@@ -1,5 +1,5 @@
 /*
- * $Id: SND_OpenALDevice.cpp,v 1.20 2005/06/08 18:54:16 ton Exp $
+ * $Id: SND_OpenALDevice.cpp,v 1.22 2006/01/07 00:14:36 erwin Exp $
  *
  * ***** BEGIN GPL/BL DUAL LICENSE BLOCK *****
  *
@@ -231,8 +231,12 @@ SND_OpenALDevice::SND_OpenALDevice()
 			m_context = alcCreateContext(dev, NULL);
 
 			if (m_context) {
-				alcMakeContextCurrent(m_context);
-				m_audio = true;
+#ifdef AL_VERSION_1_1
+		alcMakeContextCurrent((ALCcontext*)m_context);
+#else
+			alcMakeContextCurrent(m_context);
+#endif
+			m_audio = true;
 				m_device = dev;
 #ifdef __linux__
 				/*
@@ -339,7 +343,11 @@ SND_OpenALDevice::~SND_OpenALDevice()
 	
 	if (m_context) {
 		MakeCurrent();
+#ifdef AL_VERSION_1_1
+		alcDestroyContext((ALCcontext*)m_context);
+#else
 		alcDestroyContext(m_context);
+#endif
 		m_context = NULL;
 	}
 	
@@ -421,7 +429,12 @@ SND_WaveSlot* SND_OpenALDevice::LoadSample(const STR_String& name,
 #if defined(OUDE_OPENAL) || defined (__APPLE__)
 					alutLoadWAVMemory((char*)memlocation, &sampleformat, &data, &numberofsamples, &samplerate);				//	openal_2.12
 #else
-					alutLoadWAVMemory((signed char*)memlocation, &sampleformat, &data, &numberofsamples, &samplerate, &loop);//	openal_2.14+
+#ifdef AL_VERSION_1_1					
+					alutLoadWAVMemory((ALbyte*)memlocation, &sampleformat, &data, &numberofsamples, &samplerate, &loop);//	openal_2.14+
+#else
+					 alutLoadWAVMemory((signed char*)memlocation, &sampleformat, &data, &numberofsamples, &samplerate, &loop);//  openal_2.14+
+					 
+#endif 
 #endif
 					/* put it in the buffer */
 					alBufferData(m_buffers[buffer], sampleformat, data, numberofsamples, samplerate);
