@@ -1,5 +1,5 @@
 /**
- * $Id: GHOST_SystemX11.cpp,v 1.7 2004/08/24 03:30:16 kester Exp $
+ * $Id: GHOST_SystemX11.cpp,v 1.9 2005/03/29 13:27:13 sirdude Exp $
  * ***** BEGIN GPL/BL DUAL LICENSE BLOCK *****
  *
  * This program is free software; you can redistribute it and/or
@@ -30,7 +30,7 @@
  */
 
 /**
- * $Id: GHOST_SystemX11.cpp,v 1.7 2004/08/24 03:30:16 kester Exp $
+ * $Id: GHOST_SystemX11.cpp,v 1.9 2005/03/29 13:27:13 sirdude Exp $
  * ***** BEGIN GPL/BL DUAL LICENSE BLOCK *****
  *
  * This program is free software; you can redistribute it and/or
@@ -524,7 +524,7 @@ getModifierKeys(
 
 	// analyse the masks retuned from XQueryPointer.
 
-	memset(m_keyboard_vector,32,0);
+	memset(m_keyboard_vector,0,sizeof(m_keyboard_vector));
 
 	XQueryKeymap(m_display,m_keyboard_vector);
 
