@@ -1,5 +1,5 @@
 /**
- * $Id: GHOST_WindowCarbon.cpp,v 1.13 2004/08/01 22:28:56 lukep Exp $
+ * $Id: GHOST_WindowCarbon.cpp,v 1.16 2005/10/20 18:05:06 lukep Exp $
  * ***** BEGIN GPL/BL DUAL LICENSE BLOCK *****
  *
  * This program is free software; you can redistribute it and/or
@@ -31,7 +31,7 @@
 
 /**
 
- * $Id: GHOST_WindowCarbon.cpp,v 1.13 2004/08/01 22:28:56 lukep Exp $
+ * $Id: GHOST_WindowCarbon.cpp,v 1.16 2005/10/20 18:05:06 lukep Exp $
  * Copyright (C) 2001 NaN Technologies B.V.
  * @author	Maarten Gribnau
  * @date	May 10, 2001
@@ -115,15 +115,21 @@ GHOST_WindowCarbon::GHOST_WindowCarbon(
 	
 	//fprintf(stderr," main screen top %i left %i height %i width %i\n", top, left, height, width);
 	
-	if (state >= 8 ) {
-		state = state - 8;
+	if (state >= GHOST_kWindowState8Normal ) {
+		if(state == GHOST_kWindowState8Normal) state= GHOST_kWindowStateNormal;
+		else if(state == GHOST_kWindowState8Maximized) state= GHOST_kWindowStateMaximized;
+		else if(state == GHOST_kWindowState8Minimized) state= GHOST_kWindowStateMinimized;
+		else if(state == GHOST_kWindowState8FullScreen) state= GHOST_kWindowStateFullScreen;
+		
+		// state = state - 8;	this was the simple version of above code, doesnt work in gcc 4.0
+		
 		setMac_windowState(1);
 	} else 
 		setMac_windowState(0);
 
 	if (state != GHOST_kWindowStateFullScreen) {
         Rect bnds = { top, left, top+height, left+width };
-        Boolean visible = (state == GHOST_kWindowStateNormal) || (state == GHOST_kWindowStateMaximized);
+        // Boolean visible = (state == GHOST_kWindowStateNormal) || (state == GHOST_kWindowStateMaximized); /*unused*/
         gen2mac(title, title255);
         
 		err =  ::CreateNewWindow( kDocumentWindowClass,
@@ -176,7 +182,7 @@ GHOST_WindowCarbon::GHOST_WindowCarbon(
             (SInt32)this);					// Store a pointer to the class in the refCon
     */
         //GHOST_PRINT("GHOST_WindowCarbon::GHOST_WindowCarbon(): creating full-screen OpenGL context\n");
-        setDrawingContextType(GHOST_kDrawingContextTypeOpenGL);
+        setDrawingContextType(GHOST_kDrawingContextTypeOpenGL);;installDrawingContext(GHOST_kDrawingContextTypeOpenGL);
         updateDrawingContext();
         activateDrawingContext();        
     }
@@ -471,7 +477,7 @@ GHOST_TSuccess GHOST_WindowCarbon::installDrawingContext(GHOST_TDrawingContextTy
             }
             else {
                 //GHOST_PRINT("GHOST_WindowCarbon::installDrawingContext(): init full-screen OpenGL\n");
-                pixelFormat = ::aglChoosePixelFormat(0, 0, sPreferredFormatFullScreen);
+GDHandle device=::GetMainDevice();pixelFormat=::aglChoosePixelFormat(&device,1,sPreferredFormatFullScreen);
                 m_aglCtx = ::aglCreateContext(pixelFormat, 0);
                 if (!m_aglCtx) break;
 				if (!s_firstaglCtx) s_firstaglCtx = m_aglCtx;
@@ -648,6 +654,7 @@ GHOST_TSuccess GHOST_WindowCarbon::setWindowCursorShape(GHOST_TStandardCursor sh
 	return GHOST_kSuccess;
 }
 
+#if 0
 /** Reverse the bits in a GHOST_TUns8 */
 static GHOST_TUns8 uns8ReverseBits(GHOST_TUns8 ch)
 {
@@ -656,6 +663,8 @@ static GHOST_TUns8 uns8ReverseBits(GHOST_TUns8 ch)
 	ch= ((ch>>4)&0x0F) | ((ch<<4)&0xF0);
 	return ch;
 }
+#endif
+
 
 /** Reverse the bits in a GHOST_TUns16 */
 static GHOST_TUns16 uns16ReverseBits(GHOST_TUns16 shrt)
@@ -698,7 +707,7 @@ GHOST_TSuccess GHOST_WindowCarbon::setWindowCustomCursorShape(GHOST_TUns8 *bitma
 GHOST_TSuccess GHOST_WindowCarbon::setWindowCustomCursorShape(GHOST_TUns8 bitmap[16][2], 
 												GHOST_TUns8 mask[16][2], int hotX, int hotY)
 {
-	setWindowCustomCursorShape((GHOST_TUns8*)bitmap, (GHOST_TUns8*) mask, 16, 16, hotX, hotY, 0, 1);
+	return setWindowCustomCursorShape((GHOST_TUns8*)bitmap, (GHOST_TUns8*) mask, 16, 16, hotX, hotY, 0, 1);
 }
 
 
