@@ -1,6 +1,6 @@
 /* display list (or rather multi purpose list) stuff */
 /* 
-	$Id: BKE_displist.h,v 1.26 2005/05/27 09:51:06 ton Exp $
+	$Id: BKE_displist.h,v 1.41 2005/09/03 17:22:28 zuster Exp $
  *
  * ***** BEGIN GPL/BL DUAL LICENSE BLOCK *****
  *
@@ -39,12 +39,10 @@
 #define DL_POLY                 0
 #define DL_SEGM                 1
 #define DL_SURF                 2
-#define DL_TRIA                 3
 #define DL_INDEX3               4
 #define DL_INDEX4               5
 #define DL_VERTCOL              6
-#define DL_VERTS                7
-#define DL_NORS                 8
+#define DL_VERTS				7
 
 /* dl->flag */
 #define DL_CYCL_U		1
@@ -52,7 +50,6 @@
 #define DL_FRONT_CURVE	4
 #define DL_BACK_CURVE	8
 
-/* This should, of course, become a function */
 #define DL_SURFINDEX(cyclu, cyclv, sizeu, sizev)	    \
 \
 if( (cyclv)==0 && a==(sizev)-1) break;		    \
@@ -98,14 +95,16 @@ struct DispListMesh {
 	struct MFace *mface;
 	struct TFace *tface;
 	float *nors; // facenormals
+
+	int dontFreeVerts, dontFreeNors, dontFreeOther;
 };
 
 void displistmesh_free(DispListMesh *dlm);
-void displistmesh_calc_normals(DispListMesh *dlm);
 
 void displistmesh_to_mesh(DispListMesh *dlm, struct Mesh *me);
 
 DispListMesh *displistmesh_copy(DispListMesh *odlm);
+DispListMesh *displistmesh_copyShared(DispListMesh *odlm);
 
 /*
  * All the different DispList.type's use the
@@ -127,6 +126,10 @@ typedef struct DispList {
 	float *verts, *nors;
 	int *index;
 	unsigned int *col1, *col2;
+	int charidx;
+	int pad;
+
+	unsigned int *bevelSplitFlag;
 } DispList;
 
 extern void copy_displist(struct ListBase *lbn, struct ListBase *lb);
@@ -135,19 +138,20 @@ extern DispList *find_displist_create(struct ListBase *lb, int type);
 extern DispList *find_displist(struct ListBase *lb, int type);
 extern void addnormalsDispList(struct Object *ob, struct ListBase *lb);
 extern void count_displist(struct ListBase *lb, int *totvert, int *totface);
-extern void curve_to_filledpoly(struct Curve *cu, struct ListBase *nurb, struct ListBase *dispbase);
 extern void freedisplist(struct ListBase *lb);
 extern int displist_has_faces(struct ListBase *lb);
-extern float calc_taper(struct Object *taperobj, int cur, int tot);
-extern void makeDispList(struct Object *ob);
-extern void set_displist_onlyzero(int val);
+extern void makeDispListMesh(struct Object *ob);
+extern void makeDispListSurf(struct Object *ob, struct ListBase *dispbase, int forRender);
+extern void makeDispListCurveTypes(struct Object *ob, int forOrco);
+extern void makeDispListMBall(struct Object *ob);
 extern void shadeDispList(struct Object *ob);
 void freefastshade(void);
-void boundbox_displist(struct Object *ob);
 void imagestodisplist(void);
 void reshadeall_displist(void);
-void test_all_displists(void);
 void filldisplist(struct ListBase *dispbase, struct ListBase *to);
 
+void mesh_create_shadedColors(struct Object *ob, int onlyForMesh, unsigned int **col1_r, unsigned int **col2_r);
+void displistmesh_add_edges(DispListMesh *dlm);
+
 #endif
 
