@@ -1,7 +1,7 @@
 /**
  * blenlib/BKE_mesh.h (mar-2001 nzc)
  *	
- * $Id: BKE_mesh.h,v 1.11 2005/03/26 17:29:37 zuster Exp $ 
+ * $Id: BKE_mesh.h,v 1.26 2006/01/10 11:36:56 blendix Exp $ 
  *
  * ***** BEGIN GPL/BL DUAL LICENSE BLOCK *****
  *
@@ -36,12 +36,14 @@
 
 /***/
 
+struct BoundBox;
 struct DispList;
 struct ListBase;
 struct MDeformVert;
 struct Mesh;
 struct MFace;
 struct MVert;
+struct MCol;
 struct Object;
 struct TFace;
 struct VecNor;
@@ -58,32 +60,53 @@ void make_local_tface(struct Mesh *me);
 void make_local_mesh(struct Mesh *me);
 void boundbox_mesh(struct Mesh *me, float *loc, float *size);
 void tex_space_mesh(struct Mesh *me);
-void make_orco_displist_mesh(struct Object *ob, int subdivlvl);
-void make_orco_mesh(struct Mesh *me);
-void test_index_mface(struct MFace *mface, int nr);
-void test_index_face(struct MFace *mface, struct TFace *tface, int nr);
-void flipnorm_mesh(struct Mesh *me);
+float *mesh_create_orco_render(struct Object *ob);
+float *mesh_create_orco(struct Object *ob);
+void test_index_face(struct MFace *mface, struct MCol *mc, struct TFace *tface, int nr);
 struct Mesh *get_mesh(struct Object *ob);
 void set_mesh(struct Object *ob, struct Mesh *me);
 void mball_to_mesh(struct ListBase *lb, struct Mesh *me);
 void nurbs_to_mesh(struct Object *ob);
-void edge_drawflags_mesh(struct Mesh *me);
 void mcol_to_tface(struct Mesh *me, int freedata);
+struct MCol *tface_to_mcol_p(struct TFace *tface, int totface);
 void tface_to_mcol(struct Mesh *me);
 void free_dverts(struct MDeformVert *dvert, int totvert);
 void copy_dverts(struct MDeformVert *dst, struct MDeformVert *src, int totvert); /* __NLA */
-int mesh_uses_displist(struct Mesh *me);
 int update_realtime_texture(struct TFace *tface, double time);
 void mesh_delete_material_index(struct Mesh *me, int index);
-void mesh_set_smooth_flag(struct Mesh *me, int enableSmooth);
+void mesh_set_smooth_flag(struct Object *meshOb, int enableSmooth);
 
-void make_edges(struct Mesh *me);
+struct BoundBox *mesh_get_bb(struct Mesh *me);
+void mesh_get_texspace(struct Mesh *me, float *loc_r, float *rot_r, float *size_r);
 
-	/** Generate the mesh vertex normals by averaging over connected faces.
-	 *
-	 * @param me The mesh to update.
+/* if old, it converts mface->edcode to edge drawflags */
+void make_edges(struct Mesh *me, int old);
+void mesh_strip_loose_faces(struct Mesh *me);
+
+	/* Calculate vertex and face normals, face normals are returned in *faceNors_r if non-NULL
+	 * and vertex normals are stored in actual mverts.
 	 */
-void mesh_calculate_vertex_normals	(struct Mesh *me);
+void mesh_calc_normals(struct MVert *mverts, int numVerts, struct MFace *mfaces, int numFaces, float **faceNors_r);
+
+	/* Return a newly MEM_malloc'd array of all the mesh vertex locations
+	 * (_numVerts_r_ may be NULL) */
+float (*mesh_getVertexCos(struct Mesh *me, int *numVerts_r))[3];
+
+/* map from uv vertex to face (for select linked, stitch, uv suburf) */
+
+struct UvVertMap;
+typedef struct UvVertMap UvVertMap;
+
+typedef struct UvMapVert {
+	struct UvMapVert *next;
+	unsigned int f;
+	unsigned char tfindex, separate;
+} UvMapVert;
+
+UvVertMap *make_uv_vert_map(struct MFace *mface, struct TFace *tface, unsigned int totface, unsigned int totvert, int selected, float *limit);
+UvMapVert *get_uv_map_vert(UvVertMap *vmap, unsigned int v);
+UvMapVert *get_first_uv_map_vert(UvVertMap *vmap);
+void free_uv_vert_map(UvVertMap *vmap);
 
 #ifdef __cplusplus
 }
