@@ -1,5 +1,5 @@
 /**
- * $Id: BKE_object.h,v 1.8 2005/04/16 15:06:02 ton Exp $
+ * $Id: BKE_object.h,v 1.12 2005/11/27 20:49:25 ton Exp $
  *
  * ***** BEGIN GPL/BL DUAL LICENSE BLOCK *****
  *
@@ -50,7 +50,11 @@ void copy_baseflags(void);
 void copy_objectflags(void);
 struct SoftBody *copy_softbody(struct SoftBody *sb);
 void update_base_layer(struct Object *ob);
+
 void free_object(struct Object *ob);
+void object_free_display(struct Object *ob);
+void object_free_modifiers(struct Object *ob);
+
 void unlink_object(struct Object *ob);
 int exist_object(struct Object *obtest);
 void *add_camera(void);
@@ -88,13 +92,16 @@ int during_scriptlink(void);
 void where_is_object_time(struct Object *ob, float ctime);
 void where_is_object(struct Object *ob);
 void where_is_object_simul(struct Object *ob);
-void what_does_parent1(struct Object *par, int partype, int par1, int par2, int par3);
+
 void what_does_parent(struct Object *ob);
 struct BoundBox *unit_boundbox(void);
+void boundbox_set_from_min_max(struct BoundBox *bb, float min[3], float max[3]);
 void minmax_object(struct Object *ob, float *min, float *max);
 void solve_tracking (struct Object *ob, float targetmat[][4]);
 void solve_constraints (struct Object *ob, short obtype, void *obdata, float ctime);
 
+void object_handle_update(struct Object *ob);
+
 #ifdef __cplusplus
 }
 #endif
