@@ -3,7 +3,7 @@
  * 
  * common help functions and data
  * 
- * $Id: blender.c,v 1.30 2005/05/30 13:20:13 ton Exp $
+ * $Id: blender.c,v 1.40 2005/12/05 11:52:30 ton Exp $
  *
  * ***** BEGIN GPL/BL DUAL LICENSE BLOCK *****
  *
@@ -196,13 +196,11 @@ void initglobals(void)
 {
 	memset(&G, 0, sizeof(Global));
 	
+	memset(&theEditMesh, 0, sizeof(theEditMesh));
 	G.editMesh = &theEditMesh;
-	memset(G.editMesh, 0, sizeof(G.editMesh));
 
 	U.savetime= 1;
 
-	G.animspeed= 4;
-
 	G.main= MEM_callocN(sizeof(Main), "initglobals");
 
 	strcpy(G.ima, "//");
@@ -219,6 +217,10 @@ void initglobals(void)
 #endif
 
 	clear_workob();	/* object.c */
+
+	G.charstart = 0x0000;
+	G.charmin = 0x0000;
+	G.charmax = 0xffff;
 }
 
 /***/
@@ -250,7 +252,6 @@ static void clear_global(void)
 	G.main= NULL;
 	
 	G.obedit= NULL;
-	G.obpose= NULL;
 	G.saction= NULL;
 	G.buts= NULL;
 	G.v2d= NULL;
@@ -259,6 +260,8 @@ static void clear_global(void)
 	G.sima= NULL;
 	G.sipo= NULL;
 	
+	free_vertexpaint();
+	
 	G.f &= ~(G_WEIGHTPAINT + G_VERTEXPAINT + G_FACESELECT);
 }
 
@@ -310,7 +313,6 @@ static void clean_paths(Main *main)
 static void setup_app_data(BlendFileData *bfd, char *filename) 
 {
 	Object *ob;
-	Base *base;
 	bScreen *curscreen= NULL;
 	Scene *curscene= NULL;
 	char mode;
@@ -325,7 +327,7 @@ static void setup_app_data(BlendFileData *bfd, char *filename)
 	/* no load screens? */
 	if(mode) {
 		/* comes from readfile.c */
-		extern void lib_link_screen_restore(Main *, char, Scene *);
+		extern void lib_link_screen_restore(Main *, Scene *);
 		
 		SWAP(ListBase, G.main->screen, bfd->main->screen);
 		
@@ -338,7 +340,7 @@ static void setup_app_data(BlendFileData *bfd, char *filename)
 		curscreen->scene= curscene;
 
 		/* clear_global will free G.main, here we can still restore pointers */
-		lib_link_screen_restore(bfd->main, mode, curscene);
+		lib_link_screen_restore(bfd->main, curscene);
 	}
 	
 	clear_global();
@@ -381,24 +383,9 @@ static void setup_app_data(BlendFileData *bfd, char *filename)
 
 	G.f= bfd->globalf;
 
-	/* check posemode */
-	for(base= G.scene->base.first; base; base=base->next) {
-		ob= base->object;
-		if(ob->flag & OB_POSEMODE) {
-			if(ob->type==OB_ARMATURE && G.scene->basact && G.scene->basact->object==ob) 
-				G.obpose= ob;
-			else
-				ob->flag &= ~OB_POSEMODE;
-		}
-	}
-	
-	/* few DispLists, but do text_to_curve */
-	// this should be removed!!! But first a better displist system (ton)
-	for (ob= G.main->object.first; ob; ob= ob->id.next) {
-		if(ob->type==OB_FONT) {
-			Curve *cu= ob->data;
-			if(cu->nurb.first==0) text_to_curve(ob, 0);
-		}
+	/* last stage of do_versions actually, update objects (like recalc poses) */
+	for(ob= G.main->object.first; ob; ob= ob->id.next) {
+		if(ob->recalc) object_handle_update(ob);
 	}
 	
 	if (!G.background) {
@@ -508,7 +495,6 @@ typedef struct UndoElem {
 	MemFile memfile;
 } UndoElem;
 
-#define MAXUNDO	 32
 static ListBase undobase={NULL, NULL};
 static UndoElem *curundo= NULL;
 
@@ -541,7 +527,8 @@ void BKE_write_undo(char *name)
 	UndoElem *uel;
 	
 	if( (U.uiflag & USER_GLOBALUNDO)==0) return;
-
+	if( U.undosteps==0) return;
+	
 	/* remove all undos after (also when curundo==NULL) */
 	while(undobase.last != curundo) {
 		uel= undobase.last;
@@ -560,7 +547,7 @@ void BKE_write_undo(char *name)
 	uel= undobase.last;
 	while(uel) {
 		nr++;
-		if(nr==MAXUNDO) break;
+		if(nr==U.undosteps) break;
 		uel= uel->prev;
 	}
 	if(uel) {
@@ -582,7 +569,7 @@ void BKE_write_undo(char *name)
 		
 		/* calculate current filename */
 		counter++;
-		counter= counter % MAXUNDO;	
+		counter= counter % U.undosteps;	
 	
 		sprintf(numstr, "%d.blend", counter);
 		BLI_make_file_string("/", tstr, U.tempdir, numstr);
