@@ -1,5 +1,5 @@
 /**
- * $Id: constraint.c,v 1.20 2005/03/09 19:45:53 lukep Exp $
+ * $Id: constraint.c,v 1.43 2006/01/13 12:14:13 ton Exp $
  *
  * ***** BEGIN GPL/BL DUAL LICENSE BLOCK *****
  *
@@ -73,9 +73,177 @@ void Mat4BlendMat4(float [][4], float [][4], float [][4], float );
 
 /* Local function prototypes */
 
-/* Functions */
+/* ********************* Data level ****************** */
 
-char constraint_has_target (bConstraint *con) {
+void free_constraint_data (bConstraint *con)
+{
+	if (con->data){
+		switch (con->type){
+			default:
+				break;
+		};
+		
+		MEM_freeN (con->data);
+	}
+}
+
+void free_constraints (ListBase *conlist)
+{
+	bConstraint *con;
+	
+	/* Do any specific freeing */
+	for (con=conlist->first; con; con=con->next) {
+		free_constraint_data (con);
+	}
+	
+	/* Free the whole list */
+	BLI_freelistN(conlist);
+}
+
+void free_constraint_channels (ListBase *chanbase)
+{
+	bConstraintChannel *chan;
+	
+	for (chan=chanbase->first; chan; chan=chan->next)
+	{
+		if (chan->ipo){
+			chan->ipo->id.us--;
+		}
+	}
+	
+	BLI_freelistN(chanbase);
+}
+
+void relink_constraints (struct ListBase *list)
+{
+	bConstraint *con;
+	
+	for (con = list->first; con; con=con->next){
+		switch (con->type){
+			case CONSTRAINT_TYPE_KINEMATIC:
+			{
+				bKinematicConstraint *data;
+				data = con->data;
+				
+				ID_NEW(data->tar);
+			}
+				break;
+			case CONSTRAINT_TYPE_NULL:
+			{
+			}
+				break;
+			case CONSTRAINT_TYPE_TRACKTO:
+			{
+				bTrackToConstraint *data;
+				data = con->data;
+				
+				ID_NEW(data->tar);
+			}
+				break;
+			case CONSTRAINT_TYPE_MINMAX:
+			{
+				bMinMaxConstraint *data;
+				data = con->data;
+				
+				ID_NEW(data->tar);
+			}
+				break;
+			case CONSTRAINT_TYPE_LOCKTRACK:
+			{
+				bLockTrackConstraint *data;
+				data = con->data;
+				
+				ID_NEW(data->tar);
+			}
+				break;
+			case CONSTRAINT_TYPE_ACTION:
+			{
+				bActionConstraint *data;
+				data = con->data;
+				
+				ID_NEW(data->tar);
+			}
+				break;
+			case CONSTRAINT_TYPE_LOCLIKE:
+			{
+				bLocateLikeConstraint *data;
+				data = con->data;
+				
+				ID_NEW(data->tar);
+			}
+				break;
+			case CONSTRAINT_TYPE_ROTLIKE:
+			{
+				bRotateLikeConstraint *data;
+				data = con->data;
+				
+				ID_NEW(data->tar);
+			}
+				break;
+			case CONSTRAINT_TYPE_FOLLOWPATH:
+			{
+				bFollowPathConstraint *data;
+				data = con->data;
+				
+				ID_NEW(data->tar);
+			}
+				break;
+			case CONSTRAINT_TYPE_STRETCHTO:
+			{
+				bStretchToConstraint *data;
+				data = con->data;
+				
+				ID_NEW(data->tar);
+			}
+				break;
+				
+		}
+	}
+}
+
+void copy_constraint_channels (ListBase *dst, ListBase *src)
+{
+	bConstraintChannel *dchan, *schan;
+	
+	dst->first=dst->last=NULL;
+	duplicatelist(dst, src);
+	
+	for (dchan=dst->first, schan=src->first; dchan; dchan=dchan->next, schan=schan->next){
+		dchan->ipo = copy_ipo(schan->ipo);
+	}
+}
+
+void clone_constraint_channels (ListBase *dst, ListBase *src)
+{
+	bConstraintChannel *dchan, *schan;
+	
+	dst->first=dst->last=NULL;
+	duplicatelist(dst, src);
+	
+	for (dchan=dst->first, schan=src->first; dchan; dchan=dchan->next, schan=schan->next){
+		id_us_plus((ID *)dchan->ipo);
+	}
+}
+
+void copy_constraints (ListBase *dst, ListBase *src)
+{
+	bConstraint *con;
+	
+	dst->first= dst->last= NULL;
+	
+	duplicatelist (dst, src);
+	
+	for (con = dst->first; con; con=con->next) {
+		con->data = MEM_dupallocN (con->data);
+		/* removed a whole lot of useless code here (ton) */
+	}
+}
+
+
+/* **************** Editor Functions **************** */
+
+char constraint_has_target (bConstraint *con) 
+{
 	switch (con->type){
 	case CONSTRAINT_TYPE_TRACKTO:
 		{
@@ -112,6 +280,13 @@ char constraint_has_target (bConstraint *con) {
 				return 1;
 		}
 		break;
+	case CONSTRAINT_TYPE_MINMAX:
+		{
+			bMinMaxConstraint *data = con->data;
+			if (data->tar)
+				return 1;
+		}
+		break;
 	case CONSTRAINT_TYPE_ACTION:
 		{
 			bActionConstraint *data = con->data;
@@ -137,7 +312,7 @@ char constraint_has_target (bConstraint *con) {
 	return 0;
 }
 
-Object *get_constraint_target(bConstraint *con)
+Object *get_constraint_target(bConstraint *con, char **subtarget)
 {
 /*
 * If the target for this constraint is target, return a pointer 
@@ -147,57 +322,147 @@ Object *get_constraint_target(bConstraint *con)
 	case CONSTRAINT_TYPE_ACTION:
 		{
 			bActionConstraint *data = con->data;
+			*subtarget= data->subtarget;
 			return data->tar;
 		}
 		break;
 	case CONSTRAINT_TYPE_LOCLIKE:
 		{
 			bLocateLikeConstraint *data = con->data;
+			*subtarget= data->subtarget;
 			return data->tar;
 		}
 		break;
 	case CONSTRAINT_TYPE_ROTLIKE:
 		{
 			bRotateLikeConstraint *data = con->data;
+			*subtarget= data->subtarget;
 			return data->tar;
 		}
 		break;
 	case CONSTRAINT_TYPE_KINEMATIC:
 		{
 			bKinematicConstraint *data = con->data;
+			*subtarget= data->subtarget;
 			return data->tar;
 		}
 		break;
 	case CONSTRAINT_TYPE_TRACKTO:
 		{
 			bTrackToConstraint *data = con->data;
+			*subtarget= data->subtarget;
+			return data->tar;
+		}
+		break;
+	case CONSTRAINT_TYPE_MINMAX:
+		{
+			bMinMaxConstraint *data = con->data;
+			*subtarget= data->subtarget;
 			return data->tar;
 		}
 		break;
 	case CONSTRAINT_TYPE_LOCKTRACK:
 		{
 			bLockTrackConstraint *data = con->data;
+			*subtarget= data->subtarget;
 			return data->tar;
 		}
 		break;
 	case CONSTRAINT_TYPE_FOLLOWPATH: 
 		{
 			bFollowPathConstraint *data = con->data;
+			*subtarget= NULL;
 			return data->tar;
 		}
 		break;
 	case CONSTRAINT_TYPE_STRETCHTO:
 		{
 			bStretchToConstraint *data = con->data;
+			*subtarget= data->subtarget;
 			return (data->tar);
 		}
 		break;
+	default:
+		*subtarget= NULL;
+		break;
 	}
 	
 	return NULL;  
 }
 
-void unique_constraint_name (bConstraint *con, ListBase *list){
+void set_constraint_target(bConstraint *con, Object *ob, char *subtarget)
+{
+	/*
+	 * Set the target for this constraint  
+	 */
+	switch (con->type) {
+		case CONSTRAINT_TYPE_ACTION:
+		{
+			bActionConstraint *data = con->data;
+			data->tar= ob;
+			if(subtarget) BLI_strncpy(data->subtarget, subtarget, 32);
+		}
+			break;
+		case CONSTRAINT_TYPE_LOCLIKE:
+		{
+			bLocateLikeConstraint *data = con->data;
+			data->tar= ob;
+			if(subtarget) BLI_strncpy(data->subtarget, subtarget, 32);
+		}
+			break;
+		case CONSTRAINT_TYPE_ROTLIKE:
+		{
+			bRotateLikeConstraint *data = con->data;
+			data->tar= ob;
+			if(subtarget) BLI_strncpy(data->subtarget, subtarget, 32);
+		}
+			break;
+		case CONSTRAINT_TYPE_KINEMATIC:
+		{
+			bKinematicConstraint *data = con->data;
+			data->tar= ob;
+			if(subtarget) BLI_strncpy(data->subtarget, subtarget, 32);
+		}
+			break;
+		case CONSTRAINT_TYPE_TRACKTO:
+		{
+			bTrackToConstraint *data = con->data;
+			data->tar= ob;
+			if(subtarget) BLI_strncpy(data->subtarget, subtarget, 32);
+		}
+			break;
+		case CONSTRAINT_TYPE_LOCKTRACK:
+		{
+			bLockTrackConstraint *data = con->data;
+			data->tar= ob;
+			if(subtarget) BLI_strncpy(data->subtarget, subtarget, 32);
+		}
+			break;
+		case CONSTRAINT_TYPE_FOLLOWPATH: 
+		{
+			bFollowPathConstraint *data = con->data;
+			data->tar= ob;
+		}
+			break;
+		case CONSTRAINT_TYPE_STRETCHTO:
+		{
+			bStretchToConstraint *data = con->data;
+			data->tar= ob;
+			if(subtarget) BLI_strncpy(data->subtarget, subtarget, 32);
+		}
+			break;
+		case CONSTRAINT_TYPE_MINMAX:
+		{
+			bMinMaxConstraint *data = (bMinMaxConstraint*)con->data;
+			data->tar= ob;
+			if(subtarget) BLI_strncpy(data->subtarget, subtarget, 32);
+		}
+			break;
+	}
+}
+
+void unique_constraint_name (bConstraint *con, ListBase *list)
+{
 	char		tempname[64];
 	int			number;
 	char		*dot;
@@ -241,9 +506,9 @@ void unique_constraint_name (bConstraint *con, ListBase *list){
 	}
 }
 
-void	*new_constraint_data (short type)
+void *new_constraint_data (short type)
 {
-	void	*result;
+	void *result;
 	
 	switch (type){
 	case CONSTRAINT_TYPE_KINEMATIC:
@@ -252,8 +517,10 @@ void	*new_constraint_data (short type)
 			data = MEM_callocN(sizeof(bKinematicConstraint), "kinematicConstraint");
 
 			data->tolerance = (float)0.001;
+			data->weight= (float)1.0;
 			data->iterations = 500;
-
+			data->flag= CONSTRAINT_IK_TIP;
+			
 			result = data;
 		}
 		break;
@@ -273,13 +540,29 @@ void	*new_constraint_data (short type)
 
 			result = data;
 
+		}
+		break;
+	case CONSTRAINT_TYPE_MINMAX:
+		{
+			bMinMaxConstraint *data;
+			data = MEM_callocN(sizeof(bMinMaxConstraint), "minmaxConstraint");
+
+
+			data->minmaxflag = TRACK_Z;
+			data->offset = 0.0f;
+			data->cache[0] = data->cache[1] = data->cache[2] = 0.0f;
+			data->sticky = 0;
+			data->stuck = 0;
+
+			result = data;
+
 		}
 		break;
 	case CONSTRAINT_TYPE_ROTLIKE:
 		{
 			bRotateLikeConstraint *data;
 			data = MEM_callocN(sizeof(bRotateLikeConstraint), "rotlikeConstraint");
-
+			data->flag = ROTLIKE_X|ROTLIKE_Y|ROTLIKE_Z;
 			result = data;
 		}
 		break;
@@ -287,8 +570,7 @@ void	*new_constraint_data (short type)
 		{
 			bLocateLikeConstraint *data;
 			data = MEM_callocN(sizeof(bLocateLikeConstraint), "loclikeConstraint");
-
-			data->flag |= LOCLIKE_X|LOCLIKE_Y|LOCLIKE_Z;
+			data->flag = LOCLIKE_X|LOCLIKE_Y|LOCLIKE_Z;
 			result = data;
 		}
 		break;
@@ -296,7 +578,8 @@ void	*new_constraint_data (short type)
 		{
 			bActionConstraint *data;
 			data = MEM_callocN(sizeof(bActionConstraint), "actionConstraint");
-
+			data->local= 1;
+			
 			result = data;
 		}
 		break;
@@ -344,33 +627,53 @@ void	*new_constraint_data (short type)
 	return result;
 }
 
-bConstraintChannel *find_constraint_channel (ListBase *list, const char *name){
+bConstraintChannel *get_constraint_channel (ListBase *list, const char *name)
+{
 	bConstraintChannel *chan;
 
-	for (chan = list->first; chan; chan=chan->next){
-		if (!strcmp(name, chan->name)){
+	for (chan = list->first; chan; chan=chan->next) {
+		if (!strcmp(name, chan->name)) {
 			return chan;
 		}
 	}
 	return NULL;
 }
 
+/* finds or creates new constraint channel */
+bConstraintChannel *verify_constraint_channel (ListBase *list, const char *name)
+{
+	bConstraintChannel *chan;
+	
+	chan= get_constraint_channel (list, name);
+	if(chan==NULL) {
+		chan= MEM_callocN(sizeof(bConstraintChannel), "new constraint chan");
+		BLI_addtail(list, chan);
+		strcpy(chan->name, name);
+	}
+	
+	return chan;
+}
+
+
+/* ***************** Evaluating ********************* */
+
+/* does ipos only */
 void do_constraint_channels (ListBase *conbase, ListBase *chanbase, float ctime)
 {
 	bConstraint *con;
 	bConstraintChannel *chan;
-	IpoCurve *icu;
-
-	for (con=conbase->first; con; con=con->next){
-		chan = find_constraint_channel(chanbase, con->name);
+	IpoCurve *icu=NULL;
+	
+	for (con=conbase->first; con; con=con->next) {
+		chan = get_constraint_channel(chanbase, con->name);
 		if (chan && chan->ipo){
 			calc_ipo(chan->ipo, ctime);
 			for (icu=chan->ipo->curve.first; icu; icu=icu->next){
 				switch (icu->adrcode){
 				case CO_ENFORCE:
 					con->enforce = icu->curval;
-					if (con->enforce<0) con->enforce=0;
-					else if (con->enforce>1) con->enforce=1;
+					if (con->enforce<0.0f) con->enforce= 0.0f;
+					else if (con->enforce>1.0f) con->enforce= 1.0f;
 					break;
 				}
 			}
@@ -421,94 +724,34 @@ void Mat4BlendMat4(float out[][4], float dst[][4], float src[][4], float srcweig
 static void constraint_target_to_mat4 (Object *ob, const char *substring, float mat[][4], float size[3], float ctime)
 {
 
-	/*	Update the location of the target object */
-	//where_is_object_time (ob, ctime);	
-
 	/*	Case OBJECT */
-	if (!strlen(substring)){
+	if (!strlen(substring)) {
 		Mat4CpyMat4 (mat, ob->obmat);
-		VECCOPY (size, ob->size);
-		return;
+		VECCOPY (size, ob->size);  // whats this for, hack! (ton)
 	}
-
 	/*	Case BONE */
 	else {
-		bArmature *arm;
-		Bone	*bone;
-		float	bmat[4][4];
+		bPoseChannel *pchan;
 		float	bsize[3]={1, 1, 1};
 
-		arm = get_armature(ob);
-
-		/**
-		 *	Locate the bone (if there is one)
-		 *	Ensures that the bone's transformation is fully constrained
-		 *	(Cyclical relationships are disallowed elsewhere)
-		 */
-		bone = get_named_bone(arm, substring);
-		if (bone){
-			where_is_bone_time(ob, bone, ctime);
-			get_objectspace_bone_matrix(bone, bmat, 1, 1);
-			VECCOPY(bsize, bone->size);
+		pchan = get_pose_channel(ob->pose, substring);
+		if (pchan){
+			/**
+			 *	Multiply the objectspace bonematrix by the skeletons's global
+			 *	transform to obtain the worldspace transformation of the target
+			 */
+			Mat4MulMat4 (mat, pchan->pose_mat, ob->obmat);
 		} 
 		else
-			Mat4One (bmat);
+			Mat4CpyMat4 (mat, ob->obmat);
 
-		/**
-		 *	Multiply the objectspace bonematrix by the skeletons's global
-		 *	transform to obtain the worldspace transformation of the target
-		 */
-		VECCOPY(size, bsize);
-		Mat4MulMat4 (mat, bmat, ob->obmat);
-	
-		return;	
-	}
-}
-
-void clear_object_constraint_status (Object *ob)
-{
-	bConstraint *con;
-
-	if (!ob) return;
-
-	/* Clear the object's constraints */
-	for (con = ob->constraints.first; con; con=con->next){
-		con->flag &= ~CONSTRAINT_DONE;
-	}
-
-	/* Clear the object's subdata constraints */
-	switch (ob->type){
-	case OB_ARMATURE:
-		{
-			clear_pose_constraint_status (ob);
-		}
-		break;
-	default:
-		break;
-	}
-}
-
-void clear_all_constraints(void)
-{
-	Base *base;
-
-	/* Clear the constraint "done" flags -- this must be done
-	 * before displists are calculated for objects that are
-	 * deformed by armatures */
-	for (base = G.scene->base.first; base; base=base->next){
-		clear_object_constraint_status(base->object);
-	}
-}
-
-void rebuild_all_armature_displists(void) {
-	Base *base;
-
-	for (base = G.scene->base.first; base; base=base->next){
-		clear_object_constraint_status(base->object);
-		make_displists_by_armature(base->object);
+		VECCOPY(size, bsize);   // whats this for, hack! (ton)
 	}
 }
 
+/* called during solve_constraints */
+/* also for make_parent, to find correct inverse of "follow path" */
+/* warning, ownerdata is void... is not Bone anymore, but PoseChannel or Object */
 short get_constraint_target_matrix (bConstraint *con, short ownertype, void* ownerdata, float mat[][4], float size[3], float ctime)
 {
 	short valid=0;
@@ -521,55 +764,68 @@ short get_constraint_target_matrix (bConstraint *con, short ownertype, void* own
 		break;
 	case CONSTRAINT_TYPE_ACTION:
 		{
-			if (ownertype == TARGET_BONE){
+			if (ownertype == TARGET_BONE) {
+				extern void chan_calc_mat(bPoseChannel *chan);
 				bActionConstraint *data = (bActionConstraint*)con->data;
-				bPose *pose=NULL;
-				bPoseChannel *pchan=NULL;
-				float tempmat[4][4], imat[4][4], ans[4][4], restmat[4][4], irestmat[4][4];
+				bPose *pose;
+				bPoseChannel *pchan, *tchan;
 				float tempmat3[3][3];
-				float eul[3], size[3];
+				float eul[3];
 				float s,t;
-				Bone *curBone;
-				Bone tbone;
-				int i;
 				
-				curBone = (Bone*)ownerdata;
+				Mat4One(mat);	// return mat
 				
-				if (data->tar){
-					/*	Update the location of the target object */
-					where_is_object_time (data->tar, ctime);	
-					constraint_target_to_mat4(data->tar, data->subtarget, tempmat, size, ctime);
-					valid=1;
-				}
-				else
-					Mat4One (tempmat);
+				if (data->tar==NULL) return 0;
 				
-				/* If this is a bone, undo parent transforms */
-				if (strlen(data->subtarget)){
-					Bone* bone;
-
-					Mat4Invert(imat, data->tar->obmat);
-					bone = get_named_bone(get_armature(data->tar), data->subtarget);
-					if (bone){
-						get_objectspace_bone_matrix(bone, restmat, 1, 0);
-						Mat4Invert(irestmat, restmat);
+				/* need proper check for bone... */
+				if(data->subtarget[0]) {
+					pchan = get_pose_channel(data->tar->pose, data->subtarget);
+					if (pchan) {
+						float arm_mat[3][3], pose_mat[3][3];		/* arm mat should be bone mat! bug... */
+						
+						Mat3CpyMat4(arm_mat, pchan->bone->arm_mat);
+						Mat3CpyMat4(pose_mat, pchan->pose_mat);
+						
+						/* new; true local rotation constraint */
+						if(data->local) {
+							float diff_mat[3][3], par_mat[3][3], ipar_mat[3][3];
+							/* we need the local rotation = current rotation - (parent rotation + restpos) */
+							
+							if (pchan->parent) {
+								Mat3CpyMat4(par_mat, pchan->parent->pose_mat);
+								Mat3MulMat3(diff_mat, par_mat, arm_mat);
+								
+								Mat3Inv(ipar_mat, diff_mat);
+							}
+							else {
+								Mat3Inv(ipar_mat, arm_mat);
+							}
+							
+							Mat3MulMat3(tempmat3, ipar_mat, pose_mat);
+						}
+						else {	/* we use the deform mat, for backwards compatibility */
+							float imat[3][3];
+							
+							Mat3Inv(imat, arm_mat);
+							Mat3MulMat3(tempmat3, pose_mat, imat);
+						}
 					}
+					else Mat3One(tempmat3);
 				}
-				else{
-					Mat4One(imat);
-					Mat4One(irestmat);
+				else {
+					float ans[4][4];
+					
+					constraint_target_to_mat4(data->tar, data->subtarget, ans, size, ctime);
+					/* extract rotation, is in global world coordinates */
+					Mat3CpyMat4(tempmat3, ans);
 				}
-
-				Mat4MulSerie(ans, imat, tempmat, irestmat, NULL, NULL, NULL, NULL, NULL);
 				
-				Mat3CpyMat4(tempmat3, ans);
 				Mat3ToEul(tempmat3, eul);
-				
 				eul[0]*=(float)(180.0/M_PI);
 				eul[1]*=(float)(180.0/M_PI);
 				eul[2]*=(float)(180.0/M_PI);
-
-				/* Target is the animation */
+				
+				/* Target defines the animation */
 				s = (eul[data->type]-data->min)/(data->max-data->min);
 				if (s<0)
 					s=0;
@@ -577,31 +833,20 @@ short get_constraint_target_matrix (bConstraint *con, short ownertype, void* own
 					s=1;
 
 				t = ( s * (data->end-data->start)) + data->start;
-				
-				/* Get the appropriate information from the action */
+
+				/* Get the appropriate information from the action, we make temp pose */
 				pose = MEM_callocN(sizeof(bPose), "pose");
 				
-				verify_pose_channel(pose, curBone->name);
-				get_pose_from_action (&pose, data->act, t);
+				pchan = ownerdata;
+				tchan= verify_pose_channel(pose, pchan->name);
+				extract_pose_from_action (pose, data->act, t);
+				
+				chan_calc_mat(tchan);
+				
+				Mat4CpyMat4(mat, tchan->chan_mat);
 
-				/* Find the appropriate channel */
-				pchan = get_pose_channel(pose, curBone->name);
-				if (pchan){
-					memset(&tbone, 0x00, sizeof(Bone));
-
-					VECCOPY (tbone.loc, pchan->loc);
-					VECCOPY (tbone.size, pchan->size);				
-					for (i=0; i<4; i++)
-						tbone.quat[i]=pchan->quat[i];
-					
-					bone_to_mat4(&tbone, mat);
-
-				}
-				else{
-					Mat4One(mat);
-				}
 				/* Clean up */
-				clear_pose(pose);
+				free_pose_channels(pose);
 				MEM_freeN(pose);
 			}
 			
@@ -612,8 +857,18 @@ short get_constraint_target_matrix (bConstraint *con, short ownertype, void* own
 			bLocateLikeConstraint *data = (bLocateLikeConstraint*)con->data;
 
 			if (data->tar){
-				/*	Update the location of the target object */
-				where_is_object_time (data->tar, ctime);	
+				constraint_target_to_mat4(data->tar, data->subtarget, mat, size, ctime);
+				valid=1;
+			}
+			else
+				Mat4One (mat);
+		} 
+		break;
+	case CONSTRAINT_TYPE_MINMAX:
+		{
+			bMinMaxConstraint *data = (bMinMaxConstraint*)con->data;
+
+			if (data->tar){
 				constraint_target_to_mat4(data->tar, data->subtarget, mat, size, ctime);
 				valid=1;
 			}
@@ -627,8 +882,6 @@ short get_constraint_target_matrix (bConstraint *con, short ownertype, void* own
 			data = (bRotateLikeConstraint*)con->data;
 
 			if (data->tar){
-				/*	Update the location of the target object */
-				where_is_object_time (data->tar, ctime);	
 				constraint_target_to_mat4(data->tar, data->subtarget, mat, size, ctime);
 				valid=1;
 			}
@@ -642,9 +895,6 @@ short get_constraint_target_matrix (bConstraint *con, short ownertype, void* own
 			data = (bTrackToConstraint*)con->data;
 
 			if (data->tar){
-				// Refresh the object if it isn't a constraint loop
-				if (!(con->flag & CONSTRAINT_NOREFRESH))
-					where_is_object_time (data->tar, ctime);	
 				constraint_target_to_mat4(data->tar, data->subtarget, mat, size, ctime);
 				valid=1;
 			}
@@ -654,15 +904,27 @@ short get_constraint_target_matrix (bConstraint *con, short ownertype, void* own
 		break;
 	case CONSTRAINT_TYPE_KINEMATIC:
 		{
-			bTrackToConstraint *data;
-			data = (bTrackToConstraint*)con->data;
+			bKinematicConstraint *data;
+			data = (bKinematicConstraint*)con->data;
 
 			if (data->tar){
-				/*	Update the location of the target object */
-				where_is_object_time (data->tar, ctime);	
 				constraint_target_to_mat4(data->tar, data->subtarget, mat, size, ctime);
 				valid=1;
 			}
+			else if (data->flag & CONSTRAINT_IK_AUTO) {
+				Object *ob= ownerdata;
+				
+				if(ob==NULL)
+					Mat4One(mat);
+				else {
+					float vec[3];
+					/* move grabtarget into world space */
+					VECCOPY(vec, data->grabtarget);
+					Mat4MulVecfl(ob->obmat, vec);
+					Mat4CpyMat4(mat, ob->obmat);
+					VECCOPY(mat[3], vec);
+				}
+			}
 			else
 				Mat4One (mat);
 		} 
@@ -673,10 +935,6 @@ short get_constraint_target_matrix (bConstraint *con, short ownertype, void* own
 			data = (bLockTrackConstraint*)con->data;
 
 			if (data->tar){
-				// Refresh the object if it isn't a constraint loop
-				if (!(con->flag & CONSTRAINT_NOREFRESH))
-					where_is_object_time (data->tar, ctime);	
-
 				constraint_target_to_mat4(data->tar, data->subtarget, mat, size, ctime);
 				valid=1;
 			}
@@ -690,29 +948,17 @@ short get_constraint_target_matrix (bConstraint *con, short ownertype, void* own
 			data = (bFollowPathConstraint*)con->data;
 
 			if (data->tar){
-				short OldFlag;
 				Curve *cu;
 				float q[4], vec[4], dir[3], *quat, x1, totmat[4][4];
 				float curvetime;
 
-				where_is_object_time (data->tar, ctime);	
-
 				Mat4One (totmat);
 				Mat4One (mat);
 
 				cu= data->tar->data;
-				OldFlag = cu->flag;
-				
-				if(data->followflag) {
-					if(!(cu->flag & CU_FOLLOW)) cu->flag += CU_FOLLOW;
-				}
-				else {
-					if(cu->flag & CU_FOLLOW) cu->flag -= CU_FOLLOW;
-				}
 
-				if(!(cu->flag & CU_PATH)) cu->flag += CU_PATH;
-
-				if(cu->path==NULL || cu->path->data==NULL) calc_curvepath(data->tar);
+				/* note; when creating constraints that follow path, the curve gets the CU_PATH set now,
+					currently for paths to work it needs to go through the bevlist/displist system (ton) */
 				if(cu->path && cu->path->data) {
 					curvetime= bsystem_time(data->tar, data->tar->parent, (float)ctime, 0.0) - data->offset;
 
@@ -742,7 +988,6 @@ short get_constraint_target_matrix (bConstraint *con, short ownertype, void* own
 						Mat4MulSerie(mat, data->tar->obmat, totmat, NULL, NULL, NULL, NULL, NULL, NULL);
 					}
 				}
-				cu->flag = OldFlag;
 				valid=1;
 			}
 			else
@@ -755,7 +1000,6 @@ short get_constraint_target_matrix (bConstraint *con, short ownertype, void* own
 			data = (bStretchToConstraint*)con->data;
 
 			if (data->tar){
-				where_is_object_time (data->tar, ctime);	
 				constraint_target_to_mat4(data->tar, data->subtarget, mat, size, ctime);
 				valid = 1;
 			}
@@ -772,209 +1016,11 @@ short get_constraint_target_matrix (bConstraint *con, short ownertype, void* own
 	return valid;
 }
 
-void relink_constraints (struct ListBase *list)
-{
-	bConstraint *con;
-
-	for (con = list->first; con; con=con->next){
-		switch (con->type){
-		case CONSTRAINT_TYPE_KINEMATIC:
-			{
-				bKinematicConstraint *data;
-				data = con->data;
-
-				ID_NEW(data->tar);
-			}
-			break;
-		case CONSTRAINT_TYPE_NULL:
-			{
-			}
-			break;
-		case CONSTRAINT_TYPE_TRACKTO:
-			{
-				bTrackToConstraint *data;
-				data = con->data;
-
-				ID_NEW(data->tar);
-			}
-			break;
-		case CONSTRAINT_TYPE_LOCKTRACK:
-			{
-				bLockTrackConstraint *data;
-				data = con->data;
-
-				ID_NEW(data->tar);
-			}
-			break;
-		case CONSTRAINT_TYPE_ACTION:
-			{
-				bActionConstraint *data;
-				data = con->data;
-
-				ID_NEW(data->tar);
-			}
-			break;
-		case CONSTRAINT_TYPE_LOCLIKE:
-			{
-				bLocateLikeConstraint *data;
-				data = con->data;
-
-				ID_NEW(data->tar);
-			}
-			break;
-		case CONSTRAINT_TYPE_ROTLIKE:
-			{
-				bRotateLikeConstraint *data;
-				data = con->data;
-
-				ID_NEW(data->tar);
-			}
-			break;
-		case CONSTRAINT_TYPE_FOLLOWPATH:
-			{
-				bFollowPathConstraint *data;
-				data = con->data;
-
-				ID_NEW(data->tar);
-			}
-			break;
-		case CONSTRAINT_TYPE_STRETCHTO:
-			{
-				bStretchToConstraint *data;
-				data = con->data;
-				
-				ID_NEW(data->tar);
-			}
-			break;
-			
-		}
-	}
-}
-
-void *copy_constraint_channels (ListBase *dst, ListBase *src)
-{
-	bConstraintChannel *dchan, *schan;
-	bConstraintChannel *newact=NULL;
-
-	dst->first=dst->last=NULL;
-	duplicatelist(dst, src);
-	
-	for (dchan=dst->first, schan=src->first; dchan; dchan=dchan->next, schan=schan->next){
-		dchan->ipo = copy_ipo(schan->ipo);
-	}
-
-	return newact;
-}
-
-bConstraintChannel *clone_constraint_channels (ListBase *dst, ListBase *src, bConstraintChannel *oldact)
-{
-	bConstraintChannel *dchan, *schan;
-	bConstraintChannel *newact=NULL;
-
-	dst->first=dst->last=NULL;
-	duplicatelist(dst, src);
-	
-	for (dchan=dst->first, schan=src->first; dchan; dchan=dchan->next, schan=schan->next){
-		id_us_plus((ID *)dchan->ipo);
-		if (schan==oldact)
-			newact=dchan;
-	}
-
-	return newact;
-}
-
-void copy_constraints (ListBase *dst, ListBase *src)
-{
-	bConstraint *con;
-
-	dst->first=dst->last=NULL;
-
-	duplicatelist (dst, src);
-
-	/* Update specific data */
-	if (!dst->first)
-		return;
-
-	for (con = dst->first; con; con=con->next){
-		switch (con->type){
-		case CONSTRAINT_TYPE_ACTION:
-			{
-				bActionConstraint *data;
-
-				con->data = MEM_dupallocN (con->data);
-				data = (bActionConstraint*) con->data;
-			}
-			break;
-		case CONSTRAINT_TYPE_LOCLIKE:
-			{
-				bLocateLikeConstraint *data;
-
-				con->data = MEM_dupallocN (con->data);
-				data = (bLocateLikeConstraint*) con->data;
-			}
-			break;
-		case CONSTRAINT_TYPE_ROTLIKE:
-			{
-				bRotateLikeConstraint *data;
-				
-				con->data = MEM_dupallocN (con->data);
-				data = (bRotateLikeConstraint*) con->data;
-			}
-			break;
-		case CONSTRAINT_TYPE_NULL:
-			{
-				con->data = NULL;
-			}
-			break;
-		case CONSTRAINT_TYPE_TRACKTO:
-			{
-				bTrackToConstraint *data;
-				
-				con->data = MEM_dupallocN (con->data);
-				data = (bTrackToConstraint*) con->data;
-			}
-			break;
-		case CONSTRAINT_TYPE_LOCKTRACK:
-			{
-				bLockTrackConstraint *data;
-				
-				con->data = MEM_dupallocN (con->data);
-				data = (bLockTrackConstraint*) con->data;
-			}
-			break;
-		case CONSTRAINT_TYPE_KINEMATIC:
-			{
-				bKinematicConstraint *data;
-				
-				con->data = MEM_dupallocN (con->data);
-				data = (bKinematicConstraint*) con->data;
-			}
-			break;
-		case CONSTRAINT_TYPE_FOLLOWPATH:
-			{
-				bFollowPathConstraint *data;
-				
-				con->data = MEM_dupallocN (con->data);
-				data = (bFollowPathConstraint*) con->data;
-			}
-			break;
-		case CONSTRAINT_TYPE_STRETCHTO:
-			{
-				bStretchToConstraint *data;
-				
-				con->data = MEM_dupallocN (con->data);
-				data = (bStretchToConstraint*) con->data;
-			}
-			break;
-		default:
-			con->data = MEM_dupallocN (con->data);
-			break;
-		}
-	}
-}
 
+/* only called during solve_constraints */
+/* bone constraints create a fake object to work on, then ob is a workob */
+/* if ownerdata is set, it's the posechannel */
 void evaluate_constraint (bConstraint *constraint, Object *ob, short ownertype, void *ownerdata, float targetmat[][4])
-/* ob is likely to be a workob */
 {
 	float	M_oldmat[4][4];
 	float	M_identity[4][4];
@@ -984,11 +1030,6 @@ void evaluate_constraint (bConstraint *constraint, Object *ob, short ownertype,
 	
 	Mat4One (M_identity);
 	
-	/* We've already been calculated */
-	if (constraint->flag & CONSTRAINT_DONE){
-		return;
-	}
-	
 	switch (constraint->type){
 	case CONSTRAINT_TYPE_ACTION:
 		{
@@ -1017,31 +1058,107 @@ void evaluate_constraint (bConstraint *constraint, Object *ob, short ownertype,
 		break;
 	case CONSTRAINT_TYPE_ROTLIKE:
 		{
-		float	tmat[4][4];
-		float	size[3];
+			bRotateLikeConstraint *data;
+			float	tmat[3][3];
+			float	size[3];
+			
+			data = constraint->data;
+			
+			/* old files stuff only... version patch is too much code! */
+			if(data->flag==0) data->flag = ROTLIKE_X|ROTLIKE_Y|ROTLIKE_Z;
 
-		Mat4ToSize(ob->obmat, size);
-		
-		Mat4CpyMat4 (tmat, targetmat);
-		Mat4Ortho(tmat);
+			Mat4ToSize(ob->obmat, size);
+			
+			Mat3CpyMat4 (tmat, targetmat);
+			Mat3Ortho(tmat);
+			
+			if(data->flag != (ROTLIKE_X|ROTLIKE_Y|ROTLIKE_Z)) {
+				float obeul[3], eul[3], obmat[3][3];
+				
+				Mat3ToEul(tmat, eul);
+				Mat3CpyMat4(obmat, ob->obmat);
+				Mat3ToEul(obmat, obeul);
+				if(!(data->flag & ROTLIKE_X)) eul[0]= obeul[0];
+				if(!(data->flag & ROTLIKE_Y)) eul[1]= obeul[1];
+				if(!(data->flag & ROTLIKE_Z)) eul[2]= obeul[2];
+				EulToMat3(eul, tmat);
+			}
 
-		ob->obmat[0][0] = tmat[0][0]*size[0];
-		ob->obmat[0][1] = tmat[0][1]*size[1];
-		ob->obmat[0][2] = tmat[0][2]*size[2];
+			ob->obmat[0][0] = tmat[0][0]*size[0];
+			ob->obmat[0][1] = tmat[0][1]*size[1];
+			ob->obmat[0][2] = tmat[0][2]*size[2];
 
-		ob->obmat[1][0] = tmat[1][0]*size[0];
-		ob->obmat[1][1] = tmat[1][1]*size[1];
-		ob->obmat[1][2] = tmat[1][2]*size[2];
+			ob->obmat[1][0] = tmat[1][0]*size[0];
+			ob->obmat[1][1] = tmat[1][1]*size[1];
+			ob->obmat[1][2] = tmat[1][2]*size[2];
 
-		ob->obmat[2][0] = tmat[2][0]*size[0];
-		ob->obmat[2][1] = tmat[2][1]*size[1];
-		ob->obmat[2][2] = tmat[2][2]*size[2];
+			ob->obmat[2][0] = tmat[2][0]*size[0];
+			ob->obmat[2][1] = tmat[2][1]*size[1];
+			ob->obmat[2][2] = tmat[2][2]*size[2];
 		}
 		break;
 	case CONSTRAINT_TYPE_NULL:
 		{
 		}
 		break;
+	case CONSTRAINT_TYPE_MINMAX:
+		{
+			float val1, val2;
+			int index;
+			bMinMaxConstraint *data;
+
+			data = constraint->data;
+
+			switch (data->minmaxflag){
+			case TRACK_Z:
+				val1 = targetmat[3][2];
+				val2 = ob->obmat[3][2]-data->offset;
+				index = 2;
+				break;
+			case TRACK_Y:
+				val1 = targetmat[3][1];
+				val2 = ob->obmat[3][1]-data->offset;
+				index = 1;
+				break;
+			case TRACK_X:
+				val1 = targetmat[3][0];
+				val2 = ob->obmat[3][0]-data->offset;
+				index = 0;
+				break;
+			case TRACK_nZ:
+				val2 = targetmat[3][2];
+				val1 = ob->obmat[3][2]-data->offset;
+				index = 2;
+				break;
+			case TRACK_nY:
+				val2 = targetmat[3][1];
+				val1 = ob->obmat[3][1]-data->offset;
+				index = 1;
+				break;
+			case TRACK_nX:
+				val2 = targetmat[3][0];
+				val1 = ob->obmat[3][0]-data->offset;
+				index = 0;
+				break;
+			default:
+				return;
+			}
+			
+			if (val1 > val2) {
+				ob->obmat[3][index] = targetmat[3][index] + data->offset;
+				if (data->sticky==1) {
+					if (data->stuck==1) {
+						VECCOPY(ob->obmat[3], data->cache);
+					} else {
+						VECCOPY(data->cache, ob->obmat[3]);
+						data->stuck = 1;
+					}
+				}
+			} else {
+				data->stuck=0;
+			}
+		}
+		break;
 	case CONSTRAINT_TYPE_TRACKTO:
 		{
 			bTrackToConstraint *data;
@@ -1084,69 +1201,7 @@ void evaluate_constraint (bConstraint *constraint, Object *ob, short ownertype,
 		break;
 	case CONSTRAINT_TYPE_KINEMATIC:
 		{
-			bKinematicConstraint *data;
-			float	imat[4][4];
-			float	temp[4][4];
-			float totmat[4][4];
-
-			data=(bKinematicConstraint*)constraint->data;
-
-			if (data->tar && ownertype==TARGET_BONE && ownerdata){
-				Bone *curBone = (Bone*)ownerdata;
-				PoseChain *chain;
-				Object *armob;
-				
-				/* Retrieve the owner armature object from the workob */
-				armob = ob->parent;	
-				
-				/*	Make an IK chain  */				 
-				chain = ik_chain_to_posechain(armob, curBone);
-				if (!chain)
-					return;
-				chain->iterations = data->iterations;
-				chain->tolerance = data->tolerance;
-				
-				
-				{
-					float parmat[4][4];
-					
-					/* Take the obmat to objectspace */
-					Mat4CpyMat4 (temp, curBone->obmat);
-					Mat4One (curBone->obmat);
-					get_objectspace_bone_matrix(curBone, parmat, 1, 1);
-					Mat4CpyMat4 (curBone->obmat, temp);
-					Mat4MulMat4 (totmat, parmat, ob->parent->obmat);
-					
-					Mat4Invert (imat, totmat);
-					
-					Mat4CpyMat4 (temp, ob->obmat);
-					Mat4MulMat4 (ob->obmat, temp, imat);
-				}
-				
-				
-				/* Solve it */
-				if (chain->solver){
-					VECCOPY (chain->goal, targetmat[3]);					
-					solve_posechain(chain);
-				}
-				
-				free_posechain(chain);
-				
-				{
-					float parmat[4][4];
-					
-					/* Take the obmat to worldspace */
-					Mat4CpyMat4 (temp, curBone->obmat);
-					Mat4One (curBone->obmat);
-					get_objectspace_bone_matrix(curBone, parmat, 1, 1);
-					Mat4CpyMat4 (curBone->obmat, temp);
-					Mat4MulMat4 (totmat, parmat, ob->parent->obmat);
-					
-					Mat4CpyMat4 (temp, ob->obmat);
-					Mat4MulMat4 (ob->obmat, temp, totmat);
-					
-				}
-			}
+			/* removed */
 		}
 		break;
 	case CONSTRAINT_TYPE_LOCKTRACK:
@@ -1477,7 +1532,7 @@ void evaluate_constraint (bConstraint *constraint, Object *ob, short ownertype,
 			data=(bFollowPathConstraint*)constraint->data;			
 
 			if (data->tar) {
-
+				// weird, this is needed? doesnt work for workob (ton)
 				object_to_mat4(ob, obmat);
 
 				Mat4MulSerie(ob->obmat, targetmat, obmat, NULL, NULL, NULL, NULL, NULL, NULL);
@@ -1613,45 +1668,4 @@ void evaluate_constraint (bConstraint *constraint, Object *ob, short ownertype,
 		printf ("Error: Unknown constraint type\n");
 		break;
 	}
-
-}
-
-void free_constraint_data (bConstraint *con)
-{
-	if (con->data){
-		switch (con->type){
-		default:
-			break;
-		};
-		
-		MEM_freeN (con->data);
-	}
-}
-
-void free_constraints (ListBase *conlist)
-{
-	bConstraint *con;
-
-	/* Do any specific freeing */
-	for (con=conlist->first; con; con=con->next)
-	{
-		free_constraint_data (con);
-	};
-
-	/* Free the whole list */
-	BLI_freelistN(conlist);
-}
-
-void free_constraint_channels (ListBase *chanbase)
-{
-	bConstraintChannel *chan;
-
-	for (chan=chanbase->first; chan; chan=chan->next)
-	{
-		if (chan->ipo){
-			chan->ipo->id.us--;
-		}
-	}
-
-	BLI_freelistN(chanbase);
 }
