@@ -2,7 +2,7 @@
 /*  curve.c 
  * 
  *  
- * $Id: curve.c,v 1.8 2004/10/31 14:43:27 ton Exp $
+ * $Id: curve.c,v 1.27 2005/12/09 14:30:51 khughes Exp $
  *
  * ***** BEGIN GPL/BL DUAL LICENSE BLOCK *****
  *
@@ -33,8 +33,6 @@
  * ***** END GPL/BL DUAL LICENSE BLOCK *****
  */
 
-#define STRUBI hack
-
 #include <math.h>  // floor
 #include <string.h>
 #include <stdlib.h>  
@@ -43,9 +41,6 @@
 #include <config.h>
 #endif
 
-#ifdef WIN32
-#include "BLI_winstuff.h"
-#endif
 #include "MEM_guardedalloc.h"
 #include "BLI_blenlib.h"  
 #include "BLI_arithb.h"  
@@ -82,43 +77,6 @@ int cu_isectLL(float *v1, float *v2, float *v3, float *v4,
 			   short cox, short coy, 
 			   float *labda, float *mu, float *vec);
 
-
-#ifdef STRUBI
-/* hotfix; copies x*y array into extended (x+dx)*(y+dy) array
-old[] and new[] can be the same ! */
-int copyintoExtendedArray(float *old, int oldx, int oldy, float *new, int newx, int newy)
-{
-	int x, y, ttt, ooo;
-	float *oldp, *newp;
-        
-	if (newx < oldx || newy < oldy) return 0;
-	
-		
-	for (y = newy - 1; y >= oldy; y--) {	
-                ttt = y * newx;
-		for (x = newx - 1; x >= 0; x--) {
-			newp = new + 3 * (ttt + x);
-			newp[0] = 0.0; newp[1] = 0.0; newp[2] = 0.0;
-		}
-	}	
-
-	for (; y >= 0; y--) {
-		ttt = y * newx;
-                ooo = y * oldx;
-		for (x = newx - 1; x >= oldx; x--) {	
-			newp = new + 3 * (ttt + x);
-			newp[0] = 0.0; newp[1] = 0.0; newp[2] = 0.0;
-		}
-		for (; x  >= 0; x--) {
-			oldp = old + 3 * (ooo + x);
-			newp = new + 3 * (ttt + x);
-			VECCOPY(newp, oldp);
-		}
-	}
-	return 1;
-}
-#endif
-
 void unlink_curve(Curve *cu)
 {
 	int a;
@@ -148,8 +106,10 @@ void free_curve(Curve *cu)
 	
 	if(cu->mat) MEM_freeN(cu->mat);
 	if(cu->str) MEM_freeN(cu->str);
+	if(cu->strinfo) MEM_freeN(cu->strinfo);
 	if(cu->bb) MEM_freeN(cu->bb);
 	if(cu->path) free_path(cu->path);
+	if(cu->tb) MEM_freeN(cu->tb);
 }
 
 Curve *add_curve(int type)
@@ -168,8 +128,10 @@ Curve *add_curve(int type)
 	cu->pathlen= 100;
 	cu->resolu= cu->resolv= 6;
 	cu->width= 1.0;
+	cu->wordspace = 1.0;
 	cu->spacing= cu->linedist= 1.0;
 	cu->fsize= 1.0;
+	cu->ulheight = 0.05;	
 	cu->texflag= CU_AUTOSPACE;
 	
 	cu->bb= unit_boundbox();
@@ -192,6 +154,8 @@ Curve *copy_curve(Curve *cu)
 	}
 	
 	cun->str= MEM_dupallocN(cu->str);
+	cun->strinfo= MEM_dupallocN(cu->strinfo);	
+	cun->tb= MEM_dupallocN(cu->tb);
 	cun->bb= MEM_dupallocN(cu->bb);
 	
 	cun->key= copy_key(cu->key);
@@ -205,6 +169,9 @@ Curve *copy_curve(Curve *cu)
 	if(cun->ipo) cun->ipo= copy_ipo(cun->ipo);
 
 	id_us_plus((ID *)cun->vfont);
+	id_us_plus((ID *)cun->vfontb);	
+	id_us_plus((ID *)cun->vfonti);
+	id_us_plus((ID *)cun->vfontbi);
 	
 	return cun;
 }
@@ -315,28 +282,20 @@ void tex_space_curve(Curve *cu)
 		dl= dl->next;
 	}
 
-	if(doit) {
-		loc[0]= (min[0]+max[0])/2.0f;
-		loc[1]= (min[1]+max[1])/2.0f;
-		loc[2]= (min[2]+max[2])/2.0f;
-		
-		size[0]= (max[0]-min[0])/2.0f;
-		size[1]= (max[1]-min[1])/2.0f;
-		size[2]= (max[2]-min[2])/2.0f;
-	}
-	else {
-		loc[0]= loc[1]= loc[2]= 0.0f;
-		size[0]= size[1]= size[2]= 1.0f;
+	if(!doit) {
+		min[0] = min[1] = min[2] = -1.0f;
+		max[0] = max[1] = max[2] = 1.0f;
 	}
 	
-	bb->vec[0][0]=bb->vec[1][0]=bb->vec[2][0]=bb->vec[3][0]= loc[0]-size[0];
-	bb->vec[4][0]=bb->vec[5][0]=bb->vec[6][0]=bb->vec[7][0]= loc[0]+size[0];
+	loc[0]= (min[0]+max[0])/2.0f;
+	loc[1]= (min[1]+max[1])/2.0f;
+	loc[2]= (min[2]+max[2])/2.0f;
 	
-	bb->vec[0][1]=bb->vec[1][1]=bb->vec[4][1]=bb->vec[5][1]= loc[1]-size[1];
-	bb->vec[2][1]=bb->vec[3][1]=bb->vec[6][1]=bb->vec[7][1]= loc[1]+size[1];
+	size[0]= (max[0]-min[0])/2.0f;
+	size[1]= (max[1]-min[1])/2.0f;
+	size[2]= (max[2]-min[2])/2.0f;
 
-	bb->vec[0][2]=bb->vec[3][2]=bb->vec[4][2]=bb->vec[7][2]= loc[2]-size[2];
-	bb->vec[1][2]=bb->vec[2][2]=bb->vec[5][2]=bb->vec[6][2]= loc[2]+size[2];
+	boundbox_set_from_min_max(bb, min, max);
 
 	if(cu->texflag & CU_AUTOSPACE) {
 		VECCOPY(cu->loc, loc);
@@ -721,7 +680,7 @@ void basisNurb(float t, short order, short pnts, float *knots, float *basis, int
 }
 
 
-void makeNurbfaces(Nurb *nu, float *data) 
+void makeNurbfaces(Nurb *nu, float *data, int rowstride) 
 /* data  has to be 3*4*resolu*resolv in size, and zero-ed */
 {
 	BPoint *bp;
@@ -738,6 +697,9 @@ void makeNurbfaces(Nurb *nu, float *data)
 	/* allocate and initialize */
 	len= nu->pntsu*nu->pntsv;
 	if(len==0) return;
+	
+
+	
 	sum= (float *)MEM_callocN(sizeof(float)*len, "makeNurbfaces1");
 
 	resolu= nu->resolu;
@@ -866,6 +828,7 @@ void makeNurbfaces(Nurb *nu, float *data)
 			basis+= KNOTSV(nu);
 		}
 		u+= ustep;
+		if (rowstride!=0) in = (float*) (((unsigned char*) in) + (rowstride - 3*nu->resolv*sizeof(*in)));
 	}
 
 	/* free */
@@ -1050,7 +1013,8 @@ void makeNurbcurve(Nurb *nu, float *data, int dim)
 	MEM_freeN(basisu);
 }
 
-void maakbez(float q0, float q1, float q2, float q3, float *p, int it)
+/* forward differencing method for bezier curve */
+void forward_diff_bezier(float q0, float q1, float q2, float q3, float *p, int it, int stride)
 {
 	float rt0,rt1,rt2,rt3,f;
 	int a;
@@ -1070,7 +1034,7 @@ void maakbez(float q0, float q1, float q2, float q3, float *p, int it)
   
   	for(a=0; a<=it; a++) {
 		*p= q0;
-		p+= 3;
+		p+= stride;
 		q0+= q1;
  		q1+= q2;
  		q2+= q3;
@@ -1079,49 +1043,27 @@ void maakbez(float q0, float q1, float q2, float q3, float *p, int it)
 
 /* ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~ */
 
-void make_orco_surf(Curve *cu)
+float *make_orco_surf(Object *ob)
 {
+	Curve *cu = ob->data;
 	Nurb *nu;
 	int a, b, tot=0;
 	int sizeu, sizev;// ###
 	float *data;
+	float *orco;
 
-
-	/* first calculate the size of the datablock */
-	nu= cu->nurb.first;
-	while(nu) {
-#ifdef STRUBI
-/* this is a bad hack: as we want to avoid the seam in a cyclic nurbs
-texture wrapping, reserve extra orco data space to save these extra needed
-vertex based UV coordinates for the meridian vertices.
-Vertices on the 0/2pi boundary are not duplicated inside the displist but later in
-the renderface/vert construction.
-
-See also blenderWorldManipulation.c: init_render_surf()
-
-*/
-
+		/* first calculate the size of the datablock */
+	for (nu=cu->nurb.first; nu; nu=nu->next) {
 		sizeu = nu->resolu; sizev = nu->resolv;
-		if (nu->flagu & CU_CYCLIC) sizeu++;
-		if (nu->flagv & CU_CYCLIC) sizev++;
 		if(nu->pntsv>1) tot+= sizeu * sizev;
-#else
-		if(nu->pntsv>1) tot+= nu->resolu*nu->resolv;
-#endif
-		nu= nu->next;
 	}
 				/* makeNurbfaces wants zeros */
-	data= cu->orco= MEM_callocN(3*sizeof(float)*tot, "make_orco");
+	data= orco= MEM_callocN(3*sizeof(float)*tot, "make_orco");
 
-	nu= cu->nurb.first;
-	while(nu) {
+	for (nu=cu->nurb.first; nu; nu=nu->next) {
 		if(nu->pntsv>1) {
 			sizeu = nu->resolu;
 			sizev = nu->resolv;
-#ifdef STRUBI
-			if (nu->flagu & CU_CYCLIC) sizeu++;
-			if (nu->flagv & CU_CYCLIC) sizev++;
-#endif
 			
 			if(cu->flag & CU_UV_ORCO) {
 				for(b=0; b< sizeu; b++) {
@@ -1140,52 +1082,95 @@ See also blenderWorldManipulation.c: init_render_surf()
 				}
 			}
 			else {
-				makeNurbfaces(nu, data);
-#ifdef STRUBI 
-				for(b=0; b< nu->resolu; b++) {
-					for(a=0; a< nu->resolv; a++) {
-						data = cu->orco + 3 * (b * nu->resolv + a);
+				makeNurbfaces(nu, data, sizeof(*data)*sizev*3);
+
+				for(b=0; b<sizeu; b++) {
+					for(a=0; a<sizev; a++) {
+						data = orco + 3 * (b * sizev + a);
 						data[0]= (data[0]-cu->loc[0])/cu->size[0];
 						data[1]= (data[1]-cu->loc[1])/cu->size[1];
 						data[2]= (data[2]-cu->loc[2])/cu->size[2];
 					}
 				}
-				copyintoExtendedArray(cu->orco, nu->resolv, nu->resolu, cu->orco, sizev, sizeu);
-				/* copy U/V-cyclic orco's */
-				if (nu->flagv & CU_CYCLIC) {
-					b = sizeu - 1;	
-					for(a=0; a< sizev; a++) {
-						data = cu->orco + 3 * (b * sizev + a);
-						VECCOPY(data, cu->orco + 3*a);
-					}
-				}	
-				if (nu->flagu & CU_CYCLIC) {
-					a = sizev - 1;	
-					for(b=0; b< sizeu; b++) {
-						data = cu->orco + 3 * (b * sizev + a);
-						VECCOPY(data, cu->orco + 3 * b*sizev);
-					}
-				}	
-
-#else
-				tot= sizeu * sizev;
-				while(tot--) {
-					data[0]= (data[0]-cu->loc[0])/cu->size[0];
-					data[1]= (data[1]-cu->loc[1])/cu->size[1];
-					data[2]= (data[2]-cu->loc[2])/cu->size[2];
-	
-					data+= 3;
-				}
-#endif
 			}
 		}
-		nu= nu->next;
 	}
-	/* loadkeypostype(22, base, base); */
 
+	return orco;
 }
 
 
+	/* NOTE: This routine is tied to the order of vertex
+	 * built by displist and as passed to the renderer.
+	 */
+float *make_orco_curve(Object *ob)
+{
+	Curve *cu = ob->data;
+	DispList *dl;
+	int u, v, numVerts;
+	float *fp, *orco;
+	int remakeDisp = 0;
+
+	if (!(cu->flag&CU_UV_ORCO) && cu->key && cu->key->refkey) {
+		cp_cu_key(cu, cu->key->refkey, 0, count_curveverts(&cu->nurb));
+		makeDispListCurveTypes(ob, 1);
+		remakeDisp = 1;
+	}
+
+		/* Assumes displist has been built */
+
+	numVerts = 0;
+	for (dl=cu->disp.first; dl; dl=dl->next) {
+		if (dl->type==DL_INDEX3) {
+			numVerts += dl->nr;
+		} else if (dl->type==DL_SURF) {
+			numVerts += dl->parts*dl->nr;
+		}
+	}
+
+	fp= orco= MEM_mallocN(3*sizeof(float)*numVerts, "cu_orco");
+
+	for (dl=cu->disp.first; dl; dl=dl->next) {
+		if (dl->type==DL_INDEX3) {
+			for (u=0; u<dl->nr; u++,fp+=3) {
+				if (cu->flag&CU_UV_ORCO) {
+					fp[0]= 2.0f*u/(dl->nr-1) - 1.0f;
+					fp[1]= 0.0;
+					fp[2]= 0.0;
+				} else {
+					VECCOPY(fp, &dl->verts[u*3]);
+
+					fp[0]= (fp[0]-cu->loc[0])/cu->size[0];
+					fp[1]= (fp[1]-cu->loc[1])/cu->size[1];
+					fp[2]= (fp[2]-cu->loc[2])/cu->size[2];
+				}
+			}
+		} else if (dl->type==DL_SURF) {
+			for (u=0; u<dl->parts; u++) {
+				for (v=0; v<dl->nr; v++,fp+=3) {
+					if (cu->flag&CU_UV_ORCO) {
+						fp[0]= 2.0f*u/(dl->parts-1) - 1.0f;
+						fp[1]= 2.0f*v/(dl->nr-1) - 1.0f;
+						fp[2]= 0.0;
+					} else {
+						VECCOPY(fp, &dl->verts[(dl->nr*u + v)*3]);
+
+						fp[0]= (fp[0]-cu->loc[0])/cu->size[0];
+						fp[1]= (fp[1]-cu->loc[1])/cu->size[1];
+						fp[2]= (fp[2]-cu->loc[2])/cu->size[2];
+					}
+				}
+			}
+		}
+	}
+
+	if (remakeDisp) {
+		makeDispListCurveTypes(ob, 0);
+	}
+
+	return orco;
+}
+
 
 /* ***************** BEVEL ****************** */
 
@@ -1197,6 +1182,10 @@ void makebevelcurve(Object *ob, ListBase *disp)
 	int nr, a;
 
 	cu= ob->data;
+	disp->first = disp->last = NULL;
+
+	/* if a font object is being edited, then do nothing */
+	if( ob == G.obedit && ob->type == OB_FONT ) return;
 
 	if(cu->bevobj && cu->bevobj!=ob) {
 		if(cu->bevobj->type==OB_CURVE) {
@@ -1207,7 +1196,7 @@ void makebevelcurve(Object *ob, ListBase *disp)
 
 				dl= bevcu->disp.first;
 				if(dl==0) {
-					makeDispList(cu->bevobj);
+					makeDispListCurveTypes(cu->bevobj, 0);
 					dl= bevcu->disp.first;
 				}
 				while(dl) {
@@ -1234,6 +1223,9 @@ void makebevelcurve(Object *ob, ListBase *disp)
 			}
 		}
 	}
+	else if(cu->ext1==0.0 && cu->ext2==0.0) {
+		;
+	}
 	else if(cu->ext2==0.0) {
 		dl= MEM_callocN(sizeof(DispList), "makebevelcurve2");
 		dl->verts= MEM_mallocN(2*3*sizeof(float), "makebevelcurve2");
@@ -1250,12 +1242,14 @@ void makebevelcurve(Object *ob, ListBase *disp)
 		fp[5]= cu->ext1;
 	}
 	else {
-		nr= 4+2*cu->bevresol;
+		short dnr;
 		
 		/* bevel now in three parts, for proper vertex normals */
 		/* part 1 */
-		nr= 2+ cu->bevresol;
-		
+		dnr= nr= 2+ cu->bevresol;
+		if( (cu->flag & (CU_FRONT|CU_BACK))==0)	// we make a full round bevel in that case
+			nr= 3+ 2*cu->bevresol;
+		   
 		dl= MEM_callocN(sizeof(DispList), "makebevelcurve p1");
 		dl->verts= MEM_mallocN(nr*3*sizeof(float), "makebevelcurve p1");
 		BLI_addtail(disp, dl);
@@ -1266,8 +1260,8 @@ void makebevelcurve(Object *ob, ListBase *disp)
 
 		/* half a circle */
 		fp= dl->verts;
-		hoek= -0.5*M_PI;
-		dhoek= (0.5*M_PI/(nr-1));
+		dhoek= (0.5*M_PI/(dnr-1));
+		hoek= -(nr-1)*dhoek;
 		
 		for(a=0; a<nr; a++) {
 			fp[0]= 0.0;
@@ -1293,10 +1287,24 @@ void makebevelcurve(Object *ob, ListBase *disp)
 			fp[2]= -cu->ext1;
 			fp[4]= cu->ext2;
 			fp[5]= cu->ext1;
+			
+			if( (cu->flag & (CU_FRONT|CU_BACK))==0) {
+				dl= MEM_dupallocN(dl);
+				dl->verts= MEM_dupallocN(dl->verts);
+				BLI_addtail(disp, dl);
+				
+				fp= dl->verts;
+				fp[1]= -fp[1];
+				fp[2]= -fp[2];
+				fp[4]= -fp[4];
+				fp[5]= -fp[5];
+			}
 		}
 		
 		/* part 3 */
-		nr= 2+ cu->bevresol;
+		dnr= nr= 2+ cu->bevresol;
+		if( (cu->flag & (CU_FRONT|CU_BACK))==0)
+			nr= 3+ 2*cu->bevresol;
 		
 		dl= MEM_callocN(sizeof(DispList), "makebevelcurve p3");
 		dl->verts= MEM_mallocN(nr*3*sizeof(float), "makebevelcurve p3");
@@ -1309,7 +1317,7 @@ void makebevelcurve(Object *ob, ListBase *disp)
 		/* half a circle */
 		fp= dl->verts;
 		hoek= 0.0;
-		dhoek= (0.5*M_PI/(nr-1));
+		dhoek= (0.5*M_PI/(dnr-1));
 		
 		for(a=0; a<nr; a++) {
 			fp[0]= 0.0;
@@ -1319,7 +1327,6 @@ void makebevelcurve(Object *ob, ListBase *disp)
 			fp+= 3;
 		}
 	}
-
 }
 
 int cu_isectLL(float *v1, float *v2, float *v3, float *v4, short cox, short coy, float *labda, float *mu, float *vec)
@@ -1591,9 +1598,9 @@ void makeBevelList(Object *ob)
 						v2= bezt->vec[0];
 						
 						/* always do all three, to prevent data hanging around */
-						maakbez(v1[0], v1[3], v2[0], v2[3], data, nu->resolu);
-						maakbez(v1[1], v1[4], v2[1], v2[4], data+1, nu->resolu);
-						maakbez(v1[2], v1[5], v2[2], v2[5], data+2, nu->resolu);
+						forward_diff_bezier(v1[0], v1[3], v2[0], v2[3], data, nu->resolu, 3);
+						forward_diff_bezier(v1[1], v1[4], v2[1], v2[4], data+1, nu->resolu, 3);
+						forward_diff_bezier(v1[2], v1[5], v2[2], v2[5], data+2, nu->resolu, 3);
 						
 						if((nu->type & CU_2D)==0) {
 							if(cu->flag & CU_3D) {
@@ -1915,8 +1922,8 @@ void makeBevelList(Object *ob)
  *		1: nothing,  1:auto,  2:vector,  3:aligned
  */
 
-
-void calchandleNurb(BezTriple *bezt,BezTriple *prev, BezTriple *next, int mode)
+/* mode: is not zero when IpoCurve, is 2 when forced horizontal for autohandles */
+void calchandleNurb(BezTriple *bezt, BezTriple *prev, BezTriple *next, int mode)
 {
 	float *p1,*p2,*p3,pt[3];
 	float dx1,dy1,dz1,dx,dy,dz,vx,vy,vz,len,len1,len2;
@@ -1952,7 +1959,8 @@ void calchandleNurb(BezTriple *bezt,BezTriple *prev, BezTriple *next, int mode)
 		dy= p2[1]- p1[1];
 		dz= p2[2]- p1[2];
 	}
-	len1= (float)sqrt(dx*dx+dy*dy+dz*dz);
+	if(mode) len1= dx;
+	else len1= (float)sqrt(dx*dx+dy*dy+dz*dz);
 	
 	if(mode && bezt->h2==HD_AUTO && next) {
 		dx1= (p3[0]+p3[-3])/2.0f - p2[0];
@@ -1964,7 +1972,8 @@ void calchandleNurb(BezTriple *bezt,BezTriple *prev, BezTriple *next, int mode)
 		dy1= p3[1]- p2[1];
 		dz1= p3[2]- p2[2];
 	}
-	len2= (float)sqrt(dx1*dx1+dy1*dy1+dz1*dz1);
+	if(mode) len2= dx1;
+	else len2= (float)sqrt(dx1*dx1+dy1*dy1+dz1*dz1);
 
 	if(len1==0.0f) len1=1.0f;
 	if(len2==0.0f) len2=1.0f;
@@ -1974,9 +1983,10 @@ void calchandleNurb(BezTriple *bezt,BezTriple *prev, BezTriple *next, int mode)
 		vx= dx1/len2 + dx/len1;
 		vy= dy1/len2 + dy/len1;
 		vz= dz1/len2 + dz/len1;
-		len= 2.71f*(float)sqrt(vx*vx + vy*vy + vz*vz);
+		len= 2.5614f*(float)sqrt(vx*vx + vy*vy + vz*vz);
 		if(len!=0.0f) {
-		
+			int leftviolate=0, rightviolate=0;	/* for mode==2 */
+			
 			if(len1>5.0f*len2) len1= 5.0f*len2;	
 			if(len2>5.0f*len1) len2= 5.0f*len1;
 			
@@ -1985,13 +1995,79 @@ void calchandleNurb(BezTriple *bezt,BezTriple *prev, BezTriple *next, int mode)
 				*(p2-3)= *p2-vx*len1;
 				*(p2-2)= *(p2+1)-vy*len1;
 				*(p2-1)= *(p2+2)-vz*len1;
+				
+				if(mode==2 && next && prev) {	// keep horizontal if extrema
+					float ydiff1= prev->vec[1][1] - bezt->vec[1][1];
+					float ydiff2= next->vec[1][1] - bezt->vec[1][1];
+					if( (ydiff1<=0.0 && ydiff2<=0.0) || (ydiff1>=0.0 && ydiff2>=0.0) ) {
+						bezt->vec[0][1]= bezt->vec[1][1];
+					}
+					else {						// handles should not be beyond y coord of two others
+						if(ydiff1<=0.0) { 
+							if(prev->vec[1][1] > bezt->vec[0][1]) {
+								bezt->vec[0][1]= prev->vec[1][1]; 
+								leftviolate= 1;
+							}
+						}
+						else {
+							if(prev->vec[1][1] < bezt->vec[0][1]) {
+								bezt->vec[0][1]= prev->vec[1][1]; 
+								leftviolate= 1;
+							}
+						}
+					}
+				}
 			}
 			if(bezt->h2==HD_AUTO) {
 				len2/=len;
 				*(p2+3)= *p2+vx*len2;
 				*(p2+4)= *(p2+1)+vy*len2;
 				*(p2+5)= *(p2+2)+vz*len2;
+				
+				if(mode==2 && next && prev) {	// keep horizontal if extrema
+					float ydiff1= prev->vec[1][1] - bezt->vec[1][1];
+					float ydiff2= next->vec[1][1] - bezt->vec[1][1];
+					if( (ydiff1<=0.0 && ydiff2<=0.0) || (ydiff1>=0.0 && ydiff2>=0.0) ) {
+						bezt->vec[2][1]= bezt->vec[1][1];
+					}
+					else {						// handles should not be beyond y coord of two others
+						if(ydiff1<=0.0) { 
+							if(next->vec[1][1] < bezt->vec[2][1]) {
+								bezt->vec[2][1]= next->vec[1][1]; 
+								rightviolate= 1;
+							}
+						}
+						else {
+							if(next->vec[1][1] > bezt->vec[2][1]) {
+								bezt->vec[2][1]= next->vec[1][1]; 
+								rightviolate= 1;
+							}
+						}
+					}
+				}
 			}
+			if(leftviolate || rightviolate) {	/* align left handle */
+				float h1[3], h2[3];
+				
+				VecSubf(h1, p2-3, p2);
+				VecSubf(h2, p2, p2+3);
+				len1= Normalise(h1);
+				len2= Normalise(h2);
+				
+				vz= INPR(h1, h2);
+				
+				if(leftviolate) {
+					*(p2+3)= *(p2)   - vz*len2*h1[0];
+					*(p2+4)= *(p2+1) - vz*len2*h1[1];
+					*(p2+5)= *(p2+2) - vz*len2*h1[2];
+				}
+				else {
+					*(p2-3)= *(p2)   + vz*len1*h2[0];
+					*(p2-2)= *(p2+1) + vz*len1*h2[1];
+					*(p2-1)= *(p2+2) + vz*len1*h2[2];
+				}
+			}
+			
 		}
 	}
 
@@ -2016,6 +2092,7 @@ void calchandleNurb(BezTriple *bezt,BezTriple *prev, BezTriple *next, int mode)
 	len1= VecLenf(p2, p2-3);
 	if(len1==0.0) len1=1.0;
 	if(len2==0.0) len2=1.0;
+
 	if(bezt->f1 & 1) { /* order of calculation */
 		if(bezt->h2==HD_ALIGN) {	/* aligned */
 			len= len2/len1;
@@ -2203,7 +2280,8 @@ void sethandlesNurb(short code)
 {
 	/* code==1: set autohandle */
 	/* code==2: set vectorhandle */
-	/* if code==3 (HD_ALIGN) it toggle, vectorhandles become HD_FREE */
+	/* code==3 (HD_ALIGN) it toggle, vectorhandles become HD_FREE */
+	/* code==4: sets icu flag to become IPO_AUTO_HORIZ, horizontal extremes on auto-handles */
 	Nurb *nu;
 	BezTriple *bezt;
 	short a, ok=0;
@@ -2283,11 +2361,11 @@ void swapdata(void *adr1, void *adr2, int len)
 	else {
 		char *adr;
 
-		adr= (char *)malloc(len);
+		adr= (char *)MEM_mallocN(len, "curve swap");
 		memcpy(adr, adr1, len);
 		memcpy(adr1, adr2, len);
 		memcpy(adr2, adr, len);
-		free(adr);
+		MEM_freeN(adr);
 	}
 }
 
@@ -2392,3 +2470,57 @@ void switchdirectionNurb(Nurb *nu)
 		}
 	}
 }
+
+
+float (*curve_getVertexCos(Curve *cu, ListBase *lb, int *numVerts_r))[3]
+{
+	int i, numVerts = *numVerts_r = count_curveverts(lb);
+	float *co, (*cos)[3] = MEM_mallocN(sizeof(*cos)*numVerts, "cu_vcos");
+	Nurb *nu;
+
+	co = cos[0];
+	for (nu=lb->first; nu; nu=nu->next) {
+		if ((nu->type & 7)==CU_BEZIER) {
+			BezTriple *bezt = nu->bezt;
+
+			for (i=0; i<nu->pntsu; i++,bezt++) {
+				VECCOPY(co, bezt->vec[0]); co+=3;
+				VECCOPY(co, bezt->vec[1]); co+=3;
+				VECCOPY(co, bezt->vec[2]); co+=3;
+			}
+		} else {
+			BPoint *bp = nu->bp;
+
+			for (i=0; i<nu->pntsu*nu->pntsv; i++,bp++) {
+				VECCOPY(co, bp->vec); co+=3;
+			}
+		}
+	}
+
+	return cos;
+}
+
+void curve_applyVertexCos(Curve *cu, ListBase *lb, float (*vertexCos)[3])
+{
+	float *co = vertexCos[0];
+	Nurb *nu;
+	int i;
+
+	for (nu=lb->first; nu; nu=nu->next) {
+		if ((nu->type & 7)==CU_BEZIER) {
+			BezTriple *bezt = nu->bezt;
+
+			for (i=0; i<nu->pntsu; i++,bezt++) {
+				VECCOPY(bezt->vec[0], co); co+=3;
+				VECCOPY(bezt->vec[1], co); co+=3;
+				VECCOPY(bezt->vec[2], co); co+=3;
+			}
+		} else {
+			BPoint *bp = nu->bp;
+
+			for (i=0; i<nu->pntsu*nu->pntsv; i++,bp++) {
+				VECCOPY(bp->vec, co); co+=3;
+			}
+		}
+	}
+}
