@@ -2,7 +2,7 @@
 /*  key.c      
  *  
  * 
- * $Id: key.c,v 1.7 2004/12/11 12:41:51 ton Exp $
+ * $Id: key.c,v 1.19 2005/12/13 15:07:21 ton Exp $
  *
  * ***** BEGIN GPL/BL DUAL LICENSE BLOCK *****
  *
@@ -38,27 +38,29 @@
 
 #include "MEM_guardedalloc.h"
 
-#include "DNA_key_types.h"
+#include "DNA_curve_types.h"
 #include "DNA_ipo_types.h"
+#include "DNA_key_types.h"
+#include "DNA_lattice_types.h"
 #include "DNA_mesh_types.h"
 #include "DNA_meshdata_types.h"
 #include "DNA_object_types.h"
-#include "DNA_curve_types.h"
-#include "DNA_lattice_types.h"
 
-#include "BLI_blenlib.h"
-#include "BKE_utildefines.h"
+#include "BKE_action.h"
 #include "BKE_bad_level_calls.h"
-#include "BKE_global.h"
-#include "BKE_main.h"
-#include "BKE_library.h"
 #include "BKE_blender.h"
 #include "BKE_curve.h"
-#include "BKE_object.h"
-#include "BKE_mesh.h"
-#include "BKE_key.h"
+#include "BKE_global.h"
 #include "BKE_ipo.h"
+#include "BKE_key.h"
 #include "BKE_lattice.h"
+#include "BKE_library.h"
+#include "BKE_mesh.h"
+#include "BKE_main.h"
+#include "BKE_object.h"
+#include "BKE_utildefines.h"
+
+#include "BLI_blenlib.h"
 
 #ifdef HAVE_CONFIG_H
 #include <config.h>
@@ -191,23 +193,20 @@ void sort_keys(Key *key)
 	while(doit) {
 		doit= 0;
 		
-		kb= key->block.first;
-		while(kb) {
+		for(kb= key->block.first; kb; kb= kb->next) {
 			if(kb->next) {
 				if(kb->pos > kb->next->pos) {
 					BLI_remlink(&key->block, kb);
 					
-					/* insertlink(lb, prevlink, newlink): newlink komt na prevlink */
 					BLI_insertlink(&key->block, kb->next, kb);
 					
 					doit= 1;
 					break;
 				}
 			}
-			kb= kb->next;
 		}	
 	}
-	
+
 }
 
 /**************** do the key ****************/
@@ -275,7 +274,7 @@ void set_afgeleide_four_ipo(float d, float *data, int type)
 	}
 }
 
-int setkeys(float fac, ListBase *lb, KeyBlock *k[], float *t, int cycl)
+static int setkeys(float fac, ListBase *lb, KeyBlock *k[], float *t, int cycl)
 {
 	/* return 1 means k[2] is the position, return 0 means interpolate */
 	KeyBlock *k1, *firstkey;
@@ -398,7 +397,7 @@ int setkeys(float fac, ListBase *lb, KeyBlock *k[], float *t, int cycl)
 
 }
 
-void flerp(int aantal, float *in, float *f0, float *f1, float *f2, float *f3, float *t)	
+static void flerp(int aantal, float *in, float *f0, float *f1, float *f2, float *f3, float *t)	
 {
 	int a;
 
@@ -407,14 +406,23 @@ void flerp(int aantal, float *in, float *f0, float *f1, float *f2, float *f3, fl
 	}
 }
 
-void cp_key(int start, int end, int tot, char *poin, Key *key, KeyBlock *k, int mode)
+static void rel_flerp(int aantal, float *in, float *ref, float *out, float fac)
+{
+	int a;
+	
+	for(a=0; a<aantal; a++) {
+		in[a]-= fac*(ref[a]-out[a]);
+	}
+}
+
+static void cp_key(int start, int end, int tot, char *poin, Key *key, KeyBlock *k, float *weights, int mode)
 {
 	float ktot = 0.0, kd = 0.0;
 	int elemsize, poinsize = 0, a, *ofsp, ofs[32], flagflo=0;
-	char *k1;
+	char *k1, *kref;
 	char *cp, elemstr[8];
 
-	if(key->from==0) return;
+	if(key->from==NULL) return;
 
 	if( GS(key->from->name)==ID_ME ) {
 		ofs[0]= sizeof(MVert);
@@ -438,7 +446,8 @@ void cp_key(int start, int end, int tot, char *poin, Key *key, KeyBlock *k, int
 	if(end>tot) end= tot;
 	
 	k1= k->data;
-
+	kref= key->refkey->data;
+	
 	if(tot != k->totelem) {
 		ktot= 0.0;
 		flagflo= 1;
@@ -464,7 +473,6 @@ void cp_key(int start, int end, int tot, char *poin, Key *key, KeyBlock *k, int
 		else k1+= start*key->elemsize;
 	}	
 	
-
 	if(mode==KEY_BEZTRIPLE) {
 		elemstr[0]= 1;
 		elemstr[1]= IPO_BEZTRIPLE;
@@ -486,7 +494,15 @@ void cp_key(int start, int end, int tot, char *poin, Key *key, KeyBlock *k, int
 			switch(cp[1]) {
 			case IPO_FLOAT:
 				
-				memcpy(poin, k1, 4*cp[0]);
+				if(weights) {
+					memcpy(poin, kref, 4*cp[0]);
+					if(*weights!=0.0f)
+						rel_flerp(cp[0], (float *)poin, (float *)kref, (float *)k1, *weights);
+					weights++;
+				}
+				else 
+					memcpy(poin, k1, 4*cp[0]);
+
 				poin+= ofsp[0];
 
 				break;
@@ -513,9 +529,13 @@ void cp_key(int start, int end, int tot, char *poin, Key *key, KeyBlock *k, int
 			while(ktot>=1.0) {
 				ktot-= 1.0;
 				k1+= elemsize;
+				kref+= elemsize;
 			}
 		}
-		else k1+= elemsize;
+		else {
+			k1+= elemsize;
+			kref+= elemsize;
+		}
 		
 		if(mode==KEY_BEZTRIPLE) a+=2;
 	}
@@ -542,7 +562,7 @@ void cp_cu_key(Curve *cu, KeyBlock *kb, int start, int end)
 			a1= MAX2(a, start);
 			a2= MIN2(a+step, end);
 			
-			if(a1<a2) cp_key(a1, a2, tot, poin, cu->key, kb, KEY_BPOINT);
+			if(a1<a2) cp_key(a1, a2, tot, poin, cu->key, kb, NULL, KEY_BPOINT);
 		}
 		else if(nu->bezt) {
 			
@@ -554,7 +574,7 @@ void cp_cu_key(Curve *cu, KeyBlock *kb, int start, int end)
 			a1= MAX2(a, start);
 			a2= MIN2(a+step, end);
 
-			if(a1<a2) cp_key(a1, a2, tot, poin, cu->key, kb, KEY_BEZTRIPLE);
+			if(a1<a2) cp_key(a1, a2, tot, poin, cu->key, kb, NULL, KEY_BEZTRIPLE);
 			
 		}
 		a+= step;
@@ -563,27 +583,13 @@ void cp_cu_key(Curve *cu, KeyBlock *kb, int start, int end)
 }
 
 
-void rel_flerp(int aantal, float *in, float *ref, float *out, float fac)
-{
-	int a;
-
-	for(a=0; a<aantal; a++) {
-		in[a]-= fac*(ref[a]-out[a]);
-	}
-}
-
-
-
-void do_rel_key(int start, int end, int tot, char *basispoin, Key *key, float ctime, int mode)
+static void do_rel_key(int start, int end, int tot, char *basispoin, Key *key, int mode)
 {
 	KeyBlock *kb;
-	IpoCurve *icu;
-	float fac[KEY_TOTIPO], fval;
-	int *ofsp, ofs[3], elemsize, a, b;
+	int *ofsp, ofs[3], elemsize, b;
 	char *cp, *poin, *reffrom, *from, elemstr[8];
 	
-	if(key->from==0) return;
-	if(key->ipo==0) return;
+	if(key->from==NULL) return;
 	
 	if( GS(key->from->name)==ID_ME ) {
 		ofs[0]= sizeof(MVert);
@@ -611,41 +617,36 @@ void do_rel_key(int start, int end, int tot, char *basispoin, Key *key, float ct
 	elemsize= key->elemsize;
 	if(mode==KEY_BEZTRIPLE) elemsize*= 3;
 
-	/* step one: fetch ipo values */
-	icu= key->ipo->curve.first;
-	for(a=0; a<KEY_TOTIPO; a++) fac[a]= 0.0;
-	while(icu) {
-		fac[icu->adrcode]= icu->curval;
-		icu= icu->next;
-	}
+	/* step 1 init */
+	cp_key(start, end, tot, basispoin, key, key->refkey, NULL, mode);
 	
-	/* step 2 init */
-	cp_key(start, end, tot, basispoin, key, key->refkey, mode);
+	/* step 2: do it */
 	
-	/* step 3: do it */
-	
-	a= 1;
 	kb= key->block.first;
 	while(kb) {
 		
 		if(kb!=key->refkey) {
-			fval= fac[a];
-			a++;
-			if(a==32) break;
-
-			/* no difference allowed */
-			if(kb->totelem==tot) {
+			float icuval= kb->curval;
+			
+			/* only with value, and no difference allowed */
+			if(icuval!=0.0f && kb->totelem==tot) {
+				float weight, *weights= kb->weights;
 				
 				poin= basispoin;
 				reffrom= key->refkey->data;
 				from= kb->data;
 				
 				poin+= start*ofs[0];
-				reffrom+= elemsize*start;
-				from+= elemsize*start;
+				reffrom+= key->elemsize*start;	// key elemsize yes!
+				from+= key->elemsize*start;
 				
 				for(b=start; b<end; b++) {
 				
+					if(weights) 
+						weight= *weights * icuval;
+					else
+						weight= icuval;
+					
 					cp= key->elemstr;	
 					if(mode==KEY_BEZTRIPLE) cp= elemstr;
 					
@@ -655,16 +656,16 @@ void do_rel_key(int start, int end, int tot, char *basispoin, Key *key, float ct
 						
 						switch(cp[1]) {
 						case IPO_FLOAT:
-							rel_flerp(cp[0], (float *)poin, (float *)reffrom, (float *)from, fval);
+							rel_flerp(cp[0], (float *)poin, (float *)reffrom, (float *)from, weight);
 							
 							break;
 						case IPO_BPOINT:
-							rel_flerp(3, (float *)poin, (float *)reffrom, (float *)from, fval);
-							rel_flerp(1, (float *)(poin+16), (float *)(reffrom+16), (float *)(from+16), fval);
+							rel_flerp(3, (float *)poin, (float *)reffrom, (float *)from, icuval);
+							rel_flerp(1, (float *)(poin+16), (float *)(reffrom+16), (float *)(from+16), icuval);
 			
 							break;
 						case IPO_BEZTRIPLE:
-							rel_flerp(9, (float *)poin, (float *)reffrom, (float *)from, fval);
+							rel_flerp(9, (float *)poin, (float *)reffrom, (float *)from, icuval);
 			
 							break;
 						}
@@ -678,6 +679,8 @@ void do_rel_key(int start, int end, int tot, char *basispoin, Key *key, float ct
 					reffrom+= elemsize;
 					from+= elemsize;
 					
+					if(mode==KEY_BEZTRIPLE) b+= 2;
+					if(weights) weights++;
 				}
 			}
 		}
@@ -686,7 +689,7 @@ void do_rel_key(int start, int end, int tot, char *basispoin, Key *key, float ct
 }
 
 
-void do_key(int start, int end, int tot, char *poin, Key *key, KeyBlock **k, float *t, int mode)
+static void do_key(int start, int end, int tot, char *poin, Key *key, KeyBlock **k, float *t, int mode)
 {
 	float k1tot = 0.0, k2tot = 0.0, k3tot = 0.0, k4tot = 0.0;
 	float k1d = 0.0, k2d = 0.0, k3d = 0.0, k4d = 0.0;
@@ -897,17 +900,51 @@ void do_key(int start, int end, int tot, char *poin, Key *key, KeyBlock **k, flo
 	}
 }
 
+static float *get_weights_array(Object *ob, Mesh *me, char *vgroup)
+{
+	bDeformGroup *curdef;
+	int index= 0;
+	
+	if(vgroup[0]==0) return NULL;
+	if(me->dvert==NULL) return NULL;
+	
+	/* find the group (weak loop-in-loop) */
+	for (curdef = ob->defbase.first; curdef; curdef=curdef->next, index++)
+		if (!strcmp(curdef->name, vgroup))
+			break;
 
+	if(curdef) {
+		MDeformVert *dvert= me->dvert;
+		float *weights;
+		int i, j;
+		
+		weights= MEM_callocN(me->totvert*sizeof(float), "weights");
+		
+		for (i=0; i < me->totvert; i++, dvert++) {
+			for(j=0; j<dvert->totweight; j++) {
+				if (dvert->dw[j].def_nr == index) {
+					weights[i]= dvert->dw[j].weight;
+					break;
+				}
+			}
+		}
+		return weights;
+	}
+	return NULL;
+}
 
-void do_mesh_key(Mesh *me)
+static int do_mesh_key(Object *ob, Mesh *me)
 {
 	KeyBlock *k[4];
 	float cfra, ctime, t[4], delta, loc[3], size[3];
 	int a, flag = 0, step;
 	
-	if(me->totvert==0) return;
-	if(me->key==0) return;
-	if(me->key->block.first==0) return;
+	if(me->totvert==0) return 0;
+	if(me->key==NULL) return 0;
+	if(me->key->block.first==NULL) return 0;
+	
+	/* prevent python from screwing this up? anyhoo, the from pointer could be dropped */
+	me->key->from= (ID *)me;
 	
 	if(me->key->slurph && me->key->type!=KEY_RELATIVE ) {
 		delta= me->key->slurph;
@@ -936,43 +973,52 @@ void do_mesh_key(Mesh *me)
 				do_key(a, a+step, me->totvert, (char *)me->mvert->co, me->key, k, t, 0);
 			}
 			else {
-				cp_key(a, a+step, me->totvert, (char *)me->mvert->co, me->key, k[2], 0);
+				cp_key(a, a+step, me->totvert, (char *)me->mvert->co, me->key, k[2], NULL, 0);
 			}
 		}
 		
 		if(flag && k[2]==me->key->refkey) tex_space_mesh(me);
 		else boundbox_mesh(me, loc, size);
-		
 	}
 	else {
 		
-		ctime= bsystem_time(0, 0, (float)G.scene->r.cfra, 0.0);
-		if(calc_ipo_spec(me->key->ipo, KEY_SPEED, &ctime)==0) {
-			ctime /= 100.0;
-			CLAMP(ctime, 0.0, 1.0);
-		}
-		
 		if(me->key->type==KEY_RELATIVE) {
-			do_rel_key(0, me->totvert, me->totvert, (char *)me->mvert->co, me->key, ctime, 0);
+			KeyBlock *kb;
+			
+			for(kb= me->key->block.first; kb; kb= kb->next)
+				kb->weights= get_weights_array(ob, me, kb->vgroup);
+
+			do_rel_key(0, me->totvert, me->totvert, (char *)me->mvert->co, me->key, 0);
+			
+			for(kb= me->key->block.first; kb; kb= kb->next) {
+				if(kb->weights) MEM_freeN(kb->weights);
+				kb->weights= NULL;
+			}
 		}
 		else {
+			ctime= bsystem_time(ob, 0, G.scene->r.cfra, 0.0);
+
+			if(calc_ipo_spec(me->key->ipo, KEY_SPEED, &ctime)==0) {
+				ctime /= 100.0;
+				CLAMP(ctime, 0.0, 1.0);
+			}
+
 			flag= setkeys(ctime, &me->key->block, k, t, 0);
 			if(flag==0) {
-				
 				do_key(0, me->totvert, me->totvert, (char *)me->mvert->co, me->key, k, t, 0);
 			}
 			else {
-				cp_key(0, me->totvert, me->totvert, (char *)me->mvert->co, me->key, k[2], 0);
-	
+				cp_key(0, me->totvert, me->totvert, (char *)me->mvert->co, me->key, k[2], NULL, 0);
 			}
 			
 			if(flag && k[2]==me->key->refkey) tex_space_mesh(me);
 			else boundbox_mesh(me, loc, size);
 		}
 	}
+	return 1;
 }
 
-void do_cu_key(Curve *cu, KeyBlock **k, float *t)
+static void do_cu_key(Curve *cu, KeyBlock **k, float *t)
 {
 	Nurb *nu;
 	int a, step = 0, tot;
@@ -981,6 +1027,7 @@ void do_cu_key(Curve *cu, KeyBlock **k, float *t)
 	tot= count_curveverts(&cu->nurb);
 	nu= cu->nurb.first;
 	a= 0;
+	
 	while(nu) {
 		if(nu->bp) {
 			
@@ -1007,7 +1054,7 @@ void do_cu_key(Curve *cu, KeyBlock **k, float *t)
 	}
 }
 
-void do_rel_cu_key(Curve *cu, float ctime)
+static void do_rel_cu_key(Curve *cu, float ctime)
 {
 	Nurb *nu;
 	int a, step = 0, tot;
@@ -1025,7 +1072,7 @@ void do_rel_cu_key(Curve *cu, float ctime)
 			poin= (char *)nu->bp->vec;
 			poin -= a*sizeof(BPoint);
 			
-			do_rel_key(a, a+step, tot, poin, cu->key, ctime, KEY_BPOINT);
+			do_rel_key(a, a+step, tot, poin, cu->key, KEY_BPOINT);
 		}
 		else if(nu->bezt) {
 			
@@ -1034,15 +1081,15 @@ void do_rel_cu_key(Curve *cu, float ctime)
 			poin= (char *)nu->bezt->vec;
 			poin -= a*sizeof(BezTriple);
 
-			do_rel_key(a, a+step, tot, poin, cu->key, ctime, KEY_BEZTRIPLE);
-			
+			do_rel_key(a, a+step, tot, poin, cu->key, KEY_BEZTRIPLE);
 		}
 		a+= step;
+		
 		nu=nu->next;
 	}
 }
 
-void do_curve_key(Curve *cu)
+static int do_curve_key(Curve *cu)
 {
 	KeyBlock *k[4];
 	float cfra, ctime, t[4], delta;
@@ -1050,9 +1097,9 @@ void do_curve_key(Curve *cu)
 	
 	tot= count_curveverts(&cu->nurb);
 	
-	if(tot==0) return;
-	if(cu->key==0) return;
-	if(cu->key->block.first==0) return;
+	if(tot==0) return 0;
+	if(cu->key==NULL) return 0;
+	if(cu->key->block.first==NULL) return 0;
 	
 	if(cu->key->slurph) {
 		delta= cu->key->slurph;
@@ -1091,16 +1138,17 @@ void do_curve_key(Curve *cu)
 	}
 	else {
 		
-		ctime= bsystem_time(0, 0, (float)G.scene->r.cfra, 0.0);
-		if(calc_ipo_spec(cu->key->ipo, KEY_SPEED, &ctime)==0) {
-			ctime /= 100.0;
-			CLAMP(ctime, 0.0, 1.0);
-		}
+		ctime= bsystem_time(NULL, 0, (float)G.scene->r.cfra, 0.0);
 		
 		if(cu->key->type==KEY_RELATIVE) {
 			do_rel_cu_key(cu, ctime);
 		}
 		else {
+			if(calc_ipo_spec(cu->key->ipo, KEY_SPEED, &ctime)==0) {
+				ctime /= 100.0;
+				CLAMP(ctime, 0.0, 1.0);
+			}
+			
 			flag= setkeys(ctime, &cu->key->block, k, t, 0);
 			
 			if(flag==0) do_cu_key(cu, k, t);
@@ -1109,17 +1157,19 @@ void do_curve_key(Curve *cu)
 			if(flag && k[2]==cu->key->refkey) tex_space_curve(cu);
 		}
 	}
+	
+	return 1;
 }
 
-void do_latt_key(Lattice *lt)
+static int do_latt_key(Lattice *lt)
 {
 	KeyBlock *k[4];
 	float delta, cfra, ctime, t[4];
 	int a, tot, flag;
 	
-	if(lt->key==0) return;
-	if(lt->key->block.first==0) return;
-	
+	if(lt->key==NULL) return 0;
+	if(lt->key->block.first==NULL) return 0;
+
 	tot= lt->pntsu*lt->pntsv*lt->pntsw;
 
 	if(lt->key->slurph) {
@@ -1142,73 +1192,120 @@ void do_latt_key(Lattice *lt)
 				do_key(a, a+1, tot, (char *)lt->def->vec, lt->key, k, t, 0);
 			}
 			else {
-				cp_key(a, a+1, tot, (char *)lt->def->vec, lt->key, k[2], 0);
+				cp_key(a, a+1, tot, (char *)lt->def->vec, lt->key, k[2], NULL, 0);
 			}
 		}		
 	}
 	else {
-		ctime= bsystem_time(0, 0, (float)G.scene->r.cfra, 0.0);
-		if(calc_ipo_spec(lt->key->ipo, KEY_SPEED, &ctime)==0) {
-			ctime /= 100.0;
-			CLAMP(ctime, 0.0, 1.0);
-		}
+		ctime= bsystem_time(NULL, 0, (float)G.scene->r.cfra, 0.0);
 	
 		if(lt->key->type==KEY_RELATIVE) {
-			do_rel_key(0, tot, tot, (char *)lt->def->vec, lt->key, ctime, 0);
+			do_rel_key(0, tot, tot, (char *)lt->def->vec, lt->key, 0);
 		}
 		else {
-
+			if(calc_ipo_spec(lt->key->ipo, KEY_SPEED, &ctime)==0) {
+				ctime /= 100.0;
+				CLAMP(ctime, 0.0, 1.0);
+			}
+			
 			flag= setkeys(ctime, &lt->key->block, k, t, 0);
 			if(flag==0) {
 				do_key(0, tot, tot, (char *)lt->def->vec, lt->key, k, t, 0);
 			}
 			else {
-				cp_key(0, tot, tot, (char *)lt->def->vec, lt->key, k[2], 0);
+				cp_key(0, tot, tot, (char *)lt->def->vec, lt->key, k[2], NULL, 0);
 			}
 		}
 	}
 	
 	if(lt->flag & LT_OUTSIDE) outside_lattice(lt);
+	
+	return 1;
 }
 
-
-
-void do_all_keys()
+/* returns 1 when key applied */
+int do_ob_key(Object *ob)
 {
-	Key *key;
-	int idcode;
+	Key *key= ob_get_key(ob);
 	
-	key= G.main->key.first;
-	while(key) {
-		if(key->from) {
-			idcode= GS(key->from->name);
-		
-			if(idcode==ID_ME) do_mesh_key( (Mesh *)key->from);
-			else if(idcode==ID_CU) do_curve_key( (Curve *)key->from);
-			else if(idcode==ID_LT) do_latt_key( (Lattice *)key->from);
+	if(key==NULL)
+		return 0;
+	
+	if(ob->shapeflag & (OB_SHAPE_LOCK|OB_SHAPE_TEMPLOCK)) {
+		KeyBlock *kb= BLI_findlink(&key->block, ob->shapenr-1);
+
+		if(kb==NULL) {
+			kb= key->block.first;
+			ob->shapenr= 1;
 		}
-		key= key->id.next;
+		
+		if(ob->type==OB_MESH) {
+			Mesh *me= ob->data;
+			float *weights= get_weights_array(ob, me, kb->vgroup);
+
+			cp_key(0, me->totvert, me->totvert, (char *)me->mvert->co, key, kb, weights, 0);
+			
+			if(weights) MEM_freeN(weights);
+		}
+		else if(ob->type==OB_LATTICE) {
+			Lattice *lt= ob->data;
+			int tot= lt->pntsu*lt->pntsv*lt->pntsw;
+			
+			cp_key(0, tot, tot, (char *)lt->def->vec, key, kb, NULL, 0);
+		}
+		else if ELEM(ob->type, OB_CURVE, OB_SURF) {
+			Curve *cu= ob->data;
+			int tot= count_curveverts(&cu->nurb);
+			
+			cp_cu_key(cu, kb, 0, tot);
+		}
+		return 1;
 	}
+	else {
+		if(ob->ipoflag & OB_ACTION_KEY)
+			do_all_object_actions(ob);
+		else {
+			calc_ipo(key->ipo, bsystem_time(ob, 0, G.scene->r.cfra, 0.0));
+			execute_ipo((ID *)key, key->ipo);
+		}
+		
+		if(ob->type==OB_MESH) return do_mesh_key(ob, ob->data);
+		else if(ob->type==OB_CURVE) return do_curve_key( ob->data);
+		else if(ob->type==OB_SURF) return do_curve_key( ob->data);
+		else if(ob->type==OB_LATTICE) return do_latt_key( ob->data);
+	}
+	
+	return 0;
 }
 
-void do_ob_key(Object *ob)
+Key *ob_get_key(Object *ob)
 {
-	if(ob->type==OB_MESH) do_mesh_key( ob->data);
-	else if(ob->type==OB_CURVE) do_curve_key( ob->data);
-	else if(ob->type==OB_SURF) do_curve_key( ob->data);
-	else if(ob->type==OB_LATTICE) do_latt_key( ob->data);
+	if(ob==NULL) return NULL;
+	
+	if(ob->type==OB_MESH) {
+		Mesh *me= ob->data;
+		return me->key;
+	}
+	else if ELEM(ob->type, OB_CURVE, OB_SURF) {
+		Curve *cu= ob->data;
+		return cu->key;
+	}
+	else if(ob->type==OB_LATTICE) {
+		Lattice *lt= ob->data;
+		return lt->key;
+	}
+	return NULL;
 }
 
-void do_spec_key(Key *key)
+/* only the active keyblock */
+KeyBlock *ob_get_keyblock(Object *ob) 
 {
-	int idcode;
-	
-	if(key==0) return;
-	
-	idcode= GS(key->from->name);
-	
-	if(idcode==ID_ME) do_mesh_key( (Mesh *)key->from);
-	else if(idcode==ID_CU) do_curve_key( (Curve *)key->from);
-	else if(idcode==ID_LT) do_latt_key( (Lattice *)key->from);
+	Key *key= ob_get_key(ob);
 	
+	if (key) {
+		KeyBlock *kb= BLI_findlink(&key->block, ob->shapenr-1);
+		return kb;
+	}
+
+	return NULL;
 }
