@@ -4,7 +4,7 @@
  *  Contains management of ID's and libraries
  *  allocate and free of all library data
  * 
- * $Id: library.c,v 1.10 2004/04/22 19:58:54 ton Exp $
+ * $Id: library.c,v 1.15 2005/11/11 10:46:26 ton Exp $
  *
  * ***** BEGIN GPL/BL DUAL LICENSE BLOCK *****
  *
@@ -43,10 +43,6 @@
 #include <config.h>
 #endif
 
-#ifdef WIN32
-#include "BLI_winstuff.h"
-#endif
-
 #include "MEM_guardedalloc.h"
 
 /* all types are needed here, in order to do memory operations */
@@ -60,7 +56,6 @@
 #include "DNA_meta_types.h"
 #include "DNA_material_types.h"
 #include "DNA_texture_types.h"
-#include "DNA_ika_types.h"
 #include "DNA_image_types.h"
 #include "DNA_wave_types.h"
 #include "DNA_lamp_types.h"
@@ -75,6 +70,7 @@
 #include "DNA_group_types.h"
 #include "DNA_armature_types.h"
 #include "DNA_action_types.h"
+#include "DNA_userdef_types.h"
 
 #include "BLI_blenlib.h"
 #include "BLI_dynstr.h"
@@ -95,7 +91,6 @@
 #include "BKE_texture.h"
 #include "BKE_scene.h"
 #include "BKE_image.h"
-#include "BKE_ika.h"
 #include "BKE_ipo.h"
 #include "BKE_key.h"
 #include "BKE_world.h"
@@ -153,8 +148,6 @@ ListBase *wich_libbase(Main *mainlib, short type)
 			return &(mainlib->tex);
 		case ID_IM:
 			return &(mainlib->image);
-		case ID_IK:
-			return &(mainlib->ika);
 		case ID_WV:
 			return &(mainlib->wave);
 		case ID_LT:
@@ -213,26 +206,26 @@ int set_listbasepointers(Main *main, ListBase **lb)
 	lb[8]= &(main->mesh);
 	lb[9]= &(main->curve);
 	lb[10]= &(main->mball);
-	lb[11]= &(main->ika);
-	lb[12]= &(main->wave);
-	lb[13]= &(main->latt);
-	lb[14]= &(main->lamp);
-	lb[15]= &(main->camera);
 
-	lb[16]= &(main->world);
-	lb[17]= &(main->screen);
-	lb[18]= &(main->object);
-	lb[19]= &(main->scene);
-	lb[20]= &(main->library);
-	lb[21]= &(main->text);
-	lb[22]= &(main->sound);
-	lb[23]= &(main->group);
+	lb[11]= &(main->wave);
+	lb[12]= &(main->latt);
+	lb[13]= &(main->lamp);
+	lb[14]= &(main->camera);
 
-	lb[24]= samples;
-	lb[25]= &(main->script);
-	lb[26]=0;
+	lb[15]= &(main->world);
+	lb[16]= &(main->screen);
+	lb[17]= &(main->object);
+	lb[18]= &(main->scene);
+	lb[19]= &(main->library);
+	lb[20]= &(main->text);
+	lb[21]= &(main->sound);
+	lb[22]= &(main->group);
 
-	return 26;
+	lb[23]= samples;
+	lb[24]= &(main->script);
+	lb[25]= NULL;
+
+	return 25;
 }
 
 /* *********** ALLOC AND FREE *****************
@@ -277,9 +270,6 @@ static ID *alloc_libblock_notest(short type)
 		case ID_IM:
 			id= MEM_callocN(sizeof(Image), "image");
 			break;
-		case ID_IK:
-			id= MEM_callocN(sizeof(Ika), "ika");
-			break;
 		case ID_WV:
 			id= MEM_callocN(sizeof(Wave), "wave");
 			break;
@@ -425,9 +415,6 @@ void free_libblock(ListBase *lb, void *idv)
 		case ID_IM:
 			free_image((Image *)id);
 			break;
-		case ID_IK:
-			free_ika((Ika *)id);
-			break;
 		case ID_WV:
 			/* free_wave(id); */
 			break;
@@ -481,8 +468,6 @@ void free_libblock(ListBase *lb, void *idv)
 	BLI_remlink(lb, id);
 	MEM_freeN(id);
 
-	/* should not be here!! this is an interface-thing */
-	allspace(OOPS_TEST, 0);
 }
 
 void free_libblock_us(ListBase *lb, void *idv)		/* test users */
@@ -573,6 +558,9 @@ static void IDnames_to_dyn_pupstring(DynStr *pupds, ListBase *lb, ID *link, shor
 			char buf[32];
 			
 			if (nr && id==link) *nr= i+1;
+
+			if (U.uiflag & USER_HIDE_DOT && id->name[2]=='.')
+				continue;
 			
 			get_flags_for_id(id, buf);
 				
@@ -615,6 +603,9 @@ static void IPOnames_to_dyn_pupstring(DynStr *pupds, ListBase *lb, ID *link, sho
 				if (id==link)
 					*nr= i+1;
 					
+				if (U.uiflag & USER_HIDE_DOT && id->name[2]=='.')
+					continue;
+
 				get_flags_for_id(id, buf);
 				
 				BLI_dynstr_append(pupds, buf);
@@ -710,7 +701,7 @@ static void sort_alpha_id(ListBase *lb, ID *id)
 		
 		idtest= lb->first;
 		while(idtest) {
-			if(strcasecmp(idtest->name, id->name)>0 || idtest->lib) {
+			if(BLI_strcasecmp(idtest->name, id->name)>0 || idtest->lib) {
 				BLI_insertlinkbefore(lb, idtest, id);
 				break;
 			}
