@@ -6,7 +6,7 @@
  *  
  * texture coordinates are patched within the displist
  * 
- * $Id: mball.c,v 1.16 2004/11/20 11:59:44 jiri Exp $
+ * $Id: mball.c,v 1.25 2005/11/17 23:26:45 jiri Exp $
  *
  * ***** BEGIN GPL/BL DUAL LICENSE BLOCK *****
  *
@@ -65,6 +65,7 @@
 #include "BKE_library.h"
 #include "BKE_displist.h"
 #include "BKE_mball.h"
+#include "BKE_object.h"
 
 #ifdef HAVE_CONFIG_H
 #include <config.h>
@@ -217,29 +218,20 @@ void tex_space_mball(Object *ob)
 		dl= dl->next;
 	}
 
-	if(doit) {
-		loc[0]= (min[0]+max[0])/2.0f;
-		loc[1]= (min[1]+max[1])/2.0f;
-		loc[2]= (min[2]+max[2])/2.0f;
-		
-		size[0]= (max[0]-min[0])/2.0f;
-		size[1]= (max[1]-min[1])/2.0f;
-		size[2]= (max[2]-min[2])/2.0f;
-	}
-	else {
-		loc[0]= loc[1]= loc[2]= 0.0f;
-		size[0]= size[1]= size[2]= 1.0f;
+	if(!doit) {
+		min[0] = min[1] = min[2] = -1.0f;
+		max[0] = max[1] = max[2] = 1.0f;
 	}
 	
-	bb->vec[0][0]=bb->vec[1][0]=bb->vec[2][0]=bb->vec[3][0]= loc[0]-size[0];
-	bb->vec[4][0]=bb->vec[5][0]=bb->vec[6][0]=bb->vec[7][0]= loc[0]+size[0];
+	loc[0]= (min[0]+max[0])/2.0f;
+	loc[1]= (min[1]+max[1])/2.0f;
+	loc[2]= (min[2]+max[2])/2.0f;
 	
-	bb->vec[0][1]=bb->vec[1][1]=bb->vec[4][1]=bb->vec[5][1]= loc[1]-size[1];
-	bb->vec[2][1]=bb->vec[3][1]=bb->vec[6][1]=bb->vec[7][1]= loc[1]+size[1];
+	size[0]= (max[0]-min[0])/2.0f;
+	size[1]= (max[1]-min[1])/2.0f;
+	size[2]= (max[2]-min[2])/2.0f;
 
-	bb->vec[0][2]=bb->vec[3][2]=bb->vec[4][2]=bb->vec[7][2]= loc[2]-size[2];
-	bb->vec[1][2]=bb->vec[2][2]=bb->vec[5][2]=bb->vec[6][2]= loc[2]+size[2];
-	
+	boundbox_set_from_min_max(bb, min, max);
 }
 
 void make_orco_mball(Object *ob)
@@ -338,7 +330,7 @@ Object *find_basis_mball(Object *basis)
 			}
 			
 			while(ml){
-				totelem++;
+				if(!(ml->flag & MB_HIDE)) totelem++;
 				ml= ml->next;
 			}
 		}
@@ -395,7 +387,7 @@ Object *find_basis_mball(Object *basis)
 void calc_mballco(MetaElem *ml, float *vec)
 {
 	if(ml->mat) {
-		Mat4MulVecfl(ml->mat, vec);
+		Mat4MulVecfl((float ( * )[4])ml->mat, vec);
 	}
 }
 
@@ -407,7 +399,7 @@ float densfunc(MetaElem *ball, float x, float y, float z)
 	vec[0]= x;
 	vec[1]= y;
 	vec[2]= z;
-	Mat4MulVecfl(ball->imat, vec);
+	Mat4MulVecfl((float ( * )[4])ball->imat, vec);
 	dx= vec[0];
 	dy= vec[1];
 	dz= vec[2];
@@ -1323,9 +1315,9 @@ void find_first_points(PROCESS *mbproc, MetaBall *mb, int a)
 	/* Skip, when Stiffness of MetaElement is too small ... MetaElement can't be
 	 * visible alone ... but still can influence others MetaElements :-) */
 	if(f > 0.0) {
-		IN.x = in.x= 0.0;
-		IN.y = in.y= 0.0;
-		IN.z = in.z= 0.0;
+		OUT.x = IN.x = in.x= 0.0;
+		OUT.y = IN.y = in.y= 0.0;
+		OUT.z = IN.z = in.z= 0.0;
 
 		calc_mballco(ml, (float *)&in);
 		in_v = mbproc->function(in.x, in.y, in.z);
@@ -1446,8 +1438,8 @@ float init_meta(Object *ob)	/* return totsize */
 	MetaBall *mb;
 	MetaElem *ml;
 	float size, totsize, (*mat)[4] = NULL, (*imat)[4] = NULL, obinv[4][4], vec[3];
-	float temp1[4][4], temp2[4][4], max=0.0;
-	int a, obnr;
+	float temp1[4][4], temp2[4][4], temp3[4][4]; //max=0.0;
+	int a, obnr, zero_size=0;
 	char obname[32];
 	
 	Mat4Invert(obinv, ob->obmat);
@@ -1461,7 +1453,9 @@ float init_meta(Object *ob)	/* return totsize */
 	while(next_object(1, &base, &bob)) {
 
 		if(bob->type==OB_MBALL) {
-			ml= 0;
+			zero_size= 0;
+			ml= NULL;
+
 			if(bob==ob) {
 				mat= imat= 0;
 				mb= ob->data;
@@ -1483,58 +1477,140 @@ float init_meta(Object *ob)	/* return totsize */
 					else ml= mb->elems.first;
 				}
 			}
+
+			/* when metaball object hase zero scale, then MetaElem ot this MetaBall
+			 * will not be put to mainb array */
+			if(bob->size[0]==0.0 || bob->size[1]==0.0 || bob->size[2]==0.0) {
+				zero_size= 1;
+			}
+			else if(bob->parent) {
+				struct Object *pob=bob->parent;
+				while(pob) {
+					if(pob->size[0]==0.0 || pob->size[1]==0.0 || pob->size[2]==0.0) {
+						zero_size= 1;
+						break;
+					}
+					pob= pob->parent;
+				}
+			}
+
+			if (zero_size) {
+				unsigned int ml_count=0;
+				while(ml) {
+					ml_count++;
+					ml= ml->next;
+				}
+				totelem -= ml_count;
+			}
+			else {
 			while(ml) {
+				if(!(ml->flag & MB_HIDE)) {
+					int i;
+					float max_x, max_y, max_z, min_x, min_y, min_z;
 
-				Mat4One(temp2);
-				temp2[3][0]= ml->x;
-				temp2[3][1]= ml->y;
-				temp2[3][2]= ml->z;
-			
-				/* make a copy because of duplicates */
-				mainb[a]= new_pgn_element(sizeof(MetaElem));
-				*(mainb[a])= *ml;
-				mainb[a]->bb = new_pgn_element(sizeof(BoundBox));
+					max_x = max_y = max_z = -3.4e38;
+					min_x = min_y = min_z =  3.4e38;
+
+					/* too big stiffness seems only ugly due to linear interpolation
+					 * no need to have possibility for too big stiffness */
+					if(ml->s > 10.0) ml->s = 10.0;
+					
+					/* Rotation of MetaElem is stored in quat */
+ 					QuatToMat4(ml->quat, temp3);
+
+					/* Translation of MetaElem */
+					Mat4One(temp2);
+					temp2[3][0]= ml->x;
+					temp2[3][1]= ml->y;
+					temp2[3][2]= ml->z;
+
+					Mat4MulMat4(temp1, temp3, temp2);
 				
-				mat= new_pgn_element(4*4*sizeof(float));
-				imat= new_pgn_element(4*4*sizeof(float));
+					/* make a copy because of duplicates */
+					mainb[a]= new_pgn_element(sizeof(MetaElem));
+					*(mainb[a])= *ml;
+					mainb[a]->bb = new_pgn_element(sizeof(BoundBox));
 				
-				/* mat is the matrix to transform from mball into the basis-mball */
-				Mat4Invert(obinv, ob->obmat);
-				Mat4MulMat4(temp1, bob->obmat, obinv);
-				/* MetaBall transformation */
-				Mat4MulMat4(mat, temp2, temp1);
+					mat= new_pgn_element(4*4*sizeof(float));
+					imat= new_pgn_element(4*4*sizeof(float));
+					
+					/* mat is the matrix to transform from mball into the basis-mball */
+					Mat4Invert(obinv, ob->obmat);
+					Mat4MulMat4(temp2, bob->obmat, obinv);
+					/* MetaBall transformation */
+					Mat4MulMat4(mat, temp1, temp2);
+        
+					Mat4Invert(imat,mat);				
+        
+					mainb[a]->rad2= ml->rad*ml->rad;
+        
+					mainb[a]->mat= (float*) mat;
+					mainb[a]->imat= (float*) imat;
 
-				Mat4Invert(imat,mat);				
+					/* untransformed Bounding Box of MetaElem */
+					/* 0 */
+					mainb[a]->bb->vec[0][0]= -ml->expx;
+					mainb[a]->bb->vec[0][1]= -ml->expy;
+					mainb[a]->bb->vec[0][2]= -ml->expz;
+					/* 1 */
+					mainb[a]->bb->vec[1][0]=  ml->expx;
+					mainb[a]->bb->vec[1][1]= -ml->expy;
+					mainb[a]->bb->vec[1][2]= -ml->expz;
+					/* 2 */
+					mainb[a]->bb->vec[2][0]=  ml->expx;
+					mainb[a]->bb->vec[2][1]=  ml->expy;
+					mainb[a]->bb->vec[2][2]= -ml->expz;
+					/* 3 */
+					mainb[a]->bb->vec[3][0]= -ml->expx;
+					mainb[a]->bb->vec[3][1]=  ml->expy;
+					mainb[a]->bb->vec[3][2]= -ml->expz;
+					/* 4 */
+					mainb[a]->bb->vec[4][0]= -ml->expx;
+					mainb[a]->bb->vec[4][1]= -ml->expy;
+					mainb[a]->bb->vec[4][2]=  ml->expz;
+					/* 5 */
+					mainb[a]->bb->vec[5][0]=  ml->expx;
+					mainb[a]->bb->vec[5][1]= -ml->expy;
+					mainb[a]->bb->vec[5][2]=  ml->expz;
+					/* 6 */
+					mainb[a]->bb->vec[6][0]=  ml->expx;
+					mainb[a]->bb->vec[6][1]=  ml->expy;
+					mainb[a]->bb->vec[6][2]=  ml->expz;
+					/* 7 */
+					mainb[a]->bb->vec[7][0]= -ml->expx;
+					mainb[a]->bb->vec[7][1]=  ml->expy;
+					mainb[a]->bb->vec[7][2]=  ml->expz;
 
-				mainb[a]->rad2= ml->rad*ml->rad;
+					/* transformation of Metalem bb */
+					for(i=0; i<8; i++)
+						Mat4MulVecfl((float ( * )[4])mat, mainb[a]->bb->vec[i]);
 
-				mainb[a]->mat= (float*) mat;
-				mainb[a]->imat= (float*) imat;
+					/* find max and min of transformed bb */
+					for(i=0; i<8; i++){
+						/* find maximums */
+						if(mainb[a]->bb->vec[i][0] > max_x) max_x = mainb[a]->bb->vec[i][0];
+						if(mainb[a]->bb->vec[i][1] > max_y) max_y = mainb[a]->bb->vec[i][1];
+						if(mainb[a]->bb->vec[i][2] > max_z) max_z = mainb[a]->bb->vec[i][2];
+						/* find  minimums */
+						if(mainb[a]->bb->vec[i][0] < min_x) min_x = mainb[a]->bb->vec[i][0];
+						if(mainb[a]->bb->vec[i][1] < min_y) min_y = mainb[a]->bb->vec[i][1];
+						if(mainb[a]->bb->vec[i][2] < min_z) min_z = mainb[a]->bb->vec[i][2];
+					}
 
-				if(ml->type==MB_BALL){
-					max= 0.0;
+					/* create "new" bb, only point 0 and 6, which are
+					 * neccesary for octal tree filling */
+					mainb[a]->bb->vec[0][0] = min_x - ml->rad;
+					mainb[a]->bb->vec[0][1] = min_y - ml->rad;
+					mainb[a]->bb->vec[0][2] = min_z - ml->rad;
+
+					mainb[a]->bb->vec[6][0] = max_x + ml->rad;
+					mainb[a]->bb->vec[6][1] = max_y + ml->rad;
+					mainb[a]->bb->vec[6][2] = max_z + ml->rad;
+					
+					a++;
 				}
-				else if((ml->type==MB_TUBE)){
-					max= bob->size[0]*ml->expx;
-				}
-				else if((ml->type==MB_PLANE)){
-					max= MAX2(bob->size[0]*ml->expx, bob->size[1]*ml->expy);
-				}
-				else if((ml->type==MB_CUBE)||(ml->type==MB_ELIPSOID)){
-					max= MAX3(bob->size[0]*ml->expx, bob->size[1]*ml->expy, bob->size[2]*ml->expz);
-				}
-				
-				mainb[a]->bb->vec[0][0]= mat[3][0] - bob->size[0]*(max+ml->rad)/ob->size[0];
-				mainb[a]->bb->vec[0][1]= mat[3][1] - bob->size[1]*(max+ml->rad)/ob->size[1];
-				mainb[a]->bb->vec[0][2]= mat[3][2] - bob->size[2]*(max+ml->rad)/ob->size[2];
-				                                                                           
-				mainb[a]->bb->vec[6][0]= mat[3][0] + bob->size[0]*(max+ml->rad)/ob->size[0];
-				mainb[a]->bb->vec[6][1]= mat[3][1] + bob->size[1]*(max+ml->rad)/ob->size[1];
-				mainb[a]->bb->vec[6][2]= mat[3][2] + bob->size[2]*(max+ml->rad)/ob->size[2];
-				
 				ml= ml->next;
-				a++;
-				
+			}
 			}
 		}
 	}
@@ -1925,6 +2001,7 @@ void metaball_polygonize(Object *ob)
 	
 	mb= ob->data;
 
+	if(totelem==0) return;
 	if(!(R.flag & R_RENDERING) && (mb->flag==MB_UPDATE_NEVER)) return;
 	if(G.moving && mb->flag==MB_UPDATE_FAST) return;
 
@@ -1939,11 +2016,6 @@ void metaball_polygonize(Object *ob)
 	/* initialize all mainb (MetaElems) */
 	totsize= init_meta(ob);
 
-	if(totelem==0) {
-		MEM_freeN(mainb);
-		return;
-	}
-	
 	if(metaball_tree){
 		free_metaball_octal_node(metaball_tree->first);
 		MEM_freeN(metaball_tree);
