@@ -1,7 +1,7 @@
 /* text.c
  *
  *
- * $Id: text.c,v 1.16 2005/05/19 03:15:49 stiv Exp $
+ * $Id: text.c,v 1.19 2006/01/06 20:15:18 ianwill Exp $
  *
  * ***** BEGIN GPL/BL DUAL LICENSE BLOCK *****
  *
@@ -217,12 +217,15 @@ int reopen_text(Text *text)
 	TextLine *tmp;
 	char sdir[FILE_MAXDIR];
 	char sfile[FILE_MAXFILE];
+	char str[FILE_MAXDIR+FILE_MAXFILE];
 
 	if (!text || !text->name) return 0;
 	
-	BLI_split_dirfile(text->name, sdir, sfile);
+	BLI_strncpy(str, text->name, FILE_MAXDIR+FILE_MAXFILE);
+	BLI_convertstringcode(str, G.sce, G.scene->r.cfra);
+	BLI_split_dirfile(str, sdir, sfile);
 	
-	fp= fopen(text->name, "r");
+	fp= fopen(str, "r");
 	if(fp==NULL) return 0;
 
 	/* free memory: */
@@ -314,10 +317,14 @@ Text *add_text(char *file)
 	Text *ta;
 	char sdir[FILE_MAXDIR];
 	char sfile[FILE_MAXFILE];
+	char str[FILE_MAXDIR+FILE_MAXFILE];
 
-	BLI_split_dirfile(file, sdir, sfile);
+	BLI_strncpy(str, file, FILE_MAXDIR+FILE_MAXFILE);
+	if (G.scene) /* can be NULL (bg mode) */
+		BLI_convertstringcode(str, G.sce, G.scene->r.cfra);
+	BLI_split_dirfile(str, sdir, sfile);
 	
-	fp= fopen(file, "r");
+	fp= fopen(str, "r");
 	if(fp==NULL) return NULL;
 	
 	ta= alloc_libblock(&G.main->text, ID_TXT, sfile);
@@ -1331,6 +1338,14 @@ void txt_print_undo(Text *text)
 			ops= "Delete text block";
 		} else if (op==UNDO_IBLOCK) {
 			ops= "Insert text block";
+		} else if (op==UNDO_INDENT) {
+			ops= "Indent ";
+		} else if (op==UNDO_UNINDENT) {
+			ops= "Unindent ";
+		} else if (op==UNDO_COMMENT) {
+			ops= "Comment ";
+		} else if (op==UNDO_UNCOMMENT) {
+			ops= "Uncomment ";
 		} else {
 			ops= "Unknown";
 		}
@@ -1382,6 +1397,28 @@ void txt_print_undo(Text *text)
 			linep= linep+(text->undo_buf[i]<<16); i++;
 			linep= linep+(text->undo_buf[i]<<24); i++;
 			printf ("> (%d)", linep);
+		} else if (op==UNDO_INDENT || op==UNDO_UNINDENT) {
+			i++;
+
+			charp= text->undo_buf[i]; i++;
+			charp= charp+(text->undo_buf[i]<<8); i++;
+
+			linep= text->undo_buf[i]; i++;
+			linep= linep+(text->undo_buf[i]<<8); i++;
+			linep= linep+(text->undo_buf[i]<<16); i++;
+			linep= linep+(text->undo_buf[i]<<24); i++;
+			
+			printf ("to <%d, %d> ", linep, charp);
+
+			charp= text->undo_buf[i]; i++;
+			charp= charp+(text->undo_buf[i]<<8); i++;
+
+			linep= text->undo_buf[i]; i++;
+			linep= linep+(text->undo_buf[i]<<8); i++;
+			linep= linep+(text->undo_buf[i]<<16); i++;
+			linep= linep+(text->undo_buf[i]<<24); i++;
+			
+			printf ("from <%d, %d>", linep, charp);
 		}
 		
 		printf (" %d\n",  i);
@@ -1654,7 +1691,54 @@ void txt_do_undo(Text *text)
 			text->undo_pos--;
 
 			break;
+		case UNDO_INDENT:
+		case UNDO_UNINDENT:
+		case UNDO_COMMENT:
+		case UNDO_UNCOMMENT:
+			linep= text->undo_buf[text->undo_pos]; text->undo_pos--;
+			linep = (linep<<8)+text->undo_buf[text->undo_pos]; text->undo_pos--;
+			linep = (linep<<8)+text->undo_buf[text->undo_pos]; text->undo_pos--;
+			linep = (linep<<8)+text->undo_buf[text->undo_pos]; text->undo_pos--;
+			//linep is now the end line of the selection
 			
+			charp = text->undo_buf[text->undo_pos]; text->undo_pos--;
+			charp = (charp<<8)+text->undo_buf[text->undo_pos]; text->undo_pos--;
+			//charp is the last char selected or text->line->len
+			//set the selcetion for this now
+			text->selc = charp;
+			text->sell = text->lines.first;
+			for (i= 0; i < linep; i++) {
+				text->sell = text->sell->next;
+			}
+
+			linep= text->undo_buf[text->undo_pos]; text->undo_pos--;
+			linep = (linep<<8)+text->undo_buf[text->undo_pos]; text->undo_pos--;
+			linep = (linep<<8)+text->undo_buf[text->undo_pos]; text->undo_pos--;
+			linep = (linep<<8)+text->undo_buf[text->undo_pos]; text->undo_pos--;
+			//first line to be selected
+			
+			charp = text->undo_buf[text->undo_pos]; text->undo_pos--;
+			charp = (charp<<8)+text->undo_buf[text->undo_pos]; text->undo_pos--;
+			//first postion to be selected
+			text->curc = charp;
+			text->curl = text->lines.first;
+			for (i = 0; i < linep; i++) {
+				text->curl = text->curl->next;
+			}
+
+			
+			if (op==UNDO_INDENT) {
+				unindent(text);
+			} else if (op== UNDO_UNINDENT) {
+				indent(text);
+			} else if (op == UNDO_COMMENT) {
+				uncomment(text);
+			} else if (op == UNDO_UNCOMMENT) {
+				comment(text);
+			}
+
+			text->undo_pos--;
+			break;
 		default:
 			error("Undo buffer error - resetting");
 			text->undo_pos= -1;
@@ -1671,6 +1755,7 @@ void txt_do_redo(Text *text)
 	unsigned int linep;
 	unsigned short charp;
 	char *buf;
+	int i;
 	
 	text->undo_pos++;	
 	op= text->undo_buf[text->undo_pos];
@@ -1681,7 +1766,7 @@ void txt_do_redo(Text *text)
 	}
 	
 	undoing= 1;
-	
+
 	switch(op) {
 		case UNDO_CLEFT:
 			txt_move_left(text, 0);
@@ -1809,7 +1894,52 @@ void txt_do_redo(Text *text)
 			linep= linep+(text->undo_buf[text->undo_pos]<<24); text->undo_pos++;
 
 			break;
+		case UNDO_INDENT:
+		case UNDO_UNINDENT:
+		case UNDO_COMMENT:
+		case UNDO_UNCOMMENT:
+			text->undo_pos++;
+			charp = text->undo_buf[text->undo_pos]; text->undo_pos++;
+			charp = charp+(text->undo_buf[text->undo_pos]<<8); text->undo_pos++;
+			//charp is the first char selected or 0
+			
+			linep= text->undo_buf[text->undo_pos]; text->undo_pos++;
+			linep = linep+(text->undo_buf[text->undo_pos]<<8); text->undo_pos++;
+			linep = linep+(text->undo_buf[text->undo_pos]<<16); text->undo_pos++;
+			linep = linep+(text->undo_buf[text->undo_pos]<<24); text->undo_pos++;
+			//linep is now the first line of the selection			
+			//set the selcetion for this now
+			text->curc = charp;
+			text->curl = text->lines.first;
+			for (i= 0; i < linep; i++) {
+				text->curl = text->curl->next;
+			}
+			
+			charp = text->undo_buf[text->undo_pos]; text->undo_pos++;
+			charp = charp+(text->undo_buf[text->undo_pos]<<8); text->undo_pos++;
+			//last postion to be selected
+			linep= text->undo_buf[text->undo_pos]; text->undo_pos++;
+			linep = linep+(text->undo_buf[text->undo_pos]<<8); text->undo_pos++;
+			linep = linep+(text->undo_buf[text->undo_pos]<<16); text->undo_pos++;
+			linep = linep+(text->undo_buf[text->undo_pos]<<24); text->undo_pos++;
+			//Last line to be selected
+			
+			text->selc = charp;
+			text->sell = text->lines.first;
+			for (i = 0; i < linep; i++) {
+				text->sell = text->sell->next;
+			}
 
+			if (op==UNDO_INDENT) {
+				indent(text);
+			} else if (op== UNDO_UNINDENT) {
+				unindent(text);
+			} else if (op == UNDO_COMMENT) {
+				comment(text);
+			} else if (op == UNDO_UNCOMMENT) {
+				uncomment(text);
+			}
+			break;
 		default:
 			error("Undo buffer error - resetting");
 			text->undo_pos= -1;
@@ -2027,6 +2157,7 @@ void indent(Text *text)
 	if (!text) return;
 	if (!text->curl) return;
 	if (!text->sell) return;
+
 	num = 0;
 	while (TRUE)
 	{
@@ -2063,21 +2194,27 @@ void indent(Text *text)
 		text->curl = text->curl->prev;
 		num--;
 	}
+	
+	if(!undoing) 
+	{
+		txt_undo_add_toop(text, UNDO_INDENT, txt_get_span(text->lines.first, text->curl), text->curc, txt_get_span(text->lines.first, text->sell), text->selc);
+	}
 }
 
 void unindent(Text *text)
 {
 	int num = 0;
+	char remove = '\t';
 	
 	if (!text) return;
 	if (!text->curl) return;
 	if (!text->sell) return;
-	
+
 	while(TRUE)
 	{
 		int i = 0;
 		
-		if (text->curl->line[i] == '\t')
+		if (text->curl->line[i] == remove)
 		{
 			while(i< text->curl->len) {
 				text->curl->line[i]= text->curl->line[i+1];
@@ -2100,12 +2237,17 @@ void unindent(Text *text)
 		}
 		
 	}
-	
+	text->curc = 0;
 	while( num > 0 )
 	{
 		text->curl = text->curl->prev;
 		num--;
 	}
+	
+	if(!undoing) 
+	{
+		txt_undo_add_toop(text, UNDO_UNINDENT, txt_get_span(text->lines.first, text->curl), text->curc, txt_get_span(text->lines.first, text->sell), text->selc);
+	}
 }
 
 void comment(Text *text)
@@ -2116,7 +2258,8 @@ void comment(Text *text)
 	
 	if (!text) return;
 	if (!text->curl) return;
-	if (!text->sell) return;
+	if (!text->sell) return;// Need to change this need to check if only one line is selected ot more then one
+
 	num = 0;
 	while (TRUE)
 	{
@@ -2147,27 +2290,33 @@ void comment(Text *text)
 			num++;
 		}
 	}
-	
+	text->curc = 0;
 	while( num > 0 )
 	{
 		text->curl = text->curl->prev;
 		num--;
 	}
+	
+	if(!undoing) 
+	{
+		txt_undo_add_toop(text, UNDO_COMMENT, txt_get_span(text->lines.first, text->curl), text->curc, txt_get_span(text->lines.first, text->sell), text->selc);
+	}
 }
 
 void uncomment(Text *text)
 {
 	int num = 0;
+	char remove = '#';
 	
 	if (!text) return;
 	if (!text->curl) return;
 	if (!text->sell) return;
-	
+
 	while(TRUE)
 	{
 		int i = 0;
 		
-		if (text->curl->line[i] == '#')
+		if (text->curl->line[i] == remove)
 		{
 			while(i< text->curl->len) {
 				text->curl->line[i]= text->curl->line[i+1];
@@ -2190,12 +2339,17 @@ void uncomment(Text *text)
 		}
 		
 	}
-	
+	text->curc = 0;
 	while( num > 0 )
 	{
 		text->curl = text->curl->prev;
 		num--;
 	}
+	
+	if(!undoing) 
+	{
+		txt_undo_add_toop(text, UNDO_UNCOMMENT, txt_get_span(text->lines.first, text->curl), text->curc, txt_get_span(text->lines.first, text->sell), text->selc);
+	}
 }
 
 int setcurr_tab (Text *text)
