@@ -7,7 +7,7 @@
  * Functions here get counterparts with MTC prefixes. Basically, we phase
  * out the calls here in favour of fully prototyped versions.
  *
- * $Id: arithb.c,v 1.15 2004/11/11 13:47:06 ton Exp $
+ * $Id: arithb.c,v 1.33 2005/10/30 20:56:19 ton Exp $
  *
  * ***** BEGIN GPL/BL DUAL LICENSE BLOCK *****
  *
@@ -58,10 +58,6 @@
 #include <unistd.h>
 #endif
 
-#ifdef WIN32
-#include "BLI_winstuff.h"
-#endif
-
 #include <stdio.h>
 #include "BLI_arithb.h"
 
@@ -954,9 +950,21 @@ int FloatCompare( float *v1,  float *v2, float limit)
 }
 
 void printvecf( char *str,  float v[3])
+{
+	printf("%s: %.3f %.3f %.3f\n", str, v[0], v[1], v[2]);
+
+}
+
+void printquat( char *str,  float q[4])
+{
+	printf("%s: %.3f %.3f %.3f %.3f\n", str, q[0], q[1], q[2], q[3]);
+
+}
+
+void printvec4f( char *str,  float v[4])
 {
 	printf("%s\n", str);
-	printf("%f %f %f\n",v[0],v[1],v[2]);
+	printf("%f %f %f %f\n",v[0],v[1],v[2], v[3]);
 	printf("\n");
 
 }
@@ -964,10 +972,10 @@ void printvecf( char *str,  float v[3])
 void printmatrix4( char *str,  float m[][4])
 {
 	printf("%s\n", str);
-	printf("%f %f %f %f\n",m[0][0],m[0][1],m[0][2],m[0][3]);
-	printf("%f %f %f %f\n",m[1][0],m[1][1],m[1][2],m[1][3]);
-	printf("%f %f %f %f\n",m[2][0],m[2][1],m[2][2],m[2][3]);
-	printf("%f %f %f %f\n",m[3][0],m[3][1],m[3][2],m[3][3]);
+	printf("%f %f %f %f\n",m[0][0],m[1][0],m[2][0],m[3][0]);
+	printf("%f %f %f %f\n",m[0][1],m[1][1],m[2][1],m[3][1]);
+	printf("%f %f %f %f\n",m[0][2],m[1][2],m[2][2],m[3][2]);
+	printf("%f %f %f %f\n",m[0][3],m[1][3],m[2][3],m[3][3]);
 	printf("\n");
 
 }
@@ -975,9 +983,9 @@ void printmatrix4( char *str,  float m[][4])
 void printmatrix3( char *str,  float m[][3])
 {
 	printf("%s\n", str);
-	printf("%f %f %f\n",m[0][0],m[0][1],m[0][2]);
-	printf("%f %f %f\n",m[1][0],m[1][1],m[1][2]);
-	printf("%f %f %f\n",m[2][0],m[2][1],m[2][2]);
+	printf("%f %f %f\n",m[0][0],m[1][0],m[2][0]);
+	printf("%f %f %f\n",m[0][1],m[1][1],m[2][1]);
+	printf("%f %f %f\n",m[0][2],m[1][2],m[2][2]);
 	printf("\n");
 
 }
@@ -998,6 +1006,56 @@ void QuatMul(float *q, float *q1, float *q2)
 	q[2]=t2;
 }
 
+/* Assumes a unit quaternion */
+void QuatMulVecf(float *q, float *v)
+{
+	float t0, t1, t2;
+
+	t0=  -q[1]*v[0]-q[2]*v[1]-q[3]*v[2];
+	t1=   q[0]*v[0]+q[2]*v[2]-q[3]*v[1];
+	t2=   q[0]*v[1]+q[3]*v[0]-q[1]*v[2];
+	v[2]= q[0]*v[2]+q[1]*v[1]-q[2]*v[0];
+	v[0]=t1; 
+	v[1]=t2;
+
+	t1=   t0*-q[1]+v[0]*q[0]-v[1]*q[3]+v[2]*q[2];
+	t2=   t0*-q[2]+v[1]*q[0]-v[2]*q[1]+v[0]*q[3];
+	v[2]= t0*-q[3]+v[2]*q[0]-v[0]*q[2]+v[1]*q[1];
+	v[0]=t1; 
+	v[1]=t2;
+}
+
+void QuatConj(float *q)
+{
+	q[1] = -q[1];
+	q[2] = -q[2];
+	q[3] = -q[3];
+}
+
+float QuatDot(float *q1, float *q2)
+{
+	return q1[0]*q2[0] + q1[1]*q2[1] + q1[2]*q2[2] + q1[3]*q2[3];
+}
+
+void QuatInv(float *q)
+{
+	float f = QuatDot(q, q);
+
+	if (f == 0.0f)
+		return;
+
+	QuatConj(q);
+	QuatMulf(q, 1.0f/f);
+}
+
+void QuatMulf(float *q, float f)
+{
+	q[0] *= f;
+	q[1] *= f;
+	q[2] *= f;
+	q[3] *= f;
+}
+
 void QuatSub(float *q, float *q1, float *q2)
 {
 	q2[0]= -q2[0];
@@ -1391,6 +1449,7 @@ void VecUpMat3(float *vec, float mat[][3], short axis)
 }
 
 /* A & M Watt, Advanced animation and rendering techniques, 1992 ACM press */
+void QuatInterpolW(float *, float *, float *, float );
 
 void QuatInterpolW(float *result, float *quat1, float *quat2, float t)
 {
@@ -1712,6 +1771,11 @@ float VecLenf( float *v1, float *v2)
 	return (float)sqrt(x*x+y*y+z*z);
 }
 
+float VecLength(float *v)
+{
+	return (float) sqrt(v[0]*v[0] + v[1]*v[1] + v[2]*v[2]);
+}
+
 void VecAddf(float *v, float *v1, float *v2)
 {
 	v[0]= v1[0]+ v2[0];
@@ -1726,6 +1790,15 @@ void VecSubf(float *v, float *v1, float *v2)
 	v[2]= v1[2]- v2[2];
 }
 
+void VecLerpf(float *target, float *a, float *b, float t)
+{
+	float s = 1.0f-t;
+
+	target[0]= s*a[0] + t*b[0];
+	target[1]= s*a[1] + t*b[1];
+	target[2]= s*a[2] + t*b[2];
+}
+
 void VecMidf(float *v, float *v1, float *v2)
 {
 	v[0]= 0.5f*(v1[0]+ v2[0]);
@@ -2097,9 +2170,24 @@ void Mat3ToEul(float tmat[][3], float *eul)
 	cy = (float)sqrt(mat[0][0]*mat[0][0] + mat[0][1]*mat[0][1]);
 
 	if (cy > 16.0*FLT_EPSILON) {
-		eul[0] = (float)atan2(mat[1][2], mat[2][2]);
-		eul[1] = (float)atan2(-mat[0][2], cy);
-		eul[2] = (float)atan2(mat[0][1], mat[0][0]);
+		float eul1[3], eul2[3];
+		
+		eul1[0] = (float)atan2(mat[1][2], mat[2][2]);
+		eul1[1] = (float)atan2(-mat[0][2], cy);
+		eul1[2] = (float)atan2(mat[0][1], mat[0][0]);
+		
+		eul2[0] = (float)atan2(-mat[1][2], -mat[2][2]);
+		eul2[1] = (float)atan2(-mat[0][2], -cy);
+		eul2[2] = (float)atan2(-mat[0][1], -mat[0][0]);
+		
+		/* return best, which is just the one with lowest values it in */
+		if( fabs(eul1[0])+fabs(eul1[1])+fabs(eul1[2]) > fabs(eul2[0])+fabs(eul2[1])+fabs(eul2[2])) {
+			VecCopyf(eul, eul2);
+		}
+		else {
+			VecCopyf(eul, eul1);
+		}
+		
 	} else {
 		eul[0] = (float)atan2(-mat[2][1], mat[1][1]);
 		eul[1] = (float)atan2(-mat[0][2], cy);
@@ -2205,6 +2293,20 @@ void VecRotToQuat( float *vec, float phi, float *quat)
 	}
 }
 
+/* Return the angle in degrees between vecs 1-2 and 2-3 */
+float VecAngle3( float *v1, float *v2, float *v3)
+{
+	float vec1[3], vec2[3];
+
+	VecSubf(vec1, v2, v1);
+	VecSubf(vec2, v2, v3);
+	Normalise(vec1);
+	Normalise(vec2);
+	
+	return saacos(vec1[0]*vec2[0] + vec1[1]*vec2[1] + vec1[2]*vec2[2]) * 180.0/M_PI;
+}
+
+
 void euler_rot(float *beul, float ang, char axis)
 {
 	float eul[3], mat1[3][3], mat2[3][3], totmat[3][3];
@@ -2242,7 +2344,6 @@ void Mat3ToSize( float mat[][3], float *size)
 {
 	float vec[3];
 
-
 	VecCopyf(vec, mat[0]);
 	size[0]= Normalise(vec);
 	VecCopyf(vec, mat[1]);
@@ -2342,6 +2443,41 @@ void Vec2Addf(float *v, float *v1, float *v2)
 	v[1]= v1[1]+ v2[1];
 }
 
+void Vec2Subf(float *v, float *v1, float *v2)
+{
+	v[0]= v1[0]- v2[0];
+	v[1]= v1[1]- v2[1];
+}
+
+void Vec2Copyf(float *v1, float *v2)
+{
+	v1[0]= v2[0];
+	v1[1]= v2[1];
+}
+
+float Inp2f(float *v1, float *v2)
+{
+	return v1[0]*v2[0]+v1[1]*v2[1];
+}
+
+float Normalise2(float *n)
+{
+	float d;
+	
+	d= n[0]*n[0]+n[1]*n[1];
+
+	if(d>1.0e-35F) {
+		d= (float)sqrt(d);
+
+		n[0]/=d; 
+		n[1]/=d; 
+	} else {
+		n[0]=n[1]= 0.0;
+		d= 0.0;
+	}
+	return d;
+}
+
 void hsv_to_rgb(float h, float s, float v, float *r, float *g, float *b)
 {
 	int i;
@@ -2399,6 +2535,19 @@ void hsv_to_rgb(float h, float s, float v, float *r, float *g, float *b)
 	}
 }
 
+void hex_to_rgb(char *hexcol, float *r, float *g, float *b)
+{
+	unsigned int ri, gi, bi;
+	
+	if (hexcol[0] == '#') hexcol++;
+	
+	if (sscanf(hexcol, "%02x%02x%02x", &ri, &gi, &bi)) {
+		*r = ri / 255.0;
+		*g = gi / 255.0;		
+		*b = bi / 255.0;
+	}
+}
+
 void rgb_to_hsv(float r, float g, float b, float *lh, float *ls, float *lv)
 {
 	float h, s, v;
@@ -2492,3 +2641,236 @@ void cpack_to_rgb(unsigned int col, float *r, float *g, float *b)
 	*b= (float)(((col)>>16)&0xFF);
 	*b /= 255.0f;
 }
+
+
+/* *************** PROJECTIONS ******************* */
+
+void tubemap(float x, float y, float z, float *u, float *v)
+{
+	float len;
+	
+	*v = (z + 1.0) / 2.0;
+	
+	len= sqrt(x*x+y*y);
+	if(len>0) {
+		*u = (1.0 - (atan2(x/len,y/len) / M_PI)) / 2.0;
+	}
+}
+
+/* ------------------------------------------------------------------------- */
+
+void spheremap(float x, float y, float z, float *u, float *v)
+{
+	float len;
+	
+	len= sqrt(x*x+y*y+z*z);
+	if(len>0.0) {
+		
+		if(x==0.0 && y==0.0) *u= 0.0;	/* othwise domain error */
+		else *u = (1.0 - atan2(x,y)/M_PI )/2.0;
+		
+		z/=len;
+		*v = 1.0- saacos(z)/M_PI;
+	}
+}
+
+/* ------------------------------------------------------------------------- */
+
+/* *****************  m1 = m2 *****************  */
+void cpy_m3_m3(float m1[][3], float m2[][3]) 
+{	
+	memcpy(m1[0], m2[0], 9*sizeof(float));
+}
+
+/* *****************  m1 = m2 *****************  */
+void cpy_m4_m4(float m1[][4], float m2[][4]) 
+{	
+	memcpy(m1[0], m2[0], 16*sizeof(float));
+}
+
+/* ***************** identity matrix *****************  */
+void ident_m4(float m[][4])
+{
+	
+	m[0][0]= m[1][1]= m[2][2]= m[3][3]= 1.0;
+	m[0][1]= m[0][2]= m[0][3]= 0.0;
+	m[1][0]= m[1][2]= m[1][3]= 0.0;
+	m[2][0]= m[2][1]= m[2][3]= 0.0;
+	m[3][0]= m[3][1]= m[3][2]= 0.0;
+}
+
+
+/* *****************  m1 = m2 (pre) * m3 (post) ***************** */
+void mul_m3_m3m3(float m1[][3], float m2[][3], float m3[][3])
+{
+	float m[3][3];
+	
+	m[0][0]= m2[0][0]*m3[0][0] + m2[1][0]*m3[0][1] + m2[2][0]*m3[0][2]; 
+	m[0][1]= m2[0][1]*m3[0][0] + m2[1][1]*m3[0][1] + m2[2][1]*m3[0][2]; 
+	m[0][2]= m2[0][2]*m3[0][0] + m2[1][2]*m3[0][1] + m2[2][2]*m3[0][2]; 
+
+	m[1][0]= m2[0][0]*m3[1][0] + m2[1][0]*m3[1][1] + m2[2][0]*m3[1][2]; 
+	m[1][1]= m2[0][1]*m3[1][0] + m2[1][1]*m3[1][1] + m2[2][1]*m3[1][2]; 
+	m[1][2]= m2[0][2]*m3[1][0] + m2[1][2]*m3[1][1] + m2[2][2]*m3[1][2]; 
+
+	m[2][0]= m2[0][0]*m3[2][0] + m2[1][0]*m3[2][1] + m2[2][0]*m3[2][2]; 
+	m[2][1]= m2[0][1]*m3[2][0] + m2[1][1]*m3[2][1] + m2[2][1]*m3[2][2]; 
+	m[2][2]= m2[0][2]*m3[2][0] + m2[1][2]*m3[2][1] + m2[2][2]*m3[2][2]; 
+
+	cpy_m3_m3(m1, m2);
+}
+
+/*  ***************** m1 = m2 (pre) * m3 (post) ***************** */
+void mul_m4_m4m4(float m1[][4], float m2[][4], float m3[][4])
+{
+	float m[4][4];
+	
+	m[0][0]= m2[0][0]*m3[0][0] + m2[1][0]*m3[0][1] + m2[2][0]*m3[0][2] + m2[3][0]*m3[0][3]; 
+	m[0][1]= m2[0][1]*m3[0][0] + m2[1][1]*m3[0][1] + m2[2][1]*m3[0][2] + m2[3][1]*m3[0][3]; 
+	m[0][2]= m2[0][2]*m3[0][0] + m2[1][2]*m3[0][1] + m2[2][2]*m3[0][2] + m2[3][2]*m3[0][3]; 
+	m[0][3]= m2[0][3]*m3[0][0] + m2[1][3]*m3[0][1] + m2[2][3]*m3[0][2] + m2[3][3]*m3[0][3]; 
+
+	m[1][0]= m2[0][0]*m3[1][0] + m2[1][0]*m3[1][1] + m2[2][0]*m3[1][2] + m2[3][0]*m3[1][3]; 
+	m[1][1]= m2[0][1]*m3[1][0] + m2[1][1]*m3[1][1] + m2[2][1]*m3[1][2] + m2[3][1]*m3[1][3]; 
+	m[1][2]= m2[0][2]*m3[1][0] + m2[1][2]*m3[1][1] + m2[2][2]*m3[1][2] + m2[3][2]*m3[1][3]; 
+	m[1][3]= m2[0][3]*m3[1][0] + m2[1][3]*m3[1][1] + m2[2][3]*m3[1][2] + m2[3][3]*m3[1][3]; 
+
+	m[2][0]= m2[0][0]*m3[2][0] + m2[1][0]*m3[2][1] + m2[2][0]*m3[2][2] + m2[3][0]*m3[2][3]; 
+	m[2][1]= m2[0][1]*m3[2][0] + m2[1][1]*m3[2][1] + m2[2][1]*m3[2][2] + m2[3][1]*m3[2][3]; 
+	m[2][2]= m2[0][2]*m3[2][0] + m2[1][2]*m3[2][1] + m2[2][2]*m3[2][2] + m2[3][2]*m3[2][3]; 
+	m[2][3]= m2[0][3]*m3[2][0] + m2[1][3]*m3[2][1] + m2[2][3]*m3[2][2] + m2[3][3]*m3[2][3]; 
+	
+	m[3][0]= m2[0][0]*m3[3][0] + m2[1][0]*m3[3][1] + m2[2][0]*m3[3][2] + m2[3][0]*m3[3][3]; 
+	m[3][1]= m2[0][1]*m3[3][0] + m2[1][1]*m3[3][1] + m2[2][1]*m3[3][2] + m2[3][1]*m3[3][3]; 
+	m[3][2]= m2[0][2]*m3[3][0] + m2[1][2]*m3[3][1] + m2[2][2]*m3[3][2] + m2[3][2]*m3[3][3]; 
+	m[3][3]= m2[0][3]*m3[3][0] + m2[1][3]*m3[3][1] + m2[2][3]*m3[3][2] + m2[3][3]*m3[3][3]; 
+	
+	cpy_m4_m4(m1, m2);
+}
+
+/*  ***************** m1 = inverse(m2)  *****************  */
+void inv_m3_m3(float m1[][3], float m2[][3])
+{
+	short a,b;
+	float det;
+	
+	/* calc adjoint */
+	Mat3Adj(m1, m2);
+	
+	/* then determinant old matrix! */
+	det= m2[0][0]* (m2[1][1]*m2[2][2] - m2[1][2]*m2[2][1])
+	    -m2[1][0]* (m2[0][1]*m2[2][2] - m2[0][2]*m2[2][1])
+	    +m2[2][0]* (m2[0][1]*m2[1][2] - m2[0][2]*m2[1][1]);
+	
+	if(det==0.0f) det=1.0f;
+	det= 1.0f/det;
+	for(a=0;a<3;a++) {
+		for(b=0;b<3;b++) {
+			m1[a][b]*=det;
+		}
+	}
+}
+
+/*  ***************** m1 = inverse(m2)  *****************  */
+int inv_m4_m4(float inverse[][4], float mat[][4])
+{
+	int i, j, k;
+	double temp;
+	float tempmat[4][4];
+	float max;
+	int maxj;
+	
+	/* Set inverse to identity */
+	ident_m4(inverse);
+	
+	/* Copy original matrix so we don't mess it up */
+	cpy_m4_m4(tempmat, mat);
+	
+	for(i = 0; i < 4; i++) {
+		/* Look for row with max pivot */
+		max = ABS(tempmat[i][i]);
+		maxj = i;
+		for(j = i + 1; j < 4; j++) {
+			if(ABS(tempmat[j][i]) > max) {
+				max = ABS(tempmat[j][i]);
+				maxj = j;
+			}
+		}
+		/* Swap rows if necessary */
+		if (maxj != i) {
+			for( k = 0; k < 4; k++) {
+				SWAP(float, tempmat[i][k], tempmat[maxj][k]);
+				SWAP(float, inverse[i][k], inverse[maxj][k]);
+			}
+		}
+		
+		temp = tempmat[i][i];
+		if (temp == 0)
+			return 0;  /* No non-zero pivot */
+		for(k = 0; k < 4; k++) {
+			tempmat[i][k] = (float)(tempmat[i][k]/temp);
+			inverse[i][k] = (float)(inverse[i][k]/temp);
+		}
+		for(j = 0; j < 4; j++) {
+			if(j != i) {
+				temp = tempmat[j][i];
+				for(k = 0; k < 4; k++) {
+					tempmat[j][k] -= (float)(tempmat[i][k]*temp);
+					inverse[j][k] -= (float)(inverse[i][k]*temp);
+				}
+			}
+		}
+	}
+	return 1;
+}
+
+/*  ***************** v1 = v2 * mat  ***************** */
+void mul_v3_v3m4(float *v1, float *v2, float mat[][4])
+{
+	float x, y;
+	
+	x= v2[0];	// work with a copy, v1 can be same as v2
+	y= v2[1];
+	v1[0]= x*mat[0][0] + y*mat[1][0] + mat[2][0]*v2[2] + mat[3][0];
+	v1[1]= x*mat[0][1] + y*mat[1][1] + mat[2][1]*v2[2] + mat[3][1];
+	v1[2]= x*mat[0][2] + y*mat[1][2] + mat[2][2]*v2[2] + mat[3][2];
+	
+}
+
+/* moved from effect.c
+   test if the line starting at p1 ending at p2 intersects the triangle v0..v2
+   return non zero if it does 
+
+   wild guess .. says did not check the math since i don't need it
+   and if (it intersects) lambda returns intersection = p1 + lambda(p1-p2)
+*/
+int LineIntersectsTriangle(float p1[3], float p2[3], float v0[3], float v1[3], float v2[3], float *lambda)
+{
+
+	float p[3], s[3], d[3], e1[3], e2[3], q[3];
+	float a, f, u, v;
+	
+	VecSubf(e1, v1, v0);
+	VecSubf(e2, v2, v0);
+	VecSubf(d, p2, p1);
+	
+	Crossf(p, d, e2);
+	a = Inpf(e1, p);
+	if ((a > -0.000001) && (a < 0.000001)) return 0;
+	f = 1.0f/a;
+	
+	VecSubf(s, p1, v0);
+	
+	Crossf(q, s, e1);
+	*lambda = f * Inpf(e2, q);
+	if ((*lambda < 0.0)||(*lambda > 1.0)) return 0;
+	
+	u = f * Inpf(s, p);
+	if ((u < 0.0)||(u > 1.0)) return 0;
+	
+	v = f * Inpf(d, q);
+	if ((v < 0.0)||((u + v) > 1.0)) return 0;
+	
+	return 1;
+}
+
