@@ -1,5 +1,5 @@
 /**
- * $Id: storage.c,v 1.25 2005/06/04 16:22:49 lukep Exp $
+ * $Id: storage.c,v 1.31 2005/07/27 20:16:37 zuster Exp $
  *
  * ***** BEGIN GPL/BL DUAL LICENSE BLOCK *****
  *
@@ -393,7 +393,7 @@ void BLI_adddirstrings()
 		else if(num1) sprintf(files[num].size, "%15d", num1);
 		else sprintf(files[num].size, "0");
 
-		strftime(datum, 32, "%d-%b-%y %R", tm);
+		strftime(datum, 32, "%d-%b-%y %H:%M", tm);
 
 		if (st_size < 1000) {
 			sprintf(size, "%10d", (int) st_size);
