@@ -3,7 +3,7 @@
  * various string, file, list operations.
  *
  *
- * $Id: util.c,v 1.26 2005/06/10 13:12:59 jesterking Exp $
+ * $Id: util.c,v 1.32 2005/12/11 22:03:04 ton Exp $
  *
  * ***** BEGIN GPL/BL DUAL LICENSE BLOCK *****
  *
@@ -121,6 +121,11 @@ int BLI_stringdec(char *string, char *kop, char *staart, unsigned short *numlen)
 		else if (BLI_strncasecmp(string + len - 6, ".trace", 6) == 0) len -= 6;
 	}
 	
+	if (len > 9) {
+		if (BLI_strncasecmp(string + len - 9, ".blend.gz", 9) == 0) len -= 9;
+	}
+		
+	
 	if (len == len2) {
 		if (len > 4) {
 			/* handle .jf0 en .jf1 for jstreams */
@@ -426,31 +431,71 @@ int BLI_strcaseeq(char *a, char *b) {
 	return (BLI_strcasecmp(a, b)==0);
 }
 
-void BLI_makestringcode(char *fromfile, char *str)
+void BLI_makestringcode(const char *relfile, char *file)
 {
-	char *slash, len, temp[512];
-
-	strcpy(temp, fromfile);
+	char * p;
+	char * q;
+	char * lslash;
+	int len=0;
+	char temp[FILE_MAXDIR+FILE_MAXFILE];
+	char res[FILE_MAXDIR+FILE_MAXFILE];
 	
-	BLI_char_switch(temp, '\\', '/');
-	BLI_char_switch(str, '\\', '/');
-
-	/* Find the last slash */
-	slash = strrchr(temp, '/');
-	if(slash) {
-		*(slash+1)= 0;
-		len= strlen(temp);
-		if(len) {
-			if(strncmp(str, temp, len)==0) {
-				temp[0]= '/';
-				temp[1]= '/';
-				strcpy(temp+2, str+len);
-#ifdef			WIN32
-				BLI_char_switch(temp+2, '/', '\\');
+	/* if file is already relative, bail out */
+	if(file[0]=='/' && file[1]=='/') return;
+	
+	strcpy(temp, relfile);
+	
+#ifdef WIN32
+	if (strlen(file) > 2) {
+		if ( temp[1] == ':' && file[1] == ':' && temp[0] != file[0] )
+			return;
+	}
 #endif
-				strcpy(str, temp);
-			}
+
+	BLI_char_switch(temp, '\\', '/');
+	BLI_char_switch(file, '\\', '/');
+
+	/* the last slash in the file indicates where the path part ends */
+	lslash = BLI_last_slash(temp);
+
+	if (lslash) 
+	{	
+		/* find the prefix of the filename that is equal for both filenames.
+		   This is replaced by the two slashes at the beginning */
+		p = temp;
+		q = file;
+		while (*p == *q) {
+			++p; ++q;
 		}
+		/* we might have passed the slash when the beginning of a dir matches 
+		   so we rewind. Only check on the actual filename
+		*/
+		if (*q != '/') {
+			while ( (q >= file) && (*q != '/') ) { --q; --p; }
+		} 
+		else if (*p != '/') {
+			while ( (p >= temp) && (*p != '/') ) { --p; --q; }
+		}
+		
+		strcpy(res,	"//");
+
+		/* p now points to the slash that is at the beginning of the part
+		   where the path is different from the relative path. 
+		   We count the number of directories we need to go up in the
+		   hierarchy to arrive at the common 'prefix' of the path
+		*/			
+		while (p && p < lslash)	{
+			if (*p == '/') 
+				strcat(res,	"../");
+			++p;
+		}
+
+		strcat(res, q+1); /* don't copy the slash at the beginning */
+
+#ifdef	WIN32
+		BLI_char_switch(res+2, '/', '\\');
+#endif
+		strcpy(file, res);
 	}
 }
 
@@ -645,7 +690,7 @@ void BLI_make_exist(char *dir) {
 #endif
 }
 
-void BLI_make_file_string(char *relabase, char *string,  char *dir,  char *file)
+void BLI_make_file_string(const char *relabase, char *string,  const char *dir, const char *file)
 {
 
 	if (!string || !dir || !file) return; /* We don't want any NULLs */
@@ -654,9 +699,12 @@ void BLI_make_file_string(char *relabase, char *string,  char *dir,  char *file)
 
 	/* we first push all slashes into unix mode, just to make sure we don't get
 	   any mess with slashes later on. -jesterKing */
+	/* constant strings can be passed for those parameters - don't change them - elubie */
+	/*
 	BLI_char_switch(relabase, '\\', '/');
 	BLI_char_switch(dir, '\\', '/');
 	BLI_char_switch(file, '\\', '/');
+	*/
 
 	/* Resolve relative references */	
 	if (relabase && dir[0] == '/' && dir[1] == '/') {
