@@ -1,5 +1,5 @@
 /**
- * $Id: readblenentry.c,v 1.8 2005/03/19 21:08:10 zuster Exp $
+ * $Id: readblenentry.c,v 1.10 2005/07/25 18:35:49 zuster Exp $
  *
  * ***** BEGIN GPL/BL DUAL LICENSE BLOCK *****
  *
@@ -85,7 +85,6 @@ static IDType idtypes[]= {
 	{ ID_CU,		"Curve",	IDTYPE_FLAGS_ISLINKABLE}, 
 	{ ID_GR,		"Group",	0}, 
 	{ ID_ID,		"ID",		0}, 
-	{ ID_IK,		"Ika",		IDTYPE_FLAGS_ISLINKABLE}, 
 	{ ID_IM,		"Image",	IDTYPE_FLAGS_ISLINKABLE}, 
 	{ ID_IP,		"Ipo",		IDTYPE_FLAGS_ISLINKABLE}, 
 	{ ID_KE,		"Key",		0}, 
@@ -164,7 +163,9 @@ int BLO_idcode_from_name(char *name)
 	 
 BlendHandle *BLO_blendhandle_from_file(char *file) 
 {
-	return (BlendHandle*) blo_openblenderfile(file);
+	BlendReadError err;
+
+	return (BlendHandle*) blo_openblenderfile(file, &err);
 }
 
 void BLO_blendhandle_print_sizes(BlendHandle *bh, void *fp) 
@@ -255,7 +256,7 @@ BlendFileData *BLO_read_from_file(char *file, BlendReadError *error_r)
 	BlendFileData *bfd = NULL;
 	FileData *fd;
 		
-	fd = blo_openblenderfile(file);
+	fd = blo_openblenderfile(file, error_r);
 	if (fd) {
 		bfd= blo_read_file_internal(fd, error_r);
 		if (bfd) {
@@ -273,7 +274,7 @@ BlendFileData *BLO_read_from_memory(void *mem, int memsize, BlendReadError *erro
 	BlendFileData *bfd = NULL;
 	FileData *fd;
 		
-	fd = blo_openblendermemory(mem, memsize);
+	fd = blo_openblendermemory(mem, memsize,  error_r);
 	if (fd) {
 		bfd= blo_read_file_internal(fd, error_r);
 		if (bfd) {
@@ -291,7 +292,7 @@ BlendFileData *BLO_read_from_memfile(MemFile *memfile, BlendReadError *error_r)
 	BlendFileData *bfd = NULL;
 	FileData *fd;
 		
-	fd = blo_openblendermemfile(memfile);
+	fd = blo_openblendermemfile(memfile, error_r);
 	if (fd) {
 		bfd= blo_read_file_internal(fd, error_r);
 		if (bfd) {
