@@ -2,7 +2,7 @@
  *
  * .blend file writing
  *
- * $Id: writefile.c,v 1.44 2005/05/08 21:20:32 ianwill Exp $
+ * $Id: writefile.c,v 1.60 2005/10/02 20:51:35 ton Exp $
  *
  * ***** BEGIN GPL/BL DUAL LICENSE BLOCK *****
  *
@@ -83,6 +83,8 @@ Important to know is that 'streaming' has been added to files, for Blender Publi
 #include <config.h>
 #endif
 
+#include "zlib.h"
+
 #ifndef WIN32
 #include <unistd.h>
 #else
@@ -110,7 +112,6 @@ Important to know is that 'streaming' has been added to files, for Blender Publi
 #include "DNA_effect_types.h"
 #include "DNA_group_types.h"
 #include "DNA_image_types.h"
-#include "DNA_ika_types.h"
 #include "DNA_ipo_types.h"
 #include "DNA_fileglobal_types.h"
 #include "DNA_key_types.h"
@@ -121,6 +122,7 @@ Important to know is that 'streaming' has been added to files, for Blender Publi
 #include "DNA_mesh_types.h"
 #include "DNA_meshdata_types.h"
 #include "DNA_material_types.h"
+#include "DNA_modifier_types.h"
 #include "DNA_nla_types.h"
 #include "DNA_object_types.h"
 #include "DNA_object_force.h"
@@ -156,6 +158,7 @@ Important to know is that 'streaming' has been added to files, for Blender Publi
 #include "BKE_scene.h" // for do_seq
 #include "BKE_sound.h" /* ... and for samples */
 #include "BKE_utildefines.h" // for defines
+#include "BKE_modifier.h"
 
 #include "GEN_messaging.h"
 
@@ -166,6 +169,7 @@ Important to know is that 'streaming' has been added to files, for Blender Publi
 #include "readfile.h"
 #include "genfile.h"
 
+#include <errno.h>
 
 /* ********* my write, buffered writing with minimum 50k chunks ************ */
 
@@ -402,15 +406,9 @@ static void write_effects(WriteData *wd, ListBase *lb)
 	while(eff) {
 
 		switch(eff->type) {
-		case EFF_BUILD:
-			writestruct(wd, DATA, "BuildEff", 1, eff);
-			break;
 		case EFF_PARTICLE:
 			writestruct(wd, DATA, "PartEff", 1, eff);
 			break;
-		case EFF_WAVE:
-			writestruct(wd, DATA, "WaveEff", 1, eff);
-			break;
 		default:
 			writedata(wd, DATA, MEM_allocN_len(eff), eff);
 		}
@@ -614,6 +612,9 @@ static void write_constraints(WriteData *wd, ListBase *conlist)
 		case CONSTRAINT_TYPE_STRETCHTO:
 			writestruct(wd, DATA, "bStretchToConstraint", 1, con->data);
 			break;
+		case CONSTRAINT_TYPE_MINMAX:
+			writestruct(wd, DATA, "bMinMaxConstraint", 1, con->data);
+			break;
 		default:
 			break;
 		}
@@ -658,10 +659,26 @@ static void write_constraint_channels(WriteData *wd, ListBase *chanbase)
 
 }
 
+static void write_modifiers(WriteData *wd, ListBase *modbase)
+{
+	ModifierData *md;
+
+	for (md=modbase->first; md; md= md->next) {
+		ModifierTypeInfo *mti = modifierType_getInfo(md->type);
+
+		writestruct(wd, DATA, mti->structName, 1, md);
+
+		if (md->type==eModifierType_Hook) {
+			HookModifierData *hmd = (HookModifierData*) md;
+
+			writedata(wd, DATA, sizeof(int)*hmd->totindex, hmd->indexar);
+		}
+	}
+}
+
 static void write_objects(WriteData *wd, ListBase *idbase)
 {
 	Object *ob;
-	ObHook *hook;
 	int a;
 	
 	ob= idbase->first;
@@ -695,11 +712,9 @@ static void write_objects(WriteData *wd, ListBase *idbase)
 					}
 				}
 			}
+			writestruct(wd, DATA, "FluidsimSettings", 1, ob->fluidsimSettings); // NT
 			
-			for(hook= ob->hooks.first; hook; hook= hook->next) {
-				writestruct(wd, DATA, "ObHook", 1, hook);
-				writedata(wd, DATA, sizeof(int)*hook->totindex, hook->indexar);
-			}
+			write_modifiers(wd, &ob->modifiers);
 		}
 		ob= ob->id.next;
 	}
@@ -755,6 +770,7 @@ static void write_ipos(WriteData *wd, ListBase *idbase)
 			while(icu) {
 				if(icu->bezt)  writestruct(wd, DATA, "BezTriple", icu->totvert, icu->bezt);
 				if(icu->bp)  writestruct(wd, DATA, "BPoint", icu->totvert, icu->bp);
+				if(icu->driver)  writestruct(wd, DATA, "IpoDriver", 1, icu->driver);
 				icu= icu->next;
 			}
 		}
@@ -834,6 +850,13 @@ static void write_mballs(WriteData *wd, ListBase *idbase)
 	}
 }
 
+int amount_of_chars(char *str)
+{
+	// Since the data is saved as UTF-8 to the cu->str
+	// The cu->len is not same as the strlen(cu->str)
+	return strlen(str);
+}
+
 static void write_curves(WriteData *wd, ListBase *idbase)
 {
 	Curve *cu;
@@ -849,7 +872,9 @@ static void write_curves(WriteData *wd, ListBase *idbase)
 			writedata(wd, DATA, sizeof(void *)*cu->totcol, cu->mat);
 
 			if(cu->vfont) {
-				writedata(wd, DATA, cu->len+1, cu->str);
+				writedata(wd, DATA, amount_of_chars(cu->str)+1, cu->str);
+				writestruct(wd, DATA, "CharInfo", cu->len, cu->strinfo);
+				writestruct(wd, DATA, "TextBox", cu->totbox, cu->tb);				
 			}
 			else {
 				/* is also the order of reading */
@@ -1050,30 +1075,6 @@ static void write_lattices(WriteData *wd, ListBase *idbase)
 	}
 }
 
-static void write_ikas(WriteData *wd, ListBase *idbase)
-{
-	Ika *ika;
-	Limb *li;
-
-	ika= idbase->first;
-	while(ika) {
-		if(ika->id.us>0 || wd->current) {
-			/* write LibData */
-			writestruct(wd, ID_IK, "Ika", 1, ika);
-
-			/* direct data */
-			li= ika->limbbase.first;
-			while(li) {
-				writestruct(wd, DATA, "Limb", 1, li);
-				li= li->next;
-			}
-
-			writestruct(wd, DATA, "Deform", ika->totdef, ika->def);
-		}
-		ika= ika->id.next;
-	}
-}
-
 static void write_scenes(WriteData *wd, ListBase *scebase)
 {
 	Scene *sce;
@@ -1097,7 +1098,8 @@ static void write_scenes(WriteData *wd, ListBase *scebase)
 		}
 
 		writestruct(wd, DATA, "Radio", 1, sce->radio);
-
+		writestruct(wd, DATA, "ToolSettings", 1, sce->toolsettings);
+		
 		ed= sce->ed;
 		if(ed) {
 			writestruct(wd, DATA, "Editing", 1, ed);
@@ -1218,6 +1220,7 @@ static void write_screens(WriteData *wd, ListBase *scrbase)
 					writestruct(wd, DATA, "View3D", 1, v3d);
 					if(v3d->bgpic) writestruct(wd, DATA, "BGpic", 1, v3d->bgpic);
 					if(v3d->localvd) writestruct(wd, DATA, "View3D", 1, v3d->localvd);
+					if(v3d->clipbb) writestruct(wd, DATA, "BoundBox", 1, v3d->clipbb);
 				}
 				else if(sl->spacetype==SPACE_IPO) {
 					writestruct(wd, DATA, "SpaceIpo", 1, sl);
@@ -1342,11 +1345,12 @@ static void write_bone(WriteData *wd, Bone* bone)
 {
 	Bone*	cbone;
 
-//	write_constraints(wd, &bone->constraints);
-
+	// PATCH for upward compatibility after 2.37+ armature recode
+	bone->size[0]= bone->size[1]= bone->size[2]= 1.0f;
+		
 	// Write this bone
 	writestruct(wd, DATA, "Bone", 1, bone);
-
+	
 	// Write Children
 	cbone= bone->childbase.first;
 	while(cbone) {
@@ -1563,7 +1567,6 @@ static int write_file_handle(int handle, MemFile *compare, MemFile *current, int
 	write_cameras  (wd, &G.main->camera);
 	write_lamps    (wd, &G.main->lamp);
 	write_lattices (wd, &G.main->latt);
-	write_ikas     (wd, &G.main->ika);
 	write_vfonts   (wd, &G.main->vfont);
 	write_ipos     (wd, &G.main->ipo);
 	write_keys     (wd, &G.main->key);
@@ -1621,10 +1624,28 @@ int BLO_write_file(char *dir, int write_flags, char **error_r)
 	close(file);
 
 	if(!fout) {
+		if(write_flags & G_FILE_COMPRESS)
+		{	
+			// compressed files have the same ending as regular files... only from 2.4!!!
+			
+			int ret = BLI_gzip(tempname, dir);
+			
+			if(-1==ret) {
+				*error_r= "Failed opening .gz file";
+				return 0;
+			}
+			if(-2==ret) {
+				*error_r= "Failed opening .blend file for compression";
+				return 0;
+			}
+		}
+		else
 		if(BLI_rename(tempname, dir) < 0) {
 			*error_r= "Can't change old file. File saved with @";
 			return 0;
 		}
+
+		
 	} else {
 		remove(tempname);
 
