@@ -1,7 +1,7 @@
 /**
  * amiga.c
  *
- * $Id: amiga.c,v 1.12 2005/03/20 20:48:37 zuster Exp $
+ * $Id: amiga.c,v 1.13 2005/09/19 00:12:43 intrr Exp $
  *
  * ***** BEGIN GPL/BL DUAL LICENSE BLOCK *****
  *
@@ -41,6 +41,8 @@
 #include "IMB_imbuf_types.h"
 #include "IMB_imbuf.h"
 
+#include "BKE_global.h"
+
 #include "IMB_cmap.h"
 #include "IMB_allocimbuf.h"
 #include "IMB_bitplanes.h"
@@ -529,7 +531,7 @@ struct ImBuf *imb_loadamiga(int *iffmem,int flags)
 
 	if (ibuf) {
 		if (ibuf->rect) 
-			IMB_convert_rgba_to_abgr(ibuf->x*ibuf->y, ibuf->rect);
+			if (G.order == B_ENDIAN) IMB_convert_rgba_to_abgr(ibuf->x*ibuf->y, ibuf->rect);
 	}
 	
 	return (ibuf);
