@@ -29,7 +29,7 @@
  * ***** END GPL/BL DUAL LICENSE BLOCK *****
  * allocimbuf.c
  *
- * $Id: readimage.c,v 1.13 2005/04/27 11:52:50 sirdude Exp $
+ * $Id: readimage.c,v 1.15 2005/11/22 18:50:03 sirdude Exp $
  */
 
 #ifdef WIN32
@@ -50,6 +50,8 @@
 #include "IMB_hamx.h"
 #include "IMB_jpeg.h"
 #include "IMB_bmp.h"
+#include "IMB_tiff.h"
+#include "IMB_radiance_hdr.h"
 #include "BKE_global.h"
 
 #ifdef WITH_QUICKTIME
@@ -128,6 +130,14 @@ ImBuf *IMB_ibImageFromMemory(int *mem, int size, int flags) {
 		ibuf = imb_loadtarga((uchar *)mem, flags);
 		if (ibuf) return(ibuf);
 
+		if (G.have_libtiff) {
+			ibuf = imb_loadtiff((uchar *)mem, size, flags);
+			if (ibuf) return(ibuf);
+		}
+
+		ibuf = imb_loadhdr((uchar*)mem, size, flags);
+		if (ibuf) return (ibuf);
+
 #ifdef WITH_QUICKTIME
 #if defined(_WIN32) || defined (__APPLE__)
 		if(G.have_quicktime) {
