@@ -29,7 +29,7 @@
  * ***** END GPL/BL DUAL LICENSE BLOCK *****
  * util.c
  *
- * $Id: util.c,v 1.12 2005/03/23 21:10:02 phase Exp $
+ * $Id: util.c,v 1.14 2005/11/22 18:50:03 sirdude Exp $
  */
 
 #include "BLI_blenlib.h"
@@ -45,6 +45,8 @@
 #include "IMB_targa.h"
 #include "IMB_png.h"
 #include "IMB_bmp.h"
+#include "IMB_tiff.h"
+#include "IMB_radiance_hdr.h"
 
 #include "IMB_anim.h"
 
@@ -95,6 +97,12 @@ static int IMB_ispic_name(char *name)
 				}
 				if (imb_is_a_png(buf)) return(PNG);
 				if (imb_is_a_targa(buf)) return(TGA);
+
+				if (imb_is_a_tiff(buf)) return(TIF);
+
+				/* radhdr: check if hdr format */
+				if (imb_is_a_hdr(buf)) return(RADHDR);
+
 /*
 				if (imb_is_a_bmp(buf)) return(BMP);
 */
@@ -120,9 +128,16 @@ static int IMB_ispic_name(char *name)
 int IMB_ispic(char *filename)
 {
 	if(U.uiflag & USER_FILTERFILEEXTS) {
+		if (G.have_libtiff && (BLI_testextensie(filename, ".tif")
+				||	BLI_testextensie(filename, ".tiff"))) {
+				return IMB_ispic_name(filename);
+		}
 		if (G.have_quicktime){
 			if(		BLI_testextensie(filename, ".jpg")
 				||	BLI_testextensie(filename, ".jpeg")
+				||	BLI_testextensie(filename, ".tif")
+				||	BLI_testextensie(filename, ".tiff")
+				||	BLI_testextensie(filename, ".hdr")
 				||	BLI_testextensie(filename, ".tga")
 				||	BLI_testextensie(filename, ".rgb")
 				||	BLI_testextensie(filename, ".bmp")
@@ -131,8 +146,6 @@ int IMB_ispic(char *filename)
 				||	BLI_testextensie(filename, ".lbm")
 				||	BLI_testextensie(filename, ".gif")
 				||	BLI_testextensie(filename, ".psd")
-				||	BLI_testextensie(filename, ".tif")
-				||	BLI_testextensie(filename, ".tiff")
 				||	BLI_testextensie(filename, ".pct")
 				||	BLI_testextensie(filename, ".pict")
 				||	BLI_testextensie(filename, ".pntg") //macpaint
@@ -142,9 +155,10 @@ int IMB_ispic(char *filename)
 			} else {
 				return(FALSE);			
 			}
-		} else { // no quicktime
+		} else { /* no quicktime or libtiff */
 			if(		BLI_testextensie(filename, ".jpg")
 				||	BLI_testextensie(filename, ".jpeg")
+				||	BLI_testextensie(filename, ".hdr")
 				||	BLI_testextensie(filename, ".tga")
 				||	BLI_testextensie(filename, ".rgb")
 				||	BLI_testextensie(filename, ".bmp")
@@ -158,7 +172,7 @@ int IMB_ispic(char *filename)
 				return(FALSE);
 			}
 		}
-	} else { // no FILTERFILEEXTS
+	} else { /* no FILTERFILEEXTS */
 		return IMB_ispic_name(filename);
 	}
 }
