@@ -1,5 +1,5 @@
 /**
- * $Id: BDR_drawobject.h,v 1.15 2005/04/20 08:26:22 broken Exp $
+ * $Id: BDR_drawobject.h,v 1.20 2005/11/28 16:59:11 ton Exp $
  *
  * ***** BEGIN GPL/BL DUAL LICENSE BLOCK *****
  *
@@ -44,20 +44,27 @@ struct Lamp;
 struct ListBase;
 struct BoundBox;
 struct Base;
+struct BPoint;
+struct BezTriple;
+struct EditVert;
+struct EditFace;
 
-void init_draw_rects(void);
-void helpline(float *vec);
-void drawaxes(float size);
-void drawcamera(struct Object *ob);
+void mesh_foreachScreenVert(void (*func)(void *userData, struct EditVert *eve, int x, int y, int index), void *userData, int clipVerts);
+void mesh_foreachScreenEdge(void (*func)(void *userData, struct EditEdge *eed, int x0, int y0, int x1, int y1, int index), void *userData, int clipVerts);
+void mesh_foreachScreenFace(void (*func)(void *userData, struct EditFace *efa, int x, int y, int index), void *userData);
+
+void lattice_foreachScreenVert(void (*func)(void *userData, struct BPoint *bp, int x, int y), void *userData);
+void nurbs_foreachScreenVert(void (*func)(void *userData, struct Nurb *nu, struct BPoint *bp, struct BezTriple *bezt, int beztindex, int x, int y), void *userData);
 
-void calc_mesh_facedots_ext(void);
-void calc_lattverts_ext(void);
-void calc_meshverts_ext(void);
-void calc_meshverts_ext_f2(void);
-void calc_nurbverts_ext(void);
 void drawcircball(int mode, float *cent, float rad, float tmat[][4]);
 void get_local_bounds(struct Object *ob, float *centre, float *size);
-void draw_object(struct Base *base);
+
+/* drawing flags: */
+#define DRAW_PICKING	1
+#define DRAW_CONSTCOLOR	2
+void draw_object(struct Base *base, int flag);
+void drawaxes(float size, int flag);
+
 void draw_object_ext(struct Base *base);
 void drawsolidcube(float size);
 extern void draw_object_backbufsel(struct Object *ob);
