@@ -33,14 +33,17 @@
 #ifndef BIF_MESHTOOLS_H
 #define BIF_MESHTOOLS_H
 
-extern void join_mesh(void);
+struct Object;
+
+extern int join_mesh(void);
 
 extern void fasterdraw(void);
 extern void slowerdraw(void);
 
-extern void vertexnormals_mesh(Mesh *me, float *extverts);
 extern void sort_faces(void);
 
+extern int mesh_octree_table(struct Object *ob, float *co, char mode);
+extern int mesh_get_x_mirror_vert(struct Object *ob, int index);
 
 #endif
 
