@@ -1,5 +1,5 @@
 /**
- * $Id: BIF_space.h,v 1.32 2005/03/23 21:10:02 phase Exp $
+ * $Id: BIF_space.h,v 1.35 2005/12/30 14:17:15 campbellbarton Exp $
  *
  * ***** BEGIN GPL/BL DUAL LICENSE BLOCK *****
  *
@@ -42,7 +42,6 @@ struct SpaceOops;
 
 #define REMAKEIPO		1
 #define OOPS_TEST		2
-#define REMAKEALLIPO	3	/* Reevan's ipo fixing test */
 
 #define BUT_HORIZONTAL 	1
 #define BUT_VERTICAL 	2
@@ -71,6 +70,18 @@ struct SpaceOops;
 /* sequence handler codes */
 #define SEQ_HANDLER_PROPERTIES	60
 
+/* theme codes */
+#define B_ADD_THEME 	3301
+#define B_DEL_THEME 	3302
+#define B_NAME_THEME 	3303
+#define B_THEMECOL 		3304
+#define B_UPDATE_THEME 	3305
+#define B_CHANGE_THEME 	3306
+#define B_THEME_COPY 	3307
+#define B_THEME_PASTE 	3308
+
+#define B_RECALCLIGHT 	3310
+
 
 void	scrarea_do_windraw		(struct ScrArea *sa);
 void	scrarea_do_winchange	(struct ScrArea *sa);
@@ -104,6 +115,7 @@ extern       void set_rects_butspace(struct SpaceButs *buts);
 extern       void test_butspace(void);
 extern       void start_game(void);
 extern		 void select_group_menu(void);
+extern		 void join_menu(void);
 extern		 void select_group(short nr);
 
 extern 		void BIF_undo_push(char *str);
