@@ -1,5 +1,5 @@
 /**
- * $Id: BSE_drawview.h,v 1.11 2005/05/11 20:01:42 ton Exp $
+ * $Id: BSE_drawview.h,v 1.13 2005/08/20 19:18:34 ton Exp $
  *
  * ***** BEGIN GPL/BL DUAL LICENSE BLOCK *****
  *
@@ -43,9 +43,12 @@ void default_gl_light(void);
 void init_gl_stuff(void);
 void circf(float x, float y, float rad);
 void circ(float x, float y, float rad);
-void backdrawview3d(int test);
 
 void do_viewbuts(unsigned short event);
+
+void add_view3d_after(struct View3D *v3d, struct Base *base, int type);
+
+void backdrawview3d(int test);
 void drawview3dspace(struct ScrArea *sa, void *spacedata);
 void drawview3d_render(struct View3D *v3d);
 
@@ -54,6 +57,10 @@ void calc_viewborder(struct View3D *v3d, struct rcti *viewborder_r);
 void view3d_set_1_to_1_viewborder(struct View3D *v3d);
 void timestr(double time, char *str);
 
+int view3d_test_clipping(struct View3D *v3d, float *vec);
+void view3d_set_clipping(struct View3D *v3d);
+void view3d_clr_clipping(void);
+
 void sumo_callback(void *obp);
 void init_anim_sumo(void);
 void update_anim_sumo(void);
@@ -62,7 +69,6 @@ void end_anim_sumo(void);
 void inner_play_anim_loop(int init, int mode);
 int play_anim(int mode);
 
-
 void make_axis_color(char *col, char *col2, char axis);
 
 #endif /* BSE_DRAWVIEW_H */
