@@ -9,7 +9,7 @@
  *   mouse / timer / window: until 0x020
  *   custom codes: 0x4...
  * 
- * $Id: mydevice.h,v 1.19 2005/05/11 20:01:42 ton Exp $
+ * $Id: mydevice.h,v 1.20 2005/11/20 21:29:09 ton Exp $
  *
  * ***** BEGIN GPL/BL DUAL LICENSE BLOCK *****
  *
@@ -189,8 +189,10 @@
 #define COMMANDKEY	172
 #define GRLESSKEY	173
 
-/* used as fake leftmouse event, special handled in interface.c */
+/* used as fake leftmouse events, special handled in interface.c */
 #define BUT_ACTIVATE	200
+#define BUT_NEXT		201
+#define BUT_PREV		202
 
 /* **************** BLENDER QUEUE EVENTS ********************* */
 
