@@ -3,7 +3,7 @@
  *	
  * Object is a sort of wrapper for general info.
  *
- * $Id: DNA_object_types.h,v 1.29 2005/05/02 13:28:12 ton Exp $ 
+ * $Id: DNA_object_types.h,v 1.43 2005/11/17 18:01:07 ton Exp $ 
  *
  * ***** BEGIN GPL/BL DUAL LICENSE BLOCK *****
  *
@@ -53,15 +53,14 @@ struct Material;
 struct bConstraintChannel;
 struct PartDeflect;
 struct SoftBody;
+struct FluidsimSettings;
+struct DerivedMesh;
 
 typedef struct bDeformGroup {
 	struct bDeformGroup *next, *prev;
 	char name[32];
-	void *data;	/* Temporary data, most likely a pointer to the bone - no need to delete */
 } bDeformGroup;
 
-#
-#
 typedef struct BoundBox {
 	float vec[8][3];
 } BoundBox;
@@ -95,13 +94,13 @@ typedef struct Object {
 	struct bPose *pose;	
 	void *data;
 	
-	struct bConstraintChannel *activecon;		
 	ListBase constraintChannels;
-
 	ListBase effect;
 	ListBase network;
 	ListBase disp;
 	ListBase defbase;
+	ListBase modifiers; /* list of ModifierData structures */
+	
 	struct Material **mat;
 	
 	/* rot en drot have to be together! (transform('r' en 's')) */
@@ -114,11 +113,14 @@ typedef struct Object {
 	float imat[4][4];	/* for during render, old game engine, temporally: ipokeys of transform  */
 	
 	unsigned int lay;				/* copy of Base */
+	
 	short flag;			/* copy of Base */
 	short colbits;		/* when zero, from obdata */
-	char transflag, ipoflag;
-	char trackflag, upflag;
-	short ipowin, scaflag;	/* ipowin: blocktype last ipowindow */
+	
+	short transflag, ipoflag;	/* transformation and ipo settings */
+	short trackflag, upflag;
+	short nlaflag, protectflag;	/* nlaflag defines NLA override, protectflag is bits to lock transform */
+	short ipowin, scaflag;		/* ipowin: blocktype last ipowindow */
 	short scavisflag, boundtype;
 	
 	short dupon, dupoff, dupsta, dupend;
@@ -176,7 +178,8 @@ typedef struct Object {
 	 * bit 15: Always ignore activity culling 
 	 */
 	int gameflag2;
-	short softflag, dummy;			/* temporal stuff softbody experiment */
+	short softflag;			/* softboday settings */
+	short recalc;			/* dependency flag */
 	float anisotropicFriction[3];
 
 	ListBase constraints;
@@ -190,8 +193,13 @@ typedef struct Object {
 	LBuf lbuf;
 	LBuf port;
 	
-	float toonedge, smoothresh;	/* smoothresh is phong interpolation ray_shadow correction in render */
+	short fluidsimFlag;			/* NT toggle fluidsim participation on/off */
+	char shapenr, shapeflag;	/* current shape key for menu or pinned, flag for pinning */
+	float smoothresh;			/* smoothresh is phong interpolation ray_shadow correction in render */
+
+	struct FluidsimSettings *fluidsimSettings; /* if fluidsim enabled, store additional settings */
   
+	struct DerivedMesh *derivedDeform, *derivedFinal;
 } Object;
 
 typedef struct ObHook {
@@ -232,26 +240,25 @@ extern Object workob;
 #define OB_LAMP			10
 #define OB_CAMERA		11
 
-#define OB_IKA			20
 #define OB_WAVE			21
 #define OB_LATTICE		22
 
 /* 23 and 24 are for life and sector (old file compat.) */
 #define	OB_ARMATURE		25
 
-/* partype: eerste 5 bits: type */
+/* partype: first 4 bits: type */
 #define PARTYPE			15
 #define PAROBJECT		0
 #define PARCURVE		1
 #define PARKEY			2
-#define PARLIMB			3
+
 #define PARSKEL			4
 #define PARVERT1		5
 #define PARVERT3		6
 #define PARBONE			7
 #define PARSLOW			16
 
-/* char! transflag */
+/* (short) transflag */
 #define OB_OFFS_LOCAL	1
 #define OB_QUAT			2
 #define OB_NEG_SCALE	4
@@ -263,7 +270,7 @@ extern Object workob;
 
 #define OB_POWERTRACK	128
 
-/* char! ipoflag */
+/* (short) ipoflag */
 #define OB_DRAWKEY			1
 #define OB_DRAWKEYSEL		2
 #define OB_OFFS_OB			4
@@ -272,9 +279,13 @@ extern Object workob;
 #define OB_OFFS_PATH		32
 #define OB_OFFS_PARENT		64
 #define OB_OFFS_PARTICLE	128
+	/* get ipo from from action or not? */
+#define OB_ACTION_OB		256
+#define OB_ACTION_KEY		512
+	/* for stride edit */
+#define OB_DISABLE_PATH		1024
 
-
-/* trackflag / upflag */
+/* (short) trackflag / upflag */
 #define OB_POSX			0
 #define OB_POSY			1
 #define OB_POSZ			2
@@ -284,24 +295,24 @@ extern Object workob;
 
 /* gameflag in game.h */
 
-/* dt: nummers */
+/* dt: no flags */
 #define OB_BOUNDBOX		1
 #define OB_WIRE			2
 #define OB_SOLID		3
 #define OB_SHADED		4
 #define OB_TEXTURE		5
-#define OB_TOON_MONO    6
-#define OB_TOON_COLOR   7
-#define OB_TOON_TRANSP  8
-#define OB_TOON_FLAT    9
-#define OB_TOON_SMOOTH  10
 
-/* dtx: flags */
+/* dtx: flags, char! */
 #define OB_AXIS			2
 #define OB_TEXSPACE		4
 #define OB_DRAWNAME		8
 #define OB_DRAWIMAGE	16
-#define OB_DRAWWIRE		32 /* for solid+wire display */
+	/* for solid+wire display */
+#define OB_DRAWWIRE		32
+	/* for overdraw */
+#define OB_DRAWXRAY		64
+	/* enable transparent draw */
+#define OB_DRAWTRANSP	128
 
 /* boundtype */
 #define OB_BOUND_BOX		0
@@ -313,12 +324,13 @@ extern Object workob;
 
 /* also needed for base!!!!! or rather, thy interfere....*/
 /* base->flag and ob->flag */
-#define BA_WASSEL			2
-#define BA_PARSEL			4
-#define BA_WHERE_UPDATE		8
-#define BA_DISP_UPDATE		16
+#define BA_WAS_SEL			2
+#define BA_HAS_RECALC_OB	4
+#define BA_HAS_RECALC_DATA	8
+
 #define BA_DO_IPO			32
 #define OB_GONNA_MOVE		32
+
 #define BA_FROMSET			128
 #define OB_DO_IMAT			256
 #define OB_FROMDUPLI		512
@@ -327,6 +339,12 @@ extern Object workob;
 #define OB_FROMGROUP		4096
 #define OB_POSEMODE			8192
 
+/* ob->recalc (flag bits!) */
+#define OB_RECALC_OB		1
+#define OB_RECALC_DATA		2
+#define OB_RECALC			3
+
+
 /* ob->gameflag */
 #define OB_DYNAMIC		1
 #define OB_CHILD		2
@@ -364,6 +382,25 @@ extern Object workob;
 #define OB_ADDACT		1024
 #define OB_SHOWCONT		2048
 
+/* ob->shapeflag */
+#define OB_SHAPE_LOCK		1
+#define OB_SHAPE_TEMPLOCK	2
+
+/* ob->nlaflag */
+#define OB_NLA_OVERRIDE		1
+
+/* ob->protectflag */
+#define OB_LOCK_LOCX	1
+#define OB_LOCK_LOCY	2
+#define OB_LOCK_LOCZ	4
+#define OB_LOCK_LOC		7
+#define OB_LOCK_ROTX	8
+#define OB_LOCK_ROTY	16
+#define OB_LOCK_ROTZ	32
+#define OB_LOCK_SIZEX	64
+#define OB_LOCK_SIZEY	128
+#define OB_LOCK_SIZEZ	256
+
 /* ob->softflag in DNA_object_force.h */
 
 #ifdef __cplusplus
