@@ -1,7 +1,7 @@
 /**
  * blenlib/DNA_screen_types.h (mar-2001 nzc)
  *
- * $Id: DNA_screen_types.h,v 1.20 2005/05/11 20:01:42 ton Exp $ 
+ * $Id: DNA_screen_types.h,v 1.21 2005/10/27 14:11:25 ton Exp $ 
  *
  * ***** BEGIN GPL/BL DUAL LICENSE BLOCK *****
  *
@@ -147,8 +147,12 @@ typedef struct ScrArea {
 #define B_SCROLL 8
 #define HOR_SCROLL 12
 
-/* screen->flag */
+/* screen handlers */
+#define SCREEN_MAXHANDLER		8
 
+#define SCREEN_HANDLER_ANIM		1
+#define SCREEN_HANDLER_PYTHON   2
+#define SCREEN_HANDLER_VERSE	3
 
 /* dunno who thought this below is nice code, but be warned, the values are written in
    a file, and cannot be switched or altered. enum here is out of focus (ton) */
