@@ -1,7 +1,7 @@
 /**
  * blenlib/DNA_sequence_types.h (mar-2001 nzc)
  *
- * $Id: DNA_sequence_types.h,v 1.16 2005/03/24 09:37:06 ton Exp $
+ * $Id: DNA_sequence_types.h,v 1.19 2005/11/08 00:46:39 intrr Exp $
  *
  * ***** BEGIN GPL/BL DUAL LICENSE BLOCK *****
  *
@@ -82,7 +82,7 @@ typedef struct PluginSeq {
 	void (*callback)(void);
 } PluginSeq;
 
-
+/* The sequence structure is the basic struct used by any strip. each of the strips uses a different sequence structure.*/
 /* WATCH IT: first part identical to ID (for use in ipo's) */
 
 typedef struct Sequence {
@@ -91,14 +91,14 @@ typedef struct Sequence {
 	void *lib;
 	char name[24];
 
-	short flag, type;
+	short flag, type;			/*flags bitmap (see below) and the type of sequence*/
 	int len;
 	int start, startofs, endofs;
 	int startstill, endstill;
 	int machine, depth;
-	int startdisp, enddisp;
+	int startdisp, enddisp;		/*starting and ending points in the sequence*/
 	float mul, handsize;
-	int sfra;
+	int sfra;				/* starting frame according to the timeline of the scene */
 
 	Strip *strip;
 	StripElem *curelem;
@@ -119,7 +119,7 @@ typedef struct Sequence {
 	struct bSound *sound;	/* the linked "bSound" object */
 	float level, pan;		/* level in dB (0=full), pan -1..1 */
 	int curpos;				/* last sample position in audio_fill() */
-	int pad;
+	float strobe;
 
 	void *effectdata;			/* Struct pointer for effect settings */
 
@@ -164,8 +164,9 @@ typedef struct GlowVars {
 #define SEQ_FILTERY		16
 #define SEQ_MUTE		32
 #define SEQ_MAKE_PREMUL	64
+#define SEQ_REVERSE_FRAMES	128
 
-/* seq->type WATCH IT: BIT 3!!! */
+/* seq->type WATCH IT: SEQ_EFFECT BIT is used to determine if this is an effect strip!!! */
 #define SEQ_IMAGE		0
 #define SEQ_META		1
 #define SEQ_SCENE		2
