@@ -1,7 +1,7 @@
 /**
  * blenlib/DNA_view3d_types.h (mar-2001 nzc)
  *
- * $Id: DNA_view3d_types.h,v 1.22 2005/05/08 08:16:42 ton Exp $ 
+ * $Id: DNA_view3d_types.h,v 1.28 2005/11/19 10:55:06 ton Exp $ 
  *
  * ***** BEGIN GPL/BL DUAL LICENSE BLOCK *****
  *
@@ -38,6 +38,8 @@ struct Object;
 struct Image;
 struct Tex;
 struct SpaceLink;
+struct Base;
+struct BoundBox;
 
 /* This is needed to not let VC choke on near and far... old
  * proprietary MS extensions... */
@@ -48,6 +50,8 @@ struct SpaceLink;
 #define far clipend
 #endif
 
+#include "DNA_listBase.h"
+
 /* The near/far thing is a Win EXCEPTION. Thus, leave near/far in the
  * code, and patch for windows. */
 
@@ -59,6 +63,18 @@ typedef struct BGpic {
 	unsigned int *rect;
 } BGpic;
 
+#
+#
+typedef struct View3DAfter {
+	struct View3DAfter *next, *prev;
+	struct Base *base;
+	int type;
+} View3DAfter;
+
+/* View3DAfter->type */
+#define V3D_XRAY	1
+#define V3D_TRANSP	2
+
 typedef struct View3D {
 	struct SpaceLink *next, *prev;
 	int spacetype;
@@ -97,23 +113,28 @@ typedef struct View3D {
 	int lay, layact;
 	short scenelock, around, camzoom, flag;
 	
-	float lens, grid, gridview, pad, near, far;
+	float lens, grid, gridview, pixsize, near, far;
 	float ofs[3], cursor[3];
-	
-	short mx, my;	/* have to remain together, because used as single pointer */
-	short mxo, myo;
 
 	short gridlines, viewbut;
 	short gridflag;
 	short modeselect, menunr, texnr;
 	
 	/* transform widget info */
-	short twtype, twmode, twflag, twpad;
+	short twtype, twmode, twflag, twdrawflag;
 	float twmat[4][4];
 	
+	/* user defined clipping planes */
+	float clip[4][4];
+	struct BoundBox *clipbb;
+	
+	/* afterdraw, for xray & transparent */
+	struct ListBase afterdraw;
+	/* drawflags, denoting state */
+	short zbuf, transp, xray, pad2;
 } View3D;
 
-/* View3D->flag */
+/* View3D->flag (short) */
 #define V3D_MODE			(16+32+64+128+256+512)
 #define V3D_DISPIMAGE		1
 #define V3D_DISPBGPIC		2
@@ -129,6 +150,8 @@ typedef struct View3D {
 #define V3D_SELECT_OUTLINE	2048
 #define V3D_ZBUF_SELECT		4096
 #define V3D_GLOBAL_STATS	8192
+#define V3D_CLIPPING		16384
+#define V3D_DRAW_CENTERS	32768
 
 /* View3D->around */
 #define V3D_CENTRE		 0
@@ -158,6 +181,7 @@ typedef struct View3D {
 #define V3D_MANIP_GLOBAL		0
 #define V3D_MANIP_LOCAL			1
 #define V3D_MANIP_NORMAL		2
+#define V3D_MANIP_VIEW			3
 
 /* View3d->twflag */
    /* USE = user setting, DRAW = based on selection */
