@@ -1,5 +1,5 @@
 /* 
- * $Id: BPY_interface.c,v 1.71 2005/06/11 05:30:14 ianwill Exp $
+ * $Id: BPY_interface.c,v 1.80 2006/01/16 17:55:03 stiv Exp $
  *
  * ***** BEGIN GPL/BL DUAL LICENSE BLOCK *****
  *
@@ -26,56 +26,44 @@
  * This is a new part of Blender.
  *
  * Contributor(s): Michel Selten, Willian P. Germano, Stephen Swaney,
- * Chris Keith, Chris Want
+ * Chris Keith, Chris Want, Ken Hughes
  *
  * ***** END GPL/BL DUAL LICENSE BLOCK *****
 */
-#ifdef HAVE_CONFIG_H
-#include <config.h>
-#endif
 
 #include <Python.h>
+
 #include "compile.h"		/* for the PyCodeObject */
 #include "eval.h"		/* for PyEval_EvalCode */
-
-#include <stdio.h>
-
-#include <MEM_guardedalloc.h>
-#include <BLI_blenlib.h>	/* for BLI_last_slash() */
-
-#include "BIF_gl.h" /* glPushAttrib, glPopAttrib for DRAW space handlers */
-#include <BIF_interface.h>	/* for pupmenu() */
-#include <BIF_space.h>
-#include <BIF_screen.h>
-#include <BIF_toolbox.h>
-#include <BKE_global.h>
-#include <BKE_library.h>
-#include <BKE_main.h>
-#include <BKE_object.h>		/* during_scriptlink() */
-#include <BKE_text.h>
-#include <BKE_utildefines.h>
-#include <BPI_script.h>
-
-#include <DNA_camera_types.h>
-#include <DNA_ID.h>
-#include <DNA_lamp_types.h>
-#include <DNA_material_types.h>
-#include <DNA_object_types.h>
-#include <DNA_scene_types.h>
-#include <DNA_screen_types.h>
-#include <DNA_scriptlink_types.h>
-#include <DNA_space_types.h>
-#include <DNA_text_types.h>
-#include <DNA_world_types.h>
-#include <DNA_userdef_types.h>	/* for U.pythondir */
-
+#include "BLI_blenlib.h"	/* for BLI_last_slash() */
+#include "BIF_interface.h"	/* for pupmenu() */
+#include "BIF_space.h"
+#include "BIF_screen.h"
+#include "BIF_toolbox.h"
+#include "BKE_library.h"
+#include "BKE_object.h"		/* during_scriptlink() */
+#include "BKE_text.h"
+#include "DNA_screen_types.h"
+#include "DNA_userdef_types.h"	/* for U.pythondir */
+#include "MEM_guardedalloc.h"
 #include "BPY_extern.h"
 #include "BPY_menus.h"
+#include "BPI_script.h"
+#include "BKE_global.h"
+#include "BKE_main.h"
 #include "api2_2x/EXPP_interface.h"
 #include "api2_2x/constant.h"
-
 #include "api2_2x/gen_utils.h"
-#include "api2_2x/modules.h"
+#include "api2_2x/BGL.h" 
+#include "api2_2x/Blender.h"
+#include "api2_2x/Camera.h"
+#include "api2_2x/Draw.h"
+#include "api2_2x/Lamp.h"
+#include "api2_2x/NMesh.h"
+#include "api2_2x/Object.h"
+#include "api2_2x/Registry.h"
+#include "api2_2x/Scene.h"
+#include "api2_2x/World.h"
 
 /* bpy_registryDict is declared in api2_2x/Registry.h and defined
  * in api2_2x/Registry.c
@@ -83,7 +71,15 @@
  * choose to preserve after they are executed, so user changes can be
  * restored next time the script is used.  Check the Blender.Registry module. 
  */
-#include "api2_2x/Registry.h"
+//#include "api2_2x/Registry.h"
+
+/*Declares the modules and their initialization functions
+*These are TOP-LEVEL modules e.g. import `module` - there is no
+*support for packages here e.g. import `package.module` */
+static struct _inittab BPy_Inittab_Modules[] = {
+	{"Blender", M_Blender_Init},
+	{NULL, NULL}
+};
 
 /*************************************************************************
 * Structure definitions	
@@ -109,6 +105,7 @@ void ReleaseGlobalDictionary( PyObject * dict );
 void DoAllScriptsFromList( ListBase * list, short event );
 PyObject *importText( char *name );
 void init_ourImport( void );
+void init_ourReload( void );
 PyObject *blender_import( PyObject * self, PyObject * args );
 
 int BPY_txt_do_python_Text( struct Text *text );
@@ -117,11 +114,9 @@ PyObject *traceback_getFilename( PyObject * tb );
 
 void BPY_free_screen_spacehandlers(struct bScreen *sc);
 
-
 /****************************************************************************
-* Description: This function will initialise Python and all the implemented 
-*	          api variations.
-* Notes:	Currently only the api for 2.2x will be initialised. 
+* Description: This function will start the interpreter and load all modules
+* as well as search for a python installation.
 ****************************************************************************/
 void BPY_start_python( int argc, char **argv )
 {
@@ -132,35 +127,39 @@ void BPY_start_python( int argc, char **argv )
 	/* we keep a copy of the values of argc and argv so that the game engine
 	 * can call BPY_start_python(0, NULL) whenever a game ends, without having
 	 * to know argc and argv there (in source/blender/src/space.c) */
-
 	if( first_time ) {
 		argc_copy = argc;
 		argv_copy = argv;
 	}
 
+	//stuff for Registry module
 	bpy_registryDict = PyDict_New(  );/* check comment at start of this file */
-
 	if( !bpy_registryDict )
 		printf( "Error: Couldn't create the Registry Python Dictionary!" );
-
 	Py_SetProgramName( "blender" );
 
-	/* 
-	 * Py_Initialize() will attempt to import the site module and
+	/* Py_Initialize() will attempt to import the site module and
 	 * print an error if not found.  See init_syspath() for the
 	 * rest of our init msgs.
 	 */
-
-	/* Py_GetVersion() returns a ptr to astatic string */
+	// Py_GetVersion() returns a ptr to astatic string
 	printf( "Using Python version %.3s\n", Py_GetVersion() );
 
+	//Initialize the TOP-LEVEL modules
+	PyImport_ExtendInittab(BPy_Inittab_Modules);
+	
+	//Start the interpreter
 	Py_Initialize(  );
 	PySys_SetArgv( argc_copy, argv_copy );
 
+	//Overrides __import__
 	init_ourImport(  );
+	init_ourReload(  );
 
-	initBlenderApi2_2x(  );
+	//init a global dictionary
+	g_blenderdict = NULL;
 
+	//Look for a python installation
 	init_syspath( first_time ); /* not first_time: some msgs are suppressed */
 
 	return;
@@ -213,8 +212,7 @@ void init_syspath( int first_time )
 {
 	PyObject *path;
 	PyObject *mod, *d;
-	PyObject *p;
-	char *c, *progname;
+	char *progname;
 	char execdir[FILE_MAXDIR];	/*defines from DNA_space_types.h */
 
 	int n;
@@ -225,15 +223,20 @@ void init_syspath( int first_time )
 
 	if( mod ) {
 		d = PyModule_GetDict( mod );
-		PyDict_SetItemString( d, "progname", path );
+		EXPP_dict_set_item_str( d, "progname", path );
 		Py_DECREF( mod );
 	} else
 		printf( "Warning: could not set Blender.sys.progname\n" );
 
 	progname = BLI_last_slash( bprogname );	/* looks for the last dir separator */
 
-	c = Py_GetPath(  );	/* get python system path */
-	PySys_SetPath( c );	/* initialize */
+#ifdef SETSYSPATH
+	{
+		char *c; 
+		c = Py_GetPath(  );	/* get python system path */
+		PySys_SetPath( c );	/* initialize */
+	}
+#endif
 
 	n = progname - bprogname;
 	if( n > 0 ) {
@@ -248,6 +251,7 @@ void init_syspath( int first_time )
 	} else
 		printf( "Warning: could not determine argv[0] path\n" );
 
+#ifdef SETSYSPATH
 	/* 
 	 * bring in the site module so we can add 
 	 * site-package dirs to sys.path 
@@ -259,6 +263,7 @@ void init_syspath( int first_time )
 		PyObject *item;
 		int size = 0;
 		int index;
+		PyObject *p;
 
 		/* get the value of 'sitedirs' from the module */
 
@@ -286,6 +291,7 @@ void init_syspath( int first_time )
 			printf( "Continuing happily.\n" );
 		}
 	}
+#endif
 
 	/* 
 	 * initialize the sys module
@@ -296,7 +302,7 @@ void init_syspath( int first_time )
 
 	if( mod ) {
 		d = PyModule_GetDict( mod );	/* borrowed ref */
-		PyDict_SetItemString( d, "executable",
+		EXPP_dict_set_item_str( d, "executable",
 				      Py_BuildValue( "s", bprogname ) );
 		Py_DECREF( mod );
 	}
@@ -449,7 +455,7 @@ void BPY_Err_Handle( char *script_name )
  *	the last frame in the current file is adressed."' 
  */
 
-		while( 1 ) {
+		for(;;) {
 			v = PyObject_GetAttrString( tb, "tb_next" );
 
 			if( !v || v == Py_None ||
@@ -533,13 +539,13 @@ int BPY_txt_do_python_Text( struct Text *text )
 
 	script->py_globaldict = py_dict;
 
-	info = ( BPy_constant * ) M_constant_New(  );
+	info = ( BPy_constant * ) PyConstant_New(  );
 	if( info ) {
-		constant_insert( info, "name",
+		PyConstant_Insert( info, "name",
 				 PyString_FromString( script->id.name + 2 ) );
 		Py_INCREF( Py_None );
-		constant_insert( info, "arg", Py_None );
-		PyDict_SetItemString( py_dict, "__script__",
+		PyConstant_Insert( info, "arg", Py_None );
+		EXPP_dict_set_item_str( py_dict, "__script__",
 				      ( PyObject * ) info );
 	}
 
@@ -569,7 +575,8 @@ int BPY_txt_do_python_Text( struct Text *text )
 
 /****************************************************************************
 * Description: Called from command line to run a Python script
-* automatically. 
+* automatically. The script can be a file or a Blender Text in the current 
+* .blend.
 ****************************************************************************/
 void BPY_run_python_script( char *fn )
 {
@@ -609,7 +616,19 @@ void BPY_run_python_script( char *fn )
 			"%s (at line %d).\n", fn, BPY_Err_getLinenumber());
 	}
 
-	if (!is_blender_text) free_libblock(&G.main->text, text);
+	if (!is_blender_text) {
+		/* We can't simply free the text, since the script might have called
+		 * Blender.Load() to load a new .blend, freeing previous data.
+		 * So we check if the pointer is still valid. */
+		Text *txtptr = G.main->text.first;
+		while (txtptr) {
+			if (txtptr == text) {
+				free_libblock(&G.main->text, text);
+				break;
+			}
+			txtptr = txtptr->id.next;
+		}
+	}
 }
 
 /****************************************************************************
@@ -763,12 +782,12 @@ int BPY_menu_do_python( short menutype, int event )
 
 	script->py_globaldict = py_dict;
 
-	info = ( BPy_constant * ) M_constant_New(  );
+	info = ( BPy_constant * ) PyConstant_New(  );
 	if( info ) {
-		constant_insert( info, "name",
+		PyConstant_Insert( info, "name",
 				 PyString_FromString( script->id.name + 2 ) );
-		constant_insert( info, "arg", pyarg );
-		PyDict_SetItemString( py_dict, "__script__",
+		PyConstant_Insert( info, "arg", pyarg );
+		EXPP_dict_set_item_str( py_dict, "__script__",
 				      ( PyObject * ) info );
 	}
 
@@ -1059,13 +1078,13 @@ void BPY_do_pyscript( ID * id, short event )
 		 * is running nested inside another.  Blender.Load needs this info to
 		 * avoid trouble with invalid slink pointers. */
 		during_slink++;
-		disable_where_scriptlink( during_slink );
+		disable_where_scriptlink( (short)during_slink );
 
 		/* set globals in Blender module to identify scriptlink */
-		PyDict_SetItemString( g_blenderdict, "bylink", EXPP_incr_ret_True() );
-		PyDict_SetItemString( g_blenderdict, "link",
+		EXPP_dict_set_item_str( g_blenderdict, "bylink", EXPP_incr_ret_True() );
+		EXPP_dict_set_item_str( g_blenderdict, "link",
 				      ID_asPyObject( id ) );
-		PyDict_SetItemString( g_blenderdict, "event",
+		EXPP_dict_set_item_str( g_blenderdict, "event",
 				      PyString_FromString( event_to_name
 							   ( event ) ) );
 
@@ -1099,15 +1118,14 @@ void BPY_do_pyscript( ID * id, short event )
 			}
 		}
 
-		disable_where_scriptlink( during_slink - 1 );
+		disable_where_scriptlink( (short)(during_slink - 1) );
 
 		/* cleanup bylink flag and clear link so PyObject
 		 * can be released 
 		 */
-		PyDict_SetItemString( g_blenderdict, "bylink", EXPP_incr_ret_False() );
-		Py_INCREF( Py_None );
+		EXPP_dict_set_item_str( g_blenderdict, "bylink", EXPP_incr_ret_False() );
 		PyDict_SetItemString( g_blenderdict, "link", Py_None );
-		PyDict_SetItemString( g_blenderdict, "event",
+		EXPP_dict_set_item_str( g_blenderdict, "event",
 				      PyString_FromString( "" ) );
 	}
 }
@@ -1210,7 +1228,7 @@ int BPY_add_spacehandler(Text *text, ScrArea *sa, char spacetype)
 
 	if (!sa || !text) return -1;
 
-	handlertype = BPY_is_spacehandler(text, spacetype);
+	handlertype = (unsigned short)BPY_is_spacehandler(text, spacetype);
 
 	if (handlertype) {
 		ScriptLink *slink = &sa->scriptlink;
@@ -1274,16 +1292,20 @@ int BPY_do_spacehandlers( ScrArea *sa, unsigned short event,
 
 		/* tell we're running a scriptlink.  The sum also tells if this script
 		 * is running nested inside another.  Blender.Load needs this info to
-		 * avoid trouble with invalid slink pointers. */
-		during_slink++;
-		disable_where_scriptlink( during_slink );
+		 * avoid trouble with invalid slink pointers.
+		 * Update (test): allow EVENT space handlers to call file/image selectors,
+		 * still disabled for DRAW space handlers: */
+		if (event == 0) { /* event = 0: DRAW space handler */
+			during_slink++;
+			disable_where_scriptlink( (short)during_slink );
+		}
 
 		/* set globals in Blender module to identify space handler scriptlink */
-		PyDict_SetItemString(g_blenderdict, "bylink", EXPP_incr_ret_True());
+		EXPP_dict_set_item_str(g_blenderdict, "bylink", EXPP_incr_ret_True());
 		/* unlike normal scriptlinks, here Blender.link is int (space event type) */
-		PyDict_SetItemString(g_blenderdict, "link", PyInt_FromLong(space_event));
+		EXPP_dict_set_item_str(g_blenderdict, "link", PyInt_FromLong(space_event));
 		/* note: DRAW space_events set event to 0 */
-		PyDict_SetItemString(g_blenderdict, "event", PyInt_FromLong(event));
+		EXPP_dict_set_item_str(g_blenderdict, "event", PyInt_FromLong(event));
 
 		/* now run all assigned space handlers for this space and space_event */
 		for( index = 0; index < scriptlink->totscript; index++ ) {
@@ -1334,15 +1356,14 @@ int BPY_do_spacehandlers( ScrArea *sa, unsigned short event,
 				glMatrixMode(GL_MODELVIEW);
 				glPopMatrix();
 				glPopAttrib();
+				disable_where_scriptlink( (short)(during_slink - 1) );
 			}
 
 		}
 
-		disable_where_scriptlink( during_slink - 1 );
-
-		PyDict_SetItemString(g_blenderdict, "bylink", EXPP_incr_ret_False());
-		PyDict_SetItemString(g_blenderdict, "link", EXPP_incr_ret(Py_None));
-		PyDict_SetItemString(g_blenderdict, "event", PyString_FromString(""));
+		EXPP_dict_set_item_str(g_blenderdict, "bylink", EXPP_incr_ret_False());
+		PyDict_SetItemString(g_blenderdict, "link", Py_None );
+		EXPP_dict_set_item_str(g_blenderdict, "event", PyString_FromString(""));
 	}
 	
 	/* retval:
@@ -1526,7 +1547,7 @@ PyObject *CreateGlobalDictionary( void )
 	PyObject *dict = PyDict_New(  );
 
 	PyDict_SetItemString( dict, "__builtins__", PyEval_GetBuiltins(  ) );
-	PyDict_SetItemString( dict, "__name__",
+	EXPP_dict_set_item_str( dict, "__name__",
 			      PyString_FromString( "__main__" ) );
 
 	return dict;
@@ -1650,5 +1671,106 @@ void init_ourImport( void )
 
 	m = PyImport_AddModule( "__builtin__" );
 	d = PyModule_GetDict( m );
-	PyDict_SetItemString( d, "__import__", import );
+	EXPP_dict_set_item_str( d, "__import__", import );
+}
+
+/*
+ * find in-memory module and recompile
+ */
+
+static PyObject *reimportText( PyObject *module )
+{
+	Text *text;
+	char *txtname;
+	char *name;
+	char *buf = NULL;
+
+	/* get name, filename from the module itself */
+
+	txtname = PyModule_GetFilename( module );
+	name = PyModule_GetName( module );
+	if( !txtname || !name)
+		return NULL;
+
+	/* look up the text object */
+	text = ( Text * ) & ( G.main->text.first );
+	while( text ) {
+		if( !strcmp( txtname, GetName( text ) ) )
+			break;
+		text = text->id.next;
+	}
+
+	/* uh-oh.... didn't find it */
+	if( !text )
+		return NULL;
+
+	/* if previously compiled, free the object */
+	/* (can't see how could be NULL, but check just in case) */ 
+	if( text->compiled ){
+		Py_DECREF( (PyObject *)text->compiled );
+	}
+
+	/* compile the buffer */
+	buf = txt_to_buf( text );
+	text->compiled = Py_CompileString( buf, GetName( text ),
+			Py_file_input );
+	MEM_freeN( buf );
+
+	/* if compile failed.... return this error */
+	if( PyErr_Occurred(  ) ) {
+		PyErr_Print(  );
+		BPY_free_compiled_text( text );
+		return NULL;
+	}
+
+	/* make into a module */
+	return PyImport_ExecCodeModule( name, text->compiled );
+}
+
+/*
+ * our reload() module, to handle reloading in-memory scripts
+ */
+
+static PyObject *blender_reload( PyObject * self, PyObject * args )
+{
+	PyObject *exception, *err, *tb;
+	PyObject *module = NULL;
+	PyObject *newmodule = NULL;
+
+	/* check for a module arg */
+	if( !PyArg_ParseTuple( args, "O:breload", &module ) )
+		return NULL;
+
+	/* try reimporting from file */
+	newmodule = PyImport_ReloadModule( module );
+	if( newmodule )
+		return newmodule;
+
+	/* no file, try importing from memory */
+	PyErr_Fetch( &exception, &err, &tb );	/*restore for probable later use */
+
+	newmodule = reimportText( module );
+	if( newmodule ) {		/* found module, ignore above exception */
+		PyErr_Clear(  );
+		Py_XDECREF( exception );
+		Py_XDECREF( err );
+		Py_XDECREF( tb );
+	} else
+		PyErr_Restore( exception, err, tb );
+
+	return newmodule;
+}
+
+static PyMethodDef breload[] = {
+	{"blreload", blender_reload, METH_VARARGS, "our own reload"}
+};
+
+void init_ourReload( void )
+{
+	PyObject *m, *d;
+	PyObject *reload = PyCFunction_New( breload, NULL );
+
+	m = PyImport_AddModule( "__builtin__" );
+	d = PyModule_GetDict( m );
+	EXPP_dict_set_item_str( d, "reload", reload );
 }
