@@ -1,5 +1,5 @@
 /*
- * $Id: CurNurb.c,v 1.4 2004/10/07 19:25:39 stiv Exp $
+ * $Id: CurNurb.c,v 1.17 2005/12/01 04:50:04 khughes Exp $
  * ***** BEGIN GPL/BL DUAL LICENSE BLOCK *****
  *
  * This program is free software; you can redistribute it and/or
@@ -29,40 +29,41 @@
  * ***** END GPL/BL DUAL LICENSE BLOCK *****
  */
 
-#include "Python.h"
-#include "DNA_curve_types.h"
+#include "CurNurb.h" /*This must come first */
+
 #include "BKE_curve.h"
+#include "BDR_editcurve.h"	/* for convertspline */
 #include "MEM_guardedalloc.h"
-
 #include "gen_utils.h"
-#include "CurNurb.h"
 #include "BezTriple.h"
 
 
-/*-------------------------------------------------------------
-
-stuff in this section should be placed in bpy_types.h
-
------------------------------------------------------------*/
-
-
 /*
  * forward declarations go here
  */
 
 
-extern PyMethodDef BPy_CurNurb_methods[];
+static PyObject *M_CurNurb_New( PyObject * self, PyObject * args );
 PyObject *CurNurb_CreatePyObject( Nurb * blen_nurb );
 static PyObject *CurNurb_setMatIndex( BPy_CurNurb * self, PyObject * args );
 static PyObject *CurNurb_getMatIndex( BPy_CurNurb * self );
+static PyObject *CurNurb_getFlagU( BPy_CurNurb * self );
+static PyObject *CurNurb_setFlagU( BPy_CurNurb * self, PyObject * args );
+static PyObject *CurNurb_getFlagV( BPy_CurNurb * self );
+static PyObject *CurNurb_setFlagV( BPy_CurNurb * self, PyObject * args );
+static PyObject *CurNurb_getType( BPy_CurNurb * self );
+static PyObject *CurNurb_setType( BPy_CurNurb * self, PyObject * args );
 /* static PyObject* CurNurb_setXXX( BPy_CurNurb* self, PyObject* args ); */
 PyObject *CurNurb_getPoint( BPy_CurNurb * self, int index );
+static int CurNurb_setPoint( BPy_CurNurb * self, int index, PyObject * ob );
 static int CurNurb_length( PyInstanceObject * inst );
 static PyObject *CurNurb_getIter( BPy_CurNurb * self );
 static PyObject *CurNurb_iterNext( BPy_CurNurb * self );
 PyObject *CurNurb_append( BPy_CurNurb * self, PyObject * args );
 PyObject *CurNurb_pointAtIndex( Nurb * nurb, int index );
 static PyObject *CurNurb_isNurb( BPy_CurNurb * self );
+static PyObject *CurNurb_isCyclic( BPy_CurNurb * self );
+static PyObject *CurNurb_dump( BPy_CurNurb * self );
 
 char M_CurNurb_doc[] = "CurNurb";
 
@@ -79,354 +80,6 @@ static PyObject *CurNurb_repr( BPy_CurNurb * self );
 
 
 
-
-void CurNurb_dealloc( BPy_CurNurb * self )
-{
-	PyObject_DEL( self );
-}
-
-
-
-static PyObject *CurNurb_getAttr( BPy_CurNurb * self, char *name )
-{
-	PyObject *attr = Py_None;
-
-	if( strcmp( name, "mat_index" ) == 0 )
-		attr = PyInt_FromLong( self->nurb->mat_nr );
-
-	else if( strcmp( name, "points" ) == 0 )
-		attr = PyInt_FromLong( self->nurb->pntsu );
-
-	if( !attr )
-		return EXPP_ReturnPyObjError( PyExc_MemoryError,
-					      "couldn't create PyObject" );
-
-	/* member attribute found, return it */
-	if( attr != Py_None )
-		return attr;
-
-	/* not an attribute, search the methods table */
-	return Py_FindMethod( BPy_CurNurb_methods, ( PyObject * ) self, name );
-}
-
-
-/*
-  setattr
-*/
-
-static int CurNurb_setAttr( BPy_CurNurb * self, char *name, PyObject * value )
-{
-	PyObject *valtuple;
-	PyObject *error = NULL;
-
-	/* make a tuple to pass to our type methods */
-	valtuple = Py_BuildValue( "(O)", value );
-
-	if( !valtuple )
-		return EXPP_ReturnIntError( PyExc_MemoryError,
-					    "CurNurb.setAttr: cannot create pytuple" );
-
-	if( strcmp( name, "mat_index" ) == 0 )
-		error = CurNurb_setMatIndex( self, valtuple );
-
-	else {			/* error - no match for name */
-		Py_DECREF( valtuple );
-
-		if( ( strcmp( name, "ZZZZ" ) == 0 ) ||	/* user tried to change a */
-		    ( strcmp( name, "ZZZZ" ) == 0 ) )	/* constant dict type ... */
-			return EXPP_ReturnIntError( PyExc_AttributeError,
-						    "constant dictionary -- cannot be changed" );
-		else
-			return EXPP_ReturnIntError( PyExc_KeyError,
-						    "attribute not found" );
-	}
-
-
-	Py_DECREF( valtuple );	/* since it is not being returned */
-	if( error != Py_None )
-		return -1;
-
-	Py_DECREF( Py_None );
-	return 0;		/* normal exit */
-}
-
-/*
-  compare
-  in this case, we consider two CurNurbs equal, if they point to the same
-  blender data.
-*/
-
-static int CurNurb_compare( BPy_CurNurb * a, BPy_CurNurb * b )
-{
-	Nurb *pa = a->nurb;
-	Nurb *pb = b->nurb;
-
-	return ( pa == pb ) ? 0 : -1;
-}
-
-
-/*
-  factory method to create a BPy_CurNurb from a Blender Nurb
-*/
-
-PyObject *CurNurb_CreatePyObject( Nurb * blen_nurb )
-{
-	BPy_CurNurb *pyNurb;
-
-	pyNurb = ( BPy_CurNurb * ) PyObject_NEW( BPy_CurNurb, &CurNurb_Type );
-
-	if( !pyNurb )
-		return EXPP_ReturnPyObjError( PyExc_MemoryError,
-					      "could not create BPy_CurNurb PyObject" );
-
-	pyNurb->nurb = blen_nurb;
-	return ( PyObject * ) pyNurb;
-}
-
-
-/*
- *  CurNurb_repr
- */
-static PyObject *CurNurb_repr( BPy_CurNurb * self )
-{				/* used by 'repr' */
-
-	return PyString_FromFormat( "[CurNurb \"%d\"]", self->nurb->type );
-}
-
-
-static PyObject *M_CurNurb_New( PyObject * self, PyObject * args )
-{
-	return ( PyObject * ) 0;
-
-}
-
-
-
-/*
- * CurNurb_append( point )
- * append a new point to a nurb curve.
- * arg is BezTriple or list of xyzw floats 
- */
-
-PyObject *CurNurb_append( BPy_CurNurb * self, PyObject * args )
-{
-	Nurb *nurb = self->nurb;
-
-	return CurNurb_appendPointToNurb( nurb, args );
-}
-
-
-
-/*
- * CurNurb_appendPointToNurb
- * this is a non-bpy utility func to add a point to a given nurb
- */
-
-PyObject *CurNurb_appendPointToNurb( Nurb * nurb, PyObject * args )
-{
-
-	int i;
-	int size;
-	PyObject *pyOb;
-	int npoints;
-
-	/*
-	   do we have a list of four floats or a BezTriple?
-	 */
-	PyArg_ParseTuple( args, "O", &pyOb );
-
-	if( BezTriple_CheckPyObject( pyOb ) ) {
-		BezTriple *tmp;
-		npoints = nurb->pntsu;
-
-/*		printf("\ndbg: got a BezTriple\n"); */
-		tmp = nurb->bezt;	/* save old points */
-		nurb->bezt =
-			( BezTriple * ) MEM_mallocN( sizeof( BezTriple ) *
-						     ( npoints + 1 ),
-						     "CurNurb_append2" );
-
-		if( !nurb->bezt )
-			return ( EXPP_ReturnPyObjError
-				 ( PyExc_MemoryError, "allocation failed" ) );
-
-		/* copy old points to new */
-		memmove( nurb->bezt, tmp, sizeof( BezTriple ) * npoints );
-		if( tmp )
-			MEM_freeN( tmp );
-		nurb->pntsu++;
-		/* add new point to end of list */
-		memcpy( nurb->bezt + npoints,
-			BezTriple_FromPyObject( pyOb ), sizeof( BezTriple ) );
-
-	} else if( PySequence_Check( pyOb ) ) {
-		size = PySequence_Size( pyOb );
-/*		printf("\ndbg: got a sequence of size %d\n", size );  */
-		if( size == 4 ) {
-			BPoint *tmp;
-			npoints = nurb->pntsu;
-
-			tmp = nurb->bp;	/* save old pts */
-
-			nurb->bp =
-				( BPoint * ) MEM_mallocN( sizeof( BPoint ) *
-							  ( npoints + 1 ),
-							  "CurNurb_append1" );
-			if( !nurb->bp )
-				return ( EXPP_ReturnPyObjError
-					 ( PyExc_MemoryError,
-					   "allocation failed" ) );
-
-			memmove( nurb->bp, tmp, sizeof( BPoint ) * npoints );
-			if( tmp )
-				MEM_freeN( tmp );
-
-			++nurb->pntsu;
-			/* initialize new BPoint from old */
-			memcpy( nurb->bp + npoints, nurb->bp,
-				sizeof( BPoint ) );
-
-			for( i = 0; i < 4; ++i ) {
-				float tmpx =
-					( float ) PyFloat_AsDouble
-					( PySequence_GetItem( pyOb, i ) );
-				nurb->bp[npoints].vec[i] = tmpx;
-
-			}
-
-			makeknots( nurb, 1, nurb->flagu >> 1 );
-
-		} else if( size == 3 ) {	/* 3 xyz coords */
-			printf( "\nNot Yet Implemented!\n" );
-
-		}
-
-	} else {
-		/* bail with error */
-		return ( EXPP_ReturnPyObjError
-			 ( PyExc_AttributeError, "expected better stuff" ) );
-
-	}
-
-	return ( EXPP_incr_ret( Py_None ) );
-}
-
-
-/*
- *  CurNurb_setMatIndex
- *
- *  set index into material list
- */
-
-static PyObject *CurNurb_setMatIndex( BPy_CurNurb * self, PyObject * args )
-{
-	int index;
-
-	if( !PyArg_ParseTuple( args, "i", &( index ) ) )
-		return ( EXPP_ReturnPyObjError
-			 ( PyExc_AttributeError,
-			   "expected integer argument" ) );
-
-	/* fixme:  some range checking would be nice! */
-	self->nurb->mat_nr = index;
-
-	Py_INCREF( Py_None );
-	return Py_None;
-}
-
-/*
- * CurNurb_getMatIndex
- *
- * returns index into material list
- */
-
-static PyObject *CurNurb_getMatIndex( BPy_CurNurb * self )
-{
-	PyObject *index = PyInt_FromLong( ( long ) self->nurb->mat_nr );
-
-	if( index )
-		return index;
-
-	return ( EXPP_ReturnPyObjError( PyExc_RuntimeError,
-					"could not get material index" ) );
-}
-
-
-/*
- * CurNurb_getIter
- *
- * create an iterator for our CurNurb.
- * this iterator returns the points for this CurNurb.
- */
-
-static PyObject *CurNurb_getIter( BPy_CurNurb * self )
-{
-	self->bp = self->nurb->bp;
-	self->bezt = self->nurb->bezt;
-	self->atEnd = 0;
-	self->nextPoint = 0;
-
-	/* set exhausted flag if both bp and bezt are zero */
-	if( ( !self->bp ) && ( !self->bezt ) )
-		self->atEnd = 1;
-
-	Py_INCREF( self );
-	return ( PyObject * ) self;
-}
-
-
-
-static PyObject *CurNurb_iterNext( BPy_CurNurb * self )
-{
-	PyObject *po;		/* return value */
-	Nurb *pnurb = self->nurb;
-	int npoints = pnurb->pntsu;
-
-	/* are we at end already? */
-	if( self->atEnd )
-		return ( EXPP_ReturnPyObjError( PyExc_StopIteration,
-						"iterator at end" ) );
-
-	if( self->nextPoint < npoints ) {
-
-		po = CurNurb_pointAtIndex( self->nurb, self->nextPoint );
-		self->nextPoint++;
-
-		return po;
-
-	} else {
-		self->atEnd = 1;	/* set flag true */
-	}
-
-	return ( EXPP_ReturnPyObjError( PyExc_StopIteration,
-					"iterator at end" ) );
-}
-
-
-
-/*
- * CurNurb_isNurb()
- * test whether spline nurb or bezier
- */
-
-static PyObject *CurNurb_isNurb( BPy_CurNurb * self )
-{
-	/* NOTE: a Nurb has bp and bezt pointers
-	 * depending on type.
-	 * It is possible both are NULL if no points exist.
-	 * in that case, we return False
-	 */
-
-	if( self->nurb->bp ) {
-		Py_INCREF( Py_True );
-		return Py_True;
-	} else {
-		Py_INCREF( Py_False );
-		return ( Py_False );
-	}
-}
-
-
 /*
    table of module methods
    these are the equivalent of class or static methods.
@@ -459,14 +112,29 @@ static PyMethodDef BPy_CurNurb_methods[] = {
 	 "( index ) - set index into materials list"},
 	{"getMatIndex", ( PyCFunction ) CurNurb_getMatIndex, METH_NOARGS,
 	 "( ) - get current material index"},
+	{"setFlagU", ( PyCFunction ) CurNurb_setFlagU, METH_VARARGS,
+	 "( index ) - set flagU and recalculate the knots (0: uniform, 1: endpoints, 2: bezier)"},
+	{"getFlagU", ( PyCFunction ) CurNurb_getFlagU, METH_NOARGS,
+	 "( ) - get flagU of the knots"},
+	{"setFlagV", ( PyCFunction ) CurNurb_setFlagV, METH_VARARGS,
+	 "( index ) - set flagV and recalculate the knots (0: uniform, 1: endpoints, 2: bezier)"},
+	{"getFlagV", ( PyCFunction ) CurNurb_getFlagV, METH_NOARGS,
+	 "( ) - get flagV of the knots"},
+	{"setType", ( PyCFunction ) CurNurb_setType, METH_VARARGS,
+	 "( type ) - change the type of the curve (Poly: 0, Bezier: 1, NURBS: 4)"},
+	{"getType", ( PyCFunction ) CurNurb_getType, METH_NOARGS,
+	 "( ) - get the type of the curve (Poly: 0, Bezier: 1, NURBS: 4)"},
 	{"append", ( PyCFunction ) CurNurb_append, METH_VARARGS,
 	 "( point ) - add a new point.  arg is BezTriple or list of x,y,z,w floats"},
 	{"isNurb", ( PyCFunction ) CurNurb_isNurb, METH_NOARGS,
 	 "( ) - boolean function tests if this spline is type nurb or bezier"},
+	{"isCyclic", ( PyCFunction ) CurNurb_isCyclic, METH_NOARGS,
+	 "( ) - boolean function tests if this spline is cyclic (closed) or not (open)"},
+	{"dump", ( PyCFunction ) CurNurb_dump, METH_NOARGS,
+	 "( ) - dumps Nurb data)"},
 	{NULL, NULL, 0, NULL}
 };
 
-
 /* 
  *   methods for CurNurb as sequece
  */
@@ -477,7 +145,7 @@ static PySequenceMethods CurNurb_as_sequence = {
 	( intargfunc ) 0,	/* sq_repeat */
 	( intargfunc ) CurNurb_getPoint,	/* sq_item */
 	( intintargfunc ) 0,	/* sq_slice */
-	0,			/* sq_ass_item */
+	( intobjargproc ) CurNurb_setPoint,	/* sq_ass_item */
 	0,			/* sq_ass_slice */
 	( objobjproc ) 0,	/* sq_contains */
 	0,
@@ -489,6 +157,8 @@ static PySequenceMethods CurNurb_as_sequence = {
 /*
   Object Type definition
   full blown 2.3 struct
+  if you are having trouble building with an earlier version of python,
+   this is why.
 */
 
 PyTypeObject CurNurb_Type = {
@@ -574,6 +244,559 @@ PyTypeObject CurNurb_Type = {
 };
 
 
+
+
+void CurNurb_dealloc( BPy_CurNurb * self )
+{
+	PyObject_DEL( self );
+}
+
+
+
+static PyObject *CurNurb_getAttr( BPy_CurNurb * self, char *name )
+{
+	PyObject *attr = Py_None;
+
+	if( strcmp( name, "mat_index" ) == 0 )
+		attr = PyInt_FromLong( self->nurb->mat_nr );
+
+	else if( strcmp( name, "points" ) == 0 )
+		attr = PyInt_FromLong( self->nurb->pntsu );
+
+	else if( strcmp( name, "flagU" ) == 0 )
+		attr = CurNurb_getFlagU( self );
+
+	else if( strcmp( name, "flagV" ) == 0 )
+		attr = CurNurb_getFlagV( self );
+
+	else if( strcmp( name, "type" ) == 0 )
+		attr = CurNurb_getType( self );
+
+	else if( strcmp( name, "__members__" ) == 0 )
+		attr = Py_BuildValue( "[s,s,s,s,s]", "mat_index", "points", "flagU", "flagV", "type" );
+
+	if( !attr )
+		return EXPP_ReturnPyObjError( PyExc_MemoryError,
+					      "couldn't create PyObject" );
+
+	/* member attribute found, return it */
+	if( attr != Py_None )
+		return attr;
+
+	/* not an attribute, search the methods table */
+	return Py_FindMethod( BPy_CurNurb_methods, ( PyObject * ) self, name );
+}
+
+
+/*
+  setattr
+*/
+
+static int CurNurb_setAttr( BPy_CurNurb * self, char *name, PyObject * value )
+{
+	PyObject *valtuple;
+	PyObject *error = NULL;
+
+	/* make a tuple to pass to our type methods */
+	valtuple = Py_BuildValue( "(O)", value );
+
+	if( !valtuple )
+		return EXPP_ReturnIntError( PyExc_MemoryError,
+					    "CurNurb.setAttr: cannot create pytuple" );
+
+	if( strcmp( name, "mat_index" ) == 0 )
+		error = CurNurb_setMatIndex( self, valtuple );
+
+	else if( strcmp( name, "flagU" ) == 0 )
+		error = CurNurb_setFlagU( self, valtuple );
+
+	else if( strcmp( name, "flagV" ) == 0 )
+		error = CurNurb_setFlagV( self, valtuple );
+
+	else if( strcmp( name, "type" ) == 0 )
+		error = CurNurb_setType( self, valtuple );
+
+	else {			/* error - no match for name */
+		Py_DECREF( valtuple );
+
+		if( ( strcmp( name, "ZZZZ" ) == 0 ) ||	/* user tried to change a */
+		    ( strcmp( name, "ZZZZ" ) == 0 ) )	/* constant dict type ... */
+			return EXPP_ReturnIntError( PyExc_AttributeError,
+						    "constant dictionary -- cannot be changed" );
+		else
+			return EXPP_ReturnIntError( PyExc_KeyError,
+						    "attribute not found" );
+	}
+
+
+	Py_DECREF( valtuple );	/* since it is not being returned */
+	if( error != Py_None )
+		return -1;
+
+	Py_DECREF( Py_None );
+	return 0;		/* normal exit */
+}
+
+/*
+  compare
+  in this case, we consider two CurNurbs equal, if they point to the same
+  blender data.
+*/
+
+static int CurNurb_compare( BPy_CurNurb * a, BPy_CurNurb * b )
+{
+	Nurb *pa = a->nurb;
+	Nurb *pb = b->nurb;
+
+	return ( pa == pb ) ? 0 : -1;
+}
+
+
+/*
+  factory method to create a BPy_CurNurb from a Blender Nurb
+*/
+
+PyObject *CurNurb_CreatePyObject( Nurb * blen_nurb )
+{
+	BPy_CurNurb *pyNurb;
+
+	pyNurb = ( BPy_CurNurb * ) PyObject_NEW( BPy_CurNurb, &CurNurb_Type );
+
+	if( !pyNurb )
+		return EXPP_ReturnPyObjError( PyExc_MemoryError,
+					      "could not create BPy_CurNurb PyObject" );
+
+	pyNurb->nurb = blen_nurb;
+	return ( PyObject * ) pyNurb;
+}
+
+
+/*
+ *  CurNurb_repr
+ */
+static PyObject *CurNurb_repr( BPy_CurNurb * self )
+{				/* used by 'repr' */
+
+	return PyString_FromFormat( "[CurNurb \"%d\"]", self->nurb->type );
+}
+
+/* XXX Can't this be simply removed? */
+static PyObject *M_CurNurb_New( PyObject * self, PyObject * args )
+{
+	return ( PyObject * ) 0;
+
+}
+
+/*
+ *	Curve.getType
+ */
+static PyObject *CurNurb_getType( BPy_CurNurb * self )
+{
+	/* type is on 3 first bits only */
+	return PyInt_FromLong( self->nurb->type & 7 );
+}
+
+/*
+ *	Curve.setType
+ *
+ *	Convert the curve using Blender's convertspline fonction
+ */
+static PyObject *CurNurb_setType( BPy_CurNurb * self, PyObject * args )
+{
+	Nurb *nurb = self->nurb;
+	short type;
+
+	/* parameter type checking */
+	if( !PyArg_ParseTuple( args, "h", &type ) )
+		return EXPP_ReturnPyObjError
+			( PyExc_TypeError, "expected integer argument" );
+
+	/* parameter value checking */
+	if (type != CU_POLY &&
+		type != CU_BEZIER &&
+		type != CU_NURBS)
+		return EXPP_ReturnPyObjError
+			 ( PyExc_ValueError, "expected integer argument" );
+
+	/* convert and raise error if impossible */
+	if (convertspline(type, nurb))
+		return EXPP_ReturnPyObjError
+			 ( PyExc_ValueError, "Conversion Impossible" );
+
+	return EXPP_incr_ret( Py_None );
+}
+
+
+
+/*
+ * CurNurb_append( point )
+ * append a new point to a nurb curve.
+ * arg is BezTriple or list of xyzw floats 
+ */
+
+PyObject *CurNurb_append( BPy_CurNurb * self, PyObject * args )
+{
+	Nurb *nurb = self->nurb;
+
+	return CurNurb_appendPointToNurb( nurb, args );
+}
+
+
+/*
+ * CurNurb_appendPointToNurb
+ * this is a non-bpy utility func to add a point to a given nurb.
+ * notice the first arg is Nurb*.
+ */
+
+PyObject *CurNurb_appendPointToNurb( Nurb * nurb, PyObject * args )
+{
+
+	int i;
+	int size;
+	PyObject *pyOb;
+	int npoints = nurb->pntsu;
+
+	/*
+	   do we have a list of four floats or a BezTriple?
+	*/
+	if( !PyArg_ParseTuple( args, "O", &pyOb ))
+		return EXPP_ReturnPyObjError
+				( PyExc_RuntimeError,
+				  "Internal error parsing arguments" );
+
+
+
+	/* if curve is empty, adjust type depending on input type */
+	if (nurb->bezt==NULL && nurb->bp==NULL) {
+		if (BezTriple_CheckPyObject( pyOb ))
+			nurb->type |= CU_BEZIER;
+		else if (PySequence_Check( pyOb ))
+			nurb->type |= CU_NURBS;
+		else
+			return( EXPP_ReturnPyObjError( PyExc_TypeError,
+					  "Expected a BezTriple or a Sequence of 4 (or 5) floats" ) );
+	}
+
+
+
+	if ((nurb->type & 7)==CU_BEZIER) {
+		BezTriple *tmp;
+
+		if( !BezTriple_CheckPyObject( pyOb ) )
+			return( EXPP_ReturnPyObjError( PyExc_TypeError,
+					  "Expected a BezTriple\n" ) );
+
+/*		printf("\ndbg: got a BezTriple\n"); */
+		tmp = nurb->bezt;	/* save old points */
+		nurb->bezt =
+			( BezTriple * ) MEM_mallocN( sizeof( BezTriple ) *
+						     ( npoints + 1 ),
+						     "CurNurb_append2" );
+
+		if( !nurb->bezt )
+			return ( EXPP_ReturnPyObjError
+				 ( PyExc_MemoryError, "allocation failed" ) );
+
+		/* copy old points to new */
+		if( tmp ) {
+			memmove( nurb->bezt, tmp, sizeof( BezTriple ) * npoints );
+			MEM_freeN( tmp );
+		}
+
+		nurb->pntsu++;
+		/* add new point to end of list */
+		memcpy( nurb->bezt + npoints,
+			BezTriple_FromPyObject( pyOb ), sizeof( BezTriple ) );
+
+	}
+	else if( PySequence_Check( pyOb ) ) {
+		size = PySequence_Size( pyOb );
+/*		printf("\ndbg: got a sequence of size %d\n", size );  */
+		if( size == 4 || size == 5 ) {
+			BPoint *tmp;
+
+			tmp = nurb->bp;	/* save old pts */
+
+			nurb->bp =
+				( BPoint * ) MEM_mallocN( sizeof( BPoint ) *
+							  ( npoints + 1 ),
+							  "CurNurb_append1" );
+			if( !nurb->bp )
+				return ( EXPP_ReturnPyObjError
+					 ( PyExc_MemoryError,
+					   "allocation failed" ) );
+
+			memmove( nurb->bp, tmp, sizeof( BPoint ) * npoints );
+			if( tmp )
+				MEM_freeN( tmp );
+
+			++nurb->pntsu;
+			/* initialize new BPoint from old */
+			memcpy( nurb->bp + npoints, nurb->bp,
+				sizeof( BPoint ) );
+
+			for( i = 0; i < 4; ++i ) {
+				PyObject *item = PySequence_GetItem( pyOb, i );
+
+				if (item == NULL)
+					return NULL;
+
+
+				nurb->bp[npoints].vec[i] = ( float ) PyFloat_AsDouble( item );
+				Py_DECREF( item );
+			}
+
+			if (size == 5) {
+				PyObject *item = PySequence_GetItem( pyOb, i );
+
+				if (item == NULL)
+					return NULL;
+
+				nurb->bp[npoints].alfa = ( float ) PyFloat_AsDouble( item );
+				Py_DECREF( item );
+			}
+			else {
+				nurb->bp[npoints].alfa = 0.0f;
+			}
+
+			makeknots( nurb, 1, nurb->flagu >> 1 );
+
+		} else {
+			return EXPP_ReturnPyObjError( PyExc_TypeError,
+					"expected a sequence of 4 or 5 floats" );
+		}
+
+	} else {
+		/* bail with error */
+		return EXPP_ReturnPyObjError( PyExc_TypeError,
+					"expected a sequence of 4 or 5 floats" );
+
+	}
+
+	return ( EXPP_incr_ret( Py_None ) );
+}
+
+
+/*
+ *  CurNurb_setMatIndex
+ *
+ *  set index into material list
+ */
+
+static PyObject *CurNurb_setMatIndex( BPy_CurNurb * self, PyObject * args )
+{
+	int index;
+
+	if( !PyArg_ParseTuple( args, "i", &( index ) ) )
+		return ( EXPP_ReturnPyObjError
+			 ( PyExc_AttributeError,
+			   "expected integer argument" ) );
+
+	/* fixme:  some range checking would be nice! */
+	self->nurb->mat_nr = (short)index;
+
+	Py_INCREF( Py_None );
+	return Py_None;
+}
+
+/*
+ * CurNurb_getMatIndex
+ *
+ * returns index into material list
+ */
+
+static PyObject *CurNurb_getMatIndex( BPy_CurNurb * self )
+{
+	PyObject *index = PyInt_FromLong( ( long ) self->nurb->mat_nr );
+
+	if( index )
+		return index;
+
+	return ( EXPP_ReturnPyObjError( PyExc_RuntimeError,
+					"could not get material index" ) );
+}
+
+/*
+ * CurNurb_getFlagU
+ *
+ * returns curve's flagu
+ */
+
+static PyObject *CurNurb_getFlagU( BPy_CurNurb * self )
+{
+	PyObject *flagu = PyInt_FromLong( ( long ) self->nurb->flagu );
+
+	if( flagu )
+		return flagu;
+
+	return ( EXPP_ReturnPyObjError( PyExc_RuntimeError,
+					"could not get CurNurb.flagu index" ) );
+}
+
+/*
+ *  CurNurb_setFlagU
+ *
+ *  set curve's flagu and recalculate the knots
+ *
+ *  Possible values: 0 - uniform, 2 - endpoints, 4 - bezier
+ *    bit 0 controls CU_CYCLIC
+ */
+
+static PyObject *CurNurb_setFlagU( BPy_CurNurb * self, PyObject * args )
+{
+	int flagu;
+
+	if( !PyArg_ParseTuple( args, "i", &( flagu ) ) )
+		return EXPP_ReturnPyObjError( PyExc_TypeError,
+				"expected integer argument in range [0,5]" );
+
+	if( flagu < 0 || flagu > 5 )
+		return EXPP_ReturnPyObjError( PyExc_AttributeError,
+				"expected integer argument in range [0,5]" );
+
+	if( self->nurb->flagu != flagu ) {
+		self->nurb->flagu = (short)flagu;
+		makeknots( self->nurb, 1, self->nurb->flagu >> 1 );
+	}
+
+	Py_INCREF( Py_None );
+	return Py_None;
+}
+
+/*
+ * CurNurb_getFlagV
+ *
+ * returns curve's flagu
+ */
+
+static PyObject *CurNurb_getFlagV( BPy_CurNurb * self )
+{
+	PyObject *flagv = PyInt_FromLong( ( long ) self->nurb->flagv );
+
+	if( flagv )
+		return flagv;
+
+	return ( EXPP_ReturnPyObjError( PyExc_RuntimeError,
+					"could not get CurNurb.flagv" ) );
+}
+
+/*
+ *  CurNurb_setFlagV
+ *
+ *  set curve's flagu and recalculate the knots
+ *
+ *  Possible values: 0 - uniform, 1 - endpoints, 2 - bezier
+ */
+
+static PyObject *CurNurb_setFlagV( BPy_CurNurb * self, PyObject * args )
+{
+	int flagv;
+
+	if( !PyArg_ParseTuple( args, "i", &( flagv ) ) )
+		return EXPP_ReturnPyObjError( PyExc_TypeError,
+				"expected integer argument in range [0,5]" );
+
+	if( flagv < 0 || flagv > 5 )
+		return EXPP_ReturnPyObjError( PyExc_AttributeError,
+				"expected integer argument in range [0,5]" );
+
+	if( self->nurb->flagv != flagv ) {
+		self->nurb->flagv = (short)flagv;
+		makeknots( self->nurb, 2, self->nurb->flagv >> 1 );
+	}
+
+	Py_INCREF( Py_None );
+	return Py_None;
+}
+
+/*
+ * CurNurb_getIter
+ *
+ * create an iterator for our CurNurb.
+ * this iterator returns the points for this CurNurb.
+ */
+
+static PyObject *CurNurb_getIter( BPy_CurNurb * self )
+{
+	self->bp = self->nurb->bp;
+	self->bezt = self->nurb->bezt;
+	self->atEnd = 0;
+	self->nextPoint = 0;
+
+	/* set exhausted flag if both bp and bezt are zero */
+	if( ( !self->bp ) && ( !self->bezt ) )
+		self->atEnd = 1;
+
+	Py_INCREF( self );
+	return ( PyObject * ) self;
+}
+
+
+
+static PyObject *CurNurb_iterNext( BPy_CurNurb * self )
+{
+	PyObject *po;		/* return value */
+	Nurb *pnurb = self->nurb;
+	int npoints = pnurb->pntsu;
+
+	/* are we at end already? */
+	if( self->atEnd )
+		return ( EXPP_ReturnPyObjError( PyExc_StopIteration,
+						"iterator at end" ) );
+
+	if( self->nextPoint < npoints ) {
+
+		po = CurNurb_pointAtIndex( self->nurb, self->nextPoint );
+		self->nextPoint++;
+
+		return po;
+
+	} else {
+		self->atEnd = 1;	/* set flag true */
+	}
+
+	return ( EXPP_ReturnPyObjError( PyExc_StopIteration,
+					"iterator at end" ) );
+}
+
+
+
+/*
+ * CurNurb_isNurb()
+ * test whether spline nurb or bezier
+ */
+
+static PyObject *CurNurb_isNurb( BPy_CurNurb * self )
+{
+	/* NOTE: a Nurb has bp and bezt pointers
+	 * depending on type.
+	 * It is possible both are NULL if no points exist.
+	 * in that case, we return False
+	 */
+
+	if( self->nurb->bp ) {
+		return EXPP_incr_ret_True();
+	} else {
+		return EXPP_incr_ret_False();
+	}
+}
+
+/*
+ * CurNurb_isCyclic()
+ * test whether spline cyclic (closed) or not (open)
+ */
+
+static PyObject *CurNurb_isCyclic( BPy_CurNurb * self )
+{
+        /* supposing that the flagu is always set */ 
+
+	if( self->nurb->flagu & CU_CYCLIC ) {
+		return EXPP_incr_ret_True();
+	} else {
+		return EXPP_incr_ret_False();
+	}
+}
+
 /*
  * CurNurb_length
  * returns the number of points in a Nurb
@@ -605,7 +828,6 @@ static int CurNurb_length( PyInstanceObject * inst )
 
 PyObject *CurNurb_getPoint( BPy_CurNurb * self, int index )
 {
-	PyObject *pyo;
 	Nurb *myNurb;
 
 	int npoints;
@@ -617,19 +839,97 @@ PyObject *CurNurb_getPoint( BPy_CurNurb * self, int index )
 	/* DELETED: bail if index < 0 */
 	/* actually, this check is not needed since python treats */
 	/* negative indices as starting from the right end of a sequence */
+	/* 
+	THAT IS WRONG, when passing a negative index, python adjusts it to be positive
+	BUT it can still overflow in the negatives if the index is too small.
+	For example, list[-6] when list contains 5 items means index = -1 in here.
+	(theeth)
+	*/
 
 	/* bail if no Nurbs in Curve */
 	if( npoints == 0 )
 		return ( EXPP_ReturnPyObjError( PyExc_IndexError,
 						"no points in this CurNurb" ) );
 
-	if( index >= npoints )	/* out of range!  */
+	/* check index limits */
+	if( index >= npoints || index < 0 )
 		return ( EXPP_ReturnPyObjError( PyExc_IndexError,
 						"index out of range" ) );
 
-	pyo = CurNurb_pointAtIndex( myNurb, index );
+	return CurNurb_pointAtIndex( myNurb, index );
+}
 
-	return ( PyObject * ) pyo;
+/*
+ * CurNurb_setPoint
+ * modifies the Nth point in a Nurb
+ * this is one of the tp_as_sequence methods, hence the int N argument.
+ * it is called via the [] = operator, not as a usual instance method.
+ */
+static int CurNurb_setPoint( BPy_CurNurb * self, int index, PyObject * pyOb )
+{
+	Nurb *nurb = self->nurb;
+	int size;
+
+	/* check index limits */
+	if( index < 0 || index >= nurb->pntsu )
+		return EXPP_ReturnIntError( PyExc_IndexError,
+					    "array assignment index out of range\n" );
+
+
+	/* branch by curve type */
+	if ((nurb->type & 7)==CU_BEZIER) {	/* BEZIER */
+		/* check parameter type */
+		if( !BezTriple_CheckPyObject( pyOb ) )
+			return EXPP_ReturnIntError( PyExc_TypeError,
+							"expected a BezTriple\n" );
+
+		/* copy bezier in array */
+		memcpy( nurb->bezt + index,
+			BezTriple_FromPyObject( pyOb ), sizeof( BezTriple ) );
+
+		return 0;	/* finished correctly */
+	}
+	else {	/* NURBS or POLY */
+		int i;
+
+		/* check parameter type */
+		if (!PySequence_Check( pyOb ))
+			return EXPP_ReturnIntError( PyExc_TypeError,
+							"expected a list of 4 (or optionaly 5 if the curve is 3D) floats\n" );
+
+		size = PySequence_Size( pyOb );
+
+		/* check sequence size */
+		if( size != 4 && size != 5 ) 
+			return EXPP_ReturnIntError( PyExc_TypeError,
+							"expected a list of 4 (or optionaly 5 if the curve is 3D) floats\n" );
+
+		/* copy x, y, z, w */
+		for( i = 0; i < 4; ++i ) {
+			PyObject *item = PySequence_GetItem( pyOb, i );
+
+			if (item == NULL)
+				return -1;
+
+			nurb->bp[index].vec[i] = ( float ) PyFloat_AsDouble( item );
+			Py_DECREF( item );
+		}
+
+		if (size == 5) {	/* set tilt, if present */
+			PyObject *item = PySequence_GetItem( pyOb, i );
+
+			if (item == NULL)
+				return -1;
+
+			nurb->bp[index].alfa = ( float ) PyFloat_AsDouble( item );
+			Py_DECREF( item );
+		}
+		else {				/* if not, set default	*/
+			nurb->bp[index].alfa = 0.0f;
+		}
+
+		return 0;	/* finished correctly */
+	}
 }
 
 
@@ -643,15 +943,23 @@ PyObject *CurNurb_pointAtIndex( Nurb * nurb, int index )
 
 	if( nurb->bp ) {	/* we have a nurb curve */
 		int i;
-		pyo = PyList_New( 4 );
+
+		/* add Tilt only if curve is 3D */
+		if (nurb->flag & CU_3D)
+			pyo = PyList_New( 5 );
+		else
+			pyo = PyList_New( 4 );
 
 		for( i = 0; i < 4; i++ ) {
 			PyList_SetItem( pyo, i,
 					PyFloat_FromDouble( nurb->bp[index].
 							    vec[i] ) );
-
 		}
 
+		/* add Tilt only if curve is 3D */
+		if (nurb->flag & CU_3D)
+			PyList_SetItem( pyo, 4,	PyFloat_FromDouble( nurb->bp[index].alfa ) );
+
 	} else if( nurb->bezt ) {	/* we have a bezier */
 		/* if an error occurs, we just pass it on */
 		pyo = BezTriple_CreatePyObject( &( nurb->bezt[index] ) );
@@ -682,3 +990,75 @@ PyObject *CurNurb_Init( void )
 				M_CurNurb_doc );
 	return ( submodule );
 }
+
+
+/*
+  dump nurb
+*/
+
+PyObject *CurNurb_dump( BPy_CurNurb * self )
+{
+	BPoint *bp = NULL;
+	BezTriple *bezt = NULL;
+	Nurb *nurb = self->nurb;
+	int npoints = 0;
+
+	if( ! self->nurb ){  /* bail on error */
+		printf("\n no Nurb in this CurNurb");
+		Py_RETURN_NONE;
+	}
+
+	printf(" type: %d, mat_nr: %d hide: %d flag: %d",
+		   nurb->type, nurb->mat_nr, nurb->hide, nurb->flag);
+	printf("\n pntsu: %d, pntsv: %d, resolu: %d resolv: %d",
+		   nurb->pntsu, nurb->pntsv, nurb->resolu, nurb->resolv );
+	printf("\n orderu: %d  orderv: %d", nurb->orderu, nurb->orderv );
+	printf("\n flagu: %d flagv: %d",
+		   nurb->flagu, nurb->flagv );
+
+	npoints = nurb->pntsu;
+
+	if( nurb->bp ) { /* we have a BPoint  */
+		int n;
+		for( n = 0, bp = nurb->bp;
+			 n < npoints;
+			 n++, bp++ )
+		{
+			/* vec[4] */
+			printf( "\ncoords[%d]: ", n);
+			{
+				int i;
+				for( i = 0; i < 4; i++){
+					printf("%10.3f ", bp->vec[i] );
+				}
+			}
+		
+			/* alfa, s[2] */
+			printf("\n alpha: %5.2f", bp->alfa);
+			/* f1, hide */
+			printf(" f1 %d  hide %d", bp->f1, bp->hide );
+			printf("\n");
+		}
+	}
+	else { /* we have a BezTriple */
+		int n;
+		for( n = 0, bezt = nurb->bezt;
+			 n < npoints;
+			 n++, bezt++ )
+		{
+			int i, j;
+			printf("\npoint %d: ", n);
+			for( i = 0; i < 3; i++ ) {
+				printf("\nvec[%i] ",i );
+				for( j = 0; j < 3; j++ ) {
+					printf(" %5.2f ", bezt->vec[i][j] );
+				}
+			}
+				
+
+		}
+		printf("\n");
+	}
+
+	Py_RETURN_NONE;
+}
