@@ -1,5 +1,5 @@
 /* 
- * $Id: Curve.c,v 1.21 2004/10/07 19:25:39 stiv Exp $
+ * $Id: Curve.c,v 1.38 2005/11/29 01:35:45 khughes Exp $
  * ***** BEGIN GPL/BL DUAL LICENSE BLOCK *****
  *
  * This program is free software; you can redistribute it and/or
@@ -29,24 +29,20 @@
  * ***** END GPL/BL DUAL LICENSE BLOCK *****
  */
 
-#include <Python.h>
-#include "Curve.h"
-#include <stdio.h>
-
-#include <BLI_arithb.h>
-#include <BLI_blenlib.h>
-#include <BKE_main.h>
-#include <BKE_displist.h>
-#include <BKE_global.h>
-#include <BKE_object.h>
-#include <BKE_library.h>
-#include <BKE_curve.h>
-#include <BKE_utildefines.h>
-#include <MEM_guardedalloc.h>	/* because we wil be mallocing memory */
+#include "Curve.h" /*This must come first*/
 
+#include "BKE_main.h"
+#include "BKE_displist.h"
+#include "BKE_global.h"
+#include "BKE_library.h"
+#include "BKE_curve.h"
+#include "MEM_guardedalloc.h"	/* because we wil be mallocing memory */
 #include "CurNurb.h"
 #include "Material.h"
+#include "Object.h"
+#include "Key.h"
 #include "gen_utils.h"
+#include "mydevice.h"
 
 
 /*****************************************************************************/
@@ -76,26 +72,27 @@ static PyObject *M_Curve_Get( PyObject * self, PyObject * args );
 /*****************************************************************************/
 /*  Python BPy_Curve instance methods declarations:                          */
 /*****************************************************************************/
-static PyObject *Curve_getName( BPy_Curve * self );
-static PyObject *Curve_setName( BPy_Curve * self, PyObject * args );
+
+PyObject *Curve_getName( BPy_Curve * self );
+PyObject *Curve_setName( BPy_Curve * self, PyObject * args );
 static PyObject *Curve_getPathLen( BPy_Curve * self );
 static PyObject *Curve_setPathLen( BPy_Curve * self, PyObject * args );
 static PyObject *Curve_getTotcol( BPy_Curve * self );
 static PyObject *Curve_setTotcol( BPy_Curve * self, PyObject * args );
-static PyObject *Curve_getMode( BPy_Curve * self );
-static PyObject *Curve_setMode( BPy_Curve * self, PyObject * args );
-static PyObject *Curve_getBevresol( BPy_Curve * self );
-static PyObject *Curve_setBevresol( BPy_Curve * self, PyObject * args );
-static PyObject *Curve_getResolu( BPy_Curve * self );
-static PyObject *Curve_setResolu( BPy_Curve * self, PyObject * args );
-static PyObject *Curve_getResolv( BPy_Curve * self );
-static PyObject *Curve_setResolv( BPy_Curve * self, PyObject * args );
-static PyObject *Curve_getWidth( BPy_Curve * self );
-static PyObject *Curve_setWidth( BPy_Curve * self, PyObject * args );
-static PyObject *Curve_getExt1( BPy_Curve * self );
-static PyObject *Curve_setExt1( BPy_Curve * self, PyObject * args );
-static PyObject *Curve_getExt2( BPy_Curve * self );
-static PyObject *Curve_setExt2( BPy_Curve * self, PyObject * args );
+PyObject *Curve_getMode( BPy_Curve * self );
+PyObject *Curve_setMode( BPy_Curve * self, PyObject * args );
+PyObject *Curve_getBevresol( BPy_Curve * self );
+PyObject *Curve_setBevresol( BPy_Curve * self, PyObject * args );
+PyObject *Curve_getResolu( BPy_Curve * self );
+PyObject *Curve_setResolu( BPy_Curve * self, PyObject * args );
+PyObject *Curve_getResolv( BPy_Curve * self );
+PyObject *Curve_setResolv( BPy_Curve * self, PyObject * args );
+PyObject *Curve_getWidth( BPy_Curve * self );
+PyObject *Curve_setWidth( BPy_Curve * self, PyObject * args );
+PyObject *Curve_getExt1( BPy_Curve * self );
+PyObject *Curve_setExt1( BPy_Curve * self, PyObject * args );
+PyObject *Curve_getExt2( BPy_Curve * self );
+PyObject *Curve_setExt2( BPy_Curve * self, PyObject * args );
 static PyObject *Curve_getControlPoint( BPy_Curve * self, PyObject * args );
 static PyObject *Curve_setControlPoint( BPy_Curve * self, PyObject * args );
 static PyObject *Curve_getLoc( BPy_Curve * self );
@@ -105,7 +102,9 @@ static PyObject *Curve_setRot( BPy_Curve * self, PyObject * args );
 static PyObject *Curve_getSize( BPy_Curve * self );
 static PyObject *Curve_setSize( BPy_Curve * self, PyObject * args );
 static PyObject *Curve_getNumCurves( BPy_Curve * self );
+static PyObject *Curve_getKey( BPy_Curve * self );
 static PyObject *Curve_isNurb( BPy_Curve * self, PyObject * args );
+static PyObject *Curve_isCyclic( BPy_Curve * self, PyObject * args);
 static PyObject *Curve_getNumPoints( BPy_Curve * self, PyObject * args );
 static PyObject *Curve_getNumPoints( BPy_Curve * self, PyObject * args );
 
@@ -114,14 +113,19 @@ static PyObject *Curve_appendNurb( BPy_Curve * self, PyObject * args );
 
 static PyObject *Curve_getMaterials( BPy_Curve * self );
 
+static PyObject *Curve_getBevOb( BPy_Curve * self );
+static PyObject *Curve_setBevOb( BPy_Curve * self, PyObject * args );
+
+static PyObject *Curve_getTaperOb( BPy_Curve * self );
+static PyObject *Curve_setTaperOb( BPy_Curve * self, PyObject * args );
+
 static PyObject *Curve_getIter( BPy_Curve * self );
 static PyObject *Curve_iterNext( BPy_Curve * self );
-static PyObject *Curve_update( BPy_Curve * self );
+
 PyObject *Curve_getNurb( BPy_Curve * self, int n );
 static int Curve_length( PyInstanceObject * inst );
-void update_displists( void *data );
 
-void makeDispList( Object * ob );
+
 struct chartrans *text_to_curve( Object * ob, int mode );
 
 
@@ -201,9 +205,13 @@ Sets a control point "},
 	 METH_VARARGS, "(3-tuple) - Sets curve size"},
 	{"getNumCurves", ( PyCFunction ) Curve_getNumCurves,
 	 METH_NOARGS, "() - Gets number of curves in Curve"},
+	{"getKey", ( PyCFunction ) Curve_getKey,
+	 METH_NOARGS, "() - Gets curve key"},
 	{"isNurb", ( PyCFunction ) Curve_isNurb,
 	 METH_VARARGS,
 	 "(nothing or integer) - returns 1 if curve is type Nurb, O otherwise."},
+	{"isCyclic", ( PyCFunction ) Curve_isCyclic,
+	 METH_VARARGS, "( nothing or integer ) - returns true if curve is cyclic (closed), false otherwise."},
 	{"getNumPoints", ( PyCFunction ) Curve_getNumPoints,
 	 METH_VARARGS,
 	 "(nothing or integer) - returns the number of points of the specified curve"},
@@ -215,6 +223,14 @@ Sets a control point "},
 	 "( ) - updates display lists after changes to Curve"},
 	{"getMaterials", ( PyCFunction ) Curve_getMaterials, METH_NOARGS,
 	 "() - returns list of materials assigned to this Curve"},
+	{"getBevOb", ( PyCFunction ) Curve_getBevOb, METH_NOARGS,
+	 "() - returns Bevel Object assigned to this Curve"},
+	{"setBevOb", ( PyCFunction ) Curve_setBevOb, METH_VARARGS,
+	 "() - assign a Bevel Object to this Curve"},
+	{"getTaperOb", ( PyCFunction ) Curve_getTaperOb, METH_NOARGS,
+	 "() - returns Taper Object assigned to this Curve"},
+	{"setTaperOb", ( PyCFunction ) Curve_setTaperOb, METH_VARARGS,
+	 "() - assign a Taper Object to this Curve"},
 	{NULL, NULL, 0, NULL}
 };
 
@@ -425,7 +441,8 @@ PyObject *Curve_Init( void )
 {
 	PyObject *submodule;
 
-	Curve_Type.ob_type = &PyType_Type;
+	if( PyType_Ready( &Curve_Type) < 0) /* set exception.  -1 is failure */
+		return NULL;
 
 	submodule =
 		Py_InitModule3( "Blender.Curve", M_Curve_methods,
@@ -443,7 +460,7 @@ PyObject *Curve_Init( void )
 /*****************************************************************************/
 
 
-static PyObject *Curve_getName( BPy_Curve * self )
+PyObject *Curve_getName( BPy_Curve * self )
 {
 	PyObject *attr = PyString_FromString( self->curve->id.name + 2 );
 
@@ -454,7 +471,7 @@ static PyObject *Curve_getName( BPy_Curve * self )
 					"couldn't get Curve.name attribute" ) );
 }
 
-static PyObject *Curve_setName( BPy_Curve * self, PyObject * args )
+PyObject *Curve_setName( BPy_Curve * self, PyObject * args )
 {
 	char *name;
 	char buf[50];
@@ -465,6 +482,8 @@ static PyObject *Curve_setName( BPy_Curve * self, PyObject * args )
 	PyOS_snprintf( buf, sizeof( buf ), "%s", name );
 	rename_id( &self->curve->id, buf );	/* proper way in Blender */
 
+	Curve_update( self );
+
 	Py_INCREF( Py_None );
 	return Py_None;
 }
@@ -517,7 +536,7 @@ static PyObject *Curve_setTotcol( BPy_Curve * self, PyObject * args )
 }
 
 
-static PyObject *Curve_getMode( BPy_Curve * self )
+PyObject *Curve_getMode( BPy_Curve * self )
 {
 	PyObject *attr = PyInt_FromLong( ( long ) self->curve->flag );
 
@@ -529,7 +548,7 @@ static PyObject *Curve_getMode( BPy_Curve * self )
 }
 
 
-static PyObject *Curve_setMode( BPy_Curve * self, PyObject * args )
+PyObject *Curve_setMode( BPy_Curve * self, PyObject * args )
 {
 
 	if( !PyArg_ParseTuple( args, "i", &( self->curve->flag ) ) )
@@ -541,7 +560,7 @@ static PyObject *Curve_setMode( BPy_Curve * self, PyObject * args )
 }
 
 
-static PyObject *Curve_getBevresol( BPy_Curve * self )
+PyObject *Curve_getBevresol( BPy_Curve * self )
 {
 	PyObject *attr = PyInt_FromLong( ( long ) self->curve->bevresol );
 
@@ -553,19 +572,24 @@ static PyObject *Curve_getBevresol( BPy_Curve * self )
 }
 
 
-static PyObject *Curve_setBevresol( BPy_Curve * self, PyObject * args )
+PyObject *Curve_setBevresol( BPy_Curve * self, PyObject * args )
 {
+	short value;
 
-	if( !PyArg_ParseTuple( args, "i", &( self->curve->bevresol ) ) )
+	if( !PyArg_ParseTuple( args, "h", &value ) )
 		return ( EXPP_ReturnPyObjError( PyExc_AttributeError,
-						"expected int argument" ) );
+			"expected integer argument" ) );
 
-	Py_INCREF( Py_None );
-	return Py_None;
+	if(value > 10 || value < 0)
+		return ( EXPP_ReturnPyObjError( PyExc_AttributeError,
+			"acceptable values are between 10 and 0" ) );
+	self->curve->bevresol = value;
+
+	return EXPP_incr_ret( Py_None );
 }
 
 
-static PyObject *Curve_getResolu( BPy_Curve * self )
+PyObject *Curve_getResolu( BPy_Curve * self )
 {
 	PyObject *attr = PyInt_FromLong( ( long ) self->curve->resolu );
 
@@ -577,20 +601,30 @@ static PyObject *Curve_getResolu( BPy_Curve * self )
 }
 
 
-static PyObject *Curve_setResolu( BPy_Curve * self, PyObject * args )
+PyObject *Curve_setResolu( BPy_Curve * self, PyObject * args )
 {
+	short value;
+	Nurb *nu;
 
-	if( !PyArg_ParseTuple( args, "i", &( self->curve->resolu ) ) )
+	if( !PyArg_ParseTuple( args, "h", &value ) )
 		return ( EXPP_ReturnPyObjError( PyExc_AttributeError,
-						"expected int argument" ) );
+			"expected integer argument" ) );
 
-	Py_INCREF( Py_None );
-	return Py_None;
+	if(value > 128 || value < 1)
+		return ( EXPP_ReturnPyObjError( PyExc_AttributeError,
+			"acceptable values are between 128 and 1" ) );
+	self->curve->resolu = value;
+
+	/* propagate the change through all the curves */
+	for ( nu = self->curve->nurb.first; nu; nu = nu->next )
+		nu->resolu = value;
+
+	return EXPP_incr_ret( Py_None );
 }
 
 
 
-static PyObject *Curve_getResolv( BPy_Curve * self )
+PyObject *Curve_getResolv( BPy_Curve * self )
 {
 	PyObject *attr = PyInt_FromLong( ( long ) self->curve->resolv );
 
@@ -602,20 +636,25 @@ static PyObject *Curve_getResolv( BPy_Curve * self )
 }
 
 
-static PyObject *Curve_setResolv( BPy_Curve * self, PyObject * args )
+PyObject *Curve_setResolv( BPy_Curve * self, PyObject * args )
 {
+	short value;
 
-	if( !PyArg_ParseTuple( args, "i", &( self->curve->resolv ) ) )
+	if( !PyArg_ParseTuple( args, "h", &value ) )
 		return ( EXPP_ReturnPyObjError( PyExc_AttributeError,
-						"expected int argument" ) );
+			"expected integer argument" ) );
 
-	Py_INCREF( Py_None );
-	return Py_None;
+	if(value > 128 || value < 1)
+		return ( EXPP_ReturnPyObjError( PyExc_AttributeError,
+			"acceptable values are between 128 and 1" ) );
+	self->curve->resolv = value;
+
+	return EXPP_incr_ret( Py_None );
 }
 
 
 
-static PyObject *Curve_getWidth( BPy_Curve * self )
+PyObject *Curve_getWidth( BPy_Curve * self )
 {
 	PyObject *attr = PyFloat_FromDouble( ( double ) self->curve->width );
 
@@ -627,19 +666,24 @@ static PyObject *Curve_getWidth( BPy_Curve * self )
 }
 
 
-static PyObject *Curve_setWidth( BPy_Curve * self, PyObject * args )
+PyObject *Curve_setWidth( BPy_Curve * self, PyObject * args )
 {
+	float value;
 
-	if( !PyArg_ParseTuple( args, "f", &( self->curve->width ) ) )
+	if( !PyArg_ParseTuple( args, "f", &value ) )
 		return ( EXPP_ReturnPyObjError( PyExc_AttributeError,
-						"expected float argument" ) );
+			"expected float argument" ) );
 
-	Py_INCREF( Py_None );
-	return Py_None;
+	if(value > 2.0f || value < 0.0f)
+		return ( EXPP_ReturnPyObjError( PyExc_AttributeError,
+			"acceptable values are between 2.0 and 0.0" ) );
+	self->curve->width = value;
+
+	return EXPP_incr_ret( Py_None );
 }
 
 
-static PyObject *Curve_getExt1( BPy_Curve * self )
+PyObject *Curve_getExt1( BPy_Curve * self )
 {
 	PyObject *attr = PyFloat_FromDouble( ( double ) self->curve->ext1 );
 
@@ -651,20 +695,25 @@ static PyObject *Curve_getExt1( BPy_Curve * self )
 }
 
 
-static PyObject *Curve_setExt1( BPy_Curve * self, PyObject * args )
+PyObject *Curve_setExt1( BPy_Curve * self, PyObject * args )
 {
+	float value;
 
-	if( !PyArg_ParseTuple( args, "f", &( self->curve->ext1 ) ) )
+	if( !PyArg_ParseTuple( args, "f", &value ) )
 		return ( EXPP_ReturnPyObjError( PyExc_AttributeError,
-						"expected float argument" ) );
+			"expected float argument" ) );
 
-	Py_INCREF( Py_None );
-	return Py_None;
+	if(value > 5.0f || value < 0.0f)
+		return ( EXPP_ReturnPyObjError( PyExc_AttributeError,
+			"acceptable values are between 5.0 and 0.0" ) );
+	self->curve->ext1 = value;
+
+	return EXPP_incr_ret( Py_None );
 }
 
 
 
-static PyObject *Curve_getExt2( BPy_Curve * self )
+PyObject *Curve_getExt2( BPy_Curve * self )
 {
 	PyObject *attr = PyFloat_FromDouble( ( double ) self->curve->ext2 );
 
@@ -676,15 +725,20 @@ static PyObject *Curve_getExt2( BPy_Curve * self )
 }
 
 
-static PyObject *Curve_setExt2( BPy_Curve * self, PyObject * args )
+PyObject *Curve_setExt2( BPy_Curve * self, PyObject * args )
 {
+	float value;
 
-	if( !PyArg_ParseTuple( args, "f", &( self->curve->ext2 ) ) )
+	if( !PyArg_ParseTuple( args, "f", &value ) )
 		return ( EXPP_ReturnPyObjError( PyExc_AttributeError,
-						"expected float argument" ) );
+			"expected float argument" ) );
 
-	Py_INCREF( Py_None );
-	return Py_None;
+	if(value > 2.0f || value < 0.0f)
+		return ( EXPP_ReturnPyObjError( PyExc_AttributeError,
+			"acceptable values are between 2.0 and 0.0" ) );
+	self->curve->ext2 = value;
+
+	return EXPP_incr_ret( Py_None );
 }
 
 
@@ -740,7 +794,7 @@ static PyObject *Curve_setControlPoint( BPy_Curve * self, PyObject * args )
 {
 	PyObject *listargs = 0;
 	Nurb *ptrnurb = self->curve->nurb.first;
-	int numcourbe, numpoint, i, j;
+	int numcourbe = 0, numpoint = 0, i, j;
 
 	if( !ptrnurb ) {
 		Py_INCREF( Py_None );
@@ -766,14 +820,14 @@ static PyObject *Curve_setControlPoint( BPy_Curve * self, PyObject * args )
 	if( ptrnurb->bp )
 		for( i = 0; i < 4; i++ )
 			ptrnurb->bp[numpoint].vec[i] =
-				PyFloat_AsDouble( PyList_GetItem
+				(float)PyFloat_AsDouble( PyList_GetItem
 						  ( listargs, i ) );
 
 	if( ptrnurb->bezt )
 		for( i = 0; i < 3; i++ )
 			for( j = 0; j < 3; j++ )
 				ptrnurb->bezt[numpoint].vec[i][j] =
-					PyFloat_AsDouble( PyList_GetItem
+					(float)PyFloat_AsDouble( PyList_GetItem
 							  ( listargs,
 							    i * 3 + j ) );
 
@@ -784,6 +838,7 @@ static PyObject *Curve_setControlPoint( BPy_Curve * self, PyObject * args )
 static PyObject *Curve_getControlPoint( BPy_Curve * self, PyObject * args )
 {
 	PyObject *liste = PyList_New( 0 );	/* return values */
+	PyObject *item;
 
 	Nurb *ptrnurb;
 	int i, j;
@@ -816,22 +871,18 @@ static PyObject *Curve_getControlPoint( BPy_Curve * self, PyObject * args )
 						"point index out of range" ) );
 
 	if( ptrnurb->bp ) {	/* if we are a nurb curve, you get 4 values */
-		for( i = 0; i < 4; i++ )
-			PyList_Append( liste,
-				       PyFloat_FromDouble( ptrnurb->
-							   bp[numpoint].
-							   vec[i] ) );
-	}
-
-	if( ptrnurb->bezt ) {	/* if we are a bezier, you get 9 values */
+		for( i = 0; i < 4; i++ ) {
+			item = PyFloat_FromDouble( ptrnurb->bp[numpoint].vec[i] );
+			PyList_Append( liste, item );
+			Py_DECREF(item);
+		}
+	} else if( ptrnurb->bezt ) {	/* if we are a bezier, you get 9 values */
 		for( i = 0; i < 3; i++ )
-			for( j = 0; j < 3; j++ )
-				PyList_Append( liste,
-					       PyFloat_FromDouble( ptrnurb->
-								   bezt
-								   [numpoint].
-								   vec[i]
-								   [j] ) );
+			for( j = 0; j < 3; j++ ) {
+				item = PyFloat_FromDouble( ptrnurb->bezt[numpoint].vec[i][j] );
+				PyList_Append( liste, item );
+				Py_DECREF(item);
+			}
 	}
 
 	return liste;
@@ -861,7 +912,7 @@ static PyObject *Curve_setLoc( BPy_Curve * self, PyObject * args )
 			 ( PyExc_TypeError, "expected a list" ) );
 	for( i = 0; i < 3; i++ ) {
 		PyObject *xx = PyList_GetItem( listargs, i );
-		self->curve->loc[i] = PyFloat_AsDouble( xx );
+		self->curve->loc[i] = (float)PyFloat_AsDouble( xx );
 	}
 	Py_INCREF( Py_None );
 	return Py_None;
@@ -891,7 +942,7 @@ static PyObject *Curve_setRot( BPy_Curve * self, PyObject * args )
 			 ( PyExc_TypeError, "expected a list" ) );
 	for( i = 0; i < 3; i++ ) {
 		PyObject *xx = PyList_GetItem( listargs, i );
-		self->curve->rot[i] = PyFloat_AsDouble( xx );
+		self->curve->rot[i] = (float)PyFloat_AsDouble( xx );
 	}
 	Py_INCREF( Py_None );
 	return Py_None;
@@ -920,7 +971,7 @@ static PyObject *Curve_setSize( BPy_Curve * self, PyObject * args )
 			 ( PyExc_TypeError, "expected a list" ) );
 	for( i = 0; i < 3; i++ ) {
 		PyObject *xx = PyList_GetItem( listargs, i );
-		self->curve->size[i] = PyFloat_AsDouble( xx );
+		self->curve->size[i] = (float)PyFloat_AsDouble( xx );
 	}
 	Py_INCREF( Py_None );
 	return Py_None;
@@ -941,7 +992,7 @@ static PyObject *Curve_getNumCurves( BPy_Curve * self )
 	/* get curve */
 	ptrnurb = self->curve->nurb.first;
 	if( ptrnurb ) {		/* we have some nurbs in this curve */
-		while( 1 ) {
+		for(;;) {
 			++num_curves;
 			ptrnurb = ptrnurb->next;
 			if( !ptrnurb )	/* no more curves */
@@ -959,6 +1010,20 @@ static PyObject *Curve_getNumCurves( BPy_Curve * self )
 					"couldn't get number of curves" ) );
 }
 
+/*
+ * get the key object linked to this curve
+ */
+
+static PyObject *Curve_getKey( BPy_Curve * self )
+{
+	PyObject *keyObj;
+
+	if (self->curve->key)
+		keyObj = Key_CreatePyObject(self->curve->key);
+	else keyObj = EXPP_incr_ret(Py_None);
+
+	return keyObj;
+}
 
 /*
  * count the number of points in a given spline
@@ -1058,6 +1123,48 @@ static PyObject *Curve_isNurb( BPy_Curve * self, PyObject * args )
 					"couldn't get curve type" ) );
 }
 
+/* trying to make a check for closedness (cyclic), following on isNurb (above) 
+   copy-pasting done by antont@kyperjokki.fi */
+
+static PyObject *Curve_isCyclic( BPy_Curve * self, PyObject * args )
+{
+	int curve_num = 0;	/* default value */
+	/* unused:*/
+	/* int is_cyclic;
+	 * PyObject *ret_val;*/
+	Nurb *ptrnurb;
+	int i;
+
+	/* parse and check input args */
+	if( !PyArg_ParseTuple( args, "|i", &curve_num ) ) {
+		return ( EXPP_ReturnPyObjError( PyExc_AttributeError,
+						"expected int argument" ) );
+	}
+	if( curve_num < 0 ) {
+		return ( EXPP_ReturnPyObjError( PyExc_AttributeError,
+						"curve number must be non-negative" ) );
+	}
+
+	ptrnurb = self->curve->nurb.first;
+
+	if( !ptrnurb )		/* no splines in this curve */
+		return ( EXPP_ReturnPyObjError( PyExc_AttributeError,
+						"no splines in this Curve" ) );
+
+	for( i = 0; i < curve_num; i++ ) {
+		ptrnurb = ptrnurb->next;
+		if( !ptrnurb )	/* if zero, we ran just ran out of curves */
+			return ( EXPP_ReturnPyObjError( PyExc_AttributeError,
+							"curve index out of range" ) );
+	}
+
+	if(  ptrnurb->flagu & CU_CYCLIC ){
+		return EXPP_incr_ret_True();
+	} else {
+		return EXPP_incr_ret_False();
+	}
+}
+
 
 /*
  * Curve_appendPoint( numcurve, new_point )
@@ -1069,6 +1176,8 @@ static PyObject *Curve_appendPoint( BPy_Curve * self, PyObject * args )
 	int i;
 	int nurb_num;		/* index of curve we append to */
 	PyObject *coord_args;	/* coords for new point */
+	PyObject *retval = NULL;
+	PyObject *valtuple;
 	Nurb *nurb = self->curve->nurb.first;	/* first nurb in Curve */
 
 /* fixme - need to malloc new Nurb */
@@ -1092,11 +1201,22 @@ static PyObject *Curve_appendPoint( BPy_Curve * self, PyObject * args )
 				   "curve index out of range" ) );
 	}
 
-	return CurNurb_appendPointToNurb( nurb, coord_args );
+	/* rebuild our arg tuple for appendPointToNurb() */
+	valtuple = Py_BuildValue( "(O)", coord_args );
+	
+	retval =  CurNurb_appendPointToNurb( nurb, valtuple );
+	Py_DECREF( valtuple );
 
+	return retval;
 }
 
 
+/****
+  appendNurb( new_point )
+  create a new nurb in the Curve and add the point param to it.
+  returns a refernce to the newly created nurb.
+*****/
+
 static PyObject *Curve_appendNurb( BPy_Curve * self, PyObject * args )
 {
 	Nurb *nurb_ptr = self->curve->nurb.first;
@@ -1120,8 +1240,11 @@ static PyObject *Curve_appendNurb( BPy_Curve * self, PyObject * args )
 
 	if( CurNurb_appendPointToNurb( new_nurb, args ) ) {
 		*pptr = new_nurb;
-		new_nurb->resolu = 12;
-		new_nurb->resolv = 12;
+		new_nurb->resolu = self->curve->resolu;
+		new_nurb->resolv = self->curve->resolv;
+		new_nurb->hide = 0;
+		new_nurb->flag = 1;
+
 
 		if( new_nurb->bezt ) {	/* do setup for bezt */
 			new_nurb->type = CU_BEZIER;
@@ -1130,6 +1253,7 @@ static PyObject *Curve_appendNurb( BPy_Curve * self, PyObject * args )
 			new_nurb->bezt->f1 = 1;
 			new_nurb->bezt->f2 = 1;
 			new_nurb->bezt->f3 = 1;
+			new_nurb->bezt->hide = 0;
 			/* calchandlesNurb( new_nurb ); */
 		} else {	/* set up bp */
 			new_nurb->pntsv = 1;
@@ -1138,6 +1262,7 @@ static PyObject *Curve_appendNurb( BPy_Curve * self, PyObject * args )
 			new_nurb->flagu = 0;
 			new_nurb->flagv = 0;
 			new_nurb->bp->f1 = 0;
+			new_nurb->bp->hide = 0;
 			new_nurb->knotsu = 0;
 			/*makenots( new_nurb, 1, new_nurb->flagu >> 1); */
 		}
@@ -1147,7 +1272,7 @@ static PyObject *Curve_appendNurb( BPy_Curve * self, PyObject * args )
 		return NULL;	/* with PyErr already set */
 	}
 
-	return EXPP_incr_ret( Py_None );
+	return CurNurb_CreatePyObject( new_nurb );
 }
 
 
@@ -1157,13 +1282,11 @@ static PyObject *Curve_appendNurb( BPy_Curve * self, PyObject * args )
  *   used. after messing with control points
  */
 
-static PyObject *Curve_update( BPy_Curve * self )
+PyObject *Curve_update( BPy_Curve * self )
 {
-/*	update_displists( ( void * ) self->curve ); */
-	freedisplist( &self->curve->disp );
+	Object_updateDag( (void*) self->curve );
 
-	Py_INCREF( Py_None );
-	return Py_None;
+	Py_RETURN_NONE;
 }
 
 /*
@@ -1178,7 +1301,98 @@ static PyObject *Curve_getMaterials( BPy_Curve * self )
 
 }
 
+/*****************************************************************************/
+/* Function:    Curve_getBevOb                                               */
+/* Description: Get the bevel object assign to the curve.                    */
+/*****************************************************************************/
+static PyObject *Curve_getBevOb( BPy_Curve * self)
+{
+	if( self->curve->bevobj ) {
+		return Object_CreatePyObject( self->curve->bevobj );
+	}
 
+	return EXPP_incr_ret( Py_None );
+}
+
+/*****************************************************************************/
+/* Function:    Curve_setBevOb                                               */
+/* Description: Assign a bevel object to the curve.                          */
+/*****************************************************************************/
+PyObject *Curve_setBevOb( BPy_Curve * self, PyObject * args )
+{
+	BPy_Object *pybevobj;
+
+	/* Parse and check input args */
+	if( !PyArg_ParseTuple( args, "O", &pybevobj) ) {
+		return ( EXPP_ReturnPyObjError( PyExc_AttributeError,
+					"expected object or None argument" ) );
+	}
+
+	/* Accept None */
+	if( (PyObject *)pybevobj == Py_None ) {
+		self->curve->bevobj = (Object *)NULL;
+	} else {
+	/* Accept Object with type 'Curve' */
+		if( Object_CheckPyObject( ( PyObject * ) pybevobj ) && 
+			pybevobj->object->type == OB_CURVE) {
+			self->curve->bevobj = 
+				Object_FromPyObject( ( PyObject * ) pybevobj );
+		} else {
+			return ( EXPP_ReturnPyObjError( PyExc_TypeError,
+						"expected Curve object type or None argument" ) );
+		}
+	}
+
+	return EXPP_incr_ret( Py_None );
+}
+
+/*****************************************************************************/
+/* Function:    Curve_getTaperOb                                               */
+/* Description: Get the taper object assign to the curve.                    */
+/*****************************************************************************/
+
+
+static PyObject *Curve_getTaperOb( BPy_Curve * self)
+{
+	if( self->curve->taperobj ) {
+		return Object_CreatePyObject( self->curve->taperobj );
+	}
+
+	return EXPP_incr_ret( Py_None );
+}
+
+/*****************************************************************************/
+/* Function:    Curve_setTaperOb                                               */
+/* Description: Assign a taper object to the curve.                          */
+/*****************************************************************************/
+
+PyObject *Curve_setTaperOb( BPy_Curve * self, PyObject * args )
+{
+	BPy_Object *pytaperobj;
+
+	/* Parse and check input args */
+	if( !PyArg_ParseTuple( args, "O", &pytaperobj) ) {
+		return ( EXPP_ReturnPyObjError( PyExc_AttributeError,
+					"expected object or None argument" ) );
+	}
+
+	/* Accept None */
+	if( (PyObject *)pytaperobj == Py_None ) {
+		self->curve->taperobj = (Object *)NULL;
+	} else {
+	/* Accept Object with type 'Curve' */
+		if( Object_CheckPyObject( ( PyObject * ) pytaperobj ) && 
+			pytaperobj->object->type == OB_CURVE) {
+			self->curve->taperobj = 
+				Object_FromPyObject( ( PyObject * ) pytaperobj );
+		} else {
+			return ( EXPP_ReturnPyObjError( PyExc_TypeError,
+						"expected Curve object type or None argument" ) );
+		}
+	}
+
+	return EXPP_incr_ret( Py_None );
+}
 
 /*
  * Curve_getIter
@@ -1305,32 +1519,38 @@ static PyObject *CurveGetAttr( BPy_Curve * self, char *name )
 
 	if( strcmp( name, "name" ) == 0 )
 		attr = PyString_FromString( self->curve->id.name + 2 );
-	if( strcmp( name, "pathlen" ) == 0 )
+	else if( strcmp( name, "pathlen" ) == 0 )
 		attr = PyInt_FromLong( self->curve->pathlen );
-	if( strcmp( name, "totcol" ) == 0 )
+	else if( strcmp( name, "totcol" ) == 0 )
 		attr = PyInt_FromLong( self->curve->totcol );
-	if( strcmp( name, "flag" ) == 0 )
+	else if( strcmp( name, "flag" ) == 0 )
 		attr = PyInt_FromLong( self->curve->flag );
-	if( strcmp( name, "bevresol" ) == 0 )
+	else if( strcmp( name, "bevresol" ) == 0 )
 		attr = PyInt_FromLong( self->curve->bevresol );
-	if( strcmp( name, "resolu" ) == 0 )
+	else if( strcmp( name, "resolu" ) == 0 )
 		attr = PyInt_FromLong( self->curve->resolu );
-	if( strcmp( name, "resolv" ) == 0 )
+	else if( strcmp( name, "resolv" ) == 0 )
 		attr = PyInt_FromLong( self->curve->resolv );
-	if( strcmp( name, "width" ) == 0 )
+	else if( strcmp( name, "width" ) == 0 )
 		attr = PyFloat_FromDouble( self->curve->width );
-	if( strcmp( name, "ext1" ) == 0 )
+	else if( strcmp( name, "ext1" ) == 0 )
 		attr = PyFloat_FromDouble( self->curve->ext1 );
-	if( strcmp( name, "ext2" ) == 0 )
+	else if( strcmp( name, "ext2" ) == 0 )
 		attr = PyFloat_FromDouble( self->curve->ext2 );
-	if( strcmp( name, "loc" ) == 0 )
+	else if( strcmp( name, "loc" ) == 0 )
 		return Curve_getLoc( self );
-	if( strcmp( name, "rot" ) == 0 )
+	else if( strcmp( name, "rot" ) == 0 )
 		return Curve_getRot( self );
-	if( strcmp( name, "size" ) == 0 )
+	else if( strcmp( name, "size" ) == 0 )
 		return Curve_getSize( self );
+	else if( strcmp( name, "bevob" ) == 0 )
+		return Curve_getBevOb( self );
+	else if( strcmp( name, "taperob" ) == 0 )
+		return Curve_getTaperOb( self );
+	else if( strcmp( name, "key" ) == 0 )
+		return Curve_getKey( self );
 #if 0
-	if( strcmp( name, "numpts" ) == 0 )
+	else if( strcmp( name, "numpts" ) == 0 )
 		return Curve_getNumPoints( self );
 #endif
 
@@ -1365,6 +1585,8 @@ static int CurveSetAttr( BPy_Curve * self, char *name, PyObject * value )
 		error = Curve_setName( self, valtuple );
 	else if( strcmp( name, "pathlen" ) == 0 )
 		error = Curve_setPathLen( self, valtuple );
+	else if( strcmp( name, "bevresol" ) == 0 )
+		error = Curve_setBevresol( self, valtuple );
 	else if( strcmp( name, "resolu" ) == 0 )
 		error = Curve_setResolu( self, valtuple );
 	else if( strcmp( name, "resolv" ) == 0 )
@@ -1381,6 +1603,10 @@ static int CurveSetAttr( BPy_Curve * self, char *name, PyObject * value )
 		error = Curve_setRot( self, valtuple );
 	else if( strcmp( name, "size" ) == 0 )
 		error = Curve_setSize( self, valtuple );
+	else if( strcmp( name, "bevob" ) == 0 )
+		error = Curve_setBevOb( self, valtuple );
+	else if( strcmp( name, "taperob" ) == 0 )
+		error = Curve_setTaperOb( self, valtuple );
 
 	else {			/* Error */
 		Py_DECREF( valtuple );
@@ -1452,54 +1678,3 @@ struct Curve *Curve_FromPyObject( PyObject * py_obj )
 
 }
 
-
-
-/*
- * NOTE:  this func has been replaced by freedisplist() in the recent
- *        display list refactoring.
- *
- * walk across all objects looking for curves
- *  so we can update their ob's disp list
- */
-
-void update_displists( void *data )
-{
-#if 0
-	Base *base;
-	Object *ob;
-	unsigned int layer;
-
-	/* background */
-	layer = G.scene->lay;
-
-	base = G.scene->base.first;
-	while( base ) {
-		if( base->lay & layer ) {
-			ob = base->object;
-
-			if( ELEM( ob->type, OB_CURVE, OB_SURF ) ) {
-				if( ob != G.obedit ) {
-					if( ob->data == data ) {
-						makeDispList( ob );
-					}
-				}
-			} else if( ob->type == OB_FONT ) {
-				Curve *cu = ob->data;
-				if( cu->textoncurve ) {
-					if( ( ( Curve * ) cu->textoncurve->
-					      data )->key ) {
-						text_to_curve( ob, 0 );
-						makeDispList( ob );
-					}
-				}
-			}
-		}
-		if( base->next == 0 && G.scene->set
-		    && base == G.scene->base.last )
-			base = G.scene->set->base.first;
-		else
-			base = base->next;
-	}
-#endif
-}
-
