@@ -1,5 +1,5 @@
 /* 
- * $Id: Draw.c,v 1.31 2004/11/09 14:07:25 ianwill Exp $
+ * $Id: Draw.c,v 1.43 2006/01/08 18:59:55 theeth Exp $
  *
  * ***** BEGIN GPL/BL DUAL LICENSE BLOCK *****
  *
@@ -25,7 +25,7 @@
  *
  * This is a new part of Blender.
  *
- * Contributor(s): Willian P. Germano, Campbell Barton
+ * Contributor(s): Willian P. Germano, Campbell Barton, Ken Hughes
  *
  * ***** END GPL/BL DUAL LICENSE BLOCK *****
 */
@@ -35,43 +35,36 @@
  * implementation.	Non-trivial original comments are marked with an
  * @ symbol at their beginning. */
 
-#include "Draw.h"
-
-#ifdef HAVE_CONFIG_H
-#include <config.h>
-#endif
-
-#ifdef WIN32
-#include "BLI_winstuff.h"
-#endif
+#include "Draw.h" /*This must come first*/
 
 #include "BLI_blenlib.h"
 #include "MEM_guardedalloc.h"
-
 #include "BMF_Api.h"
-
 #include "DNA_screen_types.h"
-
-#include "DNA_text_types.h"
-
 #include "BKE_global.h"
-#include "BKE_library.h"
+#include "BKE_image.h"
 #include "BKE_object.h"
-
+#include "BKE_main.h"
 #include "BIF_gl.h"
 #include "BIF_screen.h"
 #include "BIF_space.h"
 #include "BIF_interface.h"
-#include "BIF_mywindow.h"
 #include "BIF_toolbox.h"
-
 #include "BPI_script.h"		/* script struct */
-
+#include "Image.h"              /* for accessing Blender.Image objects */
+#include "IMB_imbuf_types.h"    /* for the IB_rect define */
 #include "interface.h"
 #include "mydevice.h"		/*@ for all the event constants */
+#include "gen_utils.h"
+#include "Window.h"
 
-/* This one was an extern in BPY_main.h, but only opy_draw.c was using it */
-int g_window_redrawn;
+/* these delimit the free range for button events */
+#define EXPP_BUTTON_EVENTS_OFFSET 1001
+#define EXPP_BUTTON_EVENTS_MIN 0
+#define EXPP_BUTTON_EVENTS_MAX 15382 /* 16384 - 1 - OFFSET */
+
+/* pointer to main dictionary defined in Blender.c */
+extern PyObject *g_blenderdict;
 
 /*@ hack to flag that window redraw has happened inside slider callback: */
 int EXPP_disable_force_draw = 0;
@@ -112,6 +105,10 @@ static PyObject *Method_PupMenu( PyObject * self, PyObject * args );
 static PyObject *Method_PupIntInput( PyObject * self, PyObject * args );
 static PyObject *Method_PupFloatInput( PyObject * self, PyObject * args );
 static PyObject *Method_PupStrInput( PyObject * self, PyObject * args );
+/* next by Jonathan Merritt (lancelet): */
+static PyObject *Method_Image( PyObject * self, PyObject * args);
+/* CLEVER NUMBUT */
+static PyObject *Method_PupBlock( PyObject * self, PyObject * args );
 
 static uiBlock *Get_uiBlock( void );
 static void py_slider_update( void *butv, void *data2_unused );
@@ -267,6 +264,14 @@ static char Method_PupFloatInput_doc[] =
 the float value show.\n\
 Return the user input value or None on user exit";
 
+static char Method_Image_doc[] =
+	"(image, x, y, zoomx = 1.0, zoomy = 1.0, [clipx, clipy, clipw, cliph])) \n\
+    - Draw an image.\n\
+(image) - Blender.Image to draw.\n\
+(x, y) - floats specifying the location of the image.\n\
+(zoomx, zoomy) - float zoom factors in horizontal and vertical directions.\n\
+(clipx, clipy, clipw, cliph) - integers specifying a clipping rectangle within the original image.";
+
 static char Method_PupStrInput_doc[] =
 	"(text, default, max = 20) - Display a float pop-up input.\n\
 (text) - text string to display on the button;\n\
@@ -274,6 +279,22 @@ static char Method_PupStrInput_doc[] =
 (max = 20) - The maximum number of chars the user can input;\n\
 Return the user input value or None on user exit";
 
+static char Method_PupBlock_doc[] =
+	"(title, sequence) - Display a pop-up block.\n\
+(title) - The title of the block.\n\
+(sequence) - A sequence defining what the block contains. \
+The order of the list is the order of appearance, from top down.\n\
+Possible format for sequence items:\n\
+[value is an object created with Create]\n\
+\ttext: Defines a label in the block\n\
+\t(text, value, tooltip = ''): Defines a toggle button \n\
+\t(text, value, min, max, tooltip = ''): Defines a num or string button \n\
+\t\t\tdepending on the value.\n\
+\t\tFor string, max is the maximum length of the text and min is unused.\n\
+Return 1 if the pop-up is confirmed, 0 otherwise. \n\
+Warning: On cancel, the value objects are brought back to there previous values, \
+\texcept for string values which will still contain the modified values.\n";
+
 static char Method_Exit_doc[] = "() - Exit the windowing interface";
 
 /*
@@ -304,6 +325,8 @@ static struct PyMethodDef Draw_methods[] = {
 	MethodDef( PupIntInput ),
 	MethodDef( PupFloatInput ),
 	MethodDef( PupStrInput ),
+	MethodDef( PupBlock ),
+	MethodDef( Image ),
 	MethodDef( Exit ),
 	MethodDef( Redraw ),
 	MethodDef( Draw ),
@@ -323,13 +346,15 @@ PyTypeObject Button_Type = {
 	( setattrfunc ) Button_setattr,	/*tp_setattr */
 	( cmpfunc ) 0,		/*tp_cmp */
 	( reprfunc ) Button_repr,	/*tp_repr */
+	0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
+	0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
 };
 
 static void Button_dealloc( PyObject * self )
 {
 	Button *but = ( Button * ) self;
 
-	if( but->type == 3 ) {
+	if( but->type == BSTRING_TYPE ) {
 		if( but->val.asstr )
 			MEM_freeN( but->val.asstr );
 	}
@@ -342,11 +367,11 @@ static PyObject *Button_getattr( PyObject * self, char *name )
 	Button *but = ( Button * ) self;
 
 	if( strcmp( name, "val" ) == 0 ) {
-		if( but->type == 1 )
+		if( but->type == BINT_TYPE )
 			return Py_BuildValue( "i", but->val.asint );
-		else if( but->type == 2 )
+		else if( but->type == BFLOAT_TYPE )
 			return Py_BuildValue( "f", but->val.asfloat );
-		else if( but->type == 3 )
+		else if( but->type == BSTRING_TYPE )
 			return Py_BuildValue( "s", but->val.asstr );
 	}
 
@@ -359,11 +384,11 @@ static int Button_setattr( PyObject * self, char *name, PyObject * v )
 	Button *but = ( Button * ) self;
 
 	if( strcmp( name, "val" ) == 0 ) {
-		if( but->type == 1 )
+		if( but->type == BINT_TYPE )
 			PyArg_Parse( v, "i", &but->val.asint );
-		else if( but->type == 2 )
+		else if( but->type == BFLOAT_TYPE )
 			PyArg_Parse( v, "f", &but->val.asfloat );
-		else if( but->type == 3 ) {
+		else if( but->type == BSTRING_TYPE ) {
 			char *newstr;
 			PyArg_Parse( v, "s", &newstr );
 
@@ -495,20 +520,8 @@ static void spacescript_do_pywin_buttons( SpaceScript * sc,
 }
 
 void BPY_spacescript_do_pywin_event( SpaceScript * sc, unsigned short event,
-				     short val )
+	short val, char ascii )
 {
-	static int menu_hack = 0;
-
-	/* about menu_hack above: when a menu returns after an entry is chosen,
-	 * two events are generated, the second one with val = 4.  We don't want
-	 * this second one to be passed to Python, because it can be confused with
-	 * some event with same number defined by the script.
-	 * What we do is set menu_hack to 1 if a button event occurs.
-	 * Then if the next one is also a button event, w/ val = 4, we discard it. */
-
-	if( event != UI_BUT_EVENT || !val )
-		menu_hack = 0;
-
 	if( event == QKEY && G.qual & ( LR_ALTKEY | LR_CTRLKEY ) ) {
 		/* finish script: user pressed ALT+Q or CONTROL+Q */
 		Script *script = sc->script;
@@ -520,25 +533,35 @@ void BPY_spacescript_do_pywin_event( SpaceScript * sc, unsigned short event,
 		return;
 	}
 
-	if( val ) {
-		if( uiDoBlocks( &curarea->uiblocks, event ) != UI_NOTHING )
-			event = 0;
+	if (val) {
 
-		if( event == UI_BUT_EVENT ) {
-
-			if( menu_hack && ( val == 4 ) ) {	/* "false" event? */
-				menu_hack = 0;	/* if so, discard it and clear menu_hack */
-			} else {
-				menu_hack = 1;
-				spacescript_do_pywin_buttons( sc, val );
-			}
+		if (uiDoBlocks( &curarea->uiblocks, event ) != UI_NOTHING) event = 0;
 
+		if (event == UI_BUT_EVENT) {
+			/* check that event is in free range for script button events;
+			 * read the comment before check_button_event() below to understand */
+			if (val >= EXPP_BUTTON_EVENTS_OFFSET && val < 0x4000)
+				spacescript_do_pywin_buttons(sc, val - EXPP_BUTTON_EVENTS_OFFSET);
+			return;
 		}
 	}
 
-	if( sc->script->py_event )
+	/* We use the "event" main module var, used by scriptlinks, to pass the ascii
+	 * value to event callbacks (gui/event/button callbacks are not allowed
+	 * inside scriptlinks, so this is ok) */
+	if( sc->script->py_event ) {
+		int pass_ascii = 0;
+		if (ascii > 31 && ascii != 127) {
+			pass_ascii = 1;
+			EXPP_dict_set_item_str(g_blenderdict, "event",
+					PyInt_FromLong((long)ascii));
+		}
 		exec_callback( sc, sc->script->py_event,
-			       Py_BuildValue( "(ii)", event, val ) );
+			Py_BuildValue( "(ii)", event, val ) );
+		if (pass_ascii)
+			EXPP_dict_set_item_str(g_blenderdict, "event",
+					PyString_FromString(""));
+	}
 }
 
 static PyObject *Method_Exit( PyObject * self, PyObject * args )
@@ -625,7 +648,7 @@ static PyObject *Method_Register( PyObject * self, PyObject * args )
  		script = sc->script;
 		if( !script ) {
 			return EXPP_ReturnPyObjError( PyExc_RuntimeError,
-				"Draw.Register: couldn't get pointer to script struct" );
+				"Draw.Register can't be used inside script links" );
 		}
 	}
 	else sc->script = script;
@@ -701,15 +724,15 @@ static PyObject *Method_Create( PyObject * self, PyObject * args )
 
 	but = newbutton(  );
 	if( PyFloat_Check( in ) ) {
-		but->type = 2;
-		but->val.asfloat = PyFloat_AsDouble( in );
+		but->type = BFLOAT_TYPE;
+		but->val.asfloat = (float)PyFloat_AsDouble( in );
 	} else if( PyInt_Check( in ) ) {
-		but->type = 1;
+		but->type = BINT_TYPE;
 		but->val.asint = PyInt_AsLong( in );
 	} else if( PyString_Check( in ) ) {
 		char *newstr = PyString_AsString( in );
 
-		but->type = 3;
+		but->type = BSTRING_TYPE;
 		but->slen = strlen( newstr );
 		but->val.asstr = MEM_mallocN( but->slen + 1, "button string" );
 
@@ -728,6 +751,21 @@ static uiBlock *Get_uiBlock( void )
 	return uiGetBlock( butblock, curarea );
 }
 
+/* We restrict the acceptable event numbers to a proper "free" range
+ * according to other spaces in Blender.
+ * winqread***space() (space events callbacks) use short for events
+ * (called 'val' there) and we also translate by EXPP_BUTTON_EVENTS_OFFSET
+ * to get rid of unwanted events (check BPY_do_pywin_events above for
+ * explanation). This function takes care of that and proper checking: */
+static int check_button_event(int *event) {
+	if ((*event < EXPP_BUTTON_EVENTS_MIN) ||
+			(*event > EXPP_BUTTON_EVENTS_MAX)) {
+		return -1;
+	}
+	*event += EXPP_BUTTON_EVENTS_OFFSET;
+	return 0;
+}
+
 static PyObject *Method_Button( PyObject * self, PyObject * args )
 {
 	uiBlock *block;
@@ -740,10 +778,14 @@ static PyObject *Method_Button( PyObject * self, PyObject * args )
 		return EXPP_ReturnPyObjError( PyExc_TypeError,
 					      "expected a string, five ints and optionally another string as arguments" );
 
+	if (check_button_event(&event) == -1)
+		return EXPP_ReturnPyObjError( PyExc_AttributeError,
+			"button event argument must be in the range [0, 16382]");
+
 	block = Get_uiBlock(  );
 
 	if( block )
-		uiDefBut( block, BUT, event, name, x, y, w, h, 0, 0, 0, 0, 0,
+		uiDefBut( block, BUT, event, name, (short)x, (short)y, (short)w, (short)h, 0, 0, 0, 0, 0,
 			  tip );
 
 	return EXPP_incr_ret( Py_None );
@@ -762,13 +804,17 @@ static PyObject *Method_Menu( PyObject * self, PyObject * args )
 		return EXPP_ReturnPyObjError( PyExc_TypeError,
 					      "expected a string, six ints and optionally another string as arguments" );
 
+	if (check_button_event(&event) == -1)
+		return EXPP_ReturnPyObjError( PyExc_AttributeError,
+			"button event argument must be in the range [0, 16382]");
+
 	but = newbutton(  );
-	but->type = 1;
+	but->type = BINT_TYPE;
 	but->val.asint = def;
 
 	block = Get_uiBlock(  );
 	if( block )
-		uiDefButI( block, MENU, event, name, x, y, w, h,
+		uiDefButI( block, MENU, event, name, (short)x, (short)y, (short)w, (short)h,
 			   &but->val.asint, 0, 0, 0, 0, tip );
 
 	return ( PyObject * ) but;
@@ -787,13 +833,17 @@ static PyObject *Method_Toggle( PyObject * self, PyObject * args )
 		return EXPP_ReturnPyObjError( PyExc_TypeError,
 					      "expected a string, six ints and optionally another string as arguments" );
 
+	if (check_button_event(&event) == -1)
+		return EXPP_ReturnPyObjError( PyExc_AttributeError,
+			"button event argument must be in the range [0, 16382]");
+
 	but = newbutton(  );
-	but->type = 1;
+	but->type = BINT_TYPE;
 	but->val.asint = def;
 
 	block = Get_uiBlock(  );
 	if( block )
-		uiDefButI( block, TOG, event, name, x, y, w, h,
+		uiDefButI( block, TOG, event, name, (short)x, (short)y, (short)w, (short)h,
 			   &but->val.asint, 0, 0, 0, 0, tip );
 
 	return ( PyObject * ) but;
@@ -809,27 +859,28 @@ static PyObject *Method_Toggle( PyObject * self, PyObject * args )
 static void py_slider_update( void *butv, void *data2_unused )
 {
 	uiBut *but = butv;
+	PyObject *ref = Py_BuildValue( "(i)", SPACE_VIEW3D );
+	PyObject *ret = NULL;
 
 	EXPP_disable_force_draw = 1;
-	/*@
-	   Disable forced drawing, otherwise the button object which
-	   is still being used might be deleted 
-	 */
+	/*@ Disable forced drawing, otherwise the button object which
+	 * is still being used might be deleted */
 
-	/*@  
-	   spacetext_do_pywin_buttons(curarea->spacedata.first, but->retval); */
-
-	g_window_redrawn = 0;
 	curarea->win_swap = WIN_BACK_OK;
 	/* removed global uiFrontBuf (contact ton when this goes wrong here) */
-	spacescript_do_pywin_buttons( curarea->spacedata.first,
-				      uiButGetRetVal( but ) );
 
-	if( !g_window_redrawn ) {	/*@ if Redraw already called */
-		disable_where_script( 1 );
-		M_Window_Redraw( 0, Py_BuildValue( "(i)", SPACE_VIEW3D ) );
-		disable_where_script( 0 );
-	}
+	disable_where_script( 1 );
+
+	spacescript_do_pywin_buttons( curarea->spacedata.first,
+		(unsigned short)uiButGetRetVal( but ) );
+
+	/* XXX useless right now, investigate better before a bcon 5 */
+	ret = M_Window_Redraw( 0, ref );
+
+	Py_DECREF(ref);
+	if (ret) { Py_DECREF(ret); }
+
+	disable_where_script( 0 );
 
 	EXPP_disable_force_draw = 0;
 }
@@ -850,23 +901,27 @@ static PyObject *Method_Slider( PyObject * self, PyObject * args )
 					      "expected a string, five ints, three PyObjects\n\
 			and optionally another int and string as arguments" );
 
+	if (check_button_event(&event) == -1)
+		return EXPP_ReturnPyObjError( PyExc_AttributeError,
+			"button event argument must be in the range [0, 16382]");
+
 	but = newbutton(  );
 
 	if( PyFloat_Check( inio ) ) {
 		float ini, min, max;
 
-		ini = PyFloat_AsDouble( inio );
-		min = PyFloat_AsDouble( mino );
-		max = PyFloat_AsDouble( maxo );
+		ini = (float)PyFloat_AsDouble( inio );
+		min = (float)PyFloat_AsDouble( mino );
+		max = (float)PyFloat_AsDouble( maxo );
 
-		but->type = 2;
+		but->type = BFLOAT_TYPE;
 		but->val.asfloat = ini;
 
 		block = Get_uiBlock(  );
 		if( block ) {
 			uiBut *ubut;
-			ubut = uiDefButF( block, NUMSLI, event, name, x, y, w,
-					  h, &but->val.asfloat, min, max, 0, 0,
+			ubut = uiDefButF( block, NUMSLI, event, name, (short)x, (short)y, (short)w,
+					  (short)h, &but->val.asfloat, min, max, 0, 0,
 					  tip );
 			if( realtime )
 				uiButSetFunc( ubut, py_slider_update, ubut,
@@ -879,14 +934,14 @@ static PyObject *Method_Slider( PyObject * self, PyObject * args )
 		min = PyInt_AsLong( mino );
 		max = PyInt_AsLong( maxo );
 
-		but->type = 1;
+		but->type = BINT_TYPE;
 		but->val.asint = ini;
 
 		block = Get_uiBlock(  );
 		if( block ) {
 			uiBut *ubut;
-			ubut = uiDefButI( block, NUMSLI, event, name, x, y, w,
-					  h, &but->val.asint, min, max, 0, 0,
+			ubut = uiDefButI( block, NUMSLI, event, name, (short)x, (short)y, (short)w,
+					  (short)h, &but->val.asint, (float)min, (float)max, 0, 0,
 					  tip );
 			if( realtime )
 				uiButSetFunc( ubut, py_slider_update, ubut,
@@ -909,31 +964,35 @@ static PyObject *Method_Scrollbar( PyObject * self, PyObject * args )
 	if( !PyArg_ParseTuple( args, "iiiiiOOO|is", &event, &x, &y, &w, &h,
 			       &inio, &mino, &maxo, &realtime, &tip ) )
 		return EXPP_ReturnPyObjError( PyExc_TypeError,
-					      "expected five ints, three PyObjects and optionally\n\
-			another int and string as arguments" );
+			"expected five ints, three PyObjects and optionally\n\
+another int and string as arguments" );
 
 	if( !PyNumber_Check( inio ) || !PyNumber_Check( inio )
 	    || !PyNumber_Check( inio ) )
 		return EXPP_ReturnPyObjError( PyExc_AttributeError,
 					      "expected numbers for initial, min, and max" );
 
+	if (check_button_event(&event) == -1)
+		return EXPP_ReturnPyObjError( PyExc_AttributeError,
+			"button event argument must be in the range [0, 16382]");
+
 	but = newbutton(  );
 
 	if( PyFloat_Check( inio ) )
-		but->type = 2;
+		but->type = BFLOAT_TYPE;
 	else
-		but->type = 1;
+		but->type = BINT_TYPE;
 
-	ini = PyFloat_AsDouble( inio );
-	min = PyFloat_AsDouble( mino );
-	max = PyFloat_AsDouble( maxo );
+	ini = (float)PyFloat_AsDouble( inio );
+	min = (float)PyFloat_AsDouble( mino );
+	max = (float)PyFloat_AsDouble( maxo );
 
-	if( but->type == 2 ) {
+	if( but->type == BFLOAT_TYPE ) {
 		but->val.asfloat = ini;
 		block = Get_uiBlock(  );
 		if( block ) {
 			uiBut *ubut;
-			ubut = uiDefButF( block, SCROLL, event, "", x, y, w, h,
+			ubut = uiDefButF( block, SCROLL, event, "", (short)x, (short)y, (short)w, (short)h,
 					  &but->val.asfloat, min, max, 0, 0,
 					  tip );
 			if( realtime )
@@ -941,11 +1000,11 @@ static PyObject *Method_Scrollbar( PyObject * self, PyObject * args )
 					      NULL );
 		}
 	} else {
-		but->val.asint = ini;
+		but->val.asint = (int)ini;
 		block = Get_uiBlock(  );
 		if( block ) {
 			uiBut *ubut;
-			ubut = uiDefButI( block, SCROLL, event, "", x, y, w, h,
+			ubut = uiDefButI( block, SCROLL, event, "", (short)x, (short)y, (short)w, (short)h,
 					  &but->val.asint, min, max, 0, 0,
 					  tip );
 			if( realtime )
@@ -972,21 +1031,25 @@ static PyObject *Method_Number( PyObject * self, PyObject * args )
 					      "expected a string, five ints, three PyObjects and\n\
 			optionally another string as arguments" );
 
+	if (check_button_event(&event) == -1)
+		return EXPP_ReturnPyObjError( PyExc_AttributeError,
+			"button event argument must be in the range [0, 16382]");
+
 	but = newbutton(  );
 
 	if( PyFloat_Check( inio ) ) {
 		float ini, min, max;
 
-		ini = PyFloat_AsDouble( inio );
-		min = PyFloat_AsDouble( mino );
-		max = PyFloat_AsDouble( maxo );
+		ini = (float)PyFloat_AsDouble( inio );
+		min = (float)PyFloat_AsDouble( mino );
+		max = (float)PyFloat_AsDouble( maxo );
 
-		but->type = 2;
+		but->type = BFLOAT_TYPE;
 		but->val.asfloat = ini;
 
 		block = Get_uiBlock(  );
 		if( block )
-			uiDefButF( block, NUM, event, name, x, y, w, h,
+			uiDefButF( block, NUM, event, name, (short)x, (short)y, (short)w, (short)h,
 				   &but->val.asfloat, min, max, 0, 0, tip );
 	} else {
 		int ini, min, max;
@@ -995,13 +1058,13 @@ static PyObject *Method_Number( PyObject * self, PyObject * args )
 		min = PyInt_AsLong( mino );
 		max = PyInt_AsLong( maxo );
 
-		but->type = 1;
+		but->type = BINT_TYPE;
 		but->val.asint = ini;
 
 		block = Get_uiBlock(  );
 		if( block )
-			uiDefButI( block, NUM, event, name, x, y, w, h,
-				   &but->val.asint, min, max, 0, 0, tip );
+			uiDefButI( block, NUM, event, name, (short)x, (short)y, (short)w, (short)h,
+				   &but->val.asint, (float)min, (float)max, 0, 0, tip );
 	}
 
 	return ( PyObject * ) but;
@@ -1010,29 +1073,45 @@ static PyObject *Method_Number( PyObject * self, PyObject * args )
 static PyObject *Method_String( PyObject * self, PyObject * args )
 {
 	uiBlock *block;
-	char *name, *tip = NULL, *newstr;
+	char *info_arg = NULL, *tip = NULL, *newstr = NULL;
+	char *info_str = NULL, *info_str0 = " ";
 	int event;
-	int x, y, w, h, len;
+	int x, y, w, h, len, real_len = 0;
 	Button *but;
 
-	if( !PyArg_ParseTuple( args, "siiiiisi|s", &name, &event,
-			       &x, &y, &w, &h, &newstr, &len, &tip ) )
+	if( !PyArg_ParseTuple( args, "siiiiisi|s", &info_arg, &event,
+			&x, &y, &w, &h, &newstr, &len, &tip ) )
 		return EXPP_ReturnPyObjError( PyExc_TypeError,
-					      "expected a string, five ints, a string, an int and\n\
-			optionally another string as arguments" );
+			"expected a string, five ints, a string, an int and\n\
+	optionally another string as arguments" );
+
+	if (check_button_event(&event) == -1)
+		return EXPP_ReturnPyObjError( PyExc_AttributeError,
+			"button event argument must be in the range [0, 16382]");
+
+	if (len > (UI_MAX_DRAW_STR - 1)) {
+		len = UI_MAX_DRAW_STR - 1;
+		newstr[len] = '\0';
+	}
+
+	real_len = strlen(newstr);
+	if (real_len > len) real_len = len;
 
 	but = newbutton(  );
-	but->type = 3;
+	but->type = BSTRING_TYPE;
 	but->slen = len;
-	but->val.asstr = MEM_mallocN( len + 1, "button string" );
+	but->val.asstr = MEM_mallocN( len + 1, "pybutton str" );
 
-	strncpy( but->val.asstr, newstr, len );
-	but->val.asstr[len] = '\0';
+	BLI_strncpy( but->val.asstr, newstr, len + 1); /* adds '\0' */
+	but->val.asstr[real_len] = '\0';
+
+	if (info_arg[0] == '\0') info_str = info_str0;
+	else info_str = info_arg;
 
 	block = Get_uiBlock(  );
 	if( block )
-		uiDefBut( block, TEX, event, name, x, y, w, h,
-			  but->val.asstr, 0, len, 0, 0, tip );
+		uiDefBut( block, TEX, event, info_str, (short)x, (short)y, (short)w, (short)h,
+			  but->val.asstr, 0, (float)len, 0, 0, tip );
 
 	return ( PyObject * ) but;
 }
@@ -1050,13 +1129,15 @@ static PyObject *Method_GetStringWidth( PyObject * self, PyObject * args )
 
 	if( !strcmp( font_str, "normal" ) )
 		font = ( &G )->font;
+	else if( !strcmp( font_str, "large" ) )
+		font = BMF_GetFont(BMF_kScreen15);
 	else if( !strcmp( font_str, "small" ) )
 		font = ( &G )->fonts;
 	else if( !strcmp( font_str, "tiny" ) )
 		font = ( &G )->fontss;
 	else
 		return EXPP_ReturnPyObjError( PyExc_AttributeError,
-					      "\"font\" must be: 'normal' (default), 'small' or 'tiny'." );
+					      "\"font\" must be: 'large', 'normal' (default), 'small' or 'tiny'." );
 
 	width = PyInt_FromLong( BMF_GetStringWidth( font, text ) );
 
@@ -1079,6 +1160,8 @@ static PyObject *Method_Text( PyObject * self, PyObject * args )
 
 	if( !font_str )
 		font = ( &G )->font;
+	else if( !strcmp( font_str, "large" ) )
+		font = BMF_GetFont(BMF_kScreen15);
 	else if( !strcmp( font_str, "normal" ) )
 		font = ( &G )->font;
 	else if( !strcmp( font_str, "small" ) )
@@ -1087,7 +1170,7 @@ static PyObject *Method_Text( PyObject * self, PyObject * args )
 		font = ( &G )->fontss;
 	else
 		return EXPP_ReturnPyObjError( PyExc_AttributeError,
-					      "\"font\" must be: 'normal' (default), 'small' or 'tiny'." );
+					      "\"font\" must be: 'normal' (default), 'large', 'small' or 'tiny'." );
 
 	BMF_DrawString( font, text );
 
@@ -1127,7 +1210,7 @@ static PyObject *Method_PupIntInput( PyObject * self, PyObject * args )
 		return EXPP_ReturnPyObjError( PyExc_TypeError,
 					      "expected 1 string and 3 int arguments" );
 
-	if( button( &var, min, max, text ) == 0 ) {
+	if( button( &var, (short)min, (short)max, text ) == 0 ) {
 		Py_INCREF( Py_None );
 		return Py_None;
 	}
@@ -1196,6 +1279,247 @@ static PyObject *Method_PupStrInput( PyObject * self, PyObject * args )
 				      "couldn't create a PyString" );
 }
 
+static PyObject *Method_PupBlock( PyObject * self, PyObject * args )
+{
+	PyObject *pyList, *pyItem;
+	float min, max;
+	int len, i;
+	char *title;
+
+	if (!PyArg_ParseTuple( args, "sO", &title, &pyList ) || !PySequence_Check( pyList ))
+		return EXPP_ReturnPyObjError( PyExc_TypeError, "expected a string and a sequence" );
+
+
+	len = PySequence_Length(pyList);
+
+	if (len == 0)
+		return EXPP_ReturnPyObjError( PyExc_ValueError, "expected a string and a non-empty sequence." );
+
+	if (len > 24) /* LIMIT DEFINED IN toolbox.c	*/
+		return EXPP_ReturnPyObjError( PyExc_ValueError, "sequence cannot have more than 24 elements" );
+
+	for ( i=0 ; i<len ; i++ ) {
+		PyObject *pyMin = NULL, *pyMax = NULL;
+		PyObject *f1, *f2;
+		Button *but = NULL;
+		int tlen;
+		char *text, *tip = NULL;
+
+		pyItem = PySequence_GetItem( pyList, i );
+		if (!pyItem)
+			return NULL;
+
+		if (PyString_Check( pyItem )) {
+			tlen = -2;	/* single string for label, giving it a special len for later */
+		}
+		else if (PyTuple_Check( pyItem )) {
+			/* tuple for other button, get the length for later */
+			tlen = PyTuple_Size( pyItem );
+		}
+		else {
+			/* Neither a string or a tuple, error */
+			Py_DECREF( pyItem );
+			return EXPP_ReturnPyObjError( PyExc_ValueError, "expected a string or a tuple containing 2 to 5 values." );
+		}
+
+		switch (tlen) {
+		case -2:		/*	LABEL	*/
+			text = PyString_AsString( pyItem );
+			add_numbut(i, LABEL, text, 0, 0, NULL, NULL);
+			break;
+		case 2:		/*	TOGGLE	(no tooltip)	*/
+		case 3:		/*	TOGGLE	*/
+			if (!PyArg_ParseTuple( pyItem, "sO!|s", &text, &Button_Type, &but, &tip )) {
+				Py_DECREF( pyItem );
+				return EXPP_ReturnPyObjError( PyExc_ValueError, "expected a tuple containing a string, a Button object and optionally a string for toggles" );
+			}
+
+			if (but->type != BINT_TYPE) {
+				Py_DECREF( pyItem );
+				return EXPP_ReturnPyObjError( PyExc_ValueError, "Button object for toggles should hold an integer" );
+			}
+
+			add_numbut(i, TOG|INT, text, 0, 0, &but->val.asint, tip);
+			break;
+		case 4:		/*	TEX and NUM (no tooltip)	*/
+		case 5:		/*	TEX and NUM	*/
+			if (!PyArg_ParseTuple( pyItem, "sO!OO|s", &text, &Button_Type, &but, &pyMin, &pyMax, &tip )) {
+				Py_DECREF( pyItem );
+				return EXPP_ReturnPyObjError( PyExc_ValueError, "expected a tuple containing a string, a Button object, two numerical values and optionally a string for Text and Num buttons" );
+			}
+
+			f1 = PyNumber_Float(pyMin);
+			f2 = PyNumber_Float(pyMax);
+
+			if (!f1 || !f2) {
+				Py_DECREF( pyItem );
+				return EXPP_ReturnPyObjError( PyExc_ValueError, "expected a tuple containing a string, a Button object, two numerical values and optionally a string for Text and Num buttons" );
+			}
+
+			min = (float)PyFloat_AS_DOUBLE(f1);
+			max = (float)PyFloat_AS_DOUBLE(f2);
+			Py_DECREF( f1 );
+			Py_DECREF( f2 );
+
+			switch ( but->type ) {
+			case BINT_TYPE:
+				add_numbut(i, NUM|INT, text, min, max, &but->val.asint, tip);
+				break;
+			case BFLOAT_TYPE:
+				add_numbut(i, NUM|FLO, text, min, max, &but->val.asfloat, tip);
+				break;
+			case BSTRING_TYPE:
+				max = (float)floor(max);
+
+				if (max > but->slen) {
+					int old_len = but->slen;
+					char *old_str = but->val.asstr;
+					but->slen = (int)max;
+					but->val.asstr = MEM_callocN( but->slen + 1, "button pupblock");
+					BLI_strncpy( but->val.asstr, old_str, old_len + 1 );
+					MEM_freeN(old_str);
+				}
+
+				add_numbut(i, TEX, text, 0.0f, max, but->val.asstr, tip);
+			}
+
+			break;
+		default:
+			Py_DECREF( pyItem );
+			return EXPP_ReturnPyObjError( PyExc_ValueError, "expected a string or a tuple containing 2 to 5 values." );
+		}
+		Py_DECREF( pyItem );
+	}
+
+	if (do_clever_numbuts(title, len, REDRAW))
+		return EXPP_incr_ret_True();
+	else
+		return EXPP_incr_ret_False();
+}
+
+
+/*****************************************************************************
+ * Function:            Method_Image                                         *
+ * Python equivalent:   Blender.Draw.Image                                   *
+ *                                                                           *
+ * @author Jonathan Merritt <j.merritt@pgrad.unimelb.edu.au>                 *
+ ****************************************************************************/
+static PyObject *Method_Image( PyObject * self, PyObject * args )
+{
+	PyObject *pyObjImage;
+	BPy_Image *py_img;
+	Image *image;
+	float originX, originY;
+	float zoomX = 1.0, zoomY = 1.0;
+	int clipX = 0, clipY = 0, clipW = -1, clipH = -1;
+	/*GLfloat scissorBox[4];*/
+
+	/* parse the arguments passed-in from Python */
+	if( !PyArg_ParseTuple( args, "Off|ffiiii", &pyObjImage, 
+		&originX, &originY, &zoomX, &zoomY, 
+		&clipX, &clipY, &clipW, &clipH ) )
+		return EXPP_ReturnPyObjError( PyExc_TypeError,
+			"expected a Blender.Image and 2 floats, and " \
+			"optionally 2 floats and 4 ints as arguments" );
+	/* check that the first PyObject is actually a Blender.Image */
+	if( !Image_CheckPyObject( pyObjImage ) )
+		return EXPP_ReturnPyObjError( PyExc_TypeError,
+			"expected a Blender.Image and 2 floats, and " \
+			"optionally 2 floats and 4 ints as arguments" );
+	/* check that the zoom factors are valid */
+	if( ( zoomX <= 0.0 ) || ( zoomY <= 0.0 ) )
+		return EXPP_ReturnPyObjError( PyExc_TypeError,
+			"invalid zoom factors - they must be >= 0.0" );
+
+	/* fetch a C Image pointer from the passed-in Python object */
+	py_img = ( BPy_Image * ) pyObjImage;
+	image = py_img->image;
+
+	/* load the image data if necessary */
+	if( !image->ibuf )      /* if no image data is available ... */
+		load_image( image, IB_rect, "", 0 );    /* ... load it */
+	if( !image->ibuf )      /* if failed to load the image */
+		return EXPP_ReturnPyObjError( PyExc_RuntimeError,
+			"couldn't load image data in Blender" );
+
+	/* Update the time tag of the image */
+	tag_image_time(image);
+
+	/* set up a valid clipping rectangle.  if no clip rectangle was
+	 * given, this results in inclusion of the entire image.  otherwise,
+	 * the clipping is just checked against the bounds of the image.
+	 * if clipW or clipH are less than zero then they include as much of
+	 * the image as they can. */
+	clipX = EXPP_ClampInt( clipX, 0, image->ibuf->x );
+	clipY = EXPP_ClampInt( clipY, 0, image->ibuf->y );
+	if( ( clipW < 0 ) || ( clipW > ( image->ibuf->x - clipW ) ) )
+		clipW = image->ibuf->x - clipX;
+	if( ( clipH < 0 ) || ( clipH > ( image->ibuf->y - clipH ) ) )
+		clipH = image->ibuf->y - clipY;
+
+	/* -- we are "Go" to Draw! -- */
+
+	/* set the raster position.
+	 *
+	 * If the raster position is negative, then using glRasterPos2i() 
+	 * directly would cause it to be clipped.  Instead, we first establish 
+	 * a valid raster position within the clipping rectangle of the 
+	 * window and then use glBitmap() with a NULL image pointer to offset 
+	 * it to the true position we require.  To pick an initial valid 
+	 * raster position within the viewport, we query the clipping rectangle
+	 * and use its lower-left pixel.
+	 *
+	 * This particular technique is documented in the glRasterPos() man
+	 * page, although I haven't seen it used elsewhere in Blender.
+	 */
+
+	/* update (W): to fix a bug where images wouldn't get drawn if the bottom
+	 * left corner of the Scripts win were above a given height or to the right
+	 * of a given width, the code below is being commented out.  It should not
+	 * be needed anyway, because spaces in Blender are projected to lie inside
+	 * their areas, see src/drawscript.c for example.  Note: the
+	 * glaRasterPosSafe2i function in src/glutil.c does use the commented out
+	 * technique, but with 0,0 instead of scissorBox.  This function can be
+	 * a little optimized, based on glaDrawPixelsSafe in that same fine, but
+	 * we're too close to release 2.37 right now. */
+	/*
+	glGetFloatv( GL_SCISSOR_BOX, scissorBox );
+	glRasterPos2i( scissorBox[0], scissorBox[1] );
+	glBitmap( 0, 0, 0.0, 0.0, 
+		originX-scissorBox[0], originY-scissorBox[1], NULL );
+	*/
+
+	/* update (cont.): using these two lines instead:
+	 * (based on glaRasterPosSafe2i, but Ken Hughes deserves credit
+	 * for suggesting this exact fix in the bug tracker) */
+	glRasterPos2i(0, 0);
+	glBitmap( 0, 0, 0.0, 0.0, originX, originY, NULL );
+
+	/* set the zoom */
+	glPixelZoom( zoomX, zoomY );
+
+	/* set the width of the image (ROW_LENGTH), and the offset to the
+	 * clip origin within the image in x (SKIP_PIXELS) and 
+	 * y (SKIP_ROWS) */
+	glPixelStorei( GL_UNPACK_ROW_LENGTH,  image->ibuf->x );
+	glPixelStorei( GL_UNPACK_SKIP_PIXELS, clipX );
+	glPixelStorei( GL_UNPACK_SKIP_ROWS,   clipY );
+
+	/* draw the image */
+	glDrawPixels( clipW, clipH, GL_RGBA, GL_UNSIGNED_BYTE, 
+		image->ibuf->rect );
+
+	/* restore the defaults for some parameters (we could also use a
+	 * glPushClientAttrib() and glPopClientAttrib() pair). */
+	glPixelZoom( 1.0, 1.0 );
+	glPixelStorei( GL_UNPACK_SKIP_ROWS,   0 );
+	glPixelStorei( GL_UNPACK_SKIP_PIXELS, 0 );
+	glPixelStorei( GL_UNPACK_ROW_LENGTH,  0 );
+
+	Py_INCREF( Py_None );
+	return Py_None;
+
+}
 
 PyObject *Draw_Init( void )
 {
@@ -1208,11 +1532,11 @@ PyObject *Draw_Init( void )
 	dict = PyModule_GetDict( submodule );
 
 #define EXPP_ADDCONST(x) \
-	PyDict_SetItemString(dict, #x, PyInt_FromLong(x))
+	EXPP_dict_set_item_str(dict, #x, PyInt_FromLong(x))
 
 	/* So, for example:
 	 * EXPP_ADDCONST(LEFTMOUSE) becomes
-	 * PyDict_SetItemString(dict, "LEFTMOUSE", PyInt_FromLong(LEFTMOUSE)) 
+	 * EXPP_dict_set_item_str(dict, "LEFTMOUSE", PyInt_FromLong(LEFTMOUSE)) 
 	 */
 
 	EXPP_ADDCONST( LEFTMOUSE );
