@@ -1,5 +1,5 @@
 /* 
- * $Id: Draw.h,v 1.17 2005/05/08 21:20:33 ianwill Exp $
+ * $Id: Draw.h,v 1.19 2006/01/08 18:59:55 theeth Exp $
  *
  * ***** BEGIN GPL/BL DUAL LICENSE BLOCK *****
  *
@@ -37,14 +37,10 @@
 #ifndef EXPP_DRAW_H_
 #define EXPP_DRAW_H_
 
+#include <Python.h>
 #include "DNA_space_types.h"
 #include "DNA_text_types.h"
 
-#include "Python.h"
-
-#include "gen_utils.h"
-#include "Window.h"
-
 void initDraw( void );
 
 /* 
@@ -63,6 +59,10 @@ typedef struct _Button {
 	char *tooltip;
 } Button;
 
+#define BINT_TYPE		1
+#define BFLOAT_TYPE		2
+#define BSTRING_TYPE	3
+
 
 /* 
  * these are declared in ../BPY_extern.h 
