@@ -1,5 +1,5 @@
 /* 
- * $Id: EXPP_interface.c,v 1.18 2005/04/16 05:25:42 ianwill Exp $
+ * $Id: EXPP_interface.c,v 1.21 2005/08/17 14:26:00 ascotan Exp $
  *
  * ***** BEGIN GPL/BL DUAL LICENSE BLOCK *****
  *
@@ -30,29 +30,15 @@
  * ***** END GPL/BL DUAL LICENSE BLOCK *****
 */
 
-#include <stdio.h>
-
 #include <Python.h>
 
-#include <BKE_global.h>
-#include <BKE_library.h>
-#include <BKE_main.h>
-#include <BLI_blenlib.h>
-#include <DNA_space_types.h> /* for FILE_MAXDIR, FILE_MAXFILE */
-
-#include "EXPP_interface.h"
+#include "EXPP_interface.h" 
+#include "BLI_blenlib.h"
+#include "DNA_space_types.h" /* for FILE_MAXDIR, FILE_MAXFILE */
 #include "Blender.h"
-#include "Registry.h"
-#include "gen_utils.h"
 
 extern char bprogname[]; /* argv[0] from creator.c */
 
-void initBlenderApi2_2x( void )
-{
-	g_blenderdict = NULL;
-	M_Blender_Init(  );
-}
-
 /* this makes sure BLI_gethome() returns a path with '.blender' appended
  * Besides, this function now either returns userhome/.blender (if it exists)
  * or blenderInstallDir/.blender/ otherwise (can also be cvs dir).
@@ -77,8 +63,15 @@ char *bpy_gethome(int append_scriptsdir)
 	else if (homedir[0] != '\0')
 		return homedir;
 
+	/* BLI_gethome() can return NULL if env vars are not set */
 	s = BLI_gethome();
 
+	if( !s )  /* bail if no $HOME */
+	{
+		printf("$HOME is NOT set\n");
+		return NULL;
+	}
+
 	if( strstr( s, ".blender" ) )
 		PyOS_snprintf( homedir, FILE_MAXDIR, s );
 	else
