@@ -1,5 +1,5 @@
 /*
- * $Id: Ipo.c,v 1.34 2004/12/21 01:15:42 stiv Exp $
+ * $Id: Ipo.c,v 1.49 2006/01/05 19:25:38 khughes Exp $
  *
  * ***** BEGIN GPL/BL DUAL LICENSE BLOCK *****
  *
@@ -25,31 +25,26 @@
  *
  * This is a new part of Blender.
  *
- * Contributor(s): Jacques Guignot, Nathan Letwory
+ * Contributor(s): Jacques Guignot RIP 2005, Nathan Letwory, 
+ *  Stephen Swaney, Ken Hughes
  *
  * ***** END GPL/BL DUAL LICENSE BLOCK *****
  */
 
-#include "Ipo.h"
-
-#include <BKE_main.h>
-#include <BKE_global.h>
-#include <BKE_object.h>
-#include <BKE_library.h>
-#include <BSE_editipo.h>
-#include <BLI_blenlib.h>
-
-#include <DNA_curve_types.h>
+#include "Ipo.h" /*This must come first*/
 
+#include "BKE_main.h"
+#include "BKE_global.h"
+#include "BKE_library.h"
+#include "BKE_ipo.h"
+#include "BLI_blenlib.h"
+#include "BIF_space.h"
+#include "BSE_editipo.h"
+#include "MEM_guardedalloc.h"
+#include "mydevice.h"
 #include "Ipocurve.h"
-#include "constant.h"
 #include "gen_utils.h"
 
-
-
-/* forward declarations */
-char *GetIpoCurveName( IpoCurve * icu );
-
 /*****************************************************************************/
 /* Python API function prototypes for the Ipo module.                        */
 /*****************************************************************************/
@@ -93,6 +88,7 @@ static PyObject *Ipo_setRctf( BPy_Ipo * self, PyObject * args );
 static PyObject *Ipo_getCurve( BPy_Ipo * self, PyObject * args );
 static PyObject *Ipo_getCurves( BPy_Ipo * self );
 static PyObject *Ipo_addCurve( BPy_Ipo * self, PyObject * args );
+static PyObject *Ipo_delCurve( BPy_Ipo * self, PyObject * args );
 static PyObject *Ipo_getNcurves( BPy_Ipo * self );
 static PyObject *Ipo_getNBezPoints( BPy_Ipo * self, PyObject * args );
 static PyObject *Ipo_DeleteBezPoints( BPy_Ipo * self, PyObject * args );
@@ -114,37 +110,37 @@ static PyMethodDef BPy_Ipo_methods[] = {
 	{"setName", ( PyCFunction ) Ipo_setName, METH_VARARGS,
 	 "(str) - Change Ipo Data name"},
 	{"getBlocktype", ( PyCFunction ) Ipo_getBlocktype, METH_NOARGS,
-	 "() - Return Ipo blocktype -"},
+	 "() - Return Ipo blocktype"},
 	{"setBlocktype", ( PyCFunction ) Ipo_setBlocktype, METH_VARARGS,
 	 "(str) - Change Ipo blocktype"},
 	{"getRctf", ( PyCFunction ) Ipo_getRctf, METH_NOARGS,
-	 "() - Return Ipo rctf - "},
+	 "() - Return Ipo rctf"},
 	{"setRctf", ( PyCFunction ) Ipo_setRctf, METH_VARARGS,
-	 "(str) - Change Ipo rctf"},
+	 "(flt,flt,flt,flt) - Change Ipo rctf"},
 	{"addCurve", ( PyCFunction ) Ipo_addCurve, METH_VARARGS,
-	 "() - Return Ipo ncurves"},
+	 "() - Add a curve to Ipo"},
+	{"delCurve", ( PyCFunction ) Ipo_delCurve, METH_VARARGS,
+	 "(str) - Delete curve from Ipo"},
 	{"getNcurves", ( PyCFunction ) Ipo_getNcurves, METH_NOARGS,
-	 "() - Return Ipo ncurves"},
+	 "() - Return number of Ipo curves"},
 	{"getNBezPoints", ( PyCFunction ) Ipo_getNBezPoints, METH_VARARGS,
-	 "() - Return curve number of Bez points"},
+	 "(int) - Return number of Bez points on an Ipo curve"},
 	{"delBezPoint", ( PyCFunction ) Ipo_DeleteBezPoints, METH_VARARGS,
-	 "() - Return curve number of Bez points"},
+	 "(int) - deprecated: use ipocurve.delBezier()"},
 	{"getCurveBP", ( PyCFunction ) Ipo_getCurveBP, METH_VARARGS,
-	 "() - Return Ipo ncurves"},
+	 "() - unsupported"},
 	{"EvaluateCurveOn", ( PyCFunction ) Ipo_EvaluateCurveOn, METH_VARARGS,
-	 "() - Return curve value at given time"},
+	 "(int,flt) - deprecated: see ipocurve.evaluate()"},
 	{"getCurveCurval", ( PyCFunction ) Ipo_getCurvecurval, METH_VARARGS,
-	 "() - Return curval"},
-	{"getCurveBeztriple", ( PyCFunction ) Ipo_getCurveBeztriple,
-	 METH_VARARGS,
-	 "() - Return Ipo ncurves"},
-	{"setCurveBeztriple", ( PyCFunction ) Ipo_setCurveBeztriple,
-	 METH_VARARGS,
-	 "() - Return curval"},
+	 "(int) - deprecated: see ipocurve.evaluate()"},
+	{"getCurveBeztriple", ( PyCFunction ) Ipo_getCurveBeztriple, METH_VARARGS,
+	 "(int,int) - deprecated: see ipocurve.bezierPoints[]"},
+	{"setCurveBeztriple", ( PyCFunction ) Ipo_setCurveBeztriple, METH_VARARGS,
+	 "(int,int,list) - set a BezTriple"},
 	{"getCurves", ( PyCFunction ) Ipo_getCurves, METH_NOARGS,
-	 "() - Return curval"},
+	 "() - Return list of all defined Ipo curves"},
 	{"getCurve", ( PyCFunction ) Ipo_getCurve, METH_VARARGS,
-	 "() - Return curval"},
+	 "(str|int) - Returns specified Ipo curve"},
 	{NULL, NULL, 0, NULL}
 };
 
@@ -214,9 +210,9 @@ static PyObject *M_Ipo_New( PyObject * self, PyObject * args )
 	if( !strcmp( code, "Lamp" ) )
 		idcode = ID_LA;
 	if( !strcmp( code, "Action" ) )
-		idcode = ID_AC;
+		idcode = ID_PO;
 	if( !strcmp( code, "Constraint" ) )
-		idcode = IPO_CO;
+		idcode = ID_CO;
 	if( !strcmp( code, "Sequence" ) )
 		idcode = ID_SEQ;
 	if( !strcmp( code, "Curve" ) )
@@ -323,13 +319,14 @@ static PyObject *M_Ipo_Get( PyObject * self, PyObject * args )
 
 static PyObject *M_Ipo_Recalc( PyObject * self, PyObject * args )
 {
-	void testhandles_ipocurve( IpoCurve * icu );
-	PyObject *a;
+	PyObject *obj;
 	IpoCurve *icu;
-	if( !PyArg_ParseTuple( args, "O", &a ) )
-		return ( EXPP_ReturnPyObjError
-			 ( PyExc_TypeError, "expected ipo argument)" ) );
-	icu = IpoCurve_FromPyObject( a );
+
+	if( !PyArg_ParseTuple( args, "O!", &IpoCurve_Type, &obj ) )
+		return EXPP_ReturnPyObjError( PyExc_TypeError,
+				"expected Ipo curve argument" );
+
+	icu = IpoCurve_FromPyObject( obj );
 	testhandles_ipocurve( icu );
 
 	Py_INCREF( Py_None );
@@ -448,233 +445,104 @@ static PyObject *Ipo_getNcurves( BPy_Ipo * self )
 	return ( PyInt_FromLong( i ) );
 }
 
-int Ipo_laIcuName( char *s, int *param )
+
+/*
+  Lamp ipo Name to Channel
+*/
+
+static int Ipo_laIcuName( char *s, int *param )
 {
-	int ok = 0;
-	if( !strcmp( s, "Energy" ) ) {
-		*param = LA_ENERGY;
-		ok = 1;
+	extern int la_ar[];
+
+	int not_ok = 0;
+	int i;
+	char *lamp_names[LA_TOTIPO] =
+		{ "Energ", "R", "G", "B", "Dist", "SpoSi",
+		"SpoBl", "Quad1", "Quad2", "HaInt",
+		/* lamp texture names */
+		"OfsX", "OfsY", "OfsZ", "SizeX", "SizeY",
+		"SizeZ", "texR", "texG", "texB", "DefVar",
+		"Col"
+	};
+
+	for( i = 0; i < LA_TOTIPO; i++ ) {
+		if( !strcmp( s, lamp_names[i] ) ) {	/* found it! */
+			*param = la_ar[i];
+			return 1;
+		}
 	}
-	if( !strcmp( s, "R" ) ) {
-		*param = LA_COL_R;
-		ok = 1;
-	}
-	if( !strcmp( s, "G" ) ) {
-		*param = LA_COL_G;
-		ok = 1;
-	}
-	if( !strcmp( s, "B" ) ) {
-		*param = LA_COL_B;
-		ok = 1;
-	}
-	if( !strcmp( s, "Dist" ) ) {
-		*param = LA_DIST;
-		ok = 1;
-	}
-	if( !strcmp( s, "SpoSi" ) ) {
-		*param = LA_SPOTSI;
-		ok = 1;
-	}
-	if( !strcmp( s, "SpoBl" ) ) {
-		*param = LA_SPOTBL;
-		ok = 1;
-	}
-	if( !strcmp( s, "Quad1" ) ) {
-		*param = LA_QUAD1;
-		ok = 1;
-	}
-	if( !strcmp( s, "Quad2" ) ) {
-		*param = LA_QUAD2;
-		ok = 1;
-	}
-	if( !strcmp( s, "HaInt" ) ) {
-		*param = LA_HALOINT;
-		ok = 1;
-	}
-	return ok;
+
+	return not_ok;
 }
 
-int Ipo_woIcuName( char *s, int *param )
+
+/* 
+ World Ipo Name to Channel
+*/
+
+static int Ipo_woIcuName( char *s, int *param )
 {
-	int ok = 0;
-	if( !strcmp( s, "HorR" ) ) {
-		*param = WO_HOR_R;
-		ok = 1;
+	extern int wo_ar[];	/* channel values from ipo.c */
+	int not_ok = 0;
+	int i;
+	char *world_names[WO_TOTIPO] = { "HorR", "HorG", "HorB",
+		"ZenR", "ZenG", "ZenB",
+		"Expos",
+		"Misi", "MisDi", "MisSta", "MisHi",
+		"StarR", "StarB", "StarG",
+		"StarDi", "StarSi",
+		/* world textures names */
+		"OfsX", "OfsY", "OfsZ",
+		"SizeX", "SizeY", "SizeZ",
+		"texR", "texG", "texB",
+		"DefVar", "Col", "Nor", "Var",
+	};
+
+	for( i = 0; i < WO_TOTIPO; i++ ) {
+		if( !strcmp( s, world_names[i] ) ) {	/* found it! */
+			*param = wo_ar[i];
+			return 1;
+		}
 	}
-	if( !strcmp( s, "HorG" ) ) {
-		*param = WO_HOR_G;
-		ok = 1;
-	}
-	if( !strcmp( s, "HorB" ) ) {
-		*param = WO_HOR_B;
-		ok = 1;
-	}
-	if( !strcmp( s, "ZenR" ) ) {
-		*param = WO_ZEN_R;
-		ok = 1;
-	}
-	if( !strcmp( s, "ZenG" ) ) {
-		*param = WO_ZEN_G;
-		ok = 1;
-	}
-	if( !strcmp( s, "ZenB" ) ) {
-		*param = WO_ZEN_B;
-		ok = 1;
-	}
-	if( !strcmp( s, "Expos" ) ) {
-		*param = WO_EXPOS;
-		ok = 1;
-	}
-	if( !strcmp( s, "Misi" ) ) {
-		*param = WO_MISI;
-		ok = 1;
-	}
-	if( !strcmp( s, "MisDi" ) ) {
-		*param = WO_MISTDI;
-		ok = 1;
-	}
-	if( !strcmp( s, "MisHi" ) ) {
-		*param = WO_MISTHI;
-		ok = 1;
-	}
-	if( !strcmp( s, "StarR" ) ) {
-		*param = WO_STAR_R;
-		ok = 1;
-	}
-	if( !strcmp( s, "StarB" ) ) {
-		*param = WO_STAR_B;
-		ok = 1;
-	}
-	if( !strcmp( s, "StarG" ) ) {
-		*param = WO_STAR_G;
-		ok = 1;
-	}
-	if( !strcmp( s, "ClSta" ) ) {
-		*param = WO_MISTSTA;
-		ok = 1;
-	}
-	if( !strcmp( s, "StarDi" ) ) {
-		*param = WO_STARDIST;
-		ok = 1;
-	}
-	if( !strcmp( s, "StarSi" ) ) {
-		*param = WO_STARSIZE;
-		ok = 1;
-	}
-	return ok;
+
+	return not_ok;
 }
 
-int Ipo_maIcuName( char *s, int *param )
+static int Ipo_maIcuName( char *s, int *param )
 {
-	int ok = 0;
-	if( !strcmp( s, "R" ) ) {
-		*param = MA_COL_R;
-		ok = 1;
+	extern int ma_ar[];
+
+	int not_ok = 0;
+	int i;
+
+	char *material_names[MA_TOTIPO] = { "R", "G", "B",
+		"SpecR", "SpecG", "SpecB",
+		"MirR", "MirG", "MirB", "Ref", "Alpha",
+		"Emit", "Amb", "Spec",
+		"Hard", "SpTra", "Ior", "Mode",
+		"HaSize", "Translu",
+		"RayMir", "FresMir", "FresMirI",
+		"FresTra", "FresTraI",
+		"TraGlow",
+		"OfsX", "OfsY", "OfsZ",
+		"SizeX", "SizeY", "SizeZ",
+		"texR", "texG", "texB",
+		"DefVar", "Col", "Nor", "Var",
+		"Disp"
+	};
+
+
+	for( i = 0; i < MA_TOTIPO; i++ ) {
+		if( !strcmp( s, material_names[i] ) ) {	/* found it! */
+			*param = ma_ar[i];
+			return 1;
+		}
 	}
-	if( !strcmp( s, "G" ) ) {
-		*param = MA_COL_G;
-		ok = 1;
-	}
-	if( !strcmp( s, "B" ) ) {
-		*param = MA_COL_B;
-		ok = 1;
-	}
-	if( !strcmp( s, "SpecR" ) ) {
-		*param = MA_SPEC_R;
-		ok = 1;
-	}
-	if( !strcmp( s, "SpecG" ) ) {
-		*param = MA_SPEC_G;
-		ok = 1;
-	}
-	if( !strcmp( s, "SpecB" ) ) {
-		*param = MA_SPEC_B;
-		ok = 1;
-	}
-	if( !strcmp( s, "MirR" ) ) {
-		*param = MA_MIR_R;
-		ok = 1;
-	}
-	if( !strcmp( s, "MirG" ) ) {
-		*param = MA_MIR_G;
-		ok = 1;
-	}
-	if( !strcmp( s, "MirB" ) ) {
-		*param = MA_MIR_B;
-		ok = 1;
-	}
-	if( !strcmp( s, "Ref" ) ) {
-		*param = MA_REF;
-		ok = 1;
-	}
-	if( !strcmp( s, "Alpha" ) ) {
-		*param = MA_ALPHA;
-		ok = 1;
-	}
-	if( !strcmp( s, "Emit" ) ) {
-		*param = MA_EMIT;
-		ok = 1;
-	}
-	if( !strcmp( s, "Amb" ) ) {
-		*param = MA_AMB;
-		ok = 1;
-	}
-	if( !strcmp( s, "Spec" ) ) {
-		*param = MA_SPEC;
-		ok = 1;
-	}
-	if( !strcmp( s, "Hard" ) ) {
-		*param = MA_HARD;
-		ok = 1;
-	}
-	if( !strcmp( s, "SpTra" ) ) {
-		*param = MA_SPTR;
-		ok = 1;
-	}
-	if( !strcmp( s, "Ior" ) ) {
-		*param = MA_IOR;
-		ok = 1;
-	}
-	if( !strcmp( s, "Mode" ) ) {
-		*param = MA_MODE;
-		ok = 1;
-	}
-	if( !strcmp( s, "HaSize" ) ) {
-		*param = MA_HASIZE;
-		ok = 1;
-	}
-	if( !strcmp( s, "Translu" ) ) {
-		*param = MA_TRANSLU;
-		ok = 1;
-	}
-	if( !strcmp( s, "RayMir" ) ) {
-		*param = MA_RAYM;
-		ok = 1;
-	}
-	if( !strcmp( s, "FresMir" ) ) {
-		*param = MA_FRESMIR;
-		ok = 1;
-	}
-	if( !strcmp( s, "FresMirI" ) ) {
-		*param = MA_FRESMIRI;
-		ok = 1;
-	}
-	if( !strcmp( s, "FresTra" ) ) {
-		*param = MA_FRESTRA;
-		ok = 1;
-	}
-	if( !strcmp( s, "FresTraI" ) ) {
-		*param = MA_FRESTRAI;
-		ok = 1;
-	}
-	if( !strcmp( s, "TraGlow" ) ) {
-		*param = MA_ADD;
-		ok = 1;
-	}
-	return ok;
+
+	return not_ok;
 }
 
-int Ipo_keIcuName( char *s, int *param )
+static int Ipo_keIcuName( char *s, int *param )
 {
 	char key[10];
 	int ok = 0;
@@ -695,7 +563,7 @@ int Ipo_keIcuName( char *s, int *param )
 	return ok;
 }
 
-int Ipo_seqIcuName( char *s, int *param )
+static int Ipo_seqIcuName( char *s, int *param )
 {
 	int ok = 0;
 	if( !strcmp( s, "Fac" ) ) {
@@ -706,7 +574,7 @@ int Ipo_seqIcuName( char *s, int *param )
 	return ok;
 }
 
-int Ipo_cuIcuName( char *s, int *param )
+static int Ipo_cuIcuName( char *s, int *param )
 {
 	int ok = 0;
 	if( !strcmp( s, "Speed" ) ) {
@@ -717,7 +585,7 @@ int Ipo_cuIcuName( char *s, int *param )
 	return ok;
 }
 
-int Ipo_coIcuName( char *s, int *param )
+static int Ipo_coIcuName( char *s, int *param )
 {
 	int ok = 0;
 	if( !strcmp( s, "Inf" ) ) {
@@ -728,456 +596,407 @@ int Ipo_coIcuName( char *s, int *param )
 	return ok;
 }
 
-int Ipo_acIcuName( char *s, int *param )
+static int Ipo_acIcuName( char *s, int *param )
 {
 	int ok = 0;
 	if( !strcmp( s, "LocX" ) ) {
 		*param = AC_LOC_X;
-		ok = 1;
+		return 1;
 	}
 	if( !strcmp( s, "LocY" ) ) {
 		*param = AC_LOC_Y;
-		ok = 1;
+		return 1;
 	}
 	if( !strcmp( s, "LocZ" ) ) {
 		*param = AC_LOC_Z;
-		ok = 1;
+		return 1;
 	}
 	if( !strcmp( s, "SizeX" ) ) {
 		*param = AC_SIZE_X;
-		ok = 1;
+		return 1;
 	}
 	if( !strcmp( s, "SizeY" ) ) {
 		*param = AC_SIZE_Y;
-		ok = 1;
+		return 1;
 	}
 	if( !strcmp( s, "SizeZ" ) ) {
 		*param = AC_SIZE_Z;
-		ok = 1;
+		return 1;
 	}
 	if( !strcmp( s, "QuatX" ) ) {
 		*param = AC_QUAT_X;
-		ok = 1;
+		return 1;
 	}
 	if( !strcmp( s, "QuatY" ) ) {
 		*param = AC_QUAT_Y;
-		ok = 1;
+		return 1;
 	}
 	if( !strcmp( s, "QuatZ" ) ) {
 		*param = AC_QUAT_Z;
-		ok = 1;
+		return 1;
 	}
 	if( !strcmp( s, "QuatW" ) ) {
 		*param = AC_QUAT_W;
-		ok = 1;
+		return 1;
 	}
 	return ok;
 }
 
-int Ipo_caIcuName( char *s, int *param )
+
+/*
+  Camera ipo name to channel
+*/
+
+static int Ipo_caIcuName( char *s, int *param )
 {
-	int ok = 0;
-	if( !strcmp( s, "Lens" ) ) {
-		*param = CAM_LENS;
-		ok = 1;
+	/* for Camera ipos CAM_TOTNAM == CAM_TOTIPO 
+	   and cam_ic_names[] holds the complete set of names, so we use that.
+	 */
+	extern int cam_ar[];
+	extern char *cam_ic_names[];
+
+	int not_ok = 0;
+	int i;
+
+	for( i = 0; i < CAM_TOTIPO; i++ ) {
+		if( !strcmp( s, cam_ic_names[i] ) ) {	/* found it! */
+			*param = cam_ar[i];
+			return 1;
+		}
 	}
-	if( !strcmp( s, "ClSta" ) ) {
-		*param = CAM_STA;
-		ok = 1;
-	}
-	if( !strcmp( s, "ClEnd" ) ) {
-		*param = CAM_END;
-		ok = 1;
-	}
-	return ok;
+
+	return not_ok;
 }
 
-int Ipo_texIcuName( char *s, int *param )
+
+/*
+  texture ipo name to channel
+*/
+
+static int Ipo_texIcuName( char *s, int *param )
 {
-	int ok = 0;
-	if( !strcmp( s, "NSize" ) ) {
-		*param = TE_NSIZE;
-		ok = 1;
-	}
-	if( !strcmp( s, "NDepth" ) ) {
-		*param = TE_NDEPTH;
-		ok = 1;
-	}
-	if( !strcmp( s, "NType" ) ) {
-		*param = TE_NTYPE;
-		ok = 1;
-	}
-	if( !strcmp( s, "Turb" ) ) {
-		*param = TE_TURB;
-		ok = 1;
-	}
-	if( !strcmp( s, "Vnw1" ) ) {
-		*param = TE_VNW1;
-		ok = 1;
-	}
-	if( !strcmp( s, "Vnw2" ) ) {
-		*param = TE_VNW2;
-		ok = 1;
-	}
-	if( !strcmp( s, "Vnw3" ) ) {
-		*param = TE_VNW3;
-		ok = 1;
-	}
-	if( !strcmp( s, "Vnw4" ) ) {
-		*param = TE_VNW4;
-		ok = 1;
-	}
-	if( !strcmp( s, "MinkMExp" ) ) {
-		*param = TE_VNMEXP;
-		ok = 1;
-	}
-	if( !strcmp( s, "DistM" ) ) {
-		*param = TE_VN_DISTM;
-		ok = 1;
-	}
-	if( !strcmp( s, "ColT" ) ) {
-		*param = TE_VN_COLT;
-		ok = 1;
-	}
-	if( !strcmp( s, "iScale" ) ) {
-		*param = TE_ISCA;
-		ok = 1;
-	}
-	if( !strcmp( s, "DistA" ) ) {
-		*param = TE_DISTA;
-		ok = 1;
-	}
-	if( !strcmp( s, "MgType" ) ) {
-		*param = TE_MG_TYP;
-		ok = 1;
-	}
-	if( !strcmp( s, "MgH" ) ) {
-		*param = TE_MGH;
-		ok = 1;
-	}
-	if( !strcmp( s, "Lacu" ) ) {
-		*param = TE_MG_LAC;
-		ok = 1;
-	}
-	if( !strcmp( s, "Oct" ) ) {
-		*param = TE_MG_OCT;
-		ok = 1;
-	}
-	if( !strcmp( s, "MgOff" ) ) {
-		*param = TE_MG_OFF;
-		ok = 1;
-	}
-	if( !strcmp( s, "MgGain" ) ) {
-		*param = TE_MG_GAIN;
-		ok = 1;
-	}
-	if( !strcmp( s, "NBase1" ) ) {
-		*param = TE_N_BAS1;
-		ok = 1;
-	}
-	if( !strcmp( s, "NBase2" ) ) {
-		*param = TE_N_BAS2;
-		ok = 1;
+	/* this is another case where TE_TOTIPO == TE_TOTNAM.
+	   te_ic_names[] has all our names so use that.
+	*/
+	extern int te_ar[];
+	extern char *tex_ic_names[];
+	int not_ok = 0;
+	int i;
+
+	for( i = 0; i < TE_TOTIPO; i++){
+		if( !strcmp( s, tex_ic_names[i] ) ){
+			*param = te_ar[i];
+			return 1;
+		}
 	}
 
-	return ok;
+	return not_ok;
 }
 
-int Ipo_obIcuName( char *s, int *param )
+static int Ipo_obIcuName( char *s, int *param )
 {
 	int ok = 0;
 	if( !strcmp( s, "LocX" ) ) {
 		*param = OB_LOC_X;
-		ok = 1;
+		return 1;
 	}
 	if( !strcmp( s, "LocY" ) ) {
 		*param = OB_LOC_Y;
-		ok = 1;
+		return 1;
 	}
 	if( !strcmp( s, "LocZ" ) ) {
 		*param = OB_LOC_Z;
-		ok = 1;
+		return 1;
 	}
 	if( !strcmp( s, "RotX" ) ) {
 		*param = OB_ROT_X;
-		ok = 1;
+		return 1;
 	}
 	if( !strcmp( s, "RotY" ) ) {
 		*param = OB_ROT_Y;
-		ok = 1;
+		return 1;
 	}
 	if( !strcmp( s, "RotZ" ) ) {
 		*param = OB_ROT_Z;
-		ok = 1;
+		return 1;
 	}
 	if( !strcmp( s, "SizeX" ) ) {
 		*param = OB_SIZE_X;
-		ok = 1;
+		return 1;
 	}
 	if( !strcmp( s, "SizeY" ) ) {
 		*param = OB_SIZE_Y;
-		ok = 1;
+		return 1;
 	}
 	if( !strcmp( s, "SizeZ" ) ) {
 		*param = OB_SIZE_Z;
-		ok = 1;
+		return 1;
 	}
 
 	if( !strcmp( s, "dLocX" ) ) {
 		*param = OB_DLOC_X;
-		ok = 1;
+		return 1;
 	}
 	if( !strcmp( s, "dLocY" ) ) {
 		*param = OB_DLOC_Y;
-		ok = 1;
+		return 1;
 	}
 	if( !strcmp( s, "dLocZ" ) ) {
 		*param = OB_DLOC_Z;
-		ok = 1;
+		return 1;
 	}
 	if( !strcmp( s, "dRotX" ) ) {
 		*param = OB_DROT_X;
-		ok = 1;
+		return 1;
 	}
 	if( !strcmp( s, "dRotY" ) ) {
 		*param = OB_DROT_Y;
-		ok = 1;
+		return 1;
 	}
 	if( !strcmp( s, "dRotZ" ) ) {
 		*param = OB_DROT_Z;
-		ok = 1;
+		return 1;
 	}
 	if( !strcmp( s, "dSizeX" ) ) {
 		*param = OB_DSIZE_X;
-		ok = 1;
+		return 1;
 	}
 	if( !strcmp( s, "dSizeY" ) ) {
 		*param = OB_DSIZE_Y;
-		ok = 1;
+		return 1;
 	}
 	if( !strcmp( s, "dSizeZ" ) ) {
 		*param = OB_DSIZE_Z;
-		ok = 1;
+		return 1;
 	}
 
 	if( !strcmp( s, "Layer" ) ) {
 		*param = OB_LAY;
-		ok = 1;
+		return 1;
 	}
 	if( !strcmp( s, "Time" ) ) {
 		*param = OB_TIME;
-		ok = 1;
+		return 1;
 	}
 
 	if( !strcmp( s, "ColR" ) ) {
 		*param = OB_COL_R;
-		ok = 1;
+		return 1;
 	}
 	if( !strcmp( s, "ColG" ) ) {
 		*param = OB_COL_G;
-		ok = 1;
+		return 1;
 	}
 	if( !strcmp( s, "ColB" ) ) {
 		*param = OB_COL_B;
-		ok = 1;
+		return 1;
 	}
 	if( !strcmp( s, "ColA" ) ) {
 		*param = OB_COL_A;
-		ok = 1;
+		return 1;
 	}
 	if( !strcmp( s, "FStreng" ) ) {
 		*param = OB_PD_FSTR;
-		ok = 1;
+		return 1;
 	}
 	if( !strcmp( s, "FFall" ) ) {
 		*param = OB_PD_FFALL;
-		ok = 1;
+		return 1;
 	}
 	if( !strcmp( s, "Damping" ) ) {
 		*param = OB_PD_SDAMP;
-		ok = 1;
+		return 1;
 	}
 	if( !strcmp( s, "RDamp" ) ) {
 		*param = OB_PD_RDAMP;
-		ok = 1;
+		return 1;
 	}
 	if( !strcmp( s, "Perm" ) ) {
 		*param = OB_PD_PERM;
-		ok = 1;
+		return 1;
 	}
 
 	return ok;
 }
 
+
+/* 
+   Function:  Ipo_addCurve
+   Bpy:       Blender.Ipo.addCurve( 'curname')
+
+   add a new curve to an existing IPO.
+   example:
+   ipo = Blender.Ipo.New('Object','ObIpo')
+   cu = ipo.addCurve('LocX')
+*/
+
 static PyObject *Ipo_addCurve( BPy_Ipo * self, PyObject * args )
 {
-	IpoCurve *get_ipocurve( ID * from, short type, int adrcode,
-				Ipo * useipo );
-	void allspace( unsigned short event, short val );
-	void allqueue( unsigned short event, short val );
-	int param = 0, ok = 0, ipofound = 0;
-	char *s = 0;
+	int param = 0;		/* numeric curve name constant */
+	int ok;
+	char *cur_name = 0;	/* input arg: curve name */
 	Ipo *ipo = 0;
 	IpoCurve *icu = 0;
 	Link *link;
 
-	if( !PyArg_ParseTuple( args, "s", &s ) )
+	if( !PyArg_ParseTuple( args, "s", &cur_name ) )
 		return ( EXPP_ReturnPyObjError
 			 ( PyExc_TypeError, "expected string argument" ) );
 
 
+	/* chase down the ipo list looking for ours */
 	link = G.main->ipo.first;
 
 	while( link ) {
 		ipo = ( Ipo * ) link;
-		if( ipo == self->ipo ) {
-			ipofound = 1;
+		if( ipo == self->ipo )
 			break;
-		}
 		link = link->next;
 	}
 
-	if( ipo && ipofound ) {
-		switch ( ipo->blocktype ) {
-		case ID_OB:
-			ok = Ipo_obIcuName( s, &param );
-			icu = get_ipocurve( NULL, ipo->blocktype, param,
-					    self->ipo );
-			break;
-		case ID_CA:
-			ok = Ipo_caIcuName( s, &param );
-			icu = get_ipocurve( NULL, ipo->blocktype, param,
-					    self->ipo );
-			break;
-		case ID_LA:
-			ok = Ipo_laIcuName( s, &param );
-			icu = get_ipocurve( NULL, ipo->blocktype, param,
-					    self->ipo );
-			break;
-		case ID_TE:
-			ok = Ipo_texIcuName( s, &param );
-			icu = get_ipocurve( NULL, ipo->blocktype, param,
-					    self->ipo );
-			break;
-		case ID_WO:
-			ok = Ipo_woIcuName( s, &param );
-			icu = get_ipocurve( NULL, ipo->blocktype, param,
-					    self->ipo );
-			break;
-		case ID_MA:
-			ok = Ipo_maIcuName( s, &param );
-			icu = get_ipocurve( NULL, ipo->blocktype, param,
-					    self->ipo );
-			break;
-		case ID_AC:
-			ok = Ipo_acIcuName( s, &param );
-			icu = get_ipocurve( NULL, ipo->blocktype, param,
-					    self->ipo );
-			break;
-		case IPO_CO:
-			ok = Ipo_coIcuName( s, &param );
-			icu = get_ipocurve( NULL, ipo->blocktype, param,
-					    self->ipo );
-			break;
-		case ID_CU:
-			ok = Ipo_cuIcuName( s, &param );
-			icu = get_ipocurve( NULL, ipo->blocktype, param,
-					    self->ipo );
-			break;
-		case ID_KE:
-			ok = Ipo_keIcuName( s, &param );
-			icu = get_ipocurve( NULL, ipo->blocktype, param,
-					    self->ipo );
-			break;
-		case ID_SEQ:
-			ok = Ipo_seqIcuName( s, &param );
-			icu = get_ipocurve( NULL, ipo->blocktype, param,
-					    self->ipo );
-			break;
+	if( !link )
+		return EXPP_ReturnPyObjError
+				( PyExc_RuntimeError, "Ipo not found" );
+
+	/*
+	   depending on the block type, 
+	   check if the input arg curve name is valid 
+	   and set param to numeric value.
+	 */
+	switch ( ipo->blocktype ) {
+	case ID_OB:
+		ok = Ipo_obIcuName( cur_name, &param );
+		break;
+	case ID_CA:
+		ok = Ipo_caIcuName( cur_name, &param );
+		break;
+	case ID_LA:
+		ok = Ipo_laIcuName( cur_name, &param );
+		break;
+	case ID_TE:
+		ok = Ipo_texIcuName( cur_name, &param );
+		break;
+	case ID_WO:
+		ok = Ipo_woIcuName( cur_name, &param );
+		break;
+	case ID_MA:
+		ok = Ipo_maIcuName( cur_name, &param );
+		break;
+	case ID_PO:
+		ok = Ipo_acIcuName( cur_name, &param );
+		break;
+	case ID_CO:
+		ok = Ipo_coIcuName( cur_name, &param );
+		break;
+	case ID_CU:
+		ok = Ipo_cuIcuName( cur_name, &param );
+		break;
+	case ID_KE:
+		ok = Ipo_keIcuName( cur_name, &param );
+		break;
+	case ID_SEQ:
+		ok = Ipo_seqIcuName( cur_name, &param );
+		break;
+	default:
+		ok = 0;
+	}
+
+	if( !ok )		/* curve type was invalid */
+		return EXPP_ReturnPyObjError
+			( PyExc_NameError, "curve name was invalid" );
+
+	/* see if the curve already exists */
+	for( icu = ipo->curve.first; icu; icu = icu->next )
+		if( icu->adrcode == param )
+			return EXPP_ReturnPyObjError( PyExc_ValueError,
+					"Ipo curve already exists" );
+
+	/* create the new ipo curve */
+	icu = MEM_callocN(sizeof(IpoCurve), "Python added ipocurve");
+	icu->blocktype= ipo->blocktype;
+	icu->adrcode= (short)param;
+	icu->flag |= IPO_VISIBLE|IPO_AUTO_HORIZ;
+	set_icu_vars( icu );
+	BLI_addtail( &(ipo->curve), icu);
+	
+	allspace( REMAKEIPO, 0 );
+	EXPP_allqueue( REDRAWIPO, 0 );
+
+	/* create a bpy wrapper for the new ipo curve */
+	return IpoCurve_CreatePyObject( icu );
+}
+
+/* 
+   Function:  Ipo_delCurve
+   Bpy:       Blender.Ipo.delCurve(curtype)
+
+   delete an existing curve from IPO.
+   example:
+       ipo = Blender.Ipo.New('Object','ObIpo')
+       cu = ipo.delCurve('LocX')
+*/
+
+static PyObject *Ipo_delCurve( BPy_Ipo * self, PyObject * args )
+{
+	IpoCurve *icu;
+	char *strname;
+
+	if( !PyArg_ParseTuple( args, "s", &strname ) )
+		return ( EXPP_ReturnPyObjError
+			 ( PyExc_TypeError, "string argument" ) );
+
+	for( icu = self->ipo->curve.first; icu; icu = icu->next ) {
+		char *str1 = getIpoCurveName( icu );
+		if( !strcmp( str1, strname ) ) {
+			BLI_remlink( &( self->ipo->curve ), icu );
+			if( icu->bezt )
+				MEM_freeN( icu->bezt );
+			MEM_freeN( icu );
+
+			allspace( REMAKEIPO, 0 );
+			EXPP_allqueue( REDRAWIPO, 0 );
+
+			Py_INCREF( Py_None );
+			return Py_None;
 		}
 	}
 
-	if( icu == 0 )
-		return ( EXPP_ReturnPyObjError
-			 ( PyExc_ValueError, "Not a valid param." ) );
-
-#define REMAKEIPO		1
-#define REDRAWIPO		0x4023
-	allspace( REMAKEIPO, 0 );
-	allqueue( REDRAWIPO, 0 );
-
-	return IpoCurve_CreatePyObject( icu );
+	return ( EXPP_ReturnPyObjError
+		 ( PyExc_RuntimeError, "IpoCurve not found" ) );
 }
 
 
 
 static PyObject *Ipo_getCurve( BPy_Ipo * self, PyObject * args )
 {
-	char *str, *str1;
-	IpoCurve *icu = 0;
-	if( !PyArg_ParseTuple( args, "s", &str ) )
-		return ( EXPP_ReturnPyObjError
-			 ( PyExc_TypeError, "expected string argument" ) );
-	for( icu = self->ipo->curve.first; icu; icu = icu->next ) {
-		str1 = GetIpoCurveName( icu );
-		if( !strcmp( str1, str ) )
-			return IpoCurve_CreatePyObject( icu );
-	}
-
-	Py_INCREF( Py_None );
-	return Py_None;
-}
-
-char *GetIpoCurveName( IpoCurve * icu )
-{
-	switch ( icu->blocktype ) {
-	case ID_MA:
-		{
-			return getname_mat_ei( icu->adrcode );
-		}
-	case ID_WO:
-		{
-			return getname_world_ei( icu->adrcode );
-		}
-	case ID_CA:
-		{
-			return getname_cam_ei( icu->adrcode );
-		}
-	case ID_OB:
-		{
-			return getname_ob_ei( icu->adrcode, 1 );	/* solve: what if EffX/Y/Z are wanted? */
-		}
-	case ID_TE:
-		{
-			return getname_tex_ei( icu->adrcode );
-		}
-	case ID_LA:
-		{
-			return getname_la_ei( icu->adrcode );
-		}
-	case ID_AC:
-		{
-			return getname_ac_ei( icu->adrcode );
-		}
-	case ID_CU:
-		{
-			return getname_cu_ei( icu->adrcode );
-		}
-	case ID_KE:
-		{
-			return getname_key_ei( icu->adrcode );
-		}
-	case ID_SEQ:
-		{
-			return getname_seq_ei( icu->adrcode );
-		}
-	case IPO_CO:
-		{
-			return getname_co_ei( icu->adrcode );
-		}
-	}
-	return NULL;
-}
+    char *str, *str1;
+    IpoCurve *icu = NULL;
+    int adrcode;
+    PyObject *thing;
+ 
+    if( !PyArg_ParseTuple( args, "O", &thing ) )
+        return EXPP_ReturnPyObjError(PyExc_TypeError, 
+			   				"expected string or int argument" );
+ 
+    if(PyString_Check(thing)){
+        str = PyString_AsString(thing);
+        for( icu = self->ipo->curve.first; icu; icu = icu->next ) {
+            str1 = getIpoCurveName( icu );
+            if( !strcmp( str1, str ) )
+                return IpoCurve_CreatePyObject( icu );
+        }
+    } else if (PyInt_Check(thing)){
+        adrcode = (short)PyInt_AsLong(thing);
+        for( icu = self->ipo->curve.first; icu; icu = icu->next ) {
+            if(icu->adrcode == adrcode)
+                 return IpoCurve_CreatePyObject( icu );
+        }
+    } else
+        return EXPP_ReturnPyObjError(PyExc_TypeError, 
+			   				"expected string or int argument" );
+    Py_INCREF( Py_None );
+    return Py_None;
+} 
 
 static PyObject *Ipo_getCurves( BPy_Ipo * self )
 {
@@ -1357,7 +1176,7 @@ static PyObject *Ipo_setCurveBeztriple( BPy_Ipo * self, PyObject * args )
 
 	for( i = 0; i < 9; i++ ) {
 		PyObject *xx = PyTuple_GetItem( listargs, i );
-		ptrbt->vec[i / 3][i % 3] = PyFloat_AsDouble( xx );
+		ptrbt->vec[i / 3][i % 3] = (float)PyFloat_AsDouble( xx );
 	}
 
 	Py_INCREF( Py_None );
@@ -1426,9 +1245,7 @@ static PyObject *Ipo_getCurvecurval( BPy_Ipo * self, PyObject * args )
 				 ( PyExc_TypeError,
 				   "expected int or string argument" ) );
 		while( icu ) {
-			/*char str1[10];
-			   GetIpoCurveName (icu, str1); */
-			str1 = GetIpoCurveName( icu );
+			str1 = getIpoCurveName( icu );
 			if( !strcmp( str1, stringname ) )
 				break;
 			icu = icu->next;
