@@ -1,5 +1,5 @@
 /* 
- * $Id: Lattice.c,v 1.10 2005/05/22 07:22:33 ianwill Exp $
+ * $Id: Lattice.c,v 1.20 2005/11/30 08:18:05 khughes Exp $
  *
  * ***** BEGIN GPL/BL DUAL LICENSE BLOCK *****
  *
@@ -30,27 +30,26 @@
  * ***** END GPL/BL DUAL LICENSE BLOCK *****
  */
 
-#include "Lattice.h"
+#include "Lattice.h" /*This must come first*/
 
-
-#include <BKE_main.h>
-#include <BKE_global.h>
-#include <BKE_library.h>
-#include <BKE_lattice.h>
-#include <BKE_utildefines.h>
-#include <BKE_key.h>
-#include <BLI_blenlib.h>
-
-#include <DNA_key_types.h>
-#include <DNA_curve_types.h>
-#include <DNA_scene_types.h>
-#include <BIF_editlattice.h>
-#include <BIF_editkey.h>
+#include "BKE_utildefines.h"
+#include "BKE_main.h"
+#include "BKE_global.h"
+#include "BKE_library.h"
+#include "BKE_lattice.h"
+#include "BLI_blenlib.h"
+#include "DNA_object_types.h"
+#include "DNA_key_types.h"
+#include "DNA_mesh_types.h"
+#include "DNA_meshdata_types.h"
+#include "DNA_curve_types.h"
+#include "DNA_scene_types.h"
+#include "BIF_editkey.h"
+#include "BIF_editdeform.h"
 #include "blendef.h"
-#include "mydevice.h"
-#include "constant.h"
 #include "gen_utils.h"
 
+#include "Key.h"
 
 /*****************************************************************************/
 /* Python API function prototypes for the Lattice module.	*/
@@ -89,11 +88,12 @@ struct PyMethodDef M_Lattice_methods[] = {
 static PyObject *Lattice_getName( BPy_Lattice * self );
 static PyObject *Lattice_setName( BPy_Lattice * self, PyObject * args );
 static PyObject *Lattice_setPartitions( BPy_Lattice * self, PyObject * args );
-static PyObject *Lattice_getPartitions( BPy_Lattice * self, PyObject * args );
+static PyObject *Lattice_getPartitions( BPy_Lattice * self );
+static PyObject *Lattice_getKey( BPy_Lattice * self );
 static PyObject *Lattice_setKeyTypes( BPy_Lattice * self, PyObject * args );
-static PyObject *Lattice_getKeyTypes( BPy_Lattice * self, PyObject * args );
+static PyObject *Lattice_getKeyTypes( BPy_Lattice * self );
 static PyObject *Lattice_setMode( BPy_Lattice * self, PyObject * args );
-static PyObject *Lattice_getMode( BPy_Lattice * self, PyObject * args );
+static PyObject *Lattice_getMode( BPy_Lattice * self );
 static PyObject *Lattice_setPoint( BPy_Lattice * self, PyObject * args );
 static PyObject *Lattice_getPoint( BPy_Lattice * self, PyObject * args );
 static PyObject *Lattice_applyDeform( BPy_Lattice * self, PyObject *args );
@@ -115,6 +115,9 @@ static char Lattice_setPartitions_doc[] =
 static char Lattice_getPartitions_doc[] =
 	"(str) - Get the number of Partitions in x,y,z";
 
+static char Lattice_getKey_doc[] =
+	"() - Get the Key object attached to this Lattice";
+
 static char Lattice_setKeyTypes_doc[] =
 	"(str) - Set the key types for x,y,z dimensions";
 
@@ -155,6 +158,8 @@ static PyMethodDef BPy_Lattice_methods[] = {
 	 Lattice_setPartitions_doc},
 	{"getPartitions", ( PyCFunction ) Lattice_getPartitions, METH_NOARGS,
 	 Lattice_getPartitions_doc},
+	{"getKey", ( PyCFunction ) Lattice_getKey, METH_NOARGS,
+	 Lattice_getKey_doc},
 	{"setKeyTypes", ( PyCFunction ) Lattice_setKeyTypes, METH_VARARGS,
 	 Lattice_setKeyTypes_doc},
 	{"getKeyTypes", ( PyCFunction ) Lattice_getKeyTypes, METH_NOARGS,
@@ -364,12 +369,12 @@ PyObject *Lattice_Init( void )
 	Lattice_Type.ob_type = &PyType_Type;
 
 	//Module dictionary
-#define EXPP_ADDCONST(x) PyDict_SetItemString(dict, #x, PyInt_FromLong(LT_##x))
+#define EXPP_ADDCONST(x) EXPP_dict_set_item_str(dict, #x, PyInt_FromLong(LT_##x))
 	EXPP_ADDCONST( GRID );
 	EXPP_ADDCONST( OUTSIDE );
 
 #undef EXPP_ADDCONST
-#define EXPP_ADDCONST(x) PyDict_SetItemString(dict, #x, PyInt_FromLong(KEY_##x))
+#define EXPP_ADDCONST(x) EXPP_dict_set_item_str(dict, #x, PyInt_FromLong(KEY_##x))
 	EXPP_ADDCONST( LINEAR );
 	EXPP_ADDCONST( CARDINAL );
 	EXPP_ADDCONST( BSPLINE );
@@ -425,16 +430,13 @@ static PyObject *Lattice_setPartitions( BPy_Lattice * self, PyObject * args )
 		return ( EXPP_ReturnPyObjError( PyExc_RuntimeError,
 						"partition values must be 2 or greater" ) );
 
-	bl_Lattice->pntsu = ( short ) x;
-	bl_Lattice->pntsv = ( short ) y;
-	bl_Lattice->pntsw = ( short ) z;
-	resizelattice( bl_Lattice );
+	resizelattice(bl_Lattice, x, y, z, NULL);
 
 	Py_INCREF( Py_None );
 	return Py_None;
 }
 
-static PyObject *Lattice_getPartitions( BPy_Lattice * self, PyObject * args )
+static PyObject *Lattice_getPartitions( BPy_Lattice * self )
 {
 	Lattice *bl_Lattice;
 	bl_Lattice = self->Lattice;
@@ -444,7 +446,18 @@ static PyObject *Lattice_getPartitions( BPy_Lattice * self, PyObject * args )
 			      ( int ) bl_Lattice->pntsw );
 }
 
-static PyObject *Lattice_getKeyTypes( BPy_Lattice * self, PyObject * args )
+static PyObject *Lattice_getKey( BPy_Lattice * self )
+{
+	Key *key = self->Lattice->key;
+
+	if (key)
+		return Key_CreatePyObject(key);
+	else {
+		return EXPP_incr_ret(Py_None);
+	}
+}
+
+static PyObject *Lattice_getKeyTypes( BPy_Lattice * self )
 {
 	Lattice *bl_Lattice;
 	char *linear = "linear";
@@ -469,7 +482,7 @@ static PyObject *Lattice_getKeyTypes( BPy_Lattice * self, PyObject * args )
 	else if( ( bl_Lattice->typev ) == KEY_CARDINAL )
 		s_y = cardinal;
 	else if( ( bl_Lattice->typev ) == KEY_BSPLINE )
-		s_z = bspline;
+		s_y = bspline;
 	else
 		return EXPP_ReturnPyObjError( PyExc_RuntimeError,
 					      "bad key type..." );
@@ -558,7 +571,7 @@ static PyObject *Lattice_setMode( BPy_Lattice * self, PyObject * args )
 	return Py_None;
 }
 
-static PyObject *Lattice_getMode( BPy_Lattice * self, PyObject * args )
+static PyObject *Lattice_getMode( BPy_Lattice * self )
 {
 	char type[24];
 	Lattice *bl_Lattice;
@@ -684,7 +697,7 @@ static PyObject *Lattice_applyDeform( BPy_Lattice * self, PyObject *args )
 {
 	//Object* ob; unused
 	Base *base;
-	Object *par;
+	Object *par = NULL;
 	int forced = 0;
 
 	if( !Lattice_IsLinkedToObject( self ) )
@@ -698,7 +711,8 @@ static PyObject *Lattice_applyDeform( BPy_Lattice * self, PyObject *args )
 	/* deform children */
 	base = FIRSTBASE;
 	while( base ) {
-		if( ( par = base->object->parent ) ) { /* check/assign if ob has parent */
+		par = base->object->parent;
+		if( par != NULL ) { /* check/assign if ob has parent */
 			/* meshes have their mverts deformed, others ob types use displist,
 			 * so we're not doing meshes here (unless forced), or else they get
 			 * deformed twice, since parenting a Lattice to an object and redrawing
@@ -707,8 +721,12 @@ static PyObject *Lattice_applyDeform( BPy_Lattice * self, PyObject *args )
 			 * method is needed.  Or for users who actually want to apply the
 			 * deformation n times. */
 			if((self->Lattice == par->data)) {
-				if ((base->object->type != OB_MESH) || forced)
-					object_deform( base->object );
+					/* I do not know what to do with this function
+					 * at the moment given the changing modifier system.
+					 * Calling into the modifier system in the first place
+					 * isn't great... -zr 
+					 */
+				object_apply_deform(base->object);
 			}
 		}
 		base = base->next;
@@ -733,17 +751,17 @@ static PyObject *Lattice_insertKey( BPy_Lattice * self, PyObject * args )
 	if( frame > 0 ) {
 		frame = EXPP_ClampInt( frame, 1, MAXFRAME );
 		oldfra = G.scene->r.cfra;
-		G.scene->r.cfra = frame;
+		G.scene->r.cfra = (short)frame;
 	}
 //      else just use current frame, then
 //              return (EXPP_ReturnPyObjError (PyExc_RuntimeError,
 //                                              "frame value has to be greater than 0"));
 
-	//insert a keybock for the lattice
-	insert_lattkey( lt );
+	//insert a keybock for the lattice (1=relative)
+	insert_lattkey( lt , 1);
 
 	if( frame > 0 )
-		G.scene->r.cfra = oldfra;
+		G.scene->r.cfra = (short)oldfra;
 
 	Py_INCREF( Py_None );
 	return Py_None;
@@ -825,12 +843,13 @@ static PyObject *Lattice_getAttr( BPy_Lattice * self, char *name )
 				      self->Lattice->pntsw );
 	} else if( strcmp( name, "users" ) == 0 ) {
 		attr = PyInt_FromLong( self->Lattice->id.us );
-	
+	} else if( strcmp( name, "key" ) == 0 ) {
+		return Lattice_getKey(self);
 	} else if( strcmp( name, "__members__" ) == 0 )
-		attr = Py_BuildValue( "[s,s,s,s,s,s,s,s,s]", "name", "width",
+		attr = Py_BuildValue( "[s,s,s,s,s,s,s,s,s,s]", "name", "width",
 				      "height", "depth", "widthType",
 				      "heightType", "depthType", "mode",
-				      "latSize", "users" );
+				      "latSize", "users", "key" );
 	
 	if( !attr )
 		return ( EXPP_ReturnPyObjError( PyExc_MemoryError,
@@ -867,7 +886,7 @@ static int Lattice_setAttr( BPy_Lattice * self, char *name, PyObject * value )
 	if( strcmp( name, "name" ) == 0 )
 		error = Lattice_setName( self, valtuple );
 	else {			// Error: no such member in the Lattice Data structure 
-		Py_DECREF( value );
+		/*Py_DECREF( value ); borrowed reference, no need to decref */
 		Py_DECREF( valtuple );
 		return ( EXPP_ReturnIntError( PyExc_KeyError,
 					      "attribute not found or immutable" ) );
