@@ -1,5 +1,5 @@
 /**
- * $Id: Library.c,v 1.6 2005/03/09 19:45:55 lukep Exp $
+ * $Id: Library.c,v 1.12 2005/12/13 14:29:56 ascotan Exp $
  *
  * Blender.Library BPython module implementation.
  * This submodule has functions to append data from .blend files.
@@ -34,20 +34,20 @@
 */
 
 #include <Python.h>
-#include <stdio.h>
 
-#include <DNA_ID.h>
-#include <DNA_curve_types.h>
-#include <BKE_library.h>	/* for all_local */
-#include "BKE_displist.h"	/* for set_displist_onlyzero */
+#include "DNA_curve_types.h"
+#include "DNA_object_types.h"
+#include "BKE_library.h"	/* for all_local */
 #include "BKE_font.h"		/* for text_to_curve */
-#include <BLO_readfile.h>
-#include <BLI_linklist.h>
-#include <MEM_guardedalloc.h>
-
+#include "BKE_utildefines.h"
+#include "BKE_global.h"
+#include "BKE_main.h"
+#include "BLI_blenlib.h"
+#include "BLO_readfile.h"
+#include "BLI_linklist.h"
+#include "MEM_guardedalloc.h"
 #include "gen_utils.h"
 
-
 /**
  * Global variables.
  */
@@ -139,6 +139,8 @@ struct PyMethodDef M_Library_methods[] = {
 PyObject *M_Library_Open( PyObject * self, PyObject * args )
 {
 	char *fname = NULL;
+	char filename[FILE_MAXDIR+FILE_MAXFILE];
+
 	int len = 0;
 
 	if( !PyArg_ParseTuple( args, "s", &fname ) ) {
@@ -151,7 +153,10 @@ PyObject *M_Library_Open( PyObject * self, PyObject * args )
 		Py_DECREF( Py_None );	/* incref'ed by above function */
 	}
 
+   	/* G.sce = last file loaded, save for UI and restore after opening file */
+	BLI_strncpy(filename, G.sce, sizeof(filename));
 	bpy_openlib = BLO_blendhandle_from_file( fname );
+	BLI_strncpy(G.sce, filename, sizeof(filename)); 
 
 	if( !bpy_openlib )
 		return Py_BuildValue( "i", 0 );
@@ -339,28 +344,12 @@ PyObject *M_Library_Load( PyObject * self, PyObject * args )
  */
 PyObject *M_Library_Update( PyObject * self )
 {				/* code adapted from do_library_append in src/filesel.c: */
-	Object *ob = NULL;
 	Library *lib = NULL;
 
-	ob = G.main->object.first;
-	set_displist_onlyzero( 1 );
-	while( ob ) {
-		if( ob->id.lib ) {
-			if( ob->type == OB_FONT ) {
-				Curve *cu = ob->data;
-				if( cu->nurb.first == 0 )
-					text_to_curve( ob, 0 );
-			}
-			makeDispList( ob );
-		} else {
-			if( ob->type == OB_MESH && ob->parent
-			    && ob->parent->type == OB_LATTICE )
-				makeDispList( ob );
-		}
-
-		ob = ob->id.next;
-	}
-	set_displist_onlyzero( 0 );
+		/* Displist code that was here is obsolete... depending on what
+		 * this function is supposed to do (it should technically be unnecessary)
+		 * can be replaced with depgraph calls - zr
+		 */
 
 	if( bpy_openlibname ) {
 		strcpy( G.lib, bpy_openlibname );
