@@ -1,5 +1,5 @@
 /**
- * $Id: Noise.c,v 1.4 2005/03/09 19:45:55 lukep Exp $
+ * $Id: Noise.c,v 1.6 2005/08/17 14:26:00 ascotan Exp $
  *
  * Blender.Noise BPython module implementation.
  * This submodule has functions to generate noise of various types.
@@ -38,9 +38,9 @@
 /************************/
 
 #include <Python.h>
-#include <math.h>
-#include <BLI_blenlib.h>
-#include <DNA_texture_types.h>
+
+#include "BLI_blenlib.h"
+#include "DNA_texture_types.h"
 #include "constant.h"
 
 /*-----------------------------------------*/
@@ -136,10 +136,10 @@ static void randuvec( float v[3] )
 	float r;
 	v[2] = 2.f * frand(  ) - 1.f;
 	if( ( r = 1.f - v[2] * v[2] ) > 0.f ) {
-		float a = 6.283185307f * frand(  );
-		r = sqrt( r );
-		v[0] = r * cos( a );
-		v[1] = r * sin( a );
+		float a = (float)(6.283185307f * frand(  ));
+		r = (float)sqrt( r );
+		v[0] = (float)(r * cos( a ));
+		v[1] = (float)(r * sin( a ));
 	} else
 		v[2] = 1.f;
 }
@@ -191,11 +191,11 @@ static PyObject *Noise_noise( PyObject * self, PyObject * args )
 static void vNoise( float x, float y, float z, int nb, float v[3] )
 {
 	/* Simply evaluate noise at 3 different positions */
-	v[0] = 2.0 * BLI_gNoise( 1.f, x + 9.321f, y - 1.531f, z - 7.951f, 0,
-				 nb ) - 1.0;
-	v[1] = 2.0 * BLI_gNoise( 1.f, x, y, z, 0, nb ) - 1.0;
-	v[2] = 2.0 * BLI_gNoise( 1.f, x + 6.327f, y + 0.1671f, z - 2.672f, 0,
-				 nb ) - 1.0;
+	v[0] = (float)(2.0 * BLI_gNoise( 1.f, x + 9.321f, y - 1.531f, z - 7.951f, 0,
+				 nb ) - 1.0);
+	v[1] = (float)(2.0 * BLI_gNoise( 1.f, x, y, z, 0, nb ) - 1.0);
+	v[2] = (float)(2.0 * BLI_gNoise( 1.f, x + 6.327f, y + 0.1671f, z - 2.672f, 0,
+				 nb ) - 1.0);
 }
 
 static PyObject *Noise_vNoise( PyObject * self, PyObject * args )
@@ -218,17 +218,17 @@ static float turb( float x, float y, float z, int oct, int hard, int nb,
 	float amp, out, t;
 	int i;
 	amp = 1.f;
-	out = 2.0 * BLI_gNoise( 1.f, x, y, z, 0, nb ) - 1.0;
+	out = (float)(2.0 * BLI_gNoise( 1.f, x, y, z, 0, nb ) - 1.0);
 	if( hard )
-		out = fabs( out );
+		out = (float)fabs( out );
 	for( i = 1; i < oct; i++ ) {
 		amp *= ampscale;
 		x *= freqscale;
 		y *= freqscale;
 		z *= freqscale;
-		t = amp * ( 2.0 * BLI_gNoise( 1.f, x, y, z, 0, nb ) - 1.0 );
+		t = (float)(amp * ( 2.0 * BLI_gNoise( 1.f, x, y, z, 0, nb ) - 1.0 ));
 		if( hard )
-			t = fabs( t );
+			t = (float)fabs( t );
 		out += t;
 	}
 	return out;
@@ -257,9 +257,9 @@ static void vTurb( float x, float y, float z, int oct, int hard, int nb,
 	amp = 1.f;
 	vNoise( x, y, z, nb, v );
 	if( hard ) {
-		v[0] = fabs( v[0] );
-		v[1] = fabs( v[1] );
-		v[2] = fabs( v[2] );
+		v[0] = (float)fabs( v[0] );
+		v[1] = (float)fabs( v[1] );
+		v[2] = (float)fabs( v[2] );
 	}
 	for( i = 1; i < oct; i++ ) {
 		amp *= ampscale;
@@ -268,9 +268,9 @@ static void vTurb( float x, float y, float z, int oct, int hard, int nb,
 		z *= freqscale;
 		vNoise( x, y, z, nb, t );
 		if( hard ) {
-			t[0] = fabs( t[0] );
-			t[1] = fabs( t[1] );
-			t[2] = fabs( t[2] );
+			t[0] = (float)fabs( t[0] );
+			t[1] = (float)fabs( t[1] );
+			t[2] = (float)fabs( t[2] );
 		}
 		v[0] += amp * t[0];
 		v[1] += amp * t[1];
@@ -615,49 +615,49 @@ PyObject *Noise_Init(void)
 	setRndSeed( 0 );	
 
 	/* Constant noisetype dictionary */
-	NoiseTypes = M_constant_New(  );
+	NoiseTypes = PyConstant_New(  );
 	if( NoiseTypes ) {
 		BPy_constant *nt = ( BPy_constant * ) NoiseTypes;
-		constant_insert( nt, "BLENDER",
+		PyConstant_Insert( nt, "BLENDER",
 				 PyInt_FromLong( TEX_BLENDER ) );
-		constant_insert( nt, "STDPERLIN",
+		PyConstant_Insert( nt, "STDPERLIN",
 				 PyInt_FromLong( TEX_STDPERLIN ) );
-		constant_insert( nt, "NEWPERLIN",
+		PyConstant_Insert( nt, "NEWPERLIN",
 				 PyInt_FromLong( TEX_NEWPERLIN ) );
-		constant_insert( nt, "VORONOI_F1",
+		PyConstant_Insert( nt, "VORONOI_F1",
 				 PyInt_FromLong( TEX_VORONOI_F1 ) );
-		constant_insert( nt, "VORONOI_F2",
+		PyConstant_Insert( nt, "VORONOI_F2",
 				 PyInt_FromLong( TEX_VORONOI_F2 ) );
-		constant_insert( nt, "VORONOI_F3",
+		PyConstant_Insert( nt, "VORONOI_F3",
 				 PyInt_FromLong( TEX_VORONOI_F3 ) );
-		constant_insert( nt, "VORONOI_F4",
+		PyConstant_Insert( nt, "VORONOI_F4",
 				 PyInt_FromLong( TEX_VORONOI_F4 ) );
-		constant_insert( nt, "VORONOI_F2F1",
+		PyConstant_Insert( nt, "VORONOI_F2F1",
 				 PyInt_FromLong( TEX_VORONOI_F2F1 ) );
-		constant_insert( nt, "VORONOI_CRACKLE",
+		PyConstant_Insert( nt, "VORONOI_CRACKLE",
 				 PyInt_FromLong( TEX_VORONOI_CRACKLE ) );
-		constant_insert( nt, "CELLNOISE",
+		PyConstant_Insert( nt, "CELLNOISE",
 				 PyInt_FromLong( TEX_CELLNOISE ) );
 		PyModule_AddObject( md, "NoiseTypes", NoiseTypes );
 	}
 
 	/* Constant distance metric dictionary for voronoi */
-	DistanceMetrics = M_constant_New(  );
+	DistanceMetrics = PyConstant_New(  );
 	if( DistanceMetrics ) {
 		BPy_constant *dm = ( BPy_constant * ) DistanceMetrics;
-		constant_insert( dm, "DISTANCE",
+		PyConstant_Insert( dm, "DISTANCE",
 				 PyInt_FromLong( TEX_DISTANCE ) );
-		constant_insert( dm, "DISTANCE_SQUARED",
+		PyConstant_Insert( dm, "DISTANCE_SQUARED",
 				 PyInt_FromLong( TEX_DISTANCE_SQUARED ) );
-		constant_insert( dm, "MANHATTAN",
+		PyConstant_Insert( dm, "MANHATTAN",
 				 PyInt_FromLong( TEX_MANHATTAN ) );
-		constant_insert( dm, "CHEBYCHEV",
+		PyConstant_Insert( dm, "CHEBYCHEV",
 				 PyInt_FromLong( TEX_CHEBYCHEV ) );
-		constant_insert( dm, "MINKOVSKY_HALF",
+		PyConstant_Insert( dm, "MINKOVSKY_HALF",
 				 PyInt_FromLong( TEX_MINKOVSKY_HALF ) );
-		constant_insert( dm, "MINKOVSKY_FOUR",
+		PyConstant_Insert( dm, "MINKOVSKY_FOUR",
 				 PyInt_FromLong( TEX_MINKOVSKY_FOUR ) );
-		constant_insert( dm, "MINKOVSKY",
+		PyConstant_Insert( dm, "MINKOVSKY",
 				 PyInt_FromLong( TEX_MINKOVSKY ) );
 		PyModule_AddObject( md, "DistanceMetrics", DistanceMetrics );
 	}
