@@ -1,5 +1,5 @@
 /* 
- * $Id: Scene.c,v 1.32 2005/05/27 05:39:40 ianwill Exp $
+ * $Id: Scene.c,v 1.42 2006/01/06 01:53:26 campbellbarton Exp $
  *
  * ***** BEGIN GPL/BL DUAL LICENSE BLOCK *****
  *
@@ -26,33 +26,36 @@
  * This is a new part of Blender.
  *
  * Contributor(s): Willian P. Germano, Jacques Guignot, Joseph Gilbert,
- * Campbell Barton.
+ * Campbell Barton, Ken Hughes
  *
  * ***** END GPL/BL DUAL LICENSE BLOCK *****
 */
+struct View3D;
 
-#include <BKE_main.h>
-#include <BKE_global.h>
-#include <BKE_scene.h>
-#include <BKE_library.h>
-#include <BLI_blenlib.h>
-#include <BSE_drawview.h>	/* for play_anim */
-#include <BSE_headerbuttons.h>	/* for copy_scene */
-#include <BIF_drawscene.h>	/* for set_scene */
-#include <BIF_space.h>		/* for copy_view3d_lock() */
-#include <BIF_screen.h>		/* curarea */
-#include <DNA_screen_types.h>	/* SPACE_VIEW3D, SPACE_SEQ */
-#include <DNA_scriptlink_types.h>
-#include <MEM_guardedalloc.h>	/* for MEM_callocN */
-#include <mydevice.h>		/* for #define REDRAW */
+#include "Scene.h" /*This must come first */
 
+#include "BKE_global.h"
+#include "BKE_main.h"
+#include "MEM_guardedalloc.h"	/* for MEM_callocN */
+#include "DNA_screen_types.h"	/* SPACE_VIEW3D, SPACE_SEQ */
+#include "BKE_depsgraph.h"
+#include "BKE_library.h"
+#include "BKE_scene.h"
+#include "BLI_blenlib.h"
+#include "BSE_drawview.h"	/* for play_anim */
+#include "BSE_headerbuttons.h"	/* for copy_scene */
+#include "BIF_drawscene.h"	/* for set_scene */
+#include "BIF_space.h"		/* for copy_view3d_lock() */
+#include "BIF_screen.h"		/* curarea */
+#include "mydevice.h"		/* for #define REDRAW */
+#include "DNA_view3d_types.h"
 #include "Object.h"
-#include "constant.h"
+#include "Camera.h"
 #include "gen_utils.h"
 #include "sceneRender.h"
 #include "sceneRadio.h"
+#include "sceneTimeLine.h"
 
-#include "Scene.h"
 
 static Base *EXPP_Scene_getObjectBase( Scene * scene, Object * object );
 PyObject *M_Object_Get( PyObject * self, PyObject * args ); /* from Object.c */
@@ -112,6 +115,8 @@ static PyObject *Scene_getScriptLinks( BPy_Scene * self, PyObject * args );
 static PyObject *Scene_addScriptLink( BPy_Scene * self, PyObject * args );
 static PyObject *Scene_clearScriptLinks( BPy_Scene * self, PyObject * args );
 static PyObject *Scene_play( BPy_Scene * self, PyObject * args );
+static PyObject *Scene_getTimeLine( BPy_Scene * self );
+
 
 //internal
 static void Scene_dealloc( BPy_Scene * self );
@@ -141,7 +146,7 @@ static PyMethodDef BPy_Scene_methods[] = {
 	{"update", ( PyCFunction ) Scene_update, METH_VARARGS,
 	 "(full = 0) - Update scene self.\n"
 	 "full = 0: sort the base list of objects."
-	 "full = 1: full update -- also regroups, does ipos, ikas, keys"},
+	 "full = 1: full update -- also regroups, does ipos, keys"},
 	{"link", ( PyCFunction ) Scene_link, METH_VARARGS,
 	 "(obj) - Link Object obj to this scene"},
 	{"unlink", ( PyCFunction ) Scene_unlink, METH_VARARGS,
@@ -187,6 +192,8 @@ static PyMethodDef BPy_Scene_methods[] = {
 	 "since they can be used just as an interruptible timer.  If 'win' is not"
 	 "available or invalid, VIEW3D is tried, then any bigger window."
 	 "Returns 0 for normal exit or 1 when canceled by user input."},
+	{"getTimeLine", ( PyCFunction ) Scene_getTimeLine, METH_NOARGS,
+	"() - Get time line of this Scene"},
 	{NULL, NULL, 0, NULL}
 };
 //-----------------------BPy_Scene method def------------------------------
@@ -702,12 +709,12 @@ static PyObject *Scene_update( BPy_Scene * self, PyObject * args )
 		return EXPP_ReturnPyObjError( PyExc_TypeError,
 					      "expected nothing or int (0 or 1) argument" );
 
-/* Under certain circunstances, sort_baselist *here* can crash Blender.
+/* Under certain circunstances, DAG_scene_sort *here* can crash Blender.
  * A "RuntimeError: max recursion limit" happens when a scriptlink
  * on frame change has scene.update(1).
  * Investigate better how to avoid this. */
 	if( !full )
-		sort_baselist( scene );
+		DAG_scene_sort( scene );
 
 	else if( full == 1 )
 		set_scene_bg( scene );
@@ -716,7 +723,7 @@ static PyObject *Scene_update( BPy_Scene * self, PyObject * args )
 		return EXPP_ReturnPyObjError( PyExc_ValueError,
 					      "in method scene.update(full), full should be:\n"
 					      "0: to only sort scene elements (old behavior); or\n"
-					      "1: for a full update (regroups, does ipos, ikas, keys, etc.)" );
+					      "1: for a full update (regroups, does ipos, keys, etc.)" );
 
 	Py_INCREF( Py_None );
 	return Py_None;
@@ -727,6 +734,7 @@ static PyObject *Scene_link( BPy_Scene * self, PyObject * args )
 {
 	Scene *scene = self->scene;
 	BPy_Object *bpy_obj;
+	Object *object = NULL;
 
 	if( !scene )
 		return EXPP_ReturnPyObjError( PyExc_RuntimeError,
@@ -735,9 +743,19 @@ static PyObject *Scene_link( BPy_Scene * self, PyObject * args )
 	if( !PyArg_ParseTuple( args, "O!", &Object_Type, &bpy_obj ) )
 		return EXPP_ReturnPyObjError( PyExc_TypeError,
 					      "expected Object argument" );
-
-	else {			/* Ok, all is fine, let's try to link it */
-		Object *object = bpy_obj->object;
+	
+	
+		//return EXPP_ReturnPyObjError( PyExc_RuntimeError,
+		//			          "Could not create data on demand for this object type!" );
+	
+	object = bpy_obj->object;
+	
+	/* Object.c's EXPP_add_obdata does not support these objects */
+	if (!object->data && (object->type == OB_SURF || object->type == OB_FONT || object->type == OB_WAVE )) {
+		return EXPP_ReturnPyObjError( PyExc_RuntimeError,
+					      "Object has no data and new data cant be automaticaly created for Surf, Text or Wave type objects!" );
+	} else {
+		/* Ok, all is fine, let's try to link it */
 		Base *base;
 
 		/* We need to link the object to a 'Base', then link this base
@@ -759,8 +777,9 @@ static PyObject *Scene_link( BPy_Scene * self, PyObject * args )
 						      "couldn't allocate new Base for object" );
 
 		/* check if this object has obdata, case not, try to create it */
+		
 		if( !object->data && ( object->type != OB_EMPTY ) )
-			EXPP_add_obdata( object );	/* returns -1 on error, defined in Object.c */
+			EXPP_add_obdata( object ); /* returns -1 on error, defined in Object.c */
 
 		base->object = object;	/* link object to the new base */
 		base->lay = object->lay;
@@ -825,18 +844,16 @@ static PyObject *Scene_getChildren( BPy_Scene * self )
 
 	while( base ) {
 		object = base->object;
-
-		bpy_obj = M_Object_Get( Py_None,
-					Py_BuildValue( "(s)",
-						       object->id.name + 2 ) );
-
+		
+		bpy_obj = Object_CreatePyObject( object );
+		
 		if( !bpy_obj )
 			return EXPP_ReturnPyObjError( PyExc_RuntimeError,
 						      "couldn't create new object wrapper" );
 
 		PyList_Append( pylist, bpy_obj );
 		Py_XDECREF( bpy_obj );	/* PyList_Append incref'ed it */
-
+		
 		base = base->next;
 	}
 
@@ -857,11 +874,7 @@ static PyObject *Scene_getActiveObject(BPy_Scene *self)
 	ob = ((scene->basact) ? (scene->basact->object) : 0);
 
 	if (ob) {
-		PyObject *arg = Py_BuildValue("(s)", ob->id.name+2);
-
-		pyob = M_Object_Get(Py_None, arg);
-
-		Py_DECREF(arg);
+		pyob = Object_CreatePyObject( ob );
 
 		if (!pyob)
 			return EXPP_ReturnPyObjError(PyExc_MemoryError,
@@ -877,18 +890,22 @@ static PyObject *Scene_getActiveObject(BPy_Scene *self)
 static PyObject *Scene_getCurrentCamera( BPy_Scene * self )
 {
 	Object *cam_obj;
+	PyObject *pyob;
 	Scene *scene = self->scene;
-
+	
 	if( !scene )
 		return EXPP_ReturnPyObjError( PyExc_RuntimeError,
 					      "Blender Scene was deleted!" );
 
 	cam_obj = scene->camera;
 
-	if( cam_obj )		/* if found, return a wrapper for it */
-		return M_Object_Get( Py_None,
-				     Py_BuildValue( "(s)",
-						    cam_obj->id.name + 2 ) );
+	if( cam_obj ) {		/* if found, return a wrapper for it */
+		pyob = Object_CreatePyObject( cam_obj );
+		if (!pyob)
+			return EXPP_ReturnPyObjError(PyExc_MemoryError,
+					"couldn't create new object wrapper!");
+		return pyob;
+	}
 
 	Py_INCREF( Py_None );	/* none found */
 	return Py_None;
@@ -910,11 +927,14 @@ static PyObject *Scene_setCurrentCamera( BPy_Scene * self, PyObject * args )
 					      "expected Camera Object as argument" );
 
 	object = cam_obj->object;
+	if( object->type != OB_CAMERA )
+		return EXPP_ReturnPyObjError( PyExc_ValueError,
+					      "expected Camera Object as argument" );
 
 	scene->camera = object;	/* set the current Camera */
 
 	/* if this is the current scene, update its window now */
-	if( scene == G.scene )
+	if( !G.background && scene == G.scene ) /* Traced a crash to redrawing while in background mode -Campbell */
 		copy_view3d_lock( REDRAW );
 
 /* XXX copy_view3d_lock(REDRAW) prints "bad call to addqueue: 0 (18, 1)".
@@ -1052,3 +1072,20 @@ static PyObject *Scene_play( BPy_Scene * self, PyObject * args )
 
 	return ret;
 }
+
+static PyObject *Scene_getTimeLine( BPy_Scene *self ) 
+{
+	BPy_TimeLine *tm= NULL;
+
+	tm= (BPy_TimeLine *) PyObject_NEW (BPy_TimeLine, &TimeLine_Type);
+	if (!tm)
+		return EXPP_ReturnPyObjError (PyExc_MemoryError,
+					      "couldn't create BPy_TimeLine object");
+	tm->marker_list= &(self->scene->markers);
+	tm->sfra= (int) self->scene->r.sfra;
+	tm->efra= (int) self->scene->r.efra;
+
+	return (PyObject *)tm;
+}
+
+
