@@ -1,5 +1,5 @@
 /* 
- * $Id: Window.c,v 1.24 2004/12/05 04:01:57 ianwill Exp $
+ * $Id: Window.c,v 1.39 2006/01/16 02:06:02 khughes Exp $
  *
  * ***** BEGIN GPL/BL DUAL LICENSE BLOCK *****
  *
@@ -25,50 +25,43 @@
  *
  * This is a new part of Blender.
  *
- * Contributor(s): Willian P. Germano, Tom Musgrove
+ * Contributor(s): Willian P. Germano, Tom Musgrove, Michael Reimpell,
+ * Yann Vernier, Ken Hughes
  *
  * ***** END GPL/BL DUAL LICENSE BLOCK *****
 */
 
 #include <Python.h>
-#include <stdio.h>
-
-#include <blendef.h>		/* OBACT */
-#include <BDR_editobject.h>	/* enter / leave editmode */
-#include <BKE_global.h>
-#include <BKE_library.h>
-#include <BKE_object.h>		/* for during_script() */
-#include <BKE_scene.h>		/* scene_find_camera() */
-#include <BIF_usiblender.h>
-#include <BIF_mywindow.h>
-#include <BSE_headerbuttons.h>
-#include <BSE_filesel.h>
-#include <BIF_editmesh.h>	/* for undo_push_mesh() */
-#include <BIF_screen.h>
-#include <BIF_space.h>
-#include <BIF_drawtext.h>
-#include <BIF_mywindow.h> /* L/M/R_MOUSE bitflags */
-#include <BIF_spacetypes.h>
-#include <mydevice.h>
-#include <DNA_view3d_types.h>
-#include <DNA_screen_types.h>
-#include <DNA_space_types.h>
-#include <DNA_scene_types.h>
-#include <DNA_text_types.h>
-#include <DNA_vec_types.h>	/* for rcti struct */
 
+#include "BDR_editobject.h"	/* enter / leave editmode */
+#include "BKE_global.h"
+#include "BKE_main.h"
+#include "BKE_object.h"		/* for during_script() and during_scriptlink() */
+#include "BKE_scene.h"		/* scene_find_camera() */
+#include "BPI_script.h"
+#include "BIF_mywindow.h"
+#include "BSE_headerbuttons.h"
+#include "BSE_filesel.h"
+#include "BIF_editmesh.h"	/* for undo_push_mesh() */
+#include "BIF_screen.h"
+#include "BIF_space.h"
+#include "BIF_drawtext.h"
+#include "DNA_view3d_types.h"
+#include "DNA_space_types.h"
+#include "DNA_scene_types.h"
+#include "DNA_text_types.h"
+#include "DNA_object_types.h"
+#include "mydevice.h"
+#include "blendef.h"		/* OBACT */
 #include "windowTheme.h"
-#include "gen_utils.h"
-#include "matrix.h"
-#include "vector.h"
+#include "Mathutils.h"
 #include "constant.h"
+#include "gen_utils.h"
 
 
 /* See Draw.c */
 extern int EXPP_disable_force_draw;
-
-/* Callback used by the file and image selector access functions */
-static PyObject *( *EXPP_FS_PyCallback ) ( PyObject * arg ) = NULL;
+extern void setcameratoview3d(void);
 
 /*****************************************************************************/
 /* Python API function prototypes for the Window module.		*/
@@ -90,7 +83,7 @@ static PyObject *M_Window_GetPerspMatrix( PyObject * self );
 static PyObject *M_Window_FileSelector( PyObject * self, PyObject * args );
 static PyObject *M_Window_ImageSelector( PyObject * self, PyObject * args );
 static PyObject *M_Window_EditMode( PyObject * self, PyObject * args );
-static PyObject *M_Window_ViewLayer( PyObject * self, PyObject * args );
+static PyObject *M_Window_ViewLayers( PyObject * self, PyObject * args );
 static PyObject *M_Window_CameraView( PyObject * self, PyObject * args );
 static PyObject *M_Window_QTest( PyObject * self );
 static PyObject *M_Window_QRead( PyObject * self );
@@ -108,6 +101,8 @@ static PyObject *M_Window_GetScreens( PyObject * self );
 static PyObject *M_Window_SetScreen( PyObject * self, PyObject * args );
 static PyObject *M_Window_GetScreenInfo( PyObject * self, PyObject * args,
 					 PyObject * kwords );
+PyObject *Window_Init( void );
+
 
 /*****************************************************************************/
 /* The following string definitions are used for documentation strings.	    */
@@ -176,7 +171,7 @@ Returns the current status.  This function is mostly useful to leave\n\
 edit mode before applying changes to a mesh (otherwise the changes will\n\
 be lost) and then returning to it upon leaving.";
 
-static char M_Window_ViewLayer_doc[] =
+static char M_Window_ViewLayers_doc[] =
 	"(layers = []) - Get/set active layers in all 3d View windows.\n\
 () - Make no changes, only return currently visible layers.\n\
 (layers = []) - a list of integers, each in the range [1, 20].\n\
@@ -315,8 +310,11 @@ struct PyMethodDef M_Window_methods[] = {
 	 M_Window_GetPerspMatrix_doc},
 	{"EditMode", ( PyCFunction ) M_Window_EditMode, METH_VARARGS,
 	 M_Window_EditMode_doc},
-	{"ViewLayer", ( PyCFunction ) M_Window_ViewLayer, METH_VARARGS,
-	 M_Window_ViewLayer_doc},
+	{"ViewLayers", ( PyCFunction ) M_Window_ViewLayers, METH_VARARGS,
+	 M_Window_ViewLayers_doc},
+	 /* typo, deprecate someday: */
+	{"ViewLayer", ( PyCFunction ) M_Window_ViewLayers, METH_VARARGS,
+	 M_Window_ViewLayers_doc},
 	{"CameraView", ( PyCFunction ) M_Window_CameraView, METH_VARARGS,
 	 M_Window_CameraView_doc},
 	{"QTest", ( PyCFunction ) M_Window_QTest, METH_NOARGS,
@@ -376,33 +374,28 @@ PyObject *M_Window_Redraw( PyObject * self, PyObject * args )
 	if( wintype < 0 )
 		redraw_all = 1;
 
-	if( !during_script(  ) ) {
+	if( !during_script( ) && !G.background ) {
 		tempsa = curarea;
 		sa = G.curscreen->areabase.first;
 
 		while( sa ) {
 
 			if( sa->spacetype == wintype || redraw_all ) {
-				if( sa->spacetype == SPACE_TEXT ) {
-					st = sa->spacedata.first;
-					if( st->text ) {
-						if( st->text->flags & TXT_FOLLOW )	/* follow cursor display */
-							pop_space_text( st );
-
-						// XXX making a test: Jul 07, 2004.
-						// we don't need to prevent text win redraws anymore,
-						// since now there's a scripts space instead.
-						//if (EXPP_disable_force_draw) { /* defined in Draw.[ch] ... */
-						//      scrarea_queue_redraw(sa);
+				if (sa->spacetype == SPACE_SCRIPT && EXPP_disable_force_draw) {
+						scrarea_queue_redraw(sa);
+				}
+				else {
+					if( sa->spacetype == SPACE_TEXT ) {
+						st = sa->spacedata.first;
+						if( st->text ) {
+							if( st->text->flags & TXT_FOLLOW )	/* follow cursor display */
+								pop_space_text( st );
+						}
 					}
+					scrarea_do_windraw( sa );
+					if( sa->headwin ) scrarea_do_headdraw( sa );
 				}
-				//} else {
-				scrarea_do_windraw( sa );
-				if( sa->headwin )
-					scrarea_do_headdraw( sa );
-				//}
 			}
-
 			sa = sa->next;
 		}
 
@@ -435,7 +428,7 @@ static PyObject *M_Window_RedrawAll( PyObject * self, PyObject * args )
 /*****************************************************************************/
 static PyObject *M_Window_QRedrawAll( PyObject * self, PyObject * args )
 {
-	allqueue( REDRAWALL, 0 );
+	EXPP_allqueue( REDRAWALL, 0 );
 
 	Py_INCREF( Py_None );
 	return Py_None;
@@ -452,12 +445,41 @@ static PyObject *M_Window_QRedrawAll( PyObject * self, PyObject * args )
 
 static void getSelectedFile( char *name )
 {
-	if( !EXPP_FS_PyCallback )
-		return;
+	PyObject *pycallback;
+	PyObject *result;
+	Script *script;
 
-	PyObject_CallFunction( ( PyObject * ) EXPP_FS_PyCallback, "s", name );
+	/* let's find the script that owns this callback */
+	script = G.main->script.first;
+	while (script) {
+		if (script->flags & SCRIPT_RUNNING) break;
+		script = script->id.next;
+	}
 
-	EXPP_FS_PyCallback = NULL;
+	if (!script) {
+		if (curarea->spacetype == SPACE_SCRIPT) {
+			SpaceScript *sc = curarea->spacedata.first;
+			script = sc->script;
+		}
+	}
+
+	pycallback = script->py_browsercallback;
+
+	if (pycallback) {
+		result = PyObject_CallFunction( pycallback, "s", name );
+
+		if (!result) {
+			if (G.f & G_DEBUG)
+				fprintf(stderr, "BPy error: Callback call failed!\n");
+		}
+		else Py_DECREF(result);
+
+		if (script->py_browsercallback == pycallback)
+			script->py_browsercallback = NULL;
+		/* else another call to selector was made inside pycallback */
+
+		Py_DECREF(pycallback);
+	}
 
 	return;
 }
@@ -467,15 +489,25 @@ static PyObject *M_Window_FileSelector( PyObject * self, PyObject * args )
 	char *title = "SELECT FILE";
 	char *filename = G.sce;
 	SpaceScript *sc;
-	Script *script = G.main->script.last;
+	Script *script = NULL;
+	PyObject *pycallback = NULL;
 	int startspace = 0;
 
-	if( !PyArg_ParseTuple( args, "O!|ss",
-			       &PyFunction_Type, &EXPP_FS_PyCallback, &title,
-			       &filename ) )
+	if (during_scriptlink())
+		return EXPP_ReturnPyObjError(PyExc_RuntimeError,
+			"script links can't call the file selector");
+
+	if (G.background)
+		return EXPP_ReturnPyObjError(PyExc_RuntimeError,
+			"the file selector is not available in background mode");
+
+	if((!PyArg_ParseTuple( args, "O|ss", &pycallback, &title, &filename))
+		|| (!PyCallable_Check(pycallback)))
 		return EXPP_ReturnPyObjError( PyExc_AttributeError,
-					      "\nexpected a callback function (and optionally one or two strings) "
-					      "as argument(s)" );
+			"\nexpected a callback function (and optionally one or two strings) "
+			"as argument(s)" );
+
+	Py_INCREF(pycallback);
 
 /* trick: we move to a spacescript because then the fileselector will properly
  * unset our SCRIPT_FILESEL flag when the user chooses a file or cancels the
@@ -488,8 +520,14 @@ static PyObject *M_Window_FileSelector( PyObject * self, PyObject * args )
 
 	sc = curarea->spacedata.first;
 
-	/* did we get the right script? */
-	if( !( script->flags & SCRIPT_RUNNING ) ) {
+	/* let's find the script that called us */
+	script = G.main->script.first;
+	while (script) {
+		if (script->flags & SCRIPT_RUNNING) break;
+		script = script->id.next;
+	}
+
+	if( !script ) {
 		/* if not running, then we were already on a SpaceScript space, executing
 		 * a registered callback -- aka: this script has a gui */
 		script = sc->script;	/* this is the right script */
@@ -500,6 +538,12 @@ static PyObject *M_Window_FileSelector( PyObject * self, PyObject * args )
 
 	script->flags |= SCRIPT_FILESEL;
 
+	/* clear any previous callback (nested calls to selector) */
+	if (script->py_browsercallback) {
+		Py_DECREF((PyObject *)script->py_browsercallback);
+	}
+	script->py_browsercallback = pycallback;
+
 	activate_fileselect( FILE_BLENDER, title, filename, getSelectedFile );
 
 	Py_INCREF( Py_None );
@@ -511,16 +555,25 @@ static PyObject *M_Window_ImageSelector( PyObject * self, PyObject * args )
 	char *title = "SELECT IMAGE";
 	char *filename = G.sce;
 	SpaceScript *sc;
-	Script *script = G.main->script.last;
+	Script *script = NULL;
+	PyObject *pycallback = NULL;
 	int startspace = 0;
 
-	if( !PyArg_ParseTuple( args, "O!|ss",
-			       &PyFunction_Type, &EXPP_FS_PyCallback, &title,
-			       &filename ) )
-		return ( EXPP_ReturnPyObjError
-			 ( PyExc_AttributeError,
-			   "\nexpected a callback function (and optionally one or two strings) "
-			   "as argument(s)" ) );
+	if (during_scriptlink())
+		return EXPP_ReturnPyObjError(PyExc_RuntimeError,
+			"script links can't call the image selector");
+
+	if (G.background)
+		return EXPP_ReturnPyObjError(PyExc_RuntimeError,
+			"the image selector is not available in background mode");
+
+	if( !PyArg_ParseTuple( args, "O|ss", &pycallback, &title, &filename ) 
+		|| (!PyCallable_Check(pycallback)))
+		return EXPP_ReturnPyObjError ( PyExc_AttributeError,
+			"\nexpected a callback function (and optionally one or two strings) "
+			"as argument(s)" );
+
+	Py_INCREF(pycallback);
 
 /* trick: we move to a spacescript because then the fileselector will properly
  * unset our SCRIPT_FILESEL flag when the user chooses a file or cancels the
@@ -533,11 +586,16 @@ static PyObject *M_Window_ImageSelector( PyObject * self, PyObject * args )
 
 	sc = curarea->spacedata.first;
 
-	/* did we get the right script? */
-	if( !( script->flags & SCRIPT_RUNNING ) ) {
-		/* if not running, then we're on a SpaceScript space, executing a
-		 * registered callback -- aka: this script has a gui */
-		SpaceScript *sc = curarea->spacedata.first;
+	/* let's find the script that called us */
+	script = G.main->script.first;
+	while (script) {
+		if (script->flags & SCRIPT_RUNNING) break;
+		script = script->id.next;
+	}
+
+	if( !script ) {
+		/* if not running, then we were already on a SpaceScript space, executing
+		 * a registered callback -- aka: this script has a gui */
 		script = sc->script;	/* this is the right script */
 	} else {		/* still running, use the trick */
 		script->lastspace = startspace;
@@ -545,6 +603,11 @@ static PyObject *M_Window_ImageSelector( PyObject * self, PyObject * args )
 	}
 
 	script->flags |= SCRIPT_FILESEL;	/* same flag as filesel */
+	/* clear any previous callback (nested calls to selector) */
+	if (script->py_browsercallback) {
+		Py_DECREF((PyObject *)script->py_browsercallback);
+	}
+	script->py_browsercallback = pycallback;
 
 	activate_imageselect( FILE_BLENDER, title, filename, getSelectedFile );
 
@@ -561,13 +624,19 @@ static PyObject *M_Window_DrawProgressBar( PyObject * self, PyObject * args )
 	float done;
 	char *info = NULL;
 	int retval = 0;
+	ScrArea *sa = curarea;
+
+	if (G.background)
+		return EXPP_ReturnPyObjError(PyExc_RuntimeError,
+			"the progress bar is not available in background mode");
 
 	if( !PyArg_ParseTuple( args, "fs", &done, &info ) )
 		return ( EXPP_ReturnPyObjError( PyExc_AttributeError,
-						"expected a float and a string as arguments" ) );
+			"expected a float and a string as arguments" ) );
 
-	if( !G.background )
-		retval = progress_bar( done, info );
+	retval = progress_bar( done, info );
+
+	areawinset(sa->win);
 
 	return Py_BuildValue( "i", retval );
 }
@@ -782,7 +851,7 @@ static PyObject *M_Window_GetViewMatrix( PyObject * self )
 
 	viewmat =
 		( PyObject * ) newMatrixObject( ( float * ) G.vd->viewmat, 4,
-						4 );
+						4, Py_WRAP );
 
 	if( !viewmat )
 		return EXPP_ReturnPyObjError( PyExc_MemoryError,
@@ -806,7 +875,7 @@ static PyObject *M_Window_GetPerspMatrix( PyObject * self )
 
 	perspmat =
 		( PyObject * ) newMatrixObject( ( float * ) G.vd->persmat, 4,
-						4 );
+						4, Py_WRAP );
 
 	if( !perspmat )
 		return EXPP_ReturnPyObjError( PyExc_MemoryError,
@@ -820,20 +889,23 @@ static PyObject *M_Window_EditMode( PyObject * self, PyObject * args )
 	short status = -1;
 	char *undo_str = "From script";
 	int undo_str_len = 11;
+	int do_undo = 1;
 
-	if( !PyArg_ParseTuple
-	    ( args, "|hs#", &status, &undo_str, &undo_str_len ) )
+	if( !PyArg_ParseTuple( args,
+				"|hs#i", &status, &undo_str, &undo_str_len, &do_undo ) )
 		return EXPP_ReturnPyObjError( PyExc_TypeError,
-					      "expected nothing or an int (bool) and a string as arguments" );
+				"expected optional int (bool), string and int (bool) as arguments" );
 
 	if( status >= 0 ) {
 		if( status ) {
 			if( !G.obedit )
 				enter_editmode(  );
 		} else if( G.obedit ) {
-			if( undo_str_len > 63 )
-				undo_str[63] = '\0';	/* 64 is max */
-			undo_push_mesh( undo_str );	/* use better solution after 2.34 */
+			if( do_undo && U.undosteps != 0 ) {
+				if( undo_str_len > 63 )
+					undo_str[63] = '\0';	/* 64 is max */
+				undo_push_mesh( undo_str );	/* use better solution after 2.34 */
+			}
 			exit_editmode( 1 );
 		}
 	}
@@ -841,15 +913,15 @@ static PyObject *M_Window_EditMode( PyObject * self, PyObject * args )
 	return Py_BuildValue( "h", G.obedit ? 1 : 0 );
 }
 
-static PyObject *M_Window_ViewLayer( PyObject * self, PyObject * args )
+static PyObject *M_Window_ViewLayers( PyObject * self, PyObject * args )
 {
 	PyObject *item = NULL;
 	PyObject *list = NULL, *resl = NULL;
 	int val, i, bit = 0, layer = 0;
 
-	if( !G.vd ) {
+	if( !G.scene ) {
 		return EXPP_ReturnPyObjError( PyExc_RuntimeError,
-			"this function can only be used after a 3d View has been initialized" );
+			"can't get pointer to global scene" );
 	}
 
 	if( !PyArg_ParseTuple( args, "|O!", &PyList_Type, &list ) )
@@ -857,7 +929,13 @@ static PyObject *M_Window_ViewLayer( PyObject * self, PyObject * args )
 		  "expected nothing or a list of ints as argument" );
 
 	if( list ) {
-		for( i = 0; i < PyList_Size( list ); i++ ) {
+		int len_list = PyList_Size(list);
+
+		if (len_list == 0)
+			return EXPP_ReturnPyObjError( PyExc_AttributeError,
+		  	"list can't be empty, at list one layer must be set" );
+
+		for( i = 0; i < len_list; i++ ) {
 			item = PyList_GetItem( list, i );
 			if( !PyInt_Check( item ) )
 				return EXPP_ReturnPyObjError
@@ -872,15 +950,18 @@ static PyObject *M_Window_ViewLayer( PyObject * self, PyObject * args )
 
 			layer |= 1 << ( val - 1 );
 		}
-		G.vd->lay = layer;
+		G.scene->lay = layer;
+		if (G.vd) {
+			G.vd->lay = layer;
 
-		while( bit < 20 ) {
-			val = 1 << bit;
-			if( layer & val ) {
-				G.vd->layact = val;
-				break;
+			while( bit < 20 ) {
+				val = 1 << bit;
+				if( layer & val ) {
+					G.vd->layact = val;
+					break;
+				}
+				bit++;
 			}
-			bit++;
 		}
 	}
 
@@ -889,7 +970,7 @@ static PyObject *M_Window_ViewLayer( PyObject * self, PyObject * args )
 		return ( EXPP_ReturnPyObjError( PyExc_MemoryError,
 						"couldn't create pylist!" ) );
 
-	layer = G.vd->lay;
+	layer = G.scene->lay;
 
 	bit = 0;
 	while( bit < 20 ) {
@@ -945,6 +1026,10 @@ static PyObject *M_Window_QRead( PyObject * self )
 	short val = 0;
 	unsigned short event;
 
+	if (G.background)
+		return EXPP_ReturnPyObjError(PyExc_RuntimeError,
+			"QRead is not available in background mode");
+
 	event = extern_qread( &val );
 
 	return Py_BuildValue( "ii", event, val );
@@ -957,6 +1042,10 @@ static PyObject *M_Window_QAdd( PyObject * self, PyObject * args )
 	short val;
 	short after = 0;
 
+	if (G.background)
+		return EXPP_ReturnPyObjError(PyExc_RuntimeError,
+			"QAdd is not available in background mode");
+
 	if( !PyArg_ParseTuple( args, "hhh|h", &win, &evt, &val, &after ) )
 		return EXPP_ReturnPyObjError( PyExc_TypeError,
 					      "expected three or four ints as arguments" );
@@ -979,6 +1068,10 @@ static PyObject *M_Window_QHandle( PyObject * self, PyObject * args )
 	ScrArea *sa = G.curscreen->areabase.first;
 	ScrArea *oldsa = NULL;
 
+	if (G.background)
+		return EXPP_ReturnPyObjError(PyExc_RuntimeError,
+			"QHandle is not available in background mode");
+
 	if( !PyArg_ParseTuple( args, "h", &win ) )
 		return EXPP_ReturnPyObjError( PyExc_TypeError,
 					      "expected an int as argument" );
@@ -1134,7 +1227,7 @@ static PyObject *M_Window_SetScreen( PyObject * self, PyObject * args )
 
 	if( !scr )
 		return EXPP_ReturnPyObjError( PyExc_AttributeError,
-					      "no such screen, check Window.GetScreens() for valid names." );
+					      "no such screen, check Window.GetScreens() for valid names" );
 
 	return EXPP_incr_ret( Py_None );
 }
@@ -1260,27 +1353,28 @@ PyObject *Window_Init( void )
 	if( dict )
 		PyDict_SetItemString( dict, "Theme", Theme_Init(  ) );
 
-	Types = M_constant_New(  );
-	Qual = M_constant_New(  );
-	MButs = M_constant_New(  );
+	Types = PyConstant_New(  );
+	Qual = PyConstant_New(  );
+	MButs = PyConstant_New(  );
 
 	if( Types ) {
 		BPy_constant *d = ( BPy_constant * ) Types;
 
-		constant_insert( d, "VIEW3D", PyInt_FromLong( SPACE_VIEW3D ) );
-		constant_insert( d, "IPO", PyInt_FromLong( SPACE_IPO ) );
-		constant_insert( d, "OOPS", PyInt_FromLong( SPACE_OOPS ) );
-		constant_insert( d, "BUTS", PyInt_FromLong( SPACE_BUTS ) );
-		constant_insert( d, "FILE", PyInt_FromLong( SPACE_FILE ) );
-		constant_insert( d, "IMAGE", PyInt_FromLong( SPACE_IMAGE ) );
-		constant_insert( d, "INFO", PyInt_FromLong( SPACE_INFO ) );
-		constant_insert( d, "SEQ", PyInt_FromLong( SPACE_SEQ ) );
-		constant_insert( d, "IMASEL", PyInt_FromLong( SPACE_IMASEL ) );
-		constant_insert( d, "SOUND", PyInt_FromLong( SPACE_SOUND ) );
-		constant_insert( d, "ACTION", PyInt_FromLong( SPACE_ACTION ) );
-		constant_insert( d, "TEXT", PyInt_FromLong( SPACE_TEXT ) );
-		constant_insert( d, "NLA", PyInt_FromLong( SPACE_NLA ) );
-		constant_insert( d, "SCRIPT", PyInt_FromLong( SPACE_SCRIPT ) );
+		PyConstant_Insert( d, "VIEW3D", PyInt_FromLong( SPACE_VIEW3D ) );
+		PyConstant_Insert( d, "IPO", PyInt_FromLong( SPACE_IPO ) );
+		PyConstant_Insert( d, "OOPS", PyInt_FromLong( SPACE_OOPS ) );
+		PyConstant_Insert( d, "BUTS", PyInt_FromLong( SPACE_BUTS ) );
+		PyConstant_Insert( d, "FILE", PyInt_FromLong( SPACE_FILE ) );
+		PyConstant_Insert( d, "IMAGE", PyInt_FromLong( SPACE_IMAGE ) );
+		PyConstant_Insert( d, "INFO", PyInt_FromLong( SPACE_INFO ) );
+		PyConstant_Insert( d, "SEQ", PyInt_FromLong( SPACE_SEQ ) );
+		PyConstant_Insert( d, "IMASEL", PyInt_FromLong( SPACE_IMASEL ) );
+		PyConstant_Insert( d, "SOUND", PyInt_FromLong( SPACE_SOUND ) );
+		PyConstant_Insert( d, "ACTION", PyInt_FromLong( SPACE_ACTION ) );
+		PyConstant_Insert( d, "TEXT", PyInt_FromLong( SPACE_TEXT ) );
+		PyConstant_Insert( d, "NLA", PyInt_FromLong( SPACE_NLA ) );
+		PyConstant_Insert( d, "SCRIPT", PyInt_FromLong( SPACE_SCRIPT ) );
+		PyConstant_Insert( d, "TIME", PyInt_FromLong( SPACE_TIME ) );
 
 		PyModule_AddObject( submodule, "Types", Types );
 	}
@@ -1288,15 +1382,15 @@ PyObject *Window_Init( void )
 	if( Qual ) {
 		BPy_constant *d = ( BPy_constant * ) Qual;
 
-		constant_insert( d, "LALT", PyInt_FromLong( L_ALTKEY ) );
-		constant_insert( d, "RALT", PyInt_FromLong( R_ALTKEY ) );
-		constant_insert( d, "ALT", PyInt_FromLong( LR_ALTKEY ) );
-		constant_insert( d, "LCTRL", PyInt_FromLong( L_CTRLKEY ) );
-		constant_insert( d, "RCTRL", PyInt_FromLong( R_CTRLKEY ) );
-		constant_insert( d, "CTRL", PyInt_FromLong( LR_CTRLKEY ) );
-		constant_insert( d, "LSHIFT", PyInt_FromLong( L_SHIFTKEY ) );
-		constant_insert( d, "RSHIFT", PyInt_FromLong( R_SHIFTKEY ) );
-		constant_insert( d, "SHIFT", PyInt_FromLong( LR_SHIFTKEY ) );
+		PyConstant_Insert( d, "LALT", PyInt_FromLong( L_ALTKEY ) );
+		PyConstant_Insert( d, "RALT", PyInt_FromLong( R_ALTKEY ) );
+		PyConstant_Insert( d, "ALT", PyInt_FromLong( LR_ALTKEY ) );
+		PyConstant_Insert( d, "LCTRL", PyInt_FromLong( L_CTRLKEY ) );
+		PyConstant_Insert( d, "RCTRL", PyInt_FromLong( R_CTRLKEY ) );
+		PyConstant_Insert( d, "CTRL", PyInt_FromLong( LR_CTRLKEY ) );
+		PyConstant_Insert( d, "LSHIFT", PyInt_FromLong( L_SHIFTKEY ) );
+		PyConstant_Insert( d, "RSHIFT", PyInt_FromLong( R_SHIFTKEY ) );
+		PyConstant_Insert( d, "SHIFT", PyInt_FromLong( LR_SHIFTKEY ) );
 
 		PyModule_AddObject( submodule, "Qual", Qual );
 	}
@@ -1304,9 +1398,9 @@ PyObject *Window_Init( void )
 	if( MButs ) {
 		BPy_constant *d = ( BPy_constant * ) MButs;
 
-		constant_insert( d, "L", PyInt_FromLong( L_MOUSE ) );
-		constant_insert( d, "M", PyInt_FromLong( M_MOUSE ) );
-		constant_insert( d, "R", PyInt_FromLong( R_MOUSE ) );
+		PyConstant_Insert( d, "L", PyInt_FromLong( L_MOUSE ) );
+		PyConstant_Insert( d, "M", PyInt_FromLong( M_MOUSE ) );
+		PyConstant_Insert( d, "R", PyInt_FromLong( R_MOUSE ) );
 
 		PyModule_AddObject( submodule, "MButs", MButs );
 	}
