@@ -1,5 +1,5 @@
 /* 
- * $Id: World.c,v 1.23 2004/12/21 01:34:23 stiv Exp $
+ * $Id: World.c,v 1.32 2005/10/10 18:05:30 ton Exp $
  *
  * ***** BEGIN GPL/BL DUAL LICENSE BLOCK *****
  *
@@ -25,7 +25,7 @@
  *
  * This is a new part of Blender.
  *
- * Contributor(s): Jacques Guignot
+ * Contributor(s): Jacques Guignot, Johnny Matthews
  *
  * ***** END GPL/BL DUAL LICENSE BLOCK *****
 */
@@ -43,21 +43,27 @@
  * (self, name = "MyName")
  */
 
-#include <BKE_main.h>
-#include <BKE_global.h>
-#include <BKE_object.h>
-#include <BKE_library.h>
-#include <BLI_blenlib.h>
+#include "World.h"  /*This must come first*/
 
-#include <DNA_scene_types.h>  /* for G.scene */
-
-#include "World.h"
+#include "DNA_scene_types.h"  /* for G.scene */
+#include "BKE_global.h"
+#include "BKE_world.h"
+#include "BKE_main.h"
+#include "BKE_library.h"
+#include "BLI_blenlib.h"
+#include "BSE_editipo.h"
+#include "BIF_space.h"
+#include "mydevice.h"
 #include "Ipo.h"
-
-#include "constant.h"
 #include "gen_utils.h"
 
 
+#define IPOKEY_ZENITH   0
+#define IPOKEY_HORIZON  1
+#define IPOKEY_MIST     2
+#define IPOKEY_STARS    3
+#define IPOKEY_OFFSET   4
+#define IPOKEY_SIZE     5
 /*****************************************************************************/
 /* Python BPy_World methods declarations:                                   */
 /*****************************************************************************/
@@ -66,8 +72,11 @@ static PyObject *World_setRange( BPy_World * self, PyObject * args );
 static PyObject *World_getIpo( BPy_World * self );
 static PyObject *World_setIpo( BPy_World * self, PyObject * args );
 static PyObject *World_clearIpo( BPy_World * self );
+static PyObject *World_insertIpoKey( BPy_World * self, PyObject * args );
 static PyObject *World_getName( BPy_World * self );
 static PyObject *World_setName( BPy_World * self, PyObject * args );
+static PyObject *World_getMode( BPy_World * self );
+static PyObject *World_setMode( BPy_World * self, PyObject * args );
 static PyObject *World_getSkytype( BPy_World * self );
 static PyObject *World_setSkytype( BPy_World * self, PyObject * args );
 static PyObject *World_getMistype( BPy_World * self );
@@ -84,7 +93,7 @@ static PyObject *World_getMist( BPy_World * self );
 static PyObject *World_setMist( BPy_World * self, PyObject * args );
 static PyObject *World_getScriptLinks( BPy_World * self, PyObject * args );
 static PyObject *World_addScriptLink( BPy_World * self, PyObject * args );
-static PyObject *World_clearScriptLinks( BPy_World * self );
+static PyObject *World_clearScriptLinks( BPy_World * self, PyObject * args );
 static PyObject *World_setCurrent( BPy_World * self );
 
 
@@ -162,7 +171,11 @@ static PyMethodDef BPy_World_methods[] = {
 	{"getName", ( PyCFunction ) World_getName, METH_NOARGS,
 	 "() - Return World Data name"},
 	{"setName", ( PyCFunction ) World_setName, METH_VARARGS,
-	 "() - Return World Data name"},
+	 "() - Set World Data name"},
+	{"getMode", ( PyCFunction ) World_getMode, METH_NOARGS,
+	 "() - Return World Data mode"},
+	{"setMode", ( PyCFunction ) World_setMode, METH_VARARGS,
+	 "(i) - Set World Data mode"},
 	{"getSkytype", ( PyCFunction ) World_getSkytype, METH_NOARGS,
 	 "() - Return World Data skytype"},
 	{"setSkytype", ( PyCFunction ) World_setSkytype, METH_VARARGS,
@@ -194,18 +207,21 @@ static PyMethodDef BPy_World_methods[] = {
 	{"getScriptLinks", ( PyCFunction ) World_getScriptLinks, METH_VARARGS,
 	 "(eventname) - Get a list of this world's scriptlinks (Text names) "
 	 "of the given type\n"
-	 "(eventname) - string: FrameChanged or Redraw."},
+	 "(eventname) - string: FrameChanged, Redraw or Render."},
 	{"addScriptLink", ( PyCFunction ) World_addScriptLink, METH_VARARGS,
 	 "(text, evt) - Add a new world scriptlink.\n"
 	 "(text) - string: an existing Blender Text name;\n"
-	 "(evt) string: FrameChanged or Redraw."},
+	 "(evt) string: FrameChanged, Redraw or Render."},
 	{"clearScriptLinks", ( PyCFunction ) World_clearScriptLinks,
-	 METH_NOARGS,
-	 "() - Delete all scriptlinks from this world :)."},
+	 METH_VARARGS,
+	 "() - Delete all scriptlinks from this world.\n"
+	 "([s1<,s2,s3...>]) - Delete specified scriptlinks from this world."},
 	{"setCurrent", ( PyCFunction ) World_setCurrent, METH_NOARGS,
 	 "() - Makes this world the active world for the current scene."},
 	{"makeActive", ( PyCFunction ) World_setCurrent, METH_NOARGS,
 	 "please use setCurrent instead, this alias will be removed."},
+	{"insertIpoKey", ( PyCFunction ) World_insertIpoKey, METH_VARARGS,
+	 "( World IPO type ) - Inserts a key into the IPO"},
 	{NULL, NULL, 0, NULL}
 };
 
@@ -235,6 +251,7 @@ PyTypeObject World_Type = {
 	0, 0, 0, 0, 0, 0,
 	BPy_World_methods,	/* tp_methods */
 	0,			/* tp_members */
+	0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
 };
 
 /**
@@ -392,6 +409,13 @@ PyObject *World_Init( void )
 	submodule = Py_InitModule3( "Blender.World",
 				    M_World_methods, M_World_doc );
 
+	PyModule_AddIntConstant( submodule, "ZENITH",      IPOKEY_ZENITH );
+	PyModule_AddIntConstant( submodule, "HORIZON",     IPOKEY_HORIZON );
+	PyModule_AddIntConstant( submodule, "MIST",        IPOKEY_MIST );
+	PyModule_AddIntConstant( submodule, "STARS",       IPOKEY_STARS );
+	PyModule_AddIntConstant( submodule, "OFFSET",      IPOKEY_OFFSET );
+	PyModule_AddIntConstant( submodule, "SIZE",        IPOKEY_SIZE );
+
 	return ( submodule );
 }
 
@@ -474,12 +498,10 @@ static PyObject *World_clearIpo( BPy_World * self )
 			id->us--;
 		world->ipo = NULL;
 
-		Py_INCREF( Py_True );
-		return Py_True;
+		return EXPP_incr_ret_True();
 	}
 
-	Py_INCREF( Py_False );	/* no ipo found */
-	return Py_False;
+	return EXPP_incr_ret_False(); /* no ipo found */
 }
 
 /**
@@ -553,7 +575,7 @@ static PyObject *World_setSkytype( BPy_World * self, PyObject * args )
 	if( !PyArg_ParseTuple( args, "i", &skytype ) )
 		return ( EXPP_ReturnPyObjError( PyExc_TypeError,
 						"expected int argument" ) );
-	self->world->skytype = skytype;
+	self->world->skytype = (short)skytype;
 	Py_INCREF( Py_None );
 	return Py_None;
 }
@@ -590,7 +612,7 @@ static PyObject *World_setMode( BPy_World * self, PyObject * args )
 	if( !PyArg_ParseTuple( args, "i", &mode ) )
 		return ( EXPP_ReturnPyObjError( PyExc_TypeError,
 						"expected int argument" ) );
-	self->world->mode = mode;
+	self->world->mode = (short)mode;
 	Py_INCREF( Py_None );
 	return Py_None;
 }
@@ -639,7 +661,7 @@ static PyObject *World_setMistype( BPy_World * self, PyObject * args )
 	if( !PyArg_ParseTuple( args, "i", &mistype ) )
 		return ( EXPP_ReturnPyObjError( PyExc_TypeError,
 						"expected int argument" ) );
-	self->world->mistype = mistype;
+	self->world->mistype = (short)mistype;
 	Py_INCREF( Py_None );
 	return Py_None;
 }
@@ -667,9 +689,9 @@ static PyObject *World_setHor( BPy_World * self, PyObject * args )
 	if( !PyArg_ParseTuple( args, "O", &listargs ) )
 		return ( EXPP_ReturnPyObjError( PyExc_TypeError,
 						"expected list argument" ) );
-	self->world->horr = PyFloat_AsDouble( PyList_GetItem( listargs, 0 ) );
-	self->world->horg = PyFloat_AsDouble( PyList_GetItem( listargs, 1 ) );
-	self->world->horb = PyFloat_AsDouble( PyList_GetItem( listargs, 2 ) );
+	self->world->horr = (float)PyFloat_AsDouble( PyList_GetItem( listargs, 0 ) );
+	self->world->horg = (float)PyFloat_AsDouble( PyList_GetItem( listargs, 1 ) );
+	self->world->horb = (float)PyFloat_AsDouble( PyList_GetItem( listargs, 2 ) );
 	Py_INCREF( Py_None );
 	return Py_None;
 }
@@ -694,9 +716,9 @@ static PyObject *World_setZen( BPy_World * self, PyObject * args )
 	if( !PyArg_ParseTuple( args, "O", &listargs ) )
 		return ( EXPP_ReturnPyObjError( PyExc_TypeError,
 						"expected list argument" ) );
-	self->world->zenr = PyFloat_AsDouble( PyList_GetItem( listargs, 0 ) );
-	self->world->zeng = PyFloat_AsDouble( PyList_GetItem( listargs, 1 ) );
-	self->world->zenb = PyFloat_AsDouble( PyList_GetItem( listargs, 2 ) );
+	self->world->zenr = (float)PyFloat_AsDouble( PyList_GetItem( listargs, 0 ) );
+	self->world->zeng = (float)PyFloat_AsDouble( PyList_GetItem( listargs, 1 ) );
+	self->world->zenb = (float)PyFloat_AsDouble( PyList_GetItem( listargs, 2 ) );
 	Py_INCREF( Py_None );
 	return Py_None;
 }
@@ -729,9 +751,9 @@ static PyObject *World_setAmb( BPy_World * self, PyObject * args )
 	if( PyList_Size( listargs ) != 3 )
 		return ( EXPP_ReturnPyObjError
 			 ( PyExc_TypeError, "wrong list size" ) );
-	self->world->ambr = PyFloat_AsDouble( PyList_GetItem( listargs, 0 ) );
-	self->world->ambg = PyFloat_AsDouble( PyList_GetItem( listargs, 1 ) );
-	self->world->ambb = PyFloat_AsDouble( PyList_GetItem( listargs, 2 ) );
+	self->world->ambr = (float)PyFloat_AsDouble( PyList_GetItem( listargs, 0 ) );
+	self->world->ambg = (float)PyFloat_AsDouble( PyList_GetItem( listargs, 1 ) );
+	self->world->ambb = (float)PyFloat_AsDouble( PyList_GetItem( listargs, 2 ) );
 	Py_INCREF( Py_None );
 	return Py_None;
 }
@@ -766,17 +788,17 @@ static PyObject *World_setStar( BPy_World * self, PyObject * args )
 	if( PyList_Size( listargs ) != 7 )
 		return ( EXPP_ReturnPyObjError
 			 ( PyExc_TypeError, "wrong list size" ) );
-	self->world->starr = PyFloat_AsDouble( PyList_GetItem( listargs, 0 ) );
-	self->world->starg = PyFloat_AsDouble( PyList_GetItem( listargs, 1 ) );
-	self->world->starb = PyFloat_AsDouble( PyList_GetItem( listargs, 2 ) );
+	self->world->starr = (float)PyFloat_AsDouble( PyList_GetItem( listargs, 0 ) );
+	self->world->starg = (float)PyFloat_AsDouble( PyList_GetItem( listargs, 1 ) );
+	self->world->starb = (float)PyFloat_AsDouble( PyList_GetItem( listargs, 2 ) );
 	self->world->starsize =
-		PyFloat_AsDouble( PyList_GetItem( listargs, 3 ) );
+		(float)PyFloat_AsDouble( PyList_GetItem( listargs, 3 ) );
 	self->world->starmindist =
-		PyFloat_AsDouble( PyList_GetItem( listargs, 4 ) );
+		(float)PyFloat_AsDouble( PyList_GetItem( listargs, 4 ) );
 	self->world->stardist =
-		PyFloat_AsDouble( PyList_GetItem( listargs, 5 ) );
+		(float)PyFloat_AsDouble( PyList_GetItem( listargs, 5 ) );
 	self->world->starcolnoise =
-		PyFloat_AsDouble( PyList_GetItem( listargs, 6 ) );
+		(float)PyFloat_AsDouble( PyList_GetItem( listargs, 6 ) );
 	Py_INCREF( Py_None );
 	return Py_None;
 }
@@ -812,13 +834,13 @@ static PyObject *World_setMist( BPy_World * self, PyObject * args )
 	if( PyList_Size( listargs ) != 4 )
 		return ( EXPP_ReturnPyObjError
 			 ( PyExc_TypeError, "wrong list size" ) );
-	self->world->misi = PyFloat_AsDouble( PyList_GetItem( listargs, 0 ) );
+	self->world->misi = (float)PyFloat_AsDouble( PyList_GetItem( listargs, 0 ) );
 	self->world->miststa =
-		PyFloat_AsDouble( PyList_GetItem( listargs, 1 ) );
+		(float)PyFloat_AsDouble( PyList_GetItem( listargs, 1 ) );
 	self->world->mistdist =
-		PyFloat_AsDouble( PyList_GetItem( listargs, 2 ) );
+		(float)PyFloat_AsDouble( PyList_GetItem( listargs, 2 ) );
 	self->world->misthi =
-		PyFloat_AsDouble( PyList_GetItem( listargs, 3 ) );
+		(float)PyFloat_AsDouble( PyList_GetItem( listargs, 3 ) );
 	Py_INCREF( Py_None );
 	return Py_None;
 }
@@ -831,22 +853,18 @@ static PyObject *World_addScriptLink( BPy_World * self, PyObject * args )
 
 	slink = &( world )->scriptlink;
 
-	if( !EXPP_addScriptLink( slink, args, 0 ) )
-		return EXPP_incr_ret( Py_None );
-	else
-		return NULL;
+	return EXPP_addScriptLink( slink, args, 0 );
 }
 
 /* world.clearScriptLinks */
-static PyObject *World_clearScriptLinks( BPy_World * self )
+static PyObject *World_clearScriptLinks( BPy_World * self, PyObject * args )
 {
 	World *world = self->world;
 	ScriptLink *slink = NULL;
 
 	slink = &( world )->scriptlink;
 
-	return EXPP_incr_ret( Py_BuildValue
-			      ( "i", EXPP_clearScriptLinks( slink ) ) );
+	return EXPP_clearScriptLinks( slink, args );
 }
 
 /* world.getScriptLinks */
@@ -921,6 +939,8 @@ static PyObject *World_GetAttr( BPy_World * self, char *name )
 		return World_getStar( self );
 	if( strcmp( name, "mist" ) == 0 )
 		return World_getMist( self );
+	if( strcmp( name, "users" ) == 0 )
+		return PyInt_FromLong( self->world->id.us );
 	return Py_FindMethod( BPy_World_methods, ( PyObject * ) self, name );
 }
 
@@ -933,30 +953,43 @@ static PyObject *World_GetAttr( BPy_World * self, char *name )
 
 static int World_SetAttr( BPy_World * self, char *name, PyObject * value )
 {
+	PyObject *error=NULL;
 	PyObject *valtuple = Py_BuildValue( "(O)", value );
 
 	if( !valtuple )
 		return EXPP_ReturnIntError( PyExc_MemoryError,
 					    "WorldSetAttr: couldn't parse args" );
-	if( strcmp( name, "name" ) == 0 )
-		World_setName( self, valtuple );
-	if( strcmp( name, "skytype" ) == 0 )
-		World_setSkytype( self, valtuple );
-	if( strcmp( name, "mode" ) == 0 )
-		World_setMode( self, valtuple );
-	if( strcmp( name, "mistype" ) == 0 )
-		World_setMistype( self, valtuple );
-	if( strcmp( name, "hor" ) == 0 )
-		World_setHor( self, valtuple );
-	if( strcmp( name, "zen" ) == 0 )
-		World_setZen( self, valtuple );
-	if( strcmp( name, "amb" ) == 0 )
-		World_setAmb( self, valtuple );
-	if( strcmp( name, "star" ) == 0 )
-		World_setStar( self, valtuple );
-	if( strcmp( name, "mist" ) == 0 )
-		World_setMist( self, valtuple );
-	return 0;		/* normal exit */
+	else if( strcmp( name, "name" ) == 0 )
+		error = World_setName( self, valtuple );
+	else if( strcmp( name, "skytype" ) == 0 )
+		error = World_setSkytype( self, valtuple );
+	else if( strcmp( name, "mode" ) == 0 )
+		error = World_setMode( self, valtuple );
+	else if( strcmp( name, "mistype" ) == 0 )
+		error = World_setMistype( self, valtuple );
+	else if( strcmp( name, "hor" ) == 0 )
+		error = World_setHor( self, valtuple );
+	else if( strcmp( name, "zen" ) == 0 )
+		error = World_setZen( self, valtuple );
+	else if( strcmp( name, "amb" ) == 0 )
+		error = World_setAmb( self, valtuple );
+	else if( strcmp( name, "star" ) == 0 )
+		error = World_setStar( self, valtuple );
+	else if( strcmp( name, "mist" ) == 0 )
+		error = World_setMist( self, valtuple );
+	else {
+		Py_DECREF( valtuple );
+		return ( EXPP_ReturnIntError( PyExc_KeyError,
+						  "attribute not found" ) );
+	}
+
+	Py_DECREF(valtuple);
+
+	if( error != Py_None )
+		return -1;
+
+	Py_DECREF( Py_None );
+	return 0;
 }
 
 /**
@@ -1064,3 +1097,60 @@ World *GetWorldByName( char *name )
 	/* There is no object with the given name */
 	return ( NULL );
 }
+/*
+ * World_insertIpoKey()
+ *  inserts World IPO key for ZENITH,HORIZON,MIST,STARS,OFFSET,SIZE
+ */
+
+static PyObject *World_insertIpoKey( BPy_World * self, PyObject * args )
+{
+	int key = 0, map;
+
+	if( !PyArg_ParseTuple( args, "i", &( key ) ) )
+		return ( EXPP_ReturnPyObjError( PyExc_AttributeError,
+										"expected int argument" ) );
+
+	map = texchannel_to_adrcode(self->world->texact);
+
+	if(key == IPOKEY_ZENITH) {
+		insertkey((ID *)self->world, ID_WO, NULL, NULL, WO_ZEN_R);
+		insertkey((ID *)self->world, ID_WO, NULL, NULL, WO_ZEN_G);
+		insertkey((ID *)self->world, ID_WO, NULL, NULL, WO_ZEN_B);
+	}
+	if(key == IPOKEY_HORIZON) {
+		insertkey((ID *)self->world, ID_WO, NULL, NULL, WO_HOR_R);
+		insertkey((ID *)self->world, ID_WO, NULL, NULL, WO_HOR_G);
+		insertkey((ID *)self->world, ID_WO, NULL, NULL, WO_HOR_B);
+	}
+	if(key == IPOKEY_MIST) {
+		insertkey((ID *)self->world, ID_WO, NULL, NULL, WO_MISI);
+		insertkey((ID *)self->world, ID_WO, NULL, NULL, WO_MISTDI);
+		insertkey((ID *)self->world, ID_WO, NULL, NULL, WO_MISTSTA);
+		insertkey((ID *)self->world, ID_WO, NULL, NULL, WO_MISTHI);
+	}
+	if(key == IPOKEY_STARS) {
+		insertkey((ID *)self->world, ID_WO, NULL, NULL, WO_STAR_R);
+		insertkey((ID *)self->world, ID_WO, NULL, NULL, WO_STAR_G);
+		insertkey((ID *)self->world, ID_WO, NULL, NULL, WO_STAR_B);
+		insertkey((ID *)self->world, ID_WO, NULL, NULL, WO_STARDIST);
+		insertkey((ID *)self->world, ID_WO, NULL, NULL, WO_STARSIZE);
+	}
+	if(key == IPOKEY_OFFSET) {
+		insertkey((ID *)self->world, ID_WO, NULL, NULL, map+MAP_OFS_X);
+		insertkey((ID *)self->world, ID_WO, NULL, NULL, map+MAP_OFS_Y);
+		insertkey((ID *)self->world, ID_WO, NULL, NULL, map+MAP_OFS_Z);
+	}
+	if(key == IPOKEY_SIZE) {
+		insertkey((ID *)self->world, ID_WO, NULL, NULL, map+MAP_SIZE_X);
+		insertkey((ID *)self->world, ID_WO, NULL, NULL, map+MAP_SIZE_Y);
+		insertkey((ID *)self->world, ID_WO, NULL, NULL, map+MAP_SIZE_Z);
+	}
+
+	allspace(REMAKEIPO, 0);
+	EXPP_allqueue(REDRAWIPO, 0);
+	EXPP_allqueue(REDRAWVIEW3D, 0);
+	EXPP_allqueue(REDRAWACTION, 0);
+	EXPP_allqueue(REDRAWNLA, 0);
+
+	return EXPP_incr_ret( Py_None );
+}
