@@ -6,7 +6,10 @@ The Blender.Draw submodule.
 Draw
 ====
 
-B{New}: L{PupIntInput}, L{PupFloatInput}, L{PupStrInput}, mouse wheel events.
+B{New}:
+ - access to ASCII values in L{events<Register>} callbacks;
+ - 'large' fonts for L{Text} and L{GetStringWidth}.
+ - Pop-up blocks with L{PupBlock}
 
 This module provides access to a B{windowing interface} in Blender.  Its widgets
 include many kinds of buttons: push, toggle, menu, number, string, slider,
@@ -192,7 +195,23 @@ All available events:
   - ZEROKEY
   - ZKEY
 
-@note: function Button has a new alias: L{PushButton}.
+@note: function Button has an alias: L{PushButton}.
+
+@warn: B{very important}: if using your script causes "Error totblock"
+messages when Blender exits (meaning that memory has been leaked), this may
+have been caused by an ignored return value from one of the button types.  To
+avoid this, assign created buttons return values to B{global} variables,
+instead of ignoring them.  Examples::
+
+  # avoid this, it can cause memory leaks:
+  Draw.Toggle(...)
+  Draw.Number(...)
+  Draw.String(...)
+  # this is correct -- assuming the variables are globals:
+  my_toggle_button = Draw.Toggle(...)
+  my_int_button = Draw.Number(...)
+  my_str_button = Draw.String(...)
+
 
 @warn: Inside the windowing loop (after Draw.Register() has been executed and
 before Draw.Exit() is called), don't use the redraw functions from other
@@ -224,6 +243,8 @@ def Register(draw = None, event = None, button = None):
       is called only once.  It's not necessary to re-register the callbacks,
       they will stay until Draw.Exit is called.  It's enough to redraw the
       screen, when a relevant event is caught.
+  @note: only during the B{event} callback: the L{Blender}.ascii variable holds
+      the ASCII integer value (if it exists and is valid) of the current event.
   """
 
 def Redraw(after = 0):
@@ -386,6 +407,50 @@ def PupStrInput(text, default, max = 20):
   @return: The text entered by the user or None if none was chosen.
   """
 
+def PupBlock(title, sequence):
+	"""
+	Display a pop-up block.
+	
+	Possible formats for the items in the sequence parameter.
+	(Value are objects created with L{Create})
+		- string:	Defines a label
+		- (string, Value, string): Defines a toggle button. The first string is the text on the button, the optional second string is the tooltip.
+		- (string, Value, min, max, string): Defines a numeric or string button, depending on the content of Value.  The first string is the text on the button, the optional second string is the tooltip. I{For string, max is the maximum length of the string and min is unused.}
+		
+	Example::
+		import Blender
+		
+		text = Blender.Draw.Create("short text")
+		f = Blender.Draw.Create(1.0)
+		i = Blender.Draw.Create(2)
+		tog = Blender.Draw.Create(0)
+		
+		block = []
+		
+		block.append(("Name: ", text, 0, 30, "this is some tool tip"))
+		block.append("Some Label")
+		block.append(("Value: ", f, 0.0, 100.0))
+		block.append(("Value: ", i, 0, 100))
+		block.append(("Option", tog, "another tooltip"))
+		
+		retval = Blender.Draw.PupBlock("PupBlock test", block)
+		
+		print "PupBlock returned", retval
+		
+		print "text\\t", text
+		print "float\\t", f
+		print "int\\t", i
+		print "toggle\\t", tog
+
+	@warning: On cancel, the Value objects are brought back to there initial values except for string values which will still contain the modified values.
+	@type title: string
+	@param title: The title of the block.
+	@param sequence: A sequence defining what the block contains.
+		The order of the list is the order of appearance, from top down.
+	@rtype: int
+	@return: 1 if the pop-up is confirmed, 0 otherwise
+	"""
+
 def Menu(name, event, x, y, width, height, default, tooltip = None):
   """
   Create a new Menu Button object.
@@ -484,36 +549,36 @@ def Slider(name, event, x, y, width, height, initial, min, max, realtime = 1,
   @return: The Button created.
   """
 
-def Scrollbar(event, x, y, width, height, initial, min, max, realtime = 1,
-           tooltip = None):
-  """
-  Create a new Scrollbar Button object.
-  @type event: int
-  @param event: The event number to pass to the button event function when
-      activated.
-  @type x: int
-  @type y: int
-  @param x: The lower left x (horizontal) coordinate of the button.
-  @param y: The lower left y (vertical) coordinate of the button.
-  @type width: int
-  @type height: int
-  @param width: The button width.
-  @param height: The button height.
-  @type initial: int or float
-  @type min: int or float
-  @type max: int or float
-  @param initial:  The initial value.
-  @param min:  The minimum value.
-  @param max:  The maximum value.
-  @type realtime: int
-  @param realtime: If non-zero (the default), the slider will emit events as
-      it is edited.
-  @type tooltip: string
-  @param tooltip: The button's tooltip (the string that appears when the mouse
-      is kept over the button).
-  @rtype: Blender Button
-  @return: The Button created.
-  """
+#def Scrollbar(event, x, y, width, height, initial, min, max, realtime = 1,
+#           tooltip = None):
+#  """
+#  Create a new Scrollbar Button object.
+#  @type event: int
+#  @param event: The event number to pass to the button event function when
+#      activated.
+#  @type x: int
+#  @type y: int
+#  @param x: The lower left x (horizontal) coordinate of the button.
+#  @param y: The lower left y (vertical) coordinate of the button.
+#  @type width: int
+#  @type height: int
+#  @param width: The button width.
+#  @param height: The button height.
+#  @type initial: int or float
+#  @type min: int or float
+#  @type max: int or float
+#  @param initial:  The initial value.
+#  @param min:  The minimum value.
+#  @param max:  The maximum value.
+#  @type realtime: int
+#  @param realtime: If non-zero (the default), the slider will emit events as
+#      it is edited.
+#  @type tooltip: string
+#  @param tooltip: The button's tooltip (the string that appears when the mouse
+#      is kept over the button).
+#  @rtype: Blender Button
+#  @return: The Button created.
+#  """
 
 def Number(name, event, x, y, width, height, initial, min, max, tooltip = None):
   """
@@ -578,7 +643,7 @@ def GetStringWidth(string, fontsize = 'normal'):
   @type string: string
   @param string: A string.
   @type fontsize: string
-  @param fontsize: The size of the font: 'normal', 'small' or 'tiny'.
+  @param fontsize: The size of the font: 'large', 'normal', 'small' or 'tiny'.
   @rtype: int
   @return: The width of I{string} with the chosen I{fontsize}.
   """
@@ -589,16 +654,86 @@ def Text(string, fontsize = 'normal'):
   @type string: string
   @param string: The text string to draw.
   @type fontsize: string
-  @param fontsize: The size of the font: 'normal', 'small' or 'tiny'.
+  @param fontsize: The size of the font: 'large', 'normal', 'small' or 'tiny'.
   @rtype: int
   @return: The width of I{string} drawn with the chosen I{fontsize}.
   """
 
+def Image(image, x, y, zoomx=1.0, zoomy=1.0, clipx=0, clipy=0, clipw=-1, cliph=-1):
+  """
+  Draw an image on the screen.
+
+  The image is drawn at the location specified by the coordinates (x,y).  A
+  pair of optional zoom factors (in horizontal and vertical directions) can
+  be applied to the image as it is drawn, and an additional clipping rectangle
+  can be applied to extract a particular sub-region of the image to draw.
+
+  Note that the clipping rectangle is given in image space coordinates.  In
+  image space, the origin is located at the bottom left, with x coordinates 
+  increasing to the right and y coordinates increasing upwards.  No matter 
+  where the clipping rectangle is placed in image space, the lower-left pixel 
+  drawn on the screen is always placed at the coordinates (x,y).  The
+  clipping rectangle is itself clipped to the dimensions of the image.  If
+  either the width or the height of the clipping rectangle are negative then
+  the corresponding dimension (width or height) is set to include as much of 
+  the image as possible.
+  
+  For drawing images with alpha blending with the background you will need to enable blending as shown in the example.
+  
+  
+
+  Example::
+   import Blender
+   from Blender import BGL, Image, Draw
+
+   myimage = Image.Load('myimage.png')
+
+   def gui():
+        BGL.glEnable( BGL.GL_BLEND ) # Only needed for alpha blending images with background.
+        BGL.glBlendFunc(BGL.GL_SRC_ALPHA, BGL.GL_ONE_MINUS_SRC_ALPHA) 
+        
+        Draw.Image(myimage, 50, 50)
+        
+        BGL.glDisable( BGL.GL_BLEND )
+   def event(evt, val):
+        if evt == Draw.ESCKEY:
+              Draw.Exit()
+
+   Draw.Register(gui, event, None)
+
+  @type image: Blender.Image
+  @param image: The image to draw.
+  @type x: int
+  @param x: The lower left x (horizontal) position of the origin of the image.
+  @type y: int
+  @param y: The lower left y (vertical) position of the origin of the image.
+  @type zoomx: float
+  @param zoomx: The x (horizontal) zoom factor to use when drawing the image.
+  @type zoomy: float
+  @param zoomy: The y (vertical) zoom factor to use when drawing the image.
+  @type clipx: int
+  @param clipx: The lower left x (horizontal) origin of the clipping rectangle
+                within the image.  A value of 0 indicates the left of the
+                image.
+  @type clipy: int
+  @param clipy: The lower left y (vertical) origin of the clipping rectangle
+                within the image.  A value of 0 indicates the bottom of the
+                image.
+  @type clipw: int
+  @param clipw: The width of the clipping rectangle within the image. If this
+                value is negative then the clipping rectangle includes as much
+                of the image as possible in the x (horizontal) direction.
+  @type cliph: int
+  @param cliph: The height of the clipping rectangle within the image. If this
+                value is negative then the clipping rectangle includes as much
+                of the image as possible in the y (vertical) direction.
+  """
+
 class Button:
   """
   The Button object
   =================
     This object represents a button in Blender's GUI.
   @type val: int or float or string (depends on button type).
-  @cvar val: The button's value.
+  @ivar val: The button's value.
   """
