@@ -3,7 +3,9 @@
 """
 The Blender.Ipo submodule
 
-B{New}: Ipo updates to both the program and bpython access.
+B{New}: 
+  -  Ipo updates to both the program and Bpython access.
+  -  access to Blender's new Ipo driver capabilities .
 
 This module provides access to the Ipo Data in Blender. An Ipo is composed of
 several IpoCurves.
@@ -14,7 +16,7 @@ are given below.
 
 Example::
   import Blender
-  ob = Blender.Ipo.Get('ipo')    # retrieves an ipo object
+  ob = Blender.Ipo.Get('ipo')    # retrieves an Ipo object
   ob.setName('ipo1')
   print ob.name
   print ipo.getRctf()
@@ -27,7 +29,7 @@ def New (type, name):
   Creates a new Ipo.
   @type type: string
   @type name: string
-  @param type: The Ipo's blocktype. Depends on the object the ipo will be
+  @param type: The Ipo's blocktype. Depends on the object the Ipo will be
       linked to. Currently supported types are Object, Camera, World,
       Material, Texture, Lamp, Action, Constraint, Sequence, Curve, Key.
   @param name: The name for this Ipo.
@@ -68,10 +70,12 @@ class Ipo:
     @return: A list (possibly void) containing all the IpoCurves associated to the Ipo object.
     """
 
-  def getCurve(curvename):
+  def getCurve(curve):
     """
-    Return the IpoCurve with the given name. The possible values for
-    'curvename' are:
+    Return the specified IpoCurve.  If the curve does not exist in the Ipo,
+    None is returned.  I{curve} can be either a string or an integer,
+    denoting either the name of the Ipo curve or its internal adrcode.
+    The possible Ipo curve names are:
     
       1. Camera Ipo:  Lens, ClSta, ClEnd, Apert, FDist.
       2. Material Ipo: R, G, B, SpecR, SpecG, SpecB, MirR, MirG, MirB, Ref,
@@ -94,20 +98,24 @@ class Ipo:
       MinkMExp, DistM, ColT, iScale, DistA, MgType, MgH, Lacu, Oct,
       MgOff, MgGain, NBase1, NBase2.
       7. Curve Ipo: Speed.
-      8. Key Ipo: Speed, 'Key 1' - 'Key 63'.
-      9. Action Ipo: LocX, LocY, LocZ, SizeX, SizeY, SizeZ, QuatX, QuatY,
+      8. Action Ipo: LocX, LocY, LocZ, SizeX, SizeY, SizeZ, QuatX, QuatY,
       QuatZ, QuatW.
-      10. Sequence Ipo: Fac.
-      11. Constraint Ipo: Inf.
+      9. Sequence Ipo: Fac.
+      10. Constraint Ipo: Inf.
 
-    @type curvename : string
+    The adrcode for the Ipo curve can also be given; currently this is the
+    only way to access curves for Key Ipos.  The adrcodes for Key Ipo are
+    numbered consecutively starting at 0.
+    @type curve : string or int
     @rtype: IpoCurve object
     @return: the corresponding IpoCurve, or None.
+    @raise ValueError: I{curve} is not a valid name or adrcode for this Ipo
+    type.
     """
 
   def addCurve(curvename):
     """
-    Add a new curve to the IPO object. The possible values for 'curvename' are:
+    Add a new curve to the Ipo object. The possible values for I{curvename} are:
       1. Camera Ipo:  Lens, ClSta, ClEnd, Apert, FDist.
       2. Material Ipo: R, G, B, SpecR, SpecG, SpecB, MirR, MirG, MirB, Ref,
       Alpha, Emit, Amb, Spec, Hard, SpTra, Ior, Mode, HaSize, Translu,
@@ -129,20 +137,20 @@ class Ipo:
       MinkMExp, DistM, ColT, iScale, DistA, MgType, MgH, Lacu, Oct,
       MgOff, MgGain, NBase1, NBase2.
       7. Curve Ipo: Speed.
-      8. Key Ipo: Speed, 'Key 1' - 'Key 63'.
-      9. Action Ipo: LocX, LocY, LocZ, SizeX, SizeY, SizeZ, QuatX, QuatY,
+      8. Action Ipo: LocX, LocY, LocZ, SizeX, SizeY, SizeZ, QuatX, QuatY,
       QuatZ, QuatW.
-      10. Sequence Ipo: Fac.
-      11. Constraint Ipo: Inf.
+      9. Sequence Ipo: Fac.
+      10. Constraint Ipo: Inf.
 
     @type curvename : string
     @rtype: IpoCurve object
     @return: the corresponding IpoCurve, or None.
+    @raise ValueError: I{curvename} is not valid or already exists
     """
 
-  def removeCurve(curvename):
+  def delCurve(curvename):
     """
-    Remove a existing curve from the IPO object. See addCurve() for possible values for curvename.
+    Delete an existing curve from the Ipo object. See addCurve() for possible values for curvename.
     @type curvename : string
     @rtype: None
     @return: None.
@@ -209,7 +217,7 @@ class Ipo:
     """
     Gets a beztriple of the Ipo.
     @type curvepos: int
-    @param curvepos: the position of the curve in the ipo
+    @param curvepos: the position of the curve in the Ipo.
     @type pointpos: int
     @param pointpos: the position of the point in the curve.
     @rtype: list of 9 floats
@@ -220,7 +228,7 @@ class Ipo:
     """
     Sets the beztriple of the Ipo.
     @type curvepos: int
-    @param curvepos: the position of the curve in the ipo
+    @param curvepos: the position of the curve in the Ipo.
     @type pointpos: int
     @param pointpos: the position of the point in the curve.
     @type newbeztriple: list of 9 floats
@@ -231,9 +239,10 @@ class Ipo:
     
   def getCurveCurval(curvepos):
     """
-    Gets the current value of a curve of the Ipo.
+    Gets the current value of a curve of the Ipo (B{deprecated}). B{Note}:
+    new scripts should use L{IpoCurve.evaluate()}.
     @type curvepos: int or string
-    @param curvepos: the position of the curve in the ipo or the name of the
+    @param curvepos: the position of the curve in the Ipo or the name of the
         curve
     @rtype: float
     @return: the current value of the selected curve of the Ipo.
@@ -241,11 +250,12 @@ class Ipo:
 
   def EvaluateCurveOn(curvepos,time):
     """
-    Gets the current value of a curve of the Ipo.
+    Gets the value at a specific time of a curve of the Ipo (B{deprecated}).
+    B{Note}: new scripts should use L{IpoCurve.evaluate()}.
     @type curvepos: int
-    @param curvepos: the position of the curve in the ipo
+    @param curvepos: the position of the curve in the Ipo.
     @type time: float
-    @param time: the position of the curve in the ipo
+    @param time: the desired time.
     @rtype: float
     @return: the current value of the selected curve of the Ipo at the given
         time.
@@ -255,13 +265,28 @@ class IpoCurve:
   """
   The IpoCurve object
   ===================
-  This object gives access to generic data from all ipocurves objects in Blender.
+  This object gives access to generic data from all Ipo curves objects in Blender.
 
-  Important Notes for Rotation Curves:\n
-  For the rotation IpoCurves, the y values for points are in units of 10 degrees.  For example, 45.0 degrees is stored as 4.50 degrees.  These are the same numbers you see in the Transform Properties pupmenu ( NKey ) in the IPO Curve Editor window.  Positive rotations are in a counter-clockwise direction, just like in math class.
+  Important Notes for Rotation Ipo Curves:\n
+  For the rotation Ipo curves, the y values for points are in units of 10
+  degrees.  For example, 45.0 degrees is stored as 4.50 degrees.  These are the
+  same numbers you see in the Transform Properties pop-up menu ( NKey ) in
+  the IPO Curve Editor window.  Positive rotations are in a counter-clockwise
+  direction, following the standard convention.
   
-  @ivar name: The Curve Data name.
-  @ivar bezierPoints : The list of the Bezier points.
+  @ivar driver:  Status of the driver.  1= on, 0= off.
+  @type driver:  int
+  @ivar driverObject:  Object used to drive the Ipo curve.
+  @type driverObject:  Blender Object or None
+  @ivar driverChannel:  Object channel used to drive the Ipo curve.
+  Use module constants: IpoCurve.LOC_X, IpoCurve.LOC_Y, IpoCurve.LOC_Z,
+  IpoCurve.ROT_X, IpoCurve.ROT_Y, IpoCurve.ROT_Z, IpoCurve.SIZE_X,
+  IpoCurve.SIZE_Y, IpoCurve.SIZE_Z
+  @type driverChannel:  int 
+  @ivar name: The IpoCurve data name.
+  @type name: string
+  @ivar bezierPoints : The list of the curve's bezier points.
+  @type bezierPoints : list
   """
 
   def setExtrapolation(extendmode):
@@ -272,6 +297,16 @@ class IpoCurve:
         Can be Constant, Extrapolation, Cyclic or Cyclic_extrapolation.
     @rtype: None
     @return: None
+    @note: Cyclic Ipo curves never reach the end value.  If the first and
+    last bezier points do not have the same y coordinate, the value of the
+    curve when it "cycles" is that of the first point.  If a user wants to
+    get the value of the final curve point, read the final point from the
+    curve's L{bezierPoints} attribute::
+
+		ipo = Blender.Object.Get('Cube').ipo
+		icu = ipo.getCurves('LocX')
+		endtime,endvalue = icu.bezierPoints[-1].pt
+
     """
 
   def getExtrapolation():
@@ -324,7 +359,7 @@ class IpoCurve:
 
   def getName():
     """
-    Returns the name of the ipo curve. This name can be:
+    Returns the name of the Ipo curve. This name can be:
       1. Camera Ipo:  Lens, ClSta, ClEnd, Apert, FDist.
       2. Material Ipo: R, G, B, SpecR, SpecG, SpecB, MirR, MirG, MirB, Ref,
       Alpha, Emit, Amb, Spec, Hard, SpTra, Ior, Mode, HaSize, Translu,
@@ -346,26 +381,25 @@ class IpoCurve:
       MinkMExp, DistM, ColT, iScale, DistA, MgType, MgH, Lacu, Oct,
       MgOff, MgGain, NBase1, NBase2.
       7. Curve Ipo: Speed.
-      8. Key Ipo: Speed, 'Key 1' - 'Key 63'.
-      9. Action Ipo: LocX, LocY, LocZ, SizeX, SizeY, SizeZ, QuatX, QuatY,
+      8. Action Ipo: LocX, LocY, LocZ, SizeX, SizeY, SizeZ, QuatX, QuatY,
       QuatZ, QuatW.
-      10. Sequence Ipo: Fac.
-      11. Constraint Ipo: Inf.
+      9. Sequence Ipo: Fac.
+      10. Constraint Ipo: Inf.
 
     @rtype: string
-    @return: the name of the ipo curve.
+    @return: the name of the Ipo curve.
     """
 
   def getPoints():
     """
-    Returns all the points of the ipo curve.
+    Returns all the points of the Ipo curve.
     @rtype: list of BezTriples
-    @return: the points of the ipo curve.
+    @return: the points of the Ipo curve.
     """
 
   def evaluate( time ):
     """
-    Compute the value of the IpoCurve at a particular time.
+    Compute the value of the Ipo curve at a particular time.
     @type time: float
     @param time: value along the X axis
     @rtype: float
@@ -378,10 +412,25 @@ class BezTriple:
   The BezTriple object
   ====================
   This object gives access to generic data from all beztriple objects in Blender.  If an attribute is listed as being 'read-only' that means you cannot write to it.  Use the set*() methods instead.
-  @ivar pt : a list of the [x,y] coordinates for knot point of this BezTriple.  read-only.
-  @ivar vec : a list of the 3 points [ handle, knot, handle ] that comprise a BezTriple.  See the getTriple() method for an example of the format.  read-only.
+  @ivar pt : the [x,y] coordinates for knot point of this BezTriple.  Read-only.
+  @type pt: list of floats
+  @ivar vec : a list of the 3 points [ handle, knot, handle ] that comprise a BezTriple.  See the getTriple() method for an example of the format.  Read-only.
+  @type vec: list of points
+  @ivar hide: the visibility status of the control point.
+  @type hide: int
   """
 
+  def __init__(coords):
+    """
+    Create a new BezTriple object.  
+
+    @type coords: sequence of three or nine floats
+    @param coords: the coordinate values for the new control point.  If three
+    floats are given, then the handle values are automatically generated.
+    @rtype: BezTriple
+    @return: a new BezTriple object
+    """
+
   def getPoints():
     """
     Returns the xy coordinates of the Bezier knot point.
@@ -391,7 +440,9 @@ class BezTriple:
 
   def setPoints(newval):
     """
-    Sets the point xy coordinates of the Bezier knot point.
+    Sets the point xy coordinates of the Bezier knot point.  After changing
+    coordinates, it is advisable to call L{IpoCurve.recalc()} to update the
+    Ipo curves.
     @type newval: tuple of 2 floats
     @param newval: the x and y coordinates of the new Bezier point.
     @rtype: None
