@@ -4,11 +4,12 @@
 The Blender.NMesh submodule.
 
 B{New}:
- - edges class (L{NMEdge}) and nmesh methods (L{NMesh.addEdge},
-L{NMesh.addEdgesData}, etc.);
- - new optional arguments to L{NMesh.update};
- - L{NMesh.transform};
- - L{GetNames}.
+ - edges class (L{NMEdge}) and nmesh methods (L{NMesh.NMesh.addEdge},
+L{NMesh.NMesh.addEdgesData}, etc.);
+ - new optional arguments to L{NMesh.NMesh.update};
+ - L{NMesh.NMesh.transform};
+ - L{GetNames};
+ - L{GetRawFromObject} supports Surface/Curve/Text/Meta objects
 
 Mesh Data
 =========
@@ -73,7 +74,7 @@ Example::
     - TILES - uses tiled image.
     - TWOSIDE - two-sided face.
 @var FaceTranspModes: The available face transparency modes. Note: these are
-  ENUMS, they can't be combined (and'ed, or'ed, etc) like a bit vector.
+  enumerated values (enums), they can't be combined (and'ed, or'ed, etc) like a bit vector.
     - SOLID - draw solid.
     - ADD - add to background (halo).
     - ALPHA - draw with transparency.
@@ -151,19 +152,24 @@ def GetRawFromObject(name):
   Get the raw mesh data object from the Object in Blender called I{name}.\n
   Note: The mesh coordinates are in local space, not the world space of its Object.\n
   For world space vertex coordinates, each vertex location must be multiplied by the object's 4x4 matrix.
+  This function support all the geometry based objects: Mesh, Text, Surface, Curve, Meta.
   @type name: string
-  @param name: The name of an Object of type "Mesh".
+  @param name: The name of an Object.
   @rtype: NMesh
   @return: The NMesh wrapper of the mesh data from the Object called I{name}.
   @note: For "subsurfed" meshes, it's the B{display} level of subdivision that
       matters, the rendering one is only processed at the rendering pre-stage
       and is not available for scripts.  This is not a problem at all, since
       you can get and set the subdivision levels via scripting, too (see
-      L{NMesh.getSubDivLevels}, L{NMesh.setSubDivLevels}).
+      L{NMesh.NMesh.getSubDivLevels}, L{NMesh.NMesh.setSubDivLevels}).
+  @note: Meshes extracted from curve based objects (Font/2D filled curves)
+      contain both the filled surfaces and the outlines of the shapes.
   @warn: This function gets I{deformed} mesh data, already modified for
       displaying (think "display list").  It also doesn't let you overwrite the
       original mesh in Blender, so if you try to update it, a new mesh will
       be created.
+  @warn: For Meta Object's, this function will only return a NMesh with some geometry
+      when called on the base element (the one with the shortest name).
   """
 
 def PutRaw(nmesh, name = None, recalc_normals = 1, store_edges = 0):
@@ -182,7 +188,7 @@ def PutRaw(nmesh, name = None, recalc_normals = 1, store_edges = 0):
      this nmesh data.  It can be an existing mesh data object or a new one.
   @param recalc_normals: If non-zero, the vertex normals for the mesh will
      be recalculated.
-  @param store_edges: if non-zero, the edges data are stored
+  @param store_edges: deprecated, edges are always stored now.
   @rtype: None or Object
   @return: It depends on the 'name' parameter:
       - I{name} refers to an existing mesh data obj already linked to an
@@ -208,11 +214,11 @@ class NMVert:
   The NMVert object
   =================
     This object holds mesh vertex data.
-  @type co: 3D Vector object.
+  @type co: 3D Vector object. (WRAPPED DATA)
   @ivar co: The vertex coordinates (x, y, z).
-  @type no: 3D Vector object. (unit length)
+  @type no: 3D Vector object. (unit length) (WRAPPED DATA)
   @ivar no: The vertex normal vector (x, y, z).
-  @type uvco: 3D Vector object.
+  @type uvco: 3D Vector object. (WRAPPED DATA)
   @ivar uvco: The vertex texture "sticky" coordinates. The Z value of the Vector is ignored.
   @type index: int
   @ivar index: The vertex index, if owned by a mesh.
@@ -290,12 +296,12 @@ class NMFace:
   @type sel: bool
   @ivar sel: The selection state (1: selected, 0: unselected) of this NMesh's
       faces *in edit mode*.  This is not the same as the selection state of
-      the textured faces (see L{NMesh.NMFace.flag}).
+      the textured faces (see L{flag}).
   @type hide: bool
   @ivar hide: The visibility state (1: hidden, 0: visible) of this NMesh's
       faces *in edit mode*.  This is not the same as the visibility state of
-      the textured faces (see L{NMesh.NMFace.flag}).
-  @ivar col: The list of vertex colours.
+      the textured faces (see L{flag}).
+  @ivar col: The list of vertex colors.
   @ivar mat: Same as I{materialIndex} below.
   @ivar materialIndex: The index of this face's material in its NMesh materials
       list.
@@ -313,8 +319,8 @@ class NMFace:
   @note: there are normal faces and textured faces in Blender, both currently
     with their own selection and visibility states, due to a mix of old and new
     code.  To (un)select or (un)hide normal faces (visible in editmode), use
-    L{NMFace.sel} and L{NMFace.hide} vars.  For textured faces (Face Select
-    mode in Blender) use the old L{NMFace.flag} bitflag.  Also check the
+    L{sel} and L{hide} variables.  For textured faces (Face Select
+    mode in Blender) use the old L{flag} bitflag.  Also check the
     example above and note L{Window.EditMode}.
   @note: Assigning uv textures to mesh faces in Blender works like this:
     1. Select your mesh.
@@ -323,7 +329,7 @@ class NMFace:
     4. Play in both windows (better split the screen to see both at the same
        time) until the uv coordinates are where you want them.  Hint: in the
        3d window, the 'u' key opens a menu of default uv choices and the 'r'
-       key lets you rotate the uv coords.
+       key lets you rotate the uv coordinates.
     5. Leave face select mode (press f).
   """
 
@@ -343,7 +349,7 @@ class NMesh:
   @ivar name: The NMesh name.  It's common to use this field to store extra
      data about the mesh (to be exported to another program, for example).
   @ivar materials: The list of materials used by this NMesh.  See
-     L{NMesh.getMaterials} for important details.
+     L{getMaterials} for important details.
   @ivar verts: The list of NMesh vertices (NMVerts).
   @ivar users: The number of Objects using (linked to) this mesh.
   @ivar faces: The list of NMesh faces (NMFaces).
@@ -351,6 +357,7 @@ class NMesh:
   @ivar mode:  The mode flags for this mesh.  See L{setMode}.
   @ivar subDivLevels: The [display, rendering] subdivision levels in [1, 6].
   @ivar maxSmoothAngle: The max angle for auto smoothing.  See L{setMode}.
+  @cvar key: The L{Key.Key} object attached to this mesh, if any.
   """
 
   def addEdge(v1, v2):
@@ -431,7 +438,7 @@ class NMesh:
         - -1: return the current nmesh's list;
         -  0: retrieve a fresh list from the Blender mesh -- eventual
               modifications made by the script not included, unless
-              L{NMesh.update}d is called before this method;
+              L{update} is called before this method;
         -  1: like 0, but empty slots are not ignored, they are returned as
               None's.
     @note: what >= 0 also updates nmesh.materials attribute.
@@ -440,6 +447,9 @@ class NMesh:
     @note: if a user goes to the material buttons window and removes some
         mesh's link to a material, that material slot becomes empty.
         Previously such materials were ignored.
+    @note: L{Object.colbits<Object.Object.colbits>} needs to be set correctly
+        for each object in order for these materials to be used instead of
+        the object's materials.
     """
 
   def setMaterials(matlist):
@@ -450,26 +460,29 @@ class NMesh:
     @type matlist: list of materials
     @param matlist: a list with materials, None's also accepted (they become
         empty material slots in Blender.
+    @note: L{Object.colbits<Object.Object.colbits>} needs to be set correctly
+        for each object in order for these materials to be used instead of
+        the object's materials.
     """
 
   def hasVertexColours(flag = None):
     """
-    Get (and optionally set) if this NMesh has vertex colours.
+    Get (and optionally set) if this NMesh has vertex colors.
     @type flag: int
-    @param flag: If given and non-zero, the "vertex colour" flag for this NMesh
+    @param flag: If given and non-zero, the "vertex color" flag for this NMesh
         is turned I{on}.
     @rtype: bool
-    @return: The current value of the "vertex colour" flag.
-    @warn: If a mesh has both vertex colours and textured faces, this function
+    @return: The current value of the "vertex color" flag.
+    @warn: If a mesh has both vertex colors and textured faces, this function
        will return False.  This is due to the way Blender deals internally with
-       the vertex colours array (if there are textured faces, it is copied to
+       the vertex colors array (if there are textured faces, it is copied to
        the textured face structure and the original array is freed/deleted).
        If you want to know if a mesh has both textured faces and vertex
-       colours, set *in Blender* the "VCol Paint" flag for each material that
+       colors, set *in Blender* the "VCol Paint" flag for each material that
        covers an area that was also vertex painted and then check in your
        Python script if that material flag is set.  Of course also tell others
        who use your script to do the same.  The "VCol Paint" material mode flag
-       is the way to tell Blender itself to render with vertex colours, too, so
+       is the way to tell Blender itself to render with vertex colors, too, so
        it's a natural solution.
     """
 
@@ -526,6 +539,13 @@ class NMesh:
         and its weight is a float value.
     """
 
+  def getKey():
+    """
+    Get the Key object representing the Vertex Keys (absolute or
+    relative) assigned to this mesh.
+    @rtype: L{Key.Key} object or None
+    """
+
   def insertKey(frame = None, type = 'relative'):
     """
     Insert a mesh key at the given frame.  Remember to L{update} the nmesh
@@ -567,7 +587,7 @@ class NMesh:
     @type recalc_normals: int (bool)
     @param recalc_normals: if nonzero the vertex normals are recalculated.
     @type store_edges: int (bool)
-    @param store_edges: if nonzero, then edge data is stored.
+    @param store_edges: deprecated, edges are always stored now.
     @type vertex_shade: int (bool)
     @param vertex_shade: if nonzero vertices are colored based on the
         current lighting setup, like when there are no vertex colors and no
