@@ -3,8 +3,8 @@
 """
 The Blender.Scene.Render submodule.
 
-Scene
-=====
+Scene.Render
+============
 
 This module provides access to B{Scene Rendering Contexts} in Blender.
 
@@ -98,11 +98,28 @@ class RenderData:
     Render a series of frames to an output directory.
     """
 
+  def saveRenderedImage(filename):
+    """
+    Saves the image rendered using RenderData.render() to the filename and path
+    given in the variable 'filename'
+    @param filename: The path+filename for the rendered image.
+    @type filename: string 
+    @since: 2.40
+    @requires: You must have an image currently rendered before calling this method
+    """
+
   def play():
     """
     play animation of rendered images/avi (searches Pics: field).
     """
 
+  def getTimeCode():
+    """
+    Get the current frame as a string in HH:MM:SS:FF format
+    @rtype: string 
+    @return: current frame as a string in HH:MM:SS:FF format
+    """
+
   def setRenderPath(path):
     """
     Set the path to where the renderer will write to.
