@@ -221,18 +221,18 @@ def GetViewVector ():
 def GetViewMatrix ():
   """
   Get the current 3d view matrix.
-  @rtype: 4x4 float matrix
+  @rtype: 4x4 float matrix (WRAPPED DATA)
   @return: the current matrix.
   """
 
 def GetPerspMatrix ():
   """
   Get the current 3d perspective matrix.
-  @rtype: 4x4 float matrix
+  @rtype: 4x4 float matrix (WRAPPED DATA)
   @return: the current matrix.
   """
 
-def EditMode(enable = -1, undo_msg = 'From script'):
+def EditMode(enable = -1, undo_msg = 'From script', undo = 1):
   """
   Get and optionally set the current edit mode status: in or out.
 
@@ -258,6 +258,9 @@ def EditMode(enable = -1, undo_msg = 'From script'):
       string is used as the undo message in the Mesh->Undo History submenu in
       the 3d view header.  Max length is 63, strings longer than that get
       clamped.
+  @param undo: don't save Undo information (only needed when exiting edit
+  mode).
+  @type undo: int
   @rtype: int (bool)
   @return: 0 if Blender is not in edit mode right now, 1 otherwise. 
   @warn: this is an important function. NMesh operates on normal Blender
@@ -318,8 +321,8 @@ def CameraView (camtov3d = 0):
 def QTest ():
   """
   Check if there are pending events in the event queue.
-  @rtype: bool
-  @return: 1 if there are pending events, 0 otherwise.
+  @rtype: int
+  @return: 0 if there are no pending events, non-zero otherwise.
   """
 
 def QRead ():
@@ -403,14 +406,14 @@ def GetMouseButtons ():
   """
   Get the current mouse button state (see / compare against L{MButs}).
   @rtype: int
-  @return: an or'ed flag with the currently pressed buttons.
+  @return: an OR'ed flag with the currently pressed buttons.
   """
 
 def GetKeyQualifiers ():
   """
   Get the current qualifier keys state (see / compare against L{Qual}).
   @rtype: int
-  @return: an or'ed combination of values in L{Window.Qual}.
+  @return: an OR'ed combination of values in L{Qual}.
   """
 
 def SetKeyQualifiers (qual):
@@ -418,7 +421,7 @@ def SetKeyQualifiers (qual):
   Fake qualifier keys state.  This is useful because some key events require
   one or more qualifiers to be active (see L{QAdd}).
   @type qual: int
-  @param qual: an or'ed combination of values in L{Window.Qual}.
+  @param qual: an OR'ed combination of values in L{Qual}.
   @rtype: int
   @return: the current state, that should be equal to 'qual'.
   @warn: remember to reset the qual keys to 0 once they are not necessary
@@ -465,7 +468,7 @@ def GetScreenInfo (type = -1, rect = 'win', screen = ''):
   """
   Get info about the current screen setup.
   @type type: int
-  @param type: the space type (see L{Window.Types}) to restrict the
+  @param type: the space type (see L{Types}) to restrict the
      results to.  If -1 (the default), info is reported about all available
      areas.
   @type rect: string
