@@ -1,5 +1,5 @@
 /* 
- * $Id: sceneRender.c,v 1.8 2004/11/14 01:03:01 hos Exp $
+ * $Id: sceneRender.c,v 1.24 2005/11/23 15:20:44 ton Exp $
  *
  * ***** BEGIN GPL/BL DUAL LICENSE BLOCK *****
  *
@@ -29,31 +29,22 @@
  *
  * ***** END GPL/BL DUAL LICENSE BLOCK *****
 */
+struct View3D; /* keep me up here */
 
-#include <BIF_renderwin.h>
-#include <BKE_utildefines.h>
-#include <BKE_global.h>
-#include <DNA_image_types.h>
-#include <BIF_drawscene.h>
-#include <BLI_blenlib.h>
-#include <BKE_image.h>
-#include <BIF_space.h>
-#include <DNA_scene_types.h>
-#include <DNA_space_types.h>
-#include <mydevice.h>
-#include <butspace.h>
-#include <BKE_bad_level_calls.h>
-#include "sceneRender.h"
-#include "render_types.h"
+#include "sceneRender.h" /*This must come first*/
+
+#include "BIF_renderwin.h"
+#include "DNA_image_types.h"
+#include "BIF_drawscene.h"
+#include "BLI_blenlib.h"
+#include "BKE_image.h"
+#include "BKE_global.h"
+#include "mydevice.h"
+#include "butspace.h"
+#include "render.h" /* RE_animrender() */
 #include "blendef.h"
-#include "Scene.h"
 #include "gen_utils.h"
 
-
-#ifdef HAVE_CONFIG_H
-#include <config.h>
-#endif
-
 //local defines
 #define PY_NONE		0
 #define PY_LOW		1
@@ -65,6 +56,8 @@
 #define PY_FULL	 2
 
 extern RE_Render R;
+extern void schrijfplaatje(char *name);
+extern void waitcursor(int);
 
 //---------------------------------------Render prototypes-------------
 static PyObject *M_Render_CloseRenderWindow( PyObject * self );
@@ -97,6 +90,7 @@ struct PyMethodDef M_Render_methods[] = {
 };
 //------------------------------------BPy_RenderData methods/callbacks--
 static PyObject *RenderData_Render( BPy_RenderData * self );
+static PyObject *RenderData_SaveRenderedImage( BPy_RenderData *self, PyObject *args);
 static PyObject *RenderData_RenderAnim( BPy_RenderData * self );
 static PyObject *RenderData_Play( BPy_RenderData * self );
 static PyObject *RenderData_SetRenderPath( BPy_RenderData * self,
@@ -256,6 +250,8 @@ static PyObject *RenderData_OldMapValue( BPy_RenderData * self,
 static PyObject *RenderData_NewMapValue( BPy_RenderData * self,
 					 PyObject * args );
 
+static PyObject *RenderData_getTimeCode( BPy_RenderData * self);
+
 static void RenderData_dealloc( BPy_RenderData * self );
 static PyObject *RenderData_getAttr( BPy_RenderData * self, char *name );
 static PyObject *RenderData_repr( BPy_RenderData * self );
@@ -263,6 +259,8 @@ static PyObject *RenderData_repr( BPy_RenderData * self );
 static PyMethodDef BPy_RenderData_methods[] = {
 	{"render", ( PyCFunction ) RenderData_Render, METH_NOARGS,
 	 "() - render the scene\n"},
+	{"saveRenderedImage", (PyCFunction)RenderData_SaveRenderedImage, METH_VARARGS,
+	 "(filename) - save an image generated by a call to render() (set output path first)\n"},
 	{"renderAnim", ( PyCFunction ) RenderData_RenderAnim, METH_NOARGS,
 	 "() - render a sequence from start frame to end frame\n"},
 	{"play", ( PyCFunction ) RenderData_Play, METH_NOARGS,
@@ -303,7 +301,7 @@ static PyMethodDef BPy_RenderData_methods[] = {
 	 "(int) - get/set edge intensity for toon shading\n"},
 	{"setEdgeColor", ( PyCFunction ) RenderData_SetEdgeColor, METH_VARARGS,
 	 "(f,f,f) - set the edge color for toon shading - Red,Green,Blue expected.\n"},
-	{"getEdgeColor", ( PyCFunction ) RenderData_GetEdgeColor, METH_VARARGS,
+	{"getEdgeColor", ( PyCFunction ) RenderData_GetEdgeColor, METH_NOARGS,
 	 "() - get the edge color for toon shading - Red,Green,Blue expected.\n"},
 	{"edgeAntiShift", ( PyCFunction ) RenderData_EdgeAntiShift,
 	 METH_VARARGS,
@@ -377,6 +375,8 @@ static PyMethodDef BPy_RenderData_methods[] = {
 	 "(int) - get/set the current frame for rendering\n"},
 	{"endFrame", ( PyCFunction ) RenderData_EndFrame, METH_VARARGS,
 	 "(int) - get/set the ending frame for rendering\n"},
+	{"getTimeCode", ( PyCFunction ) RenderData_getTimeCode, METH_NOARGS,
+	 "get the current frame in HH:MM:SS:FF format\n"},
 	{"imageSizeX", ( PyCFunction ) RenderData_ImageSizeX, METH_VARARGS,
 	 "(int) - get/set the image width in pixels\n"},
 	{"imageSizeY", ( PyCFunction ) RenderData_ImageSizeY, METH_VARARGS,
@@ -479,7 +479,7 @@ static PyMethodDef BPy_RenderData_methods[] = {
 	 METH_VARARGS,
 	 "(f,f,f) - set the red, green, blue component of the bars\n"},
 	{"getGameFrameColor", ( PyCFunction ) RenderData_GetGameFrameColor,
-	 METH_VARARGS,
+	 METH_NOARGS,
 	 "() - get the red, green, blue component of the bars\n"},
 	{"gammaLevel", ( PyCFunction ) RenderData_GammaLevel, METH_VARARGS,
 	 "(float) - get/set the gamma value for blending oversampled images (1.0 = no correction\n"},
@@ -525,6 +525,7 @@ PyTypeObject RenderData_Type = {
 	0, 0, 0, 0, 0, 0,
 	BPy_RenderData_methods,	/* tp_methods */
 	0,			/* tp_members */
+	0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
 };
 //---------------------------------------------------Render Module Init--
 PyObject *Render_Init( void )
@@ -544,6 +545,7 @@ PyObject *Render_Init( void )
 	PyModule_AddIntConstant( submodule, "QUICKTIME", R_QUICKTIME );
 	PyModule_AddIntConstant( submodule, "TARGA", R_TARGA );
 	PyModule_AddIntConstant( submodule, "RAWTGA", R_RAWTGA );
+	PyModule_AddIntConstant( submodule, "HDR", R_RADHDR );
 	PyModule_AddIntConstant( submodule, "PNG", R_PNG );
 	PyModule_AddIntConstant( submodule, "BMP", R_BMP );
 	PyModule_AddIntConstant( submodule, "JPEG", R_JPEG90 );
@@ -640,7 +642,7 @@ static PyObject *M_Render_BitToggleInt( PyObject * args, int setting,
 		*structure |= setting;
 	else
 		*structure &= ~setting;
-	allqueue( REDRAWBUTSSCENE, 0 );
+	EXPP_allqueue( REDRAWBUTSSCENE, 0 );
 
 	return EXPP_incr_ret( Py_None );
 
@@ -666,7 +668,7 @@ static PyObject *M_Render_BitToggleShort( PyObject * args, short setting,
 		*structure |= setting;
 	else
 		*structure &= ~setting;
-	allqueue( REDRAWBUTSSCENE, 0 );
+	EXPP_allqueue( REDRAWBUTSSCENE, 0 );
 
 	return EXPP_incr_ret( Py_None );
 
@@ -693,7 +695,7 @@ static PyObject *M_Render_GetSetAttributeFloat( PyObject * args,
 		}
 
 		*structure = property;
-		allqueue( REDRAWBUTSSCENE, 0 );
+		EXPP_allqueue( REDRAWBUTSSCENE, 0 );
 		return EXPP_incr_ret( Py_None );
 	} else
 		return Py_BuildValue( "f", *structure );
@@ -720,7 +722,7 @@ static PyObject *M_Render_GetSetAttributeShort( PyObject * args,
 		}
 
 		*structure = property;
-		allqueue( REDRAWBUTSSCENE, 0 );
+		EXPP_allqueue( REDRAWBUTSSCENE, 0 );
 		return EXPP_incr_ret( Py_None );
 	} else
 		return Py_BuildValue( "h", *structure );
@@ -746,7 +748,7 @@ static PyObject *M_Render_GetSetAttributeInt( PyObject * args, int *structure,
 		}
 
 		*structure = property;
-		allqueue( REDRAWBUTSSCENE, 0 );
+		EXPP_allqueue( REDRAWBUTSSCENE, 0 );
 		return EXPP_incr_ret( Py_None );
 	} else
 		return Py_BuildValue( "i", *structure );
@@ -769,8 +771,8 @@ static void M_Render_DoSizePreset( BPy_RenderData * self, short xsch,
 	self->renderContext->yparts = yparts;
 
 	BLI_init_rctf( &self->renderContext->safety, a, b, c, d );
-	allqueue( REDRAWBUTSSCENE, 0 );
-	allqueue( REDRAWVIEWCAM, 0 );
+	EXPP_allqueue( REDRAWBUTSSCENE, 0 );
+	EXPP_allqueue( REDRAWVIEWCAM, 0 );
 }
 
 //------------------------------------Render Module Function Definitions-
@@ -792,35 +794,35 @@ PyObject *M_Render_SetRenderWinPos( PyObject * self, PyObject * args )
 		return ( EXPP_ReturnPyObjError( PyExc_AttributeError,
 						"expected a list" ) );
 
-	R.winpos = 0;
+	G.winpos = 0;
 	for( x = 0; x < PyList_Size( list ); x++ ) {
 		if( !PyArg_Parse( PyList_GetItem( list, x ), "s", &loc ) ) {
 			return EXPP_ReturnPyObjError( PyExc_TypeError,
 						      "python list not parseable\n" );
 		}
 		if( strcmp( loc, "SW" ) == 0 || strcmp( loc, "sw" ) == 0 )
-			R.winpos |= 1;
+			G.winpos |= 1;
 		else if( strcmp( loc, "S" ) == 0 || strcmp( loc, "s" ) == 0 )
-			R.winpos |= 2;
+			G.winpos |= 2;
 		else if( strcmp( loc, "SE" ) == 0 || strcmp( loc, "se" ) == 0 )
-			R.winpos |= 4;
+			G.winpos |= 4;
 		else if( strcmp( loc, "W" ) == 0 || strcmp( loc, "w" ) == 0 )
-			R.winpos |= 8;
+			G.winpos |= 8;
 		else if( strcmp( loc, "C" ) == 0 || strcmp( loc, "c" ) == 0 )
-			R.winpos |= 16;
+			G.winpos |= 16;
 		else if( strcmp( loc, "E" ) == 0 || strcmp( loc, "e" ) == 0 )
-			R.winpos |= 32;
+			G.winpos |= 32;
 		else if( strcmp( loc, "NW" ) == 0 || strcmp( loc, "nw" ) == 0 )
-			R.winpos |= 64;
+			G.winpos |= 64;
 		else if( strcmp( loc, "N" ) == 0 || strcmp( loc, "n" ) == 0 )
-			R.winpos |= 128;
+			G.winpos |= 128;
 		else if( strcmp( loc, "NE" ) == 0 || strcmp( loc, "ne" ) == 0 )
-			R.winpos |= 256;
+			G.winpos |= 256;
 		else
 			return EXPP_ReturnPyObjError( PyExc_AttributeError,
 						      "list contains unknown string\n" );
 	}
-	allqueue( REDRAWBUTSSCENE, 0 );
+	EXPP_allqueue( REDRAWBUTSSCENE, 0 );
 
 	return EXPP_incr_ret( Py_None );
 }
@@ -828,8 +830,8 @@ PyObject *M_Render_SetRenderWinPos( PyObject * self, PyObject * args )
 //------------------------------------Render.EnableDispView() -----------
 PyObject *M_Render_EnableDispView( PyObject * self )
 {
-	R.displaymode = R_DISPLAYVIEW;
-	allqueue( REDRAWBUTSSCENE, 0 );
+	G.displaymode = R_DISPLAYVIEW;
+	EXPP_allqueue( REDRAWBUTSSCENE, 0 );
 
 	return EXPP_incr_ret( Py_None );
 }
@@ -837,8 +839,8 @@ PyObject *M_Render_EnableDispView( PyObject * self )
 //------------------------------------Render.EnableDispWin() ------------
 PyObject *M_Render_EnableDispWin( PyObject * self )
 {
-	R.displaymode = R_DISPLAYWIN;
-	allqueue( REDRAWBUTSSCENE, 0 );
+	G.displaymode = R_DISPLAYWIN;
+	EXPP_allqueue( REDRAWBUTSSCENE, 0 );
 
 	return EXPP_incr_ret( Py_None );
 }
@@ -862,22 +864,99 @@ PyObject *RenderData_Render( BPy_RenderData * self )
 {
 	Scene *oldsce;
 
-	oldsce = G.scene;
-	set_scene( self->scene );
-	BIF_do_render( 0 );
-	set_scene( oldsce );
+	if (!G.background) {
+		oldsce = G.scene;
+		set_scene( self->scene );
+		BIF_do_render( 0 );
+		set_scene( oldsce );
+	}
+
+	else { /* background mode (blender -b file.blend -P script) */
+
+		int end_frame = G.scene->r.efra; /* is of type short currently */
+
+		if (G.scene != self->scene)
+			return EXPP_ReturnPyObjError (PyExc_RuntimeError,
+				"scene to render in bg mode must be the active scene");
+
+		G.scene->r.efra = G.scene->r.sfra;
+
+		RE_animrender(NULL);
+
+		G.scene->r.efra = (short)end_frame;
+	}
+
 	return EXPP_incr_ret( Py_None );
 }
+//------------------------------------RenderData.Render() ---------------
+//This will save the rendered image to an output file path already defined
+PyObject *RenderData_SaveRenderedImage ( BPy_RenderData * self, PyObject *args )
+{
+	char dir[FILE_MAXDIR * 2], str[FILE_MAXFILE * 2], strn[256];
+	char *name_str, filepath[FILE_MAXDIR+FILE_MAXFILE];
 
+	if( !PyArg_ParseTuple( args, "s", &name_str ) )
+		return EXPP_ReturnPyObjError( PyExc_TypeError,
+						"expected a filename (string)" );
+
+	if( strlen(self->renderContext->pic) + strlen(name_str) >
+			sizeof(filepath)-1 )
+		return EXPP_ReturnPyObjError( PyExc_ValueError,
+						"full filename too long" );
+
+	BLI_strncpy( filepath, self->renderContext->pic, sizeof(filepath) );
+	strcat(filepath, name_str);
+	
+	if(!R.rectot) {
+		return EXPP_ReturnPyObjError (PyExc_RuntimeError,
+			"No image rendered");
+	} else {
+		if(G.ima[0]==0) {
+			strcpy(dir, G.sce);
+			BLI_splitdirstring(dir, str);
+			strcpy(G.ima, dir);
+		}
+		
+		R.r.imtype= G.scene->r.imtype;
+		R.r.quality= G.scene->r.quality;
+		R.r.planes= G.scene->r.planes;
+	
+		strcpy(strn, filepath);
+		BLI_convertstringcode(strn, G.sce, G.scene->r.cfra);
+		if(BLI_testextensie(strn,".blend")) {
+			return EXPP_ReturnPyObjError (PyExc_RuntimeError,
+				"Wrong filename");
+		}
+
+		waitcursor(1);
+		schrijfplaatje(strn);
+		strcpy(G.ima, filepath);
+		waitcursor(0);
+	}
+
+	return EXPP_incr_ret(Py_None);
+}
 //------------------------------------RenderData.RenderAnim() -----------
 PyObject *RenderData_RenderAnim( BPy_RenderData * self )
 {
 	Scene *oldsce;
 
-	oldsce = G.scene;
-	set_scene( self->scene );
-	BIF_do_render( 1 );
-	set_scene( oldsce );
+	if (!G.background) {
+		oldsce = G.scene;
+		set_scene( self->scene );
+		BIF_do_render( 1 );
+		set_scene( oldsce );
+	}
+	else { /* background mode (blender -b file.blend -P script) */
+		if (G.scene != self->scene)
+			return EXPP_ReturnPyObjError (PyExc_RuntimeError,
+				"scene to render in bg mode must be the active scene");
+
+		if (G.scene->r.sfra > G.scene->r.efra)
+			return EXPP_ReturnPyObjError (PyExc_RuntimeError,
+				"start frame must be less or equal to end frame");
+		RE_animrender(NULL);
+	}
 	return EXPP_incr_ret( Py_None );
 }
 
@@ -894,10 +973,10 @@ PyObject *RenderData_Play( BPy_RenderData * self )
 	if( self->renderContext->imtype == R_QUICKTIME ) {
 
 		strcpy( file, self->renderContext->pic );
-		BLI_convertstringcode( file, self->scene,
+		BLI_convertstringcode( file, (char *) self->scene,
 				       self->renderContext->cfra );
 		RE_make_existing_file( file );
-		if( strcasecmp( file + strlen( file ) - 4, ".mov" ) ) {
+		if( BLI_strcasecmp( file + strlen( file ) - 4, ".mov" ) ) {
 			sprintf( txt, "%04d_%04d.mov",
 				 ( self->renderContext->sfra ),
 				 ( self->renderContext->efra ) );
@@ -911,7 +990,7 @@ PyObject *RenderData_Play( BPy_RenderData * self )
 		BLI_convertstringcode( file, G.sce,
 				       self->renderContext->cfra );
 		RE_make_existing_file( file );
-		if( strcasecmp( file + strlen( file ) - 4, ".avi" ) ) {
+		if( BLI_strcasecmp( file + strlen( file ) - 4, ".avi" ) ) {
 			sprintf( txt, "%04d_%04d.avi",
 				 ( self->renderContext->sfra ),
 				 ( self->renderContext->efra ) );
@@ -919,14 +998,14 @@ PyObject *RenderData_Play( BPy_RenderData * self )
 		}
 	}
 	if( BLI_exist( file ) ) {
-		calc_renderwin_rectangle( R.winpos, pos, size );
+		calc_renderwin_rectangle( G.winpos, pos, size );
 		sprintf( str, "%s -a -p %d %d \"%s\"", bprogname, pos[0],
 			 pos[1], file );
 		system( str );
 	} else {
 		makepicstring( file, self->renderContext->sfra );
 		if( BLI_exist( file ) ) {
-			calc_renderwin_rectangle( R.winpos, pos, size );
+			calc_renderwin_rectangle( G.winpos, pos, size );
 			sprintf( str, "%s -a -p %d %d \"%s\"", bprogname,
 				 pos[0], pos[1], file );
 			system( str );
@@ -951,7 +1030,7 @@ PyObject *RenderData_SetRenderPath( BPy_RenderData * self, PyObject * args )
 						"path is too long (SetRenderPath)" ) );
 
 	strcpy( self->renderContext->pic, name );
-	allqueue( REDRAWBUTSSCENE, 0 );
+	EXPP_allqueue( REDRAWBUTSSCENE, 0 );
 
 	return EXPP_incr_ret( Py_None );
 }
@@ -977,7 +1056,7 @@ PyObject *RenderData_SetBackbufPath( BPy_RenderData * self, PyObject * args )
 						"path is too long (SetBackbufPath)" ) );
 
 	strcpy( self->renderContext->backbuf, name );
-	allqueue( REDRAWBUTSSCENE, 0 );
+	EXPP_allqueue( REDRAWBUTSSCENE, 0 );
 
 	ima = add_image( name );
 	if( ima ) {
@@ -1015,7 +1094,7 @@ PyObject *RenderData_SetFtypePath( BPy_RenderData * self, PyObject * args )
 						"path is too long (SetFtypePath)" ) );
 
 	strcpy( self->renderContext->ftype, name );
-	allqueue( REDRAWBUTSSCENE, 0 );
+	EXPP_allqueue( REDRAWBUTSSCENE, 0 );
 
 	return EXPP_incr_ret( Py_None );
 }
@@ -1132,8 +1211,8 @@ PyObject *RenderData_SetOversamplingLevel( BPy_RenderData * self,
 		return ( EXPP_ReturnPyObjError( PyExc_AttributeError,
 						"expected 5,8,11, or 16" ) );
 
-	self->renderContext->osa = level;
-	allqueue( REDRAWBUTSSCENE, 0 );
+	self->renderContext->osa = (short)level;
+	EXPP_allqueue( REDRAWBUTSSCENE, 0 );
 
 	return EXPP_incr_ret( Py_None );
 }
@@ -1173,7 +1252,7 @@ PyObject *RenderData_PartsY( BPy_RenderData * self, PyObject * args )
 PyObject *RenderData_EnableSky( BPy_RenderData * self )
 {
 	self->renderContext->alphamode = R_ADDSKY;
-	allqueue( REDRAWBUTSSCENE, 0 );
+	EXPP_allqueue( REDRAWBUTSSCENE, 0 );
 
 	return EXPP_incr_ret( Py_None );
 }
@@ -1182,7 +1261,7 @@ PyObject *RenderData_EnableSky( BPy_RenderData * self )
 PyObject *RenderData_EnablePremultiply( BPy_RenderData * self )
 {
 	self->renderContext->alphamode = R_ALPHAPREMUL;
-	allqueue( REDRAWBUTSSCENE, 0 );
+	EXPP_allqueue( REDRAWBUTSSCENE, 0 );
 
 	return EXPP_incr_ret( Py_None );
 }
@@ -1191,7 +1270,7 @@ PyObject *RenderData_EnablePremultiply( BPy_RenderData * self )
 PyObject *RenderData_EnableKey( BPy_RenderData * self )
 {
 	self->renderContext->alphamode = R_ALPHAKEY;
-	allqueue( REDRAWBUTSSCENE, 0 );
+	EXPP_allqueue( REDRAWBUTSSCENE, 0 );
 
 	return EXPP_incr_ret( Py_None );
 }
@@ -1246,8 +1325,8 @@ PyObject *RenderData_SetRenderWinSize( BPy_RenderData * self, PyObject * args )
 		return ( EXPP_ReturnPyObjError( PyExc_AttributeError,
 						"expected 25, 50, 75, or 100" ) );
 
-	self->renderContext->size = size;
-	allqueue( REDRAWBUTSSCENE, 0 );
+	self->renderContext->size = (short)size;
+	EXPP_allqueue( REDRAWBUTSSCENE, 0 );
 
 	return EXPP_incr_ret( Py_None );
 }
@@ -1282,6 +1361,10 @@ PyObject *RenderData_EnableGaussFilter( BPy_RenderData * self,
 {
 	return M_Render_BitToggleInt( args, R_GAUSS,
 				      &self->renderContext->mode );
+	
+	/* note, this now is obsolete (ton) */
+	/* we now need a call like RenderData_SetFilter() or so */
+	/* choices are listed in DNA_scene_types.h (search filtertype) */
 }
 
 //------------------------------------RenderData.EnableBorderRender() ---
@@ -1295,7 +1378,7 @@ PyObject *RenderData_EnableBorderRender( BPy_RenderData * self,
 //------------------------------------RenderData.SetBorder() ------------
 PyObject *RenderData_SetBorder( BPy_RenderData * self, PyObject * args )
 {
-	float xmin, ymin, xmax, ymax;
+	float xmin = 0.0f, ymin = 0.0f, xmax = 0.0f, ymax = 0.0f;
 	int status = 0;
 	PyObject *ob_args;
 
@@ -1329,7 +1412,7 @@ PyObject *RenderData_SetBorder( BPy_RenderData * self, PyObject * args )
 	self->renderContext->border.xmax = xmax;
 	self->renderContext->border.ymax = ymax;
 
-	allqueue( REDRAWVIEWCAM, 1 );
+	EXPP_allqueue( REDRAWVIEWCAM, 1 );
 
 	return EXPP_incr_ret( Py_None );
 }
@@ -1354,21 +1437,21 @@ PyObject *RenderData_GaussFilterSize( BPy_RenderData * self, PyObject * args )
 PyObject *RenderData_StartFrame( BPy_RenderData * self, PyObject * args )
 {
 	return M_Render_GetSetAttributeShort( args, &self->renderContext->sfra,
-					      1, 18000 );
+					      1, MAXFRAME );
 }
 
 //------------------------------------RenderData.CurrentFrame() ---------
 PyObject *RenderData_CurrentFrame( BPy_RenderData * self, PyObject * args )
 {
 	return M_Render_GetSetAttributeShort( args, &self->renderContext->cfra,
-					      1, 18000 );
+					      1, MAXFRAME );
 }
 
 //------------------------------------RenderData.EndFrame() -------------
 PyObject *RenderData_EndFrame( BPy_RenderData * self, PyObject * args )
 {
 	return M_Render_GetSetAttributeShort( args, &self->renderContext->efra,
-					      1, 18000 );
+					      1, MAXFRAME );
 }
 
  //------------------------------------RenderData.ImageSizeX() ----------
@@ -1416,7 +1499,7 @@ PyObject *RenderData_SetRenderer( BPy_RenderData * self, PyObject * args )
 		return ( EXPP_ReturnPyObjError( PyExc_AttributeError,
 						"expected INTERN or YAFRAY" ) );
 
-	allqueue( REDRAWBUTSSCENE, 0 );
+	EXPP_allqueue( REDRAWBUTSSCENE, 0 );
 	return EXPP_incr_ret( Py_None );
 }
 
@@ -1450,6 +1533,8 @@ PyObject *RenderData_SetImageType( BPy_RenderData * self, PyObject * args )
 		self->renderContext->imtype = R_TARGA;
 	else if( type == R_RAWTGA )
 		self->renderContext->imtype = R_RAWTGA;
+	else if( type == R_RADHDR )
+		self->renderContext->imtype = R_RADHDR;
 	else if( type == R_PNG )
 		self->renderContext->imtype = R_PNG;
 	else if( type == R_BMP )
@@ -1468,7 +1553,7 @@ PyObject *RenderData_SetImageType( BPy_RenderData * self, PyObject * args )
 		return ( EXPP_ReturnPyObjError( PyExc_AttributeError,
 						"unknown constant - see modules dict for help" ) );
 
-	allqueue( REDRAWBUTSSCENE, 0 );
+	EXPP_allqueue( REDRAWBUTSSCENE, 0 );
 	return EXPP_incr_ret( Py_None );
 }
 
@@ -1492,7 +1577,7 @@ PyObject *RenderData_FramesPerSec( BPy_RenderData * self, PyObject * args )
 PyObject *RenderData_EnableGrayscale( BPy_RenderData * self )
 {
 	self->renderContext->planes = R_PLANESBW;
-	allqueue( REDRAWBUTSSCENE, 0 );
+	EXPP_allqueue( REDRAWBUTSSCENE, 0 );
 
 	return EXPP_incr_ret( Py_None );
 }
@@ -1501,7 +1586,7 @@ PyObject *RenderData_EnableGrayscale( BPy_RenderData * self )
 PyObject *RenderData_EnableRGBColor( BPy_RenderData * self )
 {
 	self->renderContext->planes = R_PLANES24;
-	allqueue( REDRAWBUTSSCENE, 0 );
+	EXPP_allqueue( REDRAWBUTSSCENE, 0 );
 
 	return EXPP_incr_ret( Py_None );
 }
@@ -1510,7 +1595,7 @@ PyObject *RenderData_EnableRGBColor( BPy_RenderData * self )
 PyObject *RenderData_EnableRGBAColor( BPy_RenderData * self )
 {
 	self->renderContext->planes = R_PLANES32;
-	allqueue( REDRAWBUTSSCENE, 0 );
+	EXPP_allqueue( REDRAWBUTSSCENE, 0 );
 
 	return EXPP_incr_ret( Py_None );
 }
@@ -1527,64 +1612,64 @@ PyObject *RenderData_SizePreset( BPy_RenderData * self, PyObject * args )
 	if( type == B_PR_PAL ) {
 		M_Render_DoSizePreset( self, 720, 576, 54, 51, 100,
 				       self->renderContext->xparts,
-				       self->renderContext->yparts, 25, 0.1,
-				       0.9, 0.1, 0.9 );
+				       self->renderContext->yparts, 25, 0.1f,
+				       0.9f, 0.1f, 0.9f );
 		self->renderContext->mode &= ~R_PANORAMA;
-		BLI_init_rctf( &self->renderContext->safety, 0.1, 0.9, 0.1,
-			       0.9 );
+		BLI_init_rctf( &self->renderContext->safety, 0.1f, 0.9f, 0.1f,
+			       0.9f );
 	} else if( type == B_PR_NTSC ) {
 		M_Render_DoSizePreset( self, 720, 480, 10, 11, 100, 1, 1,
-				       30, 0.1, 0.9, 0.1, 0.9 );
+				       30, 0.1f, 0.9f, 0.1f, 0.9f );
 		self->renderContext->mode &= ~R_PANORAMA;
-		BLI_init_rctf( &self->renderContext->safety, 0.1, 0.9, 0.1,
-			       0.9 );
+		BLI_init_rctf( &self->renderContext->safety, 0.1f, 0.9f, 0.1f,
+			       0.9f );
 	} else if( type == B_PR_PRESET ) {
 		M_Render_DoSizePreset( self, 720, 576, 54, 51, 100, 1, 1,
-				       self->renderContext->frs_sec, 0.1, 0.9,
-				       0.1, 0.9 );
+				       self->renderContext->frs_sec, 0.1f, 0.9f,
+				       0.1f, 0.9f );
 		self->renderContext->mode = R_OSA + R_SHADOW + R_FIELDS;
 		self->renderContext->imtype = R_TARGA;
-		BLI_init_rctf( &self->renderContext->safety, 0.1, 0.9, 0.1,
-			       0.9 );
+		BLI_init_rctf( &self->renderContext->safety, 0.1f, 0.9f, 0.1f,
+			       0.9f );
 	} else if( type == B_PR_PRV ) {
 		M_Render_DoSizePreset( self, 640, 512, 1, 1, 50, 1, 1,
-				       self->renderContext->frs_sec, 0.1, 0.9,
-				       0.1, 0.9 );
+				       self->renderContext->frs_sec, 0.1f, 0.9f,
+				       0.1f, 0.9f );
 		self->renderContext->mode &= ~R_PANORAMA;
-		BLI_init_rctf( &self->renderContext->safety, 0.1, 0.9, 0.1,
-			       0.9 );
+		BLI_init_rctf( &self->renderContext->safety, 0.1f, 0.9f, 0.1f,
+			       0.9f );
 	} else if( type == B_PR_PC ) {
 		M_Render_DoSizePreset( self, 640, 480, 100, 100, 100, 1, 1,
-				       self->renderContext->frs_sec, 0.0, 1.0,
-				       0.0, 1.0 );
+				       self->renderContext->frs_sec, 0.0f, 1.0f,
+				       0.0f, 1.0f );
 		self->renderContext->mode &= ~R_PANORAMA;
-		BLI_init_rctf( &self->renderContext->safety, 0.0, 1.0, 0.0,
-			       1.0 );
+		BLI_init_rctf( &self->renderContext->safety, 0.0f, 1.0f, 0.0f,
+			       1.0f );
 	} else if( type == B_PR_PAL169 ) {
 		M_Render_DoSizePreset( self, 720, 576, 64, 45, 100, 1, 1,
-				       25, 0.1, 0.9, 0.1, 0.9 );
+				       25, 0.1f, 0.9f, 0.1f, 0.9f );
 		self->renderContext->mode &= ~R_PANORAMA;
-		BLI_init_rctf( &self->renderContext->safety, 0.1, 0.9, 0.1,
-			       0.9 );
+		BLI_init_rctf( &self->renderContext->safety, 0.1f, 0.9f, 0.1f,
+			       0.9f );
 	} else if( type == B_PR_PANO ) {
 		M_Render_DoSizePreset( self, 36, 176, 115, 100, 100, 16, 1,
-				       self->renderContext->frs_sec, 0.1, 0.9,
-				       0.1, 0.9 );
+				       self->renderContext->frs_sec, 0.1f, 0.9f,
+				       0.1f, 0.9f );
 		self->renderContext->mode |= R_PANORAMA;
-		BLI_init_rctf( &self->renderContext->safety, 0.1, 0.9, 0.1,
-			       0.9 );
+		BLI_init_rctf( &self->renderContext->safety, 0.1f, 0.9f, 0.1f,
+			       0.9f );
 	} else if( type == B_PR_FULL ) {
 		M_Render_DoSizePreset( self, 1280, 1024, 1, 1, 100, 1, 1,
-				       self->renderContext->frs_sec, 0.1, 0.9,
-				       0.1, 0.9 );
+				       self->renderContext->frs_sec, 0.1f, 0.9f,
+				       0.1f, 0.9f );
 		self->renderContext->mode &= ~R_PANORAMA;
-		BLI_init_rctf( &self->renderContext->safety, 0.1, 0.9, 0.1,
-			       0.9 );
+		BLI_init_rctf( &self->renderContext->safety, 0.1f, 0.9f, 0.1f,
+			       0.9f );
 	} else
 		return ( EXPP_ReturnPyObjError( PyExc_AttributeError,
 						"unknown constant - see modules dict for help" ) );
 
-	allqueue( REDRAWBUTSSCENE, 0 );
+	EXPP_allqueue( REDRAWBUTSSCENE, 0 );
 	return EXPP_incr_ret( Py_None );
 }
 
@@ -1609,12 +1694,12 @@ PyObject *RenderData_SetYafrayGIQuality( BPy_RenderData * self,
 	if( type == PY_NONE || type == PY_LOW ||
 	    type == PY_MEDIUM || type == PY_HIGH ||
 	    type == PY_HIGHER || type == PY_BEST ) {
-		self->renderContext->GIquality = type;
+		self->renderContext->GIquality = (short)type;
 	} else
 		return ( EXPP_ReturnPyObjError( PyExc_AttributeError,
 						"unknown constant - see modules dict for help" ) );
 
-	allqueue( REDRAWBUTSSCENE, 0 );
+	EXPP_allqueue( REDRAWBUTSSCENE, 0 );
 	return EXPP_incr_ret( Py_None );
 }
 
@@ -1629,12 +1714,12 @@ PyObject *RenderData_SetYafrayGIMethod( BPy_RenderData * self,
 						"expected constant" ) );
 
 	if( type == PY_NONE || type == PY_SKYDOME || type == PY_FULL ) {
-		self->renderContext->GImethod = type;
+		self->renderContext->GImethod = (short)type;
 	} else
 		return ( EXPP_ReturnPyObjError( PyExc_AttributeError,
 						"unknown constant - see modules dict for help" ) );
 
-	allqueue( REDRAWBUTSSCENE, 0 );
+	EXPP_allqueue( REDRAWBUTSSCENE, 0 );
 	return EXPP_incr_ret( Py_None );
 }
 
@@ -1981,15 +2066,62 @@ PyObject *RenderData_EnableSGICosmo( BPy_RenderData * self, PyObject * args )
 //------------------------------------RenderData.OldMapValue() -----------
 PyObject *RenderData_OldMapValue( BPy_RenderData * self, PyObject * args )
 {
-	return M_Render_GetSetAttributeShort( args,
-					      &self->renderContext->framapto,
-					      1, 900 );
+	PyObject *tmp = M_Render_GetSetAttributeShort(args,
+		&self->renderContext->framapto, 1, 900);
+	self->renderContext->framelen =
+		(float)self->renderContext->framapto / self->renderContext->images;
+	return tmp;
 }
 
 //------------------------------------RenderData.NewMapValue() -----------
 PyObject *RenderData_NewMapValue( BPy_RenderData * self, PyObject * args )
 {
-	return M_Render_GetSetAttributeShort( args,
-					      &self->renderContext->images, 1,
-					      900 );
+	PyObject *tmp = M_Render_GetSetAttributeShort(args,
+			&self->renderContext->images, 1, 900);
+	self->renderContext->framelen =
+		(float)self->renderContext->framapto / self->renderContext->images;
+	return tmp;
 }
+
+//------------------------------------RenderData.getTimeCode() -----------
+PyObject *RenderData_getTimeCode( BPy_RenderData * self){
+    PyObject *ret = NULL;
+    char tc[12];
+    char h[3],m[3],s[3],f[3];
+    int hi,mi,si,fi,hold,fps,cfa;
+    
+    fps = self->renderContext->frs_sec;
+    cfa = self->renderContext->cfra-1;
+    fi =  cfa % fps;
+        	    
+    hi   = (cfa) / (fps*60*60);
+    hold = (cfa) % (fps*60*60);     
+    mi   = hold / (fps*60);
+    hold = hold % (fps*60);
+    si   = hold / fps;
+    
+    if(hi > 99){
+        ret = PyString_FromString("Time Greater than 99 Hours!");	
+    }
+     else{
+        if(hi > 9)
+            sprintf(h,"%d",hi);
+        else
+            sprintf(h,"0%d",hi);
+        if(mi > 9)
+            sprintf(m,"%d",mi);
+        else
+            sprintf(m,"0%d",mi);            
+        if(si > 9)
+            sprintf(s,"%d",si);
+        else
+            sprintf(s,"0%d",si);
+        if(fi > 9)
+            sprintf(f,"%d",fi);
+        else
+            sprintf(f,"0%d",fi);
+	    sprintf(tc,"%s:%s:%s:%s",h,m,s,f);
+    	ret = PyString_FromString(tc);
+    } 
+    return ret;    
+}            
