@@ -1,5 +1,5 @@
 /**
- * $Id: quicktime_import.c,v 1.13 2004/12/19 13:45:03 phase Exp $
+ * $Id: quicktime_import.c,v 1.15 2005/03/09 19:45:55 lukep Exp $
  *
  * quicktime_import.c
  *
@@ -90,7 +90,7 @@ void quicktime_exit(void)
 #if defined(_WIN32) || defined(__APPLE__)
 #ifdef WITH_QUICKTIME
 	if(G.have_quicktime) {
-		free_qtcodecdataExt();
+		free_qtcomponentdata();
 		ExitMovies();
 #ifdef _WIN32
 		TerminateQTML();
@@ -236,7 +236,7 @@ void free_anim_quicktime (struct anim *anim) {
 }
 
 
-OSErr QT_get_frameIndexes(struct anim *anim)
+static OSErr QT_get_frameIndexes(struct anim *anim)
 {
 	int i;
 	OSErr	anErr = noErr;
@@ -348,7 +348,7 @@ ImBuf * qtime_fetchibuf (struct anim *anim, int position)
 
 // following two functions only here to get movie pixeldepth
 
-int GetFirstVideoMedia(struct anim *anim)
+static int GetFirstVideoMedia(struct anim *anim)
 {
 	long    numTracks;
 	OSType  mediaType;
@@ -370,10 +370,10 @@ int GetFirstVideoMedia(struct anim *anim)
 	return 0;      // went through all tracks and no video
 }
 
-short GetFirstVideoTrackPixelDepth(struct anim *anim)
+static short GetFirstVideoTrackPixelDepth(struct anim *anim)
 {
 	SampleDescriptionHandle imageDescH =	(SampleDescriptionHandle)NewHandle(sizeof(Handle));
-	long	trackIndex = 0;
+//	long	trackIndex = 0; /*unused*/
 	
 	if(!GetFirstVideoMedia(anim))
 		return -1;
@@ -507,14 +507,16 @@ int imb_is_a_quicktime (char *name)
 	GraphicsImportComponent		theImporter = NULL;
 
 	FSSpec	theFSSpec;
-	Str255  dst;
+#ifdef _WIN32
+	Str255  dst; /*unused*/
+#endif
 	char	theFullPath[255];
 
-	Boolean						isMovieFile = false;
-	AliasHandle					myAlias = NULL;
-	Component					myImporter = NULL;
+//	Boolean						isMovieFile = false; /*unused*/
+//	AliasHandle					myAlias = NULL; /*unused*/
+//	Component					myImporter = NULL; /*unused*/
 #ifdef __APPLE__
-	FInfo						myFinderInfo;
+//	FInfo						myFinderInfo; /*unused*/
 	FSRef						myRef;
 #endif
 	OSErr						err = noErr;
@@ -564,7 +566,7 @@ ImBuf  *imb_quicktime_decode(unsigned char *mem, int size, int flags)
 	int x, y, depth;
 	int have_gw = FALSE;
 	ImBuf *ibuf = NULL;
-	ImBuf *imbuf = NULL;
+//	ImBuf *imbuf = NULL; /*unused*/
 	GWorldPtr	offGWorld;
 	PixMapHandle		myPixMap = NULL;
 
