@@ -1,7 +1,7 @@
 /*
  * jitter.h
  *
- * $Id: jitter.h,v 1.6 2002/12/30 07:15:47 mein Exp $
+ * $Id: jitter.h,v 1.7 2004/12/27 19:28:50 ton Exp $
  *
  * ***** BEGIN GPL/BL DUAL LICENSE BLOCK *****
  *
@@ -41,7 +41,8 @@ extern "C" {
 
 extern float jit[64][2];
 
-void initjit(float *jitarr, int num);
+extern void initjit(float *jitarr, int num);
+extern void init_render_jit(int nr);
 
 #ifdef __cplusplus
 }
