@@ -1,5 +1,5 @@
 /**
- * $Id: edgeRender.c,v 1.9 2004/12/04 12:02:19 ton Exp $
+ * $Id: edgeRender.c,v 1.13 2005/11/14 14:27:44 ton Exp $
  *
  * ***** BEGIN GPL/BL DUAL LICENSE BLOCK *****
  *
@@ -60,37 +60,25 @@
 
 #include "MEM_guardedalloc.h"
 #include "MTC_vectorops.h"
+#include "BKE_utildefines.h"
 
 #include "RE_callbacks.h"
 #include "edgeRender.h"
 #include "render.h"
-#include "render_intern.h"
 #include "zbuf.h" /* for zbufclipwire and zbufclip */
 #include "jitter.h"
 
-#ifdef HAVE_CONFIG_H
-#include <config.h>
-#endif
-
 #ifdef RE_EDGERENDERSAFE
 char edgeRender_h[] = EDGERENDER_H;
-char edgeRender_c[] = "$Id: edgeRender.c,v 1.9 2004/12/04 12:02:19 ton Exp $";
+char edgeRender_c[] = "$Id: edgeRender.c,v 1.13 2005/11/14 14:27:44 ton Exp $";
 #include "errorHandler.h"
 #endif
 
-/* ------------------------------------------------------------------------- */
-/* the lazy way: */
-#define MIN2(x,y)               ( (x)<(y) ? (x) : (y) )
-
 /* ------------------------------------------------------------------------- */
 
  /* These function pointers are used for z buffer filling.    */
-extern void (*zbuffunc)(float *, float *, float *);
-extern void (*zbuflinefunc)(float *, float *); 
 extern float Zmulx, Zmuly;   /* Some kind of scale?                          */
 extern float Zjitx,Zjity;    /* The x,y values for jitter offset             */
-extern unsigned int Zvlnr;           /* Face rendering pointer and counter: these    */
-extern VlakRen *Zvlr;        /* are used for 'caching' render results.       */
 
 /* ------------------------------------------------------------------------- */
 
@@ -148,12 +136,12 @@ void renderEdges(char * colourRect);
 /**
  * Buffer an edge between these two vertices in the e.r. distance buffer.
  */
-void fillEdgeRenderEdge(float *vec1, float *vec2);
+static void fillEdgeRenderEdge(int, float *vec1, float *vec2);
 
 /**
  * Buffer a face between these two vertices in the e.r. distance buffer.
  */
-void fillEdgeRenderFace(float *v1, float *v2, float *v3);
+static void fillEdgeRenderFace(struct ZSpan *zspan, int, float *v1, float *v2, float *v3);
 
 /**
  * Compose the edge render colour buffer.
@@ -302,13 +290,14 @@ void renderEdges(char *colourRect)
 {
 	/* use zbuffer to define edges, add it to the image */
 	int val, y, x, col, *rz, *rz1, *rz2, *rz3;
-	char *cp;
 	int targetoffset, heightoffset;
+	int zval1, zval2, zval3;
 	int i;
 	int matdif; /* For now: just a bogus int, 0 when all materials
 		     * under the mask are the same, non-0 otherwise*/
 	int *matptr_low = 0, *matptr_cent = 0, *matptr_high = 0;
 	int matdiffac = 0;
+	char *cp;
 
 #ifdef RE_EDGERENDER_NO_CORRECTION
 	return; /* no edge correction */
@@ -371,12 +360,12 @@ void renderEdges(char *colourRect)
 		}
 	}
 		
-	/* shift values in zbuffer 3 to the right */
+	/* shift values in zbuffer 4 to the right, for filter we need multiplying with 12 max */
   	rz = edgeBuffer;
 	if(rz==0) return;
 	
 	for(y=0; y < bufHeight * bufWidth; y++, rz++) {
-		(*rz)>>= 3;
+		(*rz)>>= 4;
 	}
 	
 	/* Distance pointers */
@@ -411,10 +400,13 @@ void renderEdges(char *colourRect)
 			    matptr_cent++,
 			    matptr_high++) {
 			
-			col= abs( -   rz1[0] -  2*rz1[1] -   rz1[2]
-				  - 2*rz2[0] + 12*rz2[1] - 2*rz2[2]
-				  -   rz3[0] -  2*rz3[1] -   rz3[2]) / 3;
-
+			/* prevent overflow with sky z values */
+			zval1=   rz1[0] + 2*rz1[1] +   rz1[2];
+			zval2=  2*rz2[0]           + 2*rz2[2];
+			zval3=   rz3[0] + 2*rz3[1] +   rz3[2];
+			
+			col= abs ( 4*rz2[1] - (zval1 + zval2 + zval3)/3 );
+			
 			/* Several options for matdif:
 			 *
 			 * - suppress all boundaries with 0 dif
@@ -439,8 +431,9 @@ void renderEdges(char *colourRect)
 				
 				matdiffac = (matdif ? 0 : same_mat_redux); 
 			}
-			
-			col= ((intensity - matdiffac) * col)>>14;
+			col >>= 5;
+			if(col > (1<<16)) col= (1<<16);
+			col= ((intensity - matdiffac) * col)>>8;
 			if(col>255) col= 255;
 			
 			/* Colour edge if
@@ -580,7 +573,7 @@ void calcEdgeRenderColBuf(char* colTargetBuffer)
 
 	/* correction for osa-sampling...*/
 	if( osaCount != 1) {
-		char *rp, *rt;
+		unsigned char *rp, *rt;
 		int a;
 		
 		rt= colTargetBuffer;
@@ -598,6 +591,8 @@ void calcEdgeRenderColBuf(char* colTargetBuffer)
 /* rendering, this should be ok.                                             */
 int zBufferEdgeRenderObjects(void)
 {
+	VlakRen *vlr= NULL;
+	unsigned int zvlnr;
     int keepLooping; 
     int faceCounter; /* counter for face number */
     Material *ma;
@@ -607,30 +602,30 @@ int zBufferEdgeRenderObjects(void)
     faceCounter = 0;
 			
     while ( (faceCounter < R.totvlak) && keepLooping) {
-	    if((faceCounter & 255)==0) { Zvlr= R.blovl[faceCounter>>8]; }
-	    else Zvlr++;
+	    if((faceCounter & 255)==0) { vlr= R.blovl[faceCounter>>8]; }
+	    else vlr++;
         
-	    ma= Zvlr->mat;
+	    ma= vlr->mat;
 
 	    /*exp*/
 	    mat_cache = ma;
 
 	    /* face number is used in the fill functions */
-	    Zvlnr = faceCounter + 1; 
+	    zvlnr = faceCounter + 1; 
         
-	    if(Zvlr->flag & R_VISIBLE) {
+	    if(vlr->flag & R_VISIBLE) {
 			
 		    /* here we cull all transparent faces if mode == 0 */
 		    if (selectmode || !(ma->mode & MA_ZTRA)) {
 			    /* here we can add all kinds of extra selection criteria */
-			    if(ma->mode & (MA_WIRE)) zbufclipwire(Zvlr);
+			    if(ma->mode & (MA_WIRE)) zbufclipwire(zvlnr, vlr);
 			    else {
-				    zbufclip(Zvlr->v1->ho,   Zvlr->v2->ho,   Zvlr->v3->ho, 
-					     Zvlr->v1->clip, Zvlr->v2->clip, Zvlr->v3->clip);
-				    if(Zvlr->v4) {
-					    Zvlnr+= 0x800000; /* in a sense, the 'adjoint' face */
-					    zbufclip(Zvlr->v1->ho,   Zvlr->v3->ho,   Zvlr->v4->ho, 
-						     Zvlr->v1->clip, Zvlr->v3->clip, Zvlr->v4->clip);
+				    zbufclip(NULL, zvlnr, vlr->v1->ho,   vlr->v2->ho,   vlr->v3->ho, 
+					     vlr->v1->clip, vlr->v2->clip, vlr->v3->clip);
+				    if(vlr->v4) {
+					    zvlnr+= 0x800000; /* in a sense, the 'adjoint' face */
+					    zbufclip(NULL, zvlnr, vlr->v1->ho,   vlr->v3->ho,   vlr->v4->ho, 
+						     vlr->v1->clip, vlr->v3->clip, vlr->v4->clip);
 				    }
 			    }
 		    }
@@ -643,7 +638,7 @@ int zBufferEdgeRenderObjects(void)
 
 /* ------------------------------------------------------------------------- */
 
-void fillEdgeRenderFace(float *v1, float *v2, float *v3)  
+static void fillEdgeRenderFace(struct ZSpan *zspan, int zvlnr, float *v1, float *v2, float *v3)  
 {
 	/* Coordinates of the vertices are specified in ZCS */
 	double z0; /* used as temp var*/
@@ -851,7 +846,7 @@ void fillEdgeRenderFace(float *v1, float *v2, float *v3)
 
 /* ------------------------------------------------------------------------- */
 
-void fillEdgeRenderEdge(float *vec1, float *vec2)
+static void fillEdgeRenderEdge(int zvlnr, float *vec1, float *vec2)
 {
 	int start, end, x, y, oldx, oldy, ofs;
 	int dz, vergz/*  , mask */;
