@@ -83,7 +83,8 @@ int imagewrap(Tex *tex, Image *ima, float *texvec, TexResult *texres)
 	texres->tin= texres->ta= texres->tr= texres->tg= texres->tb= 0.0;
 
 	if(ima==NULL || ima->ok== 0) {
-		return 0;
+		if(texres->nor) return 3;
+		else return 1;
 	}
 	
 	if(ima->ibuf==NULL) {
@@ -483,6 +484,7 @@ static void boxsample(ImBuf *ibuf, float minx, float miny, float maxx, float max
      * If global variable 'imaprepeat' has been set, the
      *  clipped-away parts are sampled as well.
      */
+	/* note: actually minx etc isnt in the proper range... this due to filter size and offset vectors for bump */
 	TexResult texr;
 	rctf *rf, stack[8];
 	float opp, tot, alphaclip= 1.0;
@@ -494,7 +496,12 @@ static void boxsample(ImBuf *ibuf, float minx, float miny, float maxx, float max
 	rf->ymin= miny*(ibuf->y);
 	rf->ymax= maxy*(ibuf->y);
 
-	if(imapextend);
+	texr.talpha= texres->talpha;	/* is read by boxsample_clip */
+	
+	if(imapextend) {
+		CLAMP(rf->xmin, 0.0f, ibuf->x-1);
+		CLAMP(rf->xmax, 0.0f, ibuf->x-1);
+	}
 	else if(imaprepeat) clipx_rctf_swap(stack, &count, 0.0, (float)(ibuf->x));
 	else {
 		alphaclip= clipx_rctf(rf, 0.0, (float)(ibuf->x));
@@ -505,7 +512,10 @@ static void boxsample(ImBuf *ibuf, float minx, float miny, float maxx, float max
 		}
 	}
 
-	if(imapextend);
+	if(imapextend) {
+		CLAMP(rf->ymin, 0.0f, ibuf->y-1);
+		CLAMP(rf->ymax, 0.0f, ibuf->y-1);
+	}
 	else if(imaprepeat) clipy_rctf_swap(stack, &count, 0.0, (float)(ibuf->y));
 	else {
 		alphaclip*= clipy_rctf(rf, 0.0, (float)(ibuf->y));
@@ -544,7 +554,7 @@ static void boxsample(ImBuf *ibuf, float minx, float miny, float maxx, float max
 	if(texres->talpha==0) texres->ta= 1.0;
 	
 	if(alphaclip!=1.0) {
-		/* this is for laetr investigation, premul or not? */
+		/* this is for later investigation, premul or not? */
 		/* texres->tr*= alphaclip; */
 		/* texres->tg*= alphaclip; */
 		/* texres->tb*= alphaclip; */
@@ -581,12 +591,15 @@ int imagewraposa(Tex *tex, Image *ima, float *texvec, float *dxt, float *dyt, Te
 	ImBuf *ibuf, *previbuf;
 	float fx, fy, minx, maxx, miny, maxy, dx, dy;
 	float maxd, pixsize, val1, val2, val3;
-	int curmap;
+	int curmap, retval;
 
 	texres->tin= texres->ta= texres->tr= texres->tg= texres->tb= 0.0;
-
+	
+	/* we need to set retval OK, otherwise texture code generates normals itself... */
+	retval= texres->nor?3:1;
+	
 	if(ima==NULL || ima->ok== 0) {
-		return 0;
+		return retval;
 	}
 	
 	if(ima->ibuf==NULL) {
@@ -643,10 +656,16 @@ int imagewraposa(Tex *tex, Image *ima, float *texvec, float *dxt, float *dyt, Te
 		maxy= MAX3(dxt[1],dyt[1],dxt[1]+dyt[1] );
 
 		/* tex_sharper has been removed */
-
 		minx= tex->filtersize*(maxx-minx)/2.0f;
 		miny= tex->filtersize*(maxy-miny)/2.0f;
 		
+		if(tex->filtersize!=1.0f) {
+			dxt[0]*= tex->filtersize;
+			dxt[1]*= tex->filtersize;
+			dyt[0]*= tex->filtersize;
+			dyt[1]*= tex->filtersize;
+		}
+
 		if(tex->imaflag & TEX_IMAROT) SWAP(float, minx, miny);
 		
 		if(minx>0.25) minx= 0.25;
@@ -683,10 +702,10 @@ int imagewraposa(Tex *tex, Image *ima, float *texvec, float *dxt, float *dyt, Te
 				if(boundary==0) {
 					if( (tex->flag & TEX_CHECKER_ODD)==0) {
 						if((xs+ys) & 1); 
-						else return 0;
+						else return retval;
 					}
 					if( (tex->flag & TEX_CHECKER_EVEN)==0) {
-						if((xs+ys) & 1) return 0;
+						if((xs+ys) & 1) return retval;
 					}
 					fx-= xs;
 					fy-= ys;
@@ -720,12 +739,12 @@ int imagewraposa(Tex *tex, Image *ima, float *texvec, float *dxt, float *dyt, Te
 
 		if(tex->extend == TEX_CLIPCUBE) {
 			if(fx+minx<0.0 || fy+miny<0.0 || fx-minx>1.0 || fy-miny>1.0 || texvec[2]<-1.0 || texvec[2]>1.0) {
-				return 0;
+				return retval;
 			}
 		}
 		else if(tex->extend==TEX_CLIP || tex->extend==TEX_CHECKER) {
 			if(fx+minx<0.0 || fy+miny<0.0 || fx-minx>1.0 || fy-miny>1.0) {
-				return 0;
+				return retval;
 			}
 		}
 		else {
@@ -781,19 +800,23 @@ int imagewraposa(Tex *tex, Image *ima, float *texvec, float *dxt, float *dyt, Te
 			
 			if(texres->nor && (tex->imaflag & TEX_NORMALMAP)==0) {
 				/* a bit extra filter */
-				minx*= 1.35f;
-				miny*= 1.35f;
+				//minx*= 1.35f;
+				//miny*= 1.35f;
 				
-				boxsample(ibuf, fx-2.0f*minx, fy-2.0f*miny, fx+minx, fy+miny, texres);
+				boxsample(ibuf, fx-minx, fy-miny, fx+minx, fy+miny, texres);
 				val1= texres->tr+texres->tg+texres->tb;
-				boxsample(ibuf, fx-minx, fy-2.0f*miny, fx+2.0f*minx, fy+miny, &texr);
+				boxsample(ibuf, fx-minx+dxt[0], fy-miny+dxt[1], fx+minx+dxt[0], fy+miny+dxt[1], &texr);
 				val2= texr.tr + texr.tg + texr.tb;
-				boxsample(ibuf, fx-2.0f*minx, fy-miny, fx+minx, fy+2.0f*miny, &texr);
+				boxsample(ibuf, fx-minx+dyt[0], fy-miny+dyt[1], fx+minx+dyt[0], fy+miny+dyt[1], &texr);
 				val3= texr.tr + texr.tg + texr.tb;
 	
+				/* don't switch x or y! */
+				texres->nor[0]= (val1-val2);
+				texres->nor[1]= (val1-val3);
+				
 				if(previbuf!=ibuf) {  /* interpolate */
 					
-					boxsample(previbuf, fx-2.0f*minx, fy-2.0f*miny, fx+minx, fy+miny, &texr);
+					boxsample(previbuf, fx-minx, fy-miny, fx+minx, fy+miny, &texr);
 					
 					/* calc rgb */
 					dx= 2.0f*(pixsize-maxd)/pixsize;
@@ -810,15 +833,22 @@ int imagewraposa(Tex *tex, Image *ima, float *texvec, float *dxt, float *dyt, Te
 					}
 					
 					val1= dy*val1+ dx*(texr.tr + texr.tg + texr.tb);
-					boxsample(previbuf, fx-minx, fy-2.0f*miny, fx+2.0f*minx, fy+miny, &texr);
+					boxsample(previbuf, fx-minx+dxt[0], fy-miny+dxt[1], fx+minx+dxt[0], fy+miny+dxt[1], &texr);
 					val2= dy*val2+ dx*(texr.tr + texr.tg + texr.tb);
-					boxsample(previbuf, fx-2.0f*minx, fy-miny, fx+minx, fy+2.0f*miny, &texr);
+					boxsample(previbuf, fx-minx+dyt[0], fy-miny+dyt[1], fx+minx+dyt[0], fy+miny+dyt[1], &texr);
 					val3= dy*val3+ dx*(texr.tr + texr.tg + texr.tb);
+					
+					texres->nor[0]= (val1-val2);	/* vals have been interpolated above! */
+					texres->nor[1]= (val1-val3);
+					
+					if(dx<1.0f) {
+						dy= 1.0f-dx;
+						texres->tb= dy*texres->tb+ dx*texr.tb;
+						texres->tg= dy*texres->tg+ dx*texr.tg;
+						texres->tr= dy*texres->tr+ dx*texr.tr;
+						texres->ta= dy*texres->ta+ dx*texr.ta;
+					}
 				}
-
-				/* don't switch x or y! */
-				texres->nor[0]= (val1-val2);
-				texres->nor[1]= (val1-val3);
 			}
 			else {
 				maxx= fx+minx;
@@ -856,18 +886,15 @@ int imagewraposa(Tex *tex, Image *ima, float *texvec, float *dxt, float *dyt, Te
 			if(texres->nor && (tex->imaflag & TEX_NORMALMAP)==0) {
 				
 				/* a bit extra filter */
-				minx*= 1.35f;
-				miny*= 1.35f;
+				//minx*= 1.35f;
+				//miny*= 1.35f;
 				
-				boxsample(ibuf, fx-2.0f*minx, fy-2.0f*miny, fx+minx, fy+miny, texres);
+				boxsample(ibuf, fx-minx, fy-miny, fx+minx, fy+miny, texres);
 				val1= texres->tr+texres->tg+texres->tb;
-
-				boxsample(ibuf, fx-minx, fy-2.0f*miny, fx+2.0f*minx, fy+miny, &texr);
+				boxsample(ibuf, fx-minx+dxt[0], fy-miny+dxt[1], fx+minx+dxt[0], fy+miny+dxt[1], &texr);
 				val2= texr.tr + texr.tg + texr.tb;
-				
-				boxsample(ibuf, fx-2.0f*minx, fy-miny, fx+miny, fy+2.0f*miny, &texr);
+				boxsample(ibuf, fx-minx+dyt[0], fy-miny+dyt[1], fx+minx+dyt[0], fy+miny+dyt[1], &texr);
 				val3= texr.tr + texr.tg + texr.tb;
-
 				/* don't switch x or y! */
 				texres->nor[0]= (val1-val2);
 				texres->nor[1]= (val1-val3);
@@ -898,9 +925,7 @@ int imagewraposa(Tex *tex, Image *ima, float *texvec, float *dxt, float *dyt, Te
 	}
 	else {
 		texres->tin= 0.0f;
-		return 0;
 	}
 	
-	if(texres->nor) return 3;
-	else return 1;
+	return retval;
 }
