@@ -1,5 +1,5 @@
 /**
- * $Id: initrender.c,v 1.35 2004/12/12 18:16:35 ton Exp $
+ * $Id: initrender.c,v 1.69 2005/11/23 15:20:45 ton Exp $
  *
  * ***** BEGIN GPL/BL DUAL LICENSE BLOCK *****
  *
@@ -42,10 +42,6 @@
 #include <config.h>
 #endif
 
-#ifdef WIN32
-#include "BLI_winstuff.h"
-#endif
-
 #include "blendef.h"
 #include "MEM_guardedalloc.h"
 
@@ -70,7 +66,6 @@
 #include "BKE_image.h"
 #include "BKE_ipo.h"
 #include "BKE_key.h"
-#include "BKE_ika.h"
 #include "BKE_action.h"
 #include "BKE_writeavi.h"
 #include "BKE_scene.h"
@@ -82,21 +77,28 @@
 #include "BSE_drawview.h"
 #include "BSE_sequence.h"
 
+#include "IMB_imbuf_types.h"
+#include "IMB_imbuf.h"
+
 #ifdef WITH_QUICKTIME
 #include "quicktime_export.h"
 #endif
 
+#include "SDL_thread.h"
+
 /* this module */
 #include "render.h"
-#include "render_intern.h"
 
 #include "RE_callbacks.h"
 #include "zbuf.h"
 #include "rendercore.h" /* part handler for the old renderer, shading functions */
+#include "pixelshading.h"
 #include "renderPreAndPost.h"
 #include "vanillaRenderPipe.h"
 #include "renderHelp.h"
 #include "jitter.h"
+#include "gammaCorrectionTables.h"
+#include "zblur.h"
 
 /* Own includes */
 #include "initrender.h"
@@ -104,78 +106,73 @@
 /* yafray: include for yafray export/render */
 #include "YafRay_Api.h"
 
-/* Some crud :/ */
-#define ELEM3(a, b, c, d)       ( ELEM(a, b, c) || (a)==(d) )
 
-
-/* from render.c */
-extern float fmask[256], centLut[16], *fmask1[9], *fmask2[9];
-extern unsigned short *mask1[9], *mask2[9], *igamtab2;
-extern char cmask[256], *centmask;
+float centLut[16], *fmask1[9], *fmask2[9];
+unsigned short *gamtab, *igamtab2, *igamtab1;
+char cmask[256], *centmask;
 
 Material defmaterial;
-short pa; /* pa is global part, for print */
-short allparts[65][4];
-int qscount;
 
-/* ********************* *********************** */
+/* ------- prototypes ----------- */ 
+void init_filt_mask(void);
 
 
-void init_def_material(void)
-{
-	Material *ma;
-
-	ma= &defmaterial;
-	
-	init_material(&defmaterial);
-
-	init_render_material(ma);
-}
-
-void RE_init_render_data(void)
-{
-	memset(&R, 0, sizeof(RE_Render));
-	memset(&O, 0, sizeof(Osa));
-	O.dxwin[0]= 1.0;
-	O.dywin[1]= 1.0;
-	
-	R.blove= (VertRen **)MEM_callocN(sizeof(void *)*(TABLEINITSIZE),"Blove");
-	R.blovl= (VlakRen **)MEM_callocN(sizeof(void *)*(TABLEINITSIZE),"Blovl");
-	R.bloha= (HaloRen **)MEM_callocN(sizeof(void *)*(TABLEINITSIZE),"Bloha");
-	R.la= (LampRen **)MEM_mallocN(LAMPINITSIZE*sizeof(void *),"renderlamparray");
-
-	init_def_material();
-}
-
-void RE_free_render_data()
-{
-	MEM_freeN(R.blove);
-	R.blove= 0;
-	MEM_freeN(R.blovl);
-	R.blovl= 0;
-	MEM_freeN(R.bloha);
-	R.bloha= 0;
-	MEM_freeN(R.la);
-	R.la= 0;
-	if(R.rectot) MEM_freeN(R.rectot);
-	if(R.rectz) MEM_freeN(R.rectz);
-	if(R.rectspare) MEM_freeN(R.rectspare);
-	R.rectot= 0;
-	R.rectz= 0;
-	R.rectspare= 0;
-
-	end_render_material(&defmaterial);
-}
-
 /* ****************** GAMMA, MASKS and LUTS **************** */
 
-float  calc_weight(float *weight, int i, int j)
+static float filt_quadratic(float x)
 {
-	float x, y, dist, totw= 0.0, fac;
-	int a;
+    if (x <  0.0f) x = -x;
+    if (x < 0.5f) return 0.75f-(x*x);
+    if (x < 1.5f) return 0.50f*(x-1.5f)*(x-1.5f);
+    return 0.0f;
+}
 
-	fac= R.r.gauss*R.r.gauss;
-	fac*= fac;
+
+static float filt_cubic(float x)
+{
+	float x2= x*x;
+	
+    if (x <  0.0f) x = -x;
+	
+    if (x < 1.0f) return 0.5*x*x2 - x2 + 2.0f/3.0f;
+    if (x < 2.0f) return (2.0-x)*(2.0-x)*(2.0-x)/6.0f;
+    return 0.0f;
+}
+
+
+static float filt_catrom(float x)
+{
+	float x2= x*x;
+	
+    if (x <  0.0f) x = -x;
+    if (x < 1.0f) return  1.5f*x2*x - 2.5f*x2  + 1.0f;
+    if (x < 2.0f) return -0.5f*x2*x + 2.5*x2 - 4.0f*x + 2.0f;
+    return 0.0f;
+}
+
+static float filt_mitchell(float x)	/* Mitchell & Netravali's two-param cubic */
+{
+	float b = 1.0f/3.0f, c = 1.0f/3.0f;
+	float p0 = (  6.0 -  2.0*b         ) / 6.0;
+	float p2 = (-18.0 + 12.0*b +  6.0*c) / 6.0;
+	float p3 = ( 12.0 -  9.0*b -  6.0*c) / 6.0;
+	float q0 = (	   8.0*b + 24.0*c) / 6.0;
+	float q1 = (      - 12.0*b - 48.0*c) / 6.0;
+	float q2 = (         6.0*b + 30.0*c) / 6.0;
+	float q3 = (       -     b -  6.0*c) / 6.0;
+
+	if (x<-2.0) return 0.0;
+	if (x<-1.0) return (q0-x*(q1-x*(q2-x*q3)));
+	if (x< 0.0) return (p0+x*x*(p2-x*p3));
+	if (x< 1.0) return (p0+x*x*(p2+x*p3));
+	if (x< 2.0) return (q0+x*(q1+x*(q2+x*q3)));
+	return 0.0;
+}
+
+static float calc_weight(float *weight, int i, int j)
+{
+	float x, y, dist, totw= 0.0;
+	int a;
 
 	for(a=0; a<R.osa; a++) {
 		x= jit[a][0] + i;
@@ -184,40 +181,62 @@ float  calc_weight(float *weight, int i, int j)
 
 		weight[a]= 0.0;
 
-		/* gaussian weighting */
-		if(R.r.mode & R_GAUSS) {
-			if(dist<R.r.gauss) {
-				x = dist*R.r.gauss;
-				weight[a]= (1.0/exp(x*x) - 1.0/exp(fac));
-			}
-		}
-		else {
+		/* Weighting choices */
+		switch(R.r.filtertype) {
+		case R_FILTER_BOX:
 			if(i==0 && j==0) weight[a]= 1.0;
+			break;
+			
+		case R_FILTER_TENT:
+			if(dist < R.r.gauss)
+				weight[a]= R.r.gauss - dist;
+			break;
+			
+		case R_FILTER_GAUSS:
+			x = dist*R.r.gauss;
+			weight[a]= (1.0/exp(x*x) - 1.0/exp(R.r.gauss*R.r.gauss*2.25));
+			break;
+		
+		case R_FILTER_MITCH:
+			weight[a]= filt_mitchell(dist*R.r.gauss);
+			break;
+		
+		case R_FILTER_QUAD:
+			weight[a]= filt_quadratic(dist*R.r.gauss);
+			break;
+			
+		case R_FILTER_CUBIC:
+			weight[a]= filt_cubic(dist*R.r.gauss);
+			break;
+			
+		case R_FILTER_CATROM:
+			weight[a]= filt_catrom(dist*R.r.gauss);
+			break;
+			
 		}
-
+		
 		totw+= weight[a];
 
 	}
 	return totw;
 }
 
-void RE_init_filt_mask(void)
+// extern called in render_envmap, to disable gauss...
+// extern called in sequence.c, it needs the gamtab
+void init_filt_mask(void)
 {
 	static int firsttime=1;
 	static int lastosa=0;
-	static int lastgauss=0;
-	static float lastgamma= 0.0;
-	float gamma, igamma, flweight[32];
+	static int lastfilter= -1;
+	static float lastgamma= 0.0f, lastgaussdist=0.0f;
+	float gamma, igamma, flweight[32], fmask[256];
 	float weight[32], totw, val, *fpx1, *fpx2, *fpy1, *fpy2, *m3, *m4;
 	int i, j, a;
-	unsigned short *m1, *m2, shweight[32];
 
 	if(firsttime) {
 		firsttime= 0;
 		
 		for(a=0; a<9;a++) {
-			mask1[a]= MEM_mallocN(256*sizeof(short), "initfilt");
-			mask2[a]= MEM_mallocN(256*sizeof(short), "initfilt");
 			fmask1[a]= MEM_mallocN(256*sizeof(float), "initfilt");
 			fmask2[a]= MEM_mallocN(256*sizeof(float), "initfilt");
 		}
@@ -245,9 +264,9 @@ void RE_init_filt_mask(void)
 	}
 
 	if(R.r.alphamode==R_ALPHAKEY) gamma= 1.0;	/* gamma correction of alpha is nasty */
-
-	if(R.r.mode & R_GAMMA) gamma= 2.0;
+	else if(R.r.mode & R_GAMMA) gamma= 2.0;
 	else gamma= 1.0;
+	
 	igamma= 1.0/gamma;
 
 	if(gamma!= lastgamma) {
@@ -284,20 +303,19 @@ void RE_init_filt_mask(void)
 		}
 	}
 
-	if(R.osa && (lastosa!=R.osa || lastgauss != (R.r.mode & R_GAUSS)) ) {
+	if(R.osa && (lastosa!=R.osa || lastfilter != (R.r.filtertype) || lastgaussdist!=R.r.gauss)) {
 		lastosa= R.osa;
-		lastgauss= R.r.mode & R_GAUSS;
-			
+		lastfilter= R.r.filtertype;
+		lastgaussdist= R.r.gauss;
+		
 		val= 1.0/((float)R.osa);
 		for(a=0; a<256; a++) {
 			fmask[a]= ((float)cmask[a])*val;
 		}
 
 		for(a=0; a<9;a++) {
-			memset(mask1[a], 0, 256*2);
-			memset(mask2[a], 0, 256*2);
-			memset(fmask1[a], 0, 256*4);
-			memset(fmask2[a], 0, 256*4);
+			memset(fmask1[a], 0, 256*sizeof(float));
+			memset(fmask2[a], 0, 256*sizeof(float));
 		}
 
 		/* calculate totw */
@@ -315,60 +333,41 @@ void RE_init_filt_mask(void)
 				memset(weight, 0, sizeof(weight));
 				calc_weight(weight, i, j);
 
-				for(a=0; a<16; a++) shweight[a]= weight[a]*(65535.0/totw);
 				for(a=0; a<16; a++) flweight[a]= weight[a]*(1.0/totw);
 
-				m1= mask1[ 3*(j+1)+i+1 ];
-				m2= mask2[ 3*(j+1)+i+1 ];
 				m3= fmask1[ 3*(j+1)+i+1 ];
 				m4= fmask2[ 3*(j+1)+i+1 ];
 
 				for(a=0; a<256; a++) {
 					if(a &   1) {
-						m1[a]+= shweight[0];
-						m2[a]+= shweight[8];
 						m3[a]+= flweight[0];
 						m4[a]+= flweight[8];
 					}
 					if(a &   2) {
-						m1[a]+= shweight[1];
-						m2[a]+= shweight[9];
 						m3[a]+= flweight[1];
 						m4[a]+= flweight[9];
 					}
 					if(a &   4) {
-						m1[a]+= shweight[2];
-						m2[a]+= shweight[10];
 						m3[a]+= flweight[2];
 						m4[a]+= flweight[10];
 					}
 					if(a &   8) {
-						m1[a]+= shweight[3];
-						m2[a]+= shweight[11];
 						m3[a]+= flweight[3];
 						m4[a]+= flweight[11];
 					}
 					if(a &  16) {
-						m1[a]+= shweight[4];
-						m2[a]+= shweight[12];
 						m3[a]+= flweight[4];
 						m4[a]+= flweight[12];
 					}
 					if(a &  32) {
-						m1[a]+= shweight[5];
-						m2[a]+= shweight[13];
 						m3[a]+= flweight[5];
 						m4[a]+= flweight[13];
 					}
 					if(a &  64) {
-						m1[a]+= shweight[6];
-						m2[a]+= shweight[14];
 						m3[a]+= flweight[6];
 						m4[a]+= flweight[14];
 					}
 					if(a & 128) {
-						m1[a]+= shweight[7];
-						m2[a]+= shweight[15];
 						m3[a]+= flweight[7];
 						m4[a]+= flweight[15];
 					}
@@ -452,13 +451,11 @@ void RE_init_filt_mask(void)
 	}
 }
 
-void RE_free_filt_mask()
+static void free_filt_mask()
 {
 	int a;
 
 	for(a=0; a<9; a++) {
-		MEM_freeN(mask1[a]);
-		MEM_freeN(mask2[a]);
 		MEM_freeN(fmask1[a]);
 		MEM_freeN(fmask2[a]);
 	}
@@ -469,9 +466,9 @@ void RE_free_filt_mask()
 	MEM_freeN(centmask);
 }
 
-/* add stuff */
-
+/* unused */
 
+#if 0
 void defaultlamp()
 {
 	LampRen *lar;
@@ -489,8 +486,60 @@ void defaultlamp()
 	lar->b= 1.0;
 	lar->lay= 65535;
 }
+#endif
 
 
+
+/* ********************* init calls *********************** */
+
+
+static void init_def_material(void)
+{
+	Material *ma;
+	
+	ma= &defmaterial;
+	
+	init_material(&defmaterial);
+	
+	init_render_material(ma);
+}
+
+void RE_init_render_data(void)
+{
+	memset(&R, 0, sizeof(RE_Render));
+	
+	R.blove= (VertRen **)MEM_callocN(sizeof(void *)*(TABLEINITSIZE),"Blove");
+	R.blovl= (VlakRen **)MEM_callocN(sizeof(void *)*(TABLEINITSIZE),"Blovl");
+	R.bloha= (HaloRen **)MEM_callocN(sizeof(void *)*(TABLEINITSIZE),"Bloha");
+	R.la= (LampRen **)MEM_mallocN(LAMPINITSIZE*sizeof(void *),"renderlamparray");
+	
+	init_def_material();
+	init_filt_mask();
+}
+
+void RE_free_render_data()
+{
+	MEM_freeN(R.blove);
+	R.blove= NULL;
+	MEM_freeN(R.blovl);
+	R.blovl= NULL;
+	MEM_freeN(R.bloha);
+	R.bloha= NULL;
+	MEM_freeN(R.la);
+	R.la= NULL;
+	if(R.rectot) MEM_freeN(R.rectot);
+	if(R.rectftot) MEM_freeN(R.rectftot);
+	if(R.rectz) MEM_freeN(R.rectz);
+	if(R.rectspare) MEM_freeN(R.rectspare);
+	R.rectot= NULL;
+	R.rectftot= NULL;
+	R.rectz= NULL;
+	R.rectspare= NULL;
+	
+	end_render_material(&defmaterial);
+	free_filt_mask();
+}
+
 /* ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~ */
 
 void RE_make_existing_file(char *name)
@@ -512,11 +561,11 @@ void RE_make_existing_file(char *name)
 void RE_setwindowclip(int mode, int jmode)
 {
 	extern float bluroffsx, bluroffsy;	// rendercore.c... hackish (ton)
-	Camera *cam=0;
+	Camera *cam=NULL;
 	float lens, minx, miny, maxx, maxy;
 	float xd, yd, afmx, afmy;
 
-	if(G.scene->camera==0) return;
+	if(G.scene->camera==NULL) return;
 
 	afmx= R.afmx;
 	afmy= R.afmy;
@@ -543,19 +592,28 @@ void RE_setwindowclip(int mode, int jmode)
 			lens= 16.0;
 		}
 
-		if( (R.r.xasp*afmx) >= (R.r.yasp*afmy) ) {
-			R.viewfac= (afmx*lens)/16.0;
+		if(R.r.mode & R_ORTHO) {
+			if( (R.r.xasp*afmx) >= (R.r.yasp*afmy) ) {
+				R.viewfac= 2.0*afmx;
+			}
+			else {
+				R.viewfac= 2.0*R.ycor*afmy;
+			}
+			/* ortho_scale == 1.0 means exact 1 to 1 mapping */
+			R.pixsize= cam->ortho_scale/R.viewfac;
 		}
 		else {
-			R.viewfac= R.ycor*(afmy*lens)/16.0;
+			if( (R.r.xasp*afmx) >= (R.r.yasp*afmy) ) {
+				R.viewfac= (afmx*lens)/16.0;
+			}
+			else {
+				R.viewfac= R.ycor*(afmy*lens)/16.0;
+			}
+			
+			R.pixsize= R.near/R.viewfac;
 		}
-		if(R.r.mode & R_ORTHO) {
-			R.near*= 100.0; 
-			R.viewfac*= 100.0;
-		}
-
-		R.pixsize= R.near/R.viewfac;
-
+		
+		/* pixsize is not a real global... get rid of it! (ton) */
 	}
 
 	/* revision / simplification of subpixel offsets:
@@ -591,23 +649,52 @@ void RE_setwindowclip(int mode, int jmode)
 	miny= R.pixsize*(miny+yd);
 	maxy= R.pixsize*(maxy+yd);
 
-	if(R.r.mode & R_ORTHO) {
-		i_window(minx, maxx, miny, maxy, R.near, 100.0*R.far, R.winmat);
-	}
-	else i_window(minx, maxx, miny, maxy, R.near, R.far, R.winmat);
+	if(R.r.mode & R_ORTHO)
+		i_ortho(minx, maxx, miny, maxy, R.near, R.far, R.winmat);
+	else 
+		i_window(minx, maxx, miny, maxy, R.near, R.far, R.winmat);
 
+	//printmatrix4("win", R.winmat);
 }
 
 
 /* ~~~~~~~~~~~~~~~~ PARTS ~~~~~~~~~~~~~~~~~~~~~~ */
 
-#define PART_EMPTY	-2
-
-void initparts()
+/** 
+* Part as in part-rendering. An image rendered in parts is rendered
+* to a list of parts, with x,y size, and a pointer to the render
+* output stored per part. Internal!
+*/
+typedef struct Part
 {
+	struct Part *next, *prev;
+	unsigned int *rect;		// color 4x8 bits
+	float *rectf;			// color 4x32 bits
+	unsigned int *rectz;	// zbuffer
+
+	short minx, miny, maxx, maxy, x, y;
+} Part;
+
+static void freeparts(void)
+{
+	Part *part= R.parts.first;
+	while(part) {
+		if(part->rect) MEM_freeN(part->rect);
+		if(part->rectz) MEM_freeN(part->rectz);
+		if(part->rectf) MEM_freeN(part->rectf);
+		part= part->next;
+	}
+	BLI_freelistN(&R.parts);
+}
+
+static void initparts(void)
+{
+	Part *pa;
 	short nr, xd, yd, xpart, ypart, xparts, yparts;
 	short a, xminb, xmaxb, yminb, ymaxb;
 
+	freeparts();
+	
 	if(R.r.mode & R_BORDER) {
 		xminb= R.r.border.xmin*R.rectx;
 		xmaxb= R.r.border.xmax*R.rectx;
@@ -629,9 +716,6 @@ void initparts()
 	xparts= R.r.xparts;	/* for border */
 	yparts= R.r.yparts;
 
-	for(nr=0;nr<xparts*yparts;nr++)
-		allparts[nr][0]= PART_EMPTY;	/* clear array */
-
 	xpart= R.rectx/xparts;
 	ypart= R.recty/yparts;
 
@@ -645,82 +729,109 @@ void initparts()
 		xpart= (xmaxb-xminb)/xparts;
 		ypart= (ymaxb-yminb)/yparts;
 	}
-
+	
 	for(nr=0; nr<xparts*yparts; nr++) {
-
+		pa= MEM_callocN(sizeof(Part), "new part");
+		
 		if(R.r.mode & R_PANORAMA) {
-			allparts[nr][0]= 0;
-			allparts[nr][1]= 0;
-			allparts[nr][2]= R.rectx;
-			allparts[nr][3]= R.recty;
+			pa->minx= pa->miny= 0;
+			pa->maxx= pa->x= R.rectx;
+			pa->maxy= pa->y= R.recty;
 		}
 		else {
 			xd= (nr % xparts);
 			yd= (nr-xd)/xparts;
 
-			allparts[nr][0]= xminb+ xd*xpart;
-			allparts[nr][1]= yminb+ yd*ypart;
-			if(xd<R.r.xparts-1) allparts[nr][2]= allparts[nr][0]+xpart;
-			else allparts[nr][2]= xmaxb;
-			if(yd<R.r.yparts-1) allparts[nr][3]= allparts[nr][1]+ypart;
-			else allparts[nr][3]= ymaxb;
+			pa->minx= xminb+ xd*xpart;
+			pa->miny= yminb+ yd*ypart;
+			if(xd<R.r.xparts-1) pa->maxx= pa->minx+xpart;
+			else pa->maxx= xmaxb;
+			if(yd<R.r.yparts-1) pa->maxy= pa->miny+ypart;
+			else pa->maxy= ymaxb;
 
-			if(allparts[nr][2]-allparts[nr][0]<=0) allparts[nr][0]= PART_EMPTY;
-			if(allparts[nr][3]-allparts[nr][1]<=0) allparts[nr][0]= PART_EMPTY;
-			
-			/* gauss needs 1 pixel extra to work */
-			if(xparts*yparts>1 && (R.r.mode & R_GAUSS)) {
-				allparts[nr][0]-= 1;
-				allparts[nr][1]-= 1;
-				allparts[nr][2]+= 1;
-				allparts[nr][3]+= 1;
-			}
+			pa->x= pa->maxx-pa->minx;
+			pa->y= pa->maxy-pa->miny;
 		}
+		
+		if(pa->x>0 && pa->y>0) {
+			/* Non-box filters might need 1 pixel extra to work */
+			if((R.r.filtertype)) {
+				pa->minx-= 1;
+				pa->miny-= 1;
+				pa->maxx+= 1;
+				pa->maxy+= 1;
+				pa->x+= 2;
+				pa->y+= 2;
+			}
+			BLI_addtail(&R.parts, pa);
+		}
+		else MEM_freeN(pa);
 	}
 	
 }
 
 /* ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~ */
-short setpart(short nr)	/* return 0 if incorrect part */
+static void setpart(Part *pa)
 {
 
-	if(allparts[nr][0]== PART_EMPTY) return 0;
-
-	R.xstart= allparts[nr][0]-R.afmx;
-	R.ystart= allparts[nr][1]-R.afmy;
-	R.xend= allparts[nr][2]-R.afmx;
-	R.yend= allparts[nr][3]-R.afmy;
-	R.rectx= R.xend-R.xstart;
-	R.recty= R.yend-R.ystart;
-
-	return 1;
+	R.xstart= pa->minx-R.afmx;
+	R.ystart= pa->miny-R.afmy;
+	R.xend= pa->maxx-R.afmx;
+	R.yend= pa->maxy-R.afmy;
+	R.rectx= pa->x;
+	R.recty= pa->y;
 }
 
 /* ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~ */
-void addparttorect(short nr, Part *part)
+static void addparttorect(Part *pa)
 {
-	unsigned int *rt, *rp;
-	int y, heigth, len, copylen;
+	float *rf, *rfp;
+	unsigned int *rt, *rtp, *rz, *rzp;
+	int y, height, len, copylen;
 
-	/* the right offset in rectot */
-
-	rp= part->rect;
-	copylen=len= (allparts[nr][2]-allparts[nr][0]);
-	heigth= (allparts[nr][3]-allparts[nr][1]);
+	/* calc the right offset in rects, zbuffer cannot exist... */
+	if(pa->rect==NULL) return;
 	
-	if(R.r.mode & R_GAUSS) {
-		rp+= 1+len;
+	rtp= pa->rect;
+	rzp= pa->rectz;
+	rfp= pa->rectf;
+	
+	copylen=len= pa->x;
+	height= pa->y;
+	
+	if(R.r.filtertype) {	/* filters added 1 pixel extra */
+		
+		rtp+= 1+len;
+		if(rzp) rzp+= 1+len;
+		if(rfp) rfp+= 4*(1+len);
+		
 		copylen= len-2;
-		heigth -= 2;
-		rt= R.rectot+ (allparts[nr][1] + 1)*R.rectx+ (allparts[nr][0]+1);
+		height -= 2;
+		rt= R.rectot+ (pa->miny + 1)*R.rectx+ (pa->minx+1);
+		rz= R.rectz+ (pa->miny + 1)*R.rectx+ (pa->minx+1);
+		rf= R.rectftot+ 4*( (pa->miny + 1)*R.rectx + (pa->minx+1) );
+	}
+	else {
+		rt= R.rectot+ pa->miny*R.rectx+ pa->minx;
+		rz= R.rectz+ pa->miny*R.rectx+ pa->minx;
+		rf= R.rectftot+ 4*(pa->miny*R.rectx+ pa->minx);
 	}
-	else rt= R.rectot+ allparts[nr][1]*R.rectx+ allparts[nr][0];
 
-
-	for(y=0; y<heigth; y++) {
-		memcpy(rt, rp, 4*copylen);
+	for(y=0; y<height; y++) {
+		memcpy(rt, rtp, 4*copylen);
 		rt+= R.rectx;
-		rp+= len;
+		rtp+= len;
+		
+		if(rzp) {
+			memcpy(rz, rzp, 4*copylen);
+			rz+= R.rectx;
+			rzp+= len;
+		}
+		if(rfp) {
+			memcpy(rf, rfp, 16*copylen);
+			rf+= 4*R.rectx;
+			rfp+= 4*len;
+		}
 	}
 }
 
@@ -728,7 +839,7 @@ void addparttorect(short nr, Part *part)
 /* ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~ */
 
  
-void add_to_blurbuf(int blur)
+static void add_to_blurbuf(int blur)
 {
 	static unsigned int *blurrect= 0;
 	int tot, gamval;
@@ -792,12 +903,26 @@ void add_to_blurbuf(int blur)
 
 
 /* yafray: main yafray render/export call */
-void yafrayRender()
+static void yafrayRender(void)
 {
 	R.flag |= R_RENDERING;	/* !!! */
 
+	/* bug #3193: these params were not initialized, needed later for winmat calc. */
+	R.xstart = -R.afmx;
+	R.ystart = -R.afmy;
+	R.xend = R.xstart + R.rectx - 1;
+	R.yend = R.ystart + R.recty - 1;
+
+	/* all allocs moved here, out of export code */
+	/* display rgba buf */
+	if (R.rectot) MEM_freeN(R.rectot);
+	R.rectot = MEM_callocN(sizeof(int)*R.rectx*R.recty, "rectot");
+	/* zbuf */
 	if (R.rectz) MEM_freeN(R.rectz);
-	R.rectz = 0;
+	R.rectz = (unsigned int *)MEM_mallocN(sizeof(int)*R.rectx*R.recty, "rectz");
+	/* float rgba buf */
+	if (R.rectftot) MEM_freeN(R.rectftot);
+	if (R.r.mode & R_FBUF) R.rectftot= MEM_callocN(4*sizeof(float)*R.rectx*R.recty, "rectftot");
 
 	// switch must be done before prepareScene()
 	if (!R.r.YFexportxml)
@@ -805,54 +930,96 @@ void yafrayRender()
 	else
 		YAF_switchPlugin();
 
+	RE_local_init_render_display();
+	RE_local_clear_render_display(R.win);
+	RE_local_timecursor((G.scene->r.cfra));
+
 	printf("Starting scene conversion.\n");
 	prepareScene();
 	printf("Scene conversion done.\n");
 
 	YAF_exportScene();
 	finalizeScene();
+
+	// show postpro effects if floatbuffer used (plugin only)
+	if (R.r.YFexportxml) {
+		if ((R.r.mode & R_FBUF) && R.rectftot)
+			RE_floatbuffer_to_output();
+	}
 }
 
 
 /* ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~ */
-void render() {
-	/* yafray: render, see above */
-	if (R.r.renderer==R_YAFRAY)
-		yafrayRender();
-	else
-		oldRenderLoop();
+
+// exported to other files, belongs in include... later
+SDL_mutex *render_abuf_lock=NULL, *load_ibuf_lock=NULL;
+
+
+static void renderloop_setblending(void)
+{
+	
+	/* this value should only be set here. do_gamma is for gammablended adding of subpixels */
+	do_gamma= 0;
+	if(R.r.mode & R_GAMMA) {
+		if(R.r.alphamode==R_ALPHAKEY);	// alpha corrected gamma doesnt work for key alpha
+		else if((R.r.mode & R_OSA)) do_gamma= 1;
+	}
+	
+	/* always call, it does gamma tables used by alphaunder, but call after R.osa and jit was set */
+	init_filt_mask();
+	
+	switch (R.r.alphamode) {
+		case R_ALPHAKEY:	
+			setSkyBlendingMode(RE_ALPHA_KEY);
+			break;
+		case R_ALPHAPREMUL:	
+			setSkyBlendingMode(RE_ALPHA_PREMUL);
+			break;
+		default:
+			setSkyBlendingMode(RE_ALPHA_SKY);		
+	}
+	
+	/* SHould use slider when the gamma button is pressed. */
+	if (do_gamma) {		
+		makeGammaTables(2.0);
+	} else {
+		makeGammaTables(1.0);
+	}
+	
 }
 
-
-void oldRenderLoop(void)  /* here the PART and FIELD loops */
+static void mainRenderLoop(void)  /* here the PART and FIELD loops */
 {
-	Part *part;
-	unsigned int *rt, *rt1, *rt2;
-	int len, y;
-	short blur, a,fields,fi,parts;  /* pa is a global because of print */
-
-	if (R.rectz) MEM_freeN(R.rectz);
-	R.rectz = 0;
+	Part *pa;
+	int blur, fields, fi, totparts, nr;
 
+	/* create mutexes for threaded render */
+	render_abuf_lock = SDL_CreateMutex();
+	load_ibuf_lock = SDL_CreateMutex();
+	
+	if(R.rectz) MEM_freeN(R.rectz);
+	R.rectz = NULL;
+	if(R.rectftot) MEM_freeN(R.rectftot);
+	R.rectftot = NULL;
+	
 	/* FIELD LOOP */
+	totparts= R.r.xparts*R.r.yparts;
 	fields= 1;
-	parts= R.r.xparts*R.r.yparts;
 
 	if(R.r.mode & R_FIELDS) {
 		fields= 2;
-		R.rectf1= R.rectf2= 0;	/* field rects */
+		R.rectf1= R.rectf2= NULL;	/* field rects */
 		R.r.ysch/= 2;
 		R.afmy/= 2;
 		R.r.yasp*= 2;
 		R.ycor= ( (float)R.r.yasp)/( (float)R.r.xasp);
 
 	}
-
 	
 	for(fi=0; fi<fields; fi++) {
 
 		/* INIT */
-		BLI_srand( 2*(G.scene->r.cfra)+fi);
+		BLI_srandom( 2*(G.scene->r.cfra)+fi);
 			
 		R.flag|= R_RENDERING;
 		if(fi==1) R.flag |= R_SEC_FIELD;
@@ -870,11 +1037,16 @@ void oldRenderLoop(void)  /* here the PART and FIELD loops */
 			R.xend= R.xstart+R.rectx-1;
 			R.yend= R.ystart+R.recty-1;
 
-
 			if(R.r.mode & R_MBLUR) set_mblur_offs(R.osa-blur);
 
-			initparts(); /* always do, because of border */
-			setpart(0);
+			initparts(); /* always do, because of border and gauss */
+			if(R.parts.first==NULL) {
+				G.afbreek=1;
+				error("Image too small");
+				break;
+			}
+			
+			setpart(R.parts.first);
 
 			RE_local_init_render_display();
 			RE_local_clear_render_display(R.win);
@@ -882,20 +1054,18 @@ void oldRenderLoop(void)  /* here the PART and FIELD loops */
 
 			prepareScene();
 
-			/* PARTS */
-			R.parts.first= R.parts.last= 0;
-			for(pa=0; pa<parts; pa++) {
+			/* PARTS LOOP */
+			nr= 0;
+			for(pa= R.parts.first; pa; pa= pa->next, nr++) {
 
 				if(RE_local_test_break()) break;
 
-				if(pa) {	/* because case pa==0 has been done */
-					if(setpart(pa)==0) break;
-				}
+				setpart(pa);
 
 				if(R.r.mode & R_MBLUR) RE_setwindowclip(0, blur);
 				else RE_setwindowclip(0,-1);
 
-				if(R.r.mode & R_PANORAMA) setPanoRot(pa);
+				if(R.r.mode & R_PANORAMA) setPanoRot(nr);
 
 				/* HOMOGENIC COORDINATES AND ZBUF AND CLIP OPTIMISATION (per part) */
 				/* There may be some interference with z-coordinate    */
@@ -903,10 +1073,13 @@ void oldRenderLoop(void)  /* here the PART and FIELD loops */
 
 				doClipping(RE_projectverto);
 				if(RE_local_test_break()) break;
-
-
+				
 				/* rectot is for result and integer face indices */
-				R.rectot= (unsigned int *)MEM_callocN(sizeof(int)*R.rectx*R.recty, "rectot");
+				if(R.rectot) MEM_freeN(R.rectot);
+				R.rectot= MEM_callocN(sizeof(int)*R.rectx*R.recty, "rectot");
+				
+				if(R.rectftot) MEM_freeN(R.rectftot);
+				if(R.r.mode & R_FBUF) R.rectftot= MEM_callocN(4*sizeof(float)*R.rectx*R.recty, "rectftot");
 				
 				if(R.r.mode & R_MBLUR) {
 					RE_local_printrenderinfo(0.0, R.osa - blur);
@@ -918,6 +1091,7 @@ void oldRenderLoop(void)  /* here the PART and FIELD loops */
 					zBufShadeAdvanced();
 				}
 				else {
+					if(R.rectz) MEM_freeN(R.rectz);
 					R.rectz =  (unsigned int *)MEM_mallocN(sizeof(int)*R.rectx*R.recty, "rectz");
 
 					if(R.r.mode & R_OSA) zbufshadeDA();
@@ -928,17 +1102,14 @@ void oldRenderLoop(void)  /* here the PART and FIELD loops */
 				if( (R.r.mode & R_BORDER) && (R.r.mode & R_MOVIECROP));
 				else {
 					/* HANDLE PART OR BORDER */
-					if(parts>1 || (R.r.mode & R_BORDER)) {
+					if(totparts>1 || (R.r.mode & R_BORDER) || (R.r.filtertype)) {
 
-						part= MEM_callocN(sizeof(Part), "part");
-						BLI_addtail(&R.parts, part);
-						part->rect= R.rectot;
+						pa->rect= R.rectot;
 						R.rectot= NULL;
-
-						if (R.rectz) {
-							MEM_freeN(R.rectz);
-							R.rectz= NULL;
-						}
+						pa->rectf= R.rectftot;
+						R.rectftot= NULL;
+						pa->rectz= R.rectz;
+						R.rectz= NULL;
 					}
 				}
 
@@ -955,40 +1126,42 @@ void oldRenderLoop(void)  /* here the PART and FIELD loops */
 
 				if(R.r.mode & R_PANORAMA) R.rectx*= R.r.xparts;
 
-				if(parts>1 || (R.r.mode & R_BORDER)) {
+				if(totparts>1 || (R.r.mode & R_BORDER) || (R.r.filtertype)) {
+					int a;
+					
 					if(R.rectot) MEM_freeN(R.rectot);
+					if(R.rectftot) MEM_freeN(R.rectftot);
+					if(R.rectz) MEM_freeN(R.rectz);
 					
-					R.rectot=(unsigned int *)MEM_callocN(sizeof(int)*R.rectx*R.recty, "rectot");
+					R.rectot= MEM_callocN(sizeof(int)*R.rectx*R.recty, "rectot");
 					
-					part= R.parts.first;
-					for(pa=0; pa<parts; pa++) {
-						if(allparts[pa][0]== PART_EMPTY) break;
-						if(part==0) break;
+					if(R.r.mode & R_UNIFIED) R.rectz= NULL;
+					else R.rectz= MEM_callocN(sizeof(int)*R.rectx*R.recty, "rectz");
+
+					if(R.r.mode & R_FBUF) R.rectftot= MEM_callocN(4*sizeof(float)*R.rectx*R.recty, "rectftot");
+					else R.rectftot= NULL;
+					
+					for(a=0, pa= R.parts.first; pa; pa= pa->next, a++) {
 						
-						if(R.r.mode & R_PANORAMA) {
-							if(pa) {
-								allparts[pa][0] += pa*R.r.xsch;
-								allparts[pa][2] += pa*R.r.xsch;
-							}
+						if(R.r.mode & R_PANORAMA) {		// pano is fake parts...
+							pa->minx += a*R.r.xsch;
+							pa->maxx += a*R.r.xsch;
 						}
-						addparttorect(pa, part);
-						
-						part= part->next;
+						addparttorect(pa);
 					}
-					
-					part= R.parts.first;
-					while(part) {
-						MEM_freeN(part->rect);
-						part= part->next;
-					}
-					BLI_freelistN(&R.parts);
 				}
 			}
-
+			
+			freeparts();
+			
 			if( (R.flag & R_HALO)) {
 				if(RE_local_test_break()==0) add_halo_flare();
 			}
 
+			if( (R.r.mode & R_ZBLUR)) {
+				if(RE_local_test_break()==0) add_zblur();
+			}
+
 			if(R.r.mode & R_MBLUR) {
 				add_to_blurbuf(blur);
 			}
@@ -1006,7 +1179,7 @@ void oldRenderLoop(void)  /* here the PART and FIELD loops */
 		if(R.r.mode & R_FIELDS) {
 			if(R.flag & R_SEC_FIELD) R.rectf2= R.rectot;
 			else R.rectf1= R.rectot;
-			R.rectot= 0;
+			R.rectot= NULL;
 		}
 
 		if(RE_local_test_break()) break;
@@ -1021,8 +1194,11 @@ void oldRenderLoop(void)  /* here the PART and FIELD loops */
 
 		if(R.rectot) MEM_freeN(R.rectot);	/* happens when a render has been stopped */
 		R.rectot=(unsigned int *)MEM_callocN(sizeof(int)*R.rectx*R.recty, "rectot");
-
+		
 		if(RE_local_test_break()==0) {
+			unsigned int *rt, *rt1, *rt2;
+			int len, a;
+			
 			rt= R.rectot;
 
 			if(R.r.mode & R_ODDFIELD) {
@@ -1045,48 +1221,58 @@ void oldRenderLoop(void)  /* here the PART and FIELD loops */
 				rt2+= R.rectx;
 			}
 		}
+		
+		if(R.rectf1) MEM_freeN(R.rectf1);
+		R.rectf1= NULL;
+		if(R.rectf2) MEM_freeN(R.rectf2);
+		R.rectf2= NULL;
+		/* fbuf and zbuf free, image size differs now */
+		if(R.rectftot) MEM_freeN(R.rectftot);
+		R.rectftot= NULL;
+		if(R.rectz) MEM_freeN(R.rectz);
+		R.rectz= NULL;
+		
 	}
 
-	/* R.rectx= R.r.xsch; */
-	/* if(R.r.mode & R_PANORAMA) R.rectx*= R.r.xparts; */
-	/* R.recty= R.r.ysch; */
-
 	/* if border: still do skybuf */
 	if(R.r.mode & R_BORDER) {
 		if( (R.r.mode & R_MOVIECROP)==0) {
 			if(R.r.bufflag & 1) {
+				unsigned int *rt;
+				int x, y;
+				
 				R.xstart= -R.afmx;
 				R.ystart= -R.afmy;
 				rt= R.rectot;
-				for(y=0; y<R.recty; y++, rt+= R.rectx) scanlinesky((char *)rt, y);
+				for(y=0; y<R.recty; y++) {
+					for(x=0; x<R.rectx; x++, rt++) {
+						if(*rt==0) fillBackgroundImageChar((char *)rt, x, y);
+					}
+				}
 			}
 		}
 	}
 
 	set_mblur_offs(0);
 
-	/* FREE */
+	/* mutexes free */
+	SDL_DestroyMutex(load_ibuf_lock);
+	SDL_DestroyMutex(render_abuf_lock);
+	load_ibuf_lock= NULL;
+	render_abuf_lock= NULL;
+}
 
-	/* zbuf test */
-
-	/* don't free R.rectz, only when its size is not the same as R.rectot */
-
-	if (R.rectz && parts == 1 && (R.r.mode & R_FIELDS) == 0);
-	else {
-		if(R.rectz) MEM_freeN(R.rectz);
-		R.rectz= 0;
-	}
-
-	if(R.rectf1) MEM_freeN(R.rectf1);
-	R.rectf1= 0;
-	if(R.rectf2) MEM_freeN(R.rectf2);
-	R.rectf2= 0;
+void render() {
+	/* yafray: render, see above */
+	if (R.r.renderer==R_YAFRAY)
+		yafrayRender();
+	else
+		mainRenderLoop();
 }
 
 
-
 /* ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~ */
-extern unsigned short usegamtab;
+
 void RE_initrender(struct View3D *ogl_render_view3d)
 {
 	double start_time;
@@ -1111,10 +1297,27 @@ void RE_initrender(struct View3D *ogl_render_view3d)
 	/* IS RENDERING ALLOWED? */
 	
 	/* forbidden combination */
-	if((R.r.mode & R_BORDER) && (R.r.mode & R_PANORAMA)) {
-		error("No border supported for Panorama");
-		G.afbreek= 1;
-		return;
+	if(R.r.mode & R_PANORAMA) {
+		if(R.r.mode & R_BORDER) {
+			error("No border supported for Panorama");
+			G.afbreek= 1;
+		}
+		if(R.r.yparts>1) {
+			error("No Y-Parts supported for Panorama");
+			G.afbreek= 1;
+		}
+		if(R.r.mode & R_ORTHO) {
+			error("No Ortho render possible for Panorama");
+			G.afbreek= 1;
+		}
+	}
+
+	if(R.r.mode & R_BORDER) {
+		if(R.r.border.xmax <= R.r.border.xmin || 
+			R.r.border.ymax <= R.r.border.ymin) {
+			error("No border area selected.");
+			G.afbreek= 1;
+		}
 	}
 	
 	if(R.r.xparts*R.r.yparts>=2 && (R.r.mode & R_MOVIECROP) && (R.r.mode & R_BORDER)) {
@@ -1135,6 +1338,7 @@ void RE_initrender(struct View3D *ogl_render_view3d)
 		return;
 	}
 	
+	if(G.afbreek==1) return;
 	
 	/* TEST BACKBUF */
 	/* If an image is specified for use as backdrop, that image is loaded    */
@@ -1148,14 +1352,20 @@ void RE_initrender(struct View3D *ogl_render_view3d)
 				R.backbuf->id.us--;
 				bima= R.backbuf;
 			}
-			else bima= 0;
+			else bima= NULL;
 			
 			R.backbuf= add_image(name);
 			
 			if(bima && bima->id.us<1) {
 				free_image_buffers(bima);
 			}
-			if(R.backbuf==0) {
+
+			if(R.backbuf && R.backbuf->ibuf==NULL) {
+				R.backbuf->ibuf= IMB_loadiffname(R.backbuf->name, IB_rect);
+				if(R.backbuf->ibuf==NULL) R.backbuf->ok= 0;
+				else R.backbuf->ok= 1;
+			}
+			if(R.backbuf==NULL || R.backbuf->ok==0) {
 				// error() doesnt work with render window open
 				//error("No backbuf there!");
 				printf("Error: No backbuf %s\n", name);
@@ -1163,23 +1373,20 @@ void RE_initrender(struct View3D *ogl_render_view3d)
 		}
 	}
 	
-	usegamtab= 0; /* see also further */
-	
 	if(R.r.mode & (R_OSA|R_MBLUR)) {
 		R.osa= R.r.osa;
 		if(R.osa>16) R.osa= 16;
 		
 		init_render_jit(R.osa);
 		
-		/* this value sometimes is reset temporally, for example in transp zbuf */
-		if(R.r.mode & R_GAMMA) {
-			if((R.r.mode & R_OSA)) usegamtab= 1;
-		}
 	}
 	else R.osa= 0;
 	
-	/* always call, it does gamma tables used by alphaunder, but call after R.osa and jit was set */
-	RE_init_filt_mask();
+	/* just prevents cpu cycles for larger render and copying */
+	if((R.r.mode & R_OSA)==0)
+		R.r.filtertype= 0;
+	
+	renderloop_setblending();	// alpha, sky, gamma
 	
 	/* when rendered without camera object */
 	/* it has to done here because of envmaps */
@@ -1195,7 +1402,20 @@ void RE_initrender(struct View3D *ogl_render_view3d)
 	
 	start_time= PIL_check_seconds_timer();
 	
-	if(R.r.scemode & R_DOSEQ) {
+	if(R.r.scemode & R_OGL) {
+		R.rectx= R.r.xsch;
+		R.recty= R.r.ysch;
+		
+		if(R.rectot) MEM_freeN(R.rectot);
+		R.rectot= (unsigned int *)MEM_callocN(sizeof(int)*R.rectx*R.recty, "rectot");
+		
+		if(R.rectftot) MEM_freeN(R.rectftot);
+		R.rectftot= NULL;
+		
+		RE_local_init_render_display();
+		drawview3d_render(ogl_render_view3d);
+	}
+	else if(R.r.scemode & R_DOSEQ) {
 		R.rectx= R.r.xsch;
 		R.recty= R.r.ysch;
 		if(R.r.mode & R_PANORAMA) {
@@ -1205,24 +1425,15 @@ void RE_initrender(struct View3D *ogl_render_view3d)
 		if(R.rectot) MEM_freeN(R.rectot);
 		R.rectot= (unsigned int *)MEM_callocN(sizeof(int)*R.rectx*R.recty, "rectot");
 		
+		if(R.rectftot) MEM_freeN(R.rectftot);
+		R.rectftot= NULL;
+		
 		RE_local_timecursor((G.scene->r.cfra));
 		
 		if(RE_local_test_break()==0) do_render_seq();
 		
 		/* display */
-		if(R.rectot) RE_local_render_display(0, R.recty-1,
-											 R.rectx, R.recty,
-											 R.rectot);
-	}
-	else if(R.r.scemode & R_OGL) {
-		R.rectx= R.r.xsch;
-		R.recty= R.r.ysch;
-		
-		if(R.rectot) MEM_freeN(R.rectot);
-		R.rectot= (unsigned int *)MEM_callocN(sizeof(int)*R.rectx*R.recty, "rectot");
-		
-		RE_local_init_render_display();
-		drawview3d_render(ogl_render_view3d);
+		if(R.rectot) RE_local_render_display(0, R.recty-1, R.rectx, R.recty,R.rectot);
 	}
 	else {
 		if(G.scene->camera==0) {
@@ -1232,8 +1443,11 @@ void RE_initrender(struct View3D *ogl_render_view3d)
 		if(G.scene->camera==0) {
 			error("No camera");
 			/* needed because R.rectx and R.recty can be unmatching R.rectot */
+			
 			if(R.rectot) MEM_freeN(R.rectot);
 			R.rectot= NULL;
+			if(R.rectftot) MEM_freeN(R.rectftot);
+			R.rectftot= NULL;
 			
 			G.afbreek=1;
 			return;
@@ -1252,22 +1466,18 @@ void RE_initrender(struct View3D *ogl_render_view3d)
 	/* display again: fields/seq/parts/pano etc */
 	if(R.rectot) {
 		RE_local_init_render_display();
-		RE_local_render_display(0, R.recty-1,
-			R.rectx, R.recty,
-			R.rectot);
+		RE_local_render_display(0, R.recty-1, R.rectx, R.recty, R.rectot);
 	}
 	else RE_local_clear_render_display(R.win);
 	
 	if ((G.scene->r.scemode & R_OGL)==0) /* header gets scrabled if renderwindow holds OGL context */	
 		RE_local_printrenderinfo((PIL_check_seconds_timer() - start_time), -1);
 	
-	/* restore variables */
-	//R.osatex= 0;
-	//R.vlr= 0;	
-	///* at cubemap */
+	/* grms... this is a nasty global */
+	do_gamma= 0;
 	
 	/* these flags remain on, until reset in caller to render (renderwin.c) */
-	R.flag &= (R_RENDERING|R_ANIMRENDER);
+	R.flag &= (R_RENDERING|R_ANIMRENDER|R_REDRAW_PRV);
 }
 
 void RE_animrender(struct View3D *ogl_render_view3d)
@@ -1275,18 +1485,22 @@ void RE_animrender(struct View3D *ogl_render_view3d)
 	int cfrao;
 	char name[256];
 
-	if(G.scene==0) return;
-
+	if(G.scene==NULL) return;
+	if(G.scene->r.sfra > G.scene->r.efra) {
+		error("Startframe larger than Endframe");
+		return;
+	}
+	
 	/* scenedata to R: (for backbuf, R.rectx etc) */
 	R.r= G.scene->r;
 
 	/* START ANIMLOOP, everywhere NOT the cfra from R.r is gebruikt: because of rest blender */
 	cfrao= (G.scene->r.cfra);
 
-	if(G.scene->r.scemode & R_OGL) R.r.mode &= ~R_PANORAMA;
+	/* disable options for ogl render */
+	if(G.scene->r.scemode & R_OGL) R.r.mode &= ~(R_PANORAMA|R_MOVIECROP);
 	
-	// these calculations apply for
-	// all movie formats
+	// these calculations apply for all movie formats
 	R.rectx= (R.r.size*R.r.xsch)/100;
 	R.recty= (R.r.size*R.r.ysch)/100;
 	if(R.r.mode & R_PANORAMA) {
@@ -1295,7 +1509,7 @@ void RE_animrender(struct View3D *ogl_render_view3d)
 	}
 	if(R.r.mode & R_MOVIECROP) {
 		initparts();
-		setpart(0);		// this will adjust r.rectx
+		setpart(R.parts.first);		// this will adjust r.rectx
 	}
 
 	if (0) {
@@ -1317,7 +1531,8 @@ void RE_animrender(struct View3D *ogl_render_view3d)
 		}
 		start_avi();
 	}
-
+// set initial conditions for softbodies here
+// ******************************************
 	for((G.scene->r.cfra)=(G.scene->r.sfra); (G.scene->r.cfra)<=(G.scene->r.efra); (G.scene->r.cfra)++) {
 		double starttime= PIL_check_seconds_timer();
 
@@ -1358,14 +1573,12 @@ void RE_animrender(struct View3D *ogl_render_view3d)
 
 	}
 
-	(G.scene->r.cfra)= cfrao;
+	G.scene->r.cfra= cfrao;
 
 	/* restore time */
 	if(R.r.mode & (R_FIELDS|R_MBLUR)) {
-		do_all_ipos();
-		do_all_keys();
-		do_all_actions();
-		do_all_ikas();
+		/* applies changes fully */
+		scene_update_for_newframe(G.scene, G.scene->lay);
 	}
 
 	if (0) {
