@@ -4,7 +4,7 @@
  * Functions to blend pixels with or without alpha, in various formats
  * nzc - June 2000
  *
- * $Id: pixelblending.c,v 1.9 2004/12/03 17:03:35 ton Exp $
+ * $Id: pixelblending.c,v 1.12 2005/01/07 14:11:00 ton Exp $
  *
  * ***** BEGIN GPL/BL DUAL LICENSE BLOCK *****
  *
@@ -39,13 +39,11 @@
 
 /* global includes */
 #include "render.h"
-#include "render_intern.h"
 
 /* local includes */
 #include "vanillaRenderPipe_types.h"
 
 /* own includes */
-#include "pixelblending_types.h"
 #include "pixelblending.h"
 #include "gammaCorrectionTables.h"
 
@@ -63,6 +61,17 @@
 /* blending difficulties otherwise. Be careful here.                         */
 #define RE_ALPHA_CLIPPING
 
+
+
+/* Threshold for a 'full' pixel: pixels with alpha above this level are      */
+/* considered opaque This is the decimal value for 0xFFF0 / 0xFFFF           */
+#define RE_FULL_COLOUR_FLOAT 0.9998
+/* Threshold for an 'empty' pixel: pixels with alpha above this level are    */
+/* considered completely transparent. This is the decimal value              */
+/* for 0x000F / 0xFFFF                                                       */
+#define RE_EMPTY_COLOUR_FLOAT 0.0002
+
+
 /* functions --------------------------------------------------------------- */
 
 /*
@@ -103,7 +112,7 @@ void applyKeyAlphaCharCol(char* target) {
 		}
 	}
 
-} /* end of void applyKeyAlphaCharCol(char* target) */
+}
 
 /* ------------------------------------------------------------------------- */
 
@@ -116,7 +125,7 @@ void addAddSampColF(float *sampvec, float *source, int mask, int osaNr,
 		if(mask & (1<<a)) addalphaAddfacFloat(sampvec, source, addfac);
 		sampvec+= 4;
 	}
-} /* end of void addAddSampColF(float, float, int, int) */
+}
 
 /* ------------------------------------------------------------------------- */
 
@@ -128,7 +137,7 @@ void addOverSampColF(float *sampvec, float *source, int mask, int osaNr)
 		if(mask & (1<<a)) addAlphaOverFloat(sampvec, source);
 		sampvec+= 4;
 	}
-} /* end of void addOverSampColF(float, float, int, int) */
+}
 
 /* ------------------------------------------------------------------------- */
 
@@ -142,99 +151,9 @@ int addUnderSampColF(float *sampvec, float *source, int mask, int osaNr)
 		sampvec+= 4;
 	}
 	return retval;
-} /* end of int addToSampColF(float, float, int, int) */
+}
 
-/* ------------------------------------------------------------------------- */
 
-int addToSampCol(unsigned short *sampcol, unsigned short *shortcol, int mask, int osaNr)
-{
-	int a, retval = osaNr;
-	
-	for(a=0; a < osaNr; a++) {
-		if(mask & (1<<a)) addAlphaUnderShort(sampcol, shortcol);
-		if(sampcol[3]>0xFFF0) retval--;
-		sampcol+= 4;
-	}
-	return retval;
-} /* end of int addToSampCol(unsigned short, uhost, int, int) */
-
-/* ------------------------------------------------------------------------- */
-
-int addtosampcol(unsigned short *sampcol, unsigned short *shortcol, int mask)
-{
-	int a, retval = R.osa;
-	
-	for(a=0; a < R.osa; a++) {
-		if(mask & (1<<a)) addAlphaUnderShort(sampcol, shortcol);
-		if(sampcol[3]>0xFFF0) retval--;
-		sampcol+= 4;
-	}
-	return retval;
-} /* end of int addtosampcol(unsigned short *sampcol, unsigned short *shortcol, int mask) */
-
-/* ------------------------------------------------------------------------- */
-
-void addAlphaOverShort(unsigned short *doel, unsigned short *bron) 
-/* fills in bron (source) over doel (target) with alpha from bron */
-{
-	unsigned int c;
-	unsigned int mul;
-
-	if( doel[3]==0 || bron[3]>=0xFFF0) {	/* has been tested */
-		*((unsigned int *)doel)= *((unsigned int *)bron);
-		*((unsigned int *)(doel+2))= *((unsigned int *)(bron+2));
-		return;
-	}
-
-	mul= 0xFFFF-bron[3];
-
-	c= ((mul*doel[0])>>16)+bron[0];
-	if(c>=0xFFF0) doel[0]=0xFFF0; 
-	else doel[0]= c;
-	c= ((mul*doel[1])>>16)+bron[1];
-	if(c>=0xFFF0) doel[1]=0xFFF0; 
-	else doel[1]= c;
-	c= ((mul*doel[2])>>16)+bron[2];
-	if(c>=0xFFF0) doel[2]=0xFFF0; 
-	else doel[2]= c;
-	c= ((mul*doel[3])>>16)+bron[3];
-	if(c>=0xFFF0) doel[3]=0xFFF0; 
-	else doel[3]= c;
-
-} /* end of void addAlphaOverShort(unsigned short *doel, unsigned short *bron) */
-
-/* ------------------------------------------------------------------------- */
-
-void addAlphaUnderShort(unsigned short *doel, unsigned short *bron)   
-/* fills in bron (source) under doel (target) using alpha from doel */
-{
-	unsigned int c;
-	unsigned int mul;
-
-	if(doel[3]>=0xFFF0) return;
-	if( doel[3]==0 ) {	/* was tested */
-		*((unsigned int *)doel)= *((unsigned int *)bron);
-		*((unsigned int *)(doel+2))= *((unsigned int *)(bron+2));
-		return;
-	}
-
-	mul= 0xFFFF-doel[3];
-
-	c= ((mul*bron[0])>>16)+doel[0];
-	if(c>=0xFFF0) doel[0]=0xFFF0; 
-	else doel[0]= c;
-	c= ((mul*bron[1])>>16)+doel[1];
-	if(c>=0xFFF0) doel[1]=0xFFF0; 
-	else doel[1]= c;
-	c= ((mul*bron[2])>>16)+doel[2];
-	if(c>=0xFFF0) doel[2]=0xFFF0;
-	else doel[2]= c;
-	c= ((mul*bron[3])>>16)+doel[3];
-	if(c>=0xFFF0) doel[3]=0xFFF0;
-	else doel[3]= c;
-
-} /* end of void addAlphaUnderShort(unsigned short *doel, unsigned short *bron) */
-  
 /* ------------------------------------------------------------------------- */
 
 void addAlphaOverFloat(float *dest, float *source)
@@ -257,8 +176,7 @@ void addAlphaOverFloat(float *dest, float *source)
 	c= (mul*dest[3]) + source[3];
        dest[3]= c;
 
-} /* end of void addAlphaOverFloat(float *doel, float *bron) */
-
+}
 
 
 /* ------------------------------------------------------------------------- */
@@ -291,7 +209,7 @@ void addAlphaUnderFloat(float *dest, float *source)
 	c= (mul*source[3]) + dest[3];
        dest[3]= c;
 
-} /* end of void addAlphaUnderFloat(float *doel, float *bron) */
+} 
 
 /* ------------------------------------------------------------------------- */
 
@@ -301,7 +219,7 @@ void cpShortColV2CharColV(unsigned short *source, char *dest)
     dest[1] = source[1]>>8;
     dest[2] = source[2]>>8;
     dest[3] = source[3]>>8;
-} /* end of void cpShortColV2CharColV(unsigned short *source, char *dest) */
+} 
 /* ------------------------------------------------------------------------- */
 
 void cpCharColV2ShortColV(char *source, unsigned short *dest)
@@ -310,7 +228,7 @@ void cpCharColV2ShortColV(char *source, unsigned short *dest)
     dest[1] = source[1]<<8;
     dest[2] = source[2]<<8;
     dest[3] = source[3]<<8;
-} /* end of void cpShortColV2CharColV(char *source, unsigned short *dest) */
+} 
 
 /* ------------------------------------------------------------------------- */
 
@@ -320,7 +238,7 @@ void cpIntColV2CharColV(unsigned int *source, char *dest)
     dest[1] = source[1]>>24;
     dest[2] = source[2]>>24;
     dest[3] = source[3]>>24;
-} /* end of void cpIntColV2CharColV(unsigned int *source, char *dest) */
+} 
 
 /* ------------------------------------------------------------------------- */
 
@@ -330,7 +248,8 @@ void cpCharColV2FloatColV(char *source, float *dest)
     dest[1] = source[1]/255.0;  
     dest[2] = source[2]/255.0;
     dest[3] = source[3]/255.0;
-} /* end of void cpCharColV2FloatColV(char *source, float *dest) */
+} 
+
 /* ------------------------------------------------------------------------- */
 
 void cpShortColV2FloatColV(unsigned short *source, float *dest)
@@ -339,7 +258,7 @@ void cpShortColV2FloatColV(unsigned short *source, float *dest)
     dest[1] = source[1]/65535.0;  
     dest[2] = source[2]/65535.0;
     dest[3] = source[3]/65535.0;
-} /* end of void cpShortColV2FloatColV(char *source, float *dest) */
+} 
 
 /* ------------------------------------------------------------------------- */
 
@@ -362,7 +281,7 @@ void cpFloatColV2CharColV(float* source, char *dest)
   else if (source[3] > 1.0) dest[3] = 255;
   else dest[3] = (char) (source[3] * 255.0);
 
-} /* end of void cpFloatColV2CharColV(float* source, char *dest) */
+} 
 
 /* ------------------------------------------------------------------------- */
 
@@ -372,7 +291,7 @@ void cpShortColV(unsigned short *source, unsigned short *dest)
     dest[1] = source[1];
     dest[2] = source[2];
     dest[3] = source[3];
-} /* end of void cpShortColV(unsigned short *source, unsigned short *dest) */
+}
 
 /* ------------------------------------------------------------------------- */
 void cpFloatColV(float *source, float *dest)
@@ -381,7 +300,7 @@ void cpFloatColV(float *source, float *dest)
     dest[1] = source[1];
     dest[2] = source[2];
     dest[3] = source[3];
-} /* end of void cpFloatColV(float *source, float *dest) */
+}
 
 /* ------------------------------------------------------------------------- */
 
@@ -391,7 +310,7 @@ void cpCharColV(char *source, char *dest)
     dest[1] = source[1];
     dest[2] = source[2];
     dest[3] = source[3];
-} /* end of void cpCharColV(char *source, char *dest) */
+}
 
 /* ------------------------------------------------------------------------- */
 void addalphaAddfacFloat(float *dest, float *source, char addfac)
@@ -425,102 +344,14 @@ void addalphaAddfacFloat(float *dest, float *source, char addfac)
 #endif
         dest[2]= c;
 
-	c= dest[3] + source[3];
+	c= (m * dest[3]) + source[3];
 #ifdef RE_ALPHA_CLIPPING
 	if(c >= RE_FULL_COLOUR_FLOAT) dest[3] = RE_FULL_COLOUR_FLOAT; 
 	else 
 #endif
        dest[3]= c;
 
-} /* end of void addalphaAddfacFloat(unsigned short *doel, unsigned short *bron, char addfac_help) */
-
-/* ------------------------------------------------------------------------- */
-
-void addalphaAddfacShort(unsigned short *doel, unsigned short *bron, char addfac)
-  /* doel= bron over doel  */
-{
-    float m; /* weiging factor of destination */
-    float c; /* intermediate colour           */
-
-    /* 1. copy bron straight away if doel has zero alpha */
-    if( doel[3] == 0) {
-        *((unsigned int *)doel)     = *((unsigned int *)bron);
-        *((unsigned int *)(doel+2)) = *((unsigned int *)(bron+2));
-        return;
-    }
-    
-    /* Addfac is a number between 0 and 1: rescale */
-    /* final target is to diminish the influence of dest when addfac rises */
-    m = 1.0 - ( bron[3] * ((255.0 - addfac) / 255.0));
-
-    /* blend colours*/
-    c = (m * doel[0]) + bron[0];
-    if( c > 65535.0 ) doel[0]=65535; 
-    else doel[0] = floor(c);
-    c = (m * doel[1]) + bron[1];
-    if( c > 65535.0 ) doel[1]=65535; 
-    else doel[1] = floor(c);
-    c = (m * doel[2]) + bron[2];
-    if( c > 65535.0 ) doel[2]=65535; 
-    else doel[2] = floor(c);
-
-    c = doel[3] + bron[3];
-    if(c > 65535.0) doel[3] = 65535; 
-    else doel[3]=  floor(c);
-
-} /* end of void addalphaAddfacShort(unsigned short *doel, unsigned short *bron, char addfac_help) */
-
-/* ------------------------------------------------------------------------- */
-
-void addHaloToHaloShort(unsigned short *d, unsigned short *s)
-{
-    /*  float m; */ /* weiging factor of destination */
-    float c[4]; /* intermediate colour           */
-    float rescale = 1.0;
-
-    /* 1. copy <s> straight away if <d> has zero alpha */
-    if( d[3] == 0) {
-        *((unsigned int *) d)      = *((unsigned int *) s);
-        *((unsigned int *)(d + 2)) = *((unsigned int *)(s + 2));
-        return;
-    }
-
-    /* 2. halo blending  */
-    /* no blending, just add */
-    c[0] = s[0] + d[0];
-    c[1] = s[1] + d[1];
-    c[2] = s[2] + d[2];
-    c[3] = s[3] + d[3];
-    /* One thing that may happen is that this pixel is over-saturated with light - */
-    /* i.e. too much light comes out, and the pixel is clipped. Currently, this    */
-    /* leads to artifacts such as overproportional undersampling of background     */
-    /* colours.                                                                    */
-    /* Compensating for over-saturation:                                           */
-    /* - increase alpha                                                            */
-    /* - increase alpha and rescale colours                                        */
-
-    /* let's try alpha increase and clipping */
-
-    /* calculate how much rescaling we need */
-    if( c[0] > 65535.0 ) { 
-      rescale *= c[0] /65535.0;
-      d[0] = 65535; 
-    } else d[0] = floor(c[0]);
-    if( c[1] > 65535.0 ) { 
-      rescale *= c[1] /65535.0;
-      d[1] = 65535; 
-    } else d[1] = floor(c[1]);
-    if( c[2] > 65535.0 ) { 
-      rescale *= c[2] /65535.0;
-      d[2] = 65535; 
-    } else d[2] = floor(c[2]);
-
-    /* a bit too hefty I think */
-    c[3] *= rescale;
-
-    if( c[3] > 65535.0 ) d[3] = 65535; else d[3]=  floor(c[3]);
-
-} /* end of void addHaloToHaloShort(unsigned short *dest, unsigned short *source, char addfac) */
+}
 
 /* ------------------------------------------------------------------------- */
 
@@ -542,17 +373,15 @@ void sampleShortColV2ShortColV(unsigned short *sample, unsigned short *dest, int
     dest[2]= intcol[2]/osaNr;
     dest[3]= intcol[3]/osaNr;
     
-} /* end of void sampleShortColVToShortColV(unsigned short *sample, unsigned short *dest) */
-
+}
 
 /* ------------------------------------------------------------------------- */
 
-float *fmask1[9], *fmask2[9];
-
 /* filtered adding to scanlines */
 void add_filt_fmask(unsigned int mask, float *col, float *rb1, float *rb2, float *rb3)
 {
 	/* calc the value of mask */
+	extern float *fmask1[], *fmask2[];
 	float val, r, g, b, al;
 	unsigned int a, maskand, maskshift;
 	int j;
@@ -615,7 +444,7 @@ void sampleFloatColV2FloatColVFilter(float *sample, float *dest1, float *dest2,
 		dest2[7]= sample[3];
 	}
 	else {
-		if (doGamma()) {
+		if (do_gamma) {
 			/* use a LUT and interpolation to do the gamma correction */
 			for(a=0; a < osaNr; a++, scol+=4) {
 				intcol[0] = gammaCorrect( (scol[0]<1.0) ? scol[0]:1.0 ); 
@@ -632,7 +461,7 @@ void sampleFloatColV2FloatColVFilter(float *sample, float *dest1, float *dest2,
 		}
 	}
 	
-} /* end void sampleFloatColVToFloatColV(unsigned short *, unsigned short *) */
+} 
 
 /* ------------------------------------------------------------------------- */
 /* The following functions are 'old' blending functions:                     */
@@ -780,29 +609,6 @@ void addalphaAdd(char *doel, char *bron)   /* adds bron (source) to doel (target
 	if(c>255) doel[3]=255; 
 	else doel[3]= c;
 }
-/* ------------------------------------------------------------------------- */
-void addalphaAddshort(unsigned short *doel, unsigned short *bron)   /* adds bron (source) to doel (target) */
-{
-	int c;
-
-	if( doel[3]==0) {
-		*((unsigned int *)doel)= *((unsigned int *)bron);
-		*((unsigned int *)(doel+2))= *((unsigned int *)(bron+2));
-		return;
-	}
-	c= doel[0]+bron[0];
-	if(c>65535) doel[0]=65535; 
-	else doel[0]= c;
-	c= doel[1]+bron[1];
-	if(c>65535) doel[1]=65535; 
-	else doel[1]= c;
-	c= doel[2]+bron[2];
-	if(c>65535) doel[2]=65535; 
-	else doel[2]= c;
-	c= doel[3]+bron[3];
-	if(c>65535) doel[3]=65535; 
-	else doel[3]= c;
-}
 
 /* ------------------------------------------------------------------------- */
 void addalphaAddFloat(float *dest, float *source)
@@ -861,30 +667,6 @@ void RE_addalphaAddfac(char *doel, char *bron, char addfac)
 	else doel[3]= c;
 }
 
-/* ------------------------------------------------------------------------- */
-/* doel= bron over doel  */
-void addalphaAddfacshort(unsigned short *doel,
-						 unsigned short *bron,
-						 short addfac)    
-{
-	int c, mul;
-
-	mul= 0xFFFF - (bron[0]*(255-addfac))/255;
-	
-	c= ((mul*doel[0])>>16)+bron[0];
-	if(c>=0xFFF0) doel[0]=0xFFF0; 
-	else doel[0]= c;
-	c= ((mul*doel[1])>>16)+bron[1];
-	if(c>=0xFFF0) doel[1]=0xFFF0; 
-	else doel[1]= c;
-	c= ((mul*doel[2])>>16)+bron[2];
-	if(c>=0xFFF0) doel[2]=0xFFF0; 
-	else doel[2]= c;
-	c= ((mul*doel[3])>>16)+bron[3];
-	if(c>=0xFFF0) doel[3]=0xFFF0; 
-	else doel[3]= c;
-
-}
 
 /* ------------------------------------------------------------------------- */
 
