@@ -31,7 +31,7 @@
  *
  * 28-06-2000 nzc
  *
- * $Id: vanillaRenderPipe.c,v 1.21 2005/06/08 12:51:03 ton Exp $
+ * $Id: vanillaRenderPipe.c,v 1.25 2005/12/13 13:17:29 ton Exp $
  *
  */
 
@@ -194,7 +194,7 @@ static void freeRenderBuffers(void) {
 /**
  * New fill function for z buffer, for edge-only rendering.
  */
-static void zBufferFillFace(unsigned int zvlnr, float *v1, float *v2, float *v3)  
+static void zBufferFillFace(ZSpan *zspan, int zvlnr, float *v1, float *v2, float *v3)  
 {
 	/* Coordinates of the vertices are specified in ZCS */
 	VlakRen *vlr;
@@ -409,7 +409,7 @@ static void zBufferFillFace(unsigned int zvlnr, float *v1, float *v2, float *v3)
 } 
 /* ------------------------------------------------------------------------- */
 
-static void zBufferFillEdge(unsigned int zvlnr, float *vec1, float *vec2)
+static void zBufferFillEdge(int zvlnr, float *vec1, float *vec2)
 {
 	int apteller;
 	int start, end, x, y, oldx, oldy, ofs;
@@ -916,7 +916,7 @@ static void integratePerSubStack(float *sampcol, struct RE_faceField* stack,
 		subStackPtr = subStackSize - 1;
 		while (subStackPtr >= 0) {
 			/* we can make a general meachanism here for operations */
-			if (stack[subStack[subStackPtr]].faceType == RE_POLY){ 
+			if (stack[subStack[subStackPtr]].faceType & RE_POLY){ 
 				VlakRen* vlr = (VlakRen*) stack[subStack[subStackPtr]].data;
 				if (vlr->mat) {
 					/* ENV faces */
@@ -1065,9 +1065,6 @@ static void renderZBufLine(int y, RE_COLBUFTYPE *colbuf1, RE_COLBUFTYPE *colbuf2
 				j[2]= collector[2]; j[3]= collector[3];
 			}
 			
-			sampleFloatColV2FloatColVFilter(sampcol, colbuf1, colbuf2, colbuf3, osaNr);
-			
-			
 			/* Spothalos are part of the normal pixelshader, so for covered  */
 			/* pixels they are handled ok. They are 'normally' alpha blended */
 			/* onto the existing colour in the collector.                    */
@@ -1078,8 +1075,15 @@ static void renderZBufLine(int y, RE_COLBUFTYPE *colbuf1, RE_COLBUFTYPE *colbuf2
 					collector[1]= gammaCorrect(collector[1]);
 					collector[2]= gammaCorrect(collector[2]);
 				}
-				addAlphaOverFloat(colbuf2+4, collector);
+				
+				j = sampcol;
+				for(i = 0; i < osaNr; i++, j+=4) { 
+					addAlphaOverFloat(j, collector);
+				}
 			}
+			
+			sampleFloatColV2FloatColVFilter(sampcol, colbuf1, colbuf2, colbuf3, osaNr);
+			
 		}
     } 
 } 
@@ -1174,11 +1178,11 @@ static int zBufferAllFaces(void)
 			
 			if(ma->mode & (MA_WIRE)) zbufclipwire(zvlnr, vlr);
 			else {
-				zbufclip(zvlnr, vlr->v1->ho,   vlr->v2->ho,   vlr->v3->ho, 
+				zbufclip(NULL, zvlnr, vlr->v1->ho,   vlr->v2->ho,   vlr->v3->ho, 
 						 vlr->v1->clip, vlr->v2->clip, vlr->v3->clip);
 				if(vlr->v4) {
 					zvlnr+= 0x800000; /* in a sense, the 'adjoint' face */
-					zbufclip(zvlnr, vlr->v1->ho,   vlr->v3->ho,   vlr->v4->ho, 
+					zbufclip(NULL, zvlnr, vlr->v1->ho,   vlr->v3->ho,   vlr->v4->ho, 
 							 vlr->v1->clip, vlr->v3->clip, vlr->v4->clip);
 				}
 			}
@@ -1457,9 +1461,6 @@ static void eraseColBuf(RE_COLBUFTYPE *buf)
  */
 static void calcZBufLine(int y)
 {
-	/* These function pointers are used for z buffer filling.    */
-	extern void (*zbuffunc)(unsigned int, float *, float *, float *);
-	extern void (*zbuflinefunc)(unsigned int, float *, float *);
 	int part;
     int keepLooping = 1;
 	
@@ -1579,7 +1580,7 @@ void zBufShadeAdvanced()
 				
 				SDL_WaitThread(thread, NULL);
 				
-				if(R.r.mode & R_GAUSS) {
+				if(R.r.filtertype) {
 					float *rb1= AColourBuffer1, *rb2= AColourBuffer2, *rb1a= AColourBuffer1a, *rb2a= AColourBuffer2a;
 					int a= 4*(R.rectx + 4);
 					while(a--) {
