@@ -1,7 +1,5 @@
-
-#include <string.h>
 /**
- * $Id: booleanops.c,v 1.10 2004/09/19 11:47:48 ton Exp $
+ * $Id: booleanops.c,v 1.24 2006/01/01 15:41:20 khughes Exp $
  *
  * ***** BEGIN GPL/BL DUAL LICENSE BLOCK *****
  *
@@ -33,6 +31,8 @@
  * CSG operations. 
  */
 
+#include <string.h>
+
 #include "MEM_guardedalloc.h"
 
 #include "BLI_ghash.h"
@@ -45,14 +45,15 @@
 
 #include "CSG_BooleanOps.h"
 
-#include "BKE_global.h"
-#include "BKE_mesh.h"
-#include "BKE_displist.h"
-#include "BKE_object.h"
 #include "BKE_booleanops.h"
-#include "BKE_utildefines.h"
+#include "BKE_depsgraph.h"
+#include "BKE_displist.h"
+#include "BKE_global.h"
 #include "BKE_library.h"
 #include "BKE_material.h"
+#include "BKE_mesh.h"
+#include "BKE_object.h"
+#include "BKE_utildefines.h"
 
 #include <math.h>
 
@@ -63,14 +64,18 @@
 #include "BLI_linklist.h"
 #include "BLI_memarena.h"
 
+static void ConvertCSGDescriptorsToDLM(
+	DispListMesh *dlm,
+	Object *ob,
+	CSG_MeshPropertyDescriptor *props,
+	CSG_FaceIteratorDescriptor *face_it,
+	CSG_VertexIteratorDescriptor *vertex_it,
+	float parinv[][4]);
 
 #ifdef HAVE_CONFIG_H
 #include <config.h>
 #endif
 
-/** check if passed mesh has faces, return zero if only edges, 1 if faces have been found */
-int has_faces(Mesh *me);
-
 /**
  * Here's the vertex iterator structure used to walk through
  * the blender vertex structure.
@@ -235,18 +240,16 @@ FaceIt_Fill(
 	MFace *mface = &mfaces[f_index];
 	FaceData *fdata = face->user_face_data;
 	
-	if (mface->v3) {
-		// ignore lines (faces with mface->v3==0)
-		face->vertex_index[0] = mface->v1;
-		face->vertex_index[1] = mface->v2;
-		face->vertex_index[2] = mface->v3;
-		if (mface->v4) {
-			face->vertex_index[3] = mface->v4;
-			face->vertex_number = 4;
-		} else {
-			face->vertex_number = 3;
-		}
-	}	
+	// ignore lines (faces with mface->v3==0)
+	face->vertex_index[0] = mface->v1;
+	face->vertex_index[1] = mface->v2;
+	face->vertex_index[2] = mface->v3;
+	if (mface->v4) {
+		face->vertex_index[3] = mface->v4;
+		face->vertex_number = 4;
+	} else {
+		face->vertex_number = 3;
+	}
 
 	fdata->faceflag = mface->flag;
 	fdata->material = give_current_material(ob, mface->mat_nr+1);
@@ -363,22 +366,208 @@ InterpFaceVertexData(
 	return 0;
 }
 
-int has_faces(Mesh *me)
-{
-	MFace *mface;
-	int a;
-
-	mface= me->mface;
-	for(a=0; a<me->totface; a++, mface++) {		
-		if(mface->v3) return 1;
-	}
-	return 0;
-}
-
 /**
  * Assumes mesh is valid and forms part of a fresh
  * blender object.
  */
+DispListMesh *NewBooleanMeshDLM(Object *ob, Object *ob_select, int int_op_type)
+{
+	Mesh *me2 = get_mesh(ob_select);
+	Mesh *me = get_mesh(ob);
+	int free_tface1,free_tface2;
+	DispListMesh *dlm = NULL;
+
+	float inv_mat[4][4];
+	int success = 0;
+	// build and fill new descriptors for these meshes
+	CSG_VertexIteratorDescriptor vd_1;
+	CSG_VertexIteratorDescriptor vd_2;
+	CSG_FaceIteratorDescriptor fd_1;
+	CSG_FaceIteratorDescriptor fd_2;
+
+	CSG_MeshPropertyDescriptor mpd1,mpd2;
+
+	// work out the operation they chose and pick the appropriate 
+	// enum from the csg module.
+
+	CSG_OperationType op_type;
+
+	if (me == NULL || me2 == NULL) return 0;
+
+	if(!me->totface || !me2->totface) return 0;
+	
+	success = 0;
+
+	switch (int_op_type) {
+		case 1 : op_type = e_csg_intersection; break;
+		case 2 : op_type = e_csg_union; break;
+		case 3 : op_type = e_csg_difference; break;
+		case 4 : op_type = e_csg_classify; break;
+		default : op_type = e_csg_intersection;
+	}
+
+	// Here is the section where we describe the properties of
+	// both meshes to the bsp module.
+
+	if (me->mcol != NULL) {
+		// Then this mesh has vertex colors only 
+		// well this is awkward because there is no equivalent 
+		// test_index_mface just for vertex colors!
+		// as a temporary hack we can convert these vertex colors 
+		// into tfaces do the operation and turn them back again.
+
+		// create some memory for the tfaces.
+		me->tface = (TFace *)MEM_callocN(sizeof(TFace)*me->totface,"BooleanOps_TempTFace");
+		mcol_to_tface(me,1);
+		free_tface1 = 1;
+	} else {
+		free_tface1 = 0;
+	}
+
+	mpd1.user_face_vertex_data_size = 0;
+	mpd1.user_data_size = sizeof(FaceData);
+
+	if (me->tface) {
+		mpd1.user_face_vertex_data_size = sizeof(FaceVertexData);
+	}
+	
+	// same for mesh2
+
+	if (me2->mcol != NULL) {
+		// create some memory for the tfaces.
+		me2->tface = (TFace *)MEM_callocN(sizeof(TFace)*me2->totface,"BooleanOps_TempTFace");
+		mcol_to_tface(me2,1);
+		free_tface2 = 1;
+	} else {
+		free_tface2 = 0;
+	}
+
+	mpd2.user_face_vertex_data_size = 0;
+	mpd2.user_data_size = sizeof(FaceData);
+
+	if (me2->tface) {
+		mpd2.user_face_vertex_data_size = sizeof(FaceVertexData);
+	}
+
+	// we map the final object back into object 1's (ob)
+	// local coordinate space. For this we need to compute
+	// the inverse transform from global to local.	
+	
+	Mat4Invert(inv_mat,ob_select->obmat);
+
+	// make a boolean operation;
+	{
+		CSG_BooleanOperation * bool_op = CSG_NewBooleanFunction();
+		CSG_MeshPropertyDescriptor output_mpd = CSG_DescibeOperands(bool_op,mpd1,mpd2);
+		// analyse the result and choose mesh descriptors accordingly
+		int output_type;			
+		if (output_mpd.user_face_vertex_data_size) {
+			output_type = 1;
+		} else {
+			output_type = 0;
+		}
+		
+		BuildMeshDescriptors(
+			ob,
+			&fd_1,
+			&vd_1
+		);
+		BuildMeshDescriptors(
+			ob_select,
+			&fd_2,
+			&vd_2
+		);
+
+		// perform the operation
+
+		if (output_type == 0) {
+
+			success = 
+			CSG_PerformBooleanOperation(
+				bool_op,
+				op_type,
+				fd_1,vd_1,fd_2,vd_2,
+				InterpNoUserData	
+			);
+		} else {
+			success = 
+			CSG_PerformBooleanOperation(
+				bool_op,
+				op_type,
+				fd_1,vd_1,fd_2,vd_2,
+				InterpFaceVertexData	
+			);
+		}
+
+		switch( success ) {
+		case 1:
+			{
+				// descriptions of the output;
+				CSG_VertexIteratorDescriptor vd_o;
+				CSG_FaceIteratorDescriptor fd_o;
+				
+				dlm = MEM_callocN(sizeof(*dlm),"dlm");
+
+				CSG_OutputFaceDescriptor(bool_op,&fd_o);
+				CSG_OutputVertexDescriptor(bool_op,&vd_o);
+
+				// iterate through results of operation and insert into new object
+
+				ConvertCSGDescriptorsToDLM(
+					dlm,
+					NULL,
+					&output_mpd,
+					&fd_o,
+					&vd_o,
+					inv_mat
+				);
+
+				// free up the memory
+
+				CSG_FreeVertexDescriptor(&vd_o);
+				CSG_FreeFaceDescriptor(&fd_o);
+			}
+			break;
+		case -1:
+			 error("Selected meshes must have faces to perform boolean operations");
+			 break;
+		case -2:
+			 error("Both meshes must be closed");
+			 break;
+		default:
+			 error("unknown internal error");
+			 break;
+		}
+
+		CSG_FreeBooleanOperation(bool_op);
+	}
+
+	// We may need to map back the tfaces to mcols here.
+	if (free_tface1) {
+		tface_to_mcol(me);
+		MEM_freeN(me->tface);
+		me->tface = NULL;
+	}
+	if (free_tface2) {
+		tface_to_mcol(me2);
+		MEM_freeN(me2->tface);
+		me2->tface = NULL;
+	}
+
+	if (free_tface1 && free_tface2) {
+		// then we need to map the output tfaces into mcols
+		if (dlm) {
+			dlm->mcol = tface_to_mcol_p(dlm->tface, dlm->totface);
+			MEM_freeN(dlm->tface);
+			dlm->tface = NULL;
+		}
+	}
+	
+	FreeMeshDescriptors(&fd_1,&vd_1);
+	FreeMeshDescriptors(&fd_2,&vd_2);
+
+	return dlm;
+}
 
 	int
 NewBooleanMesh(
@@ -389,7 +578,7 @@ NewBooleanMesh(
 	Mesh *me2 = get_mesh(base_select->object);
 	Mesh *me = get_mesh(base->object);
 	Mesh *me_new = NULL;
-	Object *ob;
+	Object *ob, *ob_new = NULL;
 	int free_tface1,free_tface2;
 
 	float inv_mat[4][4];
@@ -409,10 +598,7 @@ NewBooleanMesh(
 
 	if (me == NULL || me2 == NULL) return 0;
 
-	success = has_faces(me);
-	if(success==0) return -1;
-	success = has_faces(me2);
-	if(success==0) return -1;
+	if(!me->totface || !me2->totface) return -1;
 	
 	success = 0;
 
@@ -519,14 +705,18 @@ NewBooleanMesh(
 			);
 		}
 		
-		if (success) {
+		if (success==1) {
 			// descriptions of the output;
 			CSG_VertexIteratorDescriptor vd_o;
 			CSG_FaceIteratorDescriptor fd_o;
+			DispListMesh dlm;
 
+			// need to make sure its zeroed
+			memset (&dlm, 0, sizeof (dlm));
+			
 			// Create a new blender mesh object - using 'base' as 
 			// a template for the new object.
-			Object * ob_new=  AddNewBlenderMesh(base);
+			ob_new=  AddNewBlenderMesh(base);
 
 			// get the output descriptors
 
@@ -537,16 +727,25 @@ NewBooleanMesh(
 			// iterate through results of operation and insert into new object
 			// see subsurf.c 
 
-			ConvertCSGDescriptorsToMeshObject(
+			ConvertCSGDescriptorsToDLM(
+				&dlm,
 				ob_new,
 				&output_mpd,
 				&fd_o,
 				&vd_o,
 				inv_mat
 			);
-		
-			// initialize the object
-			tex_space_mesh(me_new);
+
+			if (dlm.nors) MEM_freeN(dlm.nors);
+
+			me_new->mvert = dlm.mvert;
+			me_new->medge = dlm.medge;
+			me_new->mface = dlm.mface;
+			me_new->tface = dlm.tface;
+			me_new->mcol = dlm.mcol;
+			me_new->totvert = dlm.totvert;
+			me_new->totedge = dlm.totedge;
+			me_new->totface = dlm.totface;
 
 			// free up the memory
 
@@ -583,6 +782,10 @@ NewBooleanMesh(
 	FreeMeshDescriptors(&fd_1,&vd_1);
 	FreeMeshDescriptors(&fd_2,&vd_2);
 
+	if (ob_new) {
+		DAG_object_flush_update(G.scene, ob_new, OB_RECALC_DATA);
+	}
+
 	return success;
 }
 
@@ -633,15 +836,15 @@ AddNewBlenderMesh(
  * the CSG module. It declares all the necessary blender cruft and 
  * fills in the vertex and face arrays.
  */
-	int
-ConvertCSGDescriptorsToMeshObject(
+	static void 
+ConvertCSGDescriptorsToDLM(
+	DispListMesh *dlm,
 	Object *ob,
 	CSG_MeshPropertyDescriptor *props,
 	CSG_FaceIteratorDescriptor *face_it,
 	CSG_VertexIteratorDescriptor *vertex_it,
 	float parinv[][4]
 ){
-	Mesh *me = ob->data;
 	FaceVertexData *user_face_vertex_data;
 	GHash *material_hash;
 	CSG_IVertex vert;
@@ -653,11 +856,6 @@ ConvertCSGDescriptorsToMeshObject(
 
 	// create some memory for the Iface according to output mesh props.
 
-	if (face_it == NULL || vertex_it == NULL || props == NULL || me == NULL) {
-		return 0;
-	}
-	if (vertex_it->num_elements > MESH_MAX_VERTS) return 0;
-
 	// initialize the face structure for readback
 	
 	face.user_face_data = MEM_callocN(sizeof(FaceData),"BooleanOp_IFaceData");
@@ -674,21 +872,18 @@ ConvertCSGDescriptorsToMeshObject(
 	
 	// create memory for the vertex array.
 
-	me->mvert = MEM_callocN(sizeof(MVert) * vertex_it->num_elements,"BooleanOp_VertexArray");
-	me->mface = MEM_callocN(sizeof(MFace) * face_it->num_elements,"BooleanOp_FaceArray");
+	dlm->mvert = MEM_callocN(sizeof(MVert) * vertex_it->num_elements,"BooleanOp_VertexArray");
+	dlm->mface = MEM_callocN(sizeof(MFace) * face_it->num_elements,"BooleanOp_FaceArray");
 
 	if (user_face_vertex_data) {
-		me->tface = MEM_callocN(sizeof(TFace) * face_it->num_elements,"BooleanOp_TFaceArray");
-		if (me->tface == NULL) return 0;
+		dlm->tface = MEM_callocN(sizeof(TFace) * face_it->num_elements,"BooleanOp_TFaceArray");
 	} else {
-		me->tface = NULL;
+		dlm->tface = NULL;
 	}
 
-	if (me->mvert == NULL || me->mface == NULL) return 0;
-
-	insert_pos = me->mvert;
-	mfaces = me->mface;
-	tfaces = me->tface;
+	insert_pos = dlm->mvert;
+	mfaces = dlm->mface;
+	tfaces = dlm->tface;
 
 	fi_insert_pos = 0;
 
@@ -709,7 +904,7 @@ ConvertCSGDescriptorsToMeshObject(
 		vertex_it->Step(vertex_it->it);
 	}
 
-	me->totvert = vertex_it->num_elements;
+	dlm->totvert = vertex_it->num_elements;
 
 	// a hash table to remap materials to indices with
 	material_hash = BLI_ghash_new(BLI_ghashutil_ptrhash, BLI_ghashutil_ptrcmp);
@@ -724,13 +919,21 @@ ConvertCSGDescriptorsToMeshObject(
 
 		// cheat CSG never dumps out quads.
 
-		mface->v1 = face.vertex_index[0];
-		mface->v2 = face.vertex_index[1];
-		mface->v3 = face.vertex_index[2];
-		mface->v4 = 0;
+		if (face.vertex_number>3) {
+			// QUAD
+			mface->v1 = face.vertex_index[0];
+			mface->v2 = face.vertex_index[1];
+			mface->v3 = face.vertex_index[2];
+			mface->v4 = face.vertex_index[3];
+		}
+		else {
+			// TRIANGLE
+			mface->v1 = face.vertex_index[0];
+			mface->v2 = face.vertex_index[1];
+			mface->v3 = face.vertex_index[2];
+			mface->v4 = 0;
+		}
 
-		mface->edcode = ME_V1V2|ME_V2V3|ME_V3V4|ME_V4V1;
-		mface->puno = 0;
 		mface->mat_nr = 0;
 		mface->flag = fdata->faceflag;
 		
@@ -738,7 +941,7 @@ ConvertCSGDescriptorsToMeshObject(
 		 * hash table, just tuck the int into a void *.
 		 */
 		
-		if (!BLI_ghash_haskey(material_hash, fdata->material)) {
+		if (ob && !BLI_ghash_haskey(material_hash, fdata->material)) {
 			int matnr = nmaterials++;
 			BLI_ghash_insert(material_hash, fdata->material, (void*) matnr);
 			assign_material(ob, fdata->material, matnr+1);
@@ -771,9 +974,9 @@ ConvertCSGDescriptorsToMeshObject(
 					((((unsigned int)floor(color[3] + 0.5f)) & 0xff) << 0);
 			}
 
-			test_index_face(mface, tface, 3);
+			test_index_face(mface, NULL, tface, face.vertex_number);
 		} else {
-			test_index_mface(mface, 3);
+			test_index_face(mface, NULL, NULL, face.vertex_number);
 		}
 
 		fi_insert_pos++;
@@ -782,17 +985,17 @@ ConvertCSGDescriptorsToMeshObject(
 
 	BLI_ghash_free(material_hash, NULL, NULL);
 
-	me->totface = face_it->num_elements;
+	dlm->totface = face_it->num_elements;
 
-	mesh_calculate_vertex_normals(me);
+	displistmesh_add_edges(dlm);
 
+	mesh_calc_normals(dlm->mvert, dlm->totvert, dlm->mface, dlm->totface, &dlm->nors);
+		
 	// thats it!
 	if (user_face_vertex_data) {
 		MEM_freeN(user_face_vertex_data);
 	}
 	MEM_freeN(face.user_face_data);
-
-	return 1;
 }	
 	
 	void
