@@ -1,5 +1,5 @@
 /**
- * $Id: 
+ * $Id: buttons_logic.c,v 1.32 2006/01/13 01:00:35 erwin Exp $ 
  *
  * ***** BEGIN GPL/BL DUAL LICENSE BLOCK *****
  *
@@ -34,10 +34,6 @@
 #include <math.h>
 #include <string.h>
 
-#ifdef HAVE_CONFIG_H
-#include <config.h>
-#endif
-
 #ifndef WIN32
 #include <unistd.h>
 #else
@@ -714,7 +710,7 @@ static char *actuator_name(int type)
 	case ACT_GAME:
 		return "Game";
 	case ACT_VISIBILITY:
-		return "Game";
+		return "Visibility";
 	}
 	return "unknown";
 }
@@ -945,7 +941,7 @@ static void set_col_sensor(int type, int medium)
 }
 
 /**
- * Draws a toggle for pulse mode, a frequency fiels and a toggle to invert
+ * Draws a toggle for pulse mode, a frequency field and a toggle to invert
  * the value of this sensor. Operates on the shared data block of sensors.
  */
 static void draw_default_sensor_header(bSensor *sens,
@@ -955,24 +951,25 @@ static void draw_default_sensor_header(bSensor *sens,
 								short w) 
 {
 	/* Pulsing and frequency */
-	uiDefIconButS(block, TOG|BIT|0, 1, ICON_DOTSUP,
-			 (short)(x + 10), (short)(y - 19), (short)(0.15 * (w-20)), 19,
+	uiDefIconButBitS(block, TOG, SENS_PULSE_REPEAT, 1, ICON_DOTSUP,
+			 (short)(x + 10 + 0. * (w-20)), (short)(y - 19), (short)(0.15 * (w-20)), 19,
 			 &sens->pulse, 0.0, 0.0, 0, 0,
-			 "Activate TRUE pulse mode");
-	uiDefIconButS(block, TOG|BIT|2, 1, ICON_DOTSDOWN,
+			 "Activate TRUE level triggering (pulse mode)");
+
+	uiDefIconButBitS(block, TOG, SENS_NEG_PULSE_MODE, 1, ICON_DOTSDOWN,
 			 (short)(x + 10 + 0.15 * (w-20)), (short)(y - 19), (short)(0.15 * (w-20)), 19,
 			 &sens->pulse, 0.0, 0.0, 0, 0,
-			 "Activate FALSE pulse mode");
+			 "Activate FALSE level triggering (pulse mode)");
 	uiDefButS(block, NUM, 1, "f:",
 			 (short)(x + 10 + 0.3 * (w-20)), (short)(y - 19), (short)(0.275 * (w-20)), 19,
 			 &sens->freq, 0.0, 10000.0, 0, 0,
-			 "Frequency of pulses (in 1/50 sec)");
+			 "Delay between repeated pulses (in logic tics, 0 = no delay)");
 	
 	/* value or shift? */
 	uiDefButS(block, TOG, 1, "Inv",
 			 (short)(x + 10 + 0.85 * (w-20)), (short)(y - 19), (short)(0.15 * (w-20)), 19,
 			 &sens->invert, 0.0, 0.0, 0, 0,
-			 "Invert the output of this sensor");
+			 "Invert the level (output) of this sensor");
 }
 
 static short draw_sensorbuttons(bSensor *sens, uiBlock *block, short xco, short yco, short width,char* objectname)
@@ -1025,7 +1022,7 @@ static short draw_sensorbuttons(bSensor *sens, uiBlock *block, short xco, short
 			ts= sens->data; 
 			
 			/* uiDefBut(block, TEX, 1, "Property:",	xco,yco-22,width, 19, &ts->name, 0, 31, 0, 0, "Only look for Objects with this property"); */
-			uiDefIDPoinBut(block, test_matpoin_but, 1, "MA:",(short)(xco + 10),(short)(yco-44), (short)(width - 20), 19, &ts->ma,  "Only look for floors with this Material"); 
+			uiDefIDPoinBut(block, test_matpoin_but, ID_MA, 1, "MA:",(short)(xco + 10),(short)(yco-44), (short)(width - 20), 19, &ts->ma,  "Only look for floors with this Material"); 
 			///* uiDefButF(block, NUM, 1, "Margin:",	xco+width/2,yco-44,width/2, 19, &ts->dist, 0.0, 10.0, 100, 0, "Extra margin (distance) for larger sensitivity"); 
 			yco-= ysize; 
 			break; 
@@ -1042,7 +1039,8 @@ static short draw_sensorbuttons(bSensor *sens, uiBlock *block, short xco, short
 			
 			/* The collision sensor will become a generic collision (i.e. it     */
 			/* absorb the old touch sensor).                                     */
-			uiDefButS(block, TOG|BIT|0, B_REDR, "M/P",(short)(xco + 10),(short)(yco - 44),
+
+			uiDefButBitS(block, TOG, SENS_COLLISION_MATERIAL, B_REDR, "M/P",(short)(xco + 10),(short)(yco - 44),
 				(short)(0.20 * (width-20)), 19, &cs->mode, 0.0, 0.0, 0, 0,
 				"Toggle collision on material or property.");
 			
@@ -1143,7 +1141,7 @@ static short draw_sensorbuttons(bSensor *sens, uiBlock *block, short xco, short
 			
 			/* part of line 1 */
 			uiBlockSetCol(block, TH_BUT_SETTING2);
-			uiDefButS(block, TOG|BIT|0, 0, "All keys",	  xco+40+(width/2), yco-44, (width/2)-50, 19,
+			uiDefButBitS(block, TOG, 1, 0, "All keys",	  xco+40+(width/2), yco-44, (width/2)-50, 19,
 				&ks->type, 0, 0, 0, 0, "");
 			
 			/* line 4: toggle property for string logging mode */
@@ -1217,7 +1215,7 @@ static short draw_sensorbuttons(bSensor *sens, uiBlock *block, short xco, short
 			/* Line 2: type selection. The number are a bit mangled to get
 			* proper compatibility with older .blend files. */
 			str= "Type %t|Left button %x1|Middle button %x2|"
-				"Right button %x4|Wheel Up %x5|Wheel Down %x6|Movement %x8|Mouse over %x16"; 
+				"Right button %x4|Wheel Up %x5|Wheel Down %x6|Movement %x8|Mouse over %x16|Mouse over any%x32"; 
 			uiDefButS(block, MENU, B_REDR, str, xco+10, yco-44, width-20, 19,
 				&ms->type, 0, 31, 0, 0,
 				"Specify the type of event this mouse sensor should trigger on.");
@@ -1254,7 +1252,7 @@ static short draw_sensorbuttons(bSensor *sens, uiBlock *block, short xco, short
 			raySens = sens->data;
 			
 			/* 1. property or material */
-			uiDefButS(block, TOG|BIT|0, B_REDR, "M/P",
+			uiDefButBitS(block, TOG, SENS_COLLISION_MATERIAL, B_REDR, "M/P",
 				xco + 10,yco - 44, 0.20 * (width-20), 19,
 				&raySens->mode, 0.0, 0.0, 0, 0,
 				"Toggle collision on material or property.");
@@ -1411,7 +1409,7 @@ static short draw_controllerbuttons(bController *cont, uiBlock *block, short xco
 		glRects(xco, yco-ysize, xco+width, yco);
 		uiEmboss((float)xco, (float)yco-ysize, (float)xco+width, (float)yco, 1);
 
-		uiDefIDPoinBut(block, test_scriptpoin_but, 1, "Script: ", xco+45,yco-24,width-90, 19, &pc->text, "");
+		uiDefIDPoinBut(block, test_scriptpoin_but, ID_SCRIPT, 1, "Script: ", xco+45,yco-24,width-90, 19, &pc->text, "");
 		
 		yco-= ysize;
 		break;
@@ -1442,14 +1440,14 @@ static int get_col_actuator(int type)
 	case ACT_SOUND:			return TH_BUT_SETTING2;
 	case ACT_CD:			return TH_BUT_NUM;
 	case ACT_CAMERA: 		return TH_BUT_TEXTFIELD;
-	case ACT_EDIT_OBJECT: 	return TH_BUT_POPUP;
+	case ACT_EDIT_OBJECT: 		return TH_BUT_POPUP;
 	case ACT_GROUP:			return TH_BUT_ACTION;
 	case ACT_RANDOM:		return TH_BUT_NEUTRAL;
 	case ACT_SCENE:			return TH_BUT_SETTING;
 	case ACT_MESSAGE:		return TH_BUT_SETTING1;
 	case ACT_GAME:			return TH_BUT_SETTING2;
-	case ACT_VISIBILITY:	return TH_BUT_NUM;
-	case ACT_CONSTRAINT:	return TH_BUT_ACTION;
+	case ACT_VISIBILITY:		return TH_BUT_NUM;
+	case ACT_CONSTRAINT:		return TH_BUT_ACTION;
 	default:				return TH_BUT_NEUTRAL;
 	}
 }
@@ -1529,14 +1527,14 @@ static short draw_actuatorbuttons(bActuator *act, uiBlock *block, short xco, sho
 			uiDefButF(block, NUM, 0, "",		xco+45+wval, yco-125, wval, 19, oa->angularvelocity+1, -10000.0, 10000.0, 10, 0, "");
 			uiDefButF(block, NUM, 0, "",		xco+45+2*wval, yco-125, wval, 19, oa->angularvelocity+2, -10000.0, 10000.0, 10, 0, "");
 			
-			uiDefButI(block, TOG|BIT|0, 0, "L",		xco+45+3*wval, yco-22, 15, 19, &oa->flag, 0.0, 0.0, 0, 0, "Local transformation");
-			uiDefButI(block, TOG|BIT|1, 0, "L",		xco+45+3*wval, yco-41, 15, 19, &oa->flag, 0.0, 0.0, 0, 0, "Local transformation");
-			uiDefButI(block, TOG|BIT|2, 0, "L",		xco+45+3*wval, yco-64, 15, 19, &oa->flag, 0.0, 0.0, 0, 0, "Local transformation");
-			uiDefButI(block, TOG|BIT|3, 0, "L",		xco+45+3*wval, yco-83, 15, 19, &oa->flag, 0.0, 0.0, 0, 0, "Local transformation");
-			uiDefButI(block, TOG|BIT|4, 0, "L",		xco+45+3*wval, yco-106, 15, 19, &oa->flag, 0.0, 0.0, 0, 0, "Local transformation");
-			uiDefButI(block, TOG|BIT|5, 0, "L",		xco+45+3*wval, yco-125, 15, 19, &oa->flag, 0.0, 0.0, 0, 0, "Local transformation");
+			uiDefButBitI(block, TOG, ACT_FORCE_LOCAL, 0, "L",		xco+45+3*wval, yco-22, 15, 19, &oa->flag, 0.0, 0.0, 0, 0, "Local transformation");
+			uiDefButBitI(block, TOG, ACT_TORQUE_LOCAL, 0, "L",		xco+45+3*wval, yco-41, 15, 19, &oa->flag, 0.0, 0.0, 0, 0, "Local transformation");
+			uiDefButBitI(block, TOG, ACT_DLOC_LOCAL, 0, "L",		xco+45+3*wval, yco-64, 15, 19, &oa->flag, 0.0, 0.0, 0, 0, "Local transformation");
+			uiDefButBitI(block, TOG, ACT_DROT_LOCAL, 0, "L",		xco+45+3*wval, yco-83, 15, 19, &oa->flag, 0.0, 0.0, 0, 0, "Local transformation");
+			uiDefButBitI(block, TOG, ACT_LIN_VEL_LOCAL, 0, "L",		xco+45+3*wval, yco-106, 15, 19, &oa->flag, 0.0, 0.0, 0, 0, "Local transformation");
+			uiDefButBitI(block, TOG, ACT_ANG_VEL_LOCAL, 0, "L",		xco+45+3*wval, yco-125, 15, 19, &oa->flag, 0.0, 0.0, 0, 0, "Local transformation");
 			
-			uiDefButI(block, TOG|BIT|6, 0, "add",xco+45+3*wval+15, yco-106, 35, 19, &oa->flag, 0.0, 0.0, 0, 0, "Toggles between ADD and SET linV");
+			uiDefButBitI(block, TOG, ACT_ADD_LIN_VEL, 0, "add",xco+45+3*wval+15, yco-106, 35, 19, &oa->flag, 0.0, 0.0, 0, 0, "Toggles between ADD and SET linV");
 			
 			yco-= ysize;
 			break;
@@ -1563,7 +1561,7 @@ static short draw_actuatorbuttons(bActuator *act, uiBlock *block, short xco, sho
 			str= "Action types   %t|Play %x0|Flipper %x2|Loop Stop %x3|Loop End %x4|Property %x6";
 #endif
 			uiDefButS(block, MENU, B_REDR, str, xco+30, yco-24, width-60, 19, &aa->type, 0.0, 0.0, 0.0, 0.0, "Action playback type");
-			uiDefIDPoinBut(block, test_actionpoin_but, 1, "AC: ", xco+30, yco-44, width-60, 19, &aa->act, "Action name");
+			uiDefIDPoinBut(block, test_actionpoin_but, ID_AC, 1, "AC: ", xco+30, yco-44, width-60, 19, &aa->act, "Action name");
 			
 			if(aa->type == ACT_ACTION_FROM_PROP)
 			{
@@ -1594,10 +1592,7 @@ static short draw_actuatorbuttons(bActuator *act, uiBlock *block, short xco, sho
 		{
 			ia= act->data;
 			
-			if(ia->type==ACT_IPO_KEY2KEY)
-				ysize= 72; 
-			else
-				ysize= 52;
+			ysize= 52;
 			
 			glRects(xco, yco-ysize, xco+width, yco);
 			uiEmboss((float)xco, (float)yco-ysize, (float)xco+width, (float)yco, 1);
@@ -1605,23 +1600,11 @@ static short draw_actuatorbuttons(bActuator *act, uiBlock *block, short xco, sho
 			str = "Ipo types   %t|Play %x0|Ping Pong %x1|Flipper %x2|Loop Stop %x3|Loop End %x4|Property %x6";
 			
 			uiDefButS(block, MENU, B_REDR, str,		xco+20, yco-24, width-40 - (width-40)/3, 19, &ia->type, 0, 0, 0, 0, "");
-			uiDefButS(block, TOG|BIT|ACT_IPOCHILD_BIT, B_REDR, 
+			uiDefButBitS(block, TOG, ACT_IPOCHILD,  B_REDR, 
 				"Child",	xco+20+0.666*(width-40), yco-24, (width-40)/3, 19, 
 				&ia->flag, 0, 0, 0, 0, 
 				"Add all children Objects as well");
-			/* 
-			Key2key was disabled.... the settings below should not be reused without 
-			thought, because they interfere with other variables.
-			
-			  if(ia->type==ACT_IPO_KEY2KEY) {
-			  
-				uiDefButS(block, TOG|BIT|0, 0, "Prev", xco+20, yco-44, (width-40)/3, 19, &ia->flag, 0, 0, 0, 0, "Play backwards");
-				uiDefButS(block, TOG|BIT|1, 0, "Cycl", xco+20+(width-40)/3, yco-44, (width-40)/3, 19, &ia->flag, 0, 0, 0, 0, "Play cyclic");
-				uiDefButS(block, TOG|BIT|3, 0, "Hold", xco+20+2*(width-40)/3, yco-44, (width-40)/3, 19, &ia->flag, 0, 0, 0, 0, "Keep playing while activated");
-				
-				  uiDefBut(block, TEX, 0, "Prop: ",		xco+20, yco-66, width-40, 19, ia->name, 0.0, 31.0, 0, 0, "Set property to key position");
-				  } else 
-			*/
+
 			if(ia->type==ACT_IPO_FROM_PROP) {
 				uiDefBut(block, TEX, 0, 
 					"Prop: ",		xco+20, yco-44, width-40, 19, 
@@ -1638,14 +1621,14 @@ static short draw_actuatorbuttons(bActuator *act, uiBlock *block, short xco, sho
 					&ia->end, 0.0, MAXFRAMEF, 0, 0, 
 					"End frame");
 				
-				uiDefButS(block, TOG|BIT|ACT_IPOFORCE_BIT, B_REDR, 
+				uiDefButBitS(block, TOG, ACT_IPOFORCE, B_REDR, 
 					"Force", xco+width-78, yco-44, 43, 19, 
 					&ia->flag, 0, 0, 0, 0, 
 					"Convert Ipo to force"); 
 				
 				/* Only show the do-force-local toggle if force is requested */
 				if (ia->flag & ACT_IPOFORCE) {
-					uiDefButS(block, TOG|BIT|ACT_IPOFORCE_LOCAL_BIT, 0, 
+					uiDefButBitS(block, TOG, ACT_IPOFORCE_LOCAL, 0, 
 						"L", xco+width-35, yco-44, 15, 19, 
 						&ia->flag, 0, 0, 0, 0, 
 						"Let the force-ipo act in local coordinates."); 
@@ -1670,7 +1653,7 @@ static short draw_actuatorbuttons(bActuator *act, uiBlock *block, short xco, sho
 			uiDefBut(block, TEX, 1, "Prop: ",		xco+30,yco-44,width-60, 19, pa->name, 0, 31, 0, 0, "Property name");
 			
 			if(pa->type==ACT_PROP_COPY) {
-				uiDefIDPoinBut(block, test_obpoin_but, 1, "OB:",	xco+10, yco-64, (width-20)/2, 19, &(pa->ob), "Copy from this Object");
+				uiDefIDPoinBut(block, test_obpoin_but, ID_OB, 1, "OB:",	xco+10, yco-64, (width-20)/2, 19, &(pa->ob), "Copy from this Object");
 				uiDefBut(block, TEX, 1, "Prop: ",		xco+10+(width-20)/2, yco-64, (width-20)/2, 19, pa->value, 0, 31, 0, 0, "Copy this property");
 			}
 			else {
@@ -1758,7 +1741,7 @@ static short draw_actuatorbuttons(bActuator *act, uiBlock *block, short xco, sho
 		
  		ca= act->data;
         
- 		uiDefIDPoinBut(block, test_obpoin_but, 1, "OB:",		xco+10, yco-24, (width-20)/2, 19, &(ca->ob), "Look at this Object");
+ 		uiDefIDPoinBut(block, test_obpoin_but, ID_OB, 1, "OB:",		xco+10, yco-24, (width-20)/2, 19, &(ca->ob), "Look at this Object");
  		uiDefButF(block, NUM, 0, "Height:",	xco+10+(width-20)/2, yco-24, (width-20)/2, 19, &ca->height, 0.0, 20.0, 0, 0, "");
 		
  		uiDefButF(block, NUM, 0, "Min:",	xco+10, yco-44, (width-60)/2, 19, &ca->min, 0.0, 20.0, 0, 0, "");
@@ -1783,7 +1766,7 @@ static short draw_actuatorbuttons(bActuator *act, uiBlock *block, short xco, sho
 			glRects(xco, yco-ysize, xco+width, yco);
 			uiEmboss((float)xco, (float)yco-ysize, (float)xco+width, (float)yco, 1);
 	 
-			uiDefIDPoinBut(block, test_obpoin_but, 1, "OB:",		xco+10, yco-44, (width-20)/2, 19, &(eoa->ob), "Add this Object");
+			uiDefIDPoinBut(block, test_obpoin_but, ID_OB, 1, "OB:",		xco+10, yco-44, (width-20)/2, 19, &(eoa->ob), "Add this Object");
 			uiDefButI(block, NUM, 0, "Time:",	xco+10+(width-20)/2, yco-44, (width-20)/2, 19, &eoa->time, 0.0, 2000.0, 0, 0, "Duration the new Object lives");
 
 			wval= (width-60)/3;
@@ -1799,7 +1782,7 @@ static short draw_actuatorbuttons(bActuator *act, uiBlock *block, short xco, sho
 			uiDefButF(block, NUM, 0, "",		xco+45+2*wval, yco-68, wval, 19,
 					 eoa->linVelocity+2, -100.0, 100.0, 10, 0,
 					 "Velocity upon creation, z component.");
-			uiDefButS(block, TOG|BIT|1, 0, "L", xco+45+3*wval, yco-68, 15, 19,
+			uiDefButBitS(block, TOG, 2, 0, "L", xco+45+3*wval, yco-68, 15, 19,
 					 &eoa->localflag, 0.0, 0.0, 0, 0,
 					 "Apply the transformation locally");
 
@@ -1814,14 +1797,14 @@ static short draw_actuatorbuttons(bActuator *act, uiBlock *block, short xco, sho
 			glRects(xco, yco-ysize, xco+width, yco);
 			uiEmboss((float)xco, (float)yco-ysize, (float)xco+width, (float)yco, 1);
 	 
-			uiDefIDPoinBut(block, test_meshpoin_but, 1, "ME:",		xco+40, yco-44, (width-80), 19, &(eoa->me), "Add this Object");
+			uiDefIDPoinBut(block, test_meshpoin_but, ID_ME, 1, "ME:",		xco+40, yco-44, (width-80), 19, &(eoa->me), "Add this Object");
 		}
 		else if(eoa->type==ACT_EDOB_TRACK_TO) {
 			ysize= 48;
 			glRects(xco, yco-ysize, xco+width, yco);
 			uiEmboss((float)xco, (float)yco-ysize, (float)xco+width, (float)yco, 1);
 	 
-			uiDefIDPoinBut(block, test_obpoin_but, 1, "OB:",		xco+10, yco-44, (width-20)/2, 19, &(eoa->ob), "Track to this Object");
+			uiDefIDPoinBut(block, test_obpoin_but, ID_OB, 1, "OB:",		xco+10, yco-44, (width-20)/2, 19, &(eoa->ob), "Track to this Object");
 			uiDefButI(block, NUM, 0, "Time:",	xco+10+(width-20)/2, yco-44, (width-20)/2-40, 19, &eoa->time, 0.0, 2000.0, 0, 0, "Duration the tracking takes");
 			uiDefButS(block, TOG, 0, "3D",	xco+width-50, yco-44, 40, 19, &eoa->flag, 0.0, 0.0, 0, 0, "Enable 3D tracking");
 		}
@@ -1878,7 +1861,7 @@ static short draw_actuatorbuttons(bActuator *act, uiBlock *block, short xco, sho
   			glRects(xco, yco-ysize, xco+width, yco); 
   			uiEmboss((float)xco, (float)yco-ysize, (float)xco+width, (float)yco, 1); 
 	 
-  			uiDefIDPoinBut(block, test_obpoin_but, 1, "OB:",		xco+40, yco-44, (width-80), 19, &(sca->camera), "Set this Camera"); 
+  			uiDefIDPoinBut(block, test_obpoin_but, ID_OB, 1, "OB:",		xco+40, yco-44, (width-80), 19, &(sca->camera), "Set this Camera"); 
   		} 
   		else if(sca->type==ACT_SCENE_SET) { 
 			
@@ -1886,7 +1869,7 @@ static short draw_actuatorbuttons(bActuator *act, uiBlock *block, short xco, sho
   			glRects(xco, yco-ysize, xco+width, yco); 
   			uiEmboss((float)xco, (float)yco-ysize, (float)xco+width, (float)yco, 1); 
 	
-  			uiDefIDPoinBut(block, test_scenepoin_but, 1, "SCE:",		xco+40, yco-44, (width-80), 19, &(sca->scene), "Set this Scene"); 
+  			uiDefIDPoinBut(block, test_scenepoin_but, ID_SCE, 1, "SCE:",		xco+40, yco-44, (width-80), 19, &(sca->scene), "Set this Scene"); 
   		} 
 		else if(sca->type==ACT_SCENE_ADD_FRONT) { 
 			
@@ -1894,7 +1877,7 @@ static short draw_actuatorbuttons(bActuator *act, uiBlock *block, short xco, sho
   			glRects(xco, yco-ysize, xco+width, yco); 
   			uiEmboss((float)xco, (float)yco-ysize, (float)xco+width, (float)yco, 1); 
 	
-  			uiDefIDPoinBut(block, test_scenepoin_but, 1, "SCE:",		xco+40, yco-44, (width-80), 19, &(sca->scene), "Add an Overlay Scene"); 
+  			uiDefIDPoinBut(block, test_scenepoin_but, ID_SCE, 1, "SCE:",		xco+40, yco-44, (width-80), 19, &(sca->scene), "Add an Overlay Scene"); 
   		} 
 		else if(sca->type==ACT_SCENE_ADD_BACK) { 
 			
@@ -1902,7 +1885,7 @@ static short draw_actuatorbuttons(bActuator *act, uiBlock *block, short xco, sho
   			glRects(xco, yco-ysize, xco+width, yco); 
   			uiEmboss((float)xco, (float)yco-ysize, (float)xco+width, (float)yco, 1); 
 	
-  			uiDefIDPoinBut(block, test_scenepoin_but, 1, "SCE:",		xco+40, yco-44, (width-80), 19, &(sca->scene), "Add a Background Scene"); 
+  			uiDefIDPoinBut(block, test_scenepoin_but, ID_SCE, 1, "SCE:",		xco+40, yco-44, (width-80), 19, &(sca->scene), "Add a Background Scene"); 
   		} 
 		else if(sca->type==ACT_SCENE_REMOVE) { 
 			
@@ -1910,7 +1893,7 @@ static short draw_actuatorbuttons(bActuator *act, uiBlock *block, short xco, sho
   			glRects(xco, yco-ysize, xco+width, yco); 
   			uiEmboss((float)xco, (float)yco-ysize, (float)xco+width, (float)yco, 1); 
 	
-  			uiDefIDPoinBut(block, test_scenepoin_but, 1, "SCE:",		xco+40, yco-44, (width-80), 19, &(sca->scene), "Remove a Scene");
+  			uiDefIDPoinBut(block, test_scenepoin_but, ID_SCE, 1, "SCE:",		xco+40, yco-44, (width-80), 19, &(sca->scene), "Remove a Scene");
   		} 
 		else if(sca->type==ACT_SCENE_SUSPEND) { 
 			
@@ -1918,7 +1901,7 @@ static short draw_actuatorbuttons(bActuator *act, uiBlock *block, short xco, sho
   			glRects(xco, yco-ysize, xco+width, yco); 
   			uiEmboss((float)xco, (float)yco-ysize, (float)xco+width, (float)yco, 1); 
 	
-  			uiDefIDPoinBut(block, test_scenepoin_but, 1, "SCE:",		xco+40, yco-44, (width-80), 19, &(sca->scene), "Pause a Scene");
+  			uiDefIDPoinBut(block, test_scenepoin_but, ID_SCE, 1, "SCE:",		xco+40, yco-44, (width-80), 19, &(sca->scene), "Pause a Scene");
   		} 
 		else if(sca->type==ACT_SCENE_RESUME) { 
 			
@@ -1926,7 +1909,7 @@ static short draw_actuatorbuttons(bActuator *act, uiBlock *block, short xco, sho
   			glRects(xco, yco-ysize, xco+width, yco); 
   			uiEmboss((float)xco, (float)yco-ysize, (float)xco+width, (float)yco, 1); 
 	
-  			uiDefIDPoinBut(block, test_scenepoin_but, 1, "SCE:",		xco+40, yco-44, (width-80), 19, &(sca->scene), "Unpause a Scene");
+  			uiDefIDPoinBut(block, test_scenepoin_but, ID_SCE, 1, "SCE:",		xco+40, yco-44, (width-80), 19, &(sca->scene), "Unpause a Scene");
   		} 
 
 		str= "Scene %t|Restart %x0|Set Scene %x1|Set Camera %x2|Add OverlayScene %x3|Add BackgroundScene %x4|Remove Scene %x5|Suspend Scene %x6|Resume Scene %x7";
@@ -2016,7 +1999,7 @@ static short draw_actuatorbuttons(bActuator *act, uiBlock *block, short xco, sho
 			  0.0, 0.0, 0, 0,
 			  "Make the object invisible or visible.");
 /*
-		uiDefButI(block, TOG|BIT|ACT_VISIBILITY_INVISIBLE_BIT, 0,
+		uiDefButBitI(block, TOG, ACT_VISIBILITY_INVISIBLE, 0,
 			  "Invisible",
 			  xco + 10, yco - 24, width - 20, 19, &visAct->flag,
 			  0.0, 0.0, 0, 0,
@@ -2060,7 +2043,7 @@ static short draw_actuatorbuttons(bActuator *act, uiBlock *block, short xco, sho
 		/*4. and 5. arguments for the distribution*/
 		switch (randAct->distribution) {
 		case ACT_RANDOM_BOOL_CONST:
-			uiDefButI(block, TOG|BIT|0, 1, "Always true", (xco+10), yco-64, (width-20), 19,
+			uiDefButBitI(block, TOG, 1, 1, "Always true", (xco+10), yco-64, (width-20), 19,
 					 &randAct->int_arg_1, 2.0, 1, 0, 0,
 					 "Always false or always true");			
 			break;
@@ -2165,7 +2148,7 @@ static short draw_actuatorbuttons(bActuator *act, uiBlock *block, short xco, sho
 		"Optional message subject. This is what can be filtered on.");
 
 		/* line 3: Text/Property */
-		uiDefButS(block, TOG|BIT|0, B_REDR, "T/P",
+		uiDefButBitS(block, TOG, 1, B_REDR, "T/P",
 			(xco+10),(yco-(myline*24)), (0.20 * (width-20)), 19,
 			&ma->bodyType, 0.0, 0.0, 0, 0,
 			"Toggle message type: either Text or a PropertyName.");
@@ -2350,42 +2333,42 @@ static uiBlock *actuator_menu(void *arg_unused)
 
 void buttons_enji(uiBlock *block, Object *ob)
 {
-	uiDefBut(block, TOG|INT|BIT|13, B_SETSECTOR, "Sector",
+	uiDefButBitI(block, TOG, OB_SECTOR, B_SETSECTOR, "Sector",
 			 10,205,65,19, &ob->gameflag, 0, 0, 0, 0, 
 			 "All game elements should be in the Sector boundbox");
-	uiDefBut(block, TOG|INT|BIT|14, B_SETPROP, "Prop",
+	uiDefButBitI(block, TOG, OB_PROP, B_SETPROP, "Prop",
 			 75,205,65,19, &ob->gameflag, 0, 0, 0, 0, 
 			 "An Object fixed within a sector");
 	uiBlockSetCol(block, BUTPURPLE);
-	uiDefBut(block, TOG|INT|BIT|2, B_SETACTOR, "Actor",
+	uiDefButBitI(block, TOG, OB_ACTOR, B_SETACTOR, "Actor",
 			 140,205,65,19, &ob->gameflag, 0, 0, 0, 0, 
 			 "Objects that are evaluated by the engine ");
 	if(ob->gameflag & OB_ACTOR) {	
-		uiDefBut(block, TOG|INT|BIT|0, B_SETDYNA, "Dynamic",
+		uiDefButBitI(block, TOG, OB_DYNAMIC, B_SETDYNA, "Dynamic",
 				 205,205,75,19, &ob->gameflag, 0, 0, 0, 0, 
 				 "Motion defined by laws of physics");
-		uiDefBut(block, TOG|INT|BIT|15, B_SETMAINACTOR, "MainActor",
+		uiDefButBitI(block, TOG, OB_MAINACTOR, B_SETMAINACTOR, "MainActor",
 				 280,205,70,19, &ob->gameflag, 0, 0, 0, 0, "");
 		
 		if(ob->gameflag & OB_DYNAMIC) {
-			uiDefBut(block, TOG|INT|BIT|6, B_DIFF, "Do Fh",
+			uiDefButBitI(block, TOG, OB_DO_FH, B_DIFF, "Do Fh",
 					 10,185,50,19, &ob->gameflag, 0, 0, 0, 0, 
 					 "Use Fh settings in Materials");
-			uiDefBut(block, TOG|INT|BIT|7, B_DIFF, "Rot Fh",
+			uiDefButBitI(block, TOG, OB_ROT_FH, B_DIFF, "Rot Fh",
 					 60,185,50,19, &ob->gameflag, 0, 0, 0, 0, 
 					 "Use face normal to rotate Object");
 		
 			uiBlockSetCol(block, BUTGREY);
-			uiDefBut(block, NUM|FLO, B_DIFF, "Mass:",
+			uiDefButF(block, NUM, B_DIFF, "Mass:",
 					 110, 185, 120, 19, &ob->mass, 0.01, 100.0, 10, 0, 
 					 "The mass of the Object");
-			uiDefBut(block, NUM|FLO, REDRAWVIEW3D, "Size:",
+			uiDefButF(block, NUM, REDRAWVIEW3D, "Size:",
 					 230, 185, 120, 19, &ob->inertia, 0.01, 10.0, 10, 0, 
 					 "Bounding sphere size");
-			uiDefBut(block, NUM|FLO, B_DIFF, "Damp:",
+			uiDefButF(block, NUM, B_DIFF, "Damp:",
 					 10, 165, 100, 19, &ob->damping, 0.0, 1.0, 10, 0, 
 					 "General movement damping");
-			uiDefBut(block, NUM|FLO, B_DIFF, "RotDamp:",
+			uiDefButF(block, NUM, B_DIFF, "RotDamp:",
 					 110, 165, 120, 19, &ob->rdamping, 0.0, 1.0, 10, 0, 
 					 "General rotation damping");
 		}
@@ -2395,25 +2378,25 @@ void buttons_enji(uiBlock *block, Object *ob)
 
 void buttons_ketsji(uiBlock *block, Object *ob)
 {
-	uiDefButI(block, TOG|BIT|2, B_REDR, "Actor",
+	uiDefButBitI(block, TOG, OB_ACTOR, B_REDR, "Actor",
 			  10,205,75,19, &ob->gameflag, 0, 0, 0, 0,
 			  "Objects that are evaluated by the engine ");
 	if(ob->gameflag & OB_ACTOR) {	
-		uiDefButI(block, TOG|BIT|9, B_REDR, "Ghost", 85,205,65,19, 
+		uiDefButBitI(block, TOG, OB_GHOST, B_REDR, "Ghost", 85,205,65,19, 
 				  &ob->gameflag, 0, 0, 0, 0, 
 				  "Objects that don't restitute collisions (like a ghost)");
-		uiDefButI(block, TOG|BIT|0, B_REDR, "Dynamic", 150,205,65,19, 
+		uiDefButBitI(block, TOG, OB_DYNAMIC, B_REDR, "Dynamic", 150,205,65,19, 
 				  &ob->gameflag, 0, 0, 0, 0, 
 				  "Motion defined by laws of physics");
 	
 		if(ob->gameflag & OB_DYNAMIC) {
-			uiDefButI(block, TOG|BIT|10, B_REDR, "Rigid Body", 215,205,135,19, 
+			uiDefButBitI(block, TOG, OB_RIGID_BODY, B_REDR, "Rigid Body", 215,205,135,19, 
 					  &ob->gameflag, 0, 0, 0, 0, 
 					  "Enable rolling physics");
-			uiDefButI(block, TOG|BIT|6, B_DIFF, "Do Fh", 10,185,50,19, 
+			uiDefButBitI(block, TOG, OB_DO_FH, B_DIFF, "Do Fh", 10,185,50,19, 
 					  &ob->gameflag, 0, 0, 0, 0, 
 					  "Use Fh settings in Materials");
-			uiDefButI(block, TOG|BIT|7, B_DIFF, "Rot Fh", 60,185,50,19, 
+			uiDefButBitI(block, TOG, OB_ROT_FH, B_DIFF, "Rot Fh", 60,185,50,19, 
 					  &ob->gameflag, 0, 0, 0, 0, 
 					  "Use face normal to rotate Object");
 			uiDefButF(block, NUM, B_DIFF, "Mass:", 110, 185, 80, 19, 
@@ -2432,7 +2415,7 @@ void buttons_ketsji(uiBlock *block, Object *ob)
 			uiDefButF(block, NUM, B_DIFF, "RotDamp:", 110, 165, 120, 19, 
 					  &ob->rdamping, 0.0, 1.0, 10, 0, 
 					  "General rotation damping");
-			uiDefButI(block, TOG|BIT|8, B_REDR, "Anisotropic", 
+			uiDefButBitI(block, TOG, OB_ANISOTROPIC_FRICTION, B_REDR, "Anisotropic", 
 					  230, 165, 120, 19,
 					  &ob->gameflag, 0.0, 1.0, 10, 0,
 					  "Enable anisotropic friction");			
@@ -2452,12 +2435,12 @@ void buttons_ketsji(uiBlock *block, Object *ob)
 	}
 
 	if (!(ob->gameflag & OB_GHOST)) {
-		uiDefButI(block, TOG|BIT|11, B_REDR, "Bounds", 10, 125, 75, 19,
+		uiDefButBitI(block, TOG, OB_BOUNDS, B_REDR, "Bounds", 10, 125, 75, 19,
 				&ob->gameflag, 0, 0,0, 0,
 				"Specify a bounds object for physics");
 		if (ob->gameflag & OB_BOUNDS) {
-			uiDefButS(block, MENU, REDRAWVIEW3D, "Boundary Display%t|Box%x0|Sphere%x1|Cylinder%x2|Cone%x3|Polyheder%x4|Polytope%x5",
-				85, 125, 100, 19, &ob->boundtype, 0, 0, 0, 0, "Selects the boundary display type");
+			uiDefButS(block, MENU, REDRAWVIEW3D, "Boundary Display%t|Box%x0|Sphere%x1|Cylinder%x2|Cone%x3|Convex Hull Polytope%x5|Static TriangleMesh %x4|",
+				85, 125, 140, 19, &ob->boundtype, 0, 0, 0, 0, "Selects the collision type");
 		}
 	}
 }
@@ -2529,8 +2512,8 @@ void logic_buts(void)
 		}
 
 		if(prop->type==PROP_BOOL) {
-			uiDefButI(block, TOG|BIT|0, B_REDR, "True",		215, (short)(70-20*a), 55, 19, &prop->data, 0, 0, 0, 0, "");
-			uiDefButI(block, TOGN|BIT|0, B_REDR, "False",	270, (short)(70-20*a), 55, 19, &prop->data, 0, 0, 0, 0, "");
+			uiDefButBitI(block, TOG, 1, B_REDR, "True",		215, (short)(70-20*a), 55, 19, &prop->data, 0, 0, 0, 0, "");
+			uiDefButBitI(block, TOGN, 1, B_REDR, "False",	270, (short)(70-20*a), 55, 19, &prop->data, 0, 0, 0, 0, "");
 		}
 		else if(prop->type==PROP_INT) 
 			uiDefButI(block, NUM, butreturn, "",			215, (short)(70-20*a), 110, 19, &prop->data, -10000, 10000, 0, 0, "");
@@ -2541,7 +2524,7 @@ void logic_buts(void)
 		else if(prop->type==PROP_TIME) 
 			uiDefButF(block, NUM, butreturn, "",			215, (short)(70-20*a), 110, 19, (float*) &prop->data, -10000, 10000, 0, 0, "");
 		
-		uiDefButS(block, TOG|BIT|0, 0, "D",		325, (short)(70-20*a), 20, 19, &prop->flag, 0, 0, 0, 0, "Print Debug info");
+		uiDefButBitS(block, TOG, PROP_DEBUG, 0, "D",		325, (short)(70-20*a), 20, 19, &prop->flag, 0, 0, 0, 0, "Print Debug info");
 		
 		a++;
 		prop= prop->next;
@@ -2557,9 +2540,9 @@ void logic_buts(void)
 	uiBlockSetEmboss(block, UI_EMBOSSP);
 	uiDefBlockBut(block, sensor_menu, NULL, "Sensors", xco-10, yco+35, 80, 19, "");
 	uiBlockSetEmboss(block, UI_EMBOSS);
-	uiDefButS(block, TOG|BIT|0, B_REDR, "Sel", xco+110, yco+35, (width-100)/3, 19, &G.buts->scaflag, 0, 0, 0, 0, "Show all selected Objects");
-	uiDefButS(block, TOG|BIT|1, B_REDR, "Act", xco+110+(width-100)/3, yco+35, (width-100)/3, 19, &G.buts->scaflag, 0, 0, 0, 0, "Show active Object");
-	uiDefButS(block, TOG|BIT|2, B_REDR, "Link", xco+110+2*(width-100)/3, yco+35, (width-100)/3, 19, &G.buts->scaflag, 0, 0, 0, 0, "Show linked Objects to Controller");
+	uiDefButBitS(block, TOG, BUTS_SENS_SEL, B_REDR, "Sel", xco+110, yco+35, (width-100)/3, 19, &G.buts->scaflag, 0, 0, 0, 0, "Show all selected Objects");
+	uiDefButBitS(block, TOG, BUTS_SENS_ACT, B_REDR, "Act", xco+110+(width-100)/3, yco+35, (width-100)/3, 19, &G.buts->scaflag, 0, 0, 0, 0, "Show active Object");
+	uiDefButBitS(block, TOG, BUTS_SENS_LINK, B_REDR, "Link", xco+110+2*(width-100)/3, yco+35, (width-100)/3, 19, &G.buts->scaflag, 0, 0, 0, 0, "Show linked Objects to Controller");
 	
 	for(a=0; a<count; a++) {
 		ob= (Object *)idar[a];
@@ -2571,9 +2554,9 @@ void logic_buts(void)
 		/* presume it is only objects for now */
 		uiBlockSetEmboss(block, UI_EMBOSS);
 		if(ob->sensors.first) uiSetCurFont(block, UI_HELVB);
-		uiDefButS(block, TOG|BIT|6, B_REDR, ob->id.name+2,(short)(xco-10), yco, (short)(width-30), 19, &ob->scaflag, 0, 31, 0, 0, "Object name, click to show/hide sensors");
+		uiDefButBitS(block, TOG, OB_SHOWSENS, B_REDR, ob->id.name+2,(short)(xco-10), yco, (short)(width-30), 19, &ob->scaflag, 0, 31, 0, 0, "Object name, click to show/hide sensors");
 		if(ob->sensors.first) uiSetCurFont(block, UI_HELV);
-		uiDefButS(block, TOG|BIT|8, B_ADD_SENS, "Add",(short)(xco+width-40), yco, 50, 19, &ob->scaflag, 0, 0, 0, 0, "Add a new Sensor");
+		uiDefButBitS(block, TOG, OB_ADDSENS, B_ADD_SENS, "Add",(short)(xco+width-40), yco, 50, 19, &ob->scaflag, 0, 0, 0, 0, "Add a new Sensor");
 		yco-=20;
 		
 		if(ob->scaflag & OB_SHOWSENS) {
@@ -2581,8 +2564,8 @@ void logic_buts(void)
 			sens= ob->sensors.first;
 			while(sens) {
 				uiBlockSetEmboss(block, UI_EMBOSSM);
-				uiDefIconButS(block, TOG|BIT|1, B_DEL_SENS, ICON_X,	xco, yco, 22, 19, &sens->flag, 0, 0, 0, 0, "Delete Sensor");
-				uiDefIconButS(block, ICONTOG|BIT|0, B_REDR, ICON_RIGHTARROW, (short)(xco+width-22), yco, 22, 19, &sens->flag, 0, 0, 0, 0, "Sensor settings");
+				uiDefIconButBitS(block, TOG, SENS_DEL, B_DEL_SENS, ICON_X,	xco, yco, 22, 19, &sens->flag, 0, 0, 0, 0, "Delete Sensor");
+				uiDefIconButBitS(block, ICONTOG, SENS_SHOW, B_REDR, ICON_RIGHTARROW, (short)(xco+width-22), yco, 22, 19, &sens->flag, 0, 0, 0, 0, "Sensor settings");
 
 				ycoo= yco;
 				if(sens->flag & SENS_SHOW)
@@ -2621,9 +2604,9 @@ void logic_buts(void)
 	uiBlockSetEmboss(block, UI_EMBOSSP);
 	uiDefBlockBut(block, controller_menu, NULL, "Controllers", xco-10, yco+35, 100, 19, "");
 	uiBlockSetEmboss(block, UI_EMBOSS);
-	uiDefButS(block, TOG|BIT|3, B_REDR, "Sel", xco+110, yco+35, (width-100)/3, 19, &G.buts->scaflag, 0, 0, 0, 0, "Show all selected Objects");
-	uiDefButS(block, TOG|BIT|4, B_REDR, "Act", xco+110+(width-100)/3, yco+35, (width-100)/3, 19, &G.buts->scaflag, 0, 0, 0, 0, "Show active Object");
-	uiDefButS(block, TOG|BIT|5, B_REDR, "Link", xco+110+2*(width-100)/3, yco+35, (width-100)/3, 19, &G.buts->scaflag, 0, 0, 0, 0, "Show linked Objects to Sensor/Actuator");
+	uiDefButBitS(block, TOG, BUTS_CONT_SEL,  B_REDR, "Sel", xco+110, yco+35, (width-100)/3, 19, &G.buts->scaflag, 0, 0, 0, 0, "Show all selected Objects");
+	uiDefButBitS(block, TOG, BUTS_CONT_ACT, B_REDR, "Act", xco+110+(width-100)/3, yco+35, (width-100)/3, 19, &G.buts->scaflag, 0, 0, 0, 0, "Show active Object");
+	uiDefButBitS(block, TOG, BUTS_CONT_LINK, B_REDR, "Link", xco+110+2*(width-100)/3, yco+35, (width-100)/3, 19, &G.buts->scaflag, 0, 0, 0, 0, "Show linked Objects to Sensor/Actuator");
 	
 	ob= OBACT;
 	
@@ -2635,9 +2618,9 @@ void logic_buts(void)
 
 		/* presume it is only objects for now */
 		uiBlockSetEmboss(block, UI_EMBOSS);
-		uiDefButS(block, TOG|BIT|9, B_ADD_CONT, "Add",(short)(xco+width-40), yco, 50, 19, &ob->scaflag, 0, 0, 0, 0, "Add a new Controller");
+		uiDefButBitS(block, TOG, OB_ADDCONT, B_ADD_CONT, "Add",(short)(xco+width-40), yco, 50, 19, &ob->scaflag, 0, 0, 0, 0, "Add a new Controller");
 		if(ob->controllers.first) uiSetCurFont(block, UI_HELVB);
-		uiDefButS(block, TOG|BIT|11, B_REDR, ob->id.name+2,(short)(xco-10), yco, (short)(width-30), 19, &ob->scaflag, 0, 0, 0, 0, "Active Object name");
+		uiDefButBitS(block, TOG, OB_SHOWCONT, B_REDR, ob->id.name+2,(short)(xco-10), yco, (short)(width-30), 19, &ob->scaflag, 0, 0, 0, 0, "Active Object name");
 		if(ob->controllers.first) uiSetCurFont(block, UI_HELV);
 		yco-=20;
 		
@@ -2646,8 +2629,8 @@ void logic_buts(void)
 			cont= ob->controllers.first;
 			while(cont) {
 				uiBlockSetEmboss(block, UI_EMBOSSM);
-				uiDefIconButS(block, TOG|BIT|1, B_DEL_CONT, ICON_X,	xco, yco, 22, 19, &cont->flag, 0, 0, 0, 0, "Delete Controller");
-				uiDefIconButS(block, ICONTOG|BIT|0, B_REDR, ICON_RIGHTARROW, (short)(xco+width-22), yco, 22, 19, &cont->flag, 0, 0, 0, 0, "Controller settings");
+				uiDefIconButBitS(block, TOG, CONT_DEL, B_DEL_CONT, ICON_X,	xco, yco, 22, 19, &cont->flag, 0, 0, 0, 0, "Delete Controller");
+				uiDefIconButBitS(block, ICONTOG, CONT_SHOW, B_REDR, ICON_RIGHTARROW, (short)(xco+width-22), yco, 22, 19, &cont->flag, 0, 0, 0, 0, "Controller settings");
 		
 				if(cont->flag & CONT_SHOW) {
 					cont->otype= cont->type;
@@ -2688,9 +2671,9 @@ void logic_buts(void)
 	uiBlockSetEmboss(block, UI_EMBOSSP);
 	uiDefBlockBut(block, actuator_menu, NULL, "Actuators", xco-10, yco+35, 100, 19, "");
 	uiBlockSetEmboss(block, UI_EMBOSS);
-	uiDefButS(block, TOG|BIT|6, B_REDR, "Sel", xco+110, yco+35, (width-110)/3, 19, &G.buts->scaflag, 0, 0, 0, 0, "Show all selected Objects");
-	uiDefButS(block, TOG|BIT|7, B_REDR, "Act", xco+110+(width-110)/3, yco+35, (width-110)/3, 19, &G.buts->scaflag, 0, 0, 0, 0, "Show active Object");
-	uiDefButS(block, TOG|BIT|8, B_REDR, "Link", xco+110+2*(width-110)/3, yco+35, (width-110)/3, 19, &G.buts->scaflag, 0, 0, 0, 0, "Show linked Objects to Controller");
+	uiDefButBitS(block, TOG, BUTS_ACT_SEL, B_REDR, "Sel", xco+110, yco+35, (width-110)/3, 19, &G.buts->scaflag, 0, 0, 0, 0, "Show all selected Objects");
+	uiDefButBitS(block, TOG, BUTS_ACT_ACT, B_REDR, "Act", xco+110+(width-110)/3, yco+35, (width-110)/3, 19, &G.buts->scaflag, 0, 0, 0, 0, "Show active Object");
+	uiDefButBitS(block, TOG, BUTS_ACT_LINK, B_REDR, "Link", xco+110+2*(width-110)/3, yco+35, (width-110)/3, 19, &G.buts->scaflag, 0, 0, 0, 0, "Show linked Objects to Controller");
 	
 	for(a=0; a<count; a++) {
 		ob= (Object *)idar[a];
@@ -2701,9 +2684,9 @@ void logic_buts(void)
 		/* presume it is only objects for now */
 		uiBlockSetEmboss(block, UI_EMBOSS);
 		if(ob->actuators.first) uiSetCurFont(block, UI_HELVB);
-		uiDefButS(block, TOG|BIT|7, B_REDR, ob->id.name+2,(short)(xco-10), yco,(short)(width-30), 19, &ob->scaflag, 0, 31, 0, 0, "Object name, click to show/hide actuators");
+		uiDefButBitS(block, TOG, OB_SHOWACT, B_REDR, ob->id.name+2,(short)(xco-10), yco,(short)(width-30), 19, &ob->scaflag, 0, 31, 0, 0, "Object name, click to show/hide actuators");
 		if(ob->actuators.first) uiSetCurFont(block, UI_HELV);
-		uiDefButS(block, TOG|BIT|10, B_ADD_ACT, "Add",(short)(xco+width-40), yco, 50, 19, &ob->scaflag, 0, 0, 0, 0, "Add a new Actuator");
+		uiDefButBitS(block, TOG, OB_ADDACT, B_ADD_ACT, "Add",(short)(xco+width-40), yco, 50, 19, &ob->scaflag, 0, 0, 0, 0, "Add a new Actuator");
 		yco-=20;
 		
 		if(ob->scaflag & OB_SHOWACT) {
@@ -2711,8 +2694,8 @@ void logic_buts(void)
 			act= ob->actuators.first;
 			while(act) {
 				uiBlockSetEmboss(block, UI_EMBOSSM);
-				uiDefIconButS(block, TOG|BIT|1, B_DEL_ACT, ICON_X,	xco, yco, 22, 19, &act->flag, 0, 0, 0, 0, "Delete Actuator");
-				uiDefIconButS(block, ICONTOG|BIT|0, B_REDR, ICON_RIGHTARROW, (short)(xco+width-22), yco, 22, 19, &act->flag, 0, 0, 0, 0, "Actuator settings");
+				uiDefIconButBitS(block, TOG, ACT_DEL, B_DEL_ACT, ICON_X,	xco, yco, 22, 19, &act->flag, 0, 0, 0, 0, "Delete Actuator");
+				uiDefIconButBitS(block, ICONTOG, ACT_SHOW, B_REDR, ICON_RIGHTARROW, (short)(xco+width-22), yco, 22, 19, &act->flag, 0, 0, 0, 0, "Actuator settings");
 
 				if(act->flag & ACT_SHOW) {
 					act->otype= act->type;
