@@ -1,5 +1,5 @@
 /**
- * $Id: 
+ * $Id: buttons_script.c,v 1.14 2005/10/28 16:49:47 ton Exp $ 
  *
  * ***** BEGIN GPL/BL DUAL LICENSE BLOCK *****
  *
@@ -48,7 +48,6 @@
 #include "DNA_effect_types.h"
 #include "DNA_group_types.h"
 #include "DNA_ID.h"
-#include "DNA_ika_types.h"
 #include "DNA_image_types.h"
 #include "DNA_key_types.h"
 #include "DNA_lamp_types.h"
@@ -75,7 +74,6 @@
 #include "BKE_displist.h"
 #include "BKE_effect.h"
 #include "BKE_font.h"
-#include "BKE_ika.h"
 #include "BKE_image.h"
 #include "BKE_ipo.h"
 #include "BKE_lattice.h"
@@ -282,7 +280,7 @@ void draw_scriptlink(uiBlock *block, ScriptLink *script, int sx, int sy, int sce
 
 		uiDefButS(block, MENU, 1, str, (short)sx, (short)sy, 140, 19, &script->flag[script->actscript-1], 0, 0, 0, 0, "Script links for this event");
 
-		uiDefIDPoinBut(block, test_scriptpoin_but, 1, "", (short)(sx+140),(short)sy, 140, 19, &script->scripts[script->actscript-1], "Name of Script to link");
+		uiDefIDPoinBut(block, test_scriptpoin_but, ID_SCRIPT, 1, "", (short)(sx+140),(short)sy, 140, 19, &script->scripts[script->actscript-1], "Name of Script to link");
 	}
 
 	sprintf(str,"%d Scr:", script->totscript);
@@ -320,7 +318,7 @@ static void  script_panel_scriptlink(void)
 	block= uiNewBlock(&curarea->uiblocks, "script_panel_scriptlink", UI_EMBOSS, UI_HELV, curarea->win);
 	if(uiNewPanel(curarea, block, "Scriptlinks", "Script", 0, 0, 318, 204)==0) return;
 
-	uiDefButI(block, TOG|BIT|13, REDRAWBUTSSCRIPT,
+	uiDefButBitI(block, TOG, G_DOSCRIPTLINKS, REDRAWBUTSSCRIPT,
 			"Enable Script Links", xco, 200, 150, 20, &G.f, 0, 0, 0, 0,
 			"Enable execution of all assigned Script links");
 	/* for proper alignment: */
