@@ -1,5 +1,5 @@
 /**
-* $Id: cursors.c,v 1.6 2005/03/19 21:08:11 zuster Exp $
+* $Id: cursors.c,v 1.7 2005/07/05 00:46:52 theeth Exp $
 *
 * ***** BEGIN GPL/BL DUAL LICENSE BLOCK *****
 *
@@ -86,6 +86,10 @@ Setting big_bm=NULL disables the large version of the cursor.
 static BCursor *BlenderCursor[BC_NUMCURSORS]; /*Points to static BCursor Structs */
 static short CurrentCursor=-1, LastCursor=-1;
 
+short GetBlenderCursor(void) {
+	return CurrentCursor;
+}
+
 void SetBlenderCursor(short curs){
 	Window *win;
 
@@ -101,7 +105,8 @@ void SetBlenderCursor(short curs){
 	if (curs==LASTCURSOR) curs=LastCursor;
 
 	if (curs==SYSCURSOR) {  /* System default Cursor */
-		set_cursor(CURSOR_STD);
+		window_set_cursor(win, CURSOR_STD);
+		//set_cursor(CURSOR_STD);
 	}
 	else if ( (U.curssize==0) || (BlenderCursor[curs]->big_bm == NULL) ) {
 		window_set_custom_cursor_ex(win, BlenderCursor[curs], 0);
