@@ -1,5 +1,5 @@
 /**
- * $Id: drawaction.c,v 1.17 2004/11/11 13:17:32 ton Exp $
+ * $Id: drawaction.c,v 1.29 2005/10/31 09:41:24 ton Exp $
  *
  * ***** BEGIN GPL/BL DUAL LICENSE BLOCK *****
  *
@@ -40,10 +40,6 @@
 #include <config.h>
 #endif
 
-#ifdef WIN32
-#include "BLI_winstuff.h"
-#endif
-
 #include "MEM_guardedalloc.h"
 
 #include "BMF_Api.h"
@@ -63,53 +59,47 @@
 #include "DNA_key_types.h"
 
 #include "BKE_action.h"
+#include "BKE_ipo.h"
 #include "BKE_global.h"
 
 /* Everything from source (BIF, BDR, BSE) ------------------------------ */ 
 
+#include "BIF_editaction.h"
+#include "BIF_editkey.h"
+#include "BIF_interface.h"
 #include "BIF_gl.h"
 #include "BIF_glutil.h"
 #include "BIF_resources.h"
 #include "BIF_screen.h"
-#include "BIF_interface.h"
 #include "BIF_mywindow.h"
 #include "BIF_space.h"
 
-#include "BDR_editcurve.h"
-#include "BSE_view.h"
-#include "BSE_drawipo.h"
-#include "BSE_editaction.h"
-#include "BSE_editaction_types.h"
 #include "BDR_drawaction.h"
+#include "BDR_editcurve.h"
+
+#include "BSE_view.h"
+#include "BSE_drawnla.h"
+#include "BSE_drawipo.h"
 
 /* 'old' stuff": defines and types, and own include -------------------- */
 
 #include "blendef.h"
 #include "mydevice.h"
 
-#include "BKE_ipo.h"
 
 /* local functions ----------------------------------------------------- */
-void drawactionspace(ScrArea *sa, void *spacedata);
-static void draw_channel_names(void);
-static void draw_channel_strips(SpaceAction *saction);
 int count_action_levels(bAction *act);
 static BezTriple **ipo_to_keylist(Ipo *ipo, int flags, int *totvert);
 static BezTriple **action_to_keylist(bAction *act, int flags, int *totvert);
 static BezTriple **ob_to_keylist(Object *ob, int flags, int *totvert);
 static BezTriple **icu_to_keylist(IpoCurve *icu, int flags, int *totvert);
-static void draw_keylist(gla2DDrawInfo *di, int totvert, BezTriple **blist, float ypos);
 void draw_icu_channel(gla2DDrawInfo *di, IpoCurve *icu, int flags, float ypos);
-static void draw_action_mesh_names(Key *key);
 
 /* implementation ------------------------------------------------------ */
-
-extern void make_rvk_slider(uiBlock *block, Key *key, int i,
-							int x, int y, int w, int h); /* editkey.c */
 extern short showsliders; /* editaction .c */
 extern short ACTWIDTH;
 
-void meshactionbuts(SpaceAction *saction, Key *key)
+static void meshactionbuts(SpaceAction *saction, Object *ob, Key *key)
 {
 	int           i;
 	char          str[64];
@@ -176,8 +166,8 @@ void meshactionbuts(SpaceAction *saction, Key *key)
 		glRects(NAMEWIDTH,  0,  NAMEWIDTH+SLIDERWIDTH,  curarea->winy);
 		uiBlockSetEmboss(block, UI_EMBOSS);
 		for (i=1 ; i < key->totkey ; ++ i) {
-			make_rvk_slider(block, key, i, 
-							x, y, SLIDERWIDTH-2, CHANNELHEIGHT-1);
+			make_rvk_slider(block, ob, i, 
+							x, y, SLIDERWIDTH-2, CHANNELHEIGHT-1, "Slider to control Shape Keys");
 
 			y-=CHANNELHEIGHT+CHANNELSKIP;
 			
@@ -361,41 +351,6 @@ int count_action_levels(bAction *act)
 
 	return y;
 }
-	/** Draw a nicely beveled button (in screen space) */
-void draw_bevel_but(int x, int y, int w, int h, int sel)
-{
-	int xmin= x, ymin= y;
-	int xmax= x+w-1, ymax= y+h-1;
-	int i;
-
-	glColor3ub(0,0,0);
-	glBegin(GL_LINE_LOOP);
-	glVertex2i(xmin, ymin);
-	glVertex2i(xmax, ymin);
-	glVertex2i(xmax, ymax);
-	glVertex2i(xmin, ymax);
-	glEnd();
-
-	glBegin(GL_LINE_LOOP);
-	if (sel) glColor3ub(0xD0, 0x7E, 0x06);
-	else glColor3ub(0x8C, 0x8C, 0x8C);
-	glVertex2i(xmax-1, ymin+1);
-	glVertex2i(xmax-1, ymax-1);
-	if (sel) glColor3ub(0xF4, 0xEE, 0x8E);
-	else glColor3ub(0xDF, 0xDF, 0xDF);
-	glVertex2i(xmin+1, ymax-1);
-	glVertex2i(xmin+1, ymin+1);
-	glEnd();
-
-	if (sel) glColor3ub(0xF1, 0xCA, 0x13);
-	else glColor3ub(0xAC, 0xAC, 0xAC);
-	glBegin(GL_LINES);
-	for (i=xmin+2; i<=xmax-2; i++) {
-		glVertex2f(i, ymin+2);
-		glVertex2f(i, ymax-1);
-	}
-	glEnd();
-}
 
 static void draw_channel_strips(SpaceAction *saction)
 {
@@ -404,7 +359,8 @@ static void draw_channel_strips(SpaceAction *saction)
 	bAction	*act;
 	bActionChannel *chan;
 	bConstraintChannel *conchan;
-	float	y;
+	float y, sta, end;
+	int act_start, act_end, dummy;
 	char col1[3], col2[3];
 	
 	BIF_GetThemeColor3ubv(TH_SHADE2, col2);
@@ -414,53 +370,86 @@ static void draw_channel_strips(SpaceAction *saction)
 	if (!act)
 		return;
 
-	scr_rct.xmin= saction->area->winrct.xmin + ACTWIDTH;
+	scr_rct.xmin= saction->area->winrct.xmin + saction->v2d.mask.xmin;
 	scr_rct.ymin= saction->area->winrct.ymin + saction->v2d.mask.ymin;
 	scr_rct.xmax= saction->area->winrct.xmin + saction->v2d.hor.xmax;
 	scr_rct.ymax= saction->area->winrct.ymin + saction->v2d.mask.ymax; 
 	di= glaBegin2DDraw(&scr_rct, &G.v2d->cur);
 
+	/* if in NLA there's a strip active, map the view */
+	if (G.saction->pin==0 && OBACT)
+		map_active_strip(di, OBACT, 0);
+	
+	/* start and end of action itself */
+	calc_action_range(act, &sta, &end);
+	gla2DDrawTranslatePt(di, sta, 0.0f, &act_start, &dummy);
+	gla2DDrawTranslatePt(di, end, 0.0f, &act_end, &dummy);
+	
+	if (G.saction->pin==0 && OBACT)
+		map_active_strip(di, OBACT, 1);
+	
+	/* first backdrop strips */
 	y= count_action_levels(act)*(CHANNELHEIGHT+CHANNELSKIP);
-
+	glEnable(GL_BLEND);
 	for (chan=act->chanbase.first; chan; chan=chan->next){
 		int frame1_x, channel_y;
-
-		gla2DDrawTranslatePt(di, 1, y, &frame1_x, &channel_y);
-
-		glEnable(GL_BLEND);
-		if (chan->flag & ACHAN_SELECTED) glColor4b(col1[0], col1[1], col1[2], 0x22);
-		else glColor4b(col2[0], col2[1], col2[2], 0x22);
-		glRectf(0,  channel_y-CHANNELHEIGHT/2,  frame1_x,  channel_y+CHANNELHEIGHT/2);
-
-		if (chan->flag & ACHAN_SELECTED) glColor4b(col1[0], col1[1], col1[2], 0x44);
-		else glColor4b(col2[0], col2[1], col2[2], 0x44);
+		
+		gla2DDrawTranslatePt(di, G.v2d->cur.xmin, y, &frame1_x, &channel_y);
+		
+		if (chan->flag & ACHAN_SELECTED) glColor4ub(col1[0], col1[1], col1[2], 0x22);
+		else glColor4ub(col2[0], col2[1], col2[2], 0x22);
 		glRectf(frame1_x,  channel_y-CHANNELHEIGHT/2,  G.v2d->hor.xmax,  channel_y+CHANNELHEIGHT/2);
-		glDisable(GL_BLEND);
-	
-		draw_ipo_channel(di, chan->ipo, 0, y);
-
+		
+		if (chan->flag & ACHAN_SELECTED) glColor4ub(col1[0], col1[1], col1[2], 0x22);
+		else glColor4ub(col2[0], col2[1], col2[2], 0x22);
+		glRectf(act_start,  channel_y-CHANNELHEIGHT/2,  act_end,  channel_y+CHANNELHEIGHT/2);
+		
 		/*	Increment the step */
 		y-=CHANNELHEIGHT+CHANNELSKIP;
-
-
+		
 		/* Draw constraint channels */
 		for (conchan=chan->constraintChannels.first; conchan; conchan=conchan->next){
 			gla2DDrawTranslatePt(di, 1, y, &frame1_x, &channel_y);
-			glEnable(GL_BLEND);
-			if (conchan->flag & ACHAN_SELECTED) glColor4b(col1[0], col1[1], col1[2], 0x22);
-			else glColor4b(col2[0], col2[1], col2[2], 0x22);
-			glRectf(0,  channel_y-CHANNELHEIGHT/2+4,  frame1_x,  channel_y+CHANNELHEIGHT/2-4);
 			
-			if (conchan->flag & ACHAN_SELECTED) glColor4b(col1[0], col1[1], col1[2], 0x44);
-			else glColor4b(col2[0], col2[1], col2[2], 0x44);
+			if (conchan->flag & ACHAN_SELECTED) glColor4ub(col1[0], col1[1], col1[2], 0x22);
+			else glColor4ub(col2[0], col2[1], col2[2], 0x22);
 			glRectf(frame1_x,  channel_y-CHANNELHEIGHT/2+4,  G.v2d->hor.xmax,  channel_y+CHANNELHEIGHT/2-4);
-			glDisable(GL_BLEND);
 			
+			if (conchan->flag & ACHAN_SELECTED) glColor4ub(col1[0], col1[1], col1[2], 0x22);
+			else glColor4ub(col2[0], col2[1], col2[2], 0x22);
+			glRectf(act_start,  channel_y-CHANNELHEIGHT/2+4,  act_end,  channel_y+CHANNELHEIGHT/2-4);
+			
+			y-=CHANNELHEIGHT+CHANNELSKIP;
+		}
+	}		
+	glDisable(GL_BLEND);
+	
+	if (G.saction->pin==0 && OBACT)
+		map_active_strip(di, OBACT, 0);
+	
+	/* dot thingies */
+	y= count_action_levels(act)*(CHANNELHEIGHT+CHANNELSKIP);
+	for (chan=act->chanbase.first; chan; chan=chan->next){
+		draw_ipo_channel(di, chan->ipo, 0, y);
+		y-=CHANNELHEIGHT+CHANNELSKIP;
+
+		/* Draw constraint channels */
+		for (conchan=chan->constraintChannels.first; conchan; conchan=conchan->next){
 			draw_ipo_channel(di, conchan->ipo, 0, y);
 			y-=CHANNELHEIGHT+CHANNELSKIP;
 		}
 	}
 
+	if(saction->flag & SACTION_MOVING) {
+		int frame1_x, channel_y;
+		gla2DDrawTranslatePt(di, saction->timeslide, 0, &frame1_x, &channel_y);
+		cpack(0x0);
+		glBegin(GL_LINES);
+		glVertex2f(frame1_x, G.v2d->mask.ymin - 100);
+		glVertex2f(frame1_x, G.v2d->mask.ymax);
+		glEnd();
+	}
+	
 	glaEnd2DDraw(di);
 }
 
@@ -502,13 +491,13 @@ static void draw_mesh_strips(SpaceAction *saction, Key *key)
 		 * get a desaturated orange background
 		 */
 		glEnable(GL_BLEND);
-		glColor4b(col2[0], col2[1], col2[2], 0x22);
+		glColor4ub(col2[0], col2[1], col2[2], 0x22);
 		glRectf(0,        channel_y-CHANNELHEIGHT/2,  
 				frame1_x, channel_y+CHANNELHEIGHT/2);
 
 		/* frames one and higher get a saturated orange background
 		 */
-		glColor4b(col2[0], col2[1], col2[2], 0x44);
+		glColor4ub(col2[0], col2[1], col2[2], 0x44);
 		glRectf(frame1_x,         channel_y-CHANNELHEIGHT/2,  
 				G.v2d->hor.xmax,  channel_y+CHANNELHEIGHT/2);
 		glDisable(GL_BLEND);
@@ -612,7 +601,7 @@ void drawactionspace(ScrArea *sa, void *spacedata)
 
 	glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA) ;
 
-	calc_scrollrcts(G.v2d, curarea->winx, curarea->winy);
+	calc_scrollrcts(sa, G.v2d, curarea->winx, curarea->winy);
 
 	/* background color for entire window (used in lefthand part tho) */
 	BIF_GetThemeColor3fv(TH_HEADER, col);
@@ -652,10 +641,8 @@ void drawactionspace(ScrArea *sa, void *spacedata)
 		 * then draw the key frames in the action window
 		 */
 		draw_mesh_strips(G.saction, key);
-		/*meshactionbuts(G.saction, key);*/
 	}
 
-
 	/* Draw current frame */
 	glViewport(ofsx+G.v2d->mask.xmin,  
              ofsy+G.v2d->mask.ymin, 
@@ -675,16 +662,20 @@ void drawactionspace(ScrArea *sa, void *spacedata)
       if(G.v2d->scroll) drawscroll(0);
 	}
 
-	/* Draw channel names */
-	draw_channel_names();
+	if(G.v2d->mask.xmin!=0) {
+		/* Draw channel names */
+		draw_channel_names();
 
-	if ( key ) {
-		/* if there is a mesh with rvk's selected,
-		 * then draw the key frames in the action window
-		 */
-		meshactionbuts(G.saction, key);
+		if(sa->winx > 50 + NAMEWIDTH + SLIDERWIDTH) {
+			if ( key ) {
+				/* if there is a mesh with rvk's selected,
+				 * then draw the key frames in the action window
+				 */
+				meshactionbuts(G.saction, OBACT, key);
+			}
+		}
 	}
-
+	
 	mywinset(curarea->win);	// reset scissor too
 	myortho2(-0.375, curarea->winx-0.375, -0.375, curarea->winy-0.375);
 	draw_area_emboss(sa);
@@ -696,11 +687,54 @@ void drawactionspace(ScrArea *sa, void *spacedata)
 	curarea->win_swap= WIN_BACK_OK;
 }
 
-
-void draw_channel_name(const char* name, short type, float ypos, int selected)
+#if 0
+/** Draw a nicely beveled diamond shape (in screen space) */
+static void draw_key_but(int x, int y, int w, int h, int sel)
 {
+	int xmin= x, ymin= y;
+	int xmax= x+w-1, ymax= y+h-1;
+	int xc= (xmin+xmax)/2, yc= (ymin+ymax)/2;
+	
+	/* interior */
+	if (sel) glColor3ub(0xF1, 0xCA, 0x13);
+	else glColor3ub(0xAC, 0xAC, 0xAC);
+	
+	glBegin(GL_QUADS);
+	glVertex2i(xc, ymin);
+	glVertex2i(xmax, yc);
+	glVertex2i(xc, ymax);
+	glVertex2i(xmin, yc);
+	glEnd();
+
+	
+	/* bevel */
+	glBegin(GL_LINE_LOOP);
+	
+	if (sel) glColor3ub(0xD0, 0x7E, 0x06);
+	else glColor3ub(0x8C, 0x8C, 0x8C);
+	/* under */
+	glVertex2i(xc, ymin+1);
+	glVertex2i(xmax-1, yc);
+	
+	if (sel) glColor3ub(0xF4, 0xEE, 0x8E);
+	else glColor3ub(0xDF, 0xDF, 0xDF);
+	/* top */
+	glVertex2i(xc, ymax-1);
+	glVertex2i(xmin+1, yc);
+	glEnd();
+	
+	/* outline */
+	glColor3ub(0,0,0);
+	glBegin(GL_LINE_LOOP);
+	glVertex2i(xc, ymin);
+	glVertex2i(xmax, yc);
+	glVertex2i(xc, ymax);
+	glVertex2i(xmin, yc);
+	glEnd();
 }
 
+#endif
+
 static void draw_keylist(gla2DDrawInfo *di, int totvert, BezTriple **blist, float ypos)
 {
 	int v;
@@ -708,13 +742,20 @@ static void draw_keylist(gla2DDrawInfo *di, int totvert, BezTriple **blist, floa
 	if (!blist)
 		return;
 
+	glEnable(GL_BLEND);
+	
 	for (v = 0; v<totvert; v++){
 		if (v==0 || (blist[v]->vec[1][0] != blist[v-1]->vec[1][0])){
 			int sc_x, sc_y;
 			gla2DDrawTranslatePt(di, blist[v]->vec[1][0], ypos, &sc_x, &sc_y);
-			draw_bevel_but(sc_x-2, sc_y-5, 7, 13, (blist[v]->f2 & 1));
+			// draw_key_but(sc_x-5, sc_y-6, 13, 13, (blist[v]->f2 & 1));
+			
+			if(blist[v]->f2 & 1) BIF_draw_icon_blended(sc_x-7, sc_y-6, ICON_SPACE2, TH_HEADER, 0);
+			else BIF_draw_icon_blended(sc_x-7, sc_y-6, ICON_SPACE3, TH_HEADER, 0);
+
 		}
 	}			
+	glDisable(GL_BLEND);
 }
 
 void draw_object_channel(gla2DDrawInfo *di, Object *ob, int flags, float ypos)
@@ -770,6 +811,7 @@ void draw_action_channel(gla2DDrawInfo *di, bAction *act, int flags, float ypos)
 static BezTriple **ob_to_keylist(Object *ob, int flags, int *totvert)
 {
 	IpoCurve *icu;
+	bConstraintChannel *conchan;
 	int v, count=0;
 
 	BezTriple **list = NULL;
@@ -784,6 +826,12 @@ static BezTriple **ob_to_keylist(Object *ob, int flags, int *totvert)
 		}
 		
 		/* Count Constraint Keys */
+		for (conchan=ob->constraintChannels.first; conchan; conchan=conchan->next){
+			if(conchan->ipo)
+				for (icu=conchan->ipo->curve.first; icu; icu=icu->next)
+					count+=icu->totvert;
+		}
+		
 		/* Count object data keys */
 
 		/* Build the list */
@@ -792,13 +840,21 @@ static BezTriple **ob_to_keylist(Object *ob, int flags, int *totvert)
 			count=0;
 			
 			/* Add object keyframes */
-			for (icu=ob->ipo->curve.first; icu; icu=icu->next){
-				for (v=0; v<icu->totvert; v++){
-					list[count++]=&icu->bezt[v];
+			if(ob->ipo) {
+				for (icu=ob->ipo->curve.first; icu; icu=icu->next){
+					for (v=0; v<icu->totvert; v++){
+						list[count++]=&icu->bezt[v];
+					}
 				}
+			}			
+			/* Add constraint keyframes */
+			for (conchan=ob->constraintChannels.first; conchan; conchan=conchan->next){
+				if(conchan->ipo)
+					for (icu=conchan->ipo->curve.first; icu; icu=icu->next)
+						for (v=0; v<icu->totvert; v++)
+							list[count++]=&icu->bezt[v];
 			}
 			
-			/* Add constraint keyframes */
 			/* Add object data keyframes */
 
 			/* Sort */
@@ -880,13 +936,13 @@ static BezTriple **action_to_keylist(bAction *act, int flags, int *totvert)
 			if(achan->ipo) {
 				for (icu=achan->ipo->curve.first; icu; icu=icu->next)
 					count+=icu->totvert;
-				
-				/* Count constraint keys */
-				for (conchan=achan->constraintChannels.first; conchan; conchan=conchan->next)
+			}
+			/* Count constraint keys */
+			for (conchan=achan->constraintChannels.first; conchan; conchan=conchan->next)
+				if(conchan->ipo) 
 					for (icu=conchan->ipo->curve.first; icu; icu=icu->next)
 						count+=icu->totvert;
 				
-			}
 		}
 		
 		/* Build the list */
@@ -895,17 +951,19 @@ static BezTriple **action_to_keylist(bAction *act, int flags, int *totvert)
 			count=0;
 			
 			for (achan=act->chanbase.first; achan; achan=achan->next){
-				/* Add transformation keys */
-				for (icu=achan->ipo->curve.first; icu; icu=icu->next){
-					for (v=0; v<icu->totvert; v++)
-						list[count++]=&icu->bezt[v];
-				}
-					
-					/* Add constraint keys */
-				for (conchan=achan->constraintChannels.first; conchan; conchan=conchan->next){
-					for (icu=conchan->ipo->curve.first; icu; icu=icu->next)
+				if(achan->ipo) {
+					/* Add transformation keys */
+					for (icu=achan->ipo->curve.first; icu; icu=icu->next){
 						for (v=0; v<icu->totvert; v++)
 							list[count++]=&icu->bezt[v];
+					}
+				}					
+					/* Add constraint keys */
+				for (conchan=achan->constraintChannels.first; conchan; conchan=conchan->next){
+					if(conchan->ipo)
+						for (icu=conchan->ipo->curve.first; icu; icu=icu->next)
+							for (v=0; v<icu->totvert; v++)
+								list[count++]=&icu->bezt[v];
 				}
 							
 			}		
