@@ -1,5 +1,5 @@
 /**
- * $Id: drawimage.c,v 1.24 2004/10/14 14:20:24 ton Exp $
+ * $Id: drawimage.c,v 1.47 2005/10/22 22:50:09 bjornmose Exp $
  *
  * ***** BEGIN GPL/BL DUAL LICENSE BLOCK *****
  *
@@ -39,12 +39,12 @@
 
 #ifdef WIN32
 #include <io.h>
-#include "BLI_winstuff.h"
 #else
 #include <unistd.h>
 #endif   
 #include "MEM_guardedalloc.h"
 
+#include "BLI_arithb.h"
 #include "BLI_blenlib.h"
 
 #include "IMB_imbuf_types.h"
@@ -53,104 +53,96 @@
 #include "DNA_mesh_types.h"
 #include "DNA_meshdata_types.h"
 #include "DNA_packedFile_types.h"
+#include "DNA_scene_types.h"
 #include "DNA_screen_types.h"
 #include "DNA_space_types.h"
 #include "DNA_userdef_types.h"
 
 #include "BKE_utildefines.h"
 #include "BKE_global.h"
+#include "BKE_main.h"
 #include "BKE_mesh.h"
 #include "BKE_image.h"
+#include "BKE_DerivedMesh.h"
+#include "BKE_displist.h"
+#include "BKE_object.h"
 
 #include "BDR_editface.h"
-#include "BDR_editobject.h"
+#include "BDR_drawobject.h"
 #include "BDR_drawmesh.h"
+#include "BDR_imagepaint.h"
 
-	
 #include "BIF_gl.h"
-#include "BIF_space.h"
-#include "BIF_screen.h"
 #include "BIF_mywindow.h"
 #include "BIF_drawimage.h"
 #include "BIF_resources.h"
 #include "BIF_interface.h"
 #include "BIF_editsima.h"
 #include "BIF_glutil.h"
+#include "BIF_space.h"
+#include "BIF_screen.h"
+#include "BIF_transform.h"
 
+#include "BSE_headerbuttons.h"
 #include "BSE_trans_types.h"
+#include "BSE_view.h"
 
 /* Modules used */
 #include "mydevice.h"
 #include "blendef.h"
-#include "render.h"
 #include "butspace.h"  // event codes
 
-
-void rectwrite_part(int winxmin, int winymin, int winxmax, int winymax, int x1, int y1, int xim, int yim, float zoomx, float zoomy, unsigned int *rect)
+static unsigned char *alloc_alpha_clone_image(int *width, int *height)
 {
-	int cx, cy, oldxim, x2, y2;
-	
-	oldxim= xim;
-		
-	/* coordinates how its drawn at the screen */
-	x2= x1+ zoomx*xim;
-	y2= y1+ zoomy*yim;
+	unsigned int size, alpha;
+	unsigned char *rect, *cp;
 
-	/* partial clip */
-	if(x1<winxmin) {
-		/* with OpenGL, rects are not allowed to start outside of the left/bottom window edge */
-		cx= winxmin-x1+(int)zoomx;
-		/* make sure the rect will be drawn pixel-exact */
-		cx/= zoomx;
-		cx++;
-		x1+= zoomx*cx;
-		xim-= cx;
-		rect+= cx;
+	if(!Gip.clone.image)
+		return NULL;
+
+	if(!Gip.clone.image->ibuf)
+		load_image(Gip.clone.image, IB_rect, G.sce, G.scene->r.cfra);
+
+	if(!Gip.clone.image->ibuf || !Gip.clone.image->ibuf->rect)
+		return NULL;
+
+	rect= MEM_dupallocN(Gip.clone.image->ibuf->rect);
+
+	if(!rect)
+		return NULL;
+
+	*width= Gip.clone.image->ibuf->x;
+	*height= Gip.clone.image->ibuf->y;
+
+	size= (*width)*(*height);
+	alpha= (unsigned char)255*Gip.clone.alpha;
+	cp= rect;
+
+	while(size-- > 0) {
+		cp[3]= alpha;
+		cp += 4;
 	}
-	if(y1<winymin) {
-		cy= winymin-y1+(int)zoomy;
-		cy/= zoomy;
-		cy++;
-		y1+= zoomy*cy;
-		rect+= cy*oldxim;
-		yim-= cy;
+
+	return rect;
+}
+
+static void setcloneimage()
+{
+	if(G.sima->menunr > 0) {
+		Image *ima= (Image*)BLI_findlink(&G.main->image, G.sima->menunr-1);
+
+		if(ima) {
+			Gip.clone.image= ima;
+			Gip.clone.offset[0]= Gip.clone.offset[0]= 0.0;
+		}
 	}
-	if(x2>=winxmax) {
-		cx= x2-winxmax;
-		cx/= zoomx;
-		xim-= cx+3;
-	}
-	if(y2>=winymax) {
-		cy= y2-winymax;
-		cy/= zoomy;
-		yim-= cy+3;
-	}
-	
-	if(xim<=0) return;
-	if(yim<=0) return;
-
-	mywinset(G.curscreen->mainwin);
-	glScissor(winxmin, winymin, winxmax-winxmin+1, winymax-winymin+1);
-	
-	glPixelStorei(GL_UNPACK_ROW_LENGTH,  oldxim);
-	
-	glPixelZoom(zoomx,  zoomy);
-
-	glRasterPos2i(x1, y1);
-	glDrawPixels(xim, yim, GL_RGBA, GL_UNSIGNED_BYTE,  rect);
-
-	glPixelZoom(1.0,  1.0);
-
-	glPixelStorei(GL_UNPACK_ROW_LENGTH,  0);
-	
-	mywinset(curarea->win);
 }
 
 /**
  * Sets up the fields of the View2D member of the SpaceImage struct
  * This routine can be called in two modes:
- * mode == 'f': float mode ???
- * mode == 'p': pixel mode ???
+ * mode == 'f': float mode (0.0 - 1.0)
+ * mode == 'p': pixel mode (0 - size)
  *
  * @param     sima  the image space to update
  * @param     mode  the mode to use for the update
@@ -242,7 +234,7 @@ void image_changed(SpaceImage *sima, int dotile)
 	if(sima->mode==SI_TEXTURE) {
 		
 		if(G.f & G_FACESELECT) {
-			me= get_mesh((G.scene->basact) ? (G.scene->basact->object) : 0);
+			me= get_mesh(OBACT);
 			if(me && me->tface) {
 				tface= me->tface;
 				a= me->totface;
@@ -268,7 +260,8 @@ void image_changed(SpaceImage *sima, int dotile)
 					}
 					tface++;
 				}
-				allqueue(REDRAWVIEW3D, 0);
+
+				object_uvs_changed(OBACT);
 				allqueue(REDRAWBUTSEDIT, 0);
 			}
 		}
@@ -280,7 +273,7 @@ void uvco_to_areaco(float *vec, short *mval)
 {
 	float x, y;
 
-	mval[0]= 3200;
+	mval[0]= IS_CLIPPED;
 	
 	x= (vec[0] - G.v2d->cur.xmin)/(G.v2d->cur.xmax-G.v2d->cur.xmin);
 	y= (vec[1] - G.v2d->cur.ymin)/(G.v2d->cur.ymax-G.v2d->cur.ymin);
@@ -293,11 +286,11 @@ void uvco_to_areaco(float *vec, short *mval)
 	}
 }
 
-void uvco_to_areaco_noclip(float *vec, short *mval)
+void uvco_to_areaco_noclip(float *vec, int *mval)
 {
 	float x, y;
 
-	mval[0]= 3200;
+	mval[0]= IS_CLIPPED;
 	
 	x= (vec[0] - G.v2d->cur.xmin)/(G.v2d->cur.xmax-G.v2d->cur.xmin);
 	y= (vec[1] - G.v2d->cur.ymin)/(G.v2d->cur.ymax-G.v2d->cur.ymin);
@@ -326,14 +319,22 @@ void draw_tfaces(void)
 			glLoadIdentity();
 			
 			/* draw shadow mesh */
-			if(G.sima->flag & SI_DRAWSHADOW){		
+			if((G.sima->flag & SI_DRAWSHADOW) && !(G.obedit==OBACT)){
+				int dmNeedsFree;
+				DerivedMesh *dm = mesh_get_derived_final(OBACT, &dmNeedsFree);
+
+				glColor3ub(112, 112, 112);
+				if (dm->drawUVEdges) dm->drawUVEdges(dm);
+
+				if (dmNeedsFree) dm->release(dm);
+			}
+			else if(G.sima->flag & SI_DRAWSHADOW){		
 				tface= me->tface;
 				mface= me->mface;
 				a= me->totface;			
 				while(a--) {
-					if(tface->flag & TF_HIDE);
-					else if(mface->v3) {
-						cpack(0x707070);
+					if(!(tface->flag & TF_HIDE)) {
+						glColor3ub(112, 112, 112);
 						glBegin(GL_LINE_LOOP);
 						glVertex2fv(tface->uv[0]);
 						glVertex2fv(tface->uv[1]);
@@ -356,7 +357,7 @@ void draw_tfaces(void)
 				mface= me->mface;
 				a= me->totface;			
 				while(a--) {
-					if(mface->v3 && (tface->flag & TF_SELECT)) {
+					if(tface->flag & TF_SELECT) {
 						if(!(~tface->flag & (TF_SEL1|TF_SEL2|TF_SEL3)) &&
 						   (!mface->v4 || tface->flag & TF_SEL4))
 							glColor4ubv(col2);
@@ -381,7 +382,7 @@ void draw_tfaces(void)
 			mface= me->mface;
 			a= me->totface;
 			while(a--) {
-				if(mface->v3 && (tface->flag & TF_SELECT) ) {
+				if(tface->flag & TF_SELECT) {
 					if(tface->flag & TF_ACTIVE){
 						activetface= tface; 
 						activemface= mface; 
@@ -468,7 +469,7 @@ void draw_tfaces(void)
 			mface= me->mface;
 			a= me->totface;
 			while(a--) {
-				if(mface->v3  && (tface->flag & TF_SELECT) ) {
+				if(tface->flag & TF_SELECT) {
 					
 					if(tface->flag & TF_SEL1); else bglVertex2fv(tface->uv[0]);
 					if(tface->flag & TF_SEL2); else bglVertex2fv(tface->uv[1]);
@@ -492,7 +493,7 @@ void draw_tfaces(void)
 			mface= me->mface;
 			a= me->totface;
 			while(a--) {
-				if(mface->v3  && (tface->flag & TF_SELECT) ) {
+				if(tface->flag & TF_SELECT) {
 					
 					if(tface->unwrap & TF_PIN1) bglVertex2fv(tface->uv[0]);
 					if(tface->unwrap & TF_PIN2) bglVertex2fv(tface->uv[1]);
@@ -515,7 +516,7 @@ void draw_tfaces(void)
 			mface= me->mface;
 			a= me->totface;
 			while(a--) {
-				if(mface->v3  && (tface->flag & TF_SELECT) ) {
+				if(tface->flag & TF_SELECT) {
 					
 					if(tface->flag & TF_SEL1) bglVertex2fv(tface->uv[0]);
 					if(tface->flag & TF_SEL2) bglVertex2fv(tface->uv[1]);
@@ -555,12 +556,12 @@ static unsigned int *get_part_from_ibuf(ImBuf *ibuf, short startx, short starty,
 	return rectmain;
 }
 
-static void draw_image_prop_circle(ImBuf *ibuf)
+static void draw_image_transform(ImBuf *ibuf)
 {
-	float aspx, aspy;
-	extern float prop_cent[3];
+	if(G.moving) {
+		float aspx, aspy, center[3];
 
-	if(G.moving && G.f & G_PROPORTIONAL) {
+        BIF_drawConstraint();
 
 		if(ibuf==0 || ibuf->rect==0 || ibuf->x==0 || ibuf->y==0) {
 			aspx= aspy= 1.0;
@@ -570,40 +571,71 @@ static void draw_image_prop_circle(ImBuf *ibuf)
 			aspy= 256.0/ibuf->y;
 		}
 
+		BIF_getPropCenter(center);
+
 		/* scale and translate the circle into place and draw it */
 		glPushMatrix();
 		glScalef(aspx, aspy, 1.0);
-		glTranslatef((1/aspx)*prop_cent[0] - prop_cent[0],
-		             (1/aspy)*prop_cent[1] - prop_cent[1], 0.0);
-		draw_prop_circle();
+		glTranslatef((1/aspx)*center[0] - center[0],
+		             (1/aspy)*center[1] - center[1], 0.0);
+
+		BIF_drawPropCircle();
+
 		glPopMatrix();
 	}
 }
 
 static void draw_image_view_icon(void)
 {
+	float xPos = 5.0;
+
 	glEnable(GL_BLEND);
 	glBlendFunc(GL_SRC_ALPHA,  GL_ONE_MINUS_SRC_ALPHA); 
 	
-	glRasterPos2f(5.0, 5.0);
-
 	if(G.sima->flag & SI_STICKYUVS) {
-		BIF_draw_icon(ICON_STICKY2_UVS);
-		glRasterPos2f(25.0, 5.0);
+		BIF_draw_icon(xPos, 5.0, ICON_STICKY2_UVS);
+		xPos = 25.0;
 	}
 	else if(G.sima->flag & SI_LOCALSTICKY) {
-		BIF_draw_icon(ICON_STICKY_UVS);
-		glRasterPos2f(25.0, 5.0);
+		BIF_draw_icon(xPos, 5.0, ICON_STICKY_UVS);
+		xPos = 25.0;
 	}
 
 	if(G.sima->flag & SI_SELACTFACE) {
-			BIF_draw_icon(ICON_DRAW_UVFACES);
+		BIF_draw_icon(xPos, 5.0, ICON_DRAW_UVFACES);
 	}
 	
 	glBlendFunc(GL_ONE,  GL_ZERO); 
 	glDisable(GL_BLEND);
 }
 
+static void draw_image_view_tool(void)
+{
+	ImagePaintTool *tool = &Gip.tool[Gip.current];
+	short mval[2];
+	float radius;
+	int draw= 0;
+
+	if(Gip.flag & IMAGEPAINT_DRAWING) {
+		if(Gip.flag & IMAGEPAINT_DRAW_TOOL_DRAWING)
+			draw= 1;
+	}
+	else if(Gip.flag & IMAGEPAINT_DRAW_TOOL)
+		draw= 1;
+	
+	if(draw) {
+		getmouseco_areawin(mval);
+
+		radius= tool->size*G.sima->zoom/2;
+		fdrawXORcirc(mval[0], mval[1], radius);
+
+		if (tool->innerradius != 1.0) {
+			radius *= tool->innerradius;
+			fdrawXORcirc(mval[0], mval[1], radius);
+		}
+	}
+}
+
 /* ************ panel stuff ************* */
 
 // button define is local, only events defined here possible
@@ -615,7 +647,7 @@ static void image_editvertex_buts(uiBlock *block)
 	static float ocent[2];
 	float cent[2]= {0.0, 0.0};
 	int imx, imy;
-	int i, nactive= 0;
+	int i, nactive= 0, step, digits;
 	Mesh *me;
 	
 	if( is_uv_tface_editing_allowed_silent()==0 ) return;
@@ -631,7 +663,7 @@ static void image_editvertex_buts(uiBlock *block)
 		MFace *mf= &((MFace*) me->mface)[i];
 		TFace *tf= &((TFace*) me->tface)[i];
 		
-		if (!mf->v3 || !(tf->flag & TF_SELECT))
+		if (!(tf->flag & TF_SELECT))
 			continue;
 		
 		if (tf->flag & TF_SEL1) {
@@ -658,38 +690,52 @@ static void image_editvertex_buts(uiBlock *block)
 		
 	if(block) {	// do the buttons
 		if (nactive) {
-			ocent[0]= (cent[0]*imx)/nactive;
-			ocent[1]= (cent[1]*imy)/nactive;
-			
-			uiDefBut(block, LABEL, 0, "UV Vertex:",10,55,302,19,0,0,0,0,0,"");
-			if(nactive==1) {
-				uiBlockBeginAlign(block);
-				uiDefButF(block, NUM, B_TRANS_IMAGE, "Vertex X:",	10, 35, 290, 19, &ocent[0], -10*imx, 10.0*imx, 100, 0, "");
-				uiDefButF(block, NUM, B_TRANS_IMAGE, "Vertex Y:",	10, 15, 290, 19, &ocent[1], -10*imy, 10.0*imy, 100, 0, "");
-				uiBlockEndAlign(block);
+			ocent[0]= cent[0]/nactive;
+			ocent[1]= cent[1]/nactive;
+			if (G.sima->flag & SI_COORDFLOATS) {
+				step= 1;
+				digits= 3;
 			}
 			else {
-				uiBlockBeginAlign(block);
-				uiDefButF(block, NUM, B_TRANS_IMAGE, "Median X:",	10, 35, 290, 19, &ocent[0], -10*imx, 10.0*imx, 100, 0, "");
-				uiDefButF(block, NUM, B_TRANS_IMAGE, "Median Y:",	10, 15, 290, 19, &ocent[1], -10*imy, 10.0*imy, 100, 0, "");
-				uiBlockEndAlign(block);
+				ocent[0] *= imx;
+				ocent[1] *= imy;
+				step= 100;
+				digits= 2;
 			}
+			
+			uiDefBut(block, LABEL, 0, "UV Vertex:",10,55,302,19,0,0,0,0,0,"");
+			uiBlockBeginAlign(block);
+			if(nactive==1) {
+				uiDefButF(block, NUM, B_TRANS_IMAGE, "Vertex X:",	10, 35, 290, 19, &ocent[0], -10*imx, 10.0*imx, step, digits, "");
+				uiDefButF(block, NUM, B_TRANS_IMAGE, "Vertex Y:",	10, 15, 290, 19, &ocent[1], -10*imy, 10.0*imy, step, digits, "");
+			}
+			else {
+				uiDefButF(block, NUM, B_TRANS_IMAGE, "Median X:",	10, 35, 290, 19, &ocent[0], -10*imx, 10.0*imx, step, digits, "");
+				uiDefButF(block, NUM, B_TRANS_IMAGE, "Median Y:",	10, 15, 290, 19, &ocent[1], -10*imy, 10.0*imy, step, digits, "");
+			}
+			uiBlockEndAlign(block);
 		}
 	}
 	else {	// apply event
 		float delta[2];
 		
-		cent[0]= (cent[0]*imx)/nactive;
-		cent[1]= (cent[1]*imy)/nactive;
+		cent[0]= cent[0]/nactive;
+		cent[1]= cent[1]/nactive;
 			
-		delta[0]= (ocent[0]-cent[0])/imx;
-		delta[1]= (ocent[1]-cent[1])/imy;
+		if (G.sima->flag & SI_COORDFLOATS) {
+			delta[0]= ocent[0]-cent[0];
+			delta[1]= ocent[1]-cent[1];
+		}
+		else {
+			delta[0]= ocent[0]/imx - cent[0];
+			delta[1]= ocent[1]/imy - cent[1];
+		}
 
 		for (i=0; i<me->totface; i++) {
 			MFace *mf= &((MFace*) me->mface)[i];
 			TFace *tf= &((TFace*) me->tface)[i];
 		
-			if (!mf->v3 || !(tf->flag & TF_SELECT))
+			if (!(tf->flag & TF_SELECT))
 				continue;
 		
 			if (tf->flag & TF_SEL1) {
@@ -755,6 +801,20 @@ void do_imagebuts(unsigned short event)
 			}
 		}
 		break;
+
+	case B_SIMACLONEBROWSE:
+		setcloneimage();
+		allqueue(REDRAWIMAGE, 0);
+		break;
+		
+	case B_SIMACLONEDELETE:
+		Gip.clone.image= NULL;
+		allqueue(REDRAWIMAGE, 0);
+		break;
+
+	case B_SIMABRUSHCHANGE:
+		allqueue(REDRAWIMAGE, 0);
+		break;
 	}
 }
 
@@ -775,14 +835,14 @@ static void image_panel_properties(short cntrl)	// IMAGE_HANDLER_PROPERTIES
 		uiDefBut(block, LABEL, B_NOP, str,		10,180,300,19, 0, 0, 0, 0, 0, "");
 
 		uiBlockBeginAlign(block);
-		uiDefButS(block, TOG|BIT|1, B_TWINANIM, "Anim", 10,150,140,19, &G.sima->image->tpageflag, 0, 0, 0, 0, "Toggles use of animated texture");
+		uiDefButBitS(block, TOG, IMA_TWINANIM, B_TWINANIM, "Anim", 10,150,140,19, &G.sima->image->tpageflag, 0, 0, 0, 0, "Toggles use of animated texture");
 		uiDefButS(block, NUM, B_TWINANIM, "Start:",		10,130,140,19, &G.sima->image->twsta, 0.0, 128.0, 0, 0, "Displays the start frame of an animated texture");
 		uiDefButS(block, NUM, B_TWINANIM, "End:",		10,110,140,19, &G.sima->image->twend, 0.0, 128.0, 0, 0, "Displays the end frame of an animated texture");
 		uiDefButS(block, NUM, B_NOP, "Speed", 				10,90,140,19, &G.sima->image->animspeed, 1.0, 100.0, 0, 0, "Displays Speed of the animation in frames per second");
 		uiBlockEndAlign(block);
 
 		uiBlockBeginAlign(block);
-		uiDefButS(block, TOG|BIT|0, B_SIMAGEDRAW1, "Tiles",	160,150,140,19, &G.sima->image->tpageflag, 0, 0, 0, 0, "Toggles use of tilemode for faces");
+		uiDefButBitS(block, TOG, IMA_TILES, B_SIMAGEDRAW1, "Tiles",	160,150,140,19, &G.sima->image->tpageflag, 0, 0, 0, 0, "Toggles use of tilemode for faces");
 		uiDefButS(block, NUM, B_SIMAGEDRAW, "X:",		160,130,70,19, &G.sima->image->xrep, 1.0, 16.0, 0, 0, "Sets the degree of repetition in the X direction");
 		uiDefButS(block, NUM, B_SIMAGEDRAW, "Y:",		230,130,70,19, &G.sima->image->yrep, 1.0, 16.0, 0, 0, "Sets the degree of repetition in the Y direction");
 		uiBlockBeginAlign(block);
@@ -793,24 +853,53 @@ static void image_panel_properties(short cntrl)	// IMAGE_HANDLER_PROPERTIES
 
 static void image_panel_paint(short cntrl)	// IMAGE_HANDLER_PROPERTIES
 {
-	extern VPaint Gvp;         /* from vpaint - this was copied from the paint panel*/
-	static float hsv[3], old[3];	// used as temp mem for picker
+	/* B_SIMABRUSHCHANGE only redraws and eats the mouse messages  */
+	/* so that LEFTMOUSE does not 'punch' through the floating panel */
+	/* B_SIMANOTHING */
+	ImagePaintTool *tool= &Gip.tool[Gip.current];
 	uiBlock *block;
+	ID *id;
 
 	block= uiNewBlock(&curarea->uiblocks, "image_panel_paint", UI_EMBOSS, UI_HELV, curarea->win);
 	uiPanelControl(UI_PNL_SOLID | UI_PNL_CLOSE | cntrl);
 	uiSetPanelHandler(IMAGE_HANDLER_PAINT);  // for close and esc
-	if(uiNewPanel(curarea, block, "Paint", "Image", 10, 230, 318, 204)==0)
+	if(uiNewPanel(curarea, block, "Image Paint", "Image", 10, 230, 318, 204)==0)
 		return;
 
-	uiBlockPickerButtons(block, &Gvp.r, hsv, old, 'f', B_NOP);	/* 'f' is for floating panel */
-
-	//	offset= FPICK+2*DPICK+BPICK in interface.c... this goes wrong when defines change
 	uiBlockBeginAlign(block);
-	uiDefButF(block, NUM, B_NOP, "A: ",		180+12+24,160,80,20, &Gvp.a, 0.0, 1.0, 0, 0, "The amount of pressure on the brush");
-	uiDefButF(block, NUM, B_NOP, "Size ",	180+12+24,140,80,20, &Gvp.size, 2.0, 64.0, 0, 0, "The size of the brush");
-}
+	uiDefButF(block, COL, B_VPCOLSLI, "",		979,160,230,19, tool->rgba, 0, 0, 0, 0, "");
+	uiDefButF(block, NUMSLI, B_SIMANOTHING , "Opacity ",		979,140,230,19, tool->rgba+3, 0.0, 1.0, 0, 0, "The amount of pressure on the brush");
+	uiDefButI(block, NUMSLI, B_SIMANOTHING , "Size ",		979,120,230,19, &tool->size, 2, 64, 0, 0, "The size of the brush");
+	uiDefButF(block, NUMSLI, B_SIMANOTHING , "Fall ",		979,100,230,19, &tool->innerradius, 0.0, 1.0, 0, 0, "The fall off radius of the brush");
 
+	if(Gip.current == IMAGEPAINT_BRUSH || Gip.current == IMAGEPAINT_SMEAR)
+		uiDefButF(block, NUMSLI, B_SIMANOTHING , "Stepsize ",979,80,230,19, &tool->timing, 1.0, 100.0, 0, 0, "Repeating Paint On %of Brush diameter");
+	else
+		uiDefButF(block, NUMSLI, B_SIMANOTHING , "Flow ",	979,80,230,19, &tool->timing, 1.0, 100.0, 0, 0, "Paint Flow for Air Brush");
+	uiBlockEndAlign(block);
+
+	uiBlockBeginAlign(block);
+	uiDefButS(block, ROW, B_SIMABRUSHCHANGE, "Brush",		890,160,80,19, &Gip.current, 7.0, IMAGEPAINT_BRUSH, 0, 0, "Brush");
+	uiDefButS(block, ROW, B_SIMABRUSHCHANGE, "AirBrush",		890,140,80,19, &Gip.current, 7.0, IMAGEPAINT_AIRBRUSH, 0, 0, "AirBrush");
+	uiDefButS(block, ROW, B_SIMABRUSHCHANGE, "Soften",		890,120,80,19, &Gip.current, 7.0, IMAGEPAINT_SOFTEN, 0, 0, "Soften");
+	uiDefButS(block, ROW, B_SIMABRUSHCHANGE, "Aux AB1",		890,100,80,19, &Gip.current, 7.0, IMAGEPAINT_AUX1, 0, 0, "Auxiliary Air Brush1");
+	uiDefButS(block, ROW, B_SIMABRUSHCHANGE, "Aux AB2",		890,80,80,19, &Gip.current, 7.0, IMAGEPAINT_AUX2, 0, 0, "Auxiliary Air Brush2");	
+	uiDefButS(block, ROW, B_SIMABRUSHCHANGE, "Smear",		890,60,80,19, &Gip.current, 7.0, IMAGEPAINT_SMEAR, 0, 0, "Smear");	
+	uiDefButS(block, ROW, B_SIMABRUSHCHANGE, "Clone",		890,40,80,19, &Gip.current, 7.0, IMAGEPAINT_CLONE, 0, 0, "Clone Brush / use RMB to drag source image");	
+	uiBlockEndAlign(block);
+
+	uiBlockBeginAlign(block);	
+	id= (ID*)Gip.clone.image;
+	std_libbuttons(block, 979, 40, 0, NULL, B_SIMACLONEBROWSE, id, 0, &G.sima->menunr, 0, 0, B_SIMACLONEDELETE, 0, 0);
+	uiDefButF(block, NUMSLI, B_SIMABRUSHCHANGE, "B ",979,20,230,19, &Gip.clone.alpha , 0.0, 1.0, 0, 0, "Blend clone image");
+	uiBlockEndAlign(block);
+
+#if 0
+	uiDefButBitS(block, TOG|BIT, IMAGEPAINT_DRAW_TOOL_DRAWING, B_SIMABRUSHCHANGE, "TD", 890,1,50,19, &Gip.flag, 0, 0, 0, 0, "Enables tool shape while drawing");
+	uiDefButBitS(block, TOG|BIT, IMAGEPAINT_DRAW_TOOL, B_SIMABRUSHCHANGE, "TP", 940,1,50,19, &Gip.flag, 0, 0, 0, 0, "Enables tool shape while not drawing");
+#endif
+	uiDefButBitS(block, TOG|BIT, IMAGEPAINT_TORUS, B_SIMABRUSHCHANGE, "Wrap", 890,1,50,19, &Gip.flag, 0, 0, 0, 0, "Enables torus wrapping");
+}
 
 static void image_blockhandlers(ScrArea *sa)
 {
@@ -836,16 +925,76 @@ static void image_blockhandlers(ScrArea *sa)
 	uiDrawBlocksPanels(sa, 0);
 }
 
-
+static void imagespace_grid(SpaceImage *sima)
+{
+	float gridsize, gridstep= 1.0f/32.0f;
+	float fac, blendfac;
+	
+	gridsize= sima->zoom;
+	
+	if(gridsize<=0.0f) return;
+	
+	if(gridsize<1.0f) {
+		while(gridsize<1.0f) {
+			gridsize*= 4.0;
+			gridstep*= 4.0;
+		}
+	}
+	else {
+		while(gridsize>=4.0f) {
+			gridsize/= 4.0;
+			gridstep/= 4.0;
+		}
+	}
+	
+	/* the fine resolution level */
+	blendfac= 0.25*gridsize - floor(0.25*gridsize);
+	CLAMP(blendfac, 0.0, 1.0);
+	BIF_ThemeColorShade(TH_BACK, (int)(20.0*(1.0-blendfac)));
+	
+	fac= 0.0f;
+	glBegin(GL_LINES);
+	while(fac<1.0) {
+		glVertex2f(0.0f, fac);
+		glVertex2f(1.0f, fac);
+		glVertex2f(fac, 0.0f);
+		glVertex2f(fac, 1.0f);
+		fac+= gridstep;
+	}
+	
+	/* the large resolution level */
+	BIF_ThemeColor(TH_BACK);
+	
+	fac= 0.0f;
+	while(fac<1.0) {
+		glVertex2f(0.0f, fac);
+		glVertex2f(1.0f, fac);
+		glVertex2f(fac, 0.0f);
+		glVertex2f(fac, 1.0f);
+		fac+= 4.0*gridstep;
+	}
+	glEnd();
+	
+}
 
 void drawimagespace(ScrArea *sa, void *spacedata)
 {
+	SpaceImage *sima= spacedata;
 	ImBuf *ibuf= NULL;
 	float col[3];
 	unsigned int *rect;
-	int x1, y1, xmin, xmax, ymin, ymax;
+	int x1, y1;
 	short sx, sy, dx, dy;
 	
+		/* If derived data is used then make sure that object
+		 * is up-to-date... might not be the case because updates
+		 * are normally done in drawview and could get here before
+		 * drawing a View3D.
+		 */
+	if (!G.obedit && OBACT && (sima->flag & SI_DRAWSHADOW)) {
+		object_handle_update(OBACT);
+	}
+
 	BIF_GetThemeColor3fv(TH_BACK, col);
 	glClearColor(col[0], col[1], col[2], 0.0);
 	glClear(GL_COLOR_BUFFER_BIT);
@@ -853,47 +1002,50 @@ void drawimagespace(ScrArea *sa, void *spacedata)
 	bwin_clear_viewmat(sa->win);	/* clear buttons view */
 	glLoadIdentity();
 	
-	xmin= curarea->winrct.xmin; xmax= curarea->winrct.xmax;
-	ymin= curarea->winrct.ymin; ymax= curarea->winrct.ymax;
+	what_image(sima);
 	
-	what_image(G.sima);
-	
-	if(G.sima->image) {
-	
-		if(G.sima->image->ibuf==0) {
-			load_image(G.sima->image, IB_rect, G.sce, G.scene->r.cfra);
+	if(sima->image) {
+		if(sima->image->ibuf==0) {
+			load_image(sima->image, IB_rect, G.sce, G.scene->r.cfra);
 		}	
-		ibuf= G.sima->image->ibuf;
+		tag_image_time(sima->image);
+		ibuf= sima->image->ibuf;
 	}
 	
-	if(ibuf==0 || ibuf->rect==0) {
-		calc_image_view(G.sima, 'f');
+	if(ibuf==NULL || ibuf->rect==NULL) {
+		calc_image_view(sima, 'f');
 		myortho2(G.v2d->cur.xmin, G.v2d->cur.xmax, G.v2d->cur.ymin, G.v2d->cur.ymax);
-		cpack(0x404040);
+		BIF_ThemeColorShade(TH_BACK, 20);
 		glRectf(0.0, 0.0, 1.0, 1.0);
+		imagespace_grid(sima);
 		draw_tfaces();
 	}
 	else {
 		/* calc location */
-		x1= xmin+(curarea->winx-G.sima->zoom*ibuf->x)/2;
-		y1= ymin+(curarea->winy-G.sima->zoom*ibuf->y)/2;
-	
-		x1-= G.sima->zoom*G.sima->xof;
-		y1-= G.sima->zoom*G.sima->yof;
+		x1= (curarea->winx-sima->zoom*ibuf->x)/2;
+		y1= (curarea->winy-sima->zoom*ibuf->y)/2;
 	
+		x1-= sima->zoom*sima->xof;
+		y1-= sima->zoom*sima->yof;
 		
-		if(G.sima->flag & SI_EDITTILE) {
-			rectwrite_part(xmin, ymin, xmax, ymax, x1, y1, ibuf->x, ibuf->y, (float)G.sima->zoom, (float)G.sima->zoom, ibuf->rect);
+		/* needed for gla draw */
+		glaDefine2DArea(&curarea->winrct);
+		glPixelZoom((float)sima->zoom, (float)sima->zoom);
+				
+		if(sima->flag & SI_EDITTILE) {
+			glaDrawPixelsSafe(x1, y1, ibuf->x, ibuf->y, ibuf->rect);
 			
-			dx= ibuf->x/G.sima->image->xrep;
-			dy= ibuf->y/G.sima->image->yrep;
-			sy= (G.sima->curtile / G.sima->image->xrep);
-			sx= G.sima->curtile - sy*G.sima->image->xrep;
+			glPixelZoom(1.0, 1.0);
+			
+			dx= ibuf->x/sima->image->xrep;
+			dy= ibuf->y/sima->image->yrep;
+			sy= (sima->curtile / sima->image->xrep);
+			sx= sima->curtile - sy*sima->image->xrep;
 	
 			sx*= dx;
 			sy*= dy;
 			
-			calc_image_view(G.sima, 'p');	/* pixel */
+			calc_image_view(sima, 'p');	/* pixel */
 			myortho2(G.v2d->cur.xmin, G.v2d->cur.xmax, G.v2d->cur.ymin, G.v2d->cur.ymax);
 			
 			cpack(0x0);
@@ -901,22 +1053,22 @@ void drawimagespace(ScrArea *sa, void *spacedata)
 			cpack(0xFFFFFF);
 			glPolygonMode(GL_FRONT_AND_BACK, GL_LINE); glRects(sx+1,  sy+1,  sx+dx,  sy+dy); glPolygonMode(GL_FRONT_AND_BACK, GL_FILL);
 		}
-		else if(G.sima->mode==SI_TEXTURE) {
-			if(G.sima->image->tpageflag & IMA_TILES) {
-				
+		else if(sima->mode==SI_TEXTURE) {
+			
+			if(sima->image->tpageflag & IMA_TILES) {
 				
 				/* just leave this a while */
-				if(G.sima->image->xrep<1) return;
-				if(G.sima->image->yrep<1) return;
+				if(sima->image->xrep<1) return;
+				if(sima->image->yrep<1) return;
 				
-				if(G.sima->curtile >= G.sima->image->xrep*G.sima->image->yrep) 
-					G.sima->curtile = G.sima->image->xrep*G.sima->image->yrep - 1; 
+				if(sima->curtile >= sima->image->xrep*sima->image->yrep) 
+					sima->curtile = sima->image->xrep*sima->image->yrep - 1; 
 				
-				dx= ibuf->x/G.sima->image->xrep;
-				dy= ibuf->y/G.sima->image->yrep;
+				dx= ibuf->x/sima->image->xrep;
+				dy= ibuf->y/sima->image->yrep;
 				
-				sy= (G.sima->curtile / G.sima->image->xrep);
-				sx= G.sima->curtile - sy*G.sima->image->xrep;
+				sy= (sima->curtile / sima->image->xrep);
+				sx= sima->curtile - sy*sima->image->xrep;
 		
 				sx*= dx;
 				sy*= dy;
@@ -926,110 +1078,158 @@ void drawimagespace(ScrArea *sa, void *spacedata)
 				/* rect= ibuf->rect; */
 				for(sy= 0; sy+dy<=ibuf->y; sy+= dy) {
 					for(sx= 0; sx+dx<=ibuf->x; sx+= dx) {
-						
-						rectwrite_part(xmin, ymin, xmax, ymax, 
-							x1+sx*G.sima->zoom, y1+sy*G.sima->zoom, dx, dy, (float)G.sima->zoom, (float)G.sima->zoom, rect);
+						glaDrawPixelsSafe(x1+sx*sima->zoom, y1+sy*sima->zoom, dx, dy, rect);
 					}
 				}
 				
 				MEM_freeN(rect);
 			}
 			else 
-				rectwrite_part(xmin, ymin, xmax, ymax, x1, y1, ibuf->x, ibuf->y, (float)G.sima->zoom,(float)G.sima->zoom, ibuf->rect);
-		
+				glaDrawPixelsSafe(x1, y1, ibuf->x, ibuf->y, ibuf->rect);
+			
+			if(Gip.current == IMAGEPAINT_CLONE) {
+				int w, h;
+				unsigned char *clonerect;
+
+				/* this is not very efficient, but glDrawPixels doesn't allow
+				   drawing with alpha */
+				clonerect= alloc_alpha_clone_image(&w, &h);
+
+				if(clonerect) {
+					int offx, offy;
+					offx = sima->zoom*ibuf->x * + Gip.clone.offset[0];
+					offy = sima->zoom*ibuf->y * + Gip.clone.offset[1];
+
+					glEnable(GL_BLEND);
+					glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
+					glaDrawPixelsSafe(x1 + offx, y1 + offy, w, h, clonerect);
+					glDisable(GL_BLEND);
+
+					MEM_freeN(clonerect);
+				}
+			}
+			
+			glPixelZoom(1.0, 1.0);
+			
 			draw_tfaces();
 		}
 	
-		calc_image_view(G.sima, 'f');	/* float */
+		calc_image_view(sima, 'f');	/* float */
 	}
 
-	draw_image_prop_circle(ibuf);
+	draw_image_transform(ibuf);
 
 	myortho2(-0.375, sa->winx-0.375, -0.375, sa->winy-0.375);
+
+	draw_image_view_tool();
 	draw_image_view_icon();
 	draw_area_emboss(sa);
 
 	/* it is important to end a view in a transform compatible with buttons */
-	bwin_scalematrix(sa->win, G.sima->blockscale, G.sima->blockscale, G.sima->blockscale);
+	bwin_scalematrix(sa->win, sima->blockscale, sima->blockscale, sima->blockscale);
 	image_blockhandlers(sa);
 
 	curarea->win_swap= WIN_BACK_OK;
 }
 
-void image_viewmove(void)
+static void image_zoom_power_of_two(void)
 {
-	short mval[2], mvalo[2], xof, yof;
+	/* Make zoom a power of 2 */
+
+	G.sima->zoom = 1 / G.sima->zoom;
+	G.sima->zoom = log(G.sima->zoom) / log(2);
+	G.sima->zoom = ceil(G.sima->zoom);
+	G.sima->zoom = pow(2, G.sima->zoom);
+	G.sima->zoom = 1 / G.sima->zoom;
+}
+
+static void image_zoom_set_factor(float zoomfac)
+{
+	SpaceImage *sima= curarea->spacedata.first;
+	int width, height;
+
+	if (zoomfac <= 0.0f)
+		return;
+
+	sima->zoom *= zoomfac;
+
+	if (sima->zoom > 0.1f && sima->zoom < 4.0f)
+		return;
+
+	/* check zoom limits */
+
+	calc_image_view(G.sima, 'p');
+	width= 256;
+	height= 256;
+	if (sima->image) {
+		if (sima->image->ibuf) {
+			width= sima->image->ibuf->x;
+			height= sima->image->ibuf->y;
+		}
+	}
+	width *= sima->zoom;
+	height *= sima->zoom;
+
+	if ((width < 4) && (height < 4))
+		sima->zoom /= zoomfac;
+	else if((curarea->winrct.xmax - curarea->winrct.xmin) <= sima->zoom)
+		sima->zoom /= zoomfac;
+	else if((curarea->winrct.ymax - curarea->winrct.ymin) <= sima->zoom)
+		sima->zoom /= zoomfac;
+}
+
+void image_viewmove(int mode)
+{
+	short mval[2], mvalo[2], zoom0;
 	
 	getmouseco_sc(mvalo);
+	zoom0= G.sima->zoom;
 	
 	while(get_mbut()&(L_MOUSE|M_MOUSE)) {
 
 		getmouseco_sc(mval);
+
+		if(mvalo[0]!=mval[0] || mvalo[1]!=mval[1]) {
 		
-		xof= (mvalo[0]-mval[0])/G.sima->zoom;
-		yof= (mvalo[1]-mval[1])/G.sima->zoom;
-		
-		if(xof || yof) {
-			
-			G.sima->xof+= xof;
-			G.sima->yof+= yof;
-			
+			if(mode==0) {
+				G.sima->xof += (mvalo[0]-mval[0])/G.sima->zoom;
+				G.sima->yof += (mvalo[1]-mval[1])/G.sima->zoom;
+			}
+			else if (mode==1) {
+				float factor;
+
+				factor= 1.0+(float)(mvalo[0]-mval[0]+mvalo[1]-mval[1])/300.0;
+				image_zoom_set_factor(factor);
+			}
+
 			mvalo[0]= mval[0];
 			mvalo[1]= mval[1];
 			
 			scrarea_do_windraw(curarea);
 			screen_swapbuffers();
-		}		
+		}
 		else BIF_wait_for_statechange();
 	}
 }
 
-void image_viewzoom(unsigned short event)
+void image_viewzoom(unsigned short event, int invert)
 {
 	SpaceImage *sima= curarea->spacedata.first;
-	int width, height;
 
-	if(U.uiflag & USER_WHEELZOOMDIR) {
-		if (event==WHEELDOWNMOUSE || event == PADPLUSKEY) {
-			sima->zoom *= 2;
-		} else {
-			sima->zoom /= 2;
-			/* Check if the image will still be visible after zooming out */
-			if (sima->zoom < 1) {
-				calc_image_view(G.sima, 'p');
-				if (sima->image) {
-					if (sima->image->ibuf) {
-						width = sima->image->ibuf->x * sima->zoom;
-						height = sima->image->ibuf->y * sima->zoom;
-						if ((width < 4) && (height < 4)) {
-							/* Image will become too small, reset value */
-							sima->zoom *= 2;
-						}
-					}
-				}
-			}
-		}
-	} else {
-		if (event==WHEELUPMOUSE || event == PADPLUSKEY) {
-			sima->zoom *= 2;
-		} else {
-			sima->zoom /= 2;
-			/* Check if the image will still be visible after zooming out */
-			if (sima->zoom < 1) {
-				calc_image_view(G.sima, 'p');
-				if (sima->image) {
-					if (sima->image->ibuf) {
-						width = sima->image->ibuf->x * sima->zoom;
-						height = sima->image->ibuf->y * sima->zoom;
-						if ((width < 4) && (height < 4)) {
-							/* Image will become too small, reset value */
-							sima->zoom *= 2;
-						}
-					}
-				}
-			}
-		}
-	}
+	if(event==WHEELDOWNMOUSE || event==PADMINUS)
+		image_zoom_set_factor((U.uiflag & USER_WHEELZOOMDIR)? 1.25: 0.8);
+	else if(event==WHEELUPMOUSE || event==PADPLUSKEY)
+		image_zoom_set_factor((U.uiflag & USER_WHEELZOOMDIR)? 0.8: 1.25);
+	else if(event==PAD1)
+		sima->zoom= 1.0;
+	else if(event==PAD2)
+		sima->zoom= (invert)? 2.0: 0.5;
+	else if(event==PAD4)
+		sima->zoom= (invert)? 4.0: 0.25;
+	else if(event==PAD8)
+		sima->zoom= (invert)? 8.0: 0.125;
+	else
+		return;
 }
 
 /**
@@ -1058,12 +1258,7 @@ void image_home(void)
 		zoomY = ((float)height) / ((float)G.sima->image->ibuf->y);
 		G.sima->zoom= MIN2(zoomX, zoomY);
 
-		/* Now make it a power of 2 */
-		G.sima->zoom = 1 / G.sima->zoom;
-		G.sima->zoom = log(G.sima->zoom) / log(2);
-		G.sima->zoom = ceil(G.sima->zoom);
-		G.sima->zoom = pow(2, G.sima->zoom);
-		G.sima->zoom = 1 / G.sima->zoom;
+		image_zoom_power_of_two();
 	}
 	else {
 		G.sima->zoom= (float)1;
@@ -1076,3 +1271,32 @@ void image_home(void)
 	scrarea_queue_winredraw(curarea);
 }
 
+void image_viewcentre(void)
+{
+	float size, min[2], max[2], d[2], xim=256.0f, yim=256.0f;
+
+	if( is_uv_tface_editing_allowed()==0 ) return;
+
+	if (!minmax_tface_uv(min, max)) return;
+
+	if(G.sima->image && G.sima->image->ibuf) {
+		xim= G.sima->image->ibuf->x;
+		yim= G.sima->image->ibuf->y;
+	}
+
+	G.sima->xof= ((min[0] + max[0])*0.5f - 0.5f)*xim;
+	G.sima->yof= ((min[1] + max[1])*0.5f - 0.5f)*yim;
+
+	d[0] = max[0] - min[0];
+	d[1] = max[1] - min[1];
+	size= 0.5*MAX2(d[0], d[1])*MAX2(xim, yim)/256.0f;
+	
+	if(size<=0.01) size= 0.01;
+
+	G.sima->zoom= 0.7/size;
+
+	calc_image_view(G.sima, 'p');
+
+	scrarea_queue_winredraw(curarea);
+}
+
