@@ -1,5 +1,5 @@
 /**
- * $Id: drawimasel.c,v 1.15 2003/11/18 11:22:16 ton Exp $
+ * $Id: drawimasel.c,v 1.19 2005/08/04 22:36:21 zuster Exp $
  *
  * ***** BEGIN GPL/BL DUAL LICENSE BLOCK *****
  *
@@ -43,7 +43,6 @@
 #endif
 
 #ifdef _WIN32
-#include "BLI_winstuff.h"
 #pragma warning (once : 4761)
 #endif
 
@@ -78,6 +77,8 @@
 /* GLOBALS */
 extern char *fsmenu;
 
+void rectwrite_imasel(int , int , int , int , int , int , int , int , float , float , unsigned int *);
+
 void rectwrite_imasel(int winxmin, int winymin, int winxmax, int winymax, int x1, int y1, int xim, int yim, float zoomx, float zoomy, unsigned int *rect)
 {
 	int cx, cy, oldxim, x2, y2;
@@ -423,9 +424,6 @@ static void str_image_type(int ftype, char *name)
 #ifdef WITH_QUICKTIME
 	if( ftype & QUICKTIME ) { strcat(name, "quicktime "); }
 #endif
-#ifdef WITH_FREEIMAGE
-	if( ftype & FREEIMAGE ) { strcat(name, "freeimage "); }
-#endif
 }
 
 void draw_sima_area(SpaceImaSel *simasel)
@@ -510,27 +508,24 @@ void draw_sima_area(SpaceImaSel *simasel)
 		sx = simasel->desx;
 		sy = simasel->deey+6;
 		
-		glRasterPos2f(sx+16*0, sy);
 		if (bitset(simasel->fase, IMS_FOUND_BIP)) {
-			BIF_draw_icon(ICON_BPIBFOLDER_HLT);
+			BIF_draw_icon(sx+16*0, sy, ICON_BPIBFOLDER_HLT);
 		} else if (bitset(simasel->fase, IMS_WRITE_NO_BIP)) {
-			BIF_draw_icon(ICON_BPIBFOLDER_DEHLT);
+			BIF_draw_icon(sx+16*0, sy, ICON_BPIBFOLDER_DEHLT);
 		} else {
-			BIF_draw_icon(ICON_BPIBFOLDER_DEHLT);
+			BIF_draw_icon(sx+16*0, sy, ICON_BPIBFOLDER_DEHLT);
 		}
 
-		glRasterPos2f(sx+16*1, sy);
 		if (bitset(simasel->fase, IMS_KNOW_INF)) {
-			BIF_draw_icon(ICON_FOLDER_HLT);
+			BIF_draw_icon(sx+16*1, sy, ICON_FOLDER_HLT);
 		} else {
-			BIF_draw_icon(ICON_FOLDER_DEHLT);
+			BIF_draw_icon(sx+16*1, sy, ICON_FOLDER_DEHLT);
 		}
 		
-		glRasterPos2f(sx+16*2, sy);
 		if (bitset(simasel->fase, IMS_KNOW_IMA)) {
-			BIF_draw_icon(ICON_BLUEIMAGE_HLT);
+			BIF_draw_icon(sx+16*2, sy, ICON_BLUEIMAGE_HLT);
 		} else {
-			BIF_draw_icon(ICON_BLUEIMAGE_DEHLT);
+			BIF_draw_icon(sx+16*2, sy, ICON_BLUEIMAGE_DEHLT);
 		}
 	}
 	
