@@ -1,5 +1,5 @@
 /**
- * $Id: drawipo.c,v 1.40 2004/12/08 15:11:52 ton Exp $
+ * $Id: drawipo.c,v 1.71 2006/01/03 20:59:51 ton Exp $
  *
  * ***** BEGIN GPL/BL DUAL LICENSE BLOCK *****
  *
@@ -42,14 +42,15 @@
 #include <unistd.h>
 #else
 #include <io.h>
-#include "BLI_winstuff.h"
 #endif   
 
 #include "BMF_Api.h"
+#include "MEM_guardedalloc.h"
 
 #include "BLI_blenlib.h"
 #include "BLI_arithb.h"
 
+#include "DNA_action_types.h"
 #include "DNA_curve_types.h"
 #include "DNA_ipo_types.h"
 #include "DNA_key_types.h"
@@ -60,11 +61,12 @@
 #include "DNA_sequence_types.h"
 #include "DNA_userdef_types.h"
 
-#include "BKE_utildefines.h"
 #include "BKE_curve.h"
-#include "BKE_ipo.h"
+#include "BKE_depsgraph.h"
 #include "BKE_global.h"
+#include "BKE_ipo.h"
 #include "BKE_key.h"
+#include "BKE_utildefines.h"
 
 #include "BIF_gl.h"
 #include "BIF_resources.h"
@@ -72,13 +74,13 @@
 #include "BIF_interface.h"
 #include "BIF_mywindow.h"
 #include "BIF_space.h"
+#include "BIF_toolbox.h"
 #include "BIF_glutil.h"
 
 #include "BSE_drawipo.h"
 #include "BSE_view.h"
 #include "BSE_editipo.h"
 #include "BSE_editipo_types.h"
-#include "BSE_editaction_types.h"
 #include "BSE_editnla_types.h"
 
 #include "mydevice.h"
@@ -91,14 +93,15 @@
 #define ISPOIN4(a, b, c, d, e)        ( (a->b) && (a->c) && (a->d) && (a->e) )   
 
 #define IPOBUTX	65
-#define IPOSTEP 35			/* minimum pixels per gridstep */
+		/* minimum pixels per gridstep */
+#define IPOSTEP 35
 
 static float ipogrid_dx, ipogrid_dy, ipogrid_startx, ipogrid_starty;
 static int ipomachtx, ipomachty;
 
-static int vertymin, vertymax, horxmin, horxmax;	/* globals om LEFTMOUSE op scrollbar te testen */
+static int vertymin, vertymax, horxmin, horxmax;	/* globals to test LEFTMOUSE for scrollbar */
 
-extern short ACTWIDTH;
+extern short ACTWIDTH;		/* this is ugly! */
 
 static void scroll_prstr(float x, float y, float val, char dir, int disptype)
 {
@@ -115,7 +118,7 @@ static void scroll_prstr(float x, float y, float val, char dir, int disptype)
 	else macht= ipomachtx;
 	
 	if (macht<=0) sprintf(str, "%.*f", 1-macht, val);
-	else sprintf(str, "%d", (int)val);
+	else sprintf(str, "%d", (int)floor(val + 0.375));
 	
 	len= strlen(str);
 	if(dir=='h') x-= 4*len;
@@ -137,7 +140,7 @@ static void step_to_grid(float *step, int *macht)
 	
 	loga= log10(*step);
 	*macht= (int)(loga);
-	
+
 	rem= loga- *macht;
 	rem= pow(10.0, rem);
 	
@@ -145,8 +148,19 @@ static void step_to_grid(float *step, int *macht)
 		if(rem < 0.2) rem= 0.2;
 		else if(rem < 0.5) rem= 0.5;
 		else rem= 1.0;
-		
+
 		*step= rem*pow(10.0, (float)*macht);
+		
+		// partial of a frame have no meaning
+		if(curarea->spacetype==SPACE_TIME) {
+			SpaceTime *stime= curarea->spacedata.first;
+			if(stime->flag & TIME_DRAWFRAMES) {
+				rem = 1.0;
+				*step = 1.0;
+			}
+		}
+		
+		if(rem==1.0) (*macht)++;	// prevents printing 1.0 2.0 3.0 etc
 	}
 	else {
 		if(rem < 2.0) rem= 2.0;
@@ -156,26 +170,36 @@ static void step_to_grid(float *step, int *macht)
 		*step= rem*pow(10.0, (float)*macht);
 		
 		(*macht)++;
+		if(rem==10.0) (*macht)++;	// prevents printing 1.0 2.0 3.0 etc
 	}
-	
 }
 
 void calc_ipogrid()
 {
-	float space, pixels;
-	
+	float space, pixels, secondiv=1.0;
+	int secondgrid= 0;
 	/* rule: gridstep is minimal IPOSTEP pixels */
 	/* how large is IPOSTEP pixels? */
 	
 	if(G.v2d==0) return;
 	
+	/* detect of we have seconds or frames, should become argument */
+	if(curarea->spacetype==SPACE_TIME) {
+		SpaceTime *stime= curarea->spacedata.first;
+		if(!(stime->flag & TIME_DRAWFRAMES)) {
+			secondgrid= 1;
+			secondiv= 0.01 * (float)G.scene->r.frs_sec;
+		}
+	}
+	
 	space= G.v2d->cur.xmax - G.v2d->cur.xmin;
 	pixels= G.v2d->mask.xmax-G.v2d->mask.xmin;
 	
-	ipogrid_dx= IPOSTEP*space/pixels;
+	ipogrid_dx= IPOSTEP*space/(secondiv*pixels);
 	step_to_grid(&ipogrid_dx, &ipomachtx);
+	ipogrid_dx*= secondiv;
 	
-	if ELEM(curarea->spacetype, SPACE_SEQ, SPACE_SOUND) {
+	if ELEM3(curarea->spacetype, SPACE_SEQ, SPACE_SOUND, SPACE_TIME) {
 		if(ipogrid_dx < 0.1) ipogrid_dx= 0.1;
 		ipomachtx-= 2;
 		if(ipomachtx<-2) ipomachtx= -2;
@@ -186,13 +210,14 @@ void calc_ipogrid()
 	ipogrid_dy= IPOSTEP*space/pixels;
 	step_to_grid(&ipogrid_dy, &ipomachty);
 	
-	if ELEM(curarea->spacetype, SPACE_SEQ, SPACE_SOUND) {
+	if ELEM3(curarea->spacetype, SPACE_SEQ, SPACE_SOUND, SPACE_TIME) {
 		if(ipogrid_dy < 1.0) ipogrid_dy= 1.0;
 		if(ipomachty<1) ipomachty= 1;
 	}
 	
-	ipogrid_startx= G.v2d->cur.xmin-fmod(G.v2d->cur.xmin, ipogrid_dx);
+	ipogrid_startx= secondiv*(G.v2d->cur.xmin/secondiv - fmod(G.v2d->cur.xmin/secondiv, ipogrid_dx/secondiv));
 	if(G.v2d->cur.xmin<0.0) ipogrid_startx-= ipogrid_dx;
+	
 	ipogrid_starty= (G.v2d->cur.ymin-fmod(G.v2d->cur.ymin, ipogrid_dy));
 	if(G.v2d->cur.ymin<0.0) ipogrid_starty-= ipogrid_dy;
 	
@@ -209,8 +234,7 @@ void draw_ipogrid(void)
 	
 	step= (G.v2d->mask.xmax-G.v2d->mask.xmin+1)/IPOSTEP;
 	
-	if(curarea->spacetype==SPACE_SOUND) glColor3ub(0x70, 0x70, 0x60);
-	else BIF_ThemeColor(TH_GRID);
+	BIF_ThemeColor(TH_GRID);
 	
 	for(a=0; a<step; a++) {
 		glBegin(GL_LINE_STRIP);
@@ -221,8 +245,7 @@ void draw_ipogrid(void)
 	
 	vec2[0]= vec1[0]-= 0.5*ipogrid_dx;
 	
-	if(curarea->spacetype==SPACE_SOUND) glColor3ub(0x80, 0x80, 0x70);
-	else  BIF_ThemeColorShade(TH_GRID, 16);
+	BIF_ThemeColorShade(TH_GRID, 16);
 	
 	step++;
 	for(a=0; a<=step; a++) {
@@ -232,7 +255,8 @@ void draw_ipogrid(void)
 		vec2[0]= vec1[0]-= ipogrid_dx;
 	}
 	
-	if(curarea->spacetype!=SPACE_SOUND && curarea->spacetype!=SPACE_ACTION && curarea->spacetype!=SPACE_NLA) {
+	if ELEM4(curarea->spacetype, SPACE_SOUND, SPACE_ACTION, SPACE_NLA, SPACE_TIME);
+	else {
 		vec1[0]= ipogrid_startx;
 		vec1[1]= vec2[1]= ipogrid_starty;
 		vec2[0]= G.v2d->cur.xmax;
@@ -291,7 +315,7 @@ void draw_ipogrid(void)
 			glRectf(0.0,  0.0,  100.0,  1.0); 
 			glPolygonMode(GL_FRONT_AND_BACK, GL_FILL);
 		}
-		else if(ELEM(G.sipo->blocktype, ID_CU, IPO_CO)) {
+		else if(ELEM(G.sipo->blocktype, ID_CU, ID_CO)) {
 			glPolygonMode(GL_FRONT_AND_BACK, GL_LINE); 
 			glRectf(0.0,  1.0,  G.v2d->cur.xmax,  1.0); 
 			glPolygonMode(GL_FRONT_AND_BACK, GL_FILL);
@@ -318,7 +342,7 @@ void ipoco_to_areaco(View2D *v2d, float *vec, short *mval)
 {
 	float x, y;
 	
-	mval[0]= 3200;
+	mval[0]= IS_CLIPPED;
 	
 	x= (vec[0] - v2d->cur.xmin)/(v2d->cur.xmax-v2d->cur.xmin);
 	y= (vec[1] - v2d->cur.ymin)/(v2d->cur.ymax-v2d->cur.ymin);
@@ -360,7 +384,62 @@ int in_ipo_buttons(void)
 	else return 1;
 }
 
+static View2D *spacelink_get_view2d(SpaceLink *sl)
+{
+	if(sl->spacetype==SPACE_IPO) 
+		return &((SpaceIpo *)sl)->v2d;
+	else if(sl->spacetype==SPACE_SOUND) 
+		return &((SpaceSound *)sl)->v2d;
+	if(sl->spacetype==SPACE_ACTION) 
+		return &((SpaceAction *)sl)->v2d;
+	if(sl->spacetype==SPACE_NLA) 
+		return &((SpaceNla *)sl)->v2d;
+	if(sl->spacetype==SPACE_TIME) 
+		return &((SpaceTime *)sl)->v2d;
+	
+	return NULL;
+}
 
+/* copies changes in this view from or to all 2d views with lock option open */
+void view2d_do_locks(ScrArea *cursa, int flag)
+{
+	ScrArea *sa;
+	View2D *v2d, *curv2d;
+	SpaceLink *sl;
+	
+	curv2d= spacelink_get_view2d(cursa->spacedata.first);
+	if(curv2d==NULL) return;
+	if((curv2d->flag & V2D_VIEWLOCK)==0) return;
+
+	for(sa= G.curscreen->areabase.first; sa; sa= sa->next) {
+		if(sa!=cursa) {
+			for(sl= sa->spacedata.first; sl; sl= sl->next) {
+				
+				v2d= spacelink_get_view2d(sl);
+				if(v2d) {
+					if(v2d->flag & V2D_VIEWLOCK) {
+						if(flag & V2D_LOCK_COPY) {
+							v2d->cur.xmin= curv2d->cur.xmin;
+							v2d->cur.xmax= curv2d->cur.xmax;
+						}
+						else {
+							curv2d->cur.xmin= v2d->cur.xmin;
+							curv2d->cur.xmax= v2d->cur.xmax;
+							scrarea_queue_winredraw(sa);
+						}
+						
+						if(flag & V2D_LOCK_REDRAW)
+							scrarea_do_windraw(sa);
+						else
+							scrarea_queue_winredraw(sa);
+					}
+				}
+			}
+		}
+	}
+}
+
+/* event based, note: curarea is in here... */
 void view2d_zoom(View2D *v2d, float factor, int winx, int winy) 
 {
 	float dx= factor*(v2d->cur.xmax-v2d->cur.xmin);
@@ -374,6 +453,7 @@ void view2d_zoom(View2D *v2d, float factor, int winx, int winy)
 		v2d->cur.ymax-= dy;
 	}
 	test_view2d(v2d, winx, winy);
+	view2d_do_locks(curarea, V2D_LOCK_COPY);
 }
 
 
@@ -565,21 +645,25 @@ void test_view2d(View2D *v2d, int winx, int winy)
 }
 
 
-void calc_scrollrcts(View2D *v2d, int winx, int winy)
+void calc_scrollrcts(ScrArea *sa, View2D *v2d, int winx, int winy)
 {
 	v2d->mask.xmin= v2d->mask.ymin= 0;
 	v2d->mask.xmax= winx;
 	v2d->mask.ymax= winy;
 	
-	if(curarea->spacetype==SPACE_ACTION) {
-		v2d->mask.xmin+= ACTWIDTH;
-		v2d->hor.xmin+=ACTWIDTH;
+	if(sa->spacetype==SPACE_ACTION) {
+		if(sa->winx > ACTWIDTH+50) { 
+			v2d->mask.xmin+= ACTWIDTH;
+			v2d->hor.xmin+=ACTWIDTH;
+		}
 	}
-	else if(curarea->spacetype==SPACE_NLA){
-		v2d->mask.xmin+= NLAWIDTH;
-		v2d->hor.xmin+=NLAWIDTH;
+	else if(sa->spacetype==SPACE_NLA){
+		if(sa->winx > NLAWIDTH+50) { 
+			v2d->mask.xmin+= NLAWIDTH;
+			v2d->hor.xmin+=NLAWIDTH;
+		}
 	}
-	else if(curarea->spacetype==SPACE_IPO) {
+	else if(sa->spacetype==SPACE_IPO) {
 		v2d->mask.xmax-= IPOBUTX;
 		
 		if(v2d->mask.xmax<IPOBUTX)
@@ -620,7 +704,7 @@ static void draw_solution_line(View2D *v2d, float h)
 	vec[0]= v2d->cur.xmin;
 	vec[1]= h;
 	ipoco_to_areaco(v2d, vec, mval);
-	if(mval[0]!=3200) {
+	if(mval[0]!=IS_CLIPPED) {
 		glBegin(GL_LINES);
 		glVertex2f(v2d->vert.xmin, mval[1]);
 		glVertex2f(v2d->vert.xmax, mval[1]);
@@ -655,6 +739,38 @@ static void draw_solution(SpaceIpo *sipo)
 	}
 }
 
+/* used for drawing timeline */
+void draw_view2d_numbers_horiz(int drawframes)
+{
+	float fac, fac2, dfac, val;
+	
+	/* the numbers: convert ipogrid_startx and -dx to scroll coordinates */
+	
+	fac= (ipogrid_startx- G.v2d->cur.xmin)/(G.v2d->cur.xmax-G.v2d->cur.xmin);
+	fac= G.v2d->mask.xmin+fac*(G.v2d->mask.xmax-G.v2d->mask.xmin);
+	
+	dfac= (ipogrid_dx)/(G.v2d->cur.xmax-G.v2d->cur.xmin);
+	dfac= dfac*(G.v2d->mask.xmax-G.v2d->mask.xmin);
+	
+	BIF_ThemeColor(TH_TEXT);
+	val= ipogrid_startx;
+	while(fac < G.v2d->mask.xmax) {
+		
+		if(drawframes) {
+			ipomachtx= 1;
+			scroll_prstr(fac, 2.0+(float)(G.v2d->mask.ymin), val, 'h', 0);
+		}
+		else {
+			fac2= val/(float)G.scene->r.frs_sec;
+			scroll_prstr(fac, 2.0+(float)(G.v2d->mask.ymin), fac2, 'h', 0);
+		}
+		
+		fac+= dfac;
+		val+= ipogrid_dx;
+	}
+}
+
+
 void drawscroll(int disptype)
 {
 	rcti vert, hor;
@@ -710,7 +826,7 @@ void drawscroll(int disptype)
 				fac2= fac2-tim;
 				scroll_prstr(fac, 3.0+(float)(hor.ymin), tim+G.scene->r.frs_sec*fac2/100.0, 'h', disptype);
 			}
-			else if(curarea->spacetype==SPACE_SOUND) {
+			else if (curarea->spacetype==SPACE_SOUND) {
 				SpaceSound *ssound= curarea->spacedata.first;
 				
 				if(ssound->flag & SND_DRAWFRAMES) {
@@ -722,6 +838,33 @@ void drawscroll(int disptype)
 					scroll_prstr(fac, 3.0+(float)(hor.ymin), fac2, 'h', disptype);
 				}
 			}
+			else if (curarea->spacetype==SPACE_TIME) {
+				SpaceTime *stime= curarea->spacedata.first;
+				
+				if(stime->flag & TIME_DRAWFRAMES) {
+					ipomachtx= 1;
+					scroll_prstr(fac, 3.0+(float)(hor.ymin), val, 'h', disptype);
+				}
+				else {
+					fac2= val/(float)G.scene->r.frs_sec;
+					scroll_prstr(fac, 3.0+(float)(hor.ymin), fac2, 'h', disptype);
+				}
+			}
+			else if (curarea->spacetype==SPACE_IPO) {
+				EditIpo *ei= get_active_editipo();
+				
+				if(ei && ei->icu && ei->icu->driver) {
+					int adrcode= ei->icu->driver->adrcode;
+					
+					if(adrcode==OB_ROT_X || adrcode==OB_ROT_Y || adrcode==OB_ROT_Z) {
+						scroll_prstr(fac, 3.0+(float)(hor.ymin), val, 'v', IPO_DISPDEGR);
+					}
+					else 
+						scroll_prstr(fac, 3.0+(float)(hor.ymin), val, 'h', disptype);
+				}
+				else 
+					scroll_prstr(fac, 3.0+(float)(hor.ymin), val, 'h', disptype);
+			}
 			else {
 				scroll_prstr(fac, 3.0+(float)(hor.ymin), val, 'h', disptype);
 			}
@@ -795,6 +938,7 @@ static void draw_ipobuts(SpaceIpo *sipo)
 {
 	ScrArea *area= sipo->area;
 	View2D *v2d= &sipo->v2d;
+	Object *ob= OBACT;
 	uiBlock *block;
 	uiBut *but;
 	EditIpo *ei;
@@ -822,12 +966,22 @@ static void draw_ipobuts(SpaceIpo *sipo)
 
 	ei= sipo->editipo;
 	y= area->winy-30+sipo->butofs;
+	
+	if(ob && sipo->blocktype==ID_KE) {
+		int icon;
+		if(ob->shapeflag & OB_SHAPE_LOCK) icon= ICON_PIN_HLT; else icon= ICON_PIN_DEHLT;
+		uiDefIconButBitC(block, TOG, OB_SHAPE_LOCK, B_SETKEY, icon, 
+						 v2d->mask.xmax+18,y,25,20, &ob->shapeflag, 0, 0, 0, 0, "Always show the current Shape for this Object");
+		y-= IPOBUTY;
+	}
+	
 	for(a=0; a<sipo->totipo; a++, ei++, y-=IPOBUTY) {
-		
-		but= uiDefButS(block, TOG|BIT|6, a+1, ei->name,  v2d->mask.xmax+18, y, IPOBUTX-15, IPOBUTY-1, &(ei->flag), 0, 0, 0, 0, "");
+		// this button defines visiblity, bit zero of flag (IPO_VISIBLE)
+		but= uiDefButBitS(block, TOG, IPO_VISIBLE, a+1, ei->name,  v2d->mask.xmax+18, y, IPOBUTX-15, IPOBUTY-1, &(ei->flag), 0, 0, 0, 0, "");
 		// no hilite, its not visible, but most of all the winmatrix is not correct later on...
 		uiButSetFlag(but, UI_TEXT_LEFT|UI_NO_HILITE);
 		
+		// this fake button defines selection of curves
 		if(ei->icu) {
 			cpack(ei->col);
 			
@@ -835,50 +989,21 @@ static void draw_ipobuts(SpaceIpo *sipo)
 			sel= ei->flag & (IPO_SELECT + IPO_EDIT);
 			
 			uiEmboss((float)(v2d->mask.xmax+8), (float)(y+2), (float)(v2d->mask.xmax+15), (float)(y+IPOBUTY-2), sel);
+			
+			if(ei->icu->driver) {
+				cpack(0x0);
+				fdrawbox((float)v2d->mask.xmax+11,  (float)y+8,  (float)v2d->mask.xmax+12.5, (float)y+9.5);
+			}
 		}
 		
-		if(sipo->blocktype==ID_KE) {
-			Key *key= (Key *)sipo->from;
-			if(key==0 || key->type==KEY_NORMAL) break;
+		if(ei->flag & IPO_ACTIVE) {
+			cpack(0x0);
+			fdrawbox(v2d->mask.xmax+7,  y+1,  v2d->mask.xmax+16, y+IPOBUTY-1);
 		}
 	}
 	uiDrawBlock(block);
 }
 
-static void calc_ipoverts(SpaceIpo *sipo)
-{
-	View2D *v2d= &sipo->v2d;
-	EditIpo *ei;
-	BezTriple *bezt;
-	BPoint *bp;
-	int a, b;
-	
-	ei= G.sipo->editipo;
-	for(a=0; a<sipo->totipo; a++, ei++) {
-		if ISPOIN(ei, flag & IPO_VISIBLE, icu) {
-			if(ei->icu->bezt) {
-				bezt= ei->icu->bezt;
-				b= ei->icu->totvert;
-				while(b--) {
-					ipoco_to_areaco(v2d, bezt->vec[0], bezt->s[0]);
-					ipoco_to_areaco(v2d, bezt->vec[1], bezt->s[1]);
-					ipoco_to_areaco(v2d, bezt->vec[2], bezt->s[2]);
-					bezt++;
-				}
-			}
-			else if(ei->icu->bp) {
-				bp= ei->icu->bp;
-				b= ei->icu->totvert;
-				while(b--) {
-					ipoco_to_areaco(v2d, bp->vec, bp->s);
-					bp++;
-				}
-			}
-		}
-	}
-}
-
-
 static void draw_ipovertices(int sel)
 {
 	EditIpo *ei;
@@ -1168,7 +1293,9 @@ static void draw_ipocurves(int sel)
 								glVertex2fv(v1);
 							}
 							else {
-								resol= 3.0*sqrt(bezt->vec[1][0] - prevbezt->vec[1][0]);
+								/* resol not depending on horizontal resolution anymore, drivers for example... */
+								if(icu->driver) resol= 32;
+								else resol= 3.0*sqrt(bezt->vec[1][0] - prevbezt->vec[1][0]);
 								
 								if(resol<2) {
 									v1[0]= prevbezt->vec[1][0]+cycxofs;
@@ -1190,8 +1317,8 @@ static void draw_ipocurves(int sel)
 									
 									correct_bezpart(v1, v2, v3, v4);
 									
-									maakbez(v1[0], v2[0], v3[0], v4[0], data, resol);
-									maakbez(v1[1], v2[1], v3[1], v4[1], data+1, resol);
+									forward_diff_bezier(v1[0], v2[0], v3[0], v4[0], data, resol, 3);
+									forward_diff_bezier(v1[1], v2[1], v3[1], v4[1], data+1, resol, 3);
 									
 									fp= data;
 									while(resol--) {
@@ -1306,16 +1433,17 @@ static void draw_key(SpaceIpo *sipo, int visible)
 	View2D *v2d= &sipo->v2d;
 	Key *key;
 	KeyBlock *kb, *act=NULL;
+	Object *ob= OBACT;
 	unsigned int col;
+	int index;
 	
-	key= (Key *)sipo->from;
-	if(key==0)
+	key= ob_get_key((Object *)sipo->from);
+	if(key==NULL)
 		return;
 	
 	if(key->type== KEY_RELATIVE) if(visible==0) return;
 	
-	kb= key->block.first;
-	while(kb) {
+	for(index=1, kb= key->block.first; kb; kb= kb->next, index++) {
 		if(kb->type==KEY_LINEAR) setlinestyle(2);
 		else if(kb->type==KEY_BSPLINE) setlinestyle(4);
 		else setlinestyle(0);
@@ -1323,7 +1451,7 @@ static void draw_key(SpaceIpo *sipo, int visible)
 		if(kb==key->refkey) col= 0x22FFFF;
 		else col= 0xFFFF00;
 		
-		if( (kb->flag & SELECT)==0)	col-= 0x225500;
+		if(ob->shapenr!=index) col-= 0x225500;
 		else act= kb;
 		
 		cpack(col);
@@ -1333,7 +1461,6 @@ static void draw_key(SpaceIpo *sipo, int visible)
 		glVertex2f(v2d->cur.xmax, kb->pos);
 		glEnd();
 		
-		kb= kb->next;
 	}
 	
 	if(act) {
@@ -1360,11 +1487,13 @@ static void draw_key(SpaceIpo *sipo, int visible)
 #define B_MUL_IPO		3402
 #define B_TRANS_IPO		3403
 #define B_IPO_NONE		3404
+#define B_IPO_DRIVER	3405
+#define B_IPO_REDR		3406
+#define B_IPO_DEPCHANGE	3407
 
 static float hspeed= 0;
 
-
-static void boundbox_ipo_visible(SpaceIpo *si)
+static void boundbox_ipo_curves(SpaceIpo *si)
 {
 	EditIpo *ei;
 	Key *key;
@@ -1391,7 +1520,7 @@ static void boundbox_ipo_visible(SpaceIpo *si)
 	}
 	/* keylines? */
 	if(si->blocktype==ID_KE) {
-		key= (Key *)si->from;
+		key= ob_get_key((Object *)si->from);
 		if(key && key->block.first) {
 			kb= key->block.first;
 			if(kb->pos < si->v2d.tot.ymin) si->v2d.tot.ymin= kb->pos;
@@ -1406,6 +1535,7 @@ static void boundbox_ipo_visible(SpaceIpo *si)
 /* is used for both read and write... */
 static void ipo_editvertex_buts(uiBlock *block, SpaceIpo *si, float min, float max)
 {
+	Object *ob= OBACT;
 	EditIpo *ei;
 	BezTriple *bezt;
 	float median[3];
@@ -1449,20 +1579,15 @@ static void ipo_editvertex_buts(uiBlock *block, SpaceIpo *si, float min, float m
 	/* check for keys */
 	if(tot==0) {
 		if(G.sipo->blocktype==ID_KE) {
-			Key *key= (Key *)G.sipo->from;
+			Key *key= ob_get_key((Object *)G.sipo->from);
 			KeyBlock *kb;
 			
-			if(key==0) return;
+			if(key==NULL || ob->shapenr==0) return;
 			iskey= 1;
 			
-			kb= key->block.first;
-			while(kb) {
-				if(kb->flag & SELECT) {
-					median[1]+= kb->pos;
-					tot++;
-				}
-				kb= kb->next;
-			}
+			kb= BLI_findlink(&key->block, ob->shapenr-1);
+			median[1]+= kb->pos;
+			tot++;
 		}
 	}
 	if(tot==0) return;
@@ -1475,20 +1600,21 @@ static void ipo_editvertex_buts(uiBlock *block, SpaceIpo *si, float min, float m
 	
 		VECCOPY(si->median, median);
 		
+		uiBlockBeginAlign(block);
 		if(tot==1) {
 			if(iskey) 
 				uiDefButF(block, NUM, B_TRANS_IPO, "Key Y:",	10, 80, 300, 19, &(si->median[1]), min, max, 10, 0, "");
 			else {
-				uiDefButF(block, NUM, B_TRANS_IPO, "Vertex X:",	10, 100, 300, 19, &(si->median[0]), min, max, 100, 0, "");
-				uiDefButF(block, NUM, B_TRANS_IPO, "Vertex Y:",	10, 80, 300, 19, &(si->median[1]), min, max, 100, 0, "");
+				uiDefButF(block, NUM, B_TRANS_IPO, "Vertex X:",	10, 100, 150, 19, &(si->median[0]), min, max, 100, 0, "");
+				uiDefButF(block, NUM, B_TRANS_IPO, "Vertex Y:",	160, 100, 150, 19, &(si->median[1]), min, max, 100, 0, "");
 			}
 		}
 		else {
 			if(iskey) 
 				uiDefButF(block, NUM, B_TRANS_IPO, "Median Key Y:",	10, 80, 300, 19, &(si->median[1]), min, max, 10, 0, "");
 			else {
-				uiDefButF(block, NUM, B_TRANS_IPO, "Median X:",	10, 100, 300, 19, &(si->median[0]), min, max, 100, 0, "");
-				uiDefButF(block, NUM, B_TRANS_IPO, "Median Y:",	10, 80, 300, 19, &(si->median[1]), min, max, 100, 0, "");
+				uiDefButF(block, NUM, B_TRANS_IPO, "Median X:",	10, 100, 150, 19, &(si->median[0]), min, max, 100, 0, "");
+				uiDefButF(block, NUM, B_TRANS_IPO, "Median Y:",	160, 100, 150, 19, &(si->median[1]), min, max, 100, 0, "");
 			}
 		}
 	}
@@ -1496,19 +1622,15 @@ static void ipo_editvertex_buts(uiBlock *block, SpaceIpo *si, float min, float m
 		VecSubf(median, si->median, median);
 
 		if(G.sipo->blocktype==ID_KE) {
-			Key *key= (Key *)G.sipo->from;
+			Key *key= ob_get_key((Object *)G.sipo->from);
 			KeyBlock *kb;
 			
-			if(key==0) return;
+			if(key==NULL || ob->shapenr==0) return;
 			
-			kb= key->block.first;
-			while(kb) {
-				if(kb->flag & SELECT) {
-					kb->pos+= median[1];
-					tot++;
-				}
-				kb= kb->next;
-			}			
+			kb= BLI_findlink(&key->block, ob->shapenr-1);
+			kb->pos+= median[1];
+			tot++;
+
 			sort_keys(key);
 		}
 	}
@@ -1552,8 +1674,25 @@ static void ipo_editvertex_buts(uiBlock *block, SpaceIpo *si, float min, float m
 
 void do_ipobuts(unsigned short event)
 {
-
+	Object *ob= OBACT;
+	EditIpo *ei;
+	
+	if(G.sipo->from==NULL) return;
+	
 	switch(event) {
+	case B_IPO_REDR:
+		ei= get_active_editipo();
+		if(ei) {
+			if(ei->icu->driver) {
+				if(G.sipo->blocktype==ID_KE || G.sipo->blocktype==ID_AC) 
+					DAG_object_flush_update(G.scene, ob, OB_RECALC_DATA);
+				else
+					DAG_object_flush_update(G.scene, ob, OB_RECALC_OB);
+			}
+		}
+		allqueue(REDRAWIPO, 0);
+		allqueue(REDRAWVIEW3D, 0);
+		break;
 	case B_SETSPEED:
 		set_speed_editipo(hspeed);
 		break;
@@ -1566,43 +1705,210 @@ void do_ipobuts(unsigned short event)
 		editipo_changed(G.sipo, 1);
 		allqueue(REDRAWIPO, 0);
 		break;
+	case B_SETKEY:
+		ob->shapeflag &= ~OB_SHAPE_TEMPLOCK;
+		DAG_object_flush_update(G.scene, ob, OB_RECALC_DATA);
+		allqueue(REDRAWVIEW3D, 0);
+		allqueue(REDRAWIPO, 0);
+		allqueue(REDRAWBUTSEDIT, 0);
+		break;
+	case B_IPO_DRIVER:
+		ei= get_active_editipo();
+		if(ei) {
+			if(ei->icu==NULL) {
+				ei->icu= verify_ipocurve(G.sipo->from, G.sipo->blocktype, G.sipo->actname, G.sipo->constname, ei->adrcode);
+				ei->flag |= IPO_SELECT;
+				ei->icu->flag= ei->flag;
+			}
+			if(ei->icu->driver) {
+				MEM_freeN(ei->icu->driver);
+				ei->icu->driver= NULL;
+				if(ei->icu->bezt==NULL) {
+					BLI_remlink( &(G.sipo->ipo->curve), ei->icu);
+					free_ipo_curve(ei->icu);
+					ei->icu= NULL;
+				}
+			}
+			else {
+				ei->icu->driver= MEM_callocN(sizeof(IpoDriver), "ipo driver");
+				ei->icu->driver->blocktype= ID_OB;
+				ei->icu->driver->adrcode= OB_LOC_X;
+			}
+
+			allqueue(REDRAWVIEW3D, 0);
+			allqueue(REDRAWIPO, 0);
+			allqueue(REDRAWBUTSEDIT, 0);
+			DAG_scene_sort(G.scene);
+			
+			BIF_undo_push("Add/Remove Ipo driver");
+		}
+		break;
+	case B_IPO_DEPCHANGE:
+		ei= get_active_editipo();
+		if(ei) {
+			if(ei->icu->driver) {
+				IpoDriver *driver= ei->icu->driver;
+				
+				if(driver->flag & IPO_DRIVER_PYTHON) {
+					driver->ob= NULL;
+				}
+				else {
+					if(driver->ob) {
+						if(ob==driver->ob) {
+							error("Cannot assign a Driver to own Object");
+							driver->ob= NULL;
+						}
+						else {
+							/* check if type is still OK */
+							if(driver->ob->type==OB_ARMATURE && driver->blocktype==ID_AR);
+							else driver->blocktype= ID_OB;
+						}
+					}
+				}
+				DAG_scene_sort(G.scene);
+				
+				if(G.sipo->blocktype==ID_KE || G.sipo->blocktype==ID_AC) 
+					DAG_object_flush_update(G.scene, ob, OB_RECALC_DATA);
+				else
+					DAG_object_flush_update(G.scene, ob, OB_RECALC_OB);
+			}
+		}
+		allqueue(REDRAWVIEW3D, 0);
+		allqueue(REDRAWIPO, 0);
+		allqueue(REDRAWBUTSEDIT, 0);
+		break;
 	}
 }
 
+static char *ipodriver_modeselect_pup(Object *ob)
+{
+	static char string[265];
+	char tmpstr[64];
+	char formatstring[64];
+	
+	strcpy(string, "Driver type: %t");
+	
+	strcpy(formatstring, "|%s %%x%d %%i%d");
+	
+	if(ob) {
+		sprintf(tmpstr,formatstring,"Object",ID_OB, ICON_OBJECT);
+		strcat(string,tmpstr);
+	}
+	if(ob && ob->type==OB_ARMATURE) {
+		sprintf(tmpstr,formatstring,"Pose",ID_AR, ICON_POSE_DEHLT);
+		strcat(string,tmpstr);
+	}
+	
+	return (string);
+}
+
+static char *ipodriver_channelselect_pup(void)
+{
+	static char string[1024];
+	char *tmp;
+	
+	strcpy(string, "Driver channel: %t");
+	tmp= string+strlen(string);
+	
+	tmp+= sprintf(tmp, "|Loc X %%x%d", OB_LOC_X);
+	tmp+= sprintf(tmp, "|Loc Y %%x%d", OB_LOC_Y);
+	tmp+= sprintf(tmp, "|Loc Z %%x%d", OB_LOC_Z);
+	tmp+= sprintf(tmp, "|Rot X %%x%d", OB_ROT_X);
+	tmp+= sprintf(tmp, "|Rot Y %%x%d", OB_ROT_Y);
+	tmp+= sprintf(tmp, "|Rot Z %%x%d", OB_ROT_Z);
+	tmp+= sprintf(tmp, "|Size X %%x%d", OB_SIZE_X);
+	tmp+= sprintf(tmp, "|Size Y %%x%d", OB_SIZE_Y);
+	tmp+= sprintf(tmp, "|Size Z %%x%d", OB_SIZE_Z);
+	
+	return (string);
+}
 
 static void ipo_panel_properties(short cntrl)	// IPO_HANDLER_PROPERTIES
 {
-	extern int totipo_vis;	// editipo.c
+	extern int totipo_curve;	// editipo.c
 	uiBlock *block;
+	EditIpo *ei;
+	char name[48];
 	
 	block= uiNewBlock(&curarea->uiblocks, "ipo_panel_properties", UI_EMBOSS, UI_HELV, curarea->win);
 	uiPanelControl(UI_PNL_SOLID | UI_PNL_CLOSE | cntrl);
 	uiSetPanelHandler(IPO_HANDLER_PROPERTIES);  // for close and esc
 	if(uiNewPanel(curarea, block, "Transform Properties", "Ipo", 10, 230, 318, 204)==0) return;
 
+	/* this is new panel height, newpanel doesnt force new size on existing panels */
+	uiNewPanelHeight(block, 204);
+	
+	/* driver buttons first */
+	ei= get_active_editipo();
+	if(ei) {
+		
+		sprintf(name, "Driven Channel: %s", ei->name);
+		uiDefBut(block, LABEL, 0, name,		10, 265, 200, 19, NULL, 1.0, 0.0, 0, 0, "");
+		
+		if(ei->icu && ei->icu->driver) {
+			IpoDriver *driver= ei->icu->driver;
+			
+			uiDefBut(block, BUT, B_IPO_DRIVER, "Remove",				210,265,100,20, NULL, 0.0f, 0.0f, 0, 0, "Remove Driver for this Ipo Channel");
+			
+			uiBlockBeginAlign(block);
+			uiDefIconButBitS(block, TOG, IPO_DRIVER_PYTHON, B_IPO_DEPCHANGE, ICON_PYTHON, 10,240,25,20, &driver->flag, 0, 0, 0, 0, "Use a one-line Python Expression as Driver");
+			
+			if(driver->flag & IPO_DRIVER_PYTHON) {
+				uiDefBut(block, TEX, B_IPO_REDR, "",				35,240,275,20, driver->name, 0, 127, 0, 0, "Python Expression");
+				uiBlockEndAlign(block);
+			}
+			else {
+				uiDefIDPoinBut(block, test_obpoin_but, ID_OB, B_IPO_DEPCHANGE, "OB:",	35, 240, 125, 20, &(driver->ob), "Driver Object");
+				if(driver->ob) {
+					int icon=ICON_OBJECT;
+					
+					if(driver->ob->type==OB_ARMATURE && driver->blocktype==ID_AR) {
+						icon = ICON_POSE_DEHLT;
+						uiDefBut(block, TEX, B_IPO_REDR, "BO:",				10,220,150,20, driver->name, 0, 31, 0, 0, "Bone name");
+					}
+					else driver->blocktype= ID_OB;	/* safety when switching object button */
+					
+					uiBlockBeginAlign(block);
+					uiDefIconTextButS(block, MENU, B_IPO_DEPCHANGE, icon, 
+									  ipodriver_modeselect_pup(driver->ob), 165,240,145,20, &(driver->blocktype), 0, 0, 0, 0, "Driver type");
 
-	boundbox_ipo_visible(G.sipo);	// should not be needed... transform/draw calls should update
+					uiDefButS(block, MENU, B_IPO_REDR, 
+								ipodriver_channelselect_pup(),			165,220,145,20, &(driver->adrcode), 0, 0, 0, 0, "Driver channel");
+				}
+				uiBlockEndAlign(block);
+			}
+		}
+		else {
+			uiDefBut(block, BUT, B_IPO_DRIVER, "Add Driver",	210,265,100,19, NULL, 0.0f, 0.0f, 0, 0, "Create a Driver for this Ipo Channel");
+		}
+	}
+	else 
+		uiDefBut(block, LABEL, 0, " ",		10, 265, 150, 19, NULL, 1.0, 0.0, 0, 0, "");
+
+	boundbox_ipo_curves(G.sipo);	// should not be needed... transform/draw calls should update
 	
 	/* note ranges for buttons below are idiot... we need 2 ranges, one for sliding scale, one for real clip */
-	if(G.sipo->ipo && G.sipo->ipo->curve.first && totipo_vis) {
+	if(G.sipo->ipo && G.sipo->ipo->curve.first && totipo_curve) {
 		extern int totipo_vertsel;	// editipo.c
 		uiDefBut(block, LABEL, 0, "Visible curves",		10, 200, 150, 19, NULL, 1.0, 0.0, 0, 0, "");
 		
-		uiDefButF(block, NUM, B_MUL_IPO, "Xmin:",		10, 180, 150, 19, &G.sipo->tot.xmin, G.sipo->tot.xmin-1000.0, 18000.0, 100, 0, "");
-		uiDefButF(block, NUM, B_MUL_IPO, "Xmax:",		160, 180, 150, 19, &G.sipo->tot.xmax, G.sipo->tot.ymin-1000.0, 18000.0, 100, 0, "");
+		uiBlockBeginAlign(block);
+		uiDefButF(block, NUM, B_MUL_IPO, "Xmin:",		10, 180, 150, 19, &G.sipo->tot.xmin, G.sipo->tot.xmin-1000.0, MAXFRAMEF, 100, 0, "");
+		uiDefButF(block, NUM, B_MUL_IPO, "Xmax:",		160, 180, 150, 19, &G.sipo->tot.xmax, G.sipo->tot.ymin-1000.0, MAXFRAMEF, 100, 0, "");
 		
 		uiDefButF(block, NUM, B_MUL_IPO, "Ymin:",		10, 160, 150, 19, &G.sipo->tot.ymin, G.sipo->tot.ymin-1000.0, 5000.0, 100, 0, "");
 		uiDefButF(block, NUM, B_MUL_IPO, "Ymax:",		160, 160, 150, 19, &G.sipo->tot.ymax, G.sipo->tot.ymin-1000.0, 5000.0, 100, 0, "");
 
 		/* SPEED BUTTON */
 		if(totipo_vertsel) {
-			uiDefButF(block, NUM, B_IPO_NONE, "Speed:",			10,130,150,19, &hspeed, 0.0, 180.0, 1, 0, "");
+			uiBlockBeginAlign(block);
+			uiDefButF(block, NUM, B_IPO_NONE, "Speed:",		10,130,150,19, &hspeed, 0.0, 180.0, 1, 0, "");
 			uiDefBut(block, BUT, B_SETSPEED,"SET",			160,130,50,19, 0, 0, 0, 0, 0, "");
 		}
 	}
 
 	/* this one also does keypositions */
-	if(G.sipo->ipo) ipo_editvertex_buts(block, G.sipo, -10000, 18000);
+	if(G.sipo->ipo) ipo_editvertex_buts(block, G.sipo, -10000, MAXFRAMEF);
 }
 
 static void ipo_blockhandlers(ScrArea *sa)
@@ -1642,7 +1948,7 @@ void drawipospace(ScrArea *sa, void *spacedata)
 	uiFreeBlocksWin(&sa->uiblocks, sa->win);	/* for panel handler to work */
 	
 	v2d->hor.xmax+=IPOBUTX;
-	calc_scrollrcts(G.v2d, sa->winx, sa->winy);
+	calc_scrollrcts(sa, G.v2d, sa->winx, sa->winy);
 
 	BIF_GetThemeColor3fv(TH_BACK, col);
 	glClearColor(col[0], col[1], col[2], 0.0); 
@@ -1663,10 +1969,9 @@ void drawipospace(ScrArea *sa, void *spacedata)
 		} 
 	}
 
-	test_editipo();	/* test if current editipo is correct, make_editipo sets v2d->cur */
+	test_editipo(0);	/* test if current editipo is correct, make_editipo sets v2d->cur */
 
 	myortho2(v2d->cur.xmin, v2d->cur.xmax, v2d->cur.ymin, v2d->cur.ymax);
-	
  
 	if(sipo->editipo) {
 		
@@ -1683,8 +1988,6 @@ void drawipospace(ScrArea *sa, void *spacedata)
 		calc_ipogrid();	
 		draw_ipogrid();
 		
-		calc_ipoverts(sipo);
-		
 		draw_cfra(sipo);
 		
 		/* ipokeys */
@@ -1770,12 +2073,15 @@ void scroll_ipobuts()
 /* total mess function, especially with mousewheel, needs cleanup badly (ton) */
 int view2dzoom(unsigned short event)
 {
+	ScrArea *sa;
 	float fac, dx, dy, wtemp;
 	short mval[2], mvalo[2];
 	
 	areawinset(curarea->win);	/* from buttons */
 	curarea->head_swap= 0;
 	getmouseco_areawin(mvalo);
+	mval[0]= mvalo[0];
+	mval[1]= mvalo[1];
 	
 	while( (get_mbut()&(L_MOUSE|M_MOUSE)) || (event==WHEELUPMOUSE) || (event==WHEELDOWNMOUSE) ) {
 	
@@ -1859,21 +2165,40 @@ int view2dzoom(unsigned short event)
 				}
 			}
 		}
-		if(mval[0]!=mvalo[0] || mval[1]!=mvalo[1]) {
+		if(ELEM(event, WHEELUPMOUSE, WHEELDOWNMOUSE) || mval[0]!=mvalo[0] || mval[1]!=mvalo[1]) {
 			
 			if(U.viewzoom!=USER_ZOOM_CONT) {
 				mvalo[0]= mval[0];
 				mvalo[1]= mval[1];
 			}
 			
-			G.v2d->cur.xmin+= dx;
-			G.v2d->cur.xmax-= dx;
-			if(curarea->spacetype!=SPACE_SEQ && curarea->spacetype!=SPACE_SOUND && curarea->spacetype!=SPACE_NLA && curarea->spacetype!=SPACE_ACTION) {
+			if( ELEM(curarea->spacetype, SPACE_NLA, SPACE_ACTION) ) {
+				if(mvalo[0] < G.v2d->mask.xmin) {
+					G.v2d->cur.ymin+= dy;
+					G.v2d->cur.ymax-= dy;
+				}
+				else {
+					G.v2d->cur.xmin+= dx;
+					G.v2d->cur.xmax-= dx;
+				}
+			}
+			else if (ELEM3(curarea->spacetype, SPACE_SEQ, SPACE_SOUND, SPACE_TIME)) {
+				G.v2d->cur.xmin+= dx;
+				G.v2d->cur.xmax-= dx;
+			}
+			else {
+				G.v2d->cur.xmin+= dx;
+				G.v2d->cur.xmax-= dx;
 				G.v2d->cur.ymin+= dy;
 				G.v2d->cur.ymax-= dy;
 			}
 		
 			test_view2d(G.v2d, curarea->winx, curarea->winy);	/* cur min max rects */
+			
+			sa= curarea;	/* now when are you going to kill this one! */
+			view2d_do_locks(curarea, V2D_LOCK_COPY|V2D_LOCK_REDRAW);
+			mywinset(sa->win);
+			
 			scrarea_do_windraw(curarea);
 			screen_swapbuffers();
 		}
@@ -1892,17 +2217,14 @@ void center_currframe(void)
 	 */
 	float width;
   
-	areawinset(curarea->win);
-	curarea->head_swap= 0;
-
 	width = G.v2d->cur.xmax - G.v2d->cur.xmin;
 	G.v2d->cur.xmin = CFRA - 0.5*(width);
 	G.v2d->cur.xmax = CFRA + 0.5*(width);
 
 	test_view2d(G.v2d, curarea->winx, curarea->winy);
-	scrarea_do_windraw(curarea);
-	screen_swapbuffers();
-	curarea->head_swap= 0;
+	view2d_do_locks(curarea, V2D_LOCK_COPY);
+
+	scrarea_queue_winredraw(curarea);
 }
 
 /* total mess function, especially with mousewheel, needs cleanup badly (ton) */
@@ -1946,12 +2268,22 @@ int view2dmove(unsigned short event)
 	/* test where mouse is */
 	getmouseco_areawin(mvalo);
 	
-	if ELEM6(curarea->spacetype, SPACE_IPO, SPACE_SEQ, SPACE_OOPS, SPACE_SOUND, SPACE_ACTION, SPACE_NLA) 
+	if ELEM7(curarea->spacetype, SPACE_IPO, SPACE_SEQ, SPACE_OOPS, SPACE_SOUND, SPACE_ACTION, SPACE_NLA, SPACE_TIME) 
 		{
+
 			if( BLI_in_rcti(&G.v2d->mask, (int)mvalo[0], (int)mvalo[1]) ) {
 				facx= (G.v2d->cur.xmax-G.v2d->cur.xmin)/(float)(G.v2d->mask.xmax-G.v2d->mask.xmin);
 				facy= (G.v2d->cur.ymax-G.v2d->cur.ymin)/(float)(G.v2d->mask.ymax-G.v2d->mask.ymin);
 			}
+			/* stoopid exception to allow scroll in lefthand side */
+			else if(curarea->spacetype==SPACE_ACTION && BLI_in_rcti(&G.v2d->mask, ACTWIDTH+(int)mvalo[0], (int)mvalo[1]) ) {
+				facx= 0.0f;
+				facy= (G.v2d->cur.ymax-G.v2d->cur.ymin)/(float)(G.v2d->mask.ymax-G.v2d->mask.ymin);
+			}
+			else if(curarea->spacetype==SPACE_NLA && BLI_in_rcti(&G.v2d->mask, NLAWIDTH+(int)mvalo[0], (int)mvalo[1]) ) {
+				facx= 0.0f;
+				facy= (G.v2d->cur.ymax-G.v2d->cur.ymin)/(float)(G.v2d->mask.ymax-G.v2d->mask.ymin);
+			}
 			else if(IN_2D_VERT_SCROLL((int)mvalo)) {
 				facy= -(G.v2d->tot.ymax-G.v2d->tot.ymin)/(float)(G.v2d->mask.ymax-G.v2d->mask.ymin);
 				if(get_mbut() & mousebut) {
@@ -1960,6 +2292,8 @@ int view2dmove(unsigned short event)
 					else left= 0.0;
 					leftret= 0;
 				}
+				if((event==WHEELUPMOUSE) || (event==WHEELDOWNMOUSE))
+					facy= -facy;
 			}
 			else if(IN_2D_HORIZ_SCROLL((int)mvalo)) {
 				facx= -(G.v2d->tot.xmax-G.v2d->tot.xmin)/(float)(G.v2d->mask.xmax-G.v2d->mask.xmin);
@@ -1979,8 +2313,8 @@ int view2dmove(unsigned short event)
 		/* no x move in outliner */
 		if(curarea->spacetype==SPACE_OOPS && G.v2d->scroll) facx= 0.0;
 		
-		/* no y move in audio */
-		if(curarea->spacetype==SPACE_SOUND) facy= 0.0;
+		/* no y move in audio & time */
+		if ELEM(curarea->spacetype, SPACE_SOUND, SPACE_TIME) facy= 0.0;
 		
 		if(get_mbut() & mousebut && leftret) return 0;
 		if(facx==0.0 && facy==0.0) return 1;
@@ -2032,14 +2366,18 @@ int view2dmove(unsigned short event)
 		}
 
 		if(mval[0]!=mvalo[0] || mval[1]!=mvalo[1]) {
-
+			ScrArea *sa;
+			
 			G.v2d->cur.xmin+= left*dx;
 			G.v2d->cur.xmax+= right*dx;
 			G.v2d->cur.ymin+= left*dy;
 			G.v2d->cur.ymax+= right*dy;
-				
+			
 			test_view2d(G.v2d, curarea->winx, curarea->winy);
-				
+			sa= curarea;	/* bad global */
+			view2d_do_locks(curarea, V2D_LOCK_COPY|V2D_LOCK_REDRAW);
+			areawinset(sa->win);
+			
 			scrarea_do_windraw(curarea);
 			screen_swapbuffers();
 				
@@ -2069,7 +2407,8 @@ EditIpo *select_proj_ipo(rctf *rectf, int event)
 	/* this was IGLuint, but it's a useless typedef... */
 	GLuint buffer[MAXPICKBUF];
 	int a, b;
-	int code, hits;
+	int hits;
+	unsigned int code;
 	short mval[2];
 	
 	G.f |= G_PICKSEL;
