@@ -1,5 +1,5 @@
 /**
- * $Id: drawmesh.c,v 1.27 2004/12/13 09:10:58 kester Exp $
+ * $Id: drawmesh.c,v 1.45 2006/01/15 16:25:13 campbellbarton Exp $
  *
  * ***** BEGIN GPL/BL DUAL LICENSE BLOCK *****
  *
@@ -37,14 +37,11 @@
 #include <config.h>
 #endif
 
-#ifdef WIN32
-#include "BLI_winstuff.h"
-#endif
-
 #include "MEM_guardedalloc.h"
 
 #include "BLI_blenlib.h"
 #include "BLI_arithb.h"
+#include "BLI_edgehash.h"
 
 #include "IMB_imbuf_types.h"
 
@@ -60,6 +57,7 @@
 
 #include "BKE_bmfont.h"
 #include "BKE_displist.h"
+#include "BKE_DerivedMesh.h"
 #include "BKE_effect.h"
 #include "BKE_global.h"
 #include "BKE_image.h"
@@ -68,6 +66,7 @@
 #include "BKE_mesh.h"
 #include "BKE_object.h"
 #include "BKE_property.h"
+#include "BKE_utildefines.h"
 
 #include "BIF_resources.h"
 #include "BIF_gl.h"
@@ -116,6 +115,10 @@ static short fTexwindx, fTexwindy, fTexwinsx, fTexwinsy;
 static int fDoMipMap = 1;
 static int fLinearMipMap = 0;
 
+/* local prototypes --------------- */
+void update_realtime_textures(void);
+
+
 /*  static int source, dest; also not used */
 
 /**
@@ -287,7 +290,7 @@ int set_tpage(TFace *tface)
 
 	if(ima->ibuf==0) {
 		load_image(ima, IB_rect, G.sce, G.scene->r.cfra);
-		
+
 		if(ima->ibuf==0) {
 			ima->ok= 0;
 
@@ -415,6 +418,8 @@ int set_tpage(TFace *tface)
 	}
 	else glBindTexture( GL_TEXTURE_2D, *bind);
 	
+	tag_image_time(ima);
+
 	glEnable(GL_TEXTURE_2D);
 
 	fCurpage= ima;
@@ -484,228 +489,192 @@ void update_realtime_textures()
 	}
 }
 
+/***/
 
-void spack(unsigned int ucol)
+	/* Flags for marked edges */
+enum {
+	eEdge_Visible = (1<<0),
+	eEdge_Select = (1<<1),
+	eEdge_Active = (1<<2),
+	eEdge_SelectAndActive = (1<<3),
+	eEdge_ActiveFirst = (1<<4),
+	eEdge_ActiveLast = (1<<5)
+};
+
+	/* Creates a hash of edges to flags indicating
+	 * adjacent tface select/active/etc flags.
+	 */
+static void get_marked_edge_info__orFlags(EdgeHash *eh, int v0, int v1, int flags)
 {
-	char *cp= (char *)&ucol;
-	
-	glColor3ub(cp[3], cp[2], cp[1]);
+	int *flags_p;
+
+	if (!BLI_edgehash_haskey(eh, v0, v1)) {
+		BLI_edgehash_insert(eh, v0, v1, 0);
+	}
+
+	flags_p = (int*) BLI_edgehash_lookup_p(eh, v0, v1);
+	*flags_p |= flags;
+}
+static EdgeHash *get_marked_edge_info(Mesh *me)
+{
+	EdgeHash *eh = BLI_edgehash_new();
+	int i;
+
+	for (i=0; i<me->totface; i++) {
+		MFace *mf = &me->mface[i];
+		TFace *tf = &me->tface[i];
+		
+		if (mf->v3) {
+			if (!(tf->flag&TF_HIDE)) {
+				unsigned int flags = eEdge_Visible;
+				if (tf->flag&TF_SELECT) flags |= eEdge_Select;
+				if (tf->flag&TF_ACTIVE) {
+					flags |= eEdge_Active;
+					if (tf->flag&TF_SELECT) flags |= eEdge_SelectAndActive;
+				}
+
+				get_marked_edge_info__orFlags(eh, mf->v1, mf->v2, flags);
+				get_marked_edge_info__orFlags(eh, mf->v2, mf->v3, flags);
+				if (mf->v4) {
+					get_marked_edge_info__orFlags(eh, mf->v3, mf->v4, flags);
+					get_marked_edge_info__orFlags(eh, mf->v4, mf->v1, flags);
+				} else {
+					get_marked_edge_info__orFlags(eh, mf->v3, mf->v1, flags);
+				}
+
+				if (tf->flag&TF_ACTIVE) {
+					get_marked_edge_info__orFlags(eh, mf->v1, mf->v2, eEdge_ActiveFirst);
+					get_marked_edge_info__orFlags(eh, mf->v1, mf->v4?mf->v4:mf->v3, eEdge_ActiveLast);
+				}
+			}
+		}
+	}
+
+	return eh;
 }
 
-void draw_tfaces3D(Object *ob, Mesh *me)
-{
-	MFace *mface;
-	TFace *tface;
-	DispList *dl;
-	float *v1, *v2, *v3, *v4;
-	float *av1= NULL, *av2= NULL, *av3= NULL, *av4= NULL, *extverts= NULL; 
-	int a, activeFaceInSelection= 0;
-	
-	if(me==0 || me->tface==0) return;
 
-	dl= find_displist(&ob->disp, DL_VERTS);
-	if (dl) extverts= dl->verts;
+static int draw_tfaces3D__setHiddenOpts(void *userData, int index)
+{
+	struct { Mesh *me; EdgeHash *eh; } *data = userData;
+	MEdge *med = &data->me->medge[index];
+	unsigned int flags = (int) BLI_edgehash_lookup(data->eh, med->v1, med->v2);
+
+	if((G.f & G_DRAWSEAMS) && (med->flag&ME_SEAM)) {
+		return 0;
+	} else if(G.f & G_DRAWEDGES){ 
+		if (G.f&G_HIDDENEDGES) {
+			return 1;
+		} else {
+			return (flags & eEdge_Visible);
+		}
+	} else {
+		return (flags & eEdge_Select);
+	}
+}
+static int draw_tfaces3D__setSeamOpts(void *userData, int index)
+{
+	struct { Mesh *me; EdgeHash *eh; } *data = userData;
+	MEdge *med = &data->me->medge[index];
+	unsigned int flags = (int) BLI_edgehash_lookup(data->eh, med->v1, med->v2);
+
+	if (med->flag&ME_SEAM) {
+		if (G.f&G_HIDDENEDGES) {
+			return 1;
+		} else {
+			return (flags & eEdge_Visible);
+		}
+	} else {
+		return 0;
+	}
+}
+static int draw_tfaces3D__setSelectOpts(void *userData, int index)
+{
+	struct { Mesh *me; EdgeHash *eh; } *data = userData;
+	MEdge *med = &data->me->medge[index];
+	unsigned int flags = (int) BLI_edgehash_lookup(data->eh, med->v1, med->v2);
+
+	return flags & eEdge_Select;
+}
+static int draw_tfaces3D__setActiveOpts(void *userData, int index)
+{
+	struct { Mesh *me; EdgeHash *eh; } *data = userData;
+	MEdge *med = &data->me->medge[index];
+	unsigned int flags = (int) BLI_edgehash_lookup(data->eh, med->v1, med->v2);
+
+	if (flags & eEdge_Active) {
+		if (flags & eEdge_ActiveLast) {
+			glColor3ub(255, 0, 0);
+		} else if (flags & eEdge_ActiveFirst) {
+			glColor3ub(0, 255, 0);
+		} else if (flags & eEdge_SelectAndActive) {
+			glColor3ub(255, 255, 0);
+		} else {
+			glColor3ub(255, 0, 255);
+		}
+
+		return 1;
+	} else {
+		return 0;
+	}
+}
+static int draw_tfaces3D__drawFaceOpts(TFace *tface, int matnr)
+{
+	if (tface && !(tface->flag&TF_HIDE) && (tface->flag&TF_SELECT)) {
+		return 2; /* Don't set color */
+	} else {
+		return 0;
+	}
+}
+static void draw_tfaces3D(Object *ob, Mesh *me, DerivedMesh *dm)
+{
+	struct { Mesh *me; EdgeHash *eh; } data;
+
+	data.me = me;
+	data.eh = get_marked_edge_info(me);
 
 	glEnable(GL_DEPTH_TEST);
 	glDisable(GL_LIGHTING);
 	bglPolygonOffset(1.0);
 
-	/* Draw (Hidden) Edges */
-	if(G.f & G_DRAWEDGES || G.f & G_HIDDENEDGES){ 
-		BIF_ThemeColor(TH_EDGE_FACESEL);
-
-		mface= me->mface;
-		tface= me->tface;
-		for(a=me->totface; a>0; a--, mface++, tface++) {
-			if(mface->v3==0) continue;
-			if(tface->flag & TF_HIDE) {
-				if(!(G.f & G_HIDDENEDGES)) continue;
-			}
-			else if(!(G.f & G_DRAWEDGES)) continue;
-			
-			if(extverts) {
-				v1= extverts+3*mface->v1;
-				v2= extverts+3*mface->v2;
-				v3= extverts+3*mface->v3;
-				v4= mface->v4?(extverts+3*mface->v4):NULL;
-			} else {
-				v1= (me->mvert+mface->v1)->co;
-				v2= (me->mvert+mface->v2)->co;
-				v3= (me->mvert+mface->v3)->co;
-				v4= mface->v4?(me->mvert+mface->v4)->co:NULL;
-			}
-
-			glBegin(GL_LINE_LOOP);
-				glVertex3fv(v1);
-				glVertex3fv(v2);
-				glVertex3fv(v3);
-				if(v4) glVertex3fv(v4);
-			glEnd();
-		}
-	}
+		/* Draw (Hidden) Edges */
+	BIF_ThemeColor(TH_EDGE_FACESEL);
+	dm->drawMappedEdges(dm, draw_tfaces3D__setHiddenOpts, &data);
 
+		/* Draw Seams */
 	if(G.f & G_DRAWSEAMS) {
 		BIF_ThemeColor(TH_EDGE_SEAM);
 		glLineWidth(2);
 
-		glBegin(GL_LINES);
-		mface= me->mface;
-		tface= me->tface;
-		for(a=me->totface; a>0; a--, mface++, tface++) {
-			if(mface->v3==0) continue;
-			if(tface->flag & TF_HIDE) continue;
-
-			if(extverts) {
-				v1= extverts+3*mface->v1;
-				v2= extverts+3*mface->v2;
-				v3= extverts+3*mface->v3;
-				v4= mface->v4?(extverts+3*mface->v4):NULL;
-			} else {
-				v1= (me->mvert+mface->v1)->co;
-				v2= (me->mvert+mface->v2)->co;
-				v3= (me->mvert+mface->v3)->co;
-				v4= mface->v4?(me->mvert+mface->v4)->co:NULL;
-			}
-
-			if(tface->unwrap & TF_SEAM1) {
-				glVertex3fv(v1);
-				glVertex3fv(v2);
-			}
-
-			if(tface->unwrap & TF_SEAM2) {
-				glVertex3fv(v2);
-				glVertex3fv(v3);
-			}
-
-			if(tface->unwrap & TF_SEAM3) {
-				glVertex3fv(v3);
-				if(v4) glVertex3fv(v4);
-				else glVertex3fv(v1);
-			}
-
-			if(v4 && tface->unwrap & TF_SEAM4) {
-				glVertex3fv(v4);
-				glVertex3fv(v1);
-			}
-		}
-		glEnd();
+		dm->drawMappedEdges(dm, draw_tfaces3D__setSeamOpts, &data);
 
 		glLineWidth(1);
 	}
 
-	/* Draw Selected Faces in transparent purple */
+		/* Draw Selected Faces */
 	if(G.f & G_DRAWFACES) {
 		glEnable(GL_BLEND);
 		glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
 		BIF_ThemeColor4(TH_FACE_SELECT);
 
-		mface= me->mface;
-		tface= me->tface;
-		for(a=me->totface; a>0; a--, mface++, tface++) {
-			if(mface->v3==0) continue;
-			if(tface->flag & TF_HIDE) continue;
-		
-			if(tface->flag & TF_SELECT) {
-				if(extverts) {
-					v1= extverts+3*mface->v1;
-					v2= extverts+3*mface->v2;
-					v3= extverts+3*mface->v3;
-					v4= mface->v4?(extverts+3*mface->v4):NULL;
-				} else {
-					v1= (me->mvert+mface->v1)->co;
-					v2= (me->mvert+mface->v2)->co;
-					v3= (me->mvert+mface->v3)->co;
-					v4= mface->v4?(me->mvert+mface->v4)->co:NULL;
-				}
-	
-				glBegin(v4?GL_QUADS:GL_TRIANGLES);
-					glVertex3fv( v1 );
-					glVertex3fv( v2 );
-					glVertex3fv( v3 );
-					if(v4) glVertex3fv( v4 );
-				glEnd();
-			}
-		}
+		dm->drawFacesTex(dm, draw_tfaces3D__drawFaceOpts);
+
 		glDisable(GL_BLEND);
 	}
 	
-	/* Draw Stippled Outline for selected faces */
-	mface= me->mface;
-	tface= me->tface;
 	bglPolygonOffset(1.0);
-	for(a=me->totface; a>0; a--, mface++, tface++) {
-		if(mface->v3==0) continue;
-		if(tface->flag & TF_HIDE) continue;
-		
-		if(tface->flag & (TF_ACTIVE|TF_SELECT) ) {
-			if(extverts) {
-				v1= extverts+3*mface->v1;
-				v2= extverts+3*mface->v2;
-				v3= extverts+3*mface->v3;
-				v4= mface->v4?(extverts+3*mface->v4):NULL;
-			} else {
-				v1= (me->mvert+mface->v1)->co;
-				v2= (me->mvert+mface->v2)->co;
-				v3= (me->mvert+mface->v3)->co;
-				v4= mface->v4?(me->mvert+mface->v4)->co:NULL;
-			}
 
-			if(tface->flag & TF_ACTIVE) {
-				activeFaceInSelection= (tface->flag & TF_SELECT);
-				av1= v1; av2= v2; av3= v3; av4= v4;
-			}
-			else {
-				cpack(0x0);
-				glBegin(GL_LINE_LOOP);
-					glVertex3fv( v1 );
-					glVertex3fv( v2 );
-					glVertex3fv( v3 );
-					if(v4) glVertex3fv( v4 );
-				glEnd();
-			}
+		/* Draw Stippled Outline for selected faces */
+	glColor3ub(255, 255, 255);
+	setlinestyle(1);
+	dm->drawMappedEdges(dm, draw_tfaces3D__setSelectOpts, &data);
+	setlinestyle(0);
 
-			if(tface->flag & TF_SELECT) {
-				cpack(0xFFFFFF);
-				setlinestyle(1);
-				glBegin(GL_LINE_LOOP);
-					glVertex3fv( v1 );
-					glVertex3fv( v2 );
-					glVertex3fv( v3 );
-					if(v4) glVertex3fv( v4 );
-				glEnd();
-				setlinestyle(0);
-			}
-		}
-	}
-
-	/* Draw Active Face on top */
-	/* colors: R=x G=y */
-	if(av1) {
-		cpack(0xFF);
-		glBegin(GL_LINE_STRIP);
-			glVertex3fv(av1);
-			if(av4) glVertex3fv(av4);
-			else glVertex3fv(av3);
-		glEnd();
-
-		cpack(0xFF00);
-		glBegin(GL_LINE_STRIP);
-			glVertex3fv(av1);
-			glVertex3fv(av2);
-		glEnd();
-
-		if(activeFaceInSelection) cpack(0x00FFFF);
-		else cpack(0xFF00FF);
-
-		glBegin(GL_LINE_STRIP);
-			glVertex3fv(av2);
-			glVertex3fv(av3);
-			if(av4) glVertex3fv(av4);
-		glEnd();
-		setlinestyle(0);
-	}
+	dm->drawMappedEdges(dm, draw_tfaces3D__setActiveOpts, &data);
 
 	bglPolygonOffset(0.0);	// resets correctly now, even after calling accumulated offsets
+
+	BLI_edgehash_free(data.eh, NULL);
 }
 
 static int set_gl_light(Object *ob)
@@ -785,7 +754,7 @@ static int set_gl_light(Object *ob)
 
 static Material *give_current_material_or_def(Object *ob, int matnr)
 {
-	extern Material defmaterial;
+	extern Material defmaterial;	// render module abuse...
 	Material *ma= give_current_material(ob, matnr);
 
 	return ma?ma:&defmaterial;
@@ -809,6 +778,14 @@ static int set_draw_settings_cached(int clearcache, int textured, TFace *texface
 		c_badtex= 0;
 	}
 
+	if (texface) {
+		lit = lit && (texface->mode&TF_LIGHT);
+		textured = textured && (texface->mode&TF_TEX);
+		doublesided = texface->mode&TF_TWOSIDE;
+	} else {
+		textured = 0;
+	}
+
 	if (doublesided!=c_doublesided) {
 		if (doublesided) glDisable(GL_CULL_FACE);
 		else glEnable(GL_CULL_FACE);
@@ -817,7 +794,7 @@ static int set_draw_settings_cached(int clearcache, int textured, TFace *texface
 	}
 
 	if (textured!=c_textured || texface!=c_texface) {
-		if (textured) {
+		if (textured ) {
 			c_badtex= !set_tpage(texface);
 		} else {
 			set_tpage(0);
@@ -855,18 +832,43 @@ static int set_draw_settings_cached(int clearcache, int textured, TFace *texface
 	return c_badtex;
 }
 
+/* Icky globals, fix with userdata parameter */
+
+static Object *g_draw_tface_mesh_ob = NULL;
+static int g_draw_tface_mesh_islight = 0;
+static int g_draw_tface_mesh_istex = 0;
+static unsigned char g_draw_tface_mesh_obcol[4];
+static int draw_tface_mesh__set_draw(TFace *tface, int matnr)
+{
+	if (tface && ((tface->flag&TF_HIDE) || (tface->mode&TF_INVISIBLE))) return 0;
+
+	if (set_draw_settings_cached(0, g_draw_tface_mesh_istex, tface, g_draw_tface_mesh_islight, g_draw_tface_mesh_ob, matnr, TF_TWOSIDE)) {
+		glColor3ub(0xFF, 0x00, 0xFF);
+		return 2; /* Don't set color */
+	} else if (tface && tface->mode&TF_OBCOL) {
+		glColor3ubv(g_draw_tface_mesh_obcol);
+		return 2; /* Don't set color */
+	} else if (!tface) {
+		Material *ma= give_current_material(g_draw_tface_mesh_ob, matnr);
+		if(ma) glColor3f(ma->r, ma->g, ma->b);
+		else glColor3f(0.5, 0.5, 0.5);
+		return 1; /* Set color from mcol if available */
+	} else {
+		return 1; /* Set color from tface */
+	}
+}
 void draw_tface_mesh(Object *ob, Mesh *me, int dt)
 /* maximum dt (drawtype): exactly according values that have been set */
 {
-	TFace *tface=NULL;
-	MFace *mface=NULL;
-	float *extverts= NULL;
 	unsigned char obcol[4];
-	int a, mode;
+	int a;
 	short islight, istex;
+	DerivedMesh *dm;
+	int dmNeedsFree;
 	
 	if(me==NULL) return;
 
+	dm = mesh_get_derived_final(ob, &dmNeedsFree);
 
 	glShadeModel(GL_SMOOTH);
 
@@ -886,142 +888,51 @@ void draw_tface_mesh(Object *ob, Mesh *me, int dt)
 	if(G.vd->drawtype==OB_TEXTURE) istex= 1;
 	else istex= 0;
 
+	g_draw_tface_mesh_ob = ob;
+	g_draw_tface_mesh_islight = islight;
+	g_draw_tface_mesh_istex = istex;
+	memcpy(g_draw_tface_mesh_obcol, obcol, sizeof(obcol));
 	set_draw_settings_cached(1, 0, 0, 0, 0, 0, 0);
 
 	if(dt > OB_SOLID) {
+		TFace *tface= me->tface;
+		MFace *mface= me->mface;
 		bProperty *prop = get_property(ob, "Text");
 		int editing= (G.f & (G_VERTEXPAINT+G_FACESELECT+G_TEXTUREPAINT+G_WEIGHTPAINT)) && (ob==((G.scene->basact) ? (G.scene->basact->object) : 0));
-		MVert *mvert=NULL;
-		int start=0, totface;
+		int start, totface;
 
-		if(mesh_uses_displist(me) && editing==0) {
-			DispList *dl= find_displist(&me->disp, DL_MESH);
-			DispListMesh *dlm= NULL;
+		dm->drawFacesTex(dm, draw_tface_mesh__set_draw);
 
-			if (!dl)
-				totface= 0;
-			else {
-				dlm = dl->mesh;
-				totface= dlm->totface;
-				mvert= dlm->mvert;
-				mface= dlm->mface;
-				tface= dlm->tface;
-			}
-		} 
-		else {
-			DispList *dl= find_displist(&ob->disp, DL_VERTS);
-			if (dl) extverts= dl->verts;
-			
-			/* although buildvars are not supported in engine, they're in Blender... */
-			totface= me->totface;
-			set_buildvars(ob, &start, &totface);
+		start = 0;
+		totface = me->totface;
 
-			mvert= me->mvert;
-			mface= me->mface;
-			tface= me->tface;
-		}
-		
-		// tface can be NULL
-		if(tface==NULL) {
-			
-			for (a=start; a<totface; a++) {
-				int v1idx, v2idx, v3idx, v4idx, mf_smooth, matnr;
-				float *v1, *v2, *v3, *v4;
-				MFace *mf= &mface[a];
-				
-				v1idx= mf->v1;
-				v2idx= mf->v2;
-				v3idx= mf->v3;
-				v4idx= mf->v4;
-				mf_smooth= mf->flag & ME_SMOOTH;
-				matnr= mf->mat_nr;
-				
-				if(v3idx==0) continue;
+		if (!editing && prop && tface) {
+			int dmNeedsFree;
+			DerivedMesh *dm = mesh_get_derived_deform(ob, &dmNeedsFree);
 
-				set_draw_settings_cached(0, 0, NULL, islight, ob, matnr, TF_TWOSIDE);
-				
-				if (extverts) {
-					v1= extverts+3*v1idx;
-					v2= extverts+3*v2idx;
-					v3= extverts+3*v3idx;
-					v4= v4idx?(extverts+3*v4idx):NULL;
-				} else {
-					v1= (mvert+v1idx)->co;
-					v2= (mvert+v2idx)->co;
-					v3= (mvert+v3idx)->co;
-					v4= v4idx?(mvert+v4idx)->co:NULL;
-				}
-				
-				{
-					Material *ma= give_current_material(ob, matnr);
-					if(ma) glColor3f(ma->r, ma->g, ma->b);
-					else glColor3f(0.5, 0.5, 0.5);
-				}
-				
-				glBegin(v4?GL_QUADS:GL_TRIANGLES);
-				
-				if (!mf_smooth) {
-					float nor[3];
-					CalcNormFloat(v1, v2, v3, nor);
-					glNormal3fv(nor);
-				}
-				
-				if (mf_smooth) glNormal3sv(mvert[v1idx].no);
-				glVertex3fv(v1);
-				
-				if (mf_smooth) glNormal3sv(mvert[v2idx].no);
-				glVertex3fv(v2);
-				
-				if (mf_smooth) glNormal3sv(mvert[v3idx].no);
-				glVertex3fv(v3);
-				
-				if(v4) {
-					if (mf_smooth) glNormal3sv(mvert[v4idx].no);
-					glVertex3fv(v4);
-				}
-				glEnd();
-			}
-		}
-		else {	// has tfaces
 			tface+= start;
 			for (a=start; a<totface; a++, tface++) {
-				int v1idx, v2idx, v3idx, v4idx, mf_smooth, matnr, badtex;
-				float *v1, *v2, *v3, *v4;
 				MFace *mf= &mface[a];
+				int mode= tface->mode;
+				int matnr= mf->mat_nr;
+				int mf_smooth= mf->flag & ME_SMOOTH;
 
-				v1idx= mf->v1;
-				v2idx= mf->v2;
-				v3idx= mf->v3;
-				v4idx= mf->v4;
-				mf_smooth= mf->flag & ME_SMOOTH;
-				matnr= mf->mat_nr;
-				
-				if(v3idx==0) continue;
-				if(tface->flag & TF_HIDE) continue;
-				if(tface->mode & TF_INVISIBLE) continue;
-				
-				mode= tface->mode;
-
-				if (extverts) {
-					v1= extverts+3*v1idx;
-					v2= extverts+3*v2idx;
-					v3= extverts+3*v3idx;
-					v4= v4idx?(extverts+3*v4idx):NULL;
-				} else {
-					v1= (mvert+v1idx)->co;
-					v2= (mvert+v2idx)->co;
-					v3= (mvert+v3idx)->co;
-					v4= v4idx?(mvert+v4idx)->co:NULL;
-				}
-
-				badtex= set_draw_settings_cached(0, istex && (mode&TF_TEX), tface, islight && (mode&TF_LIGHT), ob, matnr, mode&TF_TWOSIDE);
-
-				if (prop && !badtex && !editing && (mode & TF_BMFONT)) {
+				if (!(tface->flag&TF_HIDE) && !(mode&TF_INVISIBLE) && (mode&TF_BMFONT)) {
+					int badtex= set_draw_settings_cached(0, g_draw_tface_mesh_istex, tface, g_draw_tface_mesh_islight, g_draw_tface_mesh_ob, matnr, TF_TWOSIDE);
+					float v1[3], v2[3], v3[3], v4[3];
 					char string[MAX_PROPSTRING];
 					int characters, index;
 					Image *ima;
 					float curpos;
 
+					if (badtex)
+						continue;
+
+					dm->getVertCo(dm, mf->v1, v1);
+					dm->getVertCo(dm, mf->v2, v2);
+					dm->getVertCo(dm, mf->v3, v3);
+					if (mf->v4) dm->getVertCo(dm, mf->v4, v4);
+
 					// The BM_FONT handling code is duplicated in the gameengine
 					// Search for 'Frank van Beek' ;-)
 					// string = "Frank van Beek";
@@ -1034,7 +945,7 @@ void draw_tface_mesh(Object *ob, Mesh *me, int dt)
 						characters = 0;
 					}
 
-					if (1 || !mf_smooth) {
+					if (!mf_smooth) {
 						float nor[3];
 
 						CalcNormFloat(v1, v2, v3, nor);
@@ -1043,7 +954,7 @@ void draw_tface_mesh(Object *ob, Mesh *me, int dt)
 					}
 
 					curpos= 0.0;
-					glBegin(v4?GL_QUADS:GL_TRIANGLES);
+					glBegin(mf->v4?GL_QUADS:GL_TRIANGLES);
 					for (index = 0; index < characters; index++) {
 						float centerx, centery, sizex, sizey, transx, transy, movex, movey, advance;
 						int character = string[index];
@@ -1056,7 +967,7 @@ void draw_tface_mesh(Object *ob, Mesh *me, int dt)
 						matrixGlyph(ima->ibuf, character, & centerx, &centery, &sizex, &sizey, &transx, &transy, &movex, &movey, &advance);
 						movex+= curpos;
 
-						if (mode & TF_OBCOL) glColor3ubv(obcol);
+						if (tface->mode & TF_OBCOL) glColor3ubv(obcol);
 						else cp= (char *)&(tface->col[0]);
 
 						glTexCoord2f((tface->uv[0][0] - centerx) * sizex + transx, (tface->uv[0][1] - centery) * sizey + transy);
@@ -1071,7 +982,7 @@ void draw_tface_mesh(Object *ob, Mesh *me, int dt)
 						if (cp) glColor3ub(cp[11], cp[10], cp[9]);
 						glVertex3f(sizex * v3[0] + movex, sizey * v3[1] + movey, v3[2]);
 			
-						if(v4) {
+						if(mf->v4) {
 							glTexCoord2f((tface->uv[3][0] - centerx) * sizex + transx, (tface->uv[3][1] - centery) * sizey + transy);
 							if (cp) glColor3ub(cp[15], cp[14], cp[13]);
 							glVertex3f(sizex * v4[0] + movex, sizey * v4[1] + movey, v4[2]);
@@ -1081,57 +992,19 @@ void draw_tface_mesh(Object *ob, Mesh *me, int dt)
 					}
 					glEnd();
 				}
-				else {
-					char *cp= NULL;
-					
-					if (badtex) glColor3ub(0xFF, 0x00, 0xFF);
-					else if (mode & TF_OBCOL) glColor3ubv(obcol);
-					else cp= (char *)&(tface->col[0]);
-
-					if (!mf_smooth) {
-						float nor[3];
-
-						CalcNormFloat(v1, v2, v3, nor);
-
-						glNormal3fv(nor);
-					}
-
-					glBegin(v4?GL_QUADS:GL_TRIANGLES);
-
-					glTexCoord2fv(tface->uv[0]);
-					if (cp) glColor3ub(cp[3], cp[2], cp[1]);
-					if (mf_smooth) glNormal3sv(mvert[v1idx].no);
-					glVertex3fv(v1);
-					
-					glTexCoord2fv(tface->uv[1]);
-					if (cp) glColor3ub(cp[7], cp[6], cp[5]);
-					if (mf_smooth) glNormal3sv(mvert[v2idx].no);
-					glVertex3fv(v2);
-
-					glTexCoord2fv(tface->uv[2]);
-					if (cp) glColor3ub(cp[11], cp[10], cp[9]);
-					if (mf_smooth) glNormal3sv(mvert[v3idx].no);
-					glVertex3fv(v3);
-		
-					if(v4) {
-						glTexCoord2fv(tface->uv[3]);
-						if (cp) glColor3ub(cp[15], cp[14], cp[13]);
-						if (mf_smooth) glNormal3sv(mvert[v4idx].no);
-						glVertex3fv(v4);
-					}
-					glEnd();
-				}
 			}
 
-			/* switch off textures */
-			set_tpage(0);
+			if (dmNeedsFree) dm->release(dm);
 		}
+
+		/* switch off textures */
+		set_tpage(0);
 	}
 	glShadeModel(GL_FLAT);
 	glDisable(GL_CULL_FACE);
 	
-	if(ob==OBACT && (G.f & G_FACESELECT)) {
-		draw_tfaces3D(ob, me);
+	if(ob==OBACT && (G.f & G_FACESELECT) && me && me->tface) {
+		draw_tfaces3D(ob, me, dm);
 	}
 	
 		/* XXX, bad patch - default_gl_light() calls
@@ -1150,6 +1023,14 @@ void draw_tface_mesh(Object *ob, Mesh *me, int dt)
 	
 	glFrontFace(GL_CCW);
 
+	if(dt > OB_SOLID && !(ob==OBACT && (G.f & G_FACESELECT) && me && me->tface)) {
+		if(ob->flag & SELECT) {
+			BIF_ThemeColor((ob==OBACT)?TH_ACTIVE:TH_SELECT);
+		} else {
+			BIF_ThemeColor(TH_WIRE);
+		}
+		dm->drawLooseEdges(dm);
+	}
 }
 
 void init_realtime_GL(void)
