@@ -1,5 +1,5 @@
 /**
- * $Id: drawseq.c,v 1.26 2004/12/02 18:12:18 ton Exp $
+ * $Id: drawseq.c,v 1.38 2005/11/08 00:46:39 intrr Exp $
  *
  * ***** BEGIN GPL/BL DUAL LICENSE BLOCK *****
  *
@@ -34,10 +34,6 @@
 #include <config.h>
 #endif
 
-#ifdef _WIN32
-#include "BLI_winstuff.h"
-#endif
-
 #include <string.h>
 #include <math.h>
 
@@ -56,16 +52,17 @@
 #include "DNA_space_types.h"
 #include "DNA_view2d_types.h"
 
-#include "BKE_utildefines.h"
 #include "BKE_global.h"
 #include "BKE_plugin_types.h"
+#include "BKE_scene.h"
+#include "BKE_utildefines.h"
 
 #include "BIF_gl.h"
 #include "BIF_mywindow.h"
 #include "BIF_screen.h"
 #include "BIF_drawseq.h"
 #include "BIF_editseq.h"
-#include "BIF_drawimage.h"
+#include "BIF_glutil.h"
 #include "BIF_resources.h"
 #include "BIF_space.h"
 #include "BIF_interface.h"
@@ -202,7 +199,7 @@ static void drawmeta_contents(Sequence *seqm, float x1, float y1, float x2, floa
 	END_SEQ
 }
 
-void drawseqwave(Sequence *seq, float x1, float y1, float x2, float y2)
+static void drawseqwave(Sequence *seq, float x1, float y1, float x2, float y2)
 {
 	float f, height, midy;
 	int offset, sofs, eofs;
@@ -474,6 +471,7 @@ static void draw_image_seq(ScrArea *sa)
 	StripElem *se;
 	struct ImBuf *ibuf;
 	int x1, y1;
+	float zoom;
 
 	glClearColor(0.0, 0.0, 0.0, 0.0);
 	glClear(GL_COLOR_BUFFER_BIT);
@@ -495,13 +493,19 @@ static void draw_image_seq(ScrArea *sa)
 	sseq= sa->spacedata.first;
 	if(sseq==0) return;
 
+	if (sseq->zoom > 0) zoom = sseq->zoom;
+	else zoom = -1.0/sseq->zoom;
 	/* calc location */
-	x1= sa->winrct.xmin+(sa->winx-sseq->zoom*ibuf->x)/2;
-	y1= sa->winrct.ymin+(sa->winy-sseq->zoom*ibuf->y)/2;
+	x1= (sa->winx-zoom*ibuf->x)/2;
+	y1= (sa->winy-zoom*ibuf->y)/2;
 
-	rectwrite_part(sa->winrct.xmin, sa->winrct.ymin,
-				sa->winrct.xmax, sa->winrct.ymax,
-				x1, y1, ibuf->x, ibuf->y, (float)sseq->zoom,(float)sseq->zoom, ibuf->rect);
+	/* needed for gla draw */
+	glaDefine2DArea(&curarea->winrct);
+	glPixelZoom(zoom, zoom);
+	
+	glaDrawPixelsSafe(x1, y1, ibuf->x, ibuf->y, ibuf->rect);
+	
+	glPixelZoom(1.0, 1.0);
 
 	sa->win_swap= WIN_BACK_OK;
 }
@@ -525,7 +529,7 @@ static void draw_extra_seqinfo(void)
 
 	/* NAME */
 	glRasterPos3f(xco,  0.3, 0.0);
-	strcpy(str, give_seqname(last_seq));
+	strncpy(str, give_seqname(last_seq), 255);
 	BMF_DrawString(G.font, str);
 	xco += xfac*BMF_GetStringWidth(G.font, str) +30.0*xfac;
 
@@ -604,13 +608,12 @@ static void draw_extra_seqinfo(void)
 }
 
 #define SEQ_BUT_PLUGIN	1
-#define SEQ_BUT_MOVIE	2
+#define SEQ_BUT_RELOAD	2
 #define SEQ_BUT_EFFECT	3
 
 void do_seqbuttons(short val)
 {
 	extern Sequence *last_seq;
-	StripElem *se;
 
 	switch(val) {
 	case SEQ_BUT_PLUGIN:
@@ -618,17 +621,12 @@ void do_seqbuttons(short val)
 		free_imbuf_effect_spec(CFRA);
 		break;
 
-	case SEQ_BUT_MOVIE:
-		se= last_seq->curelem;
-		if(se && se->ibuf ) {
-			IMB_freeImBuf(se->ibuf);
-			se->ibuf= 0;
-		}
+	case SEQ_BUT_RELOAD:
+		free_imbuf_seq();	// frees all
 		break;
 	case SEQ_BUT_EFFECT:
 		new_stripdata(last_seq);
 		calc_sequence(last_seq);
-		allqueue(REDRAWSEQ, 0);
 		break;
 	}
 
@@ -672,6 +670,11 @@ static void seq_panel_properties(short cntrl)	// SEQ_HANDLER_PROPERTIES
 		uiDefBut(block, LABEL, 0, "Type: Image", 10,140,150,20, 0, 0, 0, 0, 0, "");
 		uiDefBut(block, TEX, B_NOP, "Name: ", 10,120,150,19, last_seq->name+2, 0.0, 21.0, 100, 0, "");
 
+		uiDefButBitS(block, TOG, SEQ_MAKE_PREMUL, SEQ_BUT_RELOAD, "Convert to Premul", 10,90,150,19, &last_seq->flag, 0.0, 21.0, 100, 0, "Converts RGB values to become premultiplied with Alpha");
+		uiDefButBitS(block, TOG, SEQ_FILTERY, SEQ_BUT_RELOAD, "FilterY",	10,70,150,19, &last_seq->flag, 0.0, 21.0, 100, 0, "For video movies to remove fields");
+		uiDefButF(block, NUM, SEQ_BUT_RELOAD, "Mul:",			10,50,150,19, &last_seq->mul, 0.001, 5.0, 100, 0, "Multiply colors");
+		uiDefButS(block, TOG|BIT|7, SEQ_BUT_RELOAD, "Reverse Frames", 10,30,150,19, &last_seq->flag, 0.0, 21.0, 100, 0, "Reverse frame order");
+		uiDefButF(block, NUM, SEQ_BUT_RELOAD, "Strobe:",			10,10,150,19, &last_seq->strobe, 1.0, 30.0, 100, 0, "Only display every nth frame");
 	}
 	else if(last_seq->type==SEQ_META) {
 
@@ -683,7 +686,7 @@ static void seq_panel_properties(short cntrl)	// SEQ_HANDLER_PROPERTIES
 
 		uiDefBut(block, LABEL, 0, "Type: Scene", 10,140,150,20, 0, 0, 0, 0, 0, "");
 		uiDefBut(block, TEX, B_NOP, "Name: ", 10,120,150,19, last_seq->name+2, 0.0, 21.0, 100, 0, "");
-
+		uiDefButS(block, TOG|BIT|7, SEQ_BUT_RELOAD, "Reverse Frames", 10,90,150,19, &last_seq->flag, 0.0, 21.0, 100, 0, "Reverse frame order");
 	}
 	else if(last_seq->type==SEQ_MOVIE) {
 
@@ -692,8 +695,12 @@ static void seq_panel_properties(short cntrl)	// SEQ_HANDLER_PROPERTIES
 		uiDefBut(block, LABEL, 0, "Type: Movie", 10,140,150,20, 0, 0, 0, 0, 0, "");
 		uiDefBut(block, TEX, B_NOP, "Name: ", 10,120,150,19, last_seq->name+2, 0.0, 21.0, 100, 0, "");
 
-		uiDefButS(block, TOG|BIT|4, SEQ_BUT_MOVIE, "FilterY ", 10,90,150,19, &last_seq->flag, 0.0, 21.0, 100, 0, "");
-		uiDefButF(block, NUM, SEQ_BUT_MOVIE, "Mul:", 10,70,150,19, &last_seq->mul, 0.001, 5.0, 100, 0, "");
+		uiDefButBitS(block, TOG, SEQ_MAKE_PREMUL, SEQ_BUT_RELOAD, "Make Premul Alpha ", 10,90,150,19, &last_seq->flag, 0.0, 21.0, 100, 0, "Converts RGB values to become premultiplied with Alpha");
+		uiDefButBitS(block, TOG, SEQ_FILTERY, SEQ_BUT_RELOAD, "FilterY ",	10,70,150,19, &last_seq->flag, 0.0, 21.0, 100, 0, "For video movies to remove fields");
+		uiDefButF(block, NUM, SEQ_BUT_RELOAD, "Mul:",			10,50,150,19, &last_seq->mul, 0.001, 5.0, 100, 0, "Multiply colors");
+		
+		uiDefButS(block, TOG|BIT|7, SEQ_BUT_RELOAD, "Reverse Frames", 10,30,150,19, &last_seq->flag, 0.0, 21.0, 100, 0, "Reverse frame order");
+		uiDefButF(block, NUM, SEQ_BUT_RELOAD, "Strobe:",			10,10,150,19, &last_seq->strobe, 1.0, 30.0, 100, 0, "Only display every nth frame");
 
 	}
 	else if(last_seq->type==SEQ_SOUND) {
@@ -701,9 +708,9 @@ static void seq_panel_properties(short cntrl)	// SEQ_HANDLER_PROPERTIES
 		uiDefBut(block, LABEL, 0, "Type: Audio", 10,140,150,20, 0, 0, 0, 0, 0, "");
 		uiDefBut(block, TEX, 0, "Name: ", 10,120,150,19, last_seq->name+2, 0.0, 21.0, 100, 0, "");
 
-		uiDefButS(block, TOG|BIT|5, B_NOP, "Mute", 10,90,120,19, &last_seq->flag, 0.0, 21.0, 100, 0, "");
-		uiDefButF(block, NUM, SEQ_BUT_MOVIE, "Gain (dB):", 10,70,150,19, &last_seq->level, -96.0, 6.0, 100, 0, "");
-		uiDefButF(block, NUM, SEQ_BUT_MOVIE, "Pan:", 	10,50,150,19, &last_seq->pan, -1.0, 1.0, 100, 0, "");
+		uiDefButBitS(block, TOG, SEQ_MUTE, B_NOP, "Mute", 10,90,120,19, &last_seq->flag, 0.0, 21.0, 100, 0, "");
+		uiDefButF(block, NUM, SEQ_BUT_RELOAD, "Gain (dB):", 10,70,150,19, &last_seq->level, -96.0, 6.0, 100, 0, "");
+		uiDefButF(block, NUM, SEQ_BUT_RELOAD, "Pan:", 	10,50,150,19, &last_seq->pan, -1.0, 1.0, 100, 0, "");
 	}
 	else if(last_seq->type>=SEQ_EFFECT) {
 		uiDefBut(block, LABEL, 0, "Type: Effect", 10,140,150,20, 0, 0, 0, 0, 0, "");
@@ -711,8 +718,8 @@ static void seq_panel_properties(short cntrl)	// SEQ_HANDLER_PROPERTIES
 
 		if(last_seq->type==SEQ_WIPE){
 			WipeVars *wipe = (WipeVars *)last_seq->effectdata;
-			char formatstring[1024];
-			strcpy(formatstring, "Transition Type %t|Single Wipe%x0|Double Wipe %x1|Iris Wipe %x4|Clock Wipe %x5");
+			char formatstring[256];
+			strncpy(formatstring, "Transition Type %t|Single Wipe%x0|Double Wipe %x1|Iris Wipe %x4|Clock Wipe %x5", 255);
 			uiDefButS(block, MENU,SEQ_BUT_EFFECT, formatstring,	10,90,220,22, &wipe->wipetype, 0, 0, 0, 0, "What type of wipe should be performed");
 			uiDefButF(block, NUM,SEQ_BUT_EFFECT,"Blur:",	10,65,220,22, &wipe->edgeWidth,0.0,1.0, 1, 2, "The percent width of the blur edge");
 			switch(wipe->wipetype){ /*Skip Types that do not require angle*/
@@ -777,7 +784,6 @@ void drawseqspace(ScrArea *sa, void *spacedata)
 	sseq= curarea->spacedata.first;
 	if(sseq->mainb==1) {
 		draw_image_seq(curarea);
-		curarea->win_swap= WIN_BACK_OK;
 		return;
 	}
 
@@ -790,7 +796,7 @@ void drawseqspace(ScrArea *sa, void *spacedata)
 
 	glClear(GL_COLOR_BUFFER_BIT);
 
-	calc_scrollrcts(G.v2d, curarea->winx, curarea->winy);
+	calc_scrollrcts(sa, G.v2d, curarea->winx, curarea->winy);
 
 	if(curarea->winx>SCROLLB+10 && curarea->winy>SCROLLH+10) {
 		if(G.v2d->scroll) {
