@@ -1,5 +1,5 @@
 /**
- * $Id: drawtext.c,v 1.42 2004/11/27 18:32:26 ianwill Exp $
+ * $Id: drawtext.c,v 1.54 2005/10/09 18:47:07 zuster Exp $
  *
  * ***** BEGIN GPL/BL DUAL LICENSE BLOCK *****
  *
@@ -85,9 +85,20 @@
 
 #define TEXTXLOC	38
 
-/* locals */
+/* forward declarations */
+
 void drawtextspace(ScrArea *sa, void *spacedata);
 void winqreadtextspace(struct ScrArea *sa, void *spacedata, struct BWinEvent *evt);
+void txt_copy_selectbuffer (Text *text);
+void txt_paste_clipboard(Text *text); /* blank on non Win32 */
+void txt_copy_clipboard(Text *text); /* blank on non Win32 */
+void do_brackets();
+
+int check_bracket(char *string);
+static int check_delim(char *string);
+static int check_numbers(char *string);
+static int check_builtinfuncs(char *string);
+static int check_specialvars(char *string);
 
 static void *last_txt_find_string= NULL;
 
@@ -176,7 +187,352 @@ static int render_string (char *in) {
 	return r;
 }
 
-static int text_draw(SpaceText *st, char *str, int cshift, int maxwidth, int draw, int x, int y) {
+void get_format_string(void) 
+{
+	SpaceText *st = curarea->spacedata.first;
+	Text *text = st->text;
+	TextLine *tmp;
+	char *in_line;
+	char format[2000], check[200], other[2];
+	unsigned char c;
+	int a, b, len, spot, letter, tabs, mem_amount;
+	
+	if(!text) return;
+	tmp = text->lines.first;
+	
+	while(tmp) {
+		in_line = tmp->line;
+		
+		len = strlen(in_line);
+		/* weak code... but we dont want crashes (ton) */
+		if(len>2000-1) {
+			if (tmp->format) MEM_freeN(tmp->format);
+			tmp->format= NULL;
+		}
+		else {
+			
+			spot = 0;
+			tabs = 0;
+			//see how many tabs we have
+			for(a = 0; a <len; a++) {
+				c = (unsigned char) in_line[a];
+				if(c == '\t') {
+					tabs++;
+				}
+			}
+			//calculate the amount of MEM_mallocN we neen
+			mem_amount = (((tabs*st->tabnumber)-tabs)+2)+len; // +2 for good measure
+			if (tmp->format) MEM_freeN(tmp->format);
+			tmp->format = MEM_mallocN(mem_amount, "Syntax_format");
+			
+			for (a = 0; a < len; a++) {
+				c = (unsigned char) in_line[a];
+
+				check[0] = c;
+				check[1] = '\0';
+
+				if (check_delim(check))
+				{
+					switch (c) {
+						case '\"':
+							if(in_line[a] == '\"' && in_line[a+1] == '\"' && in_line[a+2] == '\"') { 
+								format[spot] = format[spot+1] = format[spot+2] = 'l';
+								spot +=3;
+								a += 3;
+								while(in_line[a] != '\"' || in_line[a-1] != '\"' || in_line[a-2] != '\"') {
+									c = (unsigned char) in_line[a];
+									if(a >= len) {
+										format[spot] = '\0';
+										memcpy(tmp->format, format, strlen(format));
+										if(!(tmp= tmp->next)) {
+											return;
+										} else {
+											in_line = tmp->line;
+											len = strlen(in_line);
+											tabs = 0;
+											for(b = 0; b <len; b++) {
+												c = (unsigned char) in_line[b];
+												if(c == '\t') {
+													tabs++;
+												}
+											}
+											mem_amount = (((tabs*st->tabnumber)-tabs)+2)+len;
+											if (tmp->format) MEM_freeN(tmp->format);
+											tmp->format = MEM_mallocN(mem_amount, "Syntax_format");
+											a = 0; spot = 0;
+										}
+									} else {
+										if(c == '\t' || c == ' ') {
+											if(c == '\t') {
+												for(b = st->tabnumber-(spot%st->tabnumber); b > 0; b--) {
+													format[spot] = ' ';
+													spot++;
+												}
+												a++;
+											} else {
+												format[spot] = ' ';
+												a++; spot++;
+										}
+										} else {
+											format[spot] = 'l';
+											a++; spot++;
+										}
+									}
+								}
+								format[spot] = 'l';
+								spot++;
+							} else {
+								format[spot] = 'l';
+								a++; spot++;
+								while(in_line[a] != '\"') {
+									c = (unsigned char) in_line[a];
+									if(a >= len) {
+										format[spot] = '\0';
+										memcpy(tmp->format, format, strlen(format));
+										if(!(tmp= tmp->next)) {
+											return;
+										} else {
+											in_line = tmp->line;
+											len = strlen(in_line);
+											for(b = 0; b <len; b++) {
+												c = (unsigned char) in_line[b];
+												if(c == '\t') {
+													tabs++;
+												}
+											}
+											//calculate the amount of MEM_mallocN we neen
+											mem_amount = (((tabs*st->tabnumber)-tabs)+2)+len;
+											if (tmp->format) MEM_freeN(tmp->format);
+											tmp->format = MEM_mallocN(mem_amount, "Syntax_format");
+											a = 0; spot = 0;
+										}
+									}
+									if(c == '\t' || c == ' ') {
+										if(c == '\t') {
+											for(b = st->tabnumber-(spot%st->tabnumber); b > 0; b--) {
+												format[spot] = ' ';
+												spot++;
+											}
+											a++;
+										} else {
+											format[spot] = ' ';
+											a++; spot++;
+										}
+									} else {
+										format[spot] = 'l';
+										a++; spot++;
+									}
+								}
+								format[spot] = 'l';
+								spot++;
+							}
+							break;
+						case '\'':
+							if(in_line[a] == '\'' && in_line[a+1] == '\'' && in_line[a+2] == '\'') { 
+								format[spot] = format[spot+1] = format[spot+2] = 'l';
+								spot +=3;
+								a += 3;
+								while(in_line[a] != '\'' || in_line[a-1] != '\'' || in_line[a-2] != '\'') {
+									c = (unsigned char) in_line[a];
+									if(a >= len) {
+										format[spot] = '\0';
+										memcpy(tmp->format, format, strlen(format));
+										if(!(tmp= tmp->next)) {
+											return;
+										} else {
+											in_line = tmp->line;
+											len = strlen(in_line);
+											tabs = 0;
+											for(b = 0; b <len; b++) {
+												c = (unsigned char) in_line[b];
+												if(c == '\t') {
+													tabs++;
+												}
+											}
+											mem_amount = (((tabs*st->tabnumber)-tabs)+2)+len;
+											if (tmp->format) MEM_freeN(tmp->format);
+											tmp->format = MEM_mallocN(mem_amount, "Syntax_format");
+											a = 0; spot = 0;
+										}
+									} else {
+										if(c == '\t' || c == ' ') {
+											if(c == '\t') {
+												for(b = st->tabnumber-(spot%st->tabnumber); b > 0; b--) {
+													format[spot] = ' ';
+													spot++;
+												}
+												a++;
+											} else {
+												format[spot] = ' ';
+												a++; spot++;
+											}
+										} else {
+											format[spot] = 'l';
+											a++; spot++;
+										}
+									}
+								}
+								format[spot] = 'l';
+								spot++;
+							} else {
+								format[spot] = 'l';
+								a++; spot++;
+								while(in_line[a] != '\'') {
+									c = (unsigned char) in_line[a];
+									if(a >= len) {
+										format[spot] = '\0';
+										memcpy(tmp->format, format, strlen(format));
+										if(!(tmp= tmp->next)) {
+											return;
+										} else {
+											in_line = tmp->line;
+											len = strlen(in_line);
+											for(b = 0; b <len; b++) {
+												c = (unsigned char) in_line[b];
+												if(c == '\t') {
+													tabs++;
+												}
+											}
+											//calculate the amount of MEM_mallocN we neen
+											mem_amount = (((tabs*st->tabnumber)-tabs)+2)+len;
+											if (tmp->format) MEM_freeN(tmp->format);
+											tmp->format = MEM_mallocN(mem_amount, "Syntax_format");
+											a = 0; spot = 0;
+										}
+									}
+									if(c == '\t' || c == ' ') {
+										if(c == '\t') {
+											for(b = st->tabnumber-(spot%st->tabnumber); b > 0; b--) {
+												format[spot] = ' ';
+												spot++;
+											}
+											a++;
+										} else {
+											format[spot] = ' ';
+											a++; spot++;
+										}
+									} else {
+										format[spot] = 'l';
+										a++; spot++;
+									}
+								}
+								format[spot] = 'l';
+								spot++;
+							}
+							break;
+						case '#':
+							while(a<len) {
+								c = (unsigned char) in_line[a];
+								if(c == '\t' || c == ' ') {
+									if(c == '\t') {
+										for(b = st->tabnumber-(spot%st->tabnumber); b > 0; b--) {
+											format[spot] = '#';
+											spot++;
+										}
+										a++;
+									} else {
+										format[spot] = '#';
+										a++; spot++;
+									}
+								} else {
+									format[spot] = '#';
+									a++; spot++;
+								}
+							}
+							break;
+						case ' ':
+							format[spot] = ' ';
+							spot++;
+							break;
+						case '\t':
+							for(b = st->tabnumber-(spot%st->tabnumber); b > 0; b--) {
+								format[spot] = ' ';
+								spot++;
+							}
+							break;
+						default:
+							format[spot] = 'q';
+							spot++;
+							 
+							break;
+					}
+				} else if (check_numbers(check)) {
+					while (a < len) {
+						c = (unsigned char) in_line[a];
+						other[0] = c;
+						other[1] = '\0';
+						if (check_delim(other) && c != '.') {
+							a--; break;
+						} else {
+							format[spot] = 'n';
+							a++; spot++;
+						}
+					}
+				} else {
+					letter = 0;
+					while (a < len) {
+						c = (unsigned char) in_line[a];
+						other[0] = c;
+						other[1] = '\0';
+						if (check_delim(other)) {
+							a--; 
+							break;
+						} else {
+							check[letter] = (unsigned char) in_line[a];
+							letter++; 
+							a++;
+						}
+					}
+					check[letter] = '\0';
+					if (check_builtinfuncs(check)) {
+						for (b = 0; b < strlen(check); b++) {
+							format[spot] = 'b'; 
+							spot++;
+						}
+					} else if (check_specialvars(check)) { /*If TRUE then color and color next word*/
+						for (b = 0; b < strlen(check); b++) {
+							format[spot] = 'b';
+							spot++;
+						}
+						a++;
+						format[spot] = 'q';
+						spot++; a++;
+						letter = 0;
+						while (a < len) {
+							c = (unsigned char) in_line[a];
+							other[0] = c;
+							other[1] = '\0';
+							if (check_delim(other)) {
+								a--; 
+								break;
+							} else {
+								check[letter] = (unsigned char) in_line[a];
+								letter++; 
+								a++;
+							}
+						}
+						check[letter] = '\0';
+						for (b = 0; b < strlen(check); b++) {
+							format[spot] = 'v';
+							spot++;
+						}
+					}else {
+						for (b = 0; b < strlen(check); b++) {
+							format[spot] = 'q';
+							spot++;
+						}
+					}
+				}
+			}
+			format[spot] = '\0';
+			memcpy(tmp->format, format, strlen(format));
+		}
+		
+		tmp = tmp->next;
+	}
+}
+
+static int text_draw(SpaceText *st, char *str, int cshift, int maxwidth, int draw, int x, int y, char *format) {
 	int r=0, w= 0;
 	char *in;
 	int *acc;
@@ -187,10 +543,49 @@ static int text_draw(SpaceText *st, char *str, int cshift, int maxwidth, int dra
 	in= temp_char_buf+cshift;
 	acc= temp_char_accum+cshift;
 	w= w-cshift;
-	
+
 	if (draw) {
-		glRasterPos2i(x, y);
-		BMF_DrawString(spacetext_get_font(st), in);
+		if(st->showsyntax && format) {
+			int amount, a;
+			char out[2];
+			format = format+cshift;
+		
+			amount = strlen(in);
+			
+			for(a = 0; a < amount; a++) {
+				out[0] = (unsigned char) in[a]; 
+				out[1] = '\0';
+				switch (format[a]) {
+					case 'l':
+						BIF_ThemeColor(TH_SYNTAX_L);
+						break;
+					case 'b':
+						BIF_ThemeColor(TH_SYNTAX_B);
+						break;
+					case '#':
+						BIF_ThemeColor(TH_SYNTAX_C);
+						break;
+					case 'v': 
+						BIF_ThemeColor(TH_SYNTAX_V);
+						break;
+					case 'n':
+						BIF_ThemeColor(TH_SYNTAX_N);
+						break;
+					case 'q':
+						BIF_ThemeColor(TH_TEXT);
+						break;
+					default:
+						BIF_ThemeColor(TH_TEXT);
+						break;
+				}
+				glRasterPos2i(x, y);
+				BMF_DrawString(spacetext_get_font(st), out);
+				x = x+BMF_GetStringWidth(spacetext_get_font(st), out);
+			}
+		} else {
+			glRasterPos2i(x, y);
+			BMF_DrawString(spacetext_get_font(st), in);
+		}
 	} else {
 		while (w-- && *acc++ < maxwidth) {
 			r+= spacetext_get_fontwidth(st);
@@ -248,7 +643,7 @@ static void draw_cursor(SpaceText *st) {
 	int charf, charl;
 	
 	if (text->curl==text->sell && text->curc==text->selc) {
-		x= text_draw(st, text->curl->line, st->left, text->curc, 0, 0, 0);
+		x= text_draw(st, text->curl->line, st->left, text->curc, 0, 0, 0, NULL);
 
 		if (x) {
 			h= txt_get_span(text->lines.first, text->curl) - st->top;
@@ -288,7 +683,7 @@ static void draw_cursor(SpaceText *st) {
 		h= txt_get_span(text->lines.first, linef) - st->top;
 		while (h++<-1 && linef!=linel) linef= linef->next;
 	
-		x= text_draw(st, linef->line, st->left, charf, 0, 0, 0);
+		x= text_draw(st, linef->line, st->left, charf, 0, 0, 0, NULL);
 
 		BIF_ThemeColor(TH_SHADE2);
 
@@ -320,11 +715,12 @@ static void draw_cursor(SpaceText *st) {
 		
 		h= txt_get_span(text->lines.first, linef) - st->top;
 
-		i= text_draw(st, linel->line, st->left, charl, 0, 0, 0);
+		i= text_draw(st, linel->line, st->left, charl, 0, 0, 0, NULL);
 		if(i) glRecti(x, curarea->winy-st->lheight*(h)-2, i, curarea->winy-st->lheight*(h+1)-2);
 
 	}
-	
+
+	do_brackets();
 	BIF_ThemeColor(TH_TEXT);
 }
 
@@ -351,7 +747,7 @@ static void calc_text_rcts(SpaceText *st)
 	CLAMP(st->txtbar.ymax, 2, curarea->winy-2);
 
 	st->pix_per_line= (float) ltexth/curarea->winy;
-	if (st->pix_per_line<.1) st->pix_per_line=.1;
+	if (st->pix_per_line<.1) st->pix_per_line=.1f;
 
 	lbarstart= MIN2(txt_get_span(st->text->lines.first, st->text->curl), 
 				txt_get_span(st->text->lines.first, st->text->sell));
@@ -575,8 +971,16 @@ void drawtextspace(ScrArea *sa, void *spacedata)
 		tmp= tmp->next;
 		linecount++;
 	}
+	
+	if(st->showsyntax) {
+		if (!tmp->format) {
+			get_format_string();
+		}
+	}
+	
 	for (i=0; i<st->viewlines && tmp; i++, tmp= tmp->next) {
 		if(st->showlinenrs) {
+			BIF_ThemeColor(TH_TEXT);
 			if(((float)(i + linecount + 1)/10000.0) < 1.0) {
 				sprintf(linenr, "%4d", i + linecount + 1);
 				glRasterPos2i(TXT_OFFSET - 7, curarea->winy-st->lheight*(i+1));
@@ -585,9 +989,9 @@ void drawtextspace(ScrArea *sa, void *spacedata)
 				glRasterPos2i(TXT_OFFSET - 11, curarea->winy-st->lheight*(i+1));
 			}
 			BMF_DrawString(spacetext_get_font(st), linenr);
-			text_draw(st, tmp->line, st->left, 0, 1, TXT_OFFSET + TEXTXLOC, curarea->winy-st->lheight*(i+1));
+			text_draw(st, tmp->line, st->left, 0, 1, TXT_OFFSET + TEXTXLOC, curarea->winy-st->lheight*(i+1), tmp->format);
 		} else
-			text_draw(st, tmp->line, st->left, 0, 1, TXT_OFFSET, curarea->winy-st->lheight*(i+1));
+			text_draw(st, tmp->line, st->left, 0, 1, TXT_OFFSET, curarea->winy-st->lheight*(i+1), tmp->format);
 	}
 
 	draw_textscroll(st);
@@ -608,7 +1012,7 @@ void pop_space_text (SpaceText *st)
 		st->top= i - st->viewlines/2;
 	}
 	
-	x= text_draw(st, st->text->curl->line, st->left, st->text->curc, 0, 0, 0);
+	x= text_draw(st, st->text->curl->line, st->left, st->text->curc, 0, 0, 0, NULL);
 
 	if (x==0 || x>curarea->winx) {
 		st->left= st->text->curc-0.5*(curarea->winx)/spacetext_get_fontwidth(st);
@@ -631,7 +1035,8 @@ void add_text_fs(char *file)
 	st->text= text;
 
 	st->top= 0;
-			
+
+	if (st->showsyntax) get_format_string();
 	allqueue(REDRAWTEXT, 0);
 	allqueue(REDRAWHEADERS, 0);	
 }
@@ -839,7 +1244,7 @@ void txt_copy_selectbuffer (Text *text)
 
 
 #ifdef _WIN32
-char *unixNewLine(char *buffer)
+static char *unixNewLine(char *buffer)
 {
 	char *p, *p2, *output;
 	
@@ -852,7 +1257,7 @@ char *unixNewLine(char *buffer)
 	return(output);
 }
 
-char *winNewLine(char *buffer)
+static char *winNewLine(char *buffer)
 {
 	char *p, *p2, *output;
 	int add= 0;
@@ -885,6 +1290,7 @@ void txt_paste_clipboard(Text *text) {
 		txt_insert_buf(text, buffer);
 		GlobalUnlock( hData );
 		CloseClipboard();
+		MEM_freeN(buffer);
 	}
 #endif
 }
@@ -960,7 +1366,7 @@ void run_python_script(SpaceText *st)
 	char *py_filename;
 	Text *text=st->text;
 
-	if (!BPY_txt_do_python(st)) {
+	if (!BPY_txt_do_python_Text(text)) {
 		int lineno = BPY_Err_getLinenumber();
 		// jump to error if happened in current text:
 		py_filename = (char*) BPY_Err_getFilename();
@@ -983,23 +1389,10 @@ void run_python_script(SpaceText *st)
 	}
 }
 
-void set_tabs(Text *text) {
-	
-	TextLine *line = text->curl;
+static void set_tabs(Text *text)
+{
 	SpaceText *st = curarea->spacedata.first;
-	int pos = 0;
-	int max;
-	max = line->len;
-	st->currtab_set = 0;
-	while ( pos < max-1) {
-		if (line->line[pos] == '\t') {
-			st->currtab_set++;
-			pos++;
-		}
-		else {
-			pos++;
-		}
-	}
+	st->currtab_set = setcurr_tab(text);
 }
 
 void winqreadtextspace(ScrArea *sa, void *spacedata, BWinEvent *evt)
@@ -1119,6 +1512,7 @@ void winqreadtextspace(ScrArea *sa, void *spacedata, BWinEvent *evt)
 		}
 	} else if (ascii) {
 		if (txt_add_char(text, ascii)) {
+			if (st->showsyntax) get_format_string();
 			pop_space_text(st);
 			do_draw= 1;
 		}
@@ -1147,6 +1541,7 @@ void winqreadtextspace(ScrArea *sa, void *spacedata, BWinEvent *evt)
 		case DKEY:
 			if (G.qual == LR_CTRLKEY) {
 				txt_delete_char(text);
+				if (st->showsyntax) get_format_string();
 				do_draw= 1;
 				pop_space_text(st);
 			}
@@ -1164,6 +1559,7 @@ void winqreadtextspace(ScrArea *sa, void *spacedata, BWinEvent *evt)
 					break;
 				case 2:
 					txt_paste(text);
+					if (st->showsyntax) get_format_string();
 					do_draw= 1;
 					break;
 				case 3:
@@ -1251,6 +1647,7 @@ void winqreadtextspace(ScrArea *sa, void *spacedata, BWinEvent *evt)
 				if (okee("Reopen text")) {
 					if (!reopen_text(text))
 						error("Could not reopen file");
+				if (st->showsyntax) get_format_string();
 				}
 				do_draw= 1;	
 			}
@@ -1294,6 +1691,7 @@ void winqreadtextspace(ScrArea *sa, void *spacedata, BWinEvent *evt)
 			}
 			if (G.qual == LR_ALTKEY) {
 				txt_do_undo(text);
+				if (st->showsyntax) get_format_string();
 				do_draw= 1;
 			}
 			break; /* BREAK U */
@@ -1327,6 +1725,7 @@ void winqreadtextspace(ScrArea *sa, void *spacedata, BWinEvent *evt)
 					txt_paste_clipboard(text);
 				else
 					txt_paste(text);
+				if (st->showsyntax) get_format_string();
 				do_draw= 1;	
 				pop_space_text(st);
 			}
@@ -1334,6 +1733,7 @@ void winqreadtextspace(ScrArea *sa, void *spacedata, BWinEvent *evt)
 		case XKEY:
 			if (G.qual == LR_ALTKEY || G.qual == LR_CTRLKEY) {
 				txt_cut_sel(text);
+				if (st->showsyntax) get_format_string();
 				do_draw= 1;	
 				pop_space_text(st);
 			}
@@ -1345,53 +1745,61 @@ void winqreadtextspace(ScrArea *sa, void *spacedata, BWinEvent *evt)
 				} else {
 					txt_do_undo(text);
 				}
+				if (st->showsyntax) get_format_string();
 				do_draw= 1;
 			}
 			break;
 		case TABKEY:
 			if (G.qual & LR_SHIFTKEY) {
 				if (txt_has_sel(text)) {
-					txt_cut_sel(text);
+					txt_order_cursors(text);
 					unindent(text);
 					
 				}
 			} else {
 				if ( txt_has_sel(text)) {
-					txt_cut_sel(text);
-					indent_paste(text);
+					txt_order_cursors(text);
+					indent(text);
 				} else {
 					txt_add_char(text, '\t');
-					st->currtab_set++;
 				}
 			}
+			if (st->showsyntax) get_format_string();
 			pop_space_text(st);
 			do_draw= 1;
+			st->currtab_set = setcurr_tab(text);
 			break;
 		case RETKEY:
+			//double check tabs before splitting the line
+			st->currtab_set = setcurr_tab(text);
 			txt_split_curline(text);
 			{
 				int a = 0;
-				while ( a < st->currtab_set) {
-					txt_add_char(text, '\t');
-					a++;
+				if (a < st->currtab_set)
+				{
+					while ( a < st->currtab_set) {
+						txt_add_char(text, '\t');
+						a++;
+					}
 				}
 			}
+			if (st->showsyntax) get_format_string();
 			do_draw= 1;
 			pop_space_text(st);
 			break;
 		case BACKSPACEKEY:
 			txt_backspace_char(text);
 			set_tabs(text);
+			if (st->showsyntax) get_format_string();
 			do_draw= 1;
 			pop_space_text(st);
 			break;
 		case DELKEY:
-			if ( text->curl->line[text->curc] == '\t') {
-					st->currtab_set--;
-			}
 			txt_delete_char(text);
+			if (st->showsyntax) get_format_string();
 			do_draw= 1;
 			pop_space_text(st);
+			st->currtab_set = setcurr_tab(text);
 			break;
 		case DOWNARROWKEY:
 			txt_move_down(text, G.qual & LR_SHIFTKEY);
@@ -1458,3 +1866,179 @@ void winqreadtextspace(ScrArea *sa, void *spacedata, BWinEvent *evt)
 		}
 	}
 }
+
+void do_brackets(void) 
+{
+	SpaceText *st = curarea->spacedata.first;
+	Text *text = st->text;
+	TextLine *tmp, *start;
+	char test[2];
+	int d, pos, open, x, y, x2, y2, h=0;
+	
+	if(!text) return;
+	
+	tmp = text->curl;
+	start = text->curl;
+
+	test[0] = (unsigned char) tmp->line[text->curc];
+	test[1] = '\0';
+	
+	d = check_bracket(test);
+	if (!d) /*  If not pri char */
+	{
+		test[0] = (unsigned char) tmp->line[text->curc-1];
+		test[1] = '\0';
+		d = check_bracket(test);
+		if(!d) {
+			return; /*If the current char or prev is not a bracket then return*/
+		} else { /* current char */
+			h= txt_get_span(text->lines.first, start) - st->top;
+			x = text_draw(st, start->line, st->left, text->curc-1, 0, 0, 0, NULL);
+			y = text_draw(st, start->line, st->left, text->curc, 0, 0, 0, NULL);
+			if (d < 4) {
+				pos = text->curc;
+			} else {
+				pos = text->curc-2;
+			}
+		}
+	} else { /* is pri char */
+		h= txt_get_span(text->lines.first, start) - st->top;
+		x = text_draw(st, start->line, st->left, text->curc, 0, 0, 0, NULL);
+		y = text_draw(st, start->line, st->left, text->curc+1, 0, 0, 0, NULL);
+		if (d < 4) {
+			pos = text->curc+1;
+		} else {
+			pos = text->curc-1;
+		}
+	}
+	
+	if (d < 4) /*reading forward*/
+	{
+		open = 1; 
+		while ( tmp ) {
+			while (pos <= tmp->len) {
+				test[0] = (unsigned char) tmp->line[pos];
+				test[1] = '\0';
+				if(check_bracket(test) == d) {
+					open++;
+				} else if (check_bracket(test) == d+3) {
+					open--;
+					if (open == 0) {
+						BIF_ThemeColorBlend(TH_BACK, TH_SHADE2, 0.5);
+						glRecti(x, curarea->winy-st->lheight*(h)-2, y, curarea->winy-st->lheight*(h+1)-2);
+
+						h= txt_get_span(text->lines.first, tmp) - st->top;
+						x2= text_draw(st, tmp->line, st->left, pos, 0, 0, 0, NULL);
+						y2= text_draw(st, tmp->line, st->left, pos+1, 0, 0, 0, NULL);
+						glRecti(x2, curarea->winy-st->lheight*(h)-2, y2, curarea->winy-st->lheight*(h+1)-2);
+						BIF_ThemeColor(TH_TEXT);
+						return;
+					}
+				}
+				pos++;
+			}
+			tmp = tmp->next;
+			pos = 0;
+		}
+	} else { /*  reading back */
+		open = 1; 
+		while ( tmp ) {
+			while (pos >= 0) {
+				test[0] = (unsigned char) tmp->line[pos];
+				test[1] = '\0';
+				if(check_bracket(test) == d) {
+					open++;
+				} else if (check_bracket(test) == d-3) {
+					open--;
+					if (open == 0) {
+						BIF_ThemeColorBlend(TH_BACK, TH_SHADE2, 0.5);
+						glRecti(x, curarea->winy-st->lheight*(h)-2, y, curarea->winy-st->lheight*(h+1)-2);
+
+						h= txt_get_span(text->lines.first, tmp) - st->top;
+						x2= text_draw(st, tmp->line, st->left, pos, 0, 0, 0, NULL);
+						y2= text_draw(st, tmp->line, st->left, pos+1, 0, 0, 0, NULL);
+						glRecti(x2, curarea->winy-st->lheight*(h)-2, y2, curarea->winy-st->lheight*(h+1)-2);
+						BIF_ThemeColor(TH_TEXT);
+						return;
+					}
+				}
+				pos--;
+			}
+			tmp = tmp->prev;
+			if (tmp) {
+				pos = tmp->len;
+			}
+		}
+	}
+	
+}
+
+int check_bracket(char *string)
+{
+	int number, a = 0;
+	char other[][3] = {"(", "[", "{", ")", "]", "}"};
+	
+	number = 6;
+	
+	while(a < number) {
+		if(strcmp(other[a], string) == 0)
+		{
+			return a+1;
+		}
+		a++;
+	}
+	return 0;
+}
+
+static int check_builtinfuncs(char *string) 
+{
+	int number = 30, a = 0;
+	
+	char builtinfuncs[][11] = {"and", "as", "assert", "break", "class", "continue", "def",
+								"del", "elif", "else", "except", "exec", "finally",
+								"for", "from", "global", "if", "import", "in",
+								"is", "lambda", "not", "or", "pass", "print",
+								"raise", "return", "try", "while", "yield"};
+
+	for( a = 0; a < number; a++) {
+		if(!strcmp(builtinfuncs[a], string))
+			return 1;
+	}
+	return 0;
+}
+
+static int check_specialvars(char *string) 
+{
+	int number = 2, a = 0;
+	char specialvars[][7] = {"def", "class"};
+	
+	for( a = 0; a < number; a++) {
+		if(!strcmp(specialvars[a], string))
+			return a+1;
+	}
+	return 0;
+}
+
+static int check_delim(char *string) 
+{
+	int number = 28, a = 0;
+	char other[][3] = {"(", ")", ":", "\"", "\'", " ", "~", "!", "%", "^", "&", "*", "-", "+", "=", "[", "]", "{", "}", ";", "/", "<", ">", "|", ".", "#", "\t", ","};
+	
+	for( a = 0; a < number; a++) {
+		if(!strcmp(other[a], string))
+			return 1;
+	}
+	return 0;
+}
+
+static int check_numbers(char *string)
+{
+	int number = 10, a = 0;
+	char other[][2] = {"0", "1", "2", "3", "4", "5", "6", "7", "8", "9"};
+	
+	for( a = 0; a < number; a++) {
+		if(!strcmp(other[a], string))
+			return 1;
+	}
+	return 0;
+}
