@@ -1,5 +1,5 @@
 /**
- * $Id: edit.c,v 1.52 2004/12/13 17:11:33 ton Exp $
+ * $Id: edit.c,v 1.81 2006/01/02 10:40:12 campbellbarton Exp $
  *
  * ***** BEGIN GPL/BL DUAL LICENSE BLOCK *****
  *
@@ -31,6 +31,7 @@
  */
 
 #include <math.h>
+#include <string.h>
 
 #ifdef HAVE_CONFIG_H
 #include <config.h>
@@ -40,39 +41,49 @@
 #include <unistd.h>
 #else
 #include <io.h>
-#include "BLI_winstuff.h"
 #endif   
+
 #include "MEM_guardedalloc.h"
 
 #include "BMF_Api.h"
 
 #include "PIL_time.h"
 
+#include "DNA_action_types.h"
 #include "DNA_armature_types.h"
 #include "DNA_curve_types.h"
+#include "DNA_ipo_types.h"
 #include "DNA_lattice_types.h"
 #include "DNA_meta_types.h"
 #include "DNA_mesh_types.h"
+#include "DNA_modifier_types.h"
 #include "DNA_object_types.h"
 #include "DNA_screen_types.h"
 #include "DNA_scene_types.h"
 #include "DNA_space_types.h"
 #include "DNA_view3d_types.h"
-
+#include "DNA_userdef_types.h"	/* for U.dupflag */
 #include "BLI_blenlib.h"
 #include "BLI_arithb.h"
 #include "BLI_editVert.h"
 
-#include "BKE_utildefines.h"
+#include "BKE_action.h"
+#include "BKE_armature.h"
 #include "BKE_anim.h"
-#include "BKE_object.h"
+#include "BKE_curve.h"
+#include "BKE_depsgraph.h"
 #include "BKE_displist.h"
 #include "BKE_global.h"
+#include "BKE_ipo.h"
 #include "BKE_lattice.h"
 #include "BKE_mesh.h"
+#include "BKE_modifier.h"
+#include "BKE_object.h"
+#include "BKE_utildefines.h"
 
 #include "BIF_editmesh.h"
 #include "BIF_editview.h"
+#include "BIF_editarmature.h"
 #include "BIF_gl.h"
 #include "BIF_glutil.h"
 #include "BIF_interface.h"
@@ -104,12 +115,6 @@
 #include "BIF_editarmature.h"
 #endif
 
-/* editmball.c */
-extern ListBase editelems;  /* go away ! */
-
-
-/* from editobject */
-extern void make_trans_verts(float *min, float *max, int mode);     
 
 /* circle selection callback */
 typedef void (*select_CBfunc)(short selecting, Object *editobj, short *mval, float rad);
@@ -121,6 +126,10 @@ extern void uvedit_selectionCB(short selecting, Object *editobj,
 
 void circle_selectCB(select_CBfunc func);
 
+/* local protos ---------------*/
+void snap_curs_to_firstsel(void);
+
+
 int get_border(rcti *rect, short col)
 {
 	float dvec[4], fac1, fac2;
@@ -337,9 +346,11 @@ int get_border(rcti *rect, short col)
 			if (G.obedit) {
 				if ELEM4(G.obedit->type, OB_MESH, OB_CURVE, OB_SURF, OB_LATTICE) {
 					circle_selectCB(&obedit_selectionCB);
-					return 0;
 				}
 			}
+			else if (G.f&G_FACESELECT) {
+				circle_selectCB(&obedit_selectionCB);
+			}
 			return 0;
 			
 		case SPACE_IMAGE: // brush select in UV editor
@@ -394,6 +405,7 @@ void circle_selectCB(select_CBfunc callback)
 	static float rad= 40.0;
 	float rado;
 	int firsttime=1;
+	int escape= 0;
 	unsigned short event;
 	short mvalo[2], mval[2], val;
 	short selecting=0;
@@ -428,43 +440,48 @@ void circle_selectCB(select_CBfunc callback)
 
 		}
 		
-		event= extern_qread(&val);
-		if (event) {
-			int afbreek= 0;
+		while(qtest()) {
+			event= extern_qread(&val);
+			if (event) {
 
-			/* for when another window is open and a mouse cursor activates it */
-			if(event!=MOUSEY && event!=MOUSEX) mywinset(curarea->win);
-			
-			getmouseco_areawin(mval);	// important to do here, trust events!
-			
-			switch(event) {
-		
-			case LEFTMOUSE:
-			case MIDDLEMOUSE:
-				if(val) selecting= event;
-				else selecting= 0;
-				firsttime= 1;
+				/* for when another window is open and a mouse cursor activates it */
+				if(event!=MOUSEY && event!=MOUSEX) mywinset(curarea->win);
 				
-				break;
-			case WHEELDOWNMOUSE:
-			case PADPLUSKEY:
-				if(val) if(rad<200.0) rad*= 1.2;
-				break;
-			case WHEELUPMOUSE:
-			case PADMINUS:
-				if(val) if(rad>5.0) rad/= 1.2;
-				break;
+				getmouseco_areawin(mval);	// important to do here, trust events!
+				
+				switch(event) {
 			
-			case ESCKEY: case SPACEKEY: case RIGHTMOUSE: case INPUTCHANGE: 
-			case GKEY: case SKEY: case RKEY: case XKEY: case EKEY: case TABKEY:
-				afbreek= 1;
-				break;
+				case LEFTMOUSE:
+				case MIDDLEMOUSE:
+					if(val) selecting= event;
+					else selecting= 0;
+					firsttime= 1;
+					
+					break;
+				case WHEELDOWNMOUSE:
+				case PADPLUSKEY:
+				case EQUALKEY:
+					if(val) if(rad<200.0) rad*= 1.2;
+					break;
+				case WHEELUPMOUSE:
+				case PADMINUS:
+				case MINUSKEY:
+					if(val) if(rad>5.0) rad/= 1.2;
+					break;
+				
+				case ESCKEY: case SPACEKEY: case RIGHTMOUSE: case INPUTCHANGE: 
+				case GKEY: case SKEY: case RKEY: case XKEY: case EKEY: case TABKEY:
+					escape= 1;
+					break;
 
+				}
+				
+				if(escape) break;
 			}
-			
-			if(afbreek) break;
 		}
-		else PIL_sleep_ms(10);
+		PIL_sleep_ms(10);
+		
+		if(escape) break;
 	}
 	
 	/* clear circle */
@@ -474,27 +491,29 @@ void circle_selectCB(select_CBfunc callback)
 	allqueue(REDRAWINFO, 0);
 }
 
-void count_object(Object *ob, int sel)
+static void count_object(Object *ob, int sel, int totob)
 {
 	Mesh *me;
 	Curve *cu;
-	int tot=0, totf=0;
+	int tot=0, totf=0, subsurf;
 	
 	switch(ob->type) {
 	case OB_MESH:
-		G.totmesh++;
+		G.totmesh+=totob;
 		me= get_mesh(ob);
 		if(me) {
+			ModifierData *md = modifiers_findByType(ob, eModifierType_Subsurf);
 			int totvert, totface;
-				/* hack, should be getting displistmesh from a central function */
-			if (mesh_uses_displist(me) && me->disp.first && ((DispList*)me->disp.first)->type==DL_MESH) {
-				DispListMesh *dlm= ((DispList*)me->disp.first)->mesh;
-				totvert= dlm->totvert;
-				totface= dlm->totface;
-			} else {
-				totvert= me->totvert;
-				totface= me->totface;
+			
+			if (md) {
+				SubsurfModifierData *smd = (SubsurfModifierData*) md;
+				
+				subsurf= 1<<(2*smd->levels);
 			}
+			else subsurf= 1;
+			
+			totvert= subsurf*me->totvert*totob;
+			totface= subsurf*me->totface*totob;
 			
 			G.totvert+= totvert;
 			G.totface+= totface;
@@ -506,16 +525,18 @@ void count_object(Object *ob, int sel)
 		break;
 
 	case OB_LAMP:
-		G.totlamp++;
+		G.totlamp+=totob;
 		break;
 	case OB_SURF:
 	case OB_CURVE:
 	case OB_FONT:
-		G.totcurve++;
+		G.totcurve+=totob;
 		tot=totf= 0;
 		cu= ob->data;
-		if(cu->disp.first==0) makeDispList(ob);
-		count_displist( &cu->disp, &tot, &totf);
+		if(cu->disp.first)
+			count_displist( &cu->disp, &tot, &totf);
+		tot*= totob;
+		totf*= totob;
 		G.totvert+= tot;
 		G.totface+= totf;
 		if(sel) {
@@ -525,65 +546,64 @@ void count_object(Object *ob, int sel)
 		break;
 	case OB_MBALL:
 		count_displist( &ob->disp, &tot, &totf);
+		tot*= totob;
+		totf*= totob;
 		G.totvert+= tot;
 		G.totface+= totf;
 		if(sel) {
 			G.totvertsel+= tot;
 			G.totfacesel+= totf;
 		}
-		
 		break;
 	}
 	
 }
 
+/* countall does statistics */
+/* is called on most actions, like select/add/delete/layermove */
 void countall()
 {
-/*  	extern Lattice *editLatt; in BKE_lattice.h*/
 	extern ListBase editNurb;
-	/* extern ListBase bpbase; */
 	Base *base;
-	Object *ob;
+	Object *ob= OBACT;
 	Mesh *me;
 	Nurb *nu;
 	BezTriple *bezt;
 	BPoint *bp;
 	MetaElem *ml;
-	/* struct BodyPoint *bop; */
-	struct EditVert *eve;
-	struct EditFace *efa;
-#ifdef __NLA
 	struct EditBone *ebo;
-#endif
 	int a;
 
-	G.totvert= G.totvertsel= G.totfacesel= G.totface= G.totobj= 
-	    G.totmesh= G.totlamp= G.totcurve= G.totobjsel=  0;
+	G.totvert= G.totvertsel= G.totedge= G.totedgesel= G.totfacesel= G.totface= G.totobj= 
+	    G.totmesh= G.totlamp= G.totcurve= G.totobjsel= G.totbone= G.totbonesel=  0;
 
 	if(G.obedit) {
 		
 		if(G.obedit->type==OB_MESH) {
 			EditMesh *em = G.editMesh;
-			eve= em->verts.first;
-			while(eve) {
+			EditVert *eve;
+			EditEdge *eed;
+			EditFace *efa;
+			
+			for(eve= em->verts.first; eve; eve= eve->next) {
 				G.totvert++;
 				if(eve->f & SELECT) G.totvertsel++;
-				eve= eve->next;
 			}
-			
-			efa= em->faces.first;
-			while(efa) {
+			for(eed= em->edges.first; eed; eed= eed->next) {
+				G.totedge++;
+				if(eed->f & SELECT) G.totedgesel++;
+			}
+			for(efa= em->faces.first; efa; efa= efa->next) {
 				G.totface++;
 				if(efa->f & SELECT) G.totfacesel++;
-				efa= efa->next;
 			}
 		}
-#ifdef __NLA
 		else if (G.obedit->type==OB_ARMATURE){
 			for (ebo=G.edbo.first;ebo;ebo=ebo->next){
+				G.totbone++;
 				
-				/* Sync selection to parent for ik children */
-				if ((ebo->flag & BONE_IK_TOPARENT) && ebo->parent){
+				/* Sync selection to parent for connected children */
+				if ((ebo->flag & BONE_CONNECTED) && ebo->parent){
 					G.totvert--;
 					if (ebo->parent->flag & BONE_TIPSEL)
 						ebo->flag |= BONE_ROOTSEL;
@@ -601,18 +621,16 @@ void countall()
 				else
 					ebo->flag &= ~BONE_SELECTED;
 				
-				//	If this is an IK child and it's parent is being moved, remove our root
-				if ((ebo->flag & BONE_IK_TOPARENT)&& (ebo->flag & BONE_ROOTSEL) && ebo->parent && (ebo->parent->flag & BONE_TIPSEL)){
+				if(ebo->flag & BONE_SELECTED) G.totbonesel++;
+
+				//	If this is a connected child and it's parent is being moved, remove our root
+				if ((ebo->flag & BONE_CONNECTED)&& (ebo->flag & BONE_ROOTSEL) && ebo->parent && (ebo->parent->flag & BONE_TIPSEL)){
 					G.totvertsel--;
 				}
 
 				G.totvert+=2;
-				G.totface++;
-				
-
 			}
 		}
-#endif
 		else if ELEM3(G.obedit->type, OB_CURVE, OB_SURF, OB_FONT) {
 			nu= editNurb.first;
 			while(nu) {
@@ -640,6 +658,9 @@ void countall()
 			}
 		}
 		else if(G.obedit->type==OB_MBALL) {
+			/* editmball.c */
+			extern ListBase editelems;  /* go away ! */
+			
 			ml= editelems.first;
 			while(ml) {
 				G.totvert++;
@@ -661,6 +682,17 @@ void countall()
 		allqueue(REDRAWINFO, 1);	/* 1, because header->win==0! */
 		return;
 	}
+	else if(ob && (ob->flag & OB_POSEMODE)) {
+		if(ob->pose) {
+			bPoseChannel *pchan;
+			for(pchan= ob->pose->chanbase.first; pchan; pchan= pchan->next) {
+				G.totbone++;
+				if(pchan->bone && (pchan->bone->flag & BONE_SELECTED)) G.totbonesel++;
+			}
+		}
+		allqueue(REDRAWINFO, 1);	/* 1, because header->win==0! */
+		return;
+	}
 	else if(G.f & (G_FACESELECT + G_VERTEXPAINT + G_TEXTUREPAINT +G_WEIGHTPAINT)) {
 		me= get_mesh((G.scene->basact) ? (G.scene->basact->object) : 0);
 		if(me) {
@@ -676,23 +708,23 @@ void countall()
 	base= (G.scene->base.first);
 	while(base) {
 		if(G.scene->lay & base->lay) {
+			ob= base->object;	/* warning, ob not is obact anymore */
 			
-			G.totobj++;
 			if(base->flag & SELECT) G.totobjsel++;
 			
-			count_object(base->object, base->flag & SELECT);
-			
-			if(base->object->transflag & OB_DUPLI) {
-				extern ListBase duplilist;
-
-				make_duplilist(G.scene, base->object);
-				ob= duplilist.first;
-				while(ob) {
-					G.totobj++;
-					count_object(ob, base->flag & SELECT);
-					ob= ob->id.next;
-				}
-				free_duplilist();
+			if(ob->parent && (ob->parent->transflag & OB_DUPLIVERTS)) {
+				int tot= count_duplilist(ob->parent);
+				G.totobj+=tot;
+				count_object(ob, base->flag & SELECT, tot);
+			}
+			else if(ob->transflag & OB_DUPLIFRAMES) {
+				int tot= count_duplilist(ob);
+				G.totobj+=tot;
+				count_object(ob, base->flag & SELECT, tot);
+			}
+			else {
+				count_object(ob, base->flag & SELECT, 1);
+				G.totobj++;
 			}
 		}
 		base= base->next;
@@ -700,28 +732,301 @@ void countall()
 	allqueue(REDRAWINFO, 1); 	/* 1, because header->win==0! */
 }
 
-/* selected things moved, and might need update in displists */
-static void update_select_dependency(void)
+/* ************************************************** */
+/* ********************* old transform stuff ******** */
+/* ************************************************** */
+
+static TransVert *transvmain=NULL;
+static int tottrans= 0;
+
+/* copied from editobject.c, now uses (almost) proper depgraph */
+static void special_transvert_update(void)
 {
-	Base *base;
-	Object *ob;
-	for(base= (G.scene->base.first); base; base= base->next) {
-		ob= base->object;
-		if(ob->hooks.first) {
-			ObHook *hook= ob->hooks.first;
-			while(hook) {
-				if(hook->parent->flag & SELECT) freedisplist(&ob->disp);
-				hook= hook->next;
+	
+	if(G.obedit) {
+		
+		DAG_object_flush_update(G.scene, G.obedit, OB_RECALC_DATA);
+		
+		if(G.obedit->type==OB_MESH) {
+			recalc_editnormals();	// does face centers too
+		}
+		else if ELEM(G.obedit->type, OB_CURVE, OB_SURF) {
+			extern ListBase editNurb;
+			Nurb *nu= editNurb.first;
+			while(nu) {
+				test2DNurb(nu);
+				testhandlesNurb(nu); /* test for bezier too */
+				nu= nu->next;
+			}
+		}
+		else if(G.obedit->type==OB_ARMATURE){
+			bArmature *arm= G.obedit->data;
+			EditBone *ebo;
+			
+			/* Ensure all bones are correctly adjusted */
+			for (ebo=G.edbo.first; ebo; ebo=ebo->next){
+				
+				if ((ebo->flag & BONE_CONNECTED) && ebo->parent){
+					/* If this bone has a parent tip that has been moved */
+					if (ebo->parent->flag & BONE_TIPSEL){
+						VECCOPY (ebo->head, ebo->parent->tail);
+					}
+					/* If this bone has a parent tip that has NOT been moved */
+					else{
+						VECCOPY (ebo->parent->tail, ebo->head);
+					}
+				}
+			}
+			if(arm->flag & ARM_MIRROR_EDIT) 
+				transform_armature_mirror_update();
+		}
+		else if(G.obedit->type==OB_LATTICE) {
+			if(editLatt->flag & LT_OUTSIDE) outside_lattice(editLatt);
+		}
+	}
+}
+
+/* copied from editobject.c, needs to be replaced with new transform code still */
+/* mode: 1 = proportional */
+static void make_trans_verts(float *min, float *max, int mode)	
+{
+	extern ListBase editNurb;
+	EditMesh *em = G.editMesh;
+	Nurb *nu;
+	BezTriple *bezt;
+	BPoint *bp;
+	TransVert *tv=NULL;
+	MetaElem *ml;
+	EditVert *eve;
+	EditBone	*ebo;
+	float total, centre[3], centroid[3];
+	int a;
+
+	tottrans= 0; // global!
+	
+	INIT_MINMAX(min, max);
+	centroid[0]=centroid[1]=centroid[2]= 0.0;
+	
+	/* note for transform refactor: dont rely on countall anymore... its ancient */
+	/* I skip it for editmesh now (ton) */
+	if(G.obedit->type!=OB_MESH) {
+		countall();
+		if(mode) tottrans= G.totvert;
+		else tottrans= G.totvertsel;
+
+		if(G.totvertsel==0) {
+			tottrans= 0;
+			return;
+		}
+		tv=transvmain= MEM_callocN(tottrans*sizeof(TransVert), "maketransverts");
+	}
+	
+	/* we count again because of hide (old, not for mesh!) */
+	tottrans= 0;
+	
+	if(G.obedit->type==OB_MESH) {
+		int proptrans= 0;
+		
+		// transform now requires awareness for select mode, so we tag the f1 flags in verts
+		tottrans= 0;
+		if(G.scene->selectmode & SCE_SELECT_VERTEX) {
+			for(eve= em->verts.first; eve; eve= eve->next) {
+				if(eve->h==0 && (eve->f & SELECT)) {
+					eve->f1= SELECT;
+					tottrans++;
+				}
+				else eve->f1= 0;
+			}
+		}
+		else if(G.scene->selectmode & SCE_SELECT_EDGE) {
+			EditEdge *eed;
+			for(eve= em->verts.first; eve; eve= eve->next) eve->f1= 0;
+			for(eed= em->edges.first; eed; eed= eed->next) {
+				if(eed->h==0 && (eed->f & SELECT)) eed->v1->f1= eed->v2->f1= SELECT;
+			}
+			for(eve= em->verts.first; eve; eve= eve->next) if(eve->f1) tottrans++;
+		}
+		else {
+			EditFace *efa;
+			for(eve= em->verts.first; eve; eve= eve->next) eve->f1= 0;
+			for(efa= em->faces.first; efa; efa= efa->next) {
+				if(efa->h==0 && (efa->f & SELECT)) {
+					efa->v1->f1= efa->v2->f1= efa->v3->f1= SELECT;
+					if(efa->v4) efa->v4->f1= SELECT;
+				}
+			}
+			for(eve= em->verts.first; eve; eve= eve->next) if(eve->f1) tottrans++;
+		}
+		
+		/* proportional edit exception... */
+		if(mode==1 && tottrans) {
+			for(eve= em->verts.first; eve; eve= eve->next) {
+				if(eve->h==0) {
+					eve->f1 |= 2;
+					proptrans++;
+				}
+			}
+			if(proptrans>tottrans) tottrans= proptrans;
+		}
+		
+		/* and now make transverts */
+		if(tottrans) {
+			tv=transvmain= MEM_callocN(tottrans*sizeof(TransVert), "maketransverts");
+
+			for(eve= em->verts.first; eve; eve= eve->next) {
+				if(eve->f1) {
+					VECCOPY(tv->oldloc, eve->co);
+					tv->loc= eve->co;
+					if(eve->no[0]!=0.0 || eve->no[1]!=0.0 ||eve->no[2]!=0.0)
+						tv->nor= eve->no; // note this is a hackish signal (ton)
+					tv->flag= eve->f1 & SELECT;
+					tv++;
+				}
 			}
 		}
 	}
+	else if (G.obedit->type==OB_ARMATURE){
+		for (ebo=G.edbo.first;ebo;ebo=ebo->next){
+			if (ebo->flag & BONE_TIPSEL){
+				VECCOPY (tv->oldloc, ebo->tail);
+				tv->loc= ebo->tail;
+				tv->nor= NULL;
+				tv->flag= 1;
+				tv++;
+				tottrans++;
+			}
+
+			/*  Only add the root if there is no connection */
+			if (ebo->flag & BONE_ROOTSEL){
+				if (!(ebo->parent && (ebo->flag & BONE_CONNECTED) && ebo->parent->flag & BONE_TIPSEL)){
+					VECCOPY (tv->oldloc, ebo->head);
+					tv->loc= ebo->head;
+					tv->nor= NULL;
+					tv->flag= 1;
+					tv++;
+					tottrans++;
+				}		
+			}
+			
+		}
+	}
+	else if ELEM(G.obedit->type, OB_CURVE, OB_SURF) {
+		nu= editNurb.first;
+		while(nu) {
+			if((nu->type & 7)==CU_BEZIER) {
+				a= nu->pntsu;
+				bezt= nu->bezt;
+				while(a--) {
+					if(bezt->hide==0) {
+						if(mode==1 || (bezt->f1 & 1)) {
+							VECCOPY(tv->oldloc, bezt->vec[0]);
+							tv->loc= bezt->vec[0];
+							tv->flag= bezt->f1 & 1;
+							tv++;
+							tottrans++;
+						}
+						if(mode==1 || (bezt->f2 & 1)) {
+							VECCOPY(tv->oldloc, bezt->vec[1]);
+							tv->loc= bezt->vec[1];
+							tv->val= &(bezt->alfa);
+							tv->oldval= bezt->alfa;
+							tv->flag= bezt->f2 & 1;
+							tv++;
+							tottrans++;
+						}
+						if(mode==1 || (bezt->f3 & 1)) {
+							VECCOPY(tv->oldloc, bezt->vec[2]);
+							tv->loc= bezt->vec[2];
+							tv->flag= bezt->f3 & 1;
+							tv++;
+							tottrans++;
+						}
+					}
+					bezt++;
+				}
+			}
+			else {
+				a= nu->pntsu*nu->pntsv;
+				bp= nu->bp;
+				while(a--) {
+					if(bp->hide==0) {
+						if(mode==1 || (bp->f1 & 1)) {
+							VECCOPY(tv->oldloc, bp->vec);
+							tv->loc= bp->vec;
+							tv->val= &(bp->alfa);
+							tv->oldval= bp->alfa;
+							tv->flag= bp->f1 & 1;
+							tv++;
+							tottrans++;
+						}
+					}
+					bp++;
+				}
+			}
+			nu= nu->next;
+		}
+	}
+	else if(G.obedit->type==OB_MBALL) {
+		extern ListBase editelems;  /* go away ! */
+		ml= editelems.first;
+		while(ml) {
+			if(ml->flag & SELECT) {
+				tv->loc= &ml->x;
+				VECCOPY(tv->oldloc, tv->loc);
+				tv->val= &(ml->rad);
+				tv->oldval= ml->rad;
+				tv->flag= 1;
+				tv++;
+				tottrans++;
+			}
+			ml= ml->next;
+		}
+	}
+	else if(G.obedit->type==OB_LATTICE) {
+		bp= editLatt->def;
+		
+		a= editLatt->pntsu*editLatt->pntsv*editLatt->pntsw;
+		
+		while(a--) {
+			if(mode==1 || (bp->f1 & 1)) {
+				if(bp->hide==0) {
+					VECCOPY(tv->oldloc, bp->vec);
+					tv->loc= bp->vec;
+					tv->flag= bp->f1 & 1;
+					tv++;
+					tottrans++;
+				}
+			}
+			bp++;
+		}
+	}
+	
+	/* cent etc */
+	tv= transvmain;
+	total= 0.0;
+	for(a=0; a<tottrans; a++, tv++) {
+		if(tv->flag & SELECT) {
+			centroid[0]+= tv->oldloc[0];
+			centroid[1]+= tv->oldloc[1];
+			centroid[2]+= tv->oldloc[2];
+			total+= 1.0;
+			DO_MINMAX(tv->oldloc, min, max);
+		}
+	}
+	if(total!=0.0) {
+		centroid[0]/= total;
+		centroid[1]/= total;
+		centroid[2]/= total;
+	}
+
+	centre[0]= (min[0]+max[0])/2.0;
+	centre[1]= (min[1]+max[1])/2.0;
+	centre[2]= (min[2]+max[2])/2.0;
 	
 }
 
 void snap_sel_to_grid()
 {
-	extern TransVert *transvmain;
-	extern int tottrans;
 	extern float originmat[3][3];	/* object.c */
 	TransVert *tv;
 	Base *base;
@@ -729,62 +1034,57 @@ void snap_sel_to_grid()
 	float gridf, imat[3][3], bmat[3][3], vec[3];
 	int a;
 
-	gridf= G.vd->grid;
+	gridf= G.vd->gridview;
 
 
-		if(G.obedit) {
-			tottrans= 0;
-			
-#ifdef __NLA
-			if ELEM5(G.obedit->type, OB_ARMATURE, OB_LATTICE, OB_MESH, OB_SURF, OB_CURVE) make_trans_verts(bmat[0], bmat[1], 0);
-#else
-			if ELEM4(G.obedit->type, OB_LATTICE, OB_MESH, OB_SURF, OB_CURVE) make_trans_verts(bmat[0], bmat[1], 0);
-#endif
-			if(tottrans==0) return;
+	if(G.obedit) {
+		tottrans= 0;
+		
+		if ELEM6(G.obedit->type, OB_ARMATURE, OB_LATTICE, OB_MESH, OB_SURF, OB_CURVE, OB_MBALL) 
+			make_trans_verts(bmat[0], bmat[1], 0);
+		if(tottrans==0) return;
 
-			Mat3CpyMat4(bmat, G.obedit->obmat);
-			Mat3Inv(imat, bmat);
+		Mat3CpyMat4(bmat, G.obedit->obmat);
+		Mat3Inv(imat, bmat);
 
-			tv= transvmain;
-			for(a=0; a<tottrans; a++, tv++) {
+		tv= transvmain;
+		for(a=0; a<tottrans; a++, tv++) {
 
-				VECCOPY(vec, tv->loc);
-				Mat3MulVecfl(bmat, vec);
-				VecAddf(vec, vec, G.obedit->obmat[3]);
-				vec[0]= G.vd->grid*floor(.5+ vec[0]/gridf);
-				vec[1]= G.vd->grid*floor(.5+ vec[1]/gridf);
-				vec[2]= G.vd->grid*floor(.5+ vec[2]/gridf);
-				VecSubf(vec, vec, G.obedit->obmat[3]);
+			VECCOPY(vec, tv->loc);
+			Mat3MulVecfl(bmat, vec);
+			VecAddf(vec, vec, G.obedit->obmat[3]);
+			vec[0]= G.vd->gridview*floor(.5+ vec[0]/gridf);
+			vec[1]= G.vd->gridview*floor(.5+ vec[1]/gridf);
+			vec[2]= G.vd->gridview*floor(.5+ vec[2]/gridf);
+			VecSubf(vec, vec, G.obedit->obmat[3]);
 
-				Mat3MulVecfl(imat, vec);
-				VECCOPY(tv->loc, vec);
+			Mat3MulVecfl(imat, vec);
+			VECCOPY(tv->loc, vec);
 
+		}
+
+		MEM_freeN(transvmain);
+		transvmain= 0;
+		
+		special_transvert_update();
+
+		allqueue(REDRAWVIEW3D, 0);
+		return;
+	}
+
+	base= (G.scene->base.first);
+	while(base) {
+		if( ( ((base)->flag & SELECT) && ((base)->lay & G.vd->lay) && ((base)->object->id.lib==0))) {
+			ob= base->object;
+			if(ob->flag & OB_POSEMODE) {
+				;	// todo
 			}
-
-			MEM_freeN(transvmain);
-			transvmain= 0;
-			
-			calc_trans_verts(); // does test2d, makedisplist too */
-
-			special_trans_update(0);
-
-			allqueue(REDRAWVIEW3D, 0);
-			return;
-		}
-#ifdef __NLA
-		if (G.obpose){
-			allqueue(REDRAWVIEW3D, 0);
-			return;
-		}
-#endif
-		base= (G.scene->base.first);
-		while(base) {
-			if( ( ((base)->flag & SELECT) && ((base)->lay & G.vd->lay) && ((base)->object->id.lib==0))) {
-				ob= base->object;
-
-				vec[0]= -ob->obmat[3][0]+G.vd->grid*floor(.5+ ob->obmat[3][0]/gridf);
-				vec[1]= -ob->obmat[3][1]+G.vd->grid*floor(.5+ ob->obmat[3][1]/gridf);
-				vec[2]= -ob->obmat[3][2]+G.vd->grid*floor(.5+ ob->obmat[3][2]/gridf);
+			else {
+				ob->recalc |= OB_RECALC_OB;
+				
+				vec[0]= -ob->obmat[3][0]+G.vd->gridview*floor(.5+ ob->obmat[3][0]/gridf);
+				vec[1]= -ob->obmat[3][1]+G.vd->gridview*floor(.5+ ob->obmat[3][1]/gridf);
+				vec[2]= -ob->obmat[3][2]+G.vd->gridview*floor(.5+ ob->obmat[3][2]/gridf);
 
 				if(ob->parent) {
 					where_is_object(ob);
@@ -801,17 +1101,16 @@ void snap_sel_to_grid()
 					ob->loc[2]+= vec[2];
 				}
 			}
-
-			base= base->next;
 		}
-		update_select_dependency();
-		allqueue(REDRAWVIEW3D, 0);
+
+		base= base->next;
+	}
+	DAG_scene_flush_update(G.scene, screen_view3d_layers());
+	allqueue(REDRAWVIEW3D, 0);
 }
 
 void snap_sel_to_curs()
 {
-	extern TransVert *transvmain;
-	extern int tottrans;
 	extern float originmat[3][3];	/* object.c */
 	TransVert *tv;
 	Base *base;
@@ -821,57 +1120,66 @@ void snap_sel_to_curs()
 
 	curs= give_cursor();
 
-		if(G.obedit) {
-			tottrans= 0;
-#ifdef __NLA
-			if ELEM5(G.obedit->type, OB_ARMATURE, OB_LATTICE, OB_MESH, OB_SURF, OB_CURVE) make_trans_verts(bmat[0], bmat[1], 0);
-#else
-			if ELEM4(G.obedit->type, OB_LATTICE, OB_MESH, OB_SURF, OB_CURVE) make_trans_verts(bmat[0], bmat[1], 0);
-#endif
-			if(tottrans==0) return;
+	if(G.obedit) {
+		tottrans= 0;
 
-			Mat3CpyMat4(bmat, G.obedit->obmat);
-			Mat3Inv(imat, bmat);
+		if ELEM6(G.obedit->type, OB_ARMATURE, OB_LATTICE, OB_MESH, OB_SURF, OB_CURVE, OB_MBALL) 
+			make_trans_verts(bmat[0], bmat[1], 0);
+		if(tottrans==0) return;
 
-			tv= transvmain;
-			for(a=0; a<tottrans; a++, tv++) {
+		Mat3CpyMat4(bmat, G.obedit->obmat);
+		Mat3Inv(imat, bmat);
 
+		tv= transvmain;
+		for(a=0; a<tottrans; a++, tv++) {
 
-				vec[0]= curs[0]-G.obedit->obmat[3][0];
-				vec[1]= curs[1]-G.obedit->obmat[3][1];
-				vec[2]= curs[2]-G.obedit->obmat[3][2];
+			vec[0]= curs[0]-G.obedit->obmat[3][0];
+			vec[1]= curs[1]-G.obedit->obmat[3][1];
+			vec[2]= curs[2]-G.obedit->obmat[3][2];
 
+			Mat3MulVecfl(imat, vec);
+			VECCOPY(tv->loc, vec);
 
-				Mat3MulVecfl(imat, vec);
-				VECCOPY(tv->loc, vec);
+		}
+		MEM_freeN(transvmain);
+		transvmain= 0;
 
+		special_transvert_update();
+
+		allqueue(REDRAWVIEW3D, 0);
+		return;
+	}
+
+	base= (G.scene->base.first);
+	while(base) {
+		if( ( ((base)->flag & SELECT) && ((base)->lay & G.vd->lay) && ((base)->object->id.lib==0))) {
+			ob= base->object;
+			if(ob->flag & OB_POSEMODE) {
+				bPoseChannel *pchan;
+				float cursp[3];
+				
+				Mat4Invert(ob->imat, ob->obmat);
+				VECCOPY(cursp, curs);
+				Mat4MulVecfl(ob->imat, cursp);
+				
+				for (pchan = ob->pose->chanbase.first; pchan; pchan=pchan->next) {
+					if(pchan->bone->flag & BONE_SELECTED) {
+						if(pchan->parent==NULL) {
+							VECCOPY(pchan->loc, cursp);
+						}
+						/* else todo... */
+					}
+				}
+				ob->pose->flag |= (POSE_LOCKED|POSE_DO_UNLOCK);
+				ob->recalc |= OB_RECALC_DATA;
 			}
-			MEM_freeN(transvmain);
-			transvmain= 0;
-
-			calc_trans_verts(); // does test2d, makedisplist too */
-
-			special_trans_update(0);
-
-			allqueue(REDRAWVIEW3D, 0);
-			return;
-		}
-#ifdef __NLA
-		if (G.obpose){
-			allqueue(REDRAWVIEW3D, 0);
-			return;
-		}
-#endif
-		base= (G.scene->base.first);
-		while(base) {
-			if( ( ((base)->flag & SELECT) && ((base)->lay & G.vd->lay) && ((base)->object->id.lib==0))) {
-				ob= base->object;
-
+			else {
+				ob->recalc |= OB_RECALC_OB;
+				
 				vec[0]= -ob->obmat[3][0] + curs[0];
 				vec[1]= -ob->obmat[3][1] + curs[1];
 				vec[2]= -ob->obmat[3][2] + curs[2];
 
-
 				if(ob->parent) {
 					where_is_object(ob);
 
@@ -887,23 +1195,24 @@ void snap_sel_to_curs()
 					ob->loc[2]+= vec[2];
 				}
 			}
-
-			base= base->next;
 		}
-		update_select_dependency();
-		allqueue(REDRAWVIEW3D, 0);
+
+		base= base->next;
+	}
+	DAG_scene_flush_update(G.scene, screen_view3d_layers());
+	allqueue(REDRAWVIEW3D, 0);
 }
 
 void snap_curs_to_grid()
 {
 	float gridf, *curs;
 
-	gridf= G.vd->grid;
+	gridf= G.vd->gridview;
 	curs= give_cursor();
 
-	curs[0]= G.vd->grid*floor(.5+curs[0]/gridf);
-	curs[1]= G.vd->grid*floor(.5+curs[1]/gridf);
-	curs[2]= G.vd->grid*floor(.5+curs[2]/gridf);
+	curs[0]= G.vd->gridview*floor(.5+curs[0]/gridf);
+	curs[1]= G.vd->gridview*floor(.5+curs[1]/gridf);
+	curs[2]= G.vd->gridview*floor(.5+curs[2]/gridf);
 
 	allqueue(REDRAWVIEW3D, 0);
 
@@ -911,8 +1220,6 @@ void snap_curs_to_grid()
 
 void snap_curs_to_sel()
 {
-	extern TransVert *transvmain;
-	extern int tottrans;
 	TransVert *tv;
 	Base *base;
 	float *curs, bmat[3][3], vec[3], min[3], max[3], centroid[3];
@@ -924,28 +1231,64 @@ void snap_curs_to_sel()
 	INIT_MINMAX(min, max);
 	centroid[0]= centroid[1]= centroid[2]= 0.0;
 
-		if(G.obedit) {
-			tottrans=0;
-#ifdef __NLA
-			if ELEM5(G.obedit->type, OB_ARMATURE, OB_LATTICE, OB_MESH, OB_SURF, OB_CURVE) make_trans_verts(bmat[0], bmat[1], 0);
-#else
-			if ELEM4(G.obedit->type, OB_LATTICE, OB_MESH, OB_SURF, OB_CURVE) make_trans_verts(bmat[0], bmat[1], 0);
-#endif
-			if(tottrans==0) return;
+	if(G.obedit) {
+		tottrans=0;
 
-			Mat3CpyMat4(bmat, G.obedit->obmat);
+		if ELEM6(G.obedit->type, OB_ARMATURE, OB_LATTICE, OB_MESH, OB_SURF, OB_CURVE, OB_MBALL) 
+			make_trans_verts(bmat[0], bmat[1], 0);
+		if(tottrans==0) return;
 
-			tv= transvmain;
-			for(a=0; a<tottrans; a++, tv++) {
-				VECCOPY(vec, tv->loc);
-				Mat3MulVecfl(bmat, vec);
-				VecAddf(vec, vec, G.obedit->obmat[3]);
-				VecAddf(centroid, centroid, vec);
-				DO_MINMAX(vec, min, max);
+		Mat3CpyMat4(bmat, G.obedit->obmat);
+
+		tv= transvmain;
+		for(a=0; a<tottrans; a++, tv++) {
+			VECCOPY(vec, tv->loc);
+			Mat3MulVecfl(bmat, vec);
+			VecAddf(vec, vec, G.obedit->obmat[3]);
+			VecAddf(centroid, centroid, vec);
+			DO_MINMAX(vec, min, max);
+		}
+
+		if(G.vd->around==V3D_CENTROID) {
+			VecMulf(centroid, 1.0/(float)tottrans);
+			VECCOPY(curs, centroid);
+		}
+		else {
+			curs[0]= (min[0]+max[0])/2;
+			curs[1]= (min[1]+max[1])/2;
+			curs[2]= (min[2]+max[2])/2;
+		}
+		MEM_freeN(transvmain);
+		transvmain= 0;
+	}
+	else {
+		Object *ob= OBACT;
+		
+		if(ob && (ob->flag & OB_POSEMODE)) {
+			bPoseChannel *pchan;
+			for (pchan = ob->pose->chanbase.first; pchan; pchan=pchan->next) {
+				if(pchan->bone->flag & BONE_SELECTED) {
+					VECCOPY(vec, pchan->pose_head);
+					Mat4MulVecfl(ob->obmat, vec);
+					VecAddf(centroid, centroid, vec);
+					DO_MINMAX(vec, min, max);
+					count++;
+				}
 			}
-
+		}
+		else {
+			for(base= G.scene->base.first; base; base= base->next) {
+				if(((base)->flag & SELECT) && ((base)->lay & G.vd->lay) ) {
+					VECCOPY(vec, base->object->obmat[3]);
+					VecAddf(centroid, centroid, vec);
+					DO_MINMAX(vec, min, max);
+					count++;
+				}
+			}
+		}
+		if(count) {
 			if(G.vd->around==V3D_CENTROID) {
-				VecMulf(centroid, 1.0/(float)tottrans);
+				VecMulf(centroid, 1.0/(float)count);
 				VECCOPY(curs, centroid);
 			}
 			else {
@@ -953,39 +1296,13 @@ void snap_curs_to_sel()
 				curs[1]= (min[1]+max[1])/2;
 				curs[2]= (min[2]+max[2])/2;
 			}
-			MEM_freeN(transvmain);
-			transvmain= 0;
 		}
-		else {
-			base= (G.scene->base.first);
-			while(base) {
-				if(((base)->flag & SELECT) && ((base)->lay & G.vd->lay) ) {
-					VECCOPY(vec, base->object->obmat[3]);
-					VecAddf(centroid, centroid, vec);
-					DO_MINMAX(vec, min, max);
-					count++;
-				}
-				base= base->next;
-			}
-			if(count) {
-				if(G.vd->around==V3D_CENTROID) {
-					VecMulf(centroid, 1.0/(float)count);
-					VECCOPY(curs, centroid);
-				}
-				else {
-					curs[0]= (min[0]+max[0])/2;
-					curs[1]= (min[1]+max[1])/2;
-					curs[2]= (min[2]+max[2])/2;
-				}
-			}
-		}
-		allqueue(REDRAWVIEW3D, 0);
+	}
+	allqueue(REDRAWVIEW3D, 0);
 }
 
 void snap_curs_to_firstsel()
 {
-	extern TransVert *transvmain;
-	extern int tottrans;
 	TransVert *tv;
 	Base *base;
 	float *curs, bmat[3][3], vec[3], min[3], max[3], centroid[3];
@@ -997,66 +1314,62 @@ void snap_curs_to_firstsel()
 	INIT_MINMAX(min, max);
 	centroid[0]= centroid[1]= centroid[2]= 0.0;
 
-		if(G.obedit) {
-			tottrans=0;
-#ifdef __NLA
-			if ELEM5(G.obedit->type, OB_ARMATURE, OB_LATTICE, OB_MESH, OB_SURF, OB_CURVE) make_trans_verts(bmat[0], bmat[1], 0);
-#else
-			if ELEM4(G.obedit->type, OB_LATTICE, OB_MESH, OB_SURF, OB_CURVE) make_trans_verts(bmat[0], bmat[1], 0);
-#endif
-			if(tottrans==0) return;
+	if(G.obedit) {
+		tottrans=0;
 
-			Mat3CpyMat4(bmat, G.obedit->obmat);
+		if ELEM6(G.obedit->type, OB_ARMATURE, OB_LATTICE, OB_MESH, OB_SURF, OB_CURVE, OB_MBALL) 
+			make_trans_verts(bmat[0], bmat[1], 0);
+		if(tottrans==0) return;
 
-			tv= transvmain;
-			VECCOPY(vec, tv->loc);
-				/*Mat3MulVecfl(bmat, vec);
-				VecAddf(vec, vec, G.obedit->obmat[3]);
-				VecAddf(centroid, centroid, vec);
-				DO_MINMAX(vec, min, max);*/
+		Mat3CpyMat4(bmat, G.obedit->obmat);
 
-			if(G.vd->around==V3D_CENTROID) {
-				VecMulf(vec, 1.0/(float)tottrans);
-				VECCOPY(curs, vec);
-			}
-			else {
-				curs[0]= vec[0];
-				curs[1]= vec[1];
-				curs[2]= vec[2];
-			}
-			MEM_freeN(transvmain);
-			transvmain= 0;
+		tv= transvmain;
+		VECCOPY(vec, tv->loc);
+			/*Mat3MulVecfl(bmat, vec);
+			VecAddf(vec, vec, G.obedit->obmat[3]);
+			VecAddf(centroid, centroid, vec);
+			DO_MINMAX(vec, min, max);*/
+
+		if(G.vd->around==V3D_CENTROID) {
+			VecMulf(vec, 1.0/(float)tottrans);
+			VECCOPY(curs, vec);
 		}
 		else {
-			base= (G.scene->base.first);
-			while(base) {
-				if(((base)->flag & SELECT) && ((base)->lay & G.vd->lay) ) {
-					VECCOPY(vec, base->object->obmat[3]);
-					VecAddf(centroid, centroid, vec);
-					DO_MINMAX(vec, min, max);
-					count++;
-				}
-				base= base->next;
+			curs[0]= vec[0];
+			curs[1]= vec[1];
+			curs[2]= vec[2];
+		}
+		MEM_freeN(transvmain);
+		transvmain= 0;
+	}
+	else {
+		base= (G.scene->base.first);
+		while(base) {
+			if(((base)->flag & SELECT) && ((base)->lay & G.vd->lay) ) {
+				VECCOPY(vec, base->object->obmat[3]);
+				VecAddf(centroid, centroid, vec);
+				DO_MINMAX(vec, min, max);
+				count++;
 			}
-			if(count) {
-				if(G.vd->around==V3D_CENTROID) {
-					VecMulf(centroid, 1.0/(float)count);
-					VECCOPY(curs, centroid);
-				}
-				else {
-					curs[0]= (min[0]+max[0])/2;
-					curs[1]= (min[1]+max[1])/2;
-					curs[2]= (min[2]+max[2])/2;
-				}
+			base= base->next;
+		}
+		if(count) {
+			if(G.vd->around==V3D_CENTROID) {
+				VecMulf(centroid, 1.0/(float)count);
+				VECCOPY(curs, centroid);
+			}
+			else {
+				curs[0]= (min[0]+max[0])/2;
+				curs[1]= (min[1]+max[1])/2;
+				curs[2]= (min[2]+max[2])/2;
 			}
 		}
-		allqueue(REDRAWVIEW3D, 0);
+	}
+	allqueue(REDRAWVIEW3D, 0);
 }
 
 void snap_to_center()
 {
-	extern TransVert *transvmain;
-	extern int tottrans;
 	extern float originmat[3][3];
 	TransVert *tv;
 	Base *base;
@@ -1064,35 +1377,58 @@ void snap_to_center()
 	float snaploc[3], imat[3][3], bmat[3][3], vec[3], min[3], max[3], centroid[3];
 	int count, a;
 
-
-/*calculate the snaplocation (centerpoint) */
+	/*calculate the snaplocation (centerpoint) */
 	count= 0;
 	INIT_MINMAX(min, max);
 	centroid[0]= centroid[1]= centroid[2]= 0.0;
 
-		if(G.obedit) {
-			tottrans= 0;
-#ifdef __NLA
-			if ELEM5(G.obedit->type, OB_ARMATURE, OB_LATTICE, OB_MESH, OB_SURF, OB_CURVE) make_trans_verts(bmat[0], bmat[1], 0);
-#else
-			if ELEM4(G.obedit->type, OB_LATTICE, OB_MESH, OB_SURF, OB_CURVE) make_trans_verts(bmat[0], bmat[1], 0);
-#endif
-			if(tottrans==0) return;
+	if(G.obedit) {
+		tottrans= 0;
 
-			Mat3CpyMat4(bmat, G.obedit->obmat);
-			Mat3Inv(imat, bmat);
+		if ELEM6(G.obedit->type, OB_ARMATURE, OB_LATTICE, OB_MESH, OB_SURF, OB_CURVE, OB_MBALL) 
+			make_trans_verts(bmat[0], bmat[1], 0);
+		if(tottrans==0) return;
 
-			tv= transvmain;
-			for(a=0; a<tottrans; a++, tv++) {
-				VECCOPY(vec, tv->loc);
-				Mat3MulVecfl(bmat, vec);
-				VecAddf(vec, vec, G.obedit->obmat[3]);
+		Mat3CpyMat4(bmat, G.obedit->obmat);
+		Mat3Inv(imat, bmat);
+
+		tv= transvmain;
+		for(a=0; a<tottrans; a++, tv++) {
+			VECCOPY(vec, tv->loc);
+			Mat3MulVecfl(bmat, vec);
+			VecAddf(vec, vec, G.obedit->obmat[3]);
+			VecAddf(centroid, centroid, vec);
+			DO_MINMAX(vec, min, max);
+		}
+
+		if(G.vd->around==V3D_CENTROID) {
+			VecMulf(centroid, 1.0/(float)tottrans);
+			VECCOPY(snaploc, centroid);
+		}
+		else {
+			snaploc[0]= (min[0]+max[0])/2;
+			snaploc[1]= (min[1]+max[1])/2;
+			snaploc[2]= (min[2]+max[2])/2;
+		}
+		
+		MEM_freeN(transvmain);
+		transvmain= 0;
+
+	}
+	else {
+		base= (G.scene->base.first);
+		while(base) {
+			if(((base)->flag & SELECT) && ((base)->lay & G.vd->lay) ) {
+				VECCOPY(vec, base->object->obmat[3]);
 				VecAddf(centroid, centroid, vec);
 				DO_MINMAX(vec, min, max);
+				count++;
 			}
-
+			base= base->next;
+		}
+		if(count) {
 			if(G.vd->around==V3D_CENTROID) {
-				VecMulf(centroid, 1.0/(float)tottrans);
+				VecMulf(centroid, 1.0/(float)count);
 				VECCOPY(snaploc, centroid);
 			}
 			else {
@@ -1100,89 +1436,53 @@ void snap_to_center()
 				snaploc[1]= (min[1]+max[1])/2;
 				snaploc[2]= (min[2]+max[2])/2;
 			}
-			
-			MEM_freeN(transvmain);
-			transvmain= 0;
-
 		}
-		else {
-			base= (G.scene->base.first);
-			while(base) {
-				if(((base)->flag & SELECT) && ((base)->lay & G.vd->lay) ) {
-					VECCOPY(vec, base->object->obmat[3]);
-					VecAddf(centroid, centroid, vec);
-					DO_MINMAX(vec, min, max);
-					count++;
-				}
-				base= base->next;
+	}
+
+	/* Snap the selection to the snaplocation (duh!) */
+	if(G.obedit) {
+		tottrans= 0;
+
+		if ELEM6(G.obedit->type, OB_ARMATURE, OB_LATTICE, OB_MESH, OB_SURF, OB_CURVE, OB_MBALL) 
+			make_trans_verts(bmat[0], bmat[1], 0);
+		if(tottrans==0) return;
+
+		Mat3CpyMat4(bmat, G.obedit->obmat);
+		Mat3Inv(imat, bmat);
+
+		tv= transvmain;
+		for(a=0; a<tottrans; a++, tv++) {
+
+			vec[0]= snaploc[0]-G.obedit->obmat[3][0];
+			vec[1]= snaploc[1]-G.obedit->obmat[3][1];
+			vec[2]= snaploc[2]-G.obedit->obmat[3][2];
+
+			Mat3MulVecfl(imat, vec);
+			VECCOPY(tv->loc, vec);
+		}
+		MEM_freeN(transvmain);
+		transvmain= 0;
+
+		special_transvert_update();
+
+		allqueue(REDRAWVIEW3D, 0);
+		return;
+	}
+
+	base= (G.scene->base.first);
+	while(base) {
+		if( ( ((base)->flag & SELECT) && ((base)->lay & G.vd->lay) && ((base)->object->id.lib==0))) {
+			ob= base->object;
+			if(ob->flag & OB_POSEMODE) {
+				; // todo
 			}
-			if(count) {
-				if(G.vd->around==V3D_CENTROID) {
-					VecMulf(centroid, 1.0/(float)count);
-					VECCOPY(snaploc, centroid);
-				}
-				else {
-					snaploc[0]= (min[0]+max[0])/2;
-					snaploc[1]= (min[1]+max[1])/2;
-					snaploc[2]= (min[2]+max[2])/2;
-				}
-			}
-		}
-
-
-/* Snap the selection to the snaplocation (duh!) */
-
-		if(G.obedit) {
-			tottrans= 0;
-#ifdef __NLA
-			if ELEM5(G.obedit->type, OB_ARMATURE, OB_LATTICE, OB_MESH, OB_SURF, OB_CURVE) make_trans_verts(bmat[0], bmat[1], 0);
-#else
-			if ELEM4(G.obedit->type, OB_LATTICE, OB_MESH, OB_SURF, OB_CURVE) make_trans_verts(bmat[0], bmat[1], 0);
-#endif
-			if(tottrans==0) return;
-
-			Mat3CpyMat4(bmat, G.obedit->obmat);
-			Mat3Inv(imat, bmat);
-
-			tv= transvmain;
-			for(a=0; a<tottrans; a++, tv++) {
-
-
-				vec[0]= snaploc[0]-G.obedit->obmat[3][0];
-				vec[1]= snaploc[1]-G.obedit->obmat[3][1];
-				vec[2]= snaploc[2]-G.obedit->obmat[3][2];
-
-
-				Mat3MulVecfl(imat, vec);
-				VECCOPY(tv->loc, vec);
-
-			}
-			MEM_freeN(transvmain);
-			transvmain= 0;
-
-			if ELEM(G.obedit->type, OB_SURF, OB_CURVE) makeDispList(G.obedit);
-
-			special_trans_update(0);
-
-			allqueue(REDRAWVIEW3D, 0);
-			return;
-		}
-#ifdef __NLA
-		if (G.obpose){
-			allqueue(REDRAWVIEW3D, 0);
-			return;
-		}
-#endif
-		base= (G.scene->base.first);
-		while(base) {
-			if( ( ((base)->flag & SELECT) && ((base)->lay & G.vd->lay) && ((base)->object->id.lib==0))) {
-				ob= base->object;
-
+			else {
+				ob->recalc |= OB_RECALC_OB;
+				
 				vec[0]= -ob->obmat[3][0] + snaploc[0];
 				vec[1]= -ob->obmat[3][1] + snaploc[1];
 				vec[2]= -ob->obmat[3][2] + snaploc[2];
 
-
 				if(ob->parent) {
 					where_is_object(ob);
 
@@ -1198,11 +1498,12 @@ void snap_to_center()
 					ob->loc[2]+= vec[2];
 				}
 			}
-
-			base= base->next;
 		}
-		update_select_dependency();
-		allqueue(REDRAWVIEW3D, 0);
+
+		base= base->next;
+	}
+	DAG_scene_flush_update(G.scene, screen_view3d_layers());
+	allqueue(REDRAWVIEW3D, 0);
 }
 
 
@@ -1237,7 +1538,6 @@ void snapmenu()
 
 void mergemenu(void)
 {
-	extern float doublimit;
 	short event;
 
 	event = pupmenu("Merge %t|At Center%x1|At Cursor%x2");
@@ -1249,14 +1549,15 @@ void mergemenu(void)
 	else
 		snap_sel_to_curs(); /*Merge at Cursor*/
 
-	notice("Removed %d Vertices", removedoublesflag(1, doublimit));
+	notice("Removed %d Vertices", removedoublesflag(1, G.scene->toolsettings->doublimit));
 	allqueue(REDRAWVIEW3D, 0);
 	countall();
 	BIF_undo_push("Merge"); /* push the mesh down the undo pipe */
 
 }
 
-void delete_context_selected(void) {
+void delete_context_selected(void) 
+{
 	if(G.obedit) {
 		if(G.obedit->type==OB_MESH) delete_mesh();
 		else if ELEM(G.obedit->type, OB_CURVE, OB_SURF) delNurb();
@@ -1266,15 +1567,16 @@ void delete_context_selected(void) {
 	else delete_obj(0);
 }
 
-void duplicate_context_selected(void) {
+void duplicate_context_selected(void) 
+{
 	if(G.obedit) {
 		if(G.obedit->type==OB_MESH) adduplicate_mesh();
 		else if(G.obedit->type==OB_ARMATURE) adduplicate_armature();
 		else if(G.obedit->type==OB_MBALL) adduplicate_mball();
 		else if ELEM(G.obedit->type, OB_CURVE, OB_SURF) adduplicate_nurb();
 	}
-	else if(!(G.obpose)){
-		adduplicate(0);
+	else {
+		adduplicate(0, U.dupflag);
 	}
 }
 
@@ -1302,18 +1604,13 @@ void toggle_shading(void)
 
 void minmax_verts(float *min, float *max)
 {
-	extern TransVert *transvmain;
-	extern int tottrans;
 	TransVert *tv;
 	float centroid[3], vec[3], bmat[3][3];
 	int a;
 
 	tottrans=0;
-#ifdef __NLA
-	if ELEM5(G.obedit->type, OB_ARMATURE, OB_LATTICE, OB_MESH, OB_SURF, OB_CURVE) make_trans_verts(bmat[0], bmat[1], 0);
-#else
-	if ELEM4(G.obedit->type, OB_LATTICE, OB_MESH, OB_SURF, OB_CURVE) make_trans_verts(bmat[0], bmat[1], 0);
-#endif
+	if ELEM5(G.obedit->type, OB_ARMATURE, OB_LATTICE, OB_MESH, OB_SURF, OB_CURVE) 
+		make_trans_verts(bmat[0], bmat[1], 0);
 	if(tottrans==0) return;
 
 	Mat3CpyMat4(bmat, G.obedit->obmat);
