@@ -1,5 +1,5 @@
 /**
- * $Id: editface.c,v 1.40 2005/05/12 19:17:12 ton Exp $
+ * $Id: editface.c,v 1.50 2005/10/24 22:20:30 blendix Exp $
  *
  * ***** BEGIN GPL/BL DUAL LICENSE BLOCK *****
  *
@@ -59,15 +59,18 @@
 #include "DNA_userdef_types.h"
 
 #include "BKE_utildefines.h"
+#include "BKE_depsgraph.h"
 #include "BKE_displist.h"
 #include "BKE_global.h"
 #include "BKE_mesh.h"
 #include "BKE_texture.h"
+#include "BKE_object.h"
 
 #include "BSE_view.h"
 #include "BSE_edit.h"
 #include "BSE_drawview.h"	/* for backdrawview3d */
 
+#include "BIF_editsima.h"
 #include "BIF_interface.h"
 #include "BIF_mywindow.h"
 #include "BIF_toolbox.h"
@@ -87,12 +90,8 @@
 #include "blendef.h"
 #include "butspace.h"
 
-#include "TPT_DependKludge.h"
-
-#ifdef NAN_TPT
 #include "../img/IMG_Api.h"
 #include "BSE_trans_types.h"
-#endif /* NAN_TPT */
 
 #include "BDR_unwrapper.h"
 
@@ -114,7 +113,7 @@ static void uv_calc_center_vector(float *result, Object *ob, Mesh *me)
 		tface= me->tface;
 		mface= me->mface;
 		for(a=0; a<me->totface; a++, mface++, tface++) {
-			if(tface->flag & TF_SELECT && mface->v3) {
+			if(tface->flag & TF_SELECT) {
 				DO_MINMAX((me->mvert+mface->v1)->co, min, max);
 				DO_MINMAX((me->mvert+mface->v2)->co, min, max);
 				DO_MINMAX((me->mvert+mface->v3)->co, min, max);
@@ -308,7 +307,7 @@ void calculate_uv_map(unsigned short mapmode)
 		tface= me->tface;
 		mface= me->mface;
 		for(a=0; a<me->totface; a++, mface++, tface++) {
-			if(tface->flag & TF_SELECT && mface->v3) {
+			if(tface->flag & TF_SELECT) {
 				uv_calc_shift_project(tface->uv[0],cent,rotatematrix,3,(me->mvert+mface->v1)->co,min,max);
 				uv_calc_shift_project(tface->uv[1],cent,rotatematrix,3,(me->mvert+mface->v2)->co,min,max);
 				uv_calc_shift_project(tface->uv[2],cent,rotatematrix,3,(me->mvert+mface->v3)->co,min,max);
@@ -324,7 +323,7 @@ void calculate_uv_map(unsigned short mapmode)
 		tface= me->tface;
 		mface= me->mface;
 		for(a=0; a<me->totface; a++, mface++, tface++) {
-			if(tface->flag & TF_SELECT && mface->v3) {
+			if(tface->flag & TF_SELECT) {
 				if(mface->v4) b= 3; else b= 2;
 				for(; b>=0; b--) {
 					tface->uv[b][0]= ((tface->uv[b][0]-min[0])*fac)/dx;
@@ -341,7 +340,7 @@ void calculate_uv_map(unsigned short mapmode)
 		tface= me->tface;
 		mface= me->mface;
 		for(a=0; a<me->totface; a++, mface++, tface++) {
-			if(tface->flag & TF_SELECT && mface->v3) {
+			if(tface->flag & TF_SELECT) {
 				uv_calc_shift_project(tface->uv[0],cent,rotatematrix,4,(me->mvert+mface->v1)->co,NULL,NULL);
 				uv_calc_shift_project(tface->uv[1],cent,rotatematrix,4,(me->mvert+mface->v2)->co,NULL,NULL);
 				uv_calc_shift_project(tface->uv[2],cent,rotatematrix,4,(me->mvert+mface->v3)->co,NULL,NULL);
@@ -382,7 +381,7 @@ void calculate_uv_map(unsigned short mapmode)
 		tface= me->tface;
 		mface= me->mface;
 		for(a=0; a<me->totface; a++, mface++, tface++) {
-			if(tface->flag & TF_SELECT && mface->v3) {
+			if(tface->flag & TF_SELECT) {
 				uv_calc_shift_project(tface->uv[0],cent,rotatematrix,mapmode,(me->mvert+mface->v1)->co,NULL,NULL);
 				uv_calc_shift_project(tface->uv[1],cent,rotatematrix,mapmode,(me->mvert+mface->v2)->co,NULL,NULL);
 				uv_calc_shift_project(tface->uv[2],cent,rotatematrix,mapmode,(me->mvert+mface->v3)->co,NULL,NULL);
@@ -419,7 +418,7 @@ void calculate_uv_map(unsigned short mapmode)
 		tface= me->tface;
 		mface= me->mface;
 		for(a=0; a<me->totface; a++, mface++, tface++) {
-			if(tface->flag & TF_SELECT && mface->v3) {
+			if(tface->flag & TF_SELECT) {
 				CalcNormFloat((mv+mface->v1)->co, (mv+mface->v2)->co, (mv+mface->v3)->co, no);
 					
 				no[0]= fabs(no[0]);
@@ -464,7 +463,7 @@ void calculate_uv_map(unsigned short mapmode)
 		tface= me->tface;
 		mface= me->mface;
 		for(a=0; a<me->totface; a++, mface++, tface++) {
-			if(!(tface->flag & TF_SELECT && mface->v3)) continue;
+			if(!(tface->flag & TF_SELECT)) continue;
 				
 			dx= dy= 0;
 			if(mface->v4) b= 3; else b= 2;
@@ -488,6 +487,8 @@ void calculate_uv_map(unsigned short mapmode)
 
 	BIF_undo_push("UV calculation");
 
+	object_uvs_changed(OBACT);
+
 	allqueue(REDRAWVIEW3D, 0);
 	allqueue(REDRAWIMAGE, 0);
 }
@@ -588,6 +589,7 @@ void make_tfaces(Mesh *me)
 	}
 }
 
+
 void reveal_tface()
 {
 	Mesh *me;
@@ -609,8 +611,7 @@ void reveal_tface()
 
 	BIF_undo_push("Reveal UV face");
 
-	allqueue(REDRAWVIEW3D, 0);
-	allqueue(REDRAWIMAGE, 0);
+	object_tface_flags_changed(OBACT, 0);
 }
 
 void hide_tface()
@@ -643,20 +644,16 @@ void hide_tface()
 		
 		tface++;
 	}
+
 	BIF_undo_push("Hide UV face");
-	allqueue(REDRAWVIEW3D, 0);
-	allqueue(REDRAWIMAGE, 0);
-	
+
+	object_tface_flags_changed(OBACT, 0);
 }
 
 void select_linked_tfaces(int mode)
 {
 	Object *ob;
 	Mesh *me;
-	TFace *tf;
-	MFace *mf;
-	int a, doit=1, mark=0;
-	char *cpmain, *linkflag;
 	short mval[2];
 	unsigned int index=0;
 
@@ -672,96 +669,7 @@ void select_linked_tfaces(int mode)
 		if (!face_pick(me, mval[0], mval[1], &index)) return;
 	}
 
-	if(me->medge) {
-		select_linked_tfaces_with_seams(mode, me, index);
-		return;
-	}
-
-	cpmain= MEM_callocN(me->totvert, "cpmain");
-	linkflag= MEM_callocN(sizeof(char)*me->totface, "linkflaguv");
-
-	if (mode==0 || mode==1) {
-		/* only put face under cursor in array */
-		mf= ((MFace*)me->mface) + index;
-		cpmain[mf->v1]= cpmain[mf->v2]= cpmain[mf->v3]= 1;
-		if (mf->v4) cpmain[mf->v4]= 1;
-		linkflag[index]= 1;
-	}
-	else {
-		/* fill array by selection */
-		tf= me->tface;
-		mf= me->mface;
-		for(a=0; a<me->totface; a++, tf++, mf++) {
-			if(tf->flag & TF_HIDE);
-			else if(tf->flag & TF_SELECT) {
-				if(mf->v3) {
-					cpmain[mf->v1]= 1;
-					cpmain[mf->v2]= 1;
-					cpmain[mf->v3]= 1;
-					if(mf->v4) cpmain[mf->v4]= 1;
-					linkflag[a]= 1;
-				}
-			}
-		}
-	}
-	
-	while(doit) {
-		doit= 0;
-		
-		/* expand selection */
-		tf= me->tface;
-		mf= me->mface;
-		for(a=0; a<me->totface; a++, tf++, mf++) {
-			if(tf->flag & TF_HIDE);
-			else if(mf->v3 && !linkflag[a]) {
-				mark= 0;
-
-				if(cpmain[mf->v1] || cpmain[mf->v2] || cpmain[mf->v3])
-					mark= 1;
-				else if(mf->v4 && cpmain[mf->v4])
-					mark= 1;
-
-				if(mark) {
-					linkflag[a]= 1;
-					cpmain[mf->v1]= cpmain[mf->v2]= cpmain[mf->v3]= 1;
-					if(mf->v4) cpmain[mf->v4]= 1;
-					doit= 1;
-				}
-			}
-		}
-		
-	}
-
-	if(mode==0 || mode==2) {
-		for(a=0, tf=me->tface; a<me->totface; a++, tf++)
-			if(linkflag[a])
-				tf->flag |= TF_SELECT;
-			else
-				tf->flag &= ~TF_SELECT;
-	}
-	else if(mode==1) {
-		for(a=0, tf=me->tface; a<me->totface; a++, tf++)
-			if(linkflag[a] && (tf->flag & TF_SELECT))
-				break;
-
-		if (a<me->totface) {
-			for(a=0, tf=me->tface; a<me->totface; a++, tf++)
-				if(linkflag[a])
-					tf->flag &= ~TF_SELECT;
-		}
-		else {
-			for(a=0, tf=me->tface; a<me->totface; a++, tf++)
-				if(linkflag[a])
-					tf->flag |= TF_SELECT;
-		}
-	}
-	
-	MEM_freeN(cpmain);
-	MEM_freeN(linkflag);
-	
-	BIF_undo_push("Select linked UV face");
-	allqueue(REDRAWVIEW3D, 0);
-	allqueue(REDRAWIMAGE, 0);
+	select_linked_tfaces_with_seams(mode, me, index);
 }
 
 void deselectall_tface()
@@ -794,8 +702,8 @@ void deselectall_tface()
 	}
 
 	BIF_undo_push("(De)select all UV face");
-	allqueue(REDRAWVIEW3D, 0);
-	allqueue(REDRAWIMAGE, 0);
+
+	object_tface_flags_changed(OBACT, 0);
 }
 
 void selectswap_tface(void)
@@ -819,8 +727,8 @@ void selectswap_tface(void)
 	}
 
 	BIF_undo_push("Select inverse UV face");
-	allqueue(REDRAWVIEW3D, 0);
-	allqueue(REDRAWIMAGE, 0);
+
+	object_tface_flags_changed(OBACT, 0);
 }
 
 void rotate_uv_tface()
@@ -885,9 +793,79 @@ void rotate_uv_tface()
 	}
 	
 	BIF_undo_push("Rotate UV face");
-	makeDispList(OBACT);
-	allqueue(REDRAWVIEW3D, 0);
-	allqueue(REDRAWIMAGE, 0);
+
+	object_uvs_changed(OBACT);
+}
+
+void mirror_uv_tface()
+{
+	Mesh *me;
+	TFace *tface;
+	MFace *mface;
+	short mode;
+	int a;
+	
+	me= get_mesh(OBACT);
+	if(me==0 || me->tface==0) return;
+	
+	mode= pupmenu("Mirror %t|UV Co-ordinates %x1|Vertex Colors %x2");
+	
+	if(mode<1) return;
+	
+	tface= me->tface;
+	mface= me->mface;
+	a= me->totface;
+	while(a--) {
+		if(tface->flag & TF_SELECT) {
+			if(mode==1) {
+				float u1= tface->uv[0][0];
+				float v1= tface->uv[0][1];
+				if(mface->v4) {
+					tface->uv[0][0]= tface->uv[3][0];
+					tface->uv[0][1]= tface->uv[3][1];
+				
+					tface->uv[3][0]= u1;
+					tface->uv[3][1]= v1;
+
+					u1= tface->uv[1][0];
+					v1= tface->uv[1][1];
+
+					tface->uv[1][0]= tface->uv[2][0];
+					tface->uv[1][1]= tface->uv[2][1];
+				
+					tface->uv[2][0]= u1;
+					tface->uv[2][1]= v1;
+				}
+				else {
+					tface->uv[0][0]= tface->uv[2][0];
+					tface->uv[0][1]= tface->uv[2][1];
+					tface->uv[2][0]= u1;
+					tface->uv[2][1]= v1;
+				}
+			}
+			else if(mode==2) {
+				unsigned int tcol= tface->col[0];
+				if(mface->v4) {
+					tface->col[0]= tface->col[3];
+					tface->col[3]= tcol;
+
+					tcol = tface->col[1];
+					tface->col[1]= tface->col[2];
+					tface->col[2]= tcol;
+				}
+				else {
+					tface->col[0]= tface->col[2];
+					tface->col[2]= tcol;
+				}
+			}
+		}
+		tface++;
+		mface++;
+	}
+	
+	BIF_undo_push("Mirror UV face");
+
+	object_uvs_changed(OBACT);
 }
 
 void minmax_tface(float *min, float *max)
@@ -911,7 +889,7 @@ void minmax_tface(float *min, float *max)
 	mf= me->mface;
 	tf= me->tface;
 	for (a=me->totface; a>0; a--, mf++, tf++) {
-		if (!mf->v3 || tf->flag & TF_HIDE || !(tf->flag & TF_SELECT))
+		if (tf->flag & TF_HIDE || !(tf->flag & TF_SELECT))
 			continue;
 
 		VECCOPY(vec, (mv+mf->v1)->co);
@@ -979,7 +957,7 @@ int face_pick(Mesh *me, short x, short y, unsigned int *index)
 	}
 	/* Convert the color back to a face index */
 	*index = framebuffer_to_index(col);
-	if (col==0 || (*index)<=0 || (*index)>me->totface)
+	if (col==0 || (*index)<=0 || (*index)>(unsigned) me->totface)
 		return 0;
 
 	(*index)--;
@@ -1036,9 +1014,8 @@ void face_select()
 	/* image window redraw */
 	
 	BIF_undo_push("Select UV face");
-	allqueue(REDRAWIMAGE, 0);
-	allqueue(REDRAWBUTSEDIT, 0);
-	allqueue(REDRAWVIEW3D, 0);
+
+	object_tface_flags_changed(OBACT, 1);
 }
 
 void face_borderselect()
@@ -1098,29 +1075,13 @@ void face_borderselect()
 
 		BIF_undo_push("Border Select UV face");
 
-		allqueue(REDRAWVIEW3D, 0);
-		allqueue(REDRAWIMAGE, 0);
+		object_tface_flags_changed(OBACT, 0);
 	}
 #ifdef __APPLE__	
 	glReadBuffer(GL_BACK);
 #endif
 }
 
-#define TEST_STRUBI 1				
-#ifdef TEST_STRUBI
-float CalcNormUV(float *a, float *b, float *c)
-{
-	float d1[3], d2[3];
-
-	d1[0] = a[0] - b[0];
-	d1[1] = a[1] - b[1];
-	d2[0] = b[0] - c[0];
-	d2[1] = b[1] - c[1];
-	return (d1[0] * d2[1] - d1[1] * d2[0]);
-}
-#endif
-
-
 /* Pupmenu codes: */
 #define UV_CUBE_MAPPING 2
 #define UV_CYL_MAPPING 3
@@ -1202,7 +1163,7 @@ void set_faceselect()	/* toggle */
 	Mesh *me = 0;
 	
 	scrarea_queue_headredraw(curarea);
-	if(ob->id.lib) return;
+	if(ob==NULL || ob->id.lib) return;
 	
 	if(G.f & G_FACESELECT) G.f &= ~G_FACESELECT;
 	else {
@@ -1226,16 +1187,17 @@ void set_faceselect()	/* toggle */
 		BIF_undo_push("Set UV Faceselect");
 	}
 	else if((G.f & (G_WEIGHTPAINT|G_VERTEXPAINT|G_TEXTUREPAINT))==0) {
-		if(me) reveal_tface();
+		if(me) {
+			reveal_tface();
+			DAG_object_flush_update(G.scene, ob, OB_RECALC_DATA);
+		}
 		setcursor_space(SPACE_VIEW3D, CURSOR_STD);
-		makeDispList(ob);
 		BIF_undo_push("End UV Faceselect");
 	}
 	countall();
 }
 
 
-#ifdef NAN_TPT
 /**
  * Get the view ray through the screen point.
  * Uses the OpenGL settings of the active view port.
@@ -1522,7 +1484,7 @@ void face_draw()
 		error("The active object does not have a mesh obData"); return;
 	}
 
-	brush = IMG_BrushCreate(Gvp.size, Gvp.size, Gvp.r, Gvp.g, Gvp.b, Gvp.a);
+	brush = IMG_BrushCreate(Gvp.size, Gvp.size, &Gvp.r);
 	if (!brush) {
 		error("Can't create brush"); return;
 	}
@@ -1710,7 +1672,7 @@ void get_same_uv(void)
 	
 	/* image window redraw */
 	BIF_undo_push("Get same UV");
-	allqueue(REDRAWIMAGE, 0);
-	allqueue(REDRAWVIEW3D, 0);
+
+	object_tface_flags_changed(OBACT, 0);
 }
-#endif /* NAN_TPT */
+
