@@ -1,5 +1,5 @@
 /**
- * $Id: editkey.c,v 1.21 2005/03/19 21:08:12 zuster Exp $
+ * $Id: editkey.c,v 1.31 2005/11/17 21:31:49 ton Exp $
  *
  * ***** BEGIN GPL/BL DUAL LICENSE BLOCK *****
  *
@@ -31,44 +31,44 @@
  */
 
 #include <math.h>
-
-#ifdef HAVE_CONFIG_H
-#include <config.h>
-#endif
+#include <string.h>
 
 #ifndef WIN32
 #include <unistd.h>
 #else
 #include <io.h>
 #endif   
+
 #include "MEM_guardedalloc.h"
 
 #include "BLI_blenlib.h"
 #include "BLI_arithb.h"
 
+#include "DNA_action_types.h"
 #include "DNA_curve_types.h"
 #include "DNA_ipo_types.h"
 #include "DNA_key_types.h"
+#include "DNA_lattice_types.h"
 #include "DNA_mesh_types.h"
 #include "DNA_meshdata_types.h"
 #include "DNA_object_types.h"
+#include "DNA_scene_types.h"
 #include "DNA_screen_types.h"
 #include "DNA_space_types.h"
 #include "DNA_userdef_types.h"
 #include "DNA_view2d_types.h"
-#include "DNA_lattice_types.h"
-#include "DNA_scene_types.h"
 
-#include "BKE_utildefines.h"
+#include "BKE_action.h"
 #include "BKE_anim.h"
 #include "BKE_curve.h"
+#include "BKE_depsgraph.h"
 #include "BKE_global.h"
 #include "BKE_ipo.h"
 #include "BKE_key.h"
 #include "BKE_library.h"
 #include "BKE_main.h"
 #include "BKE_object.h"
-#include "BKE_displist.h"
+#include "BKE_utildefines.h"
 
 #include "BIF_editkey.h"
 #include "BIF_editview.h"
@@ -88,10 +88,6 @@
 
 extern ListBase editNurb; /* in editcurve.c */
 
-
-/* local prototypes ------------------ */
-void make_rvk_slider(uiBlock *, Key *, int , int , int , int , int );  /* used in drawaction.c too */
-
 /* temporary storage for slider values */
 float meshslidervals[64] = {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0,
                             0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0,
@@ -102,19 +98,18 @@ float meshslidervals[64] = {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0,
                             0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0,
                             0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0};
 
-static IpoCurve *get_key_icu(Key *key, int keynum) {
+static IpoCurve *get_key_icu(Ipo *ipo, int keynum) 
+{
 	/* return the Ipocurve that has the specified
 	 * keynum as ardcode -- return NULL if no such 
 	 * curve exists.
 	 */
     IpoCurve *icu;
-	if (!(key->ipo)) {
-		key->ipo = get_ipo((ID *)key, ID_KE, 1);
+	
+	if (!ipo) 
 		return NULL;
-	}
 
-
-    for (icu = key->ipo->curve.first; icu ; icu = icu->next) {
+    for (icu = ipo->curve.first; icu ; icu = icu->next) {
         if (!icu->adrcode) continue;
         if (icu->adrcode == keynum) return icu;
     }
@@ -148,21 +143,26 @@ static BezTriple *get_bezt_icu_time(IpoCurve *icu, float *frame, float *val) {
 	return bezt;
 }
 
-static void rvk_slider_func(void *voidkey, void *voidkeynum) {
+static void rvk_slider_func(void *voidob, void *voidkeynum) 
+{
 	/* the callback for the rvk sliders ... copies the
 	 * value from the temporary array into a bezier at the
 	 * right frame on the right ipo curve (creating both the
 	 * ipo curve and the bezier if needed).
 	 */
-	int       *keynum = (int *) voidkeynum;
-	Key       *key = (Key *) voidkey;
-	float     cfra, rvkval;
+	Object *ob= voidob;
 	IpoCurve  *icu=NULL;
 	BezTriple *bezt=NULL;
+	float cfra, rvkval;
+	int keynum = (int) voidkeynum;
 
 	cfra = frame_to_float(CFRA);
 
-	icu    = get_key_icu(key, *keynum);
+	/* ipo on action or ob? */
+	if(ob->ipoflag & OB_ACTION_KEY)
+		icu = verify_ipocurve(&ob->id, ID_KE, "Shape", NULL, keynum);
+	else 
+		icu = verify_ipocurve(&ob->id, ID_KE, NULL, NULL, keynum);
 
 	if (icu) {
 		/* if the ipocurve exists, try to get a bezier
@@ -170,22 +170,15 @@ static void rvk_slider_func(void *voidkey, void *voidkeynum) {
 		 */
 		bezt = get_bezt_icu_time(icu, &cfra, &rvkval);
 	}
-	else {
-		/* create an IpoCurve if one doesn't already
-		 * exist.
-		 */
-		icu = get_ipocurve(key->from, GS(key->from->name), 
-						   *keynum, key->ipo);
-	}
 	
 	/* create the bezier triple if one doesn't exist,
 	 * otherwise modify it's value
 	 */
 	if (!bezt) {
-		insert_vert_ipo(icu, cfra, meshslidervals[*keynum]);
+		insert_vert_ipo(icu, cfra, meshslidervals[keynum]);
 	}
 	else {
-		bezt->vec[1][1] = meshslidervals[*keynum];
+		bezt->vec[1][1] = meshslidervals[keynum];
 	}
 
 	/* make sure the Ipo's are properly process and
@@ -193,36 +186,19 @@ static void rvk_slider_func(void *voidkey, void *voidkeynum) {
 	 */
 	sort_time_ipocurve(icu);
 	testhandles_ipocurve(icu);
-
-	do_all_ipos();
-	do_spec_key(key);
-	/* if I'm deformed by a lattice, update my
-	 * displists
-	 */
-	makeDispList(OBACT);
-
-	/* if I'm a lattice, update the displists of
-	 * my children
-	 */
-	if (OBACT->type==OB_LATTICE ) {
-		Base *base;
-
-		base= FIRSTBASE;
-		while(base) {
-			if (base->object->parent == OBACT) {
-				makeDispList(base->object);
-			}
-			base= base->next;
-		}
-	}
+	
+	ob->shapeflag &= ~OB_SHAPE_TEMPLOCK;
+	DAG_object_flush_update(G.scene, ob, OB_RECALC_DATA);
+	
 	allqueue (REDRAWVIEW3D, 0);
 	allqueue (REDRAWACTION, 0);
 	allqueue (REDRAWNLA, 0);
 	allqueue (REDRAWIPO, 0);
-
+	allspace(REMAKEIPO, 0);
 }
 
-static float getrvkval(Key *key, int keynum) {
+static float getrvkval(Ipo *ipo, int keynum) 
+{
 	/* get the value of the rvk from the
 	 * ipo curve at the current time -- return 0
 	 * if no ipo curve exists
@@ -233,7 +209,7 @@ static float getrvkval(Key *key, int keynum) {
 	float     cfra;
 
 	cfra = frame_to_float(CFRA);
-	icu    = get_key_icu(key, keynum);
+	icu    = get_key_icu(ipo, keynum);
 	if (icu) {
 		bezt = get_bezt_icu_time(icu, &cfra, &rvkval);
 		if (!bezt) {
@@ -245,29 +221,32 @@ static float getrvkval(Key *key, int keynum) {
 
 }
 
-void make_rvk_slider(uiBlock *block, Key *key, int keynum,
-					 int x, int y, int w, int h)
+void make_rvk_slider(uiBlock *block, Object *ob, int keynum,
+					 int x, int y, int w, int h, char *tip)
 {
 	/* create a slider for the rvk */
-	uiBut         *but;
+	uiBut *but;
+	Ipo *ipo= NULL;
+	Key *key= ob_get_key(ob);
 	KeyBlock   *kb;
 	float min, max;
 	int i;
-
-	/* dang, need to pass a pointer to int to uiButSetFunc
-	 * that is on the heap, not the stack ... hence this
-	 * kludgy static array
-	 */
-	static int keynums[] = {0,1,2,3,4,5,6,7,
-							8,9,10,11,12,13,14,15,
-							16,17,18,19,20,21,22,23,
-							24,25,26,27,28,29,30,31,
-							32,33,34,35,36,37,38,39,
-							40,41,42,43,44,45,46,47,
-							48,49,50,51,52,53,54,55,
-							56,57,58,59,60,61,62,63};
-
-	meshslidervals[keynum] = getrvkval(key, keynum);
+	
+	if(key==NULL) return;
+	
+	/* ipo on action or ob? */
+	if(ob->ipoflag & OB_ACTION_KEY) {
+		if(ob->action) {
+			bActionChannel *achan;
+			
+			achan= get_action_channel(ob->action, "Shape");
+			if(achan) ipo= achan->ipo;
+		}
+	}
+	else ipo= key->ipo;
+	
+	/* global array */
+	meshslidervals[keynum] = getrvkval(ipo, keynum);
 
 	kb= key->block.first;
 	for (i=0; i<keynum; ++i) kb = kb->next; 
@@ -285,9 +264,9 @@ void make_rvk_slider(uiBlock *block, Key *key, int keynum,
 
 	but=uiDefButF(block, NUMSLI, REDRAWVIEW3D, "",
 				  x, y , w, h,
-				  meshslidervals+keynum, min, max, 10, 2,
-				  "Slider to control rvk");
-	uiButSetFunc(but, rvk_slider_func, key, keynums+keynum);
+				  meshslidervals+keynum, min, max, 10, 2, tip);
+	
+	uiButSetFunc(but, rvk_slider_func, ob, (void *)keynum);
 	// no hilite, the winmatrix is not correct later on...
 	uiButSetFlag(but, UI_NO_HILITE);
 
@@ -304,7 +283,7 @@ static void default_key_ipo(Key *key)
 			
 	icu->blocktype= ID_KE;
 	icu->adrcode= KEY_SPEED;
-	icu->flag= IPO_VISIBLE+IPO_SELECT;
+	icu->flag= IPO_VISIBLE|IPO_SELECT|IPO_AUTO_HORIZ;
 	set_icu_vars(icu);
 	
 	BLI_addtail( &(key->ipo->curve), icu);
@@ -366,60 +345,59 @@ void key_to_mesh(KeyBlock *kb, Mesh *me)
 	}
 }
 
-
-
-void insert_meshkey(Mesh *me, short offline)
+static KeyBlock *add_keyblock(Key *key)
 {
-	Key *key;
-	KeyBlock *kb, *kkb;
-	float curpos;
-	short rel;
-
-	if(me->key==0) {
-		me->key= add_key( (ID *)me);
-
-		if (!offline) /* interactive */
-			rel = pupmenu("Insert Vertex Keys %t|"
-										"Relative Keys %x1|Absolute Keys %x2");
-		else /* we were called from a script */
-			rel = offline;
-
-		switch (rel) {
-		case 1:
-			me->key->type = KEY_RELATIVE;
-			break;
-		default:
-			default_key_ipo(me->key);
-			break;
-		}
-	}
-	key= me->key;
+	KeyBlock *kb;
+	float curpos= -0.1;
+	int tot;
+	
+	kb= key->block.last;
+	if(kb) curpos= kb->pos;
 	
 	kb= MEM_callocN(sizeof(KeyBlock), "Keyblock");
 	BLI_addtail(&key->block, kb);
 	kb->type= KEY_CARDINAL;
-	
-	curpos= bsystem_time(0, 0, (float)CFRA, 0.0);
-	if(calc_ipo_spec(me->key->ipo, KEY_SPEED, &curpos)==0) {
-		curpos /= 100.0;
-	}
-	kb->pos= curpos;
+	tot= BLI_countlist(&key->block);
+	if(tot==1) strcpy(kb->name, "Basis");
+	else sprintf(kb->name, "Key %d", tot-1);
+	kb->adrcode= tot-1;
 	
 	key->totkey++;
 	if(key->totkey==1) key->refkey= kb;
 	
-	mesh_to_key(me, kb);
 	
-	sort_keys(me->key);
-
-	/* curent active: */
-	kkb= key->block.first;
-	while(kkb) {
-		kkb->flag &= ~SELECT;
-		if(kkb==kb) kkb->flag |= SELECT;
+	if(key->type == KEY_RELATIVE) 
+		kb->pos= curpos+0.1;
+	else {
+		curpos= bsystem_time(0, 0, (float)CFRA, 0.0);
+		if(calc_ipo_spec(key->ipo, KEY_SPEED, &curpos)==0) {
+			curpos /= 100.0;
+		}
+		kb->pos= curpos;
 		
-		kkb= kkb->next;
+		sort_keys(key);
 	}
+	return kb;
+}
+
+void insert_meshkey(Mesh *me, short rel)
+{
+	Key *key;
+	KeyBlock *kb;
+
+	if(me->key==NULL) {
+		me->key= add_key( (ID *)me);
+
+		if(rel)
+			me->key->type = KEY_RELATIVE;
+		else
+			default_key_ipo(me->key);
+	}
+	key= me->key;
+	
+	kb= add_keyblock(key);
+	
+	mesh_to_key(me, kb);
 }
 
 /* ******************** */
@@ -463,43 +441,21 @@ void key_to_latt(KeyBlock *kb, Lattice *lt)
 	
 }
 
-void insert_lattkey(Lattice *lt)
+/* exported to python... hrms, should not, use object levels! (ton) */
+void insert_lattkey(Lattice *lt, short rel)
 {
 	Key *key;
-	KeyBlock *kb, *kkb;
-	float curpos;
+	KeyBlock *kb;
 	
-	if(lt->key==0) {
+	if(lt->key==NULL) {
 		lt->key= add_key( (ID *)lt);
 		default_key_ipo(lt->key);
 	}
 	key= lt->key;
 	
-	kb= MEM_callocN(sizeof(KeyBlock), "Keyblock");
-	BLI_addtail(&key->block, kb);
-	kb->type= KEY_CARDINAL;
-	
-	curpos= bsystem_time(0, 0, (float)CFRA, 0.0);
-	if(calc_ipo_spec(lt->key->ipo, KEY_SPEED, &curpos)==0) {
-		curpos /= 100.0;
-	}
-	kb->pos= curpos;
-	
-	key->totkey++;
-	if(key->totkey==1) key->refkey= kb;
+	kb= add_keyblock(key);
 	
 	latt_to_key(lt, kb);
-	
-	sort_keys(lt->key);
-
-	/* curent active: */
-	kkb= key->block.first;
-	while(kkb) {
-		kkb->flag &= ~SELECT;
-		if(kkb==kb) kkb->flag |= SELECT;
-		
-		kkb= kkb->next;
-	}
 }
 
 /* ******************************** */
@@ -606,177 +562,113 @@ void key_to_curve(KeyBlock *kb, Curve  *cu, ListBase *nurb)
 }
 
 
-
-void insert_curvekey(Curve *cu)
+void insert_curvekey(Curve *cu, short rel) 
 {
 	Key *key;
-	KeyBlock *kb, *kkb;
-	float curpos;
+	KeyBlock *kb;
 	
-	if(cu->key==0) {
+	if(cu->key==NULL) {
 		cu->key= add_key( (ID *)cu);
-		default_key_ipo(cu->key);
+
+		if (rel)
+			cu->key->type = KEY_RELATIVE;
+		else
+			default_key_ipo(cu->key);
 	}
 	key= cu->key;
 	
-	kb= MEM_callocN(sizeof(KeyBlock), "Keyblock");
-	BLI_addtail(&key->block, kb);
-	kb->type= KEY_CARDINAL;
-	
-	curpos= bsystem_time(0, 0, (float)CFRA, 0.0);
-	if(calc_ipo_spec(cu->key->ipo, KEY_SPEED, &curpos)==0) {
-		curpos /= 100.0;
-	}
-	kb->pos= curpos;
-	
-	key->totkey++;
-	if(key->totkey==1) key->refkey= kb;
+	kb= add_keyblock(key);
 	
 	if(editNurb.first) curve_to_key(cu, kb, &editNurb);
 	else curve_to_key(cu, kb, &cu->nurb);
-	
-	sort_keys(cu->key);
-
-	/* curent active: */
-	kkb= key->block.first;
-	while(kkb) {
-		kkb->flag &= ~SELECT;
-		if(kkb==kb) kkb->flag |= SELECT;
-		
-		kkb= kkb->next;
-	}
 }
 
 
 /* ******************** */
 
-Key *give_current_key(Object *ob)
+void insert_shapekey(Object *ob)
 {
-	Mesh *me;
-	Curve *cu;
-	Lattice *lt;
+	Key *key;
 	
-	if(ob->type==OB_MESH) {
-		me= ob->data;
-		return me->key;
-	}
-	else if ELEM(ob->type, OB_CURVE, OB_SURF) {
-		cu= ob->data;
-		return cu->key;
-	}
-	else if(ob->type==OB_LATTICE) {
-		lt= ob->data;
-		return lt->key;
-	}
-	return 0;
+	if(ob->type==OB_MESH) insert_meshkey(ob->data, 1);
+	else if ELEM(ob->type, OB_CURVE, OB_SURF) insert_curvekey(ob->data, 1);
+	else if(ob->type==OB_LATTICE) insert_lattkey(ob->data, 1);
+	
+	key= ob_get_key(ob);
+	ob->shapenr= BLI_countlist(&key->block);
+	
+	allspace(REMAKEIPO, 0);
+	allqueue(REDRAWIPO, 0);
+	allqueue(REDRAWACTION, 0);
+	allqueue(REDRAWNLA, 0);
+	allqueue(REDRAWBUTSOBJECT, 0);
+	allqueue(REDRAWBUTSEDIT, 0);
 }
 
-void showkeypos(Key *key, KeyBlock *kb)
-{
-	Object *ob;
-	Mesh *me;
-	Lattice *lt;
-	Curve *cu;
-	int tot;
-	
-	/* from ipo */
-	ob= OBACT;
-	if(ob==0) return;
-	
-	if(key == give_current_key(ob)) {
-		
-		if(ob->type==OB_MESH) {
-			me= ob->data;
-
-			cp_key(0, me->totvert, me->totvert, (char *)me->mvert->co, me->key, kb, 0);
-
-			make_displists_by_obdata(me);
-		}
-		else if(ob->type==OB_LATTICE) {
-			lt= ob->data;
-			tot= lt->pntsu*lt->pntsv*lt->pntsw;
-			
-			cp_key(0, tot, tot, (char *)lt->def->vec, lt->key, kb, 0);
-
-			make_displists_by_parent(ob);
-		}
-		else if ELEM(ob->type, OB_CURVE, OB_SURF) {
-			cu= ob->data;
-			tot= count_curveverts(&cu->nurb);
-			cp_cu_key(cu, kb, 0, tot);
-
-			make_displists_by_obdata(cu);
-		}
-		
-		allqueue(REDRAWVIEW3D, 0);
-	}
-}
-
-void deselectall_key(void)
+void delete_key(Object *ob)
 {
 	KeyBlock *kb;
 	Key *key;
+	IpoCurve *icu;
 	
-	if(G.sipo->blocktype!=ID_KE) return;
-	key= (Key *)G.sipo->from;
-	if(key==0) return;
+	key= ob_get_key(ob);
+	if(key==NULL) return;
 	
-	kb= key->block.first;
-	while(kb) {
-		kb->flag &= ~SELECT;
-		kb= kb->next;
-	}
-}
+	kb= BLI_findlink(&key->block, ob->shapenr-1);
 
-
-void delete_key(void)
-{
-	KeyBlock *kb, *kbn;
-	Key *key;
-	
-	if(G.sipo->blocktype!=ID_KE) return;
-
-	if(okee("Erase selected keys")==0) return;
-	
-	key= (Key *)G.sipo->from;
-	if(key==0) return;
-	
-	kb= key->block.first;
-	while(kb) {
-		kbn= kb->next;
-		if(kb->flag & SELECT) {
-			BLI_remlink(&key->block, kb);
-			key->totkey--;
-			if(key->refkey== kb) key->refkey= key->block.first;
-			
-			if(kb->data) MEM_freeN(kb->data);
-			MEM_freeN(kb);
+	if(kb) {
+		BLI_remlink(&key->block, kb);
+		key->totkey--;
+		if(key->refkey== kb) key->refkey= key->block.first;
 			
+		if(kb->data) MEM_freeN(kb->data);
+		MEM_freeN(kb);
+		
+		for(kb= key->block.first; kb; kb= kb->next) {
+			if(kb->adrcode>=ob->shapenr)
+				kb->adrcode--;
 		}
-		kb= kbn;
+		
+		if(key->ipo) {
+			for(icu= key->ipo->curve.first; icu; icu= icu->next) {
+				if(icu->adrcode==ob->shapenr-1) {
+					BLI_remlink(&key->ipo->curve, icu);
+					if(icu->bezt) MEM_freeN(icu->bezt);
+					MEM_freeN(icu);
+					break;
+				}
+			}
+			for(icu= key->ipo->curve.first; icu; icu= icu->next) 
+				if(icu->adrcode>=ob->shapenr)
+					icu->adrcode--;
+		}		
+		
+		if(ob->shapenr>1) ob->shapenr--;
 	}
 	
 	if(key->totkey==0) {
-		if(GS(key->from->name)==ID_ME) ((Mesh *)key->from)->key= 0;
-		else if(GS(key->from->name)==ID_CU) ((Curve *)key->from)->key= 0;
-		else if(GS(key->from->name)==ID_LT) ((Lattice *)key->from)->key= 0;
+		if(GS(key->from->name)==ID_ME) ((Mesh *)key->from)->key= NULL;
+		else if(GS(key->from->name)==ID_CU) ((Curve *)key->from)->key= NULL;
+		else if(GS(key->from->name)==ID_LT) ((Lattice *)key->from)->key= NULL;
 
 		free_libblock_us(&(G.main->key), key);
 		scrarea_queue_headredraw(curarea);	/* ipo remove too */
 	}
-	else do_spec_key(key);
+	
+	DAG_object_flush_update(G.scene, OBACT, OB_RECALC_DATA);
 	
 	allqueue(REDRAWVIEW3D, 0);
-	scrarea_queue_winredraw(curarea);
+	allqueue(REDRAWBUTSEDIT, 0);
+	allspace(REMAKEIPO, 0);
+	allqueue(REDRAWIPO, 0);
 }
 
-void move_keys(void)
+void move_keys(Object *ob)
 {
 	Key *key;
 	KeyBlock *kb;
-	TransVert *transmain, *tv;
-	float div, dy, vec[3], dvec[3];
-	int a, tot=0, afbreek=0, firsttime= 1;
+	float div, dy, oldpos, vec[3], dvec[3];
+	int afbreek=0, firsttime= 1;
 	unsigned short event = 0;
 	short mval[2], val, xo, yo;
 	char str[32];
@@ -784,36 +676,21 @@ void move_keys(void)
 	if(G.sipo->blocktype!=ID_KE) return;
 	
 	if(G.sipo->ipo && G.sipo->ipo->id.lib) return;
-	if(G.sipo->editipo==0) return;
+	if(G.sipo->editipo==NULL) return;
 
-	key= (Key *)G.sipo->from;
-	if(key==0) return;
+	key= ob_get_key(ob);
+	if(key==NULL) return;
 	
-	/* which keys are involved */
-	kb= key->block.first;
-	while(kb) {
-		if(kb->flag & SELECT) tot++;
-		kb= kb->next;
-	}
+	/* which kb is involved */
+	kb= BLI_findlink(&key->block, ob->shapenr-1);
+	if(kb==NULL) return;	
 	
-	if(tot==0) return;	
-	
-	tv=transmain= MEM_callocN(tot*sizeof(TransVert), "transmain");
-	kb= key->block.first;
-	while(kb) {
-		if(kb->flag & SELECT) {
-			tv->loc= &kb->pos;
-			tv->oldloc[0]= kb->pos;
-			tv++;
-		}
-		kb= kb->next;
-	}
+	oldpos= kb->pos;
 	
 	getmouseco_areawin(mval);
 	xo= mval[0];
 	yo= mval[1];
 	dvec[0]=dvec[1]=dvec[2]= 0.0; 
-	
 
 	while(afbreek==0) {
 		getmouseco_areawin(mval);
@@ -830,10 +707,7 @@ void move_keys(void)
 			apply_keyb_grid(vec, 0.0, 1.0, 0.1, U.flag & USER_AUTOGRABGRID);
 			apply_keyb_grid(vec+1, 0.0, 1.0, 0.1, U.flag & USER_AUTOGRABGRID);
 
-			tv= transmain;
-			for(a=0; a<tot; a++, tv++) {
-				tv->loc[0]= tv->oldloc[0]+vec[1];
-			}
+			kb->pos= oldpos+vec[1];
 			
 			sprintf(str, "Y: %.3f  ", vec[1]);
 			headerprint(str);
@@ -862,19 +736,18 @@ void move_keys(void)
 	}
 	
 	if(event==ESCKEY) {
-		tv= transmain;
-		for(a=0; a<tot; a++, tv++) {
-			tv->loc[0]= tv->oldloc[0];
-		}
+		kb->pos= oldpos;
 	}
 	
 	sort_keys(key);
-	do_spec_key(key);
+	DAG_object_flush_update(G.scene, ob, OB_RECALC_DATA);
 	
 	/* for boundbox */
 	editipo_changed(G.sipo, 0);
 
-	MEM_freeN(transmain);	
+	allspace(REMAKEIPO, 0);
+	allqueue(REDRAWIPO, 0);
 	allqueue(REDRAWVIEW3D, 0);
+	allqueue(REDRAWBUTSEDIT, 0);
 	scrarea_queue_redraw(curarea);
 }
