@@ -1,5 +1,5 @@
 /**
- * $Id: editmesh.c,v 1.125 2004/12/21 12:10:35 ton Exp $
+ * $Id: editmesh.c,v 1.167 2006/01/02 10:40:13 campbellbarton Exp $
  *
  * ***** BEGIN GPL/BL DUAL LICENSE BLOCK *****
  *
@@ -39,9 +39,6 @@
 #include <config.h>
 #endif
 
-#ifdef WIN32
-#include "BLI_winstuff.h"
-#endif
 #include "MEM_guardedalloc.h"
 
 #include "PIL_time.h"
@@ -49,11 +46,13 @@
 #include "DNA_mesh_types.h"
 #include "DNA_meshdata_types.h"
 #include "DNA_object_types.h"
+#include "DNA_object_force.h"
 #include "DNA_screen_types.h"
 #include "DNA_key_types.h"
 #include "DNA_scene_types.h"
 #include "DNA_view3d_types.h"
 #include "DNA_material_types.h"
+#include "DNA_modifier_types.h"
 #include "DNA_texture_types.h"
 #include "DNA_userdef_types.h"
 
@@ -61,17 +60,19 @@
 #include "BLI_arithb.h"
 #include "BLI_editVert.h"
 #include "BLI_dynstr.h"
+#include "BLI_rand.h"
 
-#include "BKE_utildefines.h"
-#include "BKE_key.h"
-#include "BKE_object.h"
-#include "BKE_displist.h"
+#include "BKE_DerivedMesh.h"
+#include "BKE_depsgraph.h"
 #include "BKE_global.h"
+#include "BKE_key.h"
 #include "BKE_library.h"
 #include "BKE_main.h"
 #include "BKE_material.h"
 #include "BKE_mesh.h"
+#include "BKE_object.h"
 #include "BKE_texture.h"
+#include "BKE_utildefines.h"
 
 #include "BIF_editkey.h"
 #include "BIF_editmesh.h"
@@ -93,7 +94,6 @@
 
 #include "mydevice.h"
 #include "blendef.h"
-#include "render.h"
 
 /* own include */
 #include "editmesh.h"
@@ -156,7 +156,7 @@ EditEdge *findedgelist(EditVert *v1, EditVert *v2)
 	struct HashEdge *he;
 
 	/* swap ? */
-	if( (long)v1 > (long)v2) {
+	if( v1 > v2) {
 		v3= v2; 
 		v2= v1; 
 		v1= v3;
@@ -364,6 +364,7 @@ EditFace *addfacelist(EditVert *v1, EditVert *v2, EditVert *v3, EditVert *v4, Ed
 	if(example) {
 		efa->mat_nr= example->mat_nr;
 		efa->tf= example->tf;
+		efa->tf.flag &= ~TF_ACTIVE;
 		efa->flag= example->flag;
 	}
 	else {
@@ -391,6 +392,48 @@ EditFace *addfacelist(EditVert *v1, EditVert *v2, EditVert *v3, EditVert *v4, Ed
 
 /* ************************ end add/new/find ************  */
 
+/* ************************ Edit{Vert,Edge,Face} utilss ***************************** */
+
+/* some nice utility functions */
+
+EditVert *editedge_getOtherVert(EditEdge *eed, EditVert *eve)
+{
+	if (eve==eed->v1) {
+		return eed->v2;
+	} else if (eve==eed->v2) {
+		return eed->v1;
+	} else {
+		return NULL;
+	}
+}
+
+EditVert *editedge_getSharedVert(EditEdge *eed, EditEdge *eed2) 
+{
+	if (eed->v1==eed2->v1 || eed->v1==eed2->v2) {
+		return eed->v1;
+	} else if (eed->v2==eed2->v1 || eed->v2==eed2->v2) {
+		return eed->v2;
+	} else {
+		return NULL;
+	}
+}
+
+int editedge_containsVert(EditEdge *eed, EditVert *eve) 
+{
+	return (eed->v1==eve || eed->v2==eve);
+}
+
+int editface_containsVert(EditFace *efa, EditVert *eve) 
+{
+	return (efa->v1==eve || efa->v2==eve || efa->v3==eve || (efa->v4 && efa->v4==eve));
+}
+
+int editface_containsEdge(EditFace *efa, EditEdge *eed) 
+{
+	return (efa->e1==eed || efa->e2==eed || efa->e3==eed || (efa->e4 && efa->e4==eed));
+}
+
+
 /* ************************ stuct EditMesh manipulation ***************************** */
 
 /* fake callocs for fastmalloc below */
@@ -453,6 +496,17 @@ void free_editMesh(EditMesh *em)
 	if(em->edges.first) free_edgelist(&em->edges);
 	if(em->faces.first) free_facelist(&em->faces);
 
+	if(em->derivedFinal) {
+		if (em->derivedFinal!=em->derivedCage) {
+			em->derivedFinal->release(em->derivedFinal);
+		}
+		em->derivedFinal= NULL;
+	}
+	if(em->derivedCage) {
+		em->derivedCage->release(em->derivedCage);
+		em->derivedCage= NULL;
+	}
+
 	/* DEBUG: hashtabs are slowest part of enter/exit editmode. here a testprint */
 #if 0
 	if(em->hashedgetab) {
@@ -528,6 +582,24 @@ static void edge_normal_compare(EditEdge *eed, EditFace *efa1)
 	if(inp < -0.001 ) eed->f1= 1;
 }
 
+#if 0
+typedef struct {
+	EditEdge *eed;
+	float noLen,no[3];
+	int adjCount;
+} EdgeDrawFlagInfo;
+
+static int edgeDrawFlagInfo_cmp(const void *av, const void *bv)
+{
+	const EdgeDrawFlagInfo *a = av;
+	const EdgeDrawFlagInfo *b = bv;
+
+	if (a->noLen<b->noLen) return -1;
+	else if (a->noLen>b->noLen) return 1;
+	else return 0;
+}
+#endif
+
 static void edge_drawflags(void)
 {
 	EditMesh *em = G.editMesh;
@@ -565,10 +637,10 @@ static void edge_drawflags(void)
 		e2= efa->e2;
 		e3= efa->e3;
 		e4= efa->e4;
-		if(e1->f2<3) e1->f2+= 1;
-		if(e2->f2<3) e2->f2+= 1;
-		if(e3->f2<3) e3->f2+= 1;
-		if(e4 && e4->f2<3) e4->f2+= 1;
+		if(e1->f2<4) e1->f2+= 1;
+		if(e2->f2<4) e2->f2+= 1;
+		if(e3->f2<4) e3->f2+= 1;
+		if(e4 && e4->f2<4) e4->f2+= 1;
 		
 		if(e1->vn==0) e1->vn= (EditVert *)efa;
 		if(e2->vn==0) e2->vn= (EditVert *)efa;
@@ -603,10 +675,15 @@ static void edge_drawflags(void)
 		efa= em->faces.first;
 		while(efa) {
 			if(efa->e1->f2==2) edge_normal_compare(efa->e1, efa);
+			else efa->e1->f2= 1;
 			if(efa->e2->f2==2) edge_normal_compare(efa->e2, efa);
+			else efa->e2->f2= 1;
 			if(efa->e3->f2==2) edge_normal_compare(efa->e3, efa);
-			if(efa->e4 && efa->e4->f2==2) edge_normal_compare(efa->e4, efa);
-			
+			else efa->e3->f2= 1;
+			if(efa->e4) {
+				if(efa->e4->f2==2) edge_normal_compare(efa->e4, efa);
+				else efa->e4->f2= 1;
+			}
 			efa= efa->next;
 		}
 		
@@ -626,19 +703,16 @@ static void edge_drawflags(void)
 /* turns Mesh into editmesh */
 void make_editMesh()
 {
-	EditMesh *em = G.editMesh;
 	Mesh *me= G.obedit->data;
 	MFace *mface;
 	TFace *tface;
 	MVert *mvert;
-	KeyBlock *actkey=0;
+	KeyBlock *actkey;
 	EditVert *eve, **evlist, *eve1, *eve2, *eve3, *eve4;
 	EditFace *efa;
 	EditEdge *eed;
 	int tot, a;
 
-	if(G.obedit==NULL) return;
-
 	/* because of reload */
 	free_editMesh(G.editMesh);
 	
@@ -654,16 +728,9 @@ void make_editMesh()
 	/* initialize fastmalloc for editmesh */
 	init_editmesh_fastmalloc(G.editMesh, me->totvert, me->totedge, me->totface);
 
-	/* keys? */
-	if(me->key) {
-		actkey= me->key->block.first;
-		while(actkey) {
-			if(actkey->flag & SELECT) break;
-			actkey= actkey->next;
-		}
-	}
-
+	actkey = ob_get_keyblock(G.obedit);
 	if(actkey) {
+		strcpy(G.editModeTitleExtra, "(Key) ");
 		key_to_mesh(actkey, me);
 		tot= actkey->totelem;
 	}
@@ -700,25 +767,23 @@ void make_editMesh()
 		}
 
 	}
-
+	
 	if(actkey && actkey->totelem!=me->totvert);
 	else {
 		unsigned int *mcol;
+		MEdge *medge= me->medge;
 		
 		/* make edges */
-		if(me->medge) {
-			MEdge *medge= me->medge;
+		for(a=0; a<me->totedge; a++, medge++) {
+			eed= addedgelist(evlist[medge->v1], evlist[medge->v2], NULL);
+			eed->crease= ((float)medge->crease)/255.0;
 			
-			for(a=0; a<me->totedge; a++, medge++) {
-				eed= addedgelist(evlist[medge->v1], evlist[medge->v2], NULL);
-				eed->crease= ((float)medge->crease)/255.0;
-				
-				if(medge->flag & ME_SEAM) eed->seam= 1;
-				if(medge->flag & SELECT) eed->f |= SELECT;
-				if(medge->flag & ME_FGON) eed->h= EM_FGON;	// 2 different defines!
-				if(medge->flag & ME_HIDE) eed->h |= 1;
-			}
-
+			if(medge->flag & ME_SEAM) eed->seam= 1;
+			if(medge->flag & SELECT) eed->f |= SELECT;
+			if(medge->flag & ME_FGON) eed->h= EM_FGON;	// 2 different defines!
+			if(medge->flag & ME_HIDE) eed->h |= 1;
+			if(G.scene->selectmode==SCE_SELECT_EDGE) 
+				EM_select_edge(eed, eed->f & SELECT);		// force edge selection to vertices, seems to be needed ...
 		}
 		
 		/* make faces */
@@ -729,7 +794,8 @@ void make_editMesh()
 		for(a=0; a<me->totface; a++, mface++) {
 			eve1= evlist[mface->v1];
 			eve2= evlist[mface->v2];
-			if(mface->v3) eve3= evlist[mface->v3]; else eve3= NULL;
+			if(!mface->v3) error("Eeekadoodle! An MFACE-EDGE! Tell Zr!!!!");
+			eve3= evlist[mface->v3];
 			if(mface->v4) eve4= evlist[mface->v4]; else eve4= NULL;
 			
 			efa= addfacelist(eve1, eve2, eve3, eve4, NULL, NULL);
@@ -740,12 +806,6 @@ void make_editMesh()
 
 				if(me->tface) {
 					efa->tf= *tface;
-
-					if( tface->flag & TF_SELECT) {
-						if(G.f & G_FACESELECT) {
-							EM_select_face(efa, 1);
-						}
-					}
 				}
 			
 				efa->mat_nr= mface->mat_nr;
@@ -755,11 +815,16 @@ void make_editMesh()
 					/* select face flag, if no edges we flush down */
 					if(mface->flag & ME_FACE_SEL) {
 						efa->f |= SELECT;
-						if(me->medge==NULL) EM_select_face(efa, 1);
+					}
+					if(mface->flag & ME_HIDE) efa->h= 1;
+				}
+				else if (tface) {
+					if( tface->flag & TF_HIDE) 
+						efa->h= 1;
+					else if( tface->flag & TF_SELECT) {
+						EM_select_face(efa, 1);
 					}
 				}
-				if(mface->flag & ME_HIDE) efa->h= 1;
-				
 			}
 
 			if(me->tface) tface++;
@@ -767,18 +832,10 @@ void make_editMesh()
 		}
 	}
 	
-	/* flush hide flags when no medge */
-	if(me->medge==NULL) {
-		for(eed= em->edges.first; eed; eed= eed->next) {
-			if(eed->v1->h || eed->v2->h) eed->h |= 1;
-			else eed->h &= ~1;
-		}	
-	}
-	
 	MEM_freeN(evlist);
 
 	end_editmesh_fastmalloc();	// resets global function pointers
-
+	
 	/* this creates coherent selections. also needed for older files */
 	EM_selectmode_set();
 	/* paranoia check to enforce hide rules */
@@ -788,133 +845,24 @@ void make_editMesh()
 	
 	countall();
 	
-	if (mesh_uses_displist(me)) makeDispList(G.obedit);
-	
 	waitcursor(0);
 }
 
-/** Rotates MFace and UVFace vertices in case the last
-  * vertex index is = 0. 
-  * This function is a hack and may only be called in the
-  * conversion from EditMesh to Mesh data.
-  * This function is similar to test_index_mface in
-  * blenkernel/intern/mesh.c. 
-  * To not clutter the blenkernel code with more bad level
-  * calls/structures, this function resides here.
-  */
-
-static void fix_faceindices(MFace *mface, EditFace *efa, int nr)
-{
-	int a;
-	float tmpuv[2];
-	unsigned int tmpcol;
-
-	/* first test if the face is legal */
-
-	if(mface->v3 && mface->v3==mface->v4) {
-		mface->v4= 0;
-		nr--;
-	}
-	if(mface->v2 && mface->v2==mface->v3) {
-		mface->v3= mface->v4;
-		mface->v4= 0;
-		nr--;
-	}
-	if(mface->v1==mface->v2) {
-		mface->v2= mface->v3;
-		mface->v3= mface->v4;
-		mface->v4= 0;
-		nr--;
-	}
-
-	/* prevent a zero index value at the wrong location */
-	if(nr==2) {
-		if(mface->v2==0) SWAP(int, mface->v1, mface->v2);
-	}
-	else if(nr==3) {
-		if(mface->v3==0) {
-			SWAP(int, mface->v1, mface->v2);
-			SWAP(int, mface->v2, mface->v3);
-			/* rotate face UV coordinates, too */
-			UVCOPY(tmpuv, efa->tf.uv[0]);
-			UVCOPY(efa->tf.uv[0], efa->tf.uv[1]);
-			UVCOPY(efa->tf.uv[1], efa->tf.uv[2]);
-			UVCOPY(efa->tf.uv[2], tmpuv);
-			/* same with vertex colours */
-			tmpcol = efa->tf.col[0];
-			efa->tf.col[0] = efa->tf.col[1];
-			efa->tf.col[1] = efa->tf.col[2];
-			efa->tf.col[2] = tmpcol;
-
-			
-			a= mface->edcode;
-			mface->edcode= 0;
-			if(a & ME_V1V2) mface->edcode |= ME_V3V1;
-			if(a & ME_V2V3) mface->edcode |= ME_V1V2;
-			if(a & ME_V3V1) mface->edcode |= ME_V2V3;
-			
-			a= mface->puno;
-			mface->puno &= ~15;
-			if(a & ME_FLIPV1) mface->puno |= ME_FLIPV2;
-			if(a & ME_FLIPV2) mface->puno |= ME_FLIPV3;
-			if(a & ME_FLIPV3) mface->puno |= ME_FLIPV1;
-
-		}
-	}
-	else if(nr==4) {
-		if(mface->v3==0 || mface->v4==0) {
-			SWAP(int, mface->v1, mface->v3);
-			SWAP(int, mface->v2, mface->v4);
-			/* swap UV coordinates */
-			UVCOPY(tmpuv, efa->tf.uv[0]);
-			UVCOPY(efa->tf.uv[0], efa->tf.uv[2]);
-			UVCOPY(efa->tf.uv[2], tmpuv);
-			UVCOPY(tmpuv, efa->tf.uv[1]);
-			UVCOPY(efa->tf.uv[1], efa->tf.uv[3]);
-			UVCOPY(efa->tf.uv[3], tmpuv);
-			/* swap vertex colours */
-			tmpcol = efa->tf.col[0];
-			efa->tf.col[0] = efa->tf.col[2];
-			efa->tf.col[2] = tmpcol;
-			tmpcol = efa->tf.col[1];
-			efa->tf.col[1] = efa->tf.col[3];
-			efa->tf.col[3] = tmpcol;
-
-			a= mface->edcode;
-			mface->edcode= 0;
-			if(a & ME_V1V2) mface->edcode |= ME_V3V4;
-			if(a & ME_V2V3) mface->edcode |= ME_V2V3;
-			if(a & ME_V3V4) mface->edcode |= ME_V1V2;
-			if(a & ME_V4V1) mface->edcode |= ME_V4V1;
-
-			a= mface->puno;
-			mface->puno &= ~15;
-			if(a & ME_FLIPV1) mface->puno |= ME_FLIPV3;
-			if(a & ME_FLIPV2) mface->puno |= ME_FLIPV4;
-			if(a & ME_FLIPV3) mface->puno |= ME_FLIPV1;
-			if(a & ME_FLIPV4) mface->puno |= ME_FLIPV2;
-
-		}
-	}
-}
-
 /* makes Mesh out of editmesh */
 void load_editMesh(void)
 {
 	EditMesh *em = G.editMesh;
 	Mesh *me= G.obedit->data;
 	MVert *mvert, *oldverts;
-	MEdge *medge=NULL;
+	MEdge *medge;
 	MFace *mface;
 	MSticky *ms;
-	KeyBlock *actkey=NULL, *currkey;
 	EditVert *eve;
 	EditFace *efa;
 	EditEdge *eed;
 	float *fp, *newkey, *oldkey, nor[3];
 	int i, a, ototvert, totedge=0;
 	MDeformVert *dvert;
-	int	usedDvert = 0;
 
 	waitcursor(1);
 	countall();
@@ -926,13 +874,9 @@ void load_editMesh(void)
 	/* eve->f2 : being used in vertexnormals */
 	edge_drawflags();
 	
-	/* this sets efa->puno, punoflag (for vertex normal & projection) */
-	vertexnormals( (me->flag & ME_NOPUNOFLIP)==0 );
-		
 	eed= em->edges.first;
 	while(eed) {
 		totedge++;
-		if(me->medge==NULL && (eed->f2==0)) G.totface++;
 		eed= eed->next;
 	}
 	
@@ -941,17 +885,16 @@ void load_editMesh(void)
 	else mvert= MEM_callocN(G.totvert*sizeof(MVert), "loadeditMesh vert");
 
 	/* new Edge block */
-	if(totedge) {
-		if(me->medge==NULL) totedge= 0;	// if edges get added is defined by orig mesh
-		else medge= MEM_callocN(totedge*sizeof(MEdge), "loadeditMesh edge");
-	}
+	if(totedge==0) medge= NULL;
+	else medge= MEM_callocN(totedge*sizeof(MEdge), "loadeditMesh edge");
 	
 	/* new Face block */
 	if(G.totface==0) mface= NULL;
 	else mface= MEM_callocN(G.totface*sizeof(MFace), "loadeditMesh face");
 	
-
+	/* are we adding dverts? */
 	if (G.totvert==0) dvert= NULL;
+	else if(G.obedit->defbase.first==NULL) dvert= NULL;
 	else dvert = MEM_callocN(G.totvert*sizeof(MDeformVert), "loadeditMesh3");
 
 	if (me->dvert) free_dverts(me->dvert, me->totvert);
@@ -968,7 +911,7 @@ void load_editMesh(void)
 
 	if(me->medge) MEM_freeN(me->medge);
 	me->medge= medge;
-	if(medge) me->totedge= totedge; else me->totedge= 0;
+	me->totedge= totedge;
 	
 	if(me->mface) MEM_freeN(me->mface);
 	me->mface= mface;
@@ -987,7 +930,7 @@ void load_editMesh(void)
 		VecMulf(nor, 32767.0);
 		VECCOPY(mvert->no, nor);
 
-		/* NEW VERSION */
+		/* note: it used to remove me->dvert when it was not in use, cancelled that... annoying when you have a fresh vgroup */
 		if (dvert){
 			dvert->totweight=eve->totweight;
 			if (eve->dw){
@@ -995,7 +938,6 @@ void load_editMesh(void)
 										 "deformWeight");
 				memcpy (dvert->dw, eve->dw, 
 						sizeof(MDeformWeight)*eve->totweight);
-				usedDvert++;
 			}
 		}
 
@@ -1008,33 +950,26 @@ void load_editMesh(void)
 			
 		eve= eve->next;
 		mvert++;
-		dvert++;
-	}
-	
-	/* If we didn't actually need the dverts, get rid of them */
-	if (!usedDvert){
-		free_dverts(me->dvert, G.totvert);
-		me->dvert=NULL;
+		if(dvert) dvert++;
 	}
 
 	/* the edges */
-	if(medge) {
-		eed= em->edges.first;
-		while(eed) {
-			medge->v1= (unsigned int) eed->v1->vn;
-			medge->v2= (unsigned int) eed->v2->vn;
-			
-			medge->flag= eed->f & SELECT;
-			if(eed->f2<2) medge->flag |= ME_EDGEDRAW;
-			if(eed->seam) medge->flag |= ME_SEAM;
-			if(eed->h & EM_FGON) medge->flag |= ME_FGON;	// different defines yes
-			if(eed->h & 1) medge->flag |= ME_HIDE;
-			
-			medge->crease= (char)(255.0*eed->crease);
+	eed= em->edges.first;
+	while(eed) {
+		medge->v1= (unsigned int) eed->v1->vn;
+		medge->v2= (unsigned int) eed->v2->vn;
+		
+		medge->flag= (eed->f & SELECT) | ME_EDGERENDER;
+		if(eed->f2<2) medge->flag |= ME_EDGEDRAW;
+		if(eed->f2==0) medge->flag |= ME_LOOSEEDGE;
+		if(eed->seam) medge->flag |= ME_SEAM;
+		if(eed->h & EM_FGON) medge->flag |= ME_FGON;	// different defines yes
+		if(eed->h & 1) medge->flag |= ME_HIDE;
+		
+		medge->crease= (char)(255.0*eed->crease);
 
-			medge++;
-			eed= eed->next;
-		}
+		medge++;
+		eed= eed->next;
 	}
 
 	/* the faces */
@@ -1049,7 +984,6 @@ void load_editMesh(void)
 		if(efa->v4) mface->v4= (unsigned int) efa->v4->vn;
 			
 		mface->mat_nr= efa->mat_nr;
-		mface->puno= efa->puno;
 		
 		mface->flag= efa->flag;
 		/* bit 0 of flag is already taken for smooth... */
@@ -1074,54 +1008,26 @@ void load_editMesh(void)
 		/* watch: efa->e1->f2==0 means loose edge */ 
 			
 		if(efa->e1->f2==1) {
-			mface->edcode |= ME_V1V2; 
 			efa->e1->f2= 2;
 		}			
 		if(efa->e2->f2==1) {
-			mface->edcode |= ME_V2V3; 
 			efa->e2->f2= 2;
 		}
 		if(efa->e3->f2==1) {
-			if(efa->v4) {
-				mface->edcode |= ME_V3V4;
-			}
-			else {
-				mface->edcode |= ME_V3V1;
-			}
 			efa->e3->f2= 2;
 		}
 		if(efa->e4 && efa->e4->f2==1) {
-			mface->edcode |= ME_V4V1; 
 			efa->e4->f2= 2;
 		}
 
 
 		/* no index '0' at location 3 or 4 */
-		if(efa->v4) fix_faceindices(mface, efa, 4);
-		else fix_faceindices(mface, efa, 3);
+		test_index_face(mface, NULL, &efa->tf, efa->v4?4:3);
 			
 		i++;
 		efa= efa->next;
 	}
-		
-	/* add loose edges as a face */
-	if(medge==NULL) {
-		eed= em->edges.first;
-		while(eed) {
-			if( eed->f2==0 ) {
-				mface= &((MFace *) me->mface)[i];
-				mface->v1= (unsigned int) eed->v1->vn;
-				mface->v2= (unsigned int) eed->v2->vn;
-				test_index_mface(mface, 2);
-				mface->edcode= ME_V1V2;
-				i++;
-			}
-			eed= eed->next;
-		}
-	}
 	
-	tex_space_mesh(me);
-
 	/* tface block */
 	if( me->tface && me->totface ) {
 		TFace *tfn, *tf;
@@ -1133,6 +1039,8 @@ void load_editMesh(void)
 			*tf= efa->tf;
 				
 			if(G.f & G_FACESELECT) {
+				if( efa->h) tf->flag |= TF_HIDE;
+				else tf->flag &= ~TF_HIDE;
 				if( efa->f & SELECT)  tf->flag |= TF_SELECT;
 				else tf->flag &= ~TF_SELECT;
 			}
@@ -1169,16 +1077,51 @@ void load_editMesh(void)
 		me->mcol= 0;
 	}
 
+	/* patch hook indices */
+	{
+		Object *ob;
+		ModifierData *md;
+		EditVert *eve, **vertMap = NULL;
+		int i,j;
+
+		for (ob=G.main->object.first; ob; ob=ob->id.next) {
+			if (ob->data==me) {
+				for (md=ob->modifiers.first; md; md=md->next) {
+					if (md->type==eModifierType_Hook) {
+						HookModifierData *hmd = (HookModifierData*) md;
+
+						if (!vertMap) {
+							vertMap = MEM_callocN(sizeof(*vertMap)*ototvert, "vertMap");
+
+							for (eve=em->verts.first; eve; eve=eve->next) {
+								if (eve->keyindex!=-1)
+									vertMap[eve->keyindex] = eve;
+							}
+						}
+						
+						for (i=j=0; i<hmd->totindex; i++) {
+							if(hmd->indexar[i] < ototvert) {
+								eve = vertMap[hmd->indexar[i]];
+								
+								if (eve) {
+									hmd->indexar[j++] = (long) eve->vn;
+								}
+							}
+							else j++;
+						}
+
+						hmd->totindex = j;
+					}
+				}
+			}
+		}
+
+		if (vertMap) MEM_freeN(vertMap);
+	}
 
 	/* are there keys? */
 	if(me->key) {
-
-		/* find the active key */
-		actkey= me->key->block.first;
-		while(actkey) {
-			if(actkey->flag & SELECT) break;
-			actkey= actkey->next;
-		}
+		KeyBlock *currkey, *actkey = ob_get_keyblock(G.obedit);
 
 		/* Lets reorder the key data so that things line up roughly
 		 * with the way things were before editmode */
@@ -1225,22 +1168,27 @@ void load_editMesh(void)
 			
 			currkey= currkey->next;
 		}
-
 	}
 
 	if(oldverts) MEM_freeN(oldverts);
-
-	if(actkey) do_spec_key(me->key);
 	
-	/* te be sure: clear ->vn pointers */
+	/* to be sure: clear ->vn pointers */
 	eve= em->verts.first;
 	while(eve) {
 		eve->vn= 0;
 		eve= eve->next;
 	}
-
-	/* we do make displist here for dependencies (like particles) */
-	if (mesh_uses_displist(me)) makeDispList(G.obedit);
+	
+	/* remake softbody of all users */
+	if(me->id.us>1) {
+		Base *base;
+		for(base= G.scene->base.first; base; base= base->next) {
+			if(base->object->data==me) {
+				base->object->softflag |= OB_SB_REDO;
+				base->object->recalc |= OB_RECALC_DATA;
+			}
+		}
+	}
 	
 	/* sticky */
 	if(me->msticky) {
@@ -1252,6 +1200,9 @@ void load_editMesh(void)
 			error("Sticky was too small");
 		}
 	}
+
+	mesh_calc_normals(me->mvert, me->totvert, me->mface, me->totface, NULL);
+
 	waitcursor(0);
 }
 
@@ -1259,7 +1210,7 @@ void remake_editMesh(void)
 {
 	make_editMesh();
 	allqueue(REDRAWVIEW3D, 0);
-	makeDispList(G.obedit);
+	DAG_object_flush_update(G.scene, G.obedit, OB_RECALC_DATA);
 	BIF_undo_push("Undo all changes");
 }
 
@@ -1272,7 +1223,7 @@ void separatemenu(void)
 
 	if(G.editMesh->verts.first==NULL) return;
 	   
-	event = pupmenu("Separate (No undo!) %t|Selected%x1|All Loose Parts%x2");
+	event = pupmenu("Separate %t|Selected%x1|All Loose Parts%x2");
 	
 	if (event==0) return;
 	waitcursor(1);
@@ -1300,7 +1251,6 @@ void separate_mesh(void)
 	Mesh *me, *men;
 	Base *base, *oldbase;
 	ListBase edve, eded, edvl;
-	float trans[9];
 	
 	TEST_EDITMESH	
 
@@ -1374,11 +1324,7 @@ void separate_mesh(void)
 	oldob= G.obedit;
 	oldbase= BASACT;
 	
-	trans[0]=trans[1]=trans[2]=trans[3]=trans[4]=trans[5]= 0.0;
-	trans[6]=trans[7]=trans[8]= 1.0;
-	G.qual |= LR_ALTKEY;	/* patch to make sure we get a linked duplicate */
-	adduplicate(trans);
-	G.qual &= ~LR_ALTKEY;
+	adduplicate(1, 0); /* notrans and a linked duplicate*/
 	
 	G.obedit= BASACT->object;	/* basact was set in adduplicate()  */
 
@@ -1396,6 +1342,7 @@ void separate_mesh(void)
 	emcopy.allverts= NULL;
 	emcopy.alledges= NULL;
 	emcopy.allfaces= NULL;
+	emcopy.derivedFinal= emcopy.derivedCage= NULL;
 	free_editMesh(&emcopy);
 	
 	em->verts= edve;
@@ -1405,6 +1352,7 @@ void separate_mesh(void)
 	/* hashedges are freed now, make new! */
 	editMesh_set_hash();
 	
+	DAG_object_flush_update(G.scene, G.obedit, OB_RECALC_DATA);	
 	G.obedit= oldob;
 	BASACT= oldbase;
 	BASACT->flag |= SELECT;
@@ -1413,7 +1361,7 @@ void separate_mesh(void)
 
 	countall();
 	allqueue(REDRAWVIEW3D, 0);
-	makeDispList(G.obedit);
+	DAG_object_flush_update(G.scene, G.obedit, OB_RECALC_DATA);	
 
 }
 
@@ -1428,7 +1376,6 @@ void separate_mesh_loose(void)
 	Mesh *me, *men;
 	Base *base, *oldbase;
 	ListBase edve, eded, edvl;
-	float trans[9];
 	int vertsep=0;	
 	short done=0, check=1;
 		
@@ -1542,11 +1489,7 @@ void separate_mesh_loose(void)
 			oldob= G.obedit;
 			oldbase= BASACT;
 			
-			trans[0]=trans[1]=trans[2]=trans[3]=trans[4]=trans[5]= 0.0;
-			trans[6]=trans[7]=trans[8]= 1.0;
-			G.qual |= LR_ALTKEY;	/* patch to make sure we get a linked duplicate */
-			adduplicate(trans);
-			G.qual &= ~LR_ALTKEY;
+			adduplicate(1, 0); /* notrans and 0 for linked duplicate */
 			
 			G.obedit= BASACT->object;	/* basact was set in adduplicate()  */
 		
@@ -1564,6 +1507,7 @@ void separate_mesh_loose(void)
 			emcopy.allverts= NULL;
 			emcopy.alledges= NULL;
 			emcopy.allfaces= NULL;
+			emcopy.derivedFinal= emcopy.derivedCage= NULL;
 			free_editMesh(&emcopy);
 			
 			em->verts= edve;
@@ -1586,7 +1530,7 @@ void separate_mesh_loose(void)
 	waitcursor(0);
 	countall();
 	allqueue(REDRAWVIEW3D, 0);
-	makeDispList(G.obedit);	
+	DAG_object_flush_update(G.scene, G.obedit, OB_RECALC_DATA);	
 }
 
 /* ******************************************** */
@@ -1618,7 +1562,7 @@ typedef struct EditEdgeC
 typedef struct EditFaceC
 {
 	int v1, v2, v3, v4;
-	unsigned char mat_nr, flag, f, h, puno, fgonf;
+	unsigned char mat_nr, flag, f, h, fgonf;
 	short pad1;
 } EditFaceC;
 
@@ -1676,7 +1620,7 @@ static void *editMesh_to_undoMesh(void)
 	if(um->totedge) eedc= um->edges= MEM_callocN(um->totedge*sizeof(EditEdgeC), "alledgesC");
 	if(um->totface) efac= um->faces= MEM_callocN(um->totface*sizeof(EditFaceC), "allfacesC");
 
-	if(me->tface) tface= um->tfaces= MEM_mallocN(um->totface*sizeof(TFace), "all tfacesC");
+	if(me->tface || me->mcol) tface= um->tfaces= MEM_mallocN(um->totface*sizeof(TFace), "all tfacesC");
 
 		//printf("copy editmesh %d\n", um->totvert*sizeof(EditVert) + um->totedge*sizeof(EditEdge) + um->totface*sizeof(EditFace));
 		//printf("copy undomesh %d\n", um->totvert*sizeof(EditVertC) + um->totedge*sizeof(EditEdgeC) + um->totface*sizeof(EditFaceC));
@@ -1718,7 +1662,6 @@ static void *editMesh_to_undoMesh(void)
 		efac->flag= efa->flag;
 		efac->f= efa->f;
 		efac->h= efa->h;
-		efac->puno= efa->puno;
 		efac->fgonf= efa->fgonf;
 		
 		if(tface) {
@@ -1787,7 +1730,6 @@ static void undoMesh_to_editMesh(void *umv)
 		efa->flag= efac->flag;
 		efa->f= efac->f;
 		efa->h= efac->h;
-		efa->puno= efac->puno;
 		efa->fgonf= efac->fgonf;
 		
 		if(tface) {
@@ -1811,4 +1753,60 @@ void undo_push_mesh(char *name)
 
 /* *************** END UNDO *************/
 
+static EditVert **g_em_vert_array = NULL;
+static EditEdge **g_em_edge_array = NULL;
+static EditFace **g_em_face_array = NULL;
 
+void EM_init_index_arrays(int forVert, int forEdge, int forFace)
+{
+	EditVert *eve;
+	EditEdge *eed;
+	EditFace *efa;
+	int i;
+
+	if (forVert) {
+		g_em_vert_array = MEM_mallocN(sizeof(*g_em_vert_array)*G.totvert, "em_v_arr");
+
+		for (i=0,eve=G.editMesh->verts.first; eve; i++,eve=eve->next)
+			g_em_vert_array[i] = eve;
+	}
+
+	if (forEdge) {
+		g_em_edge_array = MEM_mallocN(sizeof(*g_em_edge_array)*G.totedge, "em_e_arr");
+
+		for (i=0,eed=G.editMesh->edges.first; eed; i++,eed=eed->next)
+			g_em_edge_array[i] = eed;
+	}
+
+	if (forFace) {
+		g_em_face_array = MEM_mallocN(sizeof(*g_em_face_array)*G.totface, "em_f_arr");
+
+		for (i=0,efa=G.editMesh->faces.first; efa; i++,efa=efa->next)
+			g_em_face_array[i] = efa;
+	}
+}
+
+void EM_free_index_arrays(void)
+{
+	if (g_em_vert_array) MEM_freeN(g_em_vert_array);
+	if (g_em_edge_array) MEM_freeN(g_em_edge_array);
+	if (g_em_face_array) MEM_freeN(g_em_face_array);
+	g_em_vert_array = NULL;
+	g_em_edge_array = NULL;
+	g_em_face_array = NULL;
+}
+
+EditVert *EM_get_vert_for_index(int index)
+{
+	return g_em_vert_array?g_em_vert_array[index]:NULL;
+}
+
+EditEdge *EM_get_edge_for_index(int index)
+{
+	return g_em_edge_array?g_em_edge_array[index]:NULL;
+}
+
+EditFace *EM_get_face_for_index(int index)
+{
+	return g_em_face_array?g_em_face_array[index]:NULL;
+}
