@@ -137,31 +137,23 @@ void EM_select_face_fgon(EditFace *efa, int val)
 /* only vertices */
 int faceselectedOR(EditFace *efa, int flag)
 {
-	
-	if(efa->v1->f & flag) return 1;
-	if(efa->v2->f & flag) return 1;
-	if(efa->v3->f & flag) return 1;
-	if(efa->v4 && (efa->v4->f & 1)) return 1;
-	return 0;
+	if ((efa->v1->f | efa->v2->f | efa->v3->f | (efa->v4?efa->v4->f:0))&flag) {
+		return 1;
+	} else {
+		return 0;
+	}
 }
 
 // replace with (efa->f & SELECT)
 int faceselectedAND(EditFace *efa, int flag)
 {
-	if(efa->v1->f & flag) {
-		if(efa->v2->f & flag) {
-			if(efa->v3->f & flag) {
-				if(efa->v4) {
-					if(efa->v4->f & flag) return 1;
-				}
-				else return 1;
-			}
-		}
+	if ((efa->v1->f & efa->v2->f & efa->v3->f & (efa->v4?efa->v4->f:flag))&flag) {
+		return 1;
+	} else {
+		return 0;
 	}
-	return 0;
 }
 
-
 int EM_nfaces_selected(void)
 {
 	EditMesh *em = G.editMesh;
@@ -409,14 +401,14 @@ void EM_selectmode_set(void)
 	EditEdge *eed;
 	EditFace *efa;
 
-	if(G.scene->selectmode & SCE_SELECT_VERTEX) {
+	if(G.scene->selectmode == SCE_SELECT_VERTEX) {
 		/* vertices -> edges -> faces */
 		for (eed= em->edges.first; eed; eed= eed->next) eed->f &= ~SELECT;
 		for (efa= em->faces.first; efa; efa= efa->next) efa->f &= ~SELECT;
 		
 		EM_select_flush();
 	}
-	else if(G.scene->selectmode & SCE_SELECT_EDGE) {
+	else if(G.scene->selectmode == SCE_SELECT_EDGE) {
 		/* deselect vertices, and select again based on edge select */
 		for(eve= em->verts.first; eve; eve= eve->next) eve->f &= ~SELECT;
 		for(eed= em->edges.first; eed; eed= eed->next) 
@@ -733,6 +725,12 @@ static short extrudeflag_edge(short flag, float *nor)
 			efa->e2->f2++;
 			efa->e3->f2++;
 			if(efa->e4) efa->e4->f2++;
+			
+			// sample for next loop
+			efa->e1->vn= (EditVert *)efa;
+			efa->e2->vn= (EditVert *)efa;
+			efa->e3->vn= (EditVert *)efa;
+			if(efa->e4) efa->e4->vn= (EditVert *)efa;
 		}
 		else {
 			efa->e1->f1++;
@@ -740,11 +738,6 @@ static short extrudeflag_edge(short flag, float *nor)
 			efa->e3->f1++;
 			if(efa->e4) efa->e4->f1++;
 		}
-		// sample for next loop
-		efa->e1->vn= (EditVert *)efa;
-		efa->e2->vn= (EditVert *)efa;
-		efa->e3->vn= (EditVert *)efa;
-		if(efa->e4) efa->e4->vn= (EditVert *)efa;
 	}
 	
 	set_edge_directions_f2(2);
@@ -878,7 +871,7 @@ short extrudeflag_vert(short flag, float *nor)
 	EditVert *eve, *v1, *v2, *v3, *v4, *nextve;
 	EditEdge *eed, *e1, *e2, *e3, *e4, *nexted;
 	EditFace *efa, *efa2, *nextvl;
-	short sel=0, del_old= 0;
+	short sel=0, del_old= 0, is_face_sel=0;
 
 	if(G.obedit==0 || get_mesh(G.obedit)==0) return 0;
 
@@ -923,6 +916,7 @@ short extrudeflag_vert(short flag, float *nor)
 			if(e4 && e4->f2 < 3) e4->f2++;
 			
 			efa->f1= 1;
+			is_face_sel= 1;	// for del_old
 		}
 		else if(faceselectedOR(efa, flag)) {
 			e1= efa->e1;
@@ -992,11 +986,13 @@ short extrudeflag_vert(short flag, float *nor)
 	*/
 	
 	 /* find if we delete old faces */
-	for(eed= em->edges.first; eed; eed= eed->next) {
-		if( (eed->f2==1 || eed->f2==2) ) {
-			if(eed->f1==2) {
-				del_old= 1;
-				break;
+	if(is_face_sel) {
+		for(eed= em->edges.first; eed; eed= eed->next) {
+			if( (eed->f2==1 || eed->f2==2) ) {
+				if(eed->f1==2) {
+					del_old= 1;
+					break;
+				}
 			}
 		}
 	}
@@ -1009,7 +1005,7 @@ short extrudeflag_vert(short flag, float *nor)
 			eed->v2->f |= 128;
 		}
 		if( (eed->f2==1 || eed->f2==2) ) {
-			
+	
 			/* if del_old, the preferred normal direction is exact opposite as for keep old faces */
 			if(eed->dir!=del_old) efa2= addfacelist(eed->v1, eed->v2, eed->v2->vn, eed->v1->vn, NULL, NULL);
 			else efa2= addfacelist(eed->v2, eed->v1, eed->v1->vn, eed->v2->vn, NULL, NULL);
@@ -1292,7 +1288,7 @@ void delfaceflag(int flag)
 }
 
 /* ********************* */
-
+#if 0
 static int check_vnormal_flip(float *n, float *vnorm) 
 {
 	float inp;
@@ -1304,181 +1300,7 @@ static int check_vnormal_flip(float *n, float *vnorm)
 
 	return 1;
 }
-
-
-void vertexnormals(int testflip)
-{
-	EditMesh *em = G.editMesh;
-	Mesh *me;
-	EditVert *eve;
-	EditFace *efa;	
-	float n1[3], n2[3], n3[3], n4[3], co[4], fac1, fac2, fac3, fac4, *temp;
-	float *f1, *f2, *f3, *f4, xn, yn, zn;
-	float len, area;
-	
-	if(G.obedit && G.obedit->type==OB_MESH) {
-		me= G.obedit->data;
-		if((me->flag & ME_TWOSIDED)==0) testflip= 0;
-	}
-
-	if(G.totvert==0) return;
-
-	if(G.totface==0) {
-		/* fake vertex normals for 'halo puno'! */
-		eve= em->verts.first;
-		while(eve) {
-			VECCOPY(eve->no, eve->co);
-			Normalise( (float *)eve->no);
-			eve= eve->next;
-		}
-		return;
-	}
-
-	/* clear normals, clear flag */	
-	eve= em->verts.first;
-	while(eve) {
-		eve->no[0]= eve->no[1]= eve->no[2]= 0.0;
-		eve->f2= 0;
-		eve= eve->next;
-	}
-	
-	/* check for vertices being shared by both solid and smooth face, 
-	   these get vertexnormal of smooth face normal only */
-	for(efa= em->faces.first; efa; efa= efa->next) {
-		if(efa->flag & ME_SMOOTH) {
-			efa->v1->f2 |= 1; efa->v2->f2 |= 1; efa->v3->f2 |= 1;
-			if(efa->v4) efa->v4->f2 |= 1;
-		}
-		else {
-			efa->v1->f2 |= 2; efa->v2->f2 |= 2; efa->v3->f2 |= 2;
-			if(efa->v4) efa->v4->f2 |= 2;
-		}
-	}
-	
-	/* calculate cosine angles and add to vertex normal */
-	for(efa= em->faces.first; efa; efa= efa->next) {
-		VecSubf(n1, efa->v2->co, efa->v1->co);
-		VecSubf(n2, efa->v3->co, efa->v2->co);
-		Normalise(n1);
-		Normalise(n2);
-
-		if(efa->v4==0) {
-			VecSubf(n3, efa->v1->co, efa->v3->co);
-			Normalise(n3);
-			
-			//area= AreaT3Dfl(efa->v1->co, efa->v2->co, efa->v3->co);
-			//if(area!=0.0) area=1.0/area; 
-			//area= sqrt(area);
-			area= 1.0;
-			
-			co[0]= area*saacos(-n3[0]*n1[0]-n3[1]*n1[1]-n3[2]*n1[2]);
-			co[1]= area*saacos(-n1[0]*n2[0]-n1[1]*n2[1]-n1[2]*n2[2]);
-			co[2]= area*saacos(-n2[0]*n3[0]-n2[1]*n3[1]-n2[2]*n3[2]);
-			
-		}
-		else {
-			VecSubf(n3, efa->v4->co, efa->v3->co);
-			VecSubf(n4, efa->v1->co, efa->v4->co);
-			Normalise(n3);
-			Normalise(n4);
-			
-			//area= AreaQ3Dfl(efa->v1->co, efa->v2->co, efa->v3->co, efa->v4->co);
-			//if(area!=0.0) area=1.0/area; 
-			//area= sqrt(area);
-			area= 1.0;
-			
-			co[0]= area*saacos(-n4[0]*n1[0]-n4[1]*n1[1]-n4[2]*n1[2]);
-			co[1]= area*saacos(-n1[0]*n2[0]-n1[1]*n2[1]-n1[2]*n2[2]);
-			co[2]= area*saacos(-n2[0]*n3[0]-n2[1]*n3[1]-n2[2]*n3[2]);
-			co[3]= area*saacos(-n3[0]*n4[0]-n3[1]*n4[1]-n3[2]*n4[2]);
-		}
-		
-		if(efa->v1->f2!=3 || (efa->flag & ME_SMOOTH)) {
-			temp= efa->v1->no;
-			if(testflip && check_vnormal_flip(efa->n, temp) ) co[0]= -co[0];
-			temp[0]+= co[0]*efa->n[0];
-			temp[1]+= co[0]*efa->n[1];
-			temp[2]+= co[0]*efa->n[2];
-		}
-		
-		if(efa->v2->f2!=3 || (efa->flag & ME_SMOOTH)) {
-			temp= efa->v2->no;
-			if(testflip && check_vnormal_flip(efa->n, temp) ) co[1]= -co[1];
-			temp[0]+= co[1]*efa->n[0];
-			temp[1]+= co[1]*efa->n[1];
-			temp[2]+= co[1]*efa->n[2];
-		}
-		
-		if(efa->v3->f2!=3 || (efa->flag & ME_SMOOTH)) {
-			temp= efa->v3->no;
-			if(testflip && check_vnormal_flip(efa->n, temp) ) co[2]= -co[2];
-			temp[0]+= co[2]*efa->n[0];
-			temp[1]+= co[2]*efa->n[1];
-			temp[2]+= co[2]*efa->n[2];
-		}
-		
-		if(efa->v4 && (efa->v4->f2!=3 || (efa->flag & ME_SMOOTH))) {
-			temp= efa->v4->no;
-			if(testflip && check_vnormal_flip(efa->n, temp) ) co[3]= -co[3];
-			temp[0]+= co[3]*efa->n[0];
-			temp[1]+= co[3]*efa->n[1];
-			temp[2]+= co[3]*efa->n[2];
-		}
-	}
-	
-	/* normalise vertex normals */
-	eve= em->verts.first;
-	while(eve) {
-		len= Normalise(eve->no);
-		if(len==0.0) {
-			VECCOPY(eve->no, eve->co);
-			Normalise( eve->no);
-		}
-		eve= eve->next;
-	}
-	
-	/* vertex normal flip-flags for shade (render) */
-	efa= em->faces.first;
-	while(efa) {
-		efa->puno=0;			
-
-		if(testflip) {
-			f1= efa->v1->no;
-			f2= efa->v2->no;
-			f3= efa->v3->no;
-			
-			fac1= efa->n[0]*f1[0] + efa->n[1]*f1[1] + efa->n[2]*f1[2];
-			if(fac1<0.0) {
-				efa->puno = ME_FLIPV1;
-			}
-			fac2= efa->n[0]*f2[0] + efa->n[1]*f2[1] + efa->n[2]*f2[2];
-			if(fac2<0.0) {
-				efa->puno += ME_FLIPV2;
-			}
-			fac3= efa->n[0]*f3[0] + efa->n[1]*f3[1] + efa->n[2]*f3[2];
-			if(fac3<0.0) {
-				efa->puno += ME_FLIPV3;
-			}
-			if(efa->v4) {
-				f4= efa->v4->no;
-				fac4= efa->n[0]*f4[0] + efa->n[1]*f4[1] + efa->n[2]*f4[2];
-				if(fac4<0.0) {
-					efa->puno += ME_FLIPV4;
-				}
-			}
-		}
-		/* projection for cubemap! */
-		xn= fabs(efa->n[0]);
-		yn= fabs(efa->n[1]);
-		zn= fabs(efa->n[2]);
-		
-		if(zn>xn && zn>yn) efa->puno += ME_PROJXY;
-		else if(yn>xn && yn>zn) efa->puno += ME_PROJXZ;
-		else efa->puno += ME_PROJYZ;
-		
-		efa= efa->next;
-	}
-}
+#endif
 
 void flipface(EditFace *efa)
 {
@@ -1522,23 +1344,35 @@ void recalc_editnormals(void)
 {
 	EditMesh *em = G.editMesh;
 	EditFace *efa;
+	EditVert *eve;
 
-	efa= em->faces.first;
-	while(efa) {
+	for(eve= em->verts.first; eve; eve=eve->next) {
+		eve->no[0] = eve->no[1] = eve->no[2] = 0.0;
+	}
+
+	for(efa= em->faces.first; efa; efa=efa->next) {
 		if(efa->v4) {
 			CalcNormFloat4(efa->v1->co, efa->v2->co, efa->v3->co, efa->v4->co, efa->n);
 			CalcCent4f(efa->cent, efa->v1->co, efa->v2->co, efa->v3->co, efa->v4->co);
+			VecAddf(efa->v4->no, efa->v4->no, efa->n);
 		}
 		else {
 			CalcNormFloat(efa->v1->co, efa->v2->co, efa->v3->co, efa->n);
 			CalcCent3f(efa->cent, efa->v1->co, efa->v2->co, efa->v3->co);
 		}
-		efa= efa->next;
+		VecAddf(efa->v1->no, efa->v1->no, efa->n);
+		VecAddf(efa->v2->no, efa->v2->no, efa->n);
+		VecAddf(efa->v3->no, efa->v3->no, efa->n);
+	}
+
+	for(eve= em->verts.first; eve; eve=eve->next) {
+		if (Normalise(eve->no)==0.0) {
+			VECCOPY(eve->no, eve->co);
+			Normalise(eve->no);
+		}
 	}
 }
 
-
-
 int compareface(EditFace *vl1, EditFace *vl2)
 {
 	EditVert *v1, *v2, *v3, *v4;
@@ -1563,7 +1397,7 @@ int compareface(EditFace *vl1, EditFace *vl2)
 		v1= vl2->v1;
 		v2= vl2->v2;
 		v3= vl2->v3;
-
+		
 		if(vl1->v1==v1 || vl1->v2==v1 || vl1->v3==v1) {
 			if(vl1->v1==v2 || vl1->v2==v2 || vl1->v3==v2) {
 				if(vl1->v1==v3 || vl1->v2==v3 || vl1->v3==v3) {
@@ -1572,10 +1406,11 @@ int compareface(EditFace *vl1, EditFace *vl2)
 			}
 		}
 	}
-
+	
 	return 0;
 }
 
+/* checks for existance, not tria overlapping inside quad */
 EditFace *exist_face(EditVert *v1, EditVert *v2, EditVert *v3, EditVert *v4)
 {
 	EditMesh *em = G.editMesh;
