@@ -44,9 +44,6 @@ editmesh_mods.c, UI level access, no geometry changes
 #include <config.h>
 #endif
 
-#ifdef WIN32
-#include "BLI_winstuff.h"
-#endif
 #include "MEM_guardedalloc.h"
 
 #include "MTC_matrixops.h"
@@ -54,6 +51,7 @@ editmesh_mods.c, UI level access, no geometry changes
 #include "DNA_mesh_types.h"
 #include "DNA_material_types.h"
 #include "DNA_meshdata_types.h"
+#include "DNA_modifier_types.h"
 #include "DNA_object_types.h"
 #include "DNA_texture_types.h"
 #include "DNA_scene_types.h"
@@ -66,6 +64,8 @@ editmesh_mods.c, UI level access, no geometry changes
 #include "BLI_rand.h"
 
 #include "BKE_displist.h"
+#include "BKE_depsgraph.h"
+#include "BKE_DerivedMesh.h"
 #include "BKE_global.h"
 #include "BKE_mesh.h"
 #include "BKE_material.h"
@@ -78,6 +78,7 @@ editmesh_mods.c, UI level access, no geometry changes
 #include "BIF_glutil.h"
 #include "BIF_graphics.h"
 #include "BIF_interface.h"
+#include "BIF_meshtools.h"
 #include "BIF_mywindow.h"
 #include "BIF_resources.h"
 #include "BIF_screen.h"
@@ -95,14 +96,44 @@ editmesh_mods.c, UI level access, no geometry changes
 
 #include "mydevice.h"
 #include "blendef.h"
-#include "render.h"  // externtex, badlevel call (ton)
+#include "render.h"  // externtex
 
 #include "editmesh.h"
 
 
+/* ****************************** MIRROR **************** */
+
+static EditVert *get_x_mirror_vert(EditVert *eve)
+{
+	int index;
+	
+	index= mesh_get_x_mirror_vert(G.obedit, POINTER_TO_INT(eve));
+	if(index != -1)
+		return INT_TO_POINTER(index);
+	return NULL;
+}
+
+void EM_select_mirrored(void)
+{
+	if(G.scene->selectmode & SCE_SELECT_VERTEX) {
+		EditMesh *em = G.editMesh;
+		EditVert *eve, *v1;
+		
+		for(eve= em->verts.first; eve; eve= eve->next) {
+			if(eve->f & SELECT) {
+				v1= get_x_mirror_vert(eve);
+				if(v1) {
+					eve->f &= ~SELECT;
+					v1->f |= SELECT;
+				}
+			}
+		}
+	}
+}
+
 /* ****************************** SELECTION ROUTINES **************** */
 
-int em_solidoffs=0, em_wireoffs=0, em_vertoffs;	// set in drawobject.c ... for colorindices
+unsigned int em_solidoffs=0, em_wireoffs=0, em_vertoffs=0;	// set in drawobject.c ... for colorindices
 
 static void check_backbuf(void)
 {
@@ -195,13 +226,22 @@ static unsigned int *read_backbuf(short xmin, short ymin, short xmax, short ymax
 
 
 /* smart function to sample a rect spiralling outside, nice for backbuf selection */
-static unsigned int sample_backbuf_rect(unsigned int *buf, int size, int min, int max, short *dist)
+static unsigned int sample_backbuf_rect(short mval[2], int size, unsigned int min, unsigned int max, short *dist)
 {
-	unsigned int *bufmin, *bufmax;
+	unsigned int *buf, *bufmin, *bufmax;
+	int minx, miny;
 	int a, b, rc, nr, amount, dirvec[4][2];
 	short distance=0;
+	unsigned int index = 0;
 	
 	amount= (size-1)/2;
+
+	minx = mval[0]-(amount+1);
+	miny = mval[1]-(amount+1);
+	buf = read_backbuf(minx, miny, minx+size-1, miny+size-1);
+	if (!buf)
+		return 0;
+
 	rc= 0;
 	
 	dirvec[0][0]= 1; dirvec[0][1]= 0;
@@ -217,28 +257,77 @@ static unsigned int sample_backbuf_rect(unsigned int *buf, int size, int min, in
 		
 		for(a=0; a<2; a++) {
 			for(b=0; b<nr; b++, distance++) {
-				
-				if(*buf && *buf>=min && *buf<max ) {
-					*dist= (short) sqrt( (float)distance );
-					return *buf - min+1;	// messy yah, but indices start at 1
+				if (*buf && *buf>=min && *buf<max) {
+					*dist= (short) sqrt( (float)distance ); // XXX, this distance is wrong - zr
+					index = *buf - min+1; // messy yah, but indices start at 1
+					goto exit;
 				}
 				
 				buf+= (dirvec[rc][0]+dirvec[rc][1]);
 				
 				if(buf<bufmin || buf>=bufmax) {
-					return 0;
+					goto exit;
 				}
 			}
 			rc++;
 			rc &= 3;
 		}
 	}
-	return 0;
+
+exit:
+	MEM_freeN(bufmin);
+	return index;
 }
 
 /* facilities for border select and circle select */
 static char *selbuf= NULL;
 
+/* opengl doesn't support concave... */
+static void draw_triangulated(short mcords[][2], short tot)
+{
+	ListBase lb={NULL, NULL};
+	DispList *dl;
+	float *fp;
+	int a;
+	
+	/* make displist */
+	dl= MEM_callocN(sizeof(DispList), "poly disp");
+	dl->type= DL_POLY;
+	dl->parts= 1;
+	dl->nr= tot;
+	dl->verts= fp=  MEM_callocN(tot*3*sizeof(float), "poly verts");
+	BLI_addtail(&lb, dl);
+	
+	for(a=0; a<tot; a++, fp+=3) {
+		fp[0]= (float)mcords[a][0];
+		fp[1]= (float)mcords[a][1];
+	}
+	
+	/* do the fill */
+	filldisplist(&lb, &lb);
+
+	/* do the draw */
+	dl= lb.first;	// filldisplist adds in head of list
+	if(dl->type==DL_INDEX3) {
+		int *index;
+		
+		a= dl->parts;
+		fp= dl->verts;
+		index= dl->index;
+		glBegin(GL_TRIANGLES);
+		while(a--) {
+			glVertex3fv(fp+3*index[0]);
+			glVertex3fv(fp+3*index[1]);
+			glVertex3fv(fp+3*index[2]);
+			index+= 3;
+		}
+		glEnd();
+	}
+	
+	freedisplist(&lb);
+}
+
+
 /* reads rect, and builds selection array for quick lookup */
 /* returns if all is OK */
 int EM_init_backbuf_border(short xmin, short ymin, short xmax, short ymax)
@@ -257,14 +346,15 @@ int EM_init_backbuf_border(short xmin, short ymin, short xmax, short ymax)
 	
 	a= (xmax-xmin+1)*(ymax-ymin+1);
 	while(a--) {
-		if(*dr>0 && *dr<=em_vertoffs) selbuf[*dr]= 1;
+		if(*dr>0 && *dr<=em_vertoffs) 
+			selbuf[*dr]= 1;
 		dr++;
 	}
 	MEM_freeN(buf);
 	return 1;
 }
 
-int EM_check_backbuf_border(int index)
+int EM_check_backbuf(unsigned int index)
 {
 	if(selbuf==NULL) return 1;
 	if(index>0 && index<=em_vertoffs)
@@ -272,7 +362,7 @@ int EM_check_backbuf_border(int index)
 	return 0;
 }
 
-void EM_free_backbuf_border(void)
+void EM_free_backbuf(void)
 {
 	if(selbuf) MEM_freeN(selbuf);
 	selbuf= NULL;
@@ -309,9 +399,10 @@ int EM_mask_init_backbuf_border(short mcords[][2], short tot, short xmin, short
 	
 	persp(PERSP_WIN);
 	glColor3ub(0, 0, 0);
-	glBegin(GL_POLYGON);
-	for(a=0; a<tot; a++) glVertex2s(mcords[a][0], mcords[a][1]);
-	glEnd();
+	
+	/* yah, opengl doesn't do concave... tsk! */
+ 	draw_triangulated(mcords, tot);	
+	
 	glBegin(GL_LINE_LOOP);	// for zero sized masks, lines
 	for(a=0; a<tot; a++) glVertex2s(mcords[a][0], mcords[a][1]);
 	glEnd();
@@ -346,7 +437,12 @@ int EM_init_backbuf_circle(short xs, short ys, short rads)
 	short xmin, ymin, xmax, ymax, xc, yc;
 	int radsq;
 	
-	if(G.obedit==NULL || G.vd->drawtype<OB_SOLID || (G.vd->flag & V3D_ZBUF_SELECT)==0) return 0;
+	/* method in use for face selecting too */
+	if(G.obedit==NULL) {
+		if(G.f & G_FACESELECT);
+		else return 0;
+	}
+	else if(G.vd->drawtype<OB_SOLID || (G.vd->flag & V3D_ZBUF_SELECT)==0) return 0;
 	if(em_vertoffs==0) return 0;
 	
 	xmin= xs-rads; xmax= xs+rads;
@@ -370,271 +466,312 @@ int EM_init_backbuf_circle(short xs, short ys, short rads)
 	
 }
 
-
-static EditVert *findnearestvert_f(short *dist, short sel)
+static void findnearestvert__doClosest(void *userData, EditVert *eve, int x, int y, int index)
 {
-	static EditVert *acto= NULL;
-	EditMesh *em = G.editMesh;
-	/* if sel==1 the vertices with flag==1 get a disadvantage */
-	EditVert *eve,*act=NULL;
-	short temp, mval[2];
+	struct { short mval[2], pass, select, dist; int lastIndex, closestIndex; EditVert *closest; } *data = userData;
 
-	if(em->verts.first==NULL) return NULL;
-
-	/* do projection */
-	calc_meshverts_ext();	/* drawobject.c */
-	
-	/* we count from acto->next to last, and from first to acto */
-	/* does acto exist? */
-	eve= em->verts.first;
-	while(eve) {
-		if(eve==acto) break;
-		eve= eve->next;
+	if (data->pass==0) {
+		if (index<=data->lastIndex)
+			return;
+	} else {
+		if (index>data->lastIndex)
+			return;
 	}
-	if(eve==NULL) acto= em->verts.first;
 
-	/* is there an indicated vertex? part 1 */
-	getmouseco_areawin(mval);
-	eve= acto->next;
-	while(eve) {
-		if(eve->h==0 && eve->xs!=3200) {
-			temp= abs(mval[0]- eve->xs)+ abs(mval[1]- eve->ys);
-			if( (eve->f & 1)==sel ) temp+=5;
-			if(temp< *dist) {
-				act= eve;
-				*dist= temp;
-				if(*dist<4) break;
-			}
-		}
-		eve= eve->next;
-	}
-	/* is there an indicated vertex? part 2 */
-	if(*dist>3) {
-		eve= em->verts.first;
-		while(eve) {
-			if(eve->h==0 && eve->xs!=3200) {
-				temp= abs(mval[0]- eve->xs)+ abs(mval[1]- eve->ys);
-				if( (eve->f & 1)==sel ) temp+=5;
-				if(temp< *dist) {
-					act= eve;
-					if(temp<4) break;
-					*dist= temp;
-				}
-				if(eve== acto) break;
-			}
-			eve= eve->next;
+	if (data->dist>3) {
+		short temp = abs(data->mval[0] - x) + abs(data->mval[1]- y);
+		if ((eve->f&1)==data->select) temp += 5;
+
+		if (temp<data->dist) {
+			data->dist = temp;
+			data->closest = eve;
+			data->closestIndex = index;
 		}
 	}
-
-	acto= act;
-	return act;
 }
-
-/* backbuffer version */
-EditVert *findnearestvert(short *dist, short sel)
+static EditVert *findnearestvert(short *dist, short sel)
 {
+	short mval[2];
+
+	getmouseco_areawin(mval);
+		
 	if(G.vd->drawtype>OB_WIRE && (G.vd->flag & V3D_ZBUF_SELECT)) {
-		EditVert *eve=NULL;
-		unsigned int *buf;
-		int a=1, index;
-		short mval[2], distance=255;
+		short distance;
+		unsigned int index = sample_backbuf_rect(mval, 50, em_wireoffs, 0xFFFFFF, &distance);
+		EditVert *eve = BLI_findlink(&G.editMesh->verts, index-1);
+
+		if (eve && distance < *dist) {
+			*dist = distance;
+			return eve;
+		} else {
+			return NULL;
+		}
+	}
+	else {
+		struct { short mval[2], pass, select, dist; int lastIndex, closestIndex; EditVert *closest; } data;
+		static int lastSelectedIndex=0;
+		static EditVert *lastSelected=NULL;
+
+		if (lastSelected && BLI_findlink(&G.editMesh->verts, lastSelectedIndex)!=lastSelected) {
+			lastSelectedIndex = 0;
+			lastSelected = NULL;
+		}
+
+		data.lastIndex = lastSelectedIndex;
+		data.mval[0] = mval[0];
+		data.mval[1] = mval[1];
+		data.select = sel;
+		data.dist = *dist;
+		data.closest = NULL;
+		data.closestIndex = 0;
+
+		data.pass = 0;
+		mesh_foreachScreenVert(findnearestvert__doClosest, &data, 1);
+
+		if (data.dist>3) {
+			data.pass = 1;
+			mesh_foreachScreenVert(findnearestvert__doClosest, &data, 1);
+		}
+
+		*dist = data.dist;
+		lastSelected = data.closest;
+		lastSelectedIndex = data.closestIndex;
+
+		return data.closest;
+	}
+}
+
+/* returns labda for closest distance v1 to line-piece v2-v3 */
+static float labda_PdistVL2Dfl( float *v1, float *v2, float *v3) 
+{
+	float rc[2], len;
+	
+	rc[0]= v3[0]-v2[0];
+	rc[1]= v3[1]-v2[1];
+	len= rc[0]*rc[0]+ rc[1]*rc[1];
+	if(len==0.0f)
+		return 0.0f;
+	
+	return ( rc[0]*(v1[0]-v2[0]) + rc[1]*(v1[1]-v2[1]) )/len;
+}
+
+/* note; uses G.vd, so needs active 3d window */
+static void findnearestedge__doClosest(void *userData, EditEdge *eed, int x0, int y0, int x1, int y1, int index)
+{
+	struct { float mval[2]; short dist; EditEdge *closest; } *data = userData;
+	float v1[2], v2[2];
+	short distance;
 		
-		getmouseco_areawin(mval);
+	v1[0] = x0;
+	v1[1] = y0;
+	v2[0] = x1;
+	v2[1] = y1;
 		
-		// sample colorcode 
-		buf= read_backbuf(mval[0]-25, mval[1]-25, mval[0]+24, mval[1]+24);
-		if(buf) {
-			index= sample_backbuf_rect(buf, 50, em_wireoffs, 0xFFFFFF, &distance); // globals, set in drawobject.c
-			MEM_freeN(buf);
-			if(distance < *dist) {
-				if(index>0) for(eve= G.editMesh->verts.first; eve; eve= eve->next, a++) if(index==a) break;
-				if(eve) *dist= distance;
+	distance= PdistVL2Dfl(data->mval, v1, v2);
+		
+	if(eed->f & SELECT) distance+=5;
+	if(distance < data->dist) {
+		if(G.vd->flag & V3D_CLIPPING) {
+			float labda= labda_PdistVL2Dfl(data->mval, v1, v2);
+			float vec[3];
+
+			vec[0]= eed->v1->co[0] + labda*(eed->v2->co[0] - eed->v1->co[0]);
+			vec[1]= eed->v1->co[1] + labda*(eed->v2->co[1] - eed->v1->co[1]);
+			vec[2]= eed->v1->co[2] + labda*(eed->v2->co[2] - eed->v1->co[2]);
+			Mat4MulVecfl(G.obedit->obmat, vec);
+
+			if(view3d_test_clipping(G.vd, vec)==0) {
+				data->dist = distance;
+				data->closest = eed;
 			}
 		}
-		return eve;
-	}
-	else return findnearestvert_f(dist, sel);
-}
-
-/* helper for findnearest edge */
-static float dist_mval_edge(short *mval, EditEdge *eed)
-{
-	float v1[2], v2[2], mval2[2];
-
-	mval2[0] = (float)mval[0];
-	mval2[1] = (float)mval[1];
-	
-	v1[0] = eed->v1->xs;
-	v1[1] = eed->v1->ys;
-	v2[0] = eed->v2->xs;
-	v2[1] = eed->v2->ys;
-	
-	return PdistVL2Dfl(mval2, v1, v2);
-}
-
-static EditEdge *findnearestedge_f(short *dist)
-{
-	EditMesh *em = G.editMesh;
-	EditEdge *closest, *eed;
-	EditVert *eve;
-	short mval[2], distance;
-	
-	if(em->edges.first==NULL) return NULL;
-	else eed= em->edges.first;	
-	
-	calc_meshverts_ext_f2();     	/* sets/clears (eve->f & 2) for vertices that aren't visible */
-
-	getmouseco_areawin(mval);
-	closest=NULL;
-	
-	/*compare the distance to the rest of the edges and find the closest one*/
-	eed=em->edges.first;
-	while(eed) {
-		/* Are both vertices of the edge ofscreen or either of them hidden? then don't select the edge*/
-		if( !((eed->v1->f & 2) && (eed->v2->f & 2)) && eed->h==0){
-			
-			distance= dist_mval_edge(mval, eed);
-			if(eed->f & SELECT) distance+=5;
-			if(distance < *dist) {
-				*dist= distance;
-				closest= eed;
-			}
+		else {
+			data->dist = distance;
+			data->closest = eed;
 		}
-		eed= eed->next;
 	}
-	
-	/* reset flags */	
-	for(eve=em->verts.first; eve; eve=eve->next){
-		eve->f &= ~2;
-	}
-	
-	return closest;
 }
-
-/* backbuffer version */
 EditEdge *findnearestedge(short *dist)
 {
-	if(G.vd->drawtype>OB_WIRE && (G.vd->flag & V3D_ZBUF_SELECT)) {
-		EditEdge *eed=NULL;
-		unsigned int *buf;
-		int a=1, index;
-		short mval[2], distance=255;
+	short mval[2];
 		
-		getmouseco_areawin(mval);
-		// sample colorcode 
-
-		buf= read_backbuf(mval[0]-25, mval[1]-25, mval[0]+24, mval[1]+24);
-		if(buf) {
-			index= sample_backbuf_rect(buf, 50, em_solidoffs, em_wireoffs, &distance); // global, set in drawobject.c
-			MEM_freeN(buf);
-			if(distance < *dist) {
-				if(index>0 && index<=em_wireoffs-em_solidoffs) {
-					for(eed= G.editMesh->edges.first; eed; eed= eed->next, a++) 
-						if(index==a) break;
-				}
-				if(eed) *dist= distance;
-			}
-		}
-		
-		return eed;
-	}
-	else return findnearestedge_f(dist);
-}
-
-
-static EditFace *findnearestface_f(short *dist)
-{
-	static EditFace *acto= NULL;
-	EditMesh *em = G.editMesh;
-	/* if selected the faces with flag==1 get a disadvantage */
-	EditFace *efa, *act=NULL;
-	short temp, mval[2];
-
-	if(em->faces.first==NULL) return NULL;
-
-	/* do projection */
-	calc_mesh_facedots_ext();
-	
-	/* we count from acto->next to last, and from first to acto */
-	/* does acto exist? */
-	efa= em->faces.first;
-	while(efa) {
-		if(efa==acto) break;
-		efa= efa->next;
-	}
-	if(efa==NULL) acto= em->faces.first;
-
-	/* is there an indicated face? part 1 */
 	getmouseco_areawin(mval);
-	efa= acto->next;
-	while(efa) {
-		if(efa->h==0 && efa->fgonf!=EM_FGON) {
-			temp= abs(mval[0]- efa->xs)+ abs(mval[1]- efa->ys);
-			if(temp< *dist) {
-				act= efa;
-				*dist= temp;
-			}
-		}
-		efa= efa->next;
-	}
-	/* is there an indicated face? part 2 */
-	if(*dist>3) {
-		efa= em->faces.first;
-		while(efa) {
-			if(efa->h==0 && efa->fgonf!=EM_FGON) {
-				temp= abs(mval[0]- efa->xs)+ abs(mval[1]- efa->ys);
-				if(temp< *dist) {
-					act= efa;
-					*dist= temp;
-				}
-				if(efa== acto) break;
-			}
-			efa= efa->next;
-		}
-	}
 
-	acto= act;
-	return act;
+	if(G.vd->drawtype>OB_WIRE && (G.vd->flag & V3D_ZBUF_SELECT)) {
+		short distance;
+		unsigned int index = sample_backbuf_rect(mval, 50, em_solidoffs, em_wireoffs, &distance);
+		EditEdge *eed = BLI_findlink(&G.editMesh->edges, index-1);
+
+		if (eed && distance<*dist) {
+			*dist = distance;
+			return eed;
+		} else {
+			return NULL;
+		}
+	}
+	else {
+		struct { float mval[2]; short dist; EditEdge *closest; } data;
+
+		data.mval[0] = mval[0];
+		data.mval[1] = mval[1];
+		data.dist = *dist;
+		data.closest = NULL;
+
+		mesh_foreachScreenEdge(findnearestedge__doClosest, &data, 2);
+
+		*dist = data.dist;
+		return data.closest;
+	}
 }
 
+static void findnearestface__getDistance(void *userData, EditFace *efa, int x, int y, int index)
+{
+	struct { short mval[2], dist; EditFace *toFace; } *data = userData;
+
+	if (efa==data->toFace) {
+		short temp = abs(data->mval[0]-x) + abs(data->mval[1]-y);
+
+		if (temp<data->dist)
+			data->dist = temp;
+	}
+}
+static void findnearestface__doClosest(void *userData, EditFace *efa, int x, int y, int index)
+{
+	struct { short mval[2], pass, dist; int lastIndex, closestIndex; EditFace *closest; } *data = userData;
+
+	if (data->pass==0) {
+		if (index<=data->lastIndex)
+			return;
+	} else {
+		if (index>data->lastIndex)
+			return;
+	}
+
+	if (data->dist>3) {
+		short temp = abs(data->mval[0]-x) + abs(data->mval[1]-y);
+
+		if (temp<data->dist) {
+			data->dist = temp;
+			data->closest = efa;
+			data->closestIndex = index;
+		}
+	}
+}
 static EditFace *findnearestface(short *dist)
 {
+	short mval[2];
+
+	getmouseco_areawin(mval);
+
 	if(G.vd->drawtype>OB_WIRE && (G.vd->flag & V3D_ZBUF_SELECT)) {
-		EditFace *efa=NULL;
-		int a=1, index;
-		short mval[2], distance;
+		unsigned int index = sample_backbuf(mval[0], mval[1]);
+		EditFace *efa = BLI_findlink(&G.editMesh->faces, index-1);
 
-		calc_mesh_facedots_ext();	// shouldnt be needed each click
-		getmouseco_areawin(mval);
+		if (efa) {
+			struct { short mval[2], dist; EditFace *toFace; } data;
 
-		// sample colorcode 
-		index= sample_backbuf(mval[0], mval[1]);
-		
-		if(index && index<=em_solidoffs) {
-			for(efa= G.editMesh->faces.first; efa; efa= efa->next, a++) if(index==a) break;
-			if(efa) {
-				distance= abs(mval[0]- efa->xs)+ abs(mval[1]- efa->ys);
+			data.mval[0] = mval[0];
+			data.mval[1] = mval[1];
+			data.dist = 0x7FFF;		// largest short
+			data.toFace = efa;
 
-				if(G.scene->selectmode == SCE_SELECT_FACE || distance<*dist) {	// only faces, no dist check
-					*dist= distance;
-					return efa;
-				}
+			mesh_foreachScreenFace(findnearestface__getDistance, &data);
+
+			if(G.scene->selectmode == SCE_SELECT_FACE || data.dist<*dist) {	// only faces, no dist check
+				*dist= data.dist;
+				return efa;
 			}
 		}
 		
 		return NULL;
 	}
-	else return findnearestface_f(dist);
+	else {
+		struct { short mval[2], pass, dist; int lastIndex, closestIndex; EditFace *closest; } data;
+		static int lastSelectedIndex=0;
+		static EditFace *lastSelected=NULL;
+
+		if (lastSelected && BLI_findlink(&G.editMesh->faces, lastSelectedIndex)!=lastSelected) {
+			lastSelectedIndex = 0;
+			lastSelected = NULL;
+		}
+
+		data.lastIndex = lastSelectedIndex;
+		data.mval[0] = mval[0];
+		data.mval[1] = mval[1];
+		data.dist = *dist;
+		data.closest = NULL;
+		data.closestIndex = 0;
+
+		data.pass = 0;
+		mesh_foreachScreenFace(findnearestface__doClosest, &data);
+
+		if (data.dist>3) {
+			data.pass = 1;
+			mesh_foreachScreenFace(findnearestface__doClosest, &data);
+		}
+
+		*dist = data.dist;
+		lastSelected = data.closest;
+		lastSelectedIndex = data.closestIndex;
+
+		return data.closest;
+	}
 }
 
 /* for interactivity, frontbuffer draw in current window */
+static void draw_dm_mapped_vert__mapFunc(void *theVert, int index, float *co, float *no_f, short *no_s)
+{
+	if (EM_get_vert_for_index(index)==theVert) {
+		bglVertex3fv(co);
+	}
+}
+static void draw_dm_mapped_vert(DerivedMesh *dm, EditVert *eve)
+{
+	EM_init_index_arrays(1, 0, 0);
+	bglBegin(GL_POINTS);
+	dm->foreachMappedVert(dm, draw_dm_mapped_vert__mapFunc, eve);
+	bglEnd();
+	EM_free_index_arrays();
+}
+
+static int draw_dm_mapped_edge__setDrawOptions(void *theEdge, int index)
+{
+	return EM_get_edge_for_index(index)==theEdge;
+}
+static void draw_dm_mapped_edge(DerivedMesh *dm, EditEdge *eed)
+{
+	EM_init_index_arrays(0, 1, 0);
+	dm->drawMappedEdges(dm, draw_dm_mapped_edge__setDrawOptions, eed);
+	EM_free_index_arrays();
+}
+
+static void draw_dm_mapped_face_center__mapFunc(void *theFace, int index, float *cent, float *no)
+{
+	if (EM_get_face_for_index(index)==theFace) {
+		bglVertex3fv(cent);
+	}
+}
+static void draw_dm_mapped_face_center(DerivedMesh *dm, EditFace *efa)
+{
+	EM_init_index_arrays(0, 0, 1);
+	bglBegin(GL_POINTS);
+	dm->foreachMappedFaceCenter(dm, draw_dm_mapped_face_center__mapFunc, efa);
+	bglEnd();
+	EM_free_index_arrays();
+}
+
 static void unified_select_draw(EditVert *eve, EditEdge *eed, EditFace *efa)
 {
-	int optimal= subsurf_optimal(G.obedit);
+	int dmNeedsFree;
+	DerivedMesh *dm = editmesh_get_derived_cage(&dmNeedsFree);
 
 	glDrawBuffer(GL_FRONT);
 
 	persp(PERSP_VIEW);
+	
+	if(G.vd->flag & V3D_CLIPPING)
+		view3d_set_clipping(G.vd);
+	
 	glPushMatrix();
 	mymultmatrix(G.obedit->obmat);
 	
@@ -655,39 +792,14 @@ static void unified_select_draw(EditVert *eve, EditEdge *eed, EditFace *efa)
 		}
 
 		if(G.scene->selectmode & (SCE_SELECT_EDGE|SCE_SELECT_FACE)) {
-			Mesh *me= G.obedit->data;
-			DispList *dl= find_displist(&me->disp, DL_MESH);
-			DispListMesh *dlm= NULL;
-			
 			if(efa->fgonf==0) {
-				if(efa->f & SELECT) BIF_ThemeColor(TH_EDGE_SELECT);
-				else BIF_ThemeColor(TH_WIRE);
-				
-				if(dl) dlm= dl->mesh;
-				if(dlm && optimal) {
-					MEdge *medge= dlm->medge;
-					MVert *mvert= dlm->mvert;
-					int b;
-					
-					glBegin(GL_LINES);
-					for (b=0; b<dlm->totedge; b++, medge++) {
-						if(medge->flag & ME_EDGEDRAW) {
-							EditEdge *eed = dlm->editedge[b];
-							if(efa->e1==eed || efa->e2==eed || efa->e3==eed || (efa->e4 && efa->e4==eed)) {
-								glVertex3fv(mvert[medge->v1].co); 
-								glVertex3fv(mvert[medge->v2].co);
-							}
-						}
-					}
-					glEnd();
-				}
-				else {
-					glBegin(GL_LINE_LOOP);
-					glVertex3fv(efa->v1->co);
-					glVertex3fv(efa->v2->co);
-					glVertex3fv(efa->v3->co);
-					if(efa->v4) glVertex3fv(efa->v4->co);
-					glEnd();
+				BIF_ThemeColor((efa->f & SELECT)?TH_EDGE_SELECT:TH_WIRE);
+	
+				draw_dm_mapped_edge(dm, efa->e1);
+				draw_dm_mapped_edge(dm, efa->e2);
+				draw_dm_mapped_edge(dm, efa->e3);
+				if (efa->e4) {
+					draw_dm_mapped_edge(dm, efa->e4);
 				}
 			}
 		}
@@ -695,81 +807,35 @@ static void unified_select_draw(EditVert *eve, EditEdge *eed, EditFace *efa)
 		if(G.scene->selectmode & SCE_SELECT_FACE) {
 			if(efa->fgonf==0) {
 				glPointSize(BIF_GetThemeValuef(TH_FACEDOT_SIZE));
-				
-				if(efa->f & SELECT) BIF_ThemeColor(TH_FACE_DOT);
-				else BIF_ThemeColor(TH_WIRE);
-				
-				bglBegin(GL_POINTS);
-				bglVertex3fv(efa->cent);
-				bglEnd();
+				BIF_ThemeColor((efa->f & SELECT)?TH_FACE_DOT:TH_WIRE);
+
+				draw_dm_mapped_face_center(dm, efa);
 			}
 		}
 	}
 	/* edge selected */
 	if(eed) {
 		if(G.scene->selectmode & (SCE_SELECT_EDGE|SCE_SELECT_FACE)) {
-			Mesh *me= G.obedit->data;
-			DispList *dl= find_displist(&me->disp, DL_MESH);
-			DispListMesh *dlm= NULL;
-			
-			if(dl) dlm= dl->mesh;
-			
-			if(eed->f & SELECT) BIF_ThemeColor(TH_EDGE_SELECT);
-			else BIF_ThemeColor(TH_WIRE);
+			BIF_ThemeColor((eed->f & SELECT)?TH_EDGE_SELECT:TH_WIRE);
 
-			if(dlm && optimal) {
-				MEdge *medge= dlm->medge;
-				MVert *mvert= dlm->mvert;
-				int b;
-				
-				glBegin(GL_LINES);
-				for (b=0; b<dlm->totedge; b++, medge++) {
-					if(medge->flag & ME_EDGEDRAW) {
-						if(eed == dlm->editedge[b]) {
-							glVertex3fv(mvert[medge->v1].co); 
-							glVertex3fv(mvert[medge->v2].co);
-						}
-					}
-				}
-				glEnd();
-			}
-			else {
-				
-				glBegin(GL_LINES);
-				glVertex3fv(eed->v1->co);
-				glVertex3fv(eed->v2->co);
-				glEnd();
-			}
+			draw_dm_mapped_edge(dm, eed);
 		}
 		if(G.scene->selectmode & SCE_SELECT_VERTEX) {
 			glPointSize(BIF_GetThemeValuef(TH_VERTEX_SIZE));
 			
-			if(eed->f & SELECT) BIF_ThemeColor(TH_VERTEX_SELECT);
-			else BIF_ThemeColor(TH_VERTEX);
+			BIF_ThemeColor((eed->f & SELECT)?TH_VERTEX_SELECT:TH_VERTEX);
 			
-			bglBegin(GL_POINTS);
-			if(optimal) {
-				bglVertex3fv(eed->v1->ssco);
-				bglVertex3fv(eed->v2->ssco);
-			} else {
-				bglVertex3fv(eed->v1->co);
-				bglVertex3fv(eed->v2->co);
-			}
-			bglEnd();
+			draw_dm_mapped_vert(dm, eed->v1);
+			draw_dm_mapped_vert(dm, eed->v2);
 		}
 	}
 	if(eve) {
 		if(G.scene->selectmode & SCE_SELECT_VERTEX) {
-
 			glPointSize(BIF_GetThemeValuef(TH_VERTEX_SIZE));
 			
-			if(eve->f & SELECT) BIF_ThemeColor(TH_VERTEX_SELECT);
-			else BIF_ThemeColor(TH_VERTEX);
+			BIF_ThemeColor((eve->f & SELECT)?TH_VERTEX_SELECT:TH_VERTEX);
 			
-			bglBegin(GL_POINTS);
-			if(optimal) bglVertex3fv(eve->ssco);
-			else bglVertex3fv(eve->co);
-			bglEnd();
+			draw_dm_mapped_vert(dm, eve);
 		}
 	}
 
@@ -779,8 +845,15 @@ static void unified_select_draw(EditVert *eve, EditEdge *eed, EditFace *efa)
 	glFlush();
 	glDrawBuffer(GL_BACK);
 
+	if(G.vd->flag & V3D_CLIPPING)
+		view3d_clr_clipping();
+	
 	/* signal that frontbuf differs from back */
 	curarea->win_swap= WIN_FRONT_OK;
+
+	if (dmNeedsFree) {
+		dm->release(dm);
+	}
 }
 
 
@@ -821,7 +894,7 @@ static int unified_findnearest(EditVert **eve, EditEdge **eed, EditFace **efa)
 
 /* selects quads in loop direction of indicated edge */
 /* only flush over edges with valence <= 2 */
-static void faceloop_select(EditEdge *startedge, int select)
+void faceloop_select(EditEdge *startedge, int select)
 {
 	EditMesh *em = G.editMesh;
 	EditEdge *eed;
@@ -883,8 +956,10 @@ static void faceloop_select(EditEdge *startedge, int select)
 	}
 	
 	/* (de)select the faces */
-	for(efa= em->faces.first; efa; efa= efa->next) {
-		if(efa->f1) EM_select_face(efa, select);
+	if(select!=2) {
+		for(efa= em->faces.first; efa; efa= efa->next) {
+			if(efa->f1) EM_select_face(efa, select);
+		}
 	}
 }
 
@@ -982,6 +1057,74 @@ static void edgeloop_select(EditEdge *starteed, int select)
 	}
 }
 
+/* 
+   Almostly exactly the same code as faceloop select
+*/
+static void edgering_select(EditEdge *startedge, int select){
+	EditMesh *em = G.editMesh;
+	EditEdge *eed;
+	EditFace *efa;
+	int looking= 1;
+	
+	/* in eed->f1 we put the valence (amount of faces in edge) */
+	/* in eed->f2 we put tagged flag as correct loop */
+	/* in efa->f1 we put tagged flag as correct to select */
+
+	for(eed= em->edges.first; eed; eed= eed->next) {
+		eed->f1= 0;
+		eed->f2= 0;
+	}
+	for(efa= em->faces.first; efa; efa= efa->next) {
+		efa->f1= 0;
+		if(efa->h==0) {
+			efa->e1->f1++;
+			efa->e2->f1++;
+			efa->e3->f1++;
+			if(efa->e4) efa->e4->f1++;
+		}
+	}
+	
+	// tag startedge OK
+	startedge->f2= 1;
+	
+	while(looking) {
+		looking= 0;
+		
+		for(efa= em->faces.first; efa; efa= efa->next) {
+			if(efa->e4 && efa->f1==0) {	// not done quad
+				if(efa->e1->f1<=2 && efa->e2->f1<=2 && efa->e3->f1<=2 && efa->e4->f1<=2) { // valence ok
+
+					// if edge tagged, select opposing edge and mark face ok
+					if(efa->e1->f2) {
+						efa->e3->f2= 1;
+						efa->f1= 1;
+						looking= 1;
+					}
+					else if(efa->e2->f2) {
+						efa->e4->f2= 1;
+						efa->f1= 1;
+						looking= 1;
+					}
+					if(efa->e3->f2) {
+						efa->e1->f2= 1;
+						efa->f1= 1;
+						looking= 1;
+					}
+					if(efa->e4->f2) {
+						efa->e2->f2= 1;
+						efa->f1= 1;
+						looking= 1;
+					}
+				}
+			}
+		}
+	}
+	
+	/* (de)select the edges */
+	for(eed= em->edges.first; eed; eed= eed->next) {
+    		if(eed->f2) EM_select_edge(eed, select);
+	}
+}
 /* ***************** MAIN MOUSE SELECTION ************** */
 
 // just to have the functions nice together
@@ -1001,15 +1144,24 @@ static void mouse_mesh_loop(void)
 		if(G.scene->selectmode & SCE_SELECT_FACE) {
 			faceloop_select(eed, eed->f & SELECT);
 		}
-		else {
-			edgeloop_select(eed, eed->f & SELECT);
+		else if(G.scene->selectmode & SCE_SELECT_EDGE) {
+            if(G.qual == (LR_CTRLKEY | LR_ALTKEY) || G.qual == (LR_CTRLKEY | LR_ALTKEY |LR_SHIFTKEY))
+    			edgering_select(eed, eed->f & SELECT);
+            else if(G.qual & LR_ALTKEY)
+    			edgeloop_select(eed, eed->f & SELECT);
+		}
+        else if(G.scene->selectmode & SCE_SELECT_VERTEX) {
+            if(G.qual == (LR_CTRLKEY | LR_ALTKEY) || G.qual == (LR_CTRLKEY | LR_ALTKEY |LR_SHIFTKEY))
+    			edgering_select(eed, eed->f & SELECT);
+            else if(G.qual & LR_ALTKEY)
+    			edgeloop_select(eed, eed->f & SELECT);
 		}
 
 		/* frontbuffer draw of last selected only */
 		unified_select_draw(NULL, eed, NULL);
 		
-		countall();
 		EM_selectmode_flush();
+		countall();
 		
 		allqueue(REDRAWVIEW3D, 0);
 	}
@@ -1053,8 +1205,8 @@ void mouse_mesh(void)
 		/* frontbuffer draw of last selected only */
 		unified_select_draw(eve, eed, efa);
 	
-		countall();
 		EM_selectmode_flush();
+		countall();
 
 		allqueue(REDRAWVIEW3D, 0);
 	}
@@ -1208,7 +1360,6 @@ void hide_mesh(int swap)
 	if(G.scene->selectmode & SCE_SELECT_VERTEX) {
 		for(eve= em->verts.first; eve; eve= eve->next) {
 			if((eve->f & SELECT)!=swap) {
-				eve->xs= 3200;
 				eve->f &= ~SELECT;
 				eve->h= 1;
 			}
@@ -1284,7 +1435,7 @@ void hide_mesh(int swap)
 	}
 		
 	allqueue(REDRAWVIEW3D, 0);
-	makeDispList(G.obedit);
+	DAG_object_flush_update(G.scene, G.obedit, OB_RECALC_DATA);	
 	BIF_undo_push("Hide");
 }
 
@@ -1304,29 +1455,68 @@ void reveal_mesh(void)
 			eve->f |= SELECT;
 		}
 	}
-
 	for(eed= em->edges.first; eed; eed= eed->next) {
 		if(eed->h & 1) {
 			eed->h &= ~1;
-			EM_select_edge(eed, 1);
+			if(G.scene->selectmode & SCE_SELECT_VERTEX); 
+			else EM_select_edge(eed, 1);
 		}
 	}
-
 	for(efa= em->faces.first; efa; efa= efa->next) {
 		if(efa->h) {
 			efa->h= 0;
-			EM_select_face(efa, 1);
+			if(G.scene->selectmode & (SCE_SELECT_EDGE|SCE_SELECT_VERTEX)); 
+			else EM_select_face(efa, 1);
 		}
 	}
-	
+
 	EM_fgon_flags();	// redo flags and indices for fgons
 	EM_selectmode_flush();
-	
+	countall();
+
 	allqueue(REDRAWVIEW3D, 0);
-	makeDispList(G.obedit);
+	DAG_object_flush_update(G.scene, G.obedit, OB_RECALC_DATA);	
 	BIF_undo_push("Reveal");
 }
 
+void select_faces_by_numverts(int numverts)
+{
+	EditMesh *em = G.editMesh;
+	EditFace *efa;
+
+	/* Selects isolated verts, and edges that do not have 2 neighboring
+	 * faces
+	 */
+	
+	if(G.scene->selectmode!=SCE_SELECT_FACE) {
+		error("Only works in face selection mode");
+		return;
+	}
+
+	efa= em->faces.first;
+	while(efa) {
+		if (efa->e4) {
+			EM_select_face(efa, (numverts==4) );
+		}
+		else if (efa->e3) {
+			EM_select_face(efa, (numverts==3) );
+		}
+		else 
+			EM_select_face(efa, (numverts!=3) && (numverts!=4) );
+		efa= efa->next;
+	}
+
+	countall();
+	addqueue(curarea->win,  REDRAW, 0);
+
+	if (numverts==3)
+		BIF_undo_push("Select Triangles");
+	else if (numverts==4)
+		BIF_undo_push("Select Quads");
+	else
+		BIF_undo_push("Select non-Triangles/Quads");
+}
+
 void select_non_manifold(void)
 {
 	EditMesh *em = G.editMesh;
@@ -1665,7 +1855,6 @@ void editmesh_mark_seam(int clear)
 {
 	EditMesh *em= G.editMesh;
 	EditEdge *eed;
-	Mesh *me= G.obedit->data;
 
 	/* auto-enable seams drawing */
 	if(clear==0) {
@@ -1673,8 +1862,6 @@ void editmesh_mark_seam(int clear)
 			G.f |= G_DRAWSEAMS;
 			allqueue(REDRAWBUTSEDIT, 0);
 		}
-		if(!me->medge)
-			me->medge= MEM_callocN(sizeof(MEdge), "fake mesh edge");
 	}
 
 	if(clear) {
@@ -1704,7 +1891,7 @@ void editmesh_mark_seam(int clear)
 void Edge_Menu() {
 	short ret;
 
-	ret= pupmenu("Edge Specials%t|Mark Seam %x1|Clear Seam %x2|Rotate Edge %x3");
+	ret= pupmenu("Edge Specials%t|Mark Seam %x1|Clear Seam %x2|Rotate Edge CW%x3|Rotate Edge CCW%x4|Loopcut%x6|Edge Slide%x5");
 
 	switch(ret)
 	{
@@ -1715,7 +1902,18 @@ void Edge_Menu() {
 		editmesh_mark_seam(1);
 		break;
 	case 3:
-		edge_rotate_selected();
+		edge_rotate_selected(2);
+		break;
+	case 4:
+		edge_rotate_selected(1);
+		break;
+	case 5:
+		EdgeSlide(0,0.0);
+    		BIF_undo_push("EdgeSlide");
+		break;
+	case 6:
+        	CutEdgeloop(1);
+		BIF_undo_push("Loopcut New");
 		break;
 	}
 }
@@ -1926,7 +2124,7 @@ void righthandfaces(int select)	/* makes faces righthand turning */
 
 	recalc_editnormals();
 	
-	makeDispList(G.obedit);
+	DAG_object_flush_update(G.scene, G.obedit, OB_RECALC_DATA);
 	
 	waitcursor(0);
 }
@@ -1968,13 +2166,9 @@ static int faceselect_nfaces_selected(Mesh *me)
 {
 	int i, count= 0;
 
-	for (i=0; i<me->totface; i++) {
-		MFace *mf= ((MFace*) me->mface) + i;
-		TFace *tf= ((TFace*) me->tface) + i;
-
-		if (mf->v3 && tface_is_selected(tf))
+	for (i=0; i<me->totface; i++)
+		if (tface_is_selected(&me->tface[i]))
 			count++;
-	}
 
 	return count;
 }
@@ -1997,7 +2191,7 @@ void faceselect_align_view_to_selected(View3D *v3d, Mesh *me, int axis)
 			MFace *mf= ((MFace*) me->mface) + i;
 			TFace *tf= ((TFace*) me->tface) + i;
 	
-			if (mf->v3 && tface_is_selected(tf)) {
+			if (tface_is_selected(tf)) {
 				float *v1, *v2, *v3, fno[3];
 
 				v1= me->mvert[mf->v1].co;
@@ -2090,6 +2284,7 @@ void vertexsmooth(void)
 	float *adror, *adr, fac;
 	float fvec[3];
 	int teller=0;
+	ModifierData *md= G.obedit->modifiers.first;
 
 	if(G.obedit==0) return;
 
@@ -2107,10 +2302,42 @@ void vertexsmooth(void)
 		if(eve->f & SELECT) {
 			eve->vn= (EditVert *)adr;
 			eve->f1= 0;
+			eve->f2= 0;
 			adr+= 3;
 		}
 		eve= eve->next;
 	}
+
+	/* if there is a mirror modifier with clipping, flag the verts that
+	 * are within tolerance of the plane(s) of reflection 
+	 */
+	for (; md; md=md->next) {
+		if (md->type==eModifierType_Mirror) {
+			MirrorModifierData *mmd = (MirrorModifierData*) md;	
+		
+			if(mmd->flag & MOD_MIR_CLIPPING) {
+				for (eve= em->verts.first; eve; eve= eve->next) {
+					if(eve->f & SELECT) {
+
+						switch(mmd->axis){
+							case 0:
+								if (fabs(eve->co[0]) < mmd->tolerance)
+									eve->f2 |= 1;
+								break;
+							case 1:
+								if (fabs(eve->co[1]) < mmd->tolerance)
+									eve->f2 |= 2;
+								break;
+							case 2:
+								if (fabs(eve->co[2]) < mmd->tolerance)
+									eve->f2 |= 4;
+								break;
+						}
+					}
+				}
+			}
+		}
+	}
 	
 	eed= em->edges.first;
 	while(eed) {
@@ -2141,6 +2368,19 @@ void vertexsmooth(void)
 				eve->co[0]= 0.5*eve->co[0]+fac*adr[0];
 				eve->co[1]= 0.5*eve->co[1]+fac*adr[1];
 				eve->co[2]= 0.5*eve->co[2]+fac*adr[2];
+
+				/* clip if needed by mirror modifier */
+				if (eve->f2) {
+					if (eve->f2 & 1) {
+						eve->co[0]= 0.0f;
+					}
+					if (eve->f2 & 2) {
+						eve->co[1]= 0.0f;
+					}
+					if (eve->f2 & 4) {
+						eve->co[2]= 0.0f;
+					}
+				}
 			}
 			eve->vn= 0;
 		}
@@ -2151,14 +2391,13 @@ void vertexsmooth(void)
 	recalc_editnormals();
 
 	allqueue(REDRAWVIEW3D, 0);
-	makeDispList(G.obedit);
+	DAG_object_flush_update(G.scene, G.obedit, OB_RECALC_DATA);
 	BIF_undo_push("Vertex Smooth");
 }
 
 void vertexnoise(void)
 {
 	EditMesh *em = G.editMesh;
-	extern float Tin;
 	Material *ma;
 	Tex *tex;
 	EditVert *eve;
@@ -2189,10 +2428,9 @@ void vertexnoise(void)
 				VecAddf(eve->co, eve->co, vec);
 			}
 			else {
-				
-				externtex(ma->mtex[0], eve->co);
-			
-				eve->co[2]+= 0.05*Tin;
+				float tin, dum;
+				externtex(ma->mtex[0], eve->co, &tin, &dum, &dum, &dum, &dum);
+				eve->co[2]+= 0.05*tin;
 			}
 		}
 		eve= eve->next;
@@ -2200,7 +2438,7 @@ void vertexnoise(void)
 
 	recalc_editnormals();
 	allqueue(REDRAWVIEW3D, 0);
-	makeDispList(G.obedit);
+	DAG_object_flush_update(G.scene, G.obedit, OB_RECALC_DATA);
 	BIF_undo_push("Vertex Noise");
 }
 
@@ -2264,7 +2502,7 @@ void vertices_to_sphere(void)
 	
 	recalc_editnormals();
 	allqueue(REDRAWVIEW3D, 0);
-	makeDispList(G.obedit);
+	DAG_object_flush_update(G.scene, G.obedit, OB_RECALC_DATA);
 	BIF_undo_push("To Sphere");
 }
 
