@@ -1,5 +1,5 @@
 /**
- * $Id: editscreen.c,v 1.86 2004/12/14 18:18:31 ton Exp $
+ * $Id: editscreen.c,v 1.120 2005/11/28 22:49:23 ton Exp $
  *
  * ***** BEGIN GPL/BL DUAL LICENSE BLOCK *****
  *
@@ -36,10 +36,6 @@
 #include <config.h>
 #endif
 
-#ifdef WIN32
-#include "BLI_winstuff.h"
-#endif   
-
 #include "nla.h"
 
 #include <stdlib.h>
@@ -96,12 +92,13 @@
 #include "BSE_edit.h"
 #include "BSE_filesel.h"
 #include "BSE_headerbuttons.h"
+#include "BSE_seqaudio.h"
 #include "BSE_view.h"
 
 #include "BPY_extern.h"
 #include "mydevice.h"
 #include "blendef.h"
-#include "render.h"
+#include "render.h"		/* R.flag */
 
 #include "winlay.h"
 
@@ -128,8 +125,16 @@ static ScrArea *g_activearea= NULL;
 short winqueue_break= 0;
 ScrArea *curarea= 0;
 
+/* prototypes -------------------*/
+int afterqtest(short win, unsigned short evt);
+unsigned short screen_qread(short *val, char *ascii);
+void add_to_mainqueue(Window *win, void *user_data, short evt, short val, char ascii);
+static void drawscredge_area(ScrArea *sa);
+
 /**********************************************************************/
 
+extern int textediting;
+
 static void screen_set_cursor(bScreen *sc) 
 {
 	if (sc->winakt>3) {
@@ -225,7 +230,8 @@ void headerprint(char *str)
 		areawinset(curarea->headwin);
 		
 		headerbox(curarea);
-		cpack(0x0);
+		
+		BIF_ThemeColor(TH_MENU_TEXT); /* better than cpack(0x0) color no? (desoto) */
 		glRasterPos2i(20+curarea->headbutofs,  6);
 		BMF_DrawString(G.font, str);
 		
@@ -318,6 +324,12 @@ void areawinset(short win)
 			G.snla= curarea->spacedata.first;
 			G.v2d= &G.snla->v2d;
 			break;
+		case SPACE_TIME:
+			{
+			SpaceTime *stime= curarea->spacedata.first;
+			G.v2d= &stime->v2d;
+			}
+			break;
 		default:
 			break;
 		}
@@ -385,6 +397,7 @@ void scrarea_do_headdraw(ScrArea *area)
 		case SPACE_SOUND:	sound_buttons();	break;
 		case SPACE_ACTION:	action_buttons();	break;
 		case SPACE_NLA:		nla_buttons();		break;
+		case SPACE_TIME:	time_buttons(area);	break;
 		}
 
 		//glScissor(area->winrct.xmin, area->winrct.xmax, area->winx, area->winy);
@@ -396,9 +409,9 @@ void scrarea_do_headchange(ScrArea *area)
 	float ofs= area->headbutofs;
 
 	if (area->headertype==HEADERDOWN) {
-		bwin_ortho2(area->headwin, -0.375+ofs, area->headrct.xmax-area->headrct.xmin-0.375+ofs, -3.375, area->headrct.ymax-area->headrct.ymin-3.375);
+		bwin_ortho2(area->headwin, -0.375+ofs, area->headrct.xmax-area->headrct.xmin-0.375+ofs, -3.375, area->headrct.ymax-area->headrct.ymin-3.375+1.0);
 	} else if (area->headertype==HEADERTOP) {
-		bwin_ortho2(area->headwin, -0.375+ofs, area->headrct.xmax-area->headrct.xmin-0.375+ofs, -2.375, area->headrct.ymax-area->headrct.ymin-2.375);
+		bwin_ortho2(area->headwin, -0.375+ofs, area->headrct.xmax-area->headrct.xmin-0.375+ofs, -2.375-1.0, area->headrct.ymax-area->headrct.ymin-2.375);
 	}
 }
 
@@ -702,7 +715,7 @@ void splash(void *data, int datasize, char *string)
 }
 
 static void moveareas(ScrEdge *edge);
-static void joinarea(ScrArea *sa, ScrEdge *onedge);
+static void joinarea_interactive(ScrArea *area, ScrEdge *onedge);
 static void splitarea_interactive(ScrArea *area, ScrEdge *onedge);
 
 static void screen_edge_edit_event(ScrArea *actarea, ScrEdge *actedge, short evt, short val) 
@@ -727,7 +740,7 @@ static void screen_edge_edit_event(ScrArea *actarea, ScrEdge *actedge, short evt
 			if (edgeop==1) {
 				splitarea_interactive(actarea, actedge);
 			} else if (edgeop==2) {
-				joinarea(actarea, actedge);
+				joinarea_interactive(actarea, actedge);
 			} else if (edgeop==3) {
 				scrarea_change_headertype(actarea, actarea->headertype?0:HEADERDOWN);
 			}
@@ -738,7 +751,7 @@ static void screen_edge_edit_event(ScrArea *actarea, ScrEdge *actedge, short evt
 
 /***/
 
-void mywindow_init_mainwin(Window *win, int orx, int ory, int sizex, int sizey);
+extern void mywindow_init_mainwin(Window *win, int orx, int ory, int sizex, int sizey);
 void test_scale_screen(bScreen *);
 
 static void resize_screens(int x, int y, int w, int h) {
@@ -947,6 +960,174 @@ void reset_autosave(void) {
 	window_set_timer(mainwin, U.savetime*60*1000, AUTOSAVE_FILE);
 }
 
+/* ************ handlers ************** */
+
+/* don't know yet how the handlers will evolve, for simplicity
+i choose for an array with eventcodes, this saves in a file!
+*/
+void add_screenhandler(bScreen *sc, short eventcode, short val)
+{
+	short a;
+	
+	// find empty spot
+	for(a=0; a<SCREEN_MAXHANDLER; a+=2) {
+		if( sc->handler[a]==eventcode ) {
+			sc->handler[a+1]= val;
+			break;
+		}
+		else if( sc->handler[a]==0) {
+			sc->handler[a]= eventcode;
+			sc->handler[a+1]= val;
+			break;
+		}
+	}
+	if(a==SCREEN_MAXHANDLER) printf("error; max (4) screen handlers reached!\n");
+}
+
+void rem_screenhandler(bScreen *sc, short eventcode)
+{
+	short a;
+	
+	for(a=0; a<SCREEN_MAXHANDLER; a+=2) {
+		if( sc->handler[a]==eventcode) {
+			sc->handler[a]= 0;
+			break;
+		}
+	}
+}
+
+int has_screenhandler(bScreen *sc, short eventcode)
+{
+	short a;
+	
+	for(a=0; a<SCREEN_MAXHANDLER; a+=2) {
+		if( sc->handler[a]==eventcode) {
+			return 1;
+		}
+	}
+	return 0;
+}
+
+static void animated_screen(bScreen *sc, short val)
+{
+	if (U.mixbufsize && (val & TIME_WITH_SEQ_AUDIO)) {
+		if(CFRA>=EFRA) {
+			CFRA= SFRA;
+			audiostream_stop();
+			audiostream_start( CFRA );
+		}
+		else {
+			int cfra = audiostream_pos();
+			if(cfra <= CFRA) CFRA++;
+			else CFRA= cfra;
+		}
+	}
+	else {
+		CFRA++;
+		if(CFRA > EFRA) CFRA= SFRA;
+	}
+	
+	update_for_newframe_nodraw(1);
+	
+	if(val & TIME_ALL_3D_WIN)
+		allqueue(REDRAWVIEW3D, 0);
+	else if(val & TIME_LEFTMOST_3D_WIN) {
+		ScrArea *sa= sc->areabase.first, *samin=NULL;
+		int min= 10000;
+		for(; sa; sa= sa->next) {
+			if(sa->spacetype==SPACE_VIEW3D) {
+				if(sa->winrct.xmin - sa->winrct.ymin < min) {
+					samin= sa;
+					min= sa->winrct.xmin - sa->winrct.ymin;
+				}
+			}
+		}
+		if(samin) scrarea_queue_winredraw(samin);
+	}
+	if(val & TIME_ALL_ANIM_WIN) allqueue(REDRAWANIM, 0);
+	if(val & TIME_ALL_BUTS_WIN) allqueue(REDRAWBUTSALL, 0);
+	
+	allqueue(REDRAWTIME, 0);
+}
+
+/* because we still have to cope with subloops, this function is called
+in viewmove() for example too */
+
+/* returns 1 if something was handled */
+/* restricts to frames-per-second setting for frequency of updates */
+int do_screenhandlers(bScreen *sc)
+{
+	static double ltime=0.0;
+	double swaptime, time;
+	short a, done= 0;
+	
+	time = PIL_check_seconds_timer();
+	swaptime= 1.0/(float)G.scene->r.frs_sec;
+	
+	/* only now do the handlers */
+	if(swaptime < time-ltime || ltime==0.0) {
+		
+		ltime= time;
+
+		for(a=0; a<SCREEN_MAXHANDLER; a+=2) {
+			switch(sc->handler[a]) {
+				case SCREEN_HANDLER_ANIM:
+					animated_screen(sc, sc->handler[a+1]);
+					done= 1;
+					break;
+				case SCREEN_HANDLER_PYTHON:
+					done= 1;
+					break;
+				case SCREEN_HANDLER_VERSE:
+					done= 1;
+					break;
+			}
+		}
+	}
+	else if( qtest()==0) PIL_sleep_ms(5);   // 5 milliseconds pause, for idle
+
+	/* separate check for if we need to add to afterqueue */
+	/* is only to keep mainqueue awqke */
+	for(a=0; a<SCREEN_MAXHANDLER; a+=2) {
+		if(sc->handler[a]) {
+			ScrArea *sa= sc->areabase.first;
+			if(sa->headwin) addafterqueue(sa->headwin, SCREEN_HANDLER, 1);
+			else addafterqueue(sa->win, SCREEN_HANDLER, 1);
+		}
+	}
+	
+	return done;
+}
+
+/* ****** end screen handlers ************ */
+
+static void drawscreen(void)
+{
+	ScrArea *sa;
+	
+	mywinset(G.curscreen->mainwin);
+	myortho2(-0.375, (float)G.curscreen->sizex-0.375, -0.375, (float)G.curscreen->sizey-0.375);
+	
+	sa= G.curscreen->areabase.first;
+	while(sa) {
+		drawscredge_area(sa);
+		sa= sa->next;
+	}
+	
+	/* this double draw patch seems to be needed for certain sgi's (octane, indigo2) */
+#if defined(__sgi) || defined(__sun__) || defined( __sun ) || defined (__sparc) || defined (__sparc__)
+	glDrawBuffer(GL_FRONT);
+	
+	sa= G.curscreen->areabase.first;
+	while(sa) {
+		drawscredge_area(sa);
+		sa= sa->next;
+	}
+	
+	glDrawBuffer(GL_BACK);
+#endif
+}
+
 static void screen_dispatch_events(void) {
 	int events_remaining= 1;
 	ScrArea *sa;
@@ -981,6 +1162,7 @@ static void screen_dispatch_events(void) {
 	}
 	
 	screen_swapbuffers();
+	do_screenhandlers(G.curscreen);
 }
 
 static ScrArea *screen_find_area_for_pt(bScreen *sc, short *mval) 
@@ -1134,7 +1316,7 @@ void screenmain(void)
 			}
 		}
 		else if(ELEM(event, LEFTARROWKEY, RIGHTARROWKEY)) {
-			if(val && (G.qual & LR_CTRLKEY)) {
+			if(textediting==0 && val && (G.qual & LR_CTRLKEY)) {
 				bScreen *sc= (event==LEFTARROWKEY)?G.curscreen->id.prev:G.curscreen->id.next;
 				if(is_allowed_to_change_screen(sc)) setscreen(sc);
 				g_activearea= NULL;
@@ -1185,11 +1367,18 @@ void screenmain(void)
 			}
 		}
 		else if (event==SPACEKEY) {
-			if((G.obedit && G.obedit->type==OB_FONT && g_activearea->spacetype==SPACE_VIEW3D)||g_activearea->spacetype==SPACE_TEXT||g_activearea->spacetype==SPACE_SCRIPT);
-			else {
-				if(val) toolbox_n();
+			if((g_activearea->spacetype!=SPACE_TEXT) && val && (G.qual & LR_SHIFTKEY)) {
+				area_fullscreen();
+				g_activearea= NULL;
 				towin= 0;
 			}
+			else {
+				if((G.obedit && G.obedit->type==OB_FONT && g_activearea->spacetype==SPACE_VIEW3D)||g_activearea->spacetype==SPACE_TEXT||g_activearea->spacetype==SPACE_SCRIPT);
+				else if(G.qual==0) {
+					if(val) toolbox_n();
+					towin= 0;
+				}
+			}
 		}
 		else if(ELEM(event, UPARROWKEY, DOWNARROWKEY)) {
 			if(val && (G.qual & LR_CTRLKEY)) {
@@ -1211,7 +1400,12 @@ void screenmain(void)
 		if (event==0 || event==EXECUTE) {
 			screen_dispatch_events();
 		}
-
+		
+		if(G.f & G_DEBUG) {
+			GLenum error = glGetError();
+			if (error)
+				printf("GL error: %s\n", gluErrorString(error));
+		}
 		/* Bizar hack. The event queue has mutated... */
 		if ( (firsttime) && (event == 0) ) {
 
@@ -1224,7 +1418,7 @@ void screenmain(void)
 				// SET AUTOPLAY in G.flags for
 				// other fileloads
 
-				G.flags |= G_FLAGS_AUTOPLAY;
+				G.flags |= G_FILE_AUTOPLAY;
 				area_autoplayscreen();
 
 				// Let The Games Begin
@@ -1246,7 +1440,8 @@ void screenmain(void)
 	}
 }
 
-#ifdef _WIN32	// FULLSCREEN
+#if 0
+//#ifdef _WIN32	// FULLSCREEN
 void mainwindow_toggle_fullscreen(int fullscreen){
 	if (fullscreen) U.uiflag |= USER_FLIPFULLSCREEN;
 	else U.uiflag &= ~USER_FLIPFULLSCREEN;
@@ -1291,24 +1486,13 @@ void setprefsize(int stax, int stay, int sizx, int sizy)
 	
 	winlay_get_screensize(&scrwidth, &scrheight);
 		
-	if(stax<0) stax= 0;
-	if(stay<0) stay= 0;
 	if(sizx<320) sizx= 320;
 	if(sizy<256) sizy= 256;
 
-	if(stax+sizx>scrwidth) {
-		fprintf(stderr," width prob \n");
-		sizx= scrwidth-stax;
-		}
-
 	if(stay+sizy>scrheight) {
 		fprintf(stderr," height prob \n");
 		sizy= scrheight-stay;
-		}
-
-/* #ifdef __APPLE__ */
-/*	if(stay+sizy>scrheight-22) sizy= scrheight-22-stay;	// 22 = topbar  */
-/* #endif*/
+	}
 
 	if(sizx<320 || sizy<256) {
 		printf("ERROR: illegal prefsize\n");
@@ -1338,7 +1522,7 @@ static void sortscrvert(ScrVert **v1, ScrVert **v2)
 {
 	ScrVert *tmp;
 	
-	if ((long)*v1 > (long)*v2) {
+	if (*v1 > *v2) {
 		tmp= *v1;
 		*v1= *v2;
 		*v2= tmp;	
@@ -1592,17 +1776,30 @@ static void del_area(ScrArea *sa)
 static void copy_areadata(ScrArea *sa1, ScrArea *sa2)
 {
 	Panel *pa1, *pa2, *patab;
-	
+	ScriptLink *slink1 = &sa1->scriptlink, *slink2 = &sa2->scriptlink;
+
 	sa1->headertype= sa2->headertype;
 	sa1->spacetype= sa2->spacetype;
 	Mat4CpyMat4(sa1->winmat, sa2->winmat);
-	
+
 	freespacelist(&sa1->spacedata);
 	duplicatespacelist(sa1, &sa1->spacedata, &sa2->spacedata);
 
 	BLI_freelistN(&sa1->panels);
 	duplicatelist(&sa1->panels, &sa2->panels);
-	
+
+	/* space handler script links */
+	if (slink1->totscript) {
+		MEM_freeN(slink1->scripts);
+		MEM_freeN(slink1->flag);
+		slink1->totscript = 0;
+	}
+	if (slink2->totscript) {
+		slink1->scripts = MEM_dupallocN(slink2->scripts);
+		slink1->flag = MEM_dupallocN(slink2->flag);
+		slink1->totscript = slink2->totscript;
+	}
+
 	/* copy pointers */
 	pa1= sa1->panels.first;
 	while(pa1) {
@@ -1875,7 +2072,8 @@ static bScreen *addscreen(char *name)		/* use setprefsize() if you want somethin
 	sc->scene= G.scene;
 	
   	if (!mainwin) {
-#ifdef _WIN32	// FULLSCREEN
+#if 0
+//#ifdef _WIN32	// FULLSCREEN
 		if (G.windowstate == G_WINDOWSTATE_FULLSCREEN)
 			mainwin= window_open("Blender", sc->startx, sc->starty, sc->sizex, sc->sizey, G_WINDOWSTATE_FULLSCREEN);
 		else
@@ -2030,7 +2228,7 @@ void area_fullscreen(void)	/* with curarea */
 		// refuse to go out of SCREENAUTOPLAY as long as G_FLAGS_AUTOPLAY
 		// is set
 
-		if (fulltype != SCREENAUTOPLAY || (G.flags & G_FLAGS_AUTOPLAY) == 0) {
+		if (fulltype != SCREENAUTOPLAY || (G.flags & G_FILE_AUTOPLAY) == 0) {
 			sc->full= 0;
 		
 			/* find old area */
@@ -2214,36 +2412,213 @@ void duplicate_screen(void)
 /* ************ END SCREEN MANAGEMENT ************** */
 /* ************  JOIN/SPLIT/MOVE ************** */
 
-static void joinarea(ScrArea *sa, ScrEdge *onedge)
+typedef struct point{
+	float x,y;
+}_point;
+
+/* draw vertical shape visualising future joining (left as well
+ * right direction of future joining) */
+static void draw_horizontal_join_shape(ScrArea *sa, char dir)
 {
-	ScrArea *sa2;
-	ScrArea *up=0, *down=0, *right=0, *left=0;
-	ScrEdge *setest;
-	short val=0;
+	_point points[10];
+	short i;
+	float w, h;
+	float width = sa->v3->vec.x - sa->v1->vec.x;
+	float height = sa->v3->vec.y - sa->v1->vec.y;
+
+	if(height<width) {
+		h = height/8;
+		w = height/4;
+	}
+	else {
+		h = width/8;
+		w = width/4;
+	}
+
+	points[0].x = sa->v1->vec.x;
+	points[0].y = sa->v1->vec.y + height/2;
+
+	points[1].x = sa->v1->vec.x;
+	points[1].y = sa->v1->vec.y;
+
+	points[2].x = sa->v4->vec.x - w;
+	points[2].y = sa->v4->vec.y;
+
+	points[3].x = sa->v4->vec.x - w;
+	points[3].y = sa->v4->vec.y + height/2 - 2*h;
+
+	points[4].x = sa->v4->vec.x - 2*w;
+	points[4].y = sa->v4->vec.y + height/2;
+
+	points[5].x = sa->v4->vec.x - w;
+	points[5].y = sa->v4->vec.y + height/2 + 2*h;
+
+	points[6].x = sa->v3->vec.x - w;
+	points[6].y = sa->v3->vec.y;
+
+	points[7].x = sa->v2->vec.x;
+	points[7].y = sa->v2->vec.y;
+
+	points[8].x = sa->v4->vec.x;
+	points[8].y = sa->v4->vec.y + height/2 - h;
+
+	points[9].x = sa->v4->vec.x;
+	points[9].y = sa->v4->vec.y + height/2 + h;
+
+	if(dir=='l') {
+		/* when direction is left, then we flip direction of arrow */
+		float cx = sa->v1->vec.x + width;
+		for(i=0;i<10;i++) {
+			points[i].x -= cx;
+			points[i].x = -points[i].x;
+			points[i].x += sa->v1->vec.x;
+		}
+	}
+
+	glBegin(GL_POLYGON);
+	for(i=0;i<5;i++)
+		glVertex2f(points[i].x, points[i].y);
+	glEnd();
+	glBegin(GL_POLYGON);
+	for(i=4;i<8;i++)
+		glVertex2f(points[i].x, points[i].y);
+	glVertex2f(points[0].x, points[0].y);
+	glEnd();
+
+	glRectf(points[2].x, points[2].y, points[8].x, points[8].y);
+	glRectf(points[6].x, points[6].y, points[9].x, points[9].y);
+}
+
+/* draw vertical shape visualising future joining (up/down direction) */
+static void draw_vertical_join_shape(ScrArea *sa, char dir)
+{
+	_point points[10];
+	short i;
+	float w, h;
+	float width = sa->v3->vec.x - sa->v1->vec.x;
+	float height = sa->v3->vec.y - sa->v1->vec.y;
+
+	if(height<width) {
+		h = height/4;
+		w = height/8;
+	}
+	else {
+		h = width/4;
+		w = width/8;
+	}
+
+	points[0].x = sa->v1->vec.x + width/2;
+	points[0].y = sa->v3->vec.y;
+
+	points[1].x = sa->v2->vec.x;
+	points[1].y = sa->v2->vec.y;
+
+	points[2].x = sa->v1->vec.x;
+	points[2].y = sa->v1->vec.y + h;
+
+	points[3].x = sa->v1->vec.x + width/2 - 2*w;
+	points[3].y = sa->v1->vec.y + h;
+
+	points[4].x = sa->v1->vec.x + width/2;
+	points[4].y = sa->v1->vec.y + 2*h;
+
+	points[5].x = sa->v1->vec.x + width/2 + 2*w;
+	points[5].y = sa->v1->vec.y + h;
+
+	points[6].x = sa->v4->vec.x;
+	points[6].y = sa->v4->vec.y + h;
 	
-	sa= test_edge_area(sa, onedge);
-	if (sa==0) return;
+	points[7].x = sa->v3->vec.x;
+	points[7].y = sa->v3->vec.y;
+
+	points[8].x = sa->v1->vec.x + width/2 - w;
+	points[8].y = sa->v1->vec.y;
+
+	points[9].x = sa->v1->vec.x + width/2 + w;
+	points[9].y = sa->v1->vec.y;
+
+	if(dir=='u') {
+		/* when direction is up, then we flip direction of arrow */
+		float cy = sa->v1->vec.y + height;
+		for(i=0;i<10;i++) {
+			points[i].y -= cy;
+			points[i].y = -points[i].y;
+			points[i].y += sa->v1->vec.y;
+		}
+	}
+
+	glBegin(GL_POLYGON);
+	for(i=0;i<5;i++)
+		glVertex2f(points[i].x, points[i].y);
+	glEnd();
+	glBegin(GL_POLYGON);
+	for(i=4;i<8;i++)
+		glVertex2f(points[i].x, points[i].y);
+	glVertex2f(points[0].x, points[0].y);
+	glEnd();
+
+	glRectf(points[2].x, points[2].y, points[8].x, points[8].y);
+	glRectf(points[6].x, points[6].y, points[9].x, points[9].y);
+}
+
+/* draw join shape due to direction of joining */
+static void draw_join_shape(ScrArea *sa, char dir)
+{
+	if(dir=='u' || dir=='d')
+		draw_vertical_join_shape(sa, dir);
+	else
+		draw_horizontal_join_shape(sa, dir);
+}
+
+/* draw screen area darker with arrow (visualisation of future joining) */
+static void scrarea_draw_shape_dark(ScrArea *sa, char dir)
+{
+	glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
+	glEnable(GL_BLEND);
+	glColor4ub(0, 0, 0, 105);
+	draw_join_shape(sa, dir);
+	glDisable(GL_BLEND);
+}
+
+/* draw screen area ligher with arrow shape ("eraser" of previous dark shape) */
+static void scrarea_draw_shape_light(ScrArea *sa, char dir)
+{
+	glBlendFunc(GL_DST_COLOR, GL_SRC_ALPHA);
+	glEnable(GL_BLEND);
+	/* value 181 was hardly computed: 181~105 */
+	glColor4ub(255, 255, 255, 181);		
+	draw_join_shape(sa, dir);
+	glDisable(GL_BLEND);
+}
+
+static void joinarea_interactive(ScrArea *area, ScrEdge *onedge)
+{
+	struct ScrArea *sa1 = area, *sa2, *scr;
+	struct ScrArea *up=0, *down=0, *right=0, *left=0;
+	struct ScrEdge *se;
+	unsigned short event;
+	short ok=0, val=0, mval[2];
+	char dir=0;
+
+	sa1 = test_edge_area(sa1, onedge);
+	if(sa1==0) return;
 
-	/* which edges? */
 	/* find directions with same edge */
 	sa2= G.curscreen->areabase.first;
 	while(sa2) {
-		if(sa2 != sa) {
-			setest= screen_findedge(G.curscreen, sa2->v1, sa2->v2);
-			if(onedge==setest) right= sa2;
-			setest= screen_findedge(G.curscreen, sa2->v2, sa2->v3);
-			if(onedge==setest) down= sa2;
-			setest= screen_findedge(G.curscreen, sa2->v3, sa2->v4);
-			if(onedge==setest) left= sa2;
-			setest= screen_findedge(G.curscreen, sa2->v4, sa2->v1);
-			if(onedge==setest) up= sa2;
+		if(sa2 != sa1) {
+			se= screen_findedge(G.curscreen, sa2->v1, sa2->v2);
+			if(onedge==se) right= sa2;
+			se= screen_findedge(G.curscreen, sa2->v2, sa2->v3);
+			if(onedge==se) down= sa2;
+			se= screen_findedge(G.curscreen, sa2->v3, sa2->v4);
+			if(onedge==se) left= sa2;
+			se= screen_findedge(G.curscreen, sa2->v4, sa2->v1);
+			if(onedge==se) up= sa2;
 		}
 		sa2= sa2->next;
 	}
 	
-	sa2= 0;
-	setest= 0;
-	
 	if(left) val++;
 	if(up) val++;
 	if(right) val++;
@@ -2251,55 +2626,154 @@ static void joinarea(ScrArea *sa, ScrEdge *onedge)
 	
 	if(val==0) return;
 	else if(val==1) {
-		if(left) sa2= left;
-		else if(up) sa2= up;
-		else if(right) sa2= right;
-		else if(down) sa2= down;
+		if(left) {
+			right = sa1;
+			sa2 = left;
+			dir = 'h';
+		}
+		else if(right) {
+			left = sa1;
+			sa2 = right;
+			dir = 'h';
+		}
+		else if(up) {
+			down = sa1;
+			sa2= up;
+			dir = 'v';
+		}
+		else if(down) {
+			up = sa1;
+			sa2 = down;
+			dir = 'v';
+		}
 	}
-	
-	
-	if(sa2) {
-		/* new area is old sa */
+
+	mywinset(G.curscreen->mainwin);
+
+	/* initial set up screen area asigned for destroying */
+	scr = sa2;
+
+	/* set up standard cursor */
+	set_cursor(CURSOR_STD);
+
+	/* should already have a good matrix */
+	glReadBuffer(GL_FRONT);
+	glDrawBuffer(GL_FRONT);
+
+	/* to prevent flickering after clicking at "Join Areas " */
+	getmouseco_sc(mval);
+	if(dir=='h') {
+		if(scr==left && mval[0]>=onedge->v1->vec.x) scr = right;
+		else if(scr==right && mval[0]<onedge->v1->vec.x) scr = left;
+	}
+	else if(dir=='v') {
+		if(scr==down && mval[1]>=onedge->v1->vec.y) scr = up;
+		else if(scr==up && mval[1]<onedge->v1->vec.y) scr = down;
+	}
+
+	/* draw scr screen area with dark shape */
+	if(scr==left)
+		scrarea_draw_shape_dark(scr,'r');
+	else if(scr==right)
+		scrarea_draw_shape_dark(scr,'l');
+	else if(scr==up)
+		scrarea_draw_shape_dark(scr,'d');
+	else if(scr==down)
+		scrarea_draw_shape_dark(scr,'u');
+	glFlush();
+
+	/* "never ending loop" of interactive selection */
+	while(!ok) {
+		getmouseco_sc(mval);
+
+		/* test if position of mouse is on the "different side" of
+		 * "joining edge" */
+		if(dir=='h') {
+			if(scr==left && mval[0]>=onedge->v1->vec.x) {
+				scrarea_draw_shape_light(scr,'r');
+				scr = right;
+				scrarea_draw_shape_dark(scr,'l');
+			}
+			else if(scr==right && mval[0]<onedge->v1->vec.x) {
+				scrarea_draw_shape_light(scr,'l');
+				scr = left;
+				scrarea_draw_shape_dark(scr,'r');
+			}
+		}
+		else if(dir=='v') {
+			if(scr==down && mval[1]>=onedge->v1->vec.y) {
+				scrarea_draw_shape_light(scr,'u');
+				scr = up;
+				scrarea_draw_shape_dark(scr,'d');
+			}
+			else if(scr==up && mval[1]<onedge->v1->vec.y){
+				scrarea_draw_shape_light(scr,'d');
+				scr = down;
+				scrarea_draw_shape_dark(scr,'u');
+			}
+		}
+
+
+		/* get pressed keys and mouse buttons */
+		event = extern_qread(&val);
+
+		/* confirm joining of two screen areas */
+		if(val && event==LEFTMOUSE) ok= 1;
+
+		/* cancel joining of joining */
+		if(val && (event==ESCKEY || event==RIGHTMOUSE)) ok= -1;
+
+		glFlush();
+	}
+
+	glReadBuffer(GL_BACK);
+	glDrawBuffer(GL_BACK);
+
+	/* joining af screen areas was confirmed ... proceed joining */
+	if(ok==1) {
+		if(sa2!=scr) {
+			sa1 = sa2;
+			sa2 = scr;
+		}
+
 		if(sa2==left) {
-			sa->v1= sa2->v1;
-			sa->v2= sa2->v2;
-			screen_addedge(G.curscreen, sa->v2, sa->v3);
-			screen_addedge(G.curscreen, sa->v1, sa->v4);
+			sa1->v1= sa2->v1;
+			sa1->v2= sa2->v2;
+			screen_addedge(G.curscreen, sa1->v2, sa1->v3);
+			screen_addedge(G.curscreen, sa1->v1, sa1->v4);
 		}
 		else if(sa2==up) {
-			sa->v2= sa2->v2;
-			sa->v3= sa2->v3;
-			screen_addedge(G.curscreen, sa->v1, sa->v2);
-			screen_addedge(G.curscreen, sa->v3, sa->v4);
+			sa1->v2= sa2->v2;
+			sa1->v3= sa2->v3;
+			screen_addedge(G.curscreen, sa1->v1, sa1->v2);
+			screen_addedge(G.curscreen, sa1->v3, sa1->v4);
 		}
 		else if(sa2==right) {
-			sa->v3= sa2->v3;
-			sa->v4= sa2->v4;
-			screen_addedge(G.curscreen, sa->v2, sa->v3);
-			screen_addedge(G.curscreen, sa->v1, sa->v4);
+			sa1->v3= sa2->v3;
+			sa1->v4= sa2->v4;
+			screen_addedge(G.curscreen, sa1->v2, sa1->v3);
+			screen_addedge(G.curscreen, sa1->v1, sa1->v4);
 		}
 		else if(sa2==down) {
-			sa->v1= sa2->v1;
-			sa->v4= sa2->v4;
-			screen_addedge(G.curscreen, sa->v1, sa->v2);
-			screen_addedge(G.curscreen, sa->v3, sa->v4);
+			sa1->v1= sa2->v1;
+			sa1->v4= sa2->v4;
+			screen_addedge(G.curscreen, sa1->v1, sa1->v2);
+			screen_addedge(G.curscreen, sa1->v3, sa1->v4);
 		}
 	
-		/* remove edge and area */
-		/* remlink(&G.curscreen->edgebase, setest); */
-		/* freeN(setest); */
 		del_area(sa2);
 		BLI_remlink(&G.curscreen->areabase, sa2);
 		MEM_freeN(sa2);
 		
 		removedouble_scredges();
 		removenotused_scredges();
-		removenotused_scrverts();	/* as last */
+		removenotused_scrverts();
 		
 		testareas();
 		mainqenter(DRAWEDGES, 1);
-			/* test cursor en inputwindow */
-		mainqenter(MOUSEY, -1);		
+
+		/* test cursor en inputwindow */
+		mainqenter(MOUSEY, -1);
 	}
 }
 
@@ -2432,7 +2906,7 @@ static void scrarea_draw_splitpoint(ScrArea *sa, char dir, float fac)
 
 static void splitarea_interactive(ScrArea *area, ScrEdge *onedge)
 {
-	ScrArea *sa= area;
+	ScrArea *scr, *sa= area;
 	float fac= 0.0;
 	unsigned short event;
 	short ok= 0, val, split = 0, mval[2], mvalo[2], first= 1;
@@ -2451,6 +2925,29 @@ static void splitarea_interactive(ScrArea *area, ScrEdge *onedge)
 	/* keep track of grid and minsize */
 	while(ok==0) {
 		getmouseco_sc(mval);
+
+		/* this part of code allows to choose, what window will be splited */
+		/* cursor is out of the current ScreenArea */
+		if((mval[0] < sa->v1->vec.x) || (mval[0] > sa->v3->vec.x) ||
+		(mval[1] < sa->v1->vec.y) || (mval[1] > sa->v3->vec.y)){
+			scr= (ScrArea*)G.curscreen->areabase.first;
+			while(scr){
+				if((mval[0] > scr->v1->vec.x) && (mval[0] < scr->v4->vec.x) &&
+				(mval[1] < scr->v2->vec.y) && (mval[1] > scr->v1->vec.y)){
+					/* test: is ScreenArea enough big for splitting */
+					short tsplit= testsplitpoint(scr, dir, fac);
+					if(tsplit){
+						split = tsplit;
+						/* delete old line from previous ScreenArea */
+						if(!first) scrarea_draw_splitpoint(sa, dir, fac);
+						sa= scr;
+						first= 1;
+						break;
+					}
+				}
+				scr= scr->next;
+			}
+		}
 		
 		if (first || mval[0]!=mvalo[0] || mval[1]!=mvalo[1]) {
 			if (!first) {
@@ -2478,6 +2975,21 @@ static void splitarea_interactive(ScrArea *area, ScrEdge *onedge)
 		}
 		
 		event= extern_qread(&val);
+
+		/* change direction of splitting between horizontal and vertical
+		 * patch was offered by Guillaume */
+		if(val && (event==TABKEY || event==MIDDLEMOUSE)) {
+			scrarea_draw_splitpoint(sa, dir, fac);
+			if(dir=='h') {
+				dir='v';
+				set_cursor(CURSOR_Y_MOVE);
+			} else {
+				dir='h';
+				set_cursor(CURSOR_X_MOVE);
+			}
+			first= 1;
+		}
+
 		if(val && event==LEFTMOUSE) {
 			if(dir=='h') {
 				fac= split- (sa->v1->vec.y);
@@ -2807,7 +3319,6 @@ static void moveareas(ScrEdge *edge)
 
 			delta= (dir=='h')?(mval[1]-mvalo[1]):(mval[0]-mvalo[0]);
 			delta= CLAMPIS(delta, -smaller, bigger);
-			
 			draw_front_xor_dirdist_line(dir, edge_position+delta, edge_start, edge_end);
 			glFlush();
 		} 
@@ -2820,6 +3331,8 @@ static void moveareas(ScrEdge *edge)
 			else if (ELEM(event, SPACEKEY, RETKEY))
 				doit= 1;
 		}
+		else BIF_wait_for_statechange();
+		
 	}
 	draw_front_xor_dirdist_line(dir, edge_position+delta, edge_start, edge_end);
 	glFlush();
@@ -2837,12 +3350,12 @@ static void moveareas(ScrEdge *edge)
 				if((dir=='h') && v1->vec.y>0 && v1->vec.y<G.curscreen->sizey-1) {
 					v1->vec.y+= delta;
 					
-					/* with these lines commented out you can pull the top bar exact to the screen border. */
-					/* if(delta != bigger && delta != -smaller) { */
-						v1->vec.y+= AREAGRID-1;
-						v1->vec.y-= (v1->vec.y % AREAGRID);
+					v1->vec.y+= AREAGRID-1;
+					v1->vec.y-= (v1->vec.y % AREAGRID);
 					
-					/* } */
+					/* prevent too small top header */
+					if(v1->vec.y > G.curscreen->sizey-HEADERY)
+						v1->vec.y= G.curscreen->sizey-HEADERY;
 				}
 			}
 			v1->flag= 0;
@@ -2988,24 +3501,20 @@ void draw_area_emboss(ScrArea *sa)
 	
 	/* for test */
 	if(FALSE && sa->spacetype==SPACE_VIEW3D) {
-	cpack(0xA0A0A0);
-	uiSetRoundBox(31);
-	uiRoundBoxEmboss(5.0, 5.0, 25.0, 100.0, 8.0, 0);
+		cpack(0xA0A0A0);
+		uiSetRoundBox(31);
+		uiRoundBoxEmboss(5.0, 5.0, 25.0, 100.0, 8.0, 0);
 
-	glEnable(GL_BLEND);
-	glBlendFunc(GL_SRC_ALPHA,  GL_ONE_MINUS_SRC_ALPHA); 
-	
-	glRasterPos2f(8.0, 10.0);
-	BIF_draw_icon(ICON_MATERIAL_HLT);
-	glRasterPos2f(8.0, 30.0);
-	BIF_draw_icon(ICON_IPO_HLT);
-	glRasterPos2f(8.0, 50.0);
-	BIF_draw_icon(ICON_HOME);
-	glRasterPos2f(8.0, 70.0);
-	BIF_draw_icon(ICON_BORDERMOVE);
-	
-	glBlendFunc(GL_ONE,  GL_ZERO); 
-	glDisable(GL_BLEND);
+		glEnable(GL_BLEND);
+		glBlendFunc(GL_SRC_ALPHA,  GL_ONE_MINUS_SRC_ALPHA); 
+		
+		BIF_draw_icon(8.0, 10.0, ICON_MATERIAL_HLT);
+		BIF_draw_icon(8.0, 30.0, ICON_IPO_HLT);
+		BIF_draw_icon(8.0, 50.0, ICON_HOME);
+		BIF_draw_icon(8.0, 70.0, ICON_BORDERMOVE);
+		
+		glBlendFunc(GL_ONE,  GL_ZERO); 
+		glDisable(GL_BLEND);
 	}
 }
 
@@ -3045,33 +3554,21 @@ void drawscredge_area(ScrArea *sa)
 	
 }
 
-void drawscreen(void)
+/* ********************************* */
+
+/* for depgraph updating, all layers visible in a screen */
+unsigned int screen_view3d_layers(void)
 {
 	ScrArea *sa;
-
-	mywinset(G.curscreen->mainwin);
-	myortho2(-0.375, (float)G.curscreen->sizex-0.375, -0.375, (float)G.curscreen->sizey-0.375);
-
-	sa= G.curscreen->areabase.first;
-	while(sa) {
-		drawscredge_area(sa);
-		sa= sa->next;
+	int layer= 0;
+	
+	for(sa= G.curscreen->areabase.first; sa; sa= sa->next) {
+		if(sa->spacetype==SPACE_VIEW3D)
+			layer |= ((View3D *)sa->spacedata.first)->lay;
 	}
-
-	/* this double draw patch seems to be needed for certain sgi's (octane, indigo2) */
-	glDrawBuffer(GL_FRONT);
-
-	sa= G.curscreen->areabase.first;
-	while(sa) {
-		drawscredge_area(sa);
-		sa= sa->next;
-	}
-
-	glDrawBuffer(GL_BACK);
+	return layer;
 }
 
-/* ********************************* */
-
 bScreen *default_twosplit() 
 {
 	bScreen *sc= addscreen("screen");
