@@ -1,5 +1,5 @@
 /**
- * $Id: editsound.c,v 1.24 2004/12/12 21:13:07 ton Exp $
+ * $Id: editsound.c,v 1.26 2005/10/20 14:41:10 ton Exp $
  *
  * ***** BEGIN GPL/BL DUAL LICENSE BLOCK *****
  *
@@ -175,7 +175,7 @@ void winqreadsoundspace(ScrArea *sa, void *spacedata, BWinEvent *evt)
 			G.v2d->cur.xmin+= dx;
 			G.v2d->cur.xmax-= dx;
 			test_view2d(G.v2d, curarea->winx, curarea->winy);
-
+			view2d_do_locks(curarea, V2D_LOCK_COPY);
 			doredraw= 1;
 			break;
 		case PADMINUS:
@@ -183,7 +183,7 @@ void winqreadsoundspace(ScrArea *sa, void *spacedata, BWinEvent *evt)
 			G.v2d->cur.xmin-= dx;
 			G.v2d->cur.xmax+= dx;
 			test_view2d(G.v2d, curarea->winx, curarea->winy);
-
+			view2d_do_locks(curarea, V2D_LOCK_COPY);
 			doredraw= 1;
 			break;
 		case HOMEKEY:
@@ -452,7 +452,7 @@ void sound_read_wav_data(bSound* sound, PackedFile* pf)
 
 
 /* ugly, but it works (for now) */
-int sound_get_filetype_from_header(bSound* sound, PackedFile* pf)
+static int sound_get_filetype_from_header(bSound* sound, PackedFile* pf)
 {
 	int filetype = SAMPLE_INVALID;
 	int i;
@@ -534,7 +534,7 @@ int sound_get_filetype_from_header(bSound* sound, PackedFile* pf)
 
 
 
-int check_filetype(bSound* sound, PackedFile* pf)
+static int check_filetype(bSound* sound, PackedFile* pf)
 {
 //	char* pdest;
 	sound->sample->type = SAMPLE_INVALID;
