@@ -1,5 +1,5 @@
 /**
- * $Id: filesel.c,v 1.40 2004/11/24 18:02:07 ton Exp $
+ * $Id: filesel.c,v 1.69 2006/01/15 13:30:56 campbellbarton Exp $
  *
  * ***** BEGIN GPL/BL DUAL LICENSE BLOCK *****
  *
@@ -34,10 +34,6 @@
 #include <string.h>
 #include <math.h>
 
-#ifdef HAVE_CONFIG_H
-#include <config.h>
-#endif
-
 #ifdef WIN32
 #include <io.h>
 #include <direct.h>
@@ -61,27 +57,32 @@
 
 #include "IMB_imbuf.h"
 
+#include "DNA_armature_types.h"
+#include "DNA_action_types.h"
 #include "DNA_curve_types.h"
 #include "DNA_image_types.h"
 #include "DNA_ipo_types.h"
-#include "DNA_vfont_types.h"
+#include "DNA_material_types.h"
 #include "DNA_mesh_types.h"
 #include "DNA_object_types.h"
 #include "DNA_texture_types.h"
 #include "DNA_space_types.h"
 #include "DNA_scene_types.h"
 #include "DNA_screen_types.h"
-#include "DNA_view3d_types.h"
 #include "DNA_userdef_types.h"
+#include "DNA_vfont_types.h"
+#include "DNA_view3d_types.h"
 
-#include "BKE_utildefines.h"
-#include "BKE_global.h"
-#include "BKE_main.h"
-#include "BKE_displist.h"
-#include "BKE_library.h"
+#include "BKE_action.h"
+#include "BKE_constraint.h"
 #include "BKE_curve.h"
+#include "BKE_depsgraph.h"
 #include "BKE_font.h"
+#include "BKE_global.h"
+#include "BKE_library.h"
+#include "BKE_main.h"
 #include "BKE_material.h"
+#include "BKE_utildefines.h"
 
 #include "BIF_gl.h"
 #include "BIF_interface.h"
@@ -95,15 +96,17 @@
 #include "BLO_readfile.h"
 
 #include "BDR_editcurve.h"
+#include "BDR_editobject.h"
+
 #include "BPI_script.h"
 #include "BSE_filesel.h"
 #include "BSE_view.h"
 
 #include "mydevice.h"
 #include "blendef.h"
-#include "render.h"
 #include "nla.h"
 
+#include "BIF_fsmenu.h"  /* include ourselves */
 
 #if defined WIN32 || defined __BeOS
 	int fnmatch(){return 0;}
@@ -134,6 +137,8 @@ static void filesel_select_objects(struct SpaceFile *sfile);
 static void active_file_object(struct SpaceFile *sfile);
 static int groupname_to_code(char *group);
 
+extern void countall(void);
+
 /* local globals */
 
 static rcti scrollrct, textrct, bar;
@@ -312,7 +317,13 @@ static int compare_name(const void *a1, const void *a2)
 	}
 	if ((entry1->type & S_IFMT) < (entry2->type & S_IFMT)) return (-1);
 	if ((entry1->type & S_IFMT) > (entry2->type & S_IFMT)) return (1);
-	return (strcasecmp(entry1->relname,entry2->relname));
+	
+	/* make sure "." and ".." are always first */
+	if( strcmp(entry1->relname, ".")==0 ) return (-1);
+	if( strcmp(entry2->relname, ".")==0 ) return (1);
+	if( strcmp(entry1->relname, "..")==0 ) return (-1);
+	
+	return (BLI_strcasecmp(entry1->relname,entry2->relname));
 }
 
 static int compare_date(const void *a1, const void *a2)	
@@ -334,13 +345,15 @@ static int compare_date(const void *a1, const void *a2)
 	if ((entry1->type & S_IFMT) < (entry2->type & S_IFMT)) return (-1);
 	if ((entry1->type & S_IFMT) > (entry2->type & S_IFMT)) return (1);
 
-/*
-	if ( entry1->s.st_ctime < entry2->s.st_ctime) return 1;
-	if ( entry1->s.st_ctime > entry2->s.st_ctime) return -1;
-*/
+	/* make sure "." and ".." are always first */
+	if( strcmp(entry1->relname, ".")==0 ) return (-1);
+	if( strcmp(entry2->relname, ".")==0 ) return (1);
+	if( strcmp(entry1->relname, "..")==0 ) return (-1);
+	
 	if ( entry1->s.st_mtime < entry2->s.st_mtime) return 1;
 	if ( entry1->s.st_mtime > entry2->s.st_mtime) return -1;
-	else return strcasecmp(entry1->relname,entry2->relname);
+	
+	else return BLI_strcasecmp(entry1->relname,entry2->relname);
 }
 
 static int compare_size(const void *a1, const void *a2)	
@@ -362,11 +375,51 @@ static int compare_size(const void *a1, const void *a2)
 	if ((entry1->type & S_IFMT) < (entry2->type & S_IFMT)) return (-1);
 	if ((entry1->type & S_IFMT) > (entry2->type & S_IFMT)) return (1);
 
+	/* make sure "." and ".." are always first */
+	if( strcmp(entry1->relname, ".")==0 ) return (-1);
+	if( strcmp(entry2->relname, ".")==0 ) return (1);
+	if( strcmp(entry1->relname, "..")==0 ) return (-1);
+	
 	if ( entry1->s.st_size < entry2->s.st_size) return 1;
 	if ( entry1->s.st_size > entry2->s.st_size) return -1;
-	else return strcasecmp(entry1->relname,entry2->relname);
+	else return BLI_strcasecmp(entry1->relname,entry2->relname);
 }
 
+static int compare_extension(const void *a1, const void *a2) {
+	const struct direntry *entry1=a1, *entry2=a2;
+	char *sufix1, *sufix2;
+	char *nil="";
+
+	if (!(sufix1= strstr (entry1->relname, ".blend.gz"))) 
+		sufix1= strrchr (entry1->relname, '.');
+	if (!(sufix2= strstr (entry2->relname, ".blend.gz")))
+		sufix2= strrchr (entry2->relname, '.');
+	if (!sufix1) sufix1= nil;
+	if (!sufix2) sufix2= nil;
+
+	/* type is is equal to stat.st_mode */
+
+	if (S_ISDIR(entry1->type)){
+		if (S_ISDIR(entry2->type)==0) return (-1);
+	} else{
+		if (S_ISDIR(entry2->type)) return (1);
+	}
+	if (S_ISREG(entry1->type)){
+		if (S_ISREG(entry2->type)==0) return (-1);
+	} else{
+		if (S_ISREG(entry2->type)) return (1);
+	}
+	if ((entry1->type & S_IFMT) < (entry2->type & S_IFMT)) return (-1);
+	if ((entry1->type & S_IFMT) > (entry2->type & S_IFMT)) return (1);
+	
+	/* make sure "." and ".." are always first */
+	if( strcmp(entry1->relname, ".")==0 ) return (-1);
+	if( strcmp(entry2->relname, ".")==0 ) return (1);
+	if( strcmp(entry1->relname, "..")==0 ) return (-1);
+	if( strcmp(entry2->relname, "..")==0 ) return (-1);
+	
+	return (BLI_strcasecmp(sufix1, sufix2));
+}
 
 /* **************************************** */
 
@@ -378,7 +431,8 @@ void clear_global_filesel_vars()
 
 void filesel_statistics(SpaceFile *sfile, int *totfile, int *selfile, float *totlen, float *sellen)
 {
-	int a, len;
+	double len;
+	int a;
 	
 	*totfile= *selfile= 0;
 	*totlen= *sellen= 0;
@@ -390,11 +444,11 @@ void filesel_statistics(SpaceFile *sfile, int *totfile, int *selfile, float *tot
 			(*totfile) ++;
 
 			len = sfile->filelist[a].s.st_size;
-			(*totlen) += (len/1048576.0); 		
+			(*totlen) += (float)(len/1048576.0); 		
 
 			if(sfile->filelist[a].flags & ACTIVE) {
 				(*selfile) ++;
-				(*sellen) += (len/1048576.0);
+				(*sellen) += (float)(len/1048576.0);
 			}
 		}
 	}
@@ -423,7 +477,7 @@ void checkdir(char *dir)
 		return;
 	}	
 
-	while (start = strstr(dir, "\\..\\")) {
+	while ( (start = strstr(dir, "\\..\\")) ) {
 		eind = start + strlen("\\..\\") - 1;
 		a = start-dir-1;
 		while (a>0) {
@@ -433,17 +487,17 @@ void checkdir(char *dir)
 		strcpy(dir+a,eind);
 	}
 
-	while (start = strstr(dir,"\\.\\")){
+	while ( (start = strstr(dir,"\\.\\")) ){
 		eind = start + strlen("\\.\\") - 1;
 		strcpy(start,eind);
 	}
 
-	while (start = strstr(dir,"\\\\" )){
+	while ( (start = strstr(dir,"\\\\" )) ){
 		eind = start + strlen("\\\\") - 1;
 		strcpy(start,eind);
 	}
 
-	if(a = strlen(dir)){				/* remove the '\\' at the end */
+	if((a = strlen(dir))){				/* remove the '\\' at the end */
 		while(a>0 && dir[a-1] == '\\'){
 			a--;
 			dir[a] = 0;
@@ -531,9 +585,14 @@ void test_flags_file(SpaceFile *sfile)
 					|| BLI_testextensie(file->relname, ".otf")
 					|| BLI_testextensie(file->relname, ".otc")) {
 				file->flags |= FTFONTFILE;			
+			} else if (G.have_libtiff &&
+					(BLI_testextensie(file->relname, ".tif")
+					||	BLI_testextensie(file->relname, ".tiff"))) {
+					file->flags |= IMAGEFILE;			
 			} else if (G.have_quicktime){
 				if(		BLI_testextensie(file->relname, ".jpg")
 					||	BLI_testextensie(file->relname, ".jpeg")
+					||	BLI_testextensie(file->relname, ".hdr")
 					||	BLI_testextensie(file->relname, ".tga")
 					||	BLI_testextensie(file->relname, ".rgb")
 					||	BLI_testextensie(file->relname, ".bmp")
@@ -560,6 +619,7 @@ void test_flags_file(SpaceFile *sfile)
 				}
 			} else { // no quicktime
 				if(BLI_testextensie(file->relname, ".jpg")
+					||	BLI_testextensie(file->relname, ".hdr")
 					||	BLI_testextensie(file->relname, ".tga")
 					||	BLI_testextensie(file->relname, ".rgb")
 					||	BLI_testextensie(file->relname, ".bmp")
@@ -598,8 +658,7 @@ void sort_filelist(SpaceFile *sfile)
 		qsort(sfile->filelist, sfile->totfile, sizeof(struct direntry), compare_size);	
 		break;
 	case FILE_SORTEXTENS:
-		qsort(sfile->filelist, sfile->totfile, sizeof(struct direntry), compare_name);	
-		break;
+		qsort(sfile->filelist, sfile->totfile, sizeof(struct direntry), compare_extension);	
 	}
 	
 	sfile->act= -1;
@@ -665,9 +724,9 @@ void freefilelist(SpaceFile *sfile)
 	if (sfile->filelist==0) return;
 	
 	for(; num>=0; num--){
-		free(sfile->filelist[num].relname);
+		MEM_freeN(sfile->filelist[num].relname);
 		
-		if (sfile->filelist[num].string) free(sfile->filelist[num].string);
+		if (sfile->filelist[num].string) MEM_freeN(sfile->filelist[num].string);
 	}
 	free(sfile->filelist);
 	sfile->filelist= 0;
@@ -688,6 +747,7 @@ static void split_sfile(SpaceFile *sfile, char *s1)
 		else test_flags_file(sfile);
 	}
 	strcpy(sfile->file, file);
+		
 	BLI_make_file_string(G.sce, sfile->dir, dir, "");
 }
 
@@ -703,21 +763,21 @@ void parent(SpaceFile *sfile)
 	dir= sfile->dir;
 	
 #ifdef WIN32
-	if(a = strlen(dir)) {				/* remove all '/' at the end */
+	if( (a = strlen(dir)) ) {				/* remove all '/' at the end */
 		while(dir[a-1] == '\\') {
 			a--;
 			dir[a] = 0;
 			if (a<=0) break;
 		}
 	}
-	if(a = strlen(dir)) {				/* then remove all until '/' */
+	if( (a = strlen(dir)) ) {				/* then remove all until '/' */
 		while(dir[a-1] != '\\') {
 			a--;
 			dir[a] = 0;
 			if (a<=0) break;
 		}
 	}
-	if (a = strlen(dir)) {
+	if( (a = strlen(dir)) ) {
 		if (dir[a-1] != '\\') strcat(dir,"\\");
 	}
 	else if(sfile->type!=FILE_MAIN) strcpy(dir,"\\");
@@ -824,14 +884,15 @@ static void calc_file_rcts(SpaceFile *sfile)
 	if(sfile->collums<1) sfile->collums= 1;
 	else if(sfile->collums>8) sfile->collums= 8;
 
-	if((U.flag & USER_FSCOLLUM)==0) if(sfile->type!=FILE_MAIN) sfile->collums= 1;
+	/* this flag aint yet defined in user menu, needed? */
+//	if((U.flag & USER_FSCOLLUM)==0) sfile->collums= 1;
 	
 	collumwidth= (textrct.xmax-textrct.xmin)/sfile->collums;
 	
 
-	totfile= sfile->totfile + 0.5;
+	totfile= sfile->totfile + 0.5f;
 
-	tot= FILESEL_DY*totfile;
+	tot= (int)(FILESEL_DY*totfile);
 	if(tot) fac= ((float)sfile->collums*(scrollrct.ymax-scrollrct.ymin))/( (float)tot);
 	else fac= 1.0;
 	
@@ -839,22 +900,22 @@ static void calc_file_rcts(SpaceFile *sfile)
 	
 	if(tot) start= ( (float)sfile->ofs)/(totfile);
 	else start= 0.0;
-	if(fac>1.0) fac= 1.0;
+	if(fac>1.0) fac= 1.0f;
 
 	if(start+fac>1.0) {
-		sfile->ofs= ceil((1.0-fac)*totfile);
+		sfile->ofs= (short)ceil((1.0-fac)*totfile);
 		start= ( (float)sfile->ofs)/(totfile);
-		fac= 1.0-start;
+		fac= 1.0f-start;
 	}
 
 	bar.xmin= scrollrct.xmin+2;
 	bar.xmax= scrollrct.xmax-2;
 	h= (scrollrct.ymax-scrollrct.ymin)-4;
-	bar.ymax= scrollrct.ymax-2- start*h;
-	bar.ymin= bar.ymax- fac*h;
+	bar.ymax= (int)(scrollrct.ymax-2- start*h);
+	bar.ymin= (int)(bar.ymax- fac*h);
 
 	pixels_to_ofs= (totfile)/(float)(h+3);
-	page_ofs= fac*totfile;
+	page_ofs= (int)(fac*totfile);
 }
 
 int filescrollselect= 0;
@@ -900,6 +961,7 @@ static void print_line(SpaceFile *sfile, struct direntry *files, int x, int y)
 		linerect(boxcol, x, y);
 	}
 
+	// this is where the little boxes in the file view are being drawn according to the file type
 	if(files->flags & BLENDERFILE) {
 		cpack(0xA0A0);
 		glRects(x-14,  y,  x-8,  y+7);
@@ -1002,7 +1064,8 @@ static int calc_filesel_line(SpaceFile *sfile, int nr, int *valx, int *valy)
 static void set_active_file(SpaceFile *sfile, int act)
 {
 	struct direntry *file;
-	int num, redraw= 0, newflag;
+	int num, redraw= 0;
+	unsigned int newflag;
 	int old=0, newi=0;
 	
 	file= sfile->filelist;
@@ -1036,28 +1099,8 @@ static void set_active_file(SpaceFile *sfile, int act)
 		}
 			
 	}
-	
-	if(redraw==2) {
-		int x, y;
-		
-		glDrawBuffer(GL_FRONT);
-
-		glScissor(curarea->winrct.xmin, curarea->winrct.ymin, curarea->winx-12, curarea->winy);
-
-		if( calc_filesel_line(sfile, old, &x, &y) ) {
-			linerect(0, x, y);
-			print_line(sfile, sfile->filelist+old, x, y);
-		}
-		if( calc_filesel_line(sfile, newi, &x, &y) ) {
-			print_line(sfile, sfile->filelist+newi, x, y);
-		}
-		
-		glScissor(curarea->winrct.xmin, curarea->winrct.ymin, curarea->winx, curarea->winy);
-
-		glFlush();		/* for geforce, to show it in the frontbuffer */
-		glDrawBuffer(GL_BACK);
-	}
-	else if(redraw) {
+	// removed frontbuffer draw here
+	if(redraw) {
 		scrarea_queue_winredraw(curarea);
 	}
 }
@@ -1099,7 +1142,6 @@ static void draw_filetext(SpaceFile *sfile)
 	for(a= sfile->ofs; a<sfile->totfile; a++, files++) {
 	
 		if( calc_filesel_line(sfile, a, &x, &y)==0 ) break;
-		
 		print_line(sfile, files, x, y);
 	}
 
@@ -1125,7 +1167,7 @@ void drawfilespace(ScrArea *sa, void *spacedata)
 	glClearColor(col[0], col[1], col[2], 0.0); 
 	glClear(GL_COLOR_BUFFER_BIT);
 
-	sfile= curarea->spacedata.first;	
+	sfile= sa->spacedata.first;	
 	if(sfile->filelist==NULL) {
 		read_dir(sfile);
 		
@@ -1140,8 +1182,8 @@ void drawfilespace(ScrArea *sa, void *spacedata)
 	else calc_file_rcts(sfile);
 
 	/* HEADER */
-	sprintf(name, "win %d", curarea->win);
-	block= uiNewBlock(&curarea->uiblocks, name, UI_EMBOSS, UI_HELV, curarea->win);
+	sprintf(name, "win %d", sa->win);
+	block= uiNewBlock(&sa->uiblocks, name, UI_EMBOSS, UI_HELV, sa->win);
 	
 	uiSetButLock( sfile->type==FILE_MAIN && sfile->returnfunc, NULL);
 
@@ -1152,14 +1194,22 @@ void drawfilespace(ScrArea *sa, void *spacedata)
 	}
 	else loadbutton= 0;
 
-	uiDefBut(block, TEX,	    1,"",	textrct.xmin, filebuty1, textrct.xmax-textrct.xmin-loadbutton, 21, sfile->file, 0.0, (float)FILE_MAXFILE-1, 0, 0, "");
-	uiDefBut(block, TEX,	    2,"",	textrct.xmin, filebuty2, textrct.xmax-textrct.xmin-loadbutton, 21, sfile->dir, 0.0, (float)FILE_MAXFILE-1, 0, 0, "");
+	uiBlockBeginAlign(block);
+	uiDefBut(block, TEX,2,"",	textrct.xmin, filebuty2, textrct.xmax-textrct.xmin-loadbutton, 21, sfile->dir, 0.0, (float)FILE_MAXFILE-1, 0, 0, "Directory, enter a directory and press enter to create it"); /* Directory input */
 	if(loadbutton) {
 		uiSetCurFont(block, UI_HELV);
 		uiDefBut(block, BUT,	    5, sfile->title,	textrct.xmax-loadbutton, filebuty2, loadbutton, 21, sfile->dir, 0.0, (float)FILE_MAXFILE-1, 0, 0, "");
+	}
+	uiBlockEndAlign(block);
+	
+	uiBlockBeginAlign(block);
+	uiDefBut(block, TEX,1,"",	textrct.xmin, filebuty1, textrct.xmax-textrct.xmin-loadbutton, 21, sfile->file, 0.0, (float)FILE_MAXFILE-1, 0, 0, "File, increment version number with (+/-)"); /* File input */
+	if(loadbutton) {
+		uiSetCurFont(block, UI_HELV);
 		uiDefBut(block, BUT,	    6, "Cancel",	textrct.xmax-loadbutton, filebuty1, loadbutton, 21, sfile->file, 0.0, (float)FILE_MAXFILE-1, 0, 0, "");
 	}
-
+	uiBlockEndAlign(block);
+	
 	menu= fsmenu_build_menu();
 	if(menu[0])	// happens when no .Bfs is there, and first time browse
 		uiDefButS(block, MENU,	3, menu, scrollrct.xmin, filebuty1, scrollrct.xmax-scrollrct.xmin, 21, &sfile->menu, 0, 0, 0, 0, "");
@@ -1173,12 +1223,12 @@ void drawfilespace(ScrArea *sa, void *spacedata)
 	draw_filetext(sfile);
 	
 	/* others diskfree etc ? */
-	scrarea_queue_headredraw(curarea);	
+	scrarea_queue_headredraw(sa);	
 	
 	myortho2(-0.375, (float)(sa->winx)-0.375, -0.375, (float)(sa->winy)-0.375);
 	draw_area_emboss(sa);
 	
-	curarea->win_swap= WIN_BACK_OK;
+	sa->win_swap= WIN_BACK_OK;
 }
 
 static void do_filescroll(SpaceFile *sfile)
@@ -1188,11 +1238,10 @@ static void do_filescroll(SpaceFile *sfile)
 	calc_file_rcts(sfile);
 	
 	filescrollselect= 1;
+	
 	/* for beauty */
-
-	glDrawBuffer(GL_FRONT);
-	draw_filescroll(sfile);
-	glDrawBuffer(GL_BACK);
+	scrarea_do_windraw(curarea);
+	screen_swapbuffers();
 	
 	getmouseco_areawin(mval);
 	oldy= yo= mval[1];
@@ -1223,9 +1272,8 @@ static void do_filescroll(SpaceFile *sfile)
 	filescrollselect= 0;
 
 	/* for beauty */
-	glDrawBuffer(GL_FRONT);
-	draw_filescroll(sfile);
-	glDrawBuffer(GL_BACK);
+	scrarea_do_windraw(curarea);
+	screen_swapbuffers();
 	
 }
 
@@ -1262,8 +1310,7 @@ void activate_fileselect(int type, char *title, char *file, void (*func)(char *)
 	scrarea_queue_winredraw(curarea);
 	
 	/* sometime double, when area already is SPACE_FILE with a different file name */
-	addqueue(curarea->headwin, CHANGED, 1);
-	
+	if(curarea->headwin) addqueue(curarea->headwin, CHANGED, 1);
 
 	name[2]= 0;
 	strcpy(name, file);
@@ -1279,6 +1326,9 @@ void activate_fileselect(int type, char *title, char *file, void (*func)(char *)
 	if(BLI_convertstringcode(name, G.sce, G.scene->r.cfra)) sfile->flag |= FILE_STRINGCODE;
 	else sfile->flag &= ~FILE_STRINGCODE;
 
+	if (U.uiflag & USER_HIDE_DOT)
+		sfile->flag |= FILE_HIDE_DOT;
+
 	if(type==FILE_MAIN) {
 		char *groupname;
 		
@@ -1310,7 +1360,8 @@ void activate_fileselect(int type, char *title, char *file, void (*func)(char *)
 	}
 	else {	/* FILE_BLENDER */
 		split_sfile(sfile, name);	/* test filelist too */
-		
+		checkdir(sfile->dir);
+
 		/* free: filelist and libfiledata became incorrect */
 		if(sfile->libfiledata) BLO_blendhandle_close(sfile->libfiledata);
 		sfile->libfiledata= 0;
@@ -1343,6 +1394,7 @@ void activate_imageselect(int type, char *title, char *file, void (*func)(char *
 	else simasel->mode &= ~IMS_STRINGCODE;
 	
 	BLI_split_dirfile(name, dir, simasel->file);
+	checkdir(simasel->dir);
 	if(strcmp(dir, simasel->dir)!=0) simasel->fase= 0;
 	strcpy(simasel->dir, dir);
 	
@@ -1492,20 +1544,43 @@ static void filesel_execute(SpaceFile *sfile)
 	else if(sfile->returnfunc) {
 		fsmenu_insert_entry(sfile->dir, 1);
 	
-		if(sfile->type==FILE_MAIN) {
-			if (sfile->menup) {
+		if(sfile->type==FILE_MAIN) { /* DATABROWSE */
+			if (sfile->menup) {	/* with value pointing to ID block index */
+				int notfound = 1;
+
+				/*	Need special handling since hiding .* datablocks means that
+					sfile->act is no longer the same as files->nr.
+
+					Also, toggle HIDE_DOT on and off can make sfile->act not longer
+					correct (meaning it doesn't point to the correct item in the filelist.
+					
+					sfile->file is always correct, so first with check if, for the item
+					corresponding to sfile->act, the name is the same.
+
+					If it isn't (or if sfile->act is not good), go over filelist and take
+					the correct one.
+
+					This means that selecting a datablock than hiding it makes it
+					unselectable. Not really a problem.
+
+					- theeth
+				 */
+
+				*sfile->menup= -1;
+
 				if(sfile->act>=0) {
 					if(sfile->filelist) {
 						files= sfile->filelist+sfile->act;
-						*sfile->menup= files->nr;
+						if ( strcmp(files->relname, sfile->file)==0) {
+							notfound = 0;
+							*sfile->menup= files->nr;
+						}
 					}	
-					else *sfile->menup= sfile->act+1;
 				}
-				else {
-					*sfile->menup= -1;
+				if (notfound) {
 					for(a=0; a<sfile->totfile; a++) {
 						if( strcmp(sfile->filelist[a].relname, sfile->file)==0) {
-							*sfile->menup= a+1;
+							*sfile->menup= sfile->filelist[a].nr;
 							break;
 						}
 					}
@@ -1804,14 +1879,6 @@ void winqreadfilespace(ScrArea *sa, void *spacedata, BWinEvent *evt)
 	sfile= curarea->spacedata.first;
 	if(sfile==0) return;
 	if(sfile->filelist==0) {
-		/* but do buttons */
-		if(val && event==LEFTMOUSE) {
-			/* FrontbufferButs(TRUE); */
-			/* event= DoButtons(); */
-			/* FrontbufferButs(FALSE); */
-					/*  NIET de headerbuttons! */
-			/* if(event) do_filesel_buttons(event, sfile); */
-		}
 		return;
 	}
 	
@@ -1875,13 +1942,6 @@ void winqreadfilespace(ScrArea *sa, void *spacedata, BWinEvent *evt)
 					}
 				}
 			}
-			else {
-				/* FrontbufferButs(TRUE); */
-				/* event= DoButtons(); */
-				/* FrontbufferButs(FALSE); */
-					/*  NOT the headerbuttons! */
-				/* if(event) do_filesel_buttons(event, sfile);	 */
-			}
 			break;
 		case RIGHTMOUSE:
 			act= find_active_file(sfile, mval[0], mval[1]);
@@ -2159,7 +2219,7 @@ static int groupname_to_code(char *group)
 	char buf[32];
 	char *lslash;
 	
-	strcpy(buf, group);
+	BLI_strncpy(buf, group, 31);
 	lslash= BLI_last_slash(buf);
 	if (lslash)
 		lslash[0]= '\0';
@@ -2201,6 +2261,73 @@ static int is_a_library(SpaceFile *sfile, char *dir, char *group)
 	return 1;
 }
 
+/* orange hack... :) */
+static void do_sync_pose(Library *lib)
+{
+	Object *ob, *obt;
+	Base *base;
+	bArmature *arm;
+	
+	/* find the armature and the pose from library */
+	for(ob= G.main->object.first; ob; ob= ob->id.next)
+		if(ob->type==OB_ARMATURE && ob->id.lib==lib)
+			break;
+	
+	if(ob==NULL || ob->pose==NULL) {
+		error("No pose appended");
+		return;
+	}
+	
+	arm= ob->data;
+
+	/* for all visible objects in this scene */
+	for(base= G.scene->base.first; base; base= base->next) {
+		if((base->flag & SELECT)) {
+			obt= base->object;
+			if(obt->type==OB_ARMATURE && obt->pose && ob!=obt) {
+				char str[128];
+				
+				sprintf(str, "Replace Object %s", obt->id.name);
+				if(okee(str)) {
+					bPoseChannel *chan;
+					bArmature *oldarm= obt->data;
+					
+					/* link armature */
+					oldarm->id.us--;
+					obt->data= arm;
+					arm->id.us++;
+					
+					/* link pose */
+					free_pose_channels(obt->pose);
+					MEM_freeN(obt->pose);
+					copy_pose(&obt->pose, ob->pose, 1);
+					
+					/* relink */
+					ob->id.newid= &obt->id;
+					for (chan = obt->pose->chanbase.first; chan; chan=chan->next){
+						relink_constraints(&chan->constraints);
+					}
+					
+					obt->pose->flag |= POSE_RECALC;
+					obt->recalc |= OB_RECALC_DATA;
+				}
+			}
+		}
+	}
+
+	/* prevent saving in file, unlink from scene */
+	for(base= G.scene->base.first; base; base= base->next) {
+		if(base->object==ob)
+			break;
+	}
+	
+	if(base) {
+		free_and_unlink_base(base);
+	}
+
+	DAG_scene_sort(G.scene);	// for accidentally appended other objects
+}
+
 static void do_library_append(SpaceFile *sfile)
 {
 	Library *lib;
@@ -2218,30 +2345,17 @@ static void do_library_append(SpaceFile *sfile)
 		Object *ob;
 		int idcode = groupname_to_code(group);
 		
-		BLO_library_append(sfile, dir, idcode);
+		BLO_library_append(sfile, dir, idcode);	/* warning; if relative, it changes the *dir to relative path */
 
-		/* DISPLISTS */
+		/* DISPLISTS? */
 		ob= G.main->object.first;
-		set_displist_onlyzero(1);
 		while(ob) {
 			if(ob->id.lib) {
-				if(ob->type==OB_FONT) {
-					Curve *cu= ob->data;
-					if(cu->nurb.first==0) text_to_curve(ob, 0);
-				}
-				makeDispList(ob);
+				ob->recalc |= OB_RECALC;
 			}
-			else if(ob->type==OB_MESH && ob->parent && ob->parent->type==OB_LATTICE ) {
-				makeDispList(ob);
-			}
-			
 			ob= ob->id.next;
 		}
-		set_displist_onlyzero(0);
 	
-		/* in sfile->dir is the whole lib name */
-		strcpy(G.lib, sfile->dir);
-		
 		/* and now find the latest append lib file */
 		lib= G.main->library.first;
 		while(lib) {
@@ -2249,15 +2363,25 @@ static void do_library_append(SpaceFile *sfile)
 			lib= lib->id.next;
 		}
 		
-		if((sfile->flag & FILE_LINK)==0) all_local(lib);
+		if(lib) {
+			if(sfile->flag & FILE_SYNCPOSE)
+				do_sync_pose(lib);
+			if((sfile->flag & FILE_LINK)==0) 
+				all_local(lib);
+		}
+		
+		/* in sfile->dir is the whole lib name */
+		strcpy(G.lib, sfile->dir);
+		
 	}
 }
 
 static void library_to_filelist(SpaceFile *sfile)
 {
-	char dir[FILE_MAXDIR], group[24];
-	int ok, i, nnames, idcode;
 	LinkNode *l, *names;
+	int ok, i, nnames, idcode;
+	char filename[FILE_MAXDIR+FILE_MAXFILE];
+	char dir[FILE_MAXDIR], group[24];
 	
 	/* name test */
 	ok= is_a_library(sfile, dir, group);
@@ -2268,10 +2392,13 @@ static void library_to_filelist(SpaceFile *sfile)
 		return;
 	}
 	
+	BLI_strncpy(filename, G.sce, sizeof(filename));	// G.sce = last file loaded, for UI
+	
 	/* there we go */
 	/* for the time being only read filedata when libfiledata==0 */
 	if (sfile->libfiledata==0) {
-		sfile->libfiledata= BLO_blendhandle_from_file(dir);
+		sfile->libfiledata= BLO_blendhandle_from_file(dir);	// this sets G.sce, we dont want it
+		
 		if(sfile->libfiledata==0) return;
 	}
 	
@@ -2291,20 +2418,20 @@ static void library_to_filelist(SpaceFile *sfile)
 	sfile->filelist= malloc(sfile->totfile * sizeof(*sfile->filelist));
 	memset(sfile->filelist, 0, sfile->totfile * sizeof(*sfile->filelist));
 
-	sfile->filelist[0].relname= strdup(".");
+	sfile->filelist[0].relname= BLI_strdup(".");
 	sfile->filelist[0].type |= S_IFDIR;
-	sfile->filelist[1].relname= strdup("..");
+	sfile->filelist[1].relname= BLI_strdup("..");
 	sfile->filelist[1].type |= S_IFDIR;
 		
 	for (i=0, l= names; i<nnames; i++, l= l->next) {
-		char *blockname= l->link;
+		char *blockname= BLI_strdup(l->link); 
 
 		sfile->filelist[i + 2].relname= blockname;
 		if (!idcode)
 			sfile->filelist[i + 2].type |= S_IFDIR;
 	}
 		
-	BLI_linklist_free(names, NULL);
+	BLI_linklist_free(names, free);
 	
 	qsort(sfile->filelist, sfile->totfile, sizeof(struct direntry), compare_name);
 	
@@ -2314,6 +2441,9 @@ static void library_to_filelist(SpaceFile *sfile)
 		if (len > sfile->maxnamelen)
 			sfile->maxnamelen = len;
 	}
+	
+	BLI_strncpy(G.sce, filename, sizeof(filename));	// prevent G.sce to change
+
 }
 
 /* ******************* DATA SELECT ********************* */
@@ -2344,7 +2474,9 @@ static void filesel_select_objects(SpaceFile *sfile)
 			base->flag= base->object->flag;
 			base= base->next;
 		}
+		countall();
 		allqueue(REDRAWVIEW3D, 0);
+		allqueue(REDRAWOOPS, 0);
 	}
 	else if( strcmp(sfile->dir, "Scene/")==0 ) {
 		
@@ -2379,6 +2511,7 @@ static void active_file_object(SpaceFile *sfile)
 					BASACT->flag |= SELECT;
 					sfile->filelist[sfile->act].flags |= ACTIVE;
 					allqueue(REDRAWVIEW3D, 0);
+					allqueue(REDRAWOOPS, 0);
 					scrarea_queue_winredraw(curarea);
 				}
 			}
@@ -2393,6 +2526,10 @@ void main_to_filelist(SpaceFile *sfile)
 	struct direntry *files, *firstlib = NULL;
 	ListBase *lb;
 	int a, fake, idcode, len, ok, totlib, totbl;
+	short hide = 0;
+
+	if (sfile->flag & FILE_HIDE_DOT)
+		hide = 1;
 
 	if(sfile->dir[0]=='/') sfile->dir[0]= 0;
 	
@@ -2404,7 +2541,7 @@ void main_to_filelist(SpaceFile *sfile)
 	if( sfile->dir[0]==0) {
 		
 		/* make directories */
-		sfile->totfile= 22;
+		sfile->totfile= 21;
 		sfile->filelist= (struct direntry *)malloc(sfile->totfile * sizeof(struct direntry));
 		
 		for(a=0; a<sfile->totfile; a++) {
@@ -2412,28 +2549,27 @@ void main_to_filelist(SpaceFile *sfile)
 			sfile->filelist[a].type |= S_IFDIR;
 		}
 		
-		sfile->filelist[0].relname= strdup("..");
-		sfile->filelist[1].relname= strdup(".");
-		sfile->filelist[2].relname= strdup("Scene");
-		sfile->filelist[3].relname= strdup("Object");
-		sfile->filelist[4].relname= strdup("Mesh");
-		sfile->filelist[5].relname= strdup("Curve");
-		sfile->filelist[6].relname= strdup("Metaball");
-		sfile->filelist[7].relname= strdup("Material");
-		sfile->filelist[8].relname= strdup("Texture");
-		sfile->filelist[9].relname= strdup("Image");
-		sfile->filelist[10].relname= strdup("Ika");
-		sfile->filelist[11].relname= strdup("Wave");
-		sfile->filelist[12].relname= strdup("Lattice");
-		sfile->filelist[13].relname= strdup("Lamp");
-		sfile->filelist[14].relname= strdup("Camera");
-		sfile->filelist[15].relname= strdup("Ipo");
-		sfile->filelist[16].relname= strdup("World");
-		sfile->filelist[17].relname= strdup("Screen");
-		sfile->filelist[18].relname= strdup("VFont");
-		sfile->filelist[19].relname= strdup("Text");
-		sfile->filelist[20].relname= strdup("Armature");
-		sfile->filelist[21].relname= strdup("Action");
+		sfile->filelist[0].relname= BLI_strdup("..");
+		sfile->filelist[1].relname= BLI_strdup(".");
+		sfile->filelist[2].relname= BLI_strdup("Scene");
+		sfile->filelist[3].relname= BLI_strdup("Object");
+		sfile->filelist[4].relname= BLI_strdup("Mesh");
+		sfile->filelist[5].relname= BLI_strdup("Curve");
+		sfile->filelist[6].relname= BLI_strdup("Metaball");
+		sfile->filelist[7].relname= BLI_strdup("Material");
+		sfile->filelist[8].relname= BLI_strdup("Texture");
+		sfile->filelist[9].relname= BLI_strdup("Image");
+		sfile->filelist[10].relname= BLI_strdup("Wave");
+		sfile->filelist[11].relname= BLI_strdup("Lattice");
+		sfile->filelist[12].relname= BLI_strdup("Lamp");
+		sfile->filelist[13].relname= BLI_strdup("Camera");
+		sfile->filelist[14].relname= BLI_strdup("Ipo");
+		sfile->filelist[15].relname= BLI_strdup("World");
+		sfile->filelist[16].relname= BLI_strdup("Screen");
+		sfile->filelist[17].relname= BLI_strdup("VFont");
+		sfile->filelist[18].relname= BLI_strdup("Text");
+		sfile->filelist[19].relname= BLI_strdup("Armature");
+		sfile->filelist[20].relname= BLI_strdup("Action");
 		qsort(sfile->filelist, sfile->totfile, sizeof(struct direntry), compare_name);
 	}
 	else {
@@ -2447,12 +2583,12 @@ void main_to_filelist(SpaceFile *sfile)
 		id= lb->first;
 		sfile->totfile= 0;
 		while(id) {
-			
 			if(sfile->returnfunc && idcode==ID_IP) {
 				if(sfile->ipotype== ((Ipo *)id)->blocktype) sfile->totfile++;
 			}
-			else sfile->totfile++;
-			
+			else if (hide==0 || id->name[2] != '.')
+				sfile->totfile++;
+
 			id= id->next;
 		}
 		
@@ -2463,10 +2599,10 @@ void main_to_filelist(SpaceFile *sfile)
 		
 		if(sfile->returnfunc==0) {
 			memset( &(sfile->filelist[0]), 0 , sizeof(struct direntry));
-			sfile->filelist[0].relname= strdup(".");
+			sfile->filelist[0].relname= BLI_strdup(".");
 			sfile->filelist[0].type |= S_IFDIR;
 			memset( &(sfile->filelist[1]), 0 , sizeof(struct direntry));
-			sfile->filelist[1].relname= strdup("..");
+			sfile->filelist[1].relname= BLI_strdup("..");
 			sfile->filelist[1].type |= S_IFDIR;
 		
 			files+= 2;
@@ -2485,32 +2621,34 @@ void main_to_filelist(SpaceFile *sfile)
 			
 			if(ok) {
 		
-				memset( files, 0 , sizeof(struct direntry));
-				files->relname= strdup(id->name+2);
-				
-				if(sfile->returnfunc==0) { /* F4 DATA BROWSE */
-					if(idcode==ID_OB) {
-						if( ((Object *)id)->flag & SELECT) files->flags |= ACTIVE;
+				if (hide==0 || id->name[2] != '.') {
+					memset( files, 0 , sizeof(struct direntry));
+					files->relname= BLI_strdup(id->name+2);
+					
+					if(sfile->returnfunc==0) { /* F4 DATA BROWSE */
+						if(idcode==ID_OB) {
+							if( ((Object *)id)->flag & SELECT) files->flags |= ACTIVE;
+						}
+						else if(idcode==ID_SCE) {
+							if( ((Scene *)id)->r.scemode & R_BG_RENDER) files->flags |= ACTIVE;
+						}
 					}
-					else if(idcode==ID_SCE) {
-						if( ((Scene *)id)->r.scemode & R_BG_RENDER) files->flags |= ACTIVE;
+					files->nr= totbl+1;
+					files->poin= id;
+					fake= id->flag & LIB_FAKEUSER;
+					
+					if(id->lib && fake) sprintf(files->extra, "LF %d", id->us);
+					else if(id->lib) sprintf(files->extra, "L    %d", id->us);
+					else if(fake) sprintf(files->extra, "F    %d", id->us);
+					else sprintf(files->extra, "      %d", id->us);
+					
+					if(id->lib) {
+						if(totlib==0) firstlib= files;
+						totlib++;
 					}
+					
+					files++;
 				}
-				files->nr= totbl+1;
-				files->poin= id;
-				fake= id->flag & LIB_FAKEUSER;
-				
-				if(id->lib && fake) sprintf(files->extra, "LF %d", id->us);
-				else if(id->lib) sprintf(files->extra, "L    %d", id->us);
-				else if(fake) sprintf(files->extra, "F    %d", id->us);
-				else sprintf(files->extra, "      %d", id->us);
-				
-				if(id->lib) {
-					if(totlib==0) firstlib= files;
-					totlib++;
-				}
-				
-				files++;
 				totbl++;
 			}
 			
