@@ -1,5 +1,5 @@
 /**
- * $Id: ghostwinlay.c,v 1.39 2005/05/04 08:46:14 ton Exp $
+ * $Id: ghostwinlay.c,v 1.41 2005/11/28 13:50:44 lukep Exp $
  *
  * ***** BEGIN GPL/BL DUAL LICENSE BLOCK *****
  *
@@ -359,6 +359,8 @@ Window *window_open(char *title, int posx, int posy, int sizex, int sizey, int s
 			
 			win->lmouse[0]= win->size[0]/2;
 			win->lmouse[1]= win->size[1]/2;
+			
+			
 		} else {
 			GHOST_DisposeWindow(g_system, ghostwin);
 		}
@@ -409,7 +411,15 @@ void window_set_cursor(Window *win, int curs) {
 		GHOST_SetCursorVisibility(win->ghostwin, 0);
 	} else {
 		GHOST_SetCursorVisibility(win->ghostwin, 1);
-		GHOST_SetCursorShape(win->ghostwin, convert_cursor(curs));
+		
+		/* detect if we use system cursor or Blender cursor */
+		switch(curs) {
+			case CURSOR_VPAINT:
+				SetBlenderCursor(BC_KNIFECURSOR);
+				break;
+			default:
+				GHOST_SetCursorShape(win->ghostwin, convert_cursor(curs));
+		}
 	}
 }
 
