@@ -4,7 +4,7 @@
  * Functions to draw the "Action Editor" window header
  * and handle user events sent to it.
  * 
- * $Id: header_action.c,v 1.20 2004/10/31 10:52:38 ton Exp $
+ * $Id: header_action.c,v 1.39 2005/12/18 18:05:10 broken Exp $
  *
  * ***** BEGIN GPL/BL DUAL LICENSE BLOCK *****
  *
@@ -43,40 +43,32 @@
 #include <config.h>
 #endif
 
-#ifdef WIN32
-#include "BLI_winstuff.h"
-#endif
-
-#include "BMF_Api.h"
-#include "BIF_language.h"
-#ifdef INTERNATIONAL
-#include "FTF_Api.h"
-#endif
-
-#include "DNA_ID.h"
 #include "DNA_action_types.h"
+#include "DNA_curve_types.h"
+#include "DNA_ID.h"
 #include "DNA_ipo_types.h"
+#include "DNA_key_types.h"
 #include "DNA_object_types.h"
 #include "DNA_scene_types.h"
 #include "DNA_screen_types.h"
 #include "DNA_space_types.h"
-#include "DNA_key_types.h"
-#include "DNA_curve_types.h"
 
+#include "BIF_editaction.h"
 #include "BIF_interface.h"
+#include "BIF_poseobject.h"
 #include "BIF_resources.h"
 #include "BIF_screen.h"
 #include "BIF_space.h"
-#include "BIF_editaction.h"
 
 #include "BKE_action.h"
+#include "BKE_armature.h"
+#include "BKE_constraint.h"
+#include "BKE_depsgraph.h"
 #include "BKE_global.h"
 #include "BKE_main.h"
-#include "BKE_constraint.h"
-#include "BKE_armature.h"
+#include "BKE_utildefines.h"
 
 #include "BSE_drawipo.h"
-#include "BSE_editaction.h"
 #include "BSE_headerbuttons.h"
 
 #include "nla.h"
@@ -90,6 +82,7 @@
 #define ACTMENU_VIEW_PLAYALL    3
 #define ACTMENU_VIEW_ALL        4
 #define ACTMENU_VIEW_MAXIMIZE   5
+#define ACTMENU_VIEW_LOCK		6
 
 #define ACTMENU_SEL_BORDER      0
 #define ACTMENU_SEL_ALL_KEYS    1
@@ -98,9 +91,16 @@
 #define ACTMENU_KEY_DUPLICATE     0
 #define ACTMENU_KEY_DELETE        1
 #define ACTMENU_KEY_BAKE          2
+#define ACTMENU_KEY_SNAP          3
+
+#define ACTMENU_KEY_CHANPOS_MOVE_CHANNEL_UP		0
+#define ACTMENU_KEY_CHANPOS_MOVE_CHANNEL_DOWN	1
+#define ACTMENU_KEY_CHANPOS_MOVE_CHANNEL_TOP	2
+#define ACTMENU_KEY_CHANPOS_MOVE_CHANNEL_BOTTOM	3
 
 #define ACTMENU_KEY_TRANSFORM_MOVE	0
 #define ACTMENU_KEY_TRANSFORM_SCALE	1
+#define ACTMENU_KEY_TRANSFORM_SLIDE	2
 
 #define ACTMENU_KEY_HANDLE_AUTO   0
 #define ACTMENU_KEY_HANDLE_ALIGN  1
@@ -111,12 +111,18 @@
 #define ACTMENU_KEY_INTERP_LINEAR 1
 #define ACTMENU_KEY_INTERP_BEZIER 2
 
+#define ACTMENU_KEY_EXTEND_CONST 0
+#define ACTMENU_KEY_EXTEND_EXTRAPOLATION 1
+#define ACTMENU_KEY_EXTEND_CYCLIC 2
+#define ACTMENU_KEY_EXTEND_CYCLICEXTRAPOLATION 3
+
 void do_action_buttons(unsigned short event)
 {
+	Object *ob= OBACT;
 
 	switch(event){
 		case B_ACTBAKE:
-			bake_action_with_client (G.saction->action, OBACT, 0.01);
+			bake_action_with_client (G.saction->action, ob, 0.01);
 			break;
 		case B_ACTCONT:
 			set_exprap_action(IPO_HORIZ);
@@ -143,21 +149,27 @@ void do_action_buttons(unsigned short event)
 			}
 			else {
 				float extra;
-				G.v2d->cur.xmin= calc_action_start(G.saction->action);
-				G.v2d->cur.xmax= calc_action_end(G.saction->action);
+				
+				calc_action_range(G.saction->action, &G.v2d->cur.xmin, &G.v2d->cur.xmax);
+				if(G.saction->pin==0 && ob) {
+					G.v2d->cur.xmin= get_action_frame_inv(ob, G.v2d->cur.xmin);
+					G.v2d->cur.xmax= get_action_frame_inv(ob, G.v2d->cur.xmax);
+				}				
 				extra= 0.05*(G.v2d->cur.xmax - G.v2d->cur.xmin);
 				G.v2d->cur.xmin-= extra;
 				G.v2d->cur.xmax+= extra;
-				
+
 				if(G.v2d->cur.xmin==G.v2d->cur.xmax) {
 					G.v2d->cur.xmax= -5;
 					G.v2d->cur.xmax= 100;
 				}
 			}
-
+			G.v2d->cur.ymin= 0.0f;
+			G.v2d->cur.ymax= 1000.0f;
+			
 			G.v2d->tot= G.v2d->cur;
 			test_view2d(G.v2d, curarea->winx, curarea->winy);
-
+			view2d_do_locks(curarea, V2D_LOCK_COPY);
 
 			addqueue (curarea->win, REDRAW, 1);
 
@@ -168,14 +180,10 @@ void do_action_buttons(unsigned short event)
 			break;
 		case B_ACTPASTE:
 			paste_posebuf(0);
-			clear_object_constraint_status(OBACT);
-			make_displists_by_armature(OBACT);
 			allqueue(REDRAWVIEW3D, 1);
 			break;
 		case B_ACTPASTEFLIP:
 			paste_posebuf(1);
-			clear_object_constraint_status(OBACT);
-			make_displists_by_armature(OBACT);
 			allqueue(REDRAWVIEW3D, 1);
 			break;
 
@@ -219,6 +227,11 @@ static void do_action_viewmenu(void *arg, int event)
 		case ACTMENU_VIEW_ALL: /* View All */
 			do_action_buttons(B_ACTHOME);
 			break;
+		case ACTMENU_VIEW_LOCK:
+			G.v2d->flag ^= V2D_VIEWLOCK;
+			if(G.v2d->flag & V2D_VIEWLOCK)
+				view2d_do_locks(curarea, 0);
+			break;
 		case ACTMENU_VIEW_MAXIMIZE: /* Maximize Window */
 			/* using event B_FULL */
 			break;
@@ -243,21 +256,13 @@ static uiBlock *action_viewmenu(void *arg_unused)
 	uiDefBut(block, SEPR, 0, "", 0, yco-=6, 
 			 menuwidth, 6, NULL, 0.0, 0.0, 0, 0, "");
 	
-	if(BTST(G.saction->lock, 0)) {
-		uiDefIconTextBut(block, BUTM, 1, ICON_CHECKBOX_HLT, 
-						 "Update Automatically|", 0, yco-=20, 
-						 menuwidth, 19, NULL, 0.0, 0.0, 1, 
-						 ACTMENU_VIEW_AUTOUPDATE, "");
-	} 
-	else {
-		uiDefIconTextBut(block, BUTM, 1, ICON_CHECKBOX_DEHLT, 
-						 "Update Automatically|", 0, yco-=20, 
-						 menuwidth, 19, NULL, 0.0, 0.0, 1, 
-						 ACTMENU_VIEW_AUTOUPDATE, "");
-	}
+	uiDefIconTextBut(block, BUTM, 1, BTST(G.saction->lock, 0)?ICON_CHECKBOX_HLT:ICON_CHECKBOX_DEHLT, 
+					 "Update Automatically|", 0, yco-=20, 
+					 menuwidth, 19, NULL, 0.0, 0.0, 1, 
+					 ACTMENU_VIEW_AUTOUPDATE, "");
 
 	uiDefBut(block, SEPR, 0, "", 0, yco-=6, 
-			 menuwidth, 6, NULL, 0.0, 0.0, 0, 0, "");
+					menuwidth, 6, NULL, 0.0, 0.0, 0, 0, "");
 	
 	uiDefIconTextBut(block, BUTM, 1, ICON_BLANK1, 
 					 "Play Back Animation|Alt A", 0, yco-=20, 
@@ -276,6 +281,11 @@ static uiBlock *action_viewmenu(void *arg_unused)
 					 menuwidth, 19, NULL, 0.0, 0.0, 1, 
 					 ACTMENU_VIEW_ALL, "");
 		
+	uiDefIconTextBut(block, BUTM, 1, (G.v2d->flag & V2D_VIEWLOCK)?ICON_CHECKBOX_HLT:ICON_CHECKBOX_DEHLT, 
+					 "Lock Time to Other Windows|", 0, yco-=20, 
+					 menuwidth, 19, NULL, 0.0, 0.0, 1, 
+					 ACTMENU_VIEW_LOCK, "");
+	
 	if (!curarea->full) 
 		uiDefIconTextBut(block, BUTM, B_FULL, ICON_BLANK1, 
 						 "Maximize Window|Ctrl UpArrow", 0, yco-=20, 
@@ -404,7 +414,7 @@ static void do_action_keymenu_transformmenu(void *arg, int event)
 				transform_meshchannel_keys('g', key);
 			} 
 			else if (act) {	
-				transform_actionchannel_keys ('g');
+				transform_actionchannel_keys ('g', 0);
 			}
 			break;
 		case ACTMENU_KEY_TRANSFORM_SCALE:
@@ -412,7 +422,15 @@ static void do_action_keymenu_transformmenu(void *arg, int event)
 				transform_meshchannel_keys('s', key);
 			} 
 			else if (act) {
-				transform_actionchannel_keys ('s');
+				transform_actionchannel_keys ('s', 0);
+			}
+			break;
+		case ACTMENU_KEY_TRANSFORM_SLIDE:
+			if (key) {
+				//transform_meshchannel_keys('t', key);
+			} 
+			else if (act) {
+				transform_actionchannel_keys ('t', 0);
 			}
 			break;
 	}
@@ -430,14 +448,16 @@ static uiBlock *action_keymenu_transformmenu(void *arg_unused)
 	uiBlockSetButmFunc(block, do_action_keymenu_transformmenu, NULL);
 
 	uiDefIconTextBut(block, BUTM, 1, ICON_BLANK1, 
-					 "Grab/Move|G", 0, yco-=20, 
-					 menuwidth, 19, NULL, 0.0, 0.0, 0, 
+					 "Grab/Move|G", 0, yco-=20, menuwidth, 19, NULL, 0.0, 0.0, 0,  
 					 ACTMENU_KEY_TRANSFORM_MOVE, "");
 	uiDefIconTextBut(block, BUTM, 1, ICON_BLANK1, 
-					 "Scale|S", 0, yco-=20, 
-					 menuwidth, 19, NULL, 0.0, 0.0, 0, 
+					 "Scale|S", 0, yco-=20, menuwidth, 19, NULL, 0.0, 0.0, 0, 
 					 ACTMENU_KEY_TRANSFORM_SCALE, "");
-
+	uiDefIconTextBut(block, BUTM, 1, ICON_BLANK1, 
+					 "Time Slide|T", 0, yco-=20, menuwidth, 19, NULL, 0.0, 0.0, 0, 
+					 ACTMENU_KEY_TRANSFORM_SLIDE, "");
+	
+	
 	uiBlockSetDirection(block, UI_RIGHT);
 	uiTextBoundsBlock(block, 60);
 
@@ -577,6 +597,140 @@ static uiBlock *action_keymenu_intpolmenu(void *arg_unused)
 	return block;
 }
 
+static void do_action_keymenu_extendmenu(void *arg, int event)
+{
+	Key *key;
+
+	key = get_action_mesh_key();
+
+	switch(event)
+	{
+		case ACTMENU_KEY_EXTEND_CONST:
+			if (key) {
+				/* to do */
+			}
+			else {
+				set_extendtype_actionchannels(SET_EXTEND_CONSTANT);
+			}
+			break;
+		case ACTMENU_KEY_EXTEND_EXTRAPOLATION:
+			if (key) {
+				/* to do */
+			}
+			else {
+				set_extendtype_actionchannels(SET_EXTEND_EXTRAPOLATION);
+			}
+			break;
+		case ACTMENU_KEY_EXTEND_CYCLIC:
+			if (key) {
+				/* to do */
+			}
+			else {
+				set_extendtype_actionchannels(SET_EXTEND_CYCLIC);
+			}
+			break;
+		case ACTMENU_KEY_EXTEND_CYCLICEXTRAPOLATION:
+			if (key) {
+				/* to do */
+			}
+			else {
+				set_extendtype_actionchannels(SET_EXTEND_CYCLICEXTRAPOLATION);
+			}
+			break;
+	}
+
+	scrarea_queue_winredraw(curarea);
+}
+
+static uiBlock *action_keymenu_extendmenu(void *arg_unused)
+{
+	uiBlock *block;
+	short yco= 0, menuwidth=120;
+
+	block= uiNewBlock(&curarea->uiblocks, "action_keymenu_extendmenu", 
+					  UI_EMBOSSP, UI_HELV, G.curscreen->mainwin);
+	uiBlockSetButmFunc(block, do_action_keymenu_extendmenu, NULL);
+
+	uiDefIconTextBut(block, BUTM, 1, ICON_BLANK1, 
+					 "Constant", 0, yco-=20, 
+					 menuwidth, 19, NULL, 0.0, 0.0, 0, 
+					 ACTMENU_KEY_EXTEND_CONST, "");
+	uiDefIconTextBut(block, BUTM, 1, ICON_BLANK1, 
+					 "Extrapolation", 0, yco-=20, 
+					 menuwidth, 19, NULL, 0.0, 0.0, 0, 
+					 ACTMENU_KEY_EXTEND_EXTRAPOLATION, "");
+	uiDefIconTextBut(block, BUTM, 1, ICON_BLANK1, 
+					 "Cyclic", 0, yco-=20, 
+					 menuwidth, 19, NULL, 0.0, 0.0, 0, 
+					 ACTMENU_KEY_EXTEND_CYCLIC, "");
+	uiDefIconTextBut(block, BUTM, 1, ICON_BLANK1, 
+					 "Cyclic Extrapolation", 0, yco-=20, 
+					 menuwidth, 19, NULL, 0.0, 0.0, 0, 
+					 ACTMENU_KEY_EXTEND_CYCLICEXTRAPOLATION, "");
+
+	uiBlockSetDirection(block, UI_RIGHT);
+	uiTextBoundsBlock(block, 60);
+
+	return block;
+}
+
+static void do_action_keymenu_chanposmenu(void *arg, int event)
+{
+	switch(event)
+	{
+		case ACTMENU_KEY_CHANPOS_MOVE_CHANNEL_DOWN:
+			down_sel_action();
+			break;
+		case ACTMENU_KEY_CHANPOS_MOVE_CHANNEL_UP:
+			up_sel_action();
+			break;
+		case ACTMENU_KEY_CHANPOS_MOVE_CHANNEL_TOP:
+			top_sel_action();
+			break;
+		case ACTMENU_KEY_CHANPOS_MOVE_CHANNEL_BOTTOM:
+			bottom_sel_action();
+			break;
+	}
+
+	scrarea_queue_winredraw(curarea);
+}
+
+static uiBlock *action_keymenu_chanposmenu(void *arg_unused)
+{
+	uiBlock *block;
+	short yco= 0, menuwidth=120;
+
+	block= uiNewBlock(&curarea->uiblocks, "action_keymenu_chanposmenu", 
+					  UI_EMBOSSP, UI_HELV, G.curscreen->mainwin);
+	uiBlockSetButmFunc(block, do_action_keymenu_chanposmenu, NULL);
+
+	uiDefIconTextBut(block, BUTM, 1, ICON_BLANK1, 
+					 "Move Up|Page Up", 0, yco-=20, 
+					 menuwidth, 19, NULL, 0.0, 0.0, 0, 
+					 ACTMENU_KEY_CHANPOS_MOVE_CHANNEL_UP, "");
+	uiDefIconTextBut(block, BUTM, 1, ICON_BLANK1, 
+					 "Move Down|Page Down", 0, yco-=20, 
+					 menuwidth, 19, NULL, 0.0, 0.0, 0, 
+					 ACTMENU_KEY_CHANPOS_MOVE_CHANNEL_DOWN, "");
+	
+	uiDefBut(block, SEPR, 0, "", 0, yco-=6, 
+					menuwidth, 6, NULL, 0.0, 0.0, 0, 0, "");
+					
+	uiDefIconTextBut(block, BUTM, 1, ICON_BLANK1, 
+					 "Move to Top|Shift Page Up", 0, yco-=20, 
+					 menuwidth, 19, NULL, 0.0, 0.0, 0, 
+					 ACTMENU_KEY_CHANPOS_MOVE_CHANNEL_TOP, "");
+	uiDefIconTextBut(block, BUTM, 1, ICON_BLANK1, 
+					 "Move to Bottom|Shift Page Down", 0, yco-=20, 
+					 menuwidth, 19, NULL, 0.0, 0.0, 0, 
+					 ACTMENU_KEY_CHANPOS_MOVE_CHANNEL_BOTTOM, "");
+
+	uiBlockSetDirection(block, UI_RIGHT);
+	uiTextBoundsBlock(block, 60);
+
+	return block;
+}
+
 static void do_action_keymenu(void *arg, int event)
 {	
 	SpaceAction *saction;
@@ -613,6 +767,9 @@ static void do_action_keymenu(void *arg, int event)
 		case ACTMENU_KEY_BAKE:
 			bake_action_with_client (G.saction->action, OBACT, 0.01);
 			break;
+		case ACTMENU_KEY_SNAP:
+			set_snap_actionchannels();
+			break;
 	}
 }
 
@@ -626,9 +783,12 @@ static uiBlock *action_keymenu(void *arg_unused)
 	uiBlockSetButmFunc(block, do_action_keymenu, NULL);
 	
 	uiDefIconTextBlockBut(block, action_keymenu_transformmenu, 
-					NULL, ICON_RIGHTARROW_THIN, 
-					"Transform", 0, yco-=20, 120, 20, "");
-						  
+						  NULL, ICON_RIGHTARROW_THIN, "Transform", 0, yco-=20, 120, 20, "");
+	
+	uiDefIconTextBut(block, BUTM, 1, ICON_BLANK1, 
+					 "Snap to Frame|Shift S", 0, yco-=20, 
+					 menuwidth, 19, NULL, 0.0, 0.0, 0, ACTMENU_KEY_SNAP, "");
+	
 	uiDefBut(block, SEPR, 0, "", 0, yco-=6, 
 					menuwidth, 6, NULL, 0.0, 0.0, 0, 0, "");
 
@@ -636,6 +796,7 @@ static uiBlock *action_keymenu(void *arg_unused)
 					"Duplicate|Shift D", 0, yco-=20, 
 					menuwidth, 19, NULL, 0.0, 0.0, 0, 
 					ACTMENU_KEY_DUPLICATE, "");
+	
 	uiDefIconTextBut(block, BUTM, 1, ICON_BLANK1, 
 					"Delete|X", 0, yco-=20, 
 					menuwidth, 19, NULL, 0.0, 0.0, 0, 
@@ -652,13 +813,24 @@ static uiBlock *action_keymenu(void *arg_unused)
 	uiDefBut(block, SEPR, 0, "", 0, yco-=6, 
 			 menuwidth, 6, NULL, 0.0, 0.0, 0, 0, "");
 
-	uiDefIconTextBlockBut(block, action_keymenu_intpolmenu, 
-						  NULL, ICON_RIGHTARROW_THIN, 
-						  "Interpolation Mode", 0, yco-=20, 120, 20, "");
 	uiDefIconTextBlockBut(block, action_keymenu_handlemenu, 
 						  NULL, ICON_RIGHTARROW_THIN, 
 						  "Handle Type", 0, yco-=20, 120, 20, "");
 	
+	uiDefBut(block, SEPR, 0, "", 0, yco-=6, 
+			 menuwidth, 6, NULL, 0.0, 0.0, 0, 0, "");
+
+	uiDefIconTextBlockBut(block, action_keymenu_extendmenu, 
+						  NULL, ICON_RIGHTARROW_THIN, 
+						  "Extend Mode", 0, yco-=20, 120, 20, "");
+	uiDefIconTextBlockBut(block, action_keymenu_intpolmenu, 
+						  NULL, ICON_RIGHTARROW_THIN, 
+						  "Interpolation Mode", 0, yco-=20, 120, 20, "");
+	uiDefIconTextBlockBut(block, action_keymenu_chanposmenu, 
+						  NULL, ICON_RIGHTARROW_THIN, 
+						  "Channel Ordering", 0, yco-=20, 120, 20, "");
+
+	
 	if(curarea->headertype==HEADERTOP) {
 		uiBlockSetDirection(block, UI_DOWN);
 	}
@@ -714,14 +886,14 @@ void action_buttons(void)
 
 	uiBlockSetEmboss(block, UI_EMBOSSN);
 	if (curarea->flag & HEADER_NO_PULLDOWN) {
-		uiDefIconButS(block, TOG|BIT|0, B_FLIPINFOMENU, 
+		uiDefIconButBitS(block, TOG, HEADER_NO_PULLDOWN, B_FLIPINFOMENU, 
 					  ICON_DISCLOSURE_TRI_RIGHT,
 					  xco,2,XIC,YIC-2,
 					  &(curarea->flag), 0, 0, 0, 0, 
 					  "Show pulldown menus");
 	}
 	else {
-		uiDefIconButS(block, TOG|BIT|0, B_FLIPINFOMENU, 
+		uiDefIconButBitS(block, TOG, HEADER_NO_PULLDOWN, B_FLIPINFOMENU, 
 					  ICON_DISCLOSURE_TRI_DOWN,
 					  xco,2,XIC,YIC-2,
 					  &(curarea->flag), 0, 0, 0, 0, 
@@ -750,32 +922,26 @@ void action_buttons(void)
 		xco+= xmax;
 	}
 
-	uiBlockSetEmboss(block, UI_EMBOSSX);
+	uiBlockSetEmboss(block, UI_EMBOSS);
 	
-	// object action is allowed to be zero!
-	/* (ton) commented out below line, since people can apparently link Action to any object (mesh) and
-		not unlink anymore when theres a mesh key. Needs to be rethought this stuff! */
-	//if (!get_action_mesh_key()) {
+	/* NAME ETC */
+	ob=OBACT;
+	from = (ID*) ob;
+
+	xco= std_libbuttons(block, xco, 0, B_ACTPIN, &G.saction->pin, 
+						B_ACTIONBROWSE, (ID*)G.saction->action, 
+						from, &(G.saction->actnr), B_ACTALONE, 
+						B_ACTLOCAL, B_ACTIONDELETE, 0, 0);	
+
+	/* Draw action baker */
+	xco+= 8;
 	
-		/* NAME ETC */
-		ob=OBACT;
-		from = (ID*) ob;
+	uiDefBut(block, BUT, B_ACTBAKE, 
+			 "Bake", xco, 0, 64, YIC, 0, 0, 0, 0, 0, 
+			 "Create an action with the constraint effects "
+			 "converted into Ipo keys");
+	xco+=64;
 
-		xco= std_libbuttons(block, xco, 0, B_ACTPIN, &G.saction->pin, 
-							B_ACTIONBROWSE, (ID*)G.saction->action, 
-							from, &(G.saction->actnr), B_ACTALONE, 
-							B_ACTLOCAL, B_ACTIONDELETE, 0, 0);	
-
-		/* Draw action baker */
-		xco+= 8;
-		
-		uiDefBut(block, BUT, B_ACTBAKE, 
-				 "Bake", xco, 0, 64, YIC, 0, 0, 0, 0, 0, 
-				 "Create an action with the constraint effects "
-				 "converted into Ipo keys");
-		xco+=64;
-
-	//}
 	uiClearButLock();
 
 	/* draw LOCK */
