@@ -4,7 +4,7 @@
  * Functions to draw the "User Preferences" window header
  * and handle user events sent to it.
  * 
- * $Id: header_info.c,v 1.50 2004/11/11 23:58:05 ton Exp $
+ * $Id: header_info.c,v 1.76 2006/01/06 03:46:50 erwin Exp $
  *
  * ***** BEGIN GPL/BL DUAL LICENSE BLOCK *****
  *
@@ -43,16 +43,6 @@
 #include <config.h>
 #endif
 
-#ifdef WIN32
-#include "BLI_winstuff.h"
-#endif
-
-#include "BMF_Api.h"
-#include "BIF_language.h"
-#ifdef INTERNATIONAL
-#include "FTF_Api.h"
-#endif
-
 #include "DNA_ID.h"
 #include "DNA_image_types.h"
 #include "DNA_lamp_types.h"
@@ -111,9 +101,8 @@
 #include "BPY_extern.h"
 #include "BPY_menus.h"
 
-#include "render.h" // for R - should use BKE_bad_level_calls.h instead?
-
 #include "blendef.h"
+#include "interface.h"
 #include "mydevice.h"
 
 extern char versionstr[]; /* from blender.c */
@@ -317,9 +306,13 @@ Scene *copy_scene(Scene *sce, int level)
 	id_us_plus((ID *)scen->world);
 	id_us_plus((ID *)scen->set);
 
-	scen->ed= 0;
-	scen->radio= 0;
+	scen->ed= NULL;
+	scen->radio= NULL;
+	scen->theDag= NULL;
+	scen->toolsettings= MEM_dupallocN(sce->toolsettings);
 
+	duplicatelist(&(scen->markers), &(sce->markers));
+	
 	obase= sce->base.first;
 	base= scen->base.first;
 	while(base) {
@@ -334,6 +327,7 @@ Scene *copy_scene(Scene *sce, int level)
 
 	/* level 1 */
 	G.scene= scen;
+	
 	single_object_users(0);
 
 	/*	camera */
@@ -348,12 +342,14 @@ Scene *copy_scene(Scene *sce, int level)
 		single_obdata_users(0);
 		single_mat_users_expand();
 		single_tex_users_expand();
+		
+		scen->radio= MEM_dupallocN(sce->radio);
+		
 	}
 
 	clear_id_newpoins();
 
 	BPY_copy_scriptlink(&sce->scriptlink);
-
 	// make a private copy of the avicodecdata
 
 	if (sce->r.avicodecdata) {
@@ -429,9 +425,6 @@ void do_info_buttons(unsigned short event)
 		break;
 	case B_INFOSCE:		/* menu select scene */
 
-		if( G.obedit) {
-			exit_editmode(2);
-		}
 		if( G.curscreen->scenenr== -2) {
 			if(curarea->winy <50) {
 				sa= closest_bigger_area();
@@ -531,7 +524,7 @@ static void check_packAll()
 	}
 }
 
-int write_runtime(char *str, char *exename)
+static int write_runtime(char *str, char *exename)
 {
 	char *freestr= NULL;
 	char *ext = 0;
@@ -585,89 +578,6 @@ static void write_runtime_check(char *str)
 /************************** MAIN MENU *****************************/
 /************************** FILE *****************************/
 
-void do_info_file_optionsmenu(void *arg, int event)
-{
-	G.fileflags ^= (1 << event);
-
-	// allqueue(REDRAWINFO, 0);
-}
-
-#if GAMEBLENDER == 1
-static uiBlock *info_file_optionsmenu(void *arg_unused)
-{
-	uiBlock *block;
-	short yco= 0, xco = 20;
-
-	block= uiNewBlock(&curarea->uiblocks, "runtime_options", UI_EMBOSSP, UI_HELV, G.curscreen->mainwin);
-	uiBlockSetButmFunc(block, do_info_file_optionsmenu, NULL);
-	uiBlockSetXOfs(block,-40);	// offset to parent button
-	
-	/* flags are case-values */
-	uiDefBut(block, BUTM, 1, "Compress File",	xco, yco-=20, 100, 19, NULL, 0.0, 0.0, 0, G_FILE_COMPRESS_BIT, "Enables file compression");
-/*
-	uiDefBut(block, BUTM, 1, "Sign File",	xco, yco-=20, 100, 19, NULL, 0.0, 0.0, 0, G_FILE_SIGN_BIT, "Add signature to file");
-	uiDefBut(block, BUTM, 1, "Lock File",	xco, yco-=20, 100, 19, NULL, 0.0, 0.0, 0, G_FILE_LOCK_BIT, "Protect the file from editing by others");
-*/
-	uiTextBoundsBlock(block, 50);
-
-	/* Toggle buttons */
-	
-	yco= 0;
-	xco -= 20;
-	uiBlockSetEmboss(block, UI_EMBOSSM);
-	uiBlockSetButmFunc(block, NULL, NULL);
-	/* flags are defines */
-	uiDefIconButI(block, ICONTOG|BIT|G_FILE_COMPRESS_BIT, 0, ICON_CHECKBOX_DEHLT, xco, yco-=20, 19, 19, &G.fileflags, 0.0, 0.0, 0, 0, "");
-/*
-	uiDefIconButI(block, ICONTOG|BIT|G_FILE_SIGN_BIT, 0, ICON_CHECKBOX_DEHLT, xco, yco-=20, 19, 19, &G.fileflags, 0.0, 0.0, 0, 0, "");
-	uiDefIconButI(block, ICONTOG|BIT|G_FILE_LOCK_BIT, 0, ICON_CHECKBOX_DEHLT, xco, yco-=20, 19, 19, &G.fileflags, 0.0, 0.0, 0, 0, "");
-*/
-	uiBlockSetDirection(block, UI_RIGHT);
-		
-	return block;
-}
-
-static uiBlock *info_runtime_optionsmenu(void *arg_unused)
-{
-	uiBlock *block;
-	short yco= 0, xco = 20;
-
-	block= uiNewBlock(&curarea->uiblocks, "add_surfacemenu", UI_EMBOSSP, UI_HELV, G.curscreen->mainwin);
-	uiBlockSetXOfs(block, -40);  // offset to parent button
-	uiBlockSetEmboss(block, UI_EMBOSSM);
-
-	uiDefBut(block, LABEL, 0, "Size options:",		xco, yco-=20, 114, 19, 0, 0.0, 0.0, 0, 0, "");
-	uiDefButS(block, NUM, 0, "X:",		xco+19, yco-=20, 95, 19,		&G.scene->r.xplay, 10.0, 2000.0, 0, 0, "Displays current X screen/window resolution. Click to change.");
-	uiDefButS(block, NUM, 0, "Y:",		xco+19, yco-=20, 95, 19, &G.scene->r.yplay, 10.0, 2000.0, 0, 0, "Displays current Y screen/window resolution. Click to change.");
-
-	uiDefBut(block, SEPR, 0, "",		xco, yco-=4, 114, 4, NULL, 0.0, 0.0, 0, 0, "");
-
-	uiDefBut(block, LABEL, 0, "Fullscreen options:",		xco, yco-=20, 114, 19, 0, 0.0, 0.0, 0, 0, "");
-	uiDefButS(block, TOG, 0, "Fullscreen", xco + 19, yco-=20, 95, 19, &G.scene->r.fullscreen, 0.0, 0.0, 0, 0, "Starts player in a new fullscreen display");
-	uiDefButS(block, NUM, 0, "Freq:",	xco+19, yco-=20, 95, 19, &G.scene->r.freqplay, 10.0, 120.0, 0, 0, "Displays clock frequency of fullscreen display. Click to change.");
-	uiDefButS(block, NUM, 0, "Bits:",	xco+19, yco-=20, 95, 19, &G.scene->r.depth, 1.0, 32.0, 0, 0, "Displays bit depth of full screen display. Click to change.");
-
-	uiDefBut(block, SEPR, 0, "",		xco, yco-=4, 114, 4, NULL, 0.0, 0.0, 0, 0, "");
-
-	/* stereo settings */
-	/* can't use any definition from the game engine here so hardcode it. Change it here when it changes there!
-	 * RAS_IRasterizer has definitions:
-	 * RAS_STEREO_NOSTEREO		 1
-	 * RAS_STEREO_QUADBUFFERED 2
-	 * RAS_STEREO_ABOVEBELOW	 3
-	 * RAS_STEREO_INTERLACED	 4	 future
-	 */
-	uiDefBut(block, LABEL, 0, "Stereo options", xco, yco-=20, 114, 19, 0, 0.0, 0.0, 0, 0, "");
-	uiDefButS(block, ROW, 0, "no stereo", xco+19, yco-=20, 95, 19, &(G.scene->r.stereomode), 6.0, 1.0, 0, 0, "Disables stereo");
-	uiDefButS(block, ROW, 0, "h/w pageflip", xco+19, yco-=20, 95, 19, &(G.scene->r.stereomode), 6.0, 2.0, 0, 0, "Enables hardware pageflip stereo method");
-	uiDefButS(block, ROW, 0, "syncdoubling", xco+19, yco-=20, 95, 19, &(G.scene->r.stereomode), 6.0, 3.0, 0, 0, "Enables syncdoubling stereo method");
-
-	uiBlockSetDirection(block, UI_RIGHT);
-	uiTextBoundsBlock(block, 50);
-		
-	return block;
-}
-#endif
 
 static void do_info_file_importmenu(void *arg, int event)
 {
@@ -852,7 +762,7 @@ static void do_info_filemenu(void *arg, int event)
 		}
 		break;
 	case 6: /* save image */
-		mainqenter(F3KEY, 1);
+		BIF_save_rendered_image();
 		break;
 	case 22: /* save runtime */
 		activate_fileselect(FILE_SPECIAL, "Save Runtime", "", write_runtime_check);
@@ -900,11 +810,16 @@ static void do_info_filemenu(void *arg, int event)
 		
 			BKE_reset_undo();
 			BKE_write_undo("original");	/* save current state */
+			refresh_interface_font();
 		}
 		break;
 	case 31: /* save default settings */
 		BIF_write_homefile();
 		break;
+
+	case 35: /* compress toggle */
+		U.flag ^= (USER_FILECOMPRESS);
+		break;
 	}
 	allqueue(REDRAWINFO, 0);
 }
@@ -927,11 +842,17 @@ static uiBlock *info_filemenu(void *arg_unused)
 
 	uiDefIconTextBut(block, BUTM, 1, ICON_BLANK1, "Save|Ctrl W",				0, yco-=20, menuwidth, 19, NULL, 0.0, 0.0, 1, 5, "");
 	uiDefIconTextBut(block, BUTM, 1, ICON_BLANK1, "Save As...|F2",			0, yco-=20, menuwidth, 19, NULL, 0.0, 0.0, 1, 4, "");
+	
+	if(U.flag & USER_FILECOMPRESS) {
+		uiDefIconTextBut(block, BUTM, 1, ICON_CHECKBOX_HLT, "Compress File",	 0, yco-=20, menuwidth, 19, NULL, 0.0, 0.0, 1, 35, "Enable file compression");
+	} else {
+		uiDefIconTextBut(block, BUTM, 1, ICON_CHECKBOX_DEHLT, "Compress File",	 0, yco-=20, menuwidth, 19, NULL, 0.0, 0.0, 1, 35, "Enable file compression");
+	}
 
 	uiDefBut(block, SEPR, 0, "",					0, yco-=6, menuwidth, 6, NULL, 0.0, 0.0, 0, 0, "");
 
 	uiDefIconTextBut(block, BUTM, 1, ICON_BLANK1, "Save Image...|F3",			0, yco-=20, menuwidth, 19, NULL, 0.0, 0.0, 1, 6, "");
-	uiDefIconTextBut(block, BUTM, 1, ICON_BLANK1, "Dump 3DView|Ctrl F3",			0, yco-=20, menuwidth, 19, NULL, 0.0, 0.0, 1, 24, "");
+	uiDefIconTextBut(block, BUTM, 1, ICON_BLANK1, "Dump Subwindow|Ctrl F3",			0, yco-=20, menuwidth, 19, NULL, 0.0, 0.0, 1, 24, "");
 	uiDefIconTextBut(block, BUTM, 1, ICON_BLANK1, "Dump Screen|Ctrl Shift F3",			0, yco-=20, menuwidth, 19, NULL, 0.0, 0.0, 1, 25, "");
 #if GAMEBLENDER == 1
 	uiDefIconTextBut(block, BUTM, 1, ICON_BLANK1, "Save Runtime...",			0, yco-=20, menuwidth, 19, NULL, 0.0, 0.0, 1, 22, "");
@@ -1283,10 +1204,6 @@ void do_info_addmenu(void *arg, int event)
 			/* Camera */
 			add_object_draw(OB_CAMERA);
 			break;
-		case 7:
-			/* Lamp */
-		//	add_object_draw(OB_LAMP);
-			break;
 		case 8:
 			/* Armature */
 			add_primitiveArmature(OB_ARMATURE);
@@ -1321,13 +1238,11 @@ static uiBlock *info_addmenu(void *arg_unused)
 	uiDefBut(block, SEPR, 0, "",					0, yco-=6, 120, 6, NULL, 0.0, 0.0, 0, 0, "");
 	
 	uiDefIconTextBut(block, BUTM, 1, ICON_BLANK1, "Camera",				0, yco-=20, 120, 19, NULL, 0.0, 0.0, 1, 6, "");
-//	uiDefIconTextBut(block, BUTM, 1, ICON_BLANK1, "Lamp",				0, yco-=20, 120, 19, NULL, 0.0, 0.0, 1, 7, "");
 	uiDefIconTextBlockBut(block, info_add_lampmenu, NULL, ICON_RIGHTARROW_THIN, "Lamp", 0, yco-=20, 120, 19, "");
 	
 	uiDefBut(block, SEPR, 0, "",					0, yco-=6, 120, 6, NULL, 0.0, 0.0, 0, 0, "");
 	
-	/* armature needs 3d window to draw */
-	//uiDefIconTextBut(block, BUTM, 1, ICON_BLANK1, "Armature",			0, yco-=20, 120, 19, NULL, 0.0, 0.0, 1, 8, "");
+	uiDefIconTextBut(block, BUTM, 1, ICON_BLANK1, "Armature",			0, yco-=20, 120, 19, NULL, 0.0, 0.0, 1, 8, "");
 	uiDefIconTextBut(block, BUTM, 1, ICON_BLANK1, "Lattice",			0, yco-=20, 120, 19, NULL, 0.0, 0.0, 1, 9, "");
 
 	uiBlockSetDirection(block, UI_DOWN);
@@ -1341,11 +1256,13 @@ static uiBlock *info_addmenu(void *arg_unused)
 static void do_info_gamemenu(void *arg, int event)
 {
 	switch (event) {
-	case G_FILE_ENABLE_ALL_FRAMES_BIT:
-	case G_FILE_SHOW_FRAMERATE_BIT:
-	case G_FILE_SHOW_DEBUG_PROPS_BIT:
-	case G_FILE_AUTOPLAY_BIT:
-		G.fileflags ^= (1 << event);
+	case G_FILE_ENABLE_ALL_FRAMES:
+	case G_FILE_SHOW_FRAMERATE:
+	case G_FILE_SHOW_DEBUG_PROPS:
+	case G_FILE_AUTOPLAY:
+	case G_FILE_GAME_TO_IPO:
+	case G_FILE_GAME_MAT:
+		G.fileflags ^= event;
 		break;
 	default:
 		; /* ignore the rest */
@@ -1367,34 +1284,48 @@ static uiBlock *info_gamemenu(void *arg_unused)
 	uiDefBut(block, SEPR, 0, "", 0, yco-=6, menuwidth, 6, NULL, 0.0, 0.0, 1, 0, "");
 
 
-	if(G.fileflags & (1 << G_FILE_ENABLE_ALL_FRAMES_BIT)) {
-		uiDefIconTextBut(block, BUTM, 1, ICON_CHECKBOX_HLT, "Enable All Frames",	 0, yco-=20, menuwidth, 19, NULL, 0.0, 0.0, 1, G_FILE_ENABLE_ALL_FRAMES_BIT, "");
+	if(G.fileflags & G_FILE_ENABLE_ALL_FRAMES) {
+		uiDefIconTextBut(block, BUTM, 1, ICON_CHECKBOX_HLT, "Enable All Frames",	 0, yco-=20, menuwidth, 19, NULL, 0.0, 0.0, 1, G_FILE_ENABLE_ALL_FRAMES, "");
 	} else {
-		uiDefIconTextBut(block, BUTM, 1, ICON_CHECKBOX_DEHLT, "Enable All Frames",	 0, yco-=20, menuwidth, 19, NULL, 0.0, 0.0, 1, G_FILE_ENABLE_ALL_FRAMES_BIT, "");
+		uiDefIconTextBut(block, BUTM, 1, ICON_CHECKBOX_DEHLT, "Enable All Frames",	 0, yco-=20, menuwidth, 19, NULL, 0.0, 0.0, 1, G_FILE_ENABLE_ALL_FRAMES, "");
+	}
+	
+	if(G.fileflags & G_FILE_GAME_TO_IPO) {
+		uiDefIconTextBut(block, BUTM, 1, ICON_CHECKBOX_HLT, "Record Game Physics to IPO",	 0, yco-=20, menuwidth, 19, NULL, 0.0, 0.0, 1, G_FILE_GAME_TO_IPO, "");
+	} else {
+		uiDefIconTextBut(block, BUTM, 1, ICON_CHECKBOX_DEHLT, "Record Game Physics to IPO",	 0, yco-=20, menuwidth, 19, NULL, 0.0, 0.0, 1, G_FILE_GAME_TO_IPO, "");
+	}
+	
+	if(G.fileflags & G_FILE_GAME_MAT) {
+		uiDefIconTextBut(block, BUTM, 1, ICON_CHECKBOX_HLT, "Use Blender Materials",	 0, yco-=20, menuwidth, 19, NULL, 0.0, 0.0, 1, G_FILE_GAME_MAT, "");
+	} else {
+		uiDefIconTextBut(block, BUTM, 1, ICON_CHECKBOX_DEHLT, "Use Blender Materials",	 0, yco-=20, menuwidth, 19, NULL, 0.0, 0.0, 1, G_FILE_GAME_MAT, "");
+	}	
+
+
+
+	if(G.fileflags & G_FILE_SHOW_FRAMERATE) {
+		uiDefIconTextBut(block, BUTM, 1, ICON_CHECKBOX_HLT, "Show Framerate and Profile",	 0, yco-=20, menuwidth, 19, NULL, 0.0, 0.0, 1, G_FILE_SHOW_FRAMERATE, "");
+	} else {
+		uiDefIconTextBut(block, BUTM, 1, ICON_CHECKBOX_DEHLT, "Show Framerate and Profile",	 0, yco-=20, menuwidth, 19, NULL, 0.0, 0.0, 1, G_FILE_SHOW_FRAMERATE, "");
 	}
 
-	if(G.fileflags & (1 << G_FILE_SHOW_FRAMERATE_BIT)) {
-		uiDefIconTextBut(block, BUTM, 1, ICON_CHECKBOX_HLT, "Show Framerate and Profile",	 0, yco-=20, menuwidth, 19, NULL, 0.0, 0.0, 1, G_FILE_SHOW_FRAMERATE_BIT, "");
+	if(G.fileflags & G_FILE_SHOW_DEBUG_PROPS) {
+		uiDefIconTextBut(block, BUTM, 1, ICON_CHECKBOX_HLT, "Show Debug Properties",		 0, yco-=20, menuwidth, 19, NULL, 0.0, 0.0, 1, G_FILE_SHOW_DEBUG_PROPS, "");
 	} else {
-		uiDefIconTextBut(block, BUTM, 1, ICON_CHECKBOX_DEHLT, "Show Framerate and Profile",	 0, yco-=20, menuwidth, 19, NULL, 0.0, 0.0, 1, G_FILE_SHOW_FRAMERATE_BIT, "");
-	}
-
-	if(G.fileflags & (1 << G_FILE_SHOW_DEBUG_PROPS_BIT)) {
-		uiDefIconTextBut(block, BUTM, 1, ICON_CHECKBOX_HLT, "Show Debug Properties",		 0, yco-=20, menuwidth, 19, NULL, 0.0, 0.0, 1, G_FILE_SHOW_DEBUG_PROPS_BIT, "");
-	} else {
-		uiDefIconTextBut(block, BUTM, 1, ICON_CHECKBOX_DEHLT, "Show Debug Properties",		 0, yco-=20, menuwidth, 19, NULL, 0.0, 0.0, 1, G_FILE_SHOW_DEBUG_PROPS_BIT, "");
+		uiDefIconTextBut(block, BUTM, 1, ICON_CHECKBOX_DEHLT, "Show Debug Properties",		 0, yco-=20, menuwidth, 19, NULL, 0.0, 0.0, 1, G_FILE_SHOW_DEBUG_PROPS, "");
 	}
 	
 	uiDefBut(block, SEPR, 0, "",				0, yco-=6, menuwidth, 6, NULL, 0.0, 0.0, 1, 0, "");
 
-	if(G.fileflags & (1 << G_FILE_AUTOPLAY_BIT)) {
-		uiDefIconTextBut(block, BUTM, 1, ICON_CHECKBOX_HLT, "Autostart",	 0, yco-=20, menuwidth, 19, NULL, 0.0, 0.0, 1, G_FILE_AUTOPLAY_BIT, "");
+	if(G.fileflags & G_FILE_AUTOPLAY) {
+		uiDefIconTextBut(block, BUTM, 1, ICON_CHECKBOX_HLT, "Autostart",	 0, yco-=20, menuwidth, 19, NULL, 0.0, 0.0, 1, G_FILE_AUTOPLAY, "");
 	} else {
-		uiDefIconTextBut(block, BUTM, 1, ICON_CHECKBOX_DEHLT, "Autostart",	 0, yco-=20, menuwidth, 19, NULL, 0.0, 0.0, 1, G_FILE_AUTOPLAY_BIT, "");
+		uiDefIconTextBut(block, BUTM, 1, ICON_CHECKBOX_DEHLT, "Autostart",	 0, yco-=20, menuwidth, 19, NULL, 0.0, 0.0, 1, G_FILE_AUTOPLAY, "");
 	}
 
 	uiBlockSetDirection(block, UI_DOWN);
-	uiTextBoundsBlock(block, 50);
+	uiTextBoundsBlock(block, 70);
 	
 	return block;
 }
@@ -1517,25 +1448,10 @@ static uiBlock *info_timelinemenu(void *arg_unused)
 
 /************************** RENDER *****************************/
 
-/* copied from buttons.c. .. probably not such a good idea!? */
-static void run_playanim(char *file)
-{
-	extern char bprogname[];	/* usiblender.c */
-	char str[FILE_MAXDIR+FILE_MAXFILE];
-	int pos[2], size[2];
-
-	calc_renderwin_rectangle(R.winpos, pos, size);
-
-	sprintf(str, "%s -a -p %d %d \"%s\"", bprogname, pos[0], pos[1], file);
-	system(str);
-}
-
 static void do_info_rendermenu(void *arg, int event)
 {
-	char file[FILE_MAXDIR+FILE_MAXFILE];
 
-	extern void makeavistring(char *string);
-	extern void makeqtstring (char *string);
+	extern void playback_anim();
 
 	switch(event) {
 		
@@ -1553,22 +1469,7 @@ static void do_info_rendermenu(void *arg, int event)
 		BIF_toggle_render_display();
 		break;
 	case 5:
-#ifdef WITH_QUICKTIME
-		if(G.scene->r.imtype == R_QUICKTIME)
-			makeqtstring(file);
-		else
-#endif
-			makeavistring(file);
-		if(BLI_exist(file)) {
-			run_playanim(file);
-		}
-		else {
-			makepicstring(file, G.scene->r.sfra);
-			if(BLI_exist(file)) {
-				run_playanim(file);
-			}
-			else error("Can't find image: %s", file);
-		}
+		playback_anim();
 		break;
 	case 6:
 		/* dodgy hack turning on SHIFT key to do a proper render border select
@@ -1771,7 +1672,7 @@ char info_time_str[32]="";
 
 static void info_text(int x, int y)
 {
-	Object *ob;
+	Object *ob= OBACT;
 	extern float hashvectf[];
 	extern int mem_in_use;
 	unsigned int swatch_color;
@@ -1781,15 +1682,36 @@ static void info_text(int x, int y)
 	int hsize;
 
 	if(G.obedit) {
-		sprintf(infostr,"Ve:%d-%d | Fa:%d-%d | Mem:%.2fM ",
-		G.totvertsel, G.totvert, G.totfacesel, G.totface,
-		(mem_in_use>>10)/1024.0);
+		char *s = infostr;
+
+		s+= sprintf(s, "%s", G.editModeTitleExtra);
+		if(G.obedit->type==OB_MESH) {
+			if(G.scene->selectmode & SCE_SELECT_VERTEX)
+				s+= sprintf(s,"Ve:%d-%d | Ed:%d-%d | Fa:%d-%d",
+						G.totvertsel, G.totvert, G.totedgesel, G.totedge, G.totfacesel, G.totface);
+			else if(G.scene->selectmode & SCE_SELECT_EDGE)
+				s+= sprintf(s,"Ed:%d-%d | Fa:%d-%d",
+						G.totedgesel, G.totedge, G.totfacesel, G.totface);
+			else 
+				s+= sprintf(s,"Fa:%d-%d", G.totfacesel, G.totface);
+		}
+		else if(G.obedit->type==OB_ARMATURE) {
+			s+= sprintf(s,"Ve:%d-%d | Bo:%d-%d", G.totvertsel, G.totvert, G.totbonesel, G.totbone);
+		}
+		else {
+			s+= sprintf(s,"Ve:%d-%d", G.totvertsel, G.totvert);
+		}
+
+		sprintf(s," | Mem:%.2fM ", (mem_in_use>>10)/1024.0);
+	}
+	else if(ob && (ob->flag & OB_POSEMODE)) {
+		sprintf(infostr,"Bo:%d-%d | Mem:%.2fM ",
+					G.totbonesel, G.totbone, (mem_in_use>>10)/1024.0);
 	}
 	else {
 		sprintf(infostr,"Ve:%d | Fa:%d | Ob:%d-%d | La:%d | Mem:%.2fM | Time:%s | ",
 			G.totvert, G.totface, G.totobj, G.totobjsel, G.totlamp,  (mem_in_use>>10)/1024.0, info_time_str);
 	}
-	ob= OBACT;
 	if(ob) {
 		strcat(infostr, ob->id.name+2);
 	}
@@ -1819,15 +1741,19 @@ static void info_text(int x, int y)
 	cpack( swatch_color );
 	glRecti(x-24,  y-6,  x-30+hsize,	y+14);
 
-	glColor3ub(0, 0, 0);
-
+	glColor3ub(0, 0, 0); /* makes text black colored rect */
+	
 	glRasterPos2i(x, y);
+	BIF_RasterPos(x, y);
 
 	BIF_DrawString(G.font, headerstr, (U.transopts & USER_TR_MENUS));
 	hsize= BIF_GetStringWidth(G.font, headerstr, (U.transopts & USER_TR_BUTTONS));
 	
+	BIF_ThemeColor(TH_MENU_TEXT); /* makes text readable on dark theme */
+	
 	glRasterPos2i(x+hsize+10,	y);
-
+	BIF_RasterPos(x+hsize+10,	y);
+	
 	BIF_DrawString(G.font, infostr, (U.transopts & USER_TR_MENUS));
 }
 
@@ -1843,11 +1769,11 @@ void info_buttons(void)
 	else uiBlockSetCol(block, TH_HEADERDESEL);
 	
 	if(curarea->flag & HEADER_NO_PULLDOWN) {
-		uiDefIconButS(block, TOG|BIT|0, B_FLIPINFOMENU, ICON_DISCLOSURE_TRI_RIGHT,
+		uiDefIconButBitS(block, TOG, HEADER_NO_PULLDOWN, B_FLIPINFOMENU, ICON_DISCLOSURE_TRI_RIGHT,
 				xco,2,XIC,YIC-2,
 				&(curarea->flag), 0, 0, 0, 0, "Enables display of pulldown menus");
 	} else {
-		uiDefIconButS(block, TOG|BIT|0, B_FLIPINFOMENU, ICON_DISCLOSURE_TRI_DOWN,
+		uiDefIconButBitS(block, TOG, HEADER_NO_PULLDOWN, B_FLIPINFOMENU, ICON_DISCLOSURE_TRI_DOWN,
 				xco,2,XIC,YIC-2,
 				&(curarea->flag), 0, 0, 0, 0, "Hides pulldown menus");
 	}
@@ -1909,6 +1835,7 @@ void info_buttons(void)
 	}
 	else xco= 430;
 	
+BIF_SetScale(block->aspect);
 	info_text(xco+24, 6);
 	
 	uiBlockSetEmboss(block, UI_EMBOSSN);
@@ -1917,7 +1844,8 @@ void info_buttons(void)
 	/* always do as last */
 	curarea->headbutlen= xco+2*XIC;
 
-#ifdef _WIN32	// FULLSCREEN
+#if 0
+// #ifdef _WIN32	// FULLSCREEN
 	if(U.uiflag & USER_FLIPFULLSCREEN) {
 		uiDefIconBut(block, BUT, B_FLIPFULLSCREEN, ICON_WINDOW_WINDOW,
 				(short)(curarea->winx-XIC-5), 0,XIC,YIC,
