@@ -4,7 +4,7 @@
  * Functions to draw the "Script Window" window header
  * and handle user events sent to it.
  * 
- * $Id: header_script.c,v 1.14 2005/03/22 04:28:36 ianwill Exp $
+ * $Id: header_script.c,v 1.17 2006/01/15 13:30:56 campbellbarton Exp $
  *
  * ***** BEGIN GPL/BL DUAL LICENSE BLOCK *****
  *
@@ -215,7 +215,7 @@ void script_buttons(void)
 	if (!sc || sc->spacetype != SPACE_SCRIPT) return;
 
 	sprintf(naam, "header %d", curarea->headwin);
-	block= uiNewBlock(&curarea->uiblocks, naam, UI_EMBOSSX, UI_HELV, curarea->headwin);
+	block= uiNewBlock(&curarea->uiblocks, naam, UI_EMBOSS, UI_HELV, curarea->headwin);
 
 	if(area_is_active_area(curarea)) uiBlockSetCol(block, TH_HEADER);
 	else uiBlockSetCol(block, TH_HEADERDESEL);
@@ -230,11 +230,11 @@ void script_buttons(void)
 
 	uiBlockSetEmboss(block, UI_EMBOSSN);
 	if(curarea->flag & HEADER_NO_PULLDOWN) {
-		uiDefIconButS(block, TOG|BIT|0, B_FLIPINFOMENU, ICON_DISCLOSURE_TRI_RIGHT,
+		uiDefIconButBitS(block, TOG, HEADER_NO_PULLDOWN, B_FLIPINFOMENU, ICON_DISCLOSURE_TRI_RIGHT,
 				xco,2,XIC,YIC-2,
 				&(curarea->flag), 0, 0, 0, 0, "Enables display of pulldown menus");
 	} else {
-		uiDefIconButS(block, TOG|BIT|0, B_FLIPINFOMENU, ICON_DISCLOSURE_TRI_DOWN,
+		uiDefIconButBitS(block, TOG, HEADER_NO_PULLDOWN, B_FLIPINFOMENU, ICON_DISCLOSURE_TRI_DOWN,
 				xco,2,XIC,YIC-2,
 				&(curarea->flag), 0, 0, 0, 0, "Hides pulldown menus");
 	}
@@ -250,8 +250,9 @@ void script_buttons(void)
 		xco+=xmax;
 	}
 
-	uiBlockSetEmboss(block, UI_EMBOSSX);
-
+	uiBlockSetEmboss(block, UI_EMBOSS);
+	
+	uiBlockBeginAlign(block);
 	uiDefIconBut(block, BUT, B_SCRIPT2PREV, ICON_GO_LEFT, xco+=XIC, 0, XIC, YIC,
 		0, 0, 0, 0, 0, "Returns to previous window");
 
@@ -262,7 +263,8 @@ void script_buttons(void)
 	else
 		uiDefIconBut(block, BUT,B_FULL, ICON_FULLSCREEN,	xco+=XIC,0,XIC,YIC, 0, 0,
 				0, 0, 0, "Makes current window full screen (CTRL+Down arrow)");
-
+	uiBlockEndAlign(block);
+	
 	/* STD SCRIPT BUTTONS */
 	xco += 2*XIC;
 	xco= std_libbuttons(block, xco, 0, 0, NULL, B_SCRIPTBROWSE, (ID*)sc->script, 0, &(sc->menunr), 0, 0, 0, 0, 0);
