@@ -1,5 +1,5 @@
 /**
- * $Id: headerbuttons.c,v 1.105 2005/05/28 13:22:09 ton Exp $
+ * $Id: headerbuttons.c,v 1.120 2005/11/19 15:16:34 ton Exp $
  *
  * ***** BEGIN GPL/BL DUAL LICENSE BLOCK *****
  *
@@ -92,11 +92,10 @@
 #include "BKE_armature.h"
 #include "BKE_blender.h"
 #include "BKE_curve.h"
-#include "BKE_displist.h"
+#include "BKE_depsgraph.h"
 #include "BKE_exotic.h"
 #include "BKE_global.h"
 #include "BKE_image.h"
-#include "BKE_ika.h"
 #include "BKE_ipo.h"
 #include "BKE_key.h"
 #include "BKE_lattice.h"
@@ -120,6 +119,7 @@
 #include "BIF_drawoops.h"
 #include "BIF_drawscene.h"
 #include "BIF_drawtext.h"
+#include "BIF_editaction.h"
 #include "BIF_editarmature.h"
 #include "BIF_editfont.h"
 #include "BIF_editlattice.h"
@@ -152,8 +152,6 @@
 #include "BSE_headerbuttons.h"
 #include "BSE_view.h"
 #include "BSE_sequence.h"
-#include "BSE_editaction.h"
-#include "BSE_editaction_types.h"
 #include "BSE_editipo.h"
 #include "BSE_drawipo.h"
 
@@ -173,8 +171,6 @@
 #include "nla.h"	/* __NLA : To be removed later */
 #include "butspace.h"  // test_idbutton
 
-#include "TPT_DependKludge.h"
-
 #include "BIF_poseobject.h"
 
 #include "SYS_System.h"
@@ -489,52 +485,18 @@ int std_libbuttons(uiBlock *block, short xco, short yco,
 }
 
 
-
+/* results in fully updated anim system */
 static void do_update_for_newframe(int mute, int events)
 {
 	extern void audiostream_scrub(unsigned int frame);	/* seqaudio.c */
-	ScrArea *sa;
 	
 	if(events) {
-		allqueue(REDRAWVIEW3D, 0);
-		allqueue(REDRAWACTION,0);
-		allqueue(REDRAWNLA,0);
-		allqueue(REDRAWIPO, 0);
-		allqueue(REDRAWINFO, 1);
-		allqueue(REDRAWSEQ, 1);
-		allqueue(REDRAWSOUND, 1);
-		allqueue(REDRAWTIME, 1);
-		allqueue(REDRAWBUTSHEAD, 0);
-		allqueue(REDRAWBUTSSHADING, 0);
-		allqueue(REDRAWBUTSOBJECT, 0);
+		allqueue(REDRAWALL, 0);
 	}
 	
-	/* layers/materials, object ipos are calculted in where_is_object (too) */
-	do_all_ipos();
-	if (G.f & G_DOSCRIPTLINKS) BPY_do_all_scripts(SCRIPT_FRAMECHANGED);
-	clear_all_constraints();
-	do_all_keys();
+	/* this function applies the changes too */
+	scene_update_for_newframe(G.scene, screen_view3d_layers()); /* BKE_scene.h */
 
-	do_all_actions(NULL);
-	rebuild_all_armature_displists();
-	/* so nice, better do it twice */
-	do_all_actions(NULL);
-	rebuild_all_armature_displists();
-
-	do_all_ikas();
-
-	test_all_displists();
-	
-	/* manipulators like updates too */
-	for(sa=G.curscreen->areabase.first; sa; sa=sa->next) {
-		if(sa->spacetype==SPACE_VIEW3D) {
-			View3D *v3d= sa->spacedata.first;
-			if(v3d->twflag & V3D_USE_MANIPULATOR) break;
-			else break;	// for now
-		}
-	}
-	if(sa) countall();	// does manipulator centers
-	
 	if ( (CFRA>1) && (!mute) && (G.scene->audio.flag & AUDIO_SCRUB)) audiostream_scrub( CFRA );
 }
 
@@ -672,17 +634,15 @@ void do_global_buttons(unsigned short event)
 	Ipo *ipo;
 	Lamp *la;
 	World *wrld;
-	Sequence *seq;
 	bAction *act;
-	ID *id, *idtest, *from;
+	ID *id, *idtest, *from=NULL;
 	ScrArea *sa;
 	int nr= 1;
 	char buf[FILE_MAXDIR+FILE_MAXFILE];
 
-
 	ob= OBACT;
 
-	id= 0;	/* id at null for texbrowse */
+	id= NULL;	/* id at null for texbrowse */
 
 
 	switch(event) {
@@ -728,12 +688,11 @@ void do_global_buttons(unsigned short event)
 					
 					if( GS(idtest->name)==ID_CU ) {
 						test_curve_type(ob);
-						allqueue(REDRAWBUTSEDIT, 0);
-						makeDispList(ob);
 					}
-					else if( ob->type==OB_MESH ) {
-						makeDispList(ob);
+					else if( ob->type==OB_ARMATURE) {
+						armature_rebuild_pose(ob, ob->data);
 					}
+					DAG_object_flush_update(G.scene, ob, OB_RECALC_DATA);
 					
 					allqueue(REDRAWBUTSEDIT, 0);
 					allqueue(REDRAWVIEW3D, 0);
@@ -766,9 +725,10 @@ void do_global_buttons(unsigned short event)
 		idtest= G.main->mesh.first;
 		while(idtest) {
 			if(nr==G.buts->menunr) {
-					
 				set_mesh(ob, (Mesh *)idtest);
 				
+				DAG_object_flush_update(G.scene, ob, OB_RECALC_DATA);
+					
 				BIF_undo_push("Browse Mesh");
 				allqueue(REDRAWBUTSEDIT, 0);
 				allqueue(REDRAWVIEW3D, 0);
@@ -854,7 +814,7 @@ void do_global_buttons(unsigned short event)
 					if(mtex) {
 						if(mtex->tex) mtex->tex->id.us--;
 						MEM_freeN(mtex);
-						ma->mtex[ ma->texact ]= 0;
+						ma->mtex[ ma->texact ]= NULL;
 						allqueue(REDRAWBUTSSHADING, 0);
 						allqueue(REDRAWIPO, 0);
 						BIF_preview_changed(G.buts);
@@ -868,7 +828,7 @@ void do_global_buttons(unsigned short event)
 					if(mtex) {
 						if(mtex->tex) mtex->tex->id.us--;
 						MEM_freeN(mtex);
-						wrld->mtex[ wrld->texact ]= 0;
+						wrld->mtex[ wrld->texact ]= NULL;
 						allqueue(REDRAWBUTSSHADING, 0);
 						allqueue(REDRAWIPO, 0);
 						BIF_preview_changed(G.buts);
@@ -882,7 +842,7 @@ void do_global_buttons(unsigned short event)
 					if(mtex) {
 						if(mtex->tex) mtex->tex->id.us--;
 						MEM_freeN(mtex);
-						la->mtex[ la->texact ]= 0;
+						la->mtex[ la->texact ]= NULL;
 						allqueue(REDRAWBUTSSHADING, 0);
 						allqueue(REDRAWIPO, 0);
 						BIF_preview_changed(G.buts);
@@ -899,7 +859,7 @@ void do_global_buttons(unsigned short event)
 			
 			id= G.buts->lockpoin;
 			if(event==B_EXTEXBROWSE) {
-				id= 0;
+				id= NULL;
 				ma= give_current_material(ob, ob->actcol);
 				if(ma) {
 					mtex= ma->mtex[ ma->texact ];
@@ -916,7 +876,7 @@ void do_global_buttons(unsigned short event)
 			
 		}
 		else {
-			id= 0;
+			id= NULL;
 			
 			ma= give_current_material(ob, ob->actcol);
 			if(ma) {
@@ -959,8 +919,14 @@ void do_global_buttons(unsigned short event)
 		if (act)
 			act->id.us--;
 		ob->action=NULL;
-		
+		if(ob->pose) {		// clear flag, also used for draw colors
+			bPoseChannel *pchan;
+			for(pchan= ob->pose->chanbase.first; pchan; pchan= pchan->next)
+				pchan->flag= 0;
+		}
 		BIF_undo_push("Unlink Action");
+		
+		allqueue(REDRAWVIEW3D, 0);
 		allqueue(REDRAWACTION, 0);
 		allqueue(REDRAWNLA, 0);
 		allqueue(REDRAWIPO, 0);
@@ -998,7 +964,7 @@ void do_global_buttons(unsigned short event)
 				if (act)
 					idtest= (ID *)copy_action(act);
 				else 
-					idtest=(ID *)add_empty_action();
+					idtest=(ID *)add_empty_action(ob->type==OB_ARMATURE?ID_PO:ID_OB);
 				idtest->us--;
 			}
 			
@@ -1007,7 +973,6 @@ void do_global_buttons(unsigned short event)
 				act= (bAction *)idtest;
 				
 				ob->action= act;
-				ob->activecon=NULL;
 				id_us_plus(idtest);
 				
 				if(id) id->us--;
@@ -1025,9 +990,10 @@ void do_global_buttons(unsigned short event)
 		break;
 	case B_IPOBROWSE:
 
-		ipo= get_ipo_to_edit(&from);
+		ipo= G.sipo->ipo;
+		from= G.sipo->from;
 		id= (ID *)ipo;
-		if(from==0) return;
+		if(from==NULL) return;
 
 		if(G.sipo->menunr== -2) {
 			activate_databrowse((ID *)G.sipo->ipo, ID_IP, GS(from->name), B_IPOBROWSE, &G.sipo->menunr, do_global_buttons);
@@ -1059,13 +1025,10 @@ void do_global_buttons(unsigned short event)
 			if(idtest==0) {
 				if(ipo) idtest= (ID *)copy_ipo(ipo);
 				else {
-					nr= GS(from->name);
-					if(nr==ID_OB){
-						if (G.sipo->blocktype==IPO_CO)
-							idtest= (ID *)add_ipo("CoIpo", IPO_CO); /* BLEARGH! */
-						else
-							idtest= (ID *)add_ipo("ObIpo", nr);
-					}
+					nr= G.sipo->blocktype;
+					if(nr==ID_OB) idtest= (ID *)add_ipo("ObIpo", ID_OB);
+					else if(nr==ID_CO) idtest= (ID *)add_ipo("CoIpo", ID_CO);
+					else if(nr==ID_PO) idtest= (ID *)add_ipo("ActIpo", nr);
 					else if(nr==ID_MA) idtest= (ID *)add_ipo("MatIpo", nr);
 					else if(nr==ID_TE) idtest= (ID *)add_ipo("TexIpo", nr);
 					else if(nr==ID_SEQ) idtest= (ID *)add_ipo("MatSeq", nr);
@@ -1075,131 +1038,26 @@ void do_global_buttons(unsigned short event)
 					else if(nr==ID_LA) idtest= (ID *)add_ipo("LaIpo", nr);
 					else if(nr==ID_CA) idtest= (ID *)add_ipo("CaIpo", nr);
 					else if(nr==ID_SO) idtest= (ID *)add_ipo("SndIpo", nr);
-					else if(nr==ID_AC) idtest= (ID *)add_ipo("ActIpo", nr);
-					else error("Warn bugs@blender.nl!");
+					else error("Warn bugtracker!");
 				}
 				idtest->us--;
 			}
 			if(idtest!=id && from) {
-				ipo= (Ipo *)idtest;
-		
-				if (ipo->blocktype==IPO_CO){
-					((Object*)from)->activecon->ipo = ipo;
-					id_us_plus(idtest);
-					allqueue(REDRAWVIEW3D, 0);
-					allqueue(REDRAWACTION, 0);
-					allqueue(REDRAWNLA, 0);
-				}
-				else if(ipo->blocktype==ID_OB) {
-					( (Object *)from)->ipo= ipo;
-					id_us_plus(idtest);
-					allqueue(REDRAWVIEW3D, 0);
-				}
-				else if(ipo->blocktype==ID_AC) {
-					bActionChannel *chan;
-					chan = get_hilighted_action_channel ((bAction*)from);
-					if (!chan){
-						error ("Create an action channel first");
-						return;
-					}
-					chan->ipo=ipo;
-					id_us_plus(idtest);
-					allqueue(REDRAWNLA, 0);
-					allqueue(REDRAWACTION, 0);
-				}
-				else if(ipo->blocktype==ID_MA) {
-					( (Material *)from)->ipo= ipo;
-					id_us_plus(idtest);
-					allqueue(REDRAWBUTSSHADING, 0);
-				}
-				else if(ipo->blocktype==ID_TE) {
-					( (Tex *)from)->ipo= ipo;
-					id_us_plus(idtest);
-					allqueue(REDRAWBUTSSHADING, 0);
-				}
-				else if(ipo->blocktype==ID_SEQ) {
-					seq= (Sequence *)from;
-					if((seq->type & SEQ_EFFECT)||(seq->type == SEQ_SOUND)) {
-						id_us_plus(idtest);
-						seq->ipo= ipo;
-					}
-				}
-				else if(ipo->blocktype==ID_CU) {
-					( (Curve *)from)->ipo= ipo;
-					id_us_plus(idtest);
-					allqueue(REDRAWVIEW3D, 0);
-				}
-				else if(ipo->blocktype==ID_KE) {
-					( (Key *)from)->ipo= ipo;
-					
-					id_us_plus(idtest);
-					allqueue(REDRAWVIEW3D, 0);
-					
-				}
-				else if(ipo->blocktype==ID_WO) {
-					( (World *)from)->ipo= ipo;
-					id_us_plus(idtest);
-					allqueue(REDRAWBUTSSHADING, 0);
-				}
-				else if(ipo->blocktype==ID_LA) {
-					( (Lamp *)from)->ipo= ipo;
-					id_us_plus(idtest);
-					allqueue(REDRAWBUTSSHADING, 0);
-				}
-				else if(ipo->blocktype==ID_CA) {
-					( (Camera *)from)->ipo= ipo;
-					id_us_plus(idtest);
-					allqueue(REDRAWBUTSEDIT, 0);
-				}
-				else if(ipo->blocktype==ID_SO) {
-					( (bSound *)from)->ipo= ipo;
-					id_us_plus(idtest);
-					allqueue(REDRAWBUTSEDIT, 0);
-				}
-				else
-					printf("error in browse ipo \n");
-				
-				if(id) id->us--;
-				
+				spaceipo_assign_ipo(G.sipo, (Ipo *)idtest);
+									
 				BIF_undo_push("Browse Ipo");
-				scrarea_queue_winredraw(curarea);
-				scrarea_queue_headredraw(curarea);
-				allqueue(REDRAWIPO, 0);
 			}
 		}
 		break;
 	case B_IPODELETE:
-		ipo= get_ipo_to_edit(&from);
-		if(from==0) return;
+		ipo= G.sipo->ipo;
+		from= G.sipo->from;
 		
-		ipo->id.us--;
-		
-		if(ipo->blocktype==ID_OB) ( (Object *)from)->ipo= 0;
-		else if(ipo->blocktype==ID_MA) ( (Material *)from)->ipo= 0;
-		else if(ipo->blocktype==ID_TE) ( (Tex *)from)->ipo= 0;
-		else if(ipo->blocktype==ID_SEQ) ( (Sequence *)from)->ipo= 0;
-		else if(ipo->blocktype==ID_CU) ( (Curve *)from)->ipo= 0;
-		else if(ipo->blocktype==ID_KE) ( (Key *)from)->ipo= 0;
-		else if(ipo->blocktype==ID_WO) ( (World *)from)->ipo= 0;
-		else if(ipo->blocktype==ID_LA) ( (Lamp *)from)->ipo= 0;
-		else if(ipo->blocktype==ID_WO) ( (World *)from)->ipo= 0;
-		else if(ipo->blocktype==ID_CA) ( (Camera *)from)->ipo= 0;
-		else if(ipo->blocktype==ID_SO) ( (bSound *)from)->ipo= 0;
-		else if(ipo->blocktype==ID_AC) {
-			bAction *act = (bAction*) from;
-			bActionChannel *chan = 
-				get_hilighted_action_channel((bAction*)from);
-			BLI_freelinkN (&act->chanbase, chan);
-		}
-		else if(ipo->blocktype==IPO_CO) ((Object *)from)->activecon->ipo= 0;
-
-		else error("Warn bugtracker!");
+		spaceipo_assign_ipo(G.sipo, NULL);
 		
 		editipo_changed(G.sipo, 1); /* doredraw */
+		
 		BIF_undo_push("Unlink Ipo");
-		allqueue(REDRAWIPO, 0);
-		allqueue(REDRAWNLA, 0);
-		allqueue (REDRAWACTION, 0);
 		
 		break;
 	case B_WORLDBROWSE:
@@ -1245,7 +1103,7 @@ void do_global_buttons(unsigned short event)
 	case B_WORLDDELETE:
 		if(G.scene->world) {
 			G.scene->world->id.us--;
-			G.scene->world= 0;
+			G.scene->world= NULL;
 
 			BIF_undo_push("Unlink World");
 			allqueue(REDRAWBUTSSHADING, 0);
@@ -1256,7 +1114,7 @@ void do_global_buttons(unsigned short event)
 	case B_WTEXBROWSE:
 
 		if(G.buts->texnr== -2) {
-			id= 0;
+			id= NULL;
 			wrld= G.scene->world;
 			if(wrld) {
 				mtex= wrld->mtex[ wrld->texact ];
@@ -1272,7 +1130,7 @@ void do_global_buttons(unsigned short event)
 			
 		}
 		else {
-			id= 0;
+			id= NULL;
 			
 			wrld= G.scene->world;
 			if(wrld) {
@@ -1357,7 +1215,7 @@ void do_global_buttons(unsigned short event)
 		if(ob->type!=OB_LAMP) return;
 
 		if(G.buts->texnr== -2) {
-			id= 0;
+			id= NULL;
 			la= ob->data;
 			mtex= la->mtex[ la->texact ];
 			if(mtex) id= (ID *)mtex->tex;
@@ -1371,7 +1229,7 @@ void do_global_buttons(unsigned short event)
 			
 		}
 		else {
-			id= 0;
+			id= NULL;
 			
 			la= ob->data;
 			mtex= la->mtex[ la->texact ];
@@ -1410,7 +1268,7 @@ void do_global_buttons(unsigned short event)
 		break;
 	
 	case B_IMAGEDELETE:
-		G.sima->image= 0;
+		G.sima->image= NULL;
 		image_changed(G.sima, 0);
 		BIF_undo_push("Unlink Image");
 		allqueue(REDRAWIMAGE, 0);
@@ -1475,6 +1333,10 @@ void do_global_buttons(unsigned short event)
 		allqueue(REDRAWVIEW3D, 0);
 		break;
 
+	case B_PLAINMENUS:     /* is button from space.c  *info* */
+		reset_toolbox();
+		break;
+
 	case B_FLIPINFOMENU:	/* is button from space.c  *info* */
 		scrarea_queue_headredraw(curarea);
 		break;
@@ -1637,13 +1499,15 @@ void do_global_buttons(unsigned short event)
 			 * can require it to be updated because its
 			 * basis might have changed... -zr
 			 */
-		if (OBACT && OBACT->type==OB_MBALL)
-			makeDispList(OBACT);
+		if (ob && ob->type==OB_MBALL)
+			DAG_object_flush_update(G.scene, ob, OB_RECALC_DATA);
 			
 		/* redraw because name has changed: new pup */
 		scrarea_queue_headredraw(curarea);
 		allqueue(REDRAWINFO, 1);
 		allqueue(REDRAWOOPS, 1);
+		allqueue(REDRAWACTION, 1);
+		allqueue(REDRAWNLA, 1);
 		/* name scene also in set PUPmenu */
 		allqueue(REDRAWBUTSALL, 0);
 		allqueue(REDRAWHEADERS, 0);
@@ -1740,6 +1604,7 @@ void do_global_buttons2(short event)
 			if(arm->id.us>1) {
 				if(okee("Single user")) {
 					ob->data= copy_armature(arm);
+					armature_rebuild_pose(ob, ob->data);
 					arm->id.us--;
 				}
 			}
@@ -1763,7 +1628,6 @@ void do_global_buttons2(short event)
 			if(act->id.us>1) {
 				if(okee("Single user")) {
 					ob->action=copy_action(act);
-					ob->activecon=NULL;
 					act->id.us--;
 					allqueue(REDRAWACTION, 0);
 				}
@@ -1866,6 +1730,8 @@ void do_global_buttons2(short event)
 				if(okee("Make local")) {
 					make_local_mesh(me);
 					make_local_key( me->key );
+
+					DAG_object_flush_update(G.scene, ob, OB_RECALC_DATA);				
 				}
 			}
 		}
@@ -1901,7 +1767,7 @@ void do_global_buttons2(short event)
 				if(okee("Single user")) {
 					ob->data= copy_curve(cu);
 					cu->id.us--;
-					makeDispList(ob);
+					DAG_object_flush_update(G.scene, ob, OB_RECALC_DATA);
 					if(ob==G.obedit) allqueue(REDRAWVIEW3D, 0);
 				}
 			}
@@ -1914,7 +1780,7 @@ void do_global_buttons2(short event)
 				if(okee("Make local")) {
 					make_local_curve(cu);
 					make_local_key( cu->key );
-					makeDispList(ob);
+					DAG_object_flush_update(G.scene, ob, OB_RECALC_DATA);
 				}
 			}
 		}
@@ -1999,33 +1865,23 @@ void do_global_buttons2(short event)
 		break;
 	
 	case B_IPOALONE:
-		ipo= get_ipo_to_edit(&idfrom);
+		ipo= G.sipo->ipo;
+		idfrom= G.sipo->from;
 		
-		if(idfrom && idfrom->lib==0) {
+		if(idfrom && idfrom->lib==NULL) {
 			if(ipo->id.us>1) {
 				if(okee("Single user")) {
-					if(ipo->blocktype==ID_OB) ((Object *)idfrom)->ipo= copy_ipo(ipo);
-					else if(ipo->blocktype==ID_MA) ((Material *)idfrom)->ipo= copy_ipo(ipo);
-					else if(ipo->blocktype==ID_TE) ((Tex *)idfrom)->ipo= copy_ipo(ipo);
-					else if(ipo->blocktype==ID_SEQ) ((Sequence *)idfrom)->ipo= copy_ipo(ipo);
-					else if(ipo->blocktype==ID_CU) ((Curve *)idfrom)->ipo= copy_ipo(ipo);
-					else if(ipo->blocktype==ID_KE) ((Key *)idfrom)->ipo= copy_ipo(ipo);
-					else if(ipo->blocktype==ID_LA) ((Lamp *)idfrom)->ipo= copy_ipo(ipo);
-					else if(ipo->blocktype==ID_WO) ((World *)idfrom)->ipo= copy_ipo(ipo);
-					else if(ipo->blocktype==ID_CA) ((Camera *)idfrom)->ipo= copy_ipo(ipo);
-					else if(ipo->blocktype==ID_SO) ((bSound *)idfrom)->ipo= copy_ipo(ipo);
-					else if(ipo->blocktype==ID_AC) get_hilighted_action_channel((bAction *)idfrom)->ipo= copy_ipo(ipo);
-					else if(ipo->blocktype==IPO_CO) ((Object *)idfrom)->activecon->ipo= copy_ipo(ipo);
-					else error("Warn ton!");
-					
-					ipo->id.us--;
+					ipo= copy_ipo(ipo);
+					ipo->id.us= 0;	/* assign_ipo adds users, copy_ipo sets to 1 */
+					spaceipo_assign_ipo(G.sipo, ipo);
 					allqueue(REDRAWIPO, 0);
 				}
 			}
 		}
 		break;
 	case B_IPOLOCAL:
-		ipo= get_ipo_to_edit(&idfrom);
+		ipo= G.sipo->ipo;
+		idfrom= G.sipo->from;
 		
 		if(idfrom && idfrom->lib==0) {
 			if(ipo->id.lib) {
@@ -2077,6 +1933,8 @@ void do_global_buttons2(short event)
 					
 					set_mesh(ob, men);
 					
+					DAG_object_flush_update(G.scene, ob, OB_RECALC_DATA);
+					
 					if(ob==G.obedit) allqueue(REDRAWVIEW3D, 0);
 				}
 			}
