@@ -1,5 +1,6 @@
+
 /**
- * $Id: interface_draw.c,v 1.33 2004/12/03 15:28:48 ton Exp $
+ * $Id: interface_draw.c,v 1.49 2006/01/19 14:42:04 campbellbarton Exp $
  *
  * ***** BEGIN GPL/BL DUAL LICENSE BLOCK *****
  *
@@ -41,15 +42,10 @@
 #include <string.h>
 #include <ctype.h>
 
-#ifdef HAVE_CONFIG_H
-#include <config.h>
-#endif
-
 #ifndef WIN32
 #include <unistd.h>
 #else
 #include <io.h>
-#include "BLI_winstuff.h"
 #endif   
 
 #include "MEM_guardedalloc.h"
@@ -63,10 +59,14 @@
 #include "DNA_space_types.h"
 #include "DNA_userdef_types.h"
 #include "DNA_vec_types.h"
+#include "DNA_vfont_types.h"
+#include "DNA_packedFile_types.h"
 
 #include "BKE_blender.h"
 #include "BKE_utildefines.h"
+#include "BKE_font.h"
 #include "BKE_global.h"
+#include "datatoc.h"            /* std font */
 
 #include "BIF_gl.h"
 #include "BIF_graphics.h"
@@ -84,6 +84,10 @@
 
 #include "BSE_view.h"
 
+#ifdef INTERNATIONAL
+#include "FTF_Api.h"
+#endif
+
 #include "mydevice.h"
 #include "interface.h"
 #include "blendef.h"
@@ -91,6 +95,10 @@
 // globals
 extern float UIwinmat[4][4];
 
+// local prototypes 
+void uiDrawBoxShadow(unsigned char alpha, float minx, float miny, float maxx, float maxy);
+
+
 /* ************** safe rasterpos for pixmap alignment with pixels ************* */
 
 void ui_rasterpos_safe(float x, float y, float aspect)
@@ -115,6 +123,9 @@ void ui_rasterpos_safe(float x, float y, float aspect)
 	}
 	
 	if(doit) glRasterPos2f(x, y);
+
+	BIF_RasterPos(x, y);
+	BIF_SetScale(aspect);
 }
 
 /* ************** generic embossed rect, for window sliders etc ************* */
@@ -144,7 +155,7 @@ void uiEmboss(float x1, float y1, float x2, float y2, int sel)
 
 static void ui_draw_icon(uiBut *but, BIFIconID icon)
 {
-	void BIF_icon_pos(float xs, float ys);
+	// void BIF_icon_pos(float xs, float ys);
 	int blend= 0;
 	float xs=0, ys=0;
 
@@ -172,9 +183,6 @@ static void ui_draw_icon(uiBut *but, BIFIconID icon)
 		ys= (but->y1+but->y2- BIF_get_icon_height(icon))/2.0;
 	}
 
-	glRasterPos2f(xs, ys);
-	// BIF_icon_pos(xs, ys);
-
 	if(but->aspect>1.1) glPixelZoom(1.0/but->aspect, 1.0/but->aspect);
 	else if(but->aspect<0.9) glPixelZoom(1.0/but->aspect, 1.0/but->aspect);
 
@@ -187,7 +195,7 @@ static void ui_draw_icon(uiBut *but, BIFIconID icon)
 		else if(but->flag & UI_ACTIVE);
 		else blend= -60;
 	}
-	BIF_draw_icon_blended(icon, but->themecol, blend);
+	BIF_draw_icon_blended(xs, ys, icon, but->themecol, blend);
 	
 	glBlendFunc(GL_ONE, GL_ZERO);
 	glDisable(GL_BLEND);
@@ -1201,12 +1209,18 @@ static void ui_draw_round(int type, int colorid, float asp, float x1, float y1,
 /* super minimal button as used in logic menu */
 static void ui_draw_minimal(int type, int colorid, float asp, float x1, float y1, float x2, float y2, int flag)
 {
-	
+	/* too much space between buttons */
+	/* 
 	x1+= asp;
 	x2-= asp;
 	y1+= asp;
 	y2-= asp;
-
+	*/
+	
+	/* Less space between buttons looks nicer */
+	y2-= asp;
+	x2-= asp;
+	
 	/* paper */
 	if(flag & UI_SELECT) {
 		if(flag & UI_ACTIVE) BIF_ThemeColorShade(colorid, -40);
@@ -1444,6 +1458,8 @@ static void ui_draw_text_icon(uiBut *but)
 	float x;
 	int len;
 	char *cpoin;
+	short t, pos, ch;
+	short selsta_tmp, selend_tmp, selsta_draw, selwidth_draw;
 	
 	/* check for button text label */
 	if (but->type == ICONTEXTROW) {
@@ -1451,28 +1467,57 @@ static void ui_draw_text_icon(uiBut *but)
 	}
 	else {
 
-		// text button cursor
+		/* text button selection and cursor */
 		if(but->pos != -1) {
-			short t, pos, ch;
-			
-			pos= but->pos+strlen(but->str);
-			if(pos >= but->ofs) {
+		
+			if (SELWIDTH > 0) {
+				/* text button selection */
+				selsta_tmp = but->selsta + strlen(but->str);
+				selend_tmp = but->selend + strlen(but->str);
+				if (but->ofs >= strlen(but->str))
+					selsta_tmp += (but->ofs - strlen(but->str));
+					
 				if(but->drawstr[0]!=0) {
-					ch= but->drawstr[pos];
-					but->drawstr[pos]= 0;
-		
-					t= but->aspect*BIF_GetStringWidth(but->font, but->drawstr+but->ofs, (U.transopts & USER_TR_BUTTONS)) + 3;
-		
-					but->drawstr[pos]= ch;
+					ch= but->drawstr[selsta_tmp];
+					but->drawstr[selsta_tmp]= 0;
+					
+					selsta_draw = but->aspect*BIF_GetStringWidth(but->font, but->drawstr+but->ofs, (U.transopts & USER_TR_BUTTONS)) + 3;
+					
+					but->drawstr[selsta_tmp]= ch;
+					
+					
+					ch= but->drawstr[selend_tmp];
+					but->drawstr[selend_tmp]= 0;
+					
+					selwidth_draw = but->aspect*BIF_GetStringWidth(but->font, but->drawstr+but->ofs, (U.transopts & USER_TR_BUTTONS)) + 3;
+					
+					but->drawstr[selend_tmp]= ch;
+					
+					BIF_ThemeColor(TH_BUT_TEXTFIELD_HI);
+					glRects(but->x1+selsta_draw+1, but->y1+2, but->x1+selwidth_draw+1, but->y2-2);
 				}
-				else t= 3;
-				
-				glColor3ub(255,0,0);
-				glRects(but->x1+t, but->y1+2, but->x1+t+3, but->y2-2);
-			}	
+			} else {
+				/* text cursor */
+				pos= but->pos+strlen(but->str);
+				if(pos >= but->ofs) {
+					if(but->drawstr[0]!=0) {
+						ch= but->drawstr[pos];
+						but->drawstr[pos]= 0;
+			
+						t= but->aspect*BIF_GetStringWidth(but->font, but->drawstr+but->ofs, (U.transopts & USER_TR_BUTTONS)) + 3;
+						
+						but->drawstr[pos]= ch;
+					}
+					else t= 3;
+					
+					glColor3ub(255,0,0);
+					glRects(but->x1+t, but->y1+2, but->x1+t+2, but->y2-2);
+				}
+			}
 		}
 		if(but->drawstr[0]!=0) {
 			int transopts;
+			int tog3= 0;
 			
 			// cut string in 2 parts
 			cpoin= strchr(but->drawstr, '|');
@@ -1492,8 +1537,24 @@ static void ui_draw_text_icon(uiBut *but)
 				else x= (but->x1+but->x2-but->strwidth+1)/2.0;
 			}
 			
+			/* tog3 button exception; draws with glColor! */
+			if(but->type==TOG3 && (but->flag & UI_SELECT)) {
+				
+				if( but->pointype==CHA ) {
+					if( BTST( *(but->poin+2), but->bitnr )) tog3= 1;
+				}
+				else if( but->pointype ==SHO ) {
+					short *sp= (short *)but->poin;
+					if( BTST( sp[1], but->bitnr )) tog3= 1;
+				}
+				
+				ui_tog3_invert(but->x1,but->y1,but->x2,but->y2, tog3);
+				if (tog3) glColor3ub(255, 255, 0);
+			}
+			
 			/* text color, with pulldown item exception */
-			if(but->dt==UI_EMBOSSP) {
+			if(tog3);	// color already set
+			else if(but->dt==UI_EMBOSSP) {
 				if((but->flag & (UI_SELECT|UI_ACTIVE)) && but->type!=LABEL) {	// LABEL = title in pulldowns
 					BIF_ThemeColor(TH_MENU_TEXT_HI);
 				} else {
@@ -1508,22 +1569,6 @@ static void ui_draw_text_icon(uiBut *but)
 				}
 			}
 
-			/* tog3 button exception */
-			if(but->type==TOG3 && (but->flag & UI_SELECT)) {
-				int ok= 0;
-				
-				if( but->pointype==CHA ) {
-					if( BTST( *(but->poin+2), but->bitnr )) ok= 1;
-				}
-				else if( but->pointype ==SHO ) {
-					short *sp= (short *)but->poin;
-					if( BTST( sp[1], but->bitnr )) ok= 1;
-				}
-				
-				ui_tog3_invert(but->x1,but->y1,but->x2,but->y2, ok);
-				if (ok) glColor3ub(255, 255, 0);
-			}
-			
 			/* LABEL button exception */
 			if(but->type==LABEL && but->min!=0.0) BIF_ThemeColor(TH_BUT_TEXT_HI);
 		
@@ -1698,6 +1743,158 @@ static void ui_draw_but_HSVCUBE(uiBut *but)
 	fdrawbox((but->x1), (but->y1), (but->x2), (but->y2));
 }
 
+#ifdef INTERNATIONAL
+static void ui_draw_but_CHARTAB(uiBut *but)
+{
+	/* Some local variables */
+	float sx, sy, ex, ey;
+	float width, height;
+	float butw, buth;
+	int x, y, cs;
+	wchar_t wstr[2];
+	unsigned char ustr[16];
+	PackedFile *pf;
+	int result = 0;
+
+	/* Calculate the size of the button */
+	width = abs(but->x2 - but->x1);
+	height = abs(but->y2 - but->y1);
+	
+	butw = floor(width / 12);
+	buth = floor(height / 6);
+	
+	/* Initialize variables */
+	sx = but->x1;
+	ex = but->x1 + butw;
+	sy = but->y1 + height - buth;
+	ey = but->y1 + height;
+
+	cs = G.charstart;
+
+	/* Set the font */
+	if(G.selfont && strcmp(G.selfont->name, "<builtin>"))
+	{
+		char tmpStr[256];
+
+		// Is the font file packed, if so then use the packed file
+		if(G.selfont->packedfile)
+		{
+			pf = G.selfont->packedfile;		
+			FTF_SetFont(pf->data, pf->size, 14.0);
+		}
+		else
+		{
+			int err;
+
+			strcpy(tmpStr, G.selfont->name);
+			BLI_convertstringcode(tmpStr, G.sce, 0);
+			err = FTF_SetFont(tmpStr, 0, 14.0);
+		}
+	}
+
+	/* Start drawing the button itself */
+	glShadeModel(GL_SMOOTH);
+
+	glColor3ub(200,  200,  200);
+	glRectf((but->x1), (but->y1), (but->x2), (but->y2));
+
+	glColor3ub(0,  0,  0);
+	for(y = 0; y < 6; y++)
+	{
+		// Do not draw more than the category allows
+		if(cs > G.charmax) break;
+
+		for(x = 0; x < 12; x++)
+		{
+			// Do not draw more than the category allows
+			if(cs > G.charmax) break;
+
+			// Draw one grid cell
+			glBegin(GL_LINE_LOOP);
+				glVertex2f(sx, sy);
+				glVertex2f(ex, sy);
+				glVertex2f(ex, ey);
+				glVertex2f(sx, ey);				
+			glEnd();	
+
+			// Draw character inside the cell
+			memset(wstr, 0, sizeof(wchar_t)*2);
+			memset(ustr, 0, 16);
+			
+			wstr[0] = cs;
+			wcs2utf8s(ustr, wstr);
+
+			if(G.selfont && strcmp(G.selfont->name, "<builtin>"))
+			{
+				float wid;
+				float llx, lly, llz, urx, ury, urz;
+				float dx, dy;
+				float px, py;
+	
+				// Calculate the position
+				wid = FTF_GetStringWidth(ustr, FTF_USE_GETTEXT | FTF_INPUT_UTF8);
+				FTF_GetBoundingBox(ustr, &llx,&lly,&llz,&urx,&ury,&urz, FTF_USE_GETTEXT | FTF_INPUT_UTF8);
+				dx = urx-llx;
+				dy = ury-lly;
+
+				// This isn't fully functional since the but->aspect isn't working like I suspected
+				px = sx + ((butw/but->aspect)-dx)/2;
+				py = sy + ((buth/but->aspect)-dy)/2;
+
+				// Set the position and draw the character
+				ui_rasterpos_safe(px, py, but->aspect);
+				FTF_DrawString(ustr, FTF_USE_GETTEXT | FTF_INPUT_UTF8);
+			}
+			else
+			{
+				ui_rasterpos_safe(sx + butw/2, sy + buth/2, but->aspect);
+				BIF_DrawString(but->font, ustr, 0);
+			}
+	
+			// Calculate the next position and character
+			sx += butw; ex +=butw;
+			cs++;
+		}
+		/* Add the y position and reset x position */
+		sy -= buth; 
+		ey -= buth;
+		sx = but->x1;
+		ex = but->x1 + butw;
+	}	
+	glShadeModel(GL_FLAT);
+
+	/* Return Font Settings to original */
+	if(U.fontsize && U.fontname[0])
+	{
+		result = FTF_SetFont(U.fontname, 0, U.fontsize);
+	}
+	else if (U.fontsize)
+	{
+		result = FTF_SetFont(datatoc_bfont_ttf, datatoc_bfont_ttf_size, U.fontsize);
+	}
+
+	if (result == 0)
+	{
+		result = FTF_SetFont(datatoc_bfont_ttf, datatoc_bfont_ttf_size, 11);
+	}
+}
+
+#endif // INTERNATIONAL
+
+static void ui_draw_roundbox(uiBut *but)
+{
+	glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
+	glEnable(GL_BLEND);
+	
+	BIF_ThemeColorShadeAlpha(TH_PANEL, but->a2, but->a2);
+
+	uiSetRoundBox(but->a1);
+	gl_round_box(GL_POLYGON, but->x1, but->y1, but->x2, but->y2, but->min);
+
+	glDisable(GL_BLEND);
+}
+
+
 /* nothing! */
 static void ui_draw_nothing(int type, int colorid, float asp, float x1, float y1, float x2, float y2, int flag)
 {
@@ -1716,11 +1913,12 @@ void ui_set_embossfunc(uiBut *but, int drawtype)
 	but->sliderfunc= ui_draw_slider;
 
 	// standard builtin first:
-	if(but->type==LABEL) but->embossfunc= ui_draw_nothing;
+	if(but->type==LABEL || but->type==ROUNDBOX) but->embossfunc= ui_draw_nothing;
 	else if(but->type==PULLDOWN) but->embossfunc= ui_draw_pulldown_round;
 	else if(drawtype==UI_EMBOSSM) but->embossfunc= ui_draw_minimal;
 	else if(drawtype==UI_EMBOSSN) but->embossfunc= ui_draw_nothing;
 	else if(drawtype==UI_EMBOSSP) but->embossfunc= ui_draw_pulldown_item;
+	else if(drawtype==UI_EMBOSSR) but->embossfunc= ui_draw_round;
 	else {
 		int theme= BIF_GetThemeValue(TH_BUT_DRAWTYPE);
 		
@@ -1743,7 +1941,6 @@ void ui_set_embossfunc(uiBut *but, int drawtype)
 	// note: if you want aligning, adapt the call uiBlockEndAlign in interface.c 
 }
 
-
 void ui_draw_but(uiBut *but)
 {
 	double value;
@@ -1785,10 +1982,20 @@ void ui_draw_but(uiBut *but)
 		ui_draw_but_HSVCUBE(but);  // box for colorpicker, three types
 		break;
 
+#ifdef INTERNATIONAL
+	case CHARTAB:
+		value= ui_get_but_val(but);
+		ui_draw_but_CHARTAB(but);
+		break;
+#endif
+
 	case LINK:
 	case INLINK:
 		ui_draw_icon(but, but->icon);
 		break;
+		
+	case ROUNDBOX:
+		ui_draw_roundbox(but);
 
 	default:
 		but->embossfunc(but->type, but->themecol, but->aspect, but->x1, but->y1, but->x2, but->y2, but->flag);
