@@ -1,5 +1,5 @@
 /**
- * $Id: interface_panel.c,v 1.30 2005/03/19 21:08:12 zuster Exp $
+ * $Id: interface_panel.c,v 1.33 2005/08/20 09:55:11 ton Exp $
  *
  * ***** BEGIN GPL/BL DUAL LICENSE BLOCK *****
  *
@@ -486,7 +486,14 @@ int uiNewPanel(ScrArea *sa, uiBlock *block, char *panelname, char *tabname, int
 		pa= pa->next;
 	}
 	
-	if(pa==NULL) {
+	if(pa) {
+		if(pa->sizex != sizex) {
+			pa->sizex= sizex;
+			pa->ofsy+= (pa->sizey - sizey);	// check uiNewPanelHeight()
+			pa->sizey= sizey; 
+		}
+	}
+	else {
 		
 		/* new panel */
 		pa= MEM_callocN(sizeof(Panel), "new panel");
@@ -714,7 +721,7 @@ void uiMatchPanel_view2d(ScrArea *sa)
 	}	
 }
 
-/* extern used ny previewrender */
+/* extern used by previewrender */
 void uiPanelPush(uiBlock *block)
 {
 	glPushMatrix(); 
@@ -889,7 +896,7 @@ static void ui_draw_panel_header(uiBlock *block)
 		/* active tab */
 		/* draw text label */
 		BIF_ThemeColor(TH_TEXT_HI);
-		ui_rasterpos_safe(4+block->minx+pnl_icons, block->maxy+5, block->aspect);
+		ui_rasterpos_safe(4.0f+block->minx+pnl_icons, block->maxy+5.0f, block->aspect);
 		BIF_DrawString(block->curfont, block->panel->panelname, (U.transopts & USER_TR_BUTTONS));
 		return;
 	}
@@ -953,7 +960,7 @@ void ui_draw_panel(uiBlock *block)
 	
 	if(panel->paneltab) return;
 
-	/* if the panel is minimised vertically:
+	/* if the panel is minimized vertically:
 	 * (------)
 	 */
 	if(panel->flag & PNL_CLOSEDY) {
@@ -981,7 +988,7 @@ void ui_draw_panel(uiBlock *block)
 		}
 	
 	}
-	/* if the panel is minimised horizontally:
+	/* if the panel is minimized horizontally:
 	 * /-\
 	 *  |
 	 *  |
@@ -1109,9 +1116,8 @@ void ui_draw_panel(uiBlock *block)
 	
 		ui_draw_x_icon(block->minx+2+ofsx, block->maxy+5);
 		/*
-		glRasterPos2f(block->minx+4, block->maxy+3);
 		if(block->aspect>1.1) glPixelZoom(1.0/block->aspect, 1.0/block->aspect);
-		BIF_draw_icon(ICON_PANEL_CLOSE);
+		BIF_draw_icon(block->minx+4, block->maxy+3, ICON_PANEL_CLOSE);
 		if(block->aspect>1.1) glPixelZoom(1.0, 1.0);
 		*/
 		ofsx= 22;
