@@ -34,13 +34,11 @@
 
 meshtools.c: no editmode, tools operating on meshes
 
-void join_mesh(void);
-void make_sticky(void);
+int join_mesh(void);
 
 void fasterdraw(void);
 void slowerdraw(void);
 
-void vertexnormals_mesh(Mesh *me, float *extverts);
 void sort_faces(void);
 
 */
@@ -53,12 +51,8 @@ void sort_faces(void);
 #include <config.h>
 #endif
 
-#ifdef WIN32
-#include "BLI_winstuff.h"
-#endif
 #include "MEM_guardedalloc.h"
 
-
 #include "DNA_mesh_types.h"
 #include "DNA_meshdata_types.h"
 #include "DNA_object_types.h"
@@ -70,7 +64,7 @@ void sort_faces(void);
 #include "BLI_blenlib.h"
 #include "BLI_arithb.h"
 
-#include "BKE_displist.h"
+#include "BKE_depsgraph.h"
 #include "BKE_global.h"
 #include "BKE_library.h"
 #include "BKE_main.h"
@@ -85,13 +79,18 @@ void sort_faces(void);
 #include "BIF_screen.h"
 #include "BIF_space.h"
 #include "BIF_toolbox.h"
+#include "BIF_editconstraint.h"
 
 #include "BDR_editobject.h" 
+#include "BDR_editface.h" 
+
+#include "BLI_editVert.h"
 
 #include "mydevice.h"
 #include "blendef.h"
 
-#include "render.h" // bad level call (ton)
+#include "BIF_meshtools.h" /* include ourself for prototypes */
+
 
 /* * ********************** no editmode!!! *********** */
 
@@ -116,7 +115,9 @@ static int testSelected_TfaceMesh(void)
 	return 0;
 }	
 
-void join_mesh(void)
+/* join selected meshes into the active mesh, context sensitive
+return 0 if no join is made (error) and 1 of the join is done */
+int join_mesh(void)
 {
 	Base *base, *nextb;
 	Object *ob;
@@ -129,25 +130,23 @@ void join_mesh(void)
 	unsigned int *mcol=NULL, *mcolmain;
 	float imat[4][4], cmat[4][4];
 	int a, b, totcol, totedge=0, totvert=0, totface=0, ok=0, vertofs, map[MAXMAT];
-	int hasedges=0;
 	int	i, j, index, haskey=0, hasdefgroup=0;
 	bDeformGroup *dg, *odg;
 	MDeformVert *dvert, *dvertmain;
 	
-	if(G.obedit) return;
+	if(G.obedit) return 0;
 	
 	ob= OBACT;
-	if(!ob || ob->type!=OB_MESH) return;
+	if(!ob || ob->type!=OB_MESH) return 0;
 	
 	/* count */
 	base= FIRSTBASE;
 	while(base) {
-		if TESTBASE(base) {
+		if TESTBASELIB(base) {
 			if(base->object->type==OB_MESH) {
 				me= base->object->data;
 				totvert+= me->totvert;
 				totface+= me->totface;
-				if(me->medge) hasedges= 1;
 
 				if(base->object == ob) ok= 1;
 
@@ -162,25 +161,21 @@ void join_mesh(void)
 	
 	if(haskey) {
 		error("Can't join meshes with vertex keys");
-		return;
+		return 0;
 	}
 	/* that way the active object is always selected */ 
-	if(ok==0) return;
+	if(ok==0) return 0;
 	
-	if(totvert==0 || totvert>MESH_MAX_VERTS) return;
+	if(totvert==0 || totvert>MESH_MAX_VERTS) return 0;
 	
-	if(okee("Join selected meshes")==0) return;
 
 
 	/* if needed add edges to other meshes */
-	if(hasedges) {
-		for(base= FIRSTBASE; base; base= base->next) {
-			if TESTBASE(base) {
-				if(base->object->type==OB_MESH) {
-					me= base->object->data;
-					if(me->medge==NULL) make_edges(me);
-					totedge += me->totedge;
-				}
+	for(base= FIRSTBASE; base; base= base->next) {
+		if TESTBASELIB(base) {
+			if(base->object->type==OB_MESH) {
+				me= base->object->data;
+				totedge += me->totedge;
 			}
 		}
 	}
@@ -198,7 +193,7 @@ void join_mesh(void)
 	
 	base= FIRSTBASE;
 	while(base) {
-		if TESTBASE(base) {
+		if TESTBASELIB(base) {
 			if(ob!=base->object && base->object->type==OB_MESH) {
 				me= base->object->data;
 
@@ -213,7 +208,7 @@ void join_mesh(void)
 						}
 					}
 					if (!odg){
-						odg = MEM_callocN (sizeof(bDeformGroup), "deformGroup");
+						odg = MEM_callocN (sizeof(bDeformGroup), "join deformGroup");
 						memcpy (odg, dg, sizeof(bDeformGroup));
 						BLI_addtail(&ob->defbase, odg);
 					}
@@ -274,7 +269,7 @@ void join_mesh(void)
 	base= FIRSTBASE;
 	while(base) {
 		nextb= base->next;
-		if TESTBASE(base) {
+		if TESTBASELIB(base) {
 			if(base->object->type==OB_MESH) {
 				
 				me= base->object->data;
@@ -344,7 +339,7 @@ void join_mesh(void)
 					while(a--) {
 						mface->v1+= vertofs;
 						mface->v2+= vertofs;
-						if(mface->v3) mface->v3+= vertofs;
+						mface->v3+= vertofs;
 						if(mface->v4) mface->v4+= vertofs;
 						
 						mface->mat_nr= map[(int)mface->mat_nr];
@@ -353,8 +348,15 @@ void join_mesh(void)
 					}
 					
 					if(tfacemain) {
-						if(me->tface) memcpy(tface, me->tface, me->totface*sizeof(TFace));
-						tface+= me->totface;
+						if(me->tface) {
+							memcpy(tface, me->tface, me->totface*sizeof(TFace));
+							tface+= me->totface;
+						}
+						else {
+							for(a=0; a<me->totface; a++, tface++) {
+								default_tface(tface);
+							}
+						}
 					}
 					
 				}
@@ -429,91 +431,23 @@ void join_mesh(void)
 	/* other mesh users */
 	test_object_materials((ID *)me);
 	
+	DAG_scene_sort(G.scene);	// removed objects, need to rebuild dag before editmode call
+	
 	enter_editmode();
 	exit_editmode(1);	// freedata, but no undo
 	
 	allqueue(REDRAWVIEW3D, 0);
 	allqueue(REDRAWBUTSSHADING, 0);
-	makeDispList(ob);
 
 	BIF_undo_push("Join Mesh");
+	return 1;
 }
 
 
-void make_sticky(void)
-{
-	Object *ob;
-	Base *base;
-	MVert *mvert;
-	Mesh *me;
-	MSticky *ms;
-	float ho[4], mat[4][4];
-	int a;
-	
-	if(G.scene->camera==0) return;
-	
-	if(G.obedit) {
-		error("Unable to make sticky in Edit Mode");
-		return;
-	}
-	base= FIRSTBASE;
-	while(base) {
-		if TESTBASELIB(base) {
-			if(base->object->type==OB_MESH) {
-				ob= base->object;
-				
-				me= ob->data;
-				mvert= me->mvert;
-				if(me->msticky) MEM_freeN(me->msticky);
-				me->msticky= MEM_mallocN(me->totvert*sizeof(MSticky), "sticky");
-				
-				/* like convert to render data */		
-				R.r= G.scene->r;
-				R.r.xsch= (R.r.size*R.r.xsch)/100;
-				R.r.ysch= (R.r.size*R.r.ysch)/100;
-				
-				R.afmx= R.r.xsch/2;
-				R.afmy= R.r.ysch/2;
-				
-				R.ycor= ( (float)R.r.yasp)/( (float)R.r.xasp);
-		
-				R.rectx= R.r.xsch; 
-				R.recty= R.r.ysch;
-				R.xstart= -R.afmx; 
-				R.ystart= -R.afmy;
-				R.xend= R.xstart+R.rectx-1;
-				R.yend= R.ystart+R.recty-1;
-		
-				where_is_object(G.scene->camera);
-				Mat4CpyMat4(R.viewinv, G.scene->camera->obmat);
-				Mat4Ortho(R.viewinv);
-				Mat4Invert(R.viewmat, R.viewinv);
-				
-				RE_setwindowclip(1, -1);
-		
-				where_is_object(ob);
-				Mat4MulMat4(mat, ob->obmat, R.viewmat);
-		
-				ms= me->msticky;
-				for(a=0; a<me->totvert; a++, ms++, mvert++) {
-					VECCOPY(ho, mvert->co);
-					Mat4MulVecfl(mat, ho);
-					RE_projectverto(ho, ho);
-					ms->co[0]= ho[0]/ho[3];
-					ms->co[1]= ho[1]/ho[3];
-				}
-			}
-		}
-		base= base->next;
-	}
-	allqueue(REDRAWBUTSEDIT, 0);
-}
-
 void fasterdraw(void)
 {
 	Base *base;
 	Mesh *me;
-	MFace *mface;
 	int toggle, a;
 
 	if(G.obedit) return;
@@ -531,25 +465,13 @@ void fasterdraw(void)
 			me= base->object->data;
 			if(me->id.lib==0 && (me->flag & ME_ISDONE)==0) {
 				me->flag |= ME_ISDONE;
-				mface= me->mface;
 				toggle= 0;
-				for(a=0; a<me->totface; a++) {
-					if( (mface->edcode & ME_V1V2) && ( (toggle++) & 1) ) {
-						mface->edcode-= ME_V1V2;
+				for(a=0; a<me->totedge; a++) {
+					MEdge *med = &me->medge[a];
+
+					if( (med->flag & ME_EDGEDRAW) && !( (toggle++) & 3) ) {
+						med->flag ^= ME_EDGEDRAW;
 					}
-					if( (mface->edcode & ME_V2V3) && ( (toggle++) & 1)) {
-						mface->edcode-= ME_V2V3;
-					}
-					if( (mface->edcode & ME_V3V1) && ( (toggle++) & 1)) {
-						mface->edcode-= ME_V3V1;
-					}
-					if( (mface->edcode & ME_V4V1) && ( (toggle++) & 1)) {
-						mface->edcode-= ME_V4V1;
-					}
-					if( (mface->edcode & ME_V3V4) && ( (toggle++) & 1)) {
-						mface->edcode-= ME_V3V4;
-					}
-					mface++;
 				}
 			}
 		}
@@ -570,7 +492,6 @@ void slowerdraw(void)		/* reset fasterdraw */
 {
 	Base *base;
 	Mesh *me;
-	MFace *mface;
 	int a;
 
 	if(G.obedit) return;
@@ -580,13 +501,8 @@ void slowerdraw(void)		/* reset fasterdraw */
 		if( TESTBASELIB(base) && (base->object->type==OB_MESH)) {
 			me= base->object->data;
 			if(me->id.lib==0) {
-				
-				mface= me->mface;
-				
-				for(a=0; a<me->totface; a++) {
-				
-					mface->edcode |= ME_V1V2|ME_V2V3;
-					mface++;
+				for(a=0; a<me->totedge; a++) {
+					me->medge[a].flag |= ME_EDGEDRAW;
 				}
 			}
 		}
@@ -596,210 +512,6 @@ void slowerdraw(void)		/* reset fasterdraw */
 	allqueue(REDRAWVIEW3D, 0);
 }
 
-/* ***************** */
-
-/* this one for NOT in editmode 
-
-(only used by external modules, that is, until now by the 
-python NMesh module) 
-
-TODO: Probably it's better to convert the mesh into a EditMesh, call
-vertexnormals() and convert it back to a Mesh again.
-
-*/
-
-static int contrpuntnorm(float *n, float *puno)  /* dutch: check vertex normal */
-{
-	float inp;
-
-	inp= n[0]*puno[0]+n[1]*puno[1]+n[2]*puno[2];
-
-	/* angles 90 degrees: dont flip */
-	if(inp> -0.000001) return 0;
-
-	return 1;
-}
-
-void vertexnormals_mesh(Mesh *me, float *extverts)
-{
-	MVert *mvert;
-	MFace *mface;
-	float n1[3], n2[3], n3[3], n4[3], co[4], fac1, fac2, fac3, fac4, *temp;
-	float *f1, *f2, *f3, *f4, xn, yn, zn, *normals;
-	float *v1, *v2, *v3, *v4, len, vnor[3];
-	int a, testflip;
-
-	if(me->totvert==0) return;
-
-	testflip= (me->flag & ME_NOPUNOFLIP)==0;
-	if((me->flag & ME_TWOSIDED)==0) testflip= 0;	/* large angles */
-	
-	if(me->totface==0) {
-		/* fake vertex normals for 'halopuno' (render option) */
-		mvert= me->mvert;
-		for(a=0; a<me->totvert; a++, mvert++) {
-			VECCOPY(n1, mvert->co);
-			Normalise(n1);
-			mvert->no[0]= 32767.0*n1[0];
-			mvert->no[1]= 32767.0*n1[1];
-			mvert->no[2]= 32767.0*n1[2];
-		}
-		return;
-	}
-
-	normals= MEM_callocN(me->totvert*3*sizeof(float), "normals");
-	
-	/* calculate cosine angles, and add to vertex normal */
-	mface= me->mface;
-	mvert= me->mvert;
-	for(a=0; a<me->totface; a++, mface++) {
-		
-		if(mface->v3==0) continue;
-		
-		if(extverts) {
-			v1= extverts+3*mface->v1;
-			v2= extverts+3*mface->v2;
-			v3= extverts+3*mface->v3;
-			v4= extverts+3*mface->v4;
-		}
-		else {		
-			v1= (mvert+mface->v1)->co;
-			v2= (mvert+mface->v2)->co;
-			v3= (mvert+mface->v3)->co;
-			v4= (mvert+mface->v4)->co;
-		}
-		
-		VecSubf(n1, v2, v1);
-		VecSubf(n2, v3, v2);
-		Normalise(n1);
-		Normalise(n2);
-
-		if(mface->v4==0) {
-			VecSubf(n3, v1, v3);
-			Normalise(n3);
-			
-			co[0]= saacos(-n3[0]*n1[0]-n3[1]*n1[1]-n3[2]*n1[2]);
-			co[1]= saacos(-n1[0]*n2[0]-n1[1]*n2[1]-n1[2]*n2[2]);
-			co[2]= saacos(-n2[0]*n3[0]-n2[1]*n3[1]-n2[2]*n3[2]);
-			
-		}
-		else {
-			VecSubf(n3, v4, v3);
-			VecSubf(n4, v1, v4);
-			Normalise(n3);
-			Normalise(n4);
-
-			co[0]= saacos(-n4[0]*n1[0]-n4[1]*n1[1]-n4[2]*n1[2]);
-			co[1]= saacos(-n1[0]*n2[0]-n1[1]*n2[1]-n1[2]*n2[2]);
-			co[2]= saacos(-n2[0]*n3[0]-n2[1]*n3[1]-n2[2]*n3[2]);
-			co[3]= saacos(-n3[0]*n4[0]-n3[1]*n4[1]-n3[2]*n4[2]);
-		}
-		
-		CalcNormFloat(v1, v2, v3, vnor);
-		
-		temp= normals+3*mface->v1;
-		if(testflip && contrpuntnorm(vnor, temp) ) co[0]= -co[0];
-		temp[0]+= co[0]*vnor[0];
-		temp[1]+= co[0]*vnor[1];
-		temp[2]+= co[0]*vnor[2];
-		
-		temp= normals+3*mface->v2;
-		if(testflip && contrpuntnorm(vnor, temp) ) co[1]= -co[1];
-		temp[0]+= co[1]*vnor[0];
-		temp[1]+= co[1]*vnor[1];
-		temp[2]+= co[1]*vnor[2];
-		
-		temp= normals+3*mface->v3;
-		if(testflip && contrpuntnorm(vnor, temp) ) co[2]= -co[2];
-		temp[0]+= co[2]*vnor[0];
-		temp[1]+= co[2]*vnor[1];
-		temp[2]+= co[2]*vnor[2];
-		
-		if(mface->v4) {
-			temp= normals+3*mface->v4;
-			if(testflip && contrpuntnorm(vnor, temp) ) co[3]= -co[3];
-			temp[0]+= co[3]*vnor[0];
-			temp[1]+= co[3]*vnor[1];
-			temp[2]+= co[3]*vnor[2];
-		}
-	}
-
-	/* normalize vertex normals */
-	mvert= me->mvert;
-	for(a=0; a<me->totvert; a++, mvert++) {
-		len= Normalise(normals+3*a);
-		if(len!=0.0) {
-			VECCOPY(n1, normals+3*a);
-			Normalise(n1);
-
-			mvert->no[0]= 32767.0*n1[0];
-			mvert->no[1]= 32767.0*n1[1];
-			mvert->no[2]= 32767.0*n1[2];
-		}
-	}
-	
-	/* vertex normal flipping flags, for during render */
-	mface= me->mface;
-	mvert= me->mvert;
-	for(a=0; a<me->totface; a++, mface++) {
-		mface->puno=0;			
-		
-		if(mface->v3==0) continue;
-		
-		if(extverts) {
-			v1= extverts+3*mface->v1;
-			v2= extverts+3*mface->v2;
-			v3= extverts+3*mface->v3;
-		}
-		else {		
-			v1= (mvert+mface->v1)->co;
-			v2= (mvert+mface->v2)->co;
-			v3= (mvert+mface->v3)->co;
-		}
-
-		CalcNormFloat(v1, v2, v3, vnor);
-
-		if(testflip) {
-			f1= normals + 3*mface->v1;
-			f2= normals + 3*mface->v2;
-			f3= normals + 3*mface->v3;
-
-			fac1= vnor[0]*f1[0] + vnor[1]*f1[1] + vnor[2]*f1[2];
-			if(fac1<0.0) {
-				mface->puno = ME_FLIPV1;
-			}
-			fac2= vnor[0]*f2[0] + vnor[1]*f2[1] + vnor[2]*f2[2];
-			if(fac2<0.0) {
-				mface->puno += ME_FLIPV2;
-			}
-			fac3= vnor[0]*f3[0] + vnor[1]*f3[1] + vnor[2]*f3[2];
-			if(fac3<0.0) {
-				mface->puno += ME_FLIPV3;
-			}
-			if(mface->v4) {
-				f4= normals + 3*mface->v4;
-				fac4= vnor[0]*f4[0] + vnor[1]*f4[1] + vnor[2]*f4[2];
-				if(fac4<0.0) {
-					mface->puno += ME_FLIPV4;
-				}
-			}
-		}
-		/* proj for cubemap! */
-		xn= fabs(vnor[0]);
-		yn= fabs(vnor[1]);
-		zn= fabs(vnor[2]);
-		
-		if(zn>xn && zn>yn) mface->puno += ME_PROJXY;
-		else if(yn>xn && yn>zn) mface->puno += ME_PROJXZ;
-		else mface->puno += ME_PROJYZ;
-		
-	}
-	
-	MEM_freeN(normals);
-}
-
-
-
 /* ********************** SORT FACES ******************* */
 
 static void permutate(void *list, int num, int size, int *index)
@@ -879,7 +591,229 @@ void sort_faces(void)
 	MEM_freeN(index);
 
 	allqueue(REDRAWVIEW3D, 0);
-	makeDispList(G.obedit);
+	DAG_object_flush_update(G.scene, ob, OB_RECALC_DATA);
+}
+
+/* ********************* MESH VERTEX OCTREE LOOKUP ************* */
+
+/* important note; this is unfinished, needs better API for editmode, and custom threshold */
+
+#define MOC_RES			8
+#define MOC_NODE_RES	8
+#define MOC_THRESH		0.0002f
+
+typedef struct MocNode {
+	struct MocNode *next;
+	int index[MOC_NODE_RES];
+} MocNode;
+
+static int mesh_octree_get_base_offs(float *co, float *offs, float *div)
+{
+	int vx, vy, vz;
+	
+	vx= floor( (co[0]-offs[0])/div[0] );
+	vy= floor( (co[1]-offs[1])/div[1] );
+	vz= floor( (co[2]-offs[2])/div[2] );
+	
+	CLAMP(vx, 0, MOC_RES-1);
+	CLAMP(vy, 0, MOC_RES-1);
+	CLAMP(vz, 0, MOC_RES-1);
+
+	return (vx*MOC_RES*MOC_RES) + vy*MOC_RES + vz;
+}
+
+static void mesh_octree_add_node(MocNode **bt, int index)
+{
+	if(*bt==NULL) {
+		*bt= MEM_callocN(sizeof(MocNode), "MocNode");
+		(*bt)->index[0]= index;
+	}
+	else {
+		int a;
+		for(a=0; a<MOC_NODE_RES; a++) {
+			if((*bt)->index[a]==index)
+				return;
+			else if((*bt)->index[a]==0) {
+				(*bt)->index[a]= index;
+				return;
+			}
+		}
+		mesh_octree_add_node(&(*bt)->next, index);
+	}
+}
+
+static void mesh_octree_free_node(MocNode **bt)
+{
+	if( (*bt)->next ) {
+		mesh_octree_free_node(&(*bt)->next);
+	}
+	MEM_freeN(*bt);
 }
 
 
+/* temporal define, just to make nicer code below */
+#define MOC_ADDNODE(vx, vy, vz)	mesh_octree_add_node(basetable + ((vx)*MOC_RES*MOC_RES) + (vy)*MOC_RES + (vz), index)
+
+static void mesh_octree_add_nodes(MocNode **basetable, float *co, float *offs, float *div, int index)
+{
+	float fx, fy, fz;
+	int vx, vy, vz;
+	
+	fx= (co[0]-offs[0])/div[0];
+	fy= (co[1]-offs[1])/div[1];
+	fz= (co[2]-offs[2])/div[2];
+	CLAMP(fx, 0.0f, MOC_RES-MOC_THRESH);
+	CLAMP(fy, 0.0f, MOC_RES-MOC_THRESH);
+	CLAMP(fz, 0.0f, MOC_RES-MOC_THRESH);
+	
+	vx= floor(fx);
+	vy= floor(fy);
+	vz= floor(fz);
+	
+	MOC_ADDNODE(vx, vy, vz);
+	
+	if( vx>0 )
+		if( fx-((float)vx)-MOC_THRESH < 0.0f)
+			MOC_ADDNODE(vx-1, vy, vz);
+	if( vx<MOC_RES-2 )
+		if( fx-((float)vx)+MOC_THRESH > 1.0f)
+			MOC_ADDNODE(vx+1, vy, vz);
+
+	if( vy>0 )
+		if( fy-((float)vy)-MOC_THRESH < 0.0f) 
+			MOC_ADDNODE(vx, vy-1, vz);
+	if( vy<MOC_RES-2 )
+		if( fy-((float)vy)+MOC_THRESH > 1.0f) 
+			MOC_ADDNODE(vx, vy+1, vz);
+
+	if( vz>0 )
+		if( fz-((float)vz)-MOC_THRESH < 0.0f) 
+			MOC_ADDNODE(vx, vy, vz-1);
+	if( vz<MOC_RES-2 )
+		if( fz-((float)vz)+MOC_THRESH > 1.0f) 
+			MOC_ADDNODE(vx, vy, vz+1);
+	
+}
+
+static int mesh_octree_find_index(MocNode **bt, MVert *mvert, float *co)
+{
+	float *vec;
+	int a;
+	
+	if(*bt==NULL)
+		return -1;
+	
+	for(a=0; a<MOC_NODE_RES; a++) {
+		if((*bt)->index[a]) {
+			/* does mesh verts and editmode, code looks potential dangerous, octree should really be filled OK! */
+			if(mvert) {
+				vec= (mvert+(*bt)->index[a]-1)->co;
+				
+				if(FloatCompare(vec, co, MOC_THRESH))
+					return (*bt)->index[a]-1;
+			}
+			else {
+				EditVert *eve= (EditVert *)INT_TO_POINTER((*bt)->index[a]);
+				
+				if(FloatCompare(eve->co, co, MOC_THRESH))
+					return (int)eve;
+			}
+		}
+		else return -1;
+	}
+	if( (*bt)->next)
+		return mesh_octree_find_index(&(*bt)->next, mvert, co);
+	
+	return -1;
+}
+
+
+/* mode is 's' start, or 'e' end, or 'u' use */
+/* if end, ob can be NULL */
+int mesh_octree_table(Object *ob, float *co, char mode)
+{
+	MocNode **bt;
+	static MocNode **basetable= NULL;
+	static float offs[3], div[3];
+	
+	if(mode=='u') {		/* use table */
+		if(basetable==NULL)
+			mesh_octree_table(ob, NULL, 's');
+	   
+		if(basetable) {
+			Mesh *me= ob->data;
+			bt= basetable + mesh_octree_get_base_offs(co, offs, div);
+			if(ob==G.obedit)
+				return mesh_octree_find_index(bt, NULL, co);
+			else
+				return mesh_octree_find_index(bt, me->mvert, co);
+		}
+		return -1;
+	}
+	else if(mode=='s') {	/* start table */
+		Mesh *me= ob->data;
+		BoundBox *bb = mesh_get_bb(me);
+		
+		/* for quick unit coordinate calculus */
+		VECCOPY(offs, bb->vec[0]);
+		offs[0]+= MOC_THRESH;		/* we offset it 1 threshold unit extra */
+		offs[1]+= MOC_THRESH;
+		offs[2]+= MOC_THRESH;
+			
+		VecSubf(div, bb->vec[6], offs);
+		div[0]+= MOC_THRESH;		/* and divide with 1 threshold unit more extra (try 8x8 unit grid on paint) */
+		div[1]+= MOC_THRESH;
+		div[2]+= MOC_THRESH;
+		
+		VecMulf(div, 1.0f/MOC_RES);
+		if(div[0]==0.0f) div[0]= 1.0f;
+		if(div[1]==0.0f) div[1]= 1.0f;
+		if(div[2]==0.0f) div[2]= 1.0f;
+	
+		if(basetable) /* happens when entering wpaint without closing it */
+			mesh_octree_table(ob, co, 'e');
+		
+		basetable= MEM_callocN(MOC_RES*MOC_RES*MOC_RES*sizeof(void *), "sym table");
+		
+		if(ob==G.obedit) {
+			EditVert *eve;
+			
+			for(eve= G.editMesh->verts.first; eve; eve= eve->next) {
+				mesh_octree_add_nodes(basetable, eve->co, offs, div, POINTER_TO_INT(eve));
+			}
+		}
+		else {		
+			MVert *mvert;
+			int a;
+			
+			for(a=1, mvert= me->mvert; a<=me->totvert; a++, mvert++) {
+				mesh_octree_add_nodes(basetable, mvert->co, offs, div, a);
+			}
+		}
+	}
+	else if(mode=='e') { /* end table */
+		if(basetable) {
+			int a;
+			
+			for(a=0, bt=basetable; a<MOC_RES*MOC_RES*MOC_RES; a++, bt++) {
+				if(*bt) mesh_octree_free_node(bt);
+			}
+			MEM_freeN(basetable);
+			basetable= NULL;
+		}
+	}
+	return 0;
+}
+
+int mesh_get_x_mirror_vert(Object *ob, int index)
+{
+	Mesh *me= ob->data;
+	MVert *mvert= me->mvert+index;
+	float vec[3];
+	
+	vec[0]= -mvert->co[0];
+	vec[1]= mvert->co[1];
+	vec[2]= mvert->co[2];
+	
+	return mesh_octree_table(ob, vec, 'u');
+}
