@@ -1,15 +1,12 @@
 /**
- * $Id: ooutliner.c
+ * $Id:
  *
- * ***** BEGIN GPL/BL DUAL LICENSE BLOCK *****
+ * ***** BEGIN GPL LICENSE BLOCK *****
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
  * as published by the Free Software Foundation; either version 2
- * of the License, or (at your option) any later version. The Blender
- * Foundation also sells licenses for use in proprietary software under
- * the Blender License.  See http://www.blender.org/BL/ for information
- * about this.
+ * of the License, or (at your option) any later version. 
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -27,7 +24,7 @@
  *
  * Contributor(s): none yet.
  *
- * ***** END GPL/BL DUAL LICENSE BLOCK *****
+ * ***** END GPL LICENSE BLOCK *****
  */
 
 #include <string.h>
@@ -35,10 +32,6 @@
 
 #include "MEM_guardedalloc.h"
 
-#ifdef WIN32
-#include "BLI_winstuff.h"
-#endif
-
 #include "DNA_action_types.h"
 #include "DNA_armature_types.h"
 #include "DNA_constraint_types.h"
@@ -51,6 +44,7 @@
 #include "DNA_material_types.h"
 #include "DNA_mesh_types.h"
 #include "DNA_meta_types.h"
+#include "DNA_modifier_types.h"
 #include "DNA_nla_types.h"
 #include "DNA_object_types.h"
 #include "DNA_oops_types.h"
@@ -63,10 +57,13 @@
 
 #include "BLI_blenlib.h"
 
+#include "BKE_constraint.h"
+#include "BKE_depsgraph.h"
 #include "BKE_global.h"
 #include "BKE_library.h"
 #include "BKE_main.h"
 #include "BKE_material.h"
+#include "BKE_modifier.h"
 #include "BKE_screen.h"
 #include "BKE_utildefines.h"
 
@@ -100,15 +97,12 @@
 #include "BDR_editobject.h"
 #include "BSE_drawipo.h"
 #include "BSE_edit.h"
-#include "BSE_editaction.h"
+
+#include "PIL_time.h" 
 
 #include "blendef.h"
 #include "mydevice.h"
 
-#ifdef HAVE_CONFIG_H
-#include <config.h>
-#endif
-
 #define OL_H	19
 #define OL_X	18
 
@@ -370,7 +364,6 @@ static void outliner_sort(SpaceOops *soops, ListBase *lb)
 static void outliner_add_bone(SpaceOops *soops, ListBase *lb, 
 							  ID *id, Bone *curBone, TreeElement *parent, int *a);
 
-
 static TreeElement *outliner_add_element(SpaceOops *soops, ListBase *lb, void *idv, 
 										 TreeElement *parent, short type, short index)
 {
@@ -409,7 +402,7 @@ static TreeElement *outliner_add_element(SpaceOops *soops, ListBase *lb, void *i
 					int a= 0;
 					tenla->name= "Scripts";
 					for (a=0; a<sce->scriptlink.totscript; a++) {
-						outliner_add_element(soops, &tenla->subtree, sce->scriptlink.scripts[a], te, 0, 0);
+						outliner_add_element(soops, &tenla->subtree, sce->scriptlink.scripts[a], tenla, 0, 0);
 					}
 				}
 			}
@@ -419,6 +412,66 @@ static TreeElement *outliner_add_element(SpaceOops *soops, ListBase *lb, void *i
 				Object *ob= (Object *)id;
 				
 				outliner_add_element(soops, &te->subtree, ob->data, te, 0, 0);
+				
+				if(ob->pose) {
+					bPoseChannel *pchan;
+					TreeElement *ten;
+					TreeElement *tenla= outliner_add_element(soops, &te->subtree, ob, te, TSE_POSE_BASE, 0);
+					
+					tenla->name= "Pose";
+					
+					if(ob!=G.obedit) {	// channels undefined in editmode, but we want the 'tenla' pose icon itself
+						int a= 0;
+						for(pchan= ob->pose->chanbase.first; pchan; pchan= pchan->next, a++) {
+							ten= outliner_add_element(soops, &tenla->subtree, ob, tenla, TSE_POSE_CHANNEL, a);
+							ten->name= pchan->name;
+							ten->directdata= pchan;
+							pchan->prev= (bPoseChannel *)ten;
+							
+							if(pchan->constraints.first) {
+								Object *target;
+								bConstraint *con;
+								TreeElement *ten1;
+								TreeElement *tenla1= outliner_add_element(soops, &ten->subtree, ob, ten, TSE_CONSTRAINT_BASE, 0);
+								int a= 0;
+								char *str;
+								
+								tenla1->name= "Constraints";
+								for(con= pchan->constraints.first; con; con= con->next, a++) {
+									ten1= outliner_add_element(soops, &tenla1->subtree, ob, tenla1, TSE_CONSTRAINT, a);
+									target= get_constraint_target(con, &str);
+									if(str && str[0]) ten1->name= str;
+									else if(target) ten1->name= target->id.name+2;
+									else ten1->name= con->name;
+									ten1->directdata= con;
+									/* possible add all other types links? */
+								}
+							}
+						}
+						/* make hierarchy */
+						ten= tenla->subtree.first;
+						while(ten) {
+							TreeElement *nten= ten->next, *par;
+							tselem= TREESTORE(ten);
+							if(tselem->type==TSE_POSE_CHANNEL) {
+								pchan= (bPoseChannel *)ten->directdata;
+								if(pchan->parent) {
+									BLI_remlink(&tenla->subtree, ten);
+									par= (TreeElement *)pchan->parent->prev;
+									BLI_addtail(&par->subtree, ten);
+								}
+							}
+							ten= nten;
+						}
+						/* restore prev pointers */
+						pchan= ob->pose->chanbase.first;
+						if(pchan) pchan->prev= NULL;
+						for(; pchan; pchan= pchan->next) {
+							if(pchan->next) pchan->next->prev= pchan;
+						}
+					}
+				}
+				
 				outliner_add_element(soops, &te->subtree, ob->ipo, te, 0, 0);
 				outliner_add_element(soops, &te->subtree, ob->action, te, 0, 0);
 				
@@ -426,30 +479,44 @@ static TreeElement *outliner_add_element(SpaceOops *soops, ListBase *lb, void *i
 					outliner_add_element(soops, &te->subtree, ob->mat[a], te, 0, a);
 				
 				if(ob->constraints.first) {
+					Object *target;
 					bConstraint *con;
 					TreeElement *ten;
 					TreeElement *tenla= outliner_add_element(soops, &te->subtree, ob, te, TSE_CONSTRAINT_BASE, 0);
 					int a= 0;
+					char *str;
 					
 					tenla->name= "Constraints";
 					for(con= ob->constraints.first; con; con= con->next, a++) {
 						ten= outliner_add_element(soops, &tenla->subtree, ob, tenla, TSE_CONSTRAINT, a);
-						ten->name= con->name;
+						target= get_constraint_target(con, &str);
+						if(str && str[0]) ten->name= str;
+						else if(target) ten->name= target->id.name+2;
+						else ten->name= con->name;
 						ten->directdata= con;
-						outliner_add_element(soops, &ten->subtree, con->ipo, ten, 0, 0);
 						/* possible add all other types links? */
 					}
 				}
-				if(ob->hooks.first) {
-					ObHook *hook;
-					TreeElement *ten;
-					TreeElement *tenla= outliner_add_element(soops, &te->subtree, ob, te, TSE_HOOKS_BASE, 0);
-					int a= 0;
-					
-					tenla->name= "Hooks";
-					for(hook=ob->hooks.first; hook; hook= hook->next, a++) {
-						ten= outliner_add_element(soops, &tenla->subtree, hook->parent, tenla, TSE_HOOK, a);
-						if(ten) ten->name= hook->name;
+				
+				if(ob->modifiers.first) {
+					ModifierData *md;
+					TreeElement *temod = outliner_add_element(soops, &te->subtree, ob, te, TSE_MODIFIER_BASE, 0);
+					int index;
+
+					temod->name = "Modifiers";
+					for (index=0,md=ob->modifiers.first; md; index++,md=md->next) {
+						TreeElement *te = outliner_add_element(soops, &temod->subtree, ob, temod, TSE_MODIFIER, index);
+						te->name= md->name;
+
+						if (md->type==eModifierType_Lattice) {
+							outliner_add_element(soops, &te->subtree, ((LatticeModifierData*) md)->object, te, TSE_MODIFIER_OB, 0);
+						} else if (md->type==eModifierType_Curve) {
+							outliner_add_element(soops, &te->subtree, ((CurveModifierData*) md)->object, te, TSE_MODIFIER_OB, 0);
+						} else if (md->type==eModifierType_Armature) {
+							outliner_add_element(soops, &te->subtree, ((ArmatureModifierData*) md)->object, te, TSE_MODIFIER_OB, 0);
+						} else if (md->type==eModifierType_Hook) {
+							outliner_add_element(soops, &te->subtree, ((HookModifierData*) md)->object, te, TSE_MODIFIER_OB, 0);
+						}
 					}
 				}
 				if(ob->defbase.first) {
@@ -470,7 +537,7 @@ static TreeElement *outliner_add_element(SpaceOops *soops, ListBase *lb, void *i
 					int a= 0;
 					
 					tenla->name= "Scripts";
-					for (a=0; a<ob->scriptlink.totscript; a++) {
+					for (a=0; a<ob->scriptlink.totscript; a++) {							/*  ** */
 						outliner_add_element(soops, &tenla->subtree, ob->scriptlink.scripts[a], te, 0, 0);
 					}
 				}
@@ -601,7 +668,6 @@ static TreeElement *outliner_add_element(SpaceOops *soops, ListBase *lb, void *i
 						}
 						ten= nten;
 					}
-					
 				}
 				else {
 					Bone *curBone;
@@ -616,22 +682,22 @@ static TreeElement *outliner_add_element(SpaceOops *soops, ListBase *lb, void *i
 	return te;
 }
 
+
 /* special handling of hierarchical non-lib data */
 static void outliner_add_bone(SpaceOops *soops, ListBase *lb, ID *id, Bone *curBone, 
 							  TreeElement *parent, int *a)
 {
 	TreeElement *te= outliner_add_element(soops, lb, id, parent, TSE_BONE, *a);
-
+	
 	(*a)++;
 	te->name= curBone->name;
 	te->directdata= curBone;
-
+	
 	for(curBone= curBone->childbase.first; curBone; curBone=curBone->next) {
 		outliner_add_bone(soops, &te->subtree, id, curBone, te, a);
 	}
 }
 
-
 static void outliner_make_hierarchy(SpaceOops *soops, ListBase *lb)
 {
 	TreeElement *te, *ten, *tep;
@@ -682,6 +748,8 @@ static void outliner_build_tree(SpaceOops *soops)
 				ten->directdata= base;
 			}
 			outliner_make_hierarchy(soops, &te->subtree);
+			/* clear id.newid, to prevent objects be inserted in wrong scenes (parent in other scene) */
+			for(base= sce->base.first; base; base= base->next) base->object->id.newid= NULL;
 		}
 	}
 	else if(soops->outlinevis == SO_CUR_SCENE) {
@@ -871,7 +939,6 @@ static void tree_element_active_object(SpaceOops *soops, TreeElement *te)
 	sce= (Scene *)outliner_search_back(soops, te, ID_SCE);
 	if(sce && G.scene != sce) {
 		if(G.obedit) exit_editmode(2);
-		if(G.obpose) exit_posemode(1);
 		set_scene(sce);
 	}
 	
@@ -903,7 +970,6 @@ static void tree_element_active_object(SpaceOops *soops, TreeElement *te)
 	}
 	
 	if(ob!=G.obedit) exit_editmode(2);
-	if(ob!=G.obpose) exit_posemode(1);
 }
 
 static int tree_element_active_material(SpaceOops *soops, TreeElement *te, int set)
@@ -1060,7 +1126,6 @@ static int tree_element_active_world(SpaceOops *soops, TreeElement *te, int set)
 	if(set) {	// make new scene active
 		if(sce && G.scene != sce) {
 			if(G.obedit) exit_editmode(2);
-			if(G.obpose) exit_posemode(1);
 			set_scene(sce);
 		}
 	}
@@ -1091,7 +1156,16 @@ static int tree_element_active_ipo(SpaceOops *soops, TreeElement *te, int set)
 	tselems= TREESTORE(tes);
 	
 	if(set) {
-		ob->ipowin= tes->idcode;
+		if(tes->idcode==ID_AC) {
+			if(ob->ipoflag & OB_ACTION_OB)
+				ob->ipowin= ID_OB;
+			else if(ob->ipoflag & OB_ACTION_KEY)
+				ob->ipowin= ID_KE;
+			else 
+				ob->ipowin= ID_PO;
+		}
+		else ob->ipowin= tes->idcode;
+		
 		if(ob->ipowin==ID_MA) tree_element_active_material(soops, tes, 1);
 		else if(ob->ipowin==ID_AC) {
 			bActionChannel *chan;
@@ -1103,26 +1177,34 @@ static int tree_element_active_ipo(SpaceOops *soops, TreeElement *te, int set)
 			deselect_actionchannels(ob->action, 0);
 			select_channel(ob->action, chan, SELECT_ADD);
 			allqueue(REDRAWACTION, ob->ipowin);
-			if(G.obpose) allqueue(REDRAWVIEW3D, ob->ipowin);
+			allqueue(REDRAWVIEW3D, ob->ipowin);
 		}
 		
 		allqueue(REDRAWIPO, ob->ipowin);
 	}
-	else if(ob->ipowin==tes->idcode) {
-		if(ob->ipowin==ID_MA) {
-			Material *ma= give_current_material(ob, ob->actcol);
-			if(ma==(Material *)tselems->id) return 1;
-		}
-		else if(ob->ipowin==ID_AC) {
-			bActionChannel *chan;
-			short a=0;
-			for(chan=ob->action->chanbase.first; chan; chan= chan->next) {
-				if(a==te->index) break;
-				if(chan->ipo) a++;
+	else {
+		if(tes->idcode==ID_AC) {
+			if(ob->ipoflag & OB_ACTION_OB)
+				return ob->ipowin==ID_OB;
+			else if(ob->ipoflag & OB_ACTION_KEY)
+				return ob->ipowin==ID_KE;
+			else if(ob->ipowin==ID_AC) {
+				bActionChannel *chan;
+				short a=0;
+				for(chan=ob->action->chanbase.first; chan; chan= chan->next) {
+					if(a==te->index) break;
+					if(chan->ipo) a++;
+				}
+				if(chan==get_hilighted_action_channel(ob->action)) return 1;
 			}
-			if(chan==get_hilighted_action_channel(ob->action)) return 1;
 		}
-		else return 1;
+		else if(ob->ipowin==tes->idcode) {
+			if(ob->ipowin==ID_MA) {
+				Material *ma= give_current_material(ob, ob->actcol);
+				if(ma==(Material *)tselems->id) return 1;
+			}
+			else return 1;
+		}
 	}
 	return 0;
 }	
@@ -1135,6 +1217,8 @@ static int tree_element_active_defgroup(TreeElement *te, TreeStoreElem *tselem,
 	ob= (Object *)tselem->id;
 	if(set) {
 		ob->actdef= te->index+1;
+		DAG_object_flush_update(G.scene, ob, OB_RECALC_DATA);
+		allqueue(REDRAWVIEW3D, ob->ipowin);
 	}
 	else {
 		if(ob==OBACT)
@@ -1163,20 +1247,46 @@ static int tree_element_active_nla_action(TreeElement *te, TreeStoreElem *tselem
 	return 0;
 }
 
+static int tree_element_active_posechannel(TreeElement *te, TreeStoreElem *tselem, int set)
+{
+	Object *ob= (Object *)tselem->id;
+	bPoseChannel *pchan= te->directdata;
+	
+	if(set) {
+		if(!(pchan->bone->flag & BONE_HIDDEN_P)) {
+			
+			if(G.qual & LR_SHIFTKEY) deselectall_posearmature(ob, 2);	// 2 = clear active tag
+			else deselectall_posearmature(ob, 0);	// 0 = deselect 
+			pchan->bone->flag |= BONE_SELECTED|BONE_ACTIVE;
+			
+			allqueue(REDRAWVIEW3D, 0);
+			allqueue(REDRAWOOPS, 0);
+			allqueue(REDRAWACTION, 0);
+		}
+	}
+	else {
+		if(ob==OBACT && ob->pose) {
+			if (pchan->bone->flag & BONE_SELECTED) return 1;
+		}
+	}
+	return 0;
+}
 
 static int tree_element_active_bone(TreeElement *te, TreeStoreElem *tselem, int set)
 {
 	bArmature *arm= (bArmature *)tselem->id;
 	Bone *bone= te->directdata;
-
+	
 	if(set) {
-		if(G.qual & LR_SHIFTKEY);
-		else deselectall_posearmature(0);
-		bone->flag |= BONE_SELECTED;
-
-		allqueue(REDRAWVIEW3D, 0);
-		allqueue(REDRAWOOPS, 0);
-		allqueue(REDRAWACTION, 0);
+		if(!(bone->flag & BONE_HIDDEN_P)) {
+			if(G.qual & LR_SHIFTKEY) deselectall_posearmature(OBACT, 2);	// 2 is clear active tag
+			else deselectall_posearmature(OBACT, 0);
+			bone->flag |= BONE_SELECTED|BONE_ACTIVE;
+			
+			allqueue(REDRAWVIEW3D, 0);
+			allqueue(REDRAWOOPS, 0);
+			allqueue(REDRAWACTION, 0);
+		}
 	}
 	else {
 		Object *ob= OBACT;
@@ -1188,24 +1298,26 @@ static int tree_element_active_bone(TreeElement *te, TreeStoreElem *tselem, int
 	return 0;
 }
 
+
 /* ebones only draw in editmode armature */
 static int tree_element_active_ebone(TreeElement *te, TreeStoreElem *tselem, int set)
 {
 	EditBone *ebone= te->directdata;
 	
 	if(set) {
-		if(G.qual & LR_SHIFTKEY);
-		else {
-			ebone->flag |= BONE_SELECTED; // trick deselectall, it toggles... duhh
-			deselectall_armature();
+		if(!(ebone->flag & BONE_HIDDEN_A)) {
+			
+			if(G.qual & LR_SHIFTKEY) deselectall_armature(2);	// only clear active tag
+			else deselectall_armature(0);	// deselect
+
+			ebone->flag |= BONE_SELECTED|BONE_ROOTSEL|BONE_TIPSEL|BONE_ACTIVE;
+			// flush to parent?
+			if(ebone->parent && (ebone->flag & BONE_CONNECTED)) ebone->parent->flag |= BONE_TIPSEL;
+			
+			allqueue(REDRAWVIEW3D, 0);
+			allqueue(REDRAWOOPS, 0);
+			allqueue(REDRAWACTION, 0);
 		}
-		ebone->flag |= BONE_SELECTED|BONE_ROOTSEL|BONE_TIPSEL;
-		// flush to parent?
-		if(ebone->parent && (ebone->flag & BONE_IK_TOPARENT)) ebone->parent->flag |= BONE_TIPSEL;
-		
-		allqueue(REDRAWVIEW3D, 0);
-		allqueue(REDRAWOOPS, 0);
-		allqueue(REDRAWACTION, 0);
 	}
 	else {
 		if (ebone->flag & BONE_SELECTED) return 1;
@@ -1213,6 +1325,14 @@ static int tree_element_active_ebone(TreeElement *te, TreeStoreElem *tselem, int
 	return 0;
 }
 
+static int tree_element_active_modifier(TreeElement *te, TreeStoreElem *tselem, int set)
+{
+	if(set) {
+		extern_set_butspace(F9KEY);
+	}
+	
+	return 0;
+}
 
 static int tree_element_active_text(SpaceOops *soops, TreeElement *te, int set)
 {
@@ -1256,6 +1376,21 @@ static int tree_element_active(SpaceOops *soops, TreeElement *te, int set)
 	return 0;
 }
 
+static int tree_element_active_pose(TreeElement *te, TreeStoreElem *tselem, int set)
+{
+	Object *ob= (Object *)tselem->id;
+	
+	if(set) {
+		if(G.obedit) exit_editmode(2);
+		if(ob->flag & OB_POSEMODE) exit_posemode();
+		else enter_posemode();
+	}
+	else {
+		if(ob->flag & OB_POSEMODE) return 1;
+	}
+	return 0;
+}
+
 /* generic call for non-id data to make/check active in UI */
 static int tree_element_type_active(SpaceOops *soops, TreeElement *te, TreeStoreElem *tselem, int set)
 {
@@ -1269,10 +1404,18 @@ static int tree_element_type_active(SpaceOops *soops, TreeElement *te, TreeStore
 			return tree_element_active_bone(te, tselem, set);
 		case TSE_EBONE:
 			return tree_element_active_ebone(te, tselem, set);
-		case TSE_HOOK: // actually object
+		case TSE_MODIFIER:
+			return tree_element_active_modifier(te, tselem, set);
+		case TSE_MODIFIER_OB:
 			if(set) tree_element_active_object(soops, te);
 			else if(tselem->id==(ID *)OBACT) return 1;
 			break;
+		case TSE_POSE_BASE:
+			return tree_element_active_pose(te, tselem, set);
+			break;
+		case TSE_POSE_CHANNEL:
+			return tree_element_active_posechannel(te, tselem, set);
+			break;
 	}
 	return 0;
 }
@@ -1311,7 +1454,7 @@ static int do_outliner_mouse_event(SpaceOops *soops, TreeElement *te, short even
 			
 			/* activate a name button? */
 			if(G.qual & LR_CTRLKEY) {
-				if(ELEM5(tselem->type, TSE_NLA, TSE_DEFGROUP_BASE, TSE_CONSTRAINT_BASE, TSE_HOOKS_BASE, TSE_SCRIPT_BASE)) 
+				if(ELEM5(tselem->type, TSE_NLA, TSE_DEFGROUP_BASE, TSE_CONSTRAINT_BASE, TSE_MODIFIER_BASE, TSE_SCRIPT_BASE)) 
 					error("Cannot edit builtin name");
 				else {
 					tselem->flag |= TSE_TEXTBUT;
@@ -1326,23 +1469,16 @@ static int do_outliner_mouse_event(SpaceOops *soops, TreeElement *te, short even
 					if(te->idcode==ID_SCE) {
 						if(G.scene!=(Scene *)tselem->id) {
 							if(G.obedit) exit_editmode(2);
-							if(G.obpose) exit_posemode(1);
 							set_scene((Scene *)tselem->id);
 						}
 					}
-					else if(ELEM4(te->idcode, ID_ME, ID_CU, ID_MB, ID_LT)) {
-						if(G.obpose) exit_posemode(1);
+					else if(ELEM5(te->idcode, ID_ME, ID_CU, ID_MB, ID_LT, ID_AR)) {
 						if(G.obedit) exit_editmode(2);
 						else {
 							enter_editmode();
 							extern_set_butspace(F9KEY);
 						}
 					}
-					else if(te->idcode==ID_AR) {
-						if(G.obedit) exit_editmode(2);
-						if(G.obpose) exit_posemode(1);
-						else enter_posemode();
-					}
 					else {	// rest of types
 						tree_element_active(soops, te, 1);
 					}
@@ -1363,7 +1499,7 @@ static int do_outliner_mouse_event(SpaceOops *soops, TreeElement *te, short even
 /* event can enterkey, then it opens/closes */
 void outliner_mouse_event(ScrArea *sa, short event)
 {
-	SpaceOops *soops= curarea->spacedata.first;
+	SpaceOops *soops= sa->spacedata.first;
 	TreeElement *te;
 	float fmval[2];
 	short mval[2];
@@ -1379,6 +1515,8 @@ void outliner_mouse_event(ScrArea *sa, short event)
 		BIF_undo_push("Outliner click event");
 		allqueue(REDRAWOOPS, 0);
 	}
+	else 
+		outliner_select(sa);
 
 }
 
@@ -1504,7 +1642,7 @@ void outliner_select(struct ScrArea *sa )
 	areamouseco_to_ipoco(&so->v2d, mval, fmval, fmval+1);
 	y1= fmval[1];
 
-	while (get_mbut() & R_MOUSE) {
+	while (get_mbut() & (L_MOUSE|R_MOUSE)) {
 		getmouseco_areawin(mval);
 		areamouseco_to_ipoco(&so->v2d, mval, fmval, fmval+1);
 		y2= fmval[1];
@@ -1516,7 +1654,9 @@ void outliner_select(struct ScrArea *sa )
 		
 			y1= y2;
 		}
+		else PIL_sleep_ms(30);
 	}
+	
 	BIF_undo_push("Outliner selection");
 
 }
@@ -1672,7 +1812,6 @@ static void object_delete_cb(TreeElement *te, TreeStoreElem *tselem)
 	
 	if(base) {
 		// check also library later
-		if(G.obpose==base->object) exit_posemode(1);
 		if(G.obedit==base->object) exit_editmode(2);
 		
 		if(base==BASACT) {
@@ -1710,6 +1849,74 @@ static void outliner_do_object_operation(SpaceOops *soops, ListBase *lb,
 	}
 }
 
+static void pchan_cb(int event, TreeElement *te, TreeStoreElem *tselem)
+{
+	bPoseChannel *pchan= (bPoseChannel *)te->directdata;
+	
+	if(event==1)
+		pchan->bone->flag |= BONE_SELECTED;
+	else if(event==2)
+		pchan->bone->flag &= ~BONE_SELECTED;
+	else if(event==3) {
+		pchan->bone->flag |= BONE_HIDDEN_P;
+		pchan->bone->flag &= ~BONE_SELECTED;
+	}
+	else if(event==4)
+		pchan->bone->flag &= ~BONE_HIDDEN_P;
+}
+
+static void bone_cb(int event, TreeElement *te, TreeStoreElem *tselem)
+{
+	Bone *bone= (Bone *)te->directdata;
+	
+	if(event==1)
+		bone->flag |= BONE_SELECTED;
+	else if(event==2)
+		bone->flag &= ~BONE_SELECTED;
+	else if(event==3) {
+		bone->flag |= BONE_HIDDEN_P;
+		bone->flag &= ~BONE_SELECTED;
+	}
+	else if(event==4)
+		bone->flag &= ~BONE_HIDDEN_P;
+}
+
+static void ebone_cb(int event, TreeElement *te, TreeStoreElem *tselem)
+{
+	EditBone *ebone= (EditBone *)te->directdata;
+	
+	if(event==1)
+		ebone->flag |= BONE_SELECTED;
+	else if(event==2)
+		ebone->flag &= ~BONE_SELECTED;
+	else if(event==3) {
+		ebone->flag |= BONE_HIDDEN_A;
+		ebone->flag &= ~BONE_SELECTED|BONE_TIPSEL|BONE_ROOTSEL;
+	}
+	else if(event==4)
+		ebone->flag &= ~BONE_HIDDEN_A;
+}
+
+static void outliner_do_data_operation(SpaceOops *soops, int type, int event, ListBase *lb, 
+										 void (*operation_cb)(int, TreeElement *, TreeStoreElem *))
+{
+	TreeElement *te;
+	TreeStoreElem *tselem;
+	
+	for(te=lb->first; te; te= te->next) {
+		tselem= TREESTORE(te);
+		if(tselem->flag & TSE_SELECTED) {
+			if(tselem->type==type) {
+				operation_cb(event, te, tselem);
+			}
+		}
+		if((tselem->flag & TSE_CLOSED)==0) {
+			outliner_do_data_operation(soops, type, event, &te->subtree, operation_cb);
+		}
+	}
+}
+
+
 void outliner_operation_menu(ScrArea *sa)
 {
 	SpaceOops *soops= sa->spacedata.first;
@@ -1741,11 +1948,11 @@ void outliner_operation_menu(ScrArea *sa)
 			}
 			else if(event==4) {
 				outliner_do_object_operation(soops, &soops->tree, object_delete_cb);
+				DAG_scene_sort(G.scene);
 				str= "Delete Objects";
 			}
 			
 			countall();
-			test_scene_constraints(); // for delete
 			
 			BIF_undo_push(str);
 			allqueue(REDRAWALL, 0); // yah... to be sure :)
@@ -1773,88 +1980,147 @@ void outliner_operation_menu(ScrArea *sa)
 						error("Not yet...");
 				}
 				allqueue(REDRAWOOPS, 0);
+				allqueue(REDRAWBUTSALL, 0);
+				allqueue(REDRAWVIEW3D, 0);
 			}
 		}
 	}
 	else if(datalevel) {
 		if(datalevel==-1) error("Mixed selection");
 		else {
-			error("Not yet...");
-			//pupmenu("Data Operations%t|Delete");
+			if(datalevel==TSE_POSE_CHANNEL) {
+				short event= pupmenu("PoseChannel Operations%t|Select%x1|Deselect%x2|Hide%x3|Unhide%x4");
+				if(event>0) {
+					outliner_do_data_operation(soops, datalevel, event, &soops->tree, pchan_cb);
+					BIF_undo_push("PoseChannel operation");
+				}
+			}
+			else if(datalevel==TSE_BONE) {
+				short event= pupmenu("Bone Operations%t|Select%x1|Deselect%x2|Hide%x3|Unhide%x4");
+				if(event>0) {
+					outliner_do_data_operation(soops, datalevel, event, &soops->tree, bone_cb);
+					BIF_undo_push("Bone operation");
+				}
+			}
+			else if(datalevel==TSE_EBONE) {
+				short event= pupmenu("EditBone Operations%t|Select%x1|Deselect%x2|Hide%x3|Unhide%x4");
+				if(event>0) {
+					outliner_do_data_operation(soops, datalevel, event, &soops->tree, ebone_cb);
+					BIF_undo_push("EditBone operation");
+				}
+			}
+			
+			allqueue(REDRAWOOPS, 0);
+			allqueue(REDRAWBUTSALL, 0);
+			allqueue(REDRAWVIEW3D, 0);
 		}
 	}
-	else error("Nothing selected");
-	
 }
 
 
 /* ***************** DRAW *************** */
 
-static void tselem_draw_icon(TreeStoreElem *tselem)
+static void tselem_draw_icon(float x, float y, TreeStoreElem *tselem, TreeElement *te)
 {
 	if(tselem->type) {
 		switch( tselem->type) {
 			case TSE_NLA:
-				BIF_draw_icon(ICON_NLA); break;
+				BIF_draw_icon(x, y, ICON_NLA); break;
 			case TSE_NLA_ACTION:
-				BIF_draw_icon(ICON_ACTION); break;
+				BIF_draw_icon(x, y, ICON_ACTION); break;
 			case TSE_DEFGROUP_BASE:
-				BIF_draw_icon(ICON_VERTEXSEL); break;
+				BIF_draw_icon(x, y, ICON_VERTEXSEL); break;
 			case TSE_BONE:
 			case TSE_EBONE:
-				BIF_draw_icon(ICON_WPAINT_DEHLT); break;
+				BIF_draw_icon(x, y, ICON_WPAINT_DEHLT); break;
 			case TSE_CONSTRAINT_BASE:
-				BIF_draw_icon(ICON_CONSTRAINT); break;
-			case TSE_HOOKS_BASE:
-				BIF_draw_icon(ICON_HOOK); break;
-			case TSE_HOOK:
-				BIF_draw_icon(ICON_OBJECT); break;
+				BIF_draw_icon(x, y, ICON_CONSTRAINT); break;
+			case TSE_MODIFIER_BASE:
+				BIF_draw_icon(x, y, ICON_MODIFIER); break;
+			case TSE_MODIFIER_OB:
+				BIF_draw_icon(x, y, ICON_OBJECT); break;
+			case TSE_MODIFIER:
+			{
+				Object *ob= (Object *)tselem->id;
+				ModifierData *md= BLI_findlink(&ob->modifiers, tselem->nr);
+				switch(md->type) {
+					case eModifierType_Subsurf: 
+						BIF_draw_icon(x, y, ICON_MOD_SUBSURF); break;
+					case eModifierType_Armature: 
+						BIF_draw_icon(x, y, ICON_ARMATURE); break;
+					case eModifierType_Lattice: 
+						BIF_draw_icon(x, y, ICON_LATTICE); break;
+					case eModifierType_Curve: 
+						BIF_draw_icon(x, y, ICON_CURVE); break;
+					case eModifierType_Build: 
+						BIF_draw_icon(x, y, ICON_MOD_BUILD); break;
+					case eModifierType_Mirror: 
+						BIF_draw_icon(x, y, ICON_MOD_MIRROR); break;
+					case eModifierType_Decimate: 
+						BIF_draw_icon(x, y, ICON_MOD_DECIM); break;
+					case eModifierType_Wave: 
+						BIF_draw_icon(x, y, ICON_MOD_WAVE); break;
+					case eModifierType_Hook: 
+						BIF_draw_icon(x, y, ICON_HOOK); break;
+					case eModifierType_Softbody: 
+						BIF_draw_icon(x, y, ICON_MOD_SOFT); break;
+					case eModifierType_Boolean: 
+						BIF_draw_icon(x, y, ICON_MOD_BOOLEAN); break;
+					default:
+						BIF_draw_icon(x, y, ICON_DOT); break;
+				}
+				break;
+			}
 			case TSE_SCRIPT_BASE:
-				BIF_draw_icon(ICON_TEXT); break;
+				BIF_draw_icon(x, y, ICON_TEXT); break;
+			case TSE_POSE_BASE:
+				BIF_draw_icon(x, y, ICON_ARMATURE_DEHLT); break;
+			case TSE_POSE_CHANNEL:
+				BIF_draw_icon(x, y, ICON_WPAINT_DEHLT); break;
 			default:
-				BIF_draw_icon(ICON_DOT); break;
+				BIF_draw_icon(x, y, ICON_DOT); break;
 		}
 	}
 	else {
 		switch( GS(tselem->id->name)) {
 			case ID_SCE:
-				BIF_draw_icon(ICON_SCENE_DEHLT); break;
+				BIF_draw_icon(x, y, ICON_SCENE_DEHLT); break;
 			case ID_OB:
-				BIF_draw_icon(ICON_OBJECT); break;
+				BIF_draw_icon(x, y, ICON_OBJECT); break;
 			case ID_ME:
-				BIF_draw_icon(ICON_MESH); break;
+				BIF_draw_icon(x, y, ICON_MESH); break;
 			case ID_CU:
-				BIF_draw_icon(ICON_CURVE); break;
+				BIF_draw_icon(x, y, ICON_CURVE); break;
 			case ID_MB:
-				BIF_draw_icon(ICON_MBALL); break;
+				BIF_draw_icon(x, y, ICON_MBALL); break;
 			case ID_LT:
-				BIF_draw_icon(ICON_LATTICE); break;
+				BIF_draw_icon(x, y, ICON_LATTICE); break;
 			case ID_LA:
-				BIF_draw_icon(ICON_LAMP_DEHLT); break;
+				BIF_draw_icon(x, y, ICON_LAMP_DEHLT); break;
 			case ID_MA:
-				BIF_draw_icon(ICON_MATERIAL_DEHLT); break;
+				BIF_draw_icon(x, y, ICON_MATERIAL_DEHLT); break;
 			case ID_TE:
-				BIF_draw_icon(ICON_TEXTURE_DEHLT); break;
+				BIF_draw_icon(x, y, ICON_TEXTURE_DEHLT); break;
 			case ID_IP:
-				BIF_draw_icon(ICON_IPO_DEHLT); break;
+				BIF_draw_icon(x, y, ICON_IPO_DEHLT); break;
 			case ID_IM:
-				BIF_draw_icon(ICON_IMAGE_DEHLT); break;
+				BIF_draw_icon(x, y, ICON_IMAGE_DEHLT); break;
 			case ID_SO:
-				BIF_draw_icon(ICON_SPEAKER); break;
+				BIF_draw_icon(x, y, ICON_SPEAKER); break;
 			case ID_AR:
-				BIF_draw_icon(ICON_ARMATURE_DEHLT); break;
+				BIF_draw_icon(x, y, ICON_ARMATURE); break;
 			case ID_CA:
-				BIF_draw_icon(ICON_CAMERA_DEHLT); break;
+				BIF_draw_icon(x, y, ICON_CAMERA_DEHLT); break;
 			case ID_KE:
-				BIF_draw_icon(ICON_EDIT_DEHLT); break;
+				BIF_draw_icon(x, y, ICON_EDIT_DEHLT); break;
 			case ID_WO:
-				BIF_draw_icon(ICON_WORLD_DEHLT); break;
+				BIF_draw_icon(x, y, ICON_WORLD_DEHLT); break;
 			case ID_AC:
-				BIF_draw_icon(ICON_ACTION); break;
+				BIF_draw_icon(x, y, ICON_ACTION); break;
 			case ID_NLA:
-				BIF_draw_icon(ICON_NLA); break;
+				BIF_draw_icon(x, y, ICON_NLA); break;
 			case ID_TXT:
-				BIF_draw_icon(ICON_SCRIPT); break;
+				BIF_draw_icon(x, y, ICON_SCRIPT); break;
 		}
 	}
 }
@@ -1875,7 +2141,6 @@ static void outliner_draw_iconrow(SpaceOops *soops, ListBase *lb, int level, int
 			active= 0;
 			if(tselem->type==0) {
 				if(te->idcode==ID_OB) active= (OBACT==(Object *)tselem->id);
-				else if(G.obpose && G.obpose->data==tselem->id) active= 1;
 				else if(G.obedit && G.obedit->data==tselem->id) active= 1;
 				else active= tree_element_active(soops, te, 0);
 			}
@@ -1888,8 +2153,7 @@ static void outliner_draw_iconrow(SpaceOops *soops, ListBase *lb, int level, int
 				glEnable(GL_BLEND);
 			}
 			
-			glRasterPos2i(*offsx, ys);
-			tselem_draw_icon(tselem);
+			tselem_draw_icon(*offsx, ys, tselem, te);
 			te->xs= *offsx;
 			te->ys= ys;
 			te->xend= *offsx+OL_X;
@@ -1939,10 +2203,6 @@ static void outliner_draw_tree_element(SpaceOops *soops, TreeElement *te, int st
 					glColor4ubv(col);
 				}
 			}
-			else if(G.obpose && G.obpose->data==tselem->id) {
-				glColor4ub(255, 255, 255, 100);
-				active= 2;
-			}
 			else if(G.obedit && G.obedit->data==tselem->id) {
 				glColor4ub(255, 255, 255, 100);
 				active= 2;
@@ -1970,21 +2230,24 @@ static void outliner_draw_tree_element(SpaceOops *soops, TreeElement *te, int st
 		
 		/* open/close icon, only when sublevels, except for scene */
 		if(te->subtree.first || te->idcode==ID_SCE) {
+			int icon_x;
 			if(tselem->type==0 && (te->idcode==ID_OB || te->idcode==ID_SCE))
-				glRasterPos2i(startx, *starty+2); // icons a bit higher
+				icon_x = startx;
 			else
-				glRasterPos2i(startx+5, *starty+2); // icons a bit higher
+				icon_x = startx+5;
+
+				// icons a bit higher
 			if(tselem->flag & TSE_CLOSED) 
-				BIF_draw_icon(ICON_TRIA_CLOSED);
+				BIF_draw_icon(icon_x, *starty+2, ICON_TRIA_RIGHT);
 			else
-				BIF_draw_icon(ICON_TRIA_OPEN);
+				BIF_draw_icon(icon_x, *starty+2, ICON_TRIA_DOWN);
 		}
 		offsx+= OL_X;
 		
 		/* datatype icon */
 		
-		glRasterPos2i(startx+offsx, *starty+2); // icons a bit higher
-		tselem_draw_icon(tselem);
+			// icons a bit higher
+		tselem_draw_icon(startx+offsx, *starty+2, tselem, te);
 		offsx+= OL_X;
 		glDisable(GL_BLEND);
 
@@ -1992,6 +2255,7 @@ static void outliner_draw_tree_element(SpaceOops *soops, TreeElement *te, int st
 		if(active==1) BIF_ThemeColor(TH_TEXT_HI); 
 		else BIF_ThemeColor(TH_TEXT);
 		glRasterPos2i(startx+offsx, *starty+5);
+		BIF_RasterPos(startx+offsx, *starty+5);
 		BIF_DrawString(G.font, te->name, 0);
 		
 		offsx+= OL_X + BIF_GetStringWidth(G.font, te->name, 0);
@@ -2084,20 +2348,23 @@ static void outliner_draw_tree(SpaceOops *soops)
 {
 	TreeElement *te;
 	int starty, startx;
+	char col[4];
 	
 #ifdef INTERNATIONAL
 	FTF_SetFontSize('l');
+	BIF_SetScale(1.0);
 #endif
 	
 	glBlendFunc(GL_SRC_ALPHA,  GL_ONE_MINUS_SRC_ALPHA); // only once
 
 	// selection first
-	glColor3ub(125, 150, 175);
+	BIF_GetThemeColor3ubv(TH_BACK, col);
+	glColor3ub(col[0]+15, col[1]+20, col[2]+25);
 	starty= soops->v2d.tot.ymax-OL_H;
 	outliner_draw_selection(soops, &soops->tree, &starty);
 	
-	// black hierarchy lines
-	glColor3ub(0,0,0);
+	// grey hierarchy lines
+	BIF_ThemeColorBlend(TH_BACK, TH_TEXT, 0.5);
 	starty= soops->v2d.tot.ymax-OL_H/2;
 	startx= 6;
 	outliner_draw_hierarchy(soops, &soops->tree, startx, &starty);
@@ -2125,67 +2392,87 @@ static void outliner_back(SpaceOops *soops)
 	}
 }
 
-static char bone_newname[40];	// temp storage for bone rename, bones are 32 max.
-
-static void namebutton_cb(void *voidp, void *arg2_unused)
+static void namebutton_cb(void *tep, void *oldnamep)
 {
-	SpaceOops *soops= voidp;
+	SpaceOops *soops= curarea->spacedata.first;
 	TreeStore *ts= soops->treestore;
-	TreeStoreElem *tselem;
-	int a;
+	TreeElement *te= tep;
 	
-	if(ts) {
-		/* only one namebutton can exist */
-		for(a=0, tselem= ts->data; a<ts->usedelem; a++, tselem++) {
-			if(tselem->flag & TSE_TEXTBUT) {
-				if(tselem->type==0) {
-					test_idbutton(tselem->id->name+2);	// library.c, unique name and alpha sort
-				}
-				else {
-					TreeElement *te= outliner_find_tree_element(&soops->tree, a);
-					
-					if(te) {
-						switch(tselem->type) {
-						case TSE_DEFGROUP:
-							unique_vertexgroup_name(te->directdata, (Object *)tselem->id); //	id = object
-							break;
-						case TSE_NLA_ACTION:
-							test_idbutton(tselem->id->name+2);
-							break;
-						case TSE_EBONE:
-							if(G.obedit && G.obedit->data==(ID *)tselem->id) {
-								extern void validate_editbonebutton(EditBone *);
-								EditBone *ebone= te->directdata;
-								validate_editbonebutton(ebone);
-							}
-							allqueue(REDRAWOOPS, 0);
-							allqueue(REDRAWVIEW3D, 1);
-							allqueue(REDRAWBUTSEDIT, 0);
-							break;
-						case TSE_BONE:
-							{
-								Bone *bone= te->directdata;
-								// always make current object active
-								tree_element_active_object(soops, te);
+	if(ts && te) {
+		TreeStoreElem *tselem= TREESTORE(te);
 
-								// dangerous call, it re-allocs the Armature bones, exits editmode too
-								rename_bone_ext(bone->name, bone_newname);
-							}
-							allqueue(REDRAWOOPS, 0);
-							allqueue(REDRAWVIEW3D, 1);
-							allqueue(REDRAWBUTSEDIT, 0);
-							break;
-						}
-					}
+		if(tselem->type==0) {
+			test_idbutton(tselem->id->name+2);	// library.c, unique name and alpha sort
+		}
+		else {
+			switch(tselem->type) {
+			case TSE_DEFGROUP:
+				unique_vertexgroup_name(te->directdata, (Object *)tselem->id); //	id = object
+				break;
+			case TSE_NLA_ACTION:
+				test_idbutton(tselem->id->name+2);
+				break;
+			case TSE_EBONE:
+				if(G.obedit && G.obedit->data==(ID *)tselem->id) {
+					EditBone *ebone= te->directdata;
+					char newname[32];
+					
+					/* restore bone name */
+					BLI_strncpy(newname, ebone->name, 32);
+					BLI_strncpy(ebone->name, oldnamep, 32);
+					armature_bone_rename(G.obedit->data, oldnamep, newname);
 				}
-				tselem->flag &= ~TSE_TEXTBUT;
+				allqueue(REDRAWOOPS, 0);
+				allqueue(REDRAWVIEW3D, 1);
+				allqueue(REDRAWBUTSEDIT, 0);
+				break;
+
+			case TSE_BONE:
+				{
+					Bone *bone= te->directdata;
+					Object *ob;
+					char newname[32];
+					
+					// always make current object active
+					tree_element_active_object(soops, te);
+					ob= OBACT;
+					
+					/* restore bone name */
+					BLI_strncpy(newname, bone->name, 32);
+					BLI_strncpy(bone->name, oldnamep, 32);
+					armature_bone_rename(ob->data, oldnamep, newname);
+				}
+				allqueue(REDRAWOOPS, 0);
+				allqueue(REDRAWVIEW3D, 1);
+				allqueue(REDRAWBUTSEDIT, 0);
+				break;
+			case TSE_POSE_CHANNEL:
+				{
+					bPoseChannel *pchan= te->directdata;
+					Object *ob;
+					char newname[32];
+					
+					// always make current object active
+					tree_element_active_object(soops, te);
+					ob= OBACT;
+					
+					/* restore bone name */
+					BLI_strncpy(newname, pchan->name, 32);
+					BLI_strncpy(pchan->name, oldnamep, 32);
+					armature_bone_rename(ob->data, oldnamep, newname);
+				}
+				allqueue(REDRAWOOPS, 0);
+				allqueue(REDRAWVIEW3D, 1);
+				allqueue(REDRAWBUTSEDIT, 0);
+				break;
+				
 			}
 		}
-		scrarea_queue_redraw(curarea);
 	}
+	scrarea_queue_redraw(curarea);
 }
 
-void outliner_buttons(uiBlock *block, SpaceOops *soops, ListBase *lb)
+static void outliner_buttons(uiBlock *block, SpaceOops *soops, ListBase *lb)
 {
 	uiBut *bt;
 	TreeElement *te;
@@ -2196,24 +2483,24 @@ void outliner_buttons(uiBlock *block, SpaceOops *soops, ListBase *lb)
 		tselem= TREESTORE(te);
 		if(tselem->flag & TSE_TEXTBUT) {
 			
-			// darn bones have complex renaming conventions... cannot edit name itself in button
-			if(tselem->type==TSE_BONE) {
-				strncpy(bone_newname, te->name, 32); // bone_newname is global
-				te->name= bone_newname;
-				len= 24;
-			}
-			else len= 19;
+			if(tselem->type==TSE_EBONE) len = sizeof(((EditBone*) 0)->name);
+			else if (tselem->type==TSE_MODIFIER) len = sizeof(((ModifierData*) 0)->name);
+			else len= sizeof(((ID*) 0)->name)-2;
 			
 			dx= BIF_GetStringWidth(G.font, te->name, 0);
 			if(dx<50) dx= 50;
 			
-			bt= uiDefBut(block, TEX, OL_NAMEBUTTON, "",  te->xs+2*OL_X-4, te->ys, dx+10, OL_H-1, te->name, 1.0, (float)len, 0, 0, "");
-			uiButSetFunc(bt, namebutton_cb, soops, NULL);
+			bt= uiDefBut(block, TEX, OL_NAMEBUTTON, "",  te->xs+2*OL_X-4, te->ys, dx+10, OL_H-1, te->name, 1.0, (float)len-1, 0, 0, "");
+			uiButSetFunc(bt, namebutton_cb, te, NULL);
 
 			// signal for button to open
 			addqueue(curarea->win, BUT_ACTIVATE, OL_NAMEBUTTON);
+			
+			/* otherwise keeps open on ESC */
+			tselem->flag &= ~TSE_TEXTBUT;
 		}
-		if((tselem->flag & TSE_CLOSED)==0) outliner_buttons(block, soops, &te->subtree);
+		else 
+			if((tselem->flag & TSE_CLOSED)==0) outliner_buttons(block, soops, &te->subtree);
 	}
 }
 
@@ -2223,12 +2510,12 @@ void draw_outliner(ScrArea *sa, SpaceOops *soops)
 	int sizey;
 	short ofsx, ofsy;
 	
-	calc_scrollrcts(G.v2d, curarea->winx, curarea->winy);
+	calc_scrollrcts(sa, G.v2d, sa->winx, sa->winy);
 
-	if(curarea->winx>SCROLLB+10 && curarea->winy>SCROLLH+10) {
+	if(sa->winx>SCROLLB+10 && sa->winy>SCROLLH+10) {
 		if(G.v2d->scroll) {	
-			ofsx= curarea->winrct.xmin;	/* because mywin */
-			ofsy= curarea->winrct.ymin;
+			ofsx= sa->winrct.xmin;	/* because mywin */
+			ofsy= sa->winrct.ymin;
 			glViewport(ofsx+G.v2d->mask.xmin,  ofsy+G.v2d->mask.ymin, ( ofsx+G.v2d->mask.xmax-1)-(ofsx+G.v2d->mask.xmin)+1, ( ofsy+G.v2d->mask.ymax-1)-( ofsy+G.v2d->mask.ymin)+1); 
 			glScissor(ofsx+G.v2d->mask.xmin,  ofsy+G.v2d->mask.ymin, ( ofsx+G.v2d->mask.xmax-1)-(ofsx+G.v2d->mask.xmin)+1, ( ofsy+G.v2d->mask.ymax-1)-( ofsy+G.v2d->mask.ymin)+1);
 		}
@@ -2243,7 +2530,7 @@ void draw_outliner(ScrArea *sa, SpaceOops *soops)
 	G.v2d->tot.xmax= (G.v2d->mask.xmax-G.v2d->mask.xmin);
 	G.v2d->tot.ymax= 0.0;
 	G.v2d->tot.ymin= -sizey*OL_H;
-	test_view2d(G.v2d, curarea->winx, curarea->winy);
+	test_view2d(G.v2d, sa->winx, sa->winy);
 
 	// align on top window if cur bigger than tot
 	if(G.v2d->cur.ymax-G.v2d->cur.ymin > sizey*OL_H) {
@@ -2263,7 +2550,7 @@ void draw_outliner(ScrArea *sa, SpaceOops *soops)
 	/* ortho corrected */
 	myortho2(G.v2d->cur.xmin-SCROLLB-0.375, G.v2d->cur.xmax-0.375, G.v2d->cur.ymin-0.375, G.v2d->cur.ymax-0.375);
 	
-	block= uiNewBlock(&curarea->uiblocks, "outliner buttons", UI_EMBOSS, UI_HELV, sa->win);
+	block= uiNewBlock(&sa->uiblocks, "outliner buttons", UI_EMBOSS, UI_HELV, sa->win);
 	outliner_buttons(block, soops, &soops->tree);
 	uiDrawBlock(block);
 	
