@@ -2,7 +2,7 @@
  * 
  * maart 95
  * 
- * $Id: previewrender.c,v 1.58 2005/04/17 17:43:07 ton Exp $
+ * $Id: previewrender.c,v 1.61 2005/09/29 13:19:07 ton Exp $
  *
  * ***** BEGIN GPL/BL DUAL LICENSE BLOCK *****
  *
@@ -652,12 +652,12 @@ static void texture_preview_pixel(Tex *tex, int x, int y, char *rect)
 			}
 			else {
 				skip= 1;
-				ta= 0.0;
+				tr= tg= tb= ta= 0.0;
 			}
 		}
 		else {
 			skip= 1;
-			ta= 0.0;
+			tr= tg= tb= ta= 0.0;
 		}
 	}
 	else {
@@ -795,6 +795,9 @@ static void shade_preview_pixel(ShadeInput *shi, float *vec, int x, int y,char *
 		if(mat->texco & TEXCO_UV) {
 			VECCOPY(shi->uv, shi->lo);
 		}
+		if(mat->texco & TEXCO_STRAND) {
+			shi->strand= shi->lo[0];
+		}
 		if(mat->texco & TEXCO_OBJECT) {
 			VECCOPY(shi->co, shi->lo);
 		}
@@ -883,15 +886,15 @@ static void shade_preview_pixel(ShadeInput *shi, float *vec, int x, int y,char *
 					float specfac;
 					
 					if(mat->spec_shader==MA_SPEC_PHONG) 
-						specfac= Phong_Spec(shi->vn, lv, shi->view, shi->har);
+						specfac= Phong_Spec(shi->vn, lv, shi->view, shi->har, 0);
 					else if(mat->spec_shader==MA_SPEC_COOKTORR) 
-						specfac= CookTorr_Spec(shi->vn, lv, shi->view, shi->har);
+						specfac= CookTorr_Spec(shi->vn, lv, shi->view, shi->har, 0);
 					else if(mat->spec_shader==MA_SPEC_BLINN) 
-						specfac= Blinn_Spec(shi->vn, lv, shi->view, mat->refrac, (float)shi->har);
+						specfac= Blinn_Spec(shi->vn, lv, shi->view, mat->refrac, (float)shi->har, 0);
 					else if(mat->spec_shader==MA_SPEC_WARDISO)
-						specfac= WardIso_Spec(shi->vn, lv, shi->view, mat->rms);
+						specfac= WardIso_Spec(shi->vn, lv, shi->view, mat->rms, 0);
 					else 
-						specfac= Toon_Spec(shi->vn, lv, shi->view, mat->param[2], mat->param[3]);
+						specfac= Toon_Spec(shi->vn, lv, shi->view, mat->param[2], mat->param[3], 0);
 				
 					inprspec= specfac*shi->spec;
 					
@@ -1140,7 +1143,7 @@ void BIF_previewrender(SpaceButs *sbuts)
 		init_render_world();
 		init_render_textures();	/* do not do it twice!! (brightness) */
 		R.totlamp= 0;
-		RE_add_render_lamp(ob, 0);	/* 0=no shadbuf */
+		RE_add_render_lamp(ob, 0);	/* 0=no shadbuf or tables */
 		lar= R.la[0];
 		
 		/* exceptions: */
