@@ -1,5 +1,5 @@
 /**
- * $Id: resources.c,v 1.43 2005/05/13 16:11:28 ton Exp $
+ * $Id: resources.c,v 1.52 2005/11/20 10:04:45 ton Exp $
  *
  * ***** BEGIN GPL/BL DUAL LICENSE BLOCK *****
  *
@@ -58,22 +58,39 @@
 #include "datatoc.h"
 
 /* global for themes */
+typedef void (*VectorDrawFunc)(int x, int y, int w, int h, float alpha);
+
 static bTheme *theme_active=NULL;
 static int theme_spacetype= SPACE_VIEW3D;
 
 typedef struct {
+		/* If drawFunc is defined then it is a vector icon, otherwise use data */
+	VectorDrawFunc drawFunc;
+
+	int w, h;
+
+		/* Data for image icons */
 	unsigned char *data;
 	float uv[4][2];
 	GLuint texid;
-	int w, h;
 } Icon;
 
+static Icon *icon_new_vector(VectorDrawFunc drawFunc, int w, int h)
+{
+	Icon *icon= MEM_callocN(sizeof(*icon), "internicon");
+	icon->drawFunc = drawFunc;
+	icon->w = w;
+	icon->h = h;
+
+	return icon;
+}
 
 static Icon *icon_from_data(unsigned char *rect, GLuint texid, int xofs, int yofs, int w, int h, int rowstride)
 {
 	Icon *icon= MEM_mallocN(sizeof(*icon), "internicon");
 	int y;
 
+	icon->drawFunc = NULL;
 	icon->texid= texid;
 	icon->uv[0][0]= ((float)xofs)/512.0;
 	icon->uv[0][1]= ((float)yofs)/256.0;
@@ -142,28 +159,36 @@ static void icon_draw_tex(Icon *icon)
 #endif
 
 
-static void icon_draw(Icon *icon)
+static void icon_draw(float x, float y, Icon *icon)
 {
-	glDrawPixels(icon->w, icon->h, GL_RGBA, GL_UNSIGNED_BYTE, icon->data);
+	if (icon->drawFunc) {
+		icon->drawFunc(x, y, icon->w, icon->h, 1.0);
+	} else {
+		glRasterPos2f(x, y);
+		glDrawPixels(icon->w, icon->h, GL_RGBA, GL_UNSIGNED_BYTE, icon->data);
+	}
 }
 
 
-static void icon_draw_blended(Icon *icon, char *blendcol, int shade)
+static void icon_draw_blended(float x, float y, Icon *icon, char *blendcol, int shade)
 {
-
-	if(shade < 0) {
-		float r= (128+shade)/128.0;
-		glPixelTransferf(GL_ALPHA_SCALE, r);
+	if (icon->drawFunc) {
+		icon->drawFunc(x, y, icon->w, icon->h, shade<0?((128+shade)/128.0):1.0);
+	} else {
+		if(shade < 0) {
+			float r= (128+shade)/128.0;
+			glPixelTransferf(GL_ALPHA_SCALE, r);
+		}
+		glRasterPos2f(x, y);
+		glDrawPixels(icon->w, icon->h, GL_RGBA, GL_UNSIGNED_BYTE, icon->data);
+		glPixelTransferf(GL_ALPHA_SCALE, 1.0);
 	}
-	glDrawPixels(icon->w, icon->h, GL_RGBA, GL_UNSIGNED_BYTE, icon->data);
-	glPixelTransferf(GL_ALPHA_SCALE, 1.0);
-
 }
 
 
 static void icon_free(Icon *icon)
 {
-	MEM_freeN(icon->data);
+	if (icon->data) MEM_freeN(icon->data);
 	MEM_freeN(icon);
 }
 
@@ -188,17 +213,16 @@ static void free_common_icons(void)
 	}
 }
 
-void BIF_draw_icon(BIFIconID icon)
+void BIF_draw_icon(float x, float y, BIFIconID icon)
 {
-	icon_draw(get_icon(icon));
+	icon_draw(x, y, get_icon(icon));
 }
 
-void BIF_draw_icon_blended(BIFIconID icon, int colorid, int shade)
+void BIF_draw_icon_blended(float x, float y, BIFIconID icon, int colorid, int shade)
 {
 	char *cp= BIF_ThemeGetColorPtr(theme_active, theme_spacetype, colorid);
-	icon_draw_blended(get_icon(icon), cp, shade);
+	icon_draw_blended(x, y, get_icon(icon), cp, shade);
 	// icon_draw_tex(get_icon(icon));
-
 }
 
 int BIF_get_icon_width(BIFIconID icon)
@@ -227,6 +251,17 @@ static void def_icon(ImBuf *bbuf, GLuint texid, BIFIconID icon, int xidx, int yi
 	}
 }
 
+static void def_vicon(BIFIconID icon, int w, int h, VectorDrawFunc drawFunc)
+{
+	int iconidx= icon-BIFICONID_FIRST;
+	
+	if (iconidx>=0 && iconidx<BIFNICONIDS) {
+		common_icons_arr[iconidx]= icon_new_vector(drawFunc, w, h);
+	} else {
+		printf("def_icon: Internal error, bad icon ID: %d\n", icon);
+	}
+}
+
 /* this only works for the hardcoded buttons image, turning the grey AA pixels to alpha, and slight off-grey to half alpha */
 
 static void clear_transp_rect_soft(unsigned char *transp, unsigned char *rect, int w, int h, int rowstride)
@@ -278,6 +313,262 @@ static void clear_transp_rect(unsigned char *transp, unsigned char *rect, int w,
 	}
 }
 
+/* Vector Icon Drawing Routines */
+
+	/* Utilities */
+
+static void viconutil_set_point(GLint pt[2], int x, int y)
+{
+	pt[0] = x;
+	pt[1] = y;
+}
+
+static void viconutil_draw_tri(GLint (*pts)[2])
+{
+	glBegin(GL_TRIANGLES);
+	glVertex2iv(pts[0]);
+	glVertex2iv(pts[1]);
+	glVertex2iv(pts[2]);
+	glEnd();
+}
+
+#if 0
+static void viconutil_draw_quad(GLint (*pts)[2])
+{
+	glBegin(GL_QUADS);
+	glVertex2iv(pts[0]);
+	glVertex2iv(pts[1]);
+	glVertex2iv(pts[2]);
+	glVertex2iv(pts[3]);
+	glEnd();
+}
+#endif
+
+static void viconutil_draw_lineloop(GLint (*pts)[2], int numPoints)
+{
+	int i;
+
+	glBegin(GL_LINE_LOOP);
+	for (i=0; i<numPoints; i++) {
+		glVertex2iv(pts[i]);
+	}
+	glEnd();
+}
+
+static void viconutil_draw_lineloop_smooth(GLint (*pts)[2], int numPoints)
+{
+	glEnable(GL_LINE_SMOOTH);
+	viconutil_draw_lineloop(pts, numPoints);
+	glDisable(GL_LINE_SMOOTH);
+}
+
+static void viconutil_draw_points(GLint (*pts)[2], int numPoints, int pointSize)
+{
+	int i;
+
+	glBegin(GL_QUADS);
+	for (i=0; i<numPoints; i++) {
+		int x = pts[i][0], y = pts[i][1];
+
+		glVertex2i(x-pointSize,y-pointSize);
+		glVertex2i(x+pointSize,y-pointSize);
+		glVertex2i(x+pointSize,y+pointSize);
+		glVertex2i(x-pointSize,y+pointSize);
+	}
+	glEnd();
+}
+
+	/* Drawing functions */
+
+static void vicon_x_draw(int x, int y, int w, int h, float alpha)
+{
+	x += 3;
+	y += 3;
+	w -= 6;
+	h -= 6;
+
+	glEnable( GL_LINE_SMOOTH );
+
+	glLineWidth(2.5);
+	
+	glColor4f(0.0, 0.0, 0.0, alpha);
+	glBegin(GL_LINES);
+	glVertex2i(x  ,y  );
+	glVertex2i(x+w,y+h);
+	glVertex2i(x+w,y  );
+	glVertex2i(x  ,y+h);
+	glEnd();
+
+	glLineWidth(1.0);
+	
+	glDisable( GL_LINE_SMOOTH );
+}
+
+static void vicon_view3d_draw(int x, int y, int w, int h, float alpha)
+{
+	int cx = x + w/2;
+	int cy = y + h/2;
+	int d = MAX2(2, h/3);
+
+	glColor4f(0.5, 0.5, 0.5, alpha);
+	glBegin(GL_LINES);
+	glVertex2i(x  , cy-d);
+	glVertex2i(x+w, cy-d);
+	glVertex2i(x  , cy+d);
+	glVertex2i(x+w, cy+d);
+
+	glVertex2i(cx-d, y  );
+	glVertex2i(cx-d, y+h);
+	glVertex2i(cx+d, y  );
+	glVertex2i(cx+d, y+h);
+	glEnd();
+	
+	glColor4f(0.0, 0.0, 0.0, alpha);
+	glBegin(GL_LINES);
+	glVertex2i(x  , cy);
+	glVertex2i(x+w, cy);
+	glVertex2i(cx, y  );
+	glVertex2i(cx, y+h);
+	glEnd();
+}
+
+static void vicon_edit_draw(int x, int y, int w, int h, float alpha)
+{
+	GLint pts[4][2];
+
+	viconutil_set_point(pts[0], x+3  , y+3  );
+	viconutil_set_point(pts[1], x+w-3, y+3  );
+	viconutil_set_point(pts[2], x+w-3, y+h-3);
+	viconutil_set_point(pts[3], x+3  , y+h-3);
+
+	glColor4f(0.0, 0.0, 0.0, alpha);
+	viconutil_draw_lineloop(pts, 4);
+
+	glColor3f(1, 1, 0.0);
+	viconutil_draw_points(pts, 4, 1);
+}
+
+static void vicon_editmode_hlt_draw(int x, int y, int w, int h, float alpha)
+{
+	GLint pts[3][2];
+
+	viconutil_set_point(pts[0], x+w/2, y+h-2);
+	viconutil_set_point(pts[1], x+3, y+4);
+	viconutil_set_point(pts[2], x+w-3, y+4);
+
+	glColor4f(0.5, 0.5, 0.5, alpha);
+	viconutil_draw_tri(pts);
+
+	glColor4f(0.0, 0.0, 0.0, 1);
+	viconutil_draw_lineloop_smooth(pts, 3);
+
+	glColor3f(1, 1, 0.0);
+	viconutil_draw_points(pts, 3, 1);
+}
+
+static void vicon_editmode_dehlt_draw(int x, int y, int w, int h, float alpha)
+{
+	GLint pts[3][2];
+
+	viconutil_set_point(pts[0], x+w/2, y+h-2);
+	viconutil_set_point(pts[1], x+3, y+4);
+	viconutil_set_point(pts[2], x+w-3, y+4);
+
+	glColor4f(0.0, 0.0, 0.0, 1);
+	viconutil_draw_lineloop_smooth(pts, 3);
+
+	glColor3f(.9, .9, .9);
+	viconutil_draw_points(pts, 3, 1);
+}
+
+static void vicon_disclosure_tri_right_draw(int x, int y, int w, int h, float alpha)
+{
+	GLint pts[3][2];
+	int cx = x+w/2;
+	int cy = y+w/2;
+	int d = w/3, d2 = w/5;
+
+	viconutil_set_point(pts[0], cx-d2, cy+d);
+	viconutil_set_point(pts[1], cx-d2, cy-d);
+	viconutil_set_point(pts[2], cx+d2, cy);
+
+	glShadeModel(GL_SMOOTH);
+	glBegin(GL_TRIANGLES);
+	glColor4f(0.8, 0.8, 0.8, alpha);
+	glVertex2iv(pts[0]);
+	glVertex2iv(pts[1]);
+	glColor4f(0.3, 0.3, 0.3, alpha);
+	glVertex2iv(pts[2]);
+	glEnd();
+	glShadeModel(GL_FLAT);
+
+	glColor4f(0.0, 0.0, 0.0, 1);
+	viconutil_draw_lineloop_smooth(pts, 3);
+}
+
+static void vicon_disclosure_tri_down_draw(int x, int y, int w, int h, float alpha)
+{
+	GLint pts[3][2];
+	int cx = x+w/2;
+	int cy = y+w/2;
+	int d = w/3, d2 = w/5;
+
+	viconutil_set_point(pts[0], cx+d, cy+d2);
+	viconutil_set_point(pts[1], cx-d, cy+d2);
+	viconutil_set_point(pts[2], cx, cy-d2);
+
+	glShadeModel(GL_SMOOTH);
+	glBegin(GL_TRIANGLES);
+	glColor4f(0.8, 0.8, 0.8, alpha);
+	glVertex2iv(pts[0]);
+	glVertex2iv(pts[1]);
+	glColor4f(0.3, 0.3, 0.3, alpha);
+	glVertex2iv(pts[2]);
+	glEnd();
+	glShadeModel(GL_FLAT);
+
+	glColor4f(0.0, 0.0, 0.0, 1);
+	viconutil_draw_lineloop_smooth(pts, 3);
+}
+
+static void vicon_move_up_draw(int x, int y, int w, int h, float alpha)
+{
+	int d=-2;
+
+	glEnable(GL_LINE_SMOOTH);
+	glLineWidth(1);
+	glColor3f(0.0, 0.0, 0.0);
+
+	glBegin(GL_LINE_STRIP);
+	glVertex2i(x+w/2-d*2, y+h/2+d);
+	glVertex2i(x+w/2, y+h/2-d + 1);
+	glVertex2i(x+w/2+d*2, y+h/2+d);
+	glEnd();
+
+	glLineWidth(1.0);
+	glDisable(GL_LINE_SMOOTH);
+}
+
+static void vicon_move_down_draw(int x, int y, int w, int h, float alpha)
+{
+	int d=2;
+
+	glEnable(GL_LINE_SMOOTH);
+	glLineWidth(1);
+	glColor3f(0.0, 0.0, 0.0);
+
+	glBegin(GL_LINE_STRIP);
+	glVertex2i(x+w/2-d*2, y+h/2+d);
+	glVertex2i(x+w/2, y+h/2-d - 1);
+	glVertex2i(x+w/2+d*2, y+h/2+d);
+	glEnd();
+
+	glLineWidth(1.0);
+	glDisable(GL_LINE_SMOOTH);
+}
+
+/***/
+
 void BIF_resources_init(void)
 {
 	ImBuf *bbuf= IMB_ibImageFromMemory((int *)datatoc_blenderbuttons, datatoc_blenderbuttons_size, IB_rect);
@@ -325,6 +616,16 @@ void BIF_resources_init(void)
 		}
 	}
 
+	def_vicon(VICON_VIEW3D, 16, 16, vicon_view3d_draw);
+	def_vicon(VICON_EDIT, 16, 16, vicon_edit_draw);
+	def_vicon(VICON_EDITMODE_DEHLT, 16, 16, vicon_editmode_dehlt_draw);
+	def_vicon(VICON_EDITMODE_HLT, 16, 16, vicon_editmode_hlt_draw);
+	def_vicon(VICON_DISCLOSURE_TRI_RIGHT, 16, 16, vicon_disclosure_tri_right_draw);
+	def_vicon(VICON_DISCLOSURE_TRI_DOWN, 16, 16, vicon_disclosure_tri_down_draw);
+	def_vicon(VICON_MOVE_UP, 16, 16, vicon_move_up_draw);
+	def_vicon(VICON_MOVE_DOWN, 16, 16, vicon_move_down_draw);
+	def_vicon(VICON_X, 16, 16, vicon_x_draw);
+
 	IMB_freeImBuf(bbuf);	
 }
 
@@ -373,6 +674,8 @@ char *BIF_ThemeGetColorPtr(bTheme *btheme, int spacetype, int colorid)
 				cp= btheme->tui.num; break;
 			case TH_BUT_TEXTFIELD:
 				cp= btheme->tui.textfield; break;
+			case TH_BUT_TEXTFIELD_HI:
+				cp= btheme->tui.textfield_hi; break;
 			case TH_BUT_POPUP:
 				cp= btheme->tui.popup; break;
 			case TH_BUT_TEXT:
@@ -479,6 +782,8 @@ char *BIF_ThemeGetColorPtr(bTheme *btheme, int spacetype, int colorid)
 				cp= ts->grid; break;
 			case TH_WIRE:
 				cp= ts->wire; break;
+			case TH_LAMP:
+				cp= ts->lamp; break;
 			case TH_SELECT:
 				cp= ts->select; break;
 			case TH_ACTIVE:
@@ -509,7 +814,15 @@ char *BIF_ThemeGetColorPtr(bTheme *btheme, int spacetype, int colorid)
 				cp= &ts->facedot_size; break;
 			case TH_NORMAL:
 				cp= ts->normal; break;
-
+			case TH_BONE_SOLID:
+				cp= ts->bone_solid; break;
+			case TH_BONE_POSE:
+				cp= ts->bone_pose; break;
+			case TH_STRIP:
+				cp= ts->strip; break;
+			case TH_STRIP_SELECT:
+				cp= ts->strip_select; break;
+				
 			case TH_SYNTAX_B:
 				cp= ts->syntaxb; break;
 			case TH_SYNTAX_V:
@@ -561,6 +874,7 @@ void BIF_InitTheme(void)
 	SETCOL(btheme->tui.setting2, 	0xA1,0x99,0xA7, 255);
 	SETCOL(btheme->tui.num,		 	0x90,0x90,0x90, 255);
 	SETCOL(btheme->tui.textfield,	0x90,0x90,0x90, 255);
+	SETCOL(btheme->tui.textfield_hi,0xc6,0x77,0x77, 255);
 	SETCOL(btheme->tui.popup,		0xA0,0xA0,0xA0, 255);
 	
 	SETCOL(btheme->tui.text,		0,0,0, 255);
@@ -585,6 +899,7 @@ void BIF_InitTheme(void)
 
 	SETCOL(btheme->tv3d.grid, 	92, 92, 92, 255);
 	SETCOL(btheme->tv3d.wire, 	0x0, 0x0, 0x0, 255);
+	SETCOL(btheme->tv3d.lamp, 	0, 0, 0, 40);
 	SETCOL(btheme->tv3d.select, 0xff, 0x88, 0xff, 255);
 	SETCOL(btheme->tv3d.active, 0xff, 0xbb, 0xff, 255);
 	SETCOL(btheme->tv3d.transform, 0xff, 0xff, 0xff, 255);
@@ -601,6 +916,10 @@ void BIF_InitTheme(void)
 	SETCOL(btheme->tv3d.face_dot, 255, 138, 48, 255);
 	btheme->tv3d.facedot_size= 4;
 	
+	SETCOL(btheme->tv3d.bone_solid, 200, 200, 200, 255);
+	SETCOL(btheme->tv3d.bone_pose, 80, 200, 255, 80);		// alpha 80 is not meant editable, used for wire+action draw
+	
+	
 	/* space buttons */
 	/* to have something initialized */
 	btheme->tbuts= btheme->tv3d;
@@ -655,6 +974,8 @@ void BIF_InitTheme(void)
 	SETCOL(btheme->tnla.shade1,  172, 172, 172, 255);		// sliders
 	SETCOL(btheme->tnla.shade2,  84, 44, 31, 100);	// bar
 	SETCOL(btheme->tnla.hilite,  17, 27, 60, 100);	// bar
+	SETCOL(btheme->tnla.strip_select, 	0xff, 0xff, 0xaa, 255);
+	SETCOL(btheme->tnla.strip, 0xe4, 0x9c, 0xc6, 255);
 	
 	/* space seq */
 	btheme->tseq= btheme->tv3d;
@@ -708,7 +1029,7 @@ void BIF_InitTheme(void)
 
 char *BIF_ThemeColorsPup(int spacetype)
 {
-	char *cp= MEM_callocN(21*32, "theme pup");
+	char *cp= MEM_callocN(32*32, "theme pup");
 	char str[32];
 	
 	if(spacetype==0) {
@@ -720,6 +1041,7 @@ char *BIF_ThemeColorsPup(int spacetype)
 		sprintf(str, "Special Setting 2 %%x%d|", TH_BUT_SETTING2); strcat(cp, str);
 		sprintf(str, "Number Input %%x%d|", TH_BUT_NUM); strcat(cp, str);
 		sprintf(str, "Text Input %%x%d|", TH_BUT_TEXTFIELD); strcat(cp, str);
+		sprintf(str, "Text Input Highlight %%x%d|", TH_BUT_TEXTFIELD_HI); strcat(cp, str);
 		sprintf(str, "Popup %%x%d|", TH_BUT_POPUP); strcat(cp, str);
 		sprintf(str, "Text %%x%d|", TH_BUT_TEXT); strcat(cp, str);
 		sprintf(str, "Text Highlight %%x%d|", TH_BUT_TEXT_HI); strcat(cp, str);
@@ -744,6 +1066,7 @@ char *BIF_ThemeColorsPup(int spacetype)
 			strcat(cp,"%l|");
 			sprintf(str, "Grid %%x%d|", TH_GRID); strcat(cp, str);
 			sprintf(str, "Wire %%x%d|", TH_WIRE); strcat(cp, str);
+			sprintf(str, "Lamp %%x%d|", TH_LAMP); strcat(cp, str);
 			sprintf(str, "Object Selected %%x%d|", TH_SELECT); strcat(cp, str);
 			sprintf(str, "Object Active %%x%d|", TH_ACTIVE); strcat(cp, str);
 			sprintf(str, "Transform %%x%d|", TH_TRANSFORM); strcat(cp, str);
@@ -758,7 +1081,9 @@ char *BIF_ThemeColorsPup(int spacetype)
 			sprintf(str, "Face Selected (transp) %%x%d|", TH_FACE_SELECT); strcat(cp, str);
 			sprintf(str, "Face Dot Selected %%x%d|", TH_FACE_DOT); strcat(cp, str);
 			sprintf(str, "Face Dot Size %%x%d|", TH_FACEDOT_SIZE); strcat(cp, str);
-			sprintf(str, "Normal %%x%d", TH_NORMAL); strcat(cp, str);
+			sprintf(str, "Normal %%x%d|", TH_NORMAL); strcat(cp, str);
+			sprintf(str, "Bone Solid %%x%d|", TH_BONE_SOLID); strcat(cp, str);
+			sprintf(str, "Bone Pose %%x%d", TH_BONE_POSE); strcat(cp, str);
 		}
 		else if(spacetype==SPACE_IPO) {
 			sprintf(str, "Panel %%x%d|", TH_PANEL); strcat(cp, str);
@@ -779,6 +1104,8 @@ char *BIF_ThemeColorsPup(int spacetype)
 			sprintf(str, "View Sliders %%x%d|", TH_SHADE1); strcat(cp, str);
 			sprintf(str, "Bars %%x%d|", TH_SHADE2); strcat(cp, str);
 			sprintf(str, "Bars selected %%x%d|", TH_HILITE); strcat(cp, str);
+			sprintf(str, "Strips %%x%d|", TH_STRIP); strcat(cp, str);
+			sprintf(str, "Strips selected %%x%d|", TH_STRIP_SELECT); strcat(cp, str);
 		}
 		else if(spacetype==SPACE_ACTION) {
 			//sprintf(str, "Panel %%x%d|", TH_PANEL); strcat(cp, str);
