@@ -1,5 +1,5 @@
 /**
- * $Id: sequence.c,v 1.19 2004/12/10 14:50:39 ton Exp $
+ * $Id: sequence.c,v 1.33 2005/11/29 21:14:42 guitargeek Exp $
  *
  * ***** BEGIN GPL/BL DUAL LICENSE BLOCK *****
  *
@@ -38,10 +38,6 @@
 #include <config.h>
 #endif
 
-#ifdef WIN32
-#include "BLI_winstuff.h"
-#endif
-
 #include "MEM_guardedalloc.h"
 #include "PIL_dynlib.h"
 
@@ -74,7 +70,7 @@
 #include "BSE_sequence.h"
 
 #include "blendef.h"
-#include "render.h"
+#include "render.h"		// talks to entire render API, and igamtab
 
 Sequence *seq_arr[MAXSEQ+1];
 int seqrectx, seqrecty;
@@ -133,7 +129,7 @@ void open_plugin_seq(PluginSeq *pis, char *seqname)
 				MEM_freeN(info);
 
 				cp= PIL_dynlib_find_symbol(pis->handle, "seqname");
-				if(cp) strcpy(cp, seqname);
+				if(cp) strncpy(cp, seqname, 21);
 			} else {
 				printf ("Plugin returned unrecognized version number\n");
 				return;
@@ -150,7 +146,7 @@ PluginSeq *add_plugin_seq(char *str, char *seqname)
 
 	pis= MEM_callocN(sizeof(PluginSeq), "PluginSeq");
 
-	strcpy(pis->name, str);
+	strncpy(pis->name, str, FILE_MAXDIR+FILE_MAXFILE);
 	open_plugin_seq(pis, seqname);
 
 	if(pis->doit==0) {
@@ -628,11 +624,13 @@ void do_cross_effect(float facf0, float facf1, int x, int y, unsigned int *rect1
 
 void do_gammacross_effect(float facf0, float facf1, int x, int y, unsigned int *rect1, unsigned int *rect2, unsigned int *out)
 {
-/*  	extern unsigned short *igamtab1, *gamtab; render.h */
+	extern void init_filt_mask(void);	// initrender.c, bad level call...
 	int fac1, fac2, col;
 	int xo;
 	char *rt1, *rt2, *rt;
 
+	init_filt_mask();	// nasty call to render code... but it uses gamtabs here
+	
 	xo= x;
 	rt1= (char *)rect1;
 	rt2= (char *)rect2;
@@ -1488,11 +1486,13 @@ void RVIsolateHighlights (unsigned char* in, unsigned char* out, int width, int
 			out[index+GlowR]=MIN2(255*clamp, (in[index+GlowR]*boost*intensity)/255);
 			out[index+GlowG]=MIN2(255*clamp, (in[index+GlowG]*boost*intensity)/255);
 			out[index+GlowB]=MIN2(255*clamp, (in[index+GlowB]*boost*intensity)/255);
+			out[index+GlowA]=MIN2(255*clamp, (in[index+GlowA]*boost*intensity)/255);
 			}
 			else{
 				out[index+GlowR]=0;
 				out[index+GlowG]=0;
 				out[index+GlowB]=0;
+				out[index+GlowA]=0;
 			}
 		}
 	}
@@ -1652,7 +1652,7 @@ void do_effect(int cfra, Sequence *seq, StripElem *se)
 			if(seq->plugin->cfra) *(seq->plugin->cfra)= frame_to_float(CFRA);
 
 			cp= PIL_dynlib_find_symbol(seq->plugin->handle, "seqname");
-			if(cp) strcpy(cp, seq->name+2);
+			if(cp) strncpy(cp, seq->name+2, 22);
 
 			if (seq->plugin->version<=2) {
 				if(se1->ibuf) IMB_convert_rgba_to_abgr(se1->ibuf->x*se1->ibuf->y, se1->ibuf->rect);
@@ -1712,16 +1712,28 @@ StripElem *give_stripelem(Sequence *seq, int cfra)
 	if(se==0) return 0;
 	if(seq->startdisp >cfra || seq->enddisp <= cfra) return 0;
 
-	if(cfra <= seq->start) nr= 0;
-	else if(cfra >= seq->start+seq->len-1) nr= seq->len-1;
-	else nr= cfra-seq->start;
+	if(seq->flag&SEQ_REVERSE_FRAMES)	{	
+		/*reverse frame in this sequence */
+		if(cfra <= seq->start) nr= seq->len-1;
+		else if(cfra >= seq->start+seq->len-1) nr= 0;
+		else nr= (seq->start + seq->len) - cfra;
+	} else {
+		if(cfra <= seq->start) nr= 0;
+		else if(cfra >= seq->start+seq->len-1) nr= seq->len-1;
+		else nr= cfra-seq->start;
+	}
+	if (seq->strobe < 1.0) seq->strobe = 1.0;
+	if (seq->strobe > 1.0) {
+		nr -= (int)fmod((double)nr, (double)seq->strobe);
+	}
 
-	se+= nr;
+	se+= nr; /* don't get confused by the increment, this is the same as strip->stripdata[nr], which works on some compilers...*/
 	se->nr= nr;
 
 	return se;
 }
 
+ 
 void set_meta_stripdata(Sequence *seqm)
 {
 	Sequence *seq, *seqim, *seqeff;
@@ -1805,7 +1817,7 @@ void do_build_seqar_cfra(ListBase *seqbase, Sequence ***seqar, int cfra)
 	Scene *oldsce;
 	unsigned int *rectot;
 	int oldx, oldy, oldcfra, doseq;
-	char name[FILE_MAXDIR];
+	char name[FILE_MAXDIR+FILE_MAXFILE];
 
 	if(seqar==NULL) return;
 	
@@ -1891,8 +1903,8 @@ void do_build_seqar_cfra(ListBase *seqbase, Sequence ***seqar, int cfra)
 							/* if playanim or render: no waitcursor */
 							if((G.f & G_PLAYANIM)==0) waitcursor(1);
 
-							strcpy(name, seq->strip->dir);
-							strcat(name, se->name);
+							strncpy(name, seq->strip->dir, FILE_MAXDIR-1);
+							strncat(name, se->name, FILE_MAXFILE);
 							BLI_convertstringcode(name, G.sce, G.scene->r.cfra);
 							se->ibuf= IMB_loadiffname(name, IB_rect);
 
@@ -1900,9 +1912,14 @@ void do_build_seqar_cfra(ListBase *seqbase, Sequence ***seqar, int cfra)
 
 							if(se->ibuf==0) se->ok= 0;
 							else {
-								if(se->ibuf->depth==32 && se->ibuf->zbuf==0) converttopremul(se->ibuf);
+								if(seq->flag & SEQ_MAKE_PREMUL) {
+									if(se->ibuf->depth==32 && se->ibuf->zbuf==0) converttopremul(se->ibuf);
+								}
 								seq->strip->orx= se->ibuf->x;
 								seq->strip->ory= se->ibuf->y;
+								if(seq->flag & SEQ_FILTERY) IMB_filtery(se->ibuf);
+								if(seq->mul==0.0) seq->mul= 1.0;
+								if(seq->mul != 1.0) multibuf(se->ibuf, seq->mul);
 							}
 						}
 					}
@@ -1913,8 +1930,8 @@ void do_build_seqar_cfra(ListBase *seqbase, Sequence ***seqar, int cfra)
 							if((G.f & G_PLAYANIM)==0) waitcursor(1);
 
 							if(seq->anim==0) {
-								strcpy(name, seq->strip->dir);
-								strcat(name, seq->strip->stripdata->name);
+								strncpy(name, seq->strip->dir, FILE_MAXDIR-1);
+								strncat(name, seq->strip->stripdata->name, FILE_MAXFILE-1);
 								BLI_convertstringcode(name, G.sce, G.scene->r.cfra);
 
 								seq->anim = openanim(name, IB_rect);
@@ -1925,7 +1942,9 @@ void do_build_seqar_cfra(ListBase *seqbase, Sequence ***seqar, int cfra)
 
 							if(se->ibuf==0) se->ok= 0;
 							else {
-								if(se->ibuf->depth==32) converttopremul(se->ibuf);
+								if(seq->flag & SEQ_MAKE_PREMUL) {
+									if(se->ibuf->depth==32) converttopremul(se->ibuf);
+								}
 								seq->strip->orx= se->ibuf->x;
 								seq->strip->ory= se->ibuf->y;
 								if(seq->flag & SEQ_FILTERY) IMB_filtery(se->ibuf);
@@ -1937,26 +1956,30 @@ void do_build_seqar_cfra(ListBase *seqbase, Sequence ***seqar, int cfra)
 					}
 					else if(seq->type==SEQ_SCENE && se->ibuf==0 && seq->scene) {	// scene can be NULL after deletions
 						View3D *vd;
+						int redisplay= (!G.background && !(R.flag & R_RENDERING));
 						
 						oldsce= G.scene;
-						set_scene_bg(seq->scene);
+						if(seq->scene!=G.scene) set_scene_bg(seq->scene);	/* set_scene does full dep updates */
 
 						/* prevent eternal loop */
 						doseq= G.scene->r.scemode & R_DOSEQ;
 						G.scene->r.scemode &= ~R_DOSEQ;
 
-						/* store stuffies */
-						oldcfra= CFRA; CFRA= seq->sfra + se->nr;
+						/* store Current FRAme */
+						oldcfra= CFRA;
+
+						CFRA= ( seq->sfra + se->nr );
+
 						waitcursor(1);
 
-						rectot= R.rectot; R.rectot= 0;
+						rectot= R.rectot; R.rectot= NULL;
 						oldx= R.rectx; oldy= R.recty;
 						/* needed because current 3D window cannot define the layers, like in a background render */
 						vd= G.vd;
-						G.vd= 0;
+						G.vd= NULL;
 
 						RE_initrender(NULL);
-						if (!G.background) {
+						if (redisplay) {
 							mainwindow_make_active();
 							if(R.r.mode & R_FIELDS) update_for_newframe_muted();
 							R.flag= 0;
@@ -1970,7 +1993,7 @@ void do_build_seqar_cfra(ListBase *seqbase, Sequence ***seqar, int cfra)
 							se->ibuf->zbuf= (int *)R.rectz;
 							/* make sure ibuf frees it */
 							se->ibuf->mall |= IB_zbuf;
-							R.rectz= 0;
+							R.rectz= NULL;
 						}
 
 						/* and restore */
@@ -1982,7 +2005,7 @@ void do_build_seqar_cfra(ListBase *seqbase, Sequence ***seqar, int cfra)
 						R.rectot= rectot;
 						R.rectx=oldx; R.recty=oldy;
 						G.scene->r.scemode |= doseq;
-						set_scene_bg(oldsce);
+						if(seq->scene!=oldsce) set_scene_bg(oldsce);	/* set_scene does full dep updates */
 
 						/* restore!! */
 						R.rectx= seqrectx;
