@@ -1,5 +1,5 @@
 /**
- * $Id: toets.c,v 1.45 2004/11/10 11:14:11 ton Exp $
+ * $Id: toets.c,v 1.67 2005/12/17 14:27:33 jesterking Exp $
  *
  * ***** BEGIN GPL/BL DUAL LICENSE BLOCK *****
  *
@@ -38,10 +38,6 @@
 #include <string.h>
 #include <math.h>
 
-#ifdef HAVE_CONFIG_H
-#include <config.h>
-#endif
-
 #ifdef WIN32
 #include "BLI_winstuff.h"
 #endif
@@ -64,10 +60,11 @@
 #include "BKE_action.h"
 #include "BKE_anim.h"
 #include "BKE_blender.h"
+#include "BKE_depsgraph.h"
 #include "BKE_displist.h"
 #include "BKE_global.h"
+#include "BKE_image.h"
 #include "BKE_ipo.h"
-#include "BKE_ika.h"
 #include "BKE_key.h"
 #include "BKE_scene.h"
 #include "BKE_utildefines.h"
@@ -77,6 +74,7 @@
 #include "BIF_editsound.h"
 #include "BIF_editmesh.h"
 #include "BIF_interface.h"
+#include "BKE_object.h"
 #include "BIF_poseobject.h"
 #include "BIF_renderwin.h"
 #include "BIF_screen.h"
@@ -95,9 +93,10 @@
 #include "BSE_edit.h"
 #include "BSE_editipo.h"
 #include "BSE_headerbuttons.h"
+#include "BSE_seqaudio.h"
 
 #include "blendef.h"
-#include "render.h"
+#include "render.h"		// darn schrijfplaatje() (ton)
 
 #include "IMB_imbuf.h"
 #include "IMB_imbuf_types.h"
@@ -106,17 +105,22 @@
 
 #include "BIF_poseobject.h"
 
-/* only used in toets.c */
+/* only used in toets.c and initrender.c */
 /* this function doesn't really belong here */
 /* ripped from render module */
 void schrijfplaatje(char *name);
 
 
-void write_imag(char *name)
+static void write_imag(char *name)
 {
 	/* from file select */
 	char str[256];
 
+	/* addImageExtension() checks for if extension was already set */
+	if(G.scene->r.scemode & R_EXTENSION) 
+		if(strlen(name)<FILE_MAXDIR+FILE_MAXFILE-5)
+			addImageExtension(name);
+	
 	strcpy(str, name);
 	BLI_convertstringcode(str, G.sce, G.scene->r.cfra);
 
@@ -143,6 +147,13 @@ void schrijfplaatje(char *name)
 	unsigned int *temprect=0;
 	char str[FILE_MAXDIR+FILE_MAXFILE];
 
+	/* radhdr: temporary call for direct float buffer save for HDR format */
+	if ((R.r.imtype==R_RADHDR) && (R.rectftot))
+	{
+		imb_savehdr_fromfloat(R.rectftot, name, R.rectx, R.recty);
+		return;
+	}
+
 	/* has RGBA been set? If so: use alpha channel for color zero */
 	IMB_alpha_to_col0(FALSE);
 
@@ -172,11 +183,12 @@ void schrijfplaatje(char *name)
 	}
 
 	if(ibuf == 0) {
-		ibuf= IMB_allocImBuf(R.rectx, R.recty, R.r.planes, 0, 0);
+		ibuf= IMB_allocImBuf(R.rectx, R.recty, (char) R.r.planes, 0, 0);
 	}
 
 	if(ibuf) {
 		ibuf->rect= (unsigned int *) R.rectot;
+//		ibuf->rect_float = R.rectftot;
 
 		if(R.r.planes == 8) IMB_cspace(ibuf, rgb_to_bw);
 
@@ -192,12 +204,20 @@ void schrijfplaatje(char *name)
 				else printf("no zbuf\n");
 			}
 		}
+		else if(R.r.imtype==R_RADHDR) {
+			/* radhdr: save hdr from rgba buffer, not really recommended, probably mistake, so warn user */
+			error("Will save, but you might want to enable the floatbuffer to save a real HDRI...");
+			ibuf->ftype= RADHDR;
+		}
 		else if(R.r.imtype==R_PNG) {
 			ibuf->ftype= PNG;
 		}
 		else if(R.r.imtype==R_BMP) {
 			ibuf->ftype= BMP;
 		}
+		else if((G.have_libtiff) && (R.r.imtype==R_TIFF)) {
+			ibuf->ftype= TIF;
+		}
 		else if((R.r.imtype==R_TARGA) || (R.r.imtype==R_PNG)) {
 			ibuf->ftype= TGA;
 		}
@@ -260,7 +280,7 @@ void persptoetsen(unsigned short event)
 			view3d_set_1_to_1_viewborder(G.vd);
 		} else {
 			if (G.vd->persp==2) {
-				G.vd->camzoom= 0.0;
+				G.vd->camzoom= 0;
 			} else {
 				G.vd->dist= 10.0;
 			}
@@ -405,16 +425,14 @@ void persptoetsen(unsigned short event)
 				if(((G.qual & LR_CTRLKEY) && (G.qual & LR_ALTKEY)) || (G.qual & LR_SHIFTKEY)) {
 					void setcameratoview3d(void);	// view.c
 					setcameratoview3d();
+					DAG_object_flush_update(G.scene, G.scene->camera, OB_RECALC_OB);
+					allqueue(REDRAWVIEW3D, 0);
 				}				
 			}
 		}
 		else if(event==PAD9) {
 			countall();
-			do_all_ipos();
-			do_all_keys();
-			do_all_actions();
-			do_all_ikas();
-			test_all_displists();
+			update_for_newframe();
 			
 			reset_slowparents();	/* editobject.c */
 		}
@@ -472,10 +490,16 @@ int untitled(char * name)
 int save_image_filesel_str(char *str)
 {
 	switch(G.scene->r.imtype) {
+	case R_RADHDR:
+		strcpy(str, "Save Radiance HDR"); return 1;
 	case R_PNG:
 		strcpy(str, "Save PNG"); return 1;
 	case R_BMP:
 		strcpy(str, "Save BMP"); return 1;
+	case R_TIFF:
+		if (G.have_libtiff) {
+			strcpy(str, "Save TIFF"); return 1;
+		}
 	case R_TARGA:
 		strcpy(str, "Save Targa"); return 1;
 	case R_RAWTGA:
@@ -525,6 +549,7 @@ int blenderqread(unsigned short event, short val)
 	/* here do the general keys handling (not screen/window/space) */
 	/* return 0: do not pass on to the other queues */
 	extern int textediting;
+	extern void playback_anim();
 	ScrArea *sa;
 	Object *ob;
 	int textspace=0;
@@ -534,7 +559,7 @@ int blenderqread(unsigned short event, short val)
 	
 	if(val==0) return 1;
 	if(event==MOUSEY || event==MOUSEX) return 1;
-	if (G.flags & G_FLAGS_AUTOPLAY) return 1;
+	if (G.flags & G_FILE_AUTOPLAY) return 1;
 
 	if (curarea && curarea->spacetype==SPACE_TEXT) textspace= 1;
 	else if (curarea && curarea->spacetype==SPACE_SCRIPT) textspace= 1;
@@ -660,6 +685,9 @@ int blenderqread(unsigned short event, short val)
 			newspace(curarea, SPACE_TEXT);
 			return 0;
 		}
+		else if (G.qual==LR_CTRLKEY) {
+			playback_anim();
+		}
 		else if(G.qual==0) {
 			BIF_toggle_render_display();
 			return 0;
@@ -674,6 +702,9 @@ int blenderqread(unsigned short event, short val)
 			newspace(curarea, SPACE_NLA);
 			return 0;
 		}
+		else if (G.qual==LR_CTRLKEY) {
+			BIF_do_render(1);
+		}
 		else {
 			/* ctrl/alt + f12 should render too, for some macs have f12 assigned to cd eject */
 			BIF_do_render(0);
@@ -681,11 +712,29 @@ int blenderqread(unsigned short event, short val)
 		return 0;
 		break;
 	
+	case WHEELUPMOUSE:
+		if(G.qual==LR_ALTKEY || G.qual==LR_COMMANDKEY) {
+			if(CFRA>1) {
+				CFRA--;
+				update_for_newframe();
+			}
+			return 0;
+		}
+		break;
+	case WHEELDOWNMOUSE:
+		if(G.qual==LR_ALTKEY || G.qual==LR_COMMANDKEY) {
+			CFRA++;
+			update_for_newframe();
+			return 0;
+		}
+		break;
+		
 	case LEFTARROWKEY:
 	case DOWNARROWKEY:
 		if(textediting==0 && textspace==0) {
 
-#ifdef _WIN32	// FULLSCREEN
+#if 0
+//#ifdef _WIN32	// FULLSCREEN
 			if(event==DOWNARROWKEY){
 				if (G.qual==LR_ALTKEY)
 					mainwindow_toggle_fullscreen(0);
@@ -712,7 +761,8 @@ int blenderqread(unsigned short event, short val)
 	case UPARROWKEY:
 		if(textediting==0 && textspace==0) {
 
-#ifdef _WIN32	// FULLSCREEN
+#if 0
+//#ifdef _WIN32	// FULLSCREEN
 			if(event==UPARROWKEY){ 
 				if(G.qual==LR_ALTKEY)
 					mainwindow_toggle_fullscreen(1);
@@ -734,7 +784,13 @@ int blenderqread(unsigned short event, short val)
 		break;
 
 	case ESCKEY:
-		sound_stop_all_sounds();
+		sound_stop_all_sounds();	// whats this?
+		
+		/* stop playback on ESC always */
+		rem_screenhandler(G.curscreen, SCREEN_HANDLER_ANIM);
+		audiostream_stop();
+		allqueue(REDRAWALL, 0);
+		
 		break;
 	case TABKEY:
 		if(G.qual==0) {
@@ -759,15 +815,16 @@ int blenderqread(unsigned short event, short val)
 			Object *ob= OBACT;
 			if(ob) {
 				if(ob->type==OB_ARMATURE) {
-					if(G.obpose) exit_posemode(1);
+					if(ob->flag & OB_POSEMODE) exit_posemode();
 					else enter_posemode();
 				}
 				else if(ob->type==OB_MESH) {
 					if(ob==G.obedit) EM_selectmode_menu();
+					else set_wpaint();
 				}
 			}
 		}
-		else if(G.qual==LR_SHIFTKEY) {
+		else if(G.qual==LR_SHIFTKEY) {	// ??
 			if(G.obedit)
 				exit_editmode(2); // freedata, and undo
 			if(G.f & G_FACESELECT)
@@ -781,8 +838,6 @@ int blenderqread(unsigned short event, short val)
 			}
 			if(G.f & G_WEIGHTPAINT)
 				set_wpaint();
-			if(G.obpose)
-				exit_posemode(1);
 		}
 		break;
 
@@ -867,7 +922,7 @@ int blenderqread(unsigned short event, short val)
 		break;
 		
 	case SKEY:
-		if(G.obpose==0 && G.obedit==0) {
+		if(G.obedit==NULL) {
 			if(G.qual==LR_CTRLKEY) {
 				strcpy(dir, G.sce);
 				if (untitled(dir)) {
@@ -883,6 +938,8 @@ int blenderqread(unsigned short event, short val)
 	
 	case TKEY:
 		if (G.qual==(LR_SHIFTKEY|LR_ALTKEY|LR_CTRLKEY)) {
+			Object *ob = OBACT;
+			int event = pupmenu(ob?"Time%t|draw|recalc ob|recalc data":"Time%t|draw");
 			int a;
 			double delta, stime;
 
@@ -890,19 +947,27 @@ int blenderqread(unsigned short event, short val)
 			
 			stime= PIL_check_seconds_timer();
 			for(a=0; a<100000; a++) {
-				scrarea_do_windraw(curarea);
+				if (event==1) {
+					scrarea_do_windraw(curarea);
+				} else if (event==2) {
+					ob->recalc |= OB_RECALC_OB;
+					object_handle_update(ob);
+				} else if (event==3) {
+					ob->recalc |= OB_RECALC_DATA;
+					object_handle_update(ob);
+				}
 
 				delta= PIL_check_seconds_timer()-stime;
 				if (delta>5.0) break;
 			}
 			
 			waitcursor(0);
-			notice("FPS: %f (%d iterations)", a/delta, a);
+			notice("%8.6f s/op - %6.2f ops/s - %d iterations", delta/a, a/delta, a);
 			return 0;
 		}
 		else if(G.qual==(LR_ALTKEY|LR_CTRLKEY)) {
 			int a;
-			int event= pupmenu("10 Timer%t|draw|draw+swap|displist|undo");
+			int event= pupmenu("10 Timer%t|draw|draw+swap|undo");
 			if(event>0) {
 				double stime= PIL_check_seconds_timer();
 				char tmpstr[128];
@@ -916,17 +981,13 @@ int blenderqread(unsigned short event, short val)
 					} else if (event==2) {
 						scrarea_do_windraw(curarea);
 						screen_swapbuffers();
-					} else if (event==3) {
-						if (OBACT) {
-							makeDispList(OBACT);
-						}
 					}
-					else if(event==4) {
+					else if(event==3) {
 						BKE_write_undo("10 timer");
 					}
 				}
 			
-				time= (PIL_check_seconds_timer()-stime)*1000;
+				time= (int) ((PIL_check_seconds_timer()-stime)*1000);
 				
 				if(event==1) sprintf(tmpstr, "draw %%t|%d ms", time);
 				if(event==2) sprintf(tmpstr, "d+sw %%t|%d ms", time);
@@ -977,7 +1038,7 @@ int blenderqread(unsigned short event, short val)
 		break;
 		
 	case XKEY:
-		if(textspace==0) {
+		if(textspace==0 && textediting==0) {
 			if(G.qual==LR_CTRLKEY) {
 				if(okee("Erase all")) {
 					if( BIF_read_homefile()==0) error("No file ~/.B.blend");
