@@ -1,5 +1,5 @@
 /**
- * $Id: toolbox.c,v 1.97 2005/06/02 10:12:13 ton Exp $
+ * $Id: toolbox.c,v 1.120 2006/01/11 16:55:31 campbellbarton Exp $
  *
  * ***** BEGIN GPL/BL DUAL LICENSE BLOCK *****
  *
@@ -67,6 +67,7 @@
 #include "BLI_blenlib.h"
 #include "BLI_arithb.h"
 
+#include "BKE_depsgraph.h"
 #include "BKE_plugin_types.h"
 #include "BKE_utildefines.h"
 #include "BKE_mesh.h"
@@ -83,15 +84,17 @@
 #include "BIF_editlattice.h"
 #include "BIF_editsima.h"
 #include "BIF_editoops.h"
+#include "BIF_editview.h"
 #include "BIF_gl.h"
 #include "BIF_graphics.h"
 #include "BIF_imasel.h"
-#include "BIF_mainqueue.h"
 #include "BIF_interface.h"
-#include "BIF_toolbox.h"
+#include "BIF_mainqueue.h"
 #include "BIF_mywindow.h"
+#include "BIF_renderwin.h"
 #include "BIF_screen.h"
 #include "BIF_space.h"
+#include "BIF_toolbox.h"
 #include "BIF_tbcallback.h"
 #include "BIF_transform.h"
 
@@ -1067,9 +1070,12 @@ int okee(char *str, ...)
 {
 	va_list ap;
 	int ret;
+	char titlestr[256];
+	
+	sprintf(titlestr, "OK? %%i%d", ICON_HELP);
 	
 	va_start(ap, str);
-	ret= vconfirm("OK?", str, ap);
+	ret= vconfirm(titlestr, str, ap);
 	va_end(ap);
 	
 	return ret;
@@ -1088,15 +1094,18 @@ void error(char *fmt, ...)
 {
 	va_list ap;
 	char nfmt[256];
-
-	sprintf(nfmt, "ERROR: %s", fmt);
-
+	char titlestr[256];
+	
+	sprintf(titlestr, "Error %%i%d", ICON_ERROR);
+	
+	sprintf(nfmt, "%s", fmt);
+	
 	va_start(ap, fmt);
 	if (G.background || !G.curscreen || (R.flag & R_RENDERING)) {
 		vprintf(nfmt, ap);
 		printf("\n");
 	} else {
-		vconfirm(NULL, nfmt, ap);
+		vconfirm(titlestr, nfmt, ap);
 	}
 	va_end(ap);
 }
@@ -1174,8 +1183,7 @@ short sbutton(char *var, float min, float max, char *str)
 	return 0;
 }
 
-short fbutton(float *var, float min, float max, float a1, float a2, 
-char *str)
+short fbutton(float *var, float min, float max, float a1, float a2, char *str)
 {
 	uiBlock *block;
 	ListBase listb={0, 0};
@@ -1243,16 +1251,16 @@ int movetolayer_buts(unsigned int *lay)
 	
 	uiBlockBeginAlign(block);
 	for(a=0; a<5; a++) 
-		uiDefButI(block, TOGR|BIT|a, 0, "",(short)(x1+a*dx),(short)(y1+dy),(short)dx,(short)dy, lay, 0, 0, 0, 0, "");
+		uiDefButBitI(block, TOGR, 1<<a, 0, "",(short)(x1+a*dx),(short)(y1+dy),(short)dx,(short)dy, lay, 0, 0, 0, 0, "");
 	for(a=0; a<5; a++) 
-		uiDefButI(block, TOGR|BIT|(a+10), 0, "",(short)(x1+a*dx),(short)y1,(short)dx,(short)dy, lay, 0, 0, 0, 0, "");
+		uiDefButBitI(block, TOGR, 1<<(a+10), 0, "",(short)(x1+a*dx),(short)y1,(short)dx,(short)dy, lay, 0, 0, 0, 0, "");
 	x1+= 5;
 	
 	uiBlockBeginAlign(block);
 	for(a=5; a<10; a++) 
-		uiDefButI(block, TOGR|BIT|a, 0, "",(short)(x1+a*dx),(short)(y1+dy),(short)dx,(short)dy, lay, 0, 0, 0, 0, "");
+		uiDefButBitI(block, TOGR, 1<<a, 0, "",(short)(x1+a*dx),(short)(y1+dy),(short)dx,(short)dy, lay, 0, 0, 0, 0, "");
 	for(a=5; a<10; a++) 
-		uiDefButI(block, TOGR|BIT|(a+10), 0, "",(short)(x1+a*dx),(short)y1,(short)dx,(short)dy, lay, 0, 0, 0, 0, "");
+		uiDefButBitI(block, TOGR, 1<<(a+10), 0, "",(short)(x1+a*dx),(short)y1,(short)dx,(short)dy, lay, 0, 0, 0, 0, "");
 	uiBlockEndAlign(block);
 
 	x1-= 5;
@@ -1302,6 +1310,19 @@ int do_clever_numbuts(char *name, int tot, int winevent)
 	int a, sizex, sizey, x1, y2;
 	short mval[2], event;
 	
+	/* Clear all events so tooltips work, this is not ideal and
+	only needed because calls from the menu still have some events
+	left over when do_clever_numbuts is called.
+	Calls from keyshortcuts do not have this problem.*/
+	ScrArea *sa;
+	BWinEvent temp_bevt;
+	for (sa= G.curscreen->areabase.first; sa; sa= sa->next) {
+		while( bwin_qread( sa->win, &temp_bevt ) ) {}
+		while( bwin_qread( sa->headwin, &temp_bevt ) ) {}
+	}
+	/* Done clearing events */
+	
+	
 	if(tot<=0 || tot>MAXNUMBUTS) return 0;
 
 	getmouseco_sc(mval);
@@ -1325,28 +1346,42 @@ int do_clever_numbuts(char *name, int tot, int winevent)
 	uiBlockSetFlag(block, UI_BLOCK_LOOP|UI_BLOCK_REDRAW|UI_BLOCK_RET_1|UI_BLOCK_ENTER_OK);
 	
 	/* WATCH IT: TEX BUTTON EXCEPTION */
-	/* WARNING: ONLY A SINGLE BIT-BUTTON POSSIBLE: WE WORK AT COPIED DATA! 
-*/
-
+	/* WARNING: ONLY A SINGLE BIT-BUTTON POSSIBLE: WE WORK AT COPIED DATA! */
+	
+	BIF_ThemeColor(TH_MENU_TEXT); /* makes text readable on dark theme */
+	
 	uiDefBut(block, LABEL, 0, name,	(short)(x1+15), (short)(y2-35), (short)(sizex-60), 19, 0, 1.0, 0.0, 0, 0, ""); 
-
+	
+	/*
 	if(name[0]=='A' && name[7]=='O') {
 		y2 -= 20;
 		uiDefBut(block, LABEL, 0, "Rotations in degrees!",	(short)(x1+15), (short)(y2-35), (short)(sizex-60), 19, 0, 0.0, 0.0, 0, 0, "");
-	}
+	}*/
 	
+	uiBlockBeginAlign(block);
 	varstr= &numbuts[0];
 	for(a=0; a<tot; a++, varstr++) {
+		
 		if(varstr->type==TEX) {
 			uiDefBut(block, TEX, 0,	varstr->name,(short)(x1+15),(short)(y2-55-20*a),(short)(sizex-60), 19, numbpoin[a], varstr->min, varstr->max, 0, 0, varstr->tip);
 		}
 		else  {
+			
+			if(varstr->type==LABEL) /* dont include the label when rounding the buttons */
+				uiBlockEndAlign(block);
+			
 			uiDefBut(block, varstr->type, 0, varstr->name,(short)(x1+15),(short)(y2-55-20*a), (short)(sizex-60), 19, &(numbdata[a]), varstr->min, varstr->max, 100, 0, varstr->tip);
+			
+			if(varstr->type==LABEL)
+				uiBlockBeginAlign(block);
 		}
+
+		
 	}
+	uiBlockEndAlign(block);
 
 	uiDefBut(block, BUT, 4000, "OK", (short)(x1+sizex-40),(short)(y2-35-20*a), 25, (short)(sizey-50), 0, 0, 0, 0, 0, "OK: Assign Values");
-
+	
 	uiBoundsBlock(block, 5);
 
 	event= uiDoBlocks(&listb, 0);
@@ -1361,12 +1396,13 @@ int do_clever_numbuts(char *name, int tot, int winevent)
 			else if ELEM( (varstr->type & BUTPOIN), FLO, INT ) memcpy(numbpoin[a], numbdata+a, 4);
 			else if((varstr->type & BUTPOIN)==SHO ) *((short *)(numbpoin[a]))= *( (short *)(numbdata+a));
 			
+			/*
 			if( strncmp(varstr->name, "Rot", 3)==0 ) {
 				float *fp;
 				
 				fp= numbpoin[a];
 				fp[0]= M_PI*fp[0]/180.0;
-			}
+			}*/
 		}
 		
 		if(winevent) {
@@ -1405,12 +1441,12 @@ void add_numbut(int nr, int type, char *str, float min, float max, void *poin, c
 	if ELEM( (type & BUTPOIN), FLO, INT ) memcpy(numbdata+nr, poin, 4);
 	if((type & BUTPOIN)==SHO ) *((short *)(numbdata+nr))= *( (short *)poin);
 	
-	if( strncmp(numbuts[nr].name, "Rot", 3)==0 ) {
+	/* if( strncmp(numbuts[nr].name, "Rot", 3)==0 ) {
 		float *fp;
 		
 		fp= (float *)(numbdata+nr);
 		fp[0]= 180.0*fp[0]/M_PI;
-	}
+	} */
 
 }
 
@@ -1626,15 +1662,16 @@ static TBitem tb_object_select_grouped[]= {
 {  -1, "", 			0, do_view3d_select_object_groupedmenu}};
 
 static TBitem tb_object_select[]= {
-{	0, "Border Select|B", 	'b', NULL},
+{	0, "Border Select|B", 	0, NULL},
 {	0, "SEPR",				0, NULL},
-{	0, "Select/Deselect All|A", 	'a', NULL},
+{	0, "Select/Deselect All|A", 	1, NULL},
+{	0, "Inverse",			2, NULL},
 {	0, "Select All by Layer", 	0, 		tb_object_select_layer},
 {	0, "Select All by Type", 	0, 		tb_object_select_type},
 {	0, "SEPR",				0, NULL},
 {	0, "Linked", 	0, 	tb_object_select_linked},
 {	0, "Grouped", 	0, 	tb_object_select_grouped},
-{  -1, "", 			0, tb_do_hotkey}};
+{  -1, "", 			0, do_view3d_select_objectmenu}};
 
 static TBitem tb_mesh_select[]= {
 {	0, "Border Select|B",               0, NULL},
@@ -1644,12 +1681,13 @@ static TBitem tb_mesh_select[]= {
 {	0, "SEPR",                          0, NULL},
 {	0, "Random...",			            5, NULL},
 {	0, "Non-Manifold|Shift Ctrl Alt M", 9, NULL},
+{	0, "Triangles|Shift Ctrl Alt 3",    11, NULL},
+{	0, "Quads|Shift Ctrl Alt 4",        12, NULL},
+{	0, "Non-Triangles/Quads|Shift Ctrl Alt 5", 13, NULL},
 {	0, "SEPR",                          0, NULL},
 {	0, "More|Ctrl NumPad +",            7, NULL},
 {	0, "Less|Ctrl NumPad -",            8, NULL},
 {	0, "SEPR",                          0, NULL},
-{	0, "Vertex Loop...|Alt B",          10, NULL},
-{	0, "Face Loop...|Shift R",          6, NULL},
 {	0, "Linked Vertices|Ctrl L",        4, NULL},
 {  -1, "", 			0, do_view3d_select_meshmenu}};
 
@@ -1663,9 +1701,9 @@ static TBitem tb_curve_select[]= {
 {  -1, "", 				0, do_view3d_select_curvemenu}};
 
 static TBitem tb__select[]= {
-{	0, "Border Select|B", 	'b', NULL},
-{	0, "(De)select All|A", 	'a', NULL},
-{  -1, "", 			0, tb_do_hotkey}};
+{       0, "Border Select|B",   'b', NULL},
+{       0, "(De)select All|A",  'a', NULL},
+{  -1, "",                      0, tb_do_hotkey}};
 
 
 /* *************Edit ********** */
@@ -1709,6 +1747,7 @@ static TBitem tb_curve_edit[]= {
 
 static TBitem tb_mesh_edit_vertex[]= {
 {	0, "Merge...|Alt M", 		5, NULL},
+{	0, "Rip|V",					7, NULL},
 {	0, "Split|Y", 				4, 		NULL},
 {	0, "Separate|P", 			3, 		NULL},
 {	0, "SEPR",					0, NULL},
@@ -1735,18 +1774,24 @@ static TBitem tb_mesh_edit_edge[]= {
 {	0, "SEPR", 								0, NULL},
 {	0, "Crease SubSurf|Shift E",	9, 		NULL},
 {	0, "SEPR", 								0, NULL},
-{	0, "Rotate Edge|Ctrl E",	10, 		NULL},
+{	0, "Rotate Edge CW|Ctrl E",	10, 		NULL},
+{	0, "Rotate Edge CCW|Ctrl E",	11, 		NULL},
+{	0, "SEPR", 								0, NULL},
+{	0, "Slide Edge|Ctrl E",	12, 		NULL},
+{	0, "Delete Edge Loop|X",	13, 		NULL},
 {  -1, "", 			0, do_view3d_edit_mesh_edgesmenu}};
 
 static TBitem tb_mesh_edit_face[]= {
-{	0, "Make Edge/Face|F", 			'f', 		NULL},
-{	0, "Fill|Shift F", 				'F', 		NULL},
-{	0, "Beautify Fill|Alt F", 			TB_ALT|'f', 		NULL},
+{	0, "Make Edge/Face|F", 			5, 		NULL},
+{	0, "Fill|Shift F", 				0, 		NULL},
+{	0, "Beautify Fill|Alt F", 			1, 		NULL},
 {	0, "SEPR",					0, NULL},
-{	0, "Convert to Triangles|Ctrl T", 	TB_CTRL|'t', 		NULL},
-{	0, "Convert to Quads|Alt J", 		TB_ALT|'j', 		NULL},
-{	0, "Flip Triangle Edges|Ctrl F", 	TB_CTRL|'f', 		NULL},
-{  -1, "", 			0, tb_do_hotkey}};
+{	0, "Convert to Triangles|Ctrl T", 	2, 		NULL},
+{	0, "Convert to Quads|Alt J", 		3, 		NULL},
+{	0, "Flip Triangle Edges|Ctrl F", 	4, 		NULL},
+{	0, "Set Smooth|W, Alt 4", 	6, 		NULL},
+{	0, "Set Solid|W, Alt 5", 	7, 		NULL},
+{  -1, "", 			0, do_view3d_edit_mesh_facesmenu}};
 
 
 static TBitem tb_mesh_edit_normal[]= {
@@ -1799,14 +1844,12 @@ static TBitem tb_obdata_hide[]= {
 {  -1, "", 			0, tb_do_hotkey}};
 
 static void tb_do_mesh(void *arg, int event){
-	Mesh *me= get_mesh(OBACT);
 	switch(event) {
 	case 1: common_insertkey(); break;
 	case 2: G.f ^= G_DRAWEDGES; break;
 	case 3: G.f ^= G_DRAWFACES; break;
 	case 4: G.f ^= G_DRAWNORMALS; break;
-	case 5: me->flag ^= ME_SUBSURF; makeDispList(OBACT); break;
-	case 6: me->flag ^= ME_OPT_EDGES; makeDispList(OBACT); break;
+	case 5: flip_subdivison(OBACT, -1); break;
 	}
 	addqueue(curarea->win, REDRAW, 1);
 }
@@ -1819,7 +1862,6 @@ static TBitem tb_mesh[]= {
 {	0, "Show/Hide Normals", 		4, 		NULL},
 {	0, "SEPR", 						0, 	NULL},
 {	0, "Subdivision Surface", 		5, 		NULL},
-{	0, "SubSurf Optimal", 			6, 		NULL},
 {	0, "SEPR", 						0, NULL},
 {	0, "Show/Hide Vertices", 	0, 		tb_obdata_hide},
 {  -1, "", 			0, tb_do_mesh}};
@@ -1880,7 +1922,7 @@ static TBitem tb_view_dt[]= {
 {	ICON_BBOX, "Bounding Box", 	1, NULL},
 {	ICON_WIRE, "Wireframe|Z", 	2, NULL},
 {	ICON_SOLID, "Solid|Z", 		3, NULL},
-{	ICON_SMOOTH, "Shaded|Ctrl Z", 		4, NULL},
+{	ICON_SMOOTH, "Shaded|Shift Z", 		4, NULL},
 {	ICON_POTATO, "Textured|Alt Z", 	5, NULL},
 {  -1, "", 			0, tb_do_view_dt}};
 
@@ -2024,6 +2066,15 @@ static void tb_do_transform(void *arg, int event)
 			if(G.scene->proportional) G.scene->proportional= 0;
 			else G.scene->proportional= 1;
 			break;
+		case 10:
+			docentre(0);
+			break;
+		case 11:
+			docentre_new();
+			break;
+		case 12:
+			docentre_cursor();
+			break;
 	}
 	allqueue(REDRAWVIEW3D, 0);
 }
@@ -2042,9 +2093,14 @@ static TBitem tb_transform[]= {
 {	0, "Scale|S", 		2, NULL},
 {	0, "Scale on Axis", 	0, tb_transform_scaleaxis},
 {	0, "SEPR", 					0, NULL},
-{	0, "Mirror", 	0, tb_transform_object_mirror},
+{	0, "ObData to Center",		10, NULL},
+{	0, "Center New",			11, NULL},
+{	0, "Center Cursor",			12, NULL},
 {	0, "SEPR", 					0, NULL},
 {	ICON_MENU_PANEL, "Properties|N", 3, NULL},
+{	0, "SEPR", 					0, NULL},
+{	0, "Mirror", 	0, tb_transform_object_mirror},
+{	0, "SEPR", 					0, NULL},
 {	0, "Snap", 		0, tb_transform_snap},
 {	0, "SEPR", 					0, NULL},
 {	0, "Clear/Apply", 	0, tb_transform_clearapply},
@@ -2072,12 +2128,16 @@ static TBitem tb_transform_editmode1[]= {
 {	0, "Scale|S", 		2, NULL},
 {	0, "Scale on Axis", 	0, tb_transform_scaleaxis},
 {	0, "SEPR", 					0, NULL},
-{	0, "Mirror", 	0, tb_transform_edit_mirror},
 {	0, "Shrink/Fatten|Alt S", 5, NULL},
 {	0, "Shear|Ctrl S", 6, NULL},
 {	0, "Warp|Shift W", 	7, NULL},
 {	0, "SEPR", 					0, NULL},
+{	0, "ObData to Center",		10, NULL},
+{	0, "SEPR", 					0, NULL},
 {	ICON_MENU_PANEL, "Properties|N", 3, NULL},
+{	0, "SEPR", 					0, NULL},
+{	0, "Mirror", 	0, tb_transform_edit_mirror},
+{	0, "SEPR", 					0, NULL},
 {	0, "Snap", 		0, tb_transform_snap},
 {	0, "SEPR", 					0, NULL},
 {	0, "Proportional Edit|O", 	8, 		NULL},
@@ -2157,7 +2217,7 @@ static TBitem addmenu_YF_lamp[]= {
 
 
 static TBitem addmenu_armature[]= {
-{	0, "Bones", 	8, NULL},
+{	0, "Bone", 	8, NULL},
 {  -1, "", 			0, do_info_addmenu}};
 
 static TBitem tb_add[]= {
@@ -2195,6 +2255,36 @@ static TBitem tb_empty[]= {
 {  -1, "", 		0, NULL}};
 
 
+/* *************RENDER ********** */
+
+static void tb_do_render(void *arg, int event){
+	switch(event)
+	{
+		case 1: /* set render border */
+			set_render_border();
+			break;
+		case 2: /* render */
+			BIF_do_render(0);
+			break;
+		case 3: /* render anim */
+			BIF_do_render(1);
+			break;
+		case 4: /* render anim */
+			if(G.scene->r.scemode & R_PASSEPARTOUT) G.scene->r.scemode &= ~R_PASSEPARTOUT;
+			else G.scene->r.scemode |= R_PASSEPARTOUT;
+			allqueue(REDRAWVIEW3D, 0);
+			break;
+	}
+}
+
+static TBitem tb_render[]= {
+	{       0, "Passepartout",                      4, NULL},
+	{       0, "Set Border",                        1, NULL},
+	{       0, "SEPR",              0, NULL},
+	{       0, "Render|F12",                        2, NULL},
+	{       0, "Anim",                                      3, NULL},
+	{  -1, "",                      0, tb_do_render}};
+
 
 static uiBlock *tb_makemenu(void *arg)
 {
@@ -2235,12 +2325,15 @@ static uiBlock *tb_makemenu(void *arg)
 	uiTextBoundsBlock(block, 60);
 	
 	/* direction is also set in the function that calls this */
-	uiBlockSetDirection(block, UI_RIGHT|UI_CENTRE);
+	if(U.uiflag & USER_PLAINMENUS)
+		uiBlockSetDirection(block, UI_RIGHT);
+	else
+		uiBlockSetDirection(block, UI_RIGHT|UI_CENTRE);
 
 	return block;
 }
 
-static int tb_mainx= 0, tb_mainy= -5;
+static int tb_mainx= 1234, tb_mainy= 0;
 static void store_main(void *arg1, void *arg2)
 {
 	tb_mainx= (int)arg1;
@@ -2253,9 +2346,21 @@ void toolbox_n(void)
 	uiBut *but;
 	TBitem *menu1=NULL, *menu2=NULL, *menu3=NULL; 
 	TBitem *menu4=NULL, *menu5=NULL, *menu6=NULL;
-	int dx;
+	TBitem *menu7=NULL;
+	int dx=0;
 	short event, mval[2], tot=0;
-	char *str1=NULL, *str2=NULL, *str3=NULL, *str4=NULL, *str5=NULL, *str6=NULL;
+	char *str1=NULL, *str2=NULL, *str3=NULL, *str4=NULL, *str5=NULL, *str6=NULL, *str7=NULL;
+	
+	/* temporal too... when this flag is (was) saved, it should initialize OK */
+	if(tb_mainx==1234) {
+		if(U.uiflag & USER_PLAINMENUS) {
+			tb_mainx= -32;
+			tb_mainy= -5;
+		} else {
+			tb_mainx= 0;
+			tb_mainy= -5;
+		}
+	}
 	
 	mywinset(G.curscreen->mainwin); // we go to screenspace
 	
@@ -2263,70 +2368,135 @@ void toolbox_n(void)
 	uiBlockSetFlag(block, UI_BLOCK_LOOP|UI_BLOCK_REDRAW|UI_BLOCK_RET_1);
 	uiBlockSetCol(block, TH_MENU_ITEM);
 	
-	dx= 64;
-	
 	/* select context for main items */
 	if(curarea->spacetype==SPACE_VIEW3D) {
-		/* standard menu */
-		menu1= tb_object; str1= "Object";
-		if (G.scene->r.renderer==R_YAFRAY) {
-			menu2= tb_add_YF; str2= "Add";
+
+		if(U.uiflag & USER_PLAINMENUS) {
+			/* column layout menu */
+			if (G.scene->r.renderer==R_YAFRAY) {
+				menu1= tb_add_YF; str1= "Add";
+			} else {
+				 menu1= tb_add; str1= "Add";
+			}
+			menu2= tb_object_edit; str2= "Edit";
+			menu3= tb_object_select; str3= "Select";
+			menu4= tb_transform; str4= "Transform";
+			menu5= tb_object; str5= "Object";
+			menu6= tb_view; str6= "View";
+			menu7= tb_render; str7= "Render";
+
+			dx= 96;
+			tot= 7;
 		} else {
-			menu2= tb_add; str2= "Add";
+			/* 3x2 layout menu */
+			menu1= tb_object; str1= "Object";
+			if (G.scene->r.renderer==R_YAFRAY) {
+				menu2= tb_add_YF; str2= "Add";
+			} else {
+				menu2= tb_add; str2= "Add";
+			}
+			menu3= tb_object_select; str3= "Select";
+			menu4= tb_object_edit; str4= "Edit";
+			menu5= tb_transform; str5= "Transform";
+			menu6= tb_view; str6= "View";
+
+			dx= 64;
+			tot= 6;
 		}
-		menu3= tb_object_select; str3= "Select";
-		menu4= tb_object_edit; str4= "Edit";
-		menu5= tb_transform; str5= "Transform";
-		menu6= tb_view; str6= "View";
 		
 		if(G.obedit) {
-			if(G.obedit->type==OB_MESH) {
-				menu1= tb_mesh; str1= "Mesh";
-				menu2= addmenu_mesh; 
-				menu3= tb_mesh_select;
-				menu4= tb_mesh_edit; 
-				menu5= tb_transform_editmode1;
+			if(U.uiflag & USER_PLAINMENUS) {
+				switch(G.obedit->type){
+				case OB_MESH:
+					menu1= addmenu_mesh;
+					menu2= tb_mesh_edit;
+					menu3= tb_mesh_select;
+					menu4= tb_transform_editmode1;
+					menu5= tb_mesh; str5= "Mesh";
+				break;
+				case OB_CURVE:
+					menu1= addmenu_curve;
+					menu2= tb_curve_edit;
+					menu3= tb_curve_select;
+					menu4= tb_transform_editmode1;
+					menu5= tb_curve; str5= "Curve";
+				break;
+				case OB_SURF:
+					menu1= addmenu_surf;
+					menu2= tb_curve_edit;
+					menu3= tb_curve_select;
+					menu4= tb_transform_editmode1;
+					menu5= tb_curve; str5= "Surface";
+				break;
+				case OB_MBALL:
+					menu1= addmenu_meta;
+					menu2= tb_edit;
+					menu3= tb__select;
+					menu4= tb_transform_editmode2;
+					menu5= tb_obdata; str5= "Meta";
+				break;
+				case OB_ARMATURE:
+					menu1= addmenu_armature;
+					menu2= tb_edit;
+					menu3= tb__select;
+					menu4= tb_transform_editmode2;
+					menu5= tb_obdata;str5= "Armature";
+				break;
+				case OB_LATTICE:
+					menu1= tb_empty;
+					menu2= tb_edit;
+					menu3= tb__select;
+					menu4= tb_transform_editmode1;
+					menu5= tb_empty;str5= "Lattice";
+				break;
+				}
+			} else {
+				if(G.obedit->type==OB_MESH) {
+					menu1= tb_mesh; str1= "Mesh";
+					menu2= addmenu_mesh; 
+					menu3= tb_mesh_select;
+					menu4= tb_mesh_edit; 
+					menu5= tb_transform_editmode1;
+				}
+				else if(G.obedit->type==OB_CURVE) {
+					menu1= tb_curve; str1= "Curve";
+					menu2= addmenu_curve;
+					menu3= tb_curve_select;
+					menu4= tb_curve_edit;
+					menu5= tb_transform_editmode1;
+				}
+				else if(G.obedit->type==OB_SURF) {
+					menu1= tb_curve; str1= "Surface";
+					menu2= addmenu_surf; 
+					menu3= tb_curve_select;
+					menu4= tb_curve_edit;
+					menu5= tb_transform_editmode1;
+				}
+				else if(G.obedit->type==OB_MBALL) {
+					menu1= tb_obdata; str1= "Meta";
+					menu2= addmenu_meta;
+					menu3= tb__select;
+					menu4= tb_edit;
+					menu5= tb_transform_editmode2;
+				}
+				else if(G.obedit->type==OB_ARMATURE) {
+					menu1= tb_obdata;str1= "Armature";
+					menu2= addmenu_armature;
+					menu3= tb__select;
+					menu4= tb_edit;
+					menu5= tb_transform_editmode2;
+				}
+				else if(G.obedit->type==OB_LATTICE) {
+					menu1= tb_empty;str1= "Lattice";
+					menu2= tb_empty;
+					menu3= tb__select;
+					menu4= tb_edit;
+					menu5= tb_transform_editmode1;
+				}
 			}
-			else if(G.obedit->type==OB_CURVE) {
-				menu1= tb_curve; str1= "Curve";
-				menu2= addmenu_curve;
-				menu3= tb_curve_select;
-				menu4= tb_curve_edit;
-				menu5= tb_transform_editmode1;
-			}
-			else if(G.obedit->type==OB_SURF) {
-				menu1= tb_curve; str1= "Surface";
-				menu2= addmenu_surf; 
-				menu3= tb_curve_select;
-				menu4= tb_curve_edit;
-				menu5= tb_transform_editmode1;
-			}
-			else if(G.obedit->type==OB_MBALL) {
-				menu1= tb_obdata; str1= "Meta";
-				menu2= addmenu_meta;
-				menu3= tb__select;
-				menu4= tb_edit;
-				menu5= tb_transform_editmode2;
-			}
-			else if(G.obedit->type==OB_ARMATURE) {
-				menu1= tb_obdata;str1= "Armature";
-				menu2= addmenu_armature;
-				menu3= tb__select;
-				menu4= tb_edit;
-				menu5= tb_transform_editmode2;
-			}
-			else if(G.obedit->type==OB_LATTICE) {
-				menu1= tb_empty;str1= "Lattice";
-				menu2= tb_empty;
-				menu3= tb__select;
-				menu4= tb_edit;
-				menu5= tb_transform_editmode1;
-			}
-			
 		}
 		else {
 		}
-		tot= 6;
 	}
 	
 	getmouseco_sc(mval);
@@ -2366,6 +2536,43 @@ void toolbox_n(void)
 		but=uiDefBlockBut(block, tb_makemenu, menu6, str6,	mval[0]+(0.5*dx)+tb_mainx,mval[1]+tb_mainy-20, dx, 19, "");
 		uiButSetFlag(but, UI_MAKE_DOWN|UI_MAKE_LEFT);
 		uiButSetFunc(but, store_main, (void *)-dx, (void *)5);
+	} else if (tot==7) {
+                /* check if it fits, dubious */
+		if(mval[0]-0.25*dx+tb_mainx < 6) mval[0]= 6 + 0.25*dx -tb_mainx;
+		else if(mval[0]+0.25*dx+tb_mainx > G.curscreen->sizex-6)
+		mval[0]= G.curscreen->sizex-6-0.25*dx-tb_mainx;
+
+		if(mval[1]-20+tb_mainy < 6) mval[1]= 6+20 -tb_mainy;
+		else if(mval[1]+20+tb_mainy > G.curscreen->sizey-6)
+			mval[1]= G.curscreen->sizey-6-20-tb_mainy;
+
+		but=uiDefIconTextBlockBut(block, tb_makemenu, menu1, ICON_RIGHTARROW_THIN, str1, mval[0]+tb_mainx,mval[1]+tb_mainy, dx, 19, "");
+		uiButSetFlag(but, UI_MAKE_RIGHT);
+		uiButSetFunc(but, store_main, (void *)-32, (void *)-5);
+
+		but=uiDefIconTextBlockBut(block, tb_makemenu, menu2, ICON_RIGHTARROW_THIN, str2, mval[0]+tb_mainx,mval[1]+tb_mainy-20, dx, 19, "");
+		uiButSetFlag(but, UI_MAKE_RIGHT);
+		uiButSetFunc(but, store_main, (void *)-32, (void *)15);
+
+		but=uiDefIconTextBlockBut(block, tb_makemenu, menu3, ICON_RIGHTARROW_THIN, str3, mval[0]+tb_mainx,mval[1]+tb_mainy-40, dx, 19, "");
+		uiButSetFlag(but, UI_MAKE_RIGHT);
+		uiButSetFunc(but, store_main, (void *)-32, (void *)35);
+
+		but=uiDefIconTextBlockBut(block, tb_makemenu, menu4, ICON_RIGHTARROW_THIN, str4, mval[0]+tb_mainx,mval[1]+tb_mainy-60, dx, 19, "");
+		uiButSetFlag(but, UI_MAKE_RIGHT);
+		uiButSetFunc(but, store_main, (void *)-32, (void *)55);
+
+		but=uiDefIconTextBlockBut(block, tb_makemenu, menu5, ICON_RIGHTARROW_THIN, str5, mval[0]+tb_mainx,mval[1]+tb_mainy-80, dx, 19, "");
+		uiButSetFlag(but, UI_MAKE_RIGHT);
+		uiButSetFunc(but, store_main, (void *)-32, (void *)75);
+
+		but=uiDefIconTextBlockBut(block, tb_makemenu, menu6, ICON_RIGHTARROW_THIN, str6, mval[0]+tb_mainx,mval[1]+tb_mainy-100, dx, 19, "");
+		uiButSetFlag(but, UI_MAKE_RIGHT);
+		uiButSetFunc(but, store_main, (void *)-32, (void *)95);
+
+		but=uiDefIconTextBlockBut(block, tb_makemenu, menu7, ICON_RIGHTARROW_THIN, str7, mval[0]+tb_mainx,mval[1]+tb_mainy-120, dx, 19, "");
+		uiButSetFlag(but, UI_MAKE_RIGHT);
+		uiButSetFunc(but, store_main, (void *)-32, (void *)105);
 	}
 	
 	uiBoundsBlock(block, 2);
@@ -2376,7 +2583,17 @@ void toolbox_n(void)
 
 void toolbox_n_add(void)
 {
-	tb_mainx= 0;
-	tb_mainy= -5;
+	reset_toolbox();
 	toolbox_n();
 }
+
+void reset_toolbox(void)
+{
+	if(U.uiflag & USER_PLAINMENUS) {
+		tb_mainx= -32;
+		tb_mainy= -5;
+	} else {
+		tb_mainx= 0;
+		tb_mainy= -5;
+	}
+}
