@@ -1,5 +1,5 @@
 /**
- * $Id: view.c,v 1.36 2004/12/16 14:42:10 ton Exp $
+ * $Id: view.c,v 1.60 2005/11/28 16:59:12 ton Exp $
  *
  * ***** BEGIN GPL/BL DUAL LICENSE BLOCK *****
  *
@@ -41,7 +41,6 @@
 
 #ifdef WIN32
 #include <io.h>
-#include "BLI_winstuff.h"
 #else
 #include <unistd.h>
 #endif   
@@ -51,14 +50,16 @@
 #include "BLI_blenlib.h"
 #include "BLI_arithb.h"
 
+#include "DNA_action_types.h"
+#include "DNA_armature_types.h"
+#include "DNA_camera_types.h"
+#include "DNA_lamp_types.h"
 #include "DNA_object_types.h"
 #include "DNA_screen_types.h"
 #include "DNA_scene_types.h"
 #include "DNA_space_types.h"
-#include "DNA_view3d_types.h"
-#include "DNA_camera_types.h"
-#include "DNA_lamp_types.h"
 #include "DNA_userdef_types.h"
+#include "DNA_view3d_types.h"
 
 #include "BKE_utildefines.h"
 #include "BKE_object.h"
@@ -76,16 +77,21 @@
 #include "BSE_drawview.h"	/* For inner_play_anim_loop */
 
 #include "BDR_drawobject.h"	/* For draw_object */
+#include "BDR_editface.h"	/* For minmax_tface */
 
 #include "mydevice.h"
 #include "blendef.h"
 
 /* Modules used */
-#include "render.h"
+#include "render.h"		/* R. stuff for ogl view render */
 
 #define TRACKBALLSIZE  (1.1)
 #define BL_NEAR_CLIP 0.001
 
+
+/* local prototypes ----------*/
+void setcameratoview3d(void); /* windows.c & toets.c */
+
 void persp_general(int a)
 {
 	/* for all window types, not 3D */
@@ -139,25 +145,19 @@ void initgrabz(float x, float y, float z)
 {
 	if(G.vd==0) return;
 	zfac= G.vd->persmat[0][3]*x+ G.vd->persmat[1][3]*y+ G.vd->persmat[2][3]*z+ G.vd->persmat[3][3];
+	
+	/* if x,y,z is exactly the viewport offset, zfac is 0 and we don't want that */
+	if (zfac==0.0f) zfac = 1.0f;
 }
 
 void window_to_3d(float *vec, short mx, short my)
 {
 	/* always call initzgrab */
 	float dx, dy;
-	float fmx, fmy, winx, winy;
 	
-	/* stupid! */
-	winx= curarea->winx;
-	winy= curarea->winy;
-	fmx= mx;
-	fmy= my;
+	dx= 2.0f*mx*zfac/curarea->winx;
+	dy= 2.0f*my*zfac/curarea->winy;
 	
-	dx= (2.0*fmx)/winx;
-	dx*= zfac;
-	dy= (2.0*fmy)/winy;
-	dy*= zfac;
-
 	vec[0]= (G.vd->persinv[0][0]*dx + G.vd->persinv[1][0]*dy);
 	vec[1]= (G.vd->persinv[0][1]*dx + G.vd->persinv[1][1]*dy);
 	vec[2]= (G.vd->persinv[0][2]*dx + G.vd->persinv[1][2]*dy);
@@ -167,20 +167,49 @@ void project_short(float *vec, short *adr)	/* clips */
 {
 	float fx, fy, vec4[4];
 
-	adr[0]= 3200;
+	adr[0]= IS_CLIPPED;
+	
+	if(G.vd->flag & V3D_CLIPPING) {
+		if(view3d_test_clipping(G.vd, vec))
+			return;
+	}
+
+	VECCOPY(vec4, vec);
+	vec4[3]= 1.0;
+	Mat4MulVec4fl(G.vd->persmat, vec4);
+	
+	if( vec4[3]>BL_NEAR_CLIP ) {	/* 0.001 is the NEAR clipping cutoff for picking */
+		fx= (curarea->winx/2)*(1 + vec4[0]/vec4[3]);
+		
+		if( fx>0 && fx<curarea->winx) {
+		
+			fy= (curarea->winy/2)*(1 + vec4[1]/vec4[3]);
+			
+			if(fy>0.0 && fy< (float)curarea->winy) {
+				adr[0]= floor(fx); 
+				adr[1]= floor(fy);
+			}
+		}
+	}
+}
+
+void project_int(float *vec, int *adr)
+{
+	float fx, fy, vec4[4];
+
+	adr[0]= 2140000000.0f;
 	VECCOPY(vec4, vec);
 	vec4[3]= 1.0;
 	
 	Mat4MulVec4fl(G.vd->persmat, vec4);
 
 	if( vec4[3]>BL_NEAR_CLIP ) {	/* 0.001 is the NEAR clipping cutoff for picking */
-		fx= (curarea->winx/2)+(curarea->winx/2)*vec4[0]/vec4[3];
+		fx= (curarea->winx/2)*(1 + vec4[0]/vec4[3]);
 		
-		if( fx>0 && fx<curarea->winx) {
-		
-			fy= (curarea->winy/2)+(curarea->winy/2)*vec4[1]/vec4[3];
+		if( fx>-2140000000.0f && fx<2140000000.0f) {
+			fy= (curarea->winy/2)*(1 + vec4[1]/vec4[3]);
 			
-			if(fy>0.0 && fy< (float)curarea->winy) {
+			if(fy>-2140000000.0f && fy<2140000000.0f) {
 				adr[0]= floor(fx); 
 				adr[1]= floor(fy);
 			}
@@ -192,18 +221,18 @@ void project_short_noclip(float *vec, short *adr)
 {
 	float fx, fy, vec4[4];
 
-	adr[0]= 3200;
+	adr[0]= IS_CLIPPED;
 	VECCOPY(vec4, vec);
 	vec4[3]= 1.0;
 	
 	Mat4MulVec4fl(G.vd->persmat, vec4);
 
 	if( vec4[3]>BL_NEAR_CLIP ) {	/* 0.001 is the NEAR clipping cutoff for picking */
-		fx= (curarea->winx/2)+(curarea->winx/2)*vec4[0]/vec4[3];
+		fx= (curarea->winx/2)*(1 + vec4[0]/vec4[3]);
 		
 		if( fx>-32700 && fx<32700) {
 		
-			fy= (curarea->winy/2)+(curarea->winy/2)*vec4[1]/vec4[3];
+			fy= (curarea->winy/2)*(1 + vec4[1]/vec4[3]);
 			
 			if(fy>-32700.0 && fy<32700.0) {
 				adr[0]= floor(fx); 
@@ -217,7 +246,7 @@ void project_float(float *vec, float *adr)
 {
 	float vec4[4];
 	
-	adr[0]= 3200.0;
+	adr[0]= IS_CLIPPED;
 	VECCOPY(vec4, vec);
 	vec4[3]= 1.0;
 	
@@ -229,6 +258,101 @@ void project_float(float *vec, float *adr)
 	}
 }
 
+void view3d_get_object_project_mat(ScrArea *area, Object *ob, float pmat[4][4], float vmat[4][4])
+{
+	if (area->spacetype!=SPACE_VIEW3D || !area->spacedata.first) {
+		Mat4One(pmat);
+		Mat4One(vmat);
+	} else {
+		View3D *vd = area->spacedata.first;
+
+		Mat4MulMat4(vmat, ob->obmat, vd->viewmat);
+		Mat4MulMat4(pmat, vmat, vd->winmat1);
+		Mat4CpyMat4(vmat, ob->obmat);
+	}
+}
+
+/* projectmat brings it to window coords, wmat to rotated world space */
+void view3d_project_short_clip(ScrArea *area, float *vec, short *adr, float projmat[4][4], float wmat[4][4])
+{
+	View3D *v3d= area->spacedata.first;
+	float fx, fy, vec4[4];
+
+	adr[0]= IS_CLIPPED;
+	
+	/* clipplanes in eye space */
+	if(v3d->flag & V3D_CLIPPING) {
+		VECCOPY(vec4, vec);
+		Mat4MulVecfl(wmat, vec4);
+		if(view3d_test_clipping(v3d, vec4))
+			return;
+	}
+	
+	VECCOPY(vec4, vec);
+	vec4[3]= 1.0;
+	
+	Mat4MulVec4fl(projmat, vec4);
+	
+	/* clipplanes in window space */
+	if( vec4[3]>BL_NEAR_CLIP ) {	/* 0.001 is the NEAR clipping cutoff for picking */
+		fx= (area->winx/2)*(1 + vec4[0]/vec4[3]);
+		
+		if( fx>0 && fx<area->winx) {
+		
+			fy= (area->winy/2)*(1 + vec4[1]/vec4[3]);
+			
+			if(fy>0.0 && fy< (float)area->winy) {
+				adr[0]= floor(fx); 
+				adr[1]= floor(fy);
+			}
+		}
+	}
+}
+
+void view3d_project_short_noclip(ScrArea *area, float *vec, short *adr, float mat[4][4])
+{
+	float fx, fy, vec4[4];
+
+	adr[0]= IS_CLIPPED;
+	
+	VECCOPY(vec4, vec);
+	vec4[3]= 1.0;
+	
+	Mat4MulVec4fl(mat, vec4);
+
+	if( vec4[3]>BL_NEAR_CLIP ) {	/* 0.001 is the NEAR clipping cutoff for picking */
+		fx= (area->winx/2)*(1 + vec4[0]/vec4[3]);
+		
+		if( fx>-32700 && fx<32700) {
+		
+			fy= (area->winy/2)*(1 + vec4[1]/vec4[3]);
+			
+			if(fy>-32700.0 && fy<32700.0) {
+				adr[0]= floor(fx); 
+				adr[1]= floor(fy);
+			}
+		}
+	}
+}
+
+void view3d_project_float(ScrArea *area, float *vec, float *adr, float mat[4][4])
+{
+	float vec4[4];
+	
+	adr[0]= IS_CLIPPED;
+	VECCOPY(vec4, vec);
+	vec4[3]= 1.0;
+	
+	Mat4MulVec4fl(mat, vec4);
+
+	if( vec4[3]>FLT_EPSILON ) {
+		adr[0] = (area->winx/2.0)+(area->winx/2.0)*vec4[0]/vec4[3];	
+		adr[1] = (area->winy/2.0)+(area->winy/2.0)*vec4[1]/vec4[3];
+	} else {
+		adr[0] = adr[1] = 0.0;
+	}
+}
+
 int boundbox_clip(float obmat[][4], BoundBox *bb)
 {
 	/* return 1: draw */
@@ -404,10 +528,13 @@ void calctrackballvec(rcti *area, short *mval, float *vec)
 
 void viewmove(int mode)
 {
+	Object *ob = OBACT;
 	float firstvec[3], newvec[3], dvec[3];
-	float oldquat[4], q1[4], si, phi, dist0;
+	float reverse, oldquat[4], q1[4], si, phi, dist0;
+	float ofs[3], obofs[3];
 	int firsttime=1;
 	short mvalball[2], mval[2], mvalo[2];
+	short use_sel = 0;
 	
 	/* sometimes this routine is called from headerbuttons */
 	areawinset(curarea->win);
@@ -425,10 +552,28 @@ void viewmove(int mode)
 
 	/* cumultime(0); */
 
+	if (G.obedit==NULL && ob && !(ob->flag & OB_POSEMODE) && U.uiflag & USER_ORBIT_SELECTION) {
+		use_sel = 1;
+		VECCOPY(ofs, G.vd->ofs);
+		if (ob) {
+			obofs[0] = -ob->obmat[3][0];
+			obofs[1] = -ob->obmat[3][1];
+			obofs[2] = -ob->obmat[3][2];
+		}
+		else {
+			VECCOPY(obofs, ofs);
+		}
+	}
+
+	reverse= 1.0f;
+	if (G.vd->persmat[2][1] < 0.0f)
+		reverse= -1.0f;
+
 	while(TRUE) {
 		getmouseco_sc(mval);
 		
-		if(mval[0]!=mvalo[0] || mval[1]!=mvalo[1] || (G.f & G_PLAYANIM)) {
+		// if playanim = alt+A, screenhandlers are for animated UI, python, etc
+		if(mval[0]!=mvalo[0] || mval[1]!=mvalo[1] || (G.f & G_PLAYANIM) || do_screenhandlers(G.curscreen)) {
 			
 			if(firsttime) {
 				
@@ -448,7 +593,6 @@ void viewmove(int mode)
 
 
 			if(mode==0) {	/* view rotate */
-
 				if (U.uiflag & USER_AUTOPERSP) G.vd->persp= 1;
 
 				if (U.flag & USER_TRACKBALL) mvalball[0]= mval[0];
@@ -482,35 +626,69 @@ void viewmove(int mode)
 					q1[0]= cos(phi);
 					q1[1]*= si;
 					q1[2]*= si;
-					q1[3]*= si;						
+					q1[3]*= si;	
 					QuatMul(G.vd->viewquat, q1, oldquat);
+
+					if (use_sel) {
+						/* compute the post multiplication quat, to rotate the offset correctly */
+						QUATCOPY(q1, oldquat);
+						QuatConj(q1);
+						QuatMul(q1, q1, G.vd->viewquat);
+
+						QuatConj(q1); /* conj == inv for unit quat */
+						VECCOPY(G.vd->ofs, ofs);
+						VecSubf(G.vd->ofs, G.vd->ofs, obofs);
+						QuatMulVecf(q1, G.vd->ofs);
+						VecAddf(G.vd->ofs, G.vd->ofs, obofs);
+					}
 				} else {
-					/* is there an acceptable solution? (180 degrees limitor) */
-					if(si<1.0) {
-						Crossf(q1+1, firstvec, newvec);
-	
-						Normalise(q1+1);
-			
-						phi= asin(si);
-		
-						si= sin(phi);
-						q1[0]= cos(phi);
-						q1[1]*= si;
-						q1[2]*= si;
-						q1[3]*= si;
-						
-						QuatMul(G.vd->viewquat, q1, oldquat);
-	
-						/* rotate around z-axis (mouse x moves)  */
-						
-						phi= 2*(mval[0]-mvalball[0]);
-						phi/= (float)curarea->winx;
-						si= sin(phi);
-						q1[0]= cos(phi);
-						q1[1]= q1[2]= 0.0;
-						q1[3]= si;
-					
-						QuatMul(G.vd->viewquat, G.vd->viewquat, q1);
+					/* New turntable view code by John Aughey */
+
+					float m[3][3];
+					float m_inv[3][3];
+					float xvec[3] = {1,0,0};
+					/* Sensitivity will control how fast the viewport rotates.  0.0035 was
+					   obtained experimentally by looking at viewport rotation sensitivities
+					   on other modeling programs. */
+					/* Perhaps this should be a configurable user parameter. */
+					const float sensitivity = 0.0035;
+
+					/* Get the 3x3 matrix and its inverse from the quaternion */
+					QuatToMat3(G.vd->viewquat, m);
+					Mat3Inv(m_inv,m);
+
+					/* Determine the direction of the x vector (for rotating up and down) */
+					/* This can likely be compuated directly from the quaternion. */
+					Mat3MulVecfl(m_inv,xvec);
+
+					/* Perform the up/down rotation */
+					phi = sensitivity * -(mval[1] - mvalo[1]);
+					si = sin(phi);
+					q1[0] = cos(phi);
+					q1[1] = si * xvec[0];
+					q1[2] = si * xvec[1];
+					q1[3] = si * xvec[2];
+					QuatMul(G.vd->viewquat, G.vd->viewquat, q1);
+
+					if (use_sel) {
+						QuatConj(q1); /* conj == inv for unit quat */
+						VecSubf(G.vd->ofs, G.vd->ofs, obofs);
+						QuatMulVecf(q1, G.vd->ofs);
+						VecAddf(G.vd->ofs, G.vd->ofs, obofs);
+					}
+
+					/* Perform the orbital rotation */
+					phi = sensitivity * reverse * (mval[0] - mvalo[0]);
+					q1[0] = cos(phi);
+					q1[1] = q1[2] = 0.0;
+					q1[3] = sin(phi);
+					QuatMul(G.vd->viewquat, G.vd->viewquat, q1);
+
+					if (use_sel) {
+						QuatConj(q1);
+						VecSubf(G.vd->ofs, G.vd->ofs, obofs);
+						QuatMulVecf(q1, G.vd->ofs);
+						VecAddf(G.vd->ofs, G.vd->ofs, obofs);
 					}
 				}
 			}
@@ -573,6 +751,11 @@ void viewmove(int mode)
 			screen_swapbuffers();
 		}
 		else {
+			short val;
+			unsigned short event;
+			/* we need to empty the queue... when you do this very long it overflows */
+			while(qtest()) event= extern_qread(&val);
+			
 			BIF_wait_for_statechange();
 		}
 		
@@ -587,8 +770,8 @@ short v3d_windowmode=0;
 void setwinmatrixview3d(rctf *rect)		/* rect: for picking */
 {
 	Camera *cam=0;
-	float d, near, far, winx = 0.0, winy = 0.0;
-	float lens, dfac, fac, x1, y1, x2, y2;
+	float near, far, winx = 0.0, winy = 0.0;
+	float lens, fac, x1, y1, x2, y2;
 	short orth;
 	
 	lens= G.vd->lens;	
@@ -617,12 +800,6 @@ void setwinmatrixview3d(rctf *rect)		/* rect: for picking */
 				lens= cam->lens;
 				near= cam->clipsta;
 				far= cam->clipend;
-				
-				if(cam->type==CAM_ORTHO) {
-					lens*= 100.0;
-					near= (near+1.0)*100.0;	/* otherwise zbuffer troubles. a Patch! */
-					far*= 100.0;
-				}
 			}
 		}
 	}
@@ -636,21 +813,17 @@ void setwinmatrixview3d(rctf *rect)		/* rect: for picking */
 		winy= curarea->winy;
 	}
 	
-	if(winx>winy) d= 0.015625*winx*lens;
-	else d= 0.015625*winy*lens;
-	
-	dfac= near/d;
-
 	if(G.vd->persp==0) {
 		if(winx>winy) x1= -G.vd->dist;
 		else x1= -winx*G.vd->dist/winy;
-		
 		x2= -x1;
 
 		if(winx>winy) y1= -winy*G.vd->dist/winx;
 		else y1= -G.vd->dist;
-		
 		y2= -y1;
+		
+		far*= 0.5;	// otherwise too extreme low zbuffer quality
+		near= -far;
 		orth= 1;
 	}
 	else {
@@ -660,12 +833,32 @@ void setwinmatrixview3d(rctf *rect)		/* rect: for picking */
 		}
 		else fac= 2.0;
 		
-		x1= -dfac*(winx/fac);
-		x2= -x1;
-		y1= -dfac*(winy/fac);
-		y2= -y1;
-
-		orth= 0;
+		/* viewplane size depends... */
+		if(cam && cam->type==CAM_ORTHO) {
+			/* ortho_scale == 1 means exact 1 to 1 mapping */
+			float dfac= 2.0*cam->ortho_scale/fac;
+			
+			if(winx>winy) x1= -dfac;
+			else x1= -winx*dfac/winy;
+			x2= -x1;
+			
+			if(winx>winy) y1= -winy*dfac/winx;
+			else y1= -dfac;
+			y2= -y1;
+			orth= 1;
+		}
+		else {
+			float dfac;
+			
+			if(winx>winy) dfac= 64.0/(fac*winx*lens);
+			else dfac= 64.0/(fac*winy*lens);
+			
+			x1= - near*winx*dfac;
+			x2= -x1;
+			y1= - near*winy*dfac;
+			y2= -y1;
+			orth= 0;
+		}
 	}
 
 	if(rect) {		/* picking */
@@ -684,18 +877,12 @@ void setwinmatrixview3d(rctf *rect)		/* rect: for picking */
 	}
 	else {
 		if(v3d_windowmode) {
-			if(orth) i_ortho(x1, x2, y1, y2, -far, far, R.winmat);
-			else {
-				if(cam && cam->type==CAM_ORTHO) i_window(x1, x2, y1, y2, near, far, R.winmat);
-				else i_window(x1, x2, y1, y2, near, far, R.winmat);
-			}
+			if(orth) i_ortho(x1, x2, y1, y2, near, far, R.winmat);
+			else i_window(x1, x2, y1, y2, near, far, R.winmat);
 		}
 		else {
-			if(orth) myortho(x1, x2, y1, y2, -far, far);
-			else {
-				if(cam && cam->type==CAM_ORTHO) mywindow(x1, x2, y1, y2, near, far);
-				else mywindow(x1, x2, y1, y2, near, far);
-			}
+			if(orth) myortho(x1, x2, y1, y2, near, far);
+			else mywindow(x1, x2, y1, y2, near, far);
 		}
 	}
 
@@ -734,7 +921,7 @@ void setviewmatrixview3d()
 			
 			if(G.vd->camera->type==OB_CAMERA) {
 				cam= G.vd->camera->data;
-				if(cam->type==CAM_ORTHO) G.vd->viewmat[3][2]*= 100.0;
+				//if(cam->type==CAM_ORTHO) G.vd->viewmat[3][2]*= 100.0;
 			}
 		}
 		else {
@@ -750,7 +937,7 @@ void setviewmatrixview3d()
 	}
 }
 
-void setcameratoview3d()
+void setcameratoview3d(void)
 {
 	Object *ob;
 	float dvec[3];
@@ -771,10 +958,9 @@ void setcameratoview3d()
 }
 
 /* IGLuint-> GLuint*/
-short selectprojektie(unsigned int *buffer, short x1, short y1, short x2, short y2)
+short  view3d_opengl_select(unsigned int *buffer, unsigned int bufsize, short x1, short y1, short x2, short y2)
 {
 	rctf rect;
-	Base *base;
 	short mval[2], code, hits;
 
 	G.f |= G_PICKSEL;
@@ -798,34 +984,40 @@ short selectprojektie(unsigned int *buffer, short x1, short y1, short x2, short
 	Mat4MulMat4(G.vd->persmat, G.vd->viewmat, curarea->winmat);
 	
 	if(G.vd->drawtype > OB_WIRE) {
-		G.zbuf= TRUE;
+		G.vd->zbuf= TRUE;
 		glEnable(GL_DEPTH_TEST);
 	}
 
-	glSelectBuffer( MAXPICKBUF, (GLuint *)buffer);
+	if(G.vd->flag & V3D_CLIPPING)
+		view3d_set_clipping(G.vd);
+	
+	glSelectBuffer( bufsize, (GLuint *)buffer);
 	glRenderMode(GL_SELECT);
 	glInitNames();	/* these two calls whatfor? It doesnt work otherwise */
 	glPushName(-1);
 	code= 1;
 	
 	if(G.obedit && G.obedit->type==OB_MBALL) {
-		draw_object(BASACT);
+		draw_object(BASACT, DRAW_PICKING|DRAW_CONSTCOLOR);
 	}
-	else if ((G.obedit && G.obedit->type==OB_ARMATURE)||(G.obpose && G.obpose->type==OB_ARMATURE)) {
-		draw_object(BASACT);
+	else if ((G.obedit && G.obedit->type==OB_ARMATURE)) {
+		draw_object(BASACT, DRAW_PICKING|DRAW_CONSTCOLOR);
 	}
 	else {
-		base= G.scene->base.first;
-		while(base) {
+		Base *base;
+		
+		G.vd->xray= TRUE;	// otherwise it postpones drawing
+		for(base= G.scene->base.first; base; base= base->next) {
 			if(base->lay & G.vd->lay) {
 				base->selcol= code;
 				glLoadName(code);
-				draw_object(base);
+				draw_object(base, DRAW_PICKING|DRAW_CONSTCOLOR);
 				code++;
 			}
-			base= base->next;
 		}
+		G.vd->xray= FALSE;	// restore
 	}
+	
 	glPopName();	/* see above (pushname) */
 	hits= glRenderMode(GL_RENDER);
 	if(hits<0) error("Too many objects in select buffer");
@@ -835,10 +1027,13 @@ short selectprojektie(unsigned int *buffer, short x1, short y1, short x2, short
 	Mat4MulMat4(G.vd->persmat, G.vd->viewmat, curarea->winmat);
 	
 	if(G.vd->drawtype > OB_WIRE) {
-		G.zbuf= 0;
+		G.vd->zbuf= 0;
 		glDisable(GL_DEPTH_TEST);
 	}
 	persp(PERSP_WIN);
+	
+	if(G.vd->flag & V3D_CLIPPING)
+		view3d_clr_clipping();
 
 	return hits;
 }
@@ -987,7 +1182,7 @@ void initlocalview()
 
 void centreview()	/* like a localview without local! */
 {
-	Base *base;
+	Object *ob= OBACT;
 	float size, min[3], max[3], afm[3];
 	int ok=0;
 
@@ -995,13 +1190,32 @@ void centreview()	/* like a localview without local! */
 	max[0]= max[1]= max[2]= -1.0e10;
 
 	if(G.obedit) {
-		minmax_verts(min, max);
-		//minmax_object(G.obedit, min, max);
-		
+		minmax_verts(min, max);	// ony selected
+		ok= 1;
+	}
+	else if(ob && (ob->flag & OB_POSEMODE)) {
+		if(ob->pose) {
+			bPoseChannel *pchan;
+			float vec[3];
+			for(pchan= ob->pose->chanbase.first; pchan; pchan= pchan->next) {
+				if(pchan->bone->flag & BONE_SELECTED) {
+					ok= 1;
+					VECCOPY(vec, pchan->pose_head);
+					Mat4MulVecfl(ob->obmat, vec);
+					DO_MINMAX(vec, min, max);
+					VECCOPY(vec, pchan->pose_tail);
+					Mat4MulVecfl(ob->obmat, vec);
+					DO_MINMAX(vec, min, max);
+				}
+			}
+		}
+	}
+	else if (G.f & G_FACESELECT) {
+		minmax_tface(min, max);
 		ok= 1;
 	}
 	else {
-		base= FIRSTBASE;
+		Base *base= FIRSTBASE;
 		while(base) {
 			if TESTBASE(base)  {
 				minmax_object(base->object, min, max);
@@ -1091,7 +1305,10 @@ void endlocalview(ScrArea *sa)
 			if( base->lay & locallay ) {
 				base->lay-= locallay;
 				if(base->lay==0) base->lay= v3d->layact;
-				if(base->object != G.obedit) base->flag |= SELECT;
+				if(base->object != G.obedit) {
+					base->flag |= SELECT;
+					base->object->flag |= SELECT;
+				}
 				base->object->lay= base->lay;
 			}
 			base= base->next;
@@ -1099,6 +1316,7 @@ void endlocalview(ScrArea *sa)
 
 		countall();
 		allqueue(REDRAWVIEW3D, 0);	/* because of select */
+		allqueue(REDRAWOOPS, 0);	/* because of select */
 		
 	}
 }
