@@ -1,5 +1,5 @@
 /**
- * $Id: vpaint.c,v 1.28 2005/05/18 20:32:32 ton Exp $
+ * $Id: vpaint.c,v 1.54 2005/11/26 12:23:33 ton Exp $
  *
  * ***** BEGIN GPL/BL DUAL LICENSE BLOCK *****
  *
@@ -51,8 +51,11 @@
 #include "BLI_arithb.h"
 #include "MTC_matrixops.h"
 
+#include "DNA_action_types.h"
+#include "DNA_armature_types.h"
 #include "DNA_mesh_types.h"
 #include "DNA_meshdata_types.h"
+#include "DNA_modifier_types.h"
 #include "DNA_object_types.h"
 #include "DNA_object_force.h"
 #include "DNA_screen_types.h"
@@ -60,21 +63,27 @@
 #include "DNA_view3d_types.h"
 #include "DNA_userdef_types.h"
 
+#include "BKE_armature.h"
+#include "BKE_DerivedMesh.h"
+#include "BKE_depsgraph.h"
+#include "BKE_deform.h"
 #include "BKE_displist.h"
 #include "BKE_global.h"
 #include "BKE_mesh.h"
+#include "BKE_modifier.h"
 #include "BKE_object.h"
 #include "BKE_utildefines.h"
 
-#include "BIF_graphics.h"
-#include "BIF_interface.h"
-#include "BIF_mywindow.h"
 #include "BIF_editview.h"
+#include "BIF_graphics.h"
+#include "BIF_glutil.h"
+#include "BIF_gl.h"
+#include "BIF_interface.h"
+#include "BIF_meshtools.h"
+#include "BIF_mywindow.h"
 #include "BIF_space.h"
 #include "BIF_screen.h"
 #include "BIF_toolbox.h"
-#include "BIF_glutil.h"
-#include "BIF_gl.h"
 
 #include "BDR_vpaint.h"
 
@@ -94,15 +103,10 @@
 #define VP_MUL	3
 #define VP_FILT	4
 
-	/* Gvp.flag */
-#define VP_COLINDEX	1
-#define VP_AREA		2
-#define VP_SOFT		4
-#define VP_NORMALS	8
-
 #define MAXINDEX	512000
 
-VPaint Gvp= {1.0, 1.0, 1.0, 0.2, 25.0, 1.0, 1.0, 0, VP_AREA+VP_SOFT};
+VPaint Gvp= {1.0, 1.0, 1.0, 0.2, 25.0, 1.0, 1.0, 0, VP_AREA+VP_SOFT+VP_SPRAY};
+VPaint Gwp= {1.0, 1.0, 1.0, 0.2, 25.0, 1.0, 1.0, 0, VP_AREA+VP_SOFT};
 float vpimat[3][3];
 unsigned int *vpaintundobuf= NULL;
 int totvpaintundo;
@@ -162,19 +166,16 @@ void do_shared_vertexcol(Mesh *me)
 	mface= me->mface;
 	mcol= (char *)me->mcol;
 	for(a=me->totface; a>0; a--, mface++, mcol+=16) {
-		if(mface->v3) {
-			if(tface==0 || (tface->mode & TF_SHAREDCOL) || (G.f & G_FACESELECT)==0) {
-				
-				scol= scolmain+4*mface->v1;
-				scol[0]++; scol[1]+= mcol[1]; scol[2]+= mcol[2]; scol[3]+= mcol[3];
-				scol= scolmain+4*mface->v2;
-				scol[0]++; scol[1]+= mcol[5]; scol[2]+= mcol[6]; scol[3]+= mcol[7];
-				scol= scolmain+4*mface->v3;
-				scol[0]++; scol[1]+= mcol[9]; scol[2]+= mcol[10]; scol[3]+= mcol[11];
-				if(mface->v4) {
-					scol= scolmain+4*mface->v4;
-					scol[0]++; scol[1]+= mcol[13]; scol[2]+= mcol[14]; scol[3]+= mcol[15];
-				}
+		if(tface==0 || (tface->mode & TF_SHAREDCOL) || (G.f & G_FACESELECT)==0) {
+			scol= scolmain+4*mface->v1;
+			scol[0]++; scol[1]+= mcol[1]; scol[2]+= mcol[2]; scol[3]+= mcol[3];
+			scol= scolmain+4*mface->v2;
+			scol[0]++; scol[1]+= mcol[5]; scol[2]+= mcol[6]; scol[3]+= mcol[7];
+			scol= scolmain+4*mface->v3;
+			scol[0]++; scol[1]+= mcol[9]; scol[2]+= mcol[10]; scol[3]+= mcol[11];
+			if(mface->v4) {
+				scol= scolmain+4*mface->v4;
+				scol[0]++; scol[1]+= mcol[13]; scol[2]+= mcol[14]; scol[3]+= mcol[15];
 			}
 		}
 		if(tface) tface++;
@@ -195,22 +196,16 @@ void do_shared_vertexcol(Mesh *me)
 	mface= me->mface;
 	mcol= (char *)me->mcol;
 	for(a=me->totface; a>0; a--, mface++, mcol+=16) {
-		if(mface->v3) {
-			if(tface==0 || (tface->mode & TF_SHAREDCOL) || (G.f & G_FACESELECT)==0) {
-				
-				scol= scolmain+4*mface->v1;
-				mcol[1]= scol[1]; mcol[2]= scol[2]; mcol[3]= scol[3];
-				
-				scol= scolmain+4*mface->v2;
-				mcol[5]= scol[1]; mcol[6]= scol[2]; mcol[7]= scol[3];
-				
-				scol= scolmain+4*mface->v3;
-				mcol[9]= scol[1]; mcol[10]= scol[2]; mcol[11]= scol[3];
-				
-				if(mface->v4) {
-					scol= scolmain+4*mface->v4;
-					mcol[13]= scol[1]; mcol[14]= scol[2]; mcol[15]= scol[3];
-				}
+		if(tface==0 || (tface->mode & TF_SHAREDCOL) || (G.f & G_FACESELECT)==0) {
+			scol= scolmain+4*mface->v1;
+			mcol[1]= scol[1]; mcol[2]= scol[2]; mcol[3]= scol[3];
+			scol= scolmain+4*mface->v2;
+			mcol[5]= scol[1]; mcol[6]= scol[2]; mcol[7]= scol[3];
+			scol= scolmain+4*mface->v3;
+			mcol[9]= scol[1]; mcol[10]= scol[2]; mcol[11]= scol[3];
+			if(mface->v4) {
+				scol= scolmain+4*mface->v4;
+				mcol[13]= scol[1]; mcol[14]= scol[2]; mcol[15]= scol[3];
 			}
 		}
 		if(tface) tface++;
@@ -223,8 +218,8 @@ void make_vertexcol()	/* single ob */
 {
 	Object *ob;
 	Mesh *me;
-	DispList *dl;
-	
+	int i;
+
 	/*
 	 * Always copies from shadedisplist to mcol.
 	 * When there are tfaces, it copies the colors and frees mcol
@@ -239,41 +234,16 @@ void make_vertexcol()	/* single ob */
 	me= get_mesh(ob);
 	if(me==0) return;
 
-	/* to be able to copy lighting info, we only use it from single side (ton) */
-	if(me->flag & ME_TWOSIDED) {
-		me->flag &= ~ME_TWOSIDED;
+	if(me->mcol) MEM_freeN(me->mcol);
+	mesh_create_shadedColors(ob, 1, (unsigned int**) &me->mcol, NULL);
+
+	for (i=0; i<me->totface*4; i++) {
+		me->mcol[i].a = 255;
 	}
-	
-	/* but subsurf vertex colors are wrong */
-	if (me->flag & ME_SUBSURF) {
-		if(me->mcol) MEM_freeN(me->mcol);
-		me->mcol= MEM_callocN(4*me->totface*sizeof(MCol), "mcol");
-	}
-	else {
-		dl= ob->disp.first;
 		
-		if(dl==NULL || dl->col1==NULL) {
-			shadeDispList(ob);
-			dl= find_displist(&ob->disp, DL_VERTCOL);
-		}
-		if(dl && dl->col1) {
-			if(me->mcol) MEM_freeN(me->mcol);
-			me->mcol= MEM_dupallocN(dl->col1);
-		}
-	}
-	
-	if(me->mcol) {
-		int i;
-		for (i=0; i<me->totface*4; i++) {
-			MCol *mcol= &me->mcol[i];
-			mcol->a= 255;
-		}
-		
-		if(me->tface) mcol_to_tface(me, 1);
-	}
-	
-	freedisplist(&(ob->disp));
+	if(me->tface) mcol_to_tface(me, 1);
 	
+	DAG_object_flush_update(G.scene, ob, OB_RECALC_DATA);
 	
 	allqueue(REDRAWBUTSEDIT, 0);
 	allqueue(REDRAWVIEW3D, 0);
@@ -321,6 +291,8 @@ void vpaint_undo()
 		*from= temp;
 		to++; from++;
 	}
+	DAG_object_flush_update(G.scene, ob, OB_RECALC_DATA);
+	
 	allqueue(REDRAWVIEW3D, 0);
 	if(me->tface) mcol_to_tface(me, 1);
 }
@@ -353,6 +325,8 @@ void clear_vpaint()
 		to++; 
 	}
 	BIF_undo_push("Clear vertex colors");
+	DAG_object_flush_update(G.scene, ob, OB_RECALC_DATA);
+	
 	allqueue(REDRAWVIEW3D, 0);
 	if(me->tface) mcol_to_tface(me, 1);
 }
@@ -487,6 +461,8 @@ void free_vertexpaint()
 	if(wpaintundobuf) 
 		free_dverts(wpaintundobuf, totwpaintundo);
 	wpaintundobuf= NULL;
+	
+	mesh_octree_table(NULL, NULL, 'e');
 }
 
 
@@ -583,31 +559,59 @@ static unsigned int mcol_mul(unsigned int col1, unsigned int col2, int fac)
 	return col;
 }
 
-static void vpaint_blend( unsigned int *col, unsigned int paintcol, int alpha)
+static void vpaint_blend( unsigned int *col, unsigned int *colorig, unsigned int paintcol, int alpha)
 {
-	
+
 	if(Gvp.mode==VP_MIX || Gvp.mode==VP_FILT) *col= mcol_blend( *col, paintcol, alpha);
 	else if(Gvp.mode==VP_ADD) *col= mcol_add( *col, paintcol, alpha);
 	else if(Gvp.mode==VP_SUB) *col= mcol_sub( *col, paintcol, alpha);
 	else if(Gvp.mode==VP_MUL) *col= mcol_mul( *col, paintcol, alpha);
+	
+	/* if no spray, clip color adding with colorig & orig alpha */
+	if((Gvp.flag & VP_SPRAY)==0) {
+		unsigned int testcol=0, a;
+		char *cp, *ct, *co;
+		
+		alpha= (int)(255.0*Gvp.a);
+		
+		if(Gvp.mode==VP_MIX || Gvp.mode==VP_FILT) testcol= mcol_blend( *colorig, paintcol, alpha);
+		else if(Gvp.mode==VP_ADD) testcol= mcol_add( *colorig, paintcol, alpha);
+		else if(Gvp.mode==VP_SUB) testcol= mcol_sub( *colorig, paintcol, alpha);
+		else if(Gvp.mode==VP_MUL) testcol= mcol_mul( *colorig, paintcol, alpha);
+		
+		cp= (char *)col;
+		ct= (char *)&testcol;
+		co= (char *)colorig;
+		
+		for(a=0; a<4; a++) {
+			if( ct[a]<co[a] ) {
+				if( cp[a]<ct[a] ) cp[a]= ct[a];
+				else if( cp[a]>co[a] ) cp[a]= co[a];
+			}
+			else {
+				if( cp[a]<co[a] ) cp[a]= co[a];
+				else if( cp[a]>ct[a] ) cp[a]= ct[a];
+			}
+		}
+	}
 }
 
 
-static int sample_backbuf_area(int x, int y)
+static int sample_backbuf_area(int x, int y, float size)
 {
 	unsigned int rect[129*129], *rt;
-	int x1, y1, x2, y2, size, a, tot=0, index;
+	int x1, y1, x2, y2, a, tot=0, index;
 	
 	if(totvpaintundo>=MAXINDEX) return 0;
 	
-	if(Gvp.size>64.0) Gvp.size= 64.0;
+	if(size>64.0) size= 64.0;
 	
-	x1= x-Gvp.size;
-	x2= x+Gvp.size;
+	x1= x-size;
+	x2= x+size;
 	CLAMP(x1, 0, curarea->winx);
 	CLAMP(x2, 0, curarea->winx);
-	y1= y-Gvp.size;
-	y2= y+Gvp.size;
+	y1= y-size;
+	y2= y+size;
 	CLAMP(y1, 0, curarea->winy);
 	CLAMP(y2, 0, curarea->winy);
 #ifdef __APPLE__
@@ -616,7 +620,7 @@ static int sample_backbuf_area(int x, int y)
 	glReadPixels(x1+curarea->winrct.xmin, y1+curarea->winrct.ymin, x2-x1+1, y2-y1+1, GL_RGBA, GL_UNSIGNED_BYTE,  rect);
 	glReadBuffer(GL_BACK);	
 
-	if(G.order==B_ENDIAN) IMB_convert_rgba_to_abgr( (int)(4*Gvp.size*Gvp.size), rect);
+	if(G.order==B_ENDIAN) IMB_convert_rgba_to_abgr( (int)(4*size*size), rect);
 
 	rt= rect;
 	size= (y2-y1)*(x2-x1);
@@ -662,33 +666,34 @@ static unsigned int sample_backbuf(int x, int y)
 	return framebuffer_to_index(col);
 }
 
-static int calc_vp_alpha(MVert *mvert, short *mval)
+static int calc_vp_alpha_dl(VPaint *vp, float *vert_nor, short *mval)
 {
-	float fac, dx, dy, nor[3];
+	float fac, dx, dy;
 	int alpha;
 	short vertco[2];
 	
-	if(Gvp.flag & VP_SOFT) {
-	 	project_short_noclip(mvert->co , vertco);
+	if(vp->flag & VP_SOFT) {
+	 	project_short_noclip(vert_nor, vertco);
 		dx= mval[0]-vertco[0];
 		dy= mval[1]-vertco[1];
 		
 		fac= sqrt(dx*dx + dy*dy);
-		if(fac > Gvp.size) return 0;
-	
-		alpha= 255.0*Gvp.a*(1.0-fac/Gvp.size);
+		if(fac > vp->size) return 0;
+
+		alpha= 255.0*vp->a*(1.0-fac/vp->size);
 	}
 	else {
-		alpha= 255.0*Gvp.a;
+		alpha= 255.0*vp->a;
 	}
-	if(Gvp.flag & VP_NORMALS) {
-		VECCOPY(nor, mvert->no);
+
+	if(vp->flag & VP_NORMALS) {
+		float *no= vert_nor+3;
 		
-		/* transpose ! */
-		fac= vpimat[2][0]*nor[0]+vpimat[2][1]*nor[1]+vpimat[2][2]*nor[2];
+			/* transpose ! */
+		fac= vpimat[2][0]*no[0]+vpimat[2][1]*no[1]+vpimat[2][2]*no[2];
 		if(fac>0.0) {
-			dx= vpimat[0][0]*nor[0]+vpimat[0][1]*nor[1]+vpimat[0][2]*nor[2];
-			dy= vpimat[1][0]*nor[0]+vpimat[1][1]*nor[1]+vpimat[1][2]*nor[2];
+			dx= vpimat[0][0]*no[0]+vpimat[0][1]*no[1]+vpimat[0][2]*no[2];
+			dy= vpimat[1][0]*no[0]+vpimat[1][1]*no[1]+vpimat[1][2]*no[2];
 			
 			alpha*= fac/sqrt(dx*dx + dy*dy + fac*fac);
 		}
@@ -696,60 +701,16 @@ static int calc_vp_alpha(MVert *mvert, short *mval)
 	}
 	
 	return alpha;
-
 }
 
-static int calc_vp_alpha_dl(DispList *disp, MVert *mvert, int vert, short *mval)
-/* Lets us do soft vertex painting onto a deformed mesh */
+
+void wpaint_undo (void)
 {
-	float fac, dx, dy, nor[3];
-	int alpha;
-	short vertco[2];
-	
-	/* For safety's sake !*/
-	if (!disp || !disp->verts)
-		return calc_vp_alpha (mvert+vert, mval);
-
-	//	use display list 
-	if(Gvp.flag & VP_SOFT) {
-	 	project_short_noclip(disp->verts+(vert*3), vertco);
-		dx= mval[0]-vertco[0];
-		dy= mval[1]-vertco[1];
-		
-		fac= sqrt(dx*dx + dy*dy);
-		if(fac > Gvp.size) return 0;
-	
-		alpha= 255.0*Gvp.a*(1.0-fac/Gvp.size);
-	}
-	else {
-		alpha= 255.0*Gvp.a;
-	}
-
-	if(Gvp.flag & VP_NORMALS) {
-		if(disp->nors) {
-			VECCOPY(nor, disp->nors+(vert*3));
-			
-			// transpose ! 
-			fac= vpimat[2][0]*nor[0]+vpimat[2][1]*nor[1]+vpimat[2][2]*nor[2];
-			if(fac>0.0) {
-				dx= vpimat[0][0]*nor[0]+vpimat[0][1]*nor[1]+vpimat[0][2]*nor[2];
-				dy= vpimat[1][0]*nor[0]+vpimat[1][1]*nor[1]+vpimat[1][2]*nor[2];
-				
-				alpha*= fac/sqrt(dx*dx + dy*dy + fac*fac);
-			}
-			else return 0;
-		}
-	}
-	
-	return alpha;
-
-}
-
-
-void wpaint_undo (void){
+	Object *ob= OBACT;
 	Mesh	*me;
+	MDeformVert *swapbuf;
 
-	me = get_mesh(OBACT);
+	me = get_mesh(ob);
 	if (!me)
 		return;
 
@@ -762,17 +723,28 @@ void wpaint_undo (void){
 	if (totwpaintundo != me->totvert)
 		return;
 
-	free_dverts(me->dvert, me->totvert);
+	swapbuf= me->dvert;
 
+	/* copy undobuf to mesh */
 	me->dvert= MEM_mallocN(sizeof(MDeformVert)*me->totvert, "deformVert");
 	copy_dverts(me->dvert, wpaintundobuf, totwpaintundo);
+	
+	/* copy previous mesh to undo */
+	free_dverts(wpaintundobuf, me->totvert);
+	wpaintundobuf= MEM_mallocN(sizeof(MDeformVert)*me->totvert, "wpaintundo");
+	copy_dverts(wpaintundobuf, swapbuf, totwpaintundo);
+	
+	/* now free previous mesh dverts */
+	free_dverts(swapbuf, me->totvert);
 
-	makeDispList(OBACT);
+	DAG_object_flush_update(G.scene, ob, OB_RECALC_DATA);
+	DAG_object_flush_update(G.scene, modifiers_isDeformedByArmature(ob), OB_RECALC_DATA);
 	scrarea_do_windraw(curarea);
 	
 }
 
-void copy_wpaint_undo (MDeformVert *dverts, int dcount){
+void copy_wpaint_undo (MDeformVert *dverts, int dcount)
+{
 	if (wpaintundobuf)
 		free_dverts(wpaintundobuf, totwpaintundo);
 
@@ -781,23 +753,249 @@ void copy_wpaint_undo (MDeformVert *dverts, int dcount){
 	copy_dverts (wpaintundobuf, dverts, dcount);
 }
 
+static void wpaint_blend(MDeformWeight *dw, MDeformWeight *uw, float alpha, float paintval)
+{
+	
+	if(dw==NULL || uw==NULL) return;
+	
+	if(Gwp.mode==VP_MIX || Gwp.mode==VP_FILT)
+		dw->weight = paintval*alpha + dw->weight*(1.0-alpha);
+	else if(Gwp.mode==VP_ADD)
+		dw->weight += paintval*alpha;
+	else if(Gwp.mode==VP_SUB) 
+		dw->weight -= paintval*alpha;
+	else if(Gwp.mode==VP_MUL) 
+		/* first mul, then blend the fac */
+		dw->weight = ((1.0-alpha) + alpha*paintval)*dw->weight;
+	
+	CLAMP(dw->weight, 0.0f, 1.0f);
+	
+	/* if no spray, clip result with orig weight & orig alpha */
+	if((Gwp.flag & VP_SPRAY)==0) {
+		float testw=0.0f;
+		
+		alpha= Gwp.a;
+		
+		if(Gwp.mode==VP_MIX || Gwp.mode==VP_FILT)
+			testw = paintval*alpha + uw->weight*(1.0-alpha);
+		else if(Gwp.mode==VP_ADD)
+			testw = uw->weight + paintval*alpha;
+		else if(Gwp.mode==VP_SUB) 
+			testw = uw->weight - paintval*alpha;
+		else if(Gwp.mode==VP_MUL) 
+			/* first mul, then blend the fac */
+			testw = ((1.0-alpha) + alpha*paintval)*uw->weight;
+		
+		CLAMP(testw, 0.0f, 1.0f);
+		
+		if( testw<uw->weight ) {
+			if(dw->weight < testw) dw->weight= testw;
+			else if(dw->weight > uw->weight) dw->weight= uw->weight;
+		}
+		else {
+			if(dw->weight > testw) dw->weight= testw;
+			else if(dw->weight < uw->weight) dw->weight= uw->weight;
+		}
+	}
+	
+}
+
+static MDeformWeight *get_defweight(MDeformVert *dv, int defgroup)
+{
+	int i;
+	for (i=0; i<dv->totweight; i++){
+		if (dv->dw[i].def_nr == defgroup)
+			return dv->dw+i;
+	}
+	return NULL;
+}
+
+/* ----------------------------------------------------- */
+
+/* used for 3d view, on active object, assumes me->dvert exists */
+/* if mode==1: */
+/*     samples cursor location, and gives menu with vertex groups to activate */
+/* else */
+/*     sets editbutvweight to the closest weight value to vertex */
+/*     note: we cant sample frontbuf, weight colors are interpolated too unpredictable */
+static void sample_wpaint(int mode)
+{
+	Object *ob= OBACT;
+	Mesh *me= get_mesh(ob);
+	int index;
+	short mval[2], sco[2];
+	
+	getmouseco_areawin(mval);
+	index= sample_backbuf(mval[0], mval[1]);
+	
+	if(index && index<=me->totface) {
+		MFace *mface;
+		
+		mface= ((MFace *)me->mface) + index-1;
+		
+		if(mode==1) {	/* sampe which groups are in here */
+			MDeformVert *dv;
+			int a, totgroup;
+			
+			totgroup= BLI_countlist(&ob->defbase);
+			if(totgroup) {
+				int totmenu=0;
+				int *groups=MEM_callocN(totgroup*sizeof(int), "groups");
+				
+				dv= me->dvert+mface->v1;
+				for(a=0; a<dv->totweight; a++) {
+					if (dv->dw[a].def_nr<totgroup)
+						groups[dv->dw[a].def_nr]= 1;
+				}
+				dv= me->dvert+mface->v2;
+				for(a=0; a<dv->totweight; a++) {
+					if (dv->dw[a].def_nr<totgroup)
+						groups[dv->dw[a].def_nr]= 1;
+				}
+				dv= me->dvert+mface->v3;
+				for(a=0; a<dv->totweight; a++) {
+					if (dv->dw[a].def_nr<totgroup)
+						groups[dv->dw[a].def_nr]= 1;
+				}
+				if(mface->v4) {
+					dv= me->dvert+mface->v4;
+					for(a=0; a<dv->totweight; a++) {
+						if (dv->dw[a].def_nr<totgroup)
+							groups[dv->dw[a].def_nr]= 1;
+					}
+				}
+				for(a=0; a<totgroup; a++)
+					if(groups[a]) totmenu++;
+				
+				if(totmenu==0) {
+					notice("No Vertex Group Selected");
+				}
+				else {
+					bDeformGroup *dg;
+					short val;
+					char item[40], *str= MEM_mallocN(40*totmenu+40, "menu");
+					
+					strcpy(str, "Vertex Groups %t");
+					for(a=0, dg=ob->defbase.first; dg && a<totgroup; a++, dg= dg->next) {
+						if(groups[a]) {
+							sprintf(item, "|%s %%x%d", dg->name, a);
+							strcat(str, item);
+						}
+					}
+					
+					val= pupmenu(str);
+					if(val>=0) {
+						ob->actdef= val+1;
+						DAG_object_flush_update(G.scene, ob, OB_RECALC_DATA);
+						allqueue(REDRAWVIEW3D, 0);
+						allqueue(REDRAWOOPS, 0);
+						allqueue(REDRAWBUTSEDIT, 0);
+					}
+					MEM_freeN(str);
+				}
+				MEM_freeN(groups);
+			}
+			else notice("No Vertex Groups in Object");
+		}
+		else {
+			DerivedMesh *dm;
+			MDeformWeight *dw;
+			extern float editbutvweight;
+			float w1, w2, w3, w4, co[3], fac;
+			int needsFree;
+			
+			dm = mesh_get_derived_final(ob, &needsFree);
+			if(dm->getVertCo==NULL) {
+				notice("Not supported yet");
+			}
+			else {
+				/* calc 3 or 4 corner weights */
+				dm->getVertCo(dm, mface->v1, co);
+				project_short_noclip(co, sco);
+				w1= ((mval[0]-sco[0])*(mval[0]-sco[0]) + (mval[1]-sco[1])*(mval[1]-sco[1]));
+				
+				dm->getVertCo(dm, mface->v2, co);
+				project_short_noclip(co, sco);
+				w2= ((mval[0]-sco[0])*(mval[0]-sco[0]) + (mval[1]-sco[1])*(mval[1]-sco[1]));
+				
+				dm->getVertCo(dm, mface->v3, co);
+				project_short_noclip(co, sco);
+				w3= ((mval[0]-sco[0])*(mval[0]-sco[0]) + (mval[1]-sco[1])*(mval[1]-sco[1]));
+				
+				if(mface->v4) {
+					dm->getVertCo(dm, mface->v4, co);
+					project_short_noclip(co, sco);
+					w4= ((mval[0]-sco[0])*(mval[0]-sco[0]) + (mval[1]-sco[1])*(mval[1]-sco[1]));
+				}
+				else w4= 1.0e10;
+				
+				fac= MIN4(w1, w2, w3, w4);
+				if(w1==fac) {
+					dw= get_defweight(me->dvert+mface->v1, ob->actdef-1);
+					if(dw) editbutvweight= dw->weight; else editbutvweight= 0.0f;
+				}
+				else if(w2==fac) {
+					dw= get_defweight(me->dvert+mface->v2, ob->actdef-1);
+					if(dw) editbutvweight= dw->weight; else editbutvweight= 0.0f;
+				}
+				else if(w3==fac) {
+					dw= get_defweight(me->dvert+mface->v3, ob->actdef-1);
+					if(dw) editbutvweight= dw->weight; else editbutvweight= 0.0f;
+				}
+				else if(w4==fac) {
+					if(mface->v4) {
+						dw= get_defweight(me->dvert+mface->v4, ob->actdef-1);
+						if(dw) editbutvweight= dw->weight; else editbutvweight= 0.0f;
+					}
+				}
+				if (needsFree)
+					dm->release(dm);
+			}
+		}		
+		
+	}
+	allqueue(REDRAWBUTSEDIT, 0);
+	
+}
+
+static void do_weight_paint_vertex(Object *ob, int index, int alpha, float paintweight, int vgroup_mirror)
+{
+	Mesh *me= ob->data;
+	MDeformWeight	*dw, *uw;
+	int vgroup= ob->actdef-1;
+	
+	dw= verify_defweight(me->dvert+index, vgroup);
+	uw= verify_defweight(wpaintundobuf+index, vgroup);
+	wpaint_blend(dw, uw, (float)alpha/255.0, paintweight);
+	
+	if(Gwp.flag & VP_MIRROR_X) {	/* x mirror painting */
+		int j= mesh_get_x_mirror_vert(ob, index);
+		if(j>=0) {
+			/* copy, not paint again */
+			if(vgroup_mirror != -1)
+				uw= verify_defweight(me->dvert+j, vgroup_mirror);
+			else
+				uw= verify_defweight(me->dvert+j, vgroup);
+				
+			uw->weight= dw->weight;
+		}
+	}
+}
+
 void weight_paint(void)
 {
+	extern float editbutvweight;
 	Object *ob; 
 	Mesh *me;
 	MFace *mface;
 	TFace *tface;
-	float mat[4][4], imat[4][4];
-	int index, totindex;
+	float mat[4][4], imat[4][4], paintweight, *vertexcosnos;
+	int index, totindex, alpha, totw;
+	int vgroup_mirror= -1;
 	short mval[2], mvalo[2], firsttime=1, mousebut;
-	MDeformWeight	*dw, *uw;
-	extern float editbutvweight;
-	DispList *dl;
-
 
 	if((G.f & G_WEIGHTPAINT)==0) return;
 	if(G.obedit) return;
-	if(G.obpose) return;
 	
 	if(indexar==NULL) init_vertexpaint();
 	
@@ -805,10 +1003,47 @@ void weight_paint(void)
 	if(ob->id.lib) return;
 
 	me= get_mesh(ob);
-	if (!me->dvert){
+	if(me==NULL || me->totface==0) return;
+	
+	/* if nothing was added yet, we make dverts and a vertex deform group */
+	if (!me->dvert)
+		create_dverts(me);
+	
+	if(G.qual & LR_CTRLKEY) {
+		sample_wpaint(0);
 		return;
 	}
-	if(me==0 || me->totface==0) return;
+	if(G.qual & LR_SHIFTKEY) {
+		sample_wpaint(1);
+		return;
+	}
+	
+	/* painting on subsurfs should give correct points too, this returns me->totvert amount */
+	vertexcosnos= mesh_get_mapped_verts_nors(ob);
+	
+	/* this happens on a Bone select, when no vgroup existed yet */
+	if(ob->actdef<=0) {
+		Object *modob;
+		if((modob = modifiers_isDeformedByArmature(ob))) {
+			bPoseChannel *pchan;
+			for(pchan= modob->pose->chanbase.first; pchan; pchan= pchan->next)
+				if(pchan->bone->flag & SELECT)
+					break;
+			if(pchan) {
+				bDeformGroup *dg= get_named_vertexgroup(ob, pchan->name);
+				if(dg==NULL)
+					dg= add_defgroup_name(ob, pchan->name);	// sets actdef
+				else
+					ob->actdef= get_defgroup_num(ob, dg);
+				allqueue(REDRAWBUTSEDIT, 0);
+			}
+		}
+	}
+	if(ob->defbase.first==NULL) {
+		add_defgroup(ob);
+		allqueue(REDRAWBUTSEDIT, 0);
+	}	
+	
 	if(ob->lay & G.vd->lay); else error("Active object is not in this layer");
 	
 	persp(PERSP_VIEW);
@@ -835,16 +1070,40 @@ void weight_paint(void)
 	if (U.flag & USER_LMOUSESELECT) mousebut = R_MOUSE;
 	else mousebut = L_MOUSE;
 	
+	/* if mirror painting, find the other group */			
+	if(Gwp.flag & VP_MIRROR_X) {
+		bDeformGroup *defgroup= BLI_findlink(&ob->defbase, ob->actdef-1);
+		if(defgroup) {
+			bDeformGroup *curdef;
+			int actdef= 0;
+			char name[32];
+
+			BLI_strncpy(name, defgroup->name, 32);
+			bone_flip_name(name, 0);		// 0 = don't strip off number extensions
+			
+			for (curdef = ob->defbase.first; curdef; curdef=curdef->next, actdef++)
+				if (!strcmp(curdef->name, name))
+					break;
+			if(curdef==NULL) {
+				int olddef= ob->actdef;	/* tsk, add_defgroup sets the active defgroup */
+				curdef= add_defgroup_name (ob, name);
+				ob->actdef= olddef;
+			}
+			
+			if(curdef && curdef!=defgroup)
+				vgroup_mirror= actdef;
+		}
+	}
+	
 	while (get_mbut() & mousebut) {
 		getmouseco_areawin(mval);
 		
 		if(firsttime || mval[0]!=mvalo[0] || mval[1]!=mvalo[1]) {
-			
 			firsttime= 0;
 			
 			/* which faces are involved */
-			if(Gvp.flag & VP_AREA) {
-				totindex= sample_backbuf_area(mval[0], mval[1]);
+			if(Gwp.flag & VP_AREA) {
+				totindex= sample_backbuf_area(mval[0], mval[1], Gwp.size);
 			}
 			else {
 				indexar[0]= sample_backbuf(mval[0], mval[1]);
@@ -854,7 +1113,7 @@ void weight_paint(void)
 			
 			MTC_Mat4SwapMat4(G.vd->persmat, mat);
 			
-			if(Gvp.flag & VP_COLINDEX) {
+			if(Gwp.flag & VP_COLINDEX) {
 				for(index=0; index<totindex; index++) {
 					if(indexar[index] && indexar[index]<=me->totface) {
 					
@@ -880,36 +1139,78 @@ void weight_paint(void)
 				}
 			}
 			
-			dl= find_displist(&ob->disp, DL_VERTS);
+			/* make sure each vertex gets treated only once */
+			/* and calculate filter weight */
+			totw= 0;
+			if(Gwp.mode==VP_FILT) 
+				paintweight= 0.0f;
+			else
+				paintweight= editbutvweight;
+			
+			for(index=0; index<totindex; index++) {
+				if(indexar[index] && indexar[index]<=me->totface) {
+					mface= me->mface + (indexar[index]-1);
+					
+					(me->dvert+mface->v1)->flag= 1;
+					(me->dvert+mface->v2)->flag= 1;
+					(me->dvert+mface->v3)->flag= 1;
+					if(mface->v4) (me->dvert+mface->v4)->flag= 1;
+					
+					if(Gwp.mode==VP_FILT) {
+						MDeformWeight *dw;
+						dw= verify_defweight(me->dvert+mface->v1, ob->actdef-1);
+						if(dw) {paintweight+= dw->weight; totw++;}
+						dw= verify_defweight(me->dvert+mface->v2, ob->actdef-1);
+						if(dw) {paintweight+= dw->weight; totw++;}
+						dw= verify_defweight(me->dvert+mface->v3, ob->actdef-1);
+						if(dw) {paintweight+= dw->weight; totw++;}
+						if(mface->v4) {
+							dw= verify_defweight(me->dvert+mface->v4, ob->actdef-1);
+							if(dw) {paintweight+= dw->weight; totw++;}
+						}
+					}
+				}
+			}
+			
+			if(Gwp.mode==VP_FILT) 
+				paintweight/= (float)totw;
+			
 			for(index=0; index<totindex; index++) {
 				
 				if(indexar[index] && indexar[index]<=me->totface) {
-					mface= ((MFace *)me->mface) + (indexar[index]-1);
+					mface= me->mface + (indexar[index]-1);
 					
-
-					if (calc_vp_alpha_dl(dl, me->mvert, mface->v1, mval)){
-						dw= verify_defweight(me->dvert+mface->v1, ob->actdef-1);
-						uw= verify_defweight(wpaintundobuf+mface->v1, ob->actdef-1);
-						if (dw) dw->weight = editbutvweight*Gvp.a + (uw->weight*(1.0-Gvp.a));
+					if((me->dvert+mface->v1)->flag) {
+						alpha= calc_vp_alpha_dl(&Gwp, vertexcosnos+6*mface->v1, mval);
+						if(alpha) {
+							do_weight_paint_vertex(ob, mface->v1, alpha, paintweight, vgroup_mirror);
+						}
+						(me->dvert+mface->v1)->flag= 0;
 					}
 					
-					if (calc_vp_alpha_dl(dl, me->mvert, mface->v2, mval)){
-						dw= verify_defweight(me->dvert+mface->v2, ob->actdef-1);
-						uw= verify_defweight(wpaintundobuf+mface->v2, ob->actdef-1);
-						if (dw) dw->weight = editbutvweight*Gvp.a + (uw->weight*(1.0-Gvp.a));
+					if((me->dvert+mface->v2)->flag) {
+						alpha= calc_vp_alpha_dl(&Gwp, vertexcosnos+6*mface->v2, mval);
+						if(alpha) {
+							do_weight_paint_vertex(ob, mface->v2, alpha, paintweight, vgroup_mirror);
+						}
+						(me->dvert+mface->v2)->flag= 0;
 					}
 					
-					if (calc_vp_alpha_dl(dl, me->mvert, mface->v3, mval)){
-						dw= verify_defweight(me->dvert+mface->v3, ob->actdef-1);
-						uw= verify_defweight(wpaintundobuf+mface->v3, ob->actdef-1);
-						if (dw) dw->weight = editbutvweight*Gvp.a + (uw->weight*(1.0-Gvp.a));
+					if((me->dvert+mface->v3)->flag) {
+						alpha= calc_vp_alpha_dl(&Gwp, vertexcosnos+6*mface->v3, mval);
+						if(alpha) {
+							do_weight_paint_vertex(ob, mface->v3, alpha, paintweight, vgroup_mirror);
+						}
+						(me->dvert+mface->v3)->flag= 0;
 					}
 					
-					if(mface->v4) {
-						if (calc_vp_alpha_dl(dl, me->mvert, mface->v4, mval)){
-							dw= verify_defweight(me->dvert+mface->v4, ob->actdef-1);
-							uw= verify_defweight(wpaintundobuf+mface->v4, ob->actdef-1);
-							if (dw) dw->weight = editbutvweight*Gvp.a + (uw->weight*(1.0-Gvp.a));
+					if((me->dvert+mface->v4)->flag) {
+						if(mface->v4) {
+							alpha= calc_vp_alpha_dl(&Gwp, vertexcosnos+6*mface->v4, mval);
+							if(alpha) {
+								do_weight_paint_vertex(ob, mface->v4, alpha, paintweight, vgroup_mirror);
+							}
+							(me->dvert+mface->v4)->flag= 0;
 						}
 					}
 				}
@@ -922,12 +1223,15 @@ void weight_paint(void)
 		
 		if(mval[0]!=mvalo[0] || mval[1]!=mvalo[1]) {
 
+			DAG_object_flush_update(G.scene, ob, OB_RECALC_DATA);
 			scrarea_do_windraw(curarea);
-
-			/* draw circle in backbuf! */
-			persp(PERSP_WIN);
-			fdrawXORcirc((float)mval[0], (float)mval[1], Gvp.size);
-			persp(PERSP_VIEW);
+			
+			if(Gwp.flag & (VP_AREA|VP_SOFT)) {
+				/* draw circle in backbuf! */
+				persp(PERSP_WIN);
+				fdrawXORcirc((float)mval[0], (float)mval[1], Gwp.size);
+				persp(PERSP_VIEW);
+			}
 
 			screen_swapbuffers();
 			backdrawview3d(0);
@@ -941,13 +1245,14 @@ void weight_paint(void)
 		MEM_freeN(me->mcol);
 		me->mcol= 0;
 	}
+	if(vertexcosnos)
+		MEM_freeN(vertexcosnos);
 	
-	makeDispList(ob);
+	DAG_object_flush_update(G.scene, ob, OB_RECALC_DATA);
 	// this flag is event for softbody to refresh weightpaint values
 	if(ob->soft) ob->softflag |= OB_SB_REDO;
 	
 	allqueue(REDRAWVIEW3D, 0);
-
 }
 
 void vertex_paint()
@@ -956,10 +1261,9 @@ void vertex_paint()
 	Mesh *me;
 	MFace *mface;
 	TFace *tface;
-	DispList *dl;
-	float mat[4][4], imat[4][4];
-	unsigned int paintcol=0, *mcol, fcol1, fcol2;
-	int index, alpha, totindex, total;
+	float mat[4][4], imat[4][4], *vertexcosnos;
+	unsigned int paintcol=0, *mcol, *mcolorig, fcol1, fcol2;
+	int index, alpha, totindex;
 	short mval[2], mvalo[2], firsttime=1, mousebut;
 	
 	if((G.f & G_VERTEXPAINT)==0) return;
@@ -978,6 +1282,9 @@ void vertex_paint()
 
 	if(me->tface==NULL && me->mcol==NULL) return;
 	
+	/* painting on subsurfs should give correct points too, this returns me->totvert amount */
+	vertexcosnos= mesh_get_mapped_verts_nors(ob);
+	
 	persp(PERSP_VIEW);
 	/* imat for normals */
 	Mat4MulMat4(mat, ob->obmat, G.vd->viewmat);
@@ -1007,12 +1314,12 @@ void vertex_paint()
 		getmouseco_areawin(mval);
 		
 		if(firsttime || mval[0]!=mvalo[0] || mval[1]!=mvalo[1]) {
-			
+
 			firsttime= 0;
 
 			/* which faces are involved */
 			if(Gvp.flag & VP_AREA) {
-				totindex= sample_backbuf_area(mval[0], mval[1]);
+				totindex= sample_backbuf_area(mval[0], mval[1], Gvp.size);
 			}
 			else {
 				indexar[0]= sample_backbuf(mval[0], mval[1]);
@@ -1047,14 +1354,13 @@ void vertex_paint()
 				}
 			}
 
-			dl= find_displist(&ob->disp, DL_VERTS);
-
 			for(index=0; index<totindex; index++) {
 
 				if(indexar[index] && indexar[index]<=me->totface) {
 				
 					mface= ((MFace *)me->mface) + (indexar[index]-1);
-					mcol= ( (unsigned int *)me->mcol) + 4*(indexar[index]-1);
+					mcol=	  ( (unsigned int *)me->mcol) + 4*(indexar[index]-1);
+					mcolorig= ( (unsigned int *)vpaintundobuf) + 4*(indexar[index]-1);
 
 					if(Gvp.mode==VP_FILT) {
 						fcol1= mcol_blend( mcol[0], mcol[1], 128);
@@ -1068,32 +1374,22 @@ void vertex_paint()
 						
 					}
 					
-					total= 0;
+					alpha= calc_vp_alpha_dl(&Gvp, vertexcosnos+6*mface->v1, mval);
+					if(alpha) vpaint_blend( mcol, mcolorig, paintcol, alpha);
 					
-					total+= alpha= calc_vp_alpha_dl(dl, me->mvert, mface->v1, mval);
-					if(alpha) vpaint_blend( mcol, paintcol, alpha);
-					
-					total+= alpha= calc_vp_alpha_dl(dl, me->mvert, mface->v2, mval);
-					if(alpha) vpaint_blend( mcol+1, paintcol, alpha);
+					alpha= calc_vp_alpha_dl(&Gvp, vertexcosnos+6*mface->v2, mval);
+					if(alpha) vpaint_blend( mcol+1, mcolorig+1, paintcol, alpha);
 	
-					total+= alpha= calc_vp_alpha_dl(dl, me->mvert, mface->v3, mval);
-					if(alpha) vpaint_blend( mcol+2, paintcol, alpha);
+					alpha= calc_vp_alpha_dl(&Gvp, vertexcosnos+6*mface->v3, mval);
+					if(alpha) vpaint_blend( mcol+2, mcolorig+2, paintcol, alpha);
 
 					if(mface->v4) {
-						total+= alpha= calc_vp_alpha_dl(dl, me->mvert, mface->v4, mval);
-						if(alpha) vpaint_blend( mcol+3, paintcol, alpha);
+						alpha= calc_vp_alpha_dl(&Gvp, vertexcosnos+6*mface->v4, mval);
+						if(alpha) vpaint_blend( mcol+3, mcolorig+3, paintcol, alpha);
 					}
-					
-					/* if(total==0) { */
-					/* 	alpha= 25*Gvp.a; */
-					/* 	vpaint_blend( mcol, paintcol, alpha); */
-					/* 	vpaint_blend( mcol+1, paintcol, alpha); */
-					/* 	vpaint_blend( mcol+2, paintcol, alpha); */
-					/* 	if(mface->v4) vpaint_blend( mcol+3, paintcol, alpha); */
-					/* } */
 				}
 			}
-			
+				
 			MTC_Mat4SwapMat4(G.vd->persmat, mat);
 			
 			do_shared_vertexcol(me);
@@ -1101,13 +1397,15 @@ void vertex_paint()
 				mcol_to_tface(me, 0);
 			}
 	
+			DAG_object_flush_update(G.scene, ob, OB_RECALC_DATA);
 			scrarea_do_windraw(curarea);
 
-			/* draw circle in backbuf! */
-			persp(PERSP_WIN);
-			fdrawXORcirc((float)mval[0], (float)mval[1], Gvp.size);
-			persp(PERSP_VIEW);
-
+			if(Gvp.flag & (VP_AREA|VP_SOFT)) {
+				/* draw circle in backbuf! */
+				persp(PERSP_WIN);
+				fdrawXORcirc((float)mval[0], (float)mval[1], Gvp.size);
+				persp(PERSP_VIEW);
+			}
 			screen_swapbuffers();
 			backdrawview3d(0);
 			
@@ -1121,7 +1419,9 @@ void vertex_paint()
 		MEM_freeN(me->mcol);
 		me->mcol= 0;
 	}
-	
+	if(vertexcosnos)
+		MEM_freeN(vertexcosnos);
+
 	allqueue(REDRAWVIEW3D, 0);
 }
 
@@ -1141,23 +1441,45 @@ void set_wpaint(void)		/* toggle */
 		return;
 	}
 	
-//	if(me && me->tface==NULL && me->mcol==NULL) make_vertexcol();
-	
 	if(G.f & G_WEIGHTPAINT) G.f &= ~G_WEIGHTPAINT;
 	else G.f |= G_WEIGHTPAINT;
 	
-	allqueue(REDRAWVIEW3D, 0);
+	allqueue(REDRAWVIEW3D, 1);	/* including header */
 	allqueue(REDRAWBUTSEDIT, 0);
 	
+		/* Weightpaint works by overriding colors in mesh,
+		 * so need to make sure we recalc on enter and
+		 * exit (exit needs doing regardless because we
+		 * should redeform).
+		 */
+	if (me) {
+		DAG_object_flush_update(G.scene, OBACT, OB_RECALC_DATA);
+	}
+
 	if(G.f & G_WEIGHTPAINT) {
+		Object *par;
+		
 		setcursor_space(SPACE_VIEW3D, CURSOR_VPAINT);
+		
+		mesh_octree_table(ob, NULL, 's');
+
+		/* verify if active weight group is also active bone */
+		par= modifiers_isDeformedByArmature(ob);
+		if(par && (par->flag & OB_POSEMODE)) {
+			bPoseChannel *pchan;
+			for(pchan= par->pose->chanbase.first; pchan; pchan= pchan->next)
+				if(pchan->bone->flag & BONE_ACTIVE)
+					break;
+			if(pchan)
+				vertexgroup_select_by_name(ob, pchan->name);
+		}
 	}
 	else {
 		freefastshade();	/* to be sure */
-		if (ob)
-			makeDispList(ob);
 		if(!(G.f & G_FACESELECT))
 			setcursor_space(SPACE_VIEW3D, CURSOR_STD);
+		
+		mesh_octree_table(ob, NULL, 'e');
 	}
 }
 
@@ -1194,7 +1516,7 @@ void set_vpaint(void)		/* toggle */
 			set_wpaint();
 	}
 	
-	allqueue(REDRAWVIEW3D, 0);
+	allqueue(REDRAWVIEW3D, 1); 	/* including header */
 	allqueue(REDRAWBUTSEDIT, 0);
 	
 	if(G.f & G_VERTEXPAINT) {
@@ -1202,7 +1524,9 @@ void set_vpaint(void)		/* toggle */
 	}
 	else {
 		freefastshade();	/* to be sure */
-		if (ob) makeDispList(ob);
+		if (me) {
+			DAG_object_flush_update(G.scene, ob, OB_RECALC_DATA);
+		}
 		if((G.f & G_FACESELECT)==0) setcursor_space(SPACE_VIEW3D, CURSOR_STD);
 	}
 }
