@@ -1,7 +1,7 @@
 /**
  * Functions for writing windows avi-format files.
  *
- * $Id: writeavicodec.c,v 1.12 2004/01/13 14:50:45 phase Exp $
+ * $Id: writeavicodec.c,v 1.15 2005/04/17 18:00:33 ton Exp $
  *
  * ***** BEGIN GPL/BL DUAL LICENSE BLOCK *****
  *
@@ -41,6 +41,9 @@
 
 #define  INC_OLE2
 #include <windows.h>
+#undef rad1
+#undef rad2
+
 #include <windowsx.h>
 #include <memory.h>
 #include <mmsystem.h>
@@ -53,7 +56,7 @@
 #include "DNA_userdef_types.h"
 
 #include "render_types.h"
-#include "render.h"
+#include "render.h"			// lotsof R. stuff
 
 #include "BKE_global.h"
 #include "BKE_scene.h"
@@ -561,12 +564,12 @@ static void acd_to_opts(AviCodecData *acd)
 		opts.cbParms = acd->cbParms;
 		
 		if (acd->lpFormat && acd->cbFormat) {
-			opts.lpFormat = malloc(opts.cbFormat);
+			opts.lpFormat = MEM_mallocN(opts.cbFormat, "opts lpFormat");
 			memcpy(opts.lpFormat, acd->lpFormat, opts.cbFormat);
 		}
 
 		if (acd->lpParms && acd->cbParms) {
-			opts.lpParms = malloc(opts.cbParms);
+			opts.lpParms = MEM_mallocN(opts.cbParms, "opts.cbParms");
 			memcpy(opts.lpParms, acd->lpParms, opts.cbParms);
 		}
 	}
@@ -575,11 +578,11 @@ static void acd_to_opts(AviCodecData *acd)
 static void free_opts_data()
 {
 	if (opts.lpFormat) {
-		free(opts.lpFormat);
+		MEM_freeN(opts.lpFormat);
 		opts.lpFormat = NULL;
 	}
 	if (opts.lpParms) {
-		free(opts.lpParms);
+		MEM_freeN(opts.lpParms);
 		opts.lpParms = NULL;
 	}
 }
