@@ -1,5 +1,5 @@
 /**
- * $Id: writeimage.c,v 1.11 2005/03/14 14:56:40 sirdude Exp $
+ * $Id: writeimage.c,v 1.14 2005/12/13 13:38:14 ton Exp $
  *
  * ***** BEGIN GPL/BL DUAL LICENSE BLOCK *****
  *
@@ -51,12 +51,18 @@ int BIF_write_ibuf(ImBuf *ibuf, char *name)
 	/* to be used for e.g. envmap, not rendered images */
 	
 	if(G.scene->r.imtype== R_IRIS) ibuf->ftype= IMAGIC;
+	else if ((G.scene->r.imtype==R_RADHDR)) {
+		ibuf->ftype= RADHDR;
+	}
 	else if ((G.scene->r.imtype==R_PNG)) {
 		ibuf->ftype= PNG;
 	}
 	else if ((G.scene->r.imtype==R_BMP)) {
 		ibuf->ftype= BMP;
 	}
+	else if ((G.have_libtiff) && (G.scene->r.imtype==R_TIFF)) {
+		ibuf->ftype= TIF;
+	}
 	else if ((G.scene->r.imtype==R_TARGA) || (G.scene->r.imtype==R_PNG)) {
 		// fall back to Targa if PNG writing is not supported
 		ibuf->ftype= TGA;
@@ -94,8 +100,9 @@ void BIF_save_envmap(EnvMap *env, char *str)
 	int dx;
 	
 	/* all interactive stuff is handled in buttons.c */
-	
-	dx= (env->cuberes * R.r.size) / 100;
+	if(env->cube[0]->ibuf==NULL) return;
+		
+	dx= env->cube[0]->ibuf->x;
 	ibuf= IMB_allocImBuf(3*dx, 2*dx, 24, IB_rect, 0);
 	
 	IMB_rectop(ibuf, env->cube[0]->ibuf, 
