@@ -7,8 +7,8 @@ using namespace std;
 static string command_path = "";
 
 #ifdef WIN32 
-
-#include <windows.h>
+#define WIN32_SKIP_HKEY_PROTECTION
+#include "BLI_winstuff.h"
 
 #ifndef FILE_MAXDIR
 #define FILE_MAXDIR  160
@@ -119,8 +119,12 @@ bool yafrayFileRender_t::initExport()
 	// try the user setting setting first, export dir must be set and exist
 	if (strlen(U.yfexportdir)==0) 
 	{
-		cout << "No export directory set in user defaults!\n";
-		dir_failed = true;
+		cout << "No export directory set in user defaults!" << endl;
+		char* temp = getenv("TEMP");
+		// if no envar, use /tmp
+		xmlpath = temp ? temp : "/tmp";
+		cout << "Will try TEMP instead: " << xmlpath << endl;
+		// no fail here, but might fail when opening file...
 	}
 	else 
 	{
@@ -209,23 +213,31 @@ bool yafrayFileRender_t::writeRender()
 		ostr << "\tAA_pixelwidth=\"" << R.r.YF_AApixelsize << "\" AA_threshold=\"" << R.r.YF_AAthreshold << "\"\n";
 	}
 	else {
-		if ((R.r.GImethod!=0) && (R.r.GIquality>1) && (!R.r.GIcache))
-			ostr << "\tAA_passes=\"5\" AA_minsamples=\"5\"\n";
-		else if ((R.r.mode & R_OSA) && (R.r.osa)) {
+		// removed the default AA settings for midquality GI, better leave it to user
+		if ((R.r.mode & R_OSA) && (R.r.osa)) {
 			int passes=(R.r.osa%4)==0 ? R.r.osa/4 : 1;
 			int minsamples=(R.r.osa%4)==0 ? 4 : R.r.osa;
 			ostr << "\tAA_passes=\"" << passes << "\" AA_minsamples=\"" << minsamples << "\"\n";
 		}
 		else ostr << "\tAA_passes=\"0\" AA_minsamples=\"1\"\n";
-		ostr << "\tAA_pixelwidth=\"1.5\" AA_threshold=\"0.05\" bias=\"" << R.r.YF_raybias << "\"\n";
+		ostr << "\tAA_pixelwidth=\"1.5\" AA_threshold=\"0.05\" bias=\"" << R.r.YF_raybias
+				 << "\" clamp_rgb=\"" << ((R.r.YF_clamprgb==0) ? "on" : "off") << "\"\n";
 	}
 
-	if (hasworld) ostr << "\tbackground_name=\"world_background\"\n";
+	World *world = G.scene->world;
+	if (world) ostr << "\tbackground_name=\"world_background\"\n";
  
 	// alpha channel render when RGBA button enabled
 	if (R.r.planes==R_PLANES32) ostr << "\n\tsave_alpha=\"on\"";
 	ostr << " >\n";
 
+	// basic fog
+	if (world && (world->mode & WO_MIST)) {
+		float fd = world->mistdist;
+		if (fd>0) fd=1.f/fd; else fd=1;
+		ostr << "\t<fog_density value=\"" << fd << "\" />\n";
+		ostr << "\t<fog_color r=\"" << world->horr << "\" g=\"" << world->horg << "\" b=\"" << world->horb << "\" />\n";
+	}
 	ostr << "\t<outfile value=\"" << imgout << "\" />\n";
 
 	ostr << "</render>\n\n";
@@ -256,9 +268,6 @@ void yafrayFileRender_t::displayImage()
 	// although it is possible to load the image using blender,
 	// maybe it is best to just do a read here, for now the yafray output is always a raw tga anyway
 
-	// rectot already freed in initrender
-	R.rectot = (unsigned int *)MEM_callocN(sizeof(int)*R.rectx*R.recty, "rectot");
-
 	FILE* fp = fopen(imgout.c_str(), "rb");
 	if (fp==NULL) {
 		cout << "YAF_displayImage(): Could not open image file\n";
@@ -269,6 +278,12 @@ void yafrayFileRender_t::displayImage()
 	fread(&header, 1, 18, fp);
 	unsigned short width = (unsigned short)(header[12] + (header[13]<<8));
 	unsigned short height = (unsigned short)(header[14] + (header[15]<<8));
+	// don't do anything if resolution doesn't match that of rectot
+	if ((width!=R.rectx) || (height!=R.recty)) {
+		fclose(fp);
+		fp = NULL;
+		return;
+	}
 	unsigned char byte_per_pix = (unsigned char)(header[16]>>3);
 	// read past any id (none in this case though)
 	unsigned int idlen = (unsigned int)header[0];
@@ -397,6 +412,10 @@ void yafrayFileRender_t::writeTextures()
 				ts = (tex->stype & 1) ? "rings" : "bands";	//stype 1&3 ringtype
 				ostr << "\t\t<wood_type value=\"" << ts << "\" />\n";
 				ostr << "\t\t<noise_type value=\"" << ntype << "\" />\n";
+				// shape parameter, for some reason noisebasis2 is used...
+				ts = "sin";
+				if (tex->noisebasis2==1) ts="saw"; else if (tex->noisebasis2==2) ts="tri";
+				ostr << "\t\t<shape value=\"" << ts << "\" />\n";
 				ostr << "\t</attributes>\n</shader>\n\n";
 				xmlfile << ostr.str();
 				break;
@@ -411,6 +430,9 @@ void yafrayFileRender_t::writeTextures()
 				ostr << "\t\t<hard value=\"" << hardnoise << "\" />\n";
 				ostr << "\t\t<sharpness value=\"" << (float)(1<<tex->stype) << "\" />\n";
 				ostr << "\t\t<noise_type value=\"" << ntype << "\" />\n";
+				ts = "sin";
+				if (tex->noisebasis2==1) ts="saw"; else if (tex->noisebasis2==2) ts="tri";
+				ostr << "\t\t<shape value=\"" << ts << "\" />\n";
 				ostr << "\t</attributes>\n</shader>\n\n";
 				xmlfile << ostr.str();
 				break;
@@ -543,6 +565,7 @@ void yafrayFileRender_t::writeTextures()
 					string texpath(ima->name);
 					adjustPath(texpath);
 					ostr << "\t\t<filename value=\"" << texpath << "\" />\n";
+					ostr << "\t\t<interpolate value=\"" << ((tex->imaflag & TEX_INTERPOL) ? "bilinear" : "none") << "\" />\n";
 					ostr << "\t</attributes>\n</shader>\n\n";
 					xmlfile << ostr.str();
 				}
@@ -600,16 +623,61 @@ void yafrayFileRender_t::writeTextures()
 
 void yafrayFileRender_t::writeShader(const string &shader_name, Material* matr, const string &facetexname)
 {
+	// if material has ramps, export colorbands first
+	if (matr->mode & (MA_RAMP_COL|MA_RAMP_SPEC))
+	{
+		// both colorbands without input shader
+		ColorBand* cb = matr->ramp_col;
+		if ((matr->mode & MA_RAMP_COL) && (cb!=NULL))
+		{
+			ostr.str("");
+			ostr << "<shader type=\"colorband\" name=\"" << shader_name+"_difframp" << "\" >\n";
+			ostr << "\t<attributes>\n\t</attributes>\n";
+			for (int i=0;i<cb->tot;i++) {
+				ostr << "\t<modulator value=\"" << cb->data[i].pos << "\" >\n";
+				ostr << "\t\t<color r=\"" << cb->data[i].r << "\"" <<
+													" g=\"" << cb->data[i].g << "\"" <<
+													" b=\"" << cb->data[i].b << "\"" <<
+													" a=\"" << cb->data[i].a << "\" />\n";
+				ostr << "\t</modulator>\n";
+			}
+			ostr << "</shader>\n\n";
+			xmlfile << ostr.str();
+		}
+		cb = matr->ramp_spec;
+		if ((matr->mode & MA_RAMP_SPEC) && (cb!=NULL))
+		{
+			ostr.str("");
+			ostr << "<shader type=\"colorband\" name=\"" << shader_name+"_specramp" << "\" >\n";
+			ostr << "\t<attributes>\n\t</attributes>\n";
+			for (int i=0;i<cb->tot;i++) {
+				ostr << "\t<modulator value=\"" << cb->data[i].pos << "\" >\n";
+				ostr << "\t\t<color r=\"" << cb->data[i].r << "\"" <<
+													" g=\"" << cb->data[i].g << "\"" <<
+													" b=\"" << cb->data[i].b << "\"" <<
+													" a=\"" << cb->data[i].a << "\" />\n";
+				ostr << "\t</modulator>\n";
+			}
+			ostr << "</shader>\n\n";
+			xmlfile << ostr.str();
+		}
+	}
+
 	ostr.str("");
 	ostr << "<shader type=\"blendershader\" name=\"" << shader_name << "\" >\n";
 	ostr << "\t<attributes>\n";
-	float diff = matr->alpha;
-	ostr << "\t\t<color r=\"" << matr->r*diff << "\" g=\"" << matr->g*diff << "\" b=\"" << matr->b*diff << "\" />\n";
-	ostr << "\t\t<specular_color r=\"" << matr->specr << "\" g=\"" << matr->specg << "\" b=\"" << matr->specb << "\" />\n";
+	ostr << "\t\t<color r=\"" << matr->r << "\" g=\"" << matr->g << "\" b=\"" << matr->b << "\" />\n";
+	float sr=matr->specr, sg=matr->specg, sb=matr->specb;
+	if (matr->spec_shader==MA_SPEC_WARDISO) {
+		// ........
+		sr /= M_PI;
+		sg /= M_PI;
+		sb /= M_PI;
+	}
+	ostr << "\t\t<specular_color r=\"" << sr << "\" g=\"" << sg << "\" b=\"" << sb << "\" />\n";
 	ostr << "\t\t<mirror_color r=\"" << matr->mirr << "\" g=\"" << matr->mirg << "\" b=\"" << matr->mirb << "\" />\n";
 	ostr << "\t\t<diffuse_reflect value=\"" << matr->ref << "\" />\n";
 	ostr << "\t\t<specular_amount value=\"" << matr->spec << "\" />\n";
-	ostr << "\t\t<hard value=\"" << matr->har << "\" />\n";
 	ostr << "\t\t<alpha value=\"" << matr->alpha << "\" />\n";
 	// if no GI used, the GIpower parameter is not always initialized, so in that case ignore it
 	float bg_mult = (R.r.GImethod==0) ? 1 : R.r.GIpower;
@@ -618,20 +686,52 @@ void yafrayFileRender_t::writeShader(const string &shader_name, Material* matr,
 	// reflection/refraction
 	if ( (matr->mode & MA_RAYMIRROR) || (matr->mode & MA_RAYTRANSP) )
 		ostr << "\t\t<IOR value=\"" << matr->ang << "\" />\n";
-	if (matr->mode & MA_RAYMIRROR) {
-		float rf = matr->ray_mirror;
-		// blender uses mir color for reflection as well
+
+	if (matr->mode & MA_RAYMIRROR)
+	{
+		// Sofar yafray's min_refle parameter (which misleadingly actually controls fresnel reflection offset)
+		// has been mapped to Blender's ray_mirror parameter.
+		// This causes it be be misinterpreted and misused as a reflection amount control however.
+		// Besides that, it also causes extra complications for the yafray Blendershader.
+		// So added an actual amount of reflection parameter instead, and another
+		// extra parameter 'frsOfs' to actually control fresnel offset (re-uses Blender fresnel_mir_i param).
+		ostr << "\t\t<reflect value=\"on\" />\n";
+		ostr << "\t\t<reflect_amount value=\""<< matr->ray_mirror << "\" />\n";
+		float fo = 1.f-(matr->fresnel_mir_i-1.f)*0.25f;	// blender param range [1,5], also here reversed (1 in Blender -> no fresnel)
+		ostr << "\t\t<fresnel_offset value=\""<< fo << "\" />\n";
+
+		// for backward compatibility, also add old 'reflected' parameter, copy of mirror_color
 		ostr << "\t\t<reflected r=\"" << matr->mirr << "\" g=\"" << matr->mirg << "\" b=\"" << matr->mirb << "\" />\n";
-		ostr << "\t\t<min_refle value=\""<< rf << "\" />\n";
-		if (matr->ray_depth>maxraydepth) maxraydepth = matr->ray_depth;
+		// same for 'min_refle' param. Instead of the ray_mirror parameter that was used before, since now
+		// the parameter's function is taken over by the fresnel offset parameter, use that instead.
+		ostr << "\t\t<min_refle value=\"" << fo << "\" />\n";
+
 	}
+
 	if (matr->mode & MA_RAYTRANSP) 
 	{
-		float tr=1.0-matr->alpha;
-		ostr << "\t\t<transmitted r=\"" << matr->r * tr << "\" g=\"" << matr->g * tr << "\" b=\"" << matr->b * tr << "\" />\n";
+		ostr << "\t\t<refract value=\"on\" />\n";
+		ostr << "\t\t<transmit_filter value=\"" << matr->filter << "\" />\n";
 		// tir on by default
 		ostr << "\t\t<tir value=\"on\" />\n";
-		if (matr->ray_depth_tra>maxraydepth) maxraydepth = matr->ray_depth_tra;
+
+		// transmit absorption color
+		// to make things easier(?) for user it now specifies the actual color at 1 unit / YF_dscale of distance
+		const float maxlog = -log(1e-38);
+		float ar = (matr->YF_ar>0) ? -log(matr->YF_ar) : maxlog;
+		float ag = (matr->YF_ag>0) ? -log(matr->YF_ag) : maxlog;
+		float ab = (matr->YF_ab>0) ? -log(matr->YF_ab) : maxlog;
+		float sc = matr->YF_dscale;
+		if (sc!=0.f) sc=1.f/sc;
+		ostr << "\t\t<absorption r=\"" << ar*sc << "\" g=\"" << ag*sc << "\" b=\"" << ab*sc << "\" />\n";
+		// dispersion
+		ostr << "\t\t<dispersion_power value=\"" << matr->YF_dpwr << "\" />\n";
+		ostr << "\t\t<dispersion_samples value=\"" << matr->YF_dsmp << "\" />\n";
+		ostr << "\t\t<dispersion_jitter value=\"" << (matr->YF_djit ? "on" : "off") << "\" />\n";
+
+		// for backward compatibility, also add old 'transmitted' parameter, copy of 'color' * (1-alpha)
+		float na = 1.f-matr->alpha;
+		ostr << "\t\t<transmitted r=\"" << matr->r*na << "\" g=\"" << matr->g*na << "\" b=\"" << matr->b*na << "\" />\n";
 	}
 
 	string Mmode = "";
@@ -643,6 +743,70 @@ void yafrayFileRender_t::writeShader(const string &shader_name, Material* matr,
 	if (matr->mode & MA_ZTRA) Mmode += " ztransp";
 	if (matr->mode & MA_ONLYSHADOW) Mmode += " onlyshadow";
 	if (Mmode!="") ostr << "\t\t<matmodes value=\"" << Mmode << "\" />\n";
+
+	// diffuse & specular brdf, lambert/cooktorr defaults
+	// diffuse
+	if (matr->diff_shader==MA_DIFF_ORENNAYAR) {
+		ostr << "\t\t<diffuse_brdf value=\"oren_nayar\" />\n";
+		ostr << "\t\t<roughness value=\"" << matr->roughness << "\" />\n";
+	}
+	else if (matr->diff_shader==MA_DIFF_TOON) {
+		ostr << "\t\t<diffuse_brdf value=\"toon\" />\n";
+		ostr << "\t\t<toondiffuse_size value=\"" << matr->param[0] << "\" />\n";
+		ostr << "\t\t<toondiffuse_smooth value=\"" << matr->param[1] << "\" />\n";
+	}
+	else if (matr->diff_shader==MA_DIFF_MINNAERT) {
+		ostr << "\t\t<diffuse_brdf value=\"minnaert\" />\n";
+		ostr << "\t\t<darkening value=\"" << matr->darkness << "\" />\n";
+	}
+	else ostr << "\t\t<diffuse_brdf value=\"lambert\" />\n";
+	// specular
+	if (matr->spec_shader==MA_SPEC_PHONG) {
+		ostr << "\t\t<specular_brdf value=\"phong\" />\n";
+		ostr << "\t\t<hard value=\"" << matr->har << "\" />\n";
+	}
+	else if (matr->spec_shader==MA_SPEC_BLINN) {
+		ostr << "\t\t<specular_brdf value=\"blinn\" />\n";
+		ostr << "\t\t<blinn_ior value=\"" << matr->refrac << "\" />\n";
+		ostr << "\t\t<hard value=\"" << matr->har << "\" />\n";
+	}
+	else if (matr->spec_shader==MA_SPEC_TOON) {
+		ostr << "\t\t<specular_brdf value=\"toon\" />\n";
+		ostr << "\t\t<toonspecular_size value=\"" << matr->param[2] << "\" />\n";
+		ostr << "\t\t<toonspecular_smooth value=\"" << matr->param[3] << "\" />\n";
+	}
+	else if (matr->spec_shader==MA_SPEC_WARDISO) {
+		ostr << "\t\t<specular_brdf value=\"ward\" />\n";
+		ostr << "\t\t<u_roughness value=\"" << matr->rms << "\" />\n";
+		ostr << "\t\t<v_roughness value=\"" << matr->rms << "\" />\n";
+	}
+	else {
+		ostr << "\t\t<specular_brdf value=\"blender_cooktorr\" />\n";
+		ostr << "\t\t<hard value=\"" << matr->har << "\" />\n";
+	}
+
+	// ramps, if used
+	if (matr->mode & (MA_RAMP_COL|MA_RAMP_SPEC))
+	{
+		const string rm_blend[9] = {"mix", "add", "mul", "sub", "screen", "divide", "difference", "darken", "lighten"};
+		const string rm_mode[4] = {"shader", "energy", "normal", "result"};
+		// diffuse
+		if ((matr->mode & MA_RAMP_COL) && (matr->ramp_col!=NULL))
+		{
+			ostr << "\t\t<diffuse_ramp value=\"" << shader_name+"_difframp" << "\" />\n";
+			ostr << "\t\t<diffuse_ramp_mode value=\"" << rm_mode[(int)matr->rampin_col] << "\" />\n";
+			ostr << "\t\t<diffuse_ramp_blend value=\"" << rm_blend[(int)matr->rampblend_col] << "\" />\n";
+			ostr << "\t\t<diffuse_ramp_factor value=\"" << matr->rampfac_col << "\" />\n";
+		}
+		// specular
+		if ((matr->mode & MA_RAMP_SPEC) && (matr->ramp_spec!=NULL)) {
+			ostr << "\t\t<specular_ramp value=\"" << shader_name+"_specramp" << "\" />\n";
+			ostr << "\t\t<specular_ramp_mode value=\"" << rm_mode[(int)matr->rampin_spec] << "\" />\n";
+			ostr << "\t\t<specular_ramp_blend value=\"" << rm_blend[(int)matr->rampblend_spec] << "\" />\n";
+			ostr << "\t\t<specular_ramp_factor value=\"" << matr->rampfac_spec << "\" />\n";
+		}
+	}
+
 	ostr << "\t</attributes>\n";
 	xmlfile << ostr.str();
 	
@@ -680,12 +844,9 @@ void yafrayFileRender_t::writeShader(const string &shader_name, Material* matr,
 			else
 				ostr << "\t\t<input value=\"" << shader_name << "_map" << m2 << "\" />\n";
 
-			// blendtype
-			string ts = "mix";
-			if (mtex->blendtype==MTEX_MUL) ts="mul";
-			else if (mtex->blendtype==MTEX_ADD) ts="add";
-			else if (mtex->blendtype==MTEX_SUB) ts="sub";
-			ostr << "\t\t<mode value=\"" << ts << "\" />\n";
+			// blendtype, would have been nice if the order would have been the same as for ramps...
+			const string blendtype[9] = {"mix", "mul", "add", "sub", "divide", "darken", "difference", "lighten", "screen"};
+			ostr << "\t\t<mode value=\"" << blendtype[(int)mtex->blendtype] << "\" />\n";
 
 			// texture color (for use with MUL and/or no_rgb etc..)
 			ostr << "\t\t<texcol r=\"" << mtex->r << "\" g=\"" << mtex->g << "\" b=\"" << mtex->b << "\" />\n";
@@ -748,7 +909,6 @@ void yafrayFileRender_t::writeShader(const string &shader_name, Material* matr,
 				int t = 1;
 				if (mtex->maptoneg & MAP_ALPHA) t = -1;
 				ostr << "\t\t<alpha value=\"" << t << "\" />\n";
-
 			}
 
 			// emit modulation
@@ -758,9 +918,16 @@ void yafrayFileRender_t::writeShader(const string &shader_name, Material* matr,
 				ostr << "\t\t<emit value=\"" << t << "\" />\n";
 			}
 
+			// raymir modulation
+			if ((mtex->mapto & MAP_RAYMIRR) || (mtex->maptoneg & MAP_RAYMIRR)) {
+				int t = 1;
+				if (mtex->maptoneg & MAP_RAYMIRR) t = -1;
+				ostr << "\t\t<raymir value=\"" << t << "\" />\n";
+			}
+
 			// texture flag, combination of strings
+			string ts = "";
 			if (mtex->texflag & (MTEX_RGBTOINT | MTEX_STENCIL | MTEX_NEGATIVE)) {
-				ts = "";
 				if (mtex->texflag & MTEX_RGBTOINT) ts += "no_rgb ";
 				if (mtex->texflag & MTEX_STENCIL) ts += "stencil ";
 				if (mtex->texflag & MTEX_NEGATIVE) ts += "negative";
@@ -826,9 +993,9 @@ void yafrayFileRender_t::writeMaterialsAndModulators()
 			if (mtexL!=used_textures.end()) {
 				ostr.str("");
 				ostr << "<shader type=\"blendermapper\" name=\"" << blendmat->first + "_map" << m <<"\"";
-				if ((mtex->texco & TEXCO_OBJECT) || (mtex->texco & TEXCO_REFL))
+				if ((mtex->texco & TEXCO_OBJECT) || (mtex->texco & TEXCO_REFL) || (mtex->texco & TEXCO_NORM))
 				{
-					// For object & reflection mapping, add the object matrix to the modulator,
+					// For object, reflection & normal mapping, add the object matrix to the modulator,
 					// as in LF script, use camera matrix if no object specified.
 					// In this case this means the inverse of that matrix
 					float texmat[4][4], itexmat[4][4];
@@ -838,13 +1005,13 @@ void yafrayFileRender_t::writeMaterialsAndModulators()
 						MTC_Mat4CpyMat4(texmat, maincam_obj->obmat);
 					MTC_Mat4Invert(itexmat, texmat);
 					ostr << "\n\t\tm00=\"" << itexmat[0][0] << "\" m01=\"" << itexmat[1][0]
-							<< "\" m02=\"" << itexmat[2][0] << "\" m03=\"" << itexmat[3][0] << "\"\n";
+							 << "\" m02=\"" << itexmat[2][0] << "\" m03=\"" << itexmat[3][0] << "\"\n";
 					ostr << "\t\tm10=\"" << itexmat[0][1] << "\" m11=\"" << itexmat[1][1]
-							<< "\" m12=\"" << itexmat[2][1] << "\" m13=\"" << itexmat[3][1] << "\"\n";
+							 << "\" m12=\"" << itexmat[2][1] << "\" m13=\"" << itexmat[3][1] << "\"\n";
 					ostr << "\t\tm20=\"" << itexmat[0][2] << "\" m21=\"" << itexmat[1][2]
-							<< "\" m22=\"" << itexmat[2][2] << "\" m23=\"" << itexmat[3][2] << "\"\n";
+							 << "\" m22=\"" << itexmat[2][2] << "\" m23=\"" << itexmat[3][2] << "\"\n";
 					ostr << "\t\tm30=\"" << itexmat[0][3] << "\" m31=\"" << itexmat[1][3]
-							<< "\" m32=\"" << itexmat[2][3] << "\" m33=\"" << itexmat[3][3] << "\">\n";
+							 << "\" m32=\"" << itexmat[2][3] << "\" m33=\"" << itexmat[3][3] << "\">\n";
 				}
 				else ostr << ">\n";
 				ostr << "\t<attributes>\n";
@@ -910,6 +1077,14 @@ void yafrayFileRender_t::writeMaterialsAndModulators()
 						ostr << "\t\t<clipping value=\"clip\" />\n";
 					else if (tex->extend==TEX_CLIPCUBE)
 						ostr << "\t\t<clipping value=\"clipcube\" />\n";
+					else if (tex->extend==TEX_CHECKER) {
+						ostr << "\t\t<clipping value=\"checker\" />\n";
+						string ts = "";
+						if (tex->flag & TEX_CHECKER_ODD) ts += "odd";
+						if (tex->flag & TEX_CHECKER_EVEN) ts += " even";
+						ostr << "\t\t<checker_mode value=\"" << ts << "\" />\n";
+						ostr << "\t\t<checker_dist value=\"" << tex->checkerdist << "\" />\n";
+					}
 					else
 						ostr << "\t\t<clipping value=\"repeat\" />\n";
 
@@ -1032,7 +1207,9 @@ void yafrayFileRender_t::writeObject(Object* obj, const vector<VlakRen*> &VLR_li
 	// Export orco coords test.
 	// Previously was done by checking orco pointer, however this can be non-null but still not initialized.
 	// Test the rendermaterial texco flag instead.
-	bool EXPORT_ORCO = ((face0mat->ren->texco & TEXCO_ORCO)!=0);
+	// update2: bug #3193 it seems it has changed again with the introduction of static 'hair' particles,
+	// now it uses the vert pointer again as an extra test to make sure there are orco coords available
+	bool EXPORT_ORCO = (((face0mat->texco & TEXCO_ORCO)!=0) && (face0->v1->orco!=NULL));
 
 	string has_orco = "off";
 	if (EXPORT_ORCO) has_orco = "on";
@@ -1054,7 +1231,7 @@ void yafrayFileRender_t::writeObject(Object* obj, const vector<VlakRen*> &VLR_li
 		// or flat shaded, the smooth flag of the first face is used to determine
 		// the shading for the whole mesh
 		if (face0->flag & ME_SMOOTH)
-			xmlfile << "\t<mesh autosmooth=\"90\" has_orco=\"" << has_orco << "\" >\n";
+			xmlfile << "\t<mesh autosmooth=\"180\" has_orco=\"" << has_orco << "\" >\n";
 		else
 			xmlfile << "\t<mesh autosmooth=\"0.1\" has_orco=\"" << has_orco << "\" >\n";	//0 shows artefacts
 	}
@@ -1065,6 +1242,13 @@ void yafrayFileRender_t::writeObject(Object* obj, const vector<VlakRen*> &VLR_li
 
 	// vertices, transformed back to world
 	xmlfile << "\t\t<points>\n";
+
+	// for deformed objects, object->imat is no longer valid,
+	// so have to create inverse render matrix ourselves here
+	float mat[4][4], imat[4][4];
+	MTC_Mat4MulMat4(mat, obj->obmat, R.viewmat);
+	MTC_Mat4Invert(imat, mat);
+
 	for (vector<VlakRen*>::const_iterator fci=VLR_list.begin();
 				fci!=VLR_list.end();++fci)
 	{
@@ -1077,7 +1261,7 @@ void yafrayFileRender_t::writeObject(Object* obj, const vector<VlakRen*> &VLR_li
 			vert_idx[vlr->v1] = vidx++;
 			ver = vlr->v1;
 			MTC_cp3Float(ver->co, tvec);
-			MTC_Mat4MulVecfl(obj->imat, tvec);
+			MTC_Mat4MulVecfl(imat, tvec);
 			ostr << "\t\t\t<p x=\"" << tvec[0]
 								 << "\" y=\"" << tvec[1]
 								 << "\" z=\"" << tvec[2] << "\" />\n";
@@ -1092,7 +1276,7 @@ void yafrayFileRender_t::writeObject(Object* obj, const vector<VlakRen*> &VLR_li
 			vert_idx[vlr->v2] = vidx++;
 			ver = vlr->v2;
 			MTC_cp3Float(ver->co, tvec);
-			MTC_Mat4MulVecfl(obj->imat, tvec);
+			MTC_Mat4MulVecfl(imat, tvec);
 			ostr << "\t\t\t<p x=\"" << tvec[0]
 								 << "\" y=\"" << tvec[1]
 								 << "\" z=\"" << tvec[2] << "\" />\n";
@@ -1107,7 +1291,7 @@ void yafrayFileRender_t::writeObject(Object* obj, const vector<VlakRen*> &VLR_li
 			vert_idx[vlr->v3] = vidx++;
 			ver = vlr->v3;
 			MTC_cp3Float(ver->co, tvec);
-			MTC_Mat4MulVecfl(obj->imat, tvec);
+			MTC_Mat4MulVecfl(imat, tvec);
 			ostr << "\t\t\t<p x=\"" << tvec[0]
 								 << "\" y=\"" << tvec[1]
 								 << "\" z=\"" << tvec[2] << "\" />\n";
@@ -1122,7 +1306,7 @@ void yafrayFileRender_t::writeObject(Object* obj, const vector<VlakRen*> &VLR_li
 			vert_idx[vlr->v4] = vidx++;
 			ver = vlr->v4;
 			MTC_cp3Float(ver->co, tvec);
-			MTC_Mat4MulVecfl(obj->imat, tvec);
+			MTC_Mat4MulVecfl(imat, tvec);
 			ostr << "\t\t\t<p x=\"" << tvec[0]
 								 << "\" y=\"" << tvec[1]
 								 << "\" z=\"" << tvec[2] << "\" />\n";
@@ -1183,19 +1367,15 @@ void yafrayFileRender_t::writeObject(Object* obj, const vector<VlakRen*> &VLR_li
 		// since Blender seems to need vcols when uvs are used, for yafray only export when the material actually uses vcols
 		if ((EXPORT_VCOL) && (vlr->vcol)) {
 			// vertex colors
-			float vr, vg, vb;
-			vr = ((vlr->vcol[ui1] >> 24) & 255)/255.0;
-			vg = ((vlr->vcol[ui1] >> 16) & 255)/255.0;
-			vb = ((vlr->vcol[ui1] >> 8) & 255)/255.0;
-			ostr << " vcol_a_r=\"" << vr << "\" vcol_a_g=\"" << vg << "\" vcol_a_b=\"" << vb << "\"";
-			vr = ((vlr->vcol[ui2] >> 24) & 255)/255.0;
-			vg = ((vlr->vcol[ui2] >> 16) & 255)/255.0;
-			vb = ((vlr->vcol[ui2] >> 8) & 255)/255.0;
-			ostr << " vcol_b_r=\"" << vr << "\" vcol_b_g=\"" << vg << "\" vcol_b_b=\"" << vb << "\"";
-			vr = ((vlr->vcol[ui3] >> 24) & 255)/255.0;
-			vg = ((vlr->vcol[ui3] >> 16) & 255)/255.0;
-			vb = ((vlr->vcol[ui3] >> 8) & 255)/255.0;
-			ostr << " vcol_c_r=\"" << vr << "\" vcol_c_g=\"" << vg << "\" vcol_c_b=\"" << vb << "\"";
+			unsigned char* pt = reinterpret_cast<unsigned char*>(&vlr->vcol[ui1]);
+			ostr << " vcol_a_r=\"" << (float)pt[3]/255.f << "\" vcol_a_g=\"" << (float)pt[2]/255.f
+					 << "\" vcol_a_b=\"" << (float)pt[1]/255.f << "\"";
+			pt = reinterpret_cast<unsigned char*>(&vlr->vcol[ui2]);
+			ostr << " vcol_b_r=\"" << (float)pt[3]/255.f << "\" vcol_b_g=\"" << (float)pt[2]/255.f
+					 << "\" vcol_b_b=\"" << (float)pt[1]/255.f << "\"";
+			pt = reinterpret_cast<unsigned char*>(&vlr->vcol[ui3]);
+			ostr << " vcol_c_r=\"" << (float)pt[3]/255.f << "\" vcol_c_g=\"" << (float)pt[2]/255.f
+					 << "\" vcol_c_b=\"" << (float)pt[1]/255.f << "\"";
 		}
 		ostr << " shader_name=\"" << fmatname << "\" />\n";
 
@@ -1222,19 +1402,15 @@ void yafrayFileRender_t::writeObject(Object* obj, const vector<VlakRen*> &VLR_li
 			}
 			if ((EXPORT_VCOL) && (vlr->vcol)) {
 				// vertex colors
-				float vr, vg, vb;
-				vr = ((vlr->vcol[ui1] >> 24) & 255)/255.0;
-				vg = ((vlr->vcol[ui1] >> 16) & 255)/255.0;
-				vb = ((vlr->vcol[ui1] >> 8) & 255)/255.0;
-				ostr << " vcol_a_r=\"" << vr << "\" vcol_a_g=\"" << vg << "\" vcol_a_b=\"" << vb << "\"";
-				vr = ((vlr->vcol[ui2] >> 24) & 255)/255.0;
-				vg = ((vlr->vcol[ui2] >> 16) & 255)/255.0;
-				vb = ((vlr->vcol[ui2] >> 8) & 255)/255.0;
-				ostr << " vcol_b_r=\"" << vr << "\" vcol_b_g=\"" << vg << "\" vcol_b_b=\"" << vb << "\"";
-				vr = ((vlr->vcol[ui3] >> 24) & 255)/255.0;
-				vg = ((vlr->vcol[ui3] >> 16) & 255)/255.0;
-				vb = ((vlr->vcol[ui3] >> 8) & 255)/255.0;
-				ostr << " vcol_c_r=\"" << vr << "\" vcol_c_g=\"" << vg << "\" vcol_c_b=\"" << vb << "\"";
+				unsigned char* pt = reinterpret_cast<unsigned char*>(&vlr->vcol[ui1]);
+				ostr << " vcol_a_r=\"" << (float)pt[3]/255.f << "\" vcol_a_g=\"" << (float)pt[2]/255.f
+						 << "\" vcol_a_b=\"" << (float)pt[1]/255.f << "\"";
+				pt = reinterpret_cast<unsigned char*>(&vlr->vcol[ui2]);
+				ostr << " vcol_b_r=\"" << (float)pt[3]/255.f << "\" vcol_b_g=\"" << (float)pt[2]/255.f
+						 << "\" vcol_b_b=\"" << (float)pt[1]/255.f << "\"";
+				pt = reinterpret_cast<unsigned char*>(&vlr->vcol[ui3]);
+				ostr << " vcol_c_r=\"" << (float)pt[3]/255.f << "\" vcol_c_g=\"" << (float)pt[2]/255.f
+						 << "\" vcol_c_b=\"" << (float)pt[1]/255.f << "\"";
 			}
 			ostr << " shader_name=\"" << fmatname << "\" />\n";
 
@@ -1371,22 +1547,24 @@ void yafrayFileRender_t::writeLamps()
 		if (lamp->type==LA_LOCAL) {
 			if (lamp->mode & LA_YF_SOFT) {
 				// shadowmapped omnidirectional light
-				ostr << "softlight";
+				ostr << "softlight\"";
 				is_softL = true;
 			}
 			else if ((lamp->mode & LA_SHAD_RAY) && (lamp->YF_ltradius>0.0)) {
 				// area sphere, only when ray shadows enabled and radius>0.0
-				ostr << "spherelight";
+				ostr << "spherelight\"";
 				is_sphereL = true;
 			}
-			else ostr << "pointlight";
+			else ostr << "pointlight\"";
+			ostr << " glow_intensity=\"" << lamp->YF_glowint << "\" glow_offset=\"" << lamp->YF_glowofs
+					 << "\" glow_type=\"" << lamp->YF_glowtype << "\"";
 		}
 		else if (lamp->type==LA_SPOT)
-			ostr << "spotlight";
+			ostr << "spotlight\"";
 		else if ((lamp->type==LA_SUN) || (lamp->type==LA_HEMI))	// hemi exported as sun
-			ostr << "sunlight";
+			ostr << "sunlight\"";
 		else if (lamp->type==LA_YF_PHOTON)
-			ostr << "photonlight";
+			ostr << "photonlight\"";
 		else {
 			// possibly unknown type, ignore
 			cout << "Unknown Blender lamp type: " << lamp->type << endl;
@@ -1394,7 +1572,7 @@ void yafrayFileRender_t::writeLamps()
 		}
 		
 		//no name available here, create one
-		ostr << "\" name=\"LAMP" << i+1;
+		ostr << " name=\"LAMP" << i+1;
 		// color already premultiplied by energy, so only need distance here
 		float pwr = 1;	// default for sun/hemi, distance irrelevant
 		if ((lamp->type!=LA_SUN) && (lamp->type!=LA_HEMI)) {
@@ -1446,7 +1624,9 @@ void yafrayFileRender_t::writeLamps()
 				ostr << "\n\thalo=\"on\" " << "res=\"" << lamp->YF_bufsize << "\"\n";
 				int hsmp = ((12-lamp->shadhalostep)*16)/12;
 				hsmp = (hsmp+1)*16;	// makes range (16, 272) for halostep(12, 0), good enough?
-				ostr << "\tsamples=\"" << hsmp <<  "\" shadow_samples=\"" << (lamp->samp*lamp->samp) << "\"\n";
+				// halo 'samples' now 'stepsize'
+				// convert from old integer samples value to some reasonable stepsize
+				ostr << "\tstepsize=\"" << (1.0/sqrt((float)hsmp)) <<  "\" shadow_samples=\"" << (lamp->samp*lamp->samp) << "\"\n";
 				ostr << "\thalo_blur=\"0\" shadow_blur=\"" << (lamp->soft*0.01f) << "\"\n";
 				ostr << "\tfog_density=\"" << (lamp->haint*0.2f) << "\"";
 			}
@@ -1527,8 +1707,10 @@ void yafrayFileRender_t::writeCamera()
 	ostr << "\" aspect_ratio=\"" << R.ycor << "\"";
 
 	// dof params, only valid for real camera
+	float fdist = 1;	// only changes for ortho
 	if (maincam_obj->type==OB_CAMERA) {
 		Camera* cam = (Camera*)maincam_obj->data;
+		if (R.r.mode & R_ORTHO) fdist = cam->ortho_scale*(mainCamLens/32.f);
 		ostr << "\n\tdof_distance=\"" << cam->YF_dofdist << "\"";
 		ostr << " aperture=\"" << cam->YF_aperture << "\"";
 		string st = "on";
@@ -1565,8 +1747,6 @@ void yafrayFileRender_t::writeCamera()
 	ostr << "\t<from x=\"" << maincam_obj->obmat[3][0] << "\""
 							<< " y=\"" << maincam_obj->obmat[3][1] << "\""
 							<< " z=\"" << maincam_obj->obmat[3][2] << "\" />\n";
-	float fdist = fabs(R.viewmat[3][2]);
-	if (R.r.mode & R_ORTHO) fdist *= 0.01f;
 	ostr << "\t<to x=\"" << maincam_obj->obmat[3][0] - fdist * R.viewmat[0][2]
 					<< "\" y=\"" << maincam_obj->obmat[3][1] - fdist * R.viewmat[1][2]
 					<< "\" z=\"" << maincam_obj->obmat[3][2] - fdist * R.viewmat[2][2] << "\" />\n";
@@ -1580,16 +1760,59 @@ void yafrayFileRender_t::writeCamera()
 
 void yafrayFileRender_t::writeHemilight()
 {
+	World *world = G.scene->world;
+	bool fromAO = false;
+	if (R.r.GIquality==6){
+		// use Blender AO params is possible
+		if (world==NULL) return;
+		if ((world->mode & WO_AMB_OCC)==0) {
+			// no AO, use default GIquality
+			cout << "No ambient occlusion enabled\nUsing defaults of 25 samples & infinite distance instead" << endl;
+		}
+		else fromAO = true;
+	}
 	ostr.str("");
-	ostr << "<light type=\"hemilight\" name=\"hemi_LT\" power=\"" << R.r.GIpower << "\"";
-	switch (R.r.GIquality)
-	{
-		case 1 :
-		case 2 : ostr << " samples=\"16\" >\n";  break;
-		case 3 : ostr << " samples=\"36\" >\n";  break;
-		case 4 : ostr << " samples=\"64\" >\n";  break;
-		case 5 : ostr << " samples=\"128\" >\n";  break;
-		default: ostr << " samples=\"25\" >\n";
+	if (R.r.GIcache) {
+		ostr << "<light type=\"pathlight\" name=\"path_LT\" power=\"" << R.r.GIpower << "\" mode=\"occlusion\"";
+		ostr << "\n\tcache=\"on\" use_QMC=\"on\" threshold=\"" << R.r.GIrefinement << "\" "
+				 << "cache_size=\"" << ((2.0/float(R.r.xsch))*R.r.GIpixelspersample) << "\"";
+		ostr << "\n\tshadow_threshold=\"" << (1.0-R.r.GIshadowquality) << "\" grid=\"82\" search=\"35\"";
+		ostr << "\n\tignore_bumpnormals=\"" << (R.r.YF_nobump ? "on" : "off") << "\"";
+		if (fromAO) {
+			// for AO, with cache, using range of 32*1 to 32*16 seems good enough
+			ostr << "\n\tsamples=\"" << 32*world->aosamp << "\" maxdistance=\"" << world->aodist << "\" >\n";
+		}
+		else {
+			switch (R.r.GIquality)
+			{
+				case 1 : ostr << " samples=\"128\" >\n";  break;
+				case 2 : ostr << " samples=\"256\" >\n";  break;
+				case 3 : ostr << " samples=\"512\" >\n";  break;
+				case 4 : ostr << " samples=\"1024\" >\n"; break;
+				case 5 : ostr << " samples=\"2048\" >\n"; break;
+				default: ostr << " samples=\"256\" >\n";
+			}
+		}
+	}
+	else {
+		ostr << "<light type=\"hemilight\" name=\"hemi_LT\" power=\"" << R.r.GIpower << "\"";
+		if (fromAO) {
+			// use minimum of 4 samples for lowest sample setting, single sample way too noisy
+			ostr << "\n\tsamples=\"" << 3 + world->aosamp*world->aosamp
+					 << "\" maxdistance=\"" << world->aodist
+					 << "\" use_QMC=\"" << ((world->aomode & WO_AORNDSMP) ? "off" : "on") << "\" >\n";
+		}
+		else {
+			switch (R.r.GIquality)
+			{
+				case 1 :
+				case 2 : ostr << " samples=\"16\" >\n";  break;
+				case 3 : ostr << " samples=\"36\" >\n";  break;
+				case 4 : ostr << " samples=\"64\" >\n";  break;
+				case 5 : ostr << " samples=\"128\" >\n";  break;
+				default: ostr << " samples=\"25\" >\n";
+			}
+		}
 	}
 	ostr << "</light>\n\n";
 	xmlfile << ostr.str();
@@ -1600,15 +1823,15 @@ void yafrayFileRender_t::writePathlight()
 	ostr.str("");
 	if (R.r.GIphotons)
 	{
-		ostr << "<light type=\"globalphotonlight\" name=\"gpm\" photons=\""<<R.r.GIphotoncount<<"\""<<endl;
-		ostr << "\tradius=\"" <<R.r.GIphotonradius << "\" depth=\""<< ((R.r.GIdepth>2) ? (R.r.GIdepth-1) : 1)
-				 << "\" caus_depth=\""<<R.r.GIcausdepth<< "\" search=\""<< R.r.GImixphotons<<"\" >"<<endl;
+		ostr << "<light type=\"globalphotonlight\" name=\"gpm\" photons=\"" << R.r.GIphotoncount << "\"" << endl;
+		ostr << "\tradius=\"" << R.r.GIphotonradius << "\" depth=\"" << ((R.r.GIdepth>2) ? (R.r.GIdepth-1) : 1)
+				 << "\" caus_depth=\"" << R.r.GIcausdepth << "\" search=\"" << R.r.GImixphotons << "\" >"<<endl;
 		ostr << "</light>"<<endl;
 	}
 	ostr << "<light type=\"pathlight\" name=\"path_LT\" power=\"" << R.r.GIindirpower << "\"";
 	ostr << " depth=\"" << ((R.r.GIphotons) ? 1 : R.r.GIdepth) << "\" caus_depth=\"" << R.r.GIcausdepth <<"\"\n";
-	if(R.r.GIdirect && R.r.GIphotons) ostr << "direct=\"on\"" << endl;
-	if (R.r.GIcache && ! (R.r.GIdirect && R.r.GIphotons))
+	if (R.r.GIdirect && R.r.GIphotons) ostr << "direct=\"on\"" << endl;
+	if (R.r.GIcache && !(R.r.GIdirect && R.r.GIphotons))
 	{
 		switch (R.r.GIquality)
 		{
@@ -1619,10 +1842,11 @@ void yafrayFileRender_t::writePathlight()
 			case 5 : ostr << " samples=\"2048\" \n";  break;
 			default: ostr << " samples=\"512\" \n";
 		}
-		float sbase = 2.0/float(R.r.xsch);
 		ostr << " cache=\"on\" use_QMC=\"on\" threshold=\"" << R.r.GIrefinement << "\"" << endl;
-		ostr << " cache_size=\"" << sbase*R.r.GIpixelspersample << "\" shadow_threshold=\"" <<
-			1.0 - R.r.GIshadowquality << "\" grid=\"82\" search=\"35\" >\n";
+		ostr << "\tignore_bumpnormals=\"" << (R.r.YF_nobump ? "on" : "off") << "\"\n";
+		float sbase = 2.0/float(R.r.xsch);
+		ostr << "\tcache_size=\"" << sbase*R.r.GIpixelspersample << "\" shadow_threshold=\"" <<
+			1.0-R.r.GIshadowquality << "\" grid=\"82\" search=\"35\" >\n";
 	}
 	else
 	{
@@ -1657,36 +1881,26 @@ bool yafrayFileRender_t::writeWorld()
 		MTex* wtex = world->mtex[i];
 		if (!wtex) continue;
 		Image* wimg = wtex->tex->ima;
-		if ((wtex->tex->type==TEX_IMAGE) && (wimg!=NULL)) {
+		// now always exports if image used as world texture (and 'Hori' mapping enabled)
+		if ((wtex->tex->type==TEX_IMAGE) && (wimg!=NULL) && (wtex->mapto & WOMAP_HORIZ)) {
 			string wt_path = wimg->name;
 			adjustPath(wt_path);
-			if (BLI_testextensie(wimg->name, ".hdr")) {
-				ostr.str("");
-				ostr << "<background type=\"HDRI\" name=\"world_background\" ";
-				// since exposure adjust is an integer, using the texbri slider isn't actually very useful here (result either -1/0/1)
-				// GIpower could be used, but is only active for GI
-				ostr << "exposure_adjust=\"" << int(world->mtex[i]->tex->bright-1) << "\" mapping=\"probe\" >\n";
-				ostr << "\t<filename value=\"" << wt_path << "\" />\n";
-				ostr << "</background>\n\n";
-				xmlfile << ostr.str();
-				return true;
-			}
-			else if (BLI_testextensie(wimg->name, ".jpg") || BLI_testextensie(wimg->name, ".jpeg") || BLI_testextensie(wimg->name, ".tga")) {
-				ostr.str("");
-				ostr << "<background type=\"image\" name=\"world_background\" >\n";
-				/*
-				// not yet in yafray, always assumes spheremap for now, not the same as in Blender,
-				// which for some reason is scaled by 2 in Blender???
-				if (wtex->texco & TEXCO_ANGMAP)
-					ostr << " mapping=\"probe\" >\n";
-				else
-					ostr << " mapping=\"sphere\" >\n";
-				*/
-				ostr << "\t<filename value=\"" << wt_path << "\" />\n";
-				ostr << "</background>\n\n";
-				xmlfile << ostr.str();
-				return true;
-			}
+			ostr.str("");
+			ostr << "<background type=\"image\" name=\"world_background\" ";
+			// exposure_adjust not restricted to integer range anymore
+			ostr << "exposure_adjust=\"" << wtex->tex->bright-1.f << "\"";
+			if (wtex->texco & TEXCO_ANGMAP)
+				ostr << " mapping=\"probe\" >\n";
+			else if (wtex->texco & TEXCO_H_SPHEREMAP)	// in yafray full sphere
+				ostr << " mapping=\"sphere\" >\n";
+			else	// assume 'tube' for anything else
+				ostr << " mapping=\"tube\" >\n";
+			ostr << "\t<filename value=\"" << wt_path << "\" />\n";
+			ostr << "\t<interpolate value=\"" << ((wtex->tex->imaflag & TEX_INTERPOL) ? "bilinear" : "none") << "\" />\n";
+			if (wtex->tex->filtersize>1.f) ostr << "\t<prefilter value=\"on\" />\n";
+			ostr << "</background>\n\n";
+			xmlfile << ostr.str();
+			return true;
 		}
 	}
 
