@@ -1,5 +1,5 @@
 /**
-* $Id: BL_ActionActuator.cpp,v 1.11 2004/11/23 23:04:30 hos Exp $
+* $Id: BL_ActionActuator.cpp,v 1.15 2006/01/16 19:34:52 erwin Exp $
 *
  * ***** BEGIN GPL/BL DUAL LICENSE BLOCK *****
  *
@@ -59,18 +59,18 @@
 BL_ActionActuator::~BL_ActionActuator()
 {
 	if (m_pose) {
-		clear_pose(m_pose);
+		free_pose_channels(m_pose);
 		MEM_freeN(m_pose);
 		m_pose = NULL;
 	};
 	
 	if (m_userpose){
-		clear_pose(m_userpose);
+		free_pose_channels(m_userpose);
 		MEM_freeN(m_userpose);
 		m_userpose=NULL;
 	}
 	if (m_blendpose) {
-		clear_pose(m_blendpose);
+		free_pose_channels(m_blendpose);
 		MEM_freeN(m_blendpose);
 		m_blendpose = NULL;
 	};
@@ -166,18 +166,28 @@ bool BL_ActionActuator::Update(double curtime, bool frame)
 	
 	// result = true if animation has to be continued, false if animation stops
 	// maybe there are events for us in the queue !
-	
-	for (vector<CValue*>::iterator i=m_events.end(); !(i==m_events.begin());)
+	if (frame)
 	{
-		i--;
-		if ((*i)->GetNumber() == 0.0f)
+		for (vector<CValue*>::iterator i=m_events.begin(); !(i==m_events.end());i++)
 		{
-			bNegativeEvent = true;
+			if ((*i)->GetNumber() == 0.0f)
+				bNegativeEvent = true;
+			else
+				bPositiveEvent= true;
+			(*i)->Release();
+		
+		}
+		m_events.clear();
+		
+		if (bPositiveEvent)
+			m_flag |= ACT_FLAG_ACTIVE;
+		
+		if (bNegativeEvent)
+		{
+			if (!(m_flag & ACT_FLAG_ACTIVE))
+				return false;
+			m_flag &= ~ACT_FLAG_ACTIVE;
 		}
-		else
-			bPositiveEvent= true;
-		(*i)->Release();
-		m_events.pop_back();
 	}
 	
 	/*	We know that action actuators have been discarded from all non armature objects:
@@ -236,7 +246,7 @@ bool BL_ActionActuator::Update(double curtime, bool frame)
 			if (!(m_flag & ACT_FLAG_LOCKINPUT)){
 				m_flag &= ~ACT_FLAG_REVERSE;
 				m_flag |= ACT_FLAG_LOCKINPUT;
-				m_starttime = curtime;
+				SetStartTime(curtime);
 			}
 		}
 		else if (bNegativeEvent){
@@ -324,6 +334,7 @@ bool BL_ActionActuator::Update(double curtime, bool frame)
 				m_localtime = m_endframe;
 				m_flag &= ~ACT_FLAG_LOCKINPUT;
 			}
+			SetStartTime(curtime);
 		}
 		break;
 	case ACT_ACTION_PLAY:
@@ -352,12 +363,12 @@ bool BL_ActionActuator::Update(double curtime, bool frame)
 			obj->GetPose(&m_pose);
 			
 			/* Override the necessary channels with ones from the action */
-			get_pose_from_action(&m_pose, m_action, m_localtime);
+			extract_pose_from_action(m_pose, m_action, m_localtime);
 
 			/* Perform the user override (if any) */
 			if (m_userpose){
-				get_pose_from_pose(&m_pose, m_userpose);
-				clear_pose(m_userpose);
+				extract_pose_from_pose(m_pose, m_userpose);
+//				clear_pose(m_userpose);
 				MEM_freeN(m_userpose);
 				m_userpose = NULL;
 			}
@@ -836,10 +847,8 @@ PyObject* BL_ActionActuator::PySetChannel(PyObject* self,
 	{
 
 /*	DO IT HERE */
-		bPoseChannel *pchan;
+		bPoseChannel *pchan= verify_pose_channel(m_userpose, string);
 
-		pchan = (bPoseChannel*) MEM_callocN(sizeof(bPoseChannel), "userChannel");
-		strcpy(pchan->name, string);
 		Mat4ToQuat(matrix, pchan->quat);
 		Mat4ToSize(matrix, pchan->size);
 		VECCOPY (pchan->loc, matrix[3]);
@@ -849,9 +858,6 @@ PyObject* BL_ActionActuator::PySetChannel(PyObject* self,
 		if (!m_userpose){
 			m_userpose = (bPose*)MEM_callocN(sizeof(bPose), "userPose");
 		}
-
-		verify_pose_channel(m_userpose, string);
-		set_pose_channel(m_userpose, pchan);
 	}
 	
 	Py_INCREF(Py_None);
