@@ -1,5 +1,5 @@
 /**
- * $Id: BL_SkinDeformer.cpp,v 1.6 2005/03/22 13:34:30 sirdude Exp $
+ * $Id: BL_SkinDeformer.cpp,v 1.12 2006/01/06 03:46:51 erwin Exp $
  *
  * ***** BEGIN GPL/BL DUAL LICENSE BLOCK *****
  *
@@ -49,6 +49,11 @@
 #include "BKE_action.h"
 #include "MT_Point3.h"
 
+extern "C"{
+	#include "BKE_lattice.h"
+}
+ #include "BKE_utildefines.h"
+
 #include "BLI_blenlib.h"
 #include "BLI_arithb.h"
 
@@ -57,8 +62,12 @@
 
 BL_SkinDeformer::~BL_SkinDeformer()
 {
+	if(m_releaseobject && m_armobj)
+		m_armobj->Release();
 };
 
+/* XXX note, this __NLA_OLDDEFORM define seems to be obsolete */
+
 bool BL_SkinDeformer::Apply(RAS_IPolyMaterial *mat)
 {
 	size_t			i, j, index;
@@ -113,7 +122,8 @@ bool BL_SkinDeformer::Apply(RAS_IPolyMaterial *mat)
 			co[2]=mvert->co[2];
 
 			//	Do the deformation
-			GB_calc_armature_deform(co, dvert);
+/* XXX note, doesnt exist anymore */
+//			GB_calc_armature_deform(co, dvert);
 			tv->SetXYZ(co);
 #else
 			//	Set the data
@@ -142,47 +152,51 @@ void BL_SkinDeformer::ProcessReplica()
 {
 }
 
+//void where_is_pose (Object *ob);
+//void armature_deform_verts(Object *armOb, Object *target, float (*vertexCos)[3], int numVerts, int deformflag); 
 void BL_SkinDeformer::Update(void)
 {
-
 	/* See if the armature has been updated for this frame */
 	if (m_lastUpdate!=m_armobj->GetLastFrame()){	
 		
 		/* Do all of the posing necessary */
-		GB_init_armature_deform (m_defbase, m_premat, m_postmat);
 		m_armobj->ApplyPose();
-		precalc_armature_posemats (m_armobj->GetArmature());
-		for (Bone *curBone=(Bone*)m_armobj->GetArmature()->bonebase.first; curBone; curBone=(Bone*)curBone->next)
-			precalc_bone_defmat(curBone);
 		
+		/* XXX note: where_is_pose() (from BKE_armature.h) calculates all matrices needed to start deforming */
+		/* but it requires the blender object pointer... */
+		//void where_is_pose (Object *ob);
+//		where_is_pose (m_blenderArmatureObj);
+		
+		/* store verts locally */
+//		for (int v =0; v<m_bmesh->totvert; v++){
+			/* XXX note, dunno about this line */
+//			m_transverts[v]=MT_Point3(m_bmesh->mvert[v].co);
+//		}
+		
+//		float	test[1000][3];
+
+//		armature_deform_verts(m_blenderArmatureObj,m_blenderMeshObject,test,m_bmesh->totvert,ARM_DEF_VGROUP);
+
+
+		Object* par_arma = m_armobj->GetArmatureObject();
+		where_is_pose( par_arma ); 
+		/* store verts locally */
 		VerifyStorage();
+		for (int v =0; v<m_bmesh->totvert; v++)
+			VECCOPY(m_transverts[v], m_bmesh->mvert[v].co);
 
-		/* Transform the verts & store locally */
-		for (int v =0; v<m_bmesh->totvert; v++){
-			float co[3];
-
-		  	co[0]=m_bmesh->mvert[v].co[0];
-			co[1]=m_bmesh->mvert[v].co[1];
-			co[2]=m_bmesh->mvert[v].co[2];
-			GB_calc_armature_deform(co, &m_bmesh->dvert[v]);
-
-			m_transverts[v]=MT_Point3(co);
-		}
-		
+		armature_deform_verts( par_arma, m_objMesh,m_transverts, m_bmesh->totvert, ARM_DEF_VGROUP );
 		RecalcNormals();
-		
 
 		/* Update the current frame */
 		m_lastUpdate=m_armobj->GetLastFrame();
 	}
 }
 
+/* XXX note: I propose to drop this function */
 void BL_SkinDeformer::SetArmature(BL_ArmatureObject *armobj)
 {
+	// --
+	// only used to set the object now
 	m_armobj = armobj;
-
-	for (bDeformGroup *dg=(bDeformGroup*)m_defbase->first; dg; dg=(bDeformGroup*)dg->next)
-			dg->data = (void*)get_named_bone(m_armobj->GetArmature(), dg->name);
-		
-		GB_validate_defgroups(m_bmesh, m_defbase);
 }
