@@ -1,5 +1,5 @@
 /**
- * $Id: KX_ConvertSensors.cpp,v 1.7 2005/03/25 10:33:35 kester Exp $
+ * $Id: KX_ConvertSensors.cpp,v 1.9 2005/08/22 15:47:56 erwin Exp $
  *
  * ***** BEGIN GPL/BL DUAL LICENSE BLOCK *****
  *
@@ -68,6 +68,7 @@ probably misplaced */
 #include "SCA_JoystickSensor.h"
 #include "KX_NetworkMessageSensor.h"
 
+
 #include "SCA_PropertySensor.h"
 #include "SCA_RandomSensor.h"
 #include "KX_RaySensor.h"
@@ -422,7 +423,7 @@ void BL_ConvertSensors(struct Object* blenderobject,
 		case SENS_MOUSE:
 			{
 				int keytype = SCA_MouseSensor::KX_MOUSESENSORMODE_NODEF;			
-				bool trackfocus = false;
+				int trackfocus = 0;
 				bMouseSensor *bmouse = (bMouseSensor *)sens->data;
 				
 				/* There are two main types of mouse sensors. If there is
@@ -455,8 +456,12 @@ void BL_ConvertSensors(struct Object* blenderobject,
 						keytype = SCA_MouseSensor::KX_MOUSESENSORMODE_MOVEMENT;
 						break;
 					case BL_SENS_MOUSE_MOUSEOVER:
-						trackfocus = true;
+						trackfocus = 1;
 						break;
+					case BL_SENS_MOUSE_MOUSEOVER_ANY:
+						trackfocus = 2;
+						break;
+
 					default:
 						; /* error */
 					}
