@@ -1,5 +1,5 @@
 /**
- * $Id: PyObjectPlus.cpp,v 1.8 2005/03/25 10:33:36 kester Exp $
+ * $Id: PyObjectPlus.cpp,v 1.9 2005/12/20 09:13:06 erwin Exp $
  * ***** BEGIN GPL/BL DUAL LICENSE BLOCK *****
  *
  * This program is free software; you can redistribute it and/or
@@ -77,6 +77,12 @@ PyTypeObject PyObjectPlus::Type = {
 	0,				/*tp_call */
 };
 
+PyObjectPlus::~PyObjectPlus()
+{
+	_Py_ForgetReference(this);
+//	assert(ob_refcnt==0);
+}
+
 PyObjectPlus::PyObjectPlus(PyTypeObject *T) 				// constructor
 {
 	MT_assert(T != NULL);
