@@ -1,5 +1,5 @@
 /**
- * $Id: GPG_Application.cpp,v 1.14 2005/03/25 10:33:36 kester Exp $
+ * $Id: GPG_Application.cpp,v 1.17 2006/01/16 22:27:29 lukep Exp $
  *
  * ***** BEGIN GPL/BL DUAL LICENSE BLOCK *****
  *
@@ -41,6 +41,7 @@
 #endif
 
 #ifdef __APPLE__
+#define GL_GLEXT_LEGACY 1
 #include <OpenGL/gl.h>
 #include <OpenGL/glu.h>
 #else
@@ -62,6 +63,8 @@ extern "C"
 #endif  // __cplusplus
 #include "BLI_blenlib.h"
 #include "BLO_readfile.h"
+#include "BKE_global.h"
+#include "BKE_main.h"
 #ifdef __cplusplus
 }
 #endif // __cplusplus
@@ -82,6 +85,7 @@ extern "C"
 #include "RAS_GLExtensionManager.h"
 #include "KX_PythonInit.h"
 #include "KX_PyConstraintBinding.h"
+#include "BL_Material.h" // MAXTEX
 
 #include "KX_BlenderSceneConverter.h"
 #include "NG_LoopBackNetworkDeviceInterface.h"
@@ -107,9 +111,9 @@ static void frameTimerProc(GHOST_ITimerTask* task, GHOST_TUns64 time);
 static GHOST_ISystem* fSystem = 0;
 static const int kTimerFreq = 10;
 
-GPG_Application::GPG_Application(GHOST_ISystem* system, struct Main *maggie, STR_String startSceneName)
+GPG_Application::GPG_Application(GHOST_ISystem* system, struct Main* maggie, STR_String startSceneName)
 	: m_startSceneName(startSceneName), 
-	  m_maggie(maggie), 
+	  m_maggie(maggie),
 	  m_exitRequested(0),
 	  m_system(system), 
 	  m_mainWindow(0), 
@@ -126,7 +130,8 @@ GPG_Application::GPG_Application(GHOST_ISystem* system, struct Main *maggie, STR
 	  m_rasterizer(0), 
 	  m_sceneconverter(0),
 	  m_networkdevice(0), 
-	  m_audiodevice(0)
+	  m_audiodevice(0),
+	  m_blendermat(0)
 {
 	fSystem = system;
 }
@@ -141,12 +146,13 @@ GPG_Application::~GPG_Application(void)
 
 
 
-bool GPG_Application::SetGameEngineData(struct Main *maggie, STR_String startSceneName)
+bool GPG_Application::SetGameEngineData(struct Main* maggie, STR_String startSceneName)
 {
 	bool result = false;
 
 	if (maggie != NULL && startSceneName != "")
 	{
+		G.scene = (Scene*)maggie->scene.first;
 		m_maggie = maggie;
 		m_startSceneName = startSceneName;
 		result = true;
@@ -488,6 +494,23 @@ bool GPG_Application::initEngine(GHOST_IWindow* window, const int stereoMode)
 		bool profile = (SYS_GetCommandLineInt(syshandle, "show_profile", 0) != 0);
 		bool frameRate = (SYS_GetCommandLineInt(syshandle, "show_framerate", 0) != 0);
 		bool useVertexArrays = SYS_GetCommandLineInt(syshandle,"vertexarrays",1) != 0;
+		
+#ifdef GL_ARB_multitexture
+		// ----------------------------------
+		if(bgl::RAS_EXT_support._ARB_multitexture && bgl::QueryVersion(1, 1)) {
+			m_blendermat = (SYS_GetCommandLineInt(syshandle, "blender_material", 0) != 0);
+			int unitmax=0;
+			glGetIntegerv(GL_MAX_TEXTURE_UNITS_ARB, (GLint*)&unitmax);
+			bgl::max_texture_units = MAXTEX>unitmax?unitmax:MAXTEX;
+			//std::cout << "using(" << bgl::max_texture_units << ") of(" << unitmax << ") texture units." << std::endl;
+		} else {
+			bgl::max_texture_units = 0;
+		}
+#else
+		m_blendermat=0;
+#endif//GL_ARB_multitexture
+		// ----------------------------------
+	
 		// create the canvas, rasterizer and rendertools
 		m_canvas = new GPG_Canvas(window);
 		if (!m_canvas)
@@ -598,7 +621,7 @@ bool GPG_Application::startEngine(void)
 	*/
 	
 	// create a scene converter, create and convert the stratingscene
-	m_sceneconverter = new KX_BlenderSceneConverter(m_maggie, m_ketsjiengine);
+	m_sceneconverter = new KX_BlenderSceneConverter(m_maggie,0, m_ketsjiengine);
 	if (m_sceneconverter)
 	{
 		STR_String startscenename = m_startSceneName.Ptr();
@@ -606,7 +629,8 @@ bool GPG_Application::startEngine(void)
 		
 		//	if (always_use_expand_framing)
 		//		sceneconverter->SetAlwaysUseExpandFraming(true);
-		
+		if(m_blendermat)
+			m_sceneconverter->SetMaterials(true);
 
 		KX_Scene* startscene = new KX_Scene(m_keyboard,
 			m_mouse,
