@@ -1,5 +1,5 @@
 /**
- * $Id: KX_ConvertPhysicsObjects.cpp,v 1.14 2004/11/22 11:49:35 kester Exp $
+ * $Id: KX_ConvertPhysicsObjects.cpp,v 1.31 2006/01/05 23:10:54 sirdude Exp $
  *
  * ***** BEGIN GPL/BL DUAL LICENSE BLOCK *****
  *
@@ -33,6 +33,8 @@
 #pragma warning (disable : 4786)
 #endif
 
+#include "MT_assert.h"
+
 // defines USE_ODE to choose physics engine
 #include "KX_ConvertPhysicsObject.h"
 #include "KX_GameObject.h"
@@ -51,10 +53,6 @@
 
 #include "KX_MotionState.h" // bridge between motionstate and scenegraph node
 
-#ifdef HAVE_CONFIG_H
-#include <config.h>
-#endif
-
 #ifdef USE_ODE
 
 #include "KX_OdePhysicsController.h"
@@ -65,6 +63,7 @@
 // USE_SUMO_SOLID is defined in headerfile KX_ConvertPhysicsObject.h
 #ifdef USE_SUMO_SOLID
 
+
 #include "SumoPhysicsEnvironment.h"
 #include "KX_SumoPhysicsController.h"
 
@@ -262,7 +261,7 @@ void	KX_ConvertSumoObject(	KX_GameObject* gameobj,
 							dynamicParent = sumoctrl->GetSumoObject();
 						}
 
-						assert(dynamicParent);
+						MT_assert(dynamicParent);
 					}
 				
 					
@@ -306,12 +305,13 @@ static void	BL_RegisterSumoObject(
 
 		// need easy access, not via 'node' etc.
 		KX_SumoPhysicsController* physicscontroller = new KX_SumoPhysicsController(sumoScene,sumoObj,motionstate,isDynamic);
-		gameobj->SetPhysicsController(physicscontroller);
-		physicscontroller->setClientInfo(gameobj);
+		gameobj->SetPhysicsController(physicscontroller,isDynamic);
+
 		
 		if (!gameobj->getClientInfo())
 			std::cout << "BL_RegisterSumoObject: WARNING: Object " << gameobj->GetName() << " has no client info" << std::endl;
-		sumoObj->setClientObject(gameobj->getClientInfo());
+		physicscontroller->setNewClientInfo(gameobj->getClientInfo());
+		
 
 		gameobj->GetSGNode()->AddSGController(physicscontroller);
 
@@ -578,6 +578,10 @@ void	KX_ClearSumoSharedShapes()
 	map_gamemesh_to_instance.clear();
 }
 
+
+
+
+
 #endif //USE_SUMO_SOLID
 
 
@@ -590,6 +594,7 @@ void	KX_ConvertODEEngineObject(KX_GameObject* gameobj,
 							struct	PHY_MaterialProps*	smmaterial,
 							struct	KX_ObjectProperties*	objprop)
 {
+	
 	// not yet, future extension :)
 	bool dyna=objprop->m_dyna;
 	bool fullRigidBody= ( objprop->m_dyna && objprop->m_angular_rigidbody) != 0;
@@ -602,49 +607,465 @@ void	KX_ConvertODEEngineObject(KX_GameObject* gameobj,
 	dxSpace* space = odeEnv->GetOdeSpace();
 	dxWorld* world = odeEnv->GetOdeWorld();
 
-	if (!objprop->m_implicitsphere &&
-		MT_fuzzyZero(objprop->m_boundingbox.m_extends[0]) ||
-		MT_fuzzyZero(objprop->m_boundingbox.m_extends[1]) ||
-		MT_fuzzyZero(objprop->m_boundingbox.m_extends[2])
-		)
+	bool isSphere = false;
+
+	switch (objprop->m_boundclass)
 	{
+	case KX_BOUNDBOX:
+		{
 
-	} else
-	{
+				KX_OdePhysicsController* physicscontroller = 
+					new KX_OdePhysicsController(
+					dyna,
+					fullRigidBody,
+					phantom,
+					motionstate,
+					space,
+					world,
+					shapeprops->m_mass,
+					smmaterial->m_friction,
+					smmaterial->m_restitution,
+					isSphere,
+					objprop->m_boundobject.box.m_center,
+					objprop->m_boundobject.box.m_extends,
+					objprop->m_boundobject.c.m_radius
+					);
 
-		KX_OdePhysicsController* physicscontroller = 
-			new KX_OdePhysicsController(
-			dyna,
-			fullRigidBody,
-			phantom,
-			motionstate,
-			space,
-			world,
-			shapeprops->m_mass,
-			smmaterial->m_friction,
-			smmaterial->m_restitution,
-			objprop->m_implicitsphere,
-			objprop->m_boundingbox.m_center,
-			objprop->m_boundingbox.m_extends,
-			objprop->m_radius
-			);
+				gameobj->SetPhysicsController(physicscontroller);
+				physicscontroller->setNewClientInfo(gameobj->getClientInfo());						
+				gameobj->GetSGNode()->AddSGController(physicscontroller);
 
-		gameobj->SetPhysicsController(physicscontroller);
-		physicscontroller->setClientInfo(gameobj);						
-		gameobj->GetSGNode()->AddSGController(physicscontroller);
+				bool isActor = objprop->m_isactor;
+				STR_String materialname;
+				if (meshobj)
+					materialname = meshobj->GetMaterialName(0);
 
-		bool isActor = objprop->m_isactor;
-		STR_String materialname;
-		if (meshobj)
-			materialname = meshobj->GetMaterialName(0);
-
-		const char* matname = materialname.ReadPtr();
+				const char* matname = materialname.ReadPtr();
 
 
-		physicscontroller->SetObject(gameobj->GetSGNode());
-				
-	}
+				physicscontroller->SetObject(gameobj->GetSGNode());
+
+				break;
+			}
+	default:
+		{
+		}
+	};
+
 }
 
 
 #endif // USE_ODE
+
+
+#ifdef USE_BULLET
+
+#include "CcdPhysicsEnvironment.h"
+#include "CcdPhysicsController.h"
+
+#include "KX_BulletPhysicsController.h"
+#include "CollisionShapes/BoxShape.h"
+#include "CollisionShapes/SphereShape.h"
+#include "CollisionShapes/ConeShape.h"
+#include "CollisionShapes/ConvexShape.h"
+#include "CollisionShapes/CylinderShape.h"
+#include "CollisionShapes/MultiSphereShape.h"
+#include "CollisionShapes/ConvexHullShape.h"
+#include "CollisionShapes/TriangleMesh.h"
+#include "CollisionShapes/TriangleMeshShape.h"
+#include "CollisionShapes/BvhTriangleMeshShape.h"
+
+
+static GEN_Map<GEN_HashedPtr,CollisionShape*>	map_gamemesh_to_bulletshape;
+
+// forward declarations
+static CollisionShape* CreateBulletShapeFromMesh(RAS_MeshObject* meshobj, bool polytope)
+{
+	if (!meshobj)
+		return 0;
+
+	CollisionShape* collisionMeshShape = 0;
+	ConvexHullShape* convexHullShape = 0;
+	TriangleMeshShape* concaveShape = 0;
+
+	TriangleMesh* collisionMeshData = 0;
+
+	//see if there is any polygons, if not, bail out.
+
+	int numUsedPolygons = 0;
+	int numPoints = 0;
+	SimdVector3* points = 0;
+
+	CollisionShape** shapeptr = map_gamemesh_to_bulletshape[GEN_HashedPtr(meshobj)];
+
+	// Mesh has already been converted: reuse
+	if (shapeptr)
+	{
+		//return *shapeptr;
+	}
+
+	// Mesh has no polygons!
+	int numpolys = meshobj->NumPolygons();
+	if (!numpolys)
+	{
+		return NULL;
+	}
+
+	// Count the number of collision polygons and check they all come from the same 
+	// vertex array
+	int numvalidpolys = 0;
+	int vtxarray = -1;
+	RAS_IPolyMaterial *poly_material = NULL;
+	bool reinstance = true;
+
+	for (int p=0; p<numpolys; p++)
+	{
+		RAS_Polygon* poly = meshobj->GetPolygon(p);
+
+		// only add polygons that have the collisionflag set
+		if (poly->IsCollider())
+		{
+			// check polygon is from the same vertex array
+			if (poly->GetVertexIndexBase().m_vtxarray != vtxarray)
+			{
+				if (vtxarray < 0)
+					vtxarray = poly->GetVertexIndexBase().m_vtxarray;
+				else
+				{
+					reinstance = false;
+					vtxarray = -1;
+				}
+			}
+
+			// check poly is from the same material
+			if (poly->GetMaterial()->GetPolyMaterial() != poly_material)
+			{
+				if (poly_material)
+				{
+					reinstance = false;
+					poly_material = NULL;
+				}
+				else
+					poly_material = poly->GetMaterial()->GetPolyMaterial();
+			}
+
+			// count the number of collision polys
+			numvalidpolys++;
+
+			// We have one collision poly, and we can't reinstance, so we
+			// might as well break here.
+			if (!reinstance)
+				break;
+		}
+	}
+
+	// No collision polygons
+	if (numvalidpolys < 1)
+		return NULL;
+
+
+	if (polytope)
+	{
+		convexHullShape = new ConvexHullShape(points,numPoints);
+		collisionMeshShape = convexHullShape;
+	} else
+	{
+		collisionMeshData = new TriangleMesh();
+//		concaveShape = new TriangleMeshShape(collisionMeshData);
+		//collisionMeshShape = concaveShape;
+
+	}
+
+
+	numvalidpolys = 0;
+
+	for (int p2=0; p2<numpolys; p2++)
+	{
+		RAS_Polygon* poly = meshobj->GetPolygon(p2);
+
+		// only add polygons that have the collisionflag set
+		if (poly->IsCollider())
+		{   
+			//Bullet can raycast any shape, so
+			if (polytope)
+			{
+				for (int i=0;i<poly->VertexCount();i++)
+				{
+					const float* vtx = meshobj->GetVertex(poly->GetVertexIndexBase().m_vtxarray, 
+						poly->GetVertexIndexBase().m_indexarray[i],
+						poly->GetMaterial()->GetPolyMaterial())->getLocalXYZ();
+					SimdPoint3 point(vtx[0],vtx[1],vtx[2]);
+					convexHullShape->AddPoint(point);
+				}
+				if (poly->VertexCount())
+					numvalidpolys++;
+
+			} else
+			{
+				{
+					const float* vtx = meshobj->GetVertex(poly->GetVertexIndexBase().m_vtxarray, 
+						poly->GetVertexIndexBase().m_indexarray[2],
+						poly->GetMaterial()->GetPolyMaterial())->getLocalXYZ();
+					SimdPoint3 vertex0(vtx[0],vtx[1],vtx[2]);
+					vtx = meshobj->GetVertex(poly->GetVertexIndexBase().m_vtxarray, 
+						poly->GetVertexIndexBase().m_indexarray[1],
+						poly->GetMaterial()->GetPolyMaterial())->getLocalXYZ();
+					SimdPoint3 vertex1(vtx[0],vtx[1],vtx[2]);
+					vtx = meshobj->GetVertex(poly->GetVertexIndexBase().m_vtxarray, 
+						poly->GetVertexIndexBase().m_indexarray[0],
+						poly->GetMaterial()->GetPolyMaterial())->getLocalXYZ();
+					SimdPoint3 vertex2(vtx[0],vtx[1],vtx[2]);
+					collisionMeshData->AddTriangle(vertex0,vertex1,vertex2);
+					numvalidpolys++;
+				}
+				if (poly->VertexCount() == 4)
+				{
+					const float* vtx = meshobj->GetVertex(poly->GetVertexIndexBase().m_vtxarray, 
+						poly->GetVertexIndexBase().m_indexarray[3],
+						poly->GetMaterial()->GetPolyMaterial())->getLocalXYZ();
+					SimdPoint3 vertex0(vtx[0],vtx[1],vtx[2]);
+					vtx = meshobj->GetVertex(poly->GetVertexIndexBase().m_vtxarray, 
+						poly->GetVertexIndexBase().m_indexarray[2],
+						poly->GetMaterial()->GetPolyMaterial())->getLocalXYZ();
+					SimdPoint3 vertex1(vtx[0],vtx[1],vtx[2]);
+					vtx = meshobj->GetVertex(poly->GetVertexIndexBase().m_vtxarray, 
+						poly->GetVertexIndexBase().m_indexarray[0],
+						poly->GetMaterial()->GetPolyMaterial())->getLocalXYZ();
+					SimdPoint3 vertex2(vtx[0],vtx[1],vtx[2]);
+					collisionMeshData->AddTriangle(vertex0,vertex1,vertex2);
+					numvalidpolys++;
+				}
+
+			}		
+		}
+	}
+
+
+
+	if (numvalidpolys > 0)
+	{
+		
+		//map_gamemesh_to_bulletshape.insert(GEN_HashedPtr(meshobj),collisionMeshShape);
+		if (!polytope)
+		{
+			concaveShape = new BvhTriangleMeshShape( collisionMeshData );
+			//concaveShape = new TriangleMeshShape( collisionMeshData );
+
+			concaveShape->RecalcLocalAabb();
+			collisionMeshShape = concaveShape;
+
+		} 
+		
+		
+
+		return collisionMeshShape;
+	}
+
+	delete collisionMeshShape;
+	return NULL;
+
+}
+
+
+
+void	KX_ConvertBulletObject(	class	KX_GameObject* gameobj,
+	class	RAS_MeshObject* meshobj,
+	class	KX_Scene* kxscene,
+	struct	PHY_ShapeProps* shapeprops,
+	struct	PHY_MaterialProps*	smmaterial,
+	struct	KX_ObjectProperties*	objprop)
+{
+
+		CcdPhysicsEnvironment* env = (CcdPhysicsEnvironment*)kxscene->GetPhysicsEnvironment();
+	assert(env);
+	
+
+	bool isbulletdyna = false;
+	CcdConstructionInfo ci;
+	class PHY_IMotionState* motionstate = new KX_MotionState(gameobj->GetSGNode());
+
+	ci.m_MotionState = motionstate;
+	ci.m_gravity = SimdVector3(0,0,0);
+	ci.m_localInertiaTensor =SimdVector3(0,0,0);
+	ci.m_mass = objprop->m_dyna ? shapeprops->m_mass : 0.f;
+	isbulletdyna = objprop->m_dyna;
+
+	ci.m_localInertiaTensor = SimdVector3(ci.m_mass/3.f,ci.m_mass/3.f,ci.m_mass/3.f);
+	
+	SimdTransform trans;
+	trans.setIdentity();
+	
+	CollisionShape* bm = 0;
+
+	switch (objprop->m_boundclass)
+	{
+	case KX_BOUNDSPHERE:
+		{
+			float radius = objprop->m_radius;
+			SimdVector3 inertiaHalfExtents (
+				radius,
+				radius,
+				radius);
+			
+			//blender doesn't support multisphere, but for testing:
+
+			//bm = new MultiSphereShape(inertiaHalfExtents,,&trans.getOrigin(),&radius,1);
+			bm = new SphereShape(objprop->m_radius);
+			bm->CalculateLocalInertia(ci.m_mass,ci.m_localInertiaTensor);
+			break;
+		};
+	case KX_BOUNDBOX:
+		{
+			MT_Vector3 halfExtents (
+				objprop->m_boundobject.box.m_extends[0],
+			objprop->m_boundobject.box.m_extends[1],
+			objprop->m_boundobject.box.m_extends[2]);
+
+			halfExtents /= 2.f;
+
+			//SimdVector3 he (halfExtents[0]-CONVEX_DISTANCE_MARGIN ,halfExtents[1]-CONVEX_DISTANCE_MARGIN ,halfExtents[2]-CONVEX_DISTANCE_MARGIN );
+			//he = he.absolute();
+
+			SimdVector3 he (halfExtents[0],halfExtents[1],halfExtents[2]);
+			he = he.absolute();
+
+
+			bm = new BoxShape(he);
+			bm->CalculateLocalInertia(ci.m_mass,ci.m_localInertiaTensor);
+			break;
+		};
+	case KX_BOUNDCYLINDER:
+		{
+			SimdVector3 halfExtents (
+				objprop->m_boundobject.c.m_radius,
+				objprop->m_boundobject.c.m_radius,
+				objprop->m_boundobject.c.m_height * 0.5f
+			);
+			bm = new CylinderShapeZ(halfExtents);
+			bm->CalculateLocalInertia(ci.m_mass,ci.m_localInertiaTensor);
+
+			break;
+		}
+
+		case KX_BOUNDCONE:
+		{
+				SimdVector3 halfExtents (objprop->m_boundobject.box.m_extends[0],
+				objprop->m_boundobject.box.m_extends[1],
+				objprop->m_boundobject.box.m_extends[2]);
+
+
+				halfExtents /= 2.f;
+
+				bm = new ConeShape(objprop->m_boundobject.c.m_radius,objprop->m_boundobject.c.m_height);
+				bm->CalculateLocalInertia(ci.m_mass,ci.m_localInertiaTensor);
+
+			break;
+		}
+		case KX_BOUNDPOLYTOPE:
+			{
+				bm = CreateBulletShapeFromMesh(meshobj,true);
+				if (bm)
+				{
+					bm->CalculateLocalInertia(ci.m_mass,ci.m_localInertiaTensor);
+				}
+				break;
+			}
+		case KX_BOUNDMESH:
+			{
+				if (!ci.m_mass)
+				{				
+					bm = CreateBulletShapeFromMesh(meshobj,false);
+					ci.m_localInertiaTensor.setValue(0.f,0.f,0.f);
+					//no moving concave meshes, so don't bother calculating inertia
+					//bm->CalculateLocalInertia(ci.m_mass,ci.m_localInertiaTensor);
+				}
+
+				break;
+			}
+
+	default:
+		//interpret the shape as a concave triangle-mesh
+		{
+			if (meshobj)
+			{
+			//	assert(0);
+
+					/*
+				meshobj->ScheduleCollisionPolygons();
+
+				KX_DeformableMesh* gfxmesh = new KX_DeformableMesh(meshobj);
+				gfxmesh->sendFixedMapping();
+				//trianglemesh
+				bm = new TriangleMeshInterface(gfxmesh,trans);
+				*/
+			}
+		}
+	}
+
+
+//	ci.m_localInertiaTensor.setValue(0.1f,0.1f,0.1f);
+
+	if (!bm)
+		return;
+
+	bm->SetMargin(0.06);
+
+	ci.m_collisionShape = bm;
+	ci.m_broadphaseHandle = 0;
+	ci.m_friction = 5.f* smmaterial->m_friction;//tweak the friction a bit, so the default 0.5 works nice
+	ci.m_restitution = smmaterial->m_restitution;
+	ci.m_physicsEnv = env;
+	// drag / damping is inverted
+	ci.m_linearDamping = 1.f - shapeprops->m_lin_drag;
+	ci.m_angularDamping = 1.f - shapeprops->m_ang_drag;
+	//need a bit of damping, else system doesn't behave well
+	ci.m_inertiaFactor = shapeprops->m_inertia/0.4f;//defaults to 0.4, don't want to change behaviour
+	
+	KX_BulletPhysicsController* physicscontroller = new KX_BulletPhysicsController(ci,isbulletdyna);
+
+	if (objprop->m_in_active_layer)
+	{
+		env->addCcdPhysicsController( physicscontroller);
+	}
+
+	
+	gameobj->SetPhysicsController(physicscontroller,isbulletdyna);
+	physicscontroller->setNewClientInfo(gameobj->getClientInfo());		
+	bool isActor = objprop->m_isactor;
+	gameobj->getClientInfo()->m_type = (isActor ? KX_ClientObjectInfo::ACTOR : KX_ClientObjectInfo::STATIC);
+	// store materialname in auxinfo, needed for touchsensors
+	//gameobj->getClientInfo()->m_auxilary_info = 0;//(matname.Length() ? (void*)(matname.ReadPtr()+2) : NULL);
+	//gameobj->getClientInfo()->m_auxilary_info = (matname.Length() ? (void*)(matname.ReadPtr()+2) : NULL);
+
+
+	gameobj->GetSGNode()->AddSGController(physicscontroller);
+
+	STR_String materialname;
+	if (meshobj)
+		materialname = meshobj->GetMaterialName(0);
+
+	const char* matname = materialname.ReadPtr();
+
+
+	physicscontroller->SetObject(gameobj->GetSGNode());
+				
+
+}
+	
+void	KX_ClearBulletSharedShapes()
+{
+	int numshapes = map_gamemesh_to_bulletshape.size();
+	int i;
+	CollisionShape*shape=0;
+	for (i=0;i<numshapes ;i++)
+	{
+		shape = *map_gamemesh_to_bulletshape.at(i);
+		//delete shape;
+	}
+	
+	map_gamemesh_to_bulletshape.clear();
+	
+}
+
+#endif
+
