@@ -1,5 +1,5 @@
 /**
- * $Id: KX_GameObject.cpp,v 1.11 2004/11/06 04:58:09 kester Exp $
+ * $Id: KX_GameObject.cpp,v 1.19 2006/01/06 03:46:52 erwin Exp $
  *
  * ***** BEGIN GPL/BL DUAL LICENSE BLOCK *****
  *
@@ -35,6 +35,12 @@
 #include <config.h>
 #endif
 
+#if defined(_WIN64)
+typedef unsigned __int64 uint_ptr;
+#else
+typedef unsigned long uint_ptr;
+#endif
+
 #ifdef WIN32
 // This warning tells us about truncation of __long__ stl-generated names.
 // It can occasionally cause DevStudio to have internal compiler warnings.
@@ -44,6 +50,7 @@
 
 #define KX_INERTIA_INFINITE 10000
 #include "RAS_IPolygonMaterial.h"
+#include "KX_BlenderMaterial.h"
 #include "KX_GameObject.h"
 #include "RAS_MeshObject.h"
 #include "KX_MeshProxy.h"
@@ -72,7 +79,8 @@ KX_GameObject::KX_GameObject(
 	m_bSuspendDynamics(false),
 	m_bUseObjectColor(false),
 	m_bVisible(true),
-	m_pPhysicsController1(NULL)
+	m_pPhysicsController1(NULL),
+	m_isDeformable(false)
 {
 	m_ignore_activity_culling = false;
 	m_pClient_info = new KX_ClientObjectInfo(this, KX_ClientObjectInfo::ACTOR);
@@ -360,31 +368,34 @@ void KX_GameObject::UpdateIPO(float curframetime,
 	UpdateTransform();
 }
 
+// IPO update
+void 
+KX_GameObject::UpdateMaterialData(
+		MT_Vector4 rgba,
+		MT_Vector3 specrgb,
+		MT_Scalar hard,
+		MT_Scalar spec,
+		MT_Scalar ref,
+		MT_Scalar emit,
+		MT_Scalar alpha
 
-/*
-void KX_GameObject::RegisterSumoObject(class SM_Scene* sumoScene,
-									   DT_SceneHandle solidscene,
-									   class SM_Object* sumoObj,
-									   const char* matname,
-									   bool isDynamic,
-									   bool isActor)
+	)
 {
-	m_bDyna = isDynamic;
-
-	// need easy access, not via 'node' etc.
-	m_pPhysicsController = new KX_PhysicsController(sumoScene,solidscene,sumoObj,isDynamic);
-				
-	GetSGNode()->AddSGController(m_pPhysicsController);
-	
-	m_pClient_info->m_type = (isActor ? 1 : 0);
-	m_pClient_info->m_clientobject = this;
-
-	// store materialname in auxinfo, needed for touchsensors
-	m_pClient_info->m_auxilary_info = (matname? (void*)(matname+2) : NULL);
-	m_pPhysicsController->SetObject(this->GetSGNode());
+	int mesh = 0;
+	if (((unsigned int)mesh < m_meshes.size()) && mesh >= 0) {
+		RAS_MaterialBucket::Set::iterator mit = m_meshes[mesh]->GetFirstMaterial();
+		for(; mit != m_meshes[mesh]->GetLastMaterial(); ++mit)
+		{
+			RAS_IPolyMaterial* poly = (*mit)->GetPolyMaterial();
+			if(poly->GetFlag() & RAS_BLENDERMAT )
+			{
+				SetObjectColor(rgba);
+				KX_BlenderMaterial *m =  static_cast<KX_BlenderMaterial*>(poly);
+				m->UpdateIPO(rgba, specrgb,hard,spec,ref,emit, alpha);
+			}
+		}
+	}
 }
-*/
-
 bool
 KX_GameObject::GetVisible(
 	void
@@ -468,10 +479,11 @@ void KX_GameObject::ResolveCombinedVelocities(
 ){
 	if (m_pPhysicsController1)
 	{
+
+		MT_Vector3 lv = lin_vel_local ? NodeGetWorldOrientation() * lin_vel : lin_vel;
+		MT_Vector3 av = ang_vel_local ? NodeGetWorldOrientation() * ang_vel : ang_vel;
 		m_pPhysicsController1->resolveCombinedVelocities(
-			lin_vel_local ? NodeGetWorldOrientation() * lin_vel : lin_vel,
-			ang_vel_local ? NodeGetWorldOrientation() * ang_vel : ang_vel			
-		);		
+			lv.x(),lv.y(),lv.z(),av.x(),av.y(),av.z());
 	}
 }
 
@@ -621,8 +633,7 @@ PyMethodDef KX_GameObject::Methods[] = {
 	{"getParent", (PyCFunction)KX_GameObject::sPyGetParent,METH_VARARGS},
 	{"getMesh", (PyCFunction)KX_GameObject::sPyGetMesh,METH_VARARGS},
 	{"getPhysicsId", (PyCFunction)KX_GameObject::sPyGetPhysicsId,METH_VARARGS},
-	
-	
+	KX_PYMETHODTABLE(KX_GameObject, getDistanceTo),
 	{NULL,NULL} //Sentinel
 };
 
@@ -954,10 +965,10 @@ PyObject* KX_GameObject::PyGetMesh(PyObject* self,
 								   PyObject* kwds)
 {
 	int mesh = 0;
-	
+
 	if (PyArg_ParseTuple(args, "|i", &mesh))
 	{
-		if (mesh < m_meshes.size() && mesh >= 0)
+		if (((unsigned int)mesh < m_meshes.size()) && mesh >= 0)
 		{
 			KX_MeshProxy* meshproxy = new KX_MeshProxy(m_meshes[mesh]);
 			return meshproxy;
@@ -967,25 +978,23 @@ PyObject* KX_GameObject::PyGetMesh(PyObject* self,
 }
 
 
-
 PyObject* KX_GameObject::PyApplyImpulse(PyObject* self, 
 										PyObject* args, 
 										PyObject* kwds)
 {
-	
-	
 	PyObject* pyattach;
 	PyObject* pyimpulse;
-	
 	if (PyArg_ParseTuple(args, "OO", &pyattach, &pyimpulse))
 	{
 		MT_Point3  attach;
 		MT_Vector3 impulse;
-	
-		if (PyVecTo(pyattach, attach) && PyVecTo(pyimpulse, impulse) && m_pPhysicsController1)
+		if (m_pPhysicsController1)
 		{
-			m_pPhysicsController1->applyImpulse(attach, impulse);
-			Py_Return;
+			if (PyVecTo(pyattach, attach) && PyVecTo(pyimpulse, impulse))
+			{
+				m_pPhysicsController1->applyImpulse(attach, impulse);
+				Py_Return;
+			}
 		}
 
 	}
@@ -1092,14 +1101,35 @@ PyObject* KX_GameObject::PyGetPhysicsId(PyObject* self,
 											   PyObject* kwds)
 {
 	KX_IPhysicsController* ctrl = GetPhysicsController();
-	int physid=0;
+	uint_ptr physid=0;
 	if (ctrl)
 	{
-		physid= (int)ctrl->GetUserData();
+		physid= (uint_ptr)ctrl->GetUserData();
 	}
-	return PyInt_FromLong(physid);
+	return PyInt_FromLong((long)physid);
 }
 
+KX_PYMETHODDEF_DOC(KX_GameObject, getDistanceTo,
+"getDistanceTo(other): get distance to another point/KX_GameObject")
+{
+	MT_Point3 b;
+	if (PyVecArgTo(args, b))
+	{
+		return PyFloat_FromDouble(NodeGetWorldPosition().distance(b));
+	}
+	PyErr_Clear();
+	
+	PyObject *pyother;
+	if (PyArg_ParseTuple(args, "O!", &KX_GameObject::Type, &pyother))
+	{
+		KX_GameObject *other = static_cast<KX_GameObject*>(pyother);
+		return PyFloat_FromDouble(NodeGetWorldPosition().distance(other->NodeGetWorldPosition()));
+	}
+	
+	return NULL;
+}
+
+
 /* --------------------------------------------------------------------- 
  * Some stuff taken from the header
  * --------------------------------------------------------------------- */
