@@ -1,5 +1,5 @@
 /*
- * $Id: KX_KetsjiEngine.cpp,v 1.14 2004/12/01 08:43:58 kester Exp $
+ * $Id: KX_KetsjiEngine.cpp,v 1.26 2006/01/18 06:04:11 erwin Exp $
  *
  * ***** BEGIN GPL/BL DUAL LICENSE BLOCK *****
  *
@@ -83,7 +83,7 @@
 // not valid, skip rendering this frame.
 //#define NZC_GUARDED_OUTPUT
 #define DEFAULT_LOGIC_TIC_RATE 30.0
-#define DEFAULT_PHYSICS_TIC_RATE 60.0
+#define DEFAULT_PHYSICS_TIC_RATE 90.0
 
 const char KX_KetsjiEngine::m_profileLabels[tc_numCategories][15] = {
 	"Physics:",		// tc_physics
@@ -99,6 +99,9 @@ const char KX_KetsjiEngine::m_profileLabels[tc_numCategories][15] = {
 
 double KX_KetsjiEngine::m_ticrate = DEFAULT_LOGIC_TIC_RATE;
 
+double KX_KetsjiEngine::m_suspendedtime = 0.0;
+double KX_KetsjiEngine::m_suspendeddelta = 0.0;
+
 
 /**
  *	Constructor of the Ketsji Engine
@@ -120,11 +123,14 @@ KX_KetsjiEngine::KX_KetsjiEngine(KX_ISystem* system)
 	m_bInitialized(false),
 	m_activecam(0),
 	m_bFixedTime(false),
+	m_game2ipo(false),
 	
 	m_firstframe(true),
 	
-	m_previoustime(0.0),
-	m_deltatime(0.0),
+	m_frameTime(0.f),
+	m_clockTime(0.f),
+	m_previousClockTime(0.f),
+
 
 	m_exitcode(KX_EXIT_REQUEST_NO_REQUEST),
 	m_exitstring(""),
@@ -177,7 +183,7 @@ KX_KetsjiEngine::~KX_KetsjiEngine()
 
 void KX_KetsjiEngine::SetKeyboardDevice(SCA_IInputDevice* keyboarddevice)
 {
-	assert(keyboarddevice);
+	MT_assert(keyboarddevice);
 	m_keyboarddevice = keyboarddevice;
 }
 
@@ -185,7 +191,7 @@ void KX_KetsjiEngine::SetKeyboardDevice(SCA_IInputDevice* keyboarddevice)
 
 void KX_KetsjiEngine::SetMouseDevice(SCA_IInputDevice* mousedevice)
 {
-	assert(mousedevice);
+	MT_assert(mousedevice);
 	m_mousedevice = mousedevice;
 }
 
@@ -193,7 +199,7 @@ void KX_KetsjiEngine::SetMouseDevice(SCA_IInputDevice* mousedevice)
 
 void KX_KetsjiEngine::SetNetworkDevice(NG_NetworkDeviceInterface* networkdevice)
 {
-	assert(networkdevice);
+	MT_assert(networkdevice);
 	m_networkdevice = networkdevice;
 }
 
@@ -201,7 +207,7 @@ void KX_KetsjiEngine::SetNetworkDevice(NG_NetworkDeviceInterface* networkdevice)
 
 void KX_KetsjiEngine::SetAudioDevice(SND_IAudioDevice* audiodevice)
 {
-	assert(audiodevice);
+	MT_assert(audiodevice);
 	m_audiodevice = audiodevice;
 }
 
@@ -209,7 +215,7 @@ void KX_KetsjiEngine::SetAudioDevice(SND_IAudioDevice* audiodevice)
 
 void KX_KetsjiEngine::SetCanvas(RAS_ICanvas* canvas)
 {
-	assert(canvas);
+	MT_assert(canvas);
 	m_canvas = canvas;
 }
 
@@ -217,7 +223,7 @@ void KX_KetsjiEngine::SetCanvas(RAS_ICanvas* canvas)
 
 void KX_KetsjiEngine::SetRenderTools(RAS_IRenderTools* rendertools)
 {
-	assert(rendertools);
+	MT_assert(rendertools);
 	m_rendertools = rendertools;
 }
 
@@ -225,7 +231,7 @@ void KX_KetsjiEngine::SetRenderTools(RAS_IRenderTools* rendertools)
 
 void KX_KetsjiEngine::SetRasterizer(RAS_IRasterizer* rasterizer)
 {
-	assert(rasterizer);
+	MT_assert(rasterizer);
 	m_rasterizer = rasterizer;
 }
 
@@ -233,7 +239,7 @@ void KX_KetsjiEngine::SetRasterizer(RAS_IRasterizer* rasterizer)
 
 void KX_KetsjiEngine::SetPythonDictionary(PyObject* pythondictionary)
 {
-	assert(pythondictionary);
+	MT_assert(pythondictionary);
 	m_pythondictionary = pythondictionary;
 }
 
@@ -241,7 +247,7 @@ void KX_KetsjiEngine::SetPythonDictionary(PyObject* pythondictionary)
 
 void KX_KetsjiEngine::SetSceneConverter(KX_ISceneConverter* sceneconverter)
 {
-	assert(sceneconverter);
+	MT_assert(sceneconverter);
 	m_sceneconverter = sceneconverter;
 }
 
@@ -254,11 +260,20 @@ void KX_KetsjiEngine::SetSceneConverter(KX_ISceneConverter* sceneconverter)
  */
 void KX_KetsjiEngine::StartEngine()
 {
-	m_previoustime = m_kxsystem->GetTimeInSeconds();
+	m_clockTime = m_kxsystem->GetTimeInSeconds();
+	m_frameTime = m_kxsystem->GetTimeInSeconds();
+	m_previousClockTime = m_kxsystem->GetTimeInSeconds();
+
 	m_firstframe = true;
 	m_bInitialized = true;
 	m_ticrate = DEFAULT_LOGIC_TIC_RATE;
-	SumoPhysicsEnvironment::setTicRate(DEFAULT_PHYSICS_TIC_RATE);
+	m_currentFrame = 0;
+
+	if (m_game2ipo)
+	{
+		m_sceneconverter->ResetPhysicsObjectsAnimationIpo();
+	}
+
 }
 
 bool KX_KetsjiEngine::BeginFrame()
@@ -306,32 +321,36 @@ void KX_KetsjiEngine::EndFrame()
 	m_rendertools->EndFrame(m_rasterizer);
 	
 	m_canvas->EndDraw();
+	
+
 }
 
-
+#include "PIL_time.h"
 
 void KX_KetsjiEngine::NextFrame()
 {
 	m_logger->StartLog(tc_services, m_kxsystem->GetTimeInSeconds(), true);
-
-	double curtime;
+	
 	if (m_bFixedTime)
-		curtime = m_previoustime + 1.0/m_ticrate;
+		m_clockTime += 1.0/m_ticrate;
 	else
-		curtime = m_kxsystem->GetTimeInSeconds();
-	m_deltatime += curtime - m_previoustime;
-	m_previoustime = curtime;
-	double localtime = curtime - m_deltatime;
+		m_clockTime = m_kxsystem->GetTimeInSeconds();
+	
+	double deltatime = m_clockTime - m_frameTime;
 
 	// Compute the number of logic frames to do each update (fixed tic bricks)
-	int frames = (int) (m_deltatime*m_ticrate);
-	m_deltatime -= double(frames)/m_ticrate;
+	int frames =int(deltatime*m_ticrate);
+	
+//	if (!frames)
+//		PIL_sleep_ms(4);
 	
 	KX_SceneList::iterator sceneit;
 	
 	while (frames)
 	{
-		localtime += 1.0/m_ticrate;
+
+		m_frameTime += 1.0/m_ticrate;
+		
 		for (sceneit = m_scenes.begin();sceneit != m_scenes.end(); ++sceneit)
 		// for each scene, call the proceed functions
 		{
@@ -346,10 +365,20 @@ void KX_KetsjiEngine::NextFrame()
 	
 			if (!scene->IsSuspended())
 			{
-				m_logger->StartLog(tc_physics, m_kxsystem->GetTimeInSeconds(), true);
+				// if the scene was suspended recalcutlate the delta tu "curtime"
+				m_suspendedtime = scene->getSuspendedTime();
+				if (scene->getSuspendedTime()!=0.0)
+					scene->setSuspendedDelta(scene->getSuspendedDelta()+m_clockTime-scene->getSuspendedTime());
+				m_suspendeddelta = scene->getSuspendedDelta();
+
+				
 				m_logger->StartLog(tc_network, m_kxsystem->GetTimeInSeconds(), true);
-				scene->GetNetworkScene()->proceed(localtime);
+				scene->GetNetworkScene()->proceed(m_frameTime);
 	
+				m_logger->StartLog(tc_scenegraph, m_kxsystem->GetTimeInSeconds(), true);
+				scene->UpdateParents(m_frameTime);
+				
+				m_logger->StartLog(tc_physics, m_kxsystem->GetTimeInSeconds(), true);
 				// set Python hooks for each scene
 				PHY_SetActiveEnvironment(scene->GetPhysicsEnvironment());
 				PHY_SetActiveScene(scene);
@@ -359,35 +388,49 @@ void KX_KetsjiEngine::NextFrame()
 				// Update scenegraph after physics step. This maps physics calculations
 				// into node positions.		
 				m_logger->StartLog(tc_scenegraph, m_kxsystem->GetTimeInSeconds(), true);
-				scene->UpdateParents(localtime);
+				scene->UpdateParents(m_frameTime);
 				
 				// Process sensors, and controllers
 				m_logger->StartLog(tc_logic, m_kxsystem->GetTimeInSeconds(), true);
-				scene->LogicBeginFrame(localtime);
+				scene->LogicBeginFrame(m_frameTime);
 	
 				// Scenegraph needs to be updated again, because Logic Controllers 
 				// can affect the local matrices.
 				m_logger->StartLog(tc_scenegraph, m_kxsystem->GetTimeInSeconds(), true);
-				scene->UpdateParents(localtime);
+				scene->UpdateParents(m_frameTime);
 	
 				// Process actuators
 	
 				// Do some cleanup work for this logic frame
 				m_logger->StartLog(tc_logic, m_kxsystem->GetTimeInSeconds(), true);
-				scene->LogicUpdateFrame(localtime, true);
+				scene->LogicUpdateFrame(m_frameTime, true);
 				scene->LogicEndFrame();
 	
 				// Actuators can affect the scenegraph
 				m_logger->StartLog(tc_scenegraph, m_kxsystem->GetTimeInSeconds(), true);
-				scene->UpdateParents(localtime);
+				scene->UpdateParents(m_frameTime);
 				
+				m_logger->StartLog(tc_physics, m_kxsystem->GetTimeInSeconds(), true);
 				scene->GetPhysicsEnvironment()->beginFrame();
 		
 				// Perform physics calculations on the scene. This can involve 
 				// many iterations of the physics solver.
-				m_logger->StartLog(tc_physics, m_kxsystem->GetTimeInSeconds(), true);
-				scene->GetPhysicsEnvironment()->proceed(localtime);
+				scene->GetPhysicsEnvironment()->proceedDeltaTime(m_frameTime,1.0/m_ticrate);//m_deltatimerealDeltaTime);
+
+				m_logger->StartLog(tc_scenegraph, m_kxsystem->GetTimeInSeconds(), true);
+				scene->UpdateParents(m_frameTime);
+			
+			
+				if (m_game2ipo)
+				{
+					m_sceneconverter->WritePhysicsObjectToAnimationIpo(m_currentFrame++);
+				}
+
+				scene->setSuspendedTime(0.0);
 			} // suspended
+			else
+				if(scene->getSuspendedTime()==0.0)
+					scene->setSuspendedTime(m_clockTime);
 	
 			DoSound(scene);
 	
@@ -414,41 +457,63 @@ void KX_KetsjiEngine::NextFrame()
 		frames--;
 	}
 
-	// Logic update sub frame: this will let some logic bricks run at the
-	// full frame rate.
-	for (sceneit = m_scenes.begin();sceneit != m_scenes.end(); ++sceneit)
-	// for each scene, call the proceed functions
-	{
-		KX_Scene* scene = *sceneit;
+	bool bUseAsyncLogicBricks= false;
 
-		if (!scene->IsSuspended())
+	if (bUseAsyncLogicBricks)
+	{	
+		// Logic update sub frame: this will let some logic bricks run at the
+		// full frame rate.
+		for (sceneit = m_scenes.begin();sceneit != m_scenes.end(); ++sceneit)
+		// for each scene, call the proceed functions
 		{
-			// set Python hooks for each scene
-			PHY_SetActiveEnvironment(scene->GetPhysicsEnvironment());
-			PHY_SetActiveScene(scene);
-			
-			// Perform physics calculations on the scene. This can involve 
-			// many iterations of the physics solver.
-			m_logger->StartLog(tc_physics, m_kxsystem->GetTimeInSeconds(), true);
-			scene->GetPhysicsEnvironment()->proceed(curtime);
-			// Update scenegraph after physics step. This maps physics calculations
-			// into node positions.		
-			m_logger->StartLog(tc_scenegraph, m_kxsystem->GetTimeInSeconds(), true);
-			scene->UpdateParents(curtime);
-			
-			// Do some cleanup work for this logic frame
-			m_logger->StartLog(tc_logic, m_kxsystem->GetTimeInSeconds(), true);
-			scene->LogicUpdateFrame(curtime, false);
+			KX_Scene* scene = *sceneit;
 
-			// Actuators can affect the scenegraph
-			m_logger->StartLog(tc_scenegraph, m_kxsystem->GetTimeInSeconds(), true);
-			scene->UpdateParents(curtime);
-		} // suspended
+			if (!scene->IsSuspended())
+			{
+				// if the scene was suspended recalcutlate the delta tu "curtime"
+				m_suspendedtime = scene->getSuspendedTime();
+				if (scene->getSuspendedTime()!=0.0)
+					scene->setSuspendedDelta(scene->getSuspendedDelta()+m_clockTime-scene->getSuspendedTime());
+				m_suspendeddelta = scene->getSuspendedDelta();
+				
+				// set Python hooks for each scene
+				PHY_SetActiveEnvironment(scene->GetPhysicsEnvironment());
+				PHY_SetActiveScene(scene);
+				
+				m_logger->StartLog(tc_scenegraph, m_kxsystem->GetTimeInSeconds(), true);
+				scene->UpdateParents(m_clockTime);
 
-		DoSound(scene);
+				// Perform physics calculations on the scene. This can involve 
+				// many iterations of the physics solver.
+				m_logger->StartLog(tc_physics, m_kxsystem->GetTimeInSeconds(), true);
+				scene->GetPhysicsEnvironment()->proceedDeltaTime(m_clockTime,0.f);
+				// Update scenegraph after physics step. This maps physics calculations
+				// into node positions.		
+				m_logger->StartLog(tc_scenegraph, m_kxsystem->GetTimeInSeconds(), true);
+				scene->UpdateParents(m_clockTime);
+				
+				// Do some cleanup work for this logic frame
+				m_logger->StartLog(tc_logic, m_kxsystem->GetTimeInSeconds(), true);
+				scene->LogicUpdateFrame(m_clockTime, false);
 
-		m_logger->StartLog(tc_services, m_kxsystem->GetTimeInSeconds(), true);
+				// Actuators can affect the scenegraph
+				m_logger->StartLog(tc_scenegraph, m_kxsystem->GetTimeInSeconds(), true);
+				scene->UpdateParents(m_clockTime);
+				 
+ 				scene->setSuspendedTime(0.0);
+			} // suspended
+ 			else
+ 				if(scene->getSuspendedTime()==0.0)
+ 					scene->setSuspendedTime(m_clockTime);
+
+			DoSound(scene);
+
+			m_logger->StartLog(tc_services, m_kxsystem->GetTimeInSeconds(), true);
+		}
 	}
+
+
+	m_previousClockTime = m_clockTime;
 	
 	// Start logging time spend outside main loop
 	m_logger->StartLog(tc_outside, m_kxsystem->GetTimeInSeconds(), true);
@@ -508,20 +573,47 @@ void KX_KetsjiEngine::Render()
 	// for each scene, call the proceed functions
 	{
 		KX_Scene* scene = *sceneit;
-
+		KX_Camera* cam = scene->GetActiveCamera();
 		// pass the scene's worldsettings to the rasterizer
 		SetWorldSettings(scene->GetWorldInfo());
+
+		// Avoid drawing the scene with the active camera twice when it's viewport is enabled
+		if(!cam->GetViewport())
+		{
+			if (scene->IsClearingZBuffer())
+				m_rasterizer->ClearDepthBuffer();
+	
+			m_rendertools->SetAuxilaryClientInfo(scene);
+	
+			//Initialize scene viewport.
+			SetupRenderFrame(scene, cam);
+	
+			// do the rendering
+			RenderFrame(scene, cam);
+		}
 		
-		if (scene->IsClearingZBuffer())
-			m_rasterizer->ClearDepthBuffer();
-
-		m_rendertools->SetAuxilaryClientInfo(scene);
-
-		//Initialize scene viewport.
-		SetupRenderFrame(scene);
-
-		// do the rendering
-		RenderFrame(scene);
+		set<class KX_Camera*>* cameras = scene->GetCameras();
+		
+		// Draw the scene once for each camera with an enabled viewport
+		set<KX_Camera*>::iterator it = cameras->begin();
+		while(it != cameras->end())
+		{
+			if((*it)->GetViewport())
+			{
+				if (scene->IsClearingZBuffer())
+					m_rasterizer->ClearDepthBuffer();
+		
+				m_rendertools->SetAuxilaryClientInfo(scene);
+		
+				//Initialize scene viewport.
+				SetupRenderFrame(scene, (*it));
+		
+				// do the rendering
+				RenderFrame(scene, (*it));
+			}
+			
+			it++;
+		}
 	}
 
 	// only one place that checks for stereo
@@ -537,6 +629,7 @@ void KX_KetsjiEngine::Render()
 		// for each scene, call the proceed functions
 		{
 			KX_Scene* scene = *sceneit;
+			KX_Camera* cam = scene->GetActiveCamera();
 
 			// pass the scene's worldsettings to the rasterizer
 			SetWorldSettings(scene->GetWorldInfo());
@@ -548,10 +641,12 @@ void KX_KetsjiEngine::Render()
 			m_rendertools->SetAuxilaryClientInfo(scene);
 
 			//Initialize scene viewport.
-			SetupRenderFrame(scene);
+			//SetupRenderFrame(scene);
+			SetupRenderFrame(scene, cam);
 
 			// do the rendering
-			RenderFrame(scene);
+			//RenderFrame(scene);
+			RenderFrame(scene, cam);
 		}
 	} // if(m_rasterizer->Stereo())
 
@@ -639,6 +734,13 @@ void KX_KetsjiEngine::SetWorldSettings(KX_WorldInfo* wi)
 {
 	if (wi->hasWorld())
 	{
+		// ...
+		m_rasterizer->SetAmbientColor(
+			wi->getAmbientColorRed(),
+			wi->getAmbientColorGreen(),
+			wi->getAmbientColorBlue()
+		);
+
 		if (m_drawingmode == RAS_IRasterizer::KX_TEXTURED)
 		{	
 			if (wi->hasMist())
@@ -702,7 +804,7 @@ void KX_KetsjiEngine::SetCameraOverrideViewMatrix(const MT_CmMatrix4x4& mat)
 }
 
 	
-void KX_KetsjiEngine::SetupRenderFrame(KX_Scene *scene)
+void KX_KetsjiEngine::SetupRenderFrame(KX_Scene *scene, KX_Camera* cam)
 {
 	// In this function we make sure the rasterizer settings are upto
 	// date. We compute the viewport so that logic
@@ -713,11 +815,13 @@ void KX_KetsjiEngine::SetupRenderFrame(KX_Scene *scene)
 
 	RAS_Rect viewport;
 
-	if (
-		m_overrideCam || 
-		(scene->GetName() != m_overrideSceneName) || 
-		m_overrideCamUseOrtho
-	) {
+	if (cam->GetViewport()) {
+		viewport.SetLeft(cam->GetViewportLeft()); 
+		viewport.SetBottom(cam->GetViewportBottom());
+		viewport.SetRight(cam->GetViewportRight());
+		viewport.SetTop(cam->GetViewportTop());
+	}
+	else if ( m_overrideCam || (scene->GetName() != m_overrideSceneName) ||  m_overrideCamUseOrtho ) {
 		RAS_FramingManager::ComputeViewport(
 			scene->GetFramingType(),
 			m_canvas->GetDisplayArea(),
@@ -745,21 +849,23 @@ void KX_KetsjiEngine::SetupRenderFrame(KX_Scene *scene)
 
 	
 // update graphics
-void KX_KetsjiEngine::RenderFrame(KX_Scene* scene)
+void KX_KetsjiEngine::RenderFrame(KX_Scene* scene, KX_Camera* cam)
 {
 	float left, right, bottom, top, nearfrust, farfrust;
 	const float ortho = 100.0;
-	KX_Camera* cam = scene->GetActiveCamera();
+//	KX_Camera* cam = scene->GetActiveCamera();
 	
 	if (!cam)
 		return;
-
+	
+	// see KX_BlenderMaterial::Activate
+	//m_rasterizer->SetAmbient();
 	m_rasterizer->DisplayFog();
 
 	if (m_overrideCam && (scene->GetName() == m_overrideSceneName) && m_overrideCamUseOrtho) {
 		MT_CmMatrix4x4 projmat = m_overrideCamProjMat;
 		m_rasterizer->SetProjectionMatrix(projmat);
-	} else if (cam->hasValidProjectionMatrix())
+	} else if (cam->hasValidProjectionMatrix() && !cam->GetViewport() )
 	{
 		m_rasterizer->SetProjectionMatrix(cam->GetProjectionMatrix());
 	} else
@@ -818,7 +924,7 @@ void KX_KetsjiEngine::RenderFrame(KX_Scene* scene)
 	// redrawn. There is a cache between the actual rescheduling
 	// and this call though. Visibility is imparted when this call
 	// runs through the individual objects.
-	scene->CalculateVisibleMeshes(m_rasterizer);
+	scene->CalculateVisibleMeshes(m_rasterizer,cam);
 
 	scene->RenderBuckets(camtrans, m_rasterizer, m_rendertools);
 }
@@ -829,6 +935,13 @@ void KX_KetsjiEngine::StopEngine()
 {
 	if (m_bInitialized)
 	{
+
+		if (m_game2ipo)
+		{
+//			printf("TestHandlesPhysicsObjectToAnimationIpo\n");
+			m_sceneconverter->TestHandlesPhysicsObjectToAnimationIpo();
+		}
+
 		KX_SceneList::iterator sceneit;
 		for (sceneit = m_scenes.begin();sceneit != m_scenes.end() ; sceneit++)
 		{
@@ -904,6 +1017,10 @@ void KX_KetsjiEngine::RenderDebugProperties()
 	if (tottime < 1e-6f) {
 		tottime = 1e-6f;
 	}
+
+	// Set viewport to entire canvas
+	RAS_Rect viewport;
+	m_canvas->SetViewPort(0, 0, int(m_canvas->GetWidth()), int(m_canvas->GetHeight()));
 	
 	/* Framerate display */
 	if (m_show_framerate) {
@@ -1185,12 +1302,27 @@ void KX_KetsjiEngine::SetUseFixedTime(bool bUseFixedTime)
 }
 
 
+void	KX_KetsjiEngine::SetGame2IpoMode(bool game2ipo,int startFrame)
+{
+	m_game2ipo = game2ipo;
+	if (game2ipo)
+	{
+		//when recording physics keyframes, always run at a fixed framerate
+		m_bFixedTime = true;
+	}
+	m_currentFrame = startFrame;
+}
 
 bool KX_KetsjiEngine::GetUseFixedTime(void) const
 {
 	return m_bFixedTime;
 }
 
+double KX_KetsjiEngine::GetSuspendedDelta()
+{
+	return m_suspendeddelta;
+}
+
 double KX_KetsjiEngine::GetTicRate()
 {
 	return m_ticrate;
