@@ -1,5 +1,5 @@
 /**
- * $Id: KX_MeshProxy.cpp,v 1.9 2005/04/23 12:25:21 kester Exp $
+ * $Id: KX_MeshProxy.cpp,v 1.11 2006/01/06 03:46:52 erwin Exp $
  * ***** BEGIN GPL/BL DUAL LICENSE BLOCK *****
  *
  * This program is free software; you can redistribute it and/or
@@ -40,6 +40,7 @@
 #include "KX_VertexProxy.h"
 
 #include "KX_PolygonMaterial.h"
+#include "KX_BlenderMaterial.h"
 
 #include "KX_PyMath.h"
 #include "KX_ConvertPhysicsObject.h"
@@ -67,6 +68,7 @@ PyParentObject KX_MeshProxy::Parents[] = {
 	&KX_MeshProxy::Type,
 	&SCA_IObject::Type,
 	&CValue::Type,
+	&PyObjectPlus::Type,
 	NULL
 };
 
@@ -87,27 +89,34 @@ KX_MeshProxy::_getattr(const STR_String& attr)
 {
 	if (attr == "materials")
 	{
-		PyObject *materials = PyList_New(0); /* new ref */
+		PyObject *materials = PyList_New(0);
 		RAS_MaterialBucket::Set::iterator mit = m_meshobj->GetFirstMaterial();
 		for(; mit != m_meshobj->GetLastMaterial(); ++mit)
-			PyList_Append(materials, static_cast<KX_PolygonMaterial*>((*mit)->GetPolyMaterial()));
+		{
+			RAS_IPolyMaterial *polymat = (*mit)->GetPolyMaterial();
+			if(polymat->GetFlag() & RAS_BLENDERMAT)
+			{
+				KX_BlenderMaterial *mat = static_cast<KX_BlenderMaterial*>(polymat);
+				PyList_Append(materials, mat);
+			}else
+			{
+				PyList_Append(materials, static_cast<KX_PolygonMaterial*>(polymat));
+			}
+		}
 		return materials;
 	}
-		
  	_getattr_up(SCA_IObject);
 }
 
 
 
 KX_MeshProxy::KX_MeshProxy(RAS_MeshObject* mesh)
-	: m_meshobj(mesh)
+	:	m_meshobj(mesh)
 {
-	
 }
 
 KX_MeshProxy::~KX_MeshProxy()
 {
-	
 }
 
 
@@ -213,5 +222,6 @@ PyObject* KX_MeshProxy::PyGetVertex(PyObject* self,
 KX_PYMETHODDEF_DOC(KX_MeshProxy, reinstancePhysicsMesh,
 "Reinstance the physics mesh.")
 {
-	return Py_Success(KX_ReInstanceShapeFromMesh(m_meshobj));
+	//this needs to be reviewed, it is dependend on Sumo/Solid. Who is using this ?
+	return Py_None;//Py_Success(KX_ReInstanceShapeFromMesh(m_meshobj));
 }
