@@ -1,7 +1,7 @@
 /**
  * Cast a ray and feel for objects
  *
- * $Id: KX_RaySensor.cpp,v 1.9 2005/03/25 10:33:37 kester Exp $
+ * $Id: KX_RaySensor.cpp,v 1.13 2005/11/11 05:02:14 erwin Exp $
  *
  * ***** BEGIN GPL/BL DUAL LICENSE BLOCK *****
  *
@@ -40,17 +40,16 @@
 #include "KX_ClientObjectInfo.h"
 #include "KX_GameObject.h"
 #include "KX_Scene.h"
-
 #include "KX_RayCast.h"
 #include "PHY_IPhysicsEnvironment.h"
-#include "PHY_IPhysicsController.h"
 #include "KX_IPhysicsController.h"
-
+#include "PHY_IPhysicsController.h"
 
 #ifdef HAVE_CONFIG_H
 #include <config.h>
 #endif
 
+
 KX_RaySensor::KX_RaySensor(class SCA_EventManager* eventmgr,
 					SCA_IObject* gameobj,
 					const STR_String& propname,
@@ -104,17 +103,18 @@ bool KX_RaySensor::IsPositiveTrigger()
 	return result;
 }
 
-bool KX_RaySensor::RayHit(KX_ClientObjectInfo* info, MT_Point3& hit_point, MT_Vector3& hit_normal, void* const data)
+bool KX_RaySensor::RayHit(KX_ClientObjectInfo* client, MT_Point3& hit_point, MT_Vector3& hit_normal, void * const data)
 {
-	KX_GameObject* obj = (KX_GameObject*)GetParent();
-	SCA_IObject *hitgameobj = info->m_gameobject;
-	if (hitgameobj == obj || info->m_type > KX_ClientObjectInfo::ACTOR)
+
+	KX_GameObject* hitKXObj = client->m_gameobject;
+	
+	if (client->m_type > KX_ClientObjectInfo::ACTOR)
 	{
 		// false hit
 		return false;
 	}
-	
 	bool bFound = false;
+
 	if (m_propertyname.Length() == 0)
 	{
 		bFound = true;
@@ -123,29 +123,32 @@ bool KX_RaySensor::RayHit(KX_ClientObjectInfo* info, MT_Point3& hit_point, MT_Ve
 	{
 		if (m_bFindMaterial)
 		{
-			if (info->m_auxilary_info)
+			if (client->m_auxilary_info)
 			{
-				bFound = (m_propertyname== ((char*)info->m_auxilary_info));
+				bFound = (m_propertyname== ((char*)client->m_auxilary_info));
 			}
 		}
 		else
 		{
-			bFound = hitgameobj->GetProperty(m_propertyname) != NULL;
+			bFound = hitKXObj->GetProperty(m_propertyname) != NULL;
 		}
 	}
 
 	if (bFound)
 	{
 		m_rayHit = true;
-		m_hitObject = hitgameobj;
+		m_hitObject = hitKXObj;
 		m_hitPosition = hit_point;
 		m_hitNormal = hit_normal;
 			
 	}
-	
+
 	return true;
+	
 }
 
+
+
 bool KX_RaySensor::Evaluate(CValue* event)
 {
 	bool result = false;
@@ -211,64 +214,33 @@ bool KX_RaySensor::Evaluate(CValue* event)
 	MT_Point3 topoint = frompoint + (m_distance) * todir;
 	MT_Point3 resultpoint;
 	MT_Vector3 resultnormal;
-	PHY_IPhysicsEnvironment* physics_environment = m_scene->GetPhysicsEnvironment();
-	if (!physics_environment)
+	bool ready = false;
+	PHY_IPhysicsEnvironment* pe = m_scene->GetPhysicsEnvironment();
+
+	if (!pe)
 	{
 		std::cout << "WARNING: Ray sensor " << GetName() << ":  There is no physics environment!" << std::endl;
 		std::cout << "         Check universe for malfunction." << std::endl;
 		return false;
 	} 
-	KX_IPhysicsController* physics_controller = obj->GetPhysicsController();
 
-	// Use the parent's physics controller if obj has no physics controller.
+	KX_IPhysicsController *spc = obj->GetPhysicsController();
 	KX_GameObject *parent = obj->GetParent();
-	if (!physics_controller && parent)
-		physics_controller = parent->GetPhysicsController();
-
+	if (!spc && parent)
+		spc = parent->GetPhysicsController();
+	
 	if (parent)
 		parent->Release();
-		
-	KX_RayCast::RayTest(physics_controller, physics_environment, frompoint, topoint, resultpoint, resultnormal, KX_RayCast::Callback<KX_RaySensor>(this));
-
-// 	do {
-// 		PHY__Vector3 respos;
-// 		PHY__Vector3 resnormal;
-// 
-// 		PHY_IPhysicsController* hitCtrl = spe->rayTest(physCtrl,
-// 			frompoint.x(),frompoint.y(),frompoint.z(),
-// 			topoint.x(),topoint.y(),topoint.z(),
-// 			respos[0],respos[1],respos[2],
-// 			resnormal[0],resnormal[1],resnormal[2]);
-// 			
-// 		if (hitCtrl)
-// 		{
-// 
-// 			resultpoint = MT_Vector3(respos);
-// 			resultnormal = MT_Vector3(resnormal);
-// 			KX_ClientObjectInfo* info = static_cast<KX_ClientObjectInfo*>(hitCtrl->getNewClientInfo());
-// 			bool bFound = false;
-// 			
-// 			if (!info)
-// 			{
-// 				std::cout<< "WARNING:  Ray sensor " << GetName() << " cannot sense PHY_IPhysicsController  - no client info.\n" << std::endl;
-// 				ready = true;
-// 				break;
-// 			} 
-// 			
-// 			
-// 
-// 		}
-// 		else
-// 		{
-// 			ready = true;
-// 		}
-// 	}
-// 	while (!ready);
-// 	
 	
+
+	PHY_IPhysicsEnvironment* physics_environment = this->m_scene->GetPhysicsEnvironment();
+	
+
+	result = KX_RayCast::RayTest(spc, physics_environment, frompoint, topoint, resultpoint, resultnormal, KX_RayCast::Callback<KX_RaySensor>(this));
+
 	
 	/* now pass this result to some controller */
-	if (m_rayHit) 
+    if (m_rayHit) 
 	{
 		if (!m_bTriggered)
 		{
@@ -276,6 +248,10 @@ bool KX_RaySensor::Evaluate(CValue* event)
 			result = true;
 			m_bTriggered = true;
 		}
+		else
+		{
+			
+		}
 	}
 	else
 	{
