@@ -1,5 +1,5 @@
 /*
- * $Id: KX_Scene.cpp,v 1.11 2004/11/22 10:19:19 kester Exp $
+ * $Id: KX_Scene.cpp,v 1.23 2006/01/06 03:46:53 erwin Exp $
  *
  * ***** BEGIN GPL/BL DUAL LICENSE BLOCK *****
  *
@@ -31,17 +31,15 @@
  * Ketsji scene. Holds references to all scene data.
  */
 
-#ifdef HAVE_CONFIG_H
-#include <config.h>
-#endif
-
 #ifdef WIN32
 #pragma warning (disable : 4786)
 #endif //WIN32
 
+#include "KX_Scene.h"
+#include "MT_assert.h"
+
 #include "KX_KetsjiEngine.h"
 #include "RAS_IPolygonMaterial.h"
-#include "KX_Scene.h"
 #include "ListValue.h"
 #include "SCA_LogicManager.h"
 #include "SCA_TimeEventManager.h"
@@ -53,8 +51,11 @@
 #include "SCA_MouseManager.h"
 #include "SCA_PropertyEventManager.h"
 #include "KX_Camera.h"
+#include "SCA_JoystickManager.h"
 
 #include "RAS_MeshObject.h"
+#include "BL_SkinMeshObject.h"
+
 #include "RAS_IRasterizer.h"
 #include "RAS_BucketManager.h"
 
@@ -74,8 +75,9 @@
 #include "PHY_IPhysicsEnvironment.h"
 #include "KX_IPhysicsController.h"
 
-#include "SM_Scene.h"
-#include "SumoPhysicsEnvironment.h"
+#include "BL_SkinDeformer.h"
+#include "BL_DeformableGameObject.h"
+
 
 void* KX_SceneReplicationFunc(SG_IObject* node,void* gameobj,void* scene)
 {
@@ -112,7 +114,8 @@ KX_Scene::KX_Scene(class SCA_IInputDevice* keyboarddevice,
 	m_active_camera(NULL),
 	m_ueberExecutionPriority(0)
 {
-		
+	m_suspendedtime = 0.0;
+	m_suspendeddelta = 0.0;
 
 	m_activity_culling = false;
 	m_suspend = false;
@@ -135,6 +138,8 @@ KX_Scene::KX_Scene(class SCA_IInputDevice* keyboarddevice,
 	KX_RayEventManager* raymgr = new KX_RayEventManager(m_logicmgr);
 
 	KX_NetworkEventManager* netmgr = new KX_NetworkEventManager(m_logicmgr, ndi);
+	
+	SCA_JoystickManager *joymgr	= new SCA_JoystickManager(m_logicmgr);
 
 	m_logicmgr->RegisterEventManager(alwaysmgr);
 	m_logicmgr->RegisterEventManager(propmgr);
@@ -144,9 +149,10 @@ KX_Scene::KX_Scene(class SCA_IInputDevice* keyboarddevice,
 	m_logicmgr->RegisterEventManager(rndmgr);
 	m_logicmgr->RegisterEventManager(raymgr);
 	m_logicmgr->RegisterEventManager(netmgr);
+	m_logicmgr->RegisterEventManager(joymgr);
 
 	m_soundScene = new SND_Scene(adi);
-	assert (m_networkDeviceInterface != NULL);
+	MT_assert (m_networkDeviceInterface != NULL);
 	m_networkScene = new NG_NetworkScene(m_networkDeviceInterface);
 	
 	m_rootnode = NULL;
@@ -187,7 +193,7 @@ KX_Scene::~KX_Scene()
 
 	if (m_euthanasyobjects)
 		m_euthanasyobjects->Release();
-	
+
 	if (m_logicmgr)
 		delete m_logicmgr;
 
@@ -204,8 +210,7 @@ KX_Scene::~KX_Scene()
 	{
 		delete m_bucketmanager;
 	}
-	
-	Py_DECREF(m_attrlist);
+	//Py_DECREF(m_attrlist);
 }
 
 
@@ -256,6 +261,13 @@ SCA_TimeEventManager* KX_Scene::GetTimeEventManager()
 
 
 
+ 
+set<class KX_Camera*>* KX_Scene::GetCameras()
+{
+	return &m_cameras;
+}
+
+
 
 void KX_Scene::SetFramingType(RAS_FrameSettings & frame_settings)
 {
@@ -397,7 +409,7 @@ KX_GameObject* KX_Scene::AddNodeReplicaObject(class SG_IObject* node, class CVal
 	}
 	
 	SG_IObject* replicanode = newobj->GetSGNode();
-	SG_Node* rootnode = (replicanode == m_rootnode ? NULL : m_rootnode);
+//	SG_Node* rootnode = (replicanode == m_rootnode ? NULL : m_rootnode);
 
 	replicanode->SetSGClientObject(newobj);
 
@@ -600,22 +612,24 @@ SCA_IObject* KX_Scene::AddReplicaObject(class CValue* originalobject,
 
 	MT_Matrix3x3 newori = ((KX_GameObject*) parentobject)->NodeGetWorldOrientation();
 	replica->NodeSetLocalOrientation(newori);
-
-	if (replica->GetPhysicsController())
-	{
-		replica->GetPhysicsController()->setPosition(newpos);
-		replica->GetPhysicsController()->setOrientation(newori.getRotation());
-	}
-
-	// here we want to set the relative scale: the rootnode's scale will override all other
-	// scalings, so lets better prepare for it
-
+	
 	// get the rootnode's scale
 	MT_Vector3 newscale = parentobj->GetSGNode()->GetRootSGParent()->GetLocalScale();
 
 	// set the replica's relative scale with the rootnode's scale
 	replica->NodeSetRelativeScale(newscale);
 
+	if (replica->GetPhysicsController())
+	{
+		replica->GetPhysicsController()->setPosition(newpos);
+		replica->GetPhysicsController()->setOrientation(newori.getRotation());
+		replica->GetPhysicsController()->setScaling(newscale);
+	}
+
+	// here we want to set the relative scale: the rootnode's scale will override all other
+	// scalings, so lets better prepare for it
+
+
 	replica->GetSGNode()->UpdateWorldData(0);
 	replica->GetSGNode()->SetBBox(originalobj->GetSGNode()->BBox());
 	replica->GetSGNode()->SetRadius(originalobj->GetSGNode()->Radius());
@@ -657,11 +671,10 @@ void KX_Scene::DelayedRemoveObject(class CValue* gameobj)
 void KX_Scene::NewRemoveObject(class CValue* gameobj)
 {
 	KX_GameObject* newobj = (KX_GameObject*) gameobj;
-	//SM_Object* sumoObj = newobj->GetSumoObject();
-	//if (sumoObj)
-	//{
-	//	this->GetSumoScene()->remove(*sumoObj);
-	//}
+
+	//todo: look at this
+	//GetPhysicsEnvironment()->RemovePhysicsController(gameobj->getPhysicsController());
+
 	// remove all sensors/controllers/actuators from logicsystem...
 	
 	SCA_SensorList& sensors = newobj->GetSensors();
@@ -710,6 +723,7 @@ void KX_Scene::NewRemoveObject(class CValue* gameobj)
 		
 	if (newobj == m_active_camera)
 	{
+		m_active_camera->Release();
 		m_active_camera = NULL;
 	}
 }
@@ -719,8 +733,42 @@ void KX_Scene::NewRemoveObject(class CValue* gameobj)
 void KX_Scene::ReplaceMesh(class CValue* gameobj,void* meshobj)
 {
 	KX_GameObject* newobj = (KX_GameObject*) gameobj;
+	RAS_MeshObject* mesh = (RAS_MeshObject*) meshobj;
+
 	newobj->RemoveMeshes();
-	newobj->AddMesh((RAS_MeshObject*)meshobj);
+	newobj->AddMesh(mesh);
+
+	if (newobj->m_isDeformable && mesh->m_class == 1) {
+		Object* blendobj = (struct Object*)m_logicmgr->FindBlendObjByGameObj(newobj);
+		Object* oldblendobj = (struct Object*)m_logicmgr->FindBlendObjByGameMeshName(mesh->GetName());
+		
+		if (blendobj->parent && blendobj->parent->type == OB_ARMATURE && 
+			blendobj->partype==PARSKEL && 
+			((Mesh*)blendobj->data)->dvert) 
+		{
+			// FIXME: should the old m_pDeformer be deleted?
+			// it shouldn't be a problem to delete it now.
+			// if we are constructing this on the fly like here,
+			// make sure to release newobj->GetParent(), and things will run shipshape
+			delete static_cast<BL_DeformableGameObject*>( newobj )->m_pDeformer;
+			
+			BL_SkinDeformer* skindeformer = new BL_SkinDeformer(
+				oldblendobj, blendobj,
+				static_cast<BL_SkinMeshObject*>(mesh),
+				true, // release ref count to BL_ArmatureObject, leak otherwise
+				static_cast<BL_ArmatureObject*>(newobj->GetParent())
+			);
+			static_cast<BL_DeformableGameObject*>( newobj )->m_pDeformer = skindeformer;
+		}
+		else if (((Mesh*)blendobj->data)->dvert) {
+			BL_MeshDeformer* meshdeformer = new BL_MeshDeformer(oldblendobj, (BL_SkinMeshObject*)mesh );
+
+			// FIXME: should the old m_pDeformer be deleted?
+			// delete ((BL_DeformableGameObject*)newobj)->m_pDeformer
+			((BL_DeformableGameObject*)newobj)->m_pDeformer = meshdeformer;
+		}
+	}
+
 	newobj->Bucketize();
 }
 
@@ -803,48 +851,48 @@ void KX_Scene::UpdateMeshTransformations()
 	}
 }
 
-void KX_Scene::MarkVisible(SG_Tree *node, RAS_IRasterizer* rasty)
+void KX_Scene::MarkVisible(SG_Tree *node, RAS_IRasterizer* rasty, KX_Camera* cam)
 {
 	int intersect = KX_Camera::INTERSECT;
 	KX_GameObject *gameobj = node->Client()?(KX_GameObject*) node->Client()->GetSGClientObject():NULL;
 	bool dotest = (gameobj && gameobj->GetVisible()) || node->Left() || node->Right();
 	
 	/* If the camera is inside the box, assume intersect. */
-	if (dotest && !node->inside(GetActiveCamera()->NodeGetWorldPosition()))
+	if (dotest && !node->inside( cam->NodeGetWorldPosition()))
 	{
 		MT_Scalar radius = node->Radius();
 		MT_Point3 centre = node->Centre();
 		
-		intersect = GetActiveCamera()->SphereInsideFrustum(centre, radius); 
+		intersect =  cam->SphereInsideFrustum(centre, radius); 
 		
 		if (intersect == KX_Camera::INTERSECT)
 		{
 			MT_Point3 box[8];
 			node->get(box);
-			intersect = GetActiveCamera()->BoxInsideFrustum(box);
+			intersect = cam->BoxInsideFrustum(box);
 		}
 	}
 
 	switch (intersect)
 	{
 		case KX_Camera::OUTSIDE:
-			MarkSubTreeVisible(node, rasty, false);
+			MarkSubTreeVisible(node, rasty, false, cam);
 			break;
 		case KX_Camera::INTERSECT:
 			if (gameobj)
-				MarkVisible(rasty, gameobj);
+				MarkVisible(rasty, gameobj,cam);
 			if (node->Left())
-				MarkVisible(node->Left(), rasty);
+				MarkVisible(node->Left(), rasty,cam);
 			if (node->Right())
-				MarkVisible(node->Right(), rasty);
+				MarkVisible(node->Right(), rasty,cam);
 			break;
 		case KX_Camera::INSIDE:
-			MarkSubTreeVisible(node, rasty, true);
+			MarkSubTreeVisible(node, rasty, true,cam);
 			break;
 	}
 }
 
-void KX_Scene::MarkSubTreeVisible(SG_Tree *node, RAS_IRasterizer* rasty, bool visible)
+void KX_Scene::MarkSubTreeVisible(SG_Tree *node, RAS_IRasterizer* rasty, bool visible,KX_Camera* cam)
 {
 	if (node->Client())
 	{
@@ -854,7 +902,7 @@ void KX_Scene::MarkSubTreeVisible(SG_Tree *node, RAS_IRasterizer* rasty, bool vi
 			if (visible)
 			{
 				int nummeshes = gameobj->GetMeshCount();
-				MT_Transform t( GetActiveCamera()->GetWorldToCamera() * gameobj->GetSGNode()->GetWorldTransform());
+				MT_Transform t( cam->GetWorldToCamera() * gameobj->GetSGNode()->GetWorldTransform());
 	
 				
 				for (int m=0;m<nummeshes;m++)
@@ -867,18 +915,18 @@ void KX_Scene::MarkSubTreeVisible(SG_Tree *node, RAS_IRasterizer* rasty, bool vi
 		}
 	}
 	if (node->Left())
-		MarkSubTreeVisible(node->Left(), rasty, visible);
+		MarkSubTreeVisible(node->Left(), rasty, visible,cam);
 	if (node->Right())
-		MarkSubTreeVisible(node->Right(), rasty, visible);
+		MarkSubTreeVisible(node->Right(), rasty, visible,cam);
 }
 
-void KX_Scene::MarkVisible(RAS_IRasterizer* rasty, KX_GameObject* gameobj)
+void KX_Scene::MarkVisible(RAS_IRasterizer* rasty, KX_GameObject* gameobj,KX_Camera*  cam)
 {
 	// User (Python/Actuator) has forced object invisible...
 	if (!gameobj->GetVisible())
 		return;
 	// If Frustum culling is off, the object is always visible.
-	bool vis = !GetActiveCamera()->GetFrustumCulling();
+	bool vis = !cam->GetFrustumCulling();
 	
 	// If the camera is inside this node, then the object is visible.
 	if (!vis)
@@ -891,7 +939,7 @@ void KX_Scene::MarkVisible(RAS_IRasterizer* rasty, KX_GameObject* gameobj)
 	{
 		MT_Vector3 scale = gameobj->GetSGNode()->GetWorldScaling();
 		MT_Scalar radius = fabs(scale[scale.closestAxis()] * gameobj->GetSGNode()->Radius());
-		switch (GetActiveCamera()->SphereInsideFrustum(gameobj->NodeGetWorldPosition(), radius))
+		switch (cam->SphereInsideFrustum(gameobj->NodeGetWorldPosition(), radius))
 		{
 			case KX_Camera::INSIDE:
 				vis = true;
@@ -903,7 +951,7 @@ void KX_Scene::MarkVisible(RAS_IRasterizer* rasty, KX_GameObject* gameobj)
 				// Test the object's bound box against the view frustum.
 				MT_Point3 box[8];
 				gameobj->GetSGNode()->getBBox(box); 
-				vis = GetActiveCamera()->BoxInsideFrustum(box) != KX_Camera::OUTSIDE;
+				vis = cam->BoxInsideFrustum(box) != KX_Camera::OUTSIDE;
 				break;
 		}
 	}
@@ -911,7 +959,7 @@ void KX_Scene::MarkVisible(RAS_IRasterizer* rasty, KX_GameObject* gameobj)
 	if (vis)
 	{
 		int nummeshes = gameobj->GetMeshCount();
-		MT_Transform t(GetActiveCamera()->GetWorldToCamera() * gameobj->GetSGNode()->GetWorldTransform());
+		MT_Transform t(cam->GetWorldToCamera() * gameobj->GetSGNode()->GetWorldTransform());
 		
 		for (int m=0;m<nummeshes;m++)
 		{
@@ -926,20 +974,20 @@ void KX_Scene::MarkVisible(RAS_IRasterizer* rasty, KX_GameObject* gameobj)
 	}
 }
 
-void KX_Scene::CalculateVisibleMeshes(RAS_IRasterizer* rasty)
+void KX_Scene::CalculateVisibleMeshes(RAS_IRasterizer* rasty,KX_Camera* cam)
 {
 // FIXME: When tree is operational
 #if 1
 	// do this incrementally in the future
 	for (int i = 0; i < m_objectlist->GetCount(); i++)
 	{
-		MarkVisible(rasty, static_cast<KX_GameObject*>(m_objectlist->GetValue(i)));
+		MarkVisible(rasty, static_cast<KX_GameObject*>(m_objectlist->GetValue(i)), cam);
 	}
 #else
-	if (GetActiveCamera()->GetFrustumCulling())
-		MarkVisible(m_objecttree, rasty);
+	if (cam->GetFrustumCulling())
+		MarkVisible(m_objecttree, rasty, cam);
 	else
-		MarkSubTreeVisible(m_objecttree, rasty, true);
+		MarkSubTreeVisible(m_objecttree, rasty, true, cam);
 #endif
 }
 
@@ -995,14 +1043,6 @@ void KX_Scene::LogicEndFrame()
 		KX_GameObject* gameobj = (KX_GameObject*)m_euthanasyobjects->GetValue(i);
 		this->RemoveObject(gameobj);
 	}
-	
-	numobj = m_euthanasyobjects->GetCount();
-	if (numobj != 0)
-	{
-		// huh?
-		int ii=0;
-	}
-	// numobj is 0 we hope
 }
 
 
@@ -1109,14 +1149,28 @@ void KX_Scene::SetNodeTree(SG_Tree* root)
 
 void KX_Scene::SetPhysicsEnvironment(class PHY_IPhysicsEnvironment* physEnv)
 {
-	SumoPhysicsEnvironment *sme = dynamic_cast<SumoPhysicsEnvironment *>(physEnv);
 	m_physicsEnvironment = physEnv;
-	if (sme)
-	{
-		KX_TouchEventManager* touchmgr = new KX_TouchEventManager(m_logicmgr, sme->GetSumoScene());
-		m_logicmgr->RegisterEventManager(touchmgr);
-		return;
-	}
+
+	KX_TouchEventManager* touchmgr = new KX_TouchEventManager(m_logicmgr, physEnv);
+	m_logicmgr->RegisterEventManager(touchmgr);
+	return;
+}
+ 
+void KX_Scene::setSuspendedTime(double suspendedtime)
+{
+	m_suspendedtime = suspendedtime;
+}
+double KX_Scene::getSuspendedTime()
+{
+	return m_suspendedtime;
+}
+void KX_Scene::setSuspendedDelta(double suspendeddelta)
+{
+	m_suspendeddelta = suspendeddelta;
+}
+double KX_Scene::getSuspendedDelta()
+{
+	return m_suspendeddelta;
 }
 
 //----------------------------------------------------------------------------
