@@ -1,5 +1,5 @@
 /**
- * $Id: KX_Scene.h,v 1.13 2005/03/25 10:33:37 kester Exp $
+ * $Id: KX_Scene.h,v 1.15 2006/01/06 03:46:53 erwin Exp $
  *
  * ***** BEGIN GPL/BL DUAL LICENSE BLOCK *****
  *
@@ -248,9 +248,12 @@ protected:
 	/**
 	 * Visibility testing functions.
 	 */
-	void MarkVisible(SG_Tree *node, RAS_IRasterizer* rasty);
-	void MarkSubTreeVisible(SG_Tree *node, RAS_IRasterizer* rasty, bool visible);
-	void MarkVisible(RAS_IRasterizer* rasty, KX_GameObject* gameobj);
+	void MarkVisible(SG_Tree *node, RAS_IRasterizer* rasty, KX_Camera*cam);
+	void MarkSubTreeVisible(SG_Tree *node, RAS_IRasterizer* rasty, bool visible, KX_Camera*cam);
+	void MarkVisible(RAS_IRasterizer* rasty, KX_GameObject* gameobj, KX_Camera*cam);
+
+	double				m_suspendedtime;
+	double				m_suspendeddelta;
 	
 	/**
 	 * This stores anything from python
@@ -319,6 +322,10 @@ public:
 	GetTimeEventManager(
 	);
 
+		set<class KX_Camera*>*
+	GetCameras(
+	);
+ 
 
 	/** Find a camera in the scene by pointer. */
 		KX_Camera*              
@@ -447,7 +454,7 @@ public:
 	void SetNetworkScene(NG_NetworkScene *newScene);
 	void SetWorldInfo(class KX_WorldInfo* wi);
 	KX_WorldInfo* GetWorldInfo();
-	void CalculateVisibleMeshes(RAS_IRasterizer* rasty);
+	void CalculateVisibleMeshes(RAS_IRasterizer* rasty, KX_Camera *cam);
 	void UpdateMeshTransformations();
 	KX_Camera* GetpCamera();
 	SND_Scene* GetSoundScene();
@@ -515,6 +522,25 @@ public:
 	virtual PyObject* _getattr(const STR_String& attr); /* name, active_camera, gravity, suspended, viewport, framing, activity_culling, activity_culling_radius */
 	virtual int _setattr(const STR_String &attr, PyObject *pyvalue);
 	virtual int _delattr(const STR_String &attr);
+
+	/**
+	 * Sets the time the scene was suspended
+	 */ 
+	void setSuspendedTime(double suspendedtime);
+	/**
+	 * Returns the "curtime" the scene was suspended
+	 */ 
+	double getSuspendedTime();
+	/**
+	 * Sets the difference between the local time of the scene (when it
+	 * was running and not suspended) and the "curtime"
+	 */ 
+	void setSuspendedDelta(double suspendeddelta);
+	/**
+	 * Returns the difference between the local time of the scene (when it
+	 * was running and not suspended) and the "curtime"
+	 */
+	double getSuspendedDelta();
 };
 
 typedef std::vector<KX_Scene*> KX_SceneList;
