@@ -1,3 +1,6 @@
+#include "KX_ConvertPhysicsObject.h"
+
+#ifdef USE_SUMO_SOLID
 
 #ifdef WIN32
 #pragma warning (disable : 4786)
@@ -154,9 +157,11 @@ void	KX_SumoPhysicsController::SetObject (SG_IObject* object)
 	SG_Controller::SetObject(object);
 
 	// cheating here...
-	KX_GameObject* gameobj = (KX_GameObject*)	object->GetSGClientObject();
-	gameobj->SetPhysicsController(this);
-	GetSumoObject()->setClientObject(gameobj->getClientInfo());
+//should not be necessary, is it for duplicates ?
+
+KX_GameObject* gameobj = (KX_GameObject*)	object->GetSGClientObject();
+gameobj->SetPhysicsController(this,gameobj->IsDynamic());
+GetSumoObject()->setClientObject(gameobj->getClientInfo());
 }
 
 
@@ -216,3 +221,4 @@ KX_SumoPhysicsController::~KX_SumoPhysicsController()
 }
 
 
+#endif//USE_SUMO_SOLID
