@@ -1,5 +1,5 @@
 /**
- * $Id: KX_VertexProxy.h,v 1.9 2005/01/16 06:02:05 kester Exp $
+ * $Id: KX_VertexProxy.h,v 1.10 2006/01/06 03:46:53 erwin Exp $
  *
  * ***** BEGIN GPL/BL DUAL LICENSE BLOCK *****
  *
@@ -63,6 +63,10 @@ public:
 	KX_PYMETHOD(KX_VertexProxy,SetXYZ);
 	KX_PYMETHOD(KX_VertexProxy,GetUV);
 	KX_PYMETHOD(KX_VertexProxy,SetUV);
+	
+	KX_PYMETHOD(KX_VertexProxy,GetUV2);
+	KX_PYMETHOD(KX_VertexProxy,SetUV2);
+
 	KX_PYMETHOD(KX_VertexProxy,GetRGBA);
 	KX_PYMETHOD(KX_VertexProxy,SetRGBA);
 	KX_PYMETHOD(KX_VertexProxy,GetNormal);
