@@ -1,5 +1,5 @@
 /**
- * $Id: OdePhysicsController.cpp,v 1.9 2004/10/16 11:41:49 kester Exp $
+ * $Id: OdePhysicsController.cpp,v 1.11 2005/07/16 21:47:52 erwin Exp $
  *
  * ***** BEGIN GPL/BL DUAL LICENSE BLOCK *****
  *
@@ -20,6 +20,7 @@
  * ***** END GPL/BL DUAL LICENSE BLOCK *****
  */
  
+#define USE_ODE
 #ifdef USE_ODE
 
 #include "OdePhysicsController.h"
@@ -421,6 +422,12 @@ void ODEPhysicsController::getOrientation(float &quatImag0,float &quatImag1,floa
   quatReal=q[3];
 }
 
+void 		ODEPhysicsController::getPosition(PHY__Vector3&	pos) const
+{
+	m_MotionState->getWorldPosition(pos[0],pos[1],pos[2]);
+
+}
+	
 void ODEPhysicsController::setPosition(float posX,float posY,float posZ)
 {
   if (!m_bPhantom)
