@@ -1,5 +1,5 @@
 /**
- * $Id: PHY_DynamicTypes.h,v 1.8 2005/03/25 10:33:39 kester Exp $
+ * $Id: PHY_DynamicTypes.h,v 1.9 2005/07/16 21:47:54 erwin Exp $
  *
  * ***** BEGIN GPL/BL DUAL LICENSE BLOCK *****
  *
@@ -32,15 +32,6 @@
 #ifndef __PHY_DYNAMIC_TYPES
 #define __PHY_DYNAMIC_TYPES
 
-/// PHY_ScalarType enumerates possible scalar types.
-/// See the PHY_IMeshInterface for its use
-typedef enum PHY_ScalarType {
-	PHY_FLOAT,
-	PHY_DOUBLE,
-	PHY_INTEGER,
-	PHY_SHORT,
-	PHY_FIXEDPOINT88
-} PHY_ScalarType;
 
 class	PHY_ResponseTable;
 
